function [At,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb,Rbar,AIMTolerance)
% ForwardBackShootingAlgorithm: Computes the anticipated shocks from a DSGE model.
%
% USAGE:
%
%       [At,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb,Rbar,AIMTolerance)
%
% REQUIRED INPUT:  Xt (vector) of dimension (q+r+1) with values for the structural shocks,
%                     the state variables and the unrestricted policy rate.
%
%                  At (vector) of dimension T+1 with initial values for the anticipated
%                     shocks.
%
%                  ePG (matrix) of dimension (T+1)x(T+q+r+2) which gives the restricted
%                     policy projections when premultiplied by the stacked vector (Xt,At).
%
%                  Rzlb (vector) of dimension (T+1) with the path for the zero lower bound.
%
%                  Rbar (vector) of dimension (T+1) with the path for the steady-state of the
%                     the restricted policy rate.
%
%                  AIMTolerance (positive integer) that determines the tolerance for the
%                     complementary slackness condition.
%
% REQUIRED OUTPUT: At (vector) of dimension T+1 with the equilibrium values for the anticipated
%                     shocks.
%
%                  status (boolean) that takes the value 1 if a solution is located, and 0 if
%                     there is no solution.
%
%                  AlgorithmInformation (vector) of dimension 1x5 where the first entry shows
%                     the BadSolution boolean variable (invertibility problem when solving for
%                     At); the second is NoSolution (elimination of tau1, first added in step 1,
%                     in step 2 implies that the lower bound is violated for some time period);
%                     the third represents the CompSlackProblem boolean (At is nonnagetive and
%                     ZLB is not violated but product of these vectors is not close to zero);
%                     the fourth gives the number of times Steps 1 and 2 of algorithm were
%                     computed; and the fifth gives the total number of tries the algorithm
%                     could have been computed. This maximum number of times is set to the
%                     maximum of 1000 and (100*T).
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: September 2, 2014.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 03-09-2014: Continued work on the function.
%
% * 04-09-2014: Finished the function.
%
% * 16-09-2014: Made the total number of tries dependent on the length of the vector of anticipated shocks.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 09-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
status = 0;
AlgorithmInformation = [];
BadSolution = 0;
NoSolution = 0;
NumTries = 0;
InnerTries = 0;
CompSlackProblem = 0;
condn = 10^(-AIMTolerance-2);
%
T1 = length(At);
qk = length(Xt);
TotalTries = max(100*(T1-1),1000);
%
% determine which entries of At are positive. This specifies the Ttau set
%
Ttau = (At'>0).*(1:T1);
Ttau = Ttau(Ttau>0);
%
LB = Rzlb-Rbar;
Atmp = zeros(T1,1);
%
% run the algorithm
%
while (status==0)&(NoSolution==0)&(NumTries<=TotalTries);
   NumTries = NumTries+1;
   %
   % step 1:
   %
   Xtilde = [Xt;At];
   tau1 = [];
   %
   % (i) compute the projected restricted policy rate and record the first
   % tau1>=1 for Rtilde_(t+tau1-1|t) yields a value below the lower bound
   %
   Rtilde = (ePG*Xtilde)';
   TestVecR = (Rtilde+condn<LB).*(1:T1);
   TestVecR = TestVecR(TestVecR>0);
   if isempty(TestVecR)==0;
      tau1 = min(setdiff(TestVecR,Ttau));
      Ttau = unique([Ttau tau1]);
   end;
   nt = length(Ttau);
   if nt>=1;
      %
      % (ii) solve for the non-zero elements of At
      %
      ePGX = ePG(Ttau,1:qk);
      ePGA = ePG(Ttau,qk+Ttau);
      if rank(ePGA)==nt;
         Atau = ePGA\(LB(Ttau)'-(ePGX*Xt));
         At = Atmp;
         At(Ttau,1) = Atau;
      else;
         BadSolution = 1;
      end;
   end;
   if BadSolution==1;
      break;
   end;
   %
   % step 2:
   %
   % (i) Record the first period tau0 for which At has a negative element.
   %     Delete this period from Ttau and set the corresponding element of At to zero
   %
   Step2 = 0;
   InnerTries = 0;
   while (Step2==0)&(NoSolution==0)&(InnerTries<=TotalTries);
      InnerTries = InnerTries+1;
      TestVecA = (At'<0).*(1:T1);
      TestVecA = TestVecA(TestVecA>0);
      if isempty(TestVecA)==0;
         tau0 = min(TestVecA);
         Ttau = setdiff(Ttau,tau0);
         At(tau0) = 0;
         %
         % check if tau0 == tau1 and then if there is no solution, i.e. stopping
         %    condition Step 3 (ii)
         %
         if tau0==tau1;
            Rtilde = (ePG*Xtilde)';
            TestVecR = (Rtilde<LB).*(1:T1);
            TestVecR = TestVecR(TestVecR>0);
            if isempty(TestVecR)==0;
               NoSolution = 1;
            end;
         end;
         %
         if NoSolution==0;
            nt = length(Ttau);
            %
            % (ii) if nt>=1 solve for non-zero elements of At
            %
            if nt>=1;
               ePGX = ePG(Ttau,1:qk);
               ePGA = ePG(Ttau,qk+Ttau);
               if rank(ePGA)==nt;
                  Atau = ePGA\(LB(Ttau)'-(ePGX*Xt));
                  At = Atmp;
                  At(Ttau,1) = Atau;
               else;
                  BadSolution = 1;
               end;
            end;
         end;
      else;
         Step2 = 1;
      end;
      %
      % redo until all At elements are non-negative
      %
   end;
   %
   %
   if (NoSolution==0)&(InnerTries<=TotalTries);
      %
      % Step 3: check the stopping conditions (i)
      %
      TestVecA = (At'>=0).*(1:T1);
      TestVecA = TestVecA(TestVecA==0);
      Xtilde = [Xt;At];
      Rtilde = (ePG*Xtilde)';
      TestVecR = (Rtilde>=LB).*(1:T1);
      TestVecR = TestVecR(TestVecR==0);
      if (isempty(TestVecA)==1)&(isempty(TestVecR)==1);
         %
         % check the complementary slackness condition
         %
         ZLB = Rtilde-LB;
         CompSlack = abs(ZLB*At);
         if CompSlack<=condn;
            status = 1;
            CompSlackProblem = 0;
         else;
            CompSlackProblem = 1;
         end;
      end;
   end;
end;
AlgorithmInformation = [BadSolution NoSolution CompSlackProblem NumTries TotalTries];

%
% end of ForwardBackShootingAlgorithm.m
%
