function [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = BVARModestyStatistics(OutputDirectory,NameOfModel,PriorType,OmegaPriorType,h,NumDraws,NumPredPaths,p)
% BVARModestyStatistics: Computes modesty statistics for a Bayesian VAR.
%
% USAGE:
%
%       [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = BVARModestyStatistics(OutputDirectory,NameOfModel,...
%                                                                   PriorType,OmegaPriorType,h,NumDraws,NumPredPaths,p)
%
% REQUIRED INPUT:  OutputDirectory (string) with the full path to the base output directory.
%
%                  NameOfModel (string) with the name of the model.
%
%                  PriorType (integer) determining the type of prior used for the parameters on lagged endogenous
%                            variables of the Bayesian VAR. The value "1" is the Minnesota-style prior, "2" is the
%                            normal conditional on the covariance matrix prior, and "3" the diffuse prior.
%
%                  OmegaPriorType (integer) determining the type of prior used for the covariance matrix parameters.
%                            The value "1" is a diffuse prior, and "2" and inverse Wishart prior.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  p (integer) with the number of endogenous variables in the BVAR.
%
% REQUIRED OUTPUT: MultiPValue (scalar) with the probability that the multivariate modesty statistic
%                              is smaller than one based on unconditional shocks.
%
%                  MeanMultiModesty (vector) with 2 elements. The first is the mean of the multivariate
%                              modesty statistic based on the conditioning consistent shocks and the second
%                              is the mean of this statistic for randomly drawn shocks.
%
%                  MeanUniModesty (vector) with means of the univariate modesty statistics for all endogenous
%                              variables.
%
%                  MeanUniModestyLZ (vector) with the means of the univariate modesty statistics suggested
%                              by Leeper-Zha (other shocks are zero over examined prediction horizon) for
%                              all endogenous variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 8, 2011.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-03-2011: Updated the documentation.
%
% * 02-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 09-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
TotalPaths = NumPredPaths*NumDraws;
MultiModestyStat = zeros(TotalPaths,2);
UniModestyStat = zeros(TotalPaths,p);
UniModestyStatLZ = zeros(NumDraws,p);
LoadIndex = (1:NumPredPaths);
for ParamIndex=1:NumDraws;
   file = FixFilePath([OutputDirectory '\predictions\BVAR-CondPrediction-Original-' NameOfModel '-' num2str(PriorType,'%0.0f') '.' num2str(OmegaPriorType,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ParamIndex,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.' num2str(NumPredPaths,'%0.0f') '.mat']);
   %
   % load the data
   %
   PredData = load(file);
   MultiModestyStat(LoadIndex,:) = PredData.MultiModestyStat;
   UniModestyStat(LoadIndex,:) = PredData.UniModestyStat;
   UniModestyStatLZ(ParamIndex,:) = PredData.UniModestyStatLZ;
   clear('PredData');
   drawnow;
   LoadIndex = LoadIndex+NumPredPaths;
end;
%
% check how often we have a greater value of column 1 than column 2 in MultiModestyStat
%
MultiPValue = sum(MultiModestyStat(:,1)<=MultiModestyStat(:,2))/TotalPaths;
MeanMultiModesty = (1/TotalPaths)*sum(MultiModestyStat);
MeanUniModesty = (1/TotalPaths)*sum(UniModestyStat);
MeanUniModestyLZ = (1/NumDraws)*sum(UniModestyStatLZ);

%
% end of BVARModestyStatistics.m
%
