function ResFile = PrintBVARModestyStatistics(DSGEModel,CurrINI,MaxPH,NumPeriods,NumDraws,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,ShockMean)
% PrintBVARModestyStatistics: Prints mutlivariate and univariate modesty results from a BVAR model to text file.
%
% USAGE:
%
%       ResFile = PrintBVARModestyStatistics(DSGEModel,CurrINI,MaxPH,NumPeriods,NumDraws,NumPredPaths,MultiPValue,...
%                                            MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,ShockMean)
%
% REQUIRED INPUT:  DSGEModel (structure)  whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  MaxPH (integer) with the maximum prediction horizon.
%
%                  NumPeriods (integer) with the number of conditional predictions.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  MultiPValue (scalar) with the probability that the multivariate modesty statistic
%                            is smaller than one based on unconditional shocks.
%
%                  MeanMultiModesty (vector) with 2 elements. The first is the mean of the multivariate
%                            modesty statistic based on the conditioning consistent shocks and the second
%                            is the mean of this statistic for randomly drawn shocks.
%
%                  MeanUniModesty (vector) with means of the univariate modesty statistics for all observed
%                            variables.
%
%                  MeanUniModestyLZ (vector) with the means of the univariate modesty statistics suggested
%                            by Leeper-Zha (other shocks are zero over examined prediction horizon) for
%                            all observed variables.
%
%                  ShockMean (matrix) of dimension n x MaxPH with the mean values of the shocks under conditioning.
%
% OPTIONAL OUTPUT: ResFile (string vector) with the full path and name of the output file.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 7, 2011.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-03-2011: Removed the "CurrChain" input variable and adapted the code for this. "NumDraws"
%               is used to separate the draws from the posterior distribution from the posterior
%               mode values.
%
% * 23-03-2011: The BVAR only supports the Waggoner-Zha method so that "ShockControlMethod" need
%               not be accounted for when writing to file.
%
% * 02-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 09-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% some useful data parameters.
%
p = length(DSGEModel.BVARY);
k = DSGEModel.BVARLags;
q = length(DSGEModel.BVARX);
%
% results file
%
if NumDraws>1;
   file = FixFilePath([DSGEModel.OutputDirectory '\BVAR-ModestyStatistics-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(DSGEModel.BVARLags) '-' DSGEModel.NameOfModel '-' int2str(MaxPH) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.txt']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\BVAR-ModestyStatistics-' int2str(DSGEModel.PriorType) '.' int2str(DSGEModel.OmegaPriorType) '-' int2str(DSGEModel.BVARLags) '-' DSGEModel.NameOfModel '-' int2str(MaxPH) '.' int2str(NumPredPaths) '.txt']);
end;
fid = fopen(file,'wt');
%
% the header information
%
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                     M O D E S T Y   S T A T I S T I C S                      *\n');
fprintf(fid,'*                             F O R   A   B V A R                              *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% Model info
%
fprintf(fid,'Name of model:                    %s\n',DSGEModel.NameOfModel);
fprintf(fid,'Data construction file:           %s\n',GetFilename(DSGEModel.DataConstructionFile));
fprintf(fid,'Steady state prior file:          %s\n',GetFilename(DSGEModel.SteadyStatePriorFile));
fprintf(fid,'Number of lags of the VAR model:  %0.0f\n',k);
fprintf(fid,'Number of endogenous variables:   %0.0f\n',p);
fprintf(fid,'Number of exogenous variables:    %0.0f\n',q);
if NumDraws>1;
   fprintf(fid,'Number of used parameter draws:   %0.0f\n',NumDraws);
   fprintf(fid,'Prediction paths/parameter draw:  %0.0f\n',NumPredPaths);
   fprintf(fid,'Total number of prediction paths: %0.0f\n\n',NumDraws*NumPredPaths);
else;
   fprintf(fid,'Total number of prediction paths: %0.0f\n\n',NumPredPaths);
end;
%
% the multivariate modesty statistics and tail probability
%
prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
fprintf(fid,'================================================================================\n');
fprintf(fid,['Tail probability for multivariate modesty statistic: ' prt_val '\n'],MultiPValue);
fprintf(fid,'--------------------------------------------------------------------------------\n');
fprintf(fid,['Mean of multivariate modesty statistic:              ' prt_val '\n'],MeanMultiModesty(1));
fprintf(fid,['Mean of multivariate reference statistic:            ' prt_val '\n'],MeanMultiModesty(2));
fprintf(fid,'--------------------------------------------------------------------------------\n');
if exist('YADAchi2cdf')==2;
   fprintf(fid,['chi-square(%2.0f) tail probability at mean:             ' prt_val '\n'],length(DSGEModel.BVARY),1-YADAchi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY)));
else;
   if exist('chi2cdf')==2;
      fprintf(fid,['chi-square(%2.0f) tail probability at mean:             ' prt_val '\n'],length(DSGEModel.BVARY),1-chi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY)));
   else;
      fprintf(fid,['chi-square(%2.0f) tail probability at mean:             ' prt_val '\n'],length(DSGEModel.BVARY),1-YADACheapchi2cdf(MeanMultiModesty(1),length(DSGEModel.BVARY)));
   end;
end;
fprintf(fid,'================================================================================\n\n');
%
% the univariate statistics
%
VariableNames = DSGEModel.VariableNames(DSGEModel.BVARY,:);
if size(DSGEModel.VariableNames,2)<15;
   add_name_str = SpaceStr(15-size(VariableNames,2));
else;
   add_name_str = '';
end;
fprintf(fid,'===================================================\n');
fprintf(fid,'           Univariate Modesty Statistics\n');
fprintf(fid,' Variable            ALLV-statistic   LZ-statistic\n');
fprintf(fid,'---------------------------------------------------\n');
for i=1:length(MeanUniModesty);
   fprintf(fid,[' %s%s     ' prt_val '     ' prt_val '\n'],VariableNames(i,:),add_name_str,MeanUniModesty(i),MeanUniModestyLZ(i));
end;
fprintf(fid,'===================================================\n\n');
fprintf(fid,'NOTE: The ALLV-statistic is the univariate modesty\n      statistic suggested by Adolfson, Lasen, Lind\n      and Villani (2005) in International Finance,\n      and modified by Warne (2011) to handle the\n      Waggoner-Zha shock distribution. The LZ-   \n      statistic is the Leeper-Zha modesty statistic\n      using the mean shock values from the Waggoner-\n      Zha shock distribution under zero measurement\n      errors and mean initial state vector for the   \n      predictions. The reference distribution\n      for these statistics is the standard normal.\n\n');
%
% check if we have data for printing the mean shock values
%
ShockNames = VariableNames;
if isempty(ShockMean)==0;
   q = size(ShockMean,1);
   if size(ShockNames,2)<20;
      AddNameStr = SpaceStr(20-size(ShockNames,2));
   else;
      AddNameStr = '';
   end;
   HeadStr = 'Residual in equation';
   PrtStr = '%s%s';
   for i=1:MaxPH;
      if (IsInteger(i/6)==0)|(i==MaxPH);
         HeadStr = [HeadStr sprintf('        %2.0f     ',i)];
         PrtStr = [PrtStr '   ' prt_val];
      else;
         HeadStr = [HeadStr sprintf('        %2.0f     \n                    ',i)];
         PrtStr = [PrtStr '   ' prt_val '\n                    '];
      end;
   end;
   TopStr = MultiCharStr('=',min(111,length(HeadStr)+1));
   DivStr = MultiCharStr('-',min(111,length(HeadStr)+1));
   fprintf(fid,'%s\n',TopStr);
   if NumDraws>1;
      fprintf(fid,'                    Mean value of residuals at mean prediction under conditioning\n');
      fprintf(fid,'                    using parameter draws from the posterior distribution.\n\n');
   else;
      fprintf(fid,'                    Value of residuals at mean prediction under conditioning\n\n');
   end;
   fprintf(fid,'                            Prediction Horizon\n');
   fprintf(fid,'%s\n',HeadStr);
   fprintf(fid,'%s\n',TopStr);
   for i=1:q;
      fprintf(fid,[PrtStr ' \n'],AddNameStr,ShockNames(i,:),ShockMean(i,:)');
      if i<q;
         fprintf(fid,'%s\n',DivStr);
      else;
         fprintf(fid,'%s\n\n',TopStr);
      end;
   end;
end;
%
% we're all done now
%
fprintf(fid,'Directory for file: %s\n',GetPath(file));
fprintf(fid,'Name of file:       %s\n',GetFilename(file));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);
%
if nargout==1;
   ResFile = file;
end;

%
% end of PrintBVARModestyStatistics.m
%
