function [status,NumPeriods] = CheckStateCondData(DSGEModel)
% CheckStateCondData: Checks if the conditioning data and the actual data make it possible
%                     to calculate conditional forecasts of the observed variables using the
%                     state variable assumptions.
%
% USAGE:
%
%       [status,NumPeriods] = CheckStateCondData(DSGEModel)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
% REQUIRED OUTPUT: status (boolean) that is 1 if conditional forecasts can be computed.
%                            and 0 otherwise.
%
% OPTIONAL OUTPUT: NumPeriods (integer) with the number of periods over which conditional
%                            forecasts can be made.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: July 14, 2009.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-07-2009: Updated the documentation.
%
% * 19-08-2009: The function now checks if the tools for conditional forecasting with
%               state variable assumptions are available.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 17-07-2013: The function no longer checks if "CondStateControls.m" exists in the "addons\sc" directory.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 09-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
status = 0;
if nargout==2;
   NumPeriods = 0;
end;
if (isempty(DSGEModel.Zeta)==1)|(isempty(DSGEModel.ZetaVariableNames)==1)|(isempty(DSGEModel.K3)==1);
   return;
end;
%
% check sample size
%
Tz = size(DSGEModel.Zeta,2);
Tx = size(DSGEModel.PredictedX,2);
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
if (Tx+(DSGEModel.T-LastPeriod)==0)|(Tz<=LastPeriod);
   return;
end;
status = 1;
if nargout==2;
   NumPeriods = min([Tx+(DSGEModel.T-LastPeriod) Tz-LastPeriod]);
end;

%
% end of CheckStateCondData.m
%
