function ParameterBlock = GetBlockingParameterPositions(BlockDraws,ParamPerBlock,NumBlocks)
% GetBlockingParameterPositions: Determines the indices of the parameters in the different blocks
%                                when using fixed or random blocking algorithms for posterior
%                                sampling.
%
% USAGE:
%
%       ParameterBlock = GetBlockingParameterPositions(BlockDraws,ParamPerBlock,NumBlocks)
%
% REQUIRED INPUT:  BlockDraws (vector) of dimension NumParamx1 with uniform random numbers used
%                             to determine which block each parameter belongs to. The lowest
%                             numbers go to the first block and the highest to the last block.
%
%                  ParamPerBlock (integer) the number of parameters to include in all blocks
%                             except for the last which takes the remaining parameters.
%
%                  NumBlocks (integer) with the number of parameter blocks.
%
% REQUIRED OUTPUT: ParameterBlock (cell array) of length NumBlocks, where each entry is a vector
%                             with indexes for the parameters that belong to each block.
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 5, 2017.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-04-2017: Updated the documentation.
%
% * 11-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
ParameterBlock = [];
%
[SortedBD,IndexBD] = sort(BlockDraws);
%
for i=1:NumBlocks;
   if i<NumBlocks;
      ParameterBlock{i} = IndexBD(1+(ParamPerBlock*(i-1)):ParamPerBlock*i,1);
   else;
      ParameterBlock{NumBlocks} = IndexBD(1+(ParamPerBlock*(NumBlocks-1)):length(BlockDraws),1);
   end;
end;

%
% end of GetBlockingParameterPositions.m
%
