function [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = GetUpdatePriorDensity(DSGEModel,CurrINI,controls,CurrentChain,NumChains,NumBurnin,NumIter)
% GetUpdatePriorDensity: Tries to estimate prior densities for the parameters that are a function of
%                        the estimated parameters by drawing from the prior distribution of the latter
%                        parameters.
%
% USAGE:
%
%       [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = GetUpdatePriorDensity(DSGEModel,CurrINI,controls,CurrentChain,NumChains,NumBurnin,NumIter)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the model,
%                            such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  CurrentChain (integer) with the number of the currently used MCMC draws.
%
%                  NumChains (integer) with the total number of MCMC chains that we can choose from.
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  NumIter (integer) with the total number of posterior draws that would have
%                            been draws based on the user settings.
%
% REQUIRED OUTPUT: UpdatePriorX (matrix) of dimension NumAdditional x NumValues with the
%                            values on the X-axis for the prior densities of the
%                            NumAdditional additional parameters.
%
%                  UpdatePriorY (matrix) of dimension NumAdditional x NumValues with the
%                            density values for the Y-axis of the prior densities of
%                            the NumAdditional additional parameters.
%
%                  UpdatePriorBounds (matrix) of dimension NumAdditional x 2 with suitable
%                            lower and upper bounds for the additional parameters.
%
%                  UpdateParameters (vector) with the entries of the updated parameters that actually
%                            are a function of the estimated parameters.
%
% OPTIONAL OUTPUT: UpdateInitialValue (vector) with the initial values of the additional
%                            parameters.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 27, 2009.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-03-2009: Renamed the output variables such that the name includes "Prior".
%
% * 09-03-2009: Added quotation marks around paths for the dos copy commands.
%
% * 13-03-2009: Added support for the Student-t, the Cauchy, the logistic, and the
%               Pareto prior distributions.
%
% * 20-03-2009: Added code for initializing the random number generators based on
%               user settings.
%
% * 31-03-2009: Draws greater than or less than 6 times the scale factor relative to
%               the location parameter are now discarded for the Student-t and Cauchy
%               distributions. Made sure that the origin parameter of the Pareto
%               distribution is taken into accound.
%
% * 02-04-2009: Took into account that "LogisticRndFcn" now supports a shape parameter.
%               Added code for the Gumbel distribution.
%
% * 23-09-2009: Fixed a bug where the BoundsMatrix was attempted to be constructed more than once
%               when "NumAdditional" is greater than 1. Since the indices "NumBeta", etc. were
%               not reset for each time the BoundsMatrix was created, the indices would turn
%               greater than the number of parameters having a particular prior. The code now
%               only creates this matrix once.
%
% * 28-09-2009: The function can now handle the case when "NumChains" is zero, reflecting that the
%               prior won't be plotted along with the posterior.
%
% * 23-12-2009: The function now uses try-catch code around the call to the kernel density estimation
%               "KernelDensityPlot".
%
% * 05-01-2010: Updated the documentation.
%
% * 17-03-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed the dos copy call to "CopyFile" calls.
%
% * 11-01-2011: Added support for unix and mac files.
%
% * 07-02-2011: Made sure that "ModeFile" is defined for all cases.
%
% * 08-02-2011: The function now only computes prior densities for parameters that actuallt are a function
%               of the estimated parameters. Added the output variable "UpdateParameters". Made sure that
%               the output directory exists when saving the prior draws to disk.
%
% * 10-02-2011: The "UpdateParameters" variable is now also influenced by the results from the kernel
%               density estimates.
%
% * 11-02-2011: The user can now optionally reuse previously computed prior densities for the update parameters
%               and, hence, optionally recompute them.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 12-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function now supprts the fixed blocking RWM sampler with a normal proposal
%               density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM sampler with a Student-t poproposal
%               density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 19-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 11-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
% * 11-01-2018: Updated the documentation.
%
% * 14-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 16-10-2018: The function now supports the SMC with data tempering posterior sampler.
%
% * 25-10-2018: Made sure that the name of the prior draws file reflects using a system prior.
%
% * 03-12-2018: The function should now support IS based on MitISEM.
%
% * 04-12-2018: Made sure that NumBurnin is removed from AddFile for IS.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed eval calls to feval.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
UpdatePriorX = [];
UpdatePriorY = [];
UpdatePriorBounds = [];
UpdateParameters = [];
if nargout==5;
   UpdateInitialValue = [];
end;
%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
% first we check if the prior densities have already been calculated and saved to file
%
if NumChains>0;
   if DSGEModel.PosteriorSampler==1;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==2;
      stat = MakeDir(DSGEModel.OutputDirectory,'slice');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Slice-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==3;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==4;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==5;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==6;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==7;
      stat = MakeDir(DSGEModel.OutputDirectory,'rwm');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\rwm\Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-Additional-t-' int2str(DSGEModel.StudenttDegFree) '-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumBurnin,'%0.0f') '.' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==8;
      stat = MakeDir(DSGEModel.OutputDirectory,'smc');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-Additional-' DSGEModel.NameOfModel '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==9;
      stat = MakeDir(DSGEModel.OutputDirectory,'smc');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\smc\SMC-Additional-' DSGEModel.NameOfModel '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
   elseif DSGEModel.PosteriorSampler==10;
      stat = MakeDir(DSGEModel.OutputDirectory,'is');
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\is\IS-Additional-' DSGEModel.NameOfModel '-' num2str(DSGEModel.InverseHessianEstimator,'%0.0f') '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF) '-' num2str(CurrentChain,'%0.0f') '.' num2str(NumChains,'%0.0f') '-' num2str(NumIter,'%0.0f') '.mat']);
   end;
else;
   stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
   if FileExist(DSGEModel.SystemPriorFile)==0;
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-Additional-' DSGEModel.NameOfModel '.mat']);
   else;
      AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-Additional-' DSGEModel.NameOfModel '.mat']);
   end;
end;
if FileExist(AddFile)==1;
   %
   % check if we can load the data
   % 
   AddData = load(AddFile);
   if isfield(AddData,'UpdatePriorX')==1;
      %
      % ask if we should load prior draws from disk
      %
      nup = size(AddData.UpdatePriorX,1);
      if nup>1;
         txt = ['The prior distributions for the ' int2str(nup) ' update parameters have already been calculated. Would you like to reuse these distributions?'];
      else;
         txt = ['The prior distribution for the single update parameter has already been calculated. Would you like to reuse this distribution?'];
      end;
      answer = Query(txt,'question',200,'Prior Densities - Update Parameters',500,'yes',CurrINI);
      if strcmp(lower(answer),'yes')==1;
         UpdatePriorX = AddData.UpdatePriorX;
         UpdatePriorY = AddData.UpdatePriorY;
         UpdatePriorBounds = AddData.UpdatePriorBounds;
         if isfield(AddData,'UpdateParameters')==1;
            UpdateParameters = AddData.UpdateParameters;
         else;
            UpdateParameters = (1:size(UpdatePriorX,1))
         end;
         if (nargout==5)&(isfield(AddData,'UpdateInitialValue')==1);
            UpdateInitialValue = AddData.UpdateInitialValue;
         end;
         return;
      end;
   end;
end;
%
% check if we can load the posterior mode data
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
if NumChains>0;
   if FileExist(ModeFile)==1;
      ModeData = load(ModeFile);
      theta = ModeData.theta;
      thetaDist = ModeData.thetaDist;
      LowerBound = ModeData.LowerBound;
      PriorDist = ModeData.PriorDist;
      ParameterNames = ModeData.ParameterNames;
      thetaPositions = ModeData.thetaPositions;
      ModelParameters = ModeData.ModelParameters;
   else;
      [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI)
      if InitStatus==0;
         return;
      end;
      %
      % check if we have a file with parameters to initialize
      %
      if FileExist(DSGEModel.InitializeParameterFile)==1;
         [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
         if DSGEModel.RunInitializeFirst==1;
            try;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            catch;
               return;
            end;
         end;
      end;
   end;
else;
   %
   % collect input from CurrINI
   %
   theta = CurrINI.theta;
   thetaDist = CurrINI.thetaDist;
   LowerBound = CurrINI.LowerBound;
   PriorDist = CurrINI.PriorDist;
   ParameterNames = CurrINI.ParameterNames;
   thetaPositions = CurrINI.thetaPositions;
   ModelParameters = CurrINI.ModelParameters;
   if FileExist(DSGEModel.InitializeParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
      if DSGEModel.RunInitializeFirst==1;
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            catch ME;
               ErrorStr = ['There was en error run the file with parameters to initialize. YADA received the following message: ' YADALastError(ME)];
            end;
         else;
            try;
               ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
            catch;
               ErrorStr = ['There was en error run the file with parameters to initialize. YADA received the following message: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==0;
            About(ErrorStr,'error','YADA - Error',500,600,CurrINI);
            return;
         end;
      end;
   end;
end;
%
% copy update parameter file to tmp folder
%
[stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
%
% make sure the ModelParameters structure is update with the initial values of the
% estimated parameters
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% run the update parameter function
%
ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
%
% check if we should run the initialize parameter function
%
if (FileExist(DSGEModel.InitializeParameterFile)==1)&((FileExist(ModeFile)==0)|(NumChains==0))&(DSGEModel.RunInitializeFirst==0);
   try;
      ModelParameters = feval(GetName(DSGEModel.InitializeParameterFile),ModelParameters);
   catch;
      return;
   end;
end;
%
% the ModelParameter structure now only has values that depend on the initial parameter values.
% determine the values for the additional parameters
%
NumParam = length(theta);
NumAdditional = size(ParameterNames.additional,1);
UpdateInitial = zeros(NumAdditional,1);
for j=1:NumAdditional;
   UpdateInitial(j,1) = getfield(ModelParameters,StringTrim(ParameterNames.additional(j,:)));
end;
%
% try to figure out which estimated parameters affect the update parameters
%
UseParameters = zeros(NumParam,1);
BoundsMatrix = zeros(NumParam,2);
NumBeta = 0;
NumLTN = 0;
NumPareto = 0;
NumUniform = 0;
for j=1:NumAdditional;
   for i=1:NumParam;
      %
      % we only need to create the BoundsMatrix once
      %
      if j==1;
         if thetaDist(i)==1;
            BoundsMatrix(i,:) = [LowerBound(i,1) Inf];
         elseif thetaDist(i)==2;
            NumBeta = NumBeta+1;
            if size(PriorDist.beta,2)==4;
               BoundsMatrix(i,:) = [PriorDist.beta(NumBeta,3) PriorDist.beta(NumBeta,4)];
            else;
               BoundsMatrix(i,:) = [0 1];
            end;
         elseif thetaDist(i)==3;
            BoundsMatrix(i,:) = [-Inf Inf];
         elseif thetaDist(i)==4;
            BoundsMatrix(i,:) = [LowerBound(i,1) Inf];
         elseif thetaDist(i)==5;
            NumLTN = NumLTN+1;
            BoundsMatrix(i,:) = [PriorDist.beta(NumLTN,3) Inf];
         elseif thetaDist(i)==6;
            NumUniform = NumUniform+1;
            BoundsMatrix(i,:) = [PriorDist.beta(NumUniform,1) PriorDist.beta(NumUniform,2)];
         elseif thetaDist(i)==7;
            BoundsMatrix(i,:) = [-Inf Inf];
         elseif thetaDist(i)==8;
            BoundsMatrix(i,:) = [-Inf Inf];
         elseif thetaDist(i)==9;
            BoundsMatrix(i,:) = [-Inf Inf];
         elseif thetaDist(i)==10;
            BoundsMatrix(i,:) = [-Inf Inf];
         elseif thetaDist(i)==11;
            NumPareto = NumPareto+1;
            BoundsMatrix(i,:) = [PriorDist.pareto(NumPareto,2) Inf];
         end;
      end;
      thetaNew = theta;
      thetaBounds = sort([0.9*thetaNew(i) 1.1*thetaNew(i)]);
      if thetaBounds(2)<=BoundsMatrix(i,2);
         thetaNew(i) = thetaBounds(2);
      else;
         thetaNew(i) = thetaBounds(1);
      end;
      %
      % update the Model Parameters structure with the thetaNew
      %
      ModelParameters = ThetaToModelParameters(ModelParameters,thetaNew,thetaPositions);
      %
      % run the update parameter function
      %
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
      %
      % check if the initial parameters have changed
      %
      UpdateNew = zeros(NumAdditional,1);
      for j1=1:NumAdditional;
         UpdateNew(j1,1) = getfield(ModelParameters,StringTrim(ParameterNames.additional(j1,:)));
      end;
      if isequal(UpdateNew,UpdateInitial)==0;
         UseParameters(i,1) = i;
      end;
   end;
end;
%
% get only the nonzero values
%
UseParameters = UseParameters(UseParameters>0);
if isempty(UseParameters)==1;
   %
   % the update parameters are all constant
   %
   return;
end;
%
% draw from the prior distributions of the parameters to use
%
if DSGEModel.RandomNumberValue==1;
   %
   % intialize random number generators based on fixed state/seed
   %
   rand('state',0);
   randn('state',0);
else;
   %
   % initialize random number generators based on clock based state/seed
   %
   rand('state',sum(100*clock));
   randn('state',sum(100*clock));
end;
UseParam = 1;
NumGamma = 0;
NumBeta = 0;
NumNormal = 0;
NumInvGamma = 0;
NumLTN = 0;
NumUniform = 0;
NumStudent = 0;
NumCauchy = 0;
NumLogistic = 0;
NumGumbel = 0;
NumPareto = 0;
thetaDraws = theta*ones(1,5000);
for i=1:NumParam;
   if thetaDist(i)==1;
      %
      % gamma dist
      %
      NumGamma = NumGamma+1;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = LowerBound(i)+GammaRndFcn(PriorDist.gamma_ab(NumGamma,1),PriorDist.gamma_ab(NumGamma,2),size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==2;
      %
      % beta dist
      %
      NumBeta = NumBeta+1;
      if size(PriorDist.beta,2)==4;
         %
         % get the lower and upper bounds (if available from PriorDist.)
         %
         c = PriorDist.beta(NumBeta,3);
         d = PriorDist.beta(NumBeta,4);
      else;
         c = 0;
         d = 1;
      end;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = BetaRndFcn(PriorDist.beta_ab(NumBeta,1),PriorDist.beta_ab(NumBeta,2),c,d,size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==3;
      %
      % normal
      %
      NumNormal = NumNormal+1;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = NormalRndFcn(PriorDist.normal(NumNormal,1),PriorDist.normal(NumNormal,2),size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==4;
      %
      % inverted gamma
      %
      NumInvGamma = NumInvGamma+1;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = LowerBound(i)+InvGammaRndFcn(PriorDist.invgamma(NumInvGamma,1)-LowerBound(i),PriorDist.invgamma(NumInvGamma,2),size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==5;
      %
      % left truncated normal
      %
      NumLTN = NumLTN+1;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = LTNormalRndFcn(PriorDist.truncnormal(NumLTN,1),PriorDist.truncnormal(NumLTN,2),PriorDist.truncnormal(NumLTN,3),size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==6;
      %
      % uniform
      %
      NumUniform = NumUniform+1;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = UniformRndFcn(PriorDist.uniform(NumUniform,1),PriorDist.uniform(NumUniform,2),size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==7;
      %
      % Student-t
      %
      NumStudent = NumStudent+1;
      if (UseParameters(UseParam)==i);
         Numz = 0;
         Oldz = [];
         lb = PriorDist.student(NumStudent,1)-(6*PriorDist.student(NumStudent,2));
         ub = PriorDist.student(NumStudent,1)+(6*PriorDist.student(NumStudent,2));
         while Numz<size(thetaDraws,2);
            z = StudentTAltRndFcn(PriorDist.student(NumStudent,1),PriorDist.student(NumStudent,2),PriorDist.student(NumStudent,3),2*size(thetaDraws,2));
            %
            % remove value of z that are too big or too small
            %
            if sum(z>=lb)<length(z);
               z = z(z>=lb);
            end;
            if sum(z<=ub)<length(z);
               z = z(z<=ub);
            end;
            Oldz = [Oldz z];
            z = Oldz;
            if length(z)>size(thetaDraws,2);
               z = z(1:size(thetaDraws,2));
            end;
            Numz = length(z);
         end;
         thetaDraws(i,:) = z;
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==8;
      %
      % Cauchy
      %
      NumCauchy = NumCauchy+1;
      if (UseParameters(UseParam)==i);
         Numz = 0;
         Oldz = [];
         lb = PriorDist.cauchy(NumCauchy,1)-(6*PriorDist.cauchy(NumCauchy,2));
         ub = PriorDist.cauchy(NumCauchy,1)+(6*PriorDist.cauchy(NumCauchy,2));
         while Numz<size(thetaDraws,2);
            z = CauchyRndFcn(PriorDist.cauchy(NumCauchy,1),PriorDist.cauchy(NumCauchy,2),2*size(thetaDraws,2));
            %
            % remove value of z that are too big or too small
            %
            if sum(z>=lb)<length(z);
               z = z(z>=lb);
            end;
            if sum(z<=ub)<length(z);
               z = z(z<=ub);
            end;
            Oldz = [Oldz z];
            z = Oldz;
            if length(z)>size(thetaDraws,2);
               z = z(1:size(thetaDraws,2));
            end;
            Numz = length(z);
         end;
         thetaDraws(i,:) = z;
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==9;
      %
      % Logistic
      %
      NumLogistic = NumLogistic+1;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = LogisticRndFcn(PriorDist.logistic_ab(NumLogistic,1),PriorDist.logistic_ab(NumLogistic,2),PriorDist.logistic_ab(NumLogistic,3),size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==10;
      %
      % Gumbel
      %
      NumGumbel = NumGumbel+1;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = GumbelRndFcn(PriorDist.gumbel_ab(NumLogistic,1),PriorDist.gumbel_ab(NumLogistic,2),size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   elseif thetaDist(i)==11;
      %
      % Pareto
      %
      NumPareto = NumPareto+1;
      if (UseParameters(UseParam)==i);
         thetaDraws(i,:) = PriorDist.pareto(NumPareto,3)+ParetoRndFcn(PriorDist.pareto(NumPareto,1),PriorDist.pareto(NumPareto,2),size(thetaDraws,2));
         UseParam = UseParam+1;
      end;
   end;
   drawnow;
   %
   % check if we're done
   %
   if UseParam>length(UseParameters);
      break;
   end;
end;
%
% we now turn to computing the values of the additional parameters under the draws from the prior
% of the estimated parameters
%
UpdateDraws = zeros(NumAdditional,size(thetaDraws,2));
%
for i=1:size(thetaDraws,2);
   %
   % update ModelParameters
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,thetaDraws(:,i),thetaPositions);
   %
   % run the update parameters function
   %
   ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   %
   % collect the update parameter values
   %
   for j=1:NumAdditional;
      UpdateDraws(j,i) = getfield(ModelParameters,StringTrim(ParameterNames.additional(j,:)));
   end;
   drawnow;
end;
%
% check if we should remove any parameters
%
UpdateParameters = (abs(max(UpdateDraws')-min(UpdateDraws'))>1.0e-12).*(1:NumAdditional);
UpdateParameters = UpdateParameters(UpdateParameters>0);
UpdateDraws = UpdateDraws(UpdateParameters,:);
NumAdditional = length(UpdateParameters);
UpdateInitial = UpdateInitial(UpdateParameters);
%
% determine upper and lower bounds for the update parameters
%
LowerB = floor(min(UpdateDraws'));
UpperB = ceil(max(UpdateDraws'));
if size(LowerB,2)>1
   LowerB = LowerB';
end;
if size(UpperB,2)>1;
   UpperB = UpperB';
end;
UpdatePriorBounds = [LowerB UpperB];
%
% we may now turn to the Kernel density estimation
%
ErrorMsg = '';
for i=1:NumAdditional;
   if MatlabNumber>=8.0;
      try;
         [XValues,YValues] = KernelDensityPlot(UpdateDraws(i,:),DSGEModel.KernelDensityEstimator);
      catch ME;
         XValues = NaN*ones(1,512);
         YValues = NaN*ones(1,512);
         ErrorMsg = ['YADA was unable to estimate the prior density of the parameter "' StringTrim(CurrINI.UpdateNames(i,:)) '". The following error message was caught: ' YADALastError(ME)];
         About(ErrorMsg,'information','Prior Densities - Update Parameters',500,500,CurrINI);
      end;
   else;
      try;
         [XValues,YValues] = KernelDensityPlot(UpdateDraws(i,:),DSGEModel.KernelDensityEstimator);
      catch;
         XValues = NaN*ones(1,512);
         YValues = NaN*ones(1,512);
         ErrorMsg = ['YADA was unable to estimate the prior density of the parameter "' StringTrim(CurrINI.UpdateNames(i,:)) '". The following error message was caught: ' YADALastError];
         About(ErrorMsg,'information','Prior Densities - Update Parameters',500,500,CurrINI);
      end;
   end;
   if i==1;
      UpdatePriorX = zeros(NumAdditional,length(XValues));
      UpdatePriorY = UpdatePriorX;
   end;
   UpdatePriorX(i,:) = XValues;
   UpdatePriorY(i,:) = YValues;
   drawnow;
end;
%
% check if some of the update parameters slipped thorugh the previous test and may be regarded as
% fixed when seen through the lense of the kernel density estimates
%
VarParam = ((abs(max(UpdatePriorX')-min(UpdatePriorX'))>1.0e-12)&(abs(max(UpdatePriorY'))<1.0e+12)).*(1:NumAdditional);
VarParam = VarParam(VarParam>0);
UpdateParameters = UpdateParameters(VarParam);
NumAdditional = length(UpdateParameters);
UpdateInitial = UpdateInitial(VarParam);
UpdatePriorX = UpdatePriorX(VarParam,:);
UpdatePriorY = UpdatePriorY(VarParam,:);
UpdatePriorBounds = UpdatePriorBounds(VarParam,:);
%
% save the results to file so that they can be quickly retrieved
%
if nargout==5;
   UpdateInitialValue = UpdateInitial;
end;
if isempty(ErrorMsg)==1;
   if NumChains>0;
      save(AddFile,'UpdatePriorX','UpdatePriorY','UpdatePriorBounds','UpdateParameters','-APPEND');
   else;
      save(AddFile,'UpdatePriorX','UpdatePriorY','UpdatePriorBounds','UpdateParameters','UpdateInitialValue');
   end;
end;

%
% GetUpdatePriorDensity.m
%
