function [PredEventData,PredictionEvent,ObsVarMean] = LoadZLBDSGEPredictionData(OutputDirectory,ModelStr,CondStr,ShockMethod,NameOfModel,ZLBSampleT,RtildePosition,REqPosition,ZLBRuleType,h,NumDraws,NumPredPaths,AnnualizeData,TransData,CurrChain,IsPosterior,IsCentered,YDataStr)
% LoadZLBDSGEPredictionData: Loads the prediction event data from file
%
% USAGE:
%
%       [PredEventData,PredictionEvent,ObsVarMean] = LoadZLBDSGEPredictionData(OutputDirectory,ModelStr,CondStr,...
%                                                        ShockMethod,NameOfModel,ZLBSampleT,RtildePosition,...
%                                                        REqPosition,ZLBRuleType,h,NumDraws,NumPredPaths,...
%                                                        AnnualizeData,TransData,CurrChain,IsPosterior,IsCentered,...
%                                                        YDataStr)
%
% REQUIRED INPUT:  OutputDirectory (string vector) with the full path to the base output directory.
%
%                  ModelStr (string vector) that is either "DSGE" or "AdaptiveLearningDSGE-x".
%
%                  CondStr (string vector) that is either "Cond" or "".
%
%                  ShockMethod (string vector) that is either "Direct-", "WaggonerZha-", "Subset-" or "".
%
%                  NameOfModel (string vector) with the name of the model.
%
%                  ZLBSampleT (integer)  with the length of the sample over which the zero lower
%                             bound may be binding.
%
%                  RtildePosition (integer) with the position of the monetary policy interest rate.
%
%                  REqPosition (integer) with the position of the monetary policy rule.
%
%                  ZLBRuleType (integer) which is either 1 for lagged unrestricted policy rate (the
%                             default) or 2 for lagged restricted policy rate.
%
%                  h (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  AnnualizeData (boolean) that take the value 1 if annualized results should be checked and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                             if draws from the prior are supplied.
%
%                  IsCentered (boolean) that takes the value 1 if the distributions not subject to the lower
%                             bound have been centered on the observed data and 0 otherwise.
%
%                  YDataStr (string) reflecting which data for the observed variables has been used. Is an
%                             empty string when IsCentered is 0.
%
% REQUIRED OUTPUT: PredEventData (n x 8) matrix where the first column holds the number of times
%                             the PredPaths data satisfies the event, the second holds the number
%                             of times the PredPaths data is greater than the event, the third
%                             the number of time the data is above the upper bound, while the
%                             fourth column holds the number of times the event has been tested.
%                             The fifth and sixth column measure the average deviation from the
%                             lower bound when below the lower bound and the square of this, while
%                             the seventh and eighth columns hold similar data for the upper bound
%                             of the event when the variables are above the upper bound.
%
%                  PredictionEvent (n x 3) matrix with the prediction event information. The first
%                             column holds the upper bound, the second the lower bound for the event,
%                             while the third holds the event length in periods.
%
%                  ObsVarMean (n x 1) vector with the steady-state values of the observed variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: September 1, 2022.
%                        This version: March 5, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-09-2022: Added the "ObsVarMean" output variables.
%
% * 08-09-2022: Added the input variable "YDataStr".
%
% * 06-01-2023: Updated the documentation.
%
% * 05-03-2024: Added the missing input variable "ZLBRuleType".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
PredEventData = [];
PredictionEvent = [];
ObsVarMean = [];
if IsPosterior==1;
   if AnnualizeData==0;
      if TransData==0;
         file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      else;
         file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      end;
   else;
      file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
   end;
else;
   if AnnualizeData==0;
      if TransData==0;
         file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Original-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      else;
         file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Transformed-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
      end;
   else;
      file = FixFilePath([OutputDirectory '\predictions\' ModelStr '-ZLB-' CondStr 'Prediction-Annual-Prior-' int2str(IsCentered) YDataStr '-' ShockMethod NameOfModel '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(h) '-' int2str(NumDraws) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
   end;
end;
if FileExist(file)==1;
   PredData = load(file);
   if isfield(PredData,'PredEventData')==1;
      if size(PredData.PredEventData,2)==8;
         PredEventData = PredData.PredEventData;
      end;
   end;
   if isfield(PredData,'PredictionEvent')==1;
      PredictionEvent = PredData.PredictionEvent;
   end;
   if isfield(PredData,'ObsVarMean')==1;
      ObsVarMean = PredData.ObsVarMean;
   end;
end;

%
% end of LoadZLBDSGEPredictionData.m
%
