function SelectDraws = SampledPosteriorDraws(DSGEModel,CurrINI,WindowNameStr,Weights,NumDraws,UseDraws)
% SampledPosteriorDraws: Determines which of the posterior draws to use. The function is intended for
%                        SMC and IS based posterior samplers.
%
% USAGE:
%
%       SelectDraws = SampledPosteriorDraws(DSGEModel,CurrINI,WindowNameStr,Weights,NumDraws,UseDraws)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (strcuture) with initialization information.
%
%                  WindowNameStr (string vector) with the name to use for the selection
%                            dialog.
%
%                  Weights (vector) of dimension NumDraws x 1 with the weights used for
%                            the posterior sampler.
%
%                  NumDraws (integer) with the total number of available posterior draws.
%
%                  UseDraws (integer) with the number of draws to use.
%
% REQUIRED OUTPUT: SelectDraws (vector) with the selected draw positions.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: May 18, 2022.
%                        This version: March 16, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-05-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 16-03-2023: Made sure that 0 values are avoided and that the maximum values is possible.
%               Furthermore, if the selected number of draws is equal to the available number,
%               then all draws are selected.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
SelectDraws = (1:NumDraws);
if isempty(UseDraws)==1;
   SelectionOptions = strvcat('100','200','300','400','500','600','700','800','900','1,000','1,500','2,000','2,500','3,000','3,500','4,000','4,500','5,000','6,000','7,000','8,000','9,000','10,000','11,000','12,000','13,000','14,000','15,000','16,000','17,000','18,000','19,000','20,000','25,000','30,000','35,000','40,000','45,000','50,000','55,000','60,000','65,000','70,000','75,000','80,000','85,000','90,000','95,000','100,000','105,000','110,000','115,000','120,000','125,000','130,000','135,000','140,000','145,000','150,000','155,000','160,000','165,000','170,000','175,000','180,000','185,000','190,000','195,000','200,000','205,000','210,000','215,000','220,000','225,000','230,000','235,000','240,000','245,000','250,000','255,000','260,000','265,000','270,000','275,000','280,000','285,000','290,000','295,000','300,000','305,000','310,000','315,000','320,000','325,000','330,000','335,000','340,000','345,000','350,000','355,000','360,000','365,000','370,000','375,000','380,000','385,000','390,000','395,000','400,000','405,000','410,000','415,000','420,000','425,000','430,000','435,000','440,000','445,000','450,000','455,000','460,000','465,000','470,000','475,000','480,000','485,000','490,000','495,000','500,000','510,000','520,000','525,000','530,000','540,000','550,000','560,000','570,000','575,000','580,000','590,000','600,000','610,000','620,000','625,000','630,000','640,000','650,000','660,000','670,000','675,000','680,000','690,000','700,000','710,000','720,000','725,000','730,000','740,000','750,000','760,000','770,000','775,000','780,000','790,000','800,000','810,000','820,000','825,000','830,000','840,000','850,000','860,000','870,000','875,000','880,000','890,000','900,000','910,000','920,000','925,000','930,000','940,000','950,000','960,000','970,000','975,000','980,000','990,000','1,000,000');
   %
   % remove all rows which imply more draws than NumDraws
   %
   MaxOpt = 0;
   for i=1:size(SelectionOptions);
      ThisOpt = str2num(strrep(StringTrim(SelectionOptions(i,:)),',',''));
      if NumDraws==ThisOpt;
         MaxOpt = i;
         break;
      elseif NumDraws<ThisOpt;
         MaxOpt = i-1;
         break;
      end;
   end;
   DefaultOpt = 1;
   if MaxOpt>0;
      SelectionOptions = SelectionOptions(1:MaxOpt,:);
      DefaultOpt = MaxOpt;
   end;
   %
   % display a selection dialog
   %
   [action,SelectedOption] = SelectionDlg(SelectionOptions,DefaultOpt,'Select the number of posterior draws to use:','',WindowNameStr,'','',CurrINI);
   if strcmp(lower(action),'cancel')==1;
      SelectDraws = [];
      return;
   end;
   UseDraws = str2num(strrep(StringTrim(SelectionOptions(SelectedOption,:)),',',''));
end;
%
if NumDraws==UseDraws;
   SelectDraws = (1:UseDraws);
   return;
end;
%
if DSGEModel.PosteriorSampler==9;
   %
   % we have draws from the SMC with data tempering sampler
   %
   if DSGEModel.RandomizeDraws==0;
      DrawFreq = ceil(NumDraws/NumPostDrawsUsage);
      SelectDraws = (1:DrawFreq:NumDraws);
      if length(SelectDraws)+1==NumPostDrawsUsage;
         SelectDraws = [SelectDraws NumDraws];
      elseif length(SelectDraws)>NumPostDrawsUsage;
         SelectDraws = SelectDraws(1:NumPostDrawsUsage);
      end;
   else;
      %
      % initialize the random number generator
      %
      if DSGEModel.RandomNumberValue==1;
         rand('state',0);
      else;
         rand('state',sum(100*clock));
      end;
      SelectDraws = sort(YADArandi(1,NumDraws,UseDraws));
   end;
else;
   %
   % we have draws from the SMC with likelihood tempering or the MitISEM samplers
   %
   WeightBins = cumsum(Weights)/NumDraws;
   if DSGEModel.RandomNumberValue==1;
      rand('state',0);
   else;
      rand('state',sum(100*clock));
   end;
   Draws = rand(1,UseDraws);
   SelectDraws = zeros(1,UseDraws);
   PosVec = (1:NumDraws)';
   for i=1:UseDraws;
      SelectDraws(i) = max((WeightBins<=Draws(i)).*PosVec)+1;
   end;
   SelectDraws(SelectDraws>UseDraws) = UseDraws;
   SelectDraws = sort(SelectDraws);
end;

%
% end of SampledPosteriorDraws.m
%
