function [status,theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = VerifyPriorData(DSGEModel,CurrINI)
% VerifyPriorData: Verifies if the data in the file with the prior distribution specification is valid. If so, it returns
%                   information about the prior.
%
% USAGE:
%
%       [status,theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist, ...
%                                                      ParameterNames,UniformBounds] = VerifyPriorData(DSGEModel,CurrINI)
%
% REQUIRED INPUT:  DSGEModel (structure) with the fields required to locate the file (and sheet) with information about the
%                            prior distributions of the various parameters.
%
%                  CurrINI (structure), with initialization information.
%
% REQUIRED OUTPUT: status (boolean) is 1 if the prior can be used by YADA, and 0 otherwise.
%
%                  theta (vector) with all parameters to be estimated
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                             0=normal, student-t, cauchy, logistic, Gumbel. This is used to determine the type of transformation
%                             that may need to be performed for the theta parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  ModelParameters (structure) with fields having the names of the inidividual parameters as they are
%                             specified under the "model parameter" header in the file with prior distribution data,
%                             and having values as specified under the header "initial value" in the same file
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                             of the model parameter in position i of theta.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                             parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                             and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                             it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                             truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                             values are the lower and the upper bound. For the student-t they are the location (mode),
%                             the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                             and scale parameters are needed. For the Pareto the parameter values are for the mode
%                             and the shape parameter.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                             "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each
%                             field returns a string matrix with the parameter names.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                             for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 4, 2006.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 07-08-2006: Added the thetaIndex and LowerBound vector outputs, as well as the ParameterNames and
%               InitialValues structures. The documentation was also updated.
%
% * 08-08-2006: Added the thetaDist output vector for quick mapping from theta into the parts that belong
%               to the different prior distributions.
%
% * 11-08-2006: Added the "all" and "estimated" fields to the ParameterNames structure.
%
% * 16-08-2006: Made sure that the initial values are valid for the different prior distributions.
%
% * 18-08-2006: Changed the StrTrim function to StringStrim.
%
% * 24-08-2006: Removed the output "InitialValues" since it is not used by YADA.
%
% * 25-08-2006: Added information collection and verification for the uniform distribution.
%
% * 28-08-2006: Made sure that the initial value is within the upper and lower bound of the uniform prior
%               distribution.
%
% * 21-09-2006: Fixed a bug when data was entered into the PriorDist.normal matrix. (The function erroneously
%               used the matrix PriorDist.invgamma as a base for each new parameter).
%
% * 13-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 02-07-2008: Added parameter name and other data to the error information text.
%
% * 21-07-2008: The gamma and invgamma priors can now have a non-zero lower bound.
%
% * 23-07-2008: The function now looks for the optional header "upper bound". Only used by the beta prior. These
%               upper and lower bounds default to 0 and 1. Added code to support a general beta prior.
%
% * 18-12-2008: Updated the documentation.
%
% * 19-12-2008: Added the check for the existence of the prior file.
%
% * 13-03-2009: Added information collection and verification for the Student-t, Cauchy, logistic, and Pareto
%               distributions.
%
% * 31-03-2009: Fixed the lower bound for the Pareto distribution.
%
% * 01-04-2009: The prior parameter 3 header is now used for the origin parameter of the Pareto distribution.
%
% * 02-04-2009: Took into account that the logistic distribution also supports a shape parameter.
%
% * 05-01-2010: Updated the documentation.
%
% * 23-11-2010: Made sure that "Parameter4" is propertly treated when 0 reflects an empty position. The information
%               was also extended for the case when there was an error with the initial value and/or the parameters
%               of the prior distribution.
%
% * 05-01-2011: Updated the documentation.
%
% * 31-01-2011: Fixed lower bound issue.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 07-03-2014: Made sure that "StartRow" is initialized.
%
% * 18-03-2014: The code after the exlread call now takes into account that the RawPriorData variables
%               can be empty (no MS office installed). For this case it attempts to set up the PriorData
%               matrix using the first two output variables of the xlsread function.
%
% * 12-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 11-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 30-04-2019: The code now allows the second parameter of the inverted Gamma to be a positive scalar
%               without necessarily being an integer. That it, the code allows for non-integer degrees
%               of freedom. The student-t distribution is likewise allows to have non-integer values
%               for the number of degrees of freedom.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the outputs
%
status = 0;
theta = [];
thetaIndex = [];
thetaDist = [];
LowerBound = [];
ModelParameters = [];
thetaPositions = [];
PriorDist = [];
UniformBounds = [];
ParameterNames.calibrated = '';
ParameterNames.beta = '';
ParameterNames.gamma = '';
ParameterNames.normal = '';
ParameterNames.invgamma = '';
ParameterNames.truncnormal = '';
ParameterNames.uniform = '';
ParameterNames.student = '';
ParameterNames.cauchy = '';
ParameterNames.logistic = '';
ParameterNames.gumbel = '';
ParameterNames.pareto = '';
ParameterNames.all = '';
ParameterNames.estimated = '';
if FileExist(DSGEModel.PriorFile)==0;
   return;
end;
StartRow = 0;
%
% Determine the file type
%
if strcmp(GetExt(DSGEModel.PriorFile),'.wk1')==1;
   %
   % Check the integrity of the Lotus 1-2-3 spreadsheet
   %
   ErrorValue = [];
   try;
    [fok,chk] = wk1finfo(DSGEModel.PriorFile);
   catch;
      ErrorValue = 1;
   end;
   if (isempty(ErrorValue)==0)|(isempty(fok)==1);
      txt = ['The Lotus 1-2-3 spreadsheet you selected "' GetFilename(DSGEModel.PriorFile) '" cannot be read by YADA. Please select another file.'];
      About(txt,'information','Bad Lotus Spreadsheet',200,500,CurrINI);
      return;
   end;
   %
   % The file can be read. Now we try to read it.
   %
   [PriorData,PriorText] = wk1read(DSGEModel.PriorFile);
   [w1,w2] = size(PriorText);
   if w2>size(PriorData,2);
      PriorData = [PriorData zeros(size(PriorData,1),w2-size(PriorData,2))];
   end;
   for i=1:w1;
      TitleText = '';
      for j=1:w2;
         TitleText = strvcat(TitleText,PriorText{i,j});
      end;
      if isempty(TitleText)==0;
         StartRow = i;
         break;
      end;
   end;
   if isempty(TitleText)==1;
      %
      % no text informations whatsoever
      %
      txt = ['The Lotus 1-2-3 spreadsheet you selected "' GetFilename(DSGEModel.PriorFile) '" doesn''t contain any text. This file is therefore not a proper prior distribution file.'];
      About(txt,'information','Bad Lotus Spreadsheet',200,500,CurrINI);
      return;
   end;
   FileType = 'Lotus 1-2-3';
else;
   %
   % Tt's an Excel spreadsheet. Check its integrity
   %
   ErrorValue = [];
   try;
      [fok,sheets] = xlsfinfo(DSGEModel.PriorFile);
   catch;
      ErrorValue = 1;
   end;
   if (isempty(ErrorValue)==0)|(isempty(fok)==1);
      txt = ['The Excel spreadsheet you selected "' GetFilename(DSGEModel.PriorFile) '" cannot be read by YADA. Please select another file.'];
      About(txt,'information','Bad Excel Spreadsheet',200,500,CurrINI);
      return;
   end;
   %
   % check if DSGEModel.PriorFileSheet is located among sheets!
   %
   sheets = char(sheets);
   pos = LocateStr(sheets,DSGEModel.PriorFileSheet);
   if pos==0;
      txt = ['The Excel spreadsheet "' GetFilename(DSGEModel.PriorFile) '" doesn''t have a sheet called "' DSGEModel.PriorFileSheet '". Please correct your selected sheet.'];
      About(txt,'information','Invalid Excel Spreadsheet Information',200,500,CurrINI);
      return;
   end;
   %
   % The file can be read. Now we try to read it.
   %
   [Data,PriorText,RawPriorData] = xlsread(DSGEModel.PriorFile,DSGEModel.PriorFileSheet);
   [w1,w2] = size(PriorText);
   %
   % User RawPriorData to create the matrix PriorData with the same information as PriorData
   % from reading a wk1-file. If it's an empty matrix we'll have to work around this with
   % PriorText and Data
   %
   if isempty(RawPriorData)==0
      [r,c] = size(RawPriorData);
      PriorData = zeros(r,c);
      for i=1:r;
         for j=1:c;
            if ischar(RawPriorData{i,j})==0;
               PriorData(i,j) = cell2mat(RawPriorData(i,j));
            end;
         end;
      end;
   else;
      PriorData = zeros(w1,w2);
      d2 = size(Data,2);
      for i=1:w1;
         for j=1:w2;
            if isempty(PriorText{i,j})==1;
               PriorData(i,j:j+d2-1) = Data(i,:);
               break;
            end;
         end;
      end;
   end;
   for i=1:w1;
      TitleText = '';
      for j=1:w2;
         TitleText = strvcat(TitleText,PriorText{i,j});
      end;
      if isempty(TitleText)==0;
         StartRow = i;
         break;
      end;
   end;
   if isempty(TitleText)==1;
      %
      % no text informations whatsoever
      %
      txt = ['The Excel spreadsheet you selected "' GetFilename(DSGEModel.PriorFile) '" with sheet "' DSGEModel.PriorFileSheet '" doesn''t contain any text. This file is therefore not a proper prior distribution file.'];
      About(txt,'information','Bad Excel Spreadsheet',200,500,CurrINI);
      return;
   end;
   FileType = 'Excel';
end;
%
% remove any leading and trailing spaces from TitleText
%
NewTitleText = '';
for i=1:size(TitleText,1);
   NewTitleText = strvcat(NewTitleText,StringTrim(TitleText(i,:)));
end;
TitleText = NewTitleText;
%
% check if we can locate the positions of the headers
%
PriorPositions = zeros(1,9);
PriorPositions(1,1) = LocateStr(TitleText,'model parameter',0);
PriorPositions(1,2) = LocateStr(TitleText,'status',0);
PriorPositions(1,3) = LocateStr(TitleText,'initial value',0);
PriorPositions(1,4) = LocateStr(TitleText,'prior type',0);
PriorPositions(1,5) = LocateStr(TitleText,'prior parameter 1',0);
PriorPositions(1,6) = LocateStr(TitleText,'prior parameter 2',0);
PriorPositions(1,7) = LocateStr(TitleText,'lower bound',0);
PriorPositions(1,8) = LocateStr(TitleText,'upper bound',0);
PriorPositions(1,9) = LocateStr(TitleText,'prior parameter 3',0);
%
% check if there are missing headers
%
if min(PriorPositions(1:7))==0;
   txt = ['The ' FileType ' spreadsheet you''ve selected for providing the prior distribution data has missing column headers. The required column headers are: "Model parameters", "Status", "Initial value", "Prior type", "Prior parameter 1", "Prior parameter 2", and "Lower bound" (without the quotation characters). The ordering of these headers is unimportant as is the case used (upper, lower, or any mixture).'];
   About(txt,'information','Missing Column Headers',200,500,CurrINI);
   return;
end;
%
% all header are present, now we try to set up the data
%
ModelParameters = [];
ParamValueExist = 1;
ValidStatus = 1;
ValidDist = 1;
ValidParameters = 1;
PriorInfStr = '';
num = 1;
PriorDist.beta = [];
PriorDist.gamma = [];
PriorDist.invgamma = [];
PriorDist.normal = [];
PriorDist.truncnormal = [];
PriorDist.uniform = [];
PriorDist.student = [];
PriorDist.cauchy = [];
PriorDist.logistic = [];
PriorDist.gumbel = [];
PriorDist.pareto = [];
%
for i=1+StartRow:w1;
   %
   % parameter name
   %
   CurrParameter = StringTrim(PriorText{i,PriorPositions(1,1)});
   %
   % check initial values
   %
   if (isnumeric(PriorData(i,PriorPositions(1,3)))==1)&(isempty(PriorData(i,PriorPositions(1,3)))==0);
      ModelParameters = setfield(ModelParameters,PriorText{i,PriorPositions(1,1)},PriorData(i,PriorPositions(1,3)));
      ParameterNames.all = strvcat(ParameterNames.all,StringTrim(PriorText{i,PriorPositions(1,1)}));
   else;
      ParamValueExist = 0;
      break;
   end;
   %
   % check the status
   %
   StatusStr = PriorText{i,PriorPositions(1,2)};
   if (isempty(StatusStr)==1)|(length(StatusStr)<2)|(ischar(StatusStr)==0);
      ValidStatus = 0;
      break;
   end;
   StatusStr = StringTrim(StatusStr);
   if (strcmp(lower(StatusStr(1,1:2)),'es')==0)&(strcmp(lower(StatusStr(1,1:2)),'ca')==0);
      ValidStatus = 0;
      break;
   end;
   %
   % Add the parameter to theta if it should be estimated and if so map its name to the position in theta
   % through the structure thetaPositions. This structure has the feature that if theta(2) = ModelParameters.gamma
   % then, thetaPositions(2).parameter = 'gamma'. Hence, when updating ModelParameters we simply use the
   % command:
   %
   %            ModelParameters = setfield(ModelParameters,thetaPositions(i).parameter,theta(i))
   %
   if strcmp(lower(StatusStr(1,1:2)),'es')==1;
      theta = [theta;PriorData(i,PriorPositions(1,3))];
      ParameterNames.estimated = strvcat(ParameterNames.estimated,PriorText{i,PriorPositions(1,1)});
      thetaPositions(num).parameter = PriorText{i,PriorPositions(1,1)};
      %
      % check if the claimed distribution is valid
      %
      DistStr = PriorText{i,PriorPositions(1,4)};
      if (isempty(DistStr)==1)|(length(DistStr)<2)|(ischar(DistStr)==0);
         ValidDist = 0;
         break;
      end;
      %
      DistStr = lower(StringTrim(DistStr));
      if (strcmp(DistStr,'beta')==0)&(strcmp(DistStr,'gamma')==0)&(strcmp(DistStr,'invgamma')==0)&(strcmp(DistStr,'normal')==0)&(strcmp(DistStr,'truncnormal')==0)&(strcmp(DistStr,'uniform')==0)&(strcmp(DistStr,'student')==0)&(strcmp(DistStr,'cauchy')==0)&(strcmp(DistStr,'logistic')==0)&(strcmp(DistStr,'gumbel')==0)&(strcmp(DistStr,'pareto')==0);
         ValidDist = 0;
         break
      end;
      %
      % we have a valid distribution
      %
      num = num+1;
      %
      % Now we check the prior parameter 1 and prior parameter 2 entries
      %
      Parameter1 = PriorData(i,PriorPositions(1,5));
      Parameter2 = PriorData(i,PriorPositions(1,6));
      Parameter3 = PriorData(i,PriorPositions(1,7));
      if isnan(Parameter3)==1;
         Parameter3 = 0;
      end;
      %
      % check if optional parameter upper bound is present
      %
      if PriorPositions(1,8)>0;
         Parameter4 = PriorData(i,PriorPositions(1,8));
         if isnan(Parameter4)==1;
            Parameter4 = 1;
         end;
         if (Parameter4==0)&(Parameter3==0);
            Parameter4 = 1;
         end;
      else;
         Parameter4 = [];
      end;
      %
      % check if optional parameter prior parameter 3 is present.
      % if not the code sets the degrees of freedom parameter of
      % the student-t to 1, i.e., to the cauchy
      %
      if PriorPositions(1,9)>0;
         Parameter5 = PriorData(i,PriorPositions(1,9));
      else;
         Parameter5 = [];
      end;
      if (isnumeric(Parameter1)==1)&(isempty(Parameter1)==0)&(isnan(Parameter1)==0)&(isnumeric(Parameter2)==1)&(isempty(Parameter2)==0)&(isnan(Parameter2)==0);
         %
         % check the distribution type
         %
         if strcmp(DistStr,'beta')==1;
            if Parameter2>0;
               ParameterNames.beta = strvcat(ParameterNames.beta,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;2];
               thetaDist = [thetaDist;2];
               if isempty(Parameter4)==0;
                  %
                  % check the upper and lower bounds
                  %
                  if (isnumeric(Parameter3)==1)&(isempty(Parameter3)==0)&(isnan(Parameter3)==0)&(isnumeric(Parameter4)==1)&(isnan(Parameter4)==0)&(Parameter4>Parameter3)&(Parameter1>Parameter3)&(Parameter1<Parameter4)&(PriorData(i,PriorPositions(1,3))>Parameter3)&(PriorData(i,PriorPositions(1,3))<Parameter4);
                     UniformBounds = [UniformBounds;Parameter3 Parameter4];
                     PriorDist.beta = [PriorDist.beta;Parameter1 Parameter2 Parameter3 Parameter4];
                     LowerBound = [LowerBound;Parameter3];
                  else;
                     ValidParameters = 0;
                     PriorInfStr = ['The following prior values were obtained: beta distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', lower bound=' num2str(Parameter3) ', upper bound=' num2str(Parameter4)];
                     break;
                  end;
               else;
                  if (PriorData(i,PriorPositions(1,3))>0)&(PriorData(i,PriorPositions(1,3))<1);
                     UniformBounds = [UniformBounds;0 1];
                     PriorDist.beta = [PriorDist.beta;Parameter1 Parameter2 0 1];
                     LowerBound = [LowerBound;0];
                  else;
                     ValidParameters = 0;
                     PriorInfStr = ['The following prior values were obtained: beta distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', lower bound=' num2str(Parameter3) ', upper bound=' num2str(Parameter4)];
                     break;
                  end;
               end;
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: beta distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', lower bound=' num2str(Parameter3) ', upper bound=' num2str(Parameter4)];
               break;
            end;
         elseif strcmp(DistStr,'gamma')==1;
            if (isnumeric(Parameter3)==1)&(isempty(Parameter3)==0)&(isnan(Parameter3)==0)&(Parameter1>Parameter3)&(Parameter2>0)&(PriorData(i,PriorPositions(1,3))>Parameter3);
               PriorDist.gamma = [PriorDist.gamma;Parameter1 Parameter2 Parameter3];
               ParameterNames.gamma = strvcat(ParameterNames.gamma,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;1];
               LowerBound = [LowerBound;Parameter3];
               thetaDist = [thetaDist;1];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: gamma distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', lower bound=' num2str(Parameter3)];
               break;
            end;
         elseif strcmp(DistStr,'invgamma')==1;
            if  (isnumeric(Parameter3)==1)&(isempty(Parameter3)==0)&(isnan(Parameter3)==0)&(Parameter1>Parameter3)&(Parameter2>0)&(PriorData(i,PriorPositions(1,3))>Parameter3);
               PriorDist.invgamma = [PriorDist.invgamma;Parameter1 Parameter2 Parameter3];
               ParameterNames.invgamma = strvcat(ParameterNames.invgamma,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;1];
               LowerBound = [LowerBound;Parameter3];
               thetaDist = [thetaDist;4];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: inverse gamma distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', lower bound=' num2str(Parameter3)];
               break;
            end;
         elseif strcmp(DistStr,'normal')==1;
            if Parameter2>0;
               PriorDist.normal = [PriorDist.normal;Parameter1 Parameter2];
               ParameterNames.normal = strvcat(ParameterNames.normal,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;0];
               LowerBound = [LowerBound;0];
               thetaDist = [thetaDist;3];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: normal distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2)];
               break;
            end;
         elseif strcmp(DistStr,'truncnormal')==1;
            if (isnumeric(Parameter3)==1)&(isempty(Parameter3)==0)&(isnan(Parameter3)==0)&(Parameter2>0)&(PriorData(i,PriorPositions(1,3))>=Parameter3);
               PriorDist.truncnormal = [PriorDist.truncnormal;Parameter1 Parameter2 Parameter3];
               ParameterNames.truncnormal = strvcat(ParameterNames.truncnormal,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;1];
               LowerBound = [LowerBound;Parameter3];
               thetaDist = [thetaDist;5];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: left truncated normal distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', lower bound=' num2str(Parameter3)];
               break;
            end;
         elseif strcmp(DistStr,'uniform')==1;
            if (Parameter2>Parameter1)&(PriorData(i,PriorPositions(1,3))>=Parameter1)&(Parameter2>=PriorData(i,PriorPositions(1,3)));
               PriorDist.uniform = [PriorDist.uniform; Parameter1 Parameter2];
               ParameterNames.uniform = strvcat(ParameterNames.uniform,PriorText{i,PriorPositions(1,1)});
               LowerBound = [LowerBound;0];
               thetaIndex = [thetaIndex;3];
               thetaDist = [thetaDist;6];
               UniformBounds = [UniformBounds;Parameter1 Parameter2];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: uniform distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2)];
               break;
            end;
         elseif strcmp(DistStr,'student')==1;
            if isempty(Parameter5)==1
               Parameter5 = 1;
            end;
            if (Parameter2>0)&(isnan(Parameter5)==0)&(Parameter5>0);
               PriorDist.student = [PriorDist.student;Parameter1 Parameter2 Parameter5];
               ParameterNames.student = strvcat(ParameterNames.student,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;0];
               LowerBound = [LowerBound;0];
               thetaDist = [thetaDist;7];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: student-t distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', parameter #3=' num2str(Parameter5)];
               break;
            end;
         elseif strcmp(DistStr,'cauchy')==1;
            if Parameter2>0;
               PriorDist.cauchy = [PriorDist.cauchy;Parameter1 Parameter2];
               ParameterNames.cauchy = strvcat(ParameterNames.cauchy,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;0];
               LowerBound = [LowerBound;0];
               thetaDist = [thetaDist;8];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: cauchy distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2)];
               break;
            end;
         elseif strcmp(DistStr,'logistic')==1;
            if isempty(Parameter5)==1
               Parameter5 = 1;
            end;
            if (Parameter2>0)&(isnumeric(Parameter5)==1)&(isnan(Parameter5)==0)&(Parameter5>0);
               PriorDist.logistic = [PriorDist.logistic;Parameter1 Parameter2 Parameter5];
               ParameterNames.logistic = strvcat(ParameterNames.logistic,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;0];
               LowerBound = [LowerBound;0];
               thetaDist = [thetaDist;9];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: logistic distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', parameter #3=' num2str(Parameter5)];
               break;
            end;
         elseif strcmp(DistStr,'gumbel')==1;
            if Parameter2>0;
               PriorDist.gumbel = [PriorDist.gumbel;Parameter1 Parameter2];
               ParameterNames.gumbel = strvcat(ParameterNames.gumbel,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;0];
               LowerBound = [LowerBound;0];
               thetaDist = [thetaDist;10];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: gumbel distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2)];
               break;
            end;
         elseif strcmp(DistStr,'pareto')==1;
            if isempty(Parameter5)==1
               Parameter5 = 0;
            end;
            if (Parameter1>0)&(Parameter2>0)&(isnumeric(Parameter5)==1)&(isnan(Parameter5)==0);
               PriorDist.pareto = [PriorDist.pareto;Parameter1 Parameter2 Parameter5];
               ParameterNames.pareto = strvcat(ParameterNames.pareto,PriorText{i,PriorPositions(1,1)});
               thetaIndex = [thetaIndex;1];
               LowerBound = [LowerBound;Parameter2+Parameter5];
               thetaDist = [thetaDist;11];
               UniformBounds = [UniformBounds;0 1];
            else;
               ValidParameters = 0;
               PriorInfStr = ['The following prior values were obtained: pareto distribution, initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2) ', parameter #3=' num2str(Parameter5)];
               break;
            end;
         end;
      else;
         ValidParameters = 0;
         PriorInfStr = ['The following prior values were obtained: initial value=' num2str(PriorData(i,PriorPositions(1,3))) ', parameter #1=' num2str(Parameter1) ', parameter #2=' num2str(Parameter2)];
         break;
      end;
   else;
      %
      % we have a calibrated parameter
      %
      ParameterNames.calibrated = strvcat(ParameterNames.calibrated,PriorText{i,PriorPositions(1,1)});
   end;
end;
%
% if there have been errors, display correct message and return function which called VerifyPriorData
%
if ParamValueExist==0;
   txt = ['The ' FileType ' spreadsheet you''ve selected has a missing initial value for the parameter "' CurrParameter '". YADA is therefore unable to continue.'];
   About(txt,'information','Missing Initial Values',200,500,CurrINI);
   return;
end;
%
if ValidStatus==0;
   txt = ['The ' FileType ' spreadsheet you''ve selected has a parameter, "' CurrParameter '", whose status, "' StatusStr '", is neither es(timated) or ca(librated). YADA is therefore unable to continue.'];
   About(txt,'information','Invalid Parameter Status',200,500,CurrINI);
   return;
end;
%
if ValidDist==0;
   txt = ['The ' FileType ' spreadsheet you''ve selected has a parameter, "' CurrParameter '", whose prior distribution, "' DistStr '", is not of a known type. These types are: "beta", "gamma", "invgamma", "normal", "truncnormal", and "uniform".'];
   About(txt,'information','Invalid Parameter Status',200,500,CurrINI);
   return;
end;
%
if ValidParameters==0;
   txt = ['The ' FileType ' spreadsheet you''ve selected has a parameter, "' CurrParameter '", whose initial value and/or prior parameters are not valid.' PriorInfStr '. YADA is therefore unable to continue.'];
   About(txt,'information','Invalid Prior Parameters',500,600,CurrINI);
   return;
end;
status = 1;

%
% end of VerifyPriorData.m
%
