function [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPaths(PhiDraws,A0Draws,PhiMode,A0Mode,X,Z,U,LastPeriod,h,ForecastType,CurrChain,IsPosterior,Weights,PME,lambda,NumPaths,DSGEModel,CurrINI,controls)
% DSGEVARCondPredictionPaths: Calculates a certain number of possible paths that the endogenous variables can take
%                             out-of-sample for a sample of the parameters in a DSGE-VAR model conditional on a
%                             set of assumptions on certain variables. The sample can be from the prior or from the
%                             posterior distribution while the direct controls of shocks conditioning method is applied.
%
% USAGE:
%
%       [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPaths(PhiDraws,...
%                                               A0Draws,PhiMode,A0Mode,X,Z,U,LastPeriod,h,ForecastType,CurrChain,...
%                                               IsPosterior,Weights,PME,lambda,NumPaths,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  PhiDraws (matrix) of dimension UsedDraws x n(np+k) with the draws from the
%                           VAR parameters. Each row holds vec(Phi(0) Phi(1) ... Phi(p))'.
%
%                  A0Draws (matrix) of dimension UsedDraws x n^2 with the draws of the A0
%                           parameters. Each row holds vec(A0)'.
%
%                  PhiMode (matrix) of dimension nx(np+k) with fixed values for the Phi parameters
%                           that can be used as replacements for PhiDraws.
%
%                  A0Mode (matrix) of dimension nxn with the contemporaneous responses in the
%                           endogenous variables from unit shocks.
%
%                  X (matrix) of dimension kxh with data on the exogenous variables over the
%                           forecast sample.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                           predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                           data and the conditioning data.
%
%                  LastPeriod (integer) with the last within-sample data point
%
%                  h (integer) with the number of out-of-sample forecast periods.
%
%                  ForecastType (integer) that is equal to 1 if the original data should be forecasted,
%                           2 if annualized data should be forecasted, and 3 if transformed data.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                           are used and 0 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension UsedDraws x 1 with the importance weights from the SMC with
%                           likelihood tempering posterior sampler.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                           are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                           used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                           when generating the posterior draws.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  NumPaths (integer) with the number of forecast paths to compute per parameter value.
%
%                  DSGEModel (structure)  whose fields contain all required information about the
%                           model, such as input files, variable and sample data. Actual observations
%                           are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                           main dialog.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (matrix) of dimension nx8 where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%                  ShockMean (matrix) of dimension nxh with the population mean of the shocks under
%                                  conditioning.
%
%                  ShockNames (string matrix) with the names of the structural shocks used by the
%                                  DSGE-VAR.
%
%                  ShockCov (matrix) of dimension nxnxh with the estimated residual or shock uncertainty.
%
%                  ParameterCov (matrix) of dimension nxnxh with the estimated parameter uncertainty.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 25, 2011.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-02-2011: Continued work on the function after the intoduction of the K1 and K2 matrices.
%               Finished the population moments from the conditional predictive distribution.
%
% * 01-03-2011: Added the path sampling code and finished the function.
%
% * 03-03-2011: Updated the documentation.
%
% * 29-03-2011: Made sure that "SigmaEpsilon" is also properly setup when the "A0Mode" value is used
%               instead of a draw from the p(oste)rior distribution.
%
% * 03-01-2012: Updated the documentation.
%
% * 06-02-2012: Added support for missing observations in the conditioning assumptions and for lag-varying
%               K2 matrices.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 11-09-2017: Added the input variable "Weights".
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
PredEventData = [];
ShockMean = [];
ShockNames = '';
UsedDraws = size(PhiDraws,1);
if isempty(Weights)==1;
   Weights = ones(UsedDraws,1);
end;
%
% initialize
%
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
if ForecastType==1;
   ShockCov = zeros(n,n,h);
   ParameterCov = ShockCov;
   DataType = 'Original';
   MeanPredictY = zeros(n,h);
else;
   ShockCov = [];
   ParameterCov = [];
   if ForecastType==2;
      DataType = 'Annualized';
   elseif ForecastType==3;
      DataType = 'Transformed';
   end;
end;
%
% setup the conditioning assumptions
%
[m,g] = size(Z);
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
%
% check the conditioning shocks to make sure that we can use them
%
ConditionalShocks = intersect(DSGEModel.ConditionalShocks,DSGEModel.DSGEVARShocks);
if length(ConditionalShocks)<m;
   txt = ['The number of conditioning shocks that can be used for the DSGE-VAR is ' int2str(length(ConditionalShocks)) ' while the required number of conditioning shocks is ' int2str(m) '. You must therefore either add more conditioning shocks or drop some conditioning assumptions.'];
   About(txt,'error','Error - Conditional Shocks',200,500,CurrINI);
   return;
end;
ConditionalShocks = DSGEModel.ConditionalShocks;
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of the ' lower(DataType) ' conditional predictive distributions have therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of the ' lower(DataType) ' conditional predictive distributions have therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% backup parameters
%
if k>0;
   Phi0star = PhiMode(:,1:k);
   Phistar = PhiMode(:,k+1:(n*p)+k);
else;
   Phi0star = [];
   Phistar = PhiMode;
end;
J = eye(n);
if p>1;
   Phistar = [Phistar;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% setup observed variables and initial conditions for the observed variables
%
if (DSGEModel.RunPredictionEvent==1)&(size(DSGEModel.Y,2)>=LastPeriod+h);
   YObserved = DSGEModel.Y(:,LastPeriod+1:LastPeriod+h);
else;
   YObserved = [];
end;
if ForecastType==2;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
   else;
      Yinit = [];
   end;
elseif ForecastType==3;
   Yinit = DSGEModel.Y(:,1:LastPeriod);
end;
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
% initial conditions for the forecasts
%
YT = vec(fliplr(DSGEModel.Y(:,LastPeriod-p+1:LastPeriod)));
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% initialize shock related data
%
ShockNames = DSGEModel.StateShockNames(DSGEModel.DSGEVARShocks,:);
ShockMean = zeros(n,h);
%
% stack certain matrices and vector for the shock distribution
%
K = zeros(n*g,m*g);
ZTg = zeros(m*g,1);
UT = zeros(m*g,1);
WTg = zeros(n*g,1);
for i=1:g;
   Ind = (1+((g-i)*n):(g-i+1)*n);
   Ind2 = (1+((g-i)*m):(g-i+1)*m);
   K(Ind,Ind2) = K1;
   for j=1:i-1;
      Ind3 = Ind+(j*n);
      if K2length==2;
         K(Ind3,Ind2) = K2;
      else;
         if j<=size(K2,3);
            K(Ind3,Ind2) = K2(:,:,j);
         end;
      end;
   end;
   ZTg(Ind2,1) = Z(:,i);
   UT(Ind2,1) = U;
end;
N = eye(n);
M = N(:,ConditionalShocks);
%
% check for missing restrictions
%
PosVec = (1:m*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
ZTg = ZTg(PosVec);
UT = UT(PosVec);
K = K(:,PosVec);
IgM = kron(eye(g),M);
IgM = IgM(:,PosVec);
IgMMorth = null(IgM','r');
IgMMorth = IgMMorth*IgMMorth';
%Morth = N(:,setdiff((1:n),ConditionalShocks));
%IgMMorth = kron(eye(g),Morth*Morth');
%
% test if we can invert the model for the standardized shocks at posterior mode
%
etaMat = K1'*A0Mode*M;
if rank(etaMat)<m;
   txt = ['The shocks you have selected for conditioning cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution has therfore been aborted.'];
   About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
   return;
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(UsedDraws*NumPaths,'%0.0f') ' Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = ['Conditional Predictive Distribution' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the conditional predictive distribution of a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags for the ' lower(DataType) ' data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Conditional Predictive Distribution ' PriorHeader],600,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
ZPos = ~isnan(Z);
%
for ParamIndex=1:UsedDraws;
   %
   % get the new parameters
   %
   A0 = reshape(A0Draws(ParamIndex,:)',n,n);
   SigmaEpsilon = A0*A0';
   etaMat = K1'*A0*M;
   if (IsPositiveSemiDefinite(SigmaEpsilon)==1)&(rank(etaMat)==m);
      Phi = reshape(PhiDraws(ParamIndex,:)',n,(n*p)+k);
      if k>0;
         Phi0 = Phi(:,1:k);
         Phi = Phi(:,k+1:(n*p)+k);
      else;
         Phi0 = [];
      end;
      if p>1;
         Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
      end;
   else;
      A0 = A0Mode;
      Phi = Phistar;
      Phi0 = Phi0star;
      etaMat = K1'*A0*M;
      SigmaEpsilon = A0*A0';
   end;
   %
   % setup exogenous variables and the G-matrix on YT
   %
   if ForecastType==1;
      Xbar = zeros(n*p,1);
      G = zeros(n*g,n*p);
      XTg = zeros(n*g,1);
      D = eye(n*g,n*g);
      PhiExp = Phi;
      for i=1:g;
         Ind = (1+((g-i)*n):(g-i+1)*n);
         Ind1 = Ind;
         Ind2 = (1+((g-i)*m):(g-i+1)*m);
         if k>0;
            Xbar = (J*Phi0*X(:,i))+(Phi*Xbar);
            XTg(Ind) = J'*Xbar;
         end;
         G(Ind,:) = J'*PhiExp;
         Phii = Phi;
         for j=1:i-1;
            Ind1 = Ind1+n;
            Ind3 = Ind+(j*n);
            D(Ind,Ind1) = J'*Phii*J;
            Phii = Phii*Phi;
         end;
         PhiExp = PhiExp*Phi;
      end;
      kTg = ZTg-UT-(K'*(XTg+(G*YT)));
      IgA0 = kron(eye(g),A0);
      IgA0M = kron(eye(g),A0)*IgM;
      %
      % compute the covariance matrix of the forecast errors over the conditioning
      % horizon
      %
      Dtilde = eye(n*g)-( (IgM/(K'*D*IgA0M))*K'*D*IgA0 );
      SigmaEShocks = IgA0*Dtilde*IgMMorth*Dtilde'*IgA0';
      CovShock = D*SigmaEShocks*D';
      %
      PhiTilde = zeros(n*p,n*g);
      Phii = eye(n*p);
      SigmaBarY = zeros(n*p,n*p);
      SigmaBarLZ = zeros(n*p,n*p);
      MultiModestyStat = zeros(NumPaths,2);
      UniModestyStat = zeros(NumPaths,n);
      UniModestyStatLZ = zeros(1,n);
   else;
      MultiModestyStat = [];
      UniModestyStat = [];
      UniModestyStatLZ = [];
   end;
   %
   % setup the population mean of the shocks over the conditioning horizon, with T+1 being
   % the leftmost and T+g being the rightmost column
   %
   YLast = YT;
   XTh = zeros(n,h);
   K2sum = zeros(m,1);
   etaShocks = zeros(n,h);
   PredictY = zeros(n,h);
   for i=1:h;
      if k>0;
         XTh(:,i) = Phi0*X(:,i);
      end;
      if i<=g
         ZPosi = ZPos(:,i);
         m1 = sum(ZPosi);
         if m1>0;
            etaShocks(ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i)-(K1(:,ZPosi)'*(XTh(:,i) + J'*Phi*YLast))-U(ZPosi)-K2sum(ZPosi));
         end;
         yNew = XTh(:,i)+(J'*Phi*YLast)+(A0*etaShocks(:,i));
         if K2length==2;
            K2sum = (K2'*yNew)+K2sum;
         else;
            K2sum = K2(:,:,1)'*yNes;
            for j=2:i;
               if j<=size(K2,3);
                  K2sum = K2sum+(K2(:,:,j)'*PredictY(:,i+1-j));
               end;
            end;
         end;
         if ForecastType==1;
            PhiTilde(:,1+((i-1)*n):n*i) = Phii*J;
            Phii = Phii*Phi;
            SigmaBarY = (J*SigmaEpsilon*J')+(Phi*SigmaBarY*Phi');
            SigmaBarLZ = (J*A0*M(:,ZPosi)*M(:,ZPosi)'*A0'*J')+(Phi*SigmaBarLZ*Phi');
            Ind1 = (1+((g-i)*n):(g-i+1)*n);
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/UsedDraws)*(diag(abs(diag(CovShock(Ind1,Ind1))))+tril(CovShock(Ind1,Ind1),-1)+tril(CovShock(Ind1,Ind1),-1)'));
            if i==g;
               %
               % compute Leeper-Zha type of univariate modesty statistics
               %
               PhiDiff = J'*PhiTilde*IgA0*vec(fliplr(etaShocks(:,1:g)));
               StdBarLZ = sqrt(abs(diag(J'*SigmaBarLZ*J)));
               StdBarLZ = StdBarLZ+(StdBarLZ==0);
               UniModestyStatLZ = (PhiDiff./StdBarLZ)';
               SigmaY = J'*SigmaBarY*J;
            end;
         end;
      else;
         yNew = XTh(:,i)+(J'*Phi*YLast);
         if ForecastType==1;
            %
            % forecast error covariance matrix
            %
            if i==(g+1);
               PhiSum = zeros(n,n);
               Phig = eye(n*p);
               PhiSigmaPhi = PhiTilde*SigmaEShocks*PhiTilde';
            end;
            PhiSum = PhiSum+(J'*Phig*J*SigmaEpsilon*J'*Phig'*J);
            Phig = Phi*Phig;
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/UsedDraws)*(PhiSum+(J'*Phig*PhiSigmaPhi*Phig'*J)));
         end;
      end;
      if p>1;
         YLast = [yNew;YLast(1:n*(p-1))];
      else;
         YLast = yNew;
      end;
      PredictY(:,i) = yNew;
      if ForecastType==1;
         ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/UsedDraws)*yNew*yNew');
      end;
   end;
   %
   % compute the average mean predictions over the different parameter draws
   %
   if ForecastType==1;
      MeanPredictY = MeanPredictY+((Weights(ParamIndex)/UsedDraws)*PredictY);
   end;
   ShockMean = ShockMean+((Weights(ParamIndex)/UsedDraws)*A0*etaShocks);
   if DSGEModel.AdjustPredictionPaths==1;
      if ForecastType==2;
         PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
      elseif ForecastType==3;
         YTmp = [Yinit PredictY];
         [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         Tt = size(YData,1);
         PredictY = YData(Tt-h+1:Tt,:)';
      end;
   end;
   %
   % we are now ready to compute possible paths at the DSGE-VAR parameter values
   %
   PredPaths = zeros(n,h,NumPaths);
   for it=1:NumPaths;
      CurrPath = it+(NumPaths*(ParamIndex-1));
      %
      % check if we should update the progress dialog
      %
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([CurrPath/(UsedDraws*NumPaths) CurrPath]);
         end;
      end;
      %
      % Draw a sequence of future innovations
      %
      upsilon = randn(n,h);
      upsilonTmp = upsilon;
      K2sum = zeros(m,1);
      YLast = YT;
      for i=1:h;
         if i<=g;
            ZPosi = ZPos(:,i);
            m1 = sum(ZPosi);
            if m1>0;
               upsilon(ConditionalShocks(ZPosi),i) = 0;
               upsilon(ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i)-(K1(:,ZPosi)'*(XTh(:,i) + (J'*Phi*YLast) + (A0*upsilon(:,i))))-U(ZPosi)-K2sum(ZPosi));
            end;
            yNew = XTh(:,i)+(J'*Phi*YLast)+(A0*upsilon(:,i));
            if K2length==2;
               K2sum = (K2'*yNew)+K2sum;
            else;
               K2sum = K2(:,:,1)'*yNew;
               for j=2:i;
                  if j<=size(K2,3);
                     K2sum = K2sum+(K2(:,:,j)'*PredPaths(:,i+1-j,it));
                  end;
               end;
            end;
            if (ForecastType==1)&(i==g);
               %
               % compute multivariate and univariate ALLV-type modesty statistics
               %
               PhiDiff = J'*PhiTilde*kron(eye(g),A0)*vec(fliplr(upsilon(:,1:g)));
               PhiDiffC = J'*PhiTilde*kron(eye(g),A0)*vec(fliplr(upsilonTmp(:,1:g)));
               MultiModestyStat(it,1) = PhiDiff'*(SigmaY\PhiDiff);
               MultiModestyStat(it,2) = PhiDiffC'*(SigmaY\PhiDiffC);
               UniModestyStat(it,:) = (PhiDiff./sqrt(diag(SigmaY)))';
            end;
         else;
            yNew = XTh(:,i)+(J'*Phi*YLast)+(A0*upsilon(:,i));
         end;
         if p>1;
            YLast = [yNew;YLast(1:n*(p-1))];
         else;
            YLast = yNew;
         end;
         PredPaths(:,i,it) = yNew;
      end;
      if ForecastType==2;
         PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
      elseif ForecastType==3;
         YTmp = [Yinit PredPaths(:,:,it)];
         [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
      end;
      drawnow;
   end;
   if StopEstimation==1;
      break;
   end;
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the sample mean is equal to the population mean.
      %
      MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
      AdjustMean = PredictY-MeanPrediction;
      for i=1:NumPaths;
         PredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   end;
   %
   % save the calculations to disk
   %
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-CondPrediction-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-Direct-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPaths) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\predictions\DSGE-VAR-CondPrediction-' DataType '-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-Direct-' DSGEModel.NameOfModel '-' int2str(h) '-' int2str(ParamIndex) '.' int2str(UsedDraws) '.' int2str(NumPaths) '.mat']);
   end;
   save(file,'PredPaths','ShockNames','ShockMean','MultiModestyStat','UniModestyStat','UniModestyStatLZ');
   if DSGEModel.RunPredictionEvent==1;
      %
      % compute the prediction events results
      %
      PredEventData = PredEventData+CalculatePredictionEvents(PredPaths,DSGEModel.PredictionEvent,Weights(ParamIndex));
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
%
% make sure the parameter uncertainty is corrected for the mean
%
DoneCalc = 1;
if isfield(DSGEModel,'PredictionEvent')==1;
   PredictionEvent = DSGEModel.PredictionEvent;
else;
   PredictionEvent = [Inf*ones(n,1) zeros(n,1) ones(n,1)];
end;
if ForecastType==1;
   for i=1:h;
      ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
      ParameterCov(:,:,i) = diag(abs(diag(ParameterCov(:,:,i))))+tril(ParameterCov(:,:,i),-1)+tril(ParameterCov(:,:,i),-1)';
   end;
   save(file,'PredEventData','PredictionEvent','ShockCov','ParameterCov','-APPEND');
else;
   save(file,'PredEventData','PredictionEvent','-APPEND');
end;

%
% end of DSGEVARCondPredictionPaths.m
%
