function [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ] = DSGEVARCondPredictionPathsParam(Phi0,Phi,A0,X,Z,U,LastPeriod,h,lambda,NumPaths,EstStr,ForecastType,DSGEVARShocks,DSGEModel,CurrINI)
% DSGEVARCondPredictionPathsParam: Calculates a certain number of possible paths that the endogenous variables can
%                                  take out-of-sample for a given value of the parameters in a DSGE-VAR model
%                                  conditional on a set of assumptions on certain variables. Uses the direct control
%                                  of shocks method.
%
% USAGE:
%
%       [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat, ...
%                                      UniModestyStatLZ] = DSGEVARCondPredictionPathsParam(Phi0,Phi,A0,X,Z,U,LastPeriod,...
%                                                 h,lambda,NumPaths,EstStr,ForecastType,DSGEVARShocks,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi0 (matrix) of dimension nxk with parameters on the exogenous variables.
%
%                  Phi (matrix) of dimension nxnp with parameters on lagged endogenous variables.
%
%                  A0 (matrix) of dimension nxn with parameters on the structural shocks.
%
%                  X (matrix) of dimension kxh with data on the exogenous variables over the
%                    forecast sample.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                    predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                    data and the conditioning data.
%
%                  LastPeriod (integer) with the last within-sample data point
%
%                  h (integer) with the number of out-of-sample forecast periods.
%
%                  lambda (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                  NumPaths (integer) with the number of forecast paths to compute.
%
%                  EstStr (string) that is equal to 'Initial Values', 'Marginal Posterior Mode',
%                    or 'Joint Posterior Mode'.
%
%                  ForecastType (integer) that is equal to 1 if the original data should be forecasted,
%                    2 if annualized data should be forecasted, and 3 if transformed data.
%
%                  DSGEVARShocks (vector) with the positions of the DSGE model shocks used for the VAR.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                    input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: PredPaths (matrix) of dimension nxhxNumPaths with the NumPaths possible paths for the n
%                    endogenous variables.
%
%                  PredData (structure) with fields:
%                    PredMean   - (matrix) of dimension nxh with the mean predictions of the observed
%                                 variables.
%                    epsShocks  - (matrix) of dimension nxh with the residuals at the population mean
%                                 predictions.
%                    epsMean    - (matrix) of dimension nxh with the mean values for the economic shocks over
%                                 the forecast period.
%                    Shocks     - (matrix) of dimension nxhxNumPaths with the residuals for the paths.
%                    KernelX    - (matrix) of dimension nx2^8xh with the x-axis values
%                                 from kernel density estimation of the marginal predictive
%                                 densities.
%                    KernelY    - (matrix) of dimension nx2^8xh with the y-axis values
%                                 from kernel density estimation of the marginal predictive
%                                 densities.
%
%                  PredEventData (matrix) of dimension nx8 where the first column holds the number of times
%                    the PredPaths data satisfies the event, the second holds the number of times
%                    the PredPaths data is below the lower bound of the event, the third the number of
%                    times the data is above the upper bound, while the fourth column holds the number
%                    of times the event has been tested. The fifth and sixth column measure the average
%                    deviation from the lower bound when below the lower bound and the square of this,
%                    while the seventh and eighth columns hold similar data for the upper bound of the
%                    event when the variables are above the upper bound.
%
%                  YObsEventData (matrix) of dimension nx8 where the first column hold the number of times the
%                    PredPaths (adjusted such that the mean is equal to the realized value) satisfies
%                    the event, the second holds the number of times the adjusted paths data is below
%                    the lower bound the event, the third the number of time the data is above the
%                    upper bound, while the fourth column holds the number of times the event has been
%                    tested. The fifth and sixth column measure the average deviation from the lower
%                    bound when below the lower bound and the square of this, while the seventh and 
%                    eighth columns hold similar data for the upper bound of the event when the
%                    variables are above the upper bound.
%
%                  MultiModestyStat (matrix) with NumPaths x 2 elements. The first column is the the
%                    multivariate modesty statistic based on the conditioning consistent shocks and
%                    the second is the value of this statistic for randomly drawn shocks.
%
%                  UniModestyStat (matrix) with NumPaths x n elements. Each column gives the univariate
%                    modesty for a particular observed variable.
%
%                  UniModestyStatLZ (vector) with the values of the univariate modesty statistics suggested
%                    by Leeper-Zha (other shocks are zero over examined prediction horizon) for all
%                    observed variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 21, 2011.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-02-2011: Updated the documentation.
%
% * 24-02-2011: Fixed the dialog name for the progress dialog.
%
% * 28-02-2011: Added the input variable "DSGEVARShocks".
%
% * 29-03-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 06-02-2012: Added support for missing observations in the conditioning assumptions and for lag-varying
%               K2 matrices.
%
% * 09-03-2012: Made sure that missing observations don't affect the prediction event calculations.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
PredEventData = [];
YObsEventData = [];
k = DSGEModel.k;
n = DSGEModel.n;
p = DSGEModel.BVARLags;
PredPaths = zeros(n,h,NumPaths);
PredData.epsShocks = zeros(n,h);
PredData.KernelX = [];
PredData.KernelY = [];
PredData.epsMean = zeros(n,h);
PredData.Shocks = zeros(n,h,NumPaths);
PredData.PredMean = zeros(n,h);
MultiModestyStat = [];
UniModestyStat = [];
UniModestyStatLZ = [];
%
% setup the conditioning assumptions
%
[m,g] = size(Z);
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
%
% check the conditioning shocks to make sure that we can use them
%
ConditionalShocks = intersect(DSGEModel.ConditionalShocks,DSGEVARShocks);
%
% make sure that the ordering of the shocks is unaffected
%
if length(ConditionalShocks)<m;
   txt = ['The number of conditioning shocks that can be used for the DSGE-VAR is ' int2str(length(ConditionalShocks)) ' while the required number of conditioning shocks is ' int2str(m) '. You must therefore either add more conditioning shocks or drop some conditioning assumptions.'];
   About(txt,'error','Error - Conditional Shocks',200,500,CurrINI);
   PredPaths = [];
   return;
end;
ConditionalShocks = DSGEModel.ConditionalShocks;
%
N = eye(n);
M = N(:,ConditionalShocks);
etaMat = K1'*A0*M;
if rank(etaMat)<m;
   txt = ['The shocks you have selected for conditioning cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution at ' lower(EstStr) ' has therfore been aborted.'];
   About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
   PredPaths = [];
   return;
end;
%
% setup observed variables and initial conditions for the observed variables
%
if (DSGEModel.RunPredictionEvent==1)&(size(DSGEModel.Y,2)>=LastPeriod+h);
   YObserved = DSGEModel.Y(:,LastPeriod+1:LastPeriod+h);
else;
   YObserved = [];
end;
if ForecastType==2;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
   else;
      Yinit = [];
   end;
elseif ForecastType==3;
   Yinit = DSGEModel.Y(:,1:LastPeriod);
end;
%
% deal with the parameters
%
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
%
% initial conditions for the forecasts
%
YT = vec(fliplr(DSGEModel.Y(:,LastPeriod-p+1:LastPeriod)));
%
% compute the population mean prediction
%
XTh = zeros(n,h);
K2sum = zeros(m,1);
etaShocks = zeros(n,h);
YLast = YT;
ZPos = ~isnan(Z);
if ForecastType==1;
   PhiTilde = zeros(n*p,n*g);
   Phii = eye(n*p);
   SigmaBarY = zeros(n*p,n*p);
   SigmaBarLZ = zeros(n*p,n*p);
   MultiModestyStat = zeros(NumPaths,2);
   UniModestyStat = zeros(NumPaths,n);
   UniModestyStatLZ = zeros(1,n);
end;
for i=1:h;
   if k>0;
      XTh(:,i) = Phi0*X(:,i);
   end;
   if i<=g;
      ZPosi = ZPos(:,i);
      m1 = sum(ZPosi);
      if m1>0;
         etaShocks(ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i)-(K1(:,ZPosi)'*(XTh(:,i) + J'*Phi*YLast))-U(ZPosi)-K2sum(ZPosi));
      end;
      yNew = XTh(:,i)+(J'*Phi*YLast)+(A0*etaShocks(:,i));
      PredData.PredMean(:,i) = yNew;
      if K2length==2;
         K2sum = (K2'*yNew)+K2sum;
      else;
         K2sum = zeros(m,1);
         for j=1:i;
            if j<=size(K2,3);
               K2sum = K2sum+(K2(:,:,j)'*PredData.PredMean(:,i+1-j));
            end;
         end;
      end;
      if ForecastType==1;
         PhiTilde(:,1+((i-1)*n):n*i) = Phii*J;
         Phii = Phii*Phi;
         SigmaBarY = (J*A0*A0'*J')+(Phi*SigmaBarY*Phi');
         SigmaBarLZ = (J*A0*M(:,ZPosi)*M(:,ZPosi)'*A0'*J')+(Phi*SigmaBarLZ*Phi');
         if i==g;
            %
            % compute Leeper-Zha type of univariate modesty statistics
            %
            PhiDiff = J'*PhiTilde*kron(eye(g),A0)*vec(fliplr(etaShocks(:,1:g)));
            StdBarLZ = sqrt(abs(diag(J'*SigmaBarLZ*J)));
            StdBarLZ = StdBarLZ+(StdBarLZ==0);
            UniModestyStatLZ = (PhiDiff./StdBarLZ)';
            SigmaY = J'*SigmaBarY*J;
         end;
      end;
   else;
      yNew = XTh(:,i)+(J'*Phi*YLast);
      PredData.PredMean(:,i) = yNew;
   end;
   if p>1;
      YLast = [yNew;YLast(1:n*(p-1))];
   else;
      YLast = yNew;
   end;
end;
PredData.epsShocks = A0*etaShocks;
%
% annualize the mean predictions when needed
%
if ForecastType==2;
   PredData.PredMean = Annualize([Yinit PredData.PredMean],DSGEModel.annual,DSGEModel.annualscale);
   if isempty(YObserved)==0;
      YObserved = Annualize([Yinit YObserved],DSGEModel.annual,DSGEModel.annualscale);;
   end;
elseif ForecastType==3;
   YTmp = [Yinit PredData.PredMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   PredData.PredMean = YData(Tt-h+1:Tt,:)';
   if isempty(YObserved)==0;
      YObserved = [Yinit YObserved];
      [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YObserved',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
      Tt = size(YData,1);
      YObserved = YData(Tt-h+1:Tt,:)';
   end;
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumPaths,'%0.0f') ' Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = ['Conditional Predictive Dist - ' EstStr];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;
%
% start the iterations
%
abort = '';
for it=1:NumPaths;
   %
   % check if we should update the progress dialog
   %
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         PredPaths = [];
         break;
      else;
         ProgressDLG([it/NumPaths it]);
      end;
   end;
   %
   % Draw a sequence of future innovations
   %
   upsilon = randn(n,h);
   upsilonTmp = upsilon;
   K2sum = zeros(m,1);
   YLast = YT;
   for i=1:h;
      if i<=g;
         ZPosi = ZPos(:,i);
         m1 = sum(ZPosi);
         if m1>0;
            upsilon(ConditionalShocks(ZPosi),i) = 0;
            upsilon(ConditionalShocks(ZPosi),i) = etaMat(ZPosi,ZPosi)\(Z(ZPosi,i)-(K1(:,ZPosi)'*(XTh(:,i) + (J'*Phi*YLast) + (A0*upsilon(:,i))))-U(ZPosi)-K2sum(ZPosi));
         end;
         yNew = XTh(:,i)+(J'*Phi*YLast)+(A0*upsilon(:,i));
         if K2length==2;
            K2sum = (K2'*yNew)+K2sum;
         else;
            K2sum = K2(:,:,1)'*yNew;
            for j=2:i;
               if j<=size(K2,3);
                  K2sum = K2sum+(K2(:,:,j)'*PredPaths(:,i+1-j,it));
               end;
            end;
         end;
         if (ForecastType==1)&(i==g);
            %
            % compute multivariate and univariate ALLV-type modesty statistics
            %
            PhiDiff = J'*PhiTilde*kron(eye(g),A0)*vec(fliplr(upsilon(:,1:g)));
            PhiDiffC = J'*PhiTilde*kron(eye(g),A0)*vec(fliplr(upsilonTmp(:,1:g)));
            MultiModestyStat(it,1) = PhiDiff'*(SigmaY\PhiDiff);
            MultiModestyStat(it,2) = PhiDiffC'*(SigmaY\PhiDiffC);
            UniModestyStat(it,:) = (PhiDiff./sqrt(diag(SigmaY)))';
         end;
      else;
         yNew = XTh(:,i)+(J'*Phi*YLast)+(A0*upsilon(:,i));
      end;
      if p>1;
         YLast = [yNew;YLast(1:n*(p-1))];
      else;
         YLast = yNew;
      end;
      PredPaths(:,i,it) = yNew;
   end;
   PredData.epsMean = PredData.epsMean+((1/NumPaths)*A0*upsilon);
   PredData.Shocks(:,:,it) = upsilon;
   %
   % check if we should annualize
   %
   if ForecastType==2;
      PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
   elseif ForecastType==3;
      YTmp = [Yinit PredPaths(:,:,it)];
      [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
      PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
   end;
   drawnow;
end;
%
% close the wait handle
%
if DSGEModel.ShowProgress==1;
   set(WaitHandle,'UserData','done');
   close(WaitHandle);
   drawnow;
   pause(0.02);
end;
if (strcmp(abort,'cancel')==0)&(DSGEModel.RunPredictionEvent==1);
   MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the mean is correct
      %
      AdjustMean = PredData.PredMean-MeanPrediction;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   else;
      PEPredPaths = PredPaths;
   end;
   %
   % compute the prediction events results
   %
   PredEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent);
   %
   if isempty(YObserved)==0;
      AdjustMean = YObserved-MeanPrediction;
      %
      % make sure that missing observations don't affect adjustment
      %
      AdjustMean(isnan(AdjustMean)) = 0;
      for i=1:NumPaths;
         YPEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
      YObsEventData = CalculatePredictionEvents(YPEPredPaths,DSGEModel.PredictionEvent);
   end;
end;
%
% check if we should compute kernel density estimates
%
if (strcmp(abort,'cancel')==0)&(NumPaths>=500);
   if DSGEModel.RunPredictionEvent==0;
      if DSGEModel.AdjustPredictionPaths==1;
         MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
         AdjustMean = PredData.PredMean-MeanPrediction;
         for i=1:NumPaths;
            PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
         end;
      else;
         PEPredPaths = PredPaths;
      end;
   end;
   %
   [PredData.KernelX,PredData.KernelY] = PredictionKernelDensity(PEPredPaths);
end;

%
% end of DSGEVARCondPredictionPathsParam.m
%
