function LogPost = DSGEVARLogPosterior(PostSample,AcceptedDraws,ModeData,lambda,EstStr,DSGEModel,CurrINI)
% DSGEVARLogPosterior: Computes the value of the log posterior for a DSGE-VAR at a sample of parameter draws.
%
% USAGE:
%
%       LogPost = DSGELogPosterior(PostSample,AcceptedDraws,ModeData,EstStr,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  PostSample (matrix) of dimension (NumIter x NumParam) with posterior draws of parameters.
%
%                  AcceptedDraws (vector) of dimension NumIter with the value 1 if the draw was accepted and
%                                0 otherwise.
%
%                  ModeData (structure) with the stored data from posterior mode estimation.
%
%                  lambda (real) hyperparameter value that is used to measure degree of misspecification.
%
%                  EstStr (string) with the name of the function for which the log posterior is computed.
%
%                  DSGEModel (structure), whose fields contain all required information
%                                about the model, such as input files, variable and sample
%                                data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: LogPost (vector) of dimension (NumIter x 1) with log posterior values for the parameters.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 27, 2012.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-02-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 14-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
% * 30-08-2017: Took into account that "logPosteriorPhiDSGEVAR" requires the input variable "NumWorkers".
%
% * 16-01-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 14-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 16-10-2018: The function now supports the SMC with data tempering posterior sampler.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 05-07-2023: The function now tries to load the AIMData file directly.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
LogPost = [];
%
% initialize dimensions
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
npk = (n*p)+k;
%
% create sample dates
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on p, the lag order
%
IVAdjusted = 0;
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
   IVAdjusted = 1;
end;
%
% actual sample that we will use
%
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
DSGEModel.KalmanFirstObservation = KalmanFirstObservation;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
if k>0;
   DetProductMoments = CalcDeterministicProductMoments(x,X);
else;
   DetProductMoments = [];
end;
%
% compute sample product moments
%
GammaHatyy = (1/T)*y*y';
GammaHatyY = (1/T)*y*Y';
GammaHatYY = (1/T)*Y*Y';
%
if isinf(lambda)==0;
   logGPR = LogGammaProductRatio(n,((1+lambda)*T)-npk,(lambda*T)-npk);
else;
   logGPR = [];
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% try to load the AIMData, other wise use ModeData
%
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      AIMData = ModeData.AIMData;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      AIMData = ModeData.AIMData;
   end;
end;
%
% now try to 
%
if MatlabNumber>=8.0;
   try;
      logPostMode = -logPosteriorPhiDSGEVAR(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
   catch ME;
      txt = ['Unable to compute the log posterior at the posterior mode values. Message caught is: ' YADALastError(ME)];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
      return;
   end;
else;
   try;
      logPostMode = -logPosteriorPhiDSGEVAR(ModeData.phiMode,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
   catch;
      txt = ['Unable to compute the log posterior at the posterior mode values. Message caught is: ' YADALastError];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
      return;
   end;
end;
%
% now we can do the rest
%
NumIter = length(AcceptedDraws);
LogPost = zeros(NumIter,1);
logPostTmp = -logPosteriorPhiDSGEVAR(PostSample(1,:)',ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
LogPost(1) = logPostTmp;
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the log posterior for a total number of ' int2str(NumIter) ' posterior draws using the DSGE-VAR(' num2str(lambda,'%0.4f') ') model with ' int2str(p) ' lags. The calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
WaitHandle = WaitDLG(txt,'information',['Log Posterior - ' EstStr],500,200,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
%
for i=2:NumIter;
   if (DSGEModel.PosteriorSampler==2)|(DSGEModel.PosteriorSampler>=8)|(AcceptedDraws(i)>AcceptedDraws(i-1));
      logPostTmp = -logPosteriorPhiDSGEVAR(PostSample(i,:)',ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
   end;
   LogPost(i) = logPostTmp;
   drawnow;
end;
%
% close the wait dialog
%
if IsHandle(WaitHandle)==1;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;

%
% end of DSGEVARLogPosterior.m
%
