function [ModelEquations,IndentStr] = DSGEVARModelEquations(Phi,A0,VariableNames,XVariableNames,ShockNames,p)
% DSGEVARModelEquations: Writes the DSGE-VAR model in equation form as a string matrix.
%
% USAGE:
%
%       [ModelEquations,IndentStr] = DSGEVARModelEquations(Phi,A0,VariableNames,XVariableNames,ShockNames,p)
%
% REQUIRED INPUT:  Phi (matrix) of dimension nx(np+k) with the parameters on the deterministic
%                      variables and the lagged endogenous variables.
%
%                  A0 (matrix) of dimension nxn with the parameters on the economic shocks.
%
%                  VariableNames (string matrix) with the names of the endogenous variables.
%
%                  XVariableNames (string matrix) with the names of the deterministic variables.
%
%                  ShockNames (string matrix) with the names of the economic shocks.
%
%                  p (integer) with the lag order of the VAR model.
%
% REQUIRED OUTPUT: ModelEquations (string matrix) where each row gives an equation of the DSGE-VAR.
%
%                  IndentStr (string) of length s for indenting the text after wrapping the
%                      "ModelEquations" rows.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 26, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-01-2010: Updated the documentation.
%
% * 19-02-2010: Added the possibility of A0 being empty.
%
% * 05-01-2011: Updated the documentation.
%
% * 14-01-2011: The function now displays "- x" instead of "+ -x".
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
ModelEquations = '';
IndentStr = '';
[n,npk] = size(Phi);
k = npk-(n*p);
if k>0;
   Phi0 = Phi(:,1:k);
   Phi = Phi(:,k+1:npk);
else;
   Phi0 = [];
end;
%
for i=1:n
   VariableName = StringTrim(VariableNames(i,:));
   if size(VariableName,2)<size(VariableNames,2);
      VariableName = [VariableName '(t)' SpaceStr(size(VariableNames,2)-size(VariableName,2))];
   else;
      VariableName = [VariableName '(t)'];
   end;
   MEStr = [VariableName ' = '];
   if i==i;
      IndentStr = SpaceStr(size(MEStr,2));
   end;
   NumPrt = 0;
   if k>0;
      for j=1:k;
         if abs(Phi0(i,j))>1.0e-12;
            XVariableName = StringTrim(XVariableNames(j,:));
            if NumPrt==1;
               if Phi0(i,j)>0;
                  if (length(XVariableName)>=3)&(strcmp(lower(XVariableName(1,1:3)),'con')==1);
                     MEStr = [MEStr sprintf(' + %0.4f*%s',Phi0(i,j),XVariableName)];
                  else;
                     MEStr = [MEStr sprintf(' + %0.4f*%s(t)',Phi0(i,j),XVariableName)];
                  end;
               else;
                  if (length(XVariableName)>=3)&(strcmp(lower(XVariableName(1,1:3)),'con')==1);
                     MEStr = [MEStr sprintf(' - %0.4f*%s',abs(Phi0(i,j)),XVariableName)];
                  else;
                     MEStr = [MEStr sprintf(' - %0.4f*%s(t)',abs(Phi0(i,j)),XVariableName)];
                  end;
               end;
            else;
               if (length(XVariableName)>=3)&(strcmp(lower(XVariableName(1,1:3)),'con')==1);
                  MEStr = [MEStr sprintf('%0.4f*%s',Phi0(i,j),XVariableName)];
               else;
                  MEStr = [MEStr sprintf('%0.4f*%s(t)',Phi0(i,j),XVariableName)];
               end;
               NumPrt = 1;
            end;
         end;
      end;
   end;
   for l=1:p;
      PhiMat = Phi(:,1+(n*(l-1)):l*n);
      for j=1:n;
         if abs(PhiMat(i,j))>1.0e-12;
            VariableName = StringTrim(VariableNames(j,:));
            if NumPrt==1;
               if PhiMat(i,j)>0
                  MEStr = [MEStr sprintf(' + %0.4f*%s(t-%0.0f)',PhiMat(i,j),VariableName,l)];
               else;
                  MEStr = [MEStr sprintf(' - %0.4f*%s(t-%0.0f)',abs(PhiMat(i,j)),VariableName,l)];
               end;
            else;
               MEStr = [MEStr sprintf('%0.4f*%s(t-%0.0f)',PhiMat(i,j),VariableName,l)];
               NumPrt = 1;
            end;
         end;
      end;
   end;
   if isempty(A0)==0;
      for j=1:n;
         if abs(A0(i,j))>1.0e-12;
            ShockName = StringTrim(ShockNames(j,:));
            if NumPrt==1;
               if A0(i,j)>0;
                  MEStr = [MEStr sprintf(' + %0.4f*%s(t)',A0(i,j),ShockName)];
               else;
                  MEStr = [MEStr sprintf(' - %0.4f*%s(t)',abs(A0(i,j)),ShockName)];
               end;
            else;
               MEStr = [MEStr sprintf('%0.4f*%s(t)',A0(i,j),ShockName)];
               NumPrt = 1;
            end;
         end;
      end;
   else;
      VariableName = StringTrim(VariableNames(i,:));
      if NumPrt==1;
         MEStr = [MEStr sprintf(' + epsilon(%s,t)',VariableName)];
      else;
         MEStr = [MEStr sprintf('epsilon(%s,t)',VariableName)];
      end;
   end;
   %
   % concatenate MEStr to ModelEquations
   %
   ModelEquations = strvcat(ModelEquations,MEStr);
end;

%
% end of DSGEVARModelEquations.m
%
