function [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = DSGEVARModestyStatistics(OutputDirectory,ShockMethod,NameOfModel,MaxPH,NumDraws,AllDraws,NumPredPaths,lambda,PME,CurrChain,IsPosterior,n,p)
% DSGEVARModestyStatistics: Computes the modesty statistics for a DSGE-VAR.
%
% USAGE:
%
%       [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = DSGEVARModestyStatistics(OutputDirectory,...
%                  ShockMethod,NameOfModel,MaxPH,NumDraws,AllDraws,NumPredPaths,lambda,PME,CurrChain,IsPosterior,n,p)
%
% REQUIRED INPUT:  OutputDirectory (string vector) with the full path to the base output directory.
%
%                  ShockMethod (string vector) that takes on the value "Direct-" or "WaggonerZha-".
%
%                  NameOfModel (string vector) with the name of the model.
%
%                  MaxPH (integer) with the length of the prediction horizon.
%
%                  NumDraws (integer) with the total number of parameter draws.
%
%                  AllDraws (integer) with the maximum number of parameter draws that could have been used.
%
%                  NumPredPaths (integer) with tht total number of prediction paths per parameter draw.
%
%                  lambda (scalar) with the value of the lambda hyperparameter for the DSGE-VAR.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                      are used, 2 if the DSGE-VAR marginal posterior mode estimates were used, and
%                      3 if the DSGE-VAR joint posterior mode estimates were used when generating
%                      the posterior draws.
%
%                  CurrChain (integer) with the number of the currently used Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                      if draws from the prior are supplied.
%
%                  n (integer) with the number of observed variables.
%
%                  p (integer) with the number of lags.
%
% REQUIRED OUTPUT: MultiPValue (scalar) with the probability that the multivariate modesty statistic
%                              is smaller than one based on unconditional shocks.
%
%                  MeanMultiModesty (vector) with 2 elements. The first is the mean of the multivariate
%                              modesty statistic based on the conditioning consistent shocks and the second
%                              is the mean of this statistic for randomly drawn shocks.
%
%                  MeanUniModesty (vector) with means of the univariate modesty statistics for all observed
%                              variables.
%
%                  MeanUniModestyLZ (vector) with the means of the univariate modesty statistics suggested
%                              by Leeper-Zha (other shocks are zero over examined prediction horizon) for
%                              all observed variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 1, 2011.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-03-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
TotalPaths = NumPredPaths*NumDraws;
MultiModestyStat = zeros(TotalPaths,2);
UniModestyStat = zeros(TotalPaths,n);
UniModestyStatLZ = zeros(NumDraws,n);
LoadIndex = (1:NumPredPaths);
for ParamIndex=1:NumDraws;
   if IsPosterior==1;
      file = FixFilePath([OutputDirectory '\predictions\DSGE-VAR-CondPrediction-Original-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' ShockMethod NameOfModel '-' int2str(CurrChain) '-' int2str(MaxPH) '-' int2str(ParamIndex) '.' int2str(NumDraws) '-' int2str(AllDraws) '.' int2str(NumPredPaths) '.mat']);
   else;
      file = FixFilePath([OutputDirectory '\predictions\DSGE-VAR-CondPrediction-Original-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' ShockMethod NameOfModel '-' int2str(MaxPH) '-' int2str(ParamIndex) '.' int2str(NumDraws) '.' int2str(NumPredPaths) '.mat']);
   end;
   %
   % load the data
   %
   PredData = load(file);
   MultiModestyStat(LoadIndex,:) = PredData.MultiModestyStat;
   UniModestyStat(LoadIndex,:) = PredData.UniModestyStat;
   UniModestyStatLZ(ParamIndex,:) = PredData.UniModestyStatLZ;
   clear('PredData');
   drawnow;
   LoadIndex = LoadIndex+NumPredPaths;
end;
%
% check how often we have a greater value of column 1 than column 2 in MultiModestyStat
%
MultiPValue = sum(MultiModestyStat(:,1)<=MultiModestyStat(:,2))/TotalPaths;
MeanMultiModesty = (1/TotalPaths)*sum(MultiModestyStat);
MeanUniModesty = (1/TotalPaths)*sum(UniModestyStat);
MeanUniModestyLZ = (1/NumDraws)*sum(UniModestyStatLZ);

%
% end of DSGEVARModestyStatistics.m
%
