function DoneCalc = DSGEVARMomentsResponse(thetaDraws,theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,PhiMode,SigmaModeEpsilon,ParameterNames,ParamNumber,CurrChain,IsPosterior,lambda,PME,DSGEModel,CurrINI)
% DSGEVARMomentsResponse: Computes the distribution of the numerical partial derivatives of the means,
%                         standard deviations, and correlations with respect to a change in one structural
%                         parameters using either prior or posterior draws of the DSGE parameters. The
%                         DSGE-VAR parameters are simply the location parameters of the conditional p(oste)riors
%                         for each theta draw.
%
% USAGE:
%
%       DoneCalc = DSGEVARMomentsResponse(thetaDraws,theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,...
%                                         PhiMode,SigmaModeEpsilon,ParameterNames,ParamNumber,CurrChain,IsPosterior,...
%                                         lambda,PME,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  thetaDraws (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                             distribution of the original DSGE model parameters.
%
%                  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                             needs the former field to know which parameter name corresponds to
%                             position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                             "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices
%                             with the parameter values for the distributions. In the case of the normal, beta, gamma,
%                             logistic and gumbel these parameter values are the mean and the standard deviation. For
%                             the invgamma it is the s and q parameters, where q is a positive integer (degrees of
%                             freedom). For the left truncated normal there's a third parameter, the lower bound. For
%                             the uniform the parameter values are the lower and the upper bound. For the student-t
%                             they are the location (mode), the scale and the degrees of freedom parameter, while for
%                             the Cauchy only the location and scale parameters are needed. For the Pareto the parameter
%                             values are for the mode and the shape parameter.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                             truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                             and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  PhiMode (matrix) of dimension nx(np+k) with the parameters on the exogenous and lagged endogenous
%                             variables of the VAR.
%
%                  SigmaModeEpsilon (matrix) of dimension nxn with the residual covariances of the VAR.
%
%                  ParameterNames (string matrix) with the names of the parameters.
%
%                  ParamNumber (integer) with the position of the parameter in theta to change.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                             if draws from the prior are supplied.
%
%                  lambda (scalar) with the value of the lambda hyperparameter.
%
%                  PME (integer)  that takes the value 1 if the DSGE model posterior mode estimates
%                             are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                             used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                             when generating the posterior draws.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 16, 2019.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-04-2019: Finished the function after the call to "DSGEVARMomentsResponseTheta".
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
[NumDraws,NumParam] = size(thetaDraws);
ParamStr = StringTrim(ParameterNames(ParamNumber,:));
%
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
J = eye(n);
if p>1;
   J = [J;zeros(n*(p-1),n)];
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'paramchange');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\paramchange". The computation distributions for the numerical partial derivatives of the change in the parameter "' ParamStr '" to the means, standard deviations, and correlations has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/paramchange". The computation distributions for the numerical partial derivatives of the change in the parameter "' ParamStr '" to the means, standard deviations, and correlations has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% determine the selected shock string
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
% compute the moments responses at the mode values
%
[MeanRespMode,StdRespMode,CorrRespMode] = DSGEVARMomentsResponseTheta(PhiMode,SigmaModeEpsilon,ParameterNames,ParamNumber,theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI);
%
% initialize index for saving
%
SaveAfterDraws = min(NumDraws,100);
NumFiles = ceil(NumDraws/SaveAfterDraws);
%
% begin the loop
%
abort = '';
MeanEstimationTime = 0;
LastDraw = 0;
ThisSave = 0;
PD = [];
mcodevec = zeros(NumDraws,1);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['Parameter Change To "' ParamStr '"' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the distribution of the partial derivatives of the means, standard deviations and correlations from a change in the parameter "' ParamStr '". Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Parameter Change To "' ParamStr '"' PriorHeader],500,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% start the iterations
%
for it=1:NumDraws;
   LastDraw = LastDraw+1;
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if strcmp(abort,'cancel')==1;
         break;
      else;
         ProgressDLG([it/NumDraws MeanEstimationTime]);
      end;
   end;
   %
   % get the VAR location parameters based on the theta draw
   %
   thetaValue = thetaDraws(it,:)';
   tic;
   [Phi,SigmaEpsilon,A0,IdentifyStatus,Status] = GetDSGEVARParameters(thetaValue,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI);
   if Status==1;
      %
      % remove parameters on the exogenous variables
      %
      if k>0;
         Phi0 = Phi(:,1:k);
      else;
         Phi0 = zeros(n,1);
      end;
      Phi = Phi(:,k+1:(n*p)+k);
      if p>1;
         Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
      end;
      %
      % we approximate the mean by J'*inv(I-Phi)*J*Phi0
      %
      MeanMat = J'*((eye(n*p)-Phi)\J)*Phi0;
      %
      % compute the autocovariances for the stacked Z vector
      %
      try;
         [SigmaZ,stat] = DoublingAlgorithmLyapunov(Phi,J*SigmaEpsilon*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      catch;
         vSigmaZ = (eye(n*n*p*p)-kron(Phi,Phi))\kron(J,J)*vec(SigmaEpsilon);
         SigmaZ = reshape(vSigmaZ,n*p,n*p);
      end;
      CovMat = SigmaZ(1:n,1:n);
      StdVec = real(sqrt(diag(CovMat)));
      if length(StdVec)>1;
         CorrVec = vechl(CovMat./(StdVec*StdVec'));
      end;
      CorrResp = [];
      %
      % now make the parameter change
      %
      thetaChange = 0.001*abs(thetaValue);
      IdentMat = eye(length(thetaValue));
      ParamDiff = thetaChange(ParamNumber);
      thetaNew = thetaValue+(ParamDiff*IdentMat(:,ParamNumber));
      %
      % compute the new DSGE-VAR parameters
      %
      [PhiNew,SigmaEpsilonNew,A0,IdentifyStatus,Status] = GetDSGEVARParameters(thetaNew,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI);
      if Status~=1;
         %
         % we try a different change vector
         %
         thetaChange = 0.0001*abs(thetaValue);
         IdentMat = eye(length(thetaValue));
         ParamDiff = thetaChange(ParamNumber);
         thetaNew = thetaValue+(ParamDiff*IdentMat(:,ParamNumber));
         [PhiNew,SigmaEpsilonNew,A0,IdentifyStatus,Status] = GetDSGEVARParameters(thetaNew,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI);
      end;
      if Status==1;
         %
         % we're OK
         %
         if k>0;
            Phi0New = PhiNew(:,1:k);
         else;
            Phi0New = zeros(n,1);
         end;
         PhiNew = PhiNew(:,k+1:(n*p)+k);
         if p>1;
            PhiNew = [PhiNew;eye(n*(p-1)) zeros(n*(p-1),n)];
         end;
         %
         % we approximate the mean by J'*inv(I-Phi)*J*Phi0
         %
         MeanMatNew = J'*((eye(n*p)-PhiNew)\J)*Phi0New;
         if k>0;
            MeanResp = (MeanMatNew-MeanMat)./ParamDiff;
         else;
            MeanResp = zeros(n,1);
         end;
         %
         % compute the autocovariances for the stacked Z vector
         %
         try;
            [SigmaZ,stat] = DoublingAlgorithmLyapunov(PhiNew,J*SigmaEpsilonNew*J',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
         catch;
            vSigmaZ = (eye(n*n*p*p)-kron(PhiNew,PhiNew))\kron(J,J)*vec(SigmaEpsilonNew);
            SigmaZ = reshape(vSigmaZ,n*p,n*p);
         end;
         CovMatNew = SigmaZ(1:n,1:n);
         StdVecNew = real(sqrt(diag(CovMatNew)));
         if length(StdVecNew)>1;
            CorrVecNew = vechl(CovMatNew./(StdVecNew*StdVecNew'));
         end;
         %
         % now compute the change in standard deviations and correlations
         %
         StdResp = (StdVecNew-StdVec)./ParamDiff;
         if length(StdVecNew)>1;
            CorrResp = (CorrVecNew-CorrVec)./ParamDiff;
         end;
      else;
         %
         % base case, we use the default values
         %
         MeanResp = MeanRespMode;
         StdResp = StdRespMode;
         CorrResp = CorrRespMode;
      end;
   else;
      %
      % base case, we use the default values
      %
      MeanResp = MeanRespMode;
      StdResp = StdRespMode;
      CorrResp = CorrRespMode;
   end;
   MeanEstimationTime = (((it-1)/it)*MeanEstimationTime)+((1/it)*toc);
   mcodevec(it) = Status;
   PD(LastDraw).MeanResp = MeanResp;
   PD(LastDraw).StdResp = StdResp;
   PD(LastDraw).CorrResp = CorrResp;
   %
   % check if we should save to disk
   %
   if (LastDraw==SaveAfterDraws)|(it==NumDraws);
      ThisSave = ThisSave+1;
      LastDraw = 0;
      %
      % set up the file name and then save
      %
      if IsPosterior==1;
         file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-VAR-ObsVar-ParamChange-' ParamStr '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
      else;
         file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-VAR-ObsVar-ParamChange-Prior-' ParamStr '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
      end;
      save(file,'PD');
      %
      % restore the data structure
      %
      PD = [];
   end;
   drawnow;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   if IsPosterior==1;
      file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-VAR-ObsVar-ParamChange-' ParamStr '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-1-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
   else;
      file = FixFilePath([DSGEModel.OutputDirectory '\paramchange\DSGE-VAR-ObsVar-ParamChange-Prior-' ParamStr '-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(NumDraws) '.mat']);
   end;
   save(file,'mcodevec','-APPEND');
   DoneCalc = 1;
end;

%
% end of DSGEVARMomentsResponse.m
%
