function [Selvalue,SelCurrChain,SelPME,SelLambda] = DSGEVARPosteriorSampling(DSGEModel,CurrINI,maingui,controls,value,CurrChain,PME,Lambda)
% DSGEVARPosteriorSampling: Runs the posterior sampler of the VAR parameters for a DSGE-VAR.
%
% USAGE:
%
%       [Selvalue,SelCurrChain,SelPME,SelLambda] = DSGEVARPosteriorSampling(DSGEModel,CurrINI,maingui,controls,...
%                                                                           value,CurrChain,PME,Lambda)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure), with initialization information.
%
%                  maingui (handle) to the YADA main dialog.
%
%                  controls (structure), whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
% OPTIONAL INPUT:  value (integer) with the selected position for the available lambda hyperparameter.
%
%                  CurrChain (integer) with the number of the selected Markov chain with posterior draws.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (vector) with the lambda hyperparameter values that can be used.
%
% OPTIONAL OUTPUT: Selvalue (integer) with the selected position for the available lambda hyperparameter.
%
%                  SelCurrChain (integer) with the number of the selected Markov chain with posterior draws.
%
%                  SelPME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  SelLambda (vector) with the lambda hyperparameter values that can be used.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 4, 2010.
%                        This version: March 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-03-2010: Continued work on the function.
%
% * 08-03-2010: Finished the function.
%
% * 09-03-2010: The function now stores the posterior mode estimates to the files with posterior draws.
%
% * 10-03-2010: Made sure that controls.tab6 gets focus when the matlab version supports this.
%
% * 19-03-2010: Added an extra input variable for "GetDSGEVARPostDrawsSelection".
%
% * 26-10-2010: Added info about the lag order to the wait dialog.
%
% * 08-12-2010: The AiM solution time "SolveTime" is now computed until the state-space form has been
%               obtained.
%
% * 16-12-2010: The function now supports the Klein and Sims DSGE model solvers.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 12-01-2011: Added additional support for unix and mac paths.
%
% * 13-01-2011: Fixed path errors for the posterior mode estimated when "PME" is 2 or 3.
%
% * 24-08-2011: Made sure "F" and "B0" are included in the "ModelParameters" structure via the "YADA" field.
%
% * 25-11-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 20-01-2012: The function now checks if the user allows or has specified unit roots in the DSGE model.
%               If unit roots are not allowed for, then YADA only accepts draws of the VAR parameters that
%               are consistent with stationarity.
%
% * 23-01-2012: Streamlined the code so that unit roots are not checked for. Instead, the function forces
%               the parameters to be consistent with stationarity.
%
% * 09-02-2012: Took "DSGEModel.StationaryVAR" into account.
%
% * 11-05-2012: The function now also stores the "thetaPostSample" draws that were used to disk.
%
% * 14-05-2012: And now also "PostSample" draws.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-08-2013: Took into account that the user can now also select the exact number of posterior draws.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 16-01-2015: Began adapting the function for use with YADA's sequential tools.
%
% * 17-03-2015: Switched to the new IsHandle function.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 01-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal proposal
%               density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 14-08-2017: The function now supports the SMC with likelihood tempering posterior sampler.
%
% * 08-09-2017: Took into account that "LoadDSGEVARPosteriorDraws" provides the "Weights" output variable.
%
% * 11-09-2017: Took into account that SMC with likelihood tempering doesn't allow for sub-samples of the
%               DSGE model posterior draws to be used when drawing VAR parameters.
%
% * 31-07-2018: The function supports user written model solver function.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 16-10-2018: The function now supports the SMC with data tempering posterior sampler.
%
% * 03-12-2018: The function should now support IS based on MitISEM.
%
% * 22-01-2019: Updated the documentation.
%
% * 27-03-2019: Ensured that the variable "Weights" has the correct dimension for the "ParameterSummary"
%               function.
%
% * 10-12-2019: Took into account that the DSGEModel.ModelSolverFile now requires the three additional
%               input variables "DSGEModel.StateVariableNames", "DSGEModel.StateEquationNames" and 
%               "DSGEModel.StateShockNames".
%
% * 08-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 04-01-2022: Updated the documentation.
%
% * 13-01-2022: Took DSGEModel.ShowEstimationLog into account.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls for feval.
%
% * 06-01-2023: Updated the documentation.
%
% * 13-03-2024: Updated the documentation.
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
if nargin==4;
   value = [];
   CurrChain = [];
   PME = [];
   Lambda = [];
end;
if nargout==4;
   Selvalue = value;
   SelCurrChain = CurrChain;
   SelPME = PME;
   SelLambda = Lambda;
end;
%
% make sure the output directory is exists
%
stat = MakeDir(DSGEModel.OutputDirectory,'dsge-var');
if stat~=1;
   if ispc==1;
      SetLogText(strvcat(' ','Couldn''t create the directory:',['  ' DSGEModel.OutputDirectory '\dsge-var'],'Posterior sampling was aborted.'));
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\dsge-var". Posterior sampling has therefore been aborted.'];
   else;
      SetLogText(strvcat(' ','Couldn''t create the directory:',['  ' DSGEModel.OutputDirectory '/dsge-var'],'Posterior sampling was aborted.'));
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/dsge-var". Posterior sampling has therefore been aborted.'];
   end;
   if isempty(CurrINI.Sequence)==1;
      About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% make sure that we have at least 10 draws
%
DrawsStr = get(controls.posterior.draws,'String');
NDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
DrawsPercentStr = get(controls.posterior.usepostdraws,'String');
DrawsPercent = str2num(strrep(StringTrim(DrawsPercentStr(DSGEModel.PostDrawsPercentValue,:)),'%',''));
DrawsNum = ceil(DrawsPercent*NDraws/100);
if DrawsNum<10;
   SetLogText('YADA requires that you should have at least 10 draws.');
   if isempty(CurrINI.Sequence)==1;
      txt = ['You have selected to use a draws percentage of ' int2str(DrawsPercent) '% for posterior sampling of the VAR parameters in the DSGE-VAR. With a total of ' int2str(NDraws) ' posterior draws of the DSGE model parameters, this results in ' int2str(DrawsNum) ' draws of the VAR parameters. YADA requires that you should have at least 10 draws, although a lot more is adviced. Please change your posterior draws settings before you run this function.'];
      About(txt,'information','VAR Posterior Sampling',300,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check
%
if isempty(value)==1;
   [value,CurrChain,PME,Lambda] = GetDSGEVARPostDrawsSelection('VAR Posterior Sampling',DSGEModel,controls,CurrINI,1);
   if value==0;
      return;
   end;
   if nargout==4;
      Selvalue = value;
      SelCurrChain = CurrChain;
      SelPME = PME;
      SelLambda = Lambda;
   end;
end;
%
% we're ready to rock'n roll
%
if DSGEModel.ShowEstimationLog==1;
   set(controls.about,'UserData',0);
   drawnow;
   oldtab = get(controls.tab1,'UserData');
   set(maingui,'CurrentObject',controls.tab6);
   YADAGUI('tab');
   if MatlabNumber>=7;
      uicontrol(controls.tab6);
   end;
   set(controls.about,'UserData',1);
   drawnow;
end;
%
% check if we should add some diving line to the log-text control
%
if isempty(CurrINI.Sequence)==1;
   LogText = get(controls.estimation_log,'String');
   if isempty(LogText)==0;
      SetLogText(strvcat(' ','*************************************************************************','*    Y    A    D    A       Y    A    D    A       Y    A    D    A     *','*************************************************************************'));
   end;
end;
%
% we can now prepare for VAR sampling
%
lambda = Lambda(value);
%
SetLogText(strvcat(' ',['Initializing VAR posterior sampling for lambda=' num2str(lambda,'%0.4f') '...'],[' * Attempting to load posterior mode data file...']));
%
% check which mode file to load
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
if PME==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
   ModeType = 'DSGE model';
elseif PME==2;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
   ModeType = 'DSGE-VAR marginal';
elseif PME==3;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
   ModeType = 'DSGE-VAR joint';
end;
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==1;
   SetLogText(strvcat('   STATUS: OK',[' * Attempting to access ' ModeType ' posterior mode data file...']));
else;
   SetLogText(strvcat(' ',['Unable to load the file "' GetFilename(ModeFile) '".']));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
%
thetaIndex = ModeData.thetaIndex;
UniformBounds = ModeData.UniformBounds;
LowerBound = ModeData.LowerBound;
thetaPositions = ModeData.thetaPositions;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
ModelParameters = ModeData.ModelParameters;
ParameterNames = ModeData.ParameterNames;
thetaMode = ModeData.thetaMode;
NumParam = length(thetaMode);
if PME>=2;
   DSGEModel.StateShockNames = ModeData.DSGEModel.StateShockNames;
   DSGEModel.DSGEVARShocks = ModeData.DSGEModel.DSGEVARShocks;
end;
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
ErrorFile = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
end;
%
% Test if there are any problems
%
firstUpdatePos = 0;
lastUpdatePos = 0;
firstInitializePos = 0;
lastInitializePos = 0;
if isempty(ErrorStr)==1;
   if DSGEModel.UseOwnInitialState==1;
      if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
         DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
      else;
         DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
      end;
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      SetLogText(strvcat('   STATUS: OK',[' * Checking ' GetName(DSGEModel.UpdateParameterFile) '...']));
      if MatlabNumber>=8.0;
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch ME;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError(ME)];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      else;
         try;
            firstUpdatePos = length(fieldnames(ModelParameters))+1;
            ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
            lastUpdatePos = length(fieldnames(ModelParameters));
         catch;
            ErrorStr = ['Error found in "' GetName(DSGEModel.UpdateParameterFile) '.m". Message caught is: ' YADALastError];
            ErrorFile = DSGEModel.UpdateParameterFile;
         end;
      end;
   end;
end;
%
% Check if we should exit
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% Try to solve the DSGE model at the posterior mode
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE model has a unique convergent','   solution for posterior mode parameter values...'));
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   SolverStr = 'AiM';
elseif DSGEModel.ModelSolver==2;
   SolverStr = 'Klein';
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   SolverStr = 'Gensys';
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==4;
   SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
if mcode~=1;
   %
   % There doesn't exist a unique convergent solution for the initial parameter values.
   %
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" The posterior mode values are no longer valid.'];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% do it again to measure the time
%
tic;
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
elseif DSGEModel.ModelSolver==4;
   [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
SolveTime = toc;
if isempty(DSGEModel.UnitRootStates)==1;
   F1 = F;
else;
   KeepPos = GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates);
   F1 = F(KeepPos,KeepPos);
end;
if (((max(abs(eig(F1)))>=1)|(IsAlmostEqual(max(abs(eig(F1))),1)==1))&(DSGEModel.AllowUnitRoot==0))|((isempty(DSGEModel.UnitRootStates)==1)&(DSGEModel.AllowUnitRoot==1)&(max(abs(eig(F1)))>1));
   SetLogText(strvcat('   STATUS: Non-stationary state equation',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The largest eigenvalue of the state equation is ' num2str(max(abs(eig(F1)))) '. The state equation is therefore non-stationary. The posterior mode values are no longer valid.'];
      About(txt,'information','YADA - DSGE Model Problem',120,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check the measurement equation file for internal validity
%
if isempty(ErrorStr)==1;
   %
   % add the values of F and B0 to the ModelParameters.YADA field
   %
   [B0Alt,KeepVar] = RemoveRedundantColumns(B0);
   ModelParameters.YADA.F = F;
   ModelParameters.YADA.B0 = B0Alt;
   SetLogText(strvcat('   STATUS: OK',['   DSGE model solver:         ' SolverStr],['   Maximum absolute error:    ' num2str(MaxAbsError)],['   Solution time:             ' num2str(SolveTime) ' seconds'],[' * Checking ' GetName(DSGEModel.MeasurementEquationFile) '...']));
   if MatlabNumber>=8.0;
      try;
         [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
      catch ME;
         ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
         ErrorFile = DSGEModel.MeasurementEquationFile;
      end;
   else;
      try;
         [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
      catch;
         ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
         ErrorFile = DSGEModel.MeasurementEquationFile;
      end;
   end;
end;
%
% Check if we should exit
%
if isempty(ErrorStr)==0;
   SetLogText(strvcat(['   STATUS: Bad input file ' GetName(ErrorFile)],' '));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
      drawnow;
   end;
   return;
end;
%
% check if we can compute the DSGE-VAR prior
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE-VAR prior can be calculated...'));
%
% initialize dimensions
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
npk = (n*p)+k;
%
% create sample dates
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Check if we should adjust this based on p, the lag order
%
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
end;
%
% actual sample that we will use
%
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
DSGEModel.FirstPeriod = FirstPeriod;
DSGEModel.LastPeriod = LastPeriod;
DSGEModel.KalmanFirstObservation = KalmanFirstObservation;
%
% get the data
%
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
T = size(y,2);
if k>0;
   DetProductMoments = CalcDeterministicProductMoments(x,X);
else;
   DetProductMoments = [];
end;
%
% check if we can compute the prior
%
[Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if PriorStatus==0;
   SetLogText(strvcat('   STATUS: Prior moment matrix GammaYY is not invertible.',' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['The sample average of the implied non-central population moment matrix GammaYY(theta) is not invertible at the ' StringTrim(PostModeStr(SelectedMode,:)) ' values. This matrix is required for setting up the DSGE-VAR prior. YADA has therefore aborted from psoterior sampling of the DSGE-VAR model.'];
      About(txt,'information','YADA - DSGE Model Problem',300,500,CurrINI);
      drawnow;
   end;
   return;
end;
%
SetLogText(strvcat('   STATUS: OK',' * Checking if the DSGE-VAR posterior can be calculated...'));
%
% compute sample product moments
%
GammaHatyy = (1/T)*y*y';
GammaHatyY = (1/T)*y*Y';
GammaHatYY = (1/T)*Y*Y';
%
[PhiMode,SigmaEpsilon,Gyy,GyY,GYY,Status] = DSGEVARParameters(A,H,R,F,B0,lambda,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if Status==0;
   SetLogText(strvcat('   STATUS: Unable to compute VAR parameters at posterior mode',['           for lambda=' num2str(lambda,'%0.4f')],' '));
   if isempty(CurrINI.Sequence)==1;
      txt = ['YADA wasn''t able to calculated the VAR parameters at the posterior mode estimates of the DSGE model parameters. The matrix GammaHatYY with a mixture of non-central prior product moments and non-central data product moments is not invertible. Posterior sampling has therefore been aborted.'];
      About(txt,'error','Error - VAR Parameters',300,500,CurrINI);
      drawnow;
   end;
   return;
end;
clear('Gyy','GyY','GYY');
SetLogText(strvcat('   STATUS: OK',' * Checking if the structural shocks can be identified...'));
%
if isinf(lambda)==0;
   SigmaModeEpsilon = ((lambda*T)/((lambda*T)+n+1))*SigmaEpsilon;
else;
   SigmaModeEpsilon = SigmaEpsilon;
end;
[A0Mode,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaModeEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
if IdentifyStatus==1;
   SetLogText(strvcat('   STATUS: OK',' * Loading DSGE-VAR posterior draws of DSGE model parameters...'));
else;
   SetLogText(strvcat('   STATUS: Unable to identify structural shocks',' * Loading DSGE-VAR posterior draws of DSGE model parameters...'));
end;
%
% keep the posterior mode values of the DSGE model
%
Mode.A = A;
Mode.H = H;
Mode.R = R;
Mode.F = F;
Mode.B0 = B0;
%
% see if we can load the estimated parameters
%
[PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEVARPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain);
NumDraws = length(LogPost);
clear('LogPost','AcceptedDraws');
drawnow;
%
% select which posterior draws to use
%
if DSGEModel.PosteriorSampler<=7;
   thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
   PostSample = PostSample(NumBurnin+1:NumDraws,:);
   NumDraws = NumDraws-NumBurnin;
   %
   % check which draws we should use
   %
   UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
   if length(UseDraws)<NumDraws;
      thetaPostSample = thetaPostSample(UseDraws,:);
      PostSample = PostSample(UseDraws,:);
      NumPostDraws = length(UseDraws);
   else;
      NumPostDraws = NumDraws;
   end;
else;
   NumPostDraws = NumDraws;
end;
%
% check if we have enough memory
%
SetLogText(strvcat(['   Number of draws available: ' int2str(NumDraws)],['   Selected number of draws:  ' int2str(NumPostDraws)],' * Testing if memory is sufficient...'));
%
tic;
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      PhiDraws = zeros(NumPostDraws,n*npk);
      SigmaDraws = zeros(NumPostDraws,n*(n+1)/2);
      if IdentifyStatus==1;
         A0Draws = zeros(NumPostDraws,n*n);
      end;
   catch ME;
      ErrorStr = ['There was an error creating the matrix to store the posterior draws. YADA caught the following error message: ' YADALastError(ME)];
   end;
else;
   try;
      PhiDraws = zeros(NumPostDraws,n*npk);
      SigmaDraws = zeros(NumPostDraws,n*(n+1)/2);
      if IdentifyStatus==1;
         A0Draws = zeros(NumPostDraws,n*n);
      end;
   catch;
      ErrorStr = ['There was an error creating the matrix to store the posterior draws. YADA caught the following error message: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   SampleMatrixTime = toc;
   SetLogText(strvcat(' ','Too many total draws: memory problem','Posterior sampling was aborted.'));
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Memory Problem',500,600,CurrINI);
      drawnow;
   end;
   return;
end;
SampleMatrixTime = toc;
SetLogText(strvcat('   STATUS: OK',['   Computation time:       ' num2str(SampleMatrixTime) ' seconds']));
%
% 2. Number of posterior sample batches to save per chain
%
if DSGEModel.PosteriorSampler<=7;
   BatchStr = get(controls.posterior.batch,'String');
   NumBatches = str2num(StringTrim(strrep(BatchStr(DSGEModel.SampleBatchValue,:),',','')));
   if NumBatches>NumPostDraws;
      NumBatches = NumPostDraws;
   end;
   NumBatchDraws = NumPostDraws/NumBatches;
   if IsInteger(NumBatchDraws)==0;
      NumBatchDraws = NumPostDraws;
      NumBatches = 1;
   end;
else;
   NumBatchDraws = NumPostDraws;
   NumBatches = 1;
end;
%
% 3. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% number of tempering stages
%
TempStagesStr = get(controls.posterior.temperingstages,'String');
TempStages = str2num(StringTrim(TempStagesStr(DSGEModel.TemperingStagesValue,:)));
%
if DSGEModel.PosteriorSampler<=7;
   SetLogText(strvcat(' ',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Number of posterior draws:          ' int2str(NumPostDraws)],[' * Number of sample batches:           ' int2str(NumBatches)],[' * Current Markov chain:               ' int2str(CurrChain)],[' * Posterior mode estimator:           ' ModeType ' posterior mode'],[' * Effective sample size:              ' int2str(T)],[' * Estimation sample:                  ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:       ' int2str(n)],[' * Number of lags:                     ' int2str(p)],[' * Number of exogenous variables:      ' int2str(k)],[' * Hyperparameter lambda:              ' num2str(lambda,'%0.4f')],' '));
else;
   SetLogText(strvcat(' ',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Number of posterior draws:          ' int2str(NumPostDraws)],[' * Current sampling chain:             ' int2str(CurrChain)],[' * Effective sample size:              ' int2str(T)],[' * Estimation sample:                  ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.SubEndYear ':' DSGEModel.SubEndPeriod],[' * Number of observed variables:       ' int2str(n)],[' * Number of lags:                     ' int2str(p)],[' * Number of exogenous variables:      ' int2str(k)],[' * Hyperparameter lambda:              ' num2str(lambda,'%0.4f')],' '));
end;
%
% check if we can locate results on disk
%
PostSampleStr = [int2str(DSGEModel.InverseHessianEstimator) '.' int2str(DSGEModel.PosteriorSampler)];
if DSGEModel.PosteriorSampler==3;
   PostSampleStr = [PostSampleStr '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==4;
   PostSampleStr = [PostSampleStr '-Fixed-' int2str(DSGEModel.FixedNumParamBlocks)];
elseif DSGEModel.PosteriorSampler==5;
   PostSampleStr = [PostSampleStr '-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==6;
   PostSampleStr = [PostSampleStr '-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2))];
elseif DSGEModel.PosteriorSampler==7;
   PostSampleStr = [PostSampleStr '-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-Student-' int2str(DSGEModel.StudenttDegFree)];
elseif DSGEModel.PosteriorSampler==8;
   PostSampleStr = [int2str(DSGEModel.PosteriorSampler) '-' int2str(TempStages) '.' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm)];
elseif DSGEModel.PosteriorSampler==9;
   PostSampleStr = [int2str(DSGEModel.PosteriorSampler) '-' int2str(DSGEModel.NumParamBlocks) '.' int2str(DSGEModel.ResamplingAlgorithm)];
elseif DSGEModel.PosteriorSampler==10;
   PostSampleStr = [PostSampleStr '-' int2str(DSGEModel.ISMaxMixCompValue) '.' int2str(DSGEModel.ISInitialDF) '.' int2str(DSGEModel.ISWeightShareValue) '.' int2str(DSGEModel.ISAddedMixWeightValue) '.' int2str(DSGEModel.ISAddedDF)];
end;
NBatchDraws = 1;
BatchIndex = (1:NumBatchDraws);
StartBatch = 1;
StartPostDraws = 1;
LoadedOldData = 0;
if DSGEModel.OverwriteDraws==0;
   ExitLoop = 0;
   for NBatch=1:NumBatches;
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\dsge-var\VAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '-' int2str(NumPostDraws) '.' int2str(NumDraws) '.mat']);
      if FileExist(PostSampleFile)==1;
         %
         % load data and add to matrices
         %
         VARData = load(PostSampleFile);
         PhiDraws(BatchIndex,:) = VARData.BatchPhiDraws;
         SigmaDraws(BatchIndex,:) = VARData.BatchSigmaDraws;
         if (IdentifyStatus==1)&(isempty(VARData.BatchA0Draws)==0);
            A0Draws(BatchIndex,:) = VARData.BatchA0Draws;
         end;
         %
         BatchIndex = BatchIndex+NumBatchDraws;
         LoadedOldData = 1;
         clear('VARData');
      else;
         ExitLoop = 1;
         break;
      end;
   end;
   if (ExitLoop==1)|(NBatch==NumBatches);
      StartBatch = NBatch;
      StartPostDraws = max(BatchIndex)-NumBatchDraws+1;
   end;
end;
%
if ((DSGEModel.OverwriteDraws==1)|(LoadedOldData==0))&(DSGEModel.PosteriorDrawsValue>1);
   %
   % Check if we should try to find data to load
   % The same number of batches is required
   %
   NPDrawsValue = DSGEModel.PosteriorDrawsValue-1;
   if DSGEModel.PosteriorDrawsValue==174;
      %
      % need to compare such that only fewer draws can be used
      %
      DrawsStr = get(controls.posterior.draws,'String');
      LowerValue = 0;
      while LowerValue==0;
         NPostDraws = str2num(StringTrim(strrep(DrawsStr(NPDrawsValue,:),',','')));
         if NPostDraws<=NumPostDraws;
            LowerValue = 1;
         else;
            NPDrawsValue = NPDrawsValue-1;
         end;
      end;
   end;
   PostSampleFile = '';
   while NPDrawsValue>=1;
      NPDraws = str2num(StringTrim(strrep(DrawsStr(NPDrawsValue,:),',','')));
      NBatches = NumBatches;
      if NBatches>NPDraws;
         NBatches = NPDraws;
      end;
      ODraws = ceil((NumPostDraws/NumDraws)*NPDraws);
      %
      % determine the number of draws that are available
      %
      %
      % check if output exists
      %
      for NBatch=1:NBatches;
         PostSampleFileI = FixFilePath([DSGEModel.OutputDirectory '\dsge-var\VAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NBatches) '-' int2str(ODraws) '.' int2str(NPDraws) '.mat']);
         if FileExist(PostSampleFileI)==1;
            PostSampleFile = PostSampleFileI;
         else;
            if NBatch>1;
               NBatch = NBatch-1;
            end;
            break;
         end;
      end;
      %
      % Go to the next entry
      %
      if isempty(PostSampleFile)==1;
         NPDrawsValue = NPDrawsValue-1;
      else;
         break;
      end;
   end;
   %
   % check if the PostSampleFile is empty. if not, disp NBatch, NBatches, NPDraws
   %
   if isempty(PostSampleFile)==0;
      if isempty(CurrINI.Sequence)==1;
         txt = ['YADA has located ' int2str(ceil(ODraws*(NBatch/NBatches))) ' previous posterior draws for lambda=' num2str(lambda,'%0.4f') '. Would you like to include these in your ' int2str(NumPostDraws) ' draws?'];
         answer = Query(txt,'question',200,'VAR Posterior Sampling',500,'no',CurrINI);
         drawnow;
      else;
         answer = CurrINI.Sequence.QueryAnswers{1};
      end;
      if strcmp(lower(answer),'yes')==1;
         %
         % first we load the data
         %
         NTempBatchDraws = ODraws/NBatches;
         %
         BatchInd = (1:NTempBatchDraws);
         %
         for i=1:NBatch;
            PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\dsge-var\VAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(i) '.' int2str(NBatch) '-' int2str(ODraws) '.' int2str(NPDraws) '.mat']);
            VARData = load(PostSampleFile);
            PhiDraws(BatchInd,:) = VARData.BatchPhiDraws;
            SigmaDraws(BatchInd,:) = VARData.BatchSigmaDraws;
            if (IdentifyStatus==1)&(isempty(VARData.BatchA0Draws)==0);
               A0Draws(BatchInd,:) = VARData.BatchA0Draws;
            end;
            BatchInd = BatchInd+NTempBatchDraws;
         end;
         StartPostDraws = ceil(ODraws*(NBatch/NBatches))+1;
         clear('VARData');
         %
         % save to file if NumBatches>1
         %
         if NumBatches>1;
            BatchIndex = (1:NumBatchDraws);
            %
            % how many batches can we store
            %
            for i=1:NumBatches;
               %
               % check if we should store data to file
               %
               BatchTest = SigmaDraws(BatchIndex,1);
               if sum(BatchTest~=0)==NumBatchDraws;
                  %
                  % we have a full sample
                  %
                  PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\dsge-var\VAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(i) '.' int2str(NumBatches) '-' int2str(NumPostDraws) '.' int2str(NumDraws) '.mat']);
                  BatchPhiDraws = PhiDraws(BatchIndex,:);
                  BatchSigmaDraws = SigmaDraws(BatchIndex,:);
                  if IdentifyStatus==1;
                     BatchA0Draws = A0Draws(BatchIndex,:);
                  else;
                     BatchA0Draws = [];
                  end;
                  %
                  % save to file
                  %
                  SetLogText(strvcat(['   Saving:           ' GetFilename(PostSampleFile)],['   Time:             ' StringTrim(datestr(now,14)) ', ' GetCurrentDate]));
                  save(PostSampleFile,'BatchPhiDraws','BatchSigmaDraws','BatchA0Draws');
                  drawnow;
                  pause(0.02);
                  %
                  % clear batched matrices from memeory
                  %
                  clear('BatchPhiDraws','BatchSigmaDraws','BatchA0Draws');
                  %
                  BatchIndex = BatchIndex+NumBatchDraws;
               else;
                  NBatchDraws = sum(BatchTest~=0)+1;
                  StartBatch = i;
                  break;
               end;
            end;
         else;
            NBatchDraws = StartPostDraws;
         end;
      end;
   end;
end;
%
% test if we have already loaded all data
%
if StartPostDraws>NumPostDraws;
   %
   % make sure that we shall not run any additional draws
   %
   SetLogText('VAR posterior sampling has already completed!');
   if isempty(CurrINI.Sequence)==1;
      txt = ['The posterior sampler for the VAR parameters with lambda=' num2str(lambda,'%0.4f') ' and p=' int2str(p) ' has already been run and stored to disk. If you wish to re-run the posterior sampling, please check mark the "Overwrite old draws" checkbox in the "DSGE Posterior Sampling" frame on the "Settings" tab.'];
      About(txt,'information','YADA - VAR Posterior Sampling',180,500,CurrINI);
      drawnow;
   end;
end;
%
% deal with the random number generator
%
if (StartPostDraws<=NumPostDraws)&(isinf(lambda)==0);
   if DSGEModel.RandomNumberValue==1;
      rand('state',0);
      randn('state',0);
      %
      if StartPostDraws>1;
         PhiTemp = randn(n*npk,StartPostDraws-1);
         InvWishTmp1 = randn(n,StartPostDraws-1);
         %
         % compute the number of degrees of freedom for the inverted Wishart
         %
         df = ((1+lambda)*T)-npk;
         for i=1:n;
            if df+1-i>=30;
               InvWishTmp2 = randn(1,StartPostDraws-1);
            else;
               InvWishTmp2 = randn(df+1-i,StartPostDraws-1);
            end;
         end;
      end;
      clear('PhiTemp','InvWishTmp1','InvWishTmp2')
   else;
      %
      % we set the state based on the current time
      %
      rand('state',sum(100*clock));
      randn('state',sum(100*clock));
   end;
end;
%
% intialize
%
abort = '';
NBatch = StartBatch;
Psi = zeros(n,n*p);
if p>1;
   Psi = [Psi;eye(n*(p-1)) zeros(n*(p-1),n)];
end;
%
% initialize the PostSampleTime variable
%
tic;
PostSampleTime = toc;
%
% check if we should show a progress dialog
%
if StartPostDraws<=NumPostDraws;
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(NumPostDraws) ' VAR posterior draws for lambda=' num2str(lambda,'%0.4f')];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Draw number:';
      %
      ProgressStructure.name = 'VAR Posterior Sampling';
      ProgressStructure.CurrINI = CurrINI;
      mywait = ProgressDLG(0,ProgressStructure);
      set(mywait,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   else;
      %
      % show a wait dialog
      %
      txt = ['Please wait while YADA runs the VAR posterior sampler for the DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and p=' int2str(p) '. The total number of posterior draws is ' int2str(NumPostDraws) '. Sampling was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
      WaitHandle = WaitDLG(txt,'information','VAR Posterior Sampling',500,200,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
end;
%
% start the posterior draws
%
tic;
for NPostDraws=StartPostDraws:NumPostDraws;
   %
   % update the progress dialog
   %
   if DSGEModel.ShowProgress==1;
      abort = get(mywait,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         break;
      else;
         if NPostDraws<=NumPostDraws;
            ProgressDLG([NPostDraws/NumPostDraws NPostDraws]);
         end;
      end;
   end;
   %
   % try to solve the DSGE model
   %
   theta = thetaPostSample(NPostDraws,:);
   [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta',thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if status~=1;
      %
      % just in case,solve the model at the posterior mode or initial parameter values (this should
      % only happen it someone has manipulated the model relative to the posterior draws of theat, i.e.,
      % when the DSGE model is different from the one used when sampling theta
      %
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   end;
   %
   DrawParam = 1;
   while DrawParam>=1;
      %
      % compute the VAR posterior parameters
      %
      if isinf(lambda)==0
         [PhiVAR,SigmaVAR,Gammayy,GammayY,GammaYY,Status] = DSGEVARParameters(A,H,R,F,B0,lambda,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
         %
         % draw a value for SigmaEpsilon
         %
         SigmaEpsilon = InvWishartRndFcn(((1+lambda)*T)*SigmaVAR,((1+lambda)*T)-npk);
         %
         % draw a value for Phi
         %
         PhiCov = (GammaYY+((1/lambda)*GammaHatYY))\eye(npk);
         Phi = MultiNormalRndFcn(vec(PhiVAR),(1/(T*lambda))*kron(PhiCov,SigmaEpsilon),1);
      else;
         [Gammayy,GammaYY,GammayY,PriorStatus] = DSGEVARPrior(A,H,R,F,B0,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
         Phi = GammayY/GammaYY;
         SigmaEpsilon = Gammayy-(Phi*GammayY');
      end;
      %
      % check if VAR must be stationary
      %
      if DSGEModel.StationaryVAR==1;
         %
         % check the eigenvalues of the VAR parameters
         %
         PhiMat = reshape(vec(Phi),n,npk);
         Psi(1:n,:) = PhiMat(:,k+1:npk);
         if max(abs(eig(Psi)))<1;
            DrawParam = 0;
         else;
            DrawParam = DrawParam+1;
            if DrawParam>100;
               %
               % use the posterior mode value of Phi and SigmaEpsilon
               %
               Phi = PhiMode;
               SigmaEpsilon = SigmaModeEpsilon;
               DrawParam = 0;
            elseif DrawParam>10;
               %
               % use the posterior mode value of theta
               %
               A = Mode.A;
               H = Mode.H;
               R = Mode.R;
               F = Mode.F;
               B0 = Mode.B0;
            end;
         end;
      else;
         DrawParam = 0;
      end;
   end;
   %
   % store draws
   %
   PhiDraws(NPostDraws,:) = vec(Phi)';
   SigmaDraws(NPostDraws,:) = vech(SigmaEpsilon)';
   %
   % check if we should compute the implies A0 matrix
   %
   if IdentifyStatus==1;
      [A0,IdStat] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
      A0Draws(NPostDraws,:) = vec(A0)';
   end;
   %
   if NBatchDraws<NumBatchDraws;
      %
      % increase
      %
      NBatchDraws = NBatchDraws+1;
   else;
      %
      % save a batch
      %
      PostSampleFile = FixFilePath([DSGEModel.OutputDirectory '\dsge-var\VAR-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NBatch) '.' int2str(NumBatches) '-' int2str(NumPostDraws) '.' int2str(NumDraws) '.mat']);
      BatchPhiDraws = PhiDraws(BatchIndex,:);
      BatchSigmaDraws = SigmaDraws(BatchIndex,:);
      if IdentifyStatus==1;
         BatchA0Draws = A0Draws(BatchIndex,:);
      else;
         BatchA0Draws = [];
      end;
      %
      SetLogText(strvcat(['   Saving:           ' GetFilename(PostSampleFile)],['   Time:             ' StringTrim(datestr(now,14)) ', ' GetCurrentDate]));
      save(PostSampleFile,'BatchPhiDraws','BatchSigmaDraws','BatchA0Draws','PhiMode','SigmaModeEpsilon','A0Mode');
      drawnow;
      pause(0.02);
      if NBatch==NumBatches;
         save(PostSampleFile,'thetaPostSample','PostSample','Weights','-APPEND');
      end;
      %
      % clear batched matrices from memory
      %
      clear('BatchPhiDraws','BatchSigmaDraws','BatchA0Draws');
      %
      NBatch = NBatch+1;
      NBatchDraws = 1;
      BatchIndex = BatchIndex+NumBatchDraws;
   end;
   drawnow;
end;
PostSampleTime = toc;
clear('thetaPostSample','PostSample');
%
% close the current waitbar
%
if StartPostDraws<=NumPostDraws;
   if DSGEModel.ShowProgress==1;
      if IsHandle(mywait)==1;
         set(mywait,'UserData','done');
         close(mywait);
         drawnow;
         pause(0.02);
      end;
   else;
      if IsHandle(WaitHandle)==1;
         set(WaitControls.text,'UserData','done');
         delete(WaitHandle);
         drawnow;
         pause(0.02);
      end;
   end;
end;
%
% update the estimation log
%
if StartPostDraws<=NumPostDraws;
   if strcmp(abort,'cancel')==0;
      SetLogText(strvcat(' ','VAR posterior sampling completed!',[' * Time:                               ' StringTrim(datestr(now,14)) ', ' GetCurrentDate],[' * Posterior sampling time:            ' SecondsToHoursStr(PostSampleTime) ' (hrs:min:sec)']));
      drawnow;
      pause(0.02);
   else;
      SetLogText('VAR Posterior sampling aborted.');
      drawnow;
      pause(0.02);
      return;
   end;
end;
%
% collect summary information about the VAR parameters
%
SetLogText(strvcat(' ','Computing summary information about the parameters...',' * Phi - The VAR parameters...',[' * Time:                               ' StringTrim(datestr(now,14))]));
tic;
if isempty(Weights)==1;
   Weights = ones(NumPostDraws,1);
end;
PhiParData = ParameterSummary(PhiDraws,Weights);
SetLogText(strvcat('   DONE',' * Sigma - The residual covariances...',[' * Time:                               ' StringTrim(datestr(now,14))]));
SigmaParData = ParameterSummary(SigmaDraws,Weights);
if IdentifyStatus==1;
   SetLogText(strvcat('   DONE',' * A0 - The contemporaneous impulse responses...',[' * Time:                               ' StringTrim(datestr(now,14))]));
   A0ParData = ParameterSummary(A0Draws,Weights);
else;
   A0ParData = []
end;
ParTime = toc;
SetLogText(strvcat('Computation completed!',[' * Estimation time:                    ' SecondsToHoursStr(ParTime) ' (hrs:min:sec)']));
%
% Ready the print the results
%
PostSampleTextFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorSamplingSummary-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' PostSampleStr '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(NumPostDraws) '.' int2str(NumDraws) '.txt']);
SetLogText(strvcat(' ','Writing posterior sampling summary results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: ' GetFilename(PostSampleTextFile)]));
PrintDSGEVARPosteriorVARSampling(PostSampleTextFile,DSGEModel,CurrINI,PhiParData,SigmaParData,A0ParData,lambda,n,p,k,PME,NumPostDraws,NumDraws,CurrChain,NumChains,PostSampleTime);
%
% display the results
%
if isempty(CurrINI.Sequence)==1;
   TextGUI(PostSampleTextFile,['VAR Posterior Sampling - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
else;
   SetLogText(' ');
end;

%
% end of DSGEVARPosteriorSampling.m
%
