function [PredPaths,PredData,PredEventData,YObsEventData] = DSGEVARPredictionPathsParam(Phi0,Phi,SigmaEpsilon,X,LastPeriod,h,lambda,NumPaths,EstStr,ForecastType,DSGEModel,CurrINI)
% DSGEVARPredictionPathsParam: Calculates a certain number of possible paths that the endogenous variables can
%                              take out-of-sample for a given value of the parameters in a DSGE-VAR model.
%
% USAGE:
%
%       [PredPaths,PredData,PredEventData,YObsEventData] = DSGEVARPredictionPathsParam(Phi0,Phi,SigmaEpsilon,X,LastPeriod,...
%                                                                h,lambda,NumPaths,EstStr,ForecastType,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  Phi0 (matrix) of dimension nxk with parameters on the exogenous variables.
%
%                  Phi (matrix) of dimension nxnp with parameters on lagged endogenous variables.
%
%                  SigmaEpsilon (matrix) of dimension nxn with residual covariances.
%
%                  X (matrix) of dimension kxh with data on the exogenous variables over the
%                    forecast sample.
%
%                  LastPeriod (integer) with the last within-sample data point
%
%                  h (integer) with the number of out-of-sample forecast periods.
%
%                  lambda (scalar) with the lambda hyperparameter of the DSGE-VAR.
%
%                  NumPaths (integer) with the number of forecast paths to compute.
%
%                  EstStr (string) that is equal to 'Initial Values', 'Marginal Posterior Mode',
%                    or 'Joint Posterior Mode'.
%
%                  ForecastType (integer) that is equal to 1 if the original data should be forecasted,
%                    2 if annualized data should be forecasted, and 3 if transformed data.
%
%                  DSGEModel (structure) whose fields contain all required information about the model, such as
%                    input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: PredPaths (n x h x NumPaths) 3D matrix with the NumPaths possible paths for the n
%                    endogenous variables.
%
%                  PredData (structure) with fields:
%                    PredMean   - (nxh) matrix with the mean predictions of the observed
%                                 variables.
%                    epsShocks  - (nxh) matrix with the residuals at the population mean
%                                 predictions.
%                    epsMean    - (nxh) matrix with the mean values for the economic shocks over
%                                 the forecast period.
%                    Shocks     - (nxhxNumPaths) matrix with the residuals for the paths.
%                    KernelX    - (3D matrix) of dimension n x 2^8 x h with the x-axis values
%                                 from kernel density estimation of the marginal predictive
%                                 densities.
%                    KernelY    - (3D matrix) of dimension n x 2^8 x h with the y-axis values
%                                 from kernel density estimation of the marginal predictive
%                                 densities.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                    the PredPaths data satisfies the event, the second holds the number of times
%                    the PredPaths data is below the lower bound of the event, the third the number of
%                    times the data is above the upper bound, while the fourth column holds the number
%                    of times the event has been tested. The fifth and sixth column measure the average
%                    deviation from the lower bound when below the lower bound and the square of this,
%                    while the seventh and eighth columns hold similar data for the upper bound of the
%                    event when the variables are above the upper bound.
%
%                  YObsEventData (n x 8) matrix where the first column hold the number of times the
%                    PredPaths (adjusted such that the mean is equal to the realized value) satisfies
%                    the event, the second holds the number of times the adjusted paths data is below
%                    the lower bound the event, the third the number of time the data is above the
%                    upper bound, while the fourth column holds the number of times the event has been
%                    tested. The fifth and sixth column measure the average deviation from the lower
%                    bound when below the lower bound and the square of this, while the seventh and 
%                    eighth columns hold similar data for the upper bound of the event when the
%                    variables are above the upper bound.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 1, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-06-2010: Fixed a bug where the residuals were not added correctly.
%
% * 04-06-2010: Removed the "FirstPeriod" input variable as it wasn't used! The "X" field was deleted
%               from the output structure "PredData".
%
% * 16-06-2010: Updated the documentation.
%
% * 18-11-2010: Took into account that "PredEventData" and "YObsEventData" now have 7 columns.
%
% * 19-11-2010: And now 8.
%
% * 05-01-2011: Updated the documentation.
%
% * 25-01-2011: Took into account that "CalcTransformData" takes the input variable "TransMatrix".
%
% * 18-02-2011: The function can now compute a Choleski-type of decomposition of "SigmaEpsilon" which
%               is consistent with a positive semidefinite matrix.
%
% * 21-02-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 09-03-2012: Made sure that missing observations don't affect the prediction event calculations.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
PredEventData = [];
YObsEventData = [];
k = DSGEModel.k;
n = DSGEModel.n;
p = DSGEModel.BVARLags;
PredPaths = zeros(n,h,NumPaths);
PredData.epsShocks = zeros(n,h);
PredData.KernelX = [];
PredData.KernelY = [];
PredData.epsMean = zeros(n,h);
PredData.Shocks = zeros(n,h,NumPaths);
PredData.PredMean = zeros(n,h);
%
% setup observed variables and initial conditions for the observed variables
%
if (DSGEModel.RunPredictionEvent==1)&(size(DSGEModel.Y,2)>=LastPeriod+h);
   YObserved = DSGEModel.Y(:,LastPeriod+1:LastPeriod+h);
else;
   YObserved = [];
end;
if ForecastType==2;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
   elseif strcmp(DSGEModel.DataFrequency,'m')==1;
      Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
   else;
      Yinit = [];
   end;
elseif ForecastType==3;
   Yinit = DSGEModel.Y(:,1:LastPeriod);
end;
%
% deal with the parameters
%
J = eye(n);
if p>1;
   Phi = [Phi;eye(n*(p-1)) zeros(n*(p-1),n)];
   J = [J;zeros(n*(p-1),n)];
end;
SigmaStdMat = PSDCholeski(SigmaEpsilon);
%
% initial conditions for the forecasts
%
YT = vec(fliplr(DSGEModel.Y(:,LastPeriod-p+1:LastPeriod)));
%
% compute the population mean prediction
%
XTh = zeros(n,h);
Xbar = zeros(n*p,1);
Phii = Phi;
for i=1:h;
   if k>0;
      Xbar = (J*Phi0*X(:,i))+(Phi*Xbar);
      XTh(:,i) = J'*Xbar;
   end;
   PredData.PredMean(:,i) = XTh(:,i)+(J'*Phii*YT);
   Phii = Phii*Phi;
end;
%
% annualize the mean predictions when needed
%
if ForecastType==2;
   PredData.PredMean = Annualize([Yinit PredData.PredMean],DSGEModel.annual,DSGEModel.annualscale);
   if isempty(YObserved)==0;
      YObserved = Annualize([Yinit YObserved],DSGEModel.annual,DSGEModel.annualscale);;
   end;
elseif ForecastType==3;
   YTmp = [Yinit PredData.PredMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   PredData.PredMean = YData(Tt-h+1:Tt,:)';
   if isempty(YObserved)==0;
      YObserved = [Yinit YObserved];
      [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YObserved',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
      Tt = size(YData,1);
      YObserved = YData(Tt-h+1:Tt,:)';
   end;
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumPaths,'%0.0f') ' Prediction Paths'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Path Number:';
   %
   ProgressStructure.name = ['Predictive Distribution - ' EstStr];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
end;
%
% start the iterations
%
abort = '';
for it=1:NumPaths;
   %
   % check if we should update the progress dialog
   %
   if DSGEModel.ShowProgress==1;
      abort = get(WaitHandle,'UserData');
      if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
         PredPaths = [];
         break;
      else;
         ProgressDLG([it/NumPaths it]);
      end;
   end;
   %
   % Draw a sequence of future innovations
   %
   ResidTh = SigmaStdMat*randn(n,h);
   PredData.Shocks(:,:,it) = ResidTh;
   PredData.epsMean = PredData.epsMean+((1/NumPaths)*ResidTh);
   Phii = Phi;
   ResidBar = zeros(n*p,1);
   for i=1:h;
      ResidBar = (J*ResidTh(:,i))+(Phi*ResidBar);
      PredPaths(:,i,it) = XTh(:,i)+(J'*Phii*YT)+(J'*ResidBar);
      Phii = Phii*Phi;
   end;
   %
   % check if we should annualize
   %
   if ForecastType==2;
      PredPaths(:,:,it) = Annualize([Yinit PredPaths(:,:,it)],DSGEModel.annual,DSGEModel.annualscale);
   elseif ForecastType==3;
      YTmp = [Yinit PredPaths(:,:,it)];
      [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
      PredPaths(:,:,it) = YData(Tt-h+1:Tt,:)';
   end;
   drawnow;
end;
%
% close the wait handle
%
if DSGEModel.ShowProgress==1;
   set(WaitHandle,'UserData','done');
   close(WaitHandle);
   drawnow;
   pause(0.02);
end;
if (strcmp(abort,'cancel')==0)&(DSGEModel.RunPredictionEvent==1);
   MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
   if DSGEModel.AdjustPredictionPaths==1;
      %
      % we need to fix the paths such that the mean is correct
      %
      AdjustMean = PredData.PredMean-MeanPrediction;
      for i=1:NumPaths;
         PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
   else;
      PEPredPaths = PredPaths;
   end;
   %
   % compute the prediction events results
   %
   PredEventData = CalculatePredictionEvents(PEPredPaths,DSGEModel.PredictionEvent);
   %
   if isempty(YObserved)==0;
      AdjustMean = YObserved-MeanPrediction;
      %
      % make sure that missing observations don't affect adjustment
      %
      AdjustMean(isnan(AdjustMean)) = 0;
      for i=1:NumPaths;
         YPEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
      end;
      YObsEventData = CalculatePredictionEvents(YPEPredPaths,DSGEModel.PredictionEvent);
   end;
end;
%
% check if we should compute kernel density estimates
%
if (strcmp(abort,'cancel')==0)&(NumPaths>=500);
   if DSGEModel.RunPredictionEvent==0;
      if DSGEModel.AdjustPredictionPaths==1;
         MeanPrediction = (1/NumPaths)*sum(PredPaths,3);
         AdjustMean = PredData.PredMean-MeanPrediction;
         for i=1:NumPaths;
            PEPredPaths(:,:,i) = PredPaths(:,:,i)+AdjustMean;
         end;
      else;
         PEPredPaths = PredPaths;
      end;
   end;
   %
   [PredData.KernelX,PredData.KernelY] = PredictionKernelDensity(PEPredPaths);
end;

%
% end of DSGEVARPredictionPathsParam.m
%
