function [Shocks,DateVector,SampleStrMat] = DSGEVARShocksDistribution(DSGEModel,CurrINI,controls,UsedDraws,CurrChain,IsPosterior,PME,lambda)
% DSGEVARShocksDistribution: Computes the mean value of the structural shocks as well as quantiles by loading data from disk.
%
% USAGE:
%
%       [Shocks,DateVector,SampleStrMat] = DSGEVARShocksDistribution(DSGEModel,CurrINI,controls,UsedDraws,CurrChain,IsPosterior,PME,lambda)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information about the
%                            model, such as input files, variable and sample data. Actual observations
%                            are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI controls on YADA's
%                            main dialog.
%
%                  UsedDraws (integer) with the number of draws of the DSGE-VAR parameters.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution
%                            are used and 0 if draws from the prior are supplied.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  lambda (scalar)  with the value of the lambda hyperparameter for the DSGE-VAR.
%
% REQUIRED OUTPUT: Shocks (structure) with fields "u" holding the mean, a matrix of dimension nxT,
%                            "Quantiles" is a vector structure with fields "u" and "percent".
%
%                  DateVector (vector) with numerical dates for the shocks.
%
%                  SampleStrMat (string matrix) with the dates for the shocks.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 27, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-05-2010: Updated the documentation.
%
% * 22-10-2010: The wait dialog now displays the lag order and lambda value for the DSGE-VAR.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: The function now supports unix and mac paths.
%
% * 08-04-2011: The function now uses "ShockAliases" instead of "StateShockNames" to represent the
%               names of the structural shocks.
%
% * 03-01-2012: Updated the documentation.
%
% * 27-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 04-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Shocks = [];
DateVector = [];
SampleStrMat = '';
%
% 1. determine the number of draws of the DSGE model parameters
%
DrawsStr = get(controls.posterior.draws,'String');
NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
%
% 2. Number of parallel chains
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
% 3. the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
p = DSGEModel.BVARLags;
ShockNames = DSGEModel.ShockAliases(DSGEModel.DSGEVARShocks,:);
for j=1:DSGEModel.n;
   ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
end;
%
% initialize index for saving
%
SaveAfterDraws = min(UsedDraws,1000);
NumFiles = ceil(UsedDraws/SaveAfterDraws);
NumQuants = length(DSGEModel.Percentiles);
%
% load the first file
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-Shocks-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-1-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-Shocks-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-1-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
end;
ShockData = load(file);
[n,T] = size(ShockData.Shocks(1).u);
DateVector = ShockData.DateVector;
SampleStrMat = ShockData.SampleStrMat;
%
% initialize output
%
Shocks.u = zeros(n,T);
for i=1:NumQuants;
   Shocks.Quantiles(i).u = zeros(n,T);
   Shocks.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% setup a wait dialog
%
txt = ['Please wait while YADA computes the mean structural shocks as well as percentile values for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(ShockNames(1,:)) '.'];
WaitHandle = WaitDLG(txt,'information',['Structural Shock Distribution' PriorHeader],500,300,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
for i=1:n;
   txt = ['Please wait while YADA computes the mean structural shocks as well as percentile values for a DSGE-VAR with lambda=' num2str(lambda,'%0.4f') ' and ' int2str(p) ' lags. The data are loaded sequentially from disk and it may therefore take some time. Current variable: ' StringTrim(ShockNames(i,:)) '.'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   try;
      TempVar = zeros(UsedDraws,T);
      CurrSaves = 0;
      for ThisSave=1:NumFiles;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-Shocks-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-Shocks-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
         end;
         ShockData = load(file);
         CurrShocks = ShockData.Shocks;
         NumSaves = length(CurrShocks);
         for s=1:NumSaves;
            TempVar(CurrSaves+s,:) = CurrShocks(s).u(i,:);
         end;
         CurrSaves = CurrSaves+NumSaves;
         drawnow;
      end;
      %
      % we can now calculate the mean
      %
      Shocks.u(i,:) = (1/UsedDraws)*sum(TempVar,1);
      %
      % now we can sort these and create the quantiles from the sorted
      %
      TempVar = sort(TempVar);
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
            end;
            Shocks.Quantiles(l).u(i,:) = TempVar(QuantVal,:);
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
            for t=1:T;
               Bounds = HighestProbabilityDensityBand(TempVar(:,t),ConfDraws);
               Shocks.Quantiles(l).u(i,t) = Bounds(1);
               Shocks.Quantiles(NumQuants-l+1).u(i,t) = Bounds(2);
            end;
         end;
      end;
   catch;
      for t=1:T;
         TempVar = zeros(UsedDraws,1);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-Shocks-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '.' int2str(NumChains) '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\state\DSGE-VAR-Shocks-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' SelectedShockStr '-' DSGEModel.NameOfModel '-' int2str(ThisSave) '-' int2str(NumFiles) '.' int2str(UsedDraws) '.mat']);
            end;
            ShockData = load(file);
            CurrShocks = ShockData.Shocks;
            NumSaves = length(CurrShocks);
            for s=1:NumSaves;
               TempVar(CurrSaves+s,1) = CurrShocks(s).u(i,t);
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % we can now calculate the mean
         %
         Shocks.u(i,t) = (1/UsedDraws)*sum(TempVar,1);
         %
         % now we can sort these and create the quantiles from the sorted
         %
         TempVar = sort(TempVar);
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*UsedDraws);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*UsedDraws);
               end;
               Shocks.Quantiles(l).u(i,t) = TempVar(QuantVal,1);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*UsedDraws/100);
               Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
               Shocks.Quantiles(l).u(i,t) = Bounds(1);
               Shocks.Quantiles(NumQuants-l+1).u(i,t) = Bounds(2);
            end;
         end;
      end;
   end;
   drawnow;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEVARShocksDistribution.m
%
