function [Phi,SigmaEpsilon,A0,IdentifyStatus,Status] = GetDSGEVARParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,lambda,DSGEModel,CurrINI)
% GetDSGEVARParameters: Determines the posterior mode estimates of the VAR parameters in a DSGE-VAR for fixed values of the
%                       DSGE model parameters.
%
% USAGE:
%
%       [Phi,SigmaEpsilon,A0,IdentifyStatus,Status] = GetDSGEVARParameters(theta,thetaPositions,ModelParameters,PriorDist,...
%                                                                          thetaDist,LowerBound,lambda,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with the values of the original DSGE model parameters.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                                 of the model parameter in position i of theta. The field "distribution" has a string
%                                 vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                                 "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                                 parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                                 and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                                 it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                                 truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                                 values are the lower and the upper bound. For the student-t they are the location (mode),
%                                 the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                                 and scale parameters are needed. For the Pareto the parameter values are for the mode
%                                 and the shape parameter.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                                 truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                                 and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  lambda (scalar) with the value of the lambda hyperparameter.
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                                 such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: Phi (matrix) of dimension nx(np+k) with the parameters on the exogenous and lagged endogenous variables
%                                 of the VAR.
%
%                  SigmaEpsilon (matrix) of dimension nxn with the residual covariances of the VAR.
%
%                  A0 (matrix) of dimension nxn with the contemporaneous effects of the n structural
%                                 shocks in the DSGE-VAR model.
%
%                  IdentifyStatus (boolean) that takes the value 1 if A0 could be determined and 0
%                                 otherwise.
%
%                  Status (boolean) that takes on the value 1 if the VAR parameter values could be determined
%                                 and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 27, 2011.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-02-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 22-01-2019: Updated the documentation.
%
% * 16-04-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Phi = [];
SigmaEpsilon = [];
A0 = [];
IdentifyStatus = 0;
Status = 0;
%
% compute initial values for the DSGE-VAR. We use prior mode
%
n = DSGEModel.n;
k = DSGEModel.k;
p = DSGEModel.BVARLags;
[y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
if k>0;
   DetProductMoments = CalcDeterministicProductMoments(x,X);
else;
   DetProductMoments = [];
end;
%
% compute sample product moments
%
T = size(y,2);
GammaHatyy = (1/T)*y*y';
GammaHatyY = (1/T)*y*Y';
GammaHatYY = (1/T)*Y*Y';
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% solve the model
%
[A,H,R,F,B0] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
%
% compute the VAR modes
%
[Phi,SigmaEpsilon,Gammayy,GammayY,GammaYY] = DSGEVARParameters(A,H,R,F,B0,lambda,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,p,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,HSample);
if isinf(lambda)==0;
   SigmaEpsilon = (((1+lambda)*T)/(((1+lambda)*T)+n+1))*SigmaEpsilon;
end;
Status = 1;
[A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);

%
% end of GetDSGEVARParameters.m
%
