function [Lambda,Tobs] = GetLambda(DSGEModel)
% GetLambda: Provides the vector with feasible lambda values for estimating DSGE-VAR models.
%
% USAGE:
%
%       [Lambda,Tobs] = GetLambda(DSGEModel)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
% REQUIRED OUTPUT: Lambda (vector) with the feasible lambda values.
%
% OPTIONAL OUTPUT: Tobs (integer) with the number of observations for the data.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 18, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-02-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 30-08-2011: Commented out the command "Lambda = ceil(Lambda*T)/T;" at the end of the function.
%               This is unnecessary and, e.g., if T is odd can lead to the wrong Lambda values if some
%               entries have 0.5.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some parameters
%
p = DSGEModel.BVARLags;
k = DSGEModel.k;
n = DSGEModel.n;
npk = (n*p)+k;
%
% determine the sample size T
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
end;
T = LastPeriod+2-FirstPeriod-KalmanFirstObservation;
%
LowBoundLambda = (n*(p+1)+k)/T;
%
% keep only the lambda values that are at least as large as the lower bound
%
Lambda = DSGEModel.Lambda(DSGEModel.Lambda>=LowBoundLambda);
if length(Lambda)<=1;
   Lambda = [LowBoundLambda Inf];
elseif Lambda(1)>LowBoundLambda;
   Lambda = [LowBoundLambda Lambda];
end;
if max(isinf(Lambda))==0;
   Lambda = [Lambda Inf];
end;
%Lambda = ceil(Lambda*T)/T;
if nargout==2;
   Tobs = T;
end;

%
% end of GetLambda.m
%
