function IRDSGEVARGraphDLG(selector,CurrINI,DSGEModel,IRF,ShockNames,lambda,NameStr,VarType,ShockType)
% IRDSGEVARGraphDLG: Displays impulse response functions for a DSGE-VAR.
%
% USAGE:
%
%       IRDSGEVARGraphDLG(selector,CurrINI,DSGEModel,IRF,ShockNames,lambda,NameStr,VarType,ShockType)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 IRF (matrix) of dimension nxnxh with the impulse responses for the endogenous
%                           variables (rows) from shocks (columns) at horizon (third dim).
%
%                 ShockNames (string matrix) with the names of the shocks/groups.
%
%                 lambda (scalar) with the value of the lambda hyperparameter.
%
%                 NameStr (string vector) taking on the values "Marginal Posterior Mode", "Joint
%                           Posterior Mode" or "Initial Values".
%
%                 VarType (string vector) taking on the values "Levels", "Original", or "Annualized".
%
%                 ShockType (string vector) taking the values "individual" or "group".
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 24, 2010.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-02-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 21-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.IRF = IRF;
   DSGEModel.NameStr = NameStr;
   DSGEModel.VarType = VarType;
else
   IRGraphGUI = findobj('Type','figure','Tag','IRDSGEVARGraphDLG');
   IRcontrol = get(IRGraphGUI,'UserData');
   CurrINI = get(IRcontrol.shocks,'UserData');
   DSGEModel = get(IRcontrol.variables,'UserData');
end;
switch selector;
   case 'init';
      IRGraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','IRDSGEVARGraphDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-120)/2 500 110], ...
         'Tag','IRDSGEVARGraphDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',[VarType ' Impulse Responses - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','IRDSGEVARGraphDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(IRGraphGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 498 100],'DSGE-VAR Graphics',100,[0.5 1],'on',CurrINI);
      %
      IRcontrol.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 55 340 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String',['Impulse responses from year 0 to ' num2str(size(IRF,3)-1,'%0.0f') ' for lambda=' num2str(lambda,'%0.4f')], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         set(IRcontrol.sample,'String',strrep(get(IRcontrol.sample,'String'),'year','quarter'));
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         set(IRcontrol.sample,'String',strrep(get(IRcontrol.sample,'String'),'year','month'));
      end;
      %
      % Select shock
      %
      IRcontrol.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 20 176 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select shock:', ...
         'HorizontalAlignment','left', ...
         'Tag','ShockText');
      %
      IRcontrol.shocks = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 25 165 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ShockNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ShocksPopup',...
         'Value',1);
      %
      if strcmp(lower(ShockType),'group')==1;
         set(IRcontrol.variables,'String','Select shock group:');
         ShockGroupNames = ShockNames;
         for i=1:size(ShockNames,1);
            ShockGroupNames(i,:) = strrep(ShockGroupNames(i,:),'_',' ');
         end;
         set(IRcontrol.shocks,'String',ShockGroupNames);
      end;
      %
      % Create graph button
      %
      IRcontrol.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','IRDSGEVARGraphDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      IRcontrol.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','IRDSGEVARGraphDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(IRGraphGUI,'UserData',IRcontrol,...
         'HandleVisibility','callback');
   case 'showgraph'
      ShockValue = get(IRcontrol.shocks,'Value');
      ShockNames = get(IRcontrol.shocks,'String');
      CurrShock = StringTrim(ShockNames(ShockValue,:));
      if ShockValue<size(ShockNames,1);
         set(IRcontrol.shocks,'Value',ShockValue+1);
      else;
         set(IRcontrol.shocks,'Value',1);
      end;
      %
      % set up the impulse responses for shock number ShockValue
      %
      IRF = DSGEModel.IRF;
      h = size(IRF,3);
      n = DSGEModel.n;
      YResponses = zeros(n,h);
      for i=1:h;
         YResponses(:,i) = IRF(:,ShockValue,i);
      end;
      %
      % Compute the number of figures (max 16 plots per graph)
      %
      NumFigs = ceil(n/16);
      %
      if NumFigs==1;
         [NRows,NCols] = VillaniConstructOptimalSubplot(n);
      else;
         NRows = 4;
         NCols = 4;
      end;
      Xvector = (1:1:h)-1;
      %
      % Initialize counters for how many plots have been added
      %
      NumPlots = 0;
      pos = [(CurrINI.scrsz(3)-800)/2 (CurrINI.scrsz(4)-600)/2 800 600];
      FigHandles = [];
      for i=1:NumFigs;
         pos(1) = pos(1)+22;
         pos(2) = pos(2)-22;
         if pos(1)+pos(3)>CurrINI.scrsz(3);
            pos(1) = 0;
         end;
         if pos(2)+pos(4)>CurrINI.scrsz(4)-80;
            pos(2) = CurrINI.scrsz(4)-80-pos(4);
         end;
         FigHandles{i} = figure('Visible','off','Position',pos);
         if NumFigs==1;
            set(FigHandles{i},'Name',[DSGEModel.VarType ' Impulse Responses At ' StringTrim(DSGEModel.NameStr) ' - ' CurrShock]);
         else;
            set(FigHandles{i},'Name',[DSGEModel.VarType ' Impulse Responses At ' StringTrim(DSGEModel.NameStr) ' For ' CurrShock ' - Figure ' num2str(i)]);
         end;
         SetFigureProperties(FigHandles{i},CurrINI);
         %
         k = 0;
         for r=1:NRows;
            for c=1:NCols;
               NumPlots = NumPlots+1;
               if NumPlots<=n;
                  k = k+1;
                  %
                  % ensure that the handle to the current figure is correct (HG2 issue)
                  %
                  set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
                  subplot(NRows,NCols,k);
                  plot(Xvector,YResponses(NumPlots,:),'-','Color',[0.75 0 0],'LineWidth',2);
                  SetAxesFonts(CurrINI,'Title',[DSGEModel.VarType ' Resp(' StringTrim(DSGEModel.VariableNames(NumPlots,:)) ') from ' CurrShock]);
                  axis('tight');
                  %
                  % fix the yaxis
                  %
                  ylim = get(gca,'YLim');
                  addY = 0.1*(ylim(2)-ylim(1));
                  ylim = ylim+[-addY addY];
                  set(gca,'YLim',ylim);
                  %
                  % add a zero line if necessary
                  %
                  if (ylim(2)>0)&(ylim(1)<0);
                     hold('on');
                     plot([Xvector(1) Xvector(h)],[0 0],'-','Color',[0 0 0]);
                     kids = get(gca,'Children');
                     kids = [kids(2) kids(1)]';
                     set(gca,'Children',kids);
                  end;
                  hold('off');
               end;
            end;
         end;
         %
         SetToolbarImages(FigHandles{i},CurrINI.images);
         set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
      end;
   case 'done'
      set(IRcontrol.done,'UserData','done');
      delete(IRGraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(IRGraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(IRGraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(IRcontrol.done);
   end;
   waitfor(IRcontrol.done,'UserData','done');
end;

%
% end of IRDSGEVARGraphDLG.m
%
