function [PME,Lambda] = SequentialDSGEVARLaplaceMarginalLikelihood(DSGEModel,CurrINI,controls,PME,Lambda,RecomputeLML)
% SequentialDSGEVARLaplaceMarginalLikelihood: Computes or collects an already estimated marginal likelihood
%                                             with the Laplace approximation for a DSGE model.
%
% USAGE:
%
%       [PME,Lambda] = SequentialDSGEVARLaplaceMarginalLikelihood(DSGEModel,CurrINI,controls,PME,Lambda,RecomputeLML)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information 
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (scalar) with the lambda hyperparameter value that should be used.
%
%                  RecomputeLML (boolean) which takes the value 1 if the log marginal likelihood
%                            using the Laplace approximation should be recomputed and 0 otherwise.
%
% REQUIRED OUTPUT: PME (integer) that takes the value 1 if the DSGE model posterior mode estimates
%                            are used, 2 if the DSGE-VAR marginal posterior mode estimates were
%                            used, and 3 if the DSGE-VAR joint posterior mode estimates were used
%                            when generating the posterior draws.
%
%                  Lambda (scalar) with the lambda hyperparameter value that should be used.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 27, 2022.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-01-2022: Finished the function from the recalculation of the marginal
%               likelihood.
%
% * 31-01-2022: Made sure that the posterior mode file is updated with a finite difference
%               based inverse Hessian when it has been recalculated and its previous value
%               indicates that it is missing.
%
% * 01-02-2022: Added the LogLikeDensity and LogPostDensity values to the output mat-file.
%
% * 06-01-2023: Updated the documentation.
%
% * 05-07-2023: The function now tries to load the AIMData file directly.
%
% * 09-01-2024: Took into account that "YADAHessianMatrix" now takes the input variable "NumWorkers".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
if isempty(PME)==1;
   %
   % check which posterior mode estimates to choose between
   %
   PostModeStr = '';
   DSGEModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
   if FileExist(DSGEModeFile)==1;
      PostModeStr = 'DSGE model posterior mode';
      CurrModes(1) = 1;
   end;
   if VerifyDSGEVARMarginalMode(DSGEModel)==1;
      PostModeStr = strvcat(PostModeStr,'DSGE-VAR marginal posterior mode');
      CurrModes(2) = 1;
   end;
   if VerifyDSGEVARJointMode(DSGEModel)==1;
      PostModeStr = strvcat(PostModeStr,'DSGE-VAR joint posterior mode');
      CurrModes(3) = 1;
   end;
   ModeSelected = 0;
   if isempty(PostModeStr)==0;
      if sum(CurrModes)>1;
         %
         % need to select the mode
         %
         [action,SelectedMode] = SelectionDlg(PostModeStr,1,'Select the posterior mode estimator:','Estimator Selection','Laplace Marginal Likelihood For DSGE-VAR','','',CurrINI);
         if strcmp(lower(action),'ok')==1;
            if strcmp(StringTrim(PostModeStr(SelectedMode,:)),'DSGE model posterior mode')==1;
               PME = 1;
            elseif strcmp(StringTrim(PostModeStr(SelectedMode,:)),'DSGE-VAR marginal posterior mode')==1;
               PME = 2;
            elseif strcmp(StringTrim(PostModeStr(SelectedMode,:)),'DSGE-VAR joint posterior mode')==1;
               PME = 3;
            end;
            ModeSelected = 1;
         else;
            SetLogText('Posterior mode estimator was not selected.');
            return;
         end;
      else;
         PME = CurrModes.*(1:3);
         PME = PME(PME>0);
         ModeSelected = 1;
      end;
   end;
else;
   ModeSelected = 1;
end;
%
% determine string with 1's and 0's for the selected shocks
%
SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
%
if ModeSelected==1;
   %
   % check if we should select a lambda-value
   %
   if isempty(Lambda)==1;
      if PME==1;
         lambda = GetLambda(DSGEModel);
         ModeFiles = '';
         LambdaValuesStr = '';
         MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         for i=1:length(lambda);
            LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
            ModeFiles = strvcat(ModeFiles,MatFile);
         end;
         DefaultValue = length(lambda);
      elseif PME==2;
         [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
         if length(lambda)>1;
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            [LambdaValuesStr,DefaultValue] = GetLambdaSelectionData(MatFile,lambda);
         end;
      elseif PME==3;
         [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
         if length(lambda)>1;
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            [LambdaValuesStr,DefaultValue] = GetLambdaSelectionData(MatFile,lambda);
         end;
      end;
      if length(lambda)>1;
         [action,value] = SelectionDlg(LambdaValuesStr,DefaultValue,'Select the value of the lambda hyperparameter:','Lambda Selection','Laplace Marginal Likelihood For DSGE-VAR','','',CurrINI);
         if strcmp(lower(action),'ok')==0;
            SetLogText('A value for lambda was not selected.');
            return;
         end;
      else;
         action = 'OK';
         value = 1;
      end;
   end;
   %
else;
   SetLogText('Unable to locate posterior mode results.');
   return;
end;
%
% determine Lambda
%
Lambda = lambda(value);
%
% try to load the mode-file
%
ModeFile = StringTrim(ModeFiles(value,:));
ErrorStr = '';
HeaderStr = 'YADA - Bad MAT File';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the posterior mode file "' GetFilename(ModeFile) '".'];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the posterior mode file "' GetFilename(ModeFile) '".'];
   end;
end;
if isempty(ErrorStr)==0;
   SetLogText(ErrorStr);
   return;
end;
%
% check if we should recompute the Laplace approximation or just take the input from the
% posterior mode file
%
if RecomputeLML==0;
   if isnan(ModeData.phiInverseHessianFinDiff)==0;
      InverseHessian = ModeData.phiInverseHessianFinDiff;
   else;
      InverseHessian = ModeData.InverseHessian;
   end;
   if isfield(ModeData,'LogMargLikeSystemLaplace')==1;
      LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
   else;
      LogMargLikeSystemLaplace = 0;
   end;
   LogLikeDensity = ModeData.LogLikeDensity;
   LogMargLike = MargLikeLaplace(ModeData.LogPostDensity,InverseHessian,1)-LogMargLikeSystemLaplace;
   p = DSGEModel.BVARLags;
else;
   %
   % need to compute certain values
   %
   n = DSGEModel.n;
   k = DSGEModel.k;
   p = DSGEModel.BVARLags;
   npk = (n*p)+k;
   %
   phi = ModeData.phiMode;
   %
   % setup the sample
   %
   [y,Y,HSample,x,X] = GetDSGEVARData(DSGEModel);
   if DSGEModel.k>0;
      DetProductMoments = CalcDeterministicProductMoments(x,X);
   else;
      DetProductMoments = [];
   end;
   T = size(y,2);
   GammaHatyy = (1/T)*y*y';
   GammaHatyY = (1/T)*y*Y';
   GammaHatYY = (1/T)*Y*Y';
   if PME<3;
      if isinf(Lambda)==0;
         logGPR = LogGammaProductRatio(n,((1+Lambda)*T)-npk,Lambda*T-npk);
      else;
         logGPR = [];
      end;
   else;
      if isinf(Lambda)==0;
         logCPC = DSGEVARLogConcLikelihoodConstant(Lambda,T,n,npk);
      else;
         logCPC = [];
      end;
   end;
   %
   % Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.InitializeParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % try to load the AIMData, other wise use ModeData
   %
   if MatlabNumber>=8.0;
      try;
         AIMData = load(DSGEModel.AIMDataFile);
      catch ME;
         AIMData = ModeData.AIMData;
      end;
   else;
      try;
         AIMData = load(DSGEModel.AIMDataFile);
      catch;
         AIMData = ModeData.AIMData;
      end;
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % compute the log posterior and the inverse Hessian at the mode
   %
   NumParam = length(phi);
   StepLengthStr = get(controls.optimize.steplength,'String');
   StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
   %
   % display a wait dialog
   %
   txt = ['Please wait while YADA computes the log posterior density kernel and the inverse Hessian with finite differences. Calculation started at: ' StringTrim(datestr(now,14)) '.'];
   WaitHandle = WaitDLG(txt,'information','Laplace Approximation Of Marginal Likelihood',500,300,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
   drawnow;
   pause(0.02);
   %
   if PME<3;
      [LogPostDensity,LogLikeDensity] = logPosteriorPhiDSGEVAR(phi,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,Lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
      LogPostDensity = -LogPostDensity;
      VechHessian = YADAHessianMatrix('logPosteriorPhiDSGEVAR',phi,StepLength,CurrINI.NumWorkers,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,Lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,CurrINI.OrderQZ,CurrINI.NumWorkers);
   else;
      [LogPostDensity,LogLikeDensity] = logConcPosteriorPhiDSGEVAR(phi,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,Lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,CurrINI.NumWorkers);
      LogPostDensity = -LogPostDensity;
      VechHessian = YADAHessianMatrix('logConcPosteriorPhiDSGEVAR',phi,StepLength,CurrINI.NumWorkers,ModeData.thetaIndex,ModeData.UniformBounds,ModeData.LowerBound,ModeData.thetaPositions,ModeData.thetaDist,ModeData.PriorDist,ModeData.ModelParameters,DSGEModel,AIMData,Lambda,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logCPC,CurrINI.OrderQZ,CurrINI.NumWorkers);
   end;
   HessianMat = GeneralizedCholeski(reshape(VechHessian,NumParam,NumParam));
   %
   % delete the wait dialog
   %
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
   %
   if IsPositiveDefinite(HessianMat)==1;
      if isfield(ModeData,'LogMargLikeSystemLaplace')==1;
         LogMargLikeSystemLaplace = ModeData.LogMargLikeSystemLaplace;
         IsSystemPrior = 1;
      else;
         LogMargLikeSystemLaplace = 0;
         IsSystemPrior = 0;
      end;
      LogMargLike = MargLikeLaplace(LogPostDensity,HessianMat,0)-LogMargLikeSystemLaplace;
      if isnan(ModeData.phiInverseHessianFinDiff)==1;
         phiInverseHessianFinDiff = HessianMat\eye(NumParam);
         save(ModeFile,'phiInverseHessianFinDiff','-APPEND');
      end;
   else;
      SetLogText(strvcat('The estimated Hessian matrix is not positive definite.','Unable to compute the Laplace approximation.'));
      return;
   end;
end;
%
% print to file
%
PrtVal = ['%' num2str(9+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
if FirstPeriod+KalmanFirstObservation-1<p+1;
   KalmanFirstObservation = p+2-FirstPeriod;
end;
[NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,KalmanFirstObservation-1);
%
if PME==1;
   ModeShort = 'DSGEMode';
   ModeType = 'DSGE Posterior Mode';
elseif PME==2;
   ModeShort = 'MargMode';
   ModeType = 'Marginal Posterior Mode';
elseif PME==3;
   ModeShort = 'JointMode';
   ModeType = 'Joint Posterior Mode';
end;
%
TextFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-LaplaceMarginalLikelihood-' num2str(Lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' ModeShort '.txt']);                  
%
fid = fopen(TextFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*           L A P L A C E   A P P R O X I M A T I O N   O F   T H E            *\n');
fprintf(fid,'*                    M A R G I N A L   L I K E L I H O O D                     *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% DSGE-VAR model information
%
fprintf(fid,'Posterior mode estimate:          %s \n',ModeType);
fprintf(fid,'Number of lags:                   %0.0f \n',p);
fprintf(fid,'Lambda:                           %0.4f \n\n',Lambda);
%
% sample information
%
fprintf(fid,'Selected sample:                  %s:%s -- %s:%s\n',DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
fprintf(fid,'Estimation sample:                %s:%s -- %s:%s\n\n',NewStartYear,NewStartPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
% now the marginal likelihood
%
fprintf(fid,['Log marginal likelihood:           ' PrtVal ' \n\n'],LogMargLike);
fprintf(fid,'where \n\n');
fprintf(fid,['   Log posterior density of phi:   ' PrtVal ' \n'],LogPostDensity);
fprintf(fid,['   Inverse Hessian term:           ' PrtVal ' \n\n'],LogMargLike-LogPostDensity+LogMargLikeSystemLaplace);
if IsSystemPrior==1;
   fprintf(fid,['   Correction for the log marginal\n   likelihood of the system prior: ' PrtVal ' \n\n'],LogMargLikeSystemLaplace);
end;
fprintf(fid,'================================================================================\n\n');
%
% Some information about the filename and its location
%
fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);
%
% save results to mat-file
%
SaveMatFile = strrep(TextFile,'.txt','.mat');
save(SaveMatFile,'LogMargLike','LogMargLikeSystemLaplace','LogLikeDensity','LogPostDensity');
%
% write to log text
%
if LogMargLikeSystemLaplace==0;
   SetLogText(strvcat(' ',['Log ML (Laplace):              ' num2str(LogMargLike,'%0.4f')],' ',['Log posterior density of phi:  ' num2str(LogPostDensity,'%0.4f')],['Inverse Hessian term:          ' num2str(LogMargLike-LogPostDensity+LogMargLikeSystemLaplace,'%0.4f')],'Output written to:',['Mat file:  ' GetFilename(SaveMatFile)],['Directory: ' GetPath(SaveMatFile)],' '));
else;
   SetLogText(strvcat(' ',['Log ML (Laplace):              ' num2str(LogMargLike,'%0.4f')],' ',['Log posterior density of phi:  ' num2str(LogPostDensity,'%0.4f')],['Inverse Hessian term:          ' num2str(LogMargLike-LogPostDensity+LogMargLikeSystemLaplace,'%0.4f')],['Log system prior ML (Laplace): ' num2str(LogMargLikeSystemLaplace,'%0.4f')],' ','Output written to:',['Mat file:  ' GetFilename(SaveMatFile)],['Directory: ' GetPath(SaveMatFile)],' '));
end;

%
% end of SequentialDSGEVARLaplaceMarginalLikelihood.m
%
