function [logLikeValue,PredVars] = logMargCondLikeDSGEVARAnnual(DSGEModel,n,p,npk,J,Y,YData,X,k,h,Phi,SigmaEpsilon)
% logMargCondLikeDSGEVARAnnual: Computes the marginal conditional likelihood for h-step-ahead forecasts of
%                               annualized observed variables in a DSGE-VAR model.
%
% USAGE:
%
%       [logLikeValue,PredVars] = logMargCondLikeDSGEVAR(DSGEModel,n,p,npk,J,Y,YData,X,k,h,Phi,SigmaEpsilon)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information
%                             about the model, such as input files, variable and sample
%                             data. Actual observations are not stored in this file.
%
%                  n (integer) with the number of endogenous variables.
%
%                  p (integer) with the lag order of the DSGE-VAR model.
%
%                  npk (integer) with the number of regressors in the VAR, equal to the number of endogenous
%                             variables times the number of lags plus the number of exogenous variables.
%
%                  J (matrix) of dimension npxn which selects the y(t) vector from the stacked Y(t) vector,
%                             i.e., y(t) = J'*Y(t), where J has an nxn identity matrix on top and zeros below.
%
%                  Y (vector) of dimension np with the historical data on the endogenous variables
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  k (integer) with the number of exoegnous variables.
%
%                  h (integer) with the prediction horizon.
%
%                  Phi (matrix) of dimension nxnpk with the parameters on the exogenous variables and the
%                             lagged endogenous of the VAR.
%
%                  SigmaEpsilon (matrix) of dimension nxn with the residual covariances of the VAR.
%
% REQUIRED OUTPUT: logLikeValue (vector) of dimension h with the value of the log conditional likelihood for
%                             marginal h-step-ahead forecasts.
%
%                  PredVars (cell array) with the positions of the predicted variables in the vector of observed
%                             variables for each prediction period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 22, 2012.
%                        This version: January 9, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 23-10-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 09-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
logLikeValue = zeros(h,1);
%
% determine the constant term and the Psi matrix
%
if k>0;
   Phi0 = Phi(:,1:k);
   Xbar = Phi0*X;
   Phi = Phi(:,k+1:npk);
else;
   Xbar = zeros(n,h);
end;
Psi = Phi;
if p>1;
   Psi = [Psi;eye(n*(p-1)) zeros(n*(p-1),n)];
end;
%
% compute the likelihood over the prediction sample
%
n = DSGEModel.n;
s = DSGEModel.Frequency;
SigmaYh = zeros(n*p,n*p);
xh = zeros(n*p,1);
Ksiti = zeros(n*p,n*p);
JXbar = J*Xbar;
Psii = Psi;
YData = YData(DSGEModel.PredictionVariables,:);
Yinit = DSGEModel.Yinit;
log2pi = log(2*pi);
%
Q = J*SigmaEpsilon*J';
F_i = eye(n*p);
Fi = eye(n*p);
F = Psi;
Psii = Psi;
Q_i = Q;
%
% compute the conditional likelihood
%
for i=1:h;
   xh = JXbar(:,i)+(Psi*xh);
   SigmaYh = Q+(Psi*SigmaYh*Psi');
   %
   % forecast yt+i
   %
   Yhatmarg = J'*(xh+(Psii*Y));
   SigmaYmarg = J'*SigmaYh*J;
   %
   % point forecasts for sums of first differences
   %
   Yagg = [Yinit Yhatmarg];
   Yhatsum = Annualize(Yagg,DSGEModel.annual,ones(n,1));
   Yinit = Yagg(:,2:s);
   %
   % annualized first differenced variables forecast and covariance
   %
   if i<=s;
      Cti = Q_i;
      %
      % covariances between levels and sums of first differenced variables
      %
      Ksiti = SigmaYh+(F*Ksiti);
      %
      % updating matrices
      %
      if i<s;
         Fi = Fi*F;
         F_i = F_i+Fi;
         Q_i = Q_i+(F_i*Q*F_i');
      end;
   else;
      Cti = (F*Cti*F')+Q_i;
      %
      Ksiti = (F*Ksiti*F')+(F_i*Q);
   end;
   Sigmati = J'*Cti*J;
   Upsilonti = J'*Ksiti*J;
   %
   % place levels variances along the diagonal
   %
   Sigmati(DSGEModel.LevPos,DSGEModel.LevPos) = SigmaYmarg(DSGEModel.LevPos,DSGEModel.LevPos);
   %
   % place levels and sums of first diff covariances in the appropriate
   % off-dialognal elements
   %
   Sigmati(DSGEModel.LevPos,DSGEModel.DiffPos) = Upsilonti(DSGEModel.LevPos,DSGEModel.DiffPos);
   Sigmati(DSGEModel.DiffPos,DSGEModel.LevPos) = Upsilonti(DSGEModel.LevPos,DSGEModel.DiffPos)';
   %
   % select the subset
   %
   Yhatsum = Yhatsum(DSGEModel.PredictionVariables);
   Sigmati = Sigmati(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
   %
   % check for missing obs
   %
   pos = ~isnan(YData(:,i));
   nstar = sum(pos);
   if nstar>0;
      zerror = YData(pos,i)-Yhatsum(pos);
      Sigmati = Sigmati(pos,pos);
      logLikeValue(i) = -((nstar*log2pi)+LogDet(Sigmati)+((zerror'/Sigmati)*zerror))/2;
      PredVars{i} = DSGEModel.PredictionVariables(pos);
   else;
      PredVars{i} = [];
   end;
   %
   % update
   %
   Psii = Psii*Psi;
end;

%
% encd of logMargCondLikeDSGEVARAnnual.m
%
