function [lnSP,SystemPriorInfo] = SystemPriorFile(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,OrderQZ,IsAdaptiveLearning)
% SystemPriorFile: Computes the value of the log prior for system features. The input variables
%                  can be used to locate a solution of the model via the DSGEModel structure, or
%                  to solve the DSGE model with the "SolveDSGEModel" function in YADA.
%
% USAGE:
%
%       [lnSP,SystemPriorInfo] = SystemPriorFile(theta,thetaPositions,ModelParameters,AIMData,DSGEModel,OrderQZ,IsAdaptiveLearning)
%
% REQUIRED INPUT:  theta (vector) with the original parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  AIMData (structure) with data collected by AIM about the model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero
%                                 otherwise.
%
%                  IsAdaptiveLearning (boolean) that takes the value 1 if the system prior should be computed
%                                 for the adaptive learning mechanism, and 0 otherwise (default).
%
% REQUIRED OUTPUT: lnSP (scalar) with the height of the log prior density of a feature conditional on
%                                 the model parameters.
%
% OPTIONAL OUTPUT: SystemPriorInfo (string matrix) with additional information about the system prior
%                                 with the data the user wants to provide to YADA.
%
% NOTE: This example function computes the population standard deviations of the observed variables and
%       computes the height of the inverted Gamma prior for the standard deviation of inflation. 
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 12, 2017.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-05-2017: Updated the documentation.
%
% * 26-05-2017: Changed the default value of lnSP from NaN to the log of eps.
%
% * 22-06-2017: Changed the input variable "CurrINI" to "OrderQZ", since this is the only field
%               of that structure which is required.
%
% * 13-12-2017: Added the optional output variable "SystemPriorInfo".
%
% * 15-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 08-01-2020: Updated the documentation.
%
% * 08-05-2020: Added the input variable mu to "AdaptiveLearningInitialValuesDSGE".
%
% * 02-07-2020: Removed the input variable mu to "AdaptiveLearningInitialValuesDSGE".
%
% * 25-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin<7;
   IsAdaptiveLearning = 0;
end;
%
if nargout==2;
   SystemPriorInfo = strvcat('Inverted gamma system prior for the population standard deviation of INFL with location','parameter 0.265 and 10 degrees of freedom conditional on the parameters.');
end;
lnSP = log(eps);
if (isfield(DSGEModel,'Solution')==1)&(isfield(DSGEModel.Solution,'F')==1)&(isempty(DSGEModel.Solution.F)==0);
   F = DSGEModel.Solution.F;
   B0 = DSGEModel.Solution.B0;
   H = DSGEModel.Solution.H;
   R = DSGEModel.Solution.R;
else;
   [A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,OrderQZ);
   if mcode~=1;
      return;
   end;
   if IsAdaptiveLearning==1;
      %
      % determine the steady-state solution matrices under adaptive learning
      %
      % 1. get the structural form matrices
      %
      [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
      BeliefSys = DSGEModel.AdaptiveLearning.BeliefSys;
      %
      % 2. Compute the transformation of the structural from that we use when solving the model
      %
      r = DSGEModel.AdaptiveLearning.r;
      f = DSGEModel.AdaptiveLearning.f;
      Sperp = DSGEModel.AdaptiveLearning.Sperp;
      SMatrix = DSGEModel.AdaptiveLearning.SMatrix;
      Gtilde = DSGEModel.AdaptiveLearning.Gtilde;
      Kgf = DSGEModel.AdaptiveLearning.Kgf;
      %
      % call the function: 
      %
      [tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
      if DSGEModel.k>0
         x = 1;
      else;
         x = [];
      end;
      %
      % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
      %
      [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,BeliefSys.m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
      %
      % compute initial values for the solution matrices
      %
      [mu,F,B0] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betavec);
   end;
end;
[Sigma,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
if length(size(H))==2;
   YCov = (H'*Sigma*H)+R;
else;
   Ht = H(:,:,size(H,3));
   YCov = (Ht'*Sigma*Ht)+R;
end;
%
% locate the position of inflation among the observed variables
%
InfPos = loc(DSGEModel.VariableNames,'INFL');
%
% the value of the standard deviation of inflation
%
sigmaInf = sqrt(abs(YCov(InfPos,InfPos)));
%
% compute the height of the inverted Gamma distribution at sigmaInf
%
lnSP = logInvertedGammaPDF(sigmaInf,0.265,10);

%
% end of SystemPriorFile.m
%
