function [status,message] = CopyFile(FileName,Location)
% CopyFile: copies a file to a location with possibly a new name
%
% USAGE:
%
%       [status,message] = CopyFile(FileName,Location)
%
% REQUIRED INPUT:  FileName (string vector) with the name of the file to be copied.
%
%                  Location (string vector) with the location and possibly the new name
%                           to use.
%
% REQUIRED OUTPUT: status (boolean) that is 1 if the copy operation was successful and 0
%                           otherwise.
% OPTIONAL OUTPUT: message (string matrix) with any message that is caught from the copy
%                           operation.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 12, 2007.
%                        This version: January 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 08-12-2008: Updated the documentation.
%
% * 09-03-2009: Added quotation marks around paths for the dos copy command.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 09-01-2011: Made the function compatible with unix and mac by changing the dos
%               command to the Matlab command copyfile.
%
% * 12-01-2011: Since "copyfile" with the option 'f' doesn't guarantee that the file
%               at "Location" is overwritten, changed to dos' copy and unix's cp command
%               with force options.
%
% * 13-01-2011: Added an additional unix cp command withou the -f switch in case it is
%               not recognized.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 21-01-2016: Updated the documentation.
%
% * 20-03-2017: Updated the documentation.
%
% * 16-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 15-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if ispc==1;
   [stat,massage] = dos(['copy /Y "' FileName '" "' Location '"']);
else;
   FileName = strrep(FileName,'\','/');
   Location = strrep(Location,'\','/');
   [stat,massage] = unix(['cp -f ' FileName ' ' Location]);
   if stat~=0;
      [stat,massage] = unix(['cp ' FileName ' ' Location]);
   end;
end;
if isempty(GetExt(Location))==0;
   NewFileName = Location;
else;
   if (ispc==1)&(strcmp(Location(1,length(Location)),'\')==0);
      Location = [Location '\'];
   elseif (isunix==1)&(strcmp(Location(1,length(Location)),'/')==0);
      Location = [Location '/'];
   end;
   NewFileName = [Location GetFilename(FileName)];
end;
status = FileExist(NewFileName);
if nargout==2;
   message = massage;
end;

%
% end of CopyFile.m
%
