function [KsiTTg,PTTg] = CondPredictionSmootherHt(KsiTT1,PTT1,KsiTT,PTT,YhatT,YT,XPred,Z,K1,A,H,R,F,B0,KalmanAlgorithm)
% CondPredictionSmootherHt: Computes the smoothed estimates and covariances of the state variables for period T
%                           taking the conditioning data into account over the prediction sample.
%
% USAGE:
%
%       [KsiTTg,PTTg] = CondPredictionSmootherHt(KsiTT1,PTT1,KsiTT,PTT,YhatT,YT,XPred,Z,K1,A,H,R,F,B0,KalmanAlgorithm)
%
% REQUIRED INPUT:  KsiTT1 (vector) of dimension rx1 with the forecast estimates of the state variables at period
%                         T using data until period T-1.
%
%                  PTT1 (matrix) of dimension rxr with the forecast estimates of the state variable covariances
%                         at period T using data until period T-1.
%
%                  KsiTT (vector) of dimension rx1 with the smooth estimates of the state variables at period
%                         T using data until period T.
%
%                  PTT (matrix) of dimension rxr with smooth estimates of the state variable covariance at
%                         period T using data until period T.
%
%                  YhatT (vector) of dimension nx1 with forecast estimates of the observed variables at T
%                         using data until period T-1.
%
%                  YT (vector) of dimension nx1 with the observed variables for period T.
%
%                  XPred (matrix) of dimension kxg with data on the exoegnous variables over the conditioning
%                         horizon.
%
%                  Z (matrix) of dimension mxg with the conditioning assumptions.
%
%                  K1 (matrix) of dimension nxm for mapping the observed variables into the conditioning
%                         assumptions.
%
%                  A (matrix) of dimension kxn with the parameters on the exogenous variables in
%                         the measurement equation.
%
%                  H (matrix) of dimension rxnx(1+g) with parameters on the state variabels in the
%                         measurement equation for the conditioning sample.
%
%                  R (matrix) of dimension nxn with the covariances of the measurement errors
%                         in the measurement equation.
%
%                  F (matrix) of dimension rxr with state transitions parameters for the state
%                         equation.
%
%                  B0 (matrix) of dimension rxq with parameters on the economic shocks in the
%                         state equation.
%
%                  KalmanAlgorithm (integer) that takes the value 1 if the standard Kalman filter should be used
%                         and 2 if the square-root filter should be applied.
%
% REQUIRED OUTPUT: KsiTTg (vector) of dimension rx1 with the smooth estimates of the state variables
%                         at T conditional on the data and the conditioning assumptions.
%
%                  PTTg (matrix) of dimension rxr with the covariances for the smooth estimates of the
%                         state variables at T conditional on the data and the conditioning assumptions.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 18, 2011.
%                        This version: January 16, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-05-2011: Made sure the "pos" is also applied to R.
%
% * 13-10-2011: Added support for the univariate filter and smoother.
%
% * 02-01-2012: Updated the documentation.
%
% * 13-01-2012: Support for missing observations.
%
% * 16-11-2012: Took into account that "KalmanAlgorithm=4" should use the same smoother as "KalmanAlgorithm=1".
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 17-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
KsiTTg = [];
PTTg = [];
%
% determine parameters
%
n = length(YT);
r = length(KsiTT1);
k = size(XPred,1);
Q = B0*B0';
q = size(B0,2);
[m,g] = size(Z);
KsiLast = F*KsiTT1;
PLast = (F*PTT1*F')+Q;
Im = eye(m);
%
% check if the standard Kalman filtering routines should be applied or the
% square root filtering routines
%
zError = zeros(m,g);
InvMSEZ = zeros(m,m,g);
if k>0;
   Ak = K1'*A'*XPred;
else;
   Ak = zeros(m,g);
end;
if (KalmanAlgorithm==1)|(KalmanAlgorithm==4);
   %
   % intialize
   %
   LZ = zeros(r,r,g);
   Rstar = K1'*R*K1;
   %
   % run the filter over the conditioning horizon
   %
   for t=1:g;
      Hstar = H(:,:,t+1)*K1;
      HstarT = Hstar';
      %
      pos = ~isnan(Z(:,t));
      n1 = sum(pos);
      if n1>0
         PLastH = PLast*Hstar(:,pos);
         InvMSEZ(pos,pos,t) = Im(pos,pos)/((HstarT(pos,:)*PLastH)+Rstar(pos,pos));
         zPred = Ak(:,t)+(HstarT*KsiLast);
         zError(pos,t) = Z(pos,t)-zPred(pos,1);
         KalmanGain = F*PLastH*InvMSEZ(pos,pos,t);
         KsiLast = (F*KsiLast)+(KalmanGain*zError(pos,t));
         LZ(:,:,t) = F-(KalmanGain*HstarT(pos,:));
         PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+(KalmanGain*Rstar(pos,pos)*KalmanGain')+Q;
      else;
         KsiLast = F*KsiLast;
         LZ(:,:,t) = F;
         PLast = (LZ(:,:,t)*PLast*LZ(:,:,t)')+Q;
      end;
   end;
   %
   % now run the disturbance smoother
   %
   rt = zeros(r,1);
   Nt = zeros(r,r);
   for t=g:-1:1;
      Hstar = H(:,:,t+1)*K1;
      pos = ~isnan(Z(:,t));
      n1 = sum(pos);
      if n1>0;
         rt = (Hstar(:,pos)*InvMSEZ(pos,pos,t)*zError(pos,t))+(LZ(:,:,t)'*rt);
         Nt = (Hstar(:,pos)*InvMSEZ(pos,pos,t)*HstarT(pos,:))+(LZ(:,:,t)'*Nt*LZ(:,:,t));
      else;
         rt = LZ(:,:,t)'*rt;
         Nt = LZ(:,:,t)'*Nt*LZ(:,:,t);
      end;
   end;
elseif KalmanAlgorithm==2;
   %
   % initialize
   %
   KalmanGain = zeros(r,m,g);
   Rstar = K1'*R*K1;
   PSq = PSDCholeski(PLast);
   RSq = PSDCholeski(Rstar);
   %
   % run the square-root filter
   %
   for t=1:g;
      Hstar = H(:,:,t+1)*K1;
      HstarT = Hstar';
      %
      pos = ~isnan(Z(:,t));
      n1 = sum(pos);
      if n1>0;
         zPred = Ak(:,t)+(HstarT*KsiLast);
         zError(pos,t) = Z(pos,t)-zPred(pos,1);
         U = [HstarT(pos,:)*PSq RSq(pos,pos) zeros(n1,q);F*PSq zeros(r,n1) B0];
         [G,US] = qr(U');
         US = US';
         Ustar1 = pinv(US(1:n1,1:n1));
         InvMSEZ(pos,pos,t) = Ustar1'*Ustar1;
         PSq = US(n1+1:n1+r,n1+1:n1+r);
         KalmanGain(:,pos,t) = US(n1+1:n1+r,1:n1)*Ustar1;
         KsiLast = (F*KsiLast)+(KalmanGain(:,pos,t)*zError(pos,t));
      else;
         U = [F*PSq B0];
         [G,US] = qr(U');
         US = US';
         PSq = US(:,1:r);
         KsiLast = F*KsiLast;
      end;
   end;
   %
   % now run the disturbance smoother
   %
   rt = zeros(r,1);
   Nt = zeros(r,r);
   for t=g:-1:1;
      Hstar = H(:,:,t+1)*K1;
      %
      pos = ~isnan(Z(:,t));
      n1 = sum(pos);
      if n1>0;
         LZt = F-(KalmanGain(:,:,t)*HstarT);
         rt = (Hstar*InvMSEZ(:,:,t)*zError(:,t))+(LZt'*rt);
         Nt = (Hstar*InvMSEZ(:,:,t)*HstarT)+(LZt'*Nt*LZt);
      else;
         rt = F'*rt;
         Nt = F'*Nt*F;
      end;
   end;
elseif KalmanAlgorithm==3;
   %
   % initialize
   %
   Z = Z-Ak;
   Rstar = K1'*R*K1;
   sigma2i = cell(1,g);
   Kti = cell(1,g);
   zti = cell(1,g);
   Hti = cell(1,g);
   n1 = size(Z,1);
   %
   if isequal(R,diag(diag(R)))==0;
      [S,L] = eig(Rstar);
      Rstar = L;
   else;
      S = eye(m);
   end;
   %
   for t=1:g;
      pos = (~isnan(Z(:,t)));
      n1 = sum(pos);
      if n1>0;
         Yt = S(pos,pos)'*Z(pos,t);
         Ht = H(:,:,t+1)*K1*S;
         Rt = Rstar(pos,pos);
      end;
      sigma2it = zeros(1,n1);
      Ktit = zeros(r,n1);
      ztit = zeros(1,n1);
      for i=1:n1;
         Hi = Ht(:,i);
         sigma2 = (Hi'*PLast*Hi)+Rt(i,i);
         if sigma2>0;
            Kt = PLast*Hi;
            zt = Yt(i)-(Hi'*KsiLast);
            sigma2it(i) = sigma2;
            Ktit(:,i) = Kt;
            ztit(i) = zt;
            %
            % update the state and the cov matrix
            %
            KsiLast = KsiLast+(Kt*(zt/sigma2));
            PLast = PLast-((1/sigma2)*Kt*Kt');
         end;
      end;
      sigma2i{t} = sigma2it;
      Kti{t} = Ktit;
      zti{t} = ztit;
      Hti{t} = Ht;
      %
      % transition stage
      %
      KsiLast = F*KsiLast;
      PLast = (F*PLast*F')+Q;
   end;
   %
   % and now the disturbance smoother
   %
   Ir = eye(r);
   rt = zeros(r,1);
   Nt = zeros(r,r);
   for t=g:-1:1;
      %
      % initialize matrices
      %
      Ht = Hti{t};
      ztit = zti{t};
      sigma2it = sigma2i{t};
      Ktit = Kti{t};
      pos = (~isnan(Z(:,t)));
      n1 = sum(pos);
      for i=(n1:-1:1);
         sigma2 = sigma2it(i);
         if sigma2>0;
            Hi = Ht(:,i);
            Lti = Ir-((1/sigma2)*Ktit(:,i)*Hi');
            rt = (Hi*(ztit(i)/sigma2))+(Lti'*rt);
            Nt = ((Hi/sigma2)*Hi')+(Lti'*Nt*Lti);
         end;
      end;
      %
      % transition
      %
      if t>1;
         rt = F'*rt;
         Nt = F'*Nt*F;
      end;
   end;
end;
%
% compute r(T|T+g) and N(T|T+g), using r(T+1|T+g) and N(T+1|T+g), respectively,
% as input for the distrubance smoother
%
pos = ~isnan(YT);
n1 = sum(pos);
if n1>0;
   InvMSEY = eye(n1)/((H(:,pos,1)'*PTT1*H(:,pos,1))+R(pos,pos));
   LT = F-(F*PTT1*H(:,pos,1)*InvMSEY*H(:,pos,1)');
   rt = (H(:,pos,1)*InvMSEY*(YT(pos,1)-YhatT(pos,1)))+(LT'*rt);
   Nt = (H(:,pos,1)*InvMSEY*H(:,pos,1)')+(LT'*Nt*LT);
else;
   rt = F'*rt;
   Nt = F'*Nt*F;
end;
%
% now compute the smoothed values for
%
KsiTTg = KsiTT1+(PTT1*rt);
PTTg = PTT1-(PTT1*Nt*PTT1);

%
% end of CondPredictionSmootherHt.m
%
