function [S,L] = SchurDecomposition(Sigma,lb)
% SchurDecomposition: Decomposes a covarince matrix based on the Schur decomposition.
%
% USAGE:
%
%       [S,L] = SchurDecomposition(Sigma,lb)
%
% REQUIRED INPUT:  Sigma (nxn matrix) that should be positive semidefinite. The rank of Sigma
%                        is s<=n.
%
% OPTIONAL INPUT:  lb (positive scalar) which gives the lower bound for a positive eigenvalue
%                        of Sigma. (default is 1.0e-12)
%
% REQUIRED OUTPUT: S (nxs matrix) containing the eigenvectors of Sigma such that Sigma = S*Lambda*S',
%                        where S'*S = eye(s), and Lambda is an sxs diagonal matrix with the non-zero
%                        eigenvalues of Sigma, ordered from the largest to the smallest.
%
% OPTIONAL OUTPUT: L (sxs matrix) containing the non-zero eigenvalues of Sigma, i.e., the matrix
%                        Lambda.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 17, 2006.
%                        This version: January 17, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-11-2006: Added the optional output L. Renamed the function from DecomposeStateMSE to SchurDecomposition.
%
% * 23-02-2007: Made sure that L has zeros below and above the diagonal.
%
% * 05-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 18-12-2008: Updated the documentation.
%
% * 26-03-2009: The code makes sure that elements close to zero are set to zero.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 23-10-2014: Made sure that the largest absolute value of each column of S is positive. This
%               ensures that changes to the underlying schur function doesn't results in sign
%               changes of the columns of S.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 30-04-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 15-04-2020: Added the optional lb input variable.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 17-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
if nargin==1;
   lb = 1.0e-12;
end;
%
[S,Lambda] = schur(Sigma);
%
% make sure we sort the diagonal elements of Lambda from the largest to the smallest
% We then reorder the columns of S accordingly
%
[NewLambda,ind] = sort(diag(Lambda));
ind = flipud(ind);
Lambda = Lambda(ind,ind);
S = S(:,ind);
%
% set numbers close to zero to zero.
%
Lpos = (Lambda>lb);
Lneg = (Lambda<-lb);
Lambda = (Lambda.*Lpos)+(Lambda.*Lneg);
rstar = rank(Lambda);
S = S(:,1:rstar);
%
% find the indices of the largest absolute elements of each column of S
%
[Scolmax,colind] = max(abs(S));
Ssign = zeros(rstar,rstar);
for i=1:rstar;
   if S(colind(i),i)>0;
      Ssign(i,i) = 1;
   else;
      Ssign(i,i) = -1;
   end;
end;
S = S*Ssign;
if nargout==2;
   L = diag(diag(Lambda(1:rstar,1:rstar)));
end;

%
% end of SchurDecomposition.m
%
