function [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = SquareRootSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,B0)
% SquareRootSmoother: Computes the update and smooth estimates of the state variables and their covariances
%                     using a square root filter.
%
% USAGE:
%
%       [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat,NtMat] = SquareRootSmoother(Yerror,SigmaSqRoot,InvMSEY,KalmanGain,Ksitt1,Ptt1,H,F,R,B0)
%
% REQUIRED INPUT:  Yerror (matrix) of dimension nxT with the 1-step ahead forecast errors of the observed variables.
%
%                  SigmaSqRoot (matrix) of dimension nxnx(T-StartPeriod+1) with the lower traingular square root
%                              of the inverse of the 1-step ahead forecast error covariance matrix of the observed
%                              variables.
%
%                  InvMSEY (matrix) of dimension nxnxT with the inverse of the 1-step ahead forecast error
%                              covariance matrix, computed from the square root filter.
%
%                  KalmanGain (matrix) of dimension rxnxT with the Kalman gains matrix from the square root filter.
%
%                  Ksitt1 (matrix) of dimension rxT with the 1-step ahead forecasts of the state variables.
%
%                  Ptt1 (matrix) of dimension rxrxT with the covariance matrix of the 1-step ahead forecasts
%                              of the state variables.
%
%                  H (matrix) of dimension rxn with parameter values on the r state variables (ksi) in the measurement equation.
%
%                  F (matrix) of dimension rxr with parameter values on lagged state variables in the state equation.
%
%                  B0 (matrix) of dimension rxq with coefficients on the structural shocks in the state equation.
%
% REQUIRED OUTPUT: Ksitt (matrix) of dimension rxT with the updated predictions of the state vector.
%
%                  Ptt (matrix) of dimension rxrxT with the covariances of the updated predictions of the state vector.
%
%                  KsitT (matrix) of dimension rxT with the smooth predictions of the state vector.
%
%                  PtT (matrix) of dimension rxrxT with the covariances of the smooth predictions of the state vector.
%
%                  rttMat (rxT) matrix with standardized forecast errors.
%
% OPTIONAL OUTPUT: rTMat (matrix) of dimension rxT with the r(t|T)-vectors.
%
%                  NtMat (matrix) of dimension rxrxT with the Nt matrices.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 29, 2010.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-12-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 27-01-2014: Updated the documentation.
%
% * 13-01-2015: Updated the documentation.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 17-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the rtvec output variable to rTMat.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% determine some dimentions
%
[r,T] = size(Ksitt1);
Q = B0*B0';
HT = H';
%
% Initializa the output
%
Ksitt = zeros(r,T);
Ptt = zeros(r,r,T);
KsitT = zeros(r,T);
PtT = zeros(r,r,T);
rttMat = zeros(r,T);
rTMatTmp = zeros(r,T);
NtMatTmp = zeros(r,r,T);
%
% we loop from period T and backwards to t=1
%
for t=(T:-1:1);
   Ptt1H = Ptt1(:,:,t)*H;
   %
   % calculate the updated state prediction and its MSE
   %
   Ksitt(:,t) = Ksitt1(:,t)+(Ptt1H*InvMSEY(:,:,t)*Yerror(:,t));
   Ptt(:,:,t) = Ptt1(:,:,t)-(Ptt1H*InvMSEY(:,:,t)*Ptt1H');
   rttMat(:,t) = H*InvMSEY(:,:,t)*Yerror(:,t);
   if t<T;
      Lt = F-(KalmanGain(:,:,t)*HT);
      rt = (H*InvMSEY(:,:,t)*Yerror(:,t))+(Lt'*rt);
      KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rt);
      Ntilde = [H*SigmaSqRoot(:,:,t)' Lt'*Nstar];
      [G,NS] = qr(Ntilde');
      Nstar = NS(1:r,:)';
      Nt = Nstar*Nstar';
      PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*Nt*Ptt1(:,:,t)');
   else;
      %
      % Updates and smooth predictions and MSEs have the same values for t=T
      %
      KsitT(:,T) = Ksitt(:,T);
      PtT(:,:,T) = Ptt(:,:,T);
      rt = H*InvMSEY(:,:,T)*Yerror(:,T);
      Ntilde = [H*SigmaSqRoot(:,:,T)' zeros(r,r)];
      [G,NS] = qr(Ntilde');
      Nstar = NS(1:r,:)';
      Nt = Nstar*Nstar';
   end;
   rTMatTmp(:,t) = rt;
   NtMatTmp(:,:,t) = Nt;
   %
   % make sure that the diagonal of Ptt and PtT are non-negative
   %
   Ptt(:,:,t) = diag(abs(diag(Ptt(:,:,t))))+tril(Ptt(:,:,t),-1)+tril(Ptt(:,:,t),-1)';
   PtT(:,:,t) = diag(abs(diag(PtT(:,:,t))))+tril(PtT(:,:,t),-1)+tril(PtT(:,:,t),-1)';
end;
if nargout>=6;
   rTMat = rTMatTmp;;
   if nargout==7;
      NtMat = NtMatTmp;
   end;
end;

%
% end of SquareRootSmoother.m
%
