function [lnL,status,lnLt,Yhat,MSEY,sigma2i,Kti,zti,Hti,Ksitt1,Ptt1] = UnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c)
% UnivariateKalmanFilterMO: Calculates the value of the log-likelihood function (based on Gaussian errors and initial entry
%                           for the state vector) for the observed data using the state-space representation and the univariate
%                           Kalman filter when there can be missing observations.
%
% USAGE:
%
%       [lnL,status,lnLt,Yhat,MSEY,sigma2i,Kti,zti,Hti,Ksitt1,Ptt1] = UnivariateKalmanFilterMO(Y,X,A,H,F,Q,R,KsiLast,initP,...
%                                                                            MaxIter,Tolerance,StartPeriod,AllowUnitRoot,c)
%
% The measurement equation is given by:
%
%                      y(t) = A'*x(t) + H'*ksi(t) + w(t)
%
% while the state equation is:
%
%                      ksi(t) = F*ksi(t-1) + v(t)
%
% while w(t) and v(t) are white noise with zero mean and covariance matrices R and Q, respectively. The vector x(t) is
% predetermined. Furthermore, ksi(1) is uncorrelated with v(t) and w(t).
%
% REQUIRED INPUTS:  Y (nxT) matrix with observed data.
%
%                   X (kxT) matrix with exogenous of predetermined data.
%
%                   A (kxn) matrix with parameter values on the k X variables in the measurement equation.
%
%                   H (rxn) matrix with parameter values on the r state variables (ksi) in the measurement equation.
%
%                   F (rxr) matrix with parameter values on lagged state variables in the state equation.
%
%                   Q (rxr) positive semidefinite matrix with covariances for the innovations in the state equation.
%
%                   R (nxn) positive semidefinite matrix with covariances for the innovations in the measurement equation.
%
%                   KsiLast (rx1) vector with initial values for the state vector.
%
%                   initP, variable indicating method of determining initial value for P(1|0), the MSE matrix for
%                          ksi(1|0). If initP=1 (default), the P(1|0) is given by the unconditional covariance matrix of ksi,
%                          calculated through the vec operator. Since this operation can be excessively slow for large systems
%                          (when r^2 is a large number) the Doubling Algorthm can be chosen to approximate the unconditional
%                          variance. This algorithm is called if initP=2. Finally, if initP=3, then P(1|0) = c*eye(r*r).
%                   NOTE: If F has eigenvalues outside the unit circle, then initP is always set to 3!
%
%                   MaxIter (integer), the maximum number of iterations to use with the doubling algorithm
%                          divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                   Tolerance (integer), determines the tolerance value for the doubling algorithm.
%
%                   StartPeriod, positive integer indicating first observation to use when calculating the log-likelihood value. The
%                       default value is 1.
%
%                   AllowUnitRoot (boolean) which is 1 if unit roots should be allowed for 0 otherwise.
%
%                   c, positive real number indicating the variance of P(1|0) when initP=3. The default value is 1000.
%
% REQUIRED OUTPUTS: lnL, scalar value of the log-likelihood function.
%
%                   status (boolean) indicating the covariance stationarity of the state variables. If all the
%                          eigenvalues of F are inside the unit circle, then status=0 and the log-likelihood function
%                          is calculated. If not, then status=1 and initP is set to 3. In addition, status can be
%                          -1 if ths forecast covariance matrix is not positive definite for some time period.
%
% OPTIONAL OUTPUTS: lnLt (1x(T-StartPeriod+1)) vector with the time t values of the log-densities that sum to the value lnL.
%
%                   Yhat (nx(T-StartPeriod+1)) matrix with the predictions [y(StartPeriod|StartPeriod-1) ... y(T|T-1).
%
%                   MSEY (nxnx(T-StartPeriod+1)) 3 dimensional matrix with MSEY(:,:,t-StartPeriod+1) = H'*P(t|t-1)*H + R, the mean
%                        squared error matrix for y(t|t-1).
%
%                   sigma2i (cell array) of dimenion 1x(T-StartPeriod+1) with vectors of dimension n containing the values
%                        for Sigma(t,i), i=1,....,n.
%
%                   Kti (cell array) of dimenion 1x(T-StartPeriod+1) with matrices of dimension rxn with the Kalman gain
%                        matrices K(t,i), i=1,...,n.
%
%                   zti (cell array) of dimension 1x(T-StartPeriod+1) with vector of dimension n containing the values for
%                        the the prediction errors z(t,i), i=1,...,n.
%
%                   Hti (cell array) of dimension 1x(T-StartPeriod+1) with matrices of dimension rxn containing the values
%                        for the (transformed) H matrices.
%
%                   Ksitt1, (rx(T-StartPeriod+1)) matrix with the predictions [ksi(StartPeriod|StartPeriod-1) ... ksi(T|T-1)].
%
%                   Ptt1 (rxrx(T-StartPeriod+1)) 3 dimensional matrix with Ptt1(:,:,t-StartPeriod+1) = P(t|t-1).
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: October 11, 2011.
%                        This version: January 16, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-10-2011: Fixed a bug with setting values for "sigma2i", "Kti", "zti", and "Hti".
%
% * 03-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 25-06-2013: The predicted values of the observables now also cover variables that are missing.
%
% * 27-01-2014: Updated the documentation.
%
% * 28-01-2014: Switched the "inv" function for "\".
%
% * 13-01-2015: Updated the documentation.
%
% * 02-09-2015: The recursive estimation breaks if the log-likelihood is a NaN.
%
% * 22-01-2016: Updated the documentation.
%
% * 21-03-2017: Updated the documentation.
%
% * 17-01-2018: Updated the documentation.
%
% * 22-01-2019: Updated the documentation.
%
% * 09-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-01-2022: Updated the documentation.
%
% * 04-01-2023: Updated the documentation.
%
% * 16-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%
% initializing
%
lnL = 0;
status = 0;
%
% check the covariance stationarity of the state vector.
%
if max(abs(eig(F)))>=1;
   if AllowUnitRoot==0;
      status = 1;
   elseif (AllowUnitRoot==1)&(max(abs(eig(F)))>1)
      status = 1;
   end;
   initP = 3;
end;
%
% Determine dimensions. Checking if dimensions match is not undertaken!
%
[n,T] = size(Y);
if isempty(X)==0;
   Ax = A'*X;
else;
   Ax = zeros(n,T);
end;
Y = Y-Ax;
%
% store the original measurement equation data
%
HOld = H;
ROld = R;
HTOld = H';
%
r = size(H,1);
%
% Adjust data for measurement error covariance matrix
%
if isequal(R,diag(diag(R)))==0;
   [S,L] = eig(R);
   H = H*S;
   R = L;
else;
   S = eye(n);
end;
HT = H';
%
% Compute initial values
%
if initP==1;
   PLast = reshape((eye(r*r)-kron(F,F))\vec(Q),r,r);
elseif initP==2;
   %
   % The maximum number of iterations is set to MaxIter*100, while the convergence value is 1.0e-8
   %
   [PLast,stat] = DoublingAlgorithmLyapunov(F,Q,MaxIter*100,10^(-(Tolerance+1)));
else;
   if c<=0;
      c = 1000;
   end;
   PLast = c*eye(r);
end;
%
% Initialize temporary matrices
%
if nargout>2;
   YhatTmp = NaN*ones(n,T);
   MSEYTmp = NaN*ones(n,n,T);
   Ksitt1Tmp = zeros(r,T);
   Ksitt1Tmp(:,1) = KsiLast;
   Ptt1Tmp = zeros(r,r,T);
   sigma2iTmp = cell(1,T);
   KtiTmp = cell(1,T);
   ztiTmp = cell(1,T);
   HtiTmp = cell(1,T);
end;
lnLtTmp = zeros(1,T);
%
% Now we can compute the Kalman filtered series
%
if nargout==2;
   for t=1:T;
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      lnLConst = -n1*log(2*pi);
      %
      if n1>0;
         Yt = S(pos,pos)'*Y(pos,t);
         Ht = H(:,pos);         
         Rt = R(pos,pos);
      end;
      for i=1:n1;
         Hi = Ht(:,i);
         sigma2 = (Hi'*PLast*Hi)+ Rt(i,i);
         if sigma2>0;
            Kt = PLast*Hi;
            zt = Yt(i)-(Hi'*KsiLast);
            %
            lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2)+((zt^2)/sigma2);
            %
            % update the state and the cov matrix
            %
            KsiLast = KsiLast+(Kt*(zt/sigma2));
            PLast = PLast-((1/sigma2)*Kt*Kt');
         end;
      end;
      lnLtTmp(1,t) = lnLConst-lnLtTmp(1,t);
      if isnan(lnLtTmp(1,t))==1;
         break;
      end;
      %
      % transition stage
      %
      KsiLast = F*KsiLast;
      PLast = (F*PLast*F')+Q;
   end;
else;
   for t=1:T;
      %
      % compute forecast error and MSE-matrix
      %
      Ksitt1Tmp(:,t) = KsiLast;
      Ptt1Tmp(:,:,t) = PLast;
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      lnLConst = -n1*log(2*pi);
      %
      if n1>0;
         Yt = S(pos,pos)'*Y(pos,t);
         Ht = H(:,pos);
         Rt = R(pos,pos);
      end;
      MSEYTmp(:,:,t) = (HTOld*PLast*HOld)+ROld;
      YhatTmp(:,t) = Ax(:,t)+(HT*KsiLast);
      %
      % initialize
      %
      sigma2it = zeros(1,n1);
      Ktit = zeros(r,n1);
      ztit = zeros(1,n1);
      for i=1:n1;
         Hi = Ht(:,i);
         sigma2 = (Hi'*PLast*Hi)+ Rt(i,i);
         if sigma2>0;
            Kt = PLast*Hi;
            zt = Yt(i)-(Hi'*KsiLast);
            sigma2it(i) = sigma2;
            Ktit(:,i) = Kt;
            ztit(i) = zt;
            %
            lnLtTmp(1,t) = lnLtTmp(1,t)+log(sigma2)+((zt^2)/sigma2);
            %
            % update the state and the cov matrix
            %
            KsiLast = KsiLast+(Kt*(zt/sigma2));
            PLast = PLast-((1/sigma2)*Kt*Kt');
         end;
      end;
      %
      % store the intermediate results
      %
      lnLtTmp(1,t) = lnLConst-lnLtTmp(1,t);
      if isnan(lnLtTmp(1,t))==1;
         break;
      end;
      sigma2iTmp{t} = sigma2it;
      KtiTmp{t} = Ktit;
      ztiTmp{t} = ztit;
      HtiTmp{t} = Ht;
      %
      % transition stage
      %
      KsiLast = F*KsiLast;
      PLast = (F*PLast*F')+Q;
   end;
end;
lnL = (1/2)*sum(lnLtTmp(1,StartPeriod:T));
%
% test if the log-likelihood is real
%
if (isreal(lnL)==0)|(isnan(lnL)==1);
   status = -1;
end;
%
% Fix output to match StartPeriod
%
if nargout>=3;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   sigma2i = sigma2iTmp(1,StartPeriod:T);
   Kti = KtiTmp(1,StartPeriod:T);
   zti = ztiTmp(1,StartPeriod:T);
   Hti = HtiTmp(1,StartPeriod:T);
end;

%
% end of UnivariateKalmanFilterMO.m
%
