function [action,DriverFunctions] = DriverSelectionDLG(selector,DSGEModel,CurrINI)
% DriverSelectionDLG: Dialog for selecting tools to include in a driver run sequence.
%
% USAGE:
%
%       [action,DriverFunctions] = DriverSelectionDLG(selector,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  selector (string vector), with the possible values "init", "reorder", "reset", "open",
%                           "remove", and "quit".
%
% OPTIONAL INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model
%                           for YADA.
%
%                  CurrINI (structure), with initialization information.
%
% OPTIONAL OUTPUT: action (string vector) taking the value "OK" or "Cancel". Depends on which button was
%                           clicked on.
%
%                  DriverFunctions (string matrix) with the names of the mat-files to
%                           use for the driver sequence.
%
% NOTE: If selector = "init", then  all optional inputs and outputs are required.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: January 23, 2023.
%                        This version: June 19, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-01-2023: Updated the documentation.
%
% * 26-01-2023: Removed the about dialog after successfully having added a driver function.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DriverFunctions = DSGEModel.DriverFunctions;
   scrsz = CurrINI.scrsz;
   ImageStructure = ReadImages;
   %
   % try to determine the width and the height
   %
   width = 800;
   height = 400;
else;
   DriverSelectionGUI = findobj('Type','figure','Tag','DriverSelectionDLG');
   controls = get(DriverSelectionGUI,'UserData');
   CurrINI = get(controls.cancel,'UserData');
end;
%
switch selector
   case 'init'
      DriverSelectionGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','DriverSelectionDLG.m', ...
         'MenuBar','none', ...
         'Units','pixels', ...
         'Position',[(scrsz(3)-width)/2 (scrsz(4)-height)/2 width height], ...
         'Tag','DriverSelectionDLG', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name','YADA - Select Driver Functions', ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','DriverSelectionDLG quit', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(DriverSelectionGUI,'DockControl','off');
      end;
      %
      AxesBox([2 2 width-2 height-10],'Driver Functions Selection',100,[0.5 1],'on',CurrINI);
      %
      % Create a listbox
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[20 60 width-135 height-85], ...
         'FontSize',10, ...
         'FontName','FixedWidth', ...
         'Style','listbox', ...
         'Enable','on', ...
         'Min',0, ...
         'Max',1, ...
         'HorizontalAlignment','left', ...
         'String',DriverFunctions, ...
         'UserData',DriverFunctions, ...
         'Visible','on', ...
         'Tag','VariablesListBox',...
         'Value',1);
      %
      % move up buttons
      %
      controls.up = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-95 ((height-80)/2)+60 75 25], ...
         'TooltipString','Move Up', ...
         'CData',ImageStructure.up, ...
         'Enable','on', ...
         'Callback','DriverSelectionDLG reorder', ...
         'Tag','MoveUpButton');
      %
      % move down button
      %
      controls.down = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-95 ((height-140)/2)+60 75 25], ...
         'TooltipString','Move Down', ...
         'CData',ImageStructure.down, ...
         'Enable','on', ...
         'Callback','DriverSelectionDLG reorder', ...
         'Tag','MoveDownButton');
      %
      % Add new model
      %
      controls.open = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[20 20 75 25], ...
         'TooltipString','Add Function', ...
         'CData',ImageStructure.open, ...
         'Enable','on', ...
         'Callback','DriverSelectionDLG open', ...
         'Tag','AddModelButton');
      %
      % remove model
      %
      controls.remove = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[100 20 75 25], ...
         'TooltipString','Remove Function', ...
         'CData',ImageStructure.clear, ...
         'Enable','on', ...
         'Callback','DriverSelectionDLG remove', ...
         'Tag','RemoveModelButton');
      %
      % Three buttons to select action
      %
      controls.ok = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-255 20 75 25], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','DriverSelectionDLG quit', ...
         'UserData','waiting', ...
         'Tag','ButtonOK');
      %
      controls.resetbutton = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-175 20 75 25], ...
         'String','Reset', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','off', ...
         'Callback','DriverSelectionDLG reset', ...
         'Tag','ButtonReset');
      %
      controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-95 20 75 25], ...
         'String','Cancel', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'UserData',CurrINI, ...
         'Callback','DriverSelectionDLG quit', ...
         'Tag','ButtonCancel');
      %
      set(DriverSelectionGUI,'UserData',controls);
   case 'reset'
      DriverFunctions = get(controls.variables,'UserData');
      set(controls.variables,'String',DriverFunctions,'Value',1);
      set(controls.resetbutton,'Enable','off');
   case 'reorder'
      DriverFunctions = get(controls.variables,'String');
      Value = get(controls.variables,'Value');
      n = size(DriverFunctions,1);
      Order = (1:n);
      if (Value>1)&(gco==controls.up);
         %
         % move current file up
         %
         DownValue = Order(Value-1);
         Order(Value-1) = Value;
         Order(Value) = DownValue;
         set(controls.variables,'String',DriverFunctions(Order,:),'Value',Value-1);
         if MatlabNumber>=7;
            uicontrol(controls.variables);
         end;
         set(controls.resetbutton,'Enable','on');
      elseif (Value<n)&(gco==controls.down);
         %
         % move current file down
         %
         UpValue = Order(Value+1);
         Order(Value+1) = Value;
         Order(Value) = UpValue;
         set(controls.variables,'String',DriverFunctions(Order,:),'Value',Value+1);
         if MatlabNumber>=7;
            uicontrol(controls.variables);
         end;
         set(controls.resetbutton,'Enable','on');
      end;
   case 'open'
      DriverFunctions = get(controls.variables,'String');
      if isempty(DriverFunctions)==0;
         File = FixFilePath([pwd '\driver\' StringTrim(DriverFunctions(size(DriverFunctions,1),:))]);
      else;
         File = FixFilePath([pwd '\driver\*.mat']);
      end;
      if MatlabNumber>=6;
         FileType = {'*.mat', 'Matlab mat Files (*.mat)'};
         [FileName,FilePath] = uigetfile(FileType,'Driver Function File - Select File',File);
      else;
         [FileName,FilePath] = uigetfile(File,'Driver Function File - Select File');
      end;
      if (isnumeric(FileName)==0)&(isunix==1);
         if strcmp(FilePath(1,length(FilePath)),'/')==0;
            FilePath = [FilePath '/'];
         end;
      end;
      %
      % check if a file was selected
      %
      if (isnumeric(FileName)==0)&(FileExist([FilePath FileName])==1);
         DriverFile = [FilePath FileName];
         %
         % check if we can import data
         %
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               DriverFileData = load(DriverFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' DriverFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               DriverFileData = load(DriverFile);
            catch;
               ErrorStr = ['Unable to load the file "' DriverFile '". Message caught is: ' YADALastError];
            end;
         end;
         if isempty(ErrorStr)==1;
            %
            % check if it has fields
            %
            if (isfield(DriverFileData,'cmd')==1)&(isfield(DriverFileData,'answers')==1)&(isfield(DriverFileData,'drivertxt')==1)&(isfield(DriverFileData,'handle')==1);
               %
               % add the new function to the variables control
               %
               DriverFunctions = get(controls.variables,'String');
               DriverFunctions = strvcat(DriverFunctions,GetFilename(DriverFile));
               n = size(DriverFunctions,1);
               set(controls.variables,'String',DriverFunctions,'Value',n);
               if MatlabNumber>=7;
                  uicontrol(controls.variables);
               end;
               set(controls.resetbutton,'Enable','on');
               %
               % information
               %
%               txt = ['YADA has successfully included the function "' strrep(strrep(GetName(DriverFile),'.',' '),'-',' - ') '" into the driver file.'];
%               About(txt,'information',['Added Function To Driver - ' GetFilename(DriverFile)],500,600,CurrINI);
            else;
               txt = ['The driver file "' GetFilename(DriverFile) '" is not valid. It doesn''t have all the necessary information, i.e., the variables "cmd", "answers", "drivertxt" and "handle". YADA is therefore unable to include this driver function.'];
               About(txt,'error',['Error - ' GetFilename(DriverFile)],CurrINI.scrsz(4)-50,600,CurrINI);
            end;
         else;
            About(ErrorStr,'error',['Error - ' GetFilename(DriverFile)],CurrINI.scrsz(4)-50,600,CurrINI);
         end;
      end;
   case 'remove'
      CurrModel = get(controls.variables,'Value');
      DriverFunctions = get(controls.variables,'String');
      n = size(DriverFunctions,1);
      if n>1;
         set(controls.variables,'String',DriverFunctions(setdiff((1:n),CurrModel),:),'Value',1);
      else;
         set(controls.variables,'String','');
      end;
      set(controls.resetbutton,'Enable','on');
   case 'quit'
      set(controls.ok,'UserData','done');
end;
%
if strcmp(selector,'init')==1;
   set(DriverSelectionGUI,'Visible','on');
   drawnow;
   SetYADAIcon(DriverSelectionGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.variables);
   end;
   %
   waitfor(controls.ok,'UserData','done');
   if (isempty(get(DriverSelectionGUI,'CurrentObject'))==1)|(strcmp(get(get(DriverSelectionGUI,'CurrentObject'),'Type'),'uicontrol')==0);
      action = 'Cancel';
   else;
      action = get(get(DriverSelectionGUI,'CurrentObject'),'String');
      if (isempty(action)==1)|(strcmp(action,'OK')==0);
         action = 'Cancel';
      end;
   end;
   %
   DriverFunctions = get(controls.variables,'String');
   %
   delete(DriverSelectionGUI);
   drawnow;
   drawnow;
   pause(0.02);
end;

%
% end of DriverSelectionDLG.m
%
