function ScreenScaling = GetScreenScaling(GraphicsRoot)
% GetScreenScaling: determines the scaling of the screen by comparing the logical number
%                   of pixels to the physical number of pixels.
%
% USAGE:
%
%       ScreenScaling = GetScreenScaling(GraphicsRoot)
%
% REQUIRED INPUT:  GraphicsRoot (handle) to the graphics root object. For modern matlab
%                               systems it is simply given by groot, while for older ones
%                               it is 0.
%
% REQUIRED OUTPUT: ScreenScaling (scalar) which is 1 if the physical and logical agree,
%                                1.25 if the logical is based on a scaling by 125%, and
%                                so on.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: February 4, 2021.
%                        This version: January 6, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-02-2021: Added a try-catch statement to the code in case the matlab version doesn't support
%               the built in java command to get the physical screen size.
%
% * 04-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%


%
% determine the scaling of the screen:
%
% Taken from the article "Working with non-standard DPI displays"
% on https://undocumentedmatlab.com/articles/working-with-non-standard-dpi-displays
%
% To get the physical screen size use the following builtin Java command
%
try;
   jScreenSize = java.awt.Toolkit.getDefaultToolkit.getScreenSize;
catch;
   ScreenScaling = 1;
   return;
end;
ScreenWidth = jScreenSize.getWidth;
ScreenHeight = jScreenSize.getHeight;
%
% to get the scaled screen size, we use matlabs command
%
ScreenSize = get(GraphicsRoot,'ScreenSize');
%
% we determine the scaling of the screen from the width
%
ScreenScaling = ScreenWidth/ScreenSize(3);

%
% end of GetScreenScaling.m
%
