function GraphLogLikelihood(lnLt,KalmanTime,DSGEModel,CurrINI,ModelStr,NameStr)
% GraphLogLikelihood: Plots the log-likelihood function over time when it is evaluated at a given parameter value
%
% USAGE:
%
%       GraphLogLikelihood(lnLt,KalmanTime,DSGEModel,CurrINI,ModelStr,NameStr)
%
% REQUIRED INPUT:  lnLt (1xT vector) with the values of the log-likelihood function based on the Kalman filter
%                       calculations.
%
%                  KalmanTime (scalar) with the computational time for the filter recursions when computing the
%                       log-likelihood.
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                       such as input files, variable and sample data.
%
%                  CurrINI (structure), with initialization information.
%
%                  ModelStr (string vector) which can be empty of state the DSGE-VAR model used.
%
%                  NameStr (string vector) which takes on the values "(Joint/Marginal/DSGE) Posterior Mode" or "Initial Values".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: November 17, 2006.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 09-11-2007: Updated the documentation.
%
% * 21-12-2007: Added a drawnow call after making the figure visibile. The function "SetXTickIntegers"
%               is now also called.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 15-09-2009: Added the log-likelihood value to the title of the accumulated log-likelihood.
%
% * 06-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 04-01-2012: Updated the documentation.
%
% * 16-11-2012: Added the "KalmanTime" input variable.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 13-03-2014: Added the "ModelStr" input variable.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% create a vector with dates
%
DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
%
% fix it if the length of the data is too short
%
if length(DateVector)>length(lnLt);
   DateVector = DateVector(1,length(DateVector)-length(lnLt)+1:length(DateVector));
end;
%
% create the figure
%
FigHandle = figure('Visible','off', ...
                   'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
                   'Tag','LogLikelihood', ...
                   'Name',['Log-Likelihood Function - ' ModelStr NameStr]);
%
SetFigureProperties(FigHandle,CurrINI);
%
subplot(2,1,1);
%
% start up a plot
%
LineHandle = plot(DateVector,lnLt,'-','Color',[0.75 0 0],'LineWidth',2);
if strcmp(NameStr,'DSGE Posterior Mode')==0;
   NameStr = lower(NameStr);
else;
   NameStr = 'DSGE posterior mode';
end;
SetAxesFonts(CurrINI,'Title',['Log-likelihood function ln p(y(t)|x(t),..,y(t-1),...) at ' NameStr]);
axis('tight');
%
% see if we should add a zero line
%
ylim = get(gca,'YLim');
add_on = 0.1*(ylim(2)-ylim(1));
ylim = [ylim(1)-add_on ylim(2)+add_on];
set(gca,'YLim',ylim);
if (ylim(1)<0)&(ylim(2)>0);
   hold('on');
   plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
   %
   % make sure the zero line is displayed below the observed variable
   %
   kids = get(gca,'Children');
   kids = [kids(2) kids(1)]';
   set(gca,'Children',kids);
end;
hold('off');
%
% Now we try to display the accumulated log-likelihood
%
subplot(2,1,2)
%
% start up a plot
%
LineHandle = plot(DateVector,cumsum(lnLt),'-','Color',[0.75 0 0],'LineWidth',2);
SetAxesFonts(CurrINI,'Title',['Accumulated log-likelihood (\Sigma_{t=1}^{T} ln p(y(t)|x(t),..,y(t-1),...) = ' num2str(sum(lnLt),['%0.' int2str(CurrINI.decimals) 'f']) '); time = ' num2str(KalmanTime) ' sec.']);
axis('tight');
SetXTickIntegers(gca);
%
% display the graphs
%
SetToolbarImages(FigHandle,CurrINI.images);
set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
drawnow;
pause(0.02);

%
% end of GraphLogLikelihood.m
%
