function [thetaMargMode,UpdateMargMode,Xprior,Yprior,BoundsMatrix,Xposterior,Yposterior] = GraphPosteriorDensity(thetaPostSample,NumBurnin,CurrChain,UpdatePostSample,UpdateMode,UpdateNames,UpdatePriorX,UpdatePriorY,UpdatePriorBounds,DSGEModel,CurrINI,controls)
% GraphPosteriorDensity: Creates plots of the posterior densities for all original parameters or estimates
%                        the posterior mode of the marginal distribution.
%
% USAGE:
%
%       [thetaMargMode,UpdateMargMode,Xprior,Yprior,BoundsMatrix,Xposterior,...
%                          Yposterior] = GraphPosteriorDensity(thetaPostSample,NumBurnin,CurrChain,UpdatePostSample,...
%                                                UpdateMode,UpdateNames,UpdatePriorX,UpdatePriorY,UpdatePriorBounds,...
%                                                DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumIterxNumParam with all the posterior
%                                  draws of the original parameters.
%
%                  NumBurnin (integer) with the number of burn-in draws.
%
%                  CurrChain (integer) with the number for the used sampling chain.
%
%                  UpdatePostSample (matrix) of dimension (NumIter-NumBurnin x NumAdditional) with
%                                  the post burnin sample draws of the parameters that are functions
%                                  of the estimated parameters.
%
%                  UpdateMode (vector) of dimension NumAdditionalx1 with the posterior mode estimates
%                                  of the parameters that are functions of the estimated parameters.
%
%                  UpdateNames (string matrix) of dimension (NumAdditional x N1) with the names of the
%                                  additional parameters.
%
%                  UpdatePriorX (matrix) of dimension NumAdditional x NumValues with the values on the
%                                  X-axis for the prior densities of the NumAdditional additional
%                                  parameters.
%
%                  UpdatePriorY (matrix) of dimension NumAdditional x NumValues with the density values
%                                  for the Y-axis of the prior densities of the NumAdditional additional
%                                  parameters.
%
%                  UpdatePriorBounds (matrix) of dimension NumAdditional x 2 with suitable lower and
%                                  upper bounds for the additional parameters.
%
%                  DSGEModel (structure)  whose fields contain all required information
%                                  about the model, such as input files, variable and sample
%                                  data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL INPUT:  controls (structure) whose fields provide handles to all the UI
%                                  controls in YADA's main dialog.
%
% OPTIONAL OUTPUT: thetaMargMode (vector) of dimension NumParamx1 with all estimated marginal posterior
%                                  mode values. The estimates are determined from the maximum of
%                                  the Kernel density values.
%
%                  UpdateMargMode (vector) of dimension NumAdditionalx1 with the estimated marginal
%                                  posterior mode values of the parameters that are a function of the
%                                  estimated parameters.
%
%                  Xprior (matrix) of dimension NumParamxNumPoints. It contains the x-axis data for all
%                            the marginal (system) prior densities.
%
%                  Yprior (matrix) of dimension NumParamxNumPoints. It contains the y-axis data for all
%                            the marginal (system) prior densities.
%
%                  BoundsMatrix (matrix) of dimension NumParamx2. It contains information about possible
%                            upper and lower bounds of the parameters.
%
%                  Xposterior (matrix) of dimension NumParam+NumAdditional x 256. It contains the x-axis
%                             data for all marginal posterior densities.
%
%                  Yposterior (matrix) of dimension NumParam+NumAdditional x 256. It contains the y-axis
%                             data for all marginal posterior densities.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 15, 2006.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-12-2006: Changed the wait dialog text such that the parameter information comes last. The function also
%               saves the figures to disk automatically.
%
% * 19-12-2006: Made sure that x and f vectors have the right dimension.
%
% * 08-01-2007: Placed a StringTrim command around parameter name is estimation wait dialog.
%
% * 08-02-2007: Added a second "drawnow" call at the end of the figure loop.
%
% * 18-04-2007: Changed the plot-style for the prior from dotted to dashed.
%
% * 09-11-2007: Updated the documentation.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 10-03-2008: The value of "CurrChain" is now used in the figure filenames.
%
% * 17-03-2008: Made sure that the y-axis height is meaningful relative to the posterior density height.
%
% * 04-04-2008: The function can now be used to estimate the posterior mode of the marginal distribution.
%
% * 23-05-2008: Updated the documentation.
%
% * 26-05-2008: The function now asks uses a subset of the posterior draws when "DSGEModel.PosteriorDensityValue"
%               is 3 or 4. The input variable controls is now also used.
%
% * 21-07-2008: Added the "LowerBound" input variable when calling "GraphPriorDensity".
%
% * 19-12-2008: Updated the documentation.
%
% * 25-02-2009: Added the input variables "UpdatePostSample", "UpdateMode" and "UpdateNames" so that the posterior
%               densities of additional parameters can also be prepared and plotted.
%
% * 26-02-2009: The function now has two optional output variables "thetaMargMode" and "UpdateMargMode". These
%               replace the old optional output variable "PostMode".
%
% * 02-03-2009: Added the three input variables "UpdatePriorX", "UpdatePriorY", and "UpdatePriorBounds" that
%               hold data for the implied prior densities of the parameters that are a function of the
%               estimated parameters. The function now checks if the user has "YADAksdensity" or "ksdensity"
%               on the path. If not, the normal kernel from the kernel density estimation toolbox is used
%               when the user has set "DSGEModel.PosteriorDensityValue" to unity, i.e., a Gaussian kernel.
%               Increased the number of points in the grid from 2^7 to 2^8 when calling the "YADAMHDensEst"
%               function.
%
% * 13-03-2009: Added support for the lower bound of the Pareto prior density, corrected the x-lim lower bound
%               for the gamma, inverted gamma, and beta priors, while the x-lim upper bound for the beta prior
%               was also corrected.
%
% * 02-04-2009: Took into account that the thetaDist value for the Pareto distribution has changed from 10
%               to 11.
%
% * 28-09-2009: The function "GraphPriorDensity" now requires the input variables "thetaPositions" and
%               "ModelParameters".
%
% * 16-12-2009: Added the code for saving the plot data to a mat-file.
%
% * 23-12-2009: The function now has try-catch code around the kernel density estimation calls. Also, the
%               "XLim" property in the graphs ensure that the lower value is smaller than the upper!
%
% * 06-01-2010: Updated the documentation.
%
% * 26-02-2010: Made sure that the optional output variables are properly initialized.
%
% * 17-03-2010: Updated the documentation.
%
% * 01-12-2010: The guestimated upper bound of the update parameters is now taken into account when setting
%               the XLim vector for such parameters.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 08-02-2011: The function now checks if updated parameters are indeed varying or fixed.
%
% * 04-01-2012: Updated the documentation.
%
% * 11-04-2012: Added try-catch code to check if memory is sufficient when concatenating "thetaPostSample"
%               and "UpdatePostSample".
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 29-08-2016: Dealt with an HG2 issue for CurrentFigure.
%
% * 10-02-2017: Updated the documentation.
%
% * 24-05-2017: The function can now separate posterior draws from prior draws from a system prior by the
%               CurrChain variable. If it's empty then prior draws are given to the function.
%
% * 01-06-2017: Added the optional output variables "Xprior,Yprior,BoundsMatrix,Xposterior,Yposterior".
%
% * 08-06-2017: Added full support for the system prior.
%
% * 28-06-2017: Updated the documentation.
%
% * 23-08-2017: Made sure that controls is treated as an optional input variable.
%
% * 11-09-2017: Took into account that a sub-sample of the posterior draws doesn't apply to SMC posterior
%               sampling.
%
% * 08-01-2018: Updated the documentation.
%
% * 15-01-2018: Took into account that the system prior draws nees not exist.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 11-10-2018: Updated the documentation.
%
% * 30-10-2018: Made sure that IsPosterior can affect the use of a sub-sample.
%
% * 21-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 19-05-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize
%
if nargout==2;
   thetaMargMode = [];
   UpdateMargMode = [];
elseif nargout==7;
   thetaMargMode = [];
   UpdateMargMode = [];
   Xprior = [];
   Yprior = [];
   BoundsMatrix = [];
   Xposterior = [];
   Yposterior = [];
end;
%
% locate the controls structure
%
if nargin==11;
   maingui = findobj('Type','figure','Tag','YADA');
   controls = get(maingui,'UserData');
end;
%
[NumIter,NumParam] = size(thetaPostSample);
%
% check if we have posterior or prior draws
%
if isempty(CurrChain)==0;
   IsPosterior = 1;
else;
   IsPosterior = 0;
end;
%
% check if the model has a system prior
%
if isequal(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
   IsSystemPrior = 0;
else;
   IsSystemPrior = 1;
end;
%
% Try to load the posterior mode estimates
%
if IsPosterior==1;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
else;
   ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\SystemPriorMode-' DSGEModel.NameOfModel '.mat']);
end;
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      ModeData = load(ModeFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      ModeData = load(ModeFile);
   catch;
      ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
theta = ModeData.theta;
thetaMode = ModeData.thetaMode;
ParameterNames = ModeData.ParameterNames;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
LowerBound = ModeData.LowerBound;
thetaPositions = ModeData.thetaPositions;
ModelParameters = ModeData.ModelParameters;
%
% directory for saving graphs
%
GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
%
% load the data for plotting the prior densities
%
if IsPosterior==0;
   DSGEModel.PriorKernel = 0;
end;
if (IsSystemPrior==0)|(IsPosterior==0);
   [Xprior,Yprior,BoundsMatrix] = GraphPriorDensity(theta,thetaDist,LowerBound,PriorDist,ParameterNames,thetaPositions,ModelParameters,controls,DSGEModel,CurrINI);
else;
   [XMargPrior,YMargPrior,BoundsMatrix,Xprior,Yprior] = GraphSystemPriorDensity(DSGEModel,CurrINI);
end;
if isempty(BoundsMatrix)==1;
   BoundsMatrix = [-Inf*ones(NumParam,1) Inf*ones(NumParam,1)];
end;
%
% check if we should use a sub-sample
%
thetaPostSample = thetaPostSample(NumBurnin+1:NumIter,:);
if (IsPosterior==1)&(DSGEModel.PosteriorSampler<=7)&(DSGEModel.PosteriorDensityValue>=3);
   %
   % select which posterior draws to use
   %
   UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumIter-NumBurnin);
   if length(UseDraws)<NumIter-NumBurnin;
      thetaPostSample = thetaPostSample(UseDraws,:);
      if isempty(UpdatePostSample)==0;
         UpdatePostSample = UpdatePostSample(UseDraws,:);
      end;
   end;
end;
%
ParameterNames = ParameterNames.estimated;
if isempty(UpdatePostSample)==1;
   NumAdditional = 0;
else;
   NumAdditional = size(UpdatePostSample,2);
   try;
      thetaPostSample = [thetaPostSample UpdatePostSample];
      clear('UpdatePostSample');
      ParameterNames = strvcat(ParameterNames,UpdateNames);
      thetaMode = [thetaMode;UpdateMode];
   catch;
      %
      % there was a problem
      %
      try;
         thetaPostSample = cat(2,thetaPostSample,UpdatePostSample);
         clear('UpdatePostSample');
         ParameterNames = strvcat(ParameterNames,UpdateNames);
         thetaMode = [thetaMode;UpdateMode];
      catch;
         NumAdditional = 0;
      end;
   end;
end;
%
% first we display a wait dialog
%
if IsPosterior==1;
   if DSGEModel.PosteriorDensityValue>2;
      WaittxtAdd = [' Estimation started at: ' StringTrim(datestr(now,14)) '. Using ' int2str(size(thetaPostSample,1)) ' draws of the ' int2str(NumIter-NumBurnin) ' post-burnin draws available. This may take a very long time...'];
      ShowCancel = 1;
   else;
      WaittxtAdd = [' Estimation started at: ' StringTrim(datestr(now,14)) '.'];
      ShowCancel = 0;
   end;
   txt = ['Please wait while YADA computes kernel density estimates of the marginal posterior densities.' WaittxtAdd ' Current parameter: ' ParameterNames(1,:) '           - 1 out of ' int2str(NumParam+NumAdditional) '.'];
   WaitHandle = WaitDLG(txt,'information','Kernel Density Estimation - Posterior',500,150,CurrINI,ShowCancel);
else;
   WaittxtAdd = [' Estimation started at: ' StringTrim(datestr(now,14)) '.'];
   txt = ['Please wait while YADA computes kernel density estimates of the marginal prior densities.' WaittxtAdd ' Current parameter: ' ParameterNames(1,:) '           - 1 out of ' int2str(NumParam+NumAdditional) '.'];
   WaitHandle = WaitDLG(txt,'information','Kernel Density Estimation - Prior',500,150,CurrINI,0);
end;
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we try to construct the x-axis and y-axis data for the posterior densities
%
StopEstimation = 0;
for i=1:NumParam+NumAdditional;
   if IsPosterior==1;
      txt = ['Please wait while YADA computes kernel density estimates of the marginal posterior densities.' WaittxtAdd ' Current parameter: ' StringTrim(ParameterNames(i,:)) ' - ' int2str(i) ' out of ' int2str(NumParam+NumAdditional) '.'];
   else;
      txt = ['Please wait while YADA computes kernel density estimates of the marginal prior densities.' WaittxtAdd ' Current parameter: ' ParameterNames(i,:) '           - ' int2str(i) ' out of ' int2str(NumParam+NumAdditional) '.'];
   end;
   set(WaitControls.text,'String',txt);
   drawnow;
   if DSGEModel.PosteriorDensityValue==1;
      if i<=NumParam;
         if exist('YADAksdensity')==2;
            [f,x] = YADAksdensity(thetaPostSample(:,i),'npoints',256);
         else;
            if exist('ksdensity')==2;
               [f,x] = ksdensity(thetaPostSample(:,i),'npoints',256);
            else;
               [x,f] = KernelDensityPlot(thetaPostSample(:,i),'knorm');
            end;
         end;
      else;
         if min(thetaPostSample(:,i))<max(thetaPostSample(:,i));
            try;
               if exist('YADAksdensity')==2;
                  [f,x] = YADAksdensity(thetaPostSample(:,i),'npoints',256);
               else;
                  if exist('ksdensity')==2;
                     [f,x] = ksdensity(thetaPostSample(:,i),'npoints',256);
                  else;
                     [x,f] = KernelDensityPlot(thetaPostSample(:,i),'knorm');
                  end;
               end;
            catch;
               if min(thetaPostSample(:,i))==max(thetaPostSample(:,i));
                  x = thetaPostSample(1,i)*ones(1,2^8);
                  f = ones(1,2^8);
               else;
                  x = NaN*ones(1,2^8);
                  f = x;
               end;
            end;
         else;
            x = thetaPostSample(1,i)*ones(1,2^8);
            f = ones(1,2^8);
         end;
      end;
   elseif DSGEModel.PosteriorDensityValue==2;
      if i<=NumParam;
         [x,f] = YADAMHDensEst(thetaPostSample(:,i),2^8,0,'gaussian',[]);
      else;
         if min(thetaPostSample(:,i))<max(thetaPostSample(:,i));
            try;
               [x,f] = YADAMHDensEst(thetaPostSample(:,i),2^8,0,'gaussian',[]);
            catch;
               if min(thetaPostSample(:,i))==max(thetaPostSample(:,i));
                  x = thetaPostSample(1,i)*ones(1,2^8);
                  f = ones(1,2^8);
               else;
                  x = NaN*ones(1,2^8);
                  f = x;
               end;
            end;
         else;
            x = thetaPostSample(1,i)*ones(1,2^8);
            f = ones(1,2^8);
         end;
      end;
   elseif DSGEModel.PosteriorDensityValue==3;
      if i<=NumParam;
         [x,f] = YADAMHDensEst(thetaPostSample(:,i),2^8,-1,'gaussian',WaitControls.cancel);
      else;
         if min(thetaPostSample(:,i))<max(thetaPostSample(:,i));
            try;
               [x,f] = YADAMHDensEst(thetaPostSample(:,i),2^8,-1,'gaussian',WaitControls.cancel);
            catch;
               if min(thetaPostSample(:,i))==max(thetaPostSample(:,i));
                  x = thetaPostSample(1,i)*ones(1,2^8);
                  f = ones(1,2^8);
               else;
                  x = NaN*ones(1,2^8);
                  f = x;
               end;
            end;
         else;
            x = thetaPostSample(1,i)*ones(1,2^8);
            f = ones(1,2^8);
         end;
      end;
   elseif DSGEModel.PosteriorDensityValue==4;
      if i<=NumParam;
         [x,f] = YADAMHDensEst(thetaPostSample(:,i),2^8,-2,'gaussian',WaitControls.cancel);
      else;
         if min(thetaPostSample(:,i))<max(thetaPostSample(:,i));
            try;
               [x,f] = YADAMHDensEst(thetaPostSample(:,i),2^8,-2,'gaussian',WaitControls.cancel);
            catch;
               if min(thetaPostSample(:,i))==max(thetaPostSample(:,i));
                  x = thetaPostSample(1,i)*ones(1,2^8);
                  f = ones(1,2^8);
               else;
                  x = NaN*ones(1,2^8);
                  f = x;
               end;
            end;
         else;
            x = thetaPostSample(1,i)*ones(1,2^8);
            f = ones(1,2^8);
         end;
      end;
   end;
   if isempty(x)==1;
      StopEstimation = 1;
      break;
   end;
   if i==1;
      Xposterior = zeros(NumParam+NumAdditional,length(x));
      Yposterior = zeros(NumParam+NumAdditional,length(x));
   end;
   if size(x,1)>size(x,2);
      Xposterior(i,:) = x';
   else;
      Xposterior(i,:) = x;
   end;
   if size(f,1)>size(f,2);
      Yposterior(i,:) = f';
   else;
      Yposterior(i,:) = f;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
if StopEstimation==1;
   return;
end;
%
% determine the mode
%
ModeData = zeros(NumParam+NumAdditional,1);
for i=1:NumParam+NumAdditional;
   [MaxVal,MaxInd] = max(Yposterior(i,:));
   ModeData(i,1) = Xposterior(i,MaxInd);
end;
%
if (nargout==2)|(nargout==7);
   thetaMargMode = ModeData(1:NumParam);
   if NumAdditional>0;
      UpdateMargMode = ModeData(NumParam+1:NumParam+NumAdditional);
   else;
      UpdateMargMode = [];
   end;
   return;
end;
%
% create the MinMaxthetaPostSample matrix
%
MinMaxthetaPostSample = [min(thetaPostSample)' max(thetaPostSample)'];
%
% ask if the user wants to save the plot data to mat-file
%
if IsPosterior==1;
   txt = 'Would you like to save the posterior and prior density data to a mat-file?';
   answer = Query(txt,'question',200,'Save - Posterior Densities',500,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      %
      file = FixFilePath([DSGEModel.OutputDirectory '\MarginalPosteriorPriorData-' int2str(DSGEModel.PosteriorDensityValue) '-' int2str(NumIter) '-' int2str(CurrChain) '.mat']);
      %
      save(file,'Xprior','Yprior','UpdatePriorX','UpdatePriorY','Xposterior','Yposterior','ParameterNames','MinMaxthetaPostSample','BoundsMatrix','thetaDist','UpdatePriorBounds','ModeData','thetaMode');
      %
      % and now the m-file
      %
      mfile = FixFilePath([DSGEModel.OutputDirectory '\MargPostData' int2str(DSGEModel.PosteriorDensityValue) '_' int2str(NumIter) '_' int2str(CurrChain) '.m']);
      %
      fid = fopen(mfile,'wt');
      fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
      fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
      fprintf(fid,'%%\n%% x-axis values for prior density of estimated parameters\n%%\n');
      fprintf(fid,'Xprior = DataStructure.Xprior;\n');
      fprintf(fid,'%%\n%% y-axis values for prior density of estimated parameters\n%%\n');
      fprintf(fid,'Yprior = DataStructure.Yprior;\n');
      fprintf(fid,'%%\n%% x-axis values for prior density of updated parameters\n%%\n');
      fprintf(fid,'UpdatePriorX = DataStructure.UpdatePriorX;\n');
      fprintf(fid,'%%\n%% y-axis values for prior density of updated parameters\n%%\n');
      fprintf(fid,'UpdatePriorY = DataStructure.UpdatePriorY;\n');
      fprintf(fid,'%%\n%% x-axis values for posterior density of all parameters\n%% (estimated on top, updated below)\n%%\n');
      fprintf(fid,'Xposterior = DataStructure.Xposterior;\n');
      fprintf(fid,'%%\n%% y-axis values for posterior density of all parameters\n%%\n');
      fprintf(fid,'Yposterior = DataStructure.Yposterior;\n');
      fprintf(fid,'%%\n%% string matrix with names of all parameters\n%%\n');
      fprintf(fid,'ParameterNames = DataStructure.ParameterNames;\n');
      fprintf(fid,'%%\n%% matrix with min and max posterior draws values of all parameters\n%%\n');
      fprintf(fid,'MinMaxthetaPostSample = DataStructure.MinMaxthetaPostSample;\n');
      fprintf(fid,'%%\n%% matrix with possible lower and upper bounds of the estimated parameters\n%%\n');
      fprintf(fid,'BoundsMatrix = DataStructure.BoundsMatrix;\n');
      fprintf(fid,'%%\n%% matrix with guestimated lower and upper bounds of the update parameters\n%%\n');
      fprintf(fid,'UpdatePriorBounds = DataStructure.UpdatePriorBounds;\n');
      fprintf(fid,'%%\n%% vector which indicates prior distribution used: 1 gamma, 2 beta, 3 normal,\n%% 4 invgamma, 5 truncnormal, 6 uniform, 7 student, 8 cauchy, 9 logistic\n%% 10 gumbel, 11 pareto\n%%\n');
      fprintf(fid,'thetaDist = DataStructure.thetaDist;\n');
      fprintf(fid,'%%\n%% vector with marginal mode of all parameters\n%%\n');
      fprintf(fid,'ModeData = DataStructure.ModeData;\n');
      fprintf(fid,'%%\n%% vector with joint mode of all parameters\n%%\n');
      fprintf(fid,'thetaMode = DataStructure.thetaMode;\n');
      fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
      %
      % end the script file
      %
      fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
      fclose(fid);
      %
      txt = ['The posterior and prior density data has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 13 entries: Xprior and Yprior (prior density data for estimated parameters), UpdatePriorX and UpdatePriorY (prior density data for updated parameters), Xposterior and Yposterior (posterior density data for all parameters), ParameterNames (string matrix with the names of all parameters), MinMaxthetaPostSample (matrix with min and max posterior draws values of all parameters), BoundsMatrix (matrix with potential lower and upper bounds for the estimated parameters), UpdatePriorBounds (matrix with guestimated lower and upper bounds for the updated parameters), thetaDist (vector indicating type of prior distribution of estimated parameters), ModeData (vector with marginal model of all parameters), and thetaMode (vector with joint mode of all parameters). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
      About(txt,'information','Posterior Densities',500,600,CurrINI);
   end;
end;
%
% we are ready to do some real plotting!
%
%
% Compute the number of figures (max 16 plots per graph)
%
NumFigs = ceil((NumParam+NumAdditional)/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam+NumAdditional);
else;
   NumRows = 4;
   NumCols = 4;
end;
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
%
%
% now we can start the loops
%
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if IsPosterior==1;
      if NumFigs==1;
         set(FigHandles{i},'Name',['Marginal Posterior Densities - Chain ' num2str(CurrChain,'%0.0f')]);
      else;
         set(FigHandles{i},'Name',['Marginal Posterior Densities - Figure ' num2str(i,'%0.0f') ' Chain ' num2str(CurrChain,'%0.0f')]);
      end;
   else;
      if NumFigs==1;
         set(FigHandles{i},'Name','Marginal Prior Densities - Chain ');
      else;
         set(FigHandles{i},'Name',['Marginal Prior Densities - Figure ' int2str(i)]);
      end;
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   %
   k = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam+NumAdditional;
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,k);
            %
            % plot the prior density
            %
            if NumPlots<=NumParam;
               if isempty(Xprior)==0;
                  PriorHandle = plot(Xprior(NumPlots,:),Yprior(NumPlots,:),'--','Color',[0.75 0 0],'LineWidth',2);
               else;
                  PriorHandle = [];
               end;
            else;
               if isempty(UpdatePriorX)==0;
                  PriorHandle = plot(UpdatePriorX(NumPlots-NumParam,:),UpdatePriorY(NumPlots-NumParam,:),'--','Color',[0.75 0 0],'LineWidth',2);
               else;
                  PriorHandle = [];
               end;
            end;
            hold('on');
            %
            % plot the posterior density
            %
            PosteriorHandle = plot(Xposterior(NumPlots,:),Yposterior(NumPlots,:),'-','Color',[0 0 0.75],'LineWidth',2);
            hold('on');
            SetAxesFonts(CurrINI,'Title',StringTrim(ParameterNames(NumPlots,:)));
            axis('tight');
            xlim = MinMaxthetaPostSample(NumPlots,:);
            add_x = 0.1*(xlim(2)-xlim(1));
            x_lim = [xlim(1)-add_x xlim(2)+add_x];
            if x_lim(1)==x_lim(2);
               x_lim(1) = x_lim(1)-0.5;
               x_lim(2) = x_lim(2)+0.5;
            end;
            set(gca,'XLim',x_lim);
            %
            ylim = [0 1.2*max(Yposterior(NumPlots,:))];
            if NumPlots<=NumParam;
               if thetaDist(NumPlots)==1;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
               elseif thetaDist(NumPlots)==2;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
                  if x_lim(2)>BoundsMatrix(NumPlots,2);
                     x_lim(2) = BoundsMatrix(NumPlots,2);
                  end;
               elseif thetaDist(NumPlots)==4;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = 0;
                  end;
               elseif thetaDist(NumPlots)==5;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
               elseif thetaDist(NumPlots)==6;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
                  if x_lim(2)>BoundsMatrix(NumPlots,2);
                     x_lim(2) = BoundsMatrix(NumPlots,2);
                  end;
               elseif thetaDist(NumPlots)==11;
                  if x_lim(1)<BoundsMatrix(NumPlots,1);
                     x_lim(1) = BoundsMatrix(NumPlots,1);
                  end;
               end;
            else;
               if x_lim(1)<UpdatePriorBounds(NumPlots-NumParam,1);
                  x_lim(1) = UpdatePriorBounds(NumPlots-NumParam,1);
               end;
               if x_lim(2)>UpdatePriorBounds(NumPlots-NumParam,2);
                  x_lim(2) = UpdatePriorBounds(NumPlots-NumParam,2);
               end;
            end;
            %
            % plot the marginal mode estimate
            %
            MarginalModeHandle = plot(ModeData(NumPlots)*ones(1,2),ylim,'-','Color',[0 0 0],'LineWidth',1);
            hold('on');
            %
            % plot the joint posterior mode estimate
            %
            JointModeHandle = plot(thetaMode(NumPlots)*ones(1,2),ylim,':','Color',[0 0 0],'LineWidth',1);
            %
            set(gca,'YLim',ylim);
            if (length(x_lim)==2)&(x_lim(1)<x_lim(2));
               set(gca,'XLim',x_lim);
            end;
            hold('off');
         end;
      end;
   end;
   if (IsSystemPrior==0)&(IsPosterior==1);
      legHandle = legend([PriorHandle PosteriorHandle MarginalModeHandle JointModeHandle],strvcat('Prior','Posterior','Marginal Mode','Joint Mode'));
   elseif (IsSystemPrior==1)&(IsPosterior==1);
      if isempty(PriorHandle)==0;
         legHandle = legend([PriorHandle PosteriorHandle MarginalModeHandle JointModeHandle],strvcat('System Prior','Posterior','Marginal Mode','Joint Mode'));
      else;
         legHandle = legend([PosteriorHandle MarginalModeHandle JointModeHandle],strvcat('Posterior','Marginal Mode','Joint Mode'));
      end;
   elseif (IsSystemPrior==1)&(IsPosterior==0);
      legHandle = legend([PriorHandle PosteriorHandle MarginalModeHandle JointModeHandle],strvcat('Marg Prior','System Prior','Marginal Mode','Joint Mode'));
   end;
   %
   SetAxesFonts(CurrINI,legHandle);
   %
   % transform units temporarily into pixels
   %
   OldAxesUnits = get(gca,'Units');
   OldLegUnits = get(legHandle,'Units');
   set(gca,'Units','pixels');
   set(legHandle,'Units','pixels');
   %
   % check if we can move legend
   %
   LegPos = get(legHandle,'Position');
   AxPos = get(gca,'Position');
   if LegPos(1)<AxPos(1)+AxPos(3);
      if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
         LegPos(1) = AxPos(1)+AxPos(3)+10;
         set(legHandle,'Position',LegPos);
      end;
   end;
   set(gca,'Units',OldAxesUnits);
   set(legHandle,'Units',OldLegUnits);
   %
   % save it to disk before
   %
   if DirExist(GraphDirectory);
      if IsPosterior==1;
         if NumFigs>1;
            file = FixFilePath([GraphDirectory '\MarginalPosteriorDensity-' num2str(NumIter,'%0.0f') '-' int2str(CurrChain) '-' num2str(i,'%0.0f') '.fig']);
         else;
            file = FixFilePath([GraphDirectory '\MarginalPosteriorDensity-' num2str(NumIter,'%0.0f') '-' int2str(CurrChain) '.fig']);
         end;
      else;
         if NumFigs>1;
            file = FixFilePath([GraphDirectory '\MarginalSystemPriorDensity-' int2str(NumIter) '-' int2str(i) '.fig']);
         else;
            file = FixFilePath([GraphDirectory '\MarginalSystemPriorDensity-' int2str(NumIter) '.fig']);
         end;
      end;
      saveas(FigHandles{i},file);
   end;
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
   %
   % Display the current figure
   %
   set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of GraphPosteriorDensity.m
%
