function ObsVarDecompDistDLG(selector,OVDs,NameStr,AnnualStr,ShockType,CurrINI,DSGEModel)
% ObsVarDecompDistDLG: Displays graphs of observed variables decomposition based on the prior or
%                      posterior mean and median of the parameters.
%
% USAGE:
%
%       ObsVarDecompDistDLG(selector,OVDs,NameStr,AnnualStr,ShockType,CurrINI,DSGEModel)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "samplebegin", "sampleend", "showgraph", and "done".
%
% OPTIONAL INPUT: OVDs (structure) with fields "Y", "DateVector", "ShockNames" with the observed variables, dates and the
%                      names of the shocks or shock groups. Furthermore, the mean estimates of the decomposition terms are
%                      called "DecompInitState", "DecompSteadyState", "DecompMeasureError" and "DecompShocks". Similarly,
%                      the median estimates are called "DecompMedianInitState", "DecompMedianSteadyState",
%                      "DecompMedianMeasureError" and "DecompMedianShocks".
%
%                 NameStr (string vector) taking on the values "Posterior" or "Prior".
%
%                 AnnualStr (string vector) taking on the values "Original" or "Annualized".
%
%                 ShockType (boolean) which takes the value 1 if shock groups are used and 0 otherwise.
%
%                 CurrINI (structure), with initialization information.
%
%                 DSGEModel (structure) whose fields contain all required information
%                                   about the model, such as input files, variable and sample
%                                   data. Actual observations are not stored in this file.
%
% NOTE: When selector = "init", then all optional input arguments are required!
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: December 11, 2023.
%                        This version: December 12, 2023.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 12-12-2023: Finished the function.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.NameStr = NameStr;
   DSGEModel.AnnualStr = AnnualStr;
   DSGEModel.OVDs = OVDs;
   DSGEModel.ShockType = ShockType;
else;
   GraphGUI = findobj('Type','figure','Tag','ObsVarDecompDistDLG');
   controls = get(GraphGUI,'UserData');
   CurrINI = get(controls.variables,'UserData');
   DSGEModel = get(controls.variablestext,'UserData');
end;
%
% Set screen size directly
%
switch selector;
   case 'init';
      GraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ObsVarDecompDistDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-525)/2 (CurrINI.scrsz(4)-180)/2 525 180], ...
         'Tag','ObsVarDecompDistDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['Observed Variable Decompositions - ' NameStr ' Distribution'], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ObsVarDecompDistDLG done', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(GraphGUI,'DockControl','off');
      end;
      %
      % Check where the sample of data actually begins
      %
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
      else;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1+(max(DSGEModel.annual)-1));
      end;
      %
      % Outer axis
      %
      AxesBox([2 2 523 170],'Graphics',45,[0.5 1],'on',CurrINI);
      %
      % Select the sample
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      %
      [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
      %
      DSGEModel = OldDSGEModel;
      controls.sampletxt = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 120 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Selected sample:', ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % sample start and sample end
      %
      controls.samplebeg = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 125 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ObsVarDecompDistDLG samplebegin', ...
         'Tag','SampleBeginPopup',...
         'Value',1);
      %
      controls.sampleend = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[303 125 87 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',SubSampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Callback','ObsVarDecompDistDLG sampleend', ...
         'Tag','SampleEndPopup',...
         'Value',size(SubSampleStrMat,1));
      %
      % Select variable
      %
      controls.variablestext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','ObsVarText');
      %
      controls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 90 180 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',DSGEModel.VariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','ObsVarPopup',...
         'Value',1);
      %
      controls.skipstateerrors = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 60 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Skip initial state and measurement errors shares', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SkipStateErrorsCheckBox');
      %
      controls.blackedgebars = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 40 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Draw bars with black edges', ...
            'Value',1, ...
            'Enable','on', ...
            'Tag','BlackEdgeBarsCheckBox');
      %
      controls.saveplotstofile = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 20 366 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Visible','on', ...
            'HorizontalAlignment','left', ...
            'String','Save figures to file', ...
            'Value',0, ...
            'Enable','on', ...
            'Tag','SavePlotsToFileCheckBox');
      %
      % Create graph button
      %
      controls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ObsVarDecompDistDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      controls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[420 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ObsVarDecompDistDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');
      %
      set(GraphGUI,'UserData',controls,...
         'HandleVisibility','callback');
   case 'samplebegin'
      %
      % check to make sure that the sample begins before it ends!
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.sampleend,'Value',samplebeg);
      end;
   case 'sampleend'
      %
      % check to make sure that the sample begins before it ends
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      if sampleend<samplebeg;
         set(controls.samplebeg,'Value',sampleend);
      end;
   case 'showgraph'
      %
      % collect variable info
      %
      VariableValue = get(controls.variables,'Value');
      CurrVariable = StringTrim(DSGEModel.VariableNames(VariableValue,:));
      n = DSGEModel.n;
      if VariableValue<n;
         set(controls.variables,'Value',VariableValue+1);
      else;
         set(controls.variables,'Value',1);
      end;
      %
      % check if the bars should have black edges or not
      %
      BlackEdgeBars = get(controls.blackedgebars,'Value');
      %
      % get the desired sample dates
      %
      samplebeg = get(controls.samplebeg,'Value');
      sampleend = get(controls.sampleend,'Value');
      SampleStrMat = get(controls.samplebeg,'String');
      FigureHandles = [];
      %
      % check if we should display measurement errors and initial state influence
      %
      if get(controls.skipstateerrors,'Value')==0;
         NumStates = 2;
         LegendNames = strvcat('Initial state','Measurement error');
      else;
         NumStates = 0;
         LegendNames = '';
      end;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                         'Tag',CurrVariable, ...
                         'Name',[DSGEModel.NameStr ' Mean Decomposition - ' CurrVariable]);
      %
      FigureHandles = [FigureHandles FigHandle];
      SetFigureProperties(FigHandle,CurrINI);
      %
      % collect the data
      %
      ShockNames = DSGEModel.OVDs.ShockNames;
      q = size(ShockNames,1);
      yObs = DSGEModel.OVDs.Y(VariableValue,:);
      DateVector = DSGEModel.OVDs.DateVector;
      T = length(DateVector);
      %
      % decomposition matrix
      %
      YmatMean = zeros(q+2,T);
      YmatMedian = YmatMean;
      %
      YmatMean(1,:) = DSGEModel.OVDs.DecompInitState(VariableValue,:);
      YmatMean(2,:) = DSGEModel.OVDs.DecompMeasureError(VariableValue,:);
      YmatMedian(1,:) = DSGEModel.OVDs.DecompMedianInitState(VariableValue,:);
      YmatMedian(2,:) = DSGEModel.OVDs.DecompMedianMeasureError(VariableValue,:);
      MeanShockContribution = squeeze(DSGEModel.OVDs.DecompShocks(VariableValue,:,:));
      MedianShockContribution = squeeze(DSGEModel.OVDs.DecompMedianShocks(VariableValue,:,:));
      %
      YmatMean(3:2+q,:) = MeanShockContribution';
      YmatMedian(3:2+q,:) = MedianShockContribution';
      %
      % now the data in deviation from steady state
      %
      yObsMean = yObs-DSGEModel.OVDs.DecompSteadyState(VariableValue,:);
      yObsMedian = yObs-DSGEModel.OVDs.DecompMedianSteadyState(VariableValue,:);
      %
      % make sure that the error from using the median decomposition (which doesn't add up
      % by contruction) adds the error to measurement errors
      %
      YmatMedian(2,:) = YmatMedian(2,:)+yObsMedian-sum(YmatMedian);
      %
      % make sure we remove the initial state and measurement error from the graph data
      % when asked to do so
      %
      if NumStates==0;
         YmatMean = YmatMean(3:q+2,:);
         YmatMedian = YmatMedian(3:q+2,:);
      end;
      %
      % check if we are using shock groups or not and provide colors for
      % the groups
      %
      UseShockGroups = DSGEModel.ShockType;
      if UseShockGroups==1;
         ShockGroups = DSGEModel.ShockGroups;
      else;
         ShockGroups = (1:length(DSGEModel.StateShockPositions));
      end;
      UseAliases = 0;
      if (UseShockGroups==1)&(isempty(DSGEModel.ShockGroupColors)==0)&(size(DSGEModel.ShockGroupColors,1)==length(unique(ShockGroups)));
         ShockGroupColors = DSGEModel.ShockGroupColors(unique(ShockGroups),:);
      else;
         if (isempty(DSGEModel.ShockColors)==0)&(q==size(DSGEModel.ShockColors,1));
            ShockGroupColors = DSGEModel.ShockColors;
         else;
            ShockGroupColors = CreateDefaultColors(q);
         end;
         if (UseShockGroups==0)|(isequal(DSGEModel.ShockGroupNames,DSGEModel.StateShockNames)==1);
            UseAliases = 1;
         end;
      end;
      if NumStates==2;
         ShockGroupColors = [0.3 0.3 0.3;0.7 0.7 0.7;ShockGroupColors];
      end;
      %
      % plot the decomposition based on the mean
      %
      if MatlabNumber>8.3;
         BarHandles = bar(DateVector',YmatMean','stacked');
      else;
         if MatlabNumber>=7.0;
            %
            % make sure the bars are written as patches
            %
            BarHandles = bar('v6',DateVector',YmatMean','stacked');
         else;
            BarHandles = bar(DateVector',YmatMean','stacked');
         end;
      end;
      %
      % remake the bar patches such that all bars are displayed properly.
      %
      BarHandles = RemakeBars(BarHandles,ShockGroupColors,BlackEdgeBars);
      %
      %
      LineHandle = plot(DateVector,yObsMean,'.-','Color',[0 0 0],'LineWidth',2);
      %
      %
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (original data) in deviation from steady-state at ' lower(DSGEModel.NameStr) ' mean']);
      else;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (annual data) in deviation from steady-state at ' lower(DSGEModel.NameStr) ' mean']);
      end;
      axis('tight');
      ylim = get(gca,'YLim');
      for h=1:length(BarHandles);
         if h==1;
            YData = get(BarHandles(h),'YData');
            if size(YData,1)~=4;
               YData = YData';
            end;
            ylim(1) = min(min(YData(:,samplebeg:sampleend)));
            ylim(2) = max(max(YData(:,samplebeg:sampleend)));
         else;
            YData = get(BarHandles(h),'YData');
            if size(YData,1)~=4;
               YData = YData';
            end;
            ylim(1) = min([min(YData(:,samplebeg:sampleend)) ylim(1)]);
            ylim(2) = max([max(YData(:,samplebeg:sampleend)) ylim(2)]);
         end;
      end;
      %
      % see if we should add a zero line
      %
      AddOn = 0.1*(ylim(2)-ylim(1));
      if AddOn==0;
         Ylim = get(gca,'YLim');
         AddOn = 0.1*(Ylim(2)-Ylim(1));
      end;
      ylim = [ylim(1)-AddOn ylim(2)+AddOn];
      set(gca,'YLim',ylim);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         kids = get(gca,'Children');
         if size(kids,1)>size(kids,2);
            kids = kids';
         end;
         kids = [kids(2:length(kids)) kids(1)]';
         set(gca,'Children',kids);
      end;
      if UseAliases==0;
         ShockGroupNames = '';
         for i=1:size(DSGEModel.ShockGroupNames,1);
            ShockGroupNames = strvcat(ShockGroupNames,StringTrim(strrep(DSGEModel.ShockGroupNames(i,:),'_',' ')));
         end;
      else;
         ShockGroupNames = DSGEModel.ShockAliases;
         for j=1:size(ShockGroupNames,1);
            ShockGroupNames(j,:) = strrep(ShockGroupNames(j,:),'_',' ');
         end;
      end;
      %
      % make sure that only the selected sample is displayed
      %
      XData = get(BarHandles(1),'XData');
      if size(XData,1)>size(XData,2);
         XData = XData';
      end;
      set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
      %
      LegendNames = strvcat(LegendNames,ShockGroupNames);
      %
      % create the legend
      %
      if MatlabNumber>=7;
         legHandle = legend([BarHandles LineHandle],strvcat(LegendNames,CurrVariable),'Location','NorthWest');
      else;
         legHandle = legend([BarHandles LineHandle],strvcat(LegendNames,CurrVariable),2);
      end;
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      % check if we should fix the X label
      %
      SetXTickIntegers(gca);
      %
      % check if we should save figure to file
      %
      if get(controls.saveplotstofile,'Value')==1;
         GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
         if DirExist(GraphDirectory)==0;
            stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
         end;
         file = FixFilePath([GraphDirectory '\' CurrVariable '-ShockDecomp-' DSGEModel.AnnualStr '-' DSGEModel.NameStr 'Mean.fig']);
         saveas(FigHandle,file);
      end;
      %
      % Fix the toolbar
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      %
      % and now the prior or posterior median graphs
      %
      FigHandle = figure('Visible','off', ...
                         'Position',[(CurrINI.scrsz(3)-750)/2+30 (CurrINI.scrsz(4)-500)/2-30 750 500], ...
                         'Tag',CurrVariable, ...
                         'Name',[DSGEModel.NameStr ' Median Decomposition - ' CurrVariable]);
      %
      FigureHandles = [FigureHandles FigHandle];
      SetFigureProperties(FigHandle,CurrINI);
      %
      % plot the decomposition based on the mean
      %
      if MatlabNumber>8.3;
         BarHandles = bar(DateVector',YmatMedian','stacked');
      else;
         if MatlabNumber>=7.0;
            %
            % make sure the bars are written as patches
            %
            BarHandles = bar('v6',DateVector',YmatMedian','stacked');
         else;
            BarHandles = bar(DateVector',YmatMedian','stacked');
         end;
      end;
      %
      % remake the bar patches such that all bars are displayed properly.
      %
      BarHandles = RemakeBars(BarHandles,ShockGroupColors,BlackEdgeBars);
      %
      % the data in deviation from steady-state
      %
      LineHandle = plot(DateVector,yObsMedian,'.-','Color',[0 0 0],'LineWidth',2);
      %
      if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (original data) in deviation from steady-state at ' lower(DSGEModel.NameStr) ' median']);
      else;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (annual data) in deviation from steady-state at ' lower(DSGEModel.NameStr) ' median']);
      end;
      axis('tight');
      ylim = get(gca,'YLim');
      for h=1:length(BarHandles);
         if h==1;
            YData = get(BarHandles(h),'YData');
            if size(YData,1)~=4;
               YData = YData';
            end;
            ylim(1) = min(min(YData(:,samplebeg:sampleend)));
            ylim(2) = max(max(YData(:,samplebeg:sampleend)));
         else;
            YData = get(BarHandles(h),'YData');
            if size(YData,1)~=4;
               YData = YData';
            end;
            ylim(1) = min([min(YData(:,samplebeg:sampleend)) ylim(1)]);
            ylim(2) = max([max(YData(:,samplebeg:sampleend)) ylim(2)]);
         end;
      end;
      %
      % see if we should add a zero line
      %
      AddOn = 0.1*(ylim(2)-ylim(1));
      if AddOn==0;
         Ylim = get(gca,'YLim');
         AddOn = 0.1*(Ylim(2)-Ylim(1));
      end;
      ylim = [ylim(1)-AddOn ylim(2)+AddOn];
      set(gca,'YLim',ylim);
      if (ylim(1)<0)&(ylim(2)>0);
         hold('on');
         plot([DateVector(1) DateVector(length(DateVector))],[0 0],'-','Color',[0 0 0]);
         %
         % make sure the zero line is displayed below the observed variable
         %
         kids = get(gca,'Children');
         if size(kids,1)>size(kids,2);
            kids = kids';
         end;
         kids = [kids(2:length(kids)) kids(1)]';
         set(gca,'Children',kids);
      end;
      %
      % make sure that only the selected sample is displayed
      %
      XData = get(BarHandles(1),'XData');
      if size(XData,1)>size(XData,2);
         XData = XData';
      end;
      set(gca,'XLim',[min(XData(:,samplebeg)) max(XData(:,sampleend))]);
      %
      % create the legend
      %
      if MatlabNumber>=7;
         legHandle = legend([BarHandles LineHandle],strvcat(LegendNames,CurrVariable),'Location','NorthWest');
      else;
         legHandle = legend([BarHandles LineHandle],strvcat(LegendNames,CurrVariable),2);
      end;
      SetAxesFonts(CurrINI,legHandle);
      hold('off');
      %
      % check if we should fix the X label
      %
      SetXTickIntegers(gca);
      %
      % check if we should save figure to file
      %
      if get(controls.saveplotstofile,'Value')==1;
         GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
         if DirExist(GraphDirectory)==0;
            stat = MakeDir(DSGEModel.OutputDirectory,'graphs');
         end;
         file = FixFilePath([GraphDirectory '\' CurrVariable '-ShockDecomp-' DSGEModel.AnnualStr '-' DSGEModel.NameStr 'Median.fig']);
         saveas(FigHandle,file);
      end;
      %
      % Fix the toolbar
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      set(FigHandle,'CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      %
      % display the graphs
      %
      set(FigureHandles,'Visible','on');
      drawnow;
      pause(0.02);
   case 'done'
      set(controls.done,'UserData','done');
      delete(GraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(GraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(GraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.done);
   end;
   waitfor(controls.done,'UserData','done');
end;

%
% end of ObsVarDecompDistDLG.m
%
