function OpenGraphics(GraphDirectory,CurrINI)
% OpenGraphics: Opens a dialog for selecting a graphics file to view and sends the file to the
%               appropriate viewer
%
% USAGE:
%
%       OpenGraphics(GraphDirectory,CurrINI)
%
% OPTIONAL INPUT: GraphDirectory (string vector) being a valid file directory.
%
%                 CurrINI (structure), with initialization information.
%
%
% NOTE: The OpenGrpahics function can open any file for which the "open" command is registered. You can
%       check this by right-clicking on a file in Windows Explorer and see if the "open" command appears
%       on the context menu.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 21, 2006.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-08-2006: Fixed a bug in the FileName cell array.
%
% * 29-08-2006: Added the CurrINI input and added the call to SetToolbarImages for fig-files.
%
% * 17-10-2006: Added a try, catch call when attempting to load fig-files. This is needed if matlab can't
%               read the fig-file, e.g., because it was created with a later version of matlab.
%
% * 09-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 22-09-2008: Added a check if the file shellcmd.exe exists.
%
% * 08-12-2008: Added quotes around the calls to exe-files.
%
% * 02-05-2009: Updated the documentation.
%
% * 06-01-2010: Updated the documentation.
%
% * 14-02-2010: Changed a drawnow command to pause(0.02) below the call to uigetfile.
%
% * 04-01-2011: Updated the documentation.
%
% * 09-01-2011: The function now supports unix and mac.
%
% * 11-01-2011: Added support for unix and mac paths.
%
% * 13-01-2011: Made sure that the path provided by uigetfile ends with '/' on unix
%               and mac.
%
% * 27-01-2011: Use "open" in a try-catch statement to try to open pdf's. The alternative
%               is the "unix" command.
%
% * 29-01-2011: Made sure that the function can open filetypes not directly supported by the open
%               function on windows and mac. Had to treat unix differently by assuming that the gimp
%               is installed.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 29-01-2014: Removed the call to "shellcmd.exe".
%
% * 05-01-2015: Updated the documentation.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 08-01-2018: Updated the documentation.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 21-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 25-01-2021: Updated the documentation.
%
% * 04-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==0;
   maingui = findobj('Type','figure','Tag','YADA');
   controls = get(maingui,'UserData');
   CurrINI = get(controls.filemenu,'UserData');
   DSGEModel = get(controls.open,'UserData');
   GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
   if DirExist(GraphDirectory)==0;
      GraphDirectory = pwd;
   end;
end;

if MatlabNumber>=6;
   FileName = {'*.fig', 'Matlab Figure Files (*.fig)'; ...
              '*.pdf', 'Portable Document Format Files (*.pdf)'; ...
              '*.emf', 'Windows Meta Files (*.emf)'; ...
              '*.jpg;*.jpeg', 'Jpeg Files (*.jpg, *.jpeg)'; ...
              '*.png', 'Portable Network Graphics Files (*.png)'; ...
              '*.eps', 'Encapsulated PostScript Files (*.eps)'; ...
              '*.*', 'All Files (*.*)'};
   [fname,fpath] = uigetfile(FileName,'Open Graphics File',FixFilePath([GraphDirectory '\']));
else;
   FileName = FixFilePath([GraphDirectory '\*.fig;*.pdf;*.emf;*jpg;*.png;*.eps']);
   [fname,fpath] = uigetfile(FileName,'Open Graphics File');
end;
drawnow;
pause(0.02);
%
if (isnumeric(fname)==0)&(isunix==1);
   if strcmp(fpath(1,length(fpath)),'/')==0;
      fpath = [fpath '/'];
   end;
end;
%
if (isnumeric(fname)==0)&(FileExist([fpath fname])==1);
   file = [fpath fname];
   if strcmp(lower(GetExt(file)),'.fig')==1;
      ErrorStr = '';
      if MatlabNumber>=8.0;
         try;
            open(file);
         catch ME;
            ErrorStr = ['Unable to load the file "' GetFilename(file) '". Message caught is: ' YADALastError(ME)];
         end;
      else;
         try;
            open(file);
         catch;
            ErrorStr = ['Unable to load the file "' GetFilename(file) '". Message caught is: ' YADALastError];
         end;
      end;
      if isempty(ErrorStr)==1;
         set(gcf,'Visible','off');
         SetToolbarImages(gcf,CurrINI.images);
         set(gcf,'Visible','on');
      else;
         About(ErrorStr,'information','YADA - Bad FIG File',500,600,CurrINI);
      end;
   else;
      %
      % use the shellcmd function
      %
      if ispc==1;
         if MatlabNumber>=7;
            winopen(file);
         else;
            cmd = ['"' file '" &'];
            stat = dos(cmd);
         end;
      else;
         %
         try;
            open(FixFilePath(file));
         catch;
            if strncmp(computer,'MAC',3)==1
               stat = unix(['open "' FixFilePath(file) '" &']);
            else;
               stat = unix(['gimp ' FixFilePath(file)]);
               if stat~=0;
                  txt = ['YADA wasn''t able to open the file "' GetFilename(FixFilePath(file)) '". Please make sure that your matlab system has an "open" function that supports the file extension "' GetExt(FixFilePath(file)) '".'];
                  About(txt,'information','YADA - Open Graphics',400,500,CurrINI);
               end;
            end;
         end;
      end;
   end;
end;

%
% end of OpenGraphics.m
%
