function PlotMargLikeSWZModifiedHarmonic(PostSample,LogPost,PValue,ModeData,NumBurnin,CurrChain,DSGEModel,CurrINI)
% PlotMargLikeSWZModifiedHarmonic: Calculates sequentially the modified harmonic mean estimator
%                                  of the log marginal likelihood with the Sims, Waggoner and Zha version
%                                  based on the truncated elliptical distribution.
%
% USAGE:
%
%       PlotMargLikeSWZModifiedHarmonic(PostSample,LogPost,PValue,ModeData,NumBurnin,CurrChain,DSGEModel,CurrINI)
%
% REQUIRED INPUT: PostSample (matrix) of dimension NumIterxNumParam with the posterior sample draws
%                            of the phi parameters.
%
%                 LogPost (vector) of dimension (NumIterx1) with all the log posterior values for the
%                            non-discarded posterior draws. These posterior values are calculated
%                            using the "logPosteriorPhiDSGE" function.
%
%                 PValue (scalar) with value between 0 and 100.
%
%                 ModeData (structure) with the stored data from posterior mode estimation.
%
%                 NumBurnin (integer), with the number of burn-in draws.
%
%                 CurrChain (integer) with the current MCMC chain number.
%
%                 DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 7, 2012.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 10-12-2012: Took into account that "MargLikeSWZModifiedHarmonic" now requires the "lambda" input variable.
%
% * 11-12-2012: Changed the name entry for the figure.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 27-08-2015: The function now supports the RWM posterior sampler with a Student-t proposal density.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 02-04-2017: The function now supports the fixed blocking RWM posterior sampler with a normal
%               proposal density.
%
% * 10-04-2017: The function now supports the fixed blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 18-04-2017: The function now supports the random blcoking RWM posterior sampler with a normal
%               proposal density.
%
% * 20-04-2017: The function now supports the random blocking RWM posterior sampler with a Student-t
%               proposal density.
%
% * 08-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% locate the controls structure
%
maingui = findobj('Type','figure','Tag','YADA');
controls = get(maingui,'UserData');
%
NumPostDraws = length(LogPost);
%
% sequential marginal likelihood start value and step length
%
SeqStartStr = get(controls.dsge.sequentialstart,'String');
DSGEModel.SequentialStartIterationValue = str2num(strrep(StringTrim(SeqStartStr(DSGEModel.SequentialStartIteration,:)),',',''));
SeqStepStr = get(controls.dsge.sequentialstep,'String');
DSGEModel.SequentialStepLengthValue = str2num(strrep(StringTrim(SeqStepStr(DSGEModel.SequentialStepLength,:)),',',''));
%
% Open up a wait dialog.
%
txt = ['Please wait while YADA computes Sims, Waggoner and Zha''s truncated elliptical distribution version of the modified harmonic mean estimator of the log marginal likelihood. Estimation started at: ' StringTrim(datestr(now,14)) '.'];
WaitHandle = WaitDLG(txt,'information','Sequential Marginal Likelihood',500,300,CurrINI,1);
WaitControls = get(WaitHandle,'UserData');
%
% estimate the marginal likelihood
%
[LogMargsMH,qL,LBound,NWStdErr] = MargLikeSWZModifiedHarmonic(PostSample(NumBurnin+1:NumPostDraws,:),LogPost(NumBurnin+1:NumPostDraws,:),NumPostDraws,PValue,1,ModeData,[],DSGEModel,CurrINI,WaitControls.cancel);
%
% shut down the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
if isempty(LogMargsMH)==0;
   %
   % ask if we write to file
   %
   txt = 'Would you like to save the sequential estimation data to file?';
   answer = Query(txt,'question',200,'Save - SWZ''s Modified Harmonic Mean',500,'yes',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      %
      Xvalues = LogMargsMH(:,1)+NumBurnin;
      LogMargLikeMH = LogMargsMH(:,2:size(LogMargsMH,2));
      %
      file = FixFilePath([DSGEModel.OutputDirectory '\SeqSWZModifiedHarmonic-' DSGEModel.NameOfModel '-' int2str(NumPostDraws) '-' int2str(CurrChain) '.mat']);
      OldPValue = PValue;
      PValue = PValue/100;
      save(file,'qL','LBound','PValue','Xvalues','LogMargLikeMH');
      PValue = OldPValue;
      %
      mfile = FixFilePath([DSGEModel.OutputDirectory '\SeqSWZMH' strrep(DSGEModel.NameOfModel,'-','') int2str(NumPostDraws) int2str(CurrChain) '.m']);
      fid = fopen(mfile,'wt');
      fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
      fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
      fprintf(fid,'%%\n%% Number of posterior draws used for sequential estimation (N x 1) vector\n%%\n');
      fprintf(fid,'Xvalues = DataStructure.Xvalues;\n');
      fprintf(fid,'%%\n%% N x 1 vector with log marginal likelihood values using\n%% Sims, Waggoner and Zha''s modified harmonic mean estimator\n%%\n');
      fprintf(fid,'LogMargLikeMH = DataStructure.LogMargLikeMH;\n');
      fprintf(fid,'%%\n%% Fraction of log posterior kernel values from iid draws of elliptical distribution\n%% that are greater than lower bound\n%%\n');
      fprintf(fid,'qL = DataStructure.qL;\n');
      fprintf(fid,'%%\n%% Fraction of log posterior kernel values from posterior draws that are\n%% greater than the lower bound\n%%\n');
      fprintf(fid,'PValue = DataStructure.PValue;\n');
      fprintf(fid,'%%\n%% Lower bound of log posterior kernel\n%%\n');
      fprintf(fid,'LBound = DataStructure.LBound;\n');
      fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
      %
      % end the script file
      %
      fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
      fclose(fid);
      %
      txt = ['The data on sequential estimation of the log marginal likelihood has been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: Xvalues (Nx1 vector with the number of posteror draws used for the sequential estimation), LogMargLikeMH (Nx1 vector with estimates of the log marginal likelihood with SWZ''s modified harmonic mean), qL (fraction of log posterior kernel values based on iid draws from an elliptical distribution that are greater than the lower bound), PValue (fraction of log posterior kernel values based on the posterior draws that are greater than the lower bound), and LBound (lower bound of the log posterior kernel based on PValue). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
      About(txt,'information','SWZ''s Modified Harmonic Mean',400,600,CurrINI);
   end;
   %
   % prepare the graph
   %
   GraphDirectory = FixFilePath([DSGEModel.OutputDirectory '\graphs']);
   %
   FigHandle =figure('Visible','off', ...
          'Position',[(CurrINI.scrsz(3)-700)/2 (CurrINI.scrsz(4)-500)/2 700 500], ...
          'Tag','MargLikeSWZ', ...
          'Name','Sequential Marginal Likelihood - SWZ''s Harmonic Mean');
   %
   %
   SetFigureProperties(FigHandle,CurrINI);
   %
   Xvalues = LogMargsMH(:,1)+NumBurnin;
   Yvalues = LogMargsMH(:,2);
   %
   plot(Xvalues,Yvalues,'LineStyle','-','Color',[0.75 0 0],'LineWidth',2,'Marker','.');
   hold('off');
   if (DSGEModel.PosteriorSampler==1)|(DSGEModel.PosteriorSampler>=3);
      SetAxesFonts(CurrINI,'Title','SWZ''s Harmonic Mean Estimates','XLabel',['RWM draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
   elseif DSGEModel.PosteriorSampler==2;
      SetAxesFonts(CurrINI,'Title','SWZ''s Harmonic Mean Estimates','XLabel',['Slice sampler draw number (with ' num2str(NumBurnin,'%0.0f') ' burn-in draws)'],'YLabel','Log marginal likelihood');
   end;
   axis('tight');
   %
   % check the YLim
   %
   ylim = get(gca,'YLim');
   add_on = 0.1*(ylim(2)-ylim(1));
   ylim = [ylim(1)-add_on ylim(2)+add_on];
   set(gca,'YLim',ylim);
   %
   % save the graph
   %
   if DirExist(GraphDirectory);
      if DSGEModel.PosteriorSampler==1;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-RWM-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==2;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Slice-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==3;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==4;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==5;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Fixed-' int2str(DSGEModel.FixedNumParamBlocks) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==6;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      elseif DSGEModel.PosteriorSampler==7;
         file = FixFilePath([GraphDirectory '\LogMarginalLikelihood-SWZ-ModifiedHarmonic-Random-' int2str(DSGEModel.BlockSize(1)) '-' int2str(DSGEModel.BlockSize(2)) '-RWM-t-' int2str(DSGEModel.StudenttDegFree) '-' num2str(NumPostDraws,'%0.0f') '-' int2str(CurrChain) '.fig']);
      end;
      saveas(FigHandle,file);
   end;
   %
   % display the graph
   %
   SetToolbarImages(FigHandle,CurrINI.images);
   set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of PlotMargLikeSWZModifiedHarmonic.m
%
