function [action,vars,positions] = SetStateVarsShocksDLG(selector,vartype,AddToName,AllVars,vars,positions,CurrINI)
% SetStateVarsShocksDLG: Dialog for either selecting the state shocks or the state variables.
%
% USAGE:
%
%       [action,vars,positions] = SetStateVarsShocksDLG(selector,vartype,AllVars,vars,positions,CurrINI)
%
% REQUIRED INPUT:  selector (string vector), with the possible values "init", "slider", "quit"
%
% OPTIONAL INPUT:  vartype (string vector), taking on the value "State Variables", "State Shocks", "Forward
%                          Looking Variables" or "Unit-Root State Variables".
%
%                  AddToName (string vector) which contains a string to add to the name header of the dialog.
%
%                  AllVars (string matrix), with all the possible state variable and state shock names. Given
%                          by the AiM output.
%
%                  vars (string matrix), with the currently suggested names for the vartype.
%
%                  positions (vector), with row number for the positions in AllVars of the vartype. These positions
%                          yield the string matrix vars from AllVars.
%
%                  CurrINI (structure), with initialization information.
%
% Optional OUTPUT: action (string vector) taking the value "OK" or "Cancel". Depends on which button was
%                         clicked on.
%
%                  vars (string matrix), with the selected vartype names.
%
%                  positions (vector), with row number for the positions in AllVars of the vartype. These positions
%                          yield the string matrix vars from AllVars.
%
% NOTE: If selector = "init", then  all optional inputs and outputs are required.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 3, 2006.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-08-2006: Changed the StrTrim function to StringStrim.
%
% * 23-11-2006: Added a check if current object is empty after waitfor function.
%
% * 04-12-2006: Made sure that the minimum number .
%
% * 08-03-2007: Updated the documentation.
%
% * 09-11-2007: Updated the documentation.
%
% * 20-12-2007: Changed the vartype input to more general strings.
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 21-05-2008: Removed unnecessary callback from checkbox.
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 06-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 07-02-2011: Changed the maximum number of columns to a screen width measure.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 15-02-2023: Added the input variable "AddToName".
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   SV.AllVars = AllVars;
   SV.vars = vars;
   SV.positions = positions;
   scrsz = CurrINI.scrsz;
   %
   % try to determine the width and the height
   %
   MaxCol = floor((scrsz(3)-35)/105);
   GUIColumns = min(floor(sqrt(size(AllVars,1))),MaxCol);
   if GUIColumns<=min(4,size(AllVars,1));
      GUIColumns = min(4,size(AllVars,1));
   end;
   GUIRows = ceil(size(AllVars,1)/GUIColumns);
   width = (100*GUIColumns)+50+(5*(GUIColumns-1));
   if width<200;
      width = 200;
   end;
   height = min(scrsz(4)-100,100+(20*GUIRows)+(5*(GUIRows-1)));
else;
   SetStateVarsShocksGUI = findobj('Type','figure','Tag','SetStateVarsShocksDLG');
   controls = get(SetStateVarsShocksGUI,'UserData');
   SV = get(controls.cancel,'UserData');
end;

switch selector
   case 'init'
      SetStateVarsShocksGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','SetStateVarsShocksDLG.m', ...
         'MenuBar','none', ...
         'Units','pixels', ...
         'Position',[(scrsz(3)-width)/2 (scrsz(4)-height)/2 width height], ...
         'Tag','SetStateVarsShocksDLG', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name',['Select ' vartype AddToName], ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','SetStateVarsShocksDLG quit', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(SetStateVarsShocksGUI,'DockControl','off');
      end;
      if strcmp(lower(vartype),'unit-root state variables')==0;
         AxesBox([2 2 width-2 height-10],[vartype ' Selection'],100,[0.5 1],'on',CurrINI);
      else;
         set(SetStateVarsShocksGUI,'Name',['Specify ' vartype]);
         AxesBox([2 2 width-2 height-10],[vartype ' Specification'],100,[0.5 1],'on',CurrINI);
      end;
      %
      % checkboxes with all variables from the AiM output
      %
      chkbx = [];
      posmat = zeros(size(AllVars,1),2);
      num = 0;
      for i=1:GUIRows;
         for j=1:GUIColumns;
            num = num+1;
            if num>size(AllVars,1);
               break;
            end;
            pos = [20+(105*(j-1)) height-35-(20*i)];
            new_chkbx = uicontrol('Units','pixels', ...
               'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
               'Position',[pos 100 20], ...
               'FontSize',CurrINI.gui_fontsize, ...
               'FontName',CurrINI.gui_fontname, ...
               'FontWeight',CurrINI.gui_fontweight, ...
               'FontAngle',CurrINI.gui_fontangle, ...
               'Style','checkbox', ...
               'Visible','on', ...
               'HorizontalAlignment','left', ...
               'String',StringTrim(AllVars(num,:)), ...
               'Value',sum(num==positions), ...
               'Enable','on', ...
               'Tag','UICheckBox');
            chkbx = [chkbx new_chkbx];
            posmat(num,:) = pos;
         end;
      end;
      SV.posmat = posmat;
      SV.chkbx = chkbx;
      %
      % Two buttons to select action
      %
      controls.ok = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-175 20 75 25], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','SetStateVarsShocksDLG quit', ...
         'UserData','waiting', ...
         'Tag','ButtonOK');
      %
      controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[width-95 20 75 25], ...
         'String','Cancel', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'UserData',SV, ...
         'Callback','SetStateVarsShocksDLG quit', ...
         'Tag','ButtonCancel');
      %
      set(SetStateVarsShocksGUI,'UserData',controls);
   case 'slider'
      %
      % in case the height of the dialog is inadequate to show all controls.
      % This requires more than 200 variables in the AiM model!
      %
   case 'quit'
      set(controls.ok,'UserData','done');
end;
%
if strcmp(selector,'init')==1;
   set(SetStateVarsShocksGUI,'Visible','on');
   drawnow;
   SetYADAIcon(SetStateVarsShocksGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.ok);
   end;
   %
   waitfor(controls.ok,'UserData','done');
   if (isempty(get(SetStateVarsShocksGUI,'CurrentObject'))==1)|(strcmp(get(get(SetStateVarsShocksGUI,'CurrentObject'),'Type'),'uicontrol')==0);
      action = 'Cancel';
   else;
      action = get(get(SetStateVarsShocksGUI,'CurrentObject'),'String');
      if (isempty(action)==1)|(strcmp(action,'OK')==0);
         action = 'Cancel';
      end;
   end;
   SV = get(controls.cancel,'UserData');
   %
   % determine output
   %
   vars = '';
   positions = [];
   for i=1:length(SV.chkbx);
      if get(SV.chkbx(i),'Value')==1;
         positions = [positions i];
         vars = strvcat(vars,get(SV.chkbx(i),'String'));
      end;
   end;
   delete(SetStateVarsShocksGUI);
   drawnow;
   pause(0.02);
end;

%
% end of SetStateVarsShocksDLG.m
%
