function ThreeDeeParamsGraphDLG(selector,DSGEModel,CurrINI,ModeData,UseTransformed,IsAdaptiveLearning)
% ThreeDeeParamsGraphDLG: Displays a dialog for selecting 2 parameters to use for computing a 3D surface
%                         of the log posterior. The function uses the original sample from the posterior
%                         mode estimation.
%
% USAGE:
%
%       ThreeDeeParamsGraphDLG(selector,DSGEModel,CurrINI,ModeData,UseTransformed,IsAdaptiveLearning)
%
% REQUIRED INPUT: selector (string vector) that takes on the values "init", "showgraph",
%                          "checkparams", and "done".
%
% OPTIONAL iNPUT: DSGEModel (structure)  whose fields contain all required information
%                          about the model, such as input files, variable and sample
%                          data. Actual observations are not stored in this file.
%
%                 CurrINI (structure) with initialization information.
%
%                 ModeData (structure) with all required information about the posterior
%                          mode of the DSGE model.
%
%                 UseTransformed (boolean) that is 1 if transformed parameters are used and 0 if
%                          the original parameters are considered.
%
%                 IsAdaptiveLearning (boolean) that takes the value 1 if the adaptive learning based solution of the
%                          DSGE model is used, and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 15, 2007.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-03-2007: Finished the function.
%
% * 20-03-2006: Made sure that the colorbar is an axes object for version 7 and later of Matlab. Added
%               the input "UseTransformed" to allow for calculations using both the transformed parameters
%               and the original parameters.
%
% * 28-03-2007: Made sure that the correct mode values are used in for plot and plot3 commands when
%               the original parameters are examined.
%
% * 09-11-2007: Updated the documentation.
%
% * 02-01-2008: Removed code using "DSGEModel.InitializeKalman".
%
% * 10-01-2008: Changed the last drawnow call to pause(0.02).
%
% * 13-02-2008: Added tests if the DSGEModel structure fields involving files are the same as the
%               fields of the DSGEModel structure stored in the ModeData structure.
%
% * 04-04-2008: Made sure that the ZLabel string uses "\phi" when trandsformed parameters are studied
%               and "\theta" when the original parameters are considered. Fixed a bug where "ColorMapValue"
%               was computed through the floor function rather than the ceil function. This meant that
%               it could take on the value 0, i.e., YADA would try to pick row 65 from the colormap.
%               This object has exactly 64 rows!
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 09-03-2009: Added quotation marks around paths for the dos copy commands.
%
% * 06-01-2010: Updated the documentation.
%
% * 16-12-2010: Took into account that "logPosteriorPhiDSGE" and "logPosteriorThetaDSGE" take the input
%               variable "OrderQZ" from the CurrINI structure.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 05-07-2011: Took into account that "logPosterioThetaDSGE" now requires the "ParameterBounds" input variable.
%
% * 04-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 24-02-2014: Decreased the height of the dialog by 20 pixels.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 10-09-2015: The function is now ready for Matlab 2014b and later.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 12-06-2017: Made sure that the number of grid points is fully consistent with the value
%               of the DSGEModel.NumberOfGridPoints value that depends on the setting in YADAGUI.
%
% * 29-08-2017: Took into account that "logPosteriorPhiDSGE" and "logPosteriorThetaDSGE" require the input
%               variable "NumWorkers".
%
% * 05-01-2018: Updated the documentation.
%
% * 22-03-2018: Made sure that the "SystemPriorFile" is copied to the tmp directory.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 29-08-2018: Updated the documentation.
%
% * 12-10-2018: Changed the position of the light object on the surface plot to half the maximum along
%               the z-axis.
%
% * 15-10-2018: Took into account that the light object is not required for HG2 graphics.
%
% * 16-10-2018: Made sure that the structure "ModeDSGEModel" has a valid field for "SystemPriorFile".
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 09-02-2021: The function now supports the input variable "IsAdaptiveLearning".
%
% * 21-04-2021: Added a WaitDLG call during the calculations when no progress dialog is displayed.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 05-07-2023: The function now tries to load the AIMData file directly.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.ModeData = ModeData;
   DSGEModel.UseTransformed = UseTransformed;
   DSGEModel.IsAdaptiveLearning = IsAdaptiveLearning;
   %
   % make sure that files are copied
   %
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy a belief system file
   %
   if IsAdaptiveLearning==1;
      if DSGEModel.BeliefSystem==2;
         [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
      end;
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
      if IsAdaptiveLearning==1;
         if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
            [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
         end;
      end;
   end;
else
   ThreeDeeParamsGraphGUI = findobj('Type','figure','Tag','ThreeDeeParamsGraphDLG');
   ThreeDeeControls = get(ThreeDeeParamsGraphGUI,'UserData');
   CurrINI = get(ThreeDeeControls.parameter1popup,'UserData');
   DSGEModel = get(ThreeDeeControls.parameter1,'UserData');
   UseTransformed = DSGEModel.UseTransformed;
   IsAdaptiveLearning = DSGEModel.IsAdaptiveLearning;
end;
%
switch selector;
   case 'init';
      ThreeDeeParamsGraphGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ThreeDeeParamsGraphDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-500)/2 (CurrINI.scrsz(4)-210)/2 500 190], ...
         'Tag','ThreeDeeParamsGraphDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name','Posterior Mode Surface', ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ThreeDeeParamsGraphDLG done', ...
         'ToolBar','none');

      if MatlabNumber>=7.0;
         set(ThreeDeeParamsGraphGUI,'DockControl','off');
      end;
      if UseTransformed==1;
         set(ThreeDeeParamsGraphGUI,'Name','Posterior Mode Surface - Transformed Parameters');
      else;
         set(ThreeDeeParamsGraphGUI,'Name','Posterior Mode Surface - Original Parameters');
      end;
      %
      % Check where the sample of data actually begins
      %
      [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.ModeData.DSGEModel.SubBeginYear,DSGEModel.ModeData.DSGEModel.SubBeginPeriod,DSGEModel.ModeData.DSGEModel.DataFrequency,DSGEModel.ModeData.DSGEModel.KalmanFirstObservation-1);
      %
      % Outer axis
      %

      AxesBox([2 2 498 180],'Graphics',45,[0.5 1],'on',CurrINI);

      ThreeDeeControls.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 140 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String',['Estimation Sample: ' NewStartYear ':' NewStartPeriod ' - ' DSGEModel.ModeData.DSGEModel.SubEndYear ':' DSGEModel.ModeData.DSGEModel.SubEndPeriod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      % Select first parameter
      %
      ThreeDeeControls.parameter1 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 105 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select parameter 1:', ...
         'HorizontalAlignment','left', ...
         'Tag','ParameterOneText');
      %
      ThreeDeeControls.parameter1popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 110 175 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ModeData.ParameterNames.estimated, ...
         'HorizontalAlignment','center', ...
         'CallBack','ThreeDeeParamsGraphDLG checkparams', ...
         'Tag','ParameterOnePopup',...
         'Value',1);
      %
      % Select second parameter
      %
      ThreeDeeControls.parameter2 = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 75 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select parameter 2:', ...
         'HorizontalAlignment','left', ...
         'Tag','ParameterTwoText');
      %
      ThreeDeeControls.parameter2popup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 80 175 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',ModeData.ParameterNames.estimated, ...
         'HorizontalAlignment','center', ...
         'CallBack','ThreeDeeParamsGraphDLG checkparams', ...
         'Tag','ParameterTwoPopup',...
         'Value',2);
      %
      % select grid width
      %
      ThreeDeeControls.GridWidthText = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 45 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select grid width:', ...
         'HorizontalAlignment','left', ...
         'Tag','GridWidthText');
      %
      ThreeDeeControls.GridWidth = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 50 175 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','mode +/-1 standard deviation|mode +/-2 standard deviations|mode +/-3 standard deviations|mode +/-4 standard deviations|mode +/-5 standard deviations|mode +/-6 standard deviations|mode +/-7 standard deviations|mode +/-8 standard deviations|mode +/-9 standard deviations|mode +/-10 standard deviations', ...
         'Value',DSGEModel.GridWidth, ...
         'Tag','GridWidthPopup');
      %
      % number of grid points
      %
      ThreeDeeControls.NumGridPointsText = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 16 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Number of grid points per axes:', ...
         'HorizontalAlignment','left', ...
         'Tag','NumGridPointsText');
      %
      ThreeDeeControls.NumGridPoints = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[200 20 175 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','popup', ...
         'Enable','on', ...
         'Visible','on', ...
         'HorizontalAlignment','left', ...
         'String','10|15|20|25|30|35|40|45|50|55|60|65|70|75|80|85|90|95|100|110|120|130|140|150|160|170|180|190|200', ...
         'Value',DSGEModel.NumberOfGridPoints, ...
         'Tag','NumberOfGridPointsPopup');
      %
      % Create graph button
      %
      ThreeDeeControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 50 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ThreeDeeParamsGraphDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      ThreeDeeControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[395 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ThreeDeeParamsGraphDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(ThreeDeeParamsGraphGUI,'UserData',ThreeDeeControls,...
         'HandleVisibility','callback');
   case 'checkparams'
      Param1Val = get(ThreeDeeControls.parameter1popup,'Value');
      Param2Val = get(ThreeDeeControls.parameter2popup,'Value');
      if Param1Val==Param2Val;
         %
         % check which controls should be adjusted
         %
         NumParams = size(get(ThreeDeeControls.parameter1popup,'String'),1);
         if gco==ThreeDeeControls.parameter1popup;
            %
            % parameter 1 has changed
            %
            if Param2Val<NumParams;
               set(ThreeDeeControls.parameter2popup,'Value',Param2Val+1);
            else;
               set(ThreeDeeControls.parameter2popup,'Value',1);
            end;
         else;
            %
            % parameter 2 has changed
            %
            if Param2Val<NumParams;
               set(ThreeDeeControls.parameter1popup,'Value',Param1Val+1);
            else;
               set(ThreeDeeControls.parameter1popup,'Value',1);
            end;
         end;
      end;
   case 'showgraph'
      set(ThreeDeeParamsGraphGUI,'Visible','off');
      drawnow;
      drawnow;
      Param1Value = get(ThreeDeeControls.parameter1popup,'Value');
      Param1String = get(ThreeDeeControls.parameter1popup,'String');
      Param1 = StringTrim(Param1String(Param1Value,:));
      %
      Param2Value = get(ThreeDeeControls.parameter2popup,'Value');
      Param2String = get(ThreeDeeControls.parameter2popup,'String');
      Param2 = StringTrim(Param2String(Param2Value,:));
      %
      % determine the grid of the two parameters
      %
      phiMode = DSGEModel.ModeData.phiMode;
      LogPostDensity = DSGEModel.ModeData.LogPostDensity;
      InverseHessian = DSGEModel.ModeData.InverseHessian;
      thetaIndex = DSGEModel.ModeData.thetaIndex;
      UniformBounds = DSGEModel.ModeData.UniformBounds;
      LowerBound = DSGEModel.ModeData.LowerBound;
      thetaPositions = DSGEModel.ModeData.thetaPositions;
      thetaDist = DSGEModel.ModeData.thetaDist;
      PriorDist = DSGEModel.ModeData.PriorDist;
      ModelParameters = DSGEModel.ModeData.ModelParameters;
      ModeDSGEModel = DSGEModel.ModeData.DSGEModel;
      %
      % try to load the AIMData, other wise use ModeData
      %
      if MatlabNumber>=8.0;
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch ME;
            AIMData = DSGEModel.ModeData.AIMData;
         end;
      else;
         try;
            AIMData = load(DSGEModel.AIMDataFile);
         catch;
            AIMData = DSGEModel.ModeData.AIMData;
         end;
      end;
      if IsAdaptiveLearning==1;
         %SMatrix,Sperp,m,G,Gtilde,Kgf,xbar
         SMatrix = DSGEModel.ModeData.SMatrix;
         Sperp = DSGEModel.ModeData.Sperp;
         Gtilde = DSGEModel.ModeData.Gtilde;
         Kgf = DSGEModel.ModeData.Kgf;
         m = DSGEModel.ModeData.BeliefSys.m;
         G = DSGEModel.ModeData.BeliefSys.G;
         ConstPos = 0;
         k = DSGEModel.k;
         if k>0;
            [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
            ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
            if ConstPos==0;
               %
               % we will now check if X has a unit vector
               %
               X = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
               T = size(X,2);
               for j=1:k;
                  ConstTest = (DSGEModel.X(j,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)==ones(1,T));
                  if sum(ConstTest)==T;
                     ConstPos = j;
                     break;
                  end;
               end;
            end;
            xbar = zeros(k,1);
            xbar(ConstPos) = 1;
         else;
            xbar = [];
         end;
      end;
      %
      % check some ModeDSGEModel fields
      %
      if isfield(ModeDSGEModel,'SystemPriorFile')==0;
         ModeDSGEModel.SystemPriorFile = pwd;
      end;
      if strcmp(lower(DSGEModel.DataConstructionFile),lower(ModeDSGEModel.DataConstructionFile))==0;
         txt = ['The data construction file that was used when estimating the posterior mode has a different name (' GetName(ModeDSGEModel.DataConstructionFile) ') than your currently specified data construction file (' GetName(DSGEModel.DataConstructionFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
         About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
         %
         set(ThreeDeeParamsGraphGUI,'Visible','on');
         drawnow;
         pause(0.02);
         return;
      end;
      if strcmp(lower(DSGEModel.MeasurementEquationFile),lower(ModeDSGEModel.MeasurementEquationFile))==0;
         txt = ['The measurement equation file that was used when estimating the posterior mode has a different name (' GetName(ModeDSGEModel.MeasurementEquationFile) ') than your currently specified measurement equation file (' GetName(DSGEModel.MeasurementEquationFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
         About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
         %
         set(ThreeDeeParamsGraphGUI,'Visible','on');
         drawnow;
         pause(0.02);
         return;
      end;
      if strcmp(lower(DSGEModel.PriorFile),lower(ModeDSGEModel.PriorFile))==0;
         txt = ['The prior distribution specification file that was used when estimating the posterior mode has a different name (' GetName(ModeDSGEModel.PriorFile) ') than your currently specified prior distribution specification file (' GetName(DSGEModel.PriorFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
         About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
         %
         set(ThreeDeeParamsGraphGUI,'Visible','on');
         drawnow;
         pause(0.02);
         return;
      end;
      if strcmp(lower(DSGEModel.UpdateParameterFile),lower(ModeDSGEModel.UpdateParameterFile))==0;
         txt = ['The file with parameters to update that was used when estimating the posterior mode has a different name (' GetName(ModeDSGEModel.UpdateParameterFile) ') than your currently specified file with parameters to update (' GetName(DSGEModel.UpdateParameterFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
         About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
         %
         set(ThreeDeeParamsGraphGUI,'Visible','on');
         drawnow;
         pause(0.02);
         return;
      end;
      if strcmp(lower(DSGEModel.InitializeParameterFile),lower(ModeDSGEModel.InitializeParameterFile))==0;
         txt = ['The file with parameters to initialize that was used when estimating the posterior mode has a different name (' GetName(ModeDSGEModel.InitializeParameterFile) ') than your currently specified file with parameters to initialize (' GetName(DSGEModel.InitializeParameterFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
         About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
         %
         set(ThreeDeeParamsGraphGUI,'Visible','on');
         drawnow;
         pause(0.02);
         return;
      end;
      if strcmp(lower(DSGEModel.AIMFile),lower(ModeDSGEModel.AIMFile))==0;
         txt = ['The AiM model file that was used when estimating the posterior mode has a different name (' GetFilename(ModeDSGEModel.AIMFile) ') than your currently specified AiM model file (' GetFilename(DSGEModel.AIMFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
         About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
         %
         set(ThreeDeeParamsGraphGUI,'Visible','on');
         drawnow;
         pause(0.02);
         return;
      end;
      if DSGEModel.ModelSolver==4;
         if strcmp(lower(DSGEModel.ModelSolverFile),lower(ModeDSGEModel.ModelSolverFile))==0;
            txt = ['The model solver that was used when estimating the posterior mode has a different name (' GetName(ModeDSGEModel.ModelSolverFile) ') than your currently specified model solver file (' GetName(DSGEModel.ModelSolverFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
            About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
            %
            set(ThreeDeeParamsGraphGUI,'Visible','on');
            drawnow;
            pause(0.02);
            return;
         end;
      end;
      %
      if IsAdaptiveLearning==1;
         if strcmp(lower(DSGEModel.BeliefSystemFile),lower(ModeDSGEModel.BeliefSystemFile))==0;
            txt = ['The belief system file for the perceived law of motion that was used when estimating the posterior mode has a different name (' GetName(ModeDSGEModel.BeliefSystemFile) ') than your currently specified belief system file (' GetName(DSGEModel.BeliefSystemFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
            About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
            %
            set(ThreeDeeParamsGraphGUI,'Visible','on');
            drawnow;
            pause(0.02);
            return;
         end;
         if DSGEModel.ModelSolver==4;
            if strcmp(lower(DSGEModel.AdaptiveLearningModelSolverFile),lower(ModeDSGEModel.AdaptiveLearningModelSolverFile))==0;
               txt = ['The model solver that was used when estimating the posterior mode has a different name (' GetName(ModeDSGEModel.AdaptiveLearningModelSolverFile) ') than your currently specified model solver file (' GetName(DSGEModel.AdaptiveLearningModelSolverFile) '). YADA has therefore aborted the calculation of the surface of the log posterior around the posterior mode.'];
               About(txt,'information','Bad Data - Posterior Mode Surface',200,500,CurrINI);
               %
               set(ThreeDeeParamsGraphGUI,'Visible','on');
               drawnow;
               pause(0.02);
               return;
            end;
         end;
      end;
      %
      % determine the grid width
      %
      GridWidth = get(ThreeDeeControls.GridWidth,'Value');
      %
      % determine the number of grid points
      %
      NumGridPointsString = get(ThreeDeeControls.NumGridPoints,'String');
      NumGridPoints = str2num(StringTrim(NumGridPointsString(get(ThreeDeeControls.NumGridPoints,'Value'),:)));
      %
      % Parameter 1
      %
      MinParam1 = phiMode(Param1Value)-(GridWidth*sqrt(InverseHessian(Param1Value,Param1Value)));
      MaxParam1 = phiMode(Param1Value)+(GridWidth*sqrt(InverseHessian(Param1Value,Param1Value)));
      Param1Grid = (MinParam1:((MaxParam1-MinParam1)/NumGridPoints):MaxParam1);
      %
      % Parameter 2
      %
      MinParam2 = phiMode(Param2Value)-(GridWidth*sqrt(InverseHessian(Param2Value,Param2Value)));
      MaxParam2 = phiMode(Param2Value)+(GridWidth*sqrt(InverseHessian(Param2Value,Param2Value)));
      Param2Grid = (MinParam2:((MaxParam2-MinParam2)/NumGridPoints):MaxParam2);
      %
      % Initialize the output
      %
      LogPostMatrix = zeros(length(Param2Grid),length(Param1Grid));
      XDataMatrix = ones(length(Param2Grid),1)*Param1Grid;
      YDataMatrix = Param2Grid'*ones(1,length(Param1Grid));
      NumPairs = length(Param1Grid)*length(Param2Grid);
      CurrPair = 0;
      phi = phiMode;
      ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
      ParameterBounds = ParameterBounds(:,1:2);
      %
      % display a progress dialog
      %
      NumParamPairs = length(Param1Grid)*length(Param2Grid);
      if DSGEModel.ShowProgress==1;
         ProgressStructure.title = ['Progress for ' int2str(NumParamPairs) ' parameter pairs'];
         ProgressStructure.facecolor = CurrINI.progress_facecolor;
         ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
         ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
         ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
         ProgressStructure.stop = 0;
         ProgressStructure.clock = DSGEModel.ShowProgressClock;
         ProgressStructure.label = ['(' Param1 ',' Param2 ') ='];
         %
         ProgressStructure.name = 'Posterior Mode Surface';
         ProgressStructure.CurrINI = CurrINI;
         WaitHandle = ProgressDLG(0,ProgressStructure);
         set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
         drawnow;
      else;
         txt = ['Please wait while YADA computes the log posterior kernel for ' int2str(NumParamPairs) ' parameter pairs for (' Param1 ',' Param2 '). Calculation was started at: ' StringTrim(datestr(now,14)) ' on ' GetCurrentDate '.'];
         WaitHandle = WaitDLG(txt,'information','Posterior Mode Surface',500,200,CurrINI,0);
      end;
      %
      if UseTransformed==0;
         Theta1Grid = zeros(1,length(Param1Grid));
         Theta2Grid = zeros(1,length(Param2Grid));
      end;
      abort = '';
      StopEstimation = 0;
      Num1 = 1;
      for i=Param1Grid;
         phi(Param1Value) = i;
         Num2 = 1;
         for j=Param2Grid;
            CurrPair = CurrPair+1;
            %
            % compute the log posterior
            %
            phi(Param2Value) = j;
            if UseTransformed==0;
               theta = PhiToTheta(phi,thetaIndex,UniformBounds,LowerBound);
               Theta1Grid(1,Num1) = theta(Param1Value);
               Theta2Grid(1,Num2) = theta(Param2Value);
            end;
            %
            % check if we should update the progress dialog
            %
            if DSGEModel.ShowProgress==1;
               abort = get(WaitHandle,'UserData');
               if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
                  StopEstimation = 1;
                  break;
               else;
                  if UseTransformed==1;
                     ProgressDLG([CurrPair/NumParamPairs phi(Param1Value) phi(Param2Value)]);
                  else;
                     ProgressDLG([CurrPair/NumParamPairs theta(Param1Value) theta(Param2Value)]);
                  end;
               end;
            end;
            if StopEstimation==1;
               break;
            end;
            if IsAdaptiveLearning==0;
               if UseTransformed==1;
                  logPost = logPosteriorPhiDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,ModeDSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
               else;
                  logPost = logPosteriorThetaDSGE(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,ModeDSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
               end;
            else;
               if UseTransformed==1;
                  logPost = logPosteriorAdaptiveLearningPhiDSGE(phi,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);;
               else;
                  logPost = logPosteriorAdaptiveLearningThetaDSGE(theta,thetaIndex,ParameterBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,CurrINI.OrderQZ,CurrINI.NumWorkers);
               end;
            end;
            %
            % check if the value is proper
            %
            if logPost~=1000000;
               LogPostMatrix(Num2,Num1) = -logPost;
               if UseTransformed==0;
                  XDataMatrix(Num2,Num1) = theta(Param1Value);
                  YDataMatrix(Num2,Num1) = theta(Param2Value);
               end;
            else;
               LogPostMatrix(Num2,Num1) = NaN;
               XDataMatrix(Num2,Num1) = NaN;
               YDataMatrix(Num2,Num1) = NaN;
            end;
            Num2 = Num2+1;
         end;
         Num1 = Num1+1;
      end;
      %
      % close the wait handle
      %
      if DSGEModel.ShowProgress==1;
         if IsHandle(WaitHandle)==1;
            set(WaitHandle,'UserData','done');
            close(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      else;
         if IsHandle(WaitHandle)==1;
            WaitControls = get(WaitHandle,'UserData');
            set(WaitControls.text,'UserData','done');
            delete(WaitHandle);
            drawnow;
            pause(0.02);
         end;
      end;
      %
      set(ThreeDeeParamsGraphGUI,'Visible','on');
      drawnow;
      pause(0.02);
      %
      % check if we should display results
      %
      if StopEstimation==0;
         if UseTransformed==0;
            thetaMode = DSGEModel.ModeData.thetaMode;
            Param1Grid = Theta1Grid;
            Param2Grid = Theta2Grid;
            LogPostDensity = LogPostDensity-DSGEModel.ModeData.LogJacobianValue;
            LogPostStr = 'log p (\theta | Data)';
         else;
            thetaMode = phiMode;
            LogPostStr = 'log p (\phi | Data)';
         end;
         %
         % Setting up the surface
         %
         FigHandle = figure('Units','pixels', ...
            'Position',[(CurrINI.scrsz(3)-650)/2 (CurrINI.scrsz(4)-500)/2 650 500], ...
            'NumberTitle','off', ...
            'Visible','off', ...
            'Tag',[Param1 '-' Param2], ...
            'Name',['Surface Plot of Log-Posterior, ' Param1 ', and ' Param2]);
         %
         SetFigureProperties(FigHandle,CurrINI);
         %
         Surfhandle = surf(XDataMatrix',YDataMatrix',LogPostMatrix','EdgeColor','none','LineWidth',1,'FaceLighting','gouraud','SpecularStrength',0.5);
         hold('on');
         %
         % ensure the presence of a colormap
         %
         colormap(jet);
         CurrColorMap = colormap;
         %
         % shading
         %
         shading('interp');
         %
         ZLim = get(gca,'ZLim');
         %
         % plot lines for the posterior mode of parameter 1
         %
         Mode1Handle = plot3(thetaMode(Param1Value)*ones(1,2),[Param2Grid(1) Param2Grid(length(Param2Grid))],LogPostDensity*ones(1,2),'LineStyle','-','Color',[0 0 0],'LineWidth',2);
         hold('on');
         Mode1HandleLeft = plot3(thetaMode(Param1Value)*ones(1,2),Param2Grid(1)*ones(1,2),[ZLim(1) LogPostDensity],'LineStyle','-','Color',[0 0 0],'LineWidth',2);
         hold('on');
         Mode1HandleRight = plot3(thetaMode(Param1Value)*ones(1,2),Param2Grid(length(Param2Grid))*ones(1,2),[ZLim(1) LogPostDensity],'LineStyle','-','Color',[0 0 0],'LineWidth',2);
         hold('on');
         Mode1HandleBottom = plot3(thetaMode(Param1Value)*ones(1,2),[Param2Grid(1) Param2Grid(length(Param2Grid))],ZLim(1)*ones(1,2),'LineStyle','-','Color',[0 0 0],'LineWidth',2);
         hold('on');
         %
         % plot lines for the posterior mode of parameter 2
         %
         Mode2Handle = plot3([Param1Grid(1) Param1Grid(length(Param1Grid))],thetaMode(Param2Value)*ones(1,2),LogPostDensity*ones(1,2),'LineStyle','-','Color',[0 0 0],'LineWidth',2);
         hold('on');
         Mode2HandleLeft = plot3(Param1Grid(1)*ones(1,2),thetaMode(Param2Value)*ones(1,2),[ZLim(1) LogPostDensity],'LineStyle','-','Color',[0 0 0],'LineWidth',2);
         hold('on');
         Mode2HandleRight = plot3(Param1Grid(length(Param1Grid))*ones(1,2),thetaMode(Param2Value)*ones(1,2),[ZLim(1) LogPostDensity],'LineStyle','-','Color',[0 0 0],'LineWidth',2);
         hold('on');
         Mode2HandleBottom = plot3([Param1Grid(1) Param1Grid(length(Param1Grid))],thetaMode(Param2Value)*ones(1,2),ZLim(1)*ones(1,2),'LineStyle','-','Color',[0 0 0],'LineWidth',2);
         hold('on');
         %
         % set the limits
         %
         set(gca,'XLim',[Param1Grid(1) Param1Grid(length(Param1Grid))],'YLim',[Param2Grid(1) Param2Grid(length(Param2Grid))]);
         %
         % add a light
         %
         if MatlabNumber<=8.3;
            LightHandle = light('Position',[Param1Grid(1) Param2Grid(1) max(get(gca,'ZLim'))/2],'Style','infinite');
            hold('on');
         end;
         %
         % add a colorbar
         %
         CurrAxes = gca;
         ColorBarHandle = colorbar;
         set(ColorBarHandle,'Tag','colorbar');
         SetAxesFonts(CurrINI,ColorBarHandle);
         %
         % make sure the the bar is not too wide
         %
         ColorBarPosition = get(ColorBarHandle,'Position');
         AxesPosition = get(CurrAxes,'Position');
         set(ColorBarHandle,'Position',[ColorBarPosition(1)+(2*ColorBarPosition(3)/3) ColorBarPosition(2) ColorBarPosition(3)/3 ColorBarPosition(4)]);
         set(CurrAxes,'Position',AxesPosition);
         hold('off');
         %
         % fix the title, xlabel, ylabel and zlabel
         %
         SetAxesFonts(CurrINI,'Title',['Log-posterior for ' Param1 ' and ' Param2],'XLabel',Param1,'Ylabel',Param2,'ZLabel',LogPostStr);
         %
         if (strcmp(get(gca,'YGrid'),'on')==1)&(strcmp(get(gca,'ZGrid'),'on')==1);
            set(gca,'XGrid','on');
         end;
         %
         % Fix the toolbar
         %
         SetToolbarImages(FigHandle,CurrINI.images);
         set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
         drawnow;
         pause(0.02);
         %
         % set up the contour plot
         %
         if GridWidth>1;
            %
            % check the colorbar in the previous plot
            %
            if MatlabNumber>8.3;
               MinMax = get(ColorBarHandle,'Limits');
            else;
               CBKids = get(ColorBarHandle,'Children');
               MinMax = get(CBKids(1),'YData');
            end;
            %
            FigHandle = figure('Units','pixels', ...
               'Position',[(CurrINI.scrsz(3)-650)/2+22 (CurrINI.scrsz(4)-500)/2-22 650 500], ...
               'NumberTitle','off', ...
               'Visible','off', ...
               'Tag',[Param1 '-' Param2], ...
               'Name',['Contour Plot of Log-Posterior, ' Param1 ', and ' Param2]);
            %
            SetFigureProperties(FigHandle,CurrINI);
            %
            % compute data for contour, i.e. 80, 90, 95, 98 and 99 percent values for lnL
            %
            ContourVec = zeros(1,min(GridWidth-1,3));
            ContourHandles = ContourVec;
            LegendText = '';
            LineStyles = {':' '--' '-'};
            for i=1:length(ContourVec);
               %
               % Find the correct x and y data
               %
               XLow = phiMode(Param1Value)-(i*sqrt(InverseHessian(Param1Value,Param1Value)));
               XHigh = phiMode(Param1Value)+(i*sqrt(InverseHessian(Param1Value,Param1Value)));
               YLow = phiMode(Param2Value)-(i*sqrt(InverseHessian(Param2Value,Param2Value)));
               YHigh = phiMode(Param2Value)+(i*sqrt(InverseHessian(Param2Value,Param2Value)));
               if UseTransformed==0;
                  %
                  % we need to map the Xlow, XHigh, YLow, and YHigh values into the
                  % original parameters
                  %
                  phiTest = phiMode;
                  phiTest(Param1Value) = XLow;
                  thetaTest = PhiToTheta(phiTest,thetaIndex,UniformBounds,LowerBound);
                  XLow = thetaTest(Param1Value);
                  %
                  phiTest(Param1Value) = XHigh;
                  thetaTest = PhiToTheta(phiTest,thetaIndex,UniformBounds,LowerBound);
                  XHigh = thetaTest(Param1Value);
                  %
                  phiTest = phiMode;
                  phiTest(Param2Value) = YLow;
                  thetaTest = PhiToTheta(phiTest,thetaIndex,UniformBounds,LowerBound);
                  YLow = thetaTest(Param2Value);
                  %
                  phiTest(Param2Value) = YHigh;
                  thetaTest = PhiToTheta(phiTest,thetaIndex,UniformBounds,LowerBound);
                  YHigh = thetaTest(Param2Value);
               end;
               %
               % find the columns for the XLow and the XHigh values
               %
               XLowPos1 = (Param1Grid>=XLow) .* (1:length(Param1Grid));
               XLowPos1 = min(XLowPos1(XLowPos1>0));
               XLowPos2 = (Param1Grid<XLow) .* (1:length(Param1Grid));
               XLowPos2 = max(XLowPos2(XLowPos2>0));
               %
               XHighPos1 = (Param1Grid>=XHigh) .* (1:length(Param1Grid));
               XHighPos1 = min(XHighPos1(XHighPos1>0));
               XHighPos2 = (Param1Grid<XHigh) .* (1:length(Param1Grid));
               XHighPos2 = max(XHighPos2(XHighPos2>0));
               %
               % find the columns of the YLow and the YHigh values
               %
               YLowPos1 = (Param2Grid>=YLow) .* (1:length(Param2Grid));
               YLowPos1 = min(YLowPos1(YLowPos1>0));
               YLowPos2 = (Param2Grid<YLow) .* (1:length(Param2Grid));
               YLowPos2 = max(YLowPos2(YLowPos2>0));
               %
               YHighPos1 = (Param2Grid>=YHigh) .* (1:length(Param2Grid));
               YHighPos1 = min(YHighPos1(YHighPos1>0));
               YHighPos2 = (Param2Grid<YHigh) .* (1:length(Param2Grid));
               YHighPos2 = max(YHighPos2(YHighPos2>0));
               %
               % collect the appropriate value
               %
               ContourVec(i) = min(min(LogPostMatrix([YLowPos1 YLowPos2 YHighPos1 YHighPos2],[XLowPos1 XLowPos2 XHighPos1 XHighPos2])));
               LegendText = strvcat(LegendText,['min log-posterior at ' num2str(i,'%0.0f') ' std = ' num2str(ContourVec(i),'%0.4f')]);
               %
               % from the ContourVec(i) value establish it relative position with respect to MinMax
               %
               ColorMapValue = ceil(((MinMax(2)-ContourVec(i))/(MinMax(2)-MinMax(1)))*64);
               if ColorMapValue>64;
                  ColorMapValue = 64;
               elseif ColorMapValue<1;
                  ColorMapValue = 1;
               end;
               CurrColor = CurrColorMap(65-ColorMapValue,:);
               %
               % plot the contour - one level at a time
               %
               [c_mat,ContourHandle] = contour(XDataMatrix',YDataMatrix',LogPostMatrix',ContourVec(i)*ones(1,2),'-');
               hold('on');
               %
               % here we can fix the properties of ContourHandle if we wish
               %
               set(ContourHandle,'Color',CurrColor,'LineWidth',2,'LineStyle',LineStyles{i});
               ContourHandles(i) = ContourHandle(1);
            end;
            %
            % plot the posterior mode of parameter 1
            %
            Mode1Handle = plot(thetaMode(Param1Value)*ones(1,2),[Param2Grid(1) Param2Grid(length(Param2Grid))],'LineStyle','-','Color',[0 0 0],'LineWidth',1);
            hold('on');
            %
            % plot the posterior mode of parameter 2
            %
            Mode2Handle = plot([Param1Grid(1) Param1Grid(length(Param1Grid))],thetaMode(Param2Value)*ones(1,2),'LineStyle','-','Color',[0 0 0],'LineWidth',1);
            hold('on');
            %
            % set the limits
            %
            set(gca,'XLim',[Param1Grid(1) Param1Grid(length(Param1Grid))],'YLim',[Param2Grid(1) Param2Grid(length(Param2Grid))]);
            %
            % fix the title, xlabel, and ylabel
            %
            SetAxesFonts(CurrINI,'Title',['Contour plots of log-posterior for ' Param1 ' and ' Param2],'XLabel',Param1,'Ylabel',Param2);
            %
            % set the legend
            %
            legHandle = legend(ContourHandles,LegendText);
            set(legHandle,'Tag','legend');
            SetAxesFonts(CurrINI,legHandle);
            hold('off');
            %
            % Fix the toolbar
            %
            SetToolbarImages(FigHandle,CurrINI.images);
            set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
            drawnow;
            pause(0.02);
         end;
      end;
   case 'done'
      set(ThreeDeeControls.done,'UserData','done');
      delete(ThreeDeeParamsGraphGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(ThreeDeeParamsGraphGUI,'Visible','on');
   drawnow;
   SetYADAIcon(ThreeDeeParamsGraphGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(ThreeDeeControls.done);
   end;
   waitfor(ThreeDeeControls.done,'UserData','done');
end;

%
% end of ThreeDeeParamsGraphDLG.m
%
