function [action,OutTextStr] = TwoSelectionsDlg(NameStr,EditTitle,TextStr,width,CurrINI)
% TwoSelectionsDlg: Displays two edit dialogs with an OK and a Cancel button.
%
% USAGE:
%
%       [action,OutTextStr] = TwoSelectionsDlg(NameStr,EditTitle,TextStr,width,CurrINI)
%
% OPTIONAL INPUT:  NameStr (string vector) with the figure name.
%
%                  EditTitle (cell array) with two string for the titles of the edit objects.
%
%                  TextStr (cell array) with two strings with the text for the edit dialogs.
%
%                  width (integer) width the dialog width in pixel units with a minimum of
%                        200. The default value is 500.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: action (string vector) that is either "OK" or "Cancel", where
%                        the latter is the default.
%
%                  OutTextStr (cell array) with two strings of the text in the edit
%                        dialogs.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 13, 2017.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-04-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% temporary initialization
%
action = 'Cancel';
OutTextStr = TextStr;
if nargin==0;
   NameStr = 'Text Dialogue';
   EditTitle{1} = 'Enter Text:';
   EditTitle{2} = 'Enter Text:';
   TextStr{1} = 'text';
   TextStr{2} = 'text';
elseif nargin==1;
   EditTitle{1} = 'Enter Text:';
   EditTitle{2} = 'Enter Text:';
   TextStr{1} = 'text';
   TextStr{2} = 'text';
elseif nargin==2;
   TextStr{1} = 'text';
   TextStr{2} = 'text';
end;
if nargin<4;
   width = 500;
end;
if nargin<5;
   CurrINI = InitializeINIFile;
end;
if width<200;
   width = 200;
end;
%
scrsz = get(CurrINI.GraphicsRoot,'ScreenSize');
%
editgui = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
   'FileName','EditDLG.m', ...
   'MenuBar','none', ...
   'Units','pixels', ...
   'Position',[(scrsz(3)-width)/2 (scrsz(4)-110)/2 width 110], ...
   'Tag','FigEditDLG', ...
   'Resize','off', ...
   'Name',NameStr, ...
   'Visible','off', ...
   'NumberTitle','off', ...
   'WindowStyle','modal', ...
   'CloseRequestFcn','CloseGUI', ...
   'ToolBar','none');
%
if MatlabNumber>=7.0;
   set(editgui,'DockControl','off');
end;
%
% edit dialog one
%
uicontrol('Units','pixels', ...
   'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
   'Position',[12 75 (width/2)-17 20], ...
   'FontSize',CurrINI.gui_fontsize, ...
   'FontName',CurrINI.gui_fontname, ...
   'FontWeight',CurrINI.gui_fontweight, ...
   'FontAngle',CurrINI.gui_fontangle, ...
   'Style','text', ...
   'HorizontalAlignment','left', ...
   'String',EditTitle{1}, ...
   'Tag','TextOne');
%
edits.editone = uicontrol('Units','pixels', ...
   'BackgroundColor',[1 1 1], ...
   'Position',[12 50 (width/2)-17 25], ...
   'Value',1, ...
   'String',TextStr{1}, ...
   'Style','edit', ...
   'HorizontalAlignment','left', ...
   'FontSize',CurrINI.gui_fontsize, ...
   'FontName',CurrINI.gui_fontname, ...
   'FontWeight',CurrINI.gui_fontweight, ...
   'FontAngle',CurrINI.gui_fontangle, ...
   'Enable','on', ...
   'Tag','EditEditOne');
%
% edit dialog two
%
uicontrol('Units','pixels', ...
   'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
   'Position',[(width/2)+5 75 (width/2)-17 20], ...
   'FontSize',CurrINI.gui_fontsize, ...
   'FontName',CurrINI.gui_fontname, ...
   'FontWeight',CurrINI.gui_fontweight, ...
   'FontAngle',CurrINI.gui_fontangle, ...
   'Style','text', ...
   'HorizontalAlignment','left', ...
   'String',EditTitle{2}, ...
   'Tag','TextTwo');
%
edits.edittwo = uicontrol('Units','pixels', ...
   'BackgroundColor',[1 1 1], ...
   'Position',[(width/2)+5 50 (width/2)-17 25], ...
   'Value',1, ...
   'String',TextStr{2}, ...
   'Style','edit', ...
   'HorizontalAlignment','left', ...
   'FontSize',CurrINI.gui_fontsize, ...
   'FontName',CurrINI.gui_fontname, ...
   'FontWeight',CurrINI.gui_fontweight, ...
   'FontAngle',CurrINI.gui_fontangle, ...
   'Enable','on', ...
   'Tag','EditEditTwo');
%
% Two buttons to select action
%
edits.ok = uicontrol('Units','pixels', ...
   'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
   'Position',[((width-5)/2)-50 12 50 25], ...
   'String','OK', ...
   'FontSize',CurrINI.gui_fontsize, ...
   'FontName',CurrINI.gui_fontname, ...
   'FontWeight',CurrINI.gui_fontweight, ...
   'FontAngle',CurrINI.gui_fontangle, ...
   'Enable','on', ...
   'Callback','CloseGUI', ...
   'UserData','waiting', ...
   'Tag','ButtonOK');
%
edits.cancel = uicontrol('Units','pixels', ...
   'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
   'Position',[(width+5)/2 12 50 25], ...
   'String','Cancel', ...
   'FontSize',CurrINI.gui_fontsize, ...
   'FontName',CurrINI.gui_fontname, ...
   'FontWeight',CurrINI.gui_fontweight, ...
   'FontAngle',CurrINI.gui_fontangle, ...
   'Enable','on', ...
   'Callback','CloseGUI', ...
   'Tag','ButtonCancel');
%
set(editgui,'Visible','on');
drawnow;
SetYADAIcon(editgui,CurrINI.ShowYADAIcon);
if MatlabNumber>=7;
   uicontrol(edits.editone);
end;
waitfor(editgui,'waitstatus','inactive');
if (isempty(get(editgui,'CurrentObject'))==1)|(strcmp(get(get(editgui,'CurrentObject'),'Type'),'uicontrol')==0);
   action = 'Cancel';
else;
   action = get(get(editgui,'CurrentObject'),'String');
   if isempty(action)==1;
      action = 'Cancel';
   end;
end;
%
if strcmp(action,'Cancel')==0;
   OutTextStr{1} = get(edits.editone,'String');
   OutTextStr{2} = get(edits.edittwo,'String');
else;
   OutTextStr{1} = '';
   OutTextStr{2} = '';
end;
%
delete(editgui);
drawnow;
pause(0.02);

%
% end of TwoSelectionsDlg.m
%
