function ZLBPredictiveDistributionDLG(selector,DSGEModel,CurrINI,ZLBYMeanPrediction,ObsVarMean,ZLBYQuantiles,KernelX,KernelY,KsitT,ZLBKsiMeanPrediction,ZLBKsiQuantiles,NameStr,AnnualStr,IsPosterior,IsConditional,IsCentered,UData,SelectedYData)
% ZLBPredictiveDistributionDLG: Displays predictive distributions of the observed and the state variables along with the mean
%                               predictions and when possible the observed values, when the monetary policy rate is subject
%                               to the zero lower bound.
%
% USAGE:
%
%       ZLBPredictiveDistributionDLG(selector,DSGEModel,CurrINI,ZLBYMeanPrediction,ObsVarMean,ZLBYQuantiles,KernelX,...
%                                    KernelY,KsitT,ZLBKsiMeanPrediction,ZLBKsiQuantiles,NameStr,AnnualStr,...
%                                    IsPosterior,IsConditional,IsCentered,UData,SelectedYData)
%
% REQUIRED INPUT: selector (string vector) taking on the values "init", "showgraph", and "done".
%
% OPTIONAL INPUT: DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 ZLBYMeanPrediction (matrix) of dimension nxh with the mean predictions of the observed
%                           variables.
%
%                 ObsVarMean (vector) of dimension x with the model mean of the observed variables.
%                           This variable may be empty.
%
%                 ZLBYQuantiles (structure) of dimension NumQuants where each has a field called "Quantile" which
%                           is an nxh matrix with the quantiles of the prediction paths. The second field
%                           is "percent" that gives the percentile value as a string vector.
%
%                 KernelX (matrix) of dimension (n x NumGridPoints x h) with the x-axis values of the
%                           kernel density estimates of the marginal predictive densities for the
%                           observed variables.
%
%                 KernelY (matrix) of dimension (n x NumGridPoints x h) with the y-axis values of the
%                           kernel density estimates of the marginal predictive densities for the
%                           observed variables.
%
%                 KsitT (matrix) of dimension rxTh with smooth estimates of the state variables.
%
%                 ZLBKsiMeanPrediction (matrix) of dimension rxh with the mean predictions of the state
%                           variables.
%
%                 ZLBKsiQuantiles (structure) of dimension NumQuants where each has a field called "Quantile" which
%                           is an rxh matrix with the quantiles of the prediction paths. The second field
%                           is "percent" that gives the percentile value as a string vector.
%
%                 NameStr (string vector) taking on the values "Posterior Mode", "Initial Values",
%                           "Prior Distribution" or "Posterior Distribution".
%
%                 AnnualStr (string vector) taking on the values "Original", "Annual", or "Transformed".
%
%                 IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                           if draws from the prior are supplied.
%
%                 IsConditional (boolean) that is 1 if conditional forecasts have been computed and 0 otherwise.
%
%                 IsCentered (boolean) that is 1 if the forecasts that don't take the ZLB into account
%                           are centered on the actual values and 0 otherwise.
%
%                 UData (structure) with data on uncertainty measures for the variables. The field names depend
%                           on which model is defined through the ModelStr input.
%
%                 SelectedYData (integer) with the selected YData set when centering the predictive distribution.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: September 10, 2014.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-09-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 20-03-2018: The code now checks if the number of paths is unity and skips the patch plots
%               for confidence regions for this case.
%
% * 21-03-2018: The confidence region patches now stretch back to the last historical value.
%
% * 23-03-2018: Fixed an issue to make legend compatible with matlab 2017a and later.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 25-05-2021: Fixed a display bug when having ragged edge data and where the patch was added to
%               a NaN value for the observed variable.
%
% * 03-01-2022: Updated the documentation.
%
% * 24-08-2022: Made the dialog similar to "PredictiveDistributionDLG.m" by adding actuals selection
%               and the variable mean to the plot of the observables.
%
% * 25-08-2022: Fixed an issue with Children after having introduced the mean estimate to the graph.
%
% * 26-08-2022: Updated the documentation.
%
% * 02-09-2022: The function now supports prior and posterior distributions.
%
% * 05-09-2022: Added the UData input variable for the normal approximation of the predictive
%               distribution.
%
% * 04-10-2022: Added the input variable "IsCentered".
%
% * 27-10-2022: Made sure that the correct actual values are shown also when the centered predictions
%               are displayed. The input variable "SelectedYData" was intorduced for that reason.
%
% * 02-01-2023: Updated the documentation.
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if strcmp(selector,'init')==1;
   DSGEModel.ZLBYMeanPrediction = ZLBYMeanPrediction;
   DSGEModel.ZLBYQuantiles = ZLBYQuantiles;
   DSGEModel.KsitT = KsitT;
   DSGEModel.ZLBKsiMeanPrediction = ZLBKsiMeanPrediction;
   DSGEModel.ZLBKsiQuantiles = ZLBKsiQuantiles;
   DSGEModel.ObsVarMean = ObsVarMean;
   DSGEModel.NameStr = NameStr;
   DSGEModel.AnnualStr = AnnualStr;
   DSGEModel.IsPosterior = IsPosterior;
   DSGEModel.IsConditional = IsConditional;
   DSGEModel.UData = UData;
   DSGEModel.SelectedYData = SelectedYData;
   DSGEModel.IsCentered = IsCentered;
   %
   DSGEModel.PriorHeader = '';
   if IsPosterior==0;
      DSGEModel.PriorHeader = ' For Prior';
   end;
   %
   % Check where the sample of prediction data actually begins
   %
   [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,DSGEModel.PredDateVector,DSGEModel.PredSampleStrMat] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(ZLBYMeanPrediction,2));
   %
   % check if we have observations for the whole prediction sample
   %
   %
   % check if we have observations for the whole prediction sample
   %
   [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%   if (LastPeriod+length(DSGEModel.PredDateVector)<=size(DSGEModel.Y,2));
%      DSGEModel.ShowAdjustOption = 1;
%      AddHeight = 35;
%   else;
%      DSGEModel.ShowAdjustOption = 0;
%      AddHeight = 0;
%   end;
   if isempty(KernelX)==0;
      DSGEModel.ShowKernel = 1;
      DSGEModel.KernelX = KernelX;
      DSGEModel.KernelY = KernelY;
      AddKHeight = 35;
   else;
      DSGEModel.ShowKernel = 0;
      AddKHeight = 0;
   end;
%      DSGEModel.ShowAdjustOption = 0;
%      DSGEModel.ShowKernel = 0;
   if (isempty(DSGEModel.Actuals)==0)&(IsCentered==0);
      AddActHeight = 35;
      ActualsStr = 'Actuals from original sample';
      for i=1:length(DSGEModel.Actuals);
         ActualsStr = strvcat(ActualsStr,DSGEModel.Actuals(i).title);
      end;
   else;
      AddActHeight = 0;
      ActualsStr = '';
   end;
else
   PredictiveDistributionGUI = findobj('Type','figure','Tag','ZLBPredictiveDistributionDLG');
   PredictiveControls = get(PredictiveDistributionGUI,'UserData');
   CurrINI = get(PredictiveControls.varnames,'UserData');
   DSGEModel = get(PredictiveControls.variables,'UserData');
   maingui = findobj('Type','figure','Tag','YADA');
   controls = get(maingui,'UserData');
end;
%
%
switch selector;
   case 'init';
      PredictiveDistributionGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ZLBPredictiveDistributionDLG.m', ...
         'MenuBar','none', ...
         'PaperUnits','points', ...
         'Units','pixels', ...
         'Position',[(CurrINI.scrsz(3)-600)/2 (CurrINI.scrsz(4)-140-AddKHeight-AddActHeight)/2 600 140+AddKHeight+AddActHeight], ...
         'Tag','ZLBPredictiveDistributionDLG', ...
         'Visible','off', ...
         'Resize','off', ...
         'Name',['ZLB Predictive Distributions' DSGEModel.PriorHeader ' - ' NameStr], ...
         'NumberTitle','off', ...
         'CloseRequestFcn','ZLBPredictiveDistributionDLG done', ...
         'ToolBar','none');
      %
      if isempty(NameStr)==1;
         set(PredictiveDistributionGUI,'Name',['ZLB Predictive Distribution With ' ModelStr ' Model']);
      end;
      %
      if MatlabNumber>=7.0;
         set(PredictiveDistributionGUI,'DockControl','off');
      end;
      %
      % Outer axis
      %
      if IsConditional==1;
         AxesBox([2 2 598 130+AddKHeight+AddActHeight],'Multistep Conditional Forecasts',45,[0.5 1],'on',CurrINI);
      else;
         AxesBox([2 2 598 130+AddKHeight+AddActHeight],'Multistep Unconditional Forecasts',45,[0.5 1],'on',CurrINI);
      end;
      %
      PredictiveControls.sample = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85+AddKHeight+AddActHeight 360 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String',['Prediction Sample: ' PredStartYear ':' PredStartPeriod ' - ' PredEndYear ':' PredEndPeriod], ...
         'HorizontalAlignment','left', ...
         'Tag','SampleText');
      %
      if (strcmp(lower(AnnualStr),'original')==1)|(strcmp(lower(AnnualStr),'transformed')==1);;
         if DSGEModel.KalmanFirstObservation>1;
            [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,DSGEModel.KalmanFirstObservation-1);
         else;
            NewStartYear = DSGEModel.SubBeginYear;
            NewStartPeriod = DSGEModel.SubBeginPeriod;
         end;
      else;
         [NewStartYear,NewStartPeriod] = AdjustSampleStart(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.DataFrequency,max(DSGEModel.annual)+DSGEModel.KalmanFirstObservation-2);
      end;
      %
      OldDSGEModel = DSGEModel;
      DSGEModel.SubBeginYear = NewStartYear;
      DSGEModel.SubBeginPeriod = NewStartPeriod;
      [FirstPeriod,LastPeriod,SampleStrMat] = CreateSubSample(DSGEModel);
      DSGEModel = OldDSGEModel;
      %
      % check if we need to change the sample from the top
      %
      if strcmp(lower(AnnualStr),'transformed')==1;
         [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
         T2 = size(YData,1);
         if DSGEModel.T-T2>FirstPeriod+DSGEModel.KalmanFirstObservation-1;
            SampleStrMat = SampleStrMat(DSGEModel.T-T2-(FirstPeriod+DSGEModel.KalmanFirstObservation-1):size(SampleStrMat,1),:);
         end;
      end;
      %
      % data sample
      %
      PredictiveControls.startdatatext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 50+AddKHeight+AddActHeight 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Start period for variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','StartDataText');
      %
      PredictiveControls.startdata = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 55+AddKHeight+AddActHeight 255 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'String',SampleStrMat, ...
         'HorizontalAlignment','center', ...
         'Tag','StartDataPopup',...
         'Value',1);
      %
      % Select variable
      %
      PredictiveControls.variables = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 15+AddKHeight+AddActHeight 180 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'UserData',DSGEModel, ...
         'String','Select variable', ...
         'HorizontalAlignment','left', ...
         'Tag','VariableNamesText');
      %
      PredictiveControls.varnames = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[210 20+AddKHeight+AddActHeight 255 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',strvcat(DSGEModel.VariableNames,DSGEModel.StateVariableNames), ...
         'HorizontalAlignment','center', ...
         'Tag','VariableNamesPopup',...
         'Value',1);
      %
      if isempty(ZLBKsiMeanPrediction)==1;
         set(PredictiveControls.varnames,'String',DSGEModel.VariableNames);
      end;
      %
      if AddActHeight>0;
         %
         % select actuals
         %
         PredictiveControls.actualstext = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 15+AddKHeight 180 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','text', ...
            'String','Select the actuals dataset', ...
            'HorizontalAlignment','left', ...
            'Tag','ActualsText');
         %
         PredictiveControls.actuals = uicontrol('Units','pixels', ...
            'BackgroundColor',[1 1 1], ...
            'Position',[210 20+AddKHeight 255 20], ...
            'Style','popupmenu', ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'UserData',CurrINI, ...
            'String',ActualsStr, ...
            'HorizontalAlignment','center', ...
            'Tag','ActualsPopup',...
            'Value',1);
      else;
         PredictiveControls.actualstext = [];
         PredictiveControls.actuals = [];
      end;
      %
      if DSGEModel.ShowKernel==1;
         PredictiveControls.ShowKernelDensity = uicontrol('Units','pixels', ...
            'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
            'Position',[24 15 441 20], ...
            'FontSize',CurrINI.gui_fontsize, ...
            'FontName',CurrINI.gui_fontname, ...
            'FontWeight',CurrINI.gui_fontweight, ...
            'FontAngle',CurrINI.gui_fontangle, ...
            'Style','checkbox', ...
            'Value',0, ...
            'String','Show kernel density estimates of marginal predictive densities', ...
            'HorizontalAlignment','left', ...
            'Tag','ShowKernelDensityCheckbox');
      else;
         PredictiveControls.ShowKernelDensity = [];
      end;
      %
      % Create graph button
      %
      PredictiveControls.show = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 55 90 20], ...
         'String','Display', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ZLBPredictiveDistributionDLG showgraph', ...
         'Enable','on', ...
         'Tag','Done');
      %
      %  Done button
      %
      PredictiveControls.done = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[495 20 90 20], ...
         'String','Done', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'CallBack','ZLBPredictiveDistributionDLG done', ...
         'Enable','on', ...
         'UserData','waiting', ...
         'Tag','Done');

      set(PredictiveDistributionGUI,'UserData',PredictiveControls,...
         'HandleVisibility','callback');
   case 'showgraph'
      VariableValue = get(PredictiveControls.varnames,'Value');
      VariableStr = get(PredictiveControls.varnames,'String');
      CurrVariable = StringTrim(VariableStr(VariableValue,:));
      if VariableValue<size(VariableStr,1);
         set(PredictiveControls.varnames,'Value',VariableValue+1);
      else;
         set(PredictiveControls.varnames,'Value',1);
      end;
      if DSGEModel.IsConditional==1;
         CondStr = 'Conditional ';
      else;
         CondStr = 'Unconditional ';
      end;
      %
      NumPredPathsStr = get(controls.forecast.predpaths,'String');
      NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
      %
      % create a vector with dates
      %
      DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
      [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
      KsiFirstPeriod = FirstPeriod;
      FirstPeriod = FirstPeriod+DSGEModel.KalmanFirstObservation+get(PredictiveControls.startdata,'Value')-2;
      %
      % create the figure
      %
      FigHandle = figure('Visible','off', ...
                     'Position',[(CurrINI.scrsz(3)-750)/2 (CurrINI.scrsz(4)-500)/2 750 500], ...
                     'Tag',[CurrVariable '-' DSGEModel.NameStr], ...
                     'Name',['ZLB ' CondStr 'Predictions' DSGEModel.PriorHeader ' - ' CurrVariable]);
      %
      SetFigureProperties(FigHandle,CurrINI);
      %
      % check which actuals data to use
      %
      if isempty(PredictiveControls.actuals)==1;
         if DSGEModel.IsCentered==0;
            YActuals = DSGEModel.Y;
         else;
            if DSGEModel.SelectedYData==1;
               YActuals = DSGEModel.Y;
            else;
               YActuals = DSGEModel.Actuals(DSGEModel.SelectedYData-1).data;
            end;
         end;
      else;
         SelData = get(PredictiveControls.actuals,'Value');
         if SelData==1;
            YActuals = DSGEModel.Y;
         else;
            YActuals = DSGEModel.Actuals(SelData-1).data;
         end;
      end;
      DSGEModel.Y = YActuals;
      %
      if VariableValue<=DSGEModel.n;
         MeanPrediction = DSGEModel.ZLBYMeanPrediction;
         Quantiles = DSGEModel.ZLBYQuantiles;
         if strcmp(lower(DSGEModel.AnnualStr),'original')==1;
            xData = DateVector(FirstPeriod:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector)));
            yData = DSGEModel.Y(VariableValue,FirstPeriod:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector)));
         elseif strcmp(lower(DSGEModel.AnnualStr),'annual')==1;
            Y = Annualize(DSGEModel.Y(:,FirstPeriod:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector))),DSGEModel.annual,DSGEModel.annualscale);
            xData = DateVector(1,max(DSGEModel.annual)+FirstPeriod-1:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector)));
            yData = Y(VariableValue,:);
         else;
            %
            % we're using the transformed data
            %
            T1 = min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector));
            OldDateVector = DateVector;
            DateVector = DateVector(FirstPeriod:T1);
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y(:,1:T1)',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            Y = YData';
            T2 = size(Y,2);
            if T2==T1;
               Y = Y(:,FirstPeriod:T1);
            else;
               if FirstPeriod-(T1-T2)>=1;
                  Y = Y(:,FirstPeriod-(T1-T2):T1);
               else;
                  DateVector = DateVector(T1-T2+1:T1);
               end;
            end;
            xData = DateVector;
            yData = Y(VariableValue,:);
         end;
         YStr = CurrVariable;
         IsObsVariable = 1;
      else;
         MeanPrediction = DSGEModel.ZLBKsiMeanPrediction;
         Quantiles = DSGEModel.ZLBKsiQuantiles;
         VariableValue = VariableValue-DSGEModel.n;
         xData = DateVector(KsiFirstPeriod:min(size(DSGEModel.Y,2),LastPeriod+length(DSGEModel.PredDateVector)));
         yData = DSGEModel.KsitT(VariableValue,KsiFirstPeriod:min(size(DSGEModel.KsitT,2),LastPeriod+length(DSGEModel.PredDateVector)));
         if length(xData)>length(yData);
            xData = xData(1:length(yData))
         end;
         YStr = ['Smoothed ' CurrVariable];
         IsObsVariable = 0;
      end;
      %
      % first we plot the predictive bounds
      %
      NumBounds = floor(length(Quantiles)/2);
      Handles = [];
      LegendStr = '';
      if NumPredPaths>1;
         for i=1:NumBounds;
            Lower = Quantiles(i).Quantile(VariableValue,:);
            Upper = Quantiles(length(Quantiles)-i+1).Quantile(VariableValue,:);
            PatchColor = ((1+(2*(i-1)))/length(Quantiles))*DSGEModel.ConfidenceBandBaseColor;
            ConfBound = Quantiles(length(Quantiles)-i+1).percent-Quantiles(i).percent;
            ConfPatch = patch([DSGEModel.PredDateVector fliplr(DSGEModel.PredDateVector)],[Upper fliplr(Lower)],PatchColor);
            set(ConfPatch,'FaceColor',PatchColor,'EdgeColor',PatchColor);
            hold('on');
            Handles = [Handles ConfPatch];
            LegendStr = strvcat(LegendStr,[num2str(ConfBound,'%0.0f') ' confidence']);
         end;
         set(gca,'Layer','Top');
      end;
      %
      % next we plot the observed variable
      %
      if isempty(xData)==0
         YHandle = plot(xData,yData,'+-','Color',[0.75 0 0],'LineWidth',2);
         hold('on');
      else;
         YHandle = [];
         YStr = '';
      end;
      %
      % set up a DateVector which does not go beyond the largest value in PredDateVector
      %
      if isempty(Handles)==0;
         PDMax = max(DSGEModel.PredDateVector);
         XData = get(YHandle,'XData');
         PDMaxPos = max((XData==PDMax).*(1:length(XData)));
         if PDMaxPos>0;
            DV = XData(1:PDMaxPos);
         else;
            DV = XData;
         end;
         [DV,Ind] = setdiff(DV,DSGEModel.PredDateVector);
         DateVec = DV(length(DV));
         Ind = Ind(length(Ind));
         YData = get(YHandle,'YData');
         YData = YData(Ind);
         %
         % we should add the position x,y = DateVec,YData to the patches
         %
         if isnan(YData)==0;
            for i=1:length(Handles);
               XD = get(Handles(i),'XData')';
               YD = get(Handles(i),'YData')';
               PatchColor = get(Handles(i),'FaceColor');
               delete(Handles(i));
               Handles(i) = patch([DateVec XD DateVec],[YData YD YData],PatchColor);
               set(Handles(i),'FaceColor',PatchColor,'EdgeColor',PatchColor);
            end;
         end;
      end;
      %
      % plot the mean prediction
      %
      PredHandle = plot(DSGEModel.PredDateVector,MeanPrediction(VariableValue,:),'--','Color',[0 0 0.75],'LineWidth',2);
      hold('on');
      %
      % check if we should also plot the mean of the observed variables
      %
      if (strcmp(lower(DSGEModel.AnnualStr),'original')==1)&(isempty(DSGEModel.ObsVarMean)==0)&(IsObsVariable==1);
         ObsVarMeanHandle = plot([DateVector(FirstPeriod) max(DSGEModel.PredDateVector)],DSGEModel.ObsVarMean(VariableValue)*ones(1,2),'.-','Color',[0 0.75 0],'LineWidth',2);
         hold('on');
         ObsVarMeanStr = [CurrVariable ' Mean'];
      else;
         ObsVarMeanHandle = [];
         ObsVarMeanStr = '';
      end;
      %
      Handles = [YHandle ObsVarMeanHandle PredHandle Handles];
      LegendStr = strvcat(YStr,ObsVarMeanStr,'Mean predictions',LegendStr);
      %
      ylim = get(gca,'YLim');
      add_on = 0.1*(ylim(2)-ylim(1));
      ylim = [ylim(1)-add_on ylim(2)+add_on];
      if (strcmp(lower(DSGEModel.AnnualStr),'original')==1)|(strcmp(lower(DSGEModel.AnnualStr),'transformed')==1);;
         set(gca,'YLim',ylim,'XLim',[DateVector(FirstPeriod) max(DSGEModel.PredDateVector)]);
      else;
         set(gca,'YLim',ylim,'XLim',[DateVector(1,max(DSGEModel.annual)+FirstPeriod-1) max(DSGEModel.PredDateVector)]);
      end;
      %
      % add a vertical line from the forecast point
      %
      fp = plot((((3*DateVector(LastPeriod))-DateVector(LastPeriod-1))/2)*[1 1],ylim,':','Color',[0 0 0]);
      hold('on');
      %
      if (ylim(1)<0)&(ylim(2)>0);
         %
         % add a zero line
         %
         zl = plot(get(gca,'XLim'),[0 0],'-','Color',[0 0 0]);
      else;
         zl = [];
      end;
      if isempty(YHandle)==0;
         if isempty(ObsVarMeanHandle)==0;
            set(gca,'Children',[PredHandle;YHandle;ObsVarMeanHandle;zl;fp;fliplr(Handles(4:length(Handles)))']);
         else;
            set(gca,'Children',[PredHandle;YHandle;zl;fp;fliplr(Handles(3:length(Handles)))']);
         end;
      else;
         if isempty(ObsVarMeanHandle)==0;
            set(gca,'Children',[PredHandle;ObsVarMeanHandle;zl;fp;fliplr(Handles(3:length(Handles)))']);
         else;
            set(gca,'Children',[PredHandle;zl;fp;fliplr(Handles(2:length(Handles)))']);
         end;
      end;
      if isempty(DSGEModel.NameStr)==0;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (' lower(DSGEModel.AnnualStr) ' data) and out-of-sample ' lower(CondStr) ' prediction paths - ' lower(DSGEModel.NameStr) ' of the parameters']);
      else;
         SetAxesFonts(CurrINI,'Title',[CurrVariable ' (' lower(DSGEModel.AnnualStr) ' data) and out-of-sample ' lower(CondStr) ' prediction paths']);
      end;
      hold('off');
      %
      % add the legend
      %
      if MatlabNumber>=7;
         legHandle = legend(Handles,LegendStr,'Location','NorthWest');
      else;
         legHandle = legend(Handles,LegendStr,2);
      end;
      SetAxesFonts(CurrINI,legHandle);
      %
      SetXTickIntegers(gca);
      %
      SetToolbarImages(FigHandle,CurrINI.images);
      %
      if (IsHandle(PredictiveControls.ShowKernelDensity)==1)&(get(PredictiveControls.ShowKernelDensity,'Value')==1);
         %
         % the prediction sample date strings are located in: DSGEModel.PredSampleStrMat
         %
         [n,NumGridPoints,h] = size(DSGEModel.KernelX);
         %
         % get the observed data and check if we can use it
         %
         y = get(YHandle,'YData');
         DateVector = get(YHandle,'XData');
         %
         % find the position in DateVector that corresponds to the first obs in PredDateVector
         %
         PredPos = (DateVector==DSGEModel.PredDateVector(1)).*(1:length(DateVector));
         PredPos = PredPos(PredPos>0);
         if isempty(PredPos)==0;
            y = y(PredPos:length(y));
            if length(y)<h;
               y = [y NaN*ones(1,h-length(y))];
            end;
         else;
            y = [];
         end;
         %
         % check how many plots we should make
         %
         NumPlots = 0;
         for i=1:h;
            if isequal(DSGEModel.KernelX(VariableValue,:,i),zeros(1,NumGridPoints))==0;
               NumPlots = NumPlots+1;
            end;
         end;
         NumFigs = ceil(NumPlots/16);
         if NumFigs==1;
            [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumPlots);
         elseif NumFigs>1;
            NumRows = 4;
            NumCols = 4;
         end;
         CurrPlot = 0;
         MadePlots = 0;
         pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
         %
         for i=1:NumFigs;
            if i>1;
               pos(1) = pos(1)+22;
               pos(2) = pos(2)-22;
               if pos(1)+1000>CurrINI.scrsz(3);
                  pos(1) = 0;
               end;
               if pos(2)<=32;
                  pos(2) = CurrINI.scrsz(4)-700;
               end;
            end;
            FigHandles{i} = figure('Visible','off','Position',pos);
            if NumFigs==1;
               set(FigHandles{i},'Name',['Marginal Predictive Densities' DSGEModel.PriorHeader ' - ' CurrVariable]);
            else;
               set(FigHandles{i},'Name',['Marginal Predictive Densities' DSGEModel.PriorHeader ' - Figure ' num2str(i,'%0.0f') ' - ' CurrVariable]);
            end;
            SetFigureProperties(FigHandles{i},CurrINI);
            %
            k = 0;
            DataHandle = [];
            DataStr = '';
            for r=1:NumRows;
               for c=1:NumCols;
                  if MadePlots<NumPlots;
                     CurrPlot = CurrPlot+1;
                     while isequal(DSGEModel.KernelX(VariableValue,:,CurrPlot),zeros(1,NumGridPoints))==1;
                        CurrPlot = CurrPlot+1;
                     end;
                     k = k+1;
                     MadePlots = MadePlots+1;
                     %
                     % ensure that the handle to the current figure is correct (HG2 issue)
                     %
                     set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
                     subplot(NumRows,NumCols,k);
                     %
                     % check if we have a normal approximation available for the predictive density
                     %
                     if (isfield(DSGEModel,'UData')==1)&(isempty(DSGEModel.UData)==0)&(isfield(DSGEModel.UData,'KernelX')==1);
                        NormalApproxHandle = plot(DSGEModel.UData.KernelX(VariableValue,:,CurrPlot),DSGEModel.UData.KernelY(VariableValue,:,CurrPlot),':','Color',[0 0.5 1],'LineWidth',2);
                        hold('on');
                        NormalApproxStr = 'Normal approx.';
                     else;
                        NormalApproxHandle = [];
                        NormalApproxStr = '';
                     end;
                     %
                     % graph the predictive density
                     %
                     PredDensityHandle = plot(DSGEModel.KernelX(VariableValue,:,CurrPlot),DSGEModel.KernelY(VariableValue,:,CurrPlot),'-','Color',[0 0.25 0],'LineWidth',2);
                     hold('on');
                     %
                     SetAxesFonts(CurrINI,'Title',[CurrVariable ' - ' DSGEModel.PredSampleStrMat(CurrPlot,:)]);
                     %
                     % deal with axis limits
                     %
                     axis('tight');
                     xlim = [min(DSGEModel.KernelX(VariableValue,:,CurrPlot)) max(DSGEModel.KernelX(VariableValue,:,CurrPlot))];
                     set(gca,'XLim',xlim);
                     %
                     ylim = [0 1.2*max(DSGEModel.KernelY(VariableValue,:,CurrPlot))];
                     %
                     % display the mean prediction
                     %
                     if (isempty(y)==0)&(isnan(y(CurrPlot))==0);
                        DataHandle = plot(y(CurrPlot)*ones(1,2),ylim,'-.','Color',[0.75 0 0],'LineWidth',2);
                        hold('on');
                        DataStr = 'Observation';
                     end;
                     MeanPredHandle = plot(DSGEModel.ZLBYMeanPrediction(VariableValue,CurrPlot)*ones(1,2),ylim,'--','Color',[0 0 0.75],'LineWidth',2);
                     MeanStr = 'Mean';
                     %
                     x_lim = get(gca,'XLim');
                     add_x = 0.1*(xlim(2)-xlim(1));
                     x_lim = [xlim(1)-add_x xlim(2)+add_x];
                     %
                     set(gca,'XLim',x_lim,'YLim',ylim);
                     hold('off');
                  end;
               end;
            end;
            legHandle = legend([PredDensityHandle NormalApproxHandle MeanPredHandle DataHandle],strvcat('Density',NormalApproxStr,MeanStr,DataStr));
            %
            SetAxesFonts(CurrINI,legHandle);
            %
            % transform units temporarily into pixels
            %
            OldAxesUnits = get(gca,'Units');
            OldLegUnits = get(legHandle,'Units');
            set(gca,'Units','pixels');
            set(legHandle,'Units','pixels');
            %
            % check if we can move legend
            %
            LegPos = get(legHandle,'Position');
            AxPos = get(gca,'Position');
            if LegPos(1)<AxPos(1)+AxPos(3);
               if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
                  LegPos(1) = AxPos(1)+AxPos(3)+10;
                  set(legHandle,'Position',LegPos);
               end;
            end;
            set(gca,'Units',OldAxesUnits);
            set(legHandle,'Units',OldLegUnits);
            %
            % Fix the toolbar cdata
            %
            SetToolbarImages(FigHandles{i},CurrINI.images);
            %
            % Display the current figure
            %
            set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
            drawnow;
            pause(0.02);
         end;
      end;
      %
      set(FigHandle,'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
      drawnow;
      pause(0.02);
   case 'done'
      set(PredictiveControls.done,'UserData','done');
      delete(PredictiveDistributionGUI);
      drawnow;
      pause(0.02);
end;
%
if strcmp(selector,'init')==1;
   set(PredictiveDistributionGUI,'Visible','on');
   drawnow;
   SetYADAIcon(PredictiveDistributionGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(PredictiveControls.done);
   end;
   waitfor(PredictiveControls.done,'UserData','done');
end;

%
% end of ZLBPredictiveDistributionDLG.m
%
