function [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG(selector,StateVariableNames,StateEquationNames,DataFrequency,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT,CurrINI)
% ZLBSelectionDLG: Dialog for setting the zero lower bound data.
%
% USAGE:
%
%       [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG(selector,StateVariableNames,StateEquationNames,DataFrequency, ...
%                                                                                    RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT,CurrINI)
%
% REQUIRED INPUT:  selector (string vector), with the possible values "init", "OK", and "Cancel"
%
% OPTIONAL INPUT:  StateVariableNames (string matrix) with the names of the state variables.
%
%                  StateEquationNames (string matrix) with the names of the state equations.
%
%                  DataFrequency (string) with the values q for quarterly data, m for monthly or a.
%
%                  RtildePosition (integer) with the position of the monetary policy interest rate.
%
%                  REqPosition (integer) with the position of the monetary policy rule.
%
%                  ZLBRuleType (integer) which is either 1 for lagged unrestricted policy rate (the
%                              default) or 2 for lagged restricted policy rate.
%
%                  ZLBSampleT (integer) with the length of the sample over which the zero lower
%                              bound may be binding.
%
%                  CurrINI (structure) with initialization information.
%
% OPTIONAL OUTPUT: action (string vector) taking the value "OK" or "Cancel". Depends on which button was
%                             clicked on.
%
%                  RtildePosition (integer) with the position of the monetary policy interest rate.
%
%                  REqPosition (integer) with the position of the monetary policy rule.
%
%                  ZLBRuleType (integer) which is either 1 for lagged unrestricted policy rate (the
%                              default) or 2 for lagged restricted policy rate.
%
%                  ZLBSampleT (integer) with the length of the sample over which the zero lower
%                              bound may be binding.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 18, 2014.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-08-2014: Updated the documentation.
%
% * 26-08-2014: Added the input and output variable "REqPosition" and the input variable "StateEquationNames".
%
% * 04-09-2014: Added the input and output variable "AdditionalDraws".
%
% * 11-09-2014: Changed the field "RhatPosition" to "RtildePosition".
%
% * 23-10-2014: Increased the number of additional draws from 100% to 1000%.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 20-01-2016: Updated the documentation.
%
% * 10-02-2017: Updated the documentation.
%
% * 05-01-2018: Updated the documentation.
%
% * 18-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 29-08-2022: Allow for the case that AdditionalDraws is empty and the corresponding
%               controls should not be shown.
%
% * 02-01-2023: Updated the documentation.
%
% * 17-02-2023: Removed all use of "AdditionalDraws".
%
% * 19-06-2023: The dialog can now display the YADA alarm clock icon.
%
% * 22-06-2023: Updated the documentation.
%
% * 23-11-2023: The function now supports the input and output variable "ZLBRuleType".
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% intialize temporarily
%
if strcmp(selector,'init')==1;
   action = 'OK';
   SV.CurrINI = CurrINI;
   SV.RtildePosition = RtildePosition;
   SV.REqPosition = REqPosition;
   SV.ZLBRuleType = ZLBRuleType;
   SV.ZLBSampleT = ZLBSampleT;
   if isempty(RtildePosition)==1;
      RtildePosition = 1;
   end;
   if isempty(REqPosition)==1;
      REqPosition = 1;
   end;
   scrsz = CurrINI.scrsz;
else;
   ZLBSelectionGUI = findobj('Type','figure','Tag','ZLBSelectionDLG');
   controls = get(ZLBSelectionGUI,'UserData');
   SV = get(controls.cancel,'UserData');
   CurrINI = SV.CurrINI;
end;
%
switch selector
   case 'init'
      ZLBSelectionGUI = figure('Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'FileName','ZLBSelectionDLG.m', ...
         'MenuBar','none', ...
         'Units','pixels', ...
         'Position',[(scrsz(3)-455)/2 (scrsz(4)-170)/2 455 200], ...
         'Tag','ZLBSelectionDLG', ...
         'Resize','off', ...
         'Visible','off', ...
         'Name','Zero Lower Bound Data', ...
         'NumberTitle','off', ...
         'WindowStyle','modal', ...
         'CloseRequestFcn','ZLBSelectionDLG quit', ...
         'ToolBar','none');
      %
      if MatlabNumber>=7.0;
         set(ZLBSelectionGUI,'DockControl','off');
      end;
      %
      AxesBox([2 2 453 190],'Zero Lower Bound Selection',100,[0.5 1],'on',CurrINI);
      %
      controls.rhatpositiontext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 145 250 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select the monetary policy rate variable:', ...
         'HorizontalAlignment','left', ...
         'Tag','RhatPositionText');
      %
      controls.rhatpositionpopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[280 150 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateVariableNames, ...
         'HorizontalAlignment','center', ...
         'Tag','RhatPositionPopup',...
         'Value',RtildePosition);
      %
      controls.reqpositiontext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 115 250 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select the monetary policy equation name:', ...
         'HorizontalAlignment','left', ...
         'Tag','REqPositionText');
      %
      controls.reqpositionpopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[280 120 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',StateEquationNames, ...
         'HorizontalAlignment','center', ...
         'Tag','REqPositionPopup',...
         'Value',REqPosition);
      %
      % select if the lagged unrestricted policy rate or the restricted policy rate appears in the
      % policy rule ZLBRuleType
      %
      controls.zlbruletypetext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 85 250 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select lagged policy rate:', ...
         'HorizontalAlignment','left', ...
         'Tag','ZLBRuleTypeText');
      %
      controls.zlbruletypepopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[280 90 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String','Unrestricted policy rate|Restricted policy rate', ...
         'HorizontalAlignment','center', ...
         'Tag','ZLBRuleTypePopup',...
         'Value',ZLBRuleType);
      %
      % Select the length of the ZLB sample
      %
      if strcmp(DataFrequency,'q')==1;
         FreqStr = 'quarter';
      elseif strcmp(DataFrequency,'m')==1;
         FreqStr = 'month';
      else;
         FreqStr = 'year';
      end;
      ZLBSampleTxt = ['1 ' FreqStr];
      for i=2:1000;
         ZLBSampleTxt = strvcat(ZLBSampleTxt,[int2str(i) ' ' FreqStr 's']);
      end;
      %
      controls.zlbsampletext = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[24 55 250 20], ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Style','text', ...
         'String','Select zero lower bound sample length:', ...
         'HorizontalAlignment','left', ...
         'Tag','ZLBSampleText');
      %
      controls.zlbsamplepopup = uicontrol('Units','pixels', ...
         'BackgroundColor',[1 1 1], ...
         'Position',[280 60 155 20], ...
         'Style','popupmenu', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'UserData',CurrINI, ...
         'String',ZLBSampleTxt, ...
         'HorizontalAlignment','center', ...
         'Tag','ZLBSamplePopup',...
         'Value',ZLBSampleT);
      %
      controls.ok = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[280 20 75 25], ...
         'String','OK', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'Callback','ZLBSelectionDLG quit', ...
         'UserData','waiting', ...
         'Tag','ButtonOK');
      %
      controls.cancel = uicontrol('Units','pixels', ...
         'BackgroundColor',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'), ...
         'Position',[360 20 75 25], ...
         'String','Cancel', ...
         'FontSize',CurrINI.gui_fontsize, ...
         'FontName',CurrINI.gui_fontname, ...
         'FontWeight',CurrINI.gui_fontweight, ...
         'FontAngle',CurrINI.gui_fontangle, ...
         'Enable','on', ...
         'UserData',SV, ...
         'Callback','ZLBSelectionDLG quit', ...
         'Tag','ButtonCancel');
      %
      set(ZLBSelectionGUI,'UserData',controls);
   case 'quit'
      set(controls.ok,'UserData','done');
end;

if strcmp(selector,'init')==1;
   set(ZLBSelectionGUI,'Visible','on');
   drawnow;
   SetYADAIcon(ZLBSelectionGUI,CurrINI.ShowYADAIcon);
   if MatlabNumber>=7;
      uicontrol(controls.ok);
   end;
   %
   waitfor(controls.ok,'UserData','done');
   %
   % check here if the user really wants to make the selection
   %
   if (isempty(get(ZLBSelectionGUI,'CurrentObject'))==1)|(strcmp(get(get(ZLBSelectionGUI,'CurrentObject'),'Type'),'uicontrol')==0);
      action = 'Cancel';
   else;
      action = get(get(ZLBSelectionGUI,'CurrentObject'),'String');
      if (isempty(action)==1)|(strcmp(action,'OK')==0);
         action = 'Cancel';
      end;
   end;
   SV = get(controls.cancel,'UserData');
   if strcmp(lower(action),'ok')==1;
      RtildePosition = get(controls.rhatpositionpopup,'Value');
      REqPosition = get(controls.reqpositionpopup,'Value');
      ZLBSampleT = get(controls.zlbsamplepopup,'Value');
      ZLBRuleType = get(controls.zlbruletypepopup,'Value');
   else;
      RtildePosition = SV.RtildePosition;
      REqPosition = SV.REqPosition;
      ZLBRuleType = SV.ZLBRuleType;
      ZLBSampleT = SV.ZLBSampleT;
   end;
   %
   delete(ZLBSelectionGUI);
   drawnow;
   pause(0.02);
end;

%
% end of ZLBSelectionDLG.m
%