function GroupColors = CreateDefaultColors(NumColors)
% CreateDefaultColors: Creates default colors based on the default colormap (jet)
%
% USAGE:
%
%       GroupColors = CreateDefaultColors(NumColors)
%
% REQUIRED INPUT:  NumColors (positive integer) with the number of colors.
%
% REQUIRED OUTPUT: GroupColors (NumColorsx3) matrix with color numbers, i.e., numbers between 0 and 1.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: January 5, 2007.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-03-2007: Updated the documentation.
%
% * 09-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 19-12-2008: Updated the documentation.
%
% * 02-05-2009: Updated the documentation.
%
% * 06-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 03-01-2012: Updated the documentation.
%
% * 02-01-2013: Updated the documentation.
%
% * 24-01-2014: Updated the documentation.
%
% * 05-01-2015: Updated the documentation.
%
% * 08-09-2015: Began work on taking HG2 into account.
%
% * 09-09-2015: Finished the code revision on account of HG2.
%
% * 20-01-2016: Updated the documentation.
%
% * 23-03-2016: Fixed a bug with using the ceil function for the "Cols" variable for
%               matlab's HG2 graphics.
%
% * 10-02-2017: Updated the documentation.
%
% * 08-01-2018: Updated the documentation.
%
% * 21-01-2019: Updated the documentation.
%
% * 07-01-2020: Updated the documentation.
%
% * 06-04-2020: Changed the colormap to jet under matlab versions after 8.3 and the
%               selection of colors is based on longest possible distance between rows
%               in the colormap matrix.
%
% * 25-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 03-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

GroupColors = zeros(NumColors,3);
if MatlabNumber>8.3
   ColMap = jet;
   sc = size(ColMap,1);
   if sc>NumColors;
      if NumColors>2;
         PossPositions = (2:sc-1);
         Distance = floor((sc-2)/(NumColors-1));
         Positions = (1+Distance:Distance:sc-1);
         if length(Positions)>NumColors-2;
            Positions = Positions(1:NumColors-2);
         elseif length(Positions)<NumColors-2;
            Distance = floor((sc-2)/(NumColors));
            Positions = (1+Distance:Distance:sc-1);
            Positions = Positions(1:NumColors-2);
         end;
         Positions = [1 Positions sc];
      elseif NumColors==2;
         Positions = [1 sc];
      else;
         Positions = sc;
      end;
      GroupColors = ColMap(Positions,:);
   elseif sc==NumColors;
      GroupColors = ColMap;
   else;
      %
      % we pick rand elements from the colormap
      %
      ColMap = jet;
      sc = size(ColMap,1);
      st = rand('state');
      rand('state',0);
      for i=1:NumColors;
         Rows = ceil(sc*rand(3,1));
         Cols = ceil(3*rand(3,1));
         for j=1:3;
            GroupColors(i,j) = ColMap(Rows(j),Cols(j));
         end;
      end;
      rand('state',st);
   end;
   return;
end;
%
% create a figure
%
fig = figure('Visible','off','Position',[0 0 800 600]);
DataVec = ones(NumColors*2,1)*(1:NumColors);
if MatlabNumber>=7.0;
   %
   % make sure the bars are written as patches
   %
   BarHandles = bar('v6',DataVec,'stacked');
else;
   BarHandles = bar(DataVec,'stacked');
end;
%
for i=1:length(BarHandles);
   if isnumeric(get(BarHandles(i),'FaceColor'))==1;
      GroupColors(i,:) = get(BarHandles(i),'FaceColor');
   else;
      %
      % colors are defined by CData, CLim
      %
      IntValue = get(BarHandles(i),'CData');
      clim = caxis;
      CurrMap = colormap;
      if strcmp(get(BarHandles(i),'CDataMapping'),'scaled')==1;
         %
         % the CDataMapping is scaled (this should be the default)
         %
         index = min(fix((IntValue(1,1)-clim(1))/(clim(2)-clim(1))*length(CurrMap))+1,length(CurrMap));
      else;
         %
         % the CDataMazpping is direct
         %
         index = min(IntValue(1,1),length(CurrMap));
      end;
      GroupColors(i,:) = CurrMap(index,:);
   end;
end;
close(fig);

%
% end of CreateDefaultColors.m
%
