function [InformationMatrix,BeliefStatus,TransStatus,status] = AdaptiveLearningDSGEInformationMatrix(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI)
% AdaptiveLearningDSGEInformationMatrix: Computes Fisher's information matrix for the DSGE model subject
%                                        to adaptive learning.
%
% USAGE:
%
%       [InformationMatrix,BeliefStatus,TransStatus,status] = ...
%                       AdaptiveLearningDSGEInformationMatrix(theta,thetaPositions,ModelParameters,...
%                                                                 ParameterNames,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  ParameterNames (string matrix) with the names of the parameters.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: InformationMatrix (matrix) of the same dimension as theta.
%
% OPTIONAL OUTPUT: BeliefStatus (boolean) which is 1 if the Kalman filter of the belief system runs without
%                        error, and 0 otherwise.
%
%                  TransStatus (boolean) which takes the value 1 if the rewrite of the structural form of
%                        the DSGE model was successful and 0 otherwise.
%
%                  status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2023 European Central Bank.
%
%                       First version: February 13, 2023.
%                        This version: March 13, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 14-02-2023: Made sure that ModelParameters is updated with the new parameters by taking
%               it as an output from SolveDSGEModel when looping over the parameters.
%
% * 24-10-2023: The function now supports the fixed belief Kalman filter.
%
% * 13-03-2024: The logical function "CheckAdaptiveLearningFixedBeliefs" is now only called once.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
InformationMatrix = [];
if nargout>1;
   BeliefStatus = 1;
end;
if nargout>2;
   TransStatus = 1;
end;
%
if nargout<=3;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 1;
end;
%
% check if we should also copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>3;
      status = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout<=3;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
         end;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
         end;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   end;
end;
if nargout>3;
   status = mcode;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% obtain the measurement equation matrices
%
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch ME;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
if length(size(H))==3;
   if size(H,3)<size(DSGEModel.Y,2);
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(size(DSGEModel.Y,2)) '). YADA has therefore aborted from the calculations.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% call the function: 
%
[tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if nargout>2;
   TransStatus = tstatus;
   if TransStatus==0;
      return;
   end;
end;
%
% get estimation sample
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if DSGEModel.k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,BeliefSys.m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
%
% simplify integers
%
m = BeliefSys.m;
k = DSGEModel.k;
%
% call the joint Kalman filter for state variables, belief coefficients and
% solution matrices
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      X = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(X,2);
      for j=1:k;
         ConstTest = (DSGEModel.X(j,FirstPeriod:LastPeriod)==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% compute the log-likelihood, forecast and update estimates of states, belief coefficients
% and solution matrices
%
tic;
if MatlabNumber>=8.0;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
      KalmanTime = toc;
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      KalmanTime = toc;
      return;
   end;
else;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
      KalmanTime = toc;
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
if nargout>1;
   BeliefStatus = bstatus;
   if BeliefStatus==0;
      return;
   end;
end;
%
% get the default values of ytilde and B
%
Y = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
[n,T] = size(Y);
p = length(theta);
Ytilde = Y-Yhat;
B = MSEY;
vecB = zeros(n*n,T);
for t=1:T;
   vecB(:,t) = vec(MSEY(:,:,t));
end;
%
% check what kind of changes to make for the parameters
%
thetaChange = 0.001*abs(theta);
thetaAltChange = 0.0001*abs(theta);
IdentMat = eye(p);
dYtilde = zeros(n,p,T);
dvecB = zeros(n*n,p,T);
StopCalculation = 0;
%
% loop over all the parameters
%
for NumParam=1:p;
   %
   % create a new parameter vector
   %
   ParamDiff = thetaChange(NumParam);
   thetaNew = theta+(ParamDiff*IdentMat(:,NumParam));
   %
   % try to solve the model at this value
   %
   [A,H,R,F,B0,mcode,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   if mcode~=1;
      %
      % try the alternative 
      %
      ParamDiff = thetaAltChange(NumParam);
      thetaNew = theta+(ParamDiff*IdentMat(:,NumParam));
      [A,H,R,F,B0,mcode,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaNew,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   end;
   if mcode==1;
      %
      % get the structural form matrices
      %
      [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
      %
      % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
      %
      [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
      %
      % call the transformation fucntion
      %
      [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
      %
      % compute initial values for the solution matrices
      %
      if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
         [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
      else;
         [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
      %
      % run the Kalman filter
      %
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
      %
      % add values to dYtilde
      %
      dYtilde(:,NumParam,:) = (1/ParamDiff)*((Y-Yhat)-Ytilde);
      vecBNew = zeros(n*n,T);
      for t=1:T;
         vecBNew(:,t) = vec(MSEY(:,:,t));
      end;
      dvecB(:,NumParam,:) = (1/ParamDiff)*(vecBNew-vecB);
   else;
      StopCalculation = 1;
      txt = ['YADA was unable to compute the numerical partial derivatives of the log-likelihood function since the DSGE model couldn''t be solved for the tested new values of the "' StringTrim(ParameterNames(NumParam,:)) '" parameter.'];
      About(ErrorStr,'information','YADA - Bad Numerical Partial Derivatives',500,600,CurrINI);
      break;
   end;
end;
%
% check if the numerical partial derivatives could be calculated.
%
if StopCalculation==1;
   return;
end;
%
% we can now compute the information matrix
%
InformationMatrix = zeros(p,p);
if DSGEModel.YNaNs==0;
   for t=1:T;
      dYtildet = dYtilde(:,:,t);
      invBt = B(:,:,t)\eye(n);
      dvecBt = dvecB(:,:,t);
      dYtildePart = dYtildet'*invBt*dYtildet;
      dvecBpart = (1/2)*dvecBt'*kron(invBt,invBt)*dvecBt;
      InformationMatrix = InformationMatrix+dYtildePart+dvecBpart;
   end;
else;
   for t=1:T;
      pos = (~isnan(Y(:,t)));
      n1 = sum(pos);
      if n1>0;
         dYtildet = dYtilde(pos,:,t);
         invBt = B(pos,pos,t)\eye(n1);
         dvecBt = dvecB(:,:,t);
         if n1<n;
            dvecBt = dvecBt(logical(kron(pos,pos)),:);
         end;
         dYtildePart = dYtildet'*invBt*dYtildet;
         dvecBpart = (1/2)*dvecBt'*kron(invBt,invBt)*dvecBt;
         InformationMatrix = InformationMatrix+dYtildePart+dvecBpart;
      end;
   end;
end;

%
% end of AdaptiveLearningDSGEInformationMatrix.m
%
