function [Xdata,Ydata,BoundsMatrix] = AdaptiveLearningGraphPriorDensity(theta,thetaDist,LowerBound,PriorDist,ParameterNames,thetaPositions,ModelParameters,controls,DSGEModel,CurrINI)
% AdaptiveLearningGraphPriorDensity: Creates plots of the prior densities for all original parameters. When the two output
%                                    arguments are listed, then the function simply resturns matrices with x-axis and y-axis
%                                    values for the densities. Applies to the adaptive learning solution of the DSGE model.
%
% USAGE:
%
%       [Xdata,Ydata,BoundsMatrix] = AdaptiveLearningGraphPriorDensity(theta,thetaDist,LowerBound,PriorDist,ParameterNames,...
%                                                                thetaPositions,ModelParameters,controls,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector), with initial values of the parameters.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  LowerBound (vector) with lower bound values.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                            "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each
%                            field returns a string matrix with the parameter names.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                           of the model parameter in position i of theta. The field "distribution" has a string
%                           vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                           DSGE model.
%
%                  controls (structure), whose fields provide handles to all the UI controls in YADA's main dialog.
%
%                  DSGEModel (structure) whose fields contain all required information about the model,
%                            such as input files, variable and sample data.
%
%                  CurrINI (structure), with initialization information.
%
% OPTIONAL OUTPUT: Xdata (matrix) with dimension NumParamxNumPoints. It contains the x-axis data for all
%                            the densities.
%
%                  Ydata (matrix) with dimension NumParamxNumPoints. It contains the y-axis data for all
%                            the densities.
%
%                  BoundsMatrix (matrix) with dimension NumParamx2. It contains information about possible
%                            upper and lower bounds of the parameters.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 19, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 20-05-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% First calculate the values to plot
%
NumGridPointsStr = get(controls.optimize.numbergridpoints,'String');
NumGridPoints = str2num(StringTrim(NumGridPointsStr(DSGEModel.NumberOfGridPoints,:)));
NumGridPointsHalf = NumGridPoints/2;
NumParam = length(theta);
if DSGEModel.PriorKernel==0;
   thetaValues = zeros(NumParam,NumGridPoints+1);
   DensityValue = thetaValues;
else;
   KernelDensityX = zeros(NumParam,512);
   KernelDensityY = KernelDensityX;
end;
%
n_beta = 0;
n_gamma = 0;
n_normal = 0;
n_invgamma = 0;
n_truncnormal = 0;
n_uniform = 0;
n_student = 0;
n_cauchy = 0;
n_logistic = 0;
n_gumbel = 0;
n_pareto = 0;
DistNames = '';
BoundMatrix = zeros(NumParam,2);
%
% check if we are using random draws from the prior
%
if DSGEModel.PriorKernel==1;
   if DSGEModel.RandomNumberValue==1;
      rand('state',0);
      randn('state',0);
   else;
      rand('state',sum(100*clock));
      randn('state',sum(100*clock));
   end;
end;
%
% lower bounds for the gamma and inverted gamma distributions
%
for i=1:NumParam;
   %
   % Determine the distribution
   %
   if thetaDist(i)==1;
      %
      % gamma dist
      %
      n_gamma = n_gamma+1;
      %
      % determine the grid
      %
      p_std = PriorDist.gamma(n_gamma,2);
      lb = PriorDist.gamma(n_gamma,1)-(DSGEModel.GridWidth*p_std);
      dev = (PriorDist.gamma(n_gamma,1)-lb)/NumGridPointsHalf;
      if lb<=LowerBound(i);
         lb = 1.0e-004+LowerBound(i);
      end;
      if DSGEModel.PriorKernel==0;
         thetaVals = (lb:dev:lb+(NumGridPoints*dev));
         logDensityVals = logGammaPDF(thetaVals'-LowerBound(i),PriorDist.gamma_ab(n_gamma,1),PriorDist.gamma_ab(n_gamma,2));
      else;
         z = LowerBound(i)+GammaRndFcn(PriorDist.gamma_ab(n_gamma,1),PriorDist.gamma_ab(n_gamma,2),5000);
      end;
      BoundMatrix(i,:) = [LowerBound(i) Inf];
      DistNames = strvcat(DistNames,'gamma');
   elseif thetaDist(i)==2;
      %
      % beta dist
      %
      n_beta = n_beta+1;
      if size(PriorDist.beta,2)==4;
         %
         % get the lower and upper bounds (if available from PriorDist.)
         %
         c = PriorDist.beta(n_beta,3);
         d = PriorDist.beta(n_beta,4);
      else;
         c = 0;
         d = 1;
      end;
      p_std = PriorDist.beta(n_beta,2);
      lb = PriorDist.beta(n_beta,1)-(DSGEModel.GridWidth*p_std);
      dev = (PriorDist.beta(n_beta,1)-lb)/NumGridPointsHalf;
      if lb<=c;
         lb = c+1.0e-004;
      end;
      if DSGEModel.PriorKernel==0;
         thetaVals = (lb:dev:lb+(NumGridPoints*dev));
         thetaVals(thetaVals>=d) = d-1.0e-004;
         logDensityVals = logBetaPDF(thetaVals',PriorDist.beta_ab(n_beta,1),PriorDist.beta_ab(n_beta,2),c,d);
      else;
         z = BetaRndFcn(PriorDist.beta_ab(n_beta,1),PriorDist.beta_ab(n_beta,2),c,d,5000);
      end;
      BoundMatrix(i,:) = [c d];
      DistNames = strvcat(DistNames,'beta');
   elseif thetaDist(i)==3;
      %
      % normal
      %
      n_normal = n_normal+1;
      p_std = PriorDist.normal(n_normal,2);
      lb = PriorDist.normal(n_normal,1)-(DSGEModel.GridWidth*p_std);
      dev = (PriorDist.normal(n_normal,1)-lb)/NumGridPointsHalf;
      thetaVals = (lb:dev:lb+(NumGridPoints*dev));
      logDensityVals = logNormalPDF(thetaVals',PriorDist.normal(n_normal,1),PriorDist.normal(n_normal,2));
      if DSGEModel.PriorKernel==1;
         z = NormalRndFcn(PriorDist.normal(n_normal,1),PriorDist.normal(n_normal,2),5000);
      end;
      BoundMatrix(i,:) = [-Inf Inf];
      DistNames = strvcat(DistNames,'normal');
   elseif thetaDist(i)==4;
      %
      % inverted gamma
      %
      n_invgamma = n_invgamma+1;
      p_par1 = PriorDist.invgamma(n_invgamma,1);
      p_par2 = PriorDist.invgamma(n_invgamma,2);
      if p_par2>2;
         p_mean = gammaln((p_par2-1)/2)-gammaln(p_par2/2);
         p_mean = exp(p_mean)*sqrt(p_par2/2)*p_par1;
         p_std = sqrt(((p_par2*p_par1*p_par1)/(p_par2-2))-(p_mean^2));
      else;
         p_mean = gammaln(1)-gammaln(3/2);
         p_mean = exp(p_mean)*sqrt(3/2)*p_par1;
         p_std = sqrt(((3*p_par1*p_par1))-(p_mean^2));
      end;
      lb = p_par1-(DSGEModel.GridWidth*p_std);
      dev = (p_par1-lb)/NumGridPointsHalf;
      if lb<=LowerBound(i);
         lb = 1.0e-004+LowerBound(i);
      end;
      if DSGEModel.PriorKernel==0;
         thetaVals = (lb:dev:lb+(NumGridPoints*dev));
         logDensityVals = logInvertedGammaPDF(thetaVals'-LowerBound(i),p_par1-LowerBound(i),p_par2);
      else;
         z = LowerBound(i)+InvGammaRndFcn(p_par1-LowerBound(i),p_par2,5000);
      end;
      BoundMatrix(i,:) = [LowerBound(i) Inf];
      DistNames = strvcat(DistNames,'inv gamma');
   elseif thetaDist(i)==5;
      %
      % left truncated normal
      %
      n_truncnormal = n_truncnormal+1;
      p_std = PriorDist.truncnormal(n_truncnormal,2);
      lb = theta(i)-(DSGEModel.GridWidth*p_std);
      dev = abs(PriorDist.truncnormal(n_truncnormal,1)-lb)/NumGridPointsHalf;
%      if lb<=PriorDist.truncnormal(n_truncnormal,3);
%         lb = PriorDist.truncnormal(n_truncnormal,3)+1.0e-008;
%      end;
      if DSGEModel.PriorKernel==0;
%         thetaVals = (lb:dev:lb+(NumGridPoints*dev));
         thetaVals = (max(lb,PriorDist.truncnormal(n_truncnormal,3)+1.0e-008):dev:max(lb,PriorDist.truncnormal(n_truncnormal,3)+1.0e-008)+(NumGridPoints*dev));
%         if length(thetaVals)>size(thetaValues,2);
%            thetaVals = thetaVals(1:size(thetaValues,2));
%         elseif length(thetaVals)<size(thetaValues,2);
%            thetaVals = [thetaVals (max(thetaVals)+dev:dev:max(thetaVals)+(dev*(size(thetaValues,2)-length(thetaVals))))];
%         end;
         logDensityVals = logLTNormalPDF(thetaVals',PriorDist.truncnormal(n_truncnormal,1),PriorDist.truncnormal(n_truncnormal,2),PriorDist.truncnormal(n_truncnormal,3));
      else;
         z = LTNormalRndFcn(PriorDist.truncnormal(n_truncnormal,1),PriorDist.truncnormal(n_truncnormal,2),PriorDist.truncnormal(n_truncnormal,3),5000);
      end;
      BoundMatrix(i,:) = [PriorDist.truncnormal(n_truncnormal,3) Inf];
      DistNames = strvcat(DistNames,'trunc normal');
   elseif thetaDist(i)==6;
      %
      % uniform
      %
      n_uniform = n_uniform+1;
      p_par1 = PriorDist.uniform(n_uniform,1);
      p_par2 = PriorDist.uniform(n_uniform,2);
      p_std = sqrt((p_par2-p_par1)/12);
      mid_point = (p_par2+p_par1)/2;
      lb = mid_point-(DSGEModel.GridWidth*p_std);
      dev = (mid_point-lb)/NumGridPointsHalf;
      if lb<p_par1;
         lb = p_par1;
      end;
      if DSGEModel.PriorKernel==0;
         thetaVals = (lb:dev:lb+(NumGridPoints*dev));
         thetaVals(thetaVals>p_par2) = p_par2;
         logDensityVals = logUniformPDF(p_par1,p_par2)*ones(length(thetaVals),1);
      else;
         z = UniformRndFcn(PriorDist.uniform(n_uniform,1),PriorDist.uniform(n_uniform,2),5000);
      end;
      BoundMatrix(i,:) = [p_par1 p_par2];
      DistNames = strvcat(DistNames,'uniform');
   elseif thetaDist(i)==7;
      %
      % Student-t
      %
      n_student = n_student+1;
      p_std = PriorDist.student(n_student,2);
      lb = PriorDist.student(n_student,1)-(DSGEModel.GridWidth*p_std);
      dev = (PriorDist.student(n_student,1)-lb)/NumGridPointsHalf;
      thetaVals = (lb:dev:lb+(NumGridPoints*dev));
      logDensityVals = logStudentTAltPDF(thetaVals',PriorDist.student(n_student,1),PriorDist.student(n_student,2),PriorDist.student(n_student,3));
      if DSGEModel.PriorKernel==1;
         Numz = 0;
         Oldz = [];
         lb = PriorDist.student(n_student,1)-(6*PriorDist.student(n_student,2));
         ub = PriorDist.student(n_student,1)+(6*PriorDist.student(n_student,2));
         while Numz<5000;
            z = StudentTAltRndFcn(PriorDist.student(n_student,1),PriorDist.student(n_student,2),PriorDist.student(n_student,3),10000);
            %
            % remove value of z that are too big or too small
            %
            if sum(z>=lb)<length(z);
               z = z(z>=lb);
            end;
            if sum(z<=ub)<length(z);
               z = z(z<=ub);
            end;
            Oldz = [Oldz z];
            z = Oldz;
            if length(z)>5000;
               z = z(1:5000);
            end;
            Numz = length(z);
         end;
      end;
      BoundMatrix(i,:) = [-Inf Inf];
      DistNames = strvcat(DistNames,'student-t');
   elseif thetaDist(i)==8;
      %
      % Cauchy
      %
      n_cauchy = n_cauchy+1;
      p_std = PriorDist.cauchy(n_cauchy,2);
      lb = PriorDist.cauchy(n_cauchy,1)-(DSGEModel.GridWidth*p_std);
      dev = (PriorDist.cauchy(n_cauchy,1)-lb)/NumGridPointsHalf;
      thetaVals = (lb:dev:lb+(NumGridPoints*dev));
      logDensityVals = logCauchyPDF(thetaVals',PriorDist.cauchy(n_cauchy,1),PriorDist.cauchy(n_cauchy,2));
      if DSGEModel.PriorKernel==1;
         Numz = 0;
         Oldz = [];
         lb = PriorDist.cauchy(n_cauchy,1)-(6*PriorDist.cauchy(n_cauchy,2));
         ub = PriorDist.cauchy(n_cauchy,1)+(6*PriorDist.cauchy(n_cauchy,2));
         while Numz<5000;
            z = CauchyRndFcn(PriorDist.cauchy(n_cauchy,1),PriorDist.cauchy(n_cauchy,2),10000);
            %
            % remove value of z that are too big or too small
            %
            if sum(z>=lb)<length(z);
               z = z(z>=lb);
            end;
            if sum(z<=ub)<length(z);
               z = z(z<=ub);
            end;
            Oldz = [Oldz z];
            z = Oldz;
            if length(z)>5000;
               z = z(1:5000);
            end;
            Numz = length(z);
         end;
      end;
      BoundMatrix(i,:) = [-Inf Inf];
      DistNames = strvcat(DistNames,'cauchy');
   elseif thetaDist(i)==9;
      %
      % logistic
      %
      n_logistic = n_logistic+1;
      a = PriorDist.logistic_ab(n_logistic,1);
      b = PriorDist.logistic_ab(n_logistic,2);
      c = PriorDist.logistic_ab(n_logistic,3);
      p_std = PriorDist.logistic(n_logistic,2);
      lb = (a+(b*log(c)))-(DSGEModel.GridWidth*p_std);
      dev = (a+(b*log(c))-lb)/NumGridPointsHalf;
      thetaVals = (lb:dev:lb+(NumGridPoints*dev));
      logDensityVals = logLogisticPDF(thetaVals',a,b,c);
      if DSGEModel.PriorKernel==1;
         z = LogisticRndFcn(a,b,c,5000);
      end;
      BoundMatrix(i,:) = [-Inf Inf];
      DistNames = strvcat(DistNames,'logistic');
   elseif thetaDist(i)==10;
      %
      % Gumbel
      %
      n_gumbel = n_gumbel+1;
      a = PriorDist.gumbel_ab(n_gumbel,1);
      b = PriorDist.gumbel_ab(n_gumbel,2);
      p_std = PriorDist.gumbel(n_gumbel,2);
      lb = a-(DSGEModel.GridWidth*p_std);
      dev = (a-lb)/NumGridPointsHalf;
      thetaVals = (lb:dev:lb+(NumGridPoints*dev));
      logDensityVals = logGumbelPDF(thetaVals',a,b);
      if DSGEModel.PriorKernel==1;
         z = GumbelRndFcn(a,b,5000);
      end;
      BoundMatrix(i,:) = [-Inf Inf];
      DistNames = strvcat(DistNames,'gumbel');
   elseif thetaDist(i)==11;
      %
      % Pareto
      %
      n_pareto = n_pareto+1;
      a = PriorDist.pareto(n_pareto,1);
      b = PriorDist.pareto(n_pareto,2);
      c = PriorDist.pareto(n_pareto,3);
      %
      % upper bound is selected such that 99.9 percent of the mass is less than or equal to the upper bound
      %
      ub = c+b/((0.01)^(1/a));
      if ub>20+c;
         ub = 20+c+b;
      end;
      dev = (ub-b-c)/NumGridPoints;
      thetaVals = (b+c:dev:ub);
      logDensityVals = logParetoPDF(thetaVals'-c,a,b);
      if DSGEModel.PriorKernel==1;
         z = c+ParetoRndFcn(a,b,5000);
      end;
      BoundMatrix(i,:) = [b+c Inf];
      DistNames = strvcat(DistNames,'pareto');
   end;
   if DSGEModel.PriorKernel==0;
      thetaValues(i,:) = thetaVals;
      DensityValue(i,:) = exp(logDensityVals)';
   else;
      %
      % Compute the Kernel density estimate of the density function
      %
      [XValues,YValues] = KernelDensityPlot(z,DSGEModel.KernelDensityEstimator);
      %
      % make sure the height is equal for the two estimates when using the normal, student-t
      % cauchy, logistic and gumbel distributions
      %
      if (thetaDist(i)==3)|(thetaDist(i)==7)|(thetaDist(i)==8)|(thetaDist(i)==9)|(thetaDist(i)==10);
         YValues = (max(exp(logDensityVals))/max(YValues))*YValues;
      end;
      KernelDensityX(i,:) = XValues';
      KernelDensityY(i,:) = YValues';
   end;
end;
%
% check if we should just return Xdata and Ydata
%
if nargout==3;
   if DSGEModel.PriorKernel==0;
      Xdata = thetaValues;
      Ydata = DensityValue;
   else;
      Xdata = KernelDensityX;
      Ydata = KernelDensityY;
   end;
   BoundsMatrix = BoundMatrix;
   return;
end;
%
% check if there are parameters to update
%
if (FileExist(DSGEModel.UpdateParameterFile)==1)&(isempty(ParameterNames.additional)==0);
   txt = ['The model "' DSGEModel.NameOfModel '" has parameters that are a function of the estimated parameters. Would you also like to plot prior densities for these additional parameters?'];
   answer = Query(txt,'question',200,'Prior Densities - Additional Parameters',500,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      %
      % get the data
      %
      UpdateNames = ParameterNames.additional;
      stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
      if stat==1;
         CurrINI.theta = theta;
         CurrINI.thetaDist = thetaDist;
         CurrINI.LowerBound = LowerBound;
         CurrINI.PriorDist = PriorDist;
         CurrINI.ParameterNames = ParameterNames;
         CurrINI.thetaPositions = thetaPositions;
         CurrINI.ModelParameters = ModelParameters;
         CurrINI.UpdateNames = UpdateNames;
         [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = AdaptiveLearningGetUpdatePriorDensity(DSGEModel,CurrINI,controls,0,0,0,0);
         %
         % test if the results were obtained from an old mat-file
         %
         if max(UpdateParameters)>size(UpdateNames,1);
            if FileExist(DSGEModel.SystemPriorFile)==0;
               AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-Additional-' DSGEModel.NameOfModel '.mat']);
            else;
               AddFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-Additional-' DSGEModel.NameOfModel '.mat']);
            end;
            delete(AddFile);
            [UpdatePriorX,UpdatePriorY,UpdatePriorBounds,UpdateParameters,UpdateInitialValue] = AdaptiveLearningGetUpdatePriorDensity(DSGEModel,CurrINI,controls,0,0,0,0);
         end;
         if isempty(UpdatePriorX)==0;
            %
            % check which parameters are actually updated
            %
            if size(UpdateNames)>length(UpdateParameters);
               UpdateNames = UpdateNames(UpdateParameters,:);
            end;
         else;
            txt = 'YADA has determined that all parameters that are supposed to be functions of the estimated parameters are, in fact, fixed. YADA will therefore skip these parameters in the plots.';
            About(txt,'information','Prior Densities - Additional Parameters',300,500,CurrINI);
            UpdateNames = '';
            UpdatePriorX = [];
            UpdatePriorY = [];
            UpdatePriorBounds = [];
            UpdateInitialValue = [];
         end;
      else;
         if ispc==1;
            txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\priordraws". The prior for the parameters that are a function of the estimated parameters can therefore not be computed.'];
         else;
            txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/priordraws". The prior for the parameters that are a function of the estimated parameters can therefore not be computed.'];
         end;
         About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
         UpdateNames = '';
         UpdatePriorX = [];
         UpdatePriorY = [];
         UpdatePriorBounds = [];
         UpdateInitialValue = [];
      end;
   else;
      UpdateNames = '';
      UpdatePriorX = [];
      UpdatePriorY = [];
      UpdatePriorBounds = [];
      UpdateInitialValue = [];
   end;
else;
   UpdateNames = '';
   UpdatePriorX = [];
   UpdatePriorY = [];
   UpdatePriorBounds = [];
   UpdateInitialValue = [];
end;
%
NumAdditional = size(UpdateNames,1);
%
% Compute the number of figures (max 16 plots per graph)
%
NumFigs = ceil((NumParam+NumAdditional)/16);
%
if NumFigs==1;
   [NumRows,NumCols] = VillaniConstructOptimalSubplot(NumParam+NumAdditional);
else;
   NumRows = 4;
   NumCols = 4;
end;
NumPlots = 0;
pos = [(CurrINI.scrsz(3)-1000)/2 (CurrINI.scrsz(4)-680)/2 1000 650];
for i=1:NumFigs;
   if i>1;
      pos(1) = pos(1)+22;
      pos(2) = pos(2)-22;
      if pos(1)+1000>CurrINI.scrsz(3);
         pos(1) = 0;
      end;
      if pos(2)<=32;
         pos(2) = CurrINI.scrsz(4)-700;
      end;
   end;
   FigHandles{i} = figure('Visible','off','Position',pos);
   if NumFigs==1;
      set(FigHandles{i},'Name','Marginal Prior Densities - Adaptive Learning');
   else;
      set(FigHandles{i},'Name',['Marginal Prior Densities - Adaptive Learning - Figure ' num2str(i)]);
   end;
   SetFigureProperties(FigHandles{i},CurrINI);
   k = 0;
   for r=1:NumRows;
      for c=1:NumCols;
         NumPlots = NumPlots+1;
         if NumPlots<=NumParam+NumAdditional;
            k = k+1;
            %
            % ensure that the handle to the current figure is correct (HG2 issue)
            %
            set(CurrINI.GraphicsRoot,'CurrentFigure',FigHandles{i});
            subplot(NumRows,NumCols,k);
            %
            % plot the density
            %
            if NumPlots<=NumParam;
               if DSGEModel.PriorKernel==0;
                  dh = plot(thetaValues(NumPlots,:),DensityValue(NumPlots,:),'-','Color',[0.75 0 0],'LineWidth',2);
               else;
                  dh = plot(KernelDensityX(NumPlots,:),KernelDensityY(NumPlots,:),'-','Color',[0.75 0 0],'LineWidth',2);
               end;
               hold('on');
               SetAxesFonts(CurrINI,'Title',[StringTrim(ParameterNames.estimated(NumPlots,:)) ' - ' StringTrim(DistNames(NumPlots,:)) ' prior']);
            else;
               dh = plot(UpdatePriorX(NumPlots-NumParam,:),UpdatePriorY(NumPlots-NumParam,:),'-','Color',[0.75 0 0],'LineWidth',2);
               hold('on');
               SetAxesFonts(CurrINI,'Title',[StringTrim(UpdateNames(NumPlots-NumParam,:)) ' - simulated prior']);
            end;
            %
            % plot the initial value
            %
            y_lim = get(gca,'Ylim');
            y_lim(1) = 0;
            if NumPlots<=NumParam;
               ivh = plot(theta(NumPlots)*ones(1,2),y_lim,'-','Color',[0 0 0]);
            else;
               ivh = plot(UpdateInitialValue(NumPlots-NumParam)*ones(1,2),y_lim,'-','Color',[0 0 0]);
            end;
            kids = [dh ivh];
            set(gca,'Children',kids);
            axis('tight');
            x_lim = get(gca,'XLim');
            if NumPlots<=NumParam;
               if DSGEModel.PriorKernel==1;
                  if strcmp(StringTrim(DistNames(NumPlots,:)),'gamma')==1;
                     x_lim(1) = BoundMatrix(NumPlots,1);
                  elseif strcmp(StringTrim(DistNames(NumPlots,:)),'beta')==1;
                     x_lim = BoundMatrix(NumPlots,:);
                  elseif strcmp(StringTrim(DistNames(NumPlots,:)),'inv gamma')==1;
                     x_lim(1) = BoundMatrix(NumPlots,1);
                  elseif strcmp(StringTrim(DistNames(NumPlots,:)),'trunc normal')==1;
                     x_lim(1) = BoundMatrix(NumPlots,1);
                  elseif strcmp(StringTrim(DistNames(NumPlots,:)),'uniform')==1;
                     x_lim = BoundMatrix(NumPlots,:);
                  end;
               end;
            else;
               if x_lim(1)<UpdatePriorBounds(NumPlots-NumParam,1);
                  x_lim(1) = UpdatePriorBounds(NumPlots-NumParam,1);
               end;
               if x_lim(2)>UpdatePriorBounds(NumPlots-NumParam,2);
                  x_lim(2) = UpdatePriorBounds(NumPlots-NumParam,2);
               end;
            end;
            set(gca,'YLim',y_lim,'XLim',x_lim);
            hold('off');
         end;
      end;
   end;
   if DSGEModel.PriorKernel==0;
      legHandle = legend([dh ivh],strvcat('Grid Density Estimate','Initial Value'));
   else;
      legHandle = legend([dh ivh],strvcat('Kernel Density Estimate','Initial Value'));
   end;
   SetAxesFonts(CurrINI,legHandle);
   %
   % transform units temporarily into pixels
   %
   OldAxesUnits = get(gca,'Units');
   OldLegUnits = get(legHandle,'Units');
   set(gca,'Units','pixels');
   set(legHandle,'Units','pixels');
   %
   % check if we can move legend
   %
   LegPos = get(legHandle,'Position');
   AxPos = get(gca,'Position');
   if LegPos(1)<AxPos(1)+AxPos(3);
      if AxPos(1)+AxPos(3)+10+LegPos(3)<1000;
         LegPos(1) = AxPos(1)+AxPos(3)+10;
         set(legHandle,'Position',LegPos);
      end;
   end;
   set(gca,'Units',OldAxesUnits);
   set(legHandle,'Units',OldLegUnits);
   %
   % Fix the toolbar cdata
   %
   SetToolbarImages(FigHandles{i},CurrINI.images);
   %
   % Display the current figure
   %
   set(FigHandles{i},'Visible','on','CloseRequestFcn','delete(gcf); drawnow; pause(0.02);');
   drawnow;
   pause(0.02);
end;

%
% end of AdaptiveLearningGraphPriorDensity.m
%
