function [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,sigmarAL,sigmaeAL,SMatrix,G,m,x,f,MaximumIterationsValue,ToleranceValue,Gtilde,Kgf)
% AdaptiveLearningInitialValuesDSGE: Computes the initial values for certain matrices of the Kalman filter for
%                                    the adaptive learning belief system.
%
% USAGE:
%
%       [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,sigmarAL,sigmaeAL,SMatrix,G,m,x,f,...
%                                                                            MaximumIterationsValue,ToleranceValue,Gtilde,Kgf)
%
% REQUIRED INPUT:  F (matrix) of dimension r x r of the state equation from the RE version of the DSGE model.
%
%                  B0 (matrix) of dimension r x q with the coefficients on the structural shocks in the same state equation.
%
%                  sigmarAL (positive scalar) with the parameter on the inverted GLS matrix to initialize R10.
%
%                  sigmaeAL (positive scalar) with the parameter on the inverted GLS matrix to determine Sigmaeps
%
%                  SMatrix (matrix) of dimension r x f that selects the forward looking variables among the state
%                          variables.
%
%                  G (cell array) of dimension f for selecting the explanatory variables among deterministic and lagged
%                          state variables in each belief equation for the forward looking variables.
%
%                  m (integer) with the maximum number of lags for the belief equations.
%
%                  x (matrix) of dimension k x T with the sample values of the deterministic variables.
%
%                  f (integer) with the number of forward looking variables.
%
%                  MaximumIterationsValue (integer) with the maximum number of iterations to use with the doubling algorithm
%                          divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                  ToleranceValue (integer) which determines the tolerance value for the doubling algorithm.
%
% OPTIONAL INPUT:  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                          of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                          A is a g x f matrix.
%
% REQUIRED OUTPUT: betavec (vector) of dimension g with the initial values for the beta-vector of the belief equations.
%
%                  Sigmau (matrix) of dimension f x f with the forecast error covariance matrix of the belief
%                          equations.
%
%                  R10 (matrix) of dimension g x g with the initial value of the covariance matrix of the
%                          forecasted belief coefficients.
%
%                  Sigmaeps (matrix) of dimension g x g with the covariance matrix of the innovation to the
%                          transition equation of the belief coefficients.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                          of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                          A is a g x f matrix.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 12, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-03-2020: Finished the function after the initialization part.
%
% * 16-03-2020: Added the output variable "Gtilde".
%
% * 17-03-2020: Fixed a bug where GLSij' was not read properly. Deleted an if-statement. Added
%               the output variable Kgf.
%
% * 30-03-2020: Added Gtilde and Kgf as input variables in the event that they have already
%               been created through a previous run of the function.
%
% * 31-03-2020: Updated the documentation.
%
% * 01-04-2020: Changed the name ending of adaptive learning parameters.
%
% * 09-04-2020: Fixed a bug where the lastpos integer was not properly updated. This affected
%               all values of f greater than 2, with betavec and Gtilde being improperly setup.
%
% * 15-04-2020: The code now makes sure to use eigenvalue/Schur decompositions of Sigmau
%               and GLSMatrix if these do not have full rank. This should ensure that the
%               "inverses" of these matrices do not have extremely large elements.
%
% * 08-05-2020: Added the input variable mu.
%
% * 02-07-2020: removed the input variable mu.
%
% * 22-01-2021: Updated the documentation.
%
% * 02-02-2021: Fixed a rows and columns bug when setting up the GLS matrix.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 15-08-2023: Added NaN and Inf tests before checking the rank of GLSMatrix. Also,
%               if F has unstable eigenvalues, then Sigmaksi is diagonal with a
%               large common variance.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
betavec = [];
Sigmau = [];
R10 = [];
Sigmaeps = [];
if nargin==11;
   Gtilde = [];
   Kgf = [];
end;
%
% compute the SigmaZ matrix in various steps
%
[k,T] = size(x);
if k>0;
   Sigmax = (1/T)*x*x';
else;
   Sigmax = [];
end;
r = size(F,1);
krm = k+(r*m);
%
% compute the population covariance matrix of the state variables in the
% DSGE model using the doubling algorithm
%
[Sigmaksi,stat] = DoublingAlgorithmLyapunov(F,B0*B0',MaximumIterationsValue*100,10^(-(ToleranceValue+1)));
if stat==1;
   %
   % it didn't converge, we try vectorization
   %
   if max(abs(eig(F)))<1;
      Sigmaksi = reshape((eye(r*r)-kron(F,F))\vec(B0*B0'),r,r);
   else;
      %
      % F is unstable, so we test with a large diagonal
      %
      Sigmaksi = 1000*eye(r);
   end;
end;
%
% we are now ready to set up SigmaZ
%
SigmaZ = zeros(krm,krm);
if k>0;
   SigmaZ(1:k,1:k) = Sigmax;
end;
SigmaZ((1:r)+k,(1:r)+k) = Sigmaksi;
if m==2;
   SigmaZ((1:r)+k+r,(1:r)+k+r) = Sigmaksi;
   SigmaZ((1:r)+k,(1:r)+k+r) = F*Sigmaksi;
   SigmaZ((1:r)+k+r,(1:r)+k) = Sigmaksi*F';
end;
%
J = zeros(krm,r);
J(k+(1:r),:) = eye(r);
%
% determine the number of explanatory variables for each forward
% looking variables
%
gsize = zeros(f,1);
for j=1:f;
   gsize(j) = size(G{j,1},2);
end;
g = sum(gsize);
if isempty(Gtilde)==1;
   Ghelp = zeros(g,krm);
   Gtilde = zeros(f*g,krm);
   MakeGtilde = 1;
else;
   MakeGtilde = 0;
end;
%
% 1. now we compute betavec and setup the Gtilde
%
betavec = zeros(g,1);
lastpos = 0;
for j=1:f;
   betavec(lastpos+(1:gsize(j))) = (G{j}'*SigmaZ*G{j})\(G{j}'*SigmaZ*J*F'*SMatrix(:,j));
   %
   % now the Gtilde matrix
   %
   if MakeGtilde==1;
      Gtildej = Ghelp;
      Gtildej(lastpos+(1:gsize(j)),:) = G{j}';
      Gtilde((g*(j-1))+1:(g*j),:) = Gtildej;
   end;
   lastpos = lastpos+gsize(j);
end;
%
% The commutation matrix Kgf
%
if isempty(Kgf)==1;
   Kgf = CommutationMatrix(g,f);
end;
%
betatilde = kron(betavec',eye(f))*Kgf*Gtilde;
%
% 2. compute Sigmau
%
SigmauHelp = betatilde*SigmaZ*J*F'*SMatrix;
Sigmau = (SMatrix'*Sigmaksi*SMatrix)-SigmauHelp-SigmauHelp'+(betatilde*SigmaZ*betatilde');
%
% find a useful generalized inverse of Sigmau if singular
%
if (rank(Sigmau)==f)&(det(Sigmau)>1.0e-10);
   Omega = Sigmau\eye(f);
else;
   [S,Lambda] = SchurDecomposition(Sigmau,1.0e-10);
   LambdaSize = size(Lambda,1);
   InvLambda = diag(ones(LambdaSize,1)./diag(Lambda));
   Omega = S*InvLambda*S';
end;
%
% now we aim for the GLS population matrix
%
GLSMatrix = zeros(g,g);
%
lastposi = 0;
for i=1:f;
   gi = gsize(i);
   rows = lastposi+(1:gi);
   lastposj = 0;
   for j=i:f;
      gj = gsize(j);
      cols = lastposj+lastposi+(1:gj);
      GLSij = Omega(i,j)*G{i,1}'*SigmaZ*G{j,1};
      GLSMatrix(rows,cols) = GLSij;
      GLSMatrix(cols,rows) = GLSij';
      lastposj = lastposj+gj;
   end;
   lastposi = lastposi+gi;
end;
%
% 3. Compute the R10 and Sigmaeps matrices based on the inverted GLS matrix
%
if (max(max(isnan(GLSMatrix)))==0)&(max(max(isinf(GLSMatrix)))==0)&(rank(GLSMatrix)==g);
   InvGLSMatrix = GLSMatrix\eye(g);
else;
   [S,Lambda] = SchurDecomposition(GLSMatrix,1.0e-10);
   LambdaSize = size(Lambda,1);
   InvLambda = diag(ones(LambdaSize,1)./diag(Lambda));
   InvGLSMatrix = S*InvLambda*S';
end;
R10 = sigmarAL*InvGLSMatrix;
Sigmaeps = sigmaeAL*InvGLSMatrix;

%
% end of AdaptiveLearningInitialValuesDSGE.m
%
