function [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,StartPeriod,AllowNaNs)
% AdaptiveLearningKalmanSmoother: Computes the Kalman smooth estimates of state variables, structural shocks and measurement errors
%                                 when expectations in the DSGE model are formed using adaptive expectations.
%
% USAGE:
%
%       [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,...
%                                                                        FirstPeriod,LastPeriod,StartPeriod,AllowNaNs)
%
% REQUIRED INPUT:  Ksitt1 (matrix) of dimension rxT with the state variable predictions.
%
%                  Ptt1 (matrix) of dimension rxrxT with covariances for the state variable prediction errors.
%
%                  Yerror (matrix) of dimension nxT with the 1-step ahead forecast errors of the observed variables.
%
%                  InvMSEY (matrix) of dimension nxnxT with the inverted 1-step ahead forecast error covariances of
%                          the observed variables.
%
%                  H (matrix) of dimension rxn (or rxnxT) with parameter values on the r state variables (ksi) in the
%                          measurement equation.
%
%                  R (matrix) of dimension nxn with covariances for the innovations in the measurement equation.
%
%                  Ftilde (matrix) of dimension r x r x T with the coefficients on lagged state variables.
%
%                  Btilde (matrix) of dimension r x q x T with the coefficients on the structural shocks.
%
%                  FirstPeriod (integer) with the first period to use for the data.
%
%                  LastPeriod (integer) with the last period to use for the data.
%
%                  StartPeriod (integer) indicating first observation to use when calculating the log-likelihood value. The
%                          default value is 1. This input along with FirstPeriod and LastPeriod are only used if H
%                          is time-varying (3D).
%
%                  AllowNaNs (boolean) that takes the value 1 if NaNs among the observed variables should be allowed
%                          or not.
%
% REQUIRED OUTPUT: KsitT (matrix) of dimension rxT with the smooth predictions of the state vector.
%
%                  PtT (matrix) of dimension rxrxT with the covariances of the smooth predictions of the state vector.
%
%                  etatt (matrix) of dimension q x T with the update estimates of the structural shocks.
%
%                  etatT (matrix) of dimension q x T with the smooth estimates of the structural shocks.
%
%                  wtt (matrix) of dimension n1 x T with the update estimates of the n1 measurement errors,
%                          where n1 is equal to the number of nonzero measurement errors, i.e. R(i,i)>0.
%
%                  wtT (matrix) of dimension n1 x T with the smooth estimates of the n1 measurement errors.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 27, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 30-03-2020: Finished the function from the missing observations and time
%               varying H matrix part.
%
% * 17-04-2020: Corrected a bug when computing etatT and etatt using Btilde(t) instead
%               of Btilde(t-1).
%
% * 22-01-2021: Updated the documentation.
%
% * 27-04-2021: Removed the input variable "VariableNames" as it is not used.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[r,T] = size(Ksitt1);
n = size(Yerror,1);
q = size(Btilde,2);
KsitT = zeros(r,T);
PtT = zeros(r,r,T);
etatt = zeros(q,T);
etatT = zeros(q,T);
rankR = rank(R);
if rankR>0;
   if AllowNaNs==0;
      wtt = zeros(n,T);
      wtT = zeros(n,T);
   else;
      wtt = NaN*ones(n,T);
      wtT = NaN*ones(n,T);
   end;
else;
   wtt = [];
   wtT = [];
end;
%
if length(size(H))==3;
   H = H(:,:,FirstPeriod+StartPeriod-1:LastPeriod);
end;
rtTLast = zeros(r,1);
NtTLast = zeros(r,r);
%
if AllowNaNs==0;
   if length(size(H))==2;
      %
      % the standard case
      %
      for t=(T:-1:1);
         rtNew = H*InvMSEY(:,:,t)*Yerror(:,t);
         %etatt(:,t) = Btilde(:,:,t)'*rtNew;
         Kt1 = Ftilde(:,:,t)*Ptt1(:,:,t)*H*InvMSEY(:,:,t);
         FKH = Ftilde(:,:,t)-(Kt1*H');
         if rankR>0;
            wtt(:,t) = R*InvMSEY(:,:,t)*Yerror(:,t);
            wtT(:,t) = wtt(:,t)-(R*InvMSEY(:,:,t)*H'*Ptt1(:,:,t)*rtTLast);
         end;
%         rtTOld = rtTLast;
         rtTLast = rtNew+(FKH'*rtTLast);
         KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rtTLast);
         if t>1;
            etatt(:,t) = Btilde(:,:,t-1)'*rtNew;
            etatT(:,t) = Btilde(:,:,t-1)'*rtTLast;
         else;
            etatt(:,t) = Btilde(:,:,1)'*rtNew;
            etatT(:,t) = Btilde(:,:,1)'*rtTLast;
         end;
         NtNew = H*InvMSEY(:,:,t)*H';
         NtTLast = NtNew+(FKH'*NtTLast*FKH);
         PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NtTLast*Ptt1(:,:,t));
%         if t==T-1;
%            disp([KsitT(:,T)   Mtilde(:,1,T-1)+Ftilde(:,:,T-1)*KsitT(:,T-1)+Btilde(:,:,T-1)*etatT(:,T)   Mtilde(:,1,T-1)+Ftilde(:,:,T-1)*KsitT(:,T-1)+Btilde(:,:,T-1)*Btilde(:,:,T-1)'*rtTOld])
%         elseif t==T-2;
%            disp([KsitT(:,T-1)   Mtilde(:,1,T-2)+Ftilde(:,:,T-2)*KsitT(:,T-2)+Btilde(:,:,T-2)*etatT(:,T-1)   Mtilde(:,1,T-2)+Ftilde(:,:,T-2)*KsitT(:,T-2)+Btilde(:,:,T-2)*Btilde(:,:,T-2)'*rtTOld])
%         end;
      end;
   else;
      %
      % time varying H matrix
      %
      for t=(T:-1:1);
         rtNew = H(:,:,t)*InvMSEY(:,:,t)*Yerror(:,t);
         %etatt(:,t) = Btilde(:,:,t)'*rtNew;
         Kt1 = Ftilde(:,:,t)*Ptt1(:,:,t)*H(:,:,t)*InvMSEY(:,:,t);
         FKH = Ftilde(:,:,t)-(Kt1*H(:,:,t)');
         if rankR>0;
            wtt(:,t) = R*InvMSEY(:,:,t)*Yerror(:,t);
            wtT(:,t) = wtt(:,t)-(R*InvMSEY(:,:,t)*H(:,:,t)'*Ptt1(:,:,t)*rtTLast);
         end;
         rtTLast = rtNew+(FKH'*rtTLast);
         KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rtTLast);
         if t>1;
            etatt(:,t) = Btilde(:,:,t-1)'*rtNew;
            etatT(:,t) = Btilde(:,:,t-1)'*rtTLast;
         else;
            etatt(:,t) = Btilde(:,:,1)'*rtNew;
            etatT(:,t) = Btilde(:,:,1)'*rtTLast;
         end;
         NtNew = H(:,:,t)*InvMSEY(:,:,t)*H(:,:,t)';
         NtTLast = NtNew+(FKH'*NtTLast*FKH);
         PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NtTLast*Ptt1(:,:,t));
      end;
   end;
else;
   %
   % missing observations
   %
   if length(size(H))==2;
      %
      % the standard case
      %
      for t=(T:-1:1);
         pos = (~isnan(Yerror(:,t)));
         n1 = sum(pos);
         if n1>0;
            rtNew = H(:,pos)*InvMSEY(pos,pos,t)*Yerror(pos,t);
            %etatt(:,t) = Btilde(:,:,t)'*rtNew;
            Kt1 = Ftilde(:,:,t)*Ptt1(:,:,t)*H(:,pos)*InvMSEY(pos,pos,t);
            FKH = Ftilde(:,:,t)-(Kt1*H(:,pos)');
            if rankR>0;
               wtt(pos,t) = R(pos,pos)*InvMSEY(pos,pos,t)*Yerror(pos,t);
               wtT(pos,t) = wtt(pos,t)-(R(pos,pos)*InvMSEY(pos,pos,t)*H(:,pos)'*Ptt1(:,:,t)*rtTLast);
            end;
            rtTLast = rtNew+(FKH'*rtTLast);
            KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rtTLast);
            %etatT(:,t) = Btilde(:,:,t)'*rtTLast;
            if t>1;
               etatt(:,t) = Btilde(:,:,t-1)'*rtNew;
               etatT(:,t) = Btilde(:,:,t-1)'*rtTLast;
            else;
               etatt(:,t) = Btilde(:,:,1)'*rtNew;
               etatT(:,t) = Btilde(:,:,1)'*rtTLast;
            end;
            NtNew = H(:,pos)*InvMSEY(pos,pos,t)*H(:,pos)';
            NtTLast = NtNew+(FKH'*NtTLast*FKH);
            PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NtTLast*Ptt1(:,:,t));
         else;
            rtTLast = Ftilde(:,:,t)'*rtTLast;
            KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rtTLast);
            etatT(:,t) = Btilde(:,:,t)'*rtTLast;
            NtTLast = Ftilde(:,:,t)'*NtTLast*Ftilde(:,:,t);
            PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NtTLast*Ptt1(:,:,t));
         end;
      end;
   else;
      %
      % time varying H matrix
      %
      for t=(T:-1:1);
         pos = (~isnan(Yerror(:,t)));
         n1 = sum(pos);
         if n1>0;
            rtNew = H(:,pos,t)*InvMSEY(pos,pos,t)*Yerror(pos,t);
            %etatt(:,t) = Btilde(:,:,t)'*rtNew;
            Kt1 = Ftilde(:,:,t)*Ptt1(:,:,t)*H(:,pos,t)*InvMSEY(pos,pos,t);
            FKH = Ftilde(:,:,t)-(Kt1*H(:,pos,t)');
            if rankR>0;
               wtt(pos,t) = R(pos,pos)*InvMSEY(pos,pos,t)*Yerror(pos,t);
               wtT(pos,t) = wtt(pos,t)-(R(pos,pos)*InvMSEY(pos,pos,t)*H(:,pos,t)'*Ptt1(:,:,t)*rtTLast);
            end;
            rtTLast = rtNew+(FKH'*rtTLast);
            KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rtTLast);
            if t>1;
               etatt(:,t) = Btilde(:,:,t-1)'*rtNew;
               etatT(:,t) = Btilde(:,:,t-1)'*rtTLast;
            else;
               etatt(:,t) = Btilde(:,:,1)'*rtNew;
               etatT(:,t) = Btilde(:,:,1)'*rtTLast;
            end;
            %etatT(:,t) = Btilde(:,:,t)'*rtTLast;
            NtNew = H(:,pos,t)*InvMSEY(pos,pos,t)*H(:,pos,t)';
            NtTLast = NtNew+(FKH'*NtTLast*FKH);
            PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NtTLast*Ptt1(:,:,t));
         else;
            rtTLast = Ftilde(:,:,t)'*rtTLast;
            KsitT(:,t) = Ksitt1(:,t)+(Ptt1(:,:,t)*rtTLast);
            etatT(:,t) = Btilde(:,:,t)'*rtTLast;
            NtTLast = Ftilde(:,:,t)'*NtTLast*Ftilde(:,:,t);
            PtT(:,:,t) = Ptt1(:,:,t)-(Ptt1(:,:,t)*NtTLast*Ptt1(:,:,t));
         end;
      end;
   end;
end;

%
% end of AdaptiveLearningKalmanSmoother.m
%
