function [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betatt)
% AdaptiveLearningSolveDSGEModel: Computes the solution matrix of the adaptive learning version of the DSGE
%                                 model that forms the state equation when computing the Kalman filter,
%                                 smoother and log-likelihood.
%
% USAGE:
%
%       [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betatt)
%
% REQUIRED INPUT:  Hstar0 (matrix) of dimension r x r with the coefficient values on the contemporaneous state variables.
%
%                  Hstarlag (matrix) of dimension r x r with the coefficient values on the lagged state variables.
%
%                  Hstar1f (matrix) of dimension r x f with the coefficient values on the expectated future values
%                          of the forward looking variables.
%
%                  D (matrix) of dimension r x q with the coefficient values on the structural shocks.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                          of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                          A is a g x f matrix.
%
%                  m (integer) with the maximum lag for the state variables in the belief equations.
%
%                  k (integer) with the number of deterministic variables.
%
%                  betatt (vector) of dimension g x 1 with the Kalman update estimates of the
%                          belief coefficients.
%
% REQUIRED OUTPUT: Mt1 (matrix) of dimension r x k with the coefficients on the deterministic
%                          variables in the solution equation.
%
%                  Ft1 (matrix) of dimension r x r with the coefficients on lagged state variables.
%
%                  Bt1 (matrix) of dimension r x q with the coefficients on the structural shocks.
%
% OPTIONAL OUTPUT: InvMat (matrix) of dimension r x r with the inverse of the coefficients in the
%                          solution on the contemporaneous state/model variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 17, 2020.
%                        This version: February 29, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-03-2020: Continued work on the function was on the transformed DSGE model matrices.
%
% * 20-03-2020: Finished the function.
%
% * 25-03-2020: The function now runs for a single time period.
%
% * 22-01-2021: Updated the documentation.
%
% * 30-03-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
% * 29-02-2024: Added the optional output variable "InvMat". It is used when dealing with anticipated
%               shocks as in the case of the ZLB.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[r,q] = size(D);
g = length(betatt);
f = size(Kgf,1)/g;
if k>0;
   Mt1 = zeros(r,k);
else;
   Mt1 = [];
end;
Ft1 = zeros(r,r);
Bt1 = zeros(r,q);
%
% create betatilde
%
betatilde = kron(betatt',eye(f))*Kgf*Gtilde;
%
% compute the coefficients on x(t)
%
InvH0H1star = (Hstar0+(Hstar1f*betatilde(:,k+1:k+r)))\eye(r);
if k>0;
   Mt1 = -InvH0H1star*Hstar1f*betatilde(:,1:k);
end;
if m==2;
   Ft1 = -InvH0H1star*(Hstarlag+(Hstar1f*betatilde(:,k+r+1:k+(r*m))));
else;
   Ft1 = -InvH0H1star*Hstarlag;
end;
Bt1 = InvH0H1star*D;
%
if nargout==4;
   InvMat = InvH0H1star;
end;

%
% end of AdaptiveLearningSolveDSGEModel.m
%
