function [StateVarStructure,BeliefStatus,TransStatus,status] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,ModelParameters,BeliefSys,DSGEModel,CurrINI,ObsVarDec)
% CalculateAdaptiveLearningStateVariables: Computes the forecasted, updated and smoother estimates of the state variables, shocks,
%                                          measurement errors as well as the log-likelihood.
%
% USAGE:
%
%       [StateVarStructure,BeliefStatus,TransStatus,status] = CalculateAdaptiveLearningStateVariables(theta,thetaPositions,...
%                                                                              ModelParameters,BeliefSys,DSGEModel,CurrINI,...
%                                                                              ObsVarDec)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters as well as the estimated
%                        parameters of the adaptive learning system.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function needs the
%                        former field to know which parameter name corresponds to position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                        DSGE model along with the adaptive learning system.
%
%                  BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags
%                        of the state variables in the belief equations; "Name" a string with the name of the
%                        belief system; and G a cell array with f matrices, one for each forward looking
%                        variable of dimension (k+r*m) x g_j where g_j is the number of explanatory
%                        variables in the belief equation for forward looking variable j. If empty, the
%                        function will compute the structure itself.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
%                  ObsVarDec (boolean) that takes the value 1 if an observed variable decomposition should
%                        be computed and 0 otherwise.
%
% REQUIRED OUTPUT: StateVarStructure (structure) with fields "Y", "Yhat" containing the observed variables and
%                        the one-step-ahead forecasts; "Ksitt1", "Ksitt" and "KsitT" with the forecasted, updated
%                        and smoothed estimates of the state variables; "etatt" and "etatT" for the update and
%                        smooth estimates of the structural shocks; "wtt" and "wtT" with the update and smooth
%                        estimates of the measurement errors; "lnLt" with the log-likelihood; "nonsingular" which
%                        indicates if there are enough shocks and measurement errors for the one-step-ahead
%                        forecast error covariance matrix to be nonsingular; "TrainSample"  is a boolean variable
%                        that indicates if a training sample was employed for the Kalman filter; "X" a matrix with
%                        the deterministic variables; "betatt" is a matrix with the Kalman filter estimates of
%                        the belief coeffcients; BeliefSys containing the structure BeliefSys; ConstPos which gives
%                        a possible position of the constant among the deterministic variables. It also contains
%                        verious other fields, such as Zlag, Gtilde, Kgf, SMatrix, m, f, Mtilde, Ftilde, Btilde,
%                        Mss, Fss and Bss (steady-state solution matrices) which can be interpreted from the
%                        documentation of the functions used below. Furthermore, for observed and state variable
%                        decompositions the fields X, etaDecomp, XiInit, DetDecomp, wthT as well as XietaDecomp,
%                        XietaInit and XiDetDecomp are included. The parameter matrices A, H and R from the
%                        measurement equation are also included as fields in the structure, as well as the solution
%                        matrices from the RE version, F and B0.
%
% OPTIONAL OUTPUT: BeliefStatus (boolean) which is 1 if the Kalman filter of the belief system runs without
%                        error, and 0 otherwise.
%
%                  TransStatus (boolean) which takes the value 1 if the rewrite of the structural form of
%                        the DSGE model was successful and 0 otherwise.
%
%                  status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 12, 2020.
%                        This version: March 13, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 13-03-2020: Updated the documentation.
%
% * 16-03-2020: Continued after calling the "AdaptiveLearningInitialValuesDSGE" function.
%
% * 19-03-2020: Added the call to "AdaptiveLearningTransformDSGEModel".
%
% * 20-03-2020: Continued the function after the call to "AdaptiveLearningSolveDSGEModel".
%
% * 24-03-2020: Changed the Kalman filtering strategy to a joint filter for state variables,
%               belief coefficients and model solution.
%
% * 25-03-2020: The function "AdaptiveLaerningKalmanFilter" provides the output Zlag.
%
% * 27-03-2020: Continued after the call to "AdaptiveLearningKalmanFilter".
%
% * 30-03-2020: Finished the function.
%
% * 31-03-2020: Added the call to "AdaptiveLearningSolveDSGEModel" prior to running the Kalman
%               filter.
%
% * 01-04-2020: Changed the name ending of adaptive learning parameters.
%
% * 03-04-2020: Added the KeepVar field to the output structure.
%
% * 06-04-2020: Added the "BeliefSys" input variable and added the output field "BeliefSys".
%
% * 09-04-2020: Looking for bugs. Ending up after the initialization of betavec etc.
%
% * 14-04-2020; Continued bug hunting. Now focus on Kalman filter.
%
% * 15-04-2020: Finished bug hunting (hopefully).
%
% * 17-04-2020: Added the possibility of computing an observed variables decomposition.
%
% * 21-04-2020: Added code for observed and state variable decompositions. The output structure
%               StateVarStructure has been amended to support the output.
%
% * 22-04-2020: Fixed a bug when computing XiDetDecomp where t should have been t-1.
%
% * 28-04-2020: Took into account that "AdaptiveLearningKalmanFilter" requires the input variables
%               "DSGEModel.UseDoublingAlgorithm" and "DSGEModel.StateCovConstValue".
%
% * 04-05-2020: Took into account that "AdaptiveLearningKalmanFilter" also provides the LastSolution
%               output variable. This variable is added as a field to "StateVarStructure".
%
% * 06-05-2020: Added the fields F and B0 to the "StateVarStructure".
%
% * 08-05-2020: Added the input variable mu to "AdaptiveLearningInitialValuesDSGE".
%
% * 11-05-2020: Added the input variable "NoSmoothedStatesForBeliefs" to "AdaptiveLearningKalmanFilter".
%
% * 15-05-2020: Made sure that the fields "wtT" and "wtt" of "StateVarStructure" are initialized.
%
% * 02-07-2020: Removed the field "NonCentralGLSMatrix" from DSGEModel and the input variable "mu" for
%               "AdaptiveLearningInitialValuesDSGE".
%
% * 06-07-2020: Updated the documentation.
%
% * 01-10-2020: Fixed a bug with j instead of : for X matrix.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 22-01-2021: Updated the documentation.
%
% * 29-01-2021: The function supports a user defined model solver under adaptive learning.
%
% * 27-04-2021: The function "AdaptiveLearningKalmanSmoother" no longer requires the input variable
%               "VariableNames".
%
% * 03-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed additional eval calls to feval.
%
% * 16-03-2022: Added the steady-state solution to StateVarStructure.
%
% * 06-01-2023: Updated the documentation.
%
% * 18-01-2023: Updated the documentation.
%
% * 24-10-2023: The function now supports the fixed belief Kalman filter.
%
% * 08-01-2024: Updated the documentation.
%
% * 13-03-2024: The logical function "CheckAdaptiveLearningFixedBeliefs" is now only called once.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
StateVarStructure = [];
if nargout>1;
   BeliefStatus = 1;
end;
if nargout>2;
   TransStatus = 1;
end;
%
if nargout<=3;
   %
   % Make sure that the ModelParameter structure takes the correct values.
   % This is not necessary to do when initial values are given to the function.
   % In that case, more substantial checking is required!
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   %
   % copy files to the tmp folder
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
      if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
else;
   status = 1;
end;
%
% check if we should also copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>3;
      status = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% See if we can solve the model
%
if nargout<=3;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   if MatlabNumber>=8.0;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch ME;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError(ME)];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError(ME)];
         end;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   else;
      try;
         if DSGEModel.ModelSolver==1;
            [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
         elseif DSGEModel.ModelSolver==2;
            [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==3;
            [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
         elseif DSGEModel.ModelSolver==4;
            [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      catch;
         if exist('mcode')>0;
            status = mcode;
         else;
            status = 0;
         end;
         if DSGEModel.ModelSolver==1;
            ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==2;
            ErrorStr = ['There was an error running the Klein solver. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==3;
            ErrorStr = ['There was an error running Gensys. YADA received the following message: ' YADALastError];
         elseif DSGEModel.ModelSolver==4;
            ErrorStr = ['There was an error running own DSGE model solver. YADA received the following message: ' YADALastError];
         end;
         About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
         drawnow;
         return;
      end;
   end;
end;
if nargout>3;
   status = mcode;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   return;
end;
%
% Convert to the state equation matrices
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% obtain the measurement equation matrices
%
if MatlabNumber>=8.0;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch ME;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError(ME)];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
else;
   try;
      [A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
   catch;
      %
      % display an error message
      %
      ErrorStr = ['Error found in "' GetName(DSGEModel.MeasurementEquationFile) '.m". Message caught is: ' YADALastError];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
if length(size(H))==3;
   if size(H,3)<size(DSGEModel.Y,2);
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(size(DSGEModel.Y,2)) '). YADA has therefore aborted from the calculations.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
StateVarStructure.nonsingular = 1;
if DSGEModel.n>(rank(R)+size(B0Alt,2));
   StateVarStructure.nonsingular = 0;
end;
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the belief system matrices
%
if isempty(BeliefSys)==1;
   if DSGEModel.BeliefSystem==1;
      BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
   else;
      BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
   end;
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% call the function: 
%
[tstatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if nargout>2;
   TransStatus = tstatus;
   if TransStatus==0;
      return;
   end;
end;
%
% get estimation sample
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if DSGEModel.k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,BeliefSys.m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
%
% check if we have a training sample or not
%
if KalmanFirstObservation>1;
   StateVarStructure.TrainSample = 1;
else;
   StateVarStructure.TrainSample = 0;
end;
%
% simplify integers
%
m = BeliefSys.m;
k = DSGEModel.k;
%
% call the joint Kalman filter for state variables, belief coefficients and
% solution matrices
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      X = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(X,2);
      for j=1:k;
         ConstTest = (DSGEModel.X(j,FirstPeriod:LastPeriod)==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
StateVarStructure.Mss = Mt1;
StateVarStructure.Fss = Ft1;
StateVarStructure.Bss = Bt1;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% compute the log-likelihood, forecast and update estimates of states, belief coefficients
% and solution matrices
%
tic;
if MatlabNumber>=8.0;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
      KalmanTime = toc;
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      KalmanTime = toc;
      return;
   end;
else;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [LogLikeValue,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
      KalmanTime = toc;
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine under adaptive learning: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
if nargout>1;
   BeliefStatus = bstatus;
   if BeliefStatus==0;
      return;
   end;
end;
%
% determine the number of observations prior to removing the training sample
%
T = size(Yhat,2);
%
% now we run the smoother
%
if MatlabNumber>=8.0;
   try;
      [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,1,DSGEModel.YNaNs);
   catch ME;
      txt = ['YADA caught the following error message from the Kalman smoother under adaptive learning: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
else;
   try;
      [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1,Ptt1,Yerror,InvMSEY,H,R,Ftilde,Btilde,FirstPeriod,LastPeriod,1,DSGEModel.YNaNs);
   catch;
      txt = ['YADA caught the following error message from the Kalman smoother under adaptive learning: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
%
% deal with the measurement errors
%
StateVarStructure.wtT = [];
StateVarStructure.wtt = [];
StateVarStructure.wtNames = '';
if rank(R)>0;
   MEPositions = [];
   for i=1:length(R);
      if R(i,i)>0;
         MEPositions = [MEPositions i];
         StateVarStructure.wtT = [StateVarStructure.wtT;wtT(i,:)];
         StateVarStructure.wtt = [StateVarStructure.wtt;wtt(i,:)];
         StateVarStructure.wtNames = strvcat(StateVarStructure.wtNames,['Error in ' StringTrim(DSGEModel.VariableNames(i,:)) ' equation']);
      end;
   end;
end;
%
% check if we should do an observed variable decomposition.
%
if ObsVarDec==1;
   %
   % now we can begin with this recalling that for t the solution matrices from t-1 should be used.
   %
   n = DSGEModel.n;
   q = size(etatT,1);
   StateVarStructure.wthT = wtT;
   if DSGEModel.YNaNs==0;
      StateVarStructure.etaDecomp = zeros(n,T,q);
      StateVarStructure.DetDecomp = zeros(n,T);
      StateVarStructure.XiInit = zeros(n,T);
   else;
      StateVarStructure.etaDecomp = NaN*ones(n,T,q);
      StateVarStructure.DetDecomp = NaN*ones(n,T);
      StateVarStructure.XiInit = NaN*ones(n,T);
   end;
   StateVarStructure.XietaDecomp = zeros(r,T,q);
   StateVarStructure.XietaInit = zeros(r,T);
   if k>0;
      StateVarStructure.XiDetDecomp = zeros(r,T);
   else;
      StateVarStructure.DetDecomp = [];
      StateVarStructure.XiDetDecomp = [];
   end;
   %
   % initialize
   %
   Mt1 = Mtilde(:,:,1);
   Ft1 = Ftilde(:,:,1);
   Bt1 = Btilde(:,:,1);
   %
   % deal with the first period of the state variables decomposition
   %
   if k>0;
      DetMat = DSGEModel.X(:,FirstPeriod:LastPeriod);
      StateVarStructure.XiDetDecomp(:,1) = Mt1*DetMat(:,1);
      StateVarStructure.XietaInit(:,1) = KsitT(:,1)-StateVarStructure.XiDetDecomp(:,1)-(Bt1*etatT(:,1));
   else;
      StateVarStructure.XietaInit(:,1) = KsitT(:,1)-(Bt1*etatT(:,1));
   end;
   for j=1:q;
      StateVarStructure.XietaDecomp(:,1,j) = Bt1(:,j)*etatT(j,1);
   end;
   %
   % compute the remaining periods for the state variables decomposition
   % giving the terms due to deterministic variables and shocks
   %
   for t=2:T;
      for j=1:q;
          StateVarStructure.XietaDecomp(:,t,j) = (Ft1*StateVarStructure.XietaDecomp(:,t-1,j))+(Bt1(:,j)*etatT(j,t));
      end;
      if k>0;
         StateVarStructure.XiDetDecomp(:,t) = (Ft1*StateVarStructure.XiDetDecomp(:,t-1))+(Mt1*DetMat(:,t));
%         StateVarStructure.XietaInit(:,t) = KsitT(:,t)-StateVarStructure.XiDetDecomp(:,t)-sum(StateVarStructure.XietaDecomp(:,t,:),3);
%      else;
%         StateVarStructure.XietaInit(:,t) = KsitT(:,t)-sum(StateVarStructure.XietaDecomp(:,t,:),3);
      end;
      StateVarStructure.XietaInit(:,t) = Ft1*StateVarStructure.XietaInit(:,t-1);
      Mt1 = Mtilde(:,:,t);
      Ft1 = Ftilde(:,:,t);
      Bt1 = Btilde(:,:,t);
   end;
   %
   % now we turn to the observed variables and the terms for the shocks
   % and the impact of the deterministic variables via the belief system
   %
   if length(size(H))==2;
      if DSGEModel.YNaNs==0;
         for j=1:q;
            StateVarStructure.etaDecomp(:,:,j) = H'*StateVarStructure.XietaDecomp(:,:,j);
         end;
         if k>0;
            StateVarStructure.DetDecomp = H'*StateVarStructure.XiDetDecomp;
         end;
         StateVarStructure.XiInit = H'*StateVarStructure.XietaInit;
      else;
         for t=1:T;
            pos = (~isnan(DSGEModel.Y(:,FirstPeriod+t-1)));
            for j=1:q;
               StateVarStructure.etaDecomp(pos,t,j) = H(:,pos)'*StateVarStructure.XietaDecomp(:,t,j);
            end;
            if k>0;
               StateVarStructure.DetDecomp(pos,t) = H(:,pos)'*StateVarStructure.XiDetDecomp(:,t);
            end;
            StateVarStructure.XiInit(pos,t) = H(:,pos)'*StateVarStructure.XietaInit(:,t);
         end;
      end;
   else;
      if DSGEModel.YNaNs==0;
         for t=1:T;
            for j=1:q;
               StateVarStructure.etaDecomp(:,t,j) = H(:,:,FirstPeriod+t-1)'*StateVarStructure.XietaDecomp(:,t,j);
            end;
            if k>0;
               StateVarStructure.DetDecomp(:,t) = H(:,:,FirstPeriod+t-1)'*StateVarStructure.XiDetDecomp(:,t);
            end;
            StateVarStructure.XiInit(:,t) = H(:,:,FirstPeriod+t-1)'*StateVarStructure.XietaInit(:,t);
         end;
      else;
         for t=1:T;
            pos = (~isnan(DSGEModel.Y(:,FirstPeriod+t-1)));
            for j=1:q;
               StateVarStructure.etaDecomp(pos,t,j) = H(:,pos,FirstPeriod+t-1)'*StateVarStructure.XietaDecomp(:,t,j);
            end;
            if k>0;
               StateVarStructure.DetDecomp(pos,t) = H(:,pos,FirstPeriod+t-1)'*StateVarStructure.XiDetDecomp(:,t);
            end;
            StateVarStructure.XiInit(pos,t) = H(:,pos,FirstPeriod+t-1)'*StateVarStructure.XietaInit(:,t);
         end;
      end;
   end;
end;
%
% collect output
%
StateVarStructure.Y = DSGEModel.Y(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
if isempty(DSGEModel.X)==0;
   StateVarStructure.X = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   StateVarStructure.X = [];
end;
StateVarStructure.BeliefSys = BeliefSys;
StateVarStructure.ConstPos = ConstPos;
StateVarStructure.KeepVar = KeepVar;
StateVarStructure.Yhat = Yhat(:,KalmanFirstObservation:T);
StateVarStructure.Ksitt1 = Ksitt1(:,KalmanFirstObservation:T);
StateVarStructure.Ksitt = Ksitt(:,KalmanFirstObservation:T);
StateVarStructure.KsitT = KsitT(:,KalmanFirstObservation:T);
StateVarStructure.lnLt = lnLt(KalmanFirstObservation:T);
StateVarStructure.Ptt1 = Ptt1(:,:,KalmanFirstObservation:T);
StateVarStructure.Ptt = Ptt(:,:,KalmanFirstObservation:T);
StateVarStructure.PtT = PtT(:,:,KalmanFirstObservation:T);
StateVarStructure.etatt = etatt(:,KalmanFirstObservation:T);
StateVarStructure.etatT = etatT(:,KalmanFirstObservation:T);
StateVarStructure.betatt = betatt(:,KalmanFirstObservation:T);
StateVarStructure.Zlag = Zlag(:,KalmanFirstObservation:T);
StateVarStructure.Kgf = Kgf;
StateVarStructure.Gtilde = Gtilde;
StateVarStructure.SMatrix = SMatrix;
StateVarStructure.m = m;
StateVarStructure.f = f;
StateVarStructure.Mtilde = Mtilde(:,:,KalmanFirstObservation:T);
StateVarStructure.Ftilde = Ftilde(:,:,KalmanFirstObservation:T);
StateVarStructure.Btilde = Btilde(:,:,KalmanFirstObservation:T);
%
if ObsVarDec==1;
   StateVarStructure.etaDecomp = StateVarStructure.etaDecomp(:,KalmanFirstObservation:T,:);
   StateVarStructure.XiInit = StateVarStructure.XiInit(:,KalmanFirstObservation:T);
   if k>0;
      StateVarStructure.DetDecomp = StateVarStructure.DetDecomp(:,KalmanFirstObservation:T);
      StateVarStructure.XiDetDecomp = StateVarStructure.XiDetDecomp(:,KalmanFirstObservation:T);
   end;
   StateVarStructure.XietaDecomp = StateVarStructure.XietaDecomp(:,KalmanFirstObservation:T,:);
   StateVarStructure.XietaInit = StateVarStructure.XietaInit(:,KalmanFirstObservation:T);
   if rank(R)>0;
      StateVarStructure.wthT = StateVarStructure.wthT(:,KalmanFirstObservation:T);
   end;
end;
%
StateVarStructure.A = A;
StateVarStructure.R = R;
if length(size(H))==3;
   StateVarStructure.H = H(:,:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
else;
   StateVarStructure.H = H;
end;
StateVarStructure.F = F;
StateVarStructure.B0 = B0;
StateVarStructure.KalmanTime = KalmanTime;
StateVarStructure.LastSolution = LastSolution;

%
% end of CalculateAdaptiveLearningStateVariables.m
%
