function [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,h,NumPaths,AnnualizeData,TransData,CurrChain,IsPosterior,Weights)
% ZLBAdaptiveLearningDSGEPredictionPaths: Calculates a certain number of possible paths that the endogenous variables
%                                         can take out-of-sample for a given sample of draws from the posterior of the
%                                         parameters in the DSGE model subject to adaptive learning and when the monetary
%                                         policy rate is subject to a zero lower bound.
%
% USAGE:
%
%       [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,...
%                                                      ModelParameters,DSGEModel,CurrINI,X,h,NumPaths,AnnualizeData,TransData,...
%                                                      CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxMaxPH with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  h (integer) with the prediction horizon.
%
%                  NumPaths (integer) the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                  likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                  the PredPaths data satisfies the event, the second hold the number
%                                  of times the PredPaths data is greater than the event, the third
%                                  the number of time the data is above the upper bound, while the
%                                  fourth column holds the number of times the event has been tested.
%                                  The fifth and sixth column measure the average deviation from the
%                                  lower bound when below the lower bound and the square of this, while
%                                  the seventh and eighth columns hold similar data for the upper bound
%                                  of the event when the variables are above the upper bound.
%
%                  ObsVarMean (n x 1) vector with the steady-state values of the observed variables.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 23, 2024.
%                        This version: March 4, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-02-2024: Continued work on the function after calling "ZLBDSGEStructuralForm".
%
% * 29-02-2024: Continued work on the function after calling "AdaptiveLearningInitialValuesDSGE".
%
% * 01-03-2024: Continued work on the function when updating beliefs for the fixed parameter case.
%
% * 04-03-2024: Continued work after setting up the "ePG" and "ePGx" matrices.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
PredEventData = [];
ObsVarMean = [];
[NumDraws,NumParam] = size(thetaPostSample);
k = DSGEModel.k;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
n = DSGEModel.n;
if (AnnualizeData==1)|(TransData==1);
   if TransData==0;
      TypeStr = 'Annualized';
   else;
      TypeStr = 'Transformed';
   end;
else;
   TypeStr = 'Original';
end;
MeanPredictY = zeros(n,h);
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,600,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the unconditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
[S,KeepShocks] = RemoveRedundantColumns(B0);
if isempty(S)==1;
   txt = 'The model doesn''t have any structural shocks. YADA has aborted from the lower bound computations.';
   About(txt,'information',['ZLB Predictive Distributions' PriorHeader],200,600,CurrINI);
   return;
end;
[r,q] = size(B0);
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
FirstPeriod = DSGEModel.FirstPeriod;
LastPeriod = DSGEModel.LastPeriod;
%
% determine the steady-state interest rate path.
%
if k>0;
   XPred = X;
   if size(DSGEModel.X,2)>DSGEModel.LastPeriod;
      X = DSGEModel.X(:,DSGEModel.LastPeriod+1:size(DSGEModel.X,2));
      LastValue = X(:,size(X,2));
   else;
      X = [];
      LastValue = DSGEModel.X(:,DSGEModel.LastPeriod);
   end;
   if isempty(DSGEModel.PredictedX)==0;
      X = [X DSGEModel.PredictedX];
   end;
   Tx = size(X,2);
   Xold = X;
   if Tx>DSGEModel.ZLBSampleT+1;
      X = X(:,1:DSGEModel.ZLBSampleT+1);
   elseif Tx<DSGEModel.ZLBSampleT+1;
      X = [X LastValue*ones(1,DSGEModel.ZLBSampleT+1-Tx)];
   end;
   Rbar = A(:,DSGEModel.ZLBPosition)'*X;
else;
   Rbar = zeros(1,DSGEModel.ZLBSampleT+1);
   X = zeros(1,DSGEModel.ZLBSampleT+1)
end;
%
% setup the zero lower bound path
%
Tzlb = length(DSGEModel.ZLBData);
if Tzlb==1;
   Rzlb = DSGEModel.ZLBData*ones(h,DSGEModel.ZLBSampleT+1);
else;
   if Tzlb>DSGEModel.LastPeriod;
      Rzlb = zeros(h,Tzlb-DSGEModel.LastPeriod);
      for i=1:h;
         Rzlb(i,1:Tzlb-DSGEModel.LastPeriod-i+2) = DSGEModel.ZLBData(DSGEModel.LastPeriod+i-1:Tzlb);
      end;
      TZLB = size(Rzlb,2);
      if TZLB>=DSGEModel.ZLBSampleT+1;
         RzlbLast = Rzlb(1,size(Rzlb,2));
         % make the matrix have the correct dimension
         Rzlb = Rzlb(:,1:DSGEModel.ZLBSampleT+1);
         % replace zeros with the last value of the lower bound
         % provided the locations appear after the last nonzero
         for i=1:h;
            for j=2:DSGEModel.ZLBSampleT+1;
               if (Rzlb(i,j)==0)&(Rzlb(i,j-1)~=0);
                  Rzlb(i,j) = RzlbLast;
               end;
            end;
         end;
         %Rzlb((Rzlb==0)) = RzlbLast;
      else;
         %
         % we should add the last value in each row to fill the matrix
         %
         RzlbLast = Rzlb(1,size(Rzlb,2));
         Rzlb((Rzlb==0)) = RzlbLast;
         Rzlb = [Rzlb RzlbLast*ones(h,DSGEModel.ZLBSampleT+1-TZLB)];
      end;
   else;
      Rzlb = DSGEModel.ZLBData(Tzlb)*ones(h,DSGEModel.ZLBSampleT+1);
   end;
end;
%
% since H may involve estimated parameters we save a copy of Rzlb for later use
%
RzlbMat = Rzlb;
%
% Transform the steady-state interest rate and the lower bound to the same units
% as the unrestricted monetary policy rate
%
if length(size(H))==2;
   Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   Rzlb = (1/Hvalue)*Rzlb;
   Rbar = (1/Hvalue)*Rbar;
else;
   Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
   Th = length(Hvalue);
   if Th>DSGEModel.ZLBSampleT;
      Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
   elseif Th<DSGEModel.ZLBSampleT;
      Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
   end;
   Rzlb = Rzlb./Hvalue;
   Rbar = Rbar./Hvalue;
end;
%
% setup the parameters that determine the projected restricted policy rate
%
e_pr = zeros(r+1,1);
e_pr(DSGEModel.RtildePosition) = 1;
%
% Determine the belief system matrices
%
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
% Compute the transformation of the structural from that we use when solving the model
%
r = length(DSGEModel.StateVariablePositions);
f = length(DSGEModel.ForwardVariablePositions);
m = BeliefSys.m;
q = size(B0,2);
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% reshuffle the equations to separate the restricted from the unrestricted policy rate
%
[Hlag,H0,Hexp,D,DA,MA] = ZLBDSGEStructuralForm(Hlag,H0,Hexp,D,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType);
%
% solve the model with the new matrices
%
[F,B0,BA,mcode] = SolveExtendedDSGEModel(Hlag,H0,Hexp,D,DA,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
%
% rewrite the G-matrices for the belief equations that match the position of the unrestricted policy rate being last and the restricted policy rate
% being in position "DSGEModel.RtildePosition". The latter variable should have zeros in the belief equations, while the former should have the
% coefficients from "DSGEModel.RtildePosition"
%
G = ZLBBeliefSystem(BeliefSys.G,m,k,r,DSGEModel.RtildePosition,DSGEModel.ZLBRuleType);
%
SMatrix = eye(r+1);
Sperp = SMatrix(:,setdiff((1:r+1),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
T = LastPeriod-(FirstPeriod+KalmanFirstObservation-1)+1;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
%
% call the transformation function
%
[TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1,InvH0H1star] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1,InvH0H1star] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% compute the solution matrix BAt1 on the anticipated shocks
%
BAt1 = InvH0H1star*DA;
%
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      Xhist = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(Xhist,2);
      for j=1:k;
         ConstTest = (Xhist==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
%
% we need to fix the H-matrix when using the extended form of the model
% by adding zeros to 
%
HOld = H;
if length(size(H))==2;
   H = [H;zeros(1,n)];
else;
   HT = zeros(size(H));
   for t=1:size(H,3);
      HT(:,:,t) = [H(:,:,t);zeros(1,n)];
   end;
   H = HT;
end;
%
% run the Kalman filter
%
if CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0;
   if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
      [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
   else;
      [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
   LastSolution.InvMatT1 = InvH0H1star;
   LastSolution.InvMatT = InvH0H1star;
end;
%
% determine initial conditions for the forecasts
%
T = size(Ksitt,2);
KsiTT = Ksitt(:,T);
PTT = Ptt(:,:,T);
PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
%
% write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
%
[Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(Mtilde(:,:,T),Ftilde(:,:,T),Btilde(:,:,T),LastSolution.InvMatT1*DA,MA);
%
% deal with deterministic variables
%
if k>0;
   Ax = A'*X;
else;
   Ax = zeros(n,h);
   X = zeros(r,h)
end;
%
% setup projections matrices for the restricted policy rate
%
% each row in ePG is the row vector multiplied by V(t) for tau=0,1,...,DSGEModel.ZLBSampleT
% each element in ePGx is the deterministic term of the projection
%
ePG = zeros(DSGEModel.ZLBSampleT+1,size(Gt,2));
ePGx = zeros(DSGEModel.ZLBSampleT+1,1);
if DSGEModel.FixedBeliefsForProj==1;
   ePG(1,:) = e_pr'*Pt(1:r+1,:);
   xbart = zeros(size(Mt,1),1);
   ePGx(1) = e_pr'*Mtilde*X(:,1);
   for tau=1:DSGEModel.ZLBSampleT;
      ePG(tau+1,:) = ePG(tau,:)*Gt;
      xbart = (Mt*X(:,tau))+Gt*xbart;
      ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
   end;
else;
   %
   % beliefs are now updated, take the values of the
   % solution matrices from period T that have been updated for T+1
   %
   MTiT = LastSolution.MTT;
   FTiT = LastSolution.FTT;
   BTiT = LastSolution.BTT;
   betaTiT = LastSolution.betaTT;
   InvMatT = LastSolution.InvMatT;
   %
   % write down the solution matrices for projections using ZLB format, with Mt, Gt and C1
   %
   [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
   ePG(1,:) = e_pr'*Pt(1:r+1,:);
   xbart = zeros(size(Mt,1),1);
   ePGx(1) = e_pr'*Mtilde*X(:,1);
   for tau=1:DSGEModel.ZLBSampleT;
      %
      % update betaTiT
      %
      betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
      %
      % update the solution matrices
      %
      if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
         [MTiT,FTiT,BTiT,InvMatT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
      else;
         [MTiT,FTiT,BTiT,InvMatT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
      %
      % obtain the new projection matrices
      %
      [Mt,Gt,C1,Mtilde,Pt] = ZLBProjectionMatrices(MTiT,FTiT,BTiT,InvMatT*DA,MA);
      ePG(tau+1,:) = ePG(tau,:)*Gt;
      xbart = (Mt*X(:,tau))+Gt*xbart;
      ePGx(tau+1) = e_pr'*((Mtilde*X(:,tau+1))+(Pt*xbart));
   end;
end;
%
% compute the mean predictions
%
ZLBYMean = zeros(n,h);
etaMean = zeros(q,h);
KsiTMean = KsiTT;
%
At = zeros(DSGEModel.ZLBSampleT+1,1);
Xt = [etaMean(:,1);KsiTMean];
t = 1;
disp(ePGx)
%for t=1:h;
   %[At,status,AlgorithmInformation] = ForwardBackShootingAlgorithmAdaptiveLearning(Xt,At,ePG,ePGx,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);






%end;



%
% set up the data matrices
%
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   Xdata = [];
end;
if strcmp(DSGEModel.DataFrequency,'q')==1;
   Yinit = DSGEModel.Y(:,LastPeriod-2:LastPeriod);
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   Yinit = DSGEModel.Y(:,LastPeriod-10:LastPeriod);
else;
   Yinit = [];
end;
maxYinit = max(max(isnan(Yinit)));
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
if TransData==1;
   Yinitial = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
   %
   % try to covert: h
   %
   YTmp = [Yinitial DSGEModel.Y(:,DSGEModel.LastPeriod)*ones(1,h)];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   maxYinitial = max(max(isnan(Yinitial)));
end;












%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
%if DSGEModel.ShowProgress==1;
%   ProgressStructure.title = ['Progress for ' num2str(NumDraws*NumPaths,'%0.0f') ' Prediction Paths'];
%   ProgressStructure.facecolor = CurrINI.progress_facecolor;
%   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
%   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
%   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
%   ProgressStructure.stop = 0;
%   ProgressStructure.clock = DSGEModel.ShowProgressClock;
%   ProgressStructure.label = 'Path Number:';
%   %
%   ProgressStructure.name = ['ZLB Predictive Distributions' PriorHeader];
%   ProgressStructure.CurrINI = CurrINI;
%   WaitHandle = ProgressDLG(0,ProgressStructure);
%   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
%   drawnow;
%else;
%   txt = ['Please wait while YADA computes predictive distributions under adaptive learning and subject to the ZLB for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
%   WaitHandle = WaitDLG(txt,'information',['ZLB Predictive Distributions ' PriorHeader ' - ' TypeStr ' Data'],600,150,CurrINI,0);
%   WaitControls = get(WaitHandle,'UserData');
%end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;














%
% close the wait dialog
%
%if DSGEModel.ShowProgress==1;
%   if IsHandle(WaitHandle)==1;
%      set(WaitHandle,'UserData','done');
%      close(WaitHandle);
%      drawnow;
%      pause(0.02);
%   end;
%else;
%   set(WaitControls.text,'UserData','done');
%   delete(WaitHandle);
%   drawnow;
%   pause(0.02);
%end;
if StopEstimation==1;
   return;
end;





%
% end of ZLBAdaptiveLearningDSGEPredictionPaths.m
%
