function [logLikeValue,lnL,PredictedVars,mcode,status] = logCondLikeAdaptiveLearningDSGE(ModelParameters,r,f,SMatrix,Sperp,BeliefSys,Gtilde,Kgf,x,xbar,YData,X,h,DSGEModel,AIMData,OrderQZ,NumWorkers)
% logCondLikeAdaptiveLearningDSGE: Computes the joint conditional likelihood for h-step-ahead forecasts of a DSGE model
%                                  under adaptive learning.
%
% USAGE:
%
%       [logLikeValue,lnL,PredictedVars,mcode,status] = logCondLikeAdaptiveLearningDSGE(ModelParameters,r,f,...
%                                                                 SMatrix,Sperp,BeliefSys,Gtilde,Kgf,x,xbar,...
%                                                                 YData,X,h,DSGEModel,AIMData,OrderQZ,NumWorkers)
%
% REQUIRED INPUT:  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  r (integer) with the number of state variables.
%
%                  f (integer) with the number of forward looking variables.
%
%                  SMatrix (matrix) of dimension r x f that selects the forward looking variables among the state
%                                  variables.
%
%                  Sperp (matrix) of dimension r x (r-f) which select the non-forward looking variables.
%
%                  BeliefSys (structure) with fields "m" an integer denoting the maximum number of lags of the state
%                                  variables in the belief equations; "Name" a string with the  name of the belief system;
%                                  and G a cell array with f matrices, one for each forward looking variable of dimension
%                                  (k+r*m) x g_j where g_j is the number of explanatory variables in the belief equation
%                                  for forward looking variable j.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                                  of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                                  A is a g x f matrix.
%
%                  x (matrix) of dimension k x T with the sample values of the deterministic variables.
%
%                  xbar (vector) of dimension k with unit element for a constant and zeros elsewhere.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  h (integer) with the prediction horizon.
%
%                  DSGEModel (structure) whose fields contain all required information
%                                  about the model, such as input files, variable and sample
%                                  data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                                  The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: logLikeValue (vector) of dimension h with the value of the log conditional likelihood for
%                                  joint h-step-ahead forecasts.
%
% OPTIONAL OUTPUT: lnL (scalar) with the value of the log-likelhood over the historical sample.
%
%                  PredictedVars (cell array) with the positions of the predicted variables in the vector of observed
%                                  variables for each prediction period.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not.
%                                  mcode is 1 when unique, 0 if not computed, and various other values when
%                                  there are problems with the solution. Specifically:
%
%                                     mcode = 2:      Roots not correctly computed by real_schur
%                                     mcode = 3:      Too many big roots
%                                     mcode = 35:     Too many big roots, and q(:,right) is singular
%                                     mcode = 4:      Too few big roots
%                                     mcode = 45:     Too few big roots, and q(:,right) is singular
%                                     mcode = 5:      q(:,right) is singular
%                                     mcode = 61:     Too many exact shiftrights
%                                     mcode = 62:     Too many numeric shiftrights
%                                     mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                     cannot be calculated.
%                                     mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                     numbers.
%                                     mcode = -1:     No stable solution.
%                                     mcode = -2:     Too many large generalized eigenvalues.
%                                     mcode = -3:     Too few large generalized eigenvalues.
%                                     else            Return code not properly specified
%
%                  status (integer) Indicates if the log-likelihood was calculated (status=0), or not (status=1).
%                                  In addition, status can be -1 if ths forecast covariance matrix is not positive
%                                  definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 4, 2021.
%                        This version: March 13, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-05-2021: Continued work on the function.
%
% * 11-05-2021: Finioshed the function with the updated belief coefficients which take
%               the actual values into account through the Kalman filter.
%
% * 03-01-2022: Updated the documentation.
%
% * 21-02-2022: Changed eval calls to feval.
%
% * 22-02-2022: Changed the last eval calls to feval.
%
% * 06-01-2023: Updated the documentation.
%
% * 24-10-2023: The function now supports the fixed belief Kalman filter.
%
% * 12-03-2024: Updated the documentation.
%
% * 13-03-2024: The logical function "CheckAdaptiveLearningFixedBeliefs" is now only called once.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

logLikeValue = zeros(h,1);
if nargout>1;
   PredictedVars = [];
   PredVars = [];
   lnL = 0;
   mcode = 0;
   status = 0;
end;
%
% See if we can solve the model
%
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==4;
   [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
if mcode~=1;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
   if NumWorkers<=1;
      save(FixFilePath([pwd '\tmp\AiMErrorData.mat']),'mcode','ModelEigenvalues');
   end;
   return;
else;
   if (NumWorkers<=1)&(FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1);
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
end;
%
% transform into state space
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
[B0Alt,KeepVar] = RemoveRedundantColumns(B0);
ModelParameters.YADA.F = F;
ModelParameters.YADA.B0 = B0Alt;
%
% Evaluate the measurement equation file
%
[A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,BeliefSys.m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
%
% call the transformation fucntion
%
[TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
if TransStatus==0;
   status = 1;
   return;
end;
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,BeliefSys.m,DSGEModel.k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% run the Kalman filter
%
if MatlabNumber>=8.0;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(BeliefSys.m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),BeliefSys.m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(BeliefSys.m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
   catch ME;
      status = 1;
      return;
   end;
else;
   try;
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(BeliefSys.m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),BeliefSys.m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(BeliefSys.m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
   catch;
      status = 1;
      return;
   end;
end;
%
if bstatus==0;
   status = 1;
   return;
end;
%
%
% initial solution matrices
%
T = size(Ksitt1,2);
MT1 = Mtilde(:,:,T);
FT1 = Ftilde(:,:,T);
BT1 = Btilde(:,:,T);
ksijoint = Ksitt(:,T);
Pjoint = Ptt(:,:,T);
%
% compute the height of the predictive densities
%
if DSGEModel.k>0;
   Ax = A'*X;
else;
   Ax = zeros(DSGEModel.n,h);
   X = zeros(size(Mtilde,1),h)
end;
YData = YData(DSGEModel.PredictionVariables,:);
log2pi = log(2*pi);
%
% compute the forecasts
%
if DSGEModel.FixedBeliefsForProj==1;
   %
   % fixed beliefs
   %
   %
   % initial forecasts of ksi and P: these are t+i|t+i-1
   %
   ksijoint = (MT1*X(:,1))+(FT1*ksijoint);
   Q = BT1*BT1';
   Pjoint = (FT1*Pjoint*FT1')+Q;
   %
   if length(size(H))==2;
      Ht = H(:,DSGEModel.PredictionVariables);
      Rt = R(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      for i=1:h;
         %
         % joint density
         %
         Yhatjoint = Ax(:,i)+(H'*ksijoint);
         SigmaYjoint = (H'*Pjoint*H)+R;
         %
         % select the subset
         %
         Yhatjoint = Yhatjoint(DSGEModel.PredictionVariables);
         SigmaYjoint = SigmaYjoint(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
         %
         % check for missing obs
         %
         pos = ~isnan(YData(:,i));
         nstar = sum(pos);
         %
         if nstar>0;
            Ystarerror = YData(pos,i)-Yhatjoint(pos);
            SigmaYstar = SigmaYjoint(pos,pos);
            logLikeValue(i) = -((nstar*log2pi)+LogDet(SigmaYstar)+((Ystarerror'/SigmaYstar)*Ystarerror))/2;
            PredVars{i} = DSGEModel.PredictionVariables(pos);
            %
            % now we compute the Kalman filter updates
            %
            if i<h;
               KalmanGain = FT1*Pjoint*(Ht(:,pos)/SigmaYstar);
               ksijoint = (MT1*X(:,i+1))+(FT1*ksijoint)+(KalmanGain*Ystarerror);
               FDev = FT1-(KalmanGain*Ht(:,pos)');
               Pjoint = (FDev*Pjoint*FDev')+(KalmanGain*Rt*KalmanGain')+Q;
            end;
         else;
            %
            % Kalman filter updates
            %
            PredVars{i} = [];
            if i<h;
               ksijoint = (MT1*X(:,i+1))+(FT1*ksijoint);
               Pjoint = (FT1*Pjoint*FT1')+Q;
            end;
         end;
      end;
   else;
      Rt = R(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      for i=1:h;
         %
         % joint density
         %
         Ht = H(:,:,DSGEModel.LastPeriod+i);
         Yhatjoint = Ax(:,i)+(Ht'*ksijoint);
         SigmaYjoint = (Ht'*Pjoint*Ht)+R;
         %
         % select the subset
         %
         Yhatjoint = Yhatjoint(DSGEModel.PredictionVariables);
         SigmaYjoint = SigmaYjoint(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
         %
         % check for missing obs
         %
         pos = ~isnan(YData(:,i));
         nstar = sum(pos);
         %
         if nstar>0;
            Ystarerror = YData(pos,i)-Yhatjoint(pos);
            SigmaYstar = SigmaYjoint(pos,pos);
            logLikeValue(i) = -((nstar*log2pi)+LogDet(SigmaYstar)+((Ystarerror'/SigmaYstar)*Ystarerror))/2;
            PredVars{i} = DSGEModel.PredictionVariables(pos);
            %
            % now we compute the Kalman filter updates
            %
            if i<h;
               Ht = Ht(:,DSGEModel.PredictionVariables);
               KalmanGain = FT1*Pjoint*(Ht(:,pos)/SigmaYstar);
               ksijoint = (MT1*X(:,i+1))+(FT1*ksijoint)+(KalmanGain*Ystarerror);
               FDev = FT1-(KalmanGain*Ht(:,pos)');
               Pjoint = (FDev*Pjoint*FDev')+(KalmanGain*Rt*KalmanGain')+Q;
            end;
         else;
            %
            % Kalman filter updates
            %
            PredVars{i} = [];
            if i<h;
               ksijoint = F*ksijoint;
               Pjoint = (F*Pjoint*F')+Q;
            end;
         end;
      end;
   end;
else;
   %
   % belief coefficients are projected forward
   %
   % 1. setup matrices to store certain caluculations in
   %
   % we here work with t|t
   %
   q = size(BT1,2);
   k = DSGEModel.k;
   n = DSGEModel.n;
   In = eye(n);
   m = BeliefSys.m;
   g = length(betavec);
   MtildeTmp = zeros(r,k,h+1);
   FtildeTmp = zeros(r,r,h+1);
   BtildeTmp = zeros(r,q,h+1);
   KsittTmp = zeros(r,h+1);
   KsittTmp(:,1) = Ksitt(:,T);
   MtildeTmp(:,:,1) = Mtilde(:,:,T);
   FtildeTmp(:,:,1) = Ftilde(:,:,T);
   BtildeTmp(:,:,1) = Btilde(:,:,T);
   InvMSEYTmp = zeros(n,n,h+1);
   InvMSEYTmp(:,:,1) = InvMSEY(:,:,T);
   YerrorTmp = NaN*zeros(n,h+1);
   YerrorTmp(:,1) = Yerror(:,T);
   ZlagTmp = zeros(k+(r*m),h);
   if k>0;
      ZlagTmp(1:k,:) = X;
   end;
   Ksitt1Tmp = zeros(r,h+1);
   Ksitt1Tmp(:,1) = Ksitt1(:,T);
   Ptt1Tmp = zeros(r,r,h+1);
   Ptt1Tmp(:,:,1) = Ptt1(:,:,T);
   %
   % initial forecasts of ksi and P: these are T+1|T
   %
   ksijoint = (MT1*X(:,1))+(FT1*ksijoint);
   Pjoint = (FT1*Pjoint*FT1')+(BT1*BT1');
   %
   if length(size(H))==2;
      Ht = H(:,DSGEModel.PredictionVariables);
      Rt = R(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      for i=1:h;
         MT1 = MtildeTmp(:,:,i);
         FT1 = FtildeTmp(:,:,i);
         BT1 = BtildeTmp(:,:,i);
         Q = BT1*BT1';
         %
         % joint density
         %
         Yhatjoint = Ax(:,i)+(H'*ksijoint);
         SigmaYjoint = (H'*Pjoint*H)+R;
         InvMSEYTmp(:,:,i+1) = In/SigmaYjoint;
         Ksitt1Tmp(:,i+1) = ksijoint;
         Ptt1Tmp(:,:,i+1) = Pjoint;
         %
         % select the subset
         %
         Yhatjoint = Yhatjoint(DSGEModel.PredictionVariables);
         SigmaYjoint = SigmaYjoint(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
         %
         % check for missing obs
         %
         pos = ~isnan(YData(:,i));
         nstar = sum(pos);
         if nstar>0;
            Ystarerror = YData(pos,i)-Yhatjoint(pos);
            YerrorTmp(pos,1+i) = Ystarerror;
            SigmaYstar = SigmaYjoint(pos,pos);
            logLikeValue(i) = -((nstar*log2pi)+LogDet(SigmaYstar)+((Ystarerror'/SigmaYstar)*Ystarerror))/2;
            PredVars{i} = DSGEModel.PredictionVariables(pos);
            %
            % now we compute the Kalman filter updates
            %
            if i<h;
               %
               % update for T+i|T+i
               %
               ksitt = ksijoint+(Pjoint*(Ht(:,pos)/SigmaYstar)*Ystarerror);
               KsittTmp(:,i+1) = ksitt;
               KalmanGain = FT1*Pjoint*(Ht(:,pos)/SigmaYstar);
               ksijoint = (MT1*X(:,i+1))+(FT1*ksijoint)+(KalmanGain*Ystarerror);
               FDev = FT1-(KalmanGain*Ht(:,pos)');
               Pjoint = (FDev*Pjoint*FDev')+(KalmanGain*Rt*KalmanGain')+Q;
            end;
         else;
            %
            % Kalman filter updates
            %
            PredVars{i} = [];
            KsittTmp(:,i+2) = ksijoint;
            if i<h;
               ksijoint = (MT1*X(:,i+1))+(FT1*ksijoint);
               Pjoint = (FT1*Pjoint*FT1')+Q;
            end;
         end;
         %
         % now we have to update the beliefs
         %
         if i==1;
            MT1 = LastSolution.MTT;
            FT1 = LastSolution.FTT;
            BT1 = LastSolution.BTT;
            betattLast = LastSolution.betaTT;
            RttLast = LastSolution.RTT;
            M = LastSolution.M;
            %
            % update the belief coefficients
            %
            betatt1Last =  betavec+(M*(betattLast-betavec));
            Rtt1Last = (M*RttLast*M)+Sigmaeps;
            MtildeTmp(:,:,2) = MT1;
            FtildeTmp(:,:,2) = FT1;
            BtildeTmp(:,:,2) = BT1;
         elseif (i>1)&(i<h);
            %
            % compute forward variable update estimates at t
            %
            ztf = SMatrix'*KsittTmp(:,i+1);
            %
            % collect lagged update state variables
            %
            ZlagTmp(k+1:k+r,i) = KsittTmp(:,i);
            %
            % check if we have a second lag
            %
            if m==2;
               if DSGEModel.NoSmoothedStatesForBeliefs==0;
                  %
                  % compute r(t-1|t-1)
                  %
                  pos1 = (~isnan(YerrorTmp(:,i)));
                  nt1 = sum(pos1);
                  pos2 = (~isnan(YerrorTmp(:,i-1)));
                  nt2 = sum(pos2);
                  if nt1>0;
                     rt1 = H(:,pos1)*InvMSEYTmp(pos1,pos1,i)*YerrorTmp(pos1,i);
                  else;
                     rt1 = zeros(r,1);
                  end;
                  %
                  % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                  %
                  if nt2>0;
                     Kt3 = FtildeTmp(:,:,i-1)*Ptt1Tmp(:,:,i-1)*H(:,pos2)*InvMSEYTmp(pos2,pos2,i-1);
                     %
                     % compute r(t-2|t-1)
                     %
                     rt2 = (H(:,pos2)*InvMSEYTmp(pos2,pos2,i-1)*YerrorTmp(pos2,i-1))+((FtildeTmp(:,:,i-1)-Kt3*H(:,pos2)')'*rt1);
                  else;
                     rt2 = FtildeTmp(:,:,i-1)'*rt1;
                  end;
                  %
                  % compute ksi(t-2|t-1)
                  %
                  ZlagTmp(k+r+1:k+(r*2),i) = Ksitt1Tmp(:,i-1)+(Ptt1Tmp(:,:,i-1)*rt2);
               else;
                  %
                  % we use the second lag of Ksitt
                  %
                  ZlagTmp(k+r+1:k+(r*2),i) = KsittTmp(:,i-1)
               end;
            end;
            %
            % retrieve qt1 transpose from Zlag
            %
            qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,i),f,g);
            %
            % update belief coefficients
            %
            Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
            if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
               InvSigmazf = Sigmazf\If;
            else;
               [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
               f1 = size(Lambda,1);
               InvLambda = diag(ones(f1,1)./diag(Lambda));
               InvSigmazf = S*InvLambda*S';
            end;
            betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
            RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
            %
            % forecast
            %
            betatt1Last = betavec+(M*(betattLast-betavec));
            Rtt1Last = (M*RttLast*M)+Sigmaeps;
            %
            % solve the model with betatt for use in the next period
            %
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MT1,FT1,BT1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
            else;
               [MT1,FT1,BT1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MtildeTmp(:,:,i+1) = MT1;
            FtildeTmp(:,:,i+1) = FT1;
            BtildeTmp(:,:,i+1) = BT1;
         end;
      end;
   else;
      Rt = R(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      for i=1:h;
         MT1 = MtildeTmp(:,:,i);
         FT1 = FtildeTmp(:,:,i);
         BT1 = BtildeTmp(:,:,i);
         Q = BT1*BT1';
         Ht = H(:,DSGEModel.PredictionVariables,DSGEModel.LastPeriod+i);
         %
         % joint density
         %
         Yhatjoint = Ax(:,i)+(H(:,:,DSGEModel.LastPeriod+i)'*ksijoint);
         SigmaYjoint = (H(:,:,DSGEModel.LastPeriod+i)'*Pjoint*H(:,:,DSGEModel.LastPeriod+i))+R;
         InvMSEYTmp(:,:,i+1) = In/SigmaYjoint;
         Ksitt1Tmp(:,i+1) = ksijoint;
         Ptt1Tmp(:,:,i+1) = Pjoint;
         %
         % select the subset
         %
         Yhatjoint = Yhatjoint(DSGEModel.PredictionVariables);
         SigmaYjoint = SigmaYjoint(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
         %
         % check for missing obs
         %
         pos = ~isnan(YData(:,i));
         nstar = sum(pos);
         if nstar>0;
            Ystarerror = YData(pos,i)-Yhatjoint(pos);
            YerrorTmp(pos,1+i) = Ystarerror;
            SigmaYstar = SigmaYjoint(pos,pos);
            logLikeValue(i) = -((nstar*log2pi)+LogDet(SigmaYstar)+((Ystarerror'/SigmaYstar)*Ystarerror))/2;
            PredVars{i} = DSGEModel.PredictionVariables(pos);
            %
            % now we compute the Kalman filter updates
            %
            if i<h;
               %
               % update for T+i|T+i
               %
               ksitt = ksijoint+(Pjoint*(Ht(:,pos)/SigmaYstar)*Ystarerror);
               KsittTmp(:,i+1) = ksitt;
               KalmanGain = FT1*Pjoint*(Ht(:,pos)/SigmaYstar);
               ksijoint = (MT1*X(:,i+1))+(FT1*ksijoint)+(KalmanGain*Ystarerror);
               FDev = FT1-(KalmanGain*Ht(:,pos)');
               Pjoint = (FDev*Pjoint*FDev')+(KalmanGain*Rt*KalmanGain')+Q;
            end;
         else;
            %
            % Kalman filter updates
            %
            PredVars{i} = [];
            KsittTmp(:,i+2) = ksijoint;
            if i<h;
               ksijoint = (MT1*X(:,i+1))+(FT1*ksijoint);
               Pjoint = (FT1*Pjoint*FT1')+Q;
            end;
         end;
         %
         % now we have to update the beliefs
         %
         if i==1;
            MT1 = LastSolution.MTT;
            FT1 = LastSolution.FTT;
            BT1 = LastSolution.BTT;
            betattLast = LastSolution.betaTT;
            RttLast = LastSolution.RTT;
            M = LastSolution.M;
            %
            % update the belief coefficients
            %
            betatt1Last =  betavec+(M*(betattLast-betavec));
            Rtt1Last = (M*RttLast*M)+Sigmaeps;
            MtildeTmp(:,:,2) = MT1;
            FtildeTmp(:,:,2) = FT1;
            BtildeTmp(:,:,2) = BT1;
         elseif (i>1)&(i<h);
            %
            % compute forward variable update estimates at t
            %
            ztf = SMatrix'*KsittTmp(:,i+1);
            %
            % collect lagged update state variables
            %
            ZlagTmp(k+1:k+r,i) = KsittTmp(:,i);
            %
            % check if we have a second lag
            %
            if m==2;
               if DSGEModel.NoSmoothedStatesForBeliefs==0;
                  %
                  % compute r(t-1|t-1)
                  %
                  pos1 = (~isnan(YerrorTmp(:,i)));
                  nt1 = sum(pos1);
                  pos2 = (~isnan(YerrorTmp(:,i-1)));
                  nt2 = sum(pos2);
                  if nt1>0;
                     rt1 = H(:,pos1,DSGEModel.LastPeriod+i-1)*InvMSEYTmp(pos1,pos1,i)*YerrorTmp(pos1,i);
                  else;
                     rt1 = zeros(r,1);
                  end;
                  %
                  % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                  %
                  if nt2>0;
                     Kt3 = FtildeTmp(:,:,i-1)*Ptt1Tmp(:,:,i-1)*H(:,pos2,DSGEModel.LastPeriod+i-2)*InvMSEYTmp(pos2,pos2,i-1);
                     %
                     % compute r(t-2|t-1)
                     %
                     rt2 = (H(:,pos2,DSGEModel.LastPeriod+i-2)*InvMSEYTmp(pos2,pos2,i-1)*YerrorTmp(pos2,i-1))+((FtildeTmp(:,:,i-1)-Kt3*H(:,pos2,DSGEModel.LastPeriod+i-2)')'*rt1);
                  else;
                     rt2 = FtildeTmp(:,:,i-1)'*rt1;
                  end;
                  %
                  % compute ksi(t-2|t-1)
                  %
                  ZlagTmp(k+r+1:k+(r*2),i) = Ksitt1Tmp(:,i-1)+(Ptt1Tmp(:,:,i-1)*rt2);
               else;
                  %
                  % we use the second lag of Ksitt
                  %
                  ZlagTmp(k+r+1:k+(r*2),i) = KsittTmp(:,i-1)
               end;
            end;
            %
            % retrieve qt1 transpose from Zlag
            %
            qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,i),f,g);
            %
            % update belief coefficients
            %
            Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
            if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
               InvSigmazf = Sigmazf\If;
            else;
               [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
               f1 = size(Lambda,1);
               InvLambda = diag(ones(f1,1)./diag(Lambda));
               InvSigmazf = S*InvLambda*S';
            end;
            betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
            RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
            %
            % forecast
            %
            betatt1Last = betavec+(M*(betattLast-betavec));
            Rtt1Last = (M*RttLast*M)+Sigmaeps;
            %
            % solve the model with betatt for use in the next period
            %
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MT1,FT1,BT1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
            else;
               [MT1,FT1,BT1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            MtildeTmp(:,:,i+1) = MT1;
            FtildeTmp(:,:,i+1) = FT1;
            BtildeTmp(:,:,i+1) = BT1;
         end;
      end;
   end;
end;
%
% we get the joint log predictive likelihood by simply accumulating the marginal
% log predictive likelihoods
%
logLikeValue = cumsum(logLikeValue);
if nargout>1;
   PredictedVars = PredVars;
end;

%
% end of logCondLikeAdaptiveLearningDSGE.m
%
