function DoneCalc = AdaptiveLearningDSGEPredictiveMoments(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,h,TotalDraws,CurrChain,IsPosterior,Weights,SmpStr,ComputePITMoment)
% AdaptiveLearningDSGEPredictiveMoments: Computes the predictive mean and covariance matrix of a DSGE model with
%                                        adaptive learning.
%
% USAGE:
%
%       DoneCalc = AdaptiveLearningDSGEPredictiveMoments(thetaPostSample,thetaMode,thetaPositions,ModelParameters, ...
%                                        DSGEModel,CurrINI,X,h,TotalDraws,CurrChain,IsPosterior,Weights,SmpStr,...
%                                        ComputePITMoment)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or prior
%                                  distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) with the posterior mode or initial value of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                  needs the former field to know which parameter name corresponds to
%                                  position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                  the prediction paths.
%
%                  h (integer) with the prediction horizon.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                  if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
%                  SmpStr (string) which reflects the first and last period in the prediction sample.
%
%                  ComputePITMoment (boolean) that takes the value 1 if moments useable for the PITs should
%                                 be computed and 0 otherwise.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 14, 2021.
%                        This version: March 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 17-05-2021: Corrected a few mistakes to take into account how Ksi_T+i|T develops.
%
% * 18-05-2021: Finished the function with back and nowcasts. Added the update beliefs
%               and time-varying H-matrix.
%
% * 20-05-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 21-02-2022: Changed eval call to feval.
%
% * 13-05-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 24-10-2023: The function now supports the fixed belief Kalman filter.
%
% * 28-02-2024: Updated the documentation.
%
% * 13-03-2024: The logical function "CheckAdaptiveLearningFixedBeliefs" is now only called once.
%
% * 19-03-2024: Added mean calculations at the mode value and checks to make sure the predictions are
%               reasonable.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
[NumDraws,NumParam] = size(thetaPostSample);
k = DSGEModel.k;
n = DSGEModel.n;
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
LastPeriod = DSGEModel.LastPeriod;
FirstPeriod = DSGEModel.FirstPeriod;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy the belief system file
%
if DSGEModel.BeliefSystem==2;
   [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   else;
      SetLogText('Missing AIM data file. Calculations aborted.');
   end;
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if isempty(CurrINI.Sequence)==1;
      About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   else;
      SetLogText('Error when trying to solve the DSGE model. Calculations aborted.');
   end;
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   if isempty(CurrINI.Sequence)==1;
      txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],200,500,CurrINI);
   else;
      SetLogText(strvcat(['The ' SolverStr ' solver provided the return code:'],mcodeStr,'Calculations aborted.'));
   end;
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      if isempty(CurrINI.Sequence)==1;
         ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the marginal predictive moments.'];
         About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      else;
         SetLogText(strvcat('Number of time periods for H(t) is less than the number of observations.','Calculations aborted.'));
      end;
      return;
   end;
end;
if DSGEModel.BeliefSystem==1;
   BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
else;
   BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
end;
%
r = length(DSGEModel.StateVariablePositions);
q = size(B0,2);
f = length(DSGEModel.ForwardVariablePositions);
m = BeliefSys.m;
SMatrix = eye(r);
Sperp = SMatrix(:,setdiff((1:r),DSGEModel.ForwardVariablePositions));
SMatrix = SMatrix(:,DSGEModel.ForwardVariablePositions);
%
% get the structural form matrices
%
[Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
%
% Determine the first observation to use when computing log-likelihood
%
KalmanFirstObservation = DSGEModel.KalmanFirstObservation;
%
% Setup all the basic data matrix for the deterministic variables
% used for initializing Sigmax
%
if DSGEModel.k>0
   x = DSGEModel.X(:,FirstPeriod+KalmanFirstObservation-1:LastPeriod);
else;
   x = [];
end;
%
% Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
%
[betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue);
ConstPos = 0;
if k>0;
   ConstPos = LocateStr(DSGEModel.XVariableNames,'const');
   if ConstPos==0;
      %
      % we will now check if X has a unit vector
      %
      Xhist = DSGEModel.X(:,FirstPeriod:LastPeriod);
      T = size(Xhist,2);
      for j=1:k;
         ConstTest = (Xhist==ones(1,T));
         if sum(ConstTest)==T;
            ConstPos = j;
            break;
         end;
      end;
   end;
   xbar = zeros(k,1);
   xbar(ConstPos) = 1;
else;
   xbar = [];
end;
if k>0;
   Xdata = DSGEModel.X(:,FirstPeriod:LastPeriod);
else;
   Xdata = [];
end;
T = LastPeriod-(FirstPeriod+KalmanFirstObservation-1)+1;
%
% call the transformation function
%
[TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
%
% compute initial values for the solution matrices
%
if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
   [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
else;
   [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
%
% check if the adaptive learning parameters allows for updating of the beliefs
%
IsALUpdate = (CheckAdaptiveLearningFixedBeliefs(ModelParameters)==0);
%
% run the Kalman filter
%
if IsALUpdate==1;
   if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
      [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
   else;
      [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
else;
   [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
end;
%
% determine initial conditions for the forecasts
%
KsiTT = Ksitt(:,T);
PTT = Ptt(:,:,T);
PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
%
% the last estimates
%
MT1 = Mtilde(:,:,T);
FT1 = Ftilde(:,:,T);
BT1 = Btilde(:,:,T);
PredictYmode = zeros(n,h);
%
%
% initialize solution matrices for possible later use
%
MTh = zeros(r,k,h);
FTh = zeros(r,r,h);
BTh = zeros(r,q,h);
%
% deal with deterministic variables
%
if k>0;
   Ax = A'*X;
else;
   Ax = zeros(n,h);
   X = zeros(r,h)
end;
if length(size(H))==2;
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % forecast with fixed beliefs
      %
      KsiTiT = KsiTT;
      for i=1:h;
         KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
         PredictYmode(:,i) = (Ax(:,i))+(H'*KsiTiT);
      end;
   else;
      %
      % forecast with updated beliefs
      %
      KsiTiT = KsiTT;
      MTiT = MT1;
      FTiT = FT1;
      BTiT = BT1;
      for i=1:h;
         KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
         PredictYmode(:,i) = (Ax(:,i))+(H'*KsiTiT);
         %
         % store the used solution matrices
         %
         MTh(:,:,i) = MTiT;
         FTh(:,:,i) = FTiT;
         BTh(:,:,i) = BTiT;
         %
         % update beliefs
         %
         if i==1;
            MTiT = LastSolution.MTT;
            FTiT = LastSolution.FTT;
            BTiT = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         else;
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
      end;
   end;
else;
   % 
   if DSGEModel.FixedBeliefsForProj==1;
      %
      % forecast with fixed beliefs
      %
      KsiTiT = KsiTT;
      for i=1:h;
         KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
         PredictYmode(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
      end;
   else;
      %
      % forecast with updated beliefs
      %
      KsiTiT = KsiTT;
      MTiT = MT1;
      FTiT = FT1;
      BTiT = BT1;
      for i=1:h;
         KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
         PredictYmode(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
         %
         % store the used solution matrices
         %
         MTh(:,:,i) = MTiT;
         FTh(:,:,i) = FTiT;
         BTh(:,:,i) = BTiT;
         %
         % update beliefs
         %
         if i==1;
            MTiT = LastSolution.MTT;
            FTiT = LastSolution.FTT;
            BTiT = LastSolution.BTT;
            betaTiT = LastSolution.betaTT;
         else;
            betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
            if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
               [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
            else;
               [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
         end;
      end;
   end;
end;
%
% store mode results for use if required
%
Ksitt1mode = Ksitt1;
Ptt1mode = Ptt1;
Yerrormode = Yerror;
InvMSEYmode = InvMSEY;
Hmode = H;
Rmode = R;
Amode = A;
Ftildemode = Ftilde;
Btildemode = Btilde;
KsiTTmode = KsiTT;
LastSolutionmode = LastSolution;
%
% use the Schur decomposition for the reduced rank matrix PTT instead of
% the Choleski decomposition
%
[Smode,Lmode] = SchurDecomposition(PTT);
BT1mode = BT1;
MT1mode = MT1;
FT1mode = FT1;
BThmode = BTh;
MThmode = MTh;
FThmode = FTh;
%
% intialize certain moments
%
StateCov = zeros(n,n,h);
ShockCov = StateCov;
MeasureCov = zeros(n,n);
ParameterCov = StateCov;
MeanPredictY = zeros(n,h);
IsMeasureCov = (rank(R)>0);
if IsMeasureCov==0;
   BNPredictY = zeros(n,2);
   BNObsCov = zeros(n,n,2);
   BNParamCov = BNObsCov;
else;
   BNPredictY = [];
   BNObsCov = [];
   BNParamCov = [];
end;
PITMoments = [];
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% check if we should setup a wait handle
%
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' int2str(NumDraws) ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Draw Number:';
   %
   ProgressStructure.name = ['Marginal Predictive Moments' PriorHeader ' - ' int2str(h) ' Periods Ahead'];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes marginal predictive moments under adaptive learning for the original data. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Marginal Predictive Moments ' PriorHeader ' - ' int2str(h) ' Periods Ahead'],600,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% begin the loop
%
abort = '';
StopEstimation = 0;
if length(size(H))==2;
   for ParamIndex=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([ParamIndex/NumDraws ParamIndex]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use posterior mode or initial values as a backup
         %
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      %
      % get the structural form matrices
      %
      [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
      %
      % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
      %
      [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
      %
      % call the transformation fucntion
      %
      [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
      %
      % compute initial values for the solution matrices
      %
      if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
         [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
      else;
         [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
      %
      % run the Kalman filter
      %
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
      %
      % determine initial conditions for the forecasts
      %
      KsiTT = Ksitt(:,T);
      PTT = Ptt(:,:,T);
      PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
      %
      % now we can start with the moments
      %
      if ComputePITMoment==1;
         PITMoments(ParamIndex).PredictYCov = zeros(n,n,h);
      end;
      PredictY = zeros(n,h);
      Fi = eye(r);
      FjSum = zeros(r,r);
      MT1 = Mtilde(:,:,T);
      FT1 = Ftilde(:,:,T);
      BT1 = Btilde(:,:,T);
      %
      % deal with deterministic variables
      %
      if k>0;
         Ax = A'*X;
      else;
         Ax = zeros(n,h);
         X = zeros(r,h)
      end;
      %
      % check beliefs
      %
      if DSGEModel.FixedBeliefsForProj==1;
         %
         % forecast with fixed beliefs
         %
         % compute mean prediction
         %
         KsiTiT = KsiTT;
         for i=1:h;
            KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
            PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
         end;
         %
         % check if mean prediction is reasonable and if not use the mode values
         %
         TestValue = max(max(abs((PredictY-PredictYmode)./PredictYmode)));
         if TestValue>=100;
            PredictY = PredictYmode;
            Ksitt1 = Ksitt1mode;
            Ptt1 = Ptt1mode;
            Yerror = Yerrormode;
            InvMSEY = InvMSEYmode;
            H = Hmode;
            R = Rmode;
            Ftilde = Ftildemode;
            Btilde = Btildemode;
            A = Amode;
            KsiTT = KsiTTmode;
            S = Smode;
            L = Lmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
         end;
         %
         % check beliefs and compute uncertainty
         %
         Q = BT1*BT1';
         for i=1:h;
            FjSum = FjSum+(Fi*Q*Fi');
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*FjSum*H);
            %
            % update Fi
            %
            Fi = FT1*Fi;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H);
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
            if ComputePITMoment==1;
               PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*(FjSum+(Fi*PTT*Fi'))*H)+R;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredictY;
         end;
         %
         % now do the now and backcast
         %
         if IsMeasureCov==0;
            %
            % run the Kalman smoother for T and T-1 only
            %
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1(:,T-1:T),Ptt1(:,:,T-1:T),Yerror(:,T-1:T),InvMSEY(:,:,T-1:T),H,R,Ftilde(:,:,T-1:T),Btilde(:,:,T-1:T),DSGEModel.LastPeriod-1,DSGEModel.LastPeriod,1,DSGEModel.YNaNs);
            if k>0;
               NCtheta = (A'*Xdata(:,T))+(H'*KsitT(:,2));
               BCtheta = (A'*Xdata(:,T-1))+(H'*KsitT(:,1));
            else;
               NCtheta = H'*KsitT(:,2);
               BCtheta = H'*KsitT(:,1);
            end;
            NSigma = H'*PtT(:,:,2)*H;
            BSigma = H'*PtT(:,:,1)*H;
            if ComputePITMoment==1;
               PITMoments(ParamIndex).BNPredictY = [BCtheta NCtheta];
               PITMoments(ParamIndex).BNPredictYCov = zeros(n,n,2);
               PITMoments(ParamIndex).BNPredictYCov(:,:,1) = BSigma;
               PITMoments(ParamIndex).BNPredictYCov(:,:,2) = NSigma;
            end;
            %
            % now we compute the Monte Carlo mean of the back and nowcast as well as
            % of their mean covariance and uncentered covariance of the mean
            %
            BNPredictY = BNPredictY+((Weights(ParamIndex)/NumDraws)*[BCtheta NCtheta]);
            BNObsCov(:,:,1) = BNObsCov(:,:,1)+((Weights(ParamIndex)/NumDraws)*BSigma);
            BNObsCov(:,:,2) = BNObsCov(:,:,2)+((Weights(ParamIndex)/NumDraws)*NSigma);
            BNParamCov(:,:,1) = BNParamCov(:,:,1)+((Weights(ParamIndex)/NumDraws)*BCtheta*BCtheta');
            BNParamCov(:,:,2) = BNParamCov(:,:,2)+((Weights(ParamIndex)/NumDraws)*NCtheta*NCtheta');
         end;
      else;
         %
         % forecast with updated beliefs
         %
         KsiTiT = KsiTT;
         MTiT = MT1;
         FTiT = FT1;
         BTiT = BT1;
         for i=1:h;
            KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
            PredictY(:,i) = (Ax(:,i))+(H'*KsiTiT);
            %
            % update beliefs
            %
            if i==1;
               MTiT = LastSolution.MTT;
               FTiT = LastSolution.FTT;
               BTiT = LastSolution.BTT;
               betaTiT = LastSolution.betaTT;
            elseif i<h;
               betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
               else;
                  [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
            end;
         end;
         %
         % check if mean prediction is reasonable and if not use the mode values
         %
         TestValue = max(max(abs((PredictY-PredictYmode)./PredictYmode)));
         if TestValue>=100;
            PredictY = PredictYmode;
            Ksitt1 = Ksitt1mode;
            Ptt1 = Ptt1mode;
            Yerror = Yerrormode;
            InvMSEY = InvMSEYmode;
            H = Hmode;
            R = Rmode;
            Ftilde = Ftildemode;
            Btilde = Btildemode;
            A = Amode;
            KsiTT = KsiTTmode;
            S = Smode;
            L = Lmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
            LastSolution = LastSolutionmode;
         end;
         %
         % check beliefs and compute uncertainty
         %
         MTiT = MT1;
         FTiT = FT1;
         BTiT = BT1;
         for i=1:h;
            FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*FjSum*H);
            %
            % update Fi
            %
            Fi = FTiT*Fi;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H'*Fi*PTT*Fi'*H);
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
            if ComputePITMoment==1;
               PITMoments(ParamIndex).PredictYCov(:,:,i) = (H'*(FjSum+(Fi*PTT*Fi'))*H)+R;
            end;
            %
            % update beliefs
            %
            if i==1;
               MTiT = LastSolution.MTT;
               FTiT = LastSolution.FTT;
               BTiT = LastSolution.BTT;
               betaTiT = LastSolution.betaTT;
            else;
               betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
               else;
                  [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredictY;
         end;
         %
         % now do the now and backcast
         %
         if IsMeasureCov==0;
            %
            % run the Kalman smoother for T and T-1 only
            %
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1(:,T-1:T),Ptt1(:,:,T-1:T),Yerror(:,T-1:T),InvMSEY(:,:,T-1:T),H,R,Ftilde(:,:,T-1:T),Btilde(:,:,T-1:T),DSGEModel.LastPeriod-1,DSGEModel.LastPeriod,1,DSGEModel.YNaNs);
            if k>0;
               NCtheta = (A'*Xdata(:,T))+(H'*KsitT(:,2));
               BCtheta = (A'*Xdata(:,T-1))+(H'*KsitT(:,1));
            else;
               NCtheta = H'*KsitT(:,2);
               BCtheta = H'*KsitT(:,1);
            end;
            NSigma = H'*PtT(:,:,2)*H;
            BSigma = H'*PtT(:,:,1)*H;
            if ComputePITMoment==1;
               PITMoments(ParamIndex).BNPredictY = [BCtheta NCtheta];
               PITMoments(ParamIndex).BNPredictYCov = zeros(n,n,2);
               PITMoments(ParamIndex).BNPredictYCov(:,:,1) = BSigma;
               PITMoments(ParamIndex).BNPredictYCov(:,:,2) = NSigma;
            end;
            %
            % now we compute the Monte Carlo mean of the back and nowcast as well as
            % of their mean covariance and uncentered covariance of the mean
            %
            BNPredictY = BNPredictY+((Weights(ParamIndex)/NumDraws)*[BCtheta NCtheta]);
            BNObsCov(:,:,1) = BNObsCov(:,:,1)+((Weights(ParamIndex)/NumDraws)*BSigma);
            BNObsCov(:,:,2) = BNObsCov(:,:,2)+((Weights(ParamIndex)/NumDraws)*NSigma);
            BNParamCov(:,:,1) = BNParamCov(:,:,1)+((Weights(ParamIndex)/NumDraws)*BCtheta*BCtheta');
            BNParamCov(:,:,2) = BNParamCov(:,:,2)+((Weights(ParamIndex)/NumDraws)*NCtheta*NCtheta');
         end;
      end;
      MeasureCov = MeasureCov+((1/NumDraws)*R*Weights(ParamIndex));
      drawnow;
   end;
else;
   %
   % time-varying H-matrix
   %
   for ParamIndex=1:NumDraws;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            StopEstimation = 1;
            break;
         else;
            ProgressDLG([ParamIndex/NumDraws ParamIndex]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(ParamIndex,:)';
      [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % use posterior mode or initial values as a backup
         %
         [A,H,R,F,B0,status,ErrMsg,ModelParameters] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      %
      % get the structural form matrices
      %
      [Hlag,H0,Hexp,D] = DSGEStructuralForm(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
      %
      % Compute the initial values of betavec, Sigmau, R10 and Sigmaeps
      %
      [betavec,Sigmau,R10,Sigmaeps,Gtilde,Kgf] = AdaptiveLearningInitialValuesDSGE(F,B0,ModelParameters.sigmarAL,ModelParameters.sigmaeAL,SMatrix,BeliefSys.G,m,x,f,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,Gtilde,Kgf);
      %
      % call the transformation fucntion
      %
      [TransStatus,Hstarlag,Hstar0,Hstar1f] = AdaptiveLearningTransformDSGEModel(Hlag,H0,Hexp,SMatrix,Sperp);
      %
      % compute initial values for the solution matrices
      %
      if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
         [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec);
      else;
         [Mt1,Ft1,Bt1] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betavec,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
      end;
      %
      % run the Kalman filter
      %
      if IsALUpdate==1;
         if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
         else;
            [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningUserSolverKalmanFilter(GetName(DSGEModel.AdaptiveLearningModelSolverFile),m,SMatrix,betavec,Sigmau,R10,Sigmaeps,ModelParameters.rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
         end;
      else;
         [lnL,bstatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningSingleKalmanFilter(m,betavec,Mt1,Ft1,Bt1,DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,xbar,A,H,R,DSGEModel.UseDoublingAlgorithm,DSGEModel.StateCovConstValue,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,KalmanFirstObservation,DSGEModel.YNaNs,DSGEModel.NoSmoothedStatesForBeliefs);
      end;
      %
      % determine initial conditions for the forecasts
      %
      KsiTT = Ksitt(:,T);
      PTT = Ptt(:,:,T);
      PTT = diag(abs(diag(PTT)))+tril(PTT,-1)+tril(PTT,-1)';
      %
      % now we can start with the moments
      %
      if ComputePITMoment==1;
         PITMoments(ParamIndex).PredictYCov = zeros(n,n,h);
      end;
      PredictY = zeros(n,h);
      Fi = eye(r);
      FjSum = zeros(r,r);
      MT1 = Mtilde(:,:,T);
      FT1 = Ftilde(:,:,T);
      BT1 = Btilde(:,:,T);
      %
      % deal with deterministic variables
      %
      if k>0;
         Ax = A'*X;
      else;
         Ax = zeros(n,h);
         X = zeros(r,h)
      end;
      %
      % check beliefs
      %
      if DSGEModel.FixedBeliefsForProj==1;
         %
         % forecast with fixed beliefs
         %
         % compute mean prediction
         %
         KsiTiT = KsiTT;
         for i=1:h;
            KsiTiT = (MT1*X(:,i))+(FT1*KsiTiT);
            PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
         end;
         %
         % check if mean prediction is reasonable and if not use the mode values
         %
         TestValue = max(max(abs((PredictY-PredictYmode)./PredictYmode)));
         if TestValue>=100;
            PredictY = PredictYmode;
            Ksitt1 = Ksitt1mode;
            Ptt1 = Ptt1mode;
            Yerror = Yerrormode;
            InvMSEY = InvMSEYmode;
            H = Hmode;
            R = Rmode;
            Ftilde = Ftildemode;
            Btilde = Btildemode;
            A = Amode;
            KsiTT = KsiTTmode;
            S = Smode;
            L = Lmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
         end;
         %
         % check beliefs and compute uncertainty
         %
         Q = BT1*BT1';
         for i=1:h;
            FjSum = FjSum+(Fi*Q*Fi');
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i));
            %
            % update Fi
            %
            Fi = FT1*Fi;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i));
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
            if ComputePITMoment==1;
               PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,LastPeriod+i)'*(FjSum+(Fi*PTT*Fi'))*H(:,:,LastPeriod+i))+R;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredictY;
         end;
         %
         % now do the now and backcast
         %
         if IsMeasureCov==0;
            %
            % run the Kalman smoother for T and T-1 only
            %
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1(:,T-1:T),Ptt1(:,:,T-1:T),Yerror(:,T-1:T),InvMSEY(:,:,T-1:T),H,R,Ftilde(:,:,T-1:T),Btilde(:,:,T-1:T),DSGEModel.LastPeriod-1,DSGEModel.LastPeriod,1,DSGEModel.YNaNs);
            if k>0;
               NCtheta = (A'*Xdata(:,T))+(H(:,:,LastPeriod+i)'*KsitT(:,2));
               BCtheta = (A'*Xdata(:,T-1))+(H(:,:,LastPeriod+i)'*KsitT(:,1));
            else;
               NCtheta = H(:,:,LastPeriod+i)'*KsitT(:,2);
               BCtheta = H(:,:,LastPeriod+i)'*KsitT(:,1);
            end;
            NSigma = H(:,:,LastPeriod+i)'*PtT(:,:,2)*H(:,:,LastPeriod+i);
            BSigma = H(:,:,LastPeriod+i)'*PtT(:,:,1)*H(:,:,LastPeriod+i);
            if ComputePITMoment==1;
               PITMoments(ParamIndex).BNPredictY = [BCtheta NCtheta];
               PITMoments(ParamIndex).BNPredictYCov = zeros(n,n,2);
               PITMoments(ParamIndex).BNPredictYCov(:,:,1) = BSigma;
               PITMoments(ParamIndex).BNPredictYCov(:,:,2) = NSigma;
            end;
            %
            % now we compute the Monte Carlo mean of the back and nowcast as well as
            % of their mean covariance and uncentered covariance of the mean
            %
            BNPredictY = BNPredictY+((Weights(ParamIndex)/NumDraws)*[BCtheta NCtheta]);
            BNObsCov(:,:,1) = BNObsCov(:,:,1)+((Weights(ParamIndex)/NumDraws)*BSigma);
            BNObsCov(:,:,2) = BNObsCov(:,:,2)+((Weights(ParamIndex)/NumDraws)*NSigma);
            BNParamCov(:,:,1) = BNParamCov(:,:,1)+((Weights(ParamIndex)/NumDraws)*BCtheta*BCtheta');
            BNParamCov(:,:,2) = BNParamCov(:,:,2)+((Weights(ParamIndex)/NumDraws)*NCtheta*NCtheta');
         end;
      else;
         %
         % forecast with updated beliefs
         %
         KsiTiT = KsiTT;
         MTiT = MT1;
         FTiT = FT1;
         BTiT = BT1;
         for i=1:h;
            KsiTiT = (MTiT*X(:,i))+(FTiT*KsiTiT);
            PredictY(:,i) = (Ax(:,i))+(H(:,:,LastPeriod+i)'*KsiTiT);
            %
            % update beliefs
            %
            if i==1;
               MTiT = LastSolution.MTT;
               FTiT = LastSolution.FTT;
               BTiT = LastSolution.BTT;
               betaTiT = LastSolution.betaTT;
            elseif i<h;
               betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
               else;
                  [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
            end;
         end;
         %
         % check if mean prediction is reasonable and if not use the mode values
         %
         TestValue = max(max(abs((PredictY-PredictYmode)./PredictYmode)));
         if TestValue>=100;
            PredictY = PredictYmode;
            Ksitt1 = Ksitt1mode;
            Ptt1 = Ptt1mode;
            Yerror = Yerrormode;
            InvMSEY = InvMSEYmode;
            H = Hmode;
            R = Rmode;
            Ftilde = Ftildemode;
            Btilde = Btildemode;
            A = Amode;
            KsiTT = KsiTTmode;
            S = Smode;
            L = Lmode;
            BT1 = BT1mode;
            MT1 = MT1mode;
            FT1 = FT1mode;
            BTh = BThmode;
            MTh = MThmode;
            FTh = FThmode;
            LastSolution = LastSolutionmode;
         end;
         %
         % check beliefs and compute uncertainty
         %
         MTiT = MT1;
         FTiT = FT1;
         BTiT = BT1;
         for i=1:h;
            FjSum = FjSum+(Fi*BTiT*BTiT'*Fi');
            ShockCov(:,:,i) = ShockCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*FjSum*H(:,:,LastPeriod+i));
            %
            % update Fi
            %
            Fi = FTiT*Fi;
            StateCov(:,:,i) = StateCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*H(:,:,LastPeriod+i)'*Fi*PTT*Fi'*H(:,:,LastPeriod+i));
            ParameterCov(:,:,i) = ParameterCov(:,:,i)+((Weights(ParamIndex)/NumDraws)*PredictY(:,i)*PredictY(:,i)');
            if ComputePITMoment==1;
               PITMoments(ParamIndex).PredictYCov(:,:,i) = (H(:,:,LastPeriod+i)'*(FjSum+(Fi*PTT*Fi'))*H(:,:,LastPeriod+i))+R;
            end;
            %
            % update beliefs
            %
            if i==1;
               MTiT = LastSolution.MTT;
               FTiT = LastSolution.FTT;
               BTiT = LastSolution.BTT;
               betaTiT = LastSolution.betaTT;
            else;
               betaTiT = betavec+(LastSolution.M*(betaTiT-betavec));
               if (DSGEModel.ModelSolver<=3)|(FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==0);
                  [MTiT,FTiT,BTiT] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT);
               else;
                  [MTiT,FTiT,BTiT] = feval(GetName(DSGEModel.AdaptiveLearningModelSolverFile),Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betaTiT,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
               end;
            end;
         end;
         MeanPredictY = MeanPredictY+((Weights(ParamIndex)/NumDraws)*PredictY);
         if ComputePITMoment==1;
            PITMoments(ParamIndex).PredictY = PredictY;
         end;
         %
         % now do the now and backcast
         %
         if IsMeasureCov==0;
            %
            % run the Kalman smoother for T and T-1 only
            %
            [KsitT,PtT,etatt,etatT,wtt,wtT] = AdaptiveLearningKalmanSmoother(Ksitt1(:,T-1:T),Ptt1(:,:,T-1:T),Yerror(:,T-1:T),InvMSEY(:,:,T-1:T),H,R,Ftilde(:,:,T-1:T),Btilde(:,:,T-1:T),DSGEModel.LastPeriod-1,DSGEModel.LastPeriod,1,DSGEModel.YNaNs);
            if k>0;
               NCtheta = (A'*Xdata(:,T))+(H(:,:,LastPeriod+i)'*KsitT(:,2));
               BCtheta = (A'*Xdata(:,T-1))+(H(:,:,LastPeriod+i)'*KsitT(:,1));
            else;
               NCtheta = H(:,:,LastPeriod+i)'*KsitT(:,2);
               BCtheta = H(:,:,LastPeriod+i)'*KsitT(:,1);
            end;
            NSigma = H(:,:,LastPeriod+i)'*PtT(:,:,2)*H(:,:,LastPeriod+i);
            BSigma = H(:,:,LastPeriod+i)'*PtT(:,:,1)*H(:,:,LastPeriod+i);
            if ComputePITMoment==1;
               PITMoments(ParamIndex).BNPredictY = [BCtheta NCtheta];
               PITMoments(ParamIndex).BNPredictYCov = zeros(n,n,2);
               PITMoments(ParamIndex).BNPredictYCov(:,:,1) = BSigma;
               PITMoments(ParamIndex).BNPredictYCov(:,:,2) = NSigma;
            end;
            %
            % now we compute the Monte Carlo mean of the back and nowcast as well as
            % of their mean covariance and uncentered covariance of the mean
            %
            BNPredictY = BNPredictY+((Weights(ParamIndex)/NumDraws)*[BCtheta NCtheta]);
            BNObsCov(:,:,1) = BNObsCov(:,:,1)+((Weights(ParamIndex)/NumDraws)*BSigma);
            BNObsCov(:,:,2) = BNObsCov(:,:,2)+((Weights(ParamIndex)/NumDraws)*NSigma);
            BNParamCov(:,:,1) = BNParamCov(:,:,1)+((Weights(ParamIndex)/NumDraws)*BCtheta*BCtheta');
            BNParamCov(:,:,2) = BNParamCov(:,:,2)+((Weights(ParamIndex)/NumDraws)*NCtheta*NCtheta');
         end;
      end;
      MeasureCov = MeasureCov+((1/NumDraws)*R*Weights(ParamIndex));
      drawnow;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
if StopEstimation==1;
   return;
end;
for i=1:h;
   ParameterCov(:,:,i) = ParameterCov(:,:,i)-(MeanPredictY(:,i)*MeanPredictY(:,i)');
end;
if IsMeasureCov==0;
   for i=1:2;
      BNParamCov(:,:,i) = BNParamCov(:,:,i)-(BNPredictY(:,i)*BNPredictY(:,i)');
   end;
   BNCov = BNObsCov+BNParamCov;
else;
   BNCov = [];
end;
%
% save the output to file
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningDSGE-PredictiveMoments-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(h) '.' int2str(NumDraws) '-' int2str(TotalDraws) '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearningDSGE-PredictiveMoments-Prior-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(h) '.' int2str(NumDraws) '.mat']);
end;
save(file,'MeanPredictY','MeasureCov','ShockCov','StateCov','ParameterCov','BNPredictY','BNObsCov','BNParamCov','BNCov','PITMoments');
DoneCalc = 1;

%
% end of AdaptiveLearningDSGEPredictiveMoments.m
%
