function [ISWeights,UnISWeights,LogPostKernel,tHeights,rhoMatrix,phiDraws] = AdaptiveLearningDSGEMitISEMWeights(phiDraws,mu,Sigma,df,piweights,phiMode,NormalizeISWeights,thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,OrderQZ,NumWorkers)
% AdaptiveLearningDSGEMitISEMWeights: Computes the importance sampling weights based on a mixture of Student-t densities for
%                                     a DSGE model with adaptive learning instead of rational expectations.
%
% USAGE:
%
%       [ISWeights,UnISWeights,LogPostKernel,tHeights,rhoMatrix,phiDraws] = ...
%                  AdaptiveLearningDSGEMitISEMWeights(phiDraws,mu,Sigma,df,piweights,phiMode,NormalizeISWeights,...
%                                                     thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,...
%                                                     PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,...
%                                                     DSGEModel,AIMData,OrderQZ,NumWorkers)
% REQUIRED INPUT:  phiDraws (matrix) of dimension NumParam x NumDraws with the candidate draws of the DSGE model
%                           parameters.
%
%                  mu (matrix) of dimension NumParam x NumH with the location parameters for the NumH mixture
%                           components.
%
%                  Sigma (matrix) of dimension NumParam x NumParam x NumH with the scale parameters for the NumH
%                           mixture components.
%
%                  df (vector) of dimension NumH x 1 with the degrees of freedom parameters for the NumH
%                           mixture components.
%
%                  piweights (vector) of dimension NumH x 1 with the weights on the NumH components.
%
%                  phiMode (vector) of dimension NumParam x 1 with the mode estimates of the DSGE model parameters.
%
%                  NormalizeISWeights (boolean) that takes the value 1 if ISWeights should be normalized to
%                           sum to NumDraws and 0 othersize.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                           0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                           that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns
%                           are 0 1 except for the uniform and the beta distribution, where they represent
%                           the upper and lower bounds.
%
%                  LowerBound (vector) with the lower bounds for the distributions.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                           needs the former field to know which parameter name corresponds to
%                           position i in theta.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                           truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                           and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                           "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                           parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                           and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                           it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                           truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                           values are the lower and the upper bound. For the student-t they are the location (mode),
%                           the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                           and scale parameters are needed. For the Pareto the parameter values are for the mode
%                           and the shape parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                           DSGE model.
%
%                  SMatrix (matrix) of dimension r x f which selects the forward looking variables among the
%                           state variables.
%
%                  Sperp (matrix) of dimension r x (r-f) which select the non-forward looking variables.
%
%                  m (integer) with the maximum lag for the state variables in the belief equations.
%
%                  G (cell array) of dimension f for selecting the explanatory variables among deterministic and lagged
%                           state variables in each belief equation for the forward looking variables.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                           of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                           A is a g x f matrix.
%
%                  xbar (vector) of dimension k with unit element for a constant and zeros elsewhere.
%
%                  DSGEModel (structure) whose fields contain all required information
%                           about the model, such as input files, variable and sample
%                           data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                           The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: ISWeights (vector) of dimension NumDraws x 1 with the importance sampling (IS) weights.
%
%                  UnISWeights (vector) of dimension NumDraws x 1 with the non-normalized IS weights.
%
%                  LogPostKernel (vector) of dimension NumDraws x 1 with the log posterior kernel values
%                           at the posterior draws.
%
%                  tHeights (matrix) of dimension NumDraws x NumH with the heights of the Student-t densities
%                           for the NumDraws parameter values and the NumH mixture components.
%
%                  rhoMatrix (matrix) of dimension NumDraws x NumH with the squared deviation terms of the Student-t
%                           density, i.e. of (z-mu)'*InvSigma*(z-mu).
%
%                  phiDraws (matrix) of dimension NumParam x NumDraws with the candidate draws of the DSGE model
%                           parameters.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 27, 2020.
%                        This version: January 8, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-05-2020: Updated the documentation.
%
% * 22-01-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
[NumParam,NumDraws] = size(phiDraws);
ISWeights = [];
UnISWeights = [];
LogPostKernel = [];
NumH = length(piweights);
tHeights = [];
rhoMatrix = [];
%
% we begin by computing all the log-posterior kernel values and
% the height of the mixture of Student-t densities
%
LogPostKernel = zeros(NumDraws,1);
LogMixComp = zeros(NumDraws,NumH);
rhoMat = zeros(NumDraws,NumH);
%
% make sure we don't compute the inverse of Sigma more than necessary
%
LogDetSigma = zeros(NumH,1);
InvSigma = zeros(NumParam,NumParam,NumH);
for j=1:NumH;
   LogDetSigma(j) = LogDet(Sigma(:,:,j));
   InvSigma(:,:,j) = eye(NumParam)/Sigma(:,:,j);
end;
%
% the following loop is the time-consuming bit
%
if NumWorkers<2;
   for i=1:NumDraws;
      %
      % keep in mind that logPosteriorAdaptiveLearningPhiDSGE provides min the log kernel value of the posterior
      %
      try;
         LPK = -logPosteriorAdaptiveLearningPhiDSGE(phiDraws(:,i),thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,OrderQZ,NumWorkers);
      catch;
         LPK = -1000000;
      end;
      LogPostKernel(i) = LPK;
      if LPK<-10000;
         %
         % replace the draw with the mode estimate, but keep the value of the log
         % posterior kernel since it will imply a weight very close to zero
         %
         phiDraws(:,i) = phiMode;
      end;
      for j=1:NumH;
         [lnS,rhoValue] = logMultiStudentTPDF(phiDraws(:,i),mu(:,j),LogDetSigma(j),InvSigma(:,:,j),df(j));
         LogMixComp(i,j) = lnS;
         rhoMat(i,j) = rhoValue;
      end;
      drawnow;
   end;
else;
   parfor i=1:NumDraws;
      %
      % keep in mind that logPosteriorAdaptiveLearningPhiDSGE provides min the log kernel value of the posterior
      %
      try;
         LPK = -logPosteriorAdaptiveLearningPhiDSGE(phiDraws(:,i),thetaIndex,UniformBounds,LowerBound,thetaPositions,thetaDist,PriorDist,ModelParameters,SMatrix,Sperp,m,G,Gtilde,Kgf,xbar,DSGEModel,AIMData,OrderQZ,NumWorkers);
      catch;
         LPK = -1000000;
      end;
      LogPostKernel(i) = LPK;
      if LPK<-10000;
         %
         % replace the draw with the mode estimate, but keep the value of the log
         % posterior kernel since it will imply a weight very close to zero
         %
         phiDraws(:,i) = phiMode;
      end;
      for j=1:NumH;
         [lnS,rhoValue] = logMultiStudentTPDF(phiDraws(:,i),mu(:,j),LogDetSigma(j),InvSigma(:,:,j),df(j));
         LogMixComp(i,j) = lnS;
         rhoMat(i,j) = rhoValue;
      end;
      drawnow;
   end;
end;
%
% now we compute exponent of LogMixComp by
%
%   a) removing the mean across rows first
%   b) multiplying the exponent of the mean corrected to the exponent of the mean
%
% as a standard way of trying to avoid numerical issues, e.g. dealing with exp(720)
%
MeanLogMixComp = (1/NumDraws)*sum(LogMixComp,1);
LogMixComp = LogMixComp-(ones(NumDraws,1)*MeanLogMixComp);
StudentMixComp = exp(LogMixComp).*(ones(NumDraws,1)*exp(MeanLogMixComp));
tHeights = StudentMixComp;
rhoMatrix = rhoMat;
%
% now compute the mixture of Student-t components
%
StudentMixComp = StudentMixComp*piweights;
%
% now we compute the weights using the above methodology for logs
%
logISWeights = LogPostKernel-log(StudentMixComp);
if max(logISWeights)<700;
   ISWeights = exp(logISWeights);
else;
   %
   % deal with numerical issues
   %
   MeanlogISWeights = (1/NumDraws)*sum(logISWeights);
   logISWeights = logISWeights-(ones(NumDraws,1)*MeanlogISWeights);
   ISWeights = exp(logISWeights)*exp(MeanlogISWeights);
end;
%
UnISWeights = ISWeights;
if NormalizeISWeights==1;
   ISWeights = (NumDraws/sum(ISWeights))*ISWeights;
end;

%
% end of AdaptiveLearningDSGEMitISEMWeights.m
%
