function [g,badg] = YADANumGradient(fcn,x,varargin)
% YADANumGradient: Computes the numerical gradient of a function based on relative (percentage)
%                  parameter changes.
%
% USAGE:
%
%       [g,badg] = YADANumGradient(fcn,x,varargin)
%
% REQUIRED INPUT:  fcn (string vector) with the name of the function being differentiated.
%                      The function returns a vector of dimension n.
%
%                  x (vector) if dimention m with the values at which the partials of fcn
%                      should be evaluated.
%
%                  varargin (variable length input argument list) which all the variables
%                      that should be sent to fcn.
%
% REQUIRED OUTPUT: g (vector) with the numerical gradient
%
%                  badg (boolean) that takes the value 1 if the gradient is "bad" and 0
%                      otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 14, 2023.
%                        This version: January 17, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-08-2023: Updated the documentation.
%
% * 17-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
badg = 0;
%
p = length(x);
xChange = 0.001*abs(x);
xAltChange = 0.0001*abs(x);
IdentMat = eye(p);
g = zeros(p,1);
%
% compute the comparison value of fcn
%
f0 = feval(fcn,x,varargin{:});
%
% make sure xChange doesn't have zeros
%
for i=1:p;
   %
   % check the addon value of x to make sure it's positive
   %
   if xChange(i)<=0;
      xChange(i) = 1e-6;
   end;
   xNew = x+(xChange(i)*IdentMat(:,i));
   xChangei = xChange(i);
   %
   % check the function value at xNew
   %
   try;
      fh = feval(fcn,xNew,varargin{:});
   catch;
      fh = 1000000;
   end;
   if (fh==1000000)|(isnan(fh)==1)|(isinf(fh)==1);
      if xAltChange(i)<=0;
         xAltChange(i) = 1e-6;
      end;
      xAltNew = x+(xAltChange(i)*IdentMat(:,i));
      xChangei = xAltChange(i);
      try;
         fh = feval(fcn,xAltNew,varargin{:});
      catch;
         fh = 1000000;
      end;
   end;
   %
   % check if the function value is kosher
   %
   if (fh~=1000000)&(isnan(fh)==0)&(isinf(fh)==0);
      g0 = (fh-f0)/xChangei;
   else;
      %
      % now we try to lower the value
      %
      xLowNew = x-(xChangei*IdentMat(:,i));
      try;
         fh = feval(fcn,xLowNew,varargin{:});
      catch;
         fh = 1000000;
      end;
      g0 = (f0 - fh)/xChangei;
   end;
   %
   % check
   %
   g(i) = g0;
   if (abs(g0)>=1e15)|(fh==1000000)|(isnan(fh)==1)|(isinf(fh)==1);
      badg = 1;
      return;
   end;
end;

%
% end of YADANumGradient.m
%
