function [mu,Sigma,piweights,df] = MitISEMAlgorithm(phiDraws,ISWeights,tHeights,rhoMatrix,df,piweights,ComputeDF,NumWorkers)
% MitISEMAlgorithm: Computes the IS-weighted expectation and maximization steps of the MitISEM algorithm.
%
% USAGE:
%
%       [mu,Sigma,piweights,df] = MitISEMAlgorithm(phiDraws,ISWeights,tHeights,rhoMatrix,df,piweights,ComputeDF,NumWorkers)
%
% REQUIRED INPUT:  phiDraws (matrix) of dimension NumParam x NumDraws with the candidate draws of the DSGE model
%                           parameters.
%
%                  ISWeights (vector) of dimension NumDraws x 1 with the importance sampling weights.
%
%                  tHeights (matrix) of dimension NumDraws x NumH with the heights of the Student-t densities
%                           for the NumDraws parameter values and the NumH mixture components.
%
%                  rhoMatrix (matrix) of dimension NumDraws x NumH with the squared deviation terms of the Student-t
%                           density, i.e. of (z-mu)'*InvSigma*(z-mu).
%
%                  df (vector) of dimension NumH x 1 with the degrees of freedom parameters for the NumH
%                           mixture components.
%
%                  piweights (vector) of dimension NumH x 1 with the weights on the NumH mixture components.
%
%                  ComputeDF (boolean) that take the value 1 if the degrees of freedom parameters should be
%                           reestimated by the algorithm, and 0 otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: mu (matrix) of dimension NumParam x NumH with the location parameters for the NumH mixture
%                           components.
%
%                  Sigma (matrix) of dimension NumParam x NumParam x NumH with the scale parameters for the NumH
%                           mixture components.
%
%                  piweights (vector) of dimension NumH x 1 with the weights on the NumH mixture components.
%
%                  df (vector) of dimension NumH x 1 with the degrees of freedom parameters for the NumH
%                           mixture components.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb9europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 5, 2018.
%                        This version: January 18, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-12-2018: Finished the function through the maximization step.
%
% * 07-12-2018: Added try-catch code around the fzero call.
%
% * 12-12-2018: Corrected the code for NumH greater than or equal to 2.
%
% * 10-01-2019: Updated the documentation.
%
% * 11-01-2019: Added vectorization code for Sigma.
%
% * 14-01-2019: Added the input variable "NumWorkers".
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 18-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
mu = [];
Sigma = [];
[NumParam,NumDraws] = size(phiDraws);
NumH = length(piweights);
iN = ones(NumDraws,1);
iH = ones(NumH,1);
im = ones(NumParam,1);
%
% starting with the expectation step:
%
% 1. computing ztilde for all components and draws
%
%    output is a NumDraws x NumH matrix
%
if NumH>1;
   zMatrix = (tHeights.*(iN*piweights'))./(tHeights*piweights*ones(1,NumH));
else;
   zMatrix = iN;
end;
%
% 2. computing vtilde for all components and draws
%
%    output is a NumDraws x NumH matrix
%
vMatrix = (zMatrix.*(NumParam+iN*df'))./(rhoMatrix+(iN*df'));
%
% 3. compute xi for all components and draws
%
psiDFVector = DiGamma(df/2);
psiDFPVector = DiGamma((df+NumParam)/2);
%
%    output is a NumDraws x NumH matrix
%
xiMatrix = ((1-zMatrix).*( iN*( log(df/2) - psiDFVector )' )) + (( log((rhoMatrix + (iN*df') )/2 ) - (iN*psiDFPVector') ).*zMatrix );
%
% 4. compute delta for all components
%
%    output is a NumDraws x NumH matrix
%
deltaMatrix = vMatrix+1-zMatrix;
%
% starting the maximization step
%
% 1. vectorizing the denominator and numerator of mu
%
wih = ISWeights*iH';
wihv = wih.*vMatrix;
muDenom = iN'*wihv;
muNumer = phiDraws*wihv;
mu = muNumer./(im*muDenom);
%
% 2. vectorizing the denominator and then computing the numerator of Sigma
%
SigmaDenom = iN'*(wih.*zMatrix);
%
%tic;
%SigmaAdd = zeros(NumParam,NumParam,NumH);
%for i=1:NumH;
%   phiDrawsDev = phiDraws-(mu(:,i)*iN');
%   for j=1:NumDraws;
%      SigmaAdd(:,:,i) = SigmaAdd(:,:,i)+((ISWeights(j)/SigmaDenom(i))*vMatrix(j,i)*phiDrawsDev(:,j)*phiDrawsDev(:,j)');
%   end;
%   drawnow;
%end;
%toc;
%
% vectorizing Sigma
%
%tic;
Sigma = zeros(NumParam,NumParam,NumH);
for i=1:NumH;
   phiDrawsDev = phiDraws-(mu(:,i)*iN');
   Sigma(:,:,i) = (1/(iN'*(ISWeights.*zMatrix(:,i))))*((phiDrawsDev.*( im*(ISWeights.*vMatrix(:,i))'))*phiDrawsDev');
end;
%toc;
%
% 3. vectorizing the denominator of piweights and utilizing
%    the fact that its numerator is identical to the denominator
%    of the Sigma estimator
%
piDenom = iN'*ISWeights;
if NumH>1;
   piweights = (SigmaDenom/piDenom)';
end;
%
if ComputeDF~=1;
   return;
end;
%
SolveOptions = optimset('TolX',1e-9,'TolFun',1e-9,'MaxIter',1000,'MaxFunEvals',1000,'Display','off'); 
%
% 4. compute the vector of constant used when solving the number of degrees of freedom,
%    while making sure it is never less than 0.01.
%
cvector = (ISWeights'*(xiMatrix+deltaMatrix))/piDenom;
for i=1:NumH;
   try;
      %
      % first we check if fsolve exists and uses that in preference to fzero
      %
      TryFZero = 1;
      if exist('fsolve')~-0;
         [dfi,funcvalue,Status] = fsolve(@(dh) SolveDF(dh,cvector(i)),df(i),SolveOptions);
         if Status==1;
            TryFZero = 0;
         end;
      end;
      if TryFZero==1;
         [dfi,funcvalue,Status] = fzero(@(dh) SolveDF(dh,cvector(i)),df(i),SolveOptions);
         if Status~=1;
            %
            % function didn't solve so we pick old value
            %
            dfi = df(i);
         end;
      end;
   catch;
      dfi = df(i);
   end;
   df(i) = max(dfi,0.01);
end;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% local function to solve the number of degrees of freedom
%
function f = SolveDF(dh,c)

f = 1-DiGamma(dh/2)+log(dh/2)-c;

%
% end of MitISEMAlgorithm.m
%
