function [logLikeValue,lnL,PredictedVars,mcode,status] = logCondLikeDSGE(ModelParameters,YData,X,h,DSGEModel,AIMData,OrderQZ,NumWorkers)
% logCondLikeDSGE: Computes the joint conditional likelihood for h-step-ahead forecasts of a DSGE model.
%
% USAGE:
%
%       [logLikeValue,lnL,PredictedVars,mcode,status] = logCondLikeDSGE(ModelParameters,YData,X,h,DSGEModel,...
%                                                                       AIMData,OrderQZ,NumWorkers)
%
% REQUIRED INPUT:  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                  DSGE model.
%
%                  YData (matrix) of dimension nxh with the realizations for the observed variables over the prediction
%                             sample.
%
%                  X (matrix) of dimension kxh with the exogenous variables over the prediction sample.
%
%                  h (integer) with the prediction horizon.
%
%                  DSGEModel (structure) whose fields contain all required information
%                                  about the model, such as input files, variable and sample
%                                  data. Actual observations are not stored in this file.
%
%                  AIMData (structure) with fields given by neq, nlag, nlead etc that is produced when running "compute_aim_data".
%                                  The neq, nlag, and nlead fields are needed by AiMSolver.
%
%                  OrderQZ (boolean) that is unity if "ordqz" is a built-in matlab function and zero otherwise.
%
%                  NumWorkers (integer) with the number of workers in a parallel pool.
%
% REQUIRED OUTPUT: logLikeValue (vector) of dimension h with the value of the log conditional likelihood for
%                                  joint h-step-ahead forecasts.
%
% OPTIONAL OUTPUT: lnL (scalar) with the value of the log-likelhood over the historical sample.
%
%                  PredictedVars (cell array) with the positions of the predicted variables in the vector of observed
%                                  variables for each prediction period.
%
%                  mcode (integer) Indicates whether the solution of the model is unique or not.
%                                  mcode is 1 when unique, 0 if not computed, and various other values when
%                                  there are problems with the solution. Specifically:
%
%                                     mcode = 2:      Roots not correctly computed by real_schur
%                                     mcode = 3:      Too many big roots
%                                     mcode = 35:     Too many big roots, and q(:,right) is singular
%                                     mcode = 4:      Too few big roots
%                                     mcode = 45:     Too few big roots, and q(:,right) is singular
%                                     mcode = 5:      q(:,right) is singular
%                                     mcode = 61:     Too many exact shiftrights
%                                     mcode = 62:     Too many numeric shiftrights
%                                     mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                     cannot be calculated.
%                                     mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                     numbers.
%                                     mcode = -1:     No stable solution.
%                                     mcode = -2:     Too many large generalized eigenvalues.
%                                     mcode = -3:     Too few large generalized eigenvalues.
%                                     else            Return code not properly specified
%
%                  mtatus (integer) Indicates if the log-likelihood was calculated (status=0), or not (status=1).
%                                  In addition, status can be -1 if ths forecast covariance matrix is not positive
%                                  definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 27, 2012.
%                        This version: January 18, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-03-2012: Added the "lnL" output variable.
%
% * 05-06-2012: Took into account that the solvers now take the "StateEquationPositions" input vector.
%
% * 20-07-2012: Robustified the computation of the log-determinant.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 30-08-2017: Added the "NumWorkers" input variable.
%
% * 14-05-2018: Updated the documentation.
%
% * 01-08-2018: The function supports user written model solver function.
%
% * 29-08-2018: Updated the documentation.
%
% * 23-01-2019: Updated the documentation.
%
% * 10-12-2019: Took into account that the DSGEModel.ModelSolverFile now requires the three additional
%               input variables "DSGEModel.StateVariableNames", "DSGEModel.StateEquationNames" and 
%               "DSGEModel.StateShockNames".
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 05-05-2021: Updated the documentation.
%
% * 06-05-2021: Fixed a bug when H is time-varying and Ht wasn't defined.
%
% * 06-01-2022: Updated the documentation.
%
% * 21-02-2022: Changed eval calls to feval.
%
% * 05-01-2023: Updated the documentation.
%
% * 18-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

logLikeValue = zeros(h,1);
if nargout>1;
   PredictedVars = [];
   PredVars = [];
   lnL = 0;
   mcode = 0;
   status = 0;
end;
%
% See if we can solve the model
%
if DSGEModel.ModelSolver==1;
   [mcode,MaxAbsError,BMatrix,SMatrix] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
elseif DSGEModel.ModelSolver==2;
   [F,B0,mcode] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==3;
   [F,B0,mcode] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
elseif DSGEModel.ModelSolver==4;
   [F,B0,mcode] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
end;
if mcode~=1;
   if DSGEModel.ModelSolver==1;
      [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
   elseif DSGEModel.ModelSolver==2;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==3;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ);
   elseif DSGEModel.ModelSolver==4;
      [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
   end;
   if NumWorkers<=1;
      save(FixFilePath([pwd '\tmp\AiMErrorData.mat']),'mcode','ModelEigenvalues');
   end;
   return;
else;
   if (NumWorkers<=1)&(FileExist(FixFilePath([pwd '\tmp\AiMErrorData.mat']))==1);
      delete(FixFilePath([pwd '\tmp\AiMErrorData.mat']));
   end;
end;
%
% transform into state space
%
if DSGEModel.ModelSolver==1;
   [F,B0] = AiMtoStateSpace(BMatrix,SMatrix,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions);
end;
%
% add the values of F and B0 to the ModelParameters.YADA field
%
%
% Evaluate the measurement equation file
%
[A,H,R] = feval(GetName(DSGEModel.MeasurementEquationFile),ModelParameters,AIMData.endog(DSGEModel.StateVariablePositions,:),DSGEModel.VariableNames,DSGEModel.XVariableNames,DSGEModel.n,size(F,1),DSGEModel.k);
%
% run the Kalman filter
%
[lnL,status,logLikeT,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.KalmanFirstObservation,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
if status~=0;
   return;
end;
%
% determine the filter estimates of ksi and P
%
T = size(Ksitt1,2);
pos = ~isnan(DSGEModel.Y(:,DSGEModel.LastPeriod));
n1 = sum(pos);
PLast = Ptt1(:,:,T);
KsiLast = Ksitt1(:,T);
if n1>0;
   YPError = DSGEModel.Y(pos,DSGEModel.LastPeriod)-Yhat(pos,T);
   if length(size(H))==2;
      Ht = H(:,pos);
      SigmaY = (Ht'*PLast*Ht)+R(pos,pos);
      ksitt = KsiLast+(PLast*(Ht/SigmaY)*YPError);
      Ptt = PLast-(PLast*(Ht/SigmaY)*Ht'*PLast);
   else;
      %
      % time-varying H matrices
      %
      Ht = H(:,pos,DSGEModel.LastPeriod)
      SigmaY = (Ht'*PLast*Ht)+R(pos,pos);
      ksitt = KsiLast+(PLast*(Ht/SigmaY)*YPError);
      Ptt = PLast-(PLast*(Ht/SigmaY)*Ht'*PLast);
   end;
else;
   ksitt = KsiLast;
   Ptt = PLast;
end;
%
% initial values for state variables and covariance matrix
%
Q = B0*B0';
%ksimarg = ksitt;
%Pmarg = Ptt;
ksijoint = F*ksitt;
Pjoint = (F*Ptt*F')+Q;
%
% compute the height of the predictive densities
%
if DSGEModel.k>0;
   Ax = A'*X;
else;
   Ax = zeros(DSGEModel.n,h);
end;
YData = YData(DSGEModel.PredictionVariables,:);
log2pi = log(2*pi);
if length(size(H))==2;
   Ht = H(:,DSGEModel.PredictionVariables);
   Rt = R(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
   for i=1:h;
      %
      % joint density
      %
      Yhatjoint = Ax(:,i)+(H'*ksijoint);
      SigmaYjoint = (H'*Pjoint*H)+R;
      %
      % select the subset
      %
      Yhatjoint = Yhatjoint(DSGEModel.PredictionVariables);
      SigmaYjoint = SigmaYjoint(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      %
      % check for missing obs
      %
      pos = ~isnan(YData(:,i));
      nstar = sum(pos);
      %
      if nstar>0;
         Ystarerror = YData(pos,i)-Yhatjoint(pos);
         SigmaYstar = SigmaYjoint(pos,pos);
         logLikeValue(i) = -((nstar*log2pi)+LogDet(SigmaYstar)+((Ystarerror'/SigmaYstar)*Ystarerror))/2;
         PredVars{i} = DSGEModel.PredictionVariables(pos);
         %
         % now we compute the Kalman filter updates
         %
         KalmanGain = F*Pjoint*(Ht(:,pos)/SigmaYstar);
         ksijoint = (F*ksijoint)+(KalmanGain*(Ystarerror));
         FDev = F-(KalmanGain*Ht(:,pos)');
         Pjoint = (FDev*Pjoint*FDev')+(KalmanGain*Rt*KalmanGain')+Q;
      else;
         %
         % Kalman filter updates
         %
         ksijoint = F*ksijoint;
         Pjoint = (F*Pjoint*F')+Q;
         PredVars{i} = [];
      end;
   end;
else;
   Rt = R(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
   for i=1:h;
      %
      % joint density
      %
      Ht = H(:,:,DSGEModel.LastPeriod+i);
      Yhatjoint = Ax(:,i)+(Ht'*ksijoint);
      SigmaYjoint = (Ht'*Pjoint*Ht)+R;
      %
      % select the subset
      %
      Yhatjoint = Yhatjoint(DSGEModel.PredictionVariables);
      SigmaYjoint = SigmaYjoint(DSGEModel.PredictionVariables,DSGEModel.PredictionVariables);
      %
      % check for missing obs
      %
      pos = ~isnan(YData(:,i));
      nstar = sum(pos);
      %
      if nstar>0;
         Ystarerror = YData(pos,i)-Yhatjoint(pos);
         SigmaYstar = SigmaYjoint(pos,pos);
         logLikeValue(i) = -((nstar*log2pi)+LogDet(SigmaYstar)+((Ystarerror'/SigmaYstar)*Ystarerror))/2;
         PredVars{i} = DSGEModel.PredictionVariables(pos);
         %
         % now we compute the Kalman filter updates
         %
         Ht = Ht(:,DSGEModel.PredictionVariables);
         KalmanGain = F*Pjoint*(Ht(:,pos)/SigmaYstar);
         ksijoint = (F*ksijoint)+(KalmanGain*(Ystarerror));
         FDev = F-(KalmanGain*Ht(:,pos)');
         Pjoint = (FDev*Pjoint*FDev')+(KalmanGain*Rt*KalmanGain')+Q;
      else;
         %
         % Kalman filter updates
         %
         ksijoint = F*ksijoint;
         Pjoint = (F*Pjoint*F')+Q;
         PredVars{i} = [];
      end;
   end;
end;
%
% we get the joint log predictive likelihood by simply accumulating the marginal
% log predictive likelihoods
%
logLikeValue = cumsum(logLikeValue);
if nargout>1;
   PredictedVars = PredVars;
end;

%
% end of logCondLikeDSGE.m
%
