function PrintParameterValues(ResFile,DSGEModel,CurrINI,ParameterNames,ModelParameters,IsAdaptiveLearning,EstType)
% PrintParameterValues: Writes the parameter values to file at either initial values or at posterior mode.
%
% USAGE:
%
%       PrintParameterValues(ResFile,DSGEModel,CurrINI,ParameterNames,ModelParameters,IsAdaptiveLearning,EstType)
%
% REQUIRED INPUT: ResFile (string vector) with the full path+name of the file to be printed.
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure) with initialization information.
%
%                 ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma",
%                                "truncnormal", "uniform", "student", "cauchy", "logistic", "gumbel", "pareto",
%                                "all", "additional" and "estimated". Each field returns a string matrix with
%                                the parameter names.
%
%                 ModelParameters (structure)  whose fields have names given by the parameter names in the
%                                DSGE model.
%
%                 IsAdaptiveLearning (boolean) that take the value 1 if the adaptive learning model is
%                                used and 0 otherwise.
%
%                 EstType (string vector) that takes on the values "Initial values" or "Posterior mode".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 21, 2011.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-06-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 14-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 13-02-2023: Added the input variable "IsAdaptiveLearning".
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% open the text-file
%
fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                       P A R A M E T E R   V A L U E S                        *\n');
if IsAdaptiveLearning==1;
   fprintf(fid,'*                      A D A P T I V E   L E A R N I N G                       *\n');
end;
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% Model info
%
fprintf(fid,'Name of model:                    %s\n',DSGEModel.NameOfModel);
fprintf(fid,'Prior distribution data file:     %s\n',GetFilename(DSGEModel.PriorFile));
fprintf(fid,'Sheet name:                       %s\n',DSGEModel.PriorFileSheet);
if IsAdaptiveLearning==1;
   fprintf(fid,'Belief system prior data file:    %s\n',GetFilename(DSGEModel.BeliefPriorFile));
   fprintf(fid,'Sheet name:                       %s\n',DSGEModel.BeliefPriorFileSheet);
end;
if FileExist(DSGEModel.UpdateParameterFile)==1;
   fprintf(fid,'Update parameters file:           %s\n',GetFilename(DSGEModel.UpdateParameterFile));
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   fprintf(fid,'Initialize parameters file:       %s\n',GetFilename(DSGEModel.InitializeParameterFile));
end;
if IsAdaptiveLearning==1;
   fprintf(fid,'Name of belief system:            %s\n',DSGEModel.BeliefSys.Name);
   if DSGEModel.BeliefSystem==1;
      fprintf(fid,'Belief system file:               YADA''s default (two own lags)\n');
   else;
      fprintf(fid,'Belief system file:               %s\n',GetFilename(DSGEModel.BeliefSystemFile));
      fprintf(fid,'Maximum lag order:                %0.0f\n', DSGEModel.BeliefSys.m);
   end;
end;
fprintf(fid,'AiM model file:                   %s\n\n',GetFilename(DSGEModel.AIMFile));
%
% data information
%
fprintf(fid,'Number of state variables:        %0.0f\n',size(DSGEModel.StateVariableNames,1));
if IsAdaptiveLearning==0;
   fprintf(fid,'Number of economic shocks:        %0.0f\n\n',size(DSGEModel.StateShockNames,1));
else;
   fprintf(fid,'Number of economic shocks:        %0.0f\n',size(DSGEModel.StateShockNames,1));
   fprintf(fid,'Number of forward variables:      %0.0f\n\n',length(DSGEModel.ForwardVariablePositions));
end;
%
StateVarNames = StrWrap(vecstr(DSGEModel.StateVariableNames,' '),40);
fprintf(fid,'Names of state variables:         %s\n',StateVarNames(1,:));
for i=2:size(StateVarNames,1);
   if strcmp(StringTrim(StateVarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateVarNames(i,:));
   end;
end;
StateShockNames = StrWrap(vecstr(DSGEModel.StateShockNames,' '),40);
fprintf(fid,'Names of the state shocks:        %s\n',StateShockNames(1,:));
for i=2:size(StateShockNames,1);
   if strcmp(StringTrim(StateShockNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateShockNames(i,:));
   end;
end;
if IsAdaptiveLearning==1;
   ForwardVarNames = StrWrap(vecstr(DSGEModel.ForwardVariableNames,' '),60);
   fprintf(fid,'Names of forward variables:       %s\n',ForwardVarNames(1,:));
   for i=2:size(ForwardVarNames,1);
      if strcmp(StringTrim(ForwardVarNames(i,:)),'')==0;
         fprintf(fid,'                                  %s\n',ForwardVarNames(i,:));
      end;
   end;
end;
%
fprintf(fid,'\n********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                             P A R A M E T E R S                              *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
fprintf(fid,'Parameter values type:            %s\n',EstType);
fprintf(fid,'Number of estimated parameters:   %0.0f\n',size(ParameterNames.estimated,1));
fprintf(fid,'Number of updated parameters:     %0.0f\n',size(ParameterNames.additional,1));
fprintf(fid,'Number of calibrated parameters:  %0.0f\n',size(ParameterNames.calibrated,1));
fprintf(fid,'Total number of parameters:       %0.0f\n\n',size(ParameterNames.estimated,1)+size(ParameterNames.additional,1)+size(ParameterNames.calibrated,1));
%
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
np = size(ParameterNames.all,2);
if np<20;
   AddStr = SpaceStr(20-np);
else;
   AddStr = '';
end;
if (isempty(ParameterNames.estimated)==0)&(size(ParameterNames.estimated,2)<np);
   AddEstStr = SpaceStr(np-size(ParameterNames.estimated,2));
else;
   AddEstStr = '';
end;
if (isempty(ParameterNames.additional)==0)&(size(ParameterNames.additional,2)<np);
   AddAddStr = SpaceStr(np-size(ParameterNames.additional,2));
else;
   AddAddStr = '';
end;
if (isempty(ParameterNames.calibrated)==0)&(size(ParameterNames.calibrated,2)<np);
   AddCalStr = SpaceStr(np-size(ParameterNames.calibrated,2));
else;
   AddCalStr = '';
end;
PrtStr = ['%s%s%s   %s%s   ' PrtVal ' \n'];
%
% deal with parameters
%
fprintf(fid,'=================================================\n');
fprintf(fid,'        Name              Type          Value\n');
fprintf(fid,'-------------------------------------------------\n');
for i=1:size(ParameterNames.estimated,1);
   fprintf(fid,PrtStr,AddStr,ParameterNames.estimated(i,:),AddEstStr,'estimated',SpaceStr(1),getfield(ModelParameters,StringTrim(ParameterNames.estimated(i,:))));
end;
for i=1:size(ParameterNames.additional,1);
   if i==1;
      fprintf(fid,'-------------------------------------------------\n');
   end;
   fprintf(fid,PrtStr,AddStr,ParameterNames.additional(i,:),AddAddStr,'updated',SpaceStr(3),getfield(ModelParameters,StringTrim(ParameterNames.additional(i,:))));
end;
for i=1:size(ParameterNames.calibrated,1);
   if i==1;
      fprintf(fid,'-------------------------------------------------\n');
   end;
   fprintf(fid,PrtStr,AddStr,ParameterNames.calibrated(i,:),AddCalStr,'calibrated','',getfield(ModelParameters,StringTrim(ParameterNames.calibrated(i,:))));
end;
fprintf(fid,'=================================================\n\n');
%
% bottom file data
%
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintParameterValues.m
%
