function PrintStateFEVD(ResFile,DSGEModel,CurrINI,VDs,PrintGroups,NumDraws,TotalDraws,CurrChain,IsPosterior)
% PrintStateFEVD: Writes the forecast error variance decompositions for the observed variables to a text file.
%
% USAGE:
%
%       PrintStateFEVD(ResFile,DSGEModel,CurrINI,CVDs,PrintGroups,NumDraws,TotalDraws,CurrChain,IsPosterior)
%
% REQUIRED INPUT: ResFile (string vector) with the full path and name of the output file.
%
%                 DSGEModel (structure) whose fields contain all required information about the model, such as input
%                           files, variable and sample data. Actual observations are not stored in this file.
%
%                 CurrINI (structure), with initialization information.
%
%                 VDs (structure) with fields "FEVD", "LRVD", "Econ", "EconLR", and "Quantiles". The "FEVD" field holds a
%                           3D matrix of dimension n x (2+q) x h, where n is the number of observed variables, q
%                           the number of shocks, and h the impulse response horizon. The values are the posterior mean
%                           forecast error variance decomposition. The "LRVD" field holds a matrix of dimension nx(1+q)
%                           with the posterior mean long-run forecast error variance decomposition. The "Econ" field holds
%                           an nxh matrix with the posterior mean of forecast error variance decomposition for all
%                           economic (structural) shocks. Similarly, "EconLR" is an nx1 vector with the posterior mean
%                           long-run forecast error variance decomposition of all economic shocks. The field "Quantiles"
%                           is a vector structure with fields "FEVD", "LRVD", "Econ", "EconLR", and "percent", where "FEVD"
%                           holds a 3D matrix of dimension n x (n+q+1) x h with values from the percent quantile of the
%                           marginal distribution of the forecast error variance decomposition, "LRVD" holds a matrix of
%                           dimension nx(n+q) with values from the percent quantile of the marginal distribution of the
%                           long-run forecast error variance decomposition, "Econ" is a matrix of dimension nxh with values
%                           from the percent quantile of the marginal distribution of the forecast error variance
%                           decomposition of all the economci shocks, "EconLR" is a matrix of dimension nx1 with values
%                           from the percent quantile of the marginal distribution of the long-run forecast error variance
%                           decomposition of all the economic shocks, while "percent" holds the percentile value in percent.
%
%                 PrintGroups (boolean) which determines if shocks groups should be printed (1) or not (0).
%
%                 NumDraws (integer) with the number of draws from the posterior distribution
%                           of the DSGE model parameters that were used in the computations.
%
%                 TotalDraws (integer) with the total number of draws from the posterior
%                           distribution of the DSGE model parameters that could have been
%                           used for the computations.
%
%                 CurrChain (integer) with the value of the current MCMC chain.
%
%                 IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                           if draws from the prior are supplied.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 21, 2017.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2018: Updated the documentation.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'* F O R E C A S T   E R R O R   V A R I A N C E   D E C O M P O S I T I O N S  *\n');
fprintf(fid,'*                F O R   T H E   S T A T E   V A R I A B L E S                 *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Number of parameter draws used:      %0.0f \n',NumDraws);
if IsPosterior==1;
   fprintf(fid,'Number of available parameter draws: %0.0f \n',TotalDraws);
   fprintf(fid,'Current MCMC chain:                  %0.0f \n\n',CurrChain);
else;
   fprintf(fid,'\n');
end;
%
if IsPosterior==1;
   fprintf(fid,'NOTE: Posterior mean values of the variance decompositions for the state\n      variables along with quantiles.\n\n');
else;
   fprintf(fid,'NOTE: Prior mean values of the variance decompositions for the state\n      variables along with quantiles.\n\n');
end;
[n,q1,h] = size(VDs.FEVD);
q = q1-1;
vq = size(DSGEModel.StateVariableNames,2);
if PrintGroups==0;
   NamesMatrix = DSGEModel.StateShockNames;
else;
   NamesMatrix = char(zeros(size(DSGEModel.ShockGroupNames)));
   for i=1:q;
      NamesMatrix(i,:) = strrep(DSGEModel.ShockGroupNames(i,:),'_',' ');
   end;
end;
NamesMatrix = strvcat('All state vars',NamesMatrix,'Sum');
vn = size(NamesMatrix,2);
%
% shift all rows of NamesMatrix such that they have a ragged left edge
%
NamesMat = '';
for i=1:q+2;
   NamesMati = StringTrim(NamesMatrix(i,:));
   vni = length(NamesMati);
   if vni<vn;
      AddNameStr = SpaceStr(vn-vni);
   else;
      AddNameStr = '';
   end;
   NamesMat = strvcat(NamesMat,[AddNameStr NamesMati]);
end;
NamesMatrix = NamesMat;
vn = size(NamesMatrix,2);
if vn<15;
   AddNameStr = SpaceStr(15-vn);
else;
   AddNameStr = '';
end;
if vq<20;
   AddShockStr = SpaceStr(20-vq);
else;
   AddShockStr = '';
end;
NameStr = '                    ';
NameStrLR = '                    ';
MEStr = '';
PrtStr = '%s';
PrtVal = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
QPrtStr = '       %4.1f quantile';
CPrtStr = '     %2.1f%% confidence   ';
CPrtVal = '%0.5f';
for i=1:q+2;
   NameStr = [NameStr '   ' AddNameStr NamesMatrix(i,:)];
   PrtStr = [PrtStr '      ' PrtVal];
   QPrtStr = [QPrtStr '      ' PrtVal];
   CPrtStr = [CPrtStr ' [' CPrtVal ',' CPrtVal ']'];
   if i==q+1;
      QPrtStrLR = QPrtStr;
      CPrtStrLR = CPrtStr;
      PrtStrLR = PrtStr;
   end;
   if i>=2;
      NameStrLR = [NameStrLR '   ' AddNameStr NamesMatrix(i,:)];
   end;
end;
%
for i=1:n;
   MEStr = strvcat(MEStr,sprintf('%s%s',AddShockStr,DSGEModel.StateVariableNames(i,:)));
end;
if DSGEModel.ConfidenceRegionMethod==1;
   FirstStr = MultiCharStr('=',length(NameStr)+4);
   DivStr = MultiCharStr('-',length(NameStr)+4);
   MeanStr = MultiCharStr('',length(NameStr)+4);
   FirstStrLR = MultiCharStr('=',length(NameStrLR)+4);
   DivStrLR = MultiCharStr('-',length(NameStrLR)+4);
   MeanStrLR = MultiCharStr('',length(NameStrLR)+4);
else;
   FirstStr = MultiCharStr('=',length(NameStr)+5);
   DivStr = MultiCharStr('-',length(NameStr)+5);
   MeanStr = MultiCharStr('',length(NameStr)+5);
   FirstStrLR = MultiCharStr('=',length(NameStrLR)+5);
   DivStrLR = MultiCharStr('-',length(NameStrLR)+5);
   MeanStrLR = MultiCharStr('',length(NameStrLR)+5);
end;
%
% check if we should relate the shocks to shock groups
%
if PrintGroups==1;
   for i=1:q;
      iGroup = (DSGEModel.ShockGroups==i) .* (1:length(DSGEModel.ShockGroups));
      iGroup = iGroup(iGroup>0);
      ShockNames = StrWrap(vecstr(DSGEModel.StateShockNames(iGroup,:),' '),60);
      GroupName = StringTrim(NamesMatrix(i,:));
      if size(GroupName,2)<25;
         AddSpace = SpaceStr(25-size(GroupName,2));
      else;
         AddSpace = '';
      end;
      fprintf(fid,'%s:%s %s\n',GroupName,AddSpace,ShockNames(1,:));
      for j=2:size(ShockNames,1);
         if strcmp(StringTrim(ShockNames(j,:)),'')==0;
            fprintf(fid,'%s%s\n',SpaceStr(27),ShockNames(j,:));
         end;
      end;
   end;
   fprintf(fid,'\n');
end;
%
% Write to output for horizon h
%
for j=1:h;
   fprintf(fid,'%s\n',FirstStr);
   fprintf(fid,'Forecast Horizon: %3.0f\n',j);
   fprintf(fid,'%s\n',NameStr);
   for i=1:n;
      fprintf(fid,'%s\n',DivStr);
      fprintf(fid,[PrtStr '\n'],MEStr(i,:),[abs(VDs.FEVD(i,:,j)) abs(VDs.Econ(i,j))]');
      fprintf(fid,'%s\n',MeanStr);
      if DSGEModel.ConfidenceRegionMethod==1;
         for k=1:length(VDs.Quantiles);
            fprintf(fid,[QPrtStr '\n'],VDs.Quantiles(k).percent,[abs(VDs.Quantiles(k).FEVD(i,:,j)) abs(VDs.Quantiles(k).Econ(i,j))]');
         end;
      else;
         for k=(length(VDs.Quantiles)/2:-1:1);
            fprintf(fid,[CPrtStr '\n'],VDs.Quantiles(length(VDs.Quantiles)-k+1).percent-VDs.Quantiles(k).percent,vec([abs(VDs.Quantiles(k).FEVD(i,:,j)) abs(VDs.Quantiles(k).Econ(i,j));abs(VDs.Quantiles(length(VDs.Quantiles)-k+1).FEVD(i,:,j)) abs(VDs.Quantiles(length(VDs.Quantiles)-k+1).Econ(i,j))]));
         end;
      end;
   end;
   fprintf(fid,'%s\n\n',FirstStr);
end;
%
% and now the long run
%
fprintf(fid,'%s\n',FirstStrLR);
fprintf(fid,'Forecast Horizon: Long Run\n');
fprintf(fid,'%s\n',NameStrLR);
for i=1:n;
   fprintf(fid,'%s\n',DivStrLR);
   fprintf(fid,[PrtStrLR '\n'],MEStr(i,:),[abs(VDs.LRVD(i,:)) abs(VDs.EconLR(i))]');
   fprintf(fid,'%s\n',MeanStrLR);
   if DSGEModel.ConfidenceRegionMethod==1;
      for k=1:length(VDs.Quantiles);
         fprintf(fid,[QPrtStrLR '\n'],VDs.Quantiles(k).percent,[abs(VDs.Quantiles(k).LRVD(i,:)) abs(VDs.Quantiles(k).EconLR(i))]');
      end;
   else;
      for k=(length(VDs.Quantiles)/2:-1:1);
         fprintf(fid,[CPrtStrLR '\n'],VDs.Quantiles(length(VDs.Quantiles)-k+1).percent-VDs.Quantiles(k).percent,vec([abs(VDs.Quantiles(k).LRVD(i,:)) abs(VDs.Quantiles(k).EconLR(i));abs(VDs.Quantiles(length(VDs.Quantiles)-k+1).LRVD(i,:)) abs(VDs.Quantiles(length(VDs.Quantiles)-k+1).EconLR(i))]));
      end;
   end;
end;
fprintf(fid,'%s\n\n',FirstStrLR);
%
% we're all done now
%
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintStateFEVD.m
%
