function PrintStructuralForm(ResFile,DSGEModel,CurrINI,Hlag,H,Hexp,D,nlead,SolutionError,SolverMessage,EstType)
% PrintStructuralForm: Prints the structural form of the DSGE model to textfile for the
%                      initial or the posterior mode parameter values.
%
% USAGE:
%
%       PrintStructuralForm(ResFile,DSGEModel,CurrINI,Hlag,H,Hexp,D,nlead,SolutionError,SolverMessage,EstType)
%
% REQUIRED INPUT: ResFile (string vector) with the full path+name of the file to be printed.
%
%                 DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                 CurrINI (structure) with initialization information.
%
%                 Hlag (matrix) of dimension rxr with coefficients on lagged state variables.
%
%                 H (matrix) of dimension rxr with coefficients on current state variables.
%
%                 Hexp (matrix) of dimension rx(r*nlead) with coefficients on expected future state variables.
%
%                 D (matrix) of dimension rxq with coefficients on current economic shocks.
%
%                 nlead (integer) with the maximum expectation time horizon.
%
%                 SolutionError (scalar) with the maximum absolution error when using the
%                               reduced form solution of the DSGE model. This means that
%                               Hlag + H*F + sum(j,nlead)Hexp(j)*F^(1+j) is checked relative to 0 and that
%                               (H + Hexp*F)\D is check relative to B0, where F and B0
%                               are the coefficients on lagged state variables and on
%                               current shocks in the solution (state equation) of the
%                               DSGE model.
%                 SolverMessage (string) with any error message from the model solver.
%
%                 EstType (string vector) that takes on the values "Initial values" or "Posterior mode".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 21, 2010.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 22-04-2010: Updated the documentation.
%
% * 07-05-2010: Added the "nlead" input variable to handle cases of E_t x(t+j) for j>1.
%
% * 05-01-2011: Updated the documentation.
%
% * 21-06-2011: Added the "SolverMessage" input variable.
%
% * 19-07-2011: Took the restructuring of the YADA manual into account.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 14-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% make sure very small numbers are zero
%
Hlag = TinyToZero(Hlag);
H = TinyToZero(H);
Hexp = TinyToZero(Hexp);
D = TinyToZero(D);
[r,q] = size(D);
%
% open the text-file
%
fid = fopen(ResFile,'wt');
fprintf(fid,'********************************************************************************\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'*                        S T R U C T U R A L   F O R M                         *\n');
fprintf(fid,'*                                                                              *\n');
fprintf(fid,'********************************************************************************\n\n');
%
% Model info
%
fprintf(fid,'Name of model:                    %s\n',DSGEModel.NameOfModel);
fprintf(fid,'Prior distribution data file:     %s\n',GetFilename(DSGEModel.PriorFile));
fprintf(fid,'Sheet name:                       %s\n',DSGEModel.PriorFileSheet);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   fprintf(fid,'Update parameters file:           %s\n',GetFilename(DSGEModel.UpdateParameterFile));
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   fprintf(fid,'Initialize parameters file:       %s\n',GetFilename(DSGEModel.InitializeParameterFile));
end;
fprintf(fid,'AiM model file:                   %s\n\n',GetFilename(DSGEModel.AIMFile));
%
% data information
%
fprintf(fid,'Number of state variables:        %0.0f\n',r);
fprintf(fid,'Number of economic shocks:        %0.0f\n\n',q);
%
StateVarNames = StrWrap(vecstr(DSGEModel.StateVariableNames,' '),40);
fprintf(fid,'Names of state variables:         %s\n',StateVarNames(1,:));
for i=2:size(StateVarNames,1);
   if strcmp(StringTrim(StateVarNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateVarNames(i,:));
   end;
end;
StateShockNames = StrWrap(vecstr(DSGEModel.StateShockNames,' '),40);
fprintf(fid,'Names of the state shocks:        %s\n',StateShockNames(1,:));
for i=2:size(StateShockNames,1);
   if strcmp(StringTrim(StateShockNames(i,:)),'')==0;
      fprintf(fid,'                                  %s\n',StateShockNames(i,:));
   end;
end;
fprintf(fid,'\n%s\n\n%s results:\n%s\n\n',MultiCharStr('*',80),EstType,MultiCharStr('-',length(EstType)+9));
%
% loop over the r equations
%
for i=1:r;
   %
   % begin with the lagged values
   %
   EqStr = '';
   NumPrt = 0;
   for j=1:r;
      if isequal(Hlag(i,j),0)==0;
         if NumPrt==1;
            if Hlag(i,j)==1;
               EqStr = [EqStr sprintf(' + %s(t-1)',StringTrim(DSGEModel.StateVariableNames(j,:)))];
            elseif Hlag(i,j)==-1;
               EqStr = [EqStr sprintf(' - %s(t-1)',StringTrim(DSGEModel.StateVariableNames(j,:)))];
            else;
               if Hlag(i,j)>0;
                  EqStr = [EqStr sprintf(' + %0.4f*%s(t-1)',Hlag(i,j),StringTrim(DSGEModel.StateVariableNames(j,:)))];
               else;
                  EqStr = [EqStr sprintf(' - %0.4f*%s(t-1)',abs(Hlag(i,j)),StringTrim(DSGEModel.StateVariableNames(j,:)))];
               end;
            end;
         else;
            if Hlag(i,j)==1;
               EqStr = [EqStr sprintf('%s(t-1)',StringTrim(DSGEModel.StateVariableNames(j,:)))];
            elseif Hlag(i,j)==-1;
               EqStr = [EqStr sprintf('- %s(t-1)',StringTrim(DSGEModel.StateVariableNames(j,:)))];
            else;
               EqStr = [EqStr sprintf('%0.4f*%s(t-1)',Hlag(i,j),StringTrim(DSGEModel.StateVariableNames(j,:)))];
            end;
            NumPrt = 1;
         end;
      end;
   end;
   %
   % contemporaneous values
   %
   for j=1:r;
      if isequal(H(i,j),0)==0;
         if NumPrt==1;
            if H(i,j)==1;
               EqStr = [EqStr sprintf(' + %s(t)',StringTrim(DSGEModel.StateVariableNames(j,:)))];
            elseif H(i,j)==-1;
               EqStr = [EqStr sprintf(' - %s(t)',StringTrim(DSGEModel.StateVariableNames(j,:)))];
            else;
               if H(i,j)>0;
                  EqStr = [EqStr sprintf(' + %0.4f*%s(t)',H(i,j),StringTrim(DSGEModel.StateVariableNames(j,:)))];
               else;
                  EqStr = [EqStr sprintf(' - %0.4f*%s(t)',abs(H(i,j)),StringTrim(DSGEModel.StateVariableNames(j,:)))];
               end;
            end;
         else;
            if H(i,j)==1;
               EqStr = [EqStr sprintf('%s(t)',StringTrim(DSGEModel.StateVariableNames(j,:)))];
            elseif H(i,j)==-1;
               EqStr = [EqStr sprintf('- %s(t)',StringTrim(DSGEModel.StateVariableNames(j,:)))];
            else;
               EqStr = [EqStr sprintf('%0.4f*%s(t)',H(i,j),StringTrim(DSGEModel.StateVariableNames(j,:)))];
            end;
            NumPrt = 1;
         end;
      end;
   end;
   %
   % expected future value
   %
   for l=1:nlead;
      Hexpl = Hexp(:,(1:r)+(r*(l-1)));
      for j=1:r;
         if isequal(Hexpl(i,j),0)==0;
            if NumPrt==1;
               if Hexpl(i,j)==1;
                  EqStr = [EqStr sprintf(' + E[%s(t+%0.0f)|I(t)]',StringTrim(DSGEModel.StateVariableNames(j,:)),l)];
               elseif Hexpl(i,j)==-1;
                  EqStr = [EqStr sprintf(' - E[%s(t+%0.0f)|I(t)]',StringTrim(DSGEModel.StateVariableNames(j,:)),l)];
               else;
                  if Hexpl(i,j)>0;
                     EqStr = [EqStr sprintf(' + %0.4f*E[%s(t+%0.0f)|I(t)]',Hexpl(i,j),StringTrim(DSGEModel.StateVariableNames(j,:)),l)];
                  else;
                     EqStr = [EqStr sprintf(' - %0.4f*E[%s(t+%0.0f)|I(t)]',abs(Hexpl(i,j)),StringTrim(DSGEModel.StateVariableNames(j,:)),l)];
                  end;
               end;
            else;
               if Hexpl(i,j)==1;
                  EqStr = [EqStr sprintf('E[%s(t+%0.0f)|I(t)]',StringTrim(DSGEModel.StateVariableNames(j,:)),l)];
               elseif Hexpl(i,j)==-1;
                  EqStr = [EqStr sprintf('- E[%s(t+%0.0f)|I(t)]',StringTrim(DSGEModel.StateVariableNames(j,:)),l)];
               else;
                  EqStr = [EqStr sprintf('%0.4f*E[%s(t+%0.0f)|I(t)]',Hexpl(i,j),StringTrim(DSGEModel.StateVariableNames(j,:)),l)];
               end;
               NumPrt = 1;
            end;
         end;
      end;
   end;
   if isempty(EqStr)==0;
      EqStr = [EqStr ' = '];
   else;
      EqStr = '0 = ';
   end;
   %
   % and now the shocks
   %
   ShockStr = '';
   NumPrt = 0;
   for j=1:q;
      if isequal(D(i,j),0)==0;
         if NumPrt==1;
            if D(i,j)==1;
               ShockStr = [ShockStr sprintf(' + %s(t)',StringTrim(DSGEModel.StateShockNames(j,:)))];
            elseif D(i,j)==-1;
               ShockStr = [ShockStr sprintf(' - %s(t)',StringTrim(DSGEModel.StateShockNames(j,:)))];
            else;
               if D(i,j)>0;
                  ShockStr = [ShockStr sprintf(' + %0.4f*%s(t)',D(i,j),StringTrim(DSGEModel.StateShockNames(j,:)))];
               else;
                  ShockStr = [ShockStr sprintf(' - %0.4f*%s(t)',abs(D(i,j)),StringTrim(DSGEModel.StateShockNames(j,:)))];
               end;
            end;
         else;
            if D(i,j)==1;
               ShockStr = [ShockStr sprintf('%s(t)',StringTrim(DSGEModel.StateShockNames(j,:)))];
            elseif D(i,j)==-1;
               ShockStr = [ShockStr sprintf('- %s(t)',StringTrim(DSGEModel.StateShockNames(j,:)))];
            else;
               ShockStr = [ShockStr sprintf('%0.4f*%s(t)',D(i,j),StringTrim(DSGEModel.StateShockNames(j,:)))];
            end;
            NumPrt = 1;
         end;
      end;
   end;
   if NumPrt==0;
      ShockStr = '0';
   end;
   if i<10;
      AddStr = ['Equation ' int2str(i) ':   '];
   elseif i<100;
      AddStr = ['Equation ' int2str(i) ':  '];
   else;
      AddStr = ['Equation ' int2str(i) ': '];
   end;
   EqStr = [AddStr EqStr ShockStr];
   EqStr = StrWrap(EqStr,65);
   for j=1:size(EqStr,1);
      if j>1;
         fprintf(fid,'              %s\n',EqStr(j,:));
      else;
         fprintf(fid,'%s\n',EqStr(j,:));
      end;
   end;
end;
%
% print the solution error data
%
if isempty(SolverMessage)==1;
   fprintf(fid,'\nMaximum absolute solution error: %s\n\n',num2str(SolutionError));
   fprintf(fid,'NOTE: The solution error concerns plugging in the reduced form into the structural\n');
   fprintf(fid,'      form and checking if equations (3.4) to (3.6) in the YADA manual hold.\n\n');
else;
   SolverMessage = StrWrap(SolverMessage,80);
   fprintf(fid,'\nNOTE: %s\n',SolverMessage(1,:));
   if size(SolverMessage,1)>1;
      for i=2:size(SolverMessage,1);
         fprintf(fid,'      %s\n',SolverMessage(i,:));
      end;
   end;
   fprintf(fid,'\n');
end;
%
% bottom file data
%
fprintf(fid,'********************************************************************************\n\n');
fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
fclose(fid);

%
% end of PrintStructuralForm.m
%
