function [Status,IsOriginal,IsMarginal,CurrChain,IsSubSample,PredictionVariables] = SequentialPredictiveLikelihoodEstimation(controls,DSGEModel,CurrINI,IsOriginal,IsMarginal,CurrChain,IsSubSample,PredictionVariables,RecomputePL)
% SequentialPredictiveLikelihoodEstimation: Can be used for computing the predictive likelihood
%                                           sequentially.
%
% USAGE:
%
%       [Status,IsOriginal,IsMarginal,...
%        CurrChain,IsSubSample,PredictionVariables] = SequentialPredictiveLikelihoodEstimation(controls,DSGEModel,CurrINI,...
%                                                                             IsOriginal,IsMarginal,CurrChain,IsSubSample,...
%                                                                             PredictionVariables,RecomputePL)
%
% REQUIRED INPUT:  DSGEModel (structure) whose fields contain all required information 
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%                  IsOriginal (boolean) that takes the value 1 if the predictive likelihood
%                            should be computed for the original data and 0 if for annualized
%                            data. When empty, the function checks which alternative should
%                            be considered.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive
%                            likelihood should be computed and 0 for the joint. When empty,
%                            the function checks which alternative should be considered.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%                            When empty, the function checks which alternative should be considered.
%
%                  PredictionVariables (vector) with the positions of the observed variables
%                            to compute the predictive likelihood for. When empty, the function
%                            checks which alternative should be considered.
%
%                  RecomputePL (boolean) which takes the value 1 if the predictive likelihood
%                            should be recomputed and 0 if old calculations should be used. When
%                            empty. the function checks which alternative should be considered.
%
% REQUIRED OUTPUT: Status (boolean) that takes the value 1 if the computations could be successfully
%                            carries out, and 0 otherwise.
%
%                  IsOriginal (boolean) that takes the value 1 if the predictive likelihood
%                            should be computed for the original data and 0 if for annualized
%                            data.
%
%                  IsMarginal (boolean) that takes the value 1 if the marginal predictive
%                            likelihood should be computed and 0 for the joint.
%
%                  CurrChain (integer) with the value of the MCMC chain to use.
%
%                  IsSubSample (boolean) that takes the value 1 if a subsample of the posterior
%                            draws should be used and 0 if all post-burnin draws should be used.
%
%                  PredictionVariables (vector) with the positions of the observed variables
%                            to compute the predictive likelihood for.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   DG-R/MPR
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2014 European Central Bank.
%
%                       First version: December 10, 2014.
%                        This version: December 10, 2014.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
Status = 0;
OriginalAnnual = [(strcmp(get(controls.predict.unconditional.original.height.posterior,'Enable'),'on')==1) (strcmp(get(controls.predict.unconditional.annual.height.posterior,'Enable'),'on')==1)];
if isempty(IsOriginal)==1;
   if OriginalAnnual(2)==1;
      txt = 'Would you like to calculate predictive likelihoods for the original data or the annualized data of the DSGE models?';
      answer = Query(txt,'question',200,'Predictive Likelihood - Posterior Distribution',550,'yes',CurrINI,{'Original' 'Annualized'});
      if strcmp(lower(answer),'original')==1;
         IsOriginal = 1;
      else;
         IsOriginal = 0;
      end;
   else;
      IsOriginal = 1;
   end;
end;
if IsOriginal==1;
   DataStr = 'Original';
else;
   DataStr = 'Annualized';
end;
%
% first attempt to load some posterior mode data
%
ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
ErrorStr = '';
HeaderStr = 'YADA - Bad MAT File';
try;
   ModeData = load(ModeFile);
catch;
   ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' lasterr];
end;
%
% check if file loading was successful
%
if isempty(ErrorStr)==0;
   SetLogText('Unable to load posterior mode data. Aborting calculation.');
   return;
end;
%
% if there are multiple chains, check which one to use if necessary
%
ChainsStr = get(controls.posterior.chains,'String');
NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
%
if (isempty(CurrChain)==1)&(NumChains>1);
   %
   % ask which chain to load
   %
   ChainsStr = '';
   for i=1:NumChains;
      ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
   end;
   [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Predictive Likelihood - Posterior Distribution','','',CurrINI);
elseif (isempty(CurrChain)==0)&(CurrChain>NumChains);
   action = 'cancel';
   SetLogText(['The selected chain number (' int2str(CurrChain) ') is greater than the available number of MCMC chains.']);
else;
   CurrChain = 1;
   action = 'ok';
end;
if strcmp(lower(action),'cancel')==1;
   SetLogText('Aborting calculation.');
   return;
end;
%
% check if the data actually exists
%
if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==0;
   SetLogText('Unable to locate the posterior parameter draws. Aborting calculation.');
   return;
end;
%
% check if sub-sample should be
%
if isempty(IsSubSample)==1;
   txt = 'Would you like to use a sub-sample of the posterior draws when estimating the predictive likelihood?';
   answer = Query(txt,'question',400,'Predictive Likelihood - Posterior Distribution',500,'no',CurrINI);
   if strcmp(lower(answer),'yes')==1;
      IsSubSample = 1;
   else;
      IsSubSample = 0;
   end;
end;
%
% check if we should compute the marginal or the joint predictive likelihood
%
if isempty(IsMarginal)==1;
   txt = 'Would you like to compute the marginal predictive likelikehood or the joint predictive likelihood?';
   answer = Query(txt,'question',400,'Predictive Likelihood - Posterior Distribution',500,'yes',CurrINI,{'Marginal' 'Joint'});
   if strcmp(lower(answer),'marginal')==1;
      IsMarginal = 1;
   else;
      IsMarginal = 0;
   end;
end;
%
if IsMarginal==1;
   MargStr = 'Marginal';
else;
   MargStr = 'Joint';
end;
%
% check the PredictionVariables
%
if isempty(PredictionVariables)==1;
   PredictionVariables = ones(1,DSGEModel.n);
   PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
   [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
   if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
      SetLogText('No prediction variables selected. Aborting calculation.');
      return;
   elseif strcmp(lower(Action),'ok')==0;
      SetLogText('Aborting calculation.');
      return;
   end;
end;
%
% check if the selected variables to predict can indeed be used.
%
if max(PredictionVariables)>DSGEModel.n;
   SetLogText('Prediction variables do not match. Aborting calculation.');
   return;
end;
DSGEModel.PredictionVariables = PredictionVariables;
%
% load the data
%
[PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
clear('LogPost','AcceptedDraws','PostSample');
[NumPostDraws,NumParam] = size(thetaPostSample);
thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
NumDraws = NumPostDraws-NumBurnin;
if IsSubSample==1;
   UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
   thetaPostSample = thetaPostSample(UseDraws,:);
   NumDraws = length(UseDraws);
end;
%
% collect some required data
%
theta = ModeData.thetaMode;
thetaPositions = ModeData.thetaPositions;
thetaIndex = ModeData.thetaIndex;
thetaDist = ModeData.thetaDist;
PriorDist = ModeData.PriorDist;
LowerBound = ModeData.LowerBound;
UniformBounds = ModeData.UniformBounds;
ModelParameters = ModeData.ModelParameters;
%
[FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%
% check how many time periods to use for predictive distributions
%
MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
%
% check the maximum when we take NaNs into account
%
YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
[MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
MaxFH = min(MaxFH,MaxPH);
YData = YData(:,1:MaxFH);
%
% matrix with exogenous variables
%
X = [];
if DSGEModel.k>0;
   if NumPredX<MaxFH;
      X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
   end;
   if NumPredX>0;
      X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
   end;
end;
%
% check if older calculations exists given that such may be used
%
tic;
if (RecomputePL==0)&(PredictiveLikelihoodExist(DSGEModel,IsOriginal,IsMarginal,1,MaxFH,NumDraws,NumBurnin,NumPostDraws,CurrChain)==1);
   DoneCalc = 1;
else;
   %
   % compute the predictive likelihood
   %
   DoneCalc = DSGEPredictiveLikelihood(thetaPostSample,theta,thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,YData,X,IsOriginal,1,IsMarginal,FirstPeriod,LastPeriod,NumPostDraws,NumBurnin,CurrChain,DSGEModel,CurrINI);
end;
clear('thetaPostSample');
if DoneCalc==0;
   SetLogText('Aborting calculation.');
   toc;
   return;
end;
%
% we can now do the remaining modified harmonic mean estimates.
%
[LogPredLike,StdErr,PredVars] = PredLikeImportanceSampling(NumDraws,MaxFH,IsOriginal,IsMarginal,1,NumBurnin,NumPostDraws,CurrChain,DSGEModel);
CompTime = toc;
if isempty(LogPredLike)==1;
   SetLogText('Calculating predictive likelihood failed.');
   return;
end;
SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
%
PredictionSampleStrMat = CreateFullSample(DSGEModel);
PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
%
% write the results to text file
%
ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE' MargStr 'PredictiveLikelihood-' DSGEModel.NameOfModel '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(NumDraws) '.' int2str(NumBurnin) '.' int2str(NumPostDraws) '-' int2str(CurrChain) '.txt']);
%
PrintDSGEPredictiveLikelihood(ResFile,LogPredLike,StdErr,PredVars,PredictionSampleStrMat,IsOriginal,IsMarginal,1,NumDraws,NumBurnin,NumPostDraws,[],CompTime,DSGEModel,CurrINI);
%
if FileExist(ResFile)==1;
   ResultsStr = '';
   for i=1:length(LogPredLike);
      ResultsStr = strvcat(ResultsStr,sprintf('%2.0f: %0.4f (%0.4f)',i,LogPredLike(i),StdErr(i)));
   end;
   SetLogText(strvcat([MargStr ' predictive likelihood:'],ResultsStr,' ',['Output written to: ' GetFilename(ResFile)],['Location:          ' GetPath(ResFile)],' '));
   Status = 1;
end;

%
% end of SequentialPredictiveLikelihoodEstimation.m
%
