function SystemPriorModeEstimation(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,IsAdaptiveLearning,DSGEModel,CurrINI,controls)
% SystemPriorModeEstimation: Runs the prior mode estimation routines for the system prior.
%
% USAGE:
%
%       SystemPriorModeEstimation(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,...
%                                 UniformBounds,LowerBound,IsAdaptiveLearning,DSGEModel,CurrINI,controls)
%
% REQUIRED INPUT:  theta (vector), with initial values of the parameters to be estimated.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                            of the model parameter in position i of theta. The field "distribution" has a string
%                            vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                            DSGE model.
%
%                  thetaDist (vector) with integers 1 if gamma, 2 if beta, 3 if normal, 4 if invgamma, 5 if
%                            truncnormal, 6 if uniform, 7 if student, 8 if cauchy, 9 if logistic, 10 if gumbel,
%                            and 11 if pareto.
%
%                  PriorDist (structure) with fields "beta", "gamma", "normal", "invgamma", "truncnormal", "uniform",
%                            "student", "cauchy", "logistic", "gumbel", and "pareto", where the values are matrices with the
%                            parameter values for the distributions. In the case of the normal, beta, gamma, logistic 
%                            and gumbel these parameter values are the mean and the standard deviation. For the invgamma
%                            it is the s and q parameters, where q is a positive integer (degrees of freedom). For the left
%                            truncated normal there's a third parameter, the lower bound. For the uniform the parameter
%                            values are the lower and the upper bound. For the student-t they are the location (mode),
%                            the scale and the degrees of freedom parameter, while for the Cauchy only the location
%                            and scale parameters are needed. For the Pareto the parameter values are for the mode
%                            and the shape parameter.
%
%                  ParameterNames (structure) with fields "calibrated", "beta", "gamma", "normal", "invgamma", "truncnormal",
%                            "uniform", "student", "cauchy", "logistic", "gumbel", "pareto", "all" and "estimated". Each field returns
%                            a string matrix with the parameter names.
%
%                  thetaIndex (vector) with integers where 1=gamma, invgamma, truncnormal, pareto 2=beta, 3=uniform, and
%                            0=normal, student-t, cauchy, logistic, gumbel. This is used to determine the type of transformation
%                            that may need to be performed for the theta parameters.
%
%                  UniformBounds (matrix) with the same number of rows as parameters and 2 columns. The columns are 0 1 except
%                            for the uniform and the beta distribution, where they represent the upper and lower bounds.
%
%                  LowerBound (vector) with lower bound values for the theta vector.
%
%                  IsAdaptiveLearning (boolean) which takes the value 1 if the adaptive learning mechanism is used for the
%                            expectations formation, and 0 otherwise.
%
%                  DSGEModel (structure) whose fields contain all required information
%                            about the model, such as input files, variable and sample
%                            data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
%                  controls (structure) whose fields provide handles to all the UI
%                            controls in YADA's main dialog.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 2, 2017.
%                        This version: March 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 06-06-2017: Took into account that PriorModeEstimation provides the additional output variables
%               "ParameterNames,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,
%               ScreenMessages".
%
% * 22-06-2017: Took into account that the system prior function now takes the input variable
%               OrderQZ instead of CurrINI.
%
% * 10-08-2017: Made sure that ModelParameters is properly updated with the different theta value. Took into
%               account that PriorModeEstimation provides ModelParameters as an output variable.
%
% * 14-05-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 16-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 02-04-2020: The function now takes the input variable "IsAdaptiveLearning".
%
% * 03-04-2020: Added the adaptive learning part to collect the variables SMatrix,
%               Sperp, r, f, Gtilde and Kgf.
%
% * 29-05-2020: Simplified the adaptive learning code.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 26-01-2021: Updated the documentation.
%
% * 29-01-2021: The function supports a user defined model solver under adaptive learning.
%
% * 06-01-2022: Updated the documentation.
%
% * 21-02-2022: Changed eval calls to feval.
%
% * 22-02-2022: Changed last eval calls to feval.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% make sure that the parameters are properly initialized
%
phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Now we need to copy files to the tmp directory and load data from DSGEModel.AIMDataFile
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.InitializeParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.InitializeParameterFile,[pwd '\tmp']);
end;
[stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
%
if IsAdaptiveLearning==1;
   if DSGEModel.BeliefSystem==2;
      [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
   end;
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.AdaptiveLearningModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.AdaptiveLearningModelSolverFile,[pwd '\tmp']);
   end;
end;
ErrorStr = '';
ErrorFile = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
      ErrorFile = DSGEModel.AIMDataFile;
   end;
end;
%
if isempty(ErrorStr)==0;
   About(ErrorStr,'error',['Error - ' GetFilename(ErrorFile)],CurrINI.scrsz(4)-50,600,CurrINI);
   drawnow;
   return;
end;
%
% check if we have the adaptive learning case
%
if IsAdaptiveLearning==1;
   %
   % we need to construct a number of variables that are stored in DSGEModel.AdaptiveLearning
   % as the fields r, f, Sperp, SMatrix, Gtilde and Kgf
   %
   f = length(DSGEModel.ForwardVariablePositions);
   r = length(DSGEModel.StateVariablePositions);
   %
   % compute the BeliefSys structure
   %
   if DSGEModel.BeliefSystem==1;
      BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
   else;
      BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
   end;
   %
   % get the adaptive learning input variables
   %
   [SMatrix,Sperp,m,G,Gtilde,Kgf,xbar] = GetAdaptiveLearningInputVariables(DSGEModel);
   DSGEModel.AdaptiveLearning.r = r;
   DSGEModel.AdaptiveLearning.f = f;
   DSGEModel.AdaptiveLearning.Sperp = Sperp;
   DSGEModel.AdaptiveLearning.SMatrix = SMatrix;
   DSGEModel.AdaptiveLearning.Gtilde = Gtilde;
   DSGEModel.AdaptiveLearning.Kgf = Kgf;
   DSGEModel.AdaptiveLearning.BeliefSys = BeliefSys;
end;
%
% run the prior mode estimation routine
%
[phiMode,ModelParameters,InverseHessian,InverseHessianFinDiff,phiFittedStd,thetaFittedStd,LogMargLikeSystemLaplace,ParameterNames,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,ScreenMessages,R,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff,EstimationStatus] = PriorModeEstimation(theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound,IsAdaptiveLearning,AIMData,DSGEModel,CurrINI,controls);
%
% print the results to file
%
if EstimationStatus==1;
   %
   % make sure that PriorDist contains all fields
   %
   if isempty(PriorDist.beta)==0;
      [a,b] = MomentToParamStdbetaPDF((PriorDist.beta(:,1)-PriorDist.beta(:,3))./(PriorDist.beta(:,4)-PriorDist.beta(:,3)),PriorDist.beta(:,2)./(PriorDist.beta(:,4)-PriorDist.beta(:,3)));
      PriorDist.beta_ab = [a b];
   else;
      PriorDist.beta_ab = [];
   end;
   %
   if isempty(PriorDist.gamma)==0;
      [a,b] = MomentToParamGammaPDF(PriorDist.gamma(:,1)-PriorDist.gamma(:,3),PriorDist.gamma(:,2));
      PriorDist.gamma_ab = [a b];
   else;
      PriorDist.gamma_ab = [];
   end;
   if isempty(PriorDist.logistic)==0;
      [a,b] = MomentToParamLogisticPDF(PriorDist.logistic(:,1),PriorDist.logistic(:,2),PriorDist.logistic(:,3));
      PriorDist.logistic_ab = [a b PriorDist.logistic(:,3)];
   else;
      PriorDist.logistic_ab = [];
   end;
   if isempty(PriorDist.gumbel)==0;
      [a,b] = MomentToParamGumbelPDF(PriorDist.gumbel(:,1),PriorDist.gumbel(:,2));
      PriorDist.gumbel_ab = [a b];
   else;
      PriorDist.gumbel_ab = [];
   end;
   %
   % compute some additional things that are used for the text file with results
   %
   ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
   ParameterBounds = ParameterBounds(:,1:2);
   %
   ThetaPhiPartial = PartialThetaPartialPhi(phiMode,thetaIndex,UniformBounds);
   thetaMode = PhiToTheta(phiMode,thetaIndex,UniformBounds,LowerBound);
   LogJacobianValue = logJacobian(phiMode,thetaIndex,UniformBounds);
   LogPriorDensity = logPriorDSGE(thetaMode,thetaDist,PriorDist,LowerBound);
   %
   % make sure that ModelParameters is updated using thetaMode
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,thetaMode,thetaPositions);
   if IsAdaptiveLearning==0;
      LogSPDensity = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);
   else;
      LogSPDensity = feval(GetName(DSGEModel.SystemPriorFile),thetaMode,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
   end;
   %
   LogJacobianIV = logJacobian(phi,thetaIndex,UniformBounds);
   LogPriorIV = logPriorDSGE(theta,thetaDist,PriorDist,LowerBound);
   %
   % make sure ModelParameters is updated using theta
   %
   ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
   if IsAdaptiveLearning==0;
      LogSPIV = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ);
   else;
      LogSPIV = feval(GetName(DSGEModel.SystemPriorFile),theta,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI.OrderQZ,1);
   end;
   %
   OptAlgorithm = 'csminwel';
   %
   % check the parameter target type
   %
   if IsOdd(DSGEModel.MaximizeAlgorithmValue)==1;
      OptParameters = 'Transformed';
   else;
      OptParameters = 'Original';
   end;
   %
   % prior mode file
   %
   if IsAdaptiveLearning==0;
      SetLogText(strvcat(' ','Writing posterior mode estimation results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: SystemPriorModeResults-' DSGEModel.NameOfModel '.txt']));
      ResultsFile = [DSGEModel.OutputDirectory '\SystemPriorModeResults-' DSGEModel.NameOfModel '.txt'];
   else;
      SetLogText(strvcat(' ','Writing posterior mode estimation results to text file:',['   Directory: ' DSGEModel.OutputDirectory],['   File name: SystemPriorModeResults-AdaptiveLearning-' DSGEModel.NameOfModel '.txt']));
      ResultsFile = [DSGEModel.OutputDirectory '\SystemPriorModeResults-AdaptiveLearning-' DSGEModel.NameOfModel '.txt'];
      DSGEModel.BeliefSys = BeliefSys;
   end;
   %
   % add: thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,ScreenMessages
   %
   PrintSystemPriorModeResults(ResultsFile,DSGEModel,CurrINI,IsAdaptiveLearning,ParameterNames,thetaMode,theta,thetaDist,PriorDist,ThetaPhiPartial,InverseHessian,InverseHessianFinDiff,phiFittedStd,LogSPDensity,LogPriorDensity,LogJacobianValue,LogSPIV,LogPriorIV,LogJacobianIV,LogMargLikeSystemLaplace,thetaCalibrated,thetaAdditionalInitial,thetaAdditionalMode,OptResults,ScreenMessages,rank(R),OptAlgorithm,OptParameters,MaxTime,ToleranceValue,MaxIter,NumIter,FuncDiff);
   %   
   if FileExist(ResultsFile)==1;
      TextGUI(ResultsFile,['System Prior Mode Estimation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Results',50,CurrINI,1,1);
   end;
end;

%
% end of SystemPriorModeEstimation.m
%
