function x = BetaRndFcn(a,b,c,d,NumDraws)
% BetaRndFcn: Draws matrices of data from the beta distribution.
%
% USAGE:
%
%       x = BetaRndFcn(a,b,c,d,NumDraws)
%
% REQUIRED INPUT:  a (vector) with the a parameter of the beta distribution.
%
%                  b (vector) with the b parameter of the beta distribution.
%
%                  c (vector) with the lower bound parameter of the beta distribution.
%
%                  d (vector) with the upper bound parameter of the beta distribution.
%
% OPTIONAL INPUT:  NumDraws (integer) with the number of draws of z
%
% REQUIRED OUTPUT: x (matrix) of draws with dimension dim(a) x NumDraws
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 28, 2006.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 29-08-2006: Added a missing ; character in the betarnd call.
%
% * 09-11-2006: Took the name change betarnd to YADAbetarnd into account.
%
% * 13-11-2007: Updated the documentation.
%
% * 23-05-2008: Updated the documentation.
%
% * 21-07-2008: The function now checks if it should call the YADA version of "betarnd" or not.
%
% * 23-07-2008: The function now takes the lower and upper bound parameters as input.
%
% * 22-01-2009: Updated the documentation.
%
% * 12-03-2009: Added the call option for "YADARndBeta".
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 22-03-2017: Updated the documentation.
%
% * 15-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==4;
   NumDraws = 1;
end;
NumDraws = floor(NumDraws);
if NumDraws<1;
   NumDraws = 1;
end;
%
% make sure a and b are column vectors
%
a = a(:);
b = b(:);
%
if exist('YADAbetarnd')==2;
   z = YADAbetarnd(a*ones(1,NumDraws),b*ones(1,NumDraws));
elseif exist('betarnd')==2;
   z = betarnd(a*ones(1,NumDraws),b*ones(1,NumDraws));
else;
   z = YADARndBeta(a,b,NumDraws);
end;
x = (c*ones(1,NumDraws))+(((d-c)*ones(1,NumDraws)).*z);

%
% end of BetaRndFcn.m
%
