function Indexes = MultinomialResampling(Weights,N)
% MultinomialResampling: Computes the indexes of the resampled particles for SMC using multinomial resampling
%                        based on the original normalized weights.
%
% USAGE:
%
%       Indexes = MultinomialResampling(Weights,N)
%
% REQUIRED INPUT:  Weights (vector) of dimension N with the normalized weights.
%
% OPTIONAL INPUT:  N (integer) with the number of weights to consider. Could be less than the
%                    length of Weights, but defaults to this when missing.
%
% REQUIRED OUTPUT: Indexed (vector) of dimension N with the positions of the resamples particles.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 17, 2017.
%                        This version: January 19, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-08-2017: Added the optional input variable N.
%
% * 21-08-2017: Updated the documentation.
%
% * 15-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 19-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

if nargin==1;
   N = length(Weights);
end;
Indexes = zeros(N,1);
%
% compute N ordered uniform random numbers
%
Utilde = fliplr(cumprod(rand(1,N).^(1./(N:-1:1))))';
CumW = cumsum(Weights/N);
%
% determine the resampled indexes
%
k = 1;
for i=1:N;
   while CumW(k)<Utilde(i);
      k = k+1;
   end;
   Indexes(i) = k;
end;

%
% end of MultinomialResampling.m
%
