function OVDPs = DSGEObsVarDecompDistribution(DSGEModel,CurrINI,AnnualizeData,ShockType,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
% DSGEObsVarDecompDistribution: Computes the mean value of observed variable decomposition term as well as
%                               quantiles by loading data from disk.
%
% USAGE:
%
%       OVDPs = DSGEObsVarDecompDistribution(DSGEModel,CurrINI,AnnualizeData,ShockType,NumDraws,...
%                                            TotalDraws,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
%                  AnnualizeData (boolean) that takes on the value 1 if the impulse responses should be annualized
%                                 and 0 otherwise.
%
%                  ShockType (string vector) taking the values 0 (individual) or 1 (group).
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                                 parameters.
%
%                  TotalDraws (integer) with the total number of draws from the posterior distribution of the
%                                 parameters that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: OVDPs (structure) with fields "DateVector", "Y", "ShockNames", "DecompInitState", "DecompSteadyState",
%                                 "DecompMeasureError", "DecompShocks", and "Quantiles". The last field is a vector with
%                                 sub-fields "percent", and "DecompY". The "DecompY" field is an nxTxq matrix where for
%                                 each i=1,...,q it holds the term "DecompInitState+DecompSteadyState+DecompMeasureError+
%                                 DecompShocks(:,:,i)". The median values are given by the fields "DecompMedianInitState",
%                                 "DecompMedianSteadyState", "DecompMedianMeasureError" and "DecompMedianShocks".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 24, 2009.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 05-01-2010: Updated the documentation.
%
% * 27-05-2010: Fixed a bug where "CurrSaves" wasn't updated. Changed the sorting procedure
%               to a vector based algorithm.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 06-01-2012: Updated the documentation.
%
% * 28-08-2012: Added support for higest probability density confidence bands.
%
% * 04-01-2013: Updated the documentation.
%
% * 03-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 25-08-2017: Added the "Weights" input variable.
%
% * 04-09-2017: Speeded up the Weights computations.
%
% * 16-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 11-12-2023: Added the median decomposition.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize: 
%
OVDPs = [];
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
NumFiles = ceil(NumDraws/min(NumDraws,1000));
TypeStr = 'Original';
if AnnualizeData==1;
   TypeStr = 'Annualized';
end;
%
NumQuants = length(DSGEModel.Percentiles);
%
% load the first file to check certain dimensions
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-1-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-Prior-' TypeStr '-' DSGEModel.NameOfModel '-1-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
end;
OVPData = load(file);
OVDPs.ShockNames = OVPData.ShockNames;
OVDPs.Y = OVPData.Y;
[n,T] = size(OVDPs.Y);
OVDPs.DateVector = OVPData.DateVector;
ShockStr = 'shock';
if ShockType==1;
   ShockStr = 'shock group';
   OVDPs.ShockNames = DSGEModel.ShockGroupNames;
   for i=1:max(DSGEModel.ShockGroups);
      OVDPs.ShockNames(i,:) = strrep(OVDPs.ShockNames(i,:),'_',' ');
   end;
end;
q = size(OVDPs.ShockNames,1);
%
% initialize output
%
OVDPs.DecompInitState = zeros(n,T);
OVDPs.DecompSteadyState = zeros(n,T);
OVDPs.DecompMeasureError = zeros(n,T);
OVDPs.DecompShocks = zeros(n,T,q);
OVDPs.DecompMedianInitState = zeros(n,T);
OVDPs.DecompMedianSteadyState = zeros(n,T);
OVDPs.DecompMedianMeasureError = zeros(n,T);
OVDPs.DecompMedianShocks = zeros(n,T,q);
for i=1:NumQuants;
   OVDPs.Quantiles(i).DecompY = zeros(n,T,q);
   OVDPs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
WeightsT = Weights*ones(1,T);
%
% set up a wait dialog
%
if IsPosterior==1;
   txt = ['Please wait while YADA computes the posterior mean as well as percentile values for the observed variable decomposition paths. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(OVDPs.ShockNames(1,:)) ')'];
else;
   txt = ['Please wait while YADA computes the prior mean as well as percentile values for the observed variable decomposition paths. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(OVDPs.ShockNames(1,:)) ')'];
end;
WaitHandle = WaitDLG(txt,'information',['Observed Variable Decomposition' PriorHeader ' - ' TypeStr ' Data'],500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we can start loading the data
%
for i=1:n;
   for j=1:q;
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean as well as percentile values for the observed variable decomposition paths. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(OVDPs.ShockNames(j,:)) ')'];
      else;
         txt = ['Please wait while YADA computes the prior mean as well as percentile values for the observed variable decomposition paths. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(OVDPs.ShockNames(j,:)) ')'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         if j==1;
            %
            % make sure we only do this once per variable
            %
            TmpDecIS = zeros(NumDraws,T);
            TmpDecSS = zeros(NumDraws,T);
            TmpDecME = zeros(NumDraws,T);
         end;
         TmpDecShocks = zeros(NumDraws,T);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-Prior-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
            end;
            OVPData = load(file);
            OVP = OVPData.OVP;
            NumSaves = length(OVP);
            %
            % load the draws
            %
            if j==1;
               for s=1:NumSaves;
                  TmpDecIS(CurrSaves+s,:) = OVP(s).DecInitState(i,:);
                  TmpDecSS(CurrSaves+s,:) = OVP(s).DecSteadyState(i,:);
                  TmpDecME(CurrSaves+s,:) = OVP(s).DecMeasureError(i,:);
               end;
            end;
            %
            % and now the shocks
            %
            if ShockType==0;
               %
               % individual shocks
               %
               for s=1:NumSaves;
                  TmpDecShocks(CurrSaves+s,:) = OVP(s).DecShocks(i,:,j);
               end;
            else;
               %
               % shock groups
               %
               Ind = (DSGEModel.ShockGroups==j).*(1:length(DSGEModel.ShockGroups));
               Ind = Ind(Ind>0);
               for s=1:NumSaves;
                  DecompTmp = OVP(s).DecShocks(i,:,Ind);
                  if length(size(DecompTmp))==3;
                     DecompTmp = sum(DecompTmp,3);
                  end;
                  TmpDecShocks(CurrSaves+s,:) = DecompTmp;
               end;
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % compute the mean & median
         %
         if j==1;
            OVDPs.DecompInitState(i,:) = (1/NumDraws)*sum(TmpDecIS.*WeightsT,1);
            OVDPs.DecompSteadyState(i,:) = (1/NumDraws)*sum(TmpDecSS.*WeightsT,1);
            OVDPs.DecompMeasureError(i,:) = (1/NumDraws)*sum(TmpDecME.*WeightsT,1);
            OVDPs.DecompMedianInitState(i,:) = median(TmpDecIS.*WeightsT);
            OVDPs.DecompMedianSteadyState(i,:) = median(TmpDecSS.*WeightsT);
            OVDPs.DecompMedianMeasureError(i,:) = median(TmpDecME.*WeightsT);
         end;
         OVDPs.DecompShocks(i,:,j) = (1/NumDraws)*sum(TmpDecShocks.*WeightsT,1);
         OVDPs.DecompMedianShocks(i,:,j) = median(TmpDecShocks.*WeightsT);
         %
         % the full decomposition for shock or shock group j
         %
         TmpDecY = sort(TmpDecIS+TmpDecSS+TmpDecME+TmpDecShocks);
         %
         % deal with the distribution for the full decomposition
         %
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
               end;
               OVDPs.Quantiles(l).DecompY(i,:,j) = TmpDecY(QuantVal,:);
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
               for t=1:T;
                  Bounds = HighestProbabilityDensityBand(TmpDecY(:,t),ConfDraws);
                  OVDPs.Quantiles(l).DecompY(i,t,j) = Bounds(1);
                  OVDPs.Quantiles(NumQuants-l+1).DecompY(i,t,j) = Bounds(2);
               end;
            end;
         end;
      catch;
         for t=1:T;
            if j==1;
               %
               % make sure we only do this once per variable
               %
               TmpDecIS = zeros(NumDraws,1);
               TmpDecSS = zeros(NumDraws,1);
               TmpDecME = zeros(NumDraws,1);
            end;
            TmpDecShocks = zeros(NumDraws,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-Prior-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
               end;
               OVPData = load(file);
               OVP = OVPData.OVP;
               NumSaves = length(OVP);
               %
               % load the draws
               %
               if j==1;
                  for s=1:NumSaves;
                     TmpDecIS(CurrSaves+s,1) = OVP(s).DecInitState(i,t);
                     TmpDecSS(CurrSaves+s,1) = OVP(s).DecSteadyState(i,t);
                     TmpDecME(CurrSaves+s,1) = OVP(s).DecMeasureError(i,t);
                  end;
               end;
               %
               % and now the shocks
               %
               if ShockType==0;
                  %
                  % individual shocks
                  %
                  for s=1:NumSaves;
                     TmpDecShocks(CurrSaves+s,1) = OVP(s).DecShocks(i,t,j);
                  end;
               else;
                  %
                  % shock groups
                  %
                  Ind = (DSGEModel.ShockGroups==j).*(1:length(DSGEModel.ShockGroups));
                  Ind = Ind(Ind>0);
                  for s=1:NumSaves;
                     DecompTmp = OVP(s).DecShocks(i,:,Ind);
                     if length(size(DecompTmp))==3;
                        DecompTmp = sum(DecompTmp,3);
                     end;
                     TmpDecShocks(CurrSaves+s,1) = DecompTmp(:,t);
                  end;
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            %
            % compute the mean & median
            %
            if j==1;
               OVDPs.DecompInitState(i,t) = (1/NumDraws)*sum(TmpDecIS.*Weights,1);
               OVDPs.DecompSteadyState(i,t) = (1/NumDraws)*sum(TmpDecSS.*Weights,1);
               OVDPs.DecompMeasureError(i,t) = (1/NumDraws)*sum(TmpDecME.*Weights,1);
               OVDPs.DecompMedianInitState(i,t) = median(TmpDecIS.*Weights);
               OVDPs.DecompMedianSteadyState(i,t) = median(TmpDecSS.*Weights);
               OVDPs.DecompMedianMeasureError(i,t) = median(TmpDecME.*Weights);
            end;
            OVDPs.DecompShocks(i,t,j) = (1/NumDraws)*sum(TmpDecShocks.*Weights,1);
            OVDPs.DecompMedianShocks(i,t,j) = median(TmpDecShocks.*Weights);
            %
            % the full decomposition for shock or shock group j
            %
            TmpDecY = sort(TmpDecIS+TmpDecSS+TmpDecME+TmpDecShocks);
            %
            % deal with the distribution for the full decomposition
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                  end;
                  OVDPs.Quantiles(l).DecompY(i,t,j) = TmpDecY(QuantVal,t);
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TmpDecY,ConfDraws);
                  OVDPs.Quantiles(l).DecompY(i,t,j) = Bounds(1);
                  OVDPs.Quantiles(NumQuants-l+1).DecompY(i,t,j) = Bounds(2);
               end;
            end;
         end;
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of DSGEObsVarDecompDistribution.m
%
