function DoneCalc = DSGEObsVarDecomposition(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,AnnualizeData,TotalDraws,CurrChain,IsPosterior)
% DSGEObsVarDecomposition: Calculates the observed variable decomposition for the posterior or prior draws of the DSGE
%                          model and stores them on disk.
%
% USAGE:
%
%       DoneCalc = DSGEObsVarDecomposition(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,...
%                                          CurrINI,AnnualizeData,TotalDraws,CurrChain,IsPosterior)
%
% REQUIRED INPUT:  thetaPostSample (matrix) of dimension NumDrawsxNumParam with draws from the posterior or the
%                                  prior distribution of the original DSGE model parameters.
%
%                  thetaMode (vector) of dimension NumParam with the posterior mode or the initial values
%                                  of the original parameters in the DSGE model.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  AnnualizeData (boolean) that takes on the value 1 if the impulse responses should be annualized
%                                 and 0 otherwise.
%
%                  TotalDraws (integer) with the total number of parameter draws that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                 if draws from the prior are supplied.
%
% REQUIRED OUTPUT: DoneCalc (boolean) that is 1 if the calculations were completed and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: April 23, 2009.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 24-04-2009: Updated the documentation.
%
% * 27-08-2009: Added extra input and output to "StateSmoother" and "StateSmootherHt" function.
%
% * 05-01-2010: Updated the documentation.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the
%               CurrINI structure.
%
% * 02-01-2011: The function now supports the square root filter.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 11-03-2011: Switched to the "RunKalmanFilter" function.
%
% * 15-03-2011: Switched to the "RunKalmanSmoother" function.
%
% * 25-10-2011: Took into account that "RunKalmanSmoother" now requires the input variable "DiffuseInit".
%
% * 06-01-2012: Updated the documentation.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 28-01-2014: Updated the documentation.
%
% * 29-01-2014: Switched the "inv" function for "\".
%
% * 14-01-2015: Updated the documentation.
%
% * 18-03-2015: Switched to the new IsHandle function.
%
% * 09-06-2015: The code takes the CurrINI.GraphicsRoot field into account.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 17-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the way the smooth structural shocks are computed.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors for "RunKalmanSmoother".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
DoneCalc = 0;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'ovp');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\ovp". The computation of observed variable decomposition distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/ovp". The computation of observed variable decomposition distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% initialize index for saving
%
NumDraws = size(thetaPostSample,1);
SaveAfterDraws = min(NumDraws,1000);
NumFiles = ceil(NumDraws/SaveAfterDraws);
TypeStr = 'Original';
if AnnualizeData==1;
   TypeStr = 'Annualized';
end;
%
% solve the model for the posterior mode estimate or the initial values
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(DSGEModel.LastPeriod) '). YADA has therefore aborted from the computation of the observed variable decomposition distribution.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
%
[B0,KeepShocks] = RemoveRedundantColumns(B0);
[r,q] = size(B0);
ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
%
% set up the date vector
%
DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
DateVector = DateVector(DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
%
% begin the loop
%
abort = '';
LastDraw = 0;
MeanEstimationTime = 0;
ThisSave = 0;
OVP = [];
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   X = DSGEModel.X(:,DSGEModel.FirstPeriod+DSGEModel.KalmanFirstObservation-1:DSGEModel.LastPeriod);
else;
   X = [];
end;
wtT = zeros(size(Y));
T = DSGEModel.LastPeriod-DSGEModel.FirstPeriod+1;
Tsum = T-DSGEModel.KalmanFirstObservation+1;
%
% check if we should setup a wait handle
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
if DSGEModel.ShowProgress==1;
   ProgressStructure.title = ['Progress for ' num2str(NumDraws,'%0.0f') ' parameter draws'];
   ProgressStructure.facecolor = CurrINI.progress_facecolor;
   ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
   ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
   ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
   ProgressStructure.stop = 0;
   ProgressStructure.clock = DSGEModel.ShowProgressClock;
   ProgressStructure.label = 'Mean estimation time:';
   %
   ProgressStructure.name = ['Observed Variable Decomposition Distribution' PriorHeader];
   ProgressStructure.CurrINI = CurrINI;
   WaitHandle = ProgressDLG(0,ProgressStructure);
   set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
   drawnow;
else;
   txt = ['Please wait while YADA computes the distribution for the observed variable decompositions. Computations started at: ' StringTrim(datestr(now,14))];
   WaitHandle = WaitDLG(txt,'information',['Observed Variable Decomposition Distribution' PriorHeader],500,150,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% start the iterations
%
if length(size(H))==2;
   for it=1:NumDraws;
      LastDraw = LastDraw+1;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            ProgressDLG([it/NumDraws MeanEstimationTime]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(it,:)';
      tic;
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % solve the model at the posterior mode or initial parameter values
         %
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      B0 = B0(:,KeepShocks);
      %
      % run the Kalman filter to get the estimate of the state forecasts
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % calculate the smooth estimates
      %
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      %
      % estimate the economic shocks
      %
      etatT = B0'*rTMat;
      if rank(R)>0;
         if min(size(DSGEModel.X))>=1;
            wtT = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)-(A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))-(H'*KsitT);
         else;
            wtT = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod)-(H'*KsitT);
         end;
         wtT = wtT(:,DSGEModel.KalmanFirstObservation:T);
      end;
      etatT = etatT(:,DSGEModel.KalmanFirstObservation:T);
      KsitT = KsitT(:,DSGEModel.KalmanFirstObservation:T);
      %
      % compute the decomposition
      %
      if DSGEModel.k>0;
         OVP(LastDraw).DecSteadyState = A'*X;
      else;
         OVP(LastDraw).DecSteadyState = zeros(size(Y));
      end;
      OVP(LastDraw).DecMeasureError = wtT;
      OVP(LastDraw).KsitT = KsitT;
      %
      % now the shock terms
      %
      OVP(LastDraw).DecShocks = zeros(DSGEModel.n,Tsum,q);
      OVP(LastDraw).DecStateShocks = zeros(r,Tsum,q);
      for t=1:Tsum;
         for j=1:q;
            Fj = eye(r);
            ShockPart = zeros(DSGEModel.n,1);
            StatePart = zeros(r,1);
            for i=0:t-1;
               ShockPart = ShockPart+(H'*Fj*B0(:,j)*etatT(j,t-i));
               StatePart = StatePart+(Fj*B0(:,j)*etatT(j,t-i));
               Fj = F*Fj;
            end;
            OVP(LastDraw).DecShocks(:,t,j) = ShockPart;
            OVP(LastDraw).DecStateShocks(:,t,j) = StatePart;
            drawnow;
         end;
      end;
      %
      % Compute the initial state term by subtraction
      %
      OVP(LastDraw).DecInitState = Y-(OVP(LastDraw).DecSteadyState+OVP(LastDraw).DecMeasureError+sum(OVP(LastDraw).DecShocks,3));
      OVP(LastDraw).DecStateInitState = OVP(LastDraw).KsitT-sum(OVP(LastDraw).DecStateShocks,3);
      %
      % check if we should annualize
      %
      if AnnualizeData==1;
         OVP(LastDraw).DecSteadyState = Annualize(OVP(LastDraw).DecSteadyState,DSGEModel.annual,DSGEModel.annualscale);
         OVP(LastDraw).DecInitState = Annualize(OVP(LastDraw).DecInitState,DSGEModel.annual,DSGEModel.annualscale);
         OVP(LastDraw).DecMeasureError = Annualize(OVP(LastDraw).DecMeasureError,DSGEModel.annual,DSGEModel.annualscale);
         DecShocks = zeros(DSGEModel.n,size(OVP(LastDraw).DecMeasureError,2),q);
         for i=1:q;
            DecShocks(:,:,i) = Annualize(OVP(LastDraw).DecShocks(:,:,i),DSGEModel.annual,DSGEModel.annualscale);
         end;
         OVP(LastDraw).DecShocks = DecShocks;
      end;
      %
      MeanEstimationTime = (((it-1)/it)*MeanEstimationTime)+((1/it)*toc);
      %
      % check if we should save to disk
      %
      if (LastDraw==SaveAfterDraws)|(it==NumDraws);
         ThisSave = ThisSave+1;
         LastDraw = 0;
         %
         % set up the file name and then save
         %
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-Prior-' TypeStr '-' DSGEModel.NameOfModel '-'  num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
         end;
         if AnnualizeData==1;
            YOld = Y;
            Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
            DateVectorOld = DateVector;
            DateVector = DateVector(Tsum-size(Y,2)+1:Tsum);
         end;
         save(file,'OVP','Y','ShockNames','DateVector');
         if AnnualizeData==1;
            Y = YOld;
            DateVector = DateVectorOld;
         end;
         %
         % restore the data structure
         %
         OVP = [];
      end;
      drawnow;
   end;
else;
   for it=1:NumDraws;
      LastDraw = LastDraw+1;
      if DSGEModel.ShowProgress==1;
         abort = get(WaitHandle,'UserData');
         if strcmp(abort,'cancel')==1;
            break;
         else;
            ProgressDLG([it/NumDraws MeanEstimationTime]);
         end;
      end;
      %
      % Solve the model for the current parameter vector
      %
      theta = thetaPostSample(it,:)';
      tic;
      [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if status~=1;
         %
         % solve the model at the posterior mode or initial parameter values
         %
         [A,H,R,F,B0,status] = SolveDSGEModel(DSGEModel,thetaMode,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      end;
      B0 = B0(:,KeepShocks);
      %
      % run the Kalman filter to get the estimate of the state forecasts
      %
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
      %
      % calculate the smooth estimates
      %
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      %
      % estimate the economic shocks
      %
      etatT = B0'*rTMat;
      if rank(R)>0;
         wtT = zeros(DSGEModel.n,size(KsitT,2));
         if min(size(DSGEModel.X))>=1;
            for t=DSGEModel.FirstPeriod:DSGEModel.LastPeriod;
               wtT(:,t-DSGEModel.FirstPeriod+1) = DSGEModel.Y(:,t)-(A'*DSGEModel.X(:,t))-(H(:,:,t)'*KsitT(:,t-DSGEModel.FirstPeriod+1));
            end;
         else;
            for t=DSGEModel.FirstPeriod:DSGEModel.LastPeriod;
               wtT(:,t-DSGEModel.FirstPeriod+1) = DSGEModel.Y(:,t)-(H(:,:,t)'*KsitT(:,t-DSGEModel.FirstPeriod+1));
            end;
         end;
         wtT = wtT(:,DSGEModel.KalmanFirstObservation:T);
      end;
      etatT = etatT(:,DSGEModel.KalmanFirstObservation:T);
      KsitT = KsitT(:,DSGEModel.KalmanFirstObservation:T);
      H = H(:,:,DSGEModel.KalmanFirstObservation:T);
      %
      %
      % compute the decomposition
      %
      if DSGEModel.k>0;
         OVP(LastDraw).DecSteadyState = A'*X;
      else;
         OVP(LastDraw).DecSteadyState = zeros(size(Y));
      end;
      OVP(LastDraw).DecMeasureError = wtT;
      OVP(LastDraw).KsitT = KsitT;
      %
      % now the shock terms
      %
      OVP(LastDraw).DecShocks = zeros(DSGEModel.n,Tsum,q);
      OVP(LastDraw).DecStateShocks = zeros(r,Tsum,q);
      for t=1:Tsum;
         for j=1:q;
            Fj = eye(r);
            ShockPart = zeros(DSGEModel.n,1);
            StatePart = zeros(r,1);
            for i=0:t-1;
               ShockPart = ShockPart+(H(:,:,t)'*Fj*B0(:,j)*etatT(j,t-i));
               StatePart = StatePart+(Fj*B0(:,j)*etatT(j,t-i));
               Fj = F*Fj;
            end;
            OVP(LastDraw).DecShocks(:,t,j) = ShockPart;
            OVP(LastDraw).DecStateShocks(:,t,j) = StatePart;
            drawnow;
         end;
      end;
      %
      % Compute the initial state term by subtraction
      %
      OVP(LastDraw).DecInitState = Y-(OVP(LastDraw).DecSteadyState+OVP(LastDraw).DecMeasureError+sum(OVP(LastDraw).DecShocks,3));
      OVP(LastDraw).DecStateInitState = OVP(LastDraw).KsitT-sum(OVP(LastDraw).DecStateShocks,3);
      %
      % check if we should annualize
      %
      if AnnualizeData==1;
         OVP(LastDraw).DecSteadyState = Annualize(OVP(LastDraw).DecSteadyState,DSGEModel.annual,DSGEModel.annualscale);
         OVP(LastDraw).DecInitState = Annualize(OVP(LastDraw).DecInitState,DSGEModel.annual,DSGEModel.annualscale);
         OVP(LastDraw).DecMeasureError = Annualize(OVP(LastDraw).DecMeasureError,DSGEModel.annual,DSGEModel.annualscale);
         DecShocks = zeros(DSGEModel.n,size(OVP(LastDraw).DecMeasureError,2),q);
         for i=1:q;
            DecShocks(:,:,i) = Annualize(DecShocks(:,:,i),DSGEModel.annual,DSGEModel.annualscale);
         end;
         OVP(LastDraw).DecShocks = DecShocks;
      end;
      %
      MeanEstimationTime = (((it-1)/it)*MeanEstimationTime)+((1/it)*toc);
      %
      % check if we should save to disk
      %
      if (LastDraw==SaveAfterDraws)|(it==NumDraws);
         ThisSave = ThisSave+1;
         LastDraw = 0;
         %
         % set up the file name and then save
         %
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\ovp\OVP-Prior-' TypeStr '-' DSGEModel.NameOfModel '-'  num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
         end;
         if AnnualizeData==1;
            YOld = Y;
            Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
            DateVectorOld = DateVector;
            DateVector = DateVector(Tsum-size(Y,2)+1:Tsum);
         end;
         save(file,'OVP','Y','ShockNames','DateVector');
         if AnnualizeData==1;
            Y = YOld;
            DateVector = DateVectorOld;
         end;
         %
         % restore the data structure
         %
         OVP = [];
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
if DSGEModel.ShowProgress==1;
   if IsHandle(WaitHandle)==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   set(WaitControls.text,'UserData','done');
   delete(WaitHandle);
   drawnow;
   pause(0.02);
end;
%
% check if we have computed all output
%
if strcmp(abort,'cancel')==0;
   DoneCalc = 1;
end;

%
% end of DSGEObsVarDecomposition.m
%
