function [StateDecomp,status,kalmanstatus] = DSGEObservationWeightsThetaDiffuse(theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI)
% DSGEObservationWeightsThetaDiffuse: Computes observation weights and decompositions into the observed variables for
%                                     the state variables and the shocks when diffuse initialization of the state variables
%                                     is used.
%
% USAGE:
%
%       [StateDecomp,status,kalmanstatus] = DSGEObservationWeightsThetaDiffuse(theta,thetaPositions,ModelParameters,...
%                                                                              VarType,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  theta (vector) with values for the original parameters of the DSGE model.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  VarType (integer) that indicates (1) state variable decompositions, (2) shock decompositions,
%                                 (3) state variable weights, and (4) shock weights.
%
%                  DSGEModel (structure) whose fields contain all required information
%                                 about the model, such as input files, variable and sample
%                                 data. Actual observations are not stored in this file.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: StateDecomp (structure) with fields:
%
%                                   1. StateForecastWeights         (TxT) cell array with rxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   2. StateUpdateWeights           (TxT) cell array with rxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   3. StateSmoothWeights           (TxT) cell array with rxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   4. ShockUpdateWeights           (TxT) cell array with qxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   5. ShockSmoothWeights           (TxT) cell array with qxn matrices. Rows represent time period
%                                                                   for state variables and columns the time period for the observed
%                                                                   variables.
%                                   6. StateForecastInitCondWeights (T) cell array with rxr matrices.
%                                   7. StateUpdateInitCondWeights   (T) cell array with rxr matrices.
%                                   8. StateSmoothInitCondWeights   (T) cell array with rxr matrices.
%                                   9. ShockUpdateInitCondWeights   (T) cell array with qxr matrices.
%                                  10. ShockSmoothInitCondWeights   (T) cell array with qxr matrices.
%                                  11. StateForecastDecomp          (rxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  12. StateUpdateDecomp            (rxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  13. StateSmoothDecomp            (rxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  14. ShockUpdateDecomp            (qxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  15. ShockSmoothDecomp            (qxTxn+1) matrix with the terms exlained by the n observed variables
%                                                                   and the initial condition for the state.
%                                  16. Ksitt1                       (rxT) matrix with the forecast estimates of the state variables.
%                                  17. Ksitt                        (rxT) matrix with the update estimates of the state variables.
%                                  18. KsitT                        (rxT) matrix with the smooth estimates of the state variables.
%                                  19. etatt                        (qxT) matrix with the update estimates of the shocks.
%                                  20. etatT                        (qxT) matrix with the update estimates of the shocks.
%                                  21. DateVector                   (T) with numeric values for the sample dates.
%                                  22. SubSampleStrMat              (TxT1) string matrix with the sample dates.
%                                  23. VariableNames                (rxR1 or qxQ1) string matrix with the names of the state
%                                                                   variables or the shocks.
%                                  24. StdDev                       nxT matrix with model-based standard deviations of the observed variables.
%                                  25. StateStdDev                  rx1 vector with model-based standard deviations of the state variables.
%
%                                The specific fields that are provided in the output structure depends on "VarType".
%
% OPTIONAL OUTPUT: status (integer) Indicates whether the solution of the model is unique or not.
%                                 mcode is 1 when unique, 0 if not computed, and various other values when
%                                 there are problems with the solution. Specifically:
%
%                                      mcode = 2:      Roots not correctly computed by real_schur
%                                      mcode = 3:      Too many big roots
%                                      mcode = 35:     Too many big roots, and q(:,right) is singular
%                                      mcode = 4:      Too few big roots
%                                      mcode = 45:     Too few big roots, and q(:,right) is singular
%                                      mcode = 5:      q(:,right) is singular
%                                      mcode = 61:     Too many exact shiftrights
%                                      mcode = 62:     Too many numeric shiftrights
%                                      mcode = 7:      The a matrix has infinite or NaN values. Eigenvalues
%                                                      cannot be calculated.
%                                      mcode = 8:      The function "compute_aim_matrices" returns complex
%                                                      numbers.
%                                      mcode = -1:     No stable solution.
%                                      mcode = -2:     Too many large generalized eigenvalues.
%                                      mcode = -3:     Too few large generalized eigenvalues.
%                                      else            Return code not properly specified
%
%                  kalmanstatus (integer) indicating the covariance stationarity of the state variables. If all the
%                                 eigenvalues of F are inside the unit circle, then kalmanstatus=0 and the log-likelihood function
%                                 is calculated. If not, then status=1 and initP is set to 0. In addition, kalmanstatus can be
%                                 -1 if ths forecast covariance matrix is not positive definite for some time period.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 27, 2019.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 28-02-2019: Continued the function of the state variable filtering case with t=1.
%
% * 01-03-2019: Continued the state variable filtering case for t=2,...,d.
%
% * 05-03-2019: Continued with the update shocks for t=1,...,d.
%
% * 06-03-2019: Verifying the update shock estimates over the initialization sample.
%
% * 07-03-2019: Continued with the update shocks for t=2,...,d.
%
% * 13-03-2019: Continued the function with time-varying H matrix for t=1,2,...,d.
%
% * 14-03-2019: Managed to initialize KalmanGain and Lt correctly for the post-initial sample
%               part, i.e., for period d taking into account that ksi(t+1|t) = F*ksi(t|t). Added
%               all code up to the smoothing part for the initial sample.
%
% * 15-03-2019: Finished the function.
%
% * 21-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 06-01-2023: Updated the documentation.
%
% * 10-02-2023: Added support for the driver.
%
% * 25-07-2023: Added extra output when computing the state variable decompositions such that
%               they can be used for forecast path decompositions.
%
% * 27-07-2023: The memory question is only posed when the function is not run via the
%               driver functionality.
%
% * 25-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the way the smooth structural shocks are computed.
%
% * 27-02-2024: Fixed the structural update shocks to use rttMat from "RunKalmanSmoother".
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize output
%
StateDecomp = [];
if nargout==1;
   %
   % copy files to the tmp directory for posterior mode values
   %
   [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
   [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
   end;
   if FileExist(DSGEModel.SystemPriorFile)==1;
      [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
   end;
   %
   % check if we should copy model solver file
   %
   if DSGEModel.ModelSolver==4;
      if FileExist(DSGEModel.ModelSolverFile)==1;
         [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
      end;
   end;
   %
   % Update any parameteters that need updating
   %
   if FileExist(DSGEModel.UpdateParameterFile)==1;
      ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
   end;
   EstType = 'Posterior Mode';
else;
   status = 1;
   kalmanstatus = 0;
   EstType = 'Initial Values';
end;
%
% check if the user wants to continue
%
if DSGEModel.RunDriver==0;
   if DSGEModel.n>30;
      txt = ['Observation weight computations require a lot of memory. Your model has many observed variables (' int2str(DSGEModel.n) ') and state variables (' int2str(length(DSGEModel.StateVariablePositions)) ') and the computations may therefore fail. YADA will try to capture any such failure, but running out of memory can make your computer unstable. Are you sure you want to continue?'];
      answer = Query(txt,'question',200,['Observation Weights - ' EstType],600,'no',CurrINI);
      if strcmp(lower(answer),'no')==1;
         return;
      end;
   end;
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   if nargout>1;
      status = 0;
      kalmanstatus = 0;
   end;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% try to solve the model
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'error','YADA Error',CurrINI.scrsz(4)-50,600,CurrINI);
   drawnow;
   return;
end;
[B0,KeepVar] = RemoveRedundantColumns(B0);
q = size(B0,2);
if q==0;
   txt = 'The DSGE model doesn''t have any structural shocks. YADA is therefore unable to compute any observation weights or decompostions.';
   About(txt,'information','YADA - No Shocks',200,500,CurrINI);
   return;
end;
n = DSGEModel.n;
k = DSGEModel.k;
%
% check if the solution is unique and convergent
%
if nargout>1;
   status = mcode;
   if mcode~=1;
      if mcode==2;
         mcodeStr = 'Roots are not correctly computed by real_schur.'
      elseif mcode==3;
         mcodeStr = 'Too many big roots.';
      elseif mcode==35;
         mcodeStr = 'Too many big roots, and q(:,right) is singular.';
      elseif mcode==4;
         mcodeStr = 'Too few big roots.';
      elseif mcode==45;
         mcodeStr = 'Too few big roots, and q(:,right) is singular.';
      elseif mcode==5;
         mcodeStr = 'q(:,right) is singular.';
      elseif mcode==61;
         mcodeStr = 'Too many exact shiftrights.';
      elseif mcode==62;
         mcodeStr = 'Too many numeric shiftrights.';
      elseif mcode==7;
         mcodeStr = 'Infinite or NaN values detected.';
      elseif mcode==8;
         mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
      elseif mcode==-1;
         mcodeStr = 'No stable solution.';
      elseif mcode==-2;
         mcodeStr = 'Too many large generalized eigenvalues.';
      elseif mcode==-3;
         mcodeStr = 'Too few large generalized eigenvalues.';
      else;
         mcodeStr = 'Return code not properly specified.';
      end;
      if DSGEModel.ModelSolver==1;
         SolverStr = 'AiM';
      elseif DSGEModel.ModelSolver==2;
         SolverStr = 'Klein';
      elseif DSGEModel.ModelSolver==3;
         SolverStr = 'Gensys';
      elseif DSGEModel.ModelSolver==4;
         SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
      end;
      txt = ['The ' SolverStr ' solver provided the return code: ' int2str(mcode) ', i.e., "' mcodeStr '"'];
      About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
      return;
   end;
end;
%
% check if measurement equations are properly defined for all time periods
%
if length(size(H))==3;
   if size(H,3)<DSGEModel.T;
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is less than the number of observations (T = ' int2str(DSGEModel.T) '). YADA has therefore aborted from the observation weight computations.'];
      About(ErrorStr,'error','Error - Measurement Equation',00,600,CurrINI);
      return;
   end;
end;
%
% check the sample (if needed for Kalman filter)
%
[FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
%
% deal with the settings for the initial value of the state variables
%
r = length(DSGEModel.StateVariablePositions);
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      InitialStateVector = zeros(r,1);
   end;
else;
   InitialStateVector = zeros(r,1);
end;
%
% We can now try to run the Kalman filter
%
if MatlabNumber>=8.0;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanG,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,A,H,F,B0,R,InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
else;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanG,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,FirstPeriod,LastPeriod,A,H,F,B0,R,InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError];
      About(txt,'error','Error - Kalman Filter',200,500,CurrINI);
      return;
   end;
end;
%
% we should adjust everything based on DSGEModel.KalmanFirstObservation
%
if nargout>1;
   kalmanstatus = stat;
   if kalmanstatus~=0;
      return;
   end;
end;
%
% Run the state smoother
%
if MatlabNumber>=8.0;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat] = RunKalmanSmoother(DSGEModel.Y,FirstPeriod,LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanG,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch ME;
      Ksitt = [];
      KsitT = [];
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
else;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat,rTMat] = RunKalmanSmoother(DSGEModel.Y,FirstPeriod,LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanG,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch;
      Ksitt = [];
      KsitT = [];
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
%
% compute the shocks
%
T = size(KsitT,2);
etatT = B0'*rTMat;
etatt = B0'*rttMat;
%
% we can now compute weights and decompositions
%
if k>0;
   Z = DSGEModel.Y(:,FirstPeriod:LastPeriod)-(A'*DSGEModel.X(:,FirstPeriod:LastPeriod));
else;
   Z = DSGEModel.Y(:,FirstPeriod:LastPeriod);
end;
%
% Adjust data for measurement error covariance matrix if we need to
% use the univariate Kalman filter during initialization
%
if isequal(R,diag(diag(R)))==1;
   Zs = Z;
   S = eye(n);
   Hs = H;
   Rs = R;
else;
   [S,L] = eig(R);
   Zs = S'*Z;
   Hs = H*S;
   Rs = L;
end;
%
% 1. The forecast and the update parts
%
StateForecastWeights = cell(T,T);
StateForecastInitCondWeights = cell(1,T);
StateForecastWeights{1,1} = zeros(r,n);
StateForecastInitCondWeights{1,1} = eye(r);
StateForecastDecomp = zeros(r,T,n+1);
StateForecastDecomp(:,1,n+1) = InitialStateVector;
%
StateUpdateWeights = cell(T,T);
StateUpdateInitCondWeights = cell(1,T);
StateUpdateDecomp = zeros(r,T,n+1);
%
ShockUpdateWeights = cell(T,T);
ShockUpdateInitCondWeights = cell(1,T);
ShockUpdateDecomp = zeros(q,T,n+1);
%
d = KalmanG.d;
%
% initialize help matrices
%
Mt = cell(1,T);
Gt = cell(1,T);
Lt = cell(1,T);
KalmanGain = cell(1,T);
%
% we should skip Bbart
%
Lhatt = cell(1,d);
Khatt = cell(1,d);
%Amat = cell(1,d);
Fhatt = cell(1,d);
Ghatt = cell(1,d);
Hhatt = cell(1,d);
%Bbart = cell(1,d);
Jr = [eye(r);zeros(r,r)];
%Mhatt = cell(1,d);
%Nhatt = cell(1,d);
%
% display a wait dialog
%
if DSGEModel.RunDriver==0;
   if VarType==1;
      txt = 'Please wait while YADA computes the decompositions of the state variables for the forecast and update estimates.';
   elseif VarType==2;
      txt = 'Please wait while YADA computes the decompositions of the shocks for the update estimates.';
   elseif VarType==3;
      txt = 'Please wait while YADA computes the weights for the decompositions of the state variables for the forecast and update estimates.';
   else;
      txt = 'Please wait while YADA computes the weights for the decompositions of the shocks for the update estimates.';
   end;
   WaitHandle = WaitDLG(txt,'information',['Observation Weights - ' EstType],500,200,CurrINI,0);
   WaitControls = get(WaitHandle,'UserData');
end;
%
% separate the cases when we have a univariate and a multivariate filtering and smoothing
%
if DSGEModel.KalmanAlgorithm~=3;
   %
   % we have standard or square-root filtering and smoothing
   %
   SigmaInf = KalmanG.SigmaInf;
   SigmaStar = KalmanG.SigmaStar;
   PInf = KalmanG.PInf;
   PStar = KalmanG.PStar;
   KInfMat = KalmanG.KInfMat;
   LInfMat = KalmanG.LInfMat;
   KStarMat = KalmanG.KStarMat;
   LStarMat = KalmanG.LStarMat;
   sigma2iInfCell = KalmanG.sigma2iInfCell;
   sigma2iStarCell = KalmanG.sigma2iStarCell;
   KtiInfCell = KalmanG.KtiInfCell;
   KtiStarCell = KalmanG.KtiStarCell;
   HtiCell = KalmanG.HtiCell;
   clear('KalmanG');
   %
   if length(size(H))==2;
      %
      % dealing with the initialization subsample
      %
      for t=1:d;
         pos = (~isnan(Z(:,t)));
         n1 = sum(pos);
         rSI = rank(SigmaInf(:,:,t),1.0e-10);
         if t==1;
            if n1>0;
               if rSI==n1;
                  %
                  % SigmaInf has full rank n1
                  %
                  StateUpdateWeights{1,1} = PInf(:,:,1)*H(:,pos)*pinv(SigmaInf(pos,pos,1));
                  StateUpdateInitCondWeights{1,1} = eye(r)-(StateUpdateWeights{1,1}*H(:,pos)');
                  Lhatt{1,1} = LInfMat(:,:,1);
                  Khatt{1,1} = KInfMat(:,:,1);
                  Fhatt{1,1} = [zeros(r,n1);H(:,pos)*pinv(SigmaInf(pos,pos,1))];
                  Ghatt{1,1} = [zeros(r,r);H(:,pos)*pinv(SigmaInf(pos,pos,1))*H(:,pos)'];
                  Hhatt{1,1} = [LInfMat(:,:,1) -KStarMat(:,pos,1)*H(:,pos)';zeros(r,r) LInfMat(:,:,1)]';
                  Fbar = H(:,pos)*pinv(SigmaInf(pos,pos,1));
                  Gbar = H(:,pos)*pinv(SigmaInf(pos,pos,1))*H(:,pos)';
               elseif rSI==0;
                  %
                  % SigmaInf is zero
                  %
                  StateUpdateWeights{1,1} = PStar(:,:,1)*H(:,pos)*pinv(SigmaStar(pos,pos,1));
                  StateUpdateInitCondWeights{1,1} = eye(r)-(StateUpdateWeights{1,1}*H(:,pos)');
                  Lhatt{1,1} = LStarMat(:,:,1);
                  Khatt{1,1} = KStarMat(:,:,1);
                  Fhatt{1,1} = [H(:,pos)*pinv(SigmaStar(pos,pos,1));zeros(r,n1)];
                  Ghatt{1,1} = [H(:,pos)*pinv(SigmaStar(pos,pos,1))*H(:,pos)';zeros(r,r)];
                  Hhatt{1,1} = [LStarMat(:,:,1) zeros(r,r);zeros(r,r) F]';
                  Fbar = H(:,pos)*pinv(SigmaStar(pos,pos,1));
                  Gbar = H(:,pos)*pinv(SigmaStar(pos,pos,1))*H(:,pos)';
               else;
                  %
                  % we have univariate filtering
                  %
                  % need to compute Lbar and Kbar
                  %
                  Hs = HtiCell{t};
                  sigma2iInft = sigma2iInfCell{t};
                  sigma2iStart = sigma2iStarCell{t};
                  KtiInft = KtiInfCell{t};
                  KtiStart = KtiStarCell{t};
                  %
                  % we loop here down from n1 to 1 to create
                  % the Lbar and Kbar matrices in equation (5.146)
                  % of the YADA manual
                  %
                  Bmat = cell(1,n1);
                  Lbar = eye(r);
                  Kbar = zeros(r,n1);
                  At = zeros(2*r,n1);
                  Lt = cell(1,n1);
                  Kt = zeros(r,n1);
                  for i=n1:-1:1;
                     Hi = Hs(:,i);
                     Bti = zeros(2*r,2*r);
                     if sigma2iInft(i)>1.0e-10;
                        Lti = eye(r)-(KtiInft(:,i)*Hi'*(1/sigma2iInft(i)));
                        Kti = KtiInft(:,i)*(1/sigma2iInft(i));
                        At(r+1:2*r,i) = Hi*(1/sigma2iInft(i));
                        Bti(1:r,1:r) = Lti;
                        Bti(r+1:2*r,r+1:2*r) = Lti;
                        Bti(1:r,r+1:2*r) = ((KtiInft(:,i)*(sigma2iStart(i)/sigma2iInft(i)))-KtiStart(:,i))*Hi'*(1/sigma2iInft(i));
                     else;
                        Lti = eye(r)-(KtiStart(:,i)*Hi'*(1/sigma2iStart(i)));
                        Kti = KtiStart(:,i)*(1/sigma2iStart(i));
                        At(1:r,i) = Hi*(1/sigma2iStart(i));
                        Bti(1:r,1:r) = Lti;
                        Bti(r+1:2*r,r+1:2*r) = Lti;
                     end;
                     Bti = Bti';
                     Bmat{1,i} = Bti;
                     if i==n1;
                        Kbar(:,i) = Kti;
                     else;
                        Kbar(:,i) = Lbar*Kti;
                     end;
                     Lbar = Lbar*Lti;
                     Lt{1,i} = Lti;
                     Kt(:,i) = Kti;
                  end;
                  %Amat{1,1} = At;
                  %
                  % now we need to loop from 1 to n1 to construct the Fbar and Gbar matrices
                  %
                  Bbar = eye(2*r);
                  Lbar = eye(r);
                  Cmat = eye(n1);
                  Fbar = zeros(2*r,n1);
                  Gbar = zeros(2*r,r);
                  for i=1:n1;
                     Ci = Cmat(i,:);
                     if i>1;
                        Hi = Hs(:,i);
                        Lbari = eye(r);
                        for j=1:i-1;
                           Ci(1,i-j) = -Hi'*Lbari*Kt(:,i-j);
                           Lbari = Lt{1,i-j}*Lbari;
                        end;
                     end;
                     Fbar = Fbar+(Bbar*At(:,i)*Ci*S(pos,pos)');
                     Gbar = Gbar+(Bbar*At(:,i)*Hi'*Lbar);
                     %
                     Lbar = Lt{1,i}*Lbar;
                     Bbar = Bbar*Bmat{1,i};
                  end;
                  Fhatt{1,1} = Fbar;
                  Ghatt{1,1} = Gbar;
                  %Bbart{1,1} = Bbar;
                  Hhatt{1,1} = Bbar*[F' zeros(r,r);zeros(r,r) F'];
                  Fbar = Jr'*Fbar;
                  Gbar = Jr'*Gbar;
                  %
                  % deal with possible measurement errors with S
                  %
                  Kbar = Kbar*S(pos,pos)';
                  StateUpdateWeights{1,1} = Kbar;
                  StateUpdateInitCondWeights{1,1} = Lbar;
                  Lhatt{1,1} = F*Lbar;
                  Khatt{1,1} = F*Kbar;
               end;
               StateUpdateDecomp(:,1,n+1) = StateUpdateInitCondWeights{1,1}*InitialStateVector;
               %
%               disp([Ksitt(:,1) StateUpdateWeights{1,1}*Z(:,1)])
%               disp([etatt(:,1) B0'*Fbar*Z(pos,1)])
               ShockUpdateWeights{1,1} = B0'*Fbar;
               ShockUpdateInitCondWeights{1,1} = -B0'*Gbar;
               ShockUpdateDecomp(:,1,n+1) = ShockUpdateInitCondWeights{1,1}*InitialStateVector;
               %
               %
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               AlphaMatU(:,pos) = StateUpdateWeights{1,1};
               AlphaMatEtaU(:,pos) = ShockUpdateWeights{1,1};
               for j=1:n;
                  if isnan(Z(j,1))==0;
                     StateUpdateDecomp(:,1,j) = AlphaMatU(:,j)*Z(j,1);
                     ShockUpdateDecomp(:,1,j) = AlphaMatEtaU(:,j)*Z(j,1);
                  end;
               end;
            else;
               %
               % all data are missing for period t
               %
               StateUpdateWeights{1,1} = zeros(r,[]);
               StateUpdateInitCondWeights{1,1} = zeros(r,r);
               StateUpdateDecomp(:,1,n+1) = zeros(r,1);
               %
               ShockUpdateWeights{1,1} = zeros(q,[]);
               %
               Lhatt{1,1} = zeros(r,r);
               Khatt{1,1} = zeros(r,[]);
               Fhatt{1,1} = zeros(2*r,[]);
               Ghatt{1,1} = zeros(2*r,2*r);
               Hhatt{1,1} = zeros(2*r,2*r);
               %Bbar{1,1} = zeros(2*r,2*r);
            end;
         else;
            %
            % t>=2 up to t=d
            %
            if n1>0;
               if rSI==n1;
                  %
                  % SigmaInf has full rank
                  %
                  Lhatt{1,t} = LInfMat(:,:,t);
                  Khatt{1,t} = KInfMat(:,:,t);
                  StateUpdateWeights{t,t} = PInf(:,:,t)*H(:,pos)*pinv(SigmaInf(pos,pos,t));
                  StateUpdateInitCondWeights{1,t} = eye(r)-(StateUpdateWeights{1,t}*H(:,pos)');
                  Fhatt{1,t} = [zeros(r,n1);H(:,pos)*pinv(SigmaInf(pos,pos,t))];
                  Ghatt{1,t} = [zeros(r,r);H(:,pos)*pinv(SigmaInf(pos,pos,t))*H(:,pos)'];
                  Hhatt{1,t} = [LInfMat(:,:,t) -KStarMat(:,pos,t)*H(:,pos)';zeros(r,r) LInfMat(:,:,t)]';
                  Fbar = H(:,pos)*pinv(SigmaInf(pos,pos,t));
                  Gbar = H(:,pos)*pinv(SigmaInf(pos,pos,t))*H(:,pos)';
               elseif rSI==0;
                  %
                  % SigmaInf is zero
                  %
                  Lhatt{1,t} = LStarMat(:,:,t);
                  Khatt{1,t} = KStarMat(:,:,t);
                  StateUpdateWeights{t,t} = PStar(:,:,t)*H(:,pos)*pinv(SigmaStar(pos,pos,t));
                  StateUpdateInitCondWeights{1,t} = eye(r)-(StateUpdateWeights{1,t}*H(:,pos)');
                  Fhatt{1,t} = [H(:,pos)*pinv(SigmaStar(pos,pos,t));zeros(r,n1)];
                  Ghatt{1,t} = [H(:,pos)*pinv(SigmaStar(pos,pos,t))*H(:,pos)';zeros(r,r)];
                  Hhatt{1,t} = [LStarMat(:,:,t) zeros(r,r);zeros(r,r) F]';
                  Fbar = H(:,pos)*pinv(SigmaStar(pos,pos,t));
                  Gbar = H(:,pos)*pinv(SigmaStar(pos,pos,t))*H(:,pos)';
               else;
                  %
                  % we have univariate filtering
                  %
                  Hs = HtiCell{t};
                  sigma2iInft = sigma2iInfCell{t};
                  sigma2iStart = sigma2iStarCell{t};
                  KtiInft = KtiInfCell{t};
                  KtiStart = KtiStarCell{t};
                  %
                  % we loop here down from n1 to 1 to create
                  % the Lbar and Kbar matrices in equation (5.146)
                  % of the YADA manual
                  %
                  Bmat = cell(1,n1);
                  Lbar = eye(r);
                  Kbar = zeros(r,n1);
                  At = zeros(2*r,n1);
                  Lt = cell(1,n1);
                  Kt = zeros(r,n1);
                  for i=n1:-1:1;
                     Hi = Hs(:,i);
                     Bti = zeros(2*r,2*r);
                     if sigma2iInft(i)>1.0e-10;
                        Lti = eye(r)-(KtiInft(:,i)*Hi'*(1/sigma2iInft(i)));
                        Kti = KtiInft(:,i)*(1/sigma2iInft(i));
                        At(r+1:2*r,i) = Hi*(1/sigma2iInft(i));
                        Bti(1:r,1:r) = Lti;
                        Bti(r+1:2*r,r+1:2*r) = Lti;
                        Bti(1:r,r+1:2*r) = ((KtiInft(:,i)*(sigma2iStart(i)/sigma2iInft(i)))-KtiStart(:,i))*Hi'*(1/sigma2iInft(i));
                     else;
                        Lti = eye(r)-(KtiStart(:,i)*Hi'*(1/sigma2iStart(i)));
                        Kti = KtiStart(:,i)*(1/sigma2iStart(i));
                        At(1:r,i) = Hi*(1/sigma2iStart(i));
                        Bti(1:r,1:r) = Lti;
                        Bti(r+1:2*r,r+1:2*r) = Lti;
                     end;
                     Bti = Bti';
                     Bmat{1,i} = Bti;
                     if i==n1;
                        Kbar(:,i) = Kti;
                     else;
                        Kbar(:,i) = Lbar*Kti;
                     end;
                     Lbar = Lbar*Lti;
                     Lt{1,i} = Lti;
                     Kt(:,i) = Kti;
                  end;
                  %Amat{1,t} = At;
                  %
                  % now we need to loop from 1 to n1 to construct the Fbar and Gbar matrices
                  %
                  Bbar = eye(2*r);
                  Lbar = eye(r);
                  Cmat = eye(n1);
                  Fbar = zeros(2*r,n1);
                  Gbar = zeros(2*r,r);
                  for i=1:n1;
                     Ci = Cmat(i,:);
                     if i>1;
                        Hi = Hs(:,i);
                        Lbari = eye(r);
                        for j=1:i-1;
                           Ci(1,i-j) = -Hi'*Lbari*Kt(:,i-j);
                           Lbari = Lt{1,i-j}*Lbari;
                        end;
                     end;
                     Fbar = Fbar+(Bbar*At(:,i)*Ci*S(pos,pos)');
                     Gbar = Gbar+(Bbar*At(:,i)*Hi'*Lbar);
                     %
                     Lbar = Lt{1,i}*Lbar;
                     Bbar = Bbar*Bmat{1,i};
                  end;
                  Fhatt{1,t} = Fbar;
                  Ghatt{1,t} = Gbar;
                  %Bbart{1,t} = Bbar;
                  Hhatt{1,t} = Bbar*[F' zeros(r,r);zeros(r,r) F'];
                  Fbar = Jr'*Fbar;
                  Gbar = Jr'*Gbar;
                  %
                  % deal with possible measurement errors with S
                  %
                  Kbar = Kbar*S(pos,pos)';
                  Lhatt{1,t} = F*Lbar;
                  Khatt{1,t} = F*Kbar;
                  StateUpdateWeights{t,t} = Kbar;
                  StateUpdateInitCondWeights{1,t} = Lbar;
               end;
            else;
               %
               % all data are missing for period t
               %
               Lhatt{1,t} = zeros(r,r);
               Khatt{1,t} = zeros(r,[]);
               StateUpdateWeights{t,t} = zeros(r,[]);
               StateUpdateInitCondWeights{1,t} = zeros(r,r);
               StateUpdateDecomp(:,t,n+1) = zeros(r,1);
               StateForecastDecomp(:,t,n+1) = zeros(r,1)
               %
               Fhatt{1,t} = zeros(2*r,[]);
               Ghatt{1,t} = zeros(2*r,2*r);
               Hhatt{1,t} = zeros(2*r,2*r);
               Gbar = zeros(r,2*r);
               Fbar = zeros(r,[]);
            end;
            %
            % we can now begin with the forecast part
            %
            for tau=1:t-1;
               if tau<t-1;
                  StateForecastWeights{t,tau} = Lhatt{1,t-1}*StateForecastWeights{t-1,tau};
               else;
                  StateForecastWeights{t,t-1} = Khatt{1,t-1};
               end;
               % StateUpdateInitCondWeights{1,t} is equal to Mhatt
               StateUpdateWeights{t,tau} = StateUpdateInitCondWeights{1,t}*StateForecastWeights{t,tau};
               ShockUpdateWeights{t,tau} = -B0'*Gbar*StateForecastWeights{t,tau};
            end;
            StateForecastInitCondWeights{1,t} = Lhatt{1,t-1}*StateForecastInitCondWeights{1,t-1};
            ShockUpdateWeights{t,t} = B0'*Fbar;
            ShockUpdateInitCondWeights{1,t} = -B0'*Gbar;
%            if t==2;
%               disp([Ksitt1(:,2) StateForecastWeights{2,1}*Z(:,1)]);
%               disp([Ksitt(:,2) StateUpdateWeights{2,2}*Z(:,2)+StateUpdateWeights{2,1}*Z(:,1)])
%               disp([etatt(:,2) ShockUpdateWeights{2,2}*Z(:,2)+ShockUpdateWeights{2,1}*Z(:,1)])
%            elseif t==3;
%               disp([Ksitt1(:,3) StateForecastWeights{3,2}*Z(:,2)+StateForecastWeights{3,1}*Z(:,1)]);
%               disp([Ksitt(:,3) StateUpdateWeights{3,3}*Z(:,3)+StateUpdateWeights{3,2}*Z(:,2)+StateUpdateWeights{3,1}*Z(:,1)])
%               disp([etatt(:,3) ShockUpdateWeights{3,3}*Z(:,3)+ShockUpdateWeights{3,2}*Z(:,2)+ShockUpdateWeights{3,1}*Z(:,2)])
%            end;
            %
            % decompositions
            %
            for j=1:n;
               for tau=1:t-1;
                  AlphaMatF = NaN*ones(r,n);
                  AlphaMatU = NaN*ones(r,n);
                  AlphaMatEtaU = NaN*ones(q,n);
                  postau = (~isnan(Z(:,tau)));
                  AlphaMatF(:,postau) = StateForecastWeights{t,tau};
                  AlphaMatU(:,postau) = StateUpdateWeights{t,tau};
                  AlphaMatEtaU(:,postau) = ShockUpdateWeights{t,tau};
                  if isnan(Z(j,tau))==0;
                     StateForecastDecomp(:,t,j) = StateForecastDecomp(:,t,j)+(AlphaMatF(:,j)*Z(j,tau));
                     StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,tau));
                     ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,tau));
                  end;
               end;
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               AlphaMatU(:,pos) = StateUpdateWeights{t,t};
               AlphaMatEtaU(:,pos) = ShockUpdateWeights{t,t};
               if isnan(Z(j,t))==0;
                  StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,t));
                  ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,t));
               end;
            end;
            %
            StateForecastDecomp(:,t,n+1) = StateForecastInitCondWeights{1,t}*InitialStateVector;
            StateUpdateDecomp(:,t,n+1) = StateUpdateInitCondWeights{1,t}*InitialStateVector;
            ShockUpdateDecomp(:,t,n+1) = ShockUpdateInitCondWeights{1,t}*InitialStateVector;
         end;
      end;
      drawnow;
      pause(0.02);
      %
      % initialize KalmanGain and Lt
      %
      HT = H';
      if n1>0;
         KalmanGain{1,d} = F*StateUpdateWeights{d,d};
         Lt{1,d} = F*StateUpdateInitCondWeights{1,d};
      else;
         KalmanGain{1,d} = zeros(r,[]);
         Lt{1,d} = F;
      end;
      %
      % we now continue from d+1 until T
      %
      for t=d+1:T;
         pos = (~isnan(Z(:,t)));
         n1 = sum(pos);
         if n1>0;
            Mt{1,t} = H(:,pos)/MSEY(pos,pos,t);
            KalmanGain{1,t} = F*Ptt1(:,:,t)*Mt{1,t};
            Gt{1,t} = Mt{1,t}*HT(pos,:);
            Lt{1,t} = F-(KalmanGain{1,t}*HT(pos,:));
         else;
            Mt{1,t} = zeros(r,[]);
            KalmanGain{1,t} = zeros(r,[]);
            Gt{1,t} = zeros(r,r);
            Lt{1,t} = F;
         end;
         for tau=1:t-1;
            if tau<t-1;
               StateForecastWeights{t,tau} = Lt{1,t-1}*StateForecastWeights{t-1,tau};
            else;
               StateForecastWeights{t,t-1} = KalmanGain{1,t-1};
            end;
            StateUpdateWeights{t,tau} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastWeights{t,tau};
            ShockUpdateWeights{t,tau} = -B0'*Gt{1,t}*StateForecastWeights{t,tau};
         end;
         StateUpdateWeights{t,t} = Ptt1(:,:,t)*Mt{1,t};
         StateForecastInitCondWeights{1,t} = Lt{1,t-1}*StateForecastInitCondWeights{1,t-1};
         StateUpdateInitCondWeights{1,t} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastInitCondWeights{1,t};
         ShockUpdateWeights{t,t} = B0'*Mt{1,t};
         ShockUpdateInitCondWeights{1,t} = -B0'*Gt{1,t}*StateForecastInitCondWeights{1,t};
         %
         % decompositions
         %
         for j=1:n;
            for tau=1:t-1;
               AlphaMatF = NaN*ones(r,n);
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               postau = (~isnan(Z(:,tau)));
               AlphaMatF(:,postau) = StateForecastWeights{t,tau};
               AlphaMatU(:,postau) = StateUpdateWeights{t,tau};
               AlphaMatEtaU(:,postau) = ShockUpdateWeights{t,tau};
               if isnan(Z(j,tau))==0;
                  StateForecastDecomp(:,t,j) = StateForecastDecomp(:,t,j)+(AlphaMatF(:,j)*Z(j,tau));
                  StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,tau));
                  ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,tau));
               end;
            end;
            AlphaMatU = NaN*ones(r,n);
            AlphaMatEtaU = NaN*ones(q,n);
            AlphaMatU(:,pos) = StateUpdateWeights{t,t};
            AlphaMatEtaU(:,pos) = ShockUpdateWeights{t,t};
            if isnan(Z(j,t))==0;
               StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,t));
               ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,t));
            end;
         end;
         StateForecastDecomp(:,t,n+1) = StateForecastInitCondWeights{1,t}*InitialStateVector;
         StateUpdateDecomp(:,t,n+1) = StateUpdateInitCondWeights{1,t}*InitialStateVector;
         ShockUpdateDecomp(:,t,n+1) = ShockUpdateInitCondWeights{1,t}*InitialStateVector;
%         if t==4;
%            disp([Ksitt1(:,4) StateForecastWeights{4,3}*Z(:,3)+StateForecastWeights{4,2}*Z(:,2)+StateForecastWeights{4,1}*Z(:,1)]);
%            disp([Ksitt(:,4) StateUpdateWeights{4,4}*Z(:,4)+StateUpdateWeights{4,3}*Z(:,3)+StateUpdateWeights{4,2}*Z(:,2)+StateUpdateWeights{4,1}*Z(:,1)])
%            disp([etatt(:,4) ShockUpdateWeights{4,4}*Z(:,4)+ShockUpdateWeights{4,3}*Z(:,3)+ShockUpdateWeights{4,2}*Z(:,2)+ShockUpdateWeights{4,1}*Z(:,2)])
%         elseif t==5;
%            disp([Ksitt1(:,5) StateForecastWeights{5,4}*Z(:,4)+StateForecastWeights{5,3}*Z(:,3)+StateForecastWeights{5,2}*Z(:,2)+StateForecastWeights{5,1}*Z(:,1)]);
%            disp([Ksitt(:,5) StateUpdateWeights{5,5}*Z(:,5)+StateUpdateWeights{5,4}*Z(:,4)+StateUpdateWeights{5,3}*Z(:,3)+StateUpdateWeights{5,2}*Z(:,2)+StateUpdateWeights{5,1}*Z(:,1)])
%            disp([etatt(:,5) ShockUpdateWeights{5,5}*Z(:,5)+ShockUpdateWeights{5,4}*Z(:,4)+ShockUpdateWeights{5,3}*Z(:,3)+ShockUpdateWeights{5,2}*Z(:,2)+ShockUpdateWeights{5,1}*Z(:,2)])
%         end;
      end;
   else;
      %
      % make sure we're using the correct H-matrices
      %
      H = H(:,:,FirstPeriod:LastPeriod);
      for t=1:d;
         pos = (~isnan(Z(:,t)));
         n1 = sum(pos);
         rSI = rank(SigmaInf(:,:,t),1.0e-10);
         if t==1;
            if n1>0;
               if rSI==n1;
                  %
                  % SigmaInf has full rank n1
                  %
                  StateUpdateWeights{1,1} = PInf(:,:,1)*H(:,pos,t)*pinv(SigmaInf(pos,pos,1));
                  StateUpdateInitCondWeights{1,1} = eye(r)-(StateUpdateWeights{1,1}*H(:,pos,t)');
                  Lhatt{1,1} = LInfMat(:,:,1);
                  Khatt{1,1} = KInfMat(:,:,1);
                  Fhatt{1,1} = [zeros(r,n1);H(:,pos,t)*pinv(SigmaInf(pos,pos,1))];
                  Ghatt{1,1} = [zeros(r,r);H(:,pos,t)*pinv(SigmaInf(pos,pos,1))*H(:,pos,t)'];
                  Hhatt{1,1} = [LInfMat(:,:,1) -KStarMat(:,pos,1)*H(:,pos,t)';zeros(r,r) LInfMat(:,:,1)]';
                  Fbar = H(:,pos,t)*pinv(SigmaInf(pos,pos,1));
                  Gbar = H(:,pos,t)*pinv(SigmaInf(pos,pos,1))*H(:,pos,t)';
               elseif rSI==0;
                  %
                  % SigmaInf is zero
                  %
                  StateUpdateWeights{1,1} = PStar(:,:,1)*H(:,pos,t)*pinv(SigmaStar(pos,pos,1));
                  StateUpdateInitCondWeights{1,1} = eye(r)-(StateUpdateWeights{1,1}*H(:,pos,t)');
                  Lhatt{1,1} = LStarMat(:,:,1);
                  Khatt{1,1} = KStarMat(:,:,1);
                  Fhatt{1,1} = [H(:,pos,t)*pinv(SigmaStar(pos,pos,1));zeros(r,n1)];
                  Ghatt{1,1} = [H(:,pos,t)*pinv(SigmaStar(pos,pos,1))*H(:,pos,t)';zeros(r,r)];
                  Hhatt{1,1} = [LStarMat(:,:,1) zeros(r,r);zeros(r,r) F]';
                  Fbar = H(:,pos,t)*pinv(SigmaStar(pos,pos,1));
                  Gbar = H(:,pos,t)*pinv(SigmaStar(pos,pos,1))*H(:,pos,t)';
               else;
                  %
                  % we have univariate filtering
                  %
                  % need to compute Lbar and Kbar
                  %
                  Hs = HtiCell{t};
                  sigma2iInft = sigma2iInfCell{t};
                  sigma2iStart = sigma2iStarCell{t};
                  KtiInft = KtiInfCell{t};
                  KtiStart = KtiStarCell{t};
                  %
                  % we loop here down from n1 to 1 to create
                  % the Lbar and Kbar matrices in equation (5.146)
                  % of the YADA manual
                  %
                  Bmat = cell(1,n1);
                  Lbar = eye(r);
                  Kbar = zeros(r,n1);
                  At = zeros(2*r,n1);
                  Lt = cell(1,n1);
                  Kt = zeros(r,n1);
                  for i=n1:-1:1;
                     Hi = Hs(:,i);
                     Bti = zeros(2*r,2*r);
                     if sigma2iInft(i)>1.0e-10;
                        Lti = eye(r)-(KtiInft(:,i)*Hi'*(1/sigma2iInft(i)));
                        Kti = KtiInft(:,i)*(1/sigma2iInft(i));
                        At(r+1:2*r,i) = Hi*(1/sigma2iInft(i));
                        Bti(1:r,1:r) = Lti;
                        Bti(r+1:2*r,r+1:2*r) = Lti;
                        Bti(1:r,r+1:2*r) = ((KtiInft(:,i)*(sigma2iStart(i)/sigma2iInft(i)))-KtiStart(:,i))*Hi'*(1/sigma2iInft(i));
                     else;
                        Lti = eye(r)-(KtiStart(:,i)*Hi'*(1/sigma2iStart(i)));
                        Kti = KtiStart(:,i)*(1/sigma2iStart(i));
                        At(1:r,i) = Hi*(1/sigma2iStart(i));
                        Bti(1:r,1:r) = Lti;
                        Bti(r+1:2*r,r+1:2*r) = Lti;
                     end;
                     Bti = Bti';
                     Bmat{1,i} = Bti;
                     if i==n1;
                        Kbar(:,i) = Kti;
                     else;
                        Kbar(:,i) = Lbar*Kti;
                     end;
                     Lbar = Lbar*Lti;
                     Lt{1,i} = Lti;
                     Kt(:,i) = Kti;
                  end;
                  %
                  % now we need to loop from 1 to n1 to construct the Fbar and Gbar matrices
                  %
                  Bbar = eye(2*r);
                  Lbar = eye(r);
                  Cmat = eye(n1);
                  Fbar = zeros(2*r,n1);
                  Gbar = zeros(2*r,r);
                  for i=1:n1;
                     Ci = Cmat(i,:);
                     if i>1;
                        Hi = Hs(:,i);
                        Lbari = eye(r);
                        for j=1:i-1;
                           Ci(1,i-j) = -Hi'*Lbari*Kt(:,i-j);
                           Lbari = Lt{1,i-j}*Lbari;
                        end;
                     end;
                     Fbar = Fbar+(Bbar*At(:,i)*Ci*S(pos,pos)');
                     Gbar = Gbar+(Bbar*At(:,i)*Hi'*Lbar);
                     %
                     Lbar = Lt{1,i}*Lbar;
                     Bbar = Bbar*Bmat{1,i};
                  end;
                  Fhatt{1,1} = Fbar;
                  Ghatt{1,1} = Gbar;
                  Hhatt{1,1} = Bbar*[F' zeros(r,r);zeros(r,r) F'];
                  Fbar = Jr'*Fbar;
                  Gbar = Jr'*Gbar;
                  %
                  % deal with possible measurement errors with S
                  %
                  Kbar = Kbar*S(pos,pos)';
                  StateUpdateWeights{1,1} = Kbar;
                  StateUpdateInitCondWeights{1,1} = Lbar;
                  Lhatt{1,1} = F*Lbar;
                  Khatt{1,1} = F*Kbar;
               end;
               StateUpdateDecomp(:,1,n+1) = StateUpdateInitCondWeights{1,1}*InitialStateVector;
               %
               ShockUpdateWeights{1,1} = B0'*Fbar;
               ShockUpdateInitCondWeights{1,1} = -B0'*Gbar;
               ShockUpdateDecomp(:,1,n+1) = ShockUpdateInitCondWeights{1,1}*InitialStateVector;
               %
               %
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               AlphaMatU(:,pos) = StateUpdateWeights{1,1};
               AlphaMatEtaU(:,pos) = ShockUpdateWeights{1,1};
               for j=1:n;
                  if isnan(Z(j,1))==0;
                     StateUpdateDecomp(:,1,j) = AlphaMatU(:,j)*Z(j,1);
                     ShockUpdateDecomp(:,1,j) = AlphaMatEtaU(:,j)*Z(j,1);
                  end;
               end;
            else;
               %
               % all data are missing for period t
               %
               StateUpdateWeights{1,1} = zeros(r,[]);
               StateUpdateInitCondWeights{1,1} = zeros(r,r);
               StateUpdateDecomp(:,1,n+1) = zeros(r,1);
               %
               ShockUpdateWeights{1,1} = zeros(q,[]);
               %
               Lhatt{1,1} = zeros(r,r);
               Khatt{1,1} = zeros(r,[]);
               Fhatt{1,1} = zeros(2*r,[]);
               Ghatt{1,1} = zeros(2*r,2*r);
               Hhatt{1,1} = zeros(2*r,2*r);
            end;
         else;
            %
            % t>=2 up to t=d
            %
            if n1>0;
               if rSI==n1;
                  %
                  % SigmaInf has full rank
                  %
                  Lhatt{1,t} = LInfMat(:,:,t);
                  Khatt{1,t} = KInfMat(:,:,t);
                  StateUpdateWeights{t,t} = PInf(:,:,t)*H(:,pos,t)*pinv(SigmaInf(pos,pos,t));
                  StateUpdateInitCondWeights{1,t} = eye(r)-(StateUpdateWeights{1,t}*H(:,pos,t)');
                  Fhatt{1,t} = [zeros(r,n1);H(:,pos,t)*pinv(SigmaInf(pos,pos,t))];
                  Ghatt{1,t} = [zeros(r,r);H(:,pos.t)*pinv(SigmaInf(pos,pos,t))*H(:,pos,t)'];
                  Hhatt{1,t} = [LInfMat(:,:,t) -KStarMat(:,pos,t)*H(:,pos,t)';zeros(r,r) LInfMat(:,:,t)]';
                  Fbar = H(:,pos,t)*pinv(SigmaInf(pos,pos,t));
                  Gbar = H(:,pos,t)*pinv(SigmaInf(pos,pos,t))*H(:,pos,t)';
               elseif rSI==0;
                  %
                  % SigmaInf is zero
                  %
                  Lhatt{1,t} = LStarMat(:,:,t);
                  Khatt{1,t} = KStarMat(:,:,t);
                  StateUpdateWeights{t,t} = PStar(:,:,t)*H(:,pos,t)*pinv(SigmaStar(pos,pos,t));
                  StateUpdateInitCondWeights{1,t} = eye(r)-(StateUpdateWeights{1,t}*H(:,pos,t)');
                  Fhatt{1,t} = [H(:,pos,t)*pinv(SigmaStar(pos,pos,t));zeros(r,n1)];
                  Ghatt{1,t} = [H(:,pos,t)*pinv(SigmaStar(pos,pos,t))*H(:,pos,t)';zeros(r,r)];
                  Hhatt{1,t} = [LStarMat(:,:,t) zeros(r,r);zeros(r,r) F]';
                  Fbar = H(:,pos,t)*pinv(SigmaStar(pos,pos,t));
                  Gbar = H(:,pos,t)*pinv(SigmaStar(pos,pos,t))*H(:,pos,t)';
               else;
                  %
                  % we have univariate filtering
                  %
                  Hs = HtiCell{t};
                  sigma2iInft = sigma2iInfCell{t};
                  sigma2iStart = sigma2iStarCell{t};
                  KtiInft = KtiInfCell{t};
                  KtiStart = KtiStarCell{t};
                  %
                  % we loop here down from n1 to 1 to create
                  % the Lbar and Kbar matrices in equation (5.146)
                  % of the YADA manual
                  %
                  Bmat = cell(1,n1);
                  Lbar = eye(r);
                  Kbar = zeros(r,n1);
                  At = zeros(2*r,n1);
                  Lt = cell(1,n1);
                  Kt = zeros(r,n1);
                  for i=n1:-1:1;
                     Hi = Hs(:,i);
                     Bti = zeros(2*r,2*r);
                     if sigma2iInft(i)>1.0e-10;
                        Lti = eye(r)-(KtiInft(:,i)*Hi'*(1/sigma2iInft(i)));
                        Kti = KtiInft(:,i)*(1/sigma2iInft(i));
                        At(r+1:2*r,i) = Hi*(1/sigma2iInft(i));
                        Bti(1:r,1:r) = Lti;
                        Bti(r+1:2*r,r+1:2*r) = Lti;
                        Bti(1:r,r+1:2*r) = ((KtiInft(:,i)*(sigma2iStart(i)/sigma2iInft(i)))-KtiStart(:,i))*Hi'*(1/sigma2iInft(i));
                     else;
                        Lti = eye(r)-(KtiStart(:,i)*Hi'*(1/sigma2iStart(i)));
                        Kti = KtiStart(:,i)*(1/sigma2iStart(i));
                        At(1:r,i) = Hi*(1/sigma2iStart(i));
                        Bti(1:r,1:r) = Lti;
                        Bti(r+1:2*r,r+1:2*r) = Lti;
                     end;
                     Bti = Bti';
                     Bmat{1,i} = Bti;
                     if i==n1;
                        Kbar(:,i) = Kti;
                     else;
                        Kbar(:,i) = Lbar*Kti;
                     end;
                     Lbar = Lbar*Lti;
                     Lt{1,i} = Lti;
                     Kt(:,i) = Kti;
                  end;
                  %
                  % now we need to loop from 1 to n1 to construct the Fbar and Gbar matrices
                  %
                  Bbar = eye(2*r);
                  Lbar = eye(r);
                  Cmat = eye(n1);
                  Fbar = zeros(2*r,n1);
                  Gbar = zeros(2*r,r);
                  for i=1:n1;
                     Ci = Cmat(i,:);
                     if i>1;
                        Hi = Hs(:,i);
                        Lbari = eye(r);
                        for j=1:i-1;
                           Ci(1,i-j) = -Hi'*Lbari*Kt(:,i-j);
                           Lbari = Lt{1,i-j}*Lbari;
                        end;
                     end;
                     Fbar = Fbar+(Bbar*At(:,i)*Ci*S(pos,pos)');
                     Gbar = Gbar+(Bbar*At(:,i)*Hi'*Lbar);
                     %
                     Lbar = Lt{1,i}*Lbar;
                     Bbar = Bbar*Bmat{1,i};
                  end;
                  Fhatt{1,t} = Fbar;
                  Ghatt{1,t} = Gbar;
                  Hhatt{1,t} = Bbar*[F' zeros(r,r);zeros(r,r) F'];
                  Fbar = Jr'*Fbar;
                  Gbar = Jr'*Gbar;
                  %
                  % deal with possible measurement errors with S
                  %
                  Kbar = Kbar*S(pos,pos)';
                  Lhatt{1,t} = F*Lbar;
                  Khatt{1,t} = F*Kbar;
                  StateUpdateWeights{t,t} = Kbar;
                  StateUpdateInitCondWeights{1,t} = Lbar;
               end;
            else;
               %
               % all data are missing for period t
               %
               Lhatt{1,t} = zeros(r,r);
               Khatt{1,t} = zeros(r,[]);
               StateUpdateWeights{t,t} = zeros(r,[]);
               StateUpdateInitCondWeights{1,t} = zeros(r,r);
               StateUpdateDecomp(:,t,n+1) = zeros(r,1);
               StateForecastDecomp(:,t,n+1) = zeros(r,1)
               %
               Fhatt{1,t} = zeros(2*r,[]);
               Ghatt{1,t} = zeros(2*r,2*r);
               Hhatt{1,t} = zeros(2*r,2*r);
               Gbar = zeros(r,2*r);
               Fbar = zeros(r,[]);
            end;
            %
            % we can now begin with the forecast part
            %
            for tau=1:t-1;
               if tau<t-1;
                  StateForecastWeights{t,tau} = Lhatt{1,t-1}*StateForecastWeights{t-1,tau};
               else;
                  StateForecastWeights{t,t-1} = Khatt{1,t-1};
               end;
               StateUpdateWeights{t,tau} = StateUpdateInitCondWeights{1,t}*StateForecastWeights{t,tau};
               ShockUpdateWeights{t,tau} = -B0'*Gbar*StateForecastWeights{t,tau};
            end;
            StateForecastInitCondWeights{1,t} = Lhatt{1,t-1}*StateForecastInitCondWeights{1,t-1};
            ShockUpdateWeights{t,t} = B0'*Fbar;
            ShockUpdateInitCondWeights{1,t} = -B0'*Gbar;
            %
            % decompositions
            %
            for j=1:n;
               for tau=1:t-1;
                  AlphaMatF = NaN*ones(r,n);
                  AlphaMatU = NaN*ones(r,n);
                  AlphaMatEtaU = NaN*ones(q,n);
                  postau = (~isnan(Z(:,tau)));
                  AlphaMatF(:,postau) = StateForecastWeights{t,tau};
                  AlphaMatU(:,postau) = StateUpdateWeights{t,tau};
                  AlphaMatEtaU(:,postau) = ShockUpdateWeights{t,tau};
                  if isnan(Z(j,tau))==0;
                     StateForecastDecomp(:,t,j) = StateForecastDecomp(:,t,j)+(AlphaMatF(:,j)*Z(j,tau));
                     StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,tau));
                     ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,tau));
                  end;
               end;
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               AlphaMatU(:,pos) = StateUpdateWeights{t,t};
               AlphaMatEtaU(:,pos) = ShockUpdateWeights{t,t};
               if isnan(Z(j,t))==0;
                  StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,t));
                  ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,t));
               end;
            end;
            %
            StateForecastDecomp(:,t,n+1) = StateForecastInitCondWeights{1,t}*InitialStateVector;
            StateUpdateDecomp(:,t,n+1) = StateUpdateInitCondWeights{1,t}*InitialStateVector;
            ShockUpdateDecomp(:,t,n+1) = ShockUpdateInitCondWeights{1,t}*InitialStateVector;
         end;
      end;
      drawnow;
      pause(0.02);
      %
      % we now continue from d+1 until T
      %
      %
      % initialize KalmanGain and Lt
      %
      if n1>0;
         KalmanGain{1,d} = F*StateUpdateWeights{d,d};
         Lt{1,d} = F*StateUpdateInitCondWeights{1,d};
      else;
         KalmanGain{1,d} = zeros(r,[]);
         Lt{1,d} = F;
      end;
      %
      % we now continue from d+1 until T
      %
      for t=d+1:T;
         pos = (~isnan(Z(:,t)));
         n1 = sum(pos);
         if n1>0;
            HT = H(:,pos,t)';
            Mt{1,t} = H(:,pos,t)/MSEY(pos,pos,t);
            KalmanGain{1,t} = F*Ptt1(:,:,t)*Mt{1,t};
            Gt{1,t} = Mt{1,t}*HT;
            Lt{1,t} = F-(KalmanGain{1,t}*HT);
         else;
            Mt{1,t} = zeros(r,[]);
            KalmanGain{1,t} = zeros(r,[]);
            Gt{1,t} = zeros(r,r);
            Lt{1,t} = F;
         end;
         for tau=1:t-1;
            if tau<t-1;
               StateForecastWeights{t,tau} = Lt{1,t-1}*StateForecastWeights{t-1,tau};
            else;
               StateForecastWeights{t,t-1} = KalmanGain{1,t-1};
            end;
            StateUpdateWeights{t,tau} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastWeights{t,tau};
            ShockUpdateWeights{t,tau} = -B0'*Gt{1,t}*StateForecastWeights{t,tau};
         end;
         StateUpdateWeights{t,t} = Ptt1(:,:,t)*Mt{1,t};
         StateForecastInitCondWeights{1,t} = Lt{1,t-1}*StateForecastInitCondWeights{1,t-1};
         StateUpdateInitCondWeights{1,t} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastInitCondWeights{1,t};
         ShockUpdateWeights{t,t} = B0'*Mt{1,t};
         ShockUpdateInitCondWeights{1,t} = -B0'*Gt{1,t}*StateForecastInitCondWeights{1,t};
         %
         % decompositions
         %
         for j=1:n;
            for tau=1:t-1;
               AlphaMatF = NaN*ones(r,n);
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               postau = (~isnan(Z(:,tau)));
               AlphaMatF(:,postau) = StateForecastWeights{t,tau};
               AlphaMatU(:,postau) = StateUpdateWeights{t,tau};
               AlphaMatEtaU(:,postau) = ShockUpdateWeights{t,tau};
               if isnan(Z(j,tau))==0;
                  StateForecastDecomp(:,t,j) = StateForecastDecomp(:,t,j)+(AlphaMatF(:,j)*Z(j,tau));
                  StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,tau));
                  ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,tau));
               end;
            end;
            AlphaMatU = NaN*ones(r,n);
            AlphaMatEtaU = NaN*ones(q,n);
            AlphaMatU(:,pos) = StateUpdateWeights{t,t};
            AlphaMatEtaU(:,pos) = ShockUpdateWeights{t,t};
            if isnan(Z(j,t))==0;
               StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,t));
               ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,t));
            end;
         end;
         StateForecastDecomp(:,t,n+1) = StateForecastInitCondWeights{1,t}*InitialStateVector;
         StateUpdateDecomp(:,t,n+1) = StateUpdateInitCondWeights{1,t}*InitialStateVector;
         ShockUpdateDecomp(:,t,n+1) = ShockUpdateInitCondWeights{1,t}*InitialStateVector;
      end;
   end;
   clear('SigmaInf','SigmaStar','KInfMat','LInfMat','KStarMat','LStarMat','sigma2iStarCell','KtiStarCell','HtiCell','sigma2iInfCell','KtiInfCell');
elseif DSGEModel.KalmanAlgorithm==3;
   %
   % we have univariate filtering and smoothing
   %
   PInf = KalmanG.PInf;
   PStar = KalmanG.PStar;
   sigma2iStarCell = KalmanG.sigma2iStarCell;
   KtiStarCell = KalmanG.KtiStarCell;
   HtiCell = KalmanG.HtiCell;
   sigma2iInfCell = KalmanG.sigma2iInfCell;
   KtiInfCell = KalmanG.KtiInfCell;
   clear('KalmanG');
   %
   % dealing with the initialization subsample
   %
   for t=1:d;
      pos = (~isnan(Z(:,t)));
      n1 = sum(pos);
      if t==1;
         if n1>0;
            %
            % need to compute Lbar and Kbar
            %
            Hs = HtiCell{t};
            sigma2iInft = sigma2iInfCell{t};
            sigma2iStart = sigma2iStarCell{t};
            KtiInft = KtiInfCell{t};
            KtiStart = KtiStarCell{t};
            %
            % we loop here down from n1 to 1 to create
            % the Lbar and Kbar matrices in equation (5.146)
            % of the YADA manual
            %
            Bmat = cell(1,n1);
            Lbar = eye(r);
            Kbar = zeros(r,n1);
            At = zeros(2*r,n1);
            Lt = cell(1,n1);
            Kt = zeros(r,n1);
            for i=n1:-1:1;
               Hi = Hs(:,i);
               Bti = zeros(2*r,2*r);
               if sigma2iInft(i)>1.0e-10;
                  Lti = eye(r)-(KtiInft(:,i)*Hi'*(1/sigma2iInft(i)));
                  Kti = KtiInft(:,i)*(1/sigma2iInft(i));
                  At(r+1:2*r,i) = Hi*(1/sigma2iInft(i));
                  Bti(1:r,1:r) = Lti;
                  Bti(r+1:2*r,r+1:2*r) = Lti;
                  Bti(1:r,r+1:2*r) = ((KtiInft(:,i)*(sigma2iStart(i)/sigma2iInft(i)))-KtiStart(:,i))*Hi'*(1/sigma2iInft(i));
               else;
                  Lti = eye(r)-(KtiStart(:,i)*Hi'*(1/sigma2iStart(i)));
                  Kti = KtiStart(:,i)*(1/sigma2iStart(i));
                  At(1:r,i) = Hi*(1/sigma2iStart(i));
                  Bti(1:r,1:r) = Lti;
                  Bti(r+1:2*r,r+1:2*r) = Lti;
               end;
               Bti = Bti';
               Bmat{1,i} = Bti;
               if i==n1;
                  Kbar(:,i) = Kti;
               else;
                  Kbar(:,i) = Lbar*Kti;
               end;
               Lbar = Lbar*Lti;
               Lt{1,i} = Lti;
               Kt(:,i) = Kti;
            end;
            %
            % now we need to loop from 1 to n1 to construct the Fbar and Gbar matrices
            %
            Bbar = eye(2*r);
            Lbar = eye(r);
            Cmat = eye(n1);
            Fbar = zeros(2*r,n1);
            Gbar = zeros(2*r,r);
            for i=1:n1;
               Ci = Cmat(i,:);
               if i>1;
                  Hi = Hs(:,i);
                  Lbari = eye(r);
                  for j=1:i-1;
                     Ci(1,i-j) = -Hi'*Lbari*Kt(:,i-j);
                     Lbari = Lt{1,i-j}*Lbari;
                  end;
               end;
               Fbar = Fbar+(Bbar*At(:,i)*Ci*S(pos,pos)');
               Gbar = Gbar+(Bbar*At(:,i)*Hi'*Lbar);
               %
               Lbar = Lt{1,i}*Lbar;
               Bbar = Bbar*Bmat{1,i};
            end;
            Fhatt{1,1} = Fbar;
            Ghatt{1,1} = Gbar;
            %Bbart{1,1} = Bbar;
            Hhatt{1,1} = Bbar*[F' zeros(r,r);zeros(r,r) F'];
            Fbar = Jr'*Fbar;
            Gbar = Jr'*Gbar;
            %
            % deal with possible measurement errors with S
            %
            Kbar = Kbar*S(pos,pos)';
            StateUpdateWeights{1,1} = Kbar;
            StateUpdateInitCondWeights{1,1} = Lbar;
            Lhatt{1,1} = F*Lbar;
            Khatt{1,1} = F*Kbar;
         else;
            %
            % all variables are missing
            %
            StateUpdateWeights{1,1} = zeros(r,[]);
            StateUpdateInitCondWeights{1,1} = zeros(r,r);
            StateUpdateDecomp(:,1,n+1) = zeros(r,1);
            %
            ShockUpdateWeights{1,1} = zeros(q,[]);
            %
            Lhatt{1,1} = zeros(r,r);
            Khatt{1,1} = zeros(r,[]);
            Fhatt{1,1} = zeros(2*r,[]);
            Ghatt{1,1} = zeros(2*r,2*r);
            Hhatt{1,1} = zeros(2*r,2*r);
         end;
      else;
         %
         % t>=2 until t=d
         %
         if n1>0;
            Hs = HtiCell{t};
            sigma2iInft = sigma2iInfCell{t};
            sigma2iStart = sigma2iStarCell{t};
            KtiInft = KtiInfCell{t};
            KtiStart = KtiStarCell{t};
            %
            % we loop here down from n1 to 1 to create
            % the Lbar and Kbar matrices in equation (5.146)
            % of the YADA manual
            %
            Bmat = cell(1,n1);
            Lbar = eye(r);
            Kbar = zeros(r,n1);
            At = zeros(2*r,n1);
            Lt = cell(1,n1);
            Kt = zeros(r,n1);
            for i=n1:-1:1;
               Hi = Hs(:,i);
               Bti = zeros(2*r,2*r);
               if sigma2iInft(i)>1.0e-10;
                  Lti = eye(r)-(KtiInft(:,i)*Hi'*(1/sigma2iInft(i)));
                  Kti = KtiInft(:,i)*(1/sigma2iInft(i));
                  At(r+1:2*r,i) = Hi*(1/sigma2iInft(i));
                  Bti(1:r,1:r) = Lti;
                  Bti(r+1:2*r,r+1:2*r) = Lti;
                  Bti(1:r,r+1:2*r) = ((KtiInft(:,i)*(sigma2iStart(i)/sigma2iInft(i)))-KtiStart(:,i))*Hi'*(1/sigma2iInft(i));
               else;
                  Lti = eye(r)-(KtiStart(:,i)*Hi'*(1/sigma2iStart(i)));
                  Kti = KtiStart(:,i)*(1/sigma2iStart(i));
                  At(1:r,i) = Hi*(1/sigma2iStart(i));
                  Bti(1:r,1:r) = Lti;
                  Bti(r+1:2*r,r+1:2*r) = Lti;
               end;
               Bti = Bti';
               Bmat{1,i} = Bti;
               if i==n1;
                  Kbar(:,i) = Kti;
               else;
                  Kbar(:,i) = Lbar*Kti;
               end;
               Lbar = Lbar*Lti;
               Lt{1,i} = Lti;
               Kt(:,i) = Kti;
            end;
            %
            % now we need to loop from 1 to n1 to construct the Fbar and Gbar matrices
            %
            Bbar = eye(2*r);
            Lbar = eye(r);
            Cmat = eye(n1);
            Fbar = zeros(2*r,n1);
            Gbar = zeros(2*r,r);
            for i=1:n1;
               Ci = Cmat(i,:);
               if i>1;
                  Hi = Hs(:,i);
                  Lbari = eye(r);
                  for j=1:i-1;
                     Ci(1,i-j) = -Hi'*Lbari*Kt(:,i-j);
                     Lbari = Lt{1,i-j}*Lbari;
                  end;
               end;
               Fbar = Fbar+(Bbar*At(:,i)*Ci*S(pos,pos)');
               Gbar = Gbar+(Bbar*At(:,i)*Hi'*Lbar);
               %
               Lbar = Lt{1,i}*Lbar;
               Bbar = Bbar*Bmat{1,i};
            end;
            Fhatt{1,t} = Fbar;
            Ghatt{1,t} = Gbar;
            %Bbart{1,t} = Bbar;
            Hhatt{1,t} = Bbar*[F' zeros(r,r);zeros(r,r) F'];
            Fbar = Jr'*Fbar;
            Gbar = Jr'*Gbar;
            %
            % deal with possible measurement errors with S
            %
            Kbar = Kbar*S(pos,pos)';
            Lhatt{1,t} = F*Lbar;
            Khatt{1,t} = F*Kbar;
            StateUpdateWeights{t,t} = Kbar;
            StateUpdateInitCondWeights{1,t} = Lbar;
         else;
            %
            % all variables are missing
            %
            Lhatt{1,t} = zeros(r,r);
            Khatt{1,t} = zeros(r,[]);
            StateUpdateWeights{t,t} = zeros(r,[]);
            StateUpdateInitCondWeights{1,t} = zeros(r,r);
            StateUpdateDecomp(:,t,n+1) = zeros(r,1);
            StateForecastDecomp(:,t,n+1) = zeros(r,1)
            %
            Fhatt{1,t} = zeros(2*r,[]);
            Ghatt{1,t} = zeros(2*r,2*r);
            Hhatt{1,t} = zeros(2*r,2*r);
            Gbar = zeros(r,2*r);
            Fbar = zeros(r,[]);
         end;
         %
         % we can now begin with the forecast part
         %
         for tau=1:t-1;
            if tau<t-1;
               StateForecastWeights{t,tau} = Lhatt{1,t-1}*StateForecastWeights{t-1,tau};
            else;
               StateForecastWeights{t,t-1} = Khatt{1,t-1};
            end;
            % StateUpdateInitCondWeights{1,t} is equal to Mhatt
            StateUpdateWeights{t,tau} = StateUpdateInitCondWeights{1,t}*StateForecastWeights{t,tau};
            ShockUpdateWeights{t,tau} = -B0'*Gbar*StateForecastWeights{t,tau};
         end;
         StateForecastInitCondWeights{1,t} = Lhatt{1,t-1}*StateForecastInitCondWeights{1,t-1};
         ShockUpdateWeights{t,t} = B0'*Fbar;
         ShockUpdateInitCondWeights{1,t} = -B0'*Gbar;
%         if t==2;
%            disp([Ksitt1(:,2) StateForecastWeights{2,1}*Z(:,1)]);
%            disp([Ksitt(:,2) StateUpdateWeights{2,2}*Z(:,2)+StateUpdateWeights{2,1}*Z(:,1)])
%            disp([etatt(:,2) ShockUpdateWeights{2,2}*Z(:,2)+ShockUpdateWeights{2,1}*Z(:,1)])
%         elseif t==3;
%            disp([Ksitt1(:,3) StateForecastWeights{3,2}*Z(:,2)+StateForecastWeights{3,1}*Z(:,1)]);
%            disp([Ksitt(:,3) StateUpdateWeights{3,3}*Z(:,3)+StateUpdateWeights{3,2}*Z(:,2)+StateUpdateWeights{3,1}*Z(:,1)])
%            disp([etatt(:,3) ShockUpdateWeights{3,3}*Z(:,3)+ShockUpdateWeights{3,2}*Z(:,2)+ShockUpdateWeights{3,1}*Z(:,2)])
%         end;
         %
         % decompositions
         %
         for j=1:n;
            for tau=1:t-1;
               AlphaMatF = NaN*ones(r,n);
               AlphaMatU = NaN*ones(r,n);
               AlphaMatEtaU = NaN*ones(q,n);
               postau = (~isnan(Z(:,tau)));
               AlphaMatF(:,postau) = StateForecastWeights{t,tau};
               AlphaMatU(:,postau) = StateUpdateWeights{t,tau};
               AlphaMatEtaU(:,postau) = ShockUpdateWeights{t,tau};
               if isnan(Z(j,tau))==0;
                  StateForecastDecomp(:,t,j) = StateForecastDecomp(:,t,j)+(AlphaMatF(:,j)*Z(j,tau));
                  StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,tau));
                  ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,tau));
               end;
            end;
            AlphaMatU = NaN*ones(r,n);
            AlphaMatEtaU = NaN*ones(q,n);
            AlphaMatU(:,pos) = StateUpdateWeights{t,t};
            AlphaMatEtaU(:,pos) = ShockUpdateWeights{t,t};
            if isnan(Z(j,t))==0;
               StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,t));
               ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,t));
            end;
         end;
         %
         StateForecastDecomp(:,t,n+1) = StateForecastInitCondWeights{1,t}*InitialStateVector;
         StateUpdateDecomp(:,t,n+1) = StateUpdateInitCondWeights{1,t}*InitialStateVector;
         ShockUpdateDecomp(:,t,n+1) = ShockUpdateInitCondWeights{1,t}*InitialStateVector;
      end;
   end;
   drawnow;
   pause(0.02);
   %
   % initialize KalmanGain and Lt
   %
   if n1>0;
      KalmanGain{1,d} = F*StateUpdateWeights{d,d};
      Lt{1,d} = F*StateUpdateInitCondWeights{1,d};
   else;
      KalmanGain{1,d} = zeros(r,[]);
      Lt{1,d} = F;
   end;
   %
   % make sure we're using the correct H-matrices
   %
   if length(size(H))==2;
      HT = H';
   else;
      H = H(:,:,FirstPeriod:LastPeriod);
   end;
   %
   % we now continue from d+1 until T
   %
   for t=d+1:T;
      pos = (~isnan(Z(:,t)));
      n1 = sum(pos);
      if n1>0;
         if length(size(H))==2;
            Mt{1,t} = H(:,pos)/MSEY(pos,pos,t);
            KalmanGain{1,t} = F*Ptt1(:,:,t)*Mt{1,t};
            Gt{1,t} = Mt{1,t}*HT(pos,:);
            Lt{1,t} = F-(KalmanGain{1,t}*HT(pos,:));
         else;
            HT = H(:,pos,t)';
            Mt{1,t} = H(:,pos,t)/MSEY(pos,pos,t);
            KalmanGain{1,t} = F*Ptt1(:,:,t)*Mt{1,t};
            Gt{1,t} = Mt{1,t}*HT;
            Lt{1,t} = F-(KalmanGain{1,t}*HT);
         end;
      else;
         Mt{1,t} = zeros(r,[]);
         KalmanGain{1,t} = zeros(r,[]);
         Gt{1,t} = zeros(r,r);
         Lt{1,t} = F;
      end;
      for tau=1:t-1;
         if tau<t-1;
            StateForecastWeights{t,tau} = Lt{1,t-1}*StateForecastWeights{t-1,tau};
         else;
            StateForecastWeights{t,t-1} = KalmanGain{1,t-1};
         end;
         StateUpdateWeights{t,tau} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastWeights{t,tau};
         ShockUpdateWeights{t,tau} = -B0'*Gt{1,t}*StateForecastWeights{t,tau};
      end;
      StateUpdateWeights{t,t} = Ptt1(:,:,t)*Mt{1,t};
      StateForecastInitCondWeights{1,t} = Lt{1,t-1}*StateForecastInitCondWeights{1,t-1};
      StateUpdateInitCondWeights{1,t} = (eye(r)-(Ptt1(:,:,t)*Gt{1,t}))*StateForecastInitCondWeights{1,t};
      ShockUpdateWeights{t,t} = B0'*Mt{1,t};
      ShockUpdateInitCondWeights{1,t} = -B0'*Gt{1,t}*StateForecastInitCondWeights{1,t};
      %
      % decompositions
      %
      for j=1:n;
         for tau=1:t-1;
            AlphaMatF = NaN*ones(r,n);
            AlphaMatU = NaN*ones(r,n);
            AlphaMatEtaU = NaN*ones(q,n);
            postau = (~isnan(Z(:,tau)));
            AlphaMatF(:,postau) = StateForecastWeights{t,tau};
            AlphaMatU(:,postau) = StateUpdateWeights{t,tau};
            AlphaMatEtaU(:,postau) = ShockUpdateWeights{t,tau};
            if isnan(Z(j,tau))==0;
               StateForecastDecomp(:,t,j) = StateForecastDecomp(:,t,j)+(AlphaMatF(:,j)*Z(j,tau));
               StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,tau));
               ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,tau));
            end;
         end;
         AlphaMatU = NaN*ones(r,n);
         AlphaMatEtaU = NaN*ones(q,n);
         AlphaMatU(:,pos) = StateUpdateWeights{t,t};
         AlphaMatEtaU(:,pos) = ShockUpdateWeights{t,t};
         if isnan(Z(j,t))==0;
            StateUpdateDecomp(:,t,j) = StateUpdateDecomp(:,t,j)+(AlphaMatU(:,j)*Z(j,t));
            ShockUpdateDecomp(:,t,j) = ShockUpdateDecomp(:,t,j)+(AlphaMatEtaU(:,j)*Z(j,t));
         end;
      end;
      StateForecastDecomp(:,t,n+1) = StateForecastInitCondWeights{1,t}*InitialStateVector;
      StateUpdateDecomp(:,t,n+1) = StateUpdateInitCondWeights{1,t}*InitialStateVector;
      ShockUpdateDecomp(:,t,n+1) = ShockUpdateInitCondWeights{1,t}*InitialStateVector;
%      if t==4;
%         disp([Ksitt1(:,4) StateForecastWeights{4,3}*Z(:,3)+StateForecastWeights{4,2}*Z(:,2)+StateForecastWeights{4,1}*Z(:,1)]);
%         disp([Ksitt(:,4) StateUpdateWeights{4,4}*Z(:,4)+StateUpdateWeights{4,3}*Z(:,3)+StateUpdateWeights{4,2}*Z(:,2)+StateUpdateWeights{4,1}*Z(:,1)])
%         disp([etatt(:,4) ShockUpdateWeights{4,4}*Z(:,4)+ShockUpdateWeights{4,3}*Z(:,3)+ShockUpdateWeights{4,2}*Z(:,2)+ShockUpdateWeights{4,1}*Z(:,2)])
%      elseif t==5;
%         disp([Ksitt1(:,5) StateForecastWeights{5,4}*Z(:,4)+StateForecastWeights{5,3}*Z(:,3)+StateForecastWeights{5,2}*Z(:,2)+StateForecastWeights{5,1}*Z(:,1)]);
%         disp([Ksitt(:,5) StateUpdateWeights{5,5}*Z(:,5)+StateUpdateWeights{5,4}*Z(:,4)+StateUpdateWeights{5,3}*Z(:,3)+StateUpdateWeights{5,2}*Z(:,2)+StateUpdateWeights{5,1}*Z(:,1)])
%         disp([etatt(:,5) ShockUpdateWeights{5,5}*Z(:,5)+ShockUpdateWeights{5,4}*Z(:,4)+ShockUpdateWeights{5,3}*Z(:,3)+ShockUpdateWeights{5,2}*Z(:,2)+ShockUpdateWeights{5,1}*Z(:,2)])
%      end;
   end;
   clear('sigma2iStarCell','KtiStarCell','HtiCell','sigma2iInfCell','KtiInfCell');
end;
clear('KalmanGain','Lhatt','Khatt');
%
% generate some of the output
%
if VarType==1;
   %
   % clear variables
   %
   clear('StateUpdateWeights','StateUpdateInitCondWeights');
   clear('ShockUpdateWeights','ShockUpdateInitCondWeights');
   drawnow;
   pause(0.02);
   %
   % state variable decompositions
   %
   StateDecomp.StateUpdateDecomp = StateUpdateDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   StateDecomp.Ksitt1 = Ksitt1(:,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.Ksitt = Ksitt(:,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.KsitT = KsitT(:,DSGEModel.KalmanFirstObservation:T);
   %
   clear('StateUpdateDecomp','ShockUpdateDecomp','Ksitt1','Ksitt','KsitT','etatt','etatT');
elseif VarType==2;
   %
   % clear variables
   %
   clear('StateUpdateWeights','StateUpdateInitCondWeights');
   clear('ShockUpdateWeights','ShockUpdateInitCondWeights');
   drawnow;
   pause(0.02);
   %
   % shock decompositions
   %
   StateDecomp.ShockUpdateDecomp = ShockUpdateDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   StateDecomp.etatt = etatt(:,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.etatT = etatT(:,DSGEModel.KalmanFirstObservation:T);
   %
   clear('StateUpdateDecomp','ShockUpdateDecomp','Ksitt1','Ksitt','KsitT','etatt','etatT');
elseif VarType==3;
   %
   % clear shock weights
   %
   clear('ShockUpdateWeights','ShockUpdateInitCondWeights','StateUpdateDecomp','ShockUpdateDecomp');
   drawnow;
   pause(0.02);
   %
   % state variable weights
   %
   StateDecomp.StateUpdateWeights = StateUpdateWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.StateUpdateInitCondWeights = StateUpdateInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   %
   % clear variables
   %
   clear('StateUpdateWeights','StateUpdateInitCondWeights');
   drawnow;
   pause(0.02);
else VarType==4;
   %
   % clear state variable weights
   %
   clear('StateUpdateWeights','StateUpdateInitCondWeights','StateUpdateDecomp','ShockUpdateDecomp');
   drawnow;
   pause(0.02);
   %
   % shock weights
   %
   StateDecomp.ShockUpdateWeights = ShockUpdateWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.ShockUpdateInitCondWeights = ShockUpdateInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   %
   % clear variables
   %
   clear('ShockUpdateWeights','ShockUpdateInitCondWeights');
   drawnow;
   pause(0.02);
end;
%
% now the smoothing part
%
rtWeights = cell(1,T);
%
StateSmoothWeights = cell(T,T);
StateSmoothInitCondWeights = cell(1,T);
StateSmoothDecomp = zeros(r,T,n+1);
%
%
ShockSmoothWeights = cell(T,T);
ShockSmoothInitCondWeights = cell(1,T);
ShockSmoothDecomp = zeros(q,T,n+1);
%
% update the text on the wait dialog
%
if DSGEModel.RunDriver==0;
   if VarType==1;
      txt = 'Please wait while YADA computes the decompositions of the state variables for the smooth estimates.';
   elseif VarType==2;
      txt = 'Please wait while YADA computes the decompositions of the shocks for the smooth estimates.';
   elseif VarType==3;
      txt = 'Please wait while YADA computes the weights for the decompositions of the state variables for the smooth estimates.';
   else;
      txt = 'Please wait while YADA computes the weights for the decompositions of the shocks for the smooth estimates.';
   end;
   set(WaitControls.text,'String',txt);
   drawnow;
   pause(0.02);
end;
%
%
% now deal with smoothing over the sample t=d+1 until T
%
t = T;
while t>=d+1;
   if t==T;
      for tau=1:T-1;
         rtWeights{1,tau} = -Gt{1,T}*StateForecastWeights{T,tau};
         StateSmoothWeights{T,tau} = StateForecastWeights{T,tau}+(Ptt1(:,:,T)*rtWeights{1,tau});
         ShockSmoothWeights{T,tau} = B0'*rtWeights{1,tau};
      end;
      rtWeights{1,T} = Mt{1,T};
      StateSmoothWeights{T,T} = Ptt1(:,:,T)*rtWeights{1,T};
      rtInitCondWeights = -Gt{1,T}*StateForecastInitCondWeights{1,T};
      %
      ShockSmoothWeights{T,T} = B0'*rtWeights{1,T};
   else;
      for tau=1:T;
         %
         % compute the common term
         %
         rtWeights{1,tau} = Lt{1,t}'*rtWeights{1,tau};
         %
         % and the rest
         %
         if tau<t;
            rtWeights{1,tau} = rtWeights{1,tau}-(Gt{1,t}*StateForecastWeights{t,tau});
         elseif tau==t;
            rtWeights{1,tau} = Mt{1,t}+rtWeights{1,tau};
         end;
         %
         % and now the weights for the smoother
         %
         StateSmoothWeights{t,tau} = Ptt1(:,:,t)*rtWeights{1,tau};
         if tau<t;
            StateSmoothWeights{t,tau} = StateForecastWeights{t,tau}+StateSmoothWeights{t,tau};
         end;
         %
         % the weights for the smoothed shocks
         %
         ShockSmoothWeights{t,tau} = B0'*rtWeights{1,tau};
      end;
      rtInitCondWeights = (Lt{1,t}'*rtInitCondWeights)-Gt{1,t}*StateForecastInitCondWeights{1,t};
   end;
   StateSmoothInitCondWeights{1,t} = StateForecastInitCondWeights{1,t}+(Ptt1(:,:,t)*rtInitCondWeights);
   ShockSmoothInitCondWeights{1,t} = B0'*rtInitCondWeights;
   %
   % decompositions
   %
   for j=1:n;
      for tau=1:T;
         AlphaMatS = NaN*ones(r,n);
         AlphaMatEtaS = NaN*ones(q,n);
         postau = (~isnan(Z(:,tau)));
         AlphaMatS(:,postau) = StateSmoothWeights{t,tau};
         AlphaMatEtaS(:,postau) = ShockSmoothWeights{t,tau};
         if isnan(Z(j,tau))==0;
            StateSmoothDecomp(:,t,j) = StateSmoothDecomp(:,t,j)+(AlphaMatS(:,j)*Z(j,tau));
            ShockSmoothDecomp(:,t,j) = ShockSmoothDecomp(:,t,j)+(AlphaMatEtaS(:,j)*Z(j,tau));
         end;
      end;
   end;
   StateSmoothDecomp(:,t,n+1) = StateSmoothInitCondWeights{1,t}*InitialStateVector;
   ShockSmoothDecomp(:,t,n+1) = ShockSmoothInitCondWeights{1,t}*InitialStateVector;
   %
%   SmoothKsi = 0;
%   for tau=1:T;
%      SmoothKsi = SmoothKsi+(StateSmoothWeights{t,tau}*Z(:,tau));
%   end;
%   disp([KsitT(:,t) SmoothKsi])
   %
   % shift back 1 period
   %
   t = t-1;
   drawnow;
   pause(0.02);
end;
%
% and finally the initial sample t=d,d-1,...,1
%
% first we initialize rtWeights for rtilde(d+1|T) based on the last rtWeights
%
OldrtWeights = rtWeights;
for tau=1:T;
   n1 = size(OldrtWeights{1,tau});
   if n1>0
      rtWeights{1,tau} = [OldrtWeights{1,tau};zeros(r,size(OldrtWeights{1,tau},2))];
   else;
      rtWeights{1,tau} = zeros(2*r,[]);
   end;
end;
rtInitCondWeights = [rtInitCondWeights;zeros(size(rtInitCondWeights))];
clear('OldrtWeights');
%
while t>=1;
   Ptilde = [PStar(:,:,t) PInf(:,:,t)];
   if t<T;
      %
      % in this normal case the initial sample is shorter than the full sample
      %
      for tau=1:T;
         %
         % compute the common term
         %
         rtWeights{1,tau} = Hhatt{1,t}*rtWeights{1,tau};
         %
         % and the rest
         %
         if tau<t;
            rtWeights{1,tau} = rtWeights{1,tau}-(Ghatt{1,t}*StateForecastWeights{t,tau});
         elseif tau==t;
            rtWeights{1,tau} = Fhatt{1,t}+rtWeights{1,tau};
         end;
         %
         % and now the weights for the smoother
         %
         StateSmoothWeights{t,tau} = Ptilde*rtWeights{1,tau};
         if tau<t;
            StateSmoothWeights{t,tau} = StateForecastWeights{t,tau}+StateSmoothWeights{t,tau};
         end;
         %
         % the weights for the smoothed shocks
         %
         ShockSmoothWeights{t,tau} = B0'*Jr'*rtWeights{1,tau};
      end;
      rtInitCondWeights = (Hhatt{1,t}*rtInitCondWeights)-Ghatt{1,t}*StateForecastInitCondWeights{1,t};
   elseif t==T;
      %
      % bad situation where d=T
      %
      for tau=1:T-1;
         rtWeights{1,tau} = -Ghatt{1,T}*StateForecastWeights{T,tau};
         StateSmoothWeights{T,tau} = StateForecastWeights{T,tau}+(Ptilde*rtWeights{1,tau});
         ShockSmoothWeights{T,tau} = B0'*Jr'*rtWeights{1,tau};
      end;
      rtWeights{1,T} = Fhatt{1,T};
      StateSmoothWeights{T,T} = Ptilde*rtWeights{1,T};
      rtInitCondWeights = -Ghatt{1,T}*StateForecastInitCondWeights{1,T};
      %
      ShockSmoothWeights{T,T} = B0'*rtWeights{1,T};
   end;
   StateSmoothInitCondWeights{1,t} = StateForecastInitCondWeights{1,t}+(Ptilde*rtInitCondWeights);
   ShockSmoothInitCondWeights{1,t} = B0'*Jr'*rtInitCondWeights;
   %
%   SmoothKsi = 0;
%   for tau=1:T;
%      SmoothKsi = SmoothKsi+(StateSmoothWeights{t,tau}*Z(:,tau));
%   end;
%   disp([KsitT(:,t) SmoothKsi])
   %
   % shift back 1 period
   %
   t = t-1;
   drawnow;
   pause(0.02);
end;
%
% shut down the wait dialog
%
if DSGEModel.RunDriver==0;
   if IsHandle(WaitHandle)==1;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
% clear some variables
%
clear('Mt','Lt','Gt','rtWeights','Fhatt','Ghatt','Hhatt');
drawnow;
pause(0.02);
%
% determine the date vector
%
DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
StateDecomp.DateVector = DateVector(DSGEModel.KalmanFirstObservation:T);
StateDecomp.SubSampleStrMat = SubSampleStrMat(DSGEModel.KalmanFirstObservation:T,:);
%
if VarType>=3;
   %
   % initializde the "StdDev" field
   %
   StateDecomp.StdDev = [];
   StateDecomp.StateStdDev = [];
   %
   % check if we can improve on these re-weighting terms of the observed variable standard deviations
   %
   if (isempty(DSGEModel.UnitRootStates)==1)&(DSGEModel.AllowUnitRoot==0);
      %
      % the covariance matrix of the state variables
      %
      [PMatrix,stat] = DoublingAlgorithmLyapunov(F,B0*B0',DSGEModel.DAMaximumIterationsValue*100,10^(-(DSGEModel.DAToleranceValue+1)));
      if length(size(H))==2;
         SigmaY = (H'*PMatrix*H)+R;
         StateDecomp.StdDev = sqrt(diag(SigmaY))*ones(1,T);
      else;
         StateDecomp.StdDev = zeros(n,T);
         for t=1:T;
            StateDecomp.StdDev(:,t) = sqrt(diag((H(:,:,t)*PMatrix*H(:,:,t))+R));
         end;
      end;
      StateDecomp.StateStdDev = sqrt(diag(PMatrix));
   end;
end;
%
% generate the output
%
if VarType==1;
   %
   % clear variables
   %
   clear('StateForecastWeights','StateSmoothWeights','StateForecastInitCondWeights','StateSmoothInitCondWeights');
   clear('ShockSmoothWeights','ShockSmoothInitCondWeights');
   drawnow;
   pause(0.02);
   %
   % state variable decompositions
   %
   StateDecomp.StateForecastDecomp = StateForecastDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   StateDecomp.StateSmoothDecomp = StateSmoothDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   StateDecomp.VariableNames = DSGEModel.StateVariableNames;
   StateDecomp.A = A;
   StateDecomp.H = H;
   StateDecomp.F = F;
   StateDecomp.Y = DSGEModel.Y(:,FirstPeriod+DSGEModel.KalmanFirstObservation-1:LastPeriod);
elseif VarType==2;
   %
   % clear variables
   %
   clear('StateForecastWeights','StateSmoothWeights','StateForecastInitCondWeights','StateSmoothInitCondWeights');
   clear('ShockSmoothWeights','ShockSmoothInitCondWeights');
   drawnow;
   pause(0.02);
   %
   % shock decompositions
   %
   StateDecomp.ShockSmoothDecomp = ShockSmoothDecomp(:,DSGEModel.KalmanFirstObservation:T,:);
   VariableNames = DSGEModel.ShockAliases(KeepVar,:);
   for j=1:size(VariableNames);
      VariableNames(j,:) = strrep(VariableNames(j,:),'_',' ');
   end;
   StateDecomp.VariableNames = VariableNames;
elseif VarType==3;
   %
   % clear shock weights
   %
   clear('ShockSmoothWeights','ShockSmoothInitCondWeights','StateForecastDecomp','StateSmoothDecomp','ShockSmoothDecomp');
   drawnow;
   pause(0.02);
   %
   % state variable weights
   %
   StateDecomp.StateForecastWeights = StateForecastWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.StateSmoothWeights = StateSmoothWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.StateForecastInitCondWeights = StateForecastInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.StateSmoothInitCondWeights = StateSmoothInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.VariableNames = DSGEModel.StateVariableNames;
   if isempty(StateDecomp.StdDev)==0;
      StateDecomp.StdDev = StateDecomp.StdDev(:,DSGEModel.KalmanFirstObservation:T);
   end;
   %
   % clear variables
   %
   clear('StateForecastWeights','StateSmoothWeights','StateForecastInitCondWeights','StateSmoothInitCondWeights');
   drawnow;
   pause(0.02);
else VarType==4;
   %
   % clear state variable weights
   %
   clear('StateForecastWeights','StateSmoothWeights','StateForecastInitCondWeights','StateSmoothInitCondWeights','StateForecastDecomp','StateSmoothDecomp','ShockSmoothDecomp');
   drawnow;
   pause(0.02);
   %
   % shock weights
   %
   StateDecomp.ShockSmoothWeights = ShockSmoothWeights(DSGEModel.KalmanFirstObservation:T,DSGEModel.KalmanFirstObservation:T);
   StateDecomp.ShockSmoothInitCondWeights = ShockSmoothInitCondWeights(1,DSGEModel.KalmanFirstObservation:T);
   VariableNames = DSGEModel.ShockAliases(KeepVar,:);
   for j=1:size(VariableNames);
      VariableNames(j,:) = strrep(VariableNames(j,:),'_',' ');
   end;
   StateDecomp.VariableNames = VariableNames;
   if isempty(StateDecomp.StdDev)==0;
      StateDecomp.StdDev = StateDecomp.StdDev(:,DSGEModel.KalmanFirstObservation:T);
   end;
   %
   % clear variables
   %
   clear('ShockSmoothWeights','ShockSmoothInitCondWeights');
   drawnow;
   pause(0.02);
end;
%
% when the dimensions are large, repack memory in YADA's working directory
%
if (n>20)&(MatlabNumber>=7);
   YADApack;
end;

%
% end of DSGEObservationWeightsThetaDiffuse.m
%
