function [PredMean,Yss,YTh,status] = DSGEPointForecastObsWeightDecomp(KsiT,KsiTTDecomp,A,H,F,Y,X,h,IsOriginal,DSGEModel)
% DSGEPointForecastObsWeightDecomp: Computes the decomposition of the point forecasts
%                                   of the observed variables based on the observed
%                                   variables over the historical sample using fixed
%                                   parameter values.
%
% USAGE:
%
%       [PredMean,Yss,YTh,status] = DSGEPointForecastObsWeightDecomp(KsiTTDecomp,A,H,F,Y,X,h,IsOriginal,DSGEModel)
%
% REQUIRED INPUT:  KsiTTDecomp (matrix) of dimension rx(n+1) of the decomposition
%                              of the state variables in the last historical period
%                              based on the historical data (this is obtained from
%                              the observation weight decomposition of the state
%                              variables over the full historical sample). The
%                              decomposition gives the weighted sum of the individual
%                              observed variables and of the initial value of the
%                              state variables (typically 0).
%
%                  A (matrix) of dimension kxn with the steady-state coefficients on
%                              the deterministic variables (typically a constant).
%
%                  H (matrix) of dimension rxn (and possibly a third dimension giving
%                              time).
%
%                  F (matrix) of dimension rxr with the coefficients on lagged state
%                              variables in the state equation.
%
%                  Y (matrix) of dimension nxT with the historical data of the observed variables.
%
%                  X (matrix) of dimension kxh with the deterministic variables over the forecast
%                              sample.
%
%                  h (integer) with the forecast horizon.
%
%                  IsOriginal (boolean) which takes the value 1 if the original data should be
%                              forecasted and 0 if the annualized data should be forecasted.
%
%                  DSGEModel (structure) whose fields contain all required information about the
%                              model, such as input files, variable and sample data.
%
% REQUIRED OUTPUT: PredMean (matrix) of dimension nxh with the point forecasts of the observed
%                              variables.
%
%                  Yss (matrix) of dimension nxh with the steady-state values of the observed
%                              variables over the forecast sample.
%
%                  YTh (matrix) of dimension nx(n+1)xh with the decomposition of the point
%                              forecasts of the observed variables into their historical data
%                              and initial value of the state variables.
%
%                  status (boolean) which takes the value 1 if the calculations were completed
%                              and 0 otherwise.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: July 26, 2023.
%                        This version: January 26, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 27-07-2023: Cleaned up the code.
%
% * 26-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
status = 1;
n = DSGEModel.n;
k = DSGEModel.k;
r = length(KsiT);
PredMean = zeros(n,h);
Yss = zeros(n,h);
YTh = zeros(n,n+1,h);
T = size(Y,2);
%
% scaling issues in the measurement equation between the
% observed variables and the state variables (like in the
% An&Schorfheide model) can lead to a difference between
% KsiT and sum(KsiTTDecomp,2). Any such difference will be
% contributed to the initial value of the decomposition.
%
KsiDiff = KsiT-sum(KsiTTDecomp,2);
KsiTTDecomp(:,n+1) = KsiTTDecomp(:,n+1)+KsiDiff;
%
% compute the steady-state
%
if k>0;
   Yss = A'*X;
end;
%
if length(size(H))==3;
   %
   % we need to determine which elements of H to use
   %
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      status = 0;
      if DSGEModel.RunDriver==0;
         ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations and forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the point forecast observation weight decompositions.'];
         About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      else;
         SetLogText(strvcat(['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower'],['than the number of observations and forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore'],'aborted from the point forecast observation weight decompositions.'));
      end;
      return;
   end;
end;
%
% compute the decomposition for the original data
%
Fi = eye(r);
if length(size(H))==2;
   for i=1:h;
      Fi = Fi*F;
      YTh(:,:,i) = H'*Fi*KsiTTDecomp;
      PredMean(:,i) = Yss(:,i)+(H'*Fi*KsiT);
   end;
else;
   for i=1:h;
      Fi = Fi*F;
      YTh(:,:,i) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*KsiTTDecomp;
      PredMean(:,i) = Yss(:,i)+(H(:,:,DSGEModel.LastPeriod+i)'*Fi*KsiT);
   end;
end;
%
% check if we should annualize the steady-state
% and determine historical data to use for
% annualization
%
YInit = [];
Freq = 1;
if IsOriginal==0;
   if strcmp(DSGEModel.DataFrequency,'q')==1;
      Freq = 4;
   elseif strcmp(DSGEModel.DataFrequency,'q')==1;
      Freq = 12;
   end;
   %
   % get all the deterministic data prior to DSGEModel.LastPeriod
   %
   if Freq>1;
      if k>0;
         XOld = DSGEModel.X(:,DSGEModel.LastPeriod-Freq+2:DSGEModel.LastPeriod);
         YssOld = A'*XOld;
         Yss = Annualize([YssOld Yss],DSGEModel.annual,DSGEModel.annualscale);
      else;
         YssOld = zeros(n,Freq-1);
      end;
      YInit = Y(:,T-Freq+2:T);
   end;
end;
%
if (IsOriginal==0)&(Freq>1);
   PredMean = Annualize([YInit PredMean],DSGEModel.annual,DSGEModel.annualscale);
   %
   % it remains to annualize the decomposition
   % we try to do this component by component
   %
   f = size(YInit,2);
   YThInit = zeros(n,n+1,f);
   %
   % make sure initial values are measured in deviation from steady state
   %
   YInit = YInit-YssOld;
   for j=1:f;
      YThInit(:,1:n,j) = diag(YInit(:,j));
   end;
   YThAnnual = zeros(n,n+1,h);
   Tmp = zeros(n,h);
   for j=1:n+1;
      YThAnnual(:,j,:) = Annualize([squeeze(YThInit(:,j,:)) squeeze(YTh(:,j,:))],DSGEModel.annual,DSGEModel.annualscale);;
   end;
   YTh = YThAnnual;
end;

%
% end of DSGEPointForecastObsWeightDecomp.m
%
