function HFED = HFEDDistribution(DSGEModel,CurrINI,h,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
% HFEDDistribution: Computes the mean value of the historical forecast error decompositions as well as
%                   quantiles by loading data from disk.
%
% USAGE:
%
%       HFED = HFEDDistribution(DSGEModel,CurrINI,h,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with intialization information.
%
%                  h (positive integer), with the forecast horizon.
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                           parameters.
%
%                  TotalDraws (integer) with the number of draws from the posterior distribution of the parameters
%                           that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                           if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                           likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: HFED (structure) with fields "epstth", an nxT matrix with the mean h-steps ahead historical
%                           forecast errors, "wthT", an nxT matrix with the mean h-steps ahead smooth estimate
%                           of the measurement error, "KsiErr", nxT matrix with the mean of the share of the
%                           historical forecast error due to the state prediction error, "etathT", nxTxq matrix
%                           with the mean of the shares of the historical forecast error due to the structural
%                           shocks. Furthermore, "R" (nxn) holds the posterior mode estimate of the measurement
%                           error covariance matrix, "KeepVar" is a qx1 vector with intgeres indicating the
%                           nonzero structural shocks, "forecastHorizon" postive integer with the forecast
%                           horizon, and "Quantiles" a vector structura with the quantile values for "epstth",
%                           "wthT", "KsiErr", and "etathT".
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 7, 2008.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 08-05-2008: Finished the function.
%
% * 13-05-2008: Fixed a bug when updating the CurrSaves variable.
%
% * 23-05-2008: Updated the documentation.
%
% * 22-01-2009: Updated the documentation.
%
% * 27-03-2009: Added the input variable "IsPosterior".
%
% * 24-04-2009: Added figure refresh code for wait dialog.
%
% * 05-01-2010: Updated the documentation.
%
% * 05-01-2011: Updated the documentation.
%
% * 12-01-2011: The function now supports unix and mac paths.
%
% * 05-01-2012: Updated the documentation.
%
% * 28-08-2012: Added support for higest probability density confidence bands.
%
% * 03-01-2013: Updated the documentation.
%
% * 03-09-2013: Added "try-catch" code around possibly too large matrices.
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 25-08-2017: Added the "Weights" input variable.
%
% * 04-09-2017: Speeded up the Weights computations.
%
% * 16-05-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
HFED.ForecastHorizon = h;
NumFiles = ceil(NumDraws/min(NumDraws,100));
NumQuants = length(DSGEModel.Percentiles);
%
% load one file to record the size of the data
%
if IsPosterior==1;
   file = FixFilePath([DSGEModel.OutputDirectory '\hfed\DSGE-HFED-' int2str(h) '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-1-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
else;
   file = FixFilePath([DSGEModel.OutputDirectory '\hfed\DSGE-HFED-Prior-' int2str(h) '-' DSGEModel.NameOfModel '-1-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
end;
FEData = load(file);
[n,T] = size(FEData.HFED(1).HistDecompStructure.epstth);
q = length(FEData.KeepShocks);
HFED.R = FEData.Rmode;
HFED.KeepVar = FEData.KeepShocks;
ShockNames = DSGEModel.StateShockNames(FEData.KeepShocks,:);
%
HFED.epstth = zeros(n,T);
HFED.wthT = zeros(n,T);
HFED.KsiErr = zeros(n,T);
HFED.etathT = zeros(n,T,q);
for i=1:NumQuants;
   HFED.Quantiles(i).epstth = zeros(n,T);
   HFED.Quantiles(i).wthT = zeros(n,T);
   HFED.Quantiles(i).KsiErr = zeros(n,T);
   HFED.Quantiles(i).etathT = zeros(n,T,q);
   HFED.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
% start with the nxT matrices
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' Prior';
end;
WeightsT = Weights*ones(1,T);
%
% set up a wait dialog
%
txt = ['Please wait while YADA computes the mean historical forecast error decompositions as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,shock): (' StringTrim(DSGEModel.VariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ').'];
WaitHandle = WaitDLG(txt,'information',['Forecast Error Decompositions -' PriorHeader ' Distribution'],500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
for i=1:n;
   txt = ['Please wait while YADA computes the mean historical forecast error decompositions as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,shock): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(1,:)) ').'];
   set(WaitControls.text,'String',txt);
   drawnow;
   %
   % initialize the data
   %
   try;
      TmpEps = zeros(NumDraws,T);
      TmpKsi = zeros(NumDraws,T);
      TmpShock = zeros(NumDraws,T);
      if rank(HFED.R)>0;
         TmpwthT = zeros(NumDraws,T);
      end;
      CurrSaves = 0;
      %
      % load the data
      %
      for ThisSave=1:NumFiles;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\hfed\DSGE-HFED-' int2str(h) '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\hfed\DSGE-HFED-Prior-' int2str(h) '-' DSGEModel.NameOfModel '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
         end;
         HFEDData = load(file);
         FED = HFEDData.HFED;
         NumSaves = length(FED);
         for s=1:NumSaves;
            TmpEps(CurrSaves+s,:) = FED(s).HistDecompStructure.epstth(i,:);
            TmpKsi(CurrSaves+s,:) = FED(s).HistDecompStructure.KsiErr(i,:);
            TmpShock(CurrSaves+s,:) = FED(s).HistDecompStructure.etathT(i,:,1);
            if rank(HFED.R)>0;
               TmpwthT(CurrSaves+s,:) = FED(s).HistDecompStructure.wthT(i,:);
            end;
         end;
         CurrSaves = CurrSaves+NumSaves;
         drawnow;
      end;
      %
      % compute the mean
      %
      HFED.epstth(i,:) = (1/NumDraws)*sum(TmpEps.*WeightsT);
      HFED.KsiErr(i,:) = (1/NumDraws)*sum(TmpKsi.*WeightsT);
      HFED.etathT(i,:,1) = (1/NumDraws)*sum(TmpShock.*WeightsT);
      if rank(HFED.R)>0;
         HFED.wthT(i,:) = (1/NumDraws)*sum(TmpwthT.*WeightsT);
      end;
      %
      % sort the data
      %
      TmpEps = sort(TmpEps);
      TmpKsi = sort(TmpKsi);
      TmpShock = sort(TmpShock);
      if rank(HFED.R)>0;
         TmpwthT = sort(TmpwthT);
      end;
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
            end;
            HFED.Quantiles(l).epstth(i,:) = TmpEps(QuantVal,:);
            HFED.Quantiles(l).KsiErr(i,:) = TmpKsi(QuantVal,:);
            HFED.Quantiles(l).etathT(i,:,1) = TmpShock(QuantVal,:);
            if rank(HFED.R)>0;
               HFED.Quantiles(l).wthT(i,:) = TmpwthT(QuantVal,:);
            end;
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
            for t=1:T;
               Bounds = HighestProbabilityDensityBand(TmpEps(:,t),ConfDraws);
               HFED.Quantiles(l).epstth(i,t) = Bounds(1);
               HFED.Quantiles(NumQuants-l+1).epstth(i,t) = Bounds(2);
               Bounds = HighestProbabilityDensityBand(TmpKsi(:,t),ConfDraws);
               HFED.Quantiles(l).KsiErr(i,t) = Bounds(1);
               HFED.Quantiles(NumQuants-l+1).KsiErr(i,t) = Bounds(2);
               Bounds = HighestProbabilityDensityBand(TmpShock(:,t),ConfDraws);
               HFED.Quantiles(l).etathT(i,t,1) = Bounds(1);
               HFED.Quantiles(NumQuants-l+1).etathT(i,t,1) = Bounds(2);
               if rank(HFED.R)>0;
                  Bounds = HighestProbabilityDensityBand(TmpwthT(:,t),ConfDraws);
                  HFED.Quantiles(l).wthT(i,t) = Bounds(1);
                  HFED.Quantiles(NumQuants-l+1).wthT(i,t) = Bounds(2);
               end;
            end;
         end;
      end;
   catch;
      for t=1:T;
         TmpEps = zeros(NumDraws,1);
         TmpKsi = zeros(NumDraws,1);
         TmpShock = zeros(NumDraws,1);
         if rank(HFED.R)>0;
            TmpwthT = zeros(NumDraws,1);
         end;
         CurrSaves = 0;
         %
         % load the data
         %
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\hfed\DSGE-HFED-' int2str(h) '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\hfed\DSGE-HFED-Prior-' int2str(h) '-' DSGEModel.NameOfModel '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
            end;
            HFEDData = load(file);
            FED = HFEDData.HFED;
            NumSaves = length(FED);
            for s=1:NumSaves;
               TmpEps(CurrSaves+s,1) = FED(s).HistDecompStructure.epstth(i,t);
               TmpKsi(CurrSaves+s,1) = FED(s).HistDecompStructure.KsiErr(i,t);
               TmpShock(CurrSaves+s,1) = FED(s).HistDecompStructure.etathT(i,t,1);
               if rank(HFED.R)>0;
                  TmpwthT(CurrSaves+s,1) = FED(s).HistDecompStructure.wthT(i,t);
               end;
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % compute the mean
         %
         HFED.epstth(i,t) = (1/NumDraws)*sum(TmpEps.*Weights);
         HFED.KsiErr(i,t) = (1/NumDraws)*sum(TmpKsi.*Weights);
         HFED.etathT(i,t,1) = (1/NumDraws)*sum(TmpShock.*Weights);
         if rank(HFED.R)>0;
            HFED.wthT(i,t) = (1/NumDraws)*sum(TmpwthT.*Weights);
         end;
         %
         % sort the data
         %
         TmpEps = sort(TmpEps);
         TmpKsi = sort(TmpKsi);
         TmpShock = sort(TmpShock);
         if rank(HFED.R)>0;
            TmpwthT = sort(TmpwthT);
         end;
         if DSGEModel.ConfidenceRegionMethod==1;
            for l=1:NumQuants;
               if DSGEModel.Percentiles(l)<50;
                  QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
               else;
                  QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
               end;
               HFED.Quantiles(l).epstth(i,t) = TmpEps(QuantVal,1);
               HFED.Quantiles(l).KsiErr(i,t) = TmpKsi(QuantVal,1);
               HFED.Quantiles(l).etathT(i,t,1) = TmpShock(QuantVal,1);
               if rank(HFED.R)>0;
                  HFED.Quantiles(l).wthT(i,t) = TmpwthT(QuantVal,1);
               end;
            end;
         else;
            for l=1:floor(NumQuants/2);
               ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
               Bounds = HighestProbabilityDensityBand(TmpEps,ConfDraws);
               HFED.Quantiles(l).epstth(i,t) = Bounds(1);
               HFED.Quantiles(NumQuants-l+1).epstth(i,t) = Bounds(2);
               Bounds = HighestProbabilityDensityBand(TmpKsi,ConfDraws);
               HFED.Quantiles(l).KsiErr(i,t) = Bounds(1);
               HFED.Quantiles(NumQuants-l+1).KsiErr(i,t) = Bounds(2);
               Bounds = HighestProbabilityDensityBand(TmpShock,ConfDraws);
               HFED.Quantiles(l).etathT(i,t,1) = Bounds(1);
               HFED.Quantiles(NumQuants-l+1).etathT(i,t,1) = Bounds(2);
               if rank(HFED.R)>0;
                  Bounds = HighestProbabilityDensityBand(TmpwthT,ConfDraws);
                  HFED.Quantiles(l).wthT(i,t) = Bounds(1);
                  HFED.Quantiles(NumQuants-l+1).wthT(i,t) = Bounds(2);
               end;
            end;
         end;
      end;
   end;
   drawnow;
end;
%
% now we turn to the structural shocks
%
for i=1:n;
   for j=2:q;
      txt = ['Please wait while YADA computes the mean historical forecast error decompositions as well as percentile values. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,shock): (' StringTrim(DSGEModel.VariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ').'];
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      % initialize
      %
      TmpShock = zeros(NumDraws,T);
      CurrSaves = 0;
      %
      % load the data
      %
      for ThisSave=1:NumFiles;
         if IsPosterior==1;
            file = FixFilePath([DSGEModel.OutputDirectory '\hfed\DSGE-HFED-' int2str(h) '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
         else;
            file = FixFilePath([DSGEModel.OutputDirectory '\hfed\DSGE-HFED-Prior-' int2str(h) '-' DSGEModel.NameOfModel '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
         end;
         HFEDData = load(file);
         FED = HFEDData.HFED;
         NumSaves = length(FED);
         for s=1:NumSaves;
            TmpShock(CurrSaves+s,:) = FED(s).HistDecompStructure.etathT(i,:,j);
         end;
         CurrSaves = CurrSaves+NumSaves;
         drawnow;
      end;
      %
      % compute the mean
      %
      HFED.etathT(i,:,j) = (1/NumDraws)*sum(TmpShock.*WeightsT);
      %
      % sort the data
      %
      TmpShock = sort(TmpShock);
      if DSGEModel.ConfidenceRegionMethod==1;
         for l=1:NumQuants;
            if DSGEModel.Percentiles(l)<50;
               QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
            else;
               QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
            end;
            HFED.Quantiles(l).etathT(i,:,j) = TmpShock(QuantVal,:);
         end;
      else;
         for l=1:floor(NumQuants/2);
            ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
            for t=1:T;
               Bounds = HighestProbabilityDensityBand(TmpShock(:,t),ConfDraws);
               HFED.Quantiles(l).etathT(i,t,j) = Bounds(1);
               HFED.Quantiles(NumQuants-l+1).etathT(i,t,j) = Bounds(2);
            end;
         end;
      end;
      drawnow;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of HFEDDistribution.m
%
