function SolutionCode = MonteCarloFiltering(thetaDraws,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI)
% MonteCarloFiltering: Runs the Monte Carlo filtering routine that checks if the DSGE model has a unique convergent
%                      solution, indeterminacy, or no stable solution at various parameter values taken from the
%                      prior distribution.
%
% USAGE:
%
%       SolutionCode = MonteCarloFiltering(thetaDraws,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI)
%
% REQUIRED INPUT:  thetaDraws (matrix) of dimension NumParamxNumDraws with values of the parameters of the
%                             DSGE model.
%
%                  thetaPositions (structure) where thetaPositions(i).parameter is a string vector giving the name
%                             of the model parameter in position i of theta. The field "distribution" has a string
%                             vector giving the assumed prior distribution of the parameter.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                             DSGE model.
%
%                  AIMData (structure) with data collected by AIM about the model.

%                  DSGEModel (structure) whose fields contain all required information about the model,
%                             such as input files, variable and sample data.
%
%                  CurrINI (structure) with initialization information.
%
% REQUIRED OUTPUT: SolutionCode (vector) of dimension NumDraws with integer 1 for unique, convergent solution,
%                             2 for indeterminacy, and 0 for no stable solution.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 14, 2011.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 15-12-2011: Continued work on the function.
%
% * 19-12-2011: Updated the documentation.
%
% * 05-01-2012: Updated the documentation.
%
% * 02-01-2013: Added try-catch code around the call to "SolveDSGEModel".
%
% * 28-01-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 22-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 16-05-2018: Updated the documentation.
%
% * 03-08-2018: The function supports user written model solver function.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
[NumParam,NumDraws] = size(thetaDraws);
%
% setup a wait dialog while checking the properties of the prior draws
%
txt = ['Please wait while YADA checks the DSGE model solution for ' int2str(NumDraws) ' values from the prior distribution. Computations started at: ' StringTrim(datestr(now,14)) '.'];
WaitHandle = WaitDLG(txt,'information',['Monte Carlo Filtering - ' int2str(NumDraws) ' Draws'],500,300,CurrINI,1);
WaitControls = get(WaitHandle,'UserData');
drawnow;
%
% intialize
%
SolutionCode = [];
Solutions = zeros(1,NumDraws);
StopEstimation = 0;
%
for i=1:NumDraws;
   %
   % check if the cancel button was pressed
   %
   if strcmp(get(WaitControls.cancel,'UserData'),'cancel')==1;
      StopEstimation = 1;
      break;
   end;
   try;
      [A,H,R,F,B0,mcode] = SolveDSGEModel(DSGEModel,thetaDraws(:,i),thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
   catch;
      mcode = 0;
   end;
   if mcode==1;
      status = 1;
   elseif (mcode==4)|(mcode==-3);
      %
      % indeterminacy in AiM and in Klein or Sims
      %
      status = 2;
   else;
      status = 0;
   end;
   Solutions(i) = status;
   drawnow;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);
%
if StopEstimation==1;
   return;
end;
SolutionCode = Solutions;

%
% end of MonteCarloFiltering.m
%
