function IRFs = StateVarIRFsDistribution(DSGEModel,CurrINI,ShockType,NumDraws,TotalDraws,CurrChain,IsPosterior,Weights)
% StateVarIRFsDistribution: Computes the mean value of impulse responses for the state variables as well as
%                           quantiles by loading data from disk.
%
% USAGE:
%
%       IRFs = StateVarIRFsDistribution(DSGEModel,CurrINI,ShockType,NumDraws,TotalDraws,CurrChain,...
%                                      IsPosterior,Weights)
%
% REQUIRED INPUT:  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure) with initialization information.
%
%                  ShockType (string vector) taking the values "individual" or "group".
%
%                  NumDraws (integer) with the number of effective draws from the posterior distribution of the
%                                 parameters.
%
%                  TotalDraws (integer) with the total number of draws from the posterior distribution of the
%                                 parameters that could have been used.
%
%                  CurrChain (integer) with the number for the current Markov chain.
%
%                  IsPosterior (boolean) that is 1 if draws from the posterior distribution are used and 0
%                                 if draws from the prior are supplied.
%
%                  Weights (vector) of dimension NumDraws x 1 with the importance weights from the SMC with
%                                 likelihood tempering posterior sampler.
%
% REQUIRED OUTPUT: IRFs (structure) with fields "Ksi" and "Quantiles". The "Ksi" field holds a 3D matrix of dimension
%                                 r x q x h+1, where r is the number of state variables, q the number of shocks,
%                                 and h the impulse response horizon. The values are the posterior mean impulse
%                                 responses. The field "Quantiles" is a vector structure with fields "Ksi" and
%                                 "percent", where "Ksi" holds a 3D matrix of dimension r x q x h+1 with values from
%                                 the percent quantile of the marginal distribution of the impulse responses and
%                                 "percent" holds the percentile value in percent.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: December 17, 2014.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 18-12-2014: Updated the documentation.
%
% * 14-01-2015: Updated the documentation.
%
% * 23-02-2016: Updated the documentation.
%
% * 29-03-2017: Updated the documentation.
%
% * 24-08-2017: Added the Weights input variable.
%
% * 09-04-2018: Added a clear command after IR has been obtained.
%
% * 10-04-2018: Memory issue code for large models.
%
% * 23-04-2018: Took into account that the number of stored files increased when n>20.
%
% * 29-08-2018: Updated the documentation.
%
% * 24-01-2019: Updated the documentation.
%
% * 21-01-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
IRFs = [];
if isempty(Weights)==1;
   Weights = ones(NumDraws,1);
end;
%
if DSGEModel.n<=20;
   NumFiles = ceil(NumDraws/min(NumDraws,1000));
else;
   NumFiles = ceil(NumDraws/min(NumDraws,100));
end;
%
% Determine the impulse response horizon
%
if strcmp(DSGEModel.DataFrequency,'q')==1;
   SampleFreq = 4;
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   SampleFreq = 12;
else;
   SampleFreq = 1;
end;
h = SampleFreq*DSGEModel.IRHorizon;
TypeStr = 'Original';
if strcmp(ShockType,'individual')==1;
   IMatrix = [];
   q = length(DSGEModel.StateShockPositions);
   ShockNames = DSGEModel.StateShockNames;
   ShockStr = 'shock';
else;
   IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups)';
   q = size(IMatrix,1);
   ShockNames = DSGEModel.ShockGroupNames;
   ShockStr = 'shock group';
   for j=1:size(ShockNames,1);
      ShockNames(j,:) = strrep(ShockNames(j,:),'_',' ');
   end;
end;
%
r = length(DSGEModel.StateVariablePositions);
NumQuants = length(DSGEModel.Percentiles);
%
if DSGEModel.n>20;
   YADApack;
end;
%
% initialize output
%
IRFs.Ksi = zeros(r,q,h+1);
for i=1:NumQuants;
   IRFs.Quantiles(i).Ksi = zeros(r,q,h+1);
   IRFs.Quantiles(i).percent = DSGEModel.Percentiles(i);
end;
%
PriorHeader = '';
if IsPosterior==0;
   PriorHeader = ' For Prior';
end;
%
% set up a wait dialog
%
if IsPosterior==1;
   txt = ['Please wait while YADA computes the posterior mean impulse responses as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.StateVariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ')'];
else;
   txt = ['Please wait while YADA computes the prior mean impulse responses as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.StateVariableNames(1,:)) ',' StringTrim(ShockNames(1,:)) ')'];
end;
WaitHandle = WaitDLG(txt,'information',['Impulse Responses' PriorHeader ' - State Variables'],500,150,CurrINI,0);
WaitControls = get(WaitHandle,'UserData');
drawnow;
pause(0.02);
%
% now we can start loading the data
%
for i=1:r;
   for j=1:q;
      %
      % update the wait dialog
      %
      if IsPosterior==1;
         txt = ['Please wait while YADA computes the posterior mean impulse responses as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.StateVariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ')'];
      else;
         txt = ['Please wait while YADA computes the prior mean impulse responses as well as percentile values for the state variables. The data are loaded sequentially from disk and it may therefore take some time. Current (variable,' ShockStr '): (' StringTrim(DSGEModel.StateVariableNames(i,:)) ',' StringTrim(ShockNames(j,:)) ')'];
      end;
      set(WaitControls.text,'String',txt);
      drawnow;
      %
      try;
         TempVar = zeros(NumDraws,h+1);
         CurrSaves = 0;
         for ThisSave=1:NumFiles;
            if IsPosterior==1;
               file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-IR-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
            else;
               file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-IR-Prior-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
            end;
            IRData = load(file);
            IR = IRData.IR;
            clear('IRData');
            NumSaves = length(IR);
            if DSGEModel.n>20;
               for s=1:NumSaves
                  IR(s).Y = [];
               end;
            end;
            %
            % load data from CurrSaves to NumSaves+1-CurrSaves
            %
            if isempty(IMatrix)==1;
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,:) = IR(s).Ksi(i,j,:);
               end;
            else;
               for s=1:NumSaves;
                  TempVar(CurrSaves+s,:) = IMatrix(j,:)*reshape(IR(s).Ksi(i,:,:),size(IR(s).Ksi,2),h+1);
               end;
            end;
            CurrSaves = CurrSaves+NumSaves;
            drawnow;
         end;
         %
         % now we sort and compute the mean the data
         %
         for k=1:h+1;
            if max(abs(TempVar(:,k)))>1.0e-10;
               IRFs.Ksi(i,j,k) = mean(TempVar(:,k).*Weights);
            end;
            TempVar(:,k) = sort(TempVar(:,k));
            %
            % now we put the quantile information in its correct place
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                  end;
                  if abs(TempVar(QuantVal,k))>1.0e-10;
                     IRFs.Quantiles(l).Ksi(i,j,k) = TempVar(QuantVal,k);
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar(:,k),ConfDraws);
                  IRFs.Quantiles(l).Ksi(i,j,k) = Bounds(1);
                  IRFs.Quantiles(NumQuants-l+1).Ksi(i,j,k) = Bounds(2);
               end;
            end;
            drawnow;
         end;
      catch;
         for k=1:h+1;
            TempVar = zeros(NumDraws,1);
            CurrSaves = 0;
            for ThisSave=1:NumFiles;
               if IsPosterior==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-IR-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(CurrChain,'%0.0f') '-' num2str(h,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '-' num2str(TotalDraws,'%0.0f') '.mat']);
               else;
                  file = FixFilePath([DSGEModel.OutputDirectory '\irfs\DSGE-IR-Prior-' TypeStr '-' DSGEModel.NameOfModel '-' num2str(h,'%0.0f') '-' num2str(ThisSave,'%0.0f') '-' num2str(NumFiles,'%0.0f') '.' num2str(NumDraws,'%0.0f') '.mat']);
               end;
               IRData = load(file);
               IR = IRData.IR;
               clear('IRData');
               NumSaves = length(IR);
               if DSGEModel.n>20;
                  for s=1:NumSaves
                     IR(s).Y = [];
                  end;
               end;
               %
               % load data from CurrSaves to NumSaves+1-CurrSaves
               %
               if isempty(IMatrix)==1;
                  for s=1:NumSaves;
                     TempVar(CurrSaves+s,1) = IR(s).Ksi(i,j,k);
                  end;
               else;
                  for s=1:NumSaves;
                     MT = IMatrix(j,:)*reshape(IR(s).Ksi(i,:,:),size(IR(s).Ksi,2),h+1);
                     TempVar(CurrSaves+s,1) = MT(k);
                  end;
               end;
               CurrSaves = CurrSaves+NumSaves;
               drawnow;
            end;
            if max(abs(TempVar))>1.0e-10;
               IRFs.Ksi(i,j,k) = mean(TempVar.*Weights);
            end;
            TempVar = sort(TempVar);
            %
            % now we put the quantile information in its correct place
            %
            if DSGEModel.ConfidenceRegionMethod==1;
               for l=1:NumQuants;
                  if DSGEModel.Percentiles(l)<50;
                     QuantVal = ceil((DSGEModel.Percentiles(l)/100)*NumDraws);
                  else;
                     QuantVal = floor((DSGEModel.Percentiles(l)/100)*NumDraws);
                  end;
                  if abs(TempVar(QuantVal,k))>1.0e-10;
                     IRFs.Quantiles(l).Ksi(i,j,k) = TempVar(QuantVal,1);
                  end;
               end;
            else;
               for l=1:floor(NumQuants/2);
                  ConfDraws = floor((DSGEModel.Percentiles(NumQuants-l+1)-DSGEModel.Percentiles(l))*NumDraws/100);
                  Bounds = HighestProbabilityDensityBand(TempVar,ConfDraws);
                  IRFs.Quantiles(l).Ksi(i,j,k) = Bounds(1);
                  IRFs.Quantiles(NumQuants-l+1).Ksi(i,j,k) = Bounds(2);
               end;
            end;
            drawnow;
         end;
      end;
   end;
end;
%
% close the wait dialog
%
set(WaitControls.text,'UserData','done');
delete(WaitHandle);
drawnow;
pause(0.02);

%
% end of StateVarIRFsDistribution.m
%
