function CRPScore = CRPS(Xpaths,xtrue,NumWorkers)
% CRPS: Computes the continuous ranked probability score (CRPS) as the negative of the score in Gneiting
%       and Raftery (2007).
%
% USAGE:
%
%       CRPScore = CRPS(Xpaths,xtrue,NumWorkers)
%
% REQUIRED INPUT:  Xpaths (vector) of dimension T x 1 with T paths for the scalar
%                         variable X.
%
%                  xtrue (scalar) with the true value (reference value) for the CRPS.
%
%                  NumWorkers (integer) with the number of available cores.
%
% REQUIRED OUTPUT: CRPScore (scalar) with the score value.
%
%
% References: See Gneiting, T. and A.E. Raftery (2007), "Strictly Proper Scoring Rules,
%                 Prediction, and Estimation", JASA, 102(407), 359-378.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 28, 2020.
%                        This version: January 25, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 31-08-2020: Updated the documentation.
%
% * 27-01-2021: Remember Shoah.
%
% * 06-01-2022: Updated the documentation.
%
% * 10-02-2022: Fixed for CRPS2 calculation when T is large.
%
% * 11-02-2022: added the input variable NumWorkers.
%
% * 25-02-2022: Changed the definition of CRPS to match Gneiting & Raftery (JASA, 2007).
%
% * 06-01-2023: Updated the documentation.
%
% * 25-01-2024: Updated the documentation.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

T = length(Xpaths);
CRPS1 = (1/T)*sum(abs(Xpaths-xtrue));
if T<=1000;
   CRPS2 = (1/(2*T*T))*sum(vec(abs(repmat(Xpaths,1,T)-repmat(Xpaths',T,1))));
else;
   CRPS2 = 0;
   T2 = 2*(T*T);
   for j=1:T;
      CRPS2 = CRPS2+((1/T2)*sum(abs(Xpaths-Xpaths(j))));
   end;
end;
%CRPScore = CRPS1-CRPS2;
CRPScore = CRPS2-CRPS1;

%
% end of CRPS.m
%
