function [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredCondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,h,NumPaths,AnnualizeData,TransData,NameStr,SelectedYData)
% ZLBDSGECenteredCondPredictionPathsThetaWZ: Computes conditional prediction paths for a given parameter vector when the policy rate is subject
%                                            to the ZLB and the conditional distribution is not centered on the observed variables when the ZLB
%                                            is not binding. The Waggoner-Zha approach is used.
%
% USAGE:
%
%       [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredCondPredictionPathsThetaWZ(theta,thetaPositions,...
%                                                                                   ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,...
%                                                                                   h,NumPaths,AnnualizeData,TransData,NameStr,SelectedYData)
%
% REQUIRED INPUT:  theta (vector) with values of the original DSGE model parameters.
%
%                  thetaPositions (structure) with fields parameter and distribution. This function
%                                 needs the former field to know which parameter name corresponds to
%                                 position i in theta.
%
%                  ModelParameters (structure) whose fields have names given by the parameter names in the
%                                 DSGE model.
%
%                  DSGEModel (structure) with fields containing all the information about a DSGE Model for YADA.
%
%                  CurrINI (structure), with initialization information.
%
%                  YData (matrix) of dimension nxh with the data on the observed variables over the
%                                 prediction sample.
%
%                  X (matrix) of dimension kxh with the observations of the exogenous variables for
%                                 the prediction paths.
%
%                  Z (matrix) of dimension mxg with the out-of-sample data we shall condition the
%                                 predictions on.
%
%                  U (matrix) of dimension mxg with initial values for the mapping between the observed
%                                 data and the conditioning data.
%
%                  h (integer) with the the prediction horizon.
%
%                  NumPaths (integer) the number of paths to draw.
%
%                  AnnualizeData (boolean) that is 1 if the prediction paths should be annualized and 0 otherwise.
%
%                  TransData (boolean) that is 1 if the prediction paths should be transformed and 0 otherwise.
%
%                  NameStr (string vector) that is sent to the progress dialog name after the string
%                                 "Zero Lower Bound".
%
%                  SelectedYData (integer) with the selected YData set when centering the predictive distribution.
%
% REQUIRED OUTPUT: Status (boolean) that takes the value 1 if the calculations were successful and 0 otherwise.
%
%                  ZLBYPaths (matrix) of dimension nxhxNumPaths with zero lower bound consistent prediction
%                                 paths for the observed variables.
%
%                  ZLBKsiPaths (matrix) of dimension rxhxNumPaths with zero lower bound consistent prediction
%                                 paths for the state variables.
%
%                  KsitTh (matrix) of dimension rxTh with the smooth estimates of the state variables taking
%                                 data from the prediction sample into account.
%
%                  PredData (structure) with fields:
%                                 ZLBYMean   - (nxh) matrix with the mean predictions of the observed
%                                              variables.
%                                 ZLBKsiMean - (rxh) matrix with the values of the states at the mean
%                                              predictions.
%                                 X          - (kxh) matrix with the deterministic variables for the
%                                              prediction horizon.
%                                 A          - (kxn) matrix with the parameters on the exogenous variables in
%                                              the measurement equation.
%                                 H          - (rxn) matrix with parameters on the state variabels in the
%                                              measurement equation.
%                                 R          - (nxn) covariance matrix for the measurement errors.
%                                 F          - (rxr) matrix with state transitions parameters for the state
%                                              equation.
%                                 B0         - (rxq) matrix with parameters on the economic shocks in the
%                                              state equation.
%                                 KernelX    - (3D matrix) of dimension n x 2^8 x h with the x-axis values
%                                              from kernel density estimation of the marginal predictive
%                                              densities.
%                                 KernelY    - (3D matrix) of dimension n x 2^8 x h with the y-axis values
%                                              from kernel density estimation of the marginal predictive
%                                              densities.
%
%                  PredEventData (n x 8) matrix where the first column holds the number of times
%                                 the ZLBYPaths data satisfies the event, the second holds the number
%                                 of times the ZLBYPaths data is greater than the event, the third
%                                 the number of time the data is above the upper bound, while the
%                                 fourth column holds the number of times the event has been tested.
%                                 The fifth and sixth column measure the average deviation from the
%                                 lower bound when below the lower bound and the square of this, while
%                                 the seventh and eighth columns hold similar data for the upper bound
%                                 of the event when the variables are above the upper bound.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 28, 2023.
%                        This version: February 27, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 01-03-2023: Updated the documentation.
%
% * 06-03-2023: Added the time-varying H-matrix case.
%
% * 07-03-2023: Continued work on the function correcting bugs with re-centering shock values.
%
% * 08-03-2023: Finished work on the function.
%
% * 14-03-2023: Changed the condition for the conditioning restrictions to hold to 1.0e-6.
%
% * 22-03-2023: Updated the header of the progress dialog etc.
%
% * 18-04-2023: The function now supports parallel computations.
%
% * 27-04-2023: Changed random number initialization to avoid shuffle under parallel computations.
%
% * 23-11-2023: Took into account that ZLBKleinSolver requires the input variable "ZLBRuleType".
%
% * 26-01-2024: Updated the documentation.
%
% * 26-02-2024: Changed the way the smooth structural shocks are computed.
%
% * 27-02-2024: Changed "Ksit1t" output to "rttMat" with standardized forecast errors.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize the output
%
ZLBYPaths = [];
ZLBKsiPaths = [];
KsitTh = [];
PredData = [];
PredEventData = [];
Status = 0;
MultiModestyStat = [];
UniModestyStat = [];
UniModestyStatLZ = [];
n = DSGEModel.n;
k = DSGEModel.k;
if (AnnualizeData==1)|(TransData==1);
   if TransData==0;
      TypeStr = 'Annualized';
   else;
      TypeStr = 'Transformed';
   end;
else;
   TypeStr = 'Original';
end;
%
% create the output directory
%
stat = MakeDir(DSGEModel.OutputDirectory,'predictions');
if stat~=1;
   if ispc==1;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '\predictions". The computation of predictive distributions has therefore been aborted.'];
   else;
      txt = ['YADA was for some reason unable to create the directory "' DSGEModel.OutputDirectory '/predictions". The computation of predictive distributions has therefore been aborted.'];
   end;
   About(txt,'information','YADA - Directory Creation Problem',180,500,CurrINI);
   return;
end;
%
% copy files to the tmp folder
%
[stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
[stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
if FileExist(DSGEModel.UpdateParameterFile)==1;
   [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
end;
if FileExist(DSGEModel.SystemPriorFile)==1;
   [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
end;
%
% check if we should copy model solver file
%
if DSGEModel.ModelSolver==4;
   if FileExist(DSGEModel.ModelSolverFile)==1;
      [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
   end;
end;
%
% Make sure that the ModelParameter structure takes the correct values.
% This is not necessary to do when initial values are given to the function.
% In that case, more substantial checking is required!
%
ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
%
% Update any parameteters that need updating
%
if FileExist(DSGEModel.UpdateParameterFile)==1;
   ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
end;
%
% load the AIMData file
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch ME;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      AIMData = load(DSGEModel.AIMDataFile);
   catch;
      ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
   return;
end;
%
% set the InitialStateVector value
%
if DSGEModel.UseOwnInitialState==1;
   if length(DSGEModel.InitialStateValues)==size(AIMData.endog,1);
      DSGEModel.InitialStateVector = DSGEModel.InitialStateValues(DSGEModel.StateVariablePositions);
   else;
      DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
   end;
else;
   DSGEModel.InitialStateVector = zeros(length(DSGEModel.StateVariablePositions),1);
end;
%
% Get the sub-sample
%
[DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
%
% try to solve the model for the posterior mode estimates
%
ErrorStr = '';
if MatlabNumber>=8.0;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch ME;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
   end;
else;
   try;
      [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
      if isempty(ErrMsg)==0;
         ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
      end;
   catch;
      ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
   end;
end;
if isempty(ErrorStr)==0;
   About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
   return;
end;
if mcode~=1;
   if mcode==2;
      mcodeStr = 'Roots are not correctly computed by real_schur.'
   elseif mcode==3;
      mcodeStr = 'Too many big roots.';
   elseif mcode==35;
      mcodeStr = 'Too many big roots, and q(:,right) is singular.';
   elseif mcode==4;
      mcodeStr = 'Too few big roots.';
   elseif mcode==45;
      mcodeStr = 'Too few big roots, and q(:,right) is singular.';
   elseif mcode==5;
      mcodeStr = 'q(:,right) is singular.';
   elseif mcode==61;
      mcodeStr = 'Too many exact shiftrights.';
   elseif mcode==62;
      mcodeStr = 'Too many numeric shiftrights.';
   elseif mcode==7;
      mcodeStr = 'Infinite or NaN values detected.';
   elseif mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   if DSGEModel.ModelSolver==1;
      SolverStr = 'AiM';
   elseif DSGEModel.ModelSolver==2;
      SolverStr = 'Klein';
   elseif DSGEModel.ModelSolver==3;
      SolverStr = 'Gensys';
   elseif DSGEModel.ModelSolver==4;
      SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
   end;
   txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - ' SolverStr ' Solver Error'],500,600,CurrINI);
   return;
end;
if length(size(H))==3;
   if size(H,3)<DSGEModel.LastPeriod+h;
      %
      % too few data points in the 3D H matrix.
      %
      ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations plus forecasts (T+h = ' int2str(DSGEModel.LastPeriod+h) '). YADA has therefore aborted from the unconditional predictions.'];
      About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
      return;
   end;
end;
[S,KeepShocks] = RemoveRedundantColumns(B0);
if isempty(S)==1;
   txt = 'The model doesn''t have any structural shocks. YADA has aborted from the lower bound computations.';
   About(txt,'information',['ZLB Predictions - ' NameStr],200,600,CurrINI);
   return;
end;
[r,q] = size(B0);
%
% test if the measurement equation for the policy rate maps to the correct state variables only
%
if length(size(H))==2;
   ZLBLink = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   NonZLBLink = H(setdiff((1:r),DSGEModel.RtildePosition),DSGEModel.ZLBPosition);
   if r>1;
      TestValue = isequal(NonZLBLink,zeros(r-1,1));
   else;
      TestValue = isempty(NonZLBLink);
   end;
   txt = '';
   BadValue = 0;
   if ZLBLink==0;
      txt = ['The observed monetary policy interest rate (' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ') doesn''t map to the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. YADA is therefore unable to use the lower bound and the steady-state interest rate and will abort.'];
      BadValue = 1;
   elseif TestValue==0;
      txt = ['The measurement equation for ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ' maps to other state variables than the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. Are you sure you want to use this mapping?'];
   end;
   if isempty(txt)==0;
      if BadValue==1;
         About(txt,'information',['ZLB Predictions - ' NameStr],300,600,CurrINI);
         return;
      else;
         answer = Query(txt,'question',300,['ZLB Predictions - ' NameStr],600,'no',CurrINI);
         if strcmp(lower(answer),'no')==1;
            return;
         end;
      end;
   end;
else;
   Hlength = size(H,3);
   ZLBLink = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2);
   ZLBLTest = min(ZLBLink~=zeros(Hlength-DSGEModel.LastPeriod+1,1));
   NonZLBLink = shiftdim(H(setdiff((1:r),DSGEModel.RtildePosition),DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2);
   if r>1;
      TestValue = isequal(NonZLBLink,zeros(Hlength-DSGEModel.LastPeriod+1,r-1))
   else;
      TestValue = isempty(NonZLBLink);
   end;
   txt = '';
   BadValue = 0;
   if ZLBLink==0;
      txt = ['The observed monetary policy interest rate (' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ') doesn''t map to the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model. YADA is therefore unable to use the lower bound and the steady-state interest rate and will abort.'];
      BadValue = 1;
   elseif TestValue==0;
      txt = ['The measurement equation for ' StringTrim(DSGEModel.VariableNames(DSGEModel.ZLBPosition,:)) ' maps to other state variables than the claimed unrestricted monetary policy rate (' StringTrim(DSGEModel.StateVariableNames(DSGEModel.RtildePosition,:)) ') in the DSGE model for some time periods. Are you sure you want to use this mapping?'];
   end;
   if isempty(txt)==0;
      if BadValue==1;
         About(txt,'information',['ZLB Predictions - ' NameStr],300,600,CurrIN);
         return;
      else;
         answer = Query(txt,'question',300,['ZLB Predictions - ' NameStr],600,'no',CurrINI);
         if strcmp(lower(answer),'no')==1;
            return;
         end;
      end;
   end;
end;
%
% determine the steady-state interest rate path.
%
Xinput = X;
if k>0;
   Ax = A'*X;
   XPred = X;
   if size(DSGEModel.X,2)>DSGEModel.LastPeriod;
      X = DSGEModel.X(:,DSGEModel.LastPeriod+1:size(DSGEModel.X,2));
      LastValue = X(:,size(X,2));
   else;
      X = [];
      LastValue = DSGEModel.X(:,DSGEModel.LastPeriod);
   end;
   if isempty(DSGEModel.PredictedX)==0;
      X = [X DSGEModel.PredictedX];
   end;
   Tx = size(X,2);
   Xold = X;
   if Tx>DSGEModel.ZLBSampleT+1;
      X = X(:,1:DSGEModel.ZLBSampleT+1);
   elseif Tx<DSGEModel.ZLBSampleT+1;
      X = [X LastValue*ones(1,DSGEModel.ZLBSampleT+1-Tx)];
   end;
   Rbar = A(:,DSGEModel.ZLBPosition)'*X;
else;
   Ax = zeros(n,h);
   Rbar = zeros(1,DSGEModel.ZLBSampleT+1);
end;
%
% setup the zero lower bound path
%
Tzlb = length(DSGEModel.ZLBData);
if Tzlb==1;
   Rzlb = DSGEModel.ZLBData*ones(h,DSGEModel.ZLBSampleT+1);
else;
   if Tzlb>DSGEModel.LastPeriod;
      Rzlb = zeros(h,Tzlb-DSGEModel.LastPeriod);
      for i=1:h;
         Rzlb(i,1:Tzlb-DSGEModel.LastPeriod-i+2) = DSGEModel.ZLBData(DSGEModel.LastPeriod+i-1:Tzlb);
      end;
      TZLB = size(Rzlb,2);
      if TZLB>=DSGEModel.ZLBSampleT+1;
         RzlbLast = Rzlb(1,size(Rzlb,2));
         % make the matrix have the correct dimension
         Rzlb = Rzlb(:,1:DSGEModel.ZLBSampleT+1);
         % replace zeros with the last value of the lower bound
         % provided the locations appear after the last nonzero
         for i=1:h;
            for j=2:DSGEModel.ZLBSampleT+1;
               if (Rzlb(i,j)==0)&(Rzlb(i,j-1)~=0);
                  Rzlb(i,j) = RzlbLast;
               end;
            end;
         end;
         %Rzlb((Rzlb==0)) = RzlbLast;
      else;
         %
         % we should add the last value in each row to fill the matrix
         %
         RzlbLast = Rzlb(1,size(Rzlb,2));
         Rzlb((Rzlb==0)) = RzlbLast;
         Rzlb = [Rzlb RzlbLast*ones(h,DSGEModel.ZLBSampleT+1-TZLB)];
      end;
   else;
      Rzlb = DSGEModel.ZLBData(Tzlb)*ones(h,DSGEModel.ZLBSampleT+1);
   end;
end;
%
% since H may involve estimated parameters we save a copy of Rzlb for later use
%
RzlbMat = Rzlb;
%
% Transform the steady-state interest rate and the lower bound to the same units
% as the unrestricted monetary policy rate
%
if length(size(H))==2;
   Hvalue = H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition);
   Rzlb = (1/Hvalue)*Rzlb;
   Rbar = (1/Hvalue)*Rbar;
else;
   Hvalue = shiftdim(H(DSGEModel.RtildePosition,DSGEModel.ZLBPosition,DSGEModel.LastPeriod:Hlength),2)';
   Th = length(Hvalue);
   if Th>DSGEModel.ZLBSampleT;
      Hvalue = Hvalue(1,1:DSGEModel.ZLBSampleT);
   elseif Th<DSGEModel.ZLBSampleT;
      Hvalue = [Hvalue Hvalue(1,Th)*ones(1,DSGEModel.ZLBSampleT-Th)];
   end;
   Rzlb = Rzlb./Hvalue;
   Rbar = Rbar./Hvalue;
end;
%
% We can now run the Kalman filter
%
if MatlabNumber>=8.0;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   catch ME;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Filter',200,600,CurrINI);
      return;
   end;
else;
   try;
      [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter(DSGEModel.Y,DSGEModel.X,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
   catch;
      txt = ['YADA caught the following error message from the Kalman filter routine: ' YADALastError];
      About(txt,'error','Error - Kalman Filter',200,600,CurrINI);
      return;
   end;
end;
if stat~=0;
   if stat==1;
      txt = 'The state variables are not stationary at the initial values. Issue recorded while running the Kalman filter on the linear DSGE model.';
   else;
      txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab. Issue recorded while running the Kalman filter on the linear DSGE model.';
   end;
   About(txt,'information',['ZLB Predictions - ' NameStr],200,600,CurrINI);
   return;
end;
%
% check if there are numerical issues with diffuse initialization of the Kalman filter
%
if DSGEModel.UseDoublingAlgorithm==4;
   txt = '';
   if KalmanGain.d==KalmanGain.T;
      txt = 'The diffuse initialization of the Kalman filter didn''t converge within the selected sample. This may be due to non-stationarity or near non-stationarity of some state variable which isn''t observed. YADA therefore recommends that you choose a different initialization method.';
   elseif (KalmanGain.d<KalmanGain.T)&(max(diff(sum(SigmaSqRoot,1)))>0);
      txt = ['Diffuse initialization of the Kalman filter converged within the selected sample in period ' int2str(KalmanGain.d) ' relative to period ' int2str(KalmanGain.T) ', but the rank of the Sigma_Inf matrix has increased over the initialization sample. This indicates a numerical issue with the procedure and it may be due to non-stationarity or near non-stationarity of a state variable which is not observed. YADA therefore recommends that you choose a different initialization method.'];
   end;
   if isempty(txt)==0;
      answer = Query(strvcat(txt,' ','Are you sure you want to continue?'),'question',300,['ZLB Predictions - ' NameStr],600,'no',CurrINI);
      if strcmp(lower(answer),'no')==1;
         return;
      end;
   end;
end;
%
% get some parameter values
%
T = size(Ksitt1,2);
g = size(Z,2);
%
% create the mapping matrices for conditioning variables
%
K1 = DSGEModel.K1(:,DSGEModel.ConditionalVariables);
if length(size(DSGEModel.K2))==2;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables);
   K2length = 2;
else;
   K2 = DSGEModel.K2(:,DSGEModel.ConditionalVariables,:);
   K2length = 3;
end;
SCond = (isequal(U,zeros(size(U)))&isequal(K2,zeros(size(K2))));
%
% run the Kalman smoother for g periods
%
if MatlabNumber>=8.0;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch ME;
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError(ME)];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
else;
   try;
      [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
   catch;
      txt = ['YADA caught the following error message from the Kalman smoother: ' YADALastError];
      About(txt,'error','Error - Kalman Smoother',200,500,CurrINI);
      return;
   end;
end;
PredData.KsitT = KsitT;
%
% calculate measurement errors for the sample T-g+1 until T
%
if (DSGEModel.KsiUseCondData==1)&(SCond==1);
   if (k>0);
      XPred = Xinput(:,1:g);
   else;
      XPred = [];
   end;
   if length(size(H))==2;
      [KsiTTz,PTTz] = CondPredictionSmoother(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,T),PtT(:,:,T),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H,R,F,B0,DSGEModel.KalmanAlgorithm);
   else;
      [KsiTTz,PTTz] = CondPredictionSmootherHt(Ksitt1(:,T),Ptt1(:,:,T),KsitT(:,T),PtT(:,:,T),Yhat(:,T),DSGEModel.Y(:,DSGEModel.LastPeriod),XPred,Z,K1,A,H(:,:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+g),R,F,B0,DSGEModel.KalmanAlgorithm);
   end;
elseif (DSGEModel.KsiUseCondData==1)&(size(KsitT,2)>=g);
   PtT = PtT(:,:,size(PtT,3)-g+1:size(PtT,3));
   KsitT = KsitT(:,size(KsitT,2)-g+1:size(KsitT,2));
   if length(size(H))==2;
      if rank(R)>0;
         if k>0;
            wtT = DSGEModel.Y(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod)-(A'*DSGEModel.X(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod))-(H'*KsitT);
         else;
            wtT = DSGEModel.Y(:,DSGEModel.LastPeriod-g+1:DSGEModel.LastPeriod)-(H'*KsitT);
         end;
      else;
         wtT = zeros(n,g);
      end;
   else;
      wtT = zeros(n,g);
      if rank(R)>0;
         if k>0;
            for i=1:g;
               wtT(:,i) = DSGEModel.Y(:,DSGEModel.LastPeriod-g+i)-(A'*DSGEModel.X(:,DSGEModel.LastPeriod-g+i))-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
            end;
         else;
            for i=1:g;
               wtT(:,i) = DSGEModel.Y(:,DSGEModel.LastPeriod-g+i)-(H(:,:,DSGEModel.LastPeriod-g+i)'*KsitT(:,i));
            end;
         end;
      end;
   end;
   %
   % Smooth estimation of the states at time T using the conditioning assumptions.
   %
   if length(size(H))==2;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[Xtra Xinput(:,1:g)],K1,K2,A,H,R,F,B0);
      else;
         [KsiTTz,PTTz] = CondPredictionKalmanSmoother(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H,R,F,B0);
      end;
   else;
      if k>0;
         if g>1;
            Xtra = DSGEModel.X(:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod);
         else;
            Xtra = [];
         end;
         [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[Xtra Xinput(:,1:g)],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
      else;
         [KsiTTz,PTTz] = CondPredictionKalmanSmootherHt(KsitT,PtT,wtT,Z,U,[],K1,K2,A,H(:,:,DSGEModel.LastPeriod-g+2:DSGEModel.LastPeriod+g),R,F,B0);
      end;
   end;
else;
   KsiTTz = KsitT(:,size(KsitT,2));
   PTTz = PtT(:,:,size(KsitT,2));
end;
PTTz = diag(abs(diag(PTTz)))+tril(PTTz,-1)+tril(PTTz,-1)';
%
% Now we need to determine KsiTT for just the last period
%
KsiTT = KsitT(:,size(KsitT,2));
%
% set up the data matrices
%
Y = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
if isempty(DSGEModel.X)==0;
   Xdata = DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
else;
   Xdata = [];
end;
Yinit = [];
if strcmp(DSGEModel.DataFrequency,'q')==1;
   Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
elseif strcmp(DSGEModel.DataFrequency,'m')==1;
   Yinit = DSGEModel.Y(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
else;
   Yinit = [];
end;
maxYinit = max(max(isnan(Yinit)));
if DSGEModel.RunPredictionEvent==1;
   PredEventData = zeros(n,8);
end;
%
Yinitial = [];
Tt = [];
if TransData==1;
   Yinitial = DSGEModel.Y(:,1:DSGEModel.LastPeriod);
   %
   % try to covert: h
   %
   YTmp = [Yinitial DSGEModel.Y(:,DSGEModel.LastPeriod)*ones(1,h)];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   Tt = size(YData,1);
   maxYinitial = max(max(isnan(Yinitial)));
end;
%
KsiTT = KsitT(:,T);
PTT = PtT(:,:,T);
%
% compute KsitTh
% 1. filter until T+h
%
if length(size(H))==2;
   [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter([Y YData],[Xdata XPred],1,size(Y,2)+h,A,H,F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
else;
   [logLikeValue,stat,lnLt,Yerror,Yhat,SigmaSqRoot,MSEY,InvMSEY,KalmanGain,Ksitt1,Ptt1] = RunKalmanFilter([Y YData],[Xdata XPred],1,size(Y,2)+h,A,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod+h),F,B0,R,DSGEModel.InitialStateVector,DSGEModel.UseDoublingAlgorithm,DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,1,DSGEModel.AllowUnitRoot,DSGEModel.StateCovConstValue,GetPosition(DSGEModel.StateVariablePositions,DSGEModel.UnitRootStates),DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm);
end;
%
% 2. smoother
%
if length(size(H))==2;
   [Ksitth,Ptth,KsitTh,PtTh,rttMath,rTMath] = RunKalmanSmoother([Y YData],1,size(Y,2)+h,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
else;
   [Ksitth,Ptth,KsitTh,PtTh,rttMath,rTMath] = RunKalmanSmoother([Y YData],1,size(Y,2)+h,Yhat,Ksitt1,Ptt1,H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod+h),F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
end;
%
% compute variables for recentering
%
% estimated smooth shocks over the prediction sample
%
etaTTh = B0'*rTMath(:,DSGEModel.LastPeriod+1:DSGEModel.LastPeriod+h);
%etaTTh = ((B0'*B0)\B0')*(KsitTh(:,DSGEModel.LastPeriod+1:DSGEModel.LastPeriod+h)-(F*KsitTh(:,DSGEModel.LastPeriod:DSGEModel.LastPeriod+h-1)));
%
% estimated smooth shocks at T using the data from the prediction sample
%
KsiTTh = KsitTh(:,DSGEModel.LastPeriod);
%
% estimate the measurement errors
%
wTTh = zeros(n,h);
if rank(R)>0;
   if length(size(H))==2;
      wTTh = YData-Ax-(H'*KsitTh(:,DSGEModel.LastPeriod+1:DSGEModel.LastPeriod+h));
   else;
      for t=1:h;
         wTTh(:,t) = YData(:,t)-Ax(:,t)-(H(:,:,DSGEModel.LastPeriod+t)'*KsitTh(:,DSGEModel.LastPeriod+t));
      end;
   end;
end;
wTTh(DSGEModel.ZLBPosition,:) = 0;
WbarTg = vec(fliplr(wTTh(:,1:g)));
%
% Try to solve the model subject to the anticipated shocks
%
[G,C1,P,mcode] = ZLBKleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
%
% notice that the row r+1 of P corresponds to the unrestricted interest rate
% and row DSGEModel.RtildePosition to the actual policy interest rate
%
if mcode~=1;
   if mcode==8;
      mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
   elseif mcode==-1;
      mcodeStr = 'No stable solution.';
   elseif mcode==-2;
      mcodeStr = 'Too many large generalized eigenvalues.';
   elseif mcode==-3;
      mcodeStr = 'Too few large generalized eigenvalues.';
   else;
      mcodeStr = 'Return code not properly specified.';
   end;
   txt = ['The Klein solver with anticipated shocks for handling the zero lower bound provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
   About(txt,'information',['YADA - Klein Solver Error'],200,500,CurrINI);
   return;
end;
%
% setup the parameters that determine the projected restricted policy rate
%
e_pr = zeros(r+1,1);
e_pr(DSGEModel.RtildePosition) = 1;
%
% setup ePG for conmputing the restricted policy rate projections
%
ePG = zeros(DSGEModel.ZLBSampleT+1,size(G,2));
ePG(1,:) = e_pr'*P(1:r+1,:);
for tau=1:DSGEModel.ZLBSampleT;
   ePG(tau+1,:) = ePG(tau,:)*G;
end;
if DSGEModel.ZLBSampleT>1;
   M = [zeros(DSGEModel.ZLBSampleT,1) eye(DSGEModel.ZLBSampleT);zeros(1,DSGEModel.ZLBSampleT+1)];
else;
   M = 0;
end;
%
% create the matrices for selecting shocks
%
m = size(Z,1);
%
% check if shocks can be used
%
if length(size(H))==2;
   if rank(K1'*H'*B0)<m;
      txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
      About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
      PredPaths = [];
      return;
   end;
else;
   BadEtaMat = 0;
   for i=1:g;
      if rank(K1'*H(:,:,DSGEModel.LastPeriod+i)'*B0)<m;
         txt = ['The conditioning assumptions you have selected cannot be used due to a singularity in the solution algorithm. Estimation of the conditional predictive distribution' lower(NameStr) ' has therfore been aborted.'];
         About(txt,'error','Error - Conditional Shocks',150,500,CurrINI);
         PredPaths = [];
         BadEtaMat = 1;
         break;
      end;
   end;
   if BadEtaMat==1;
      return;
   end;
end;
%
% stack the model to determine the shock distribution
%
Xbar = zeros(n*g,1);
G = zeros(n*g,r);
WTg = zeros(n*g,1);
K = zeros(n*g,m*g);
D = zeros(n*g,q*g);
Fexp = F;
ZTg = zeros(m*g,1);
UT = zeros(m*g,1);
if length(size(H))==2;
   for i=1:g;
      Ind = (1+((g-i)*n):(g-i+1)*n);
      Ind1 = (1+((g-i)*q):(g-i+1)*q);
      Ind2 = (1+((g-i)*m):(g-i+1)*m);
      if k>0;
         Xbar(Ind,1) = Ax(:,i);
      end;
      G(Ind,:) = H'*Fexp;
      D(Ind,Ind1) = H'*B0;
      K(Ind,Ind2) = K1;
      Fi = F;
      for j=1:i-1;
         Ind1 = Ind1+q;
         Ind3 = Ind+(j*n);
         D(Ind,Ind1) = H'*Fi*B0;
         if K2length==2;
            K(Ind3,Ind2) = K2;
         else;
            if j<=size(K2,3);
               K(Ind3,Ind2) = K2(:,:,j);
            end;
         end;
         Fi = F*Fi;
      end;
      ZTg(Ind2,1) = Z(:,i);
      UT(Ind2,1) = U(:,g);
      Fexp = F*Fexp;
   end;
else;
   for i=1:g;
      Ind = (1+((g-i)*n):(g-i+1)*n);
      Ind1 = (1+((g-i)*q):(g-i+1)*q);
      Ind2 = (1+((g-i)*m):(g-i+1)*m);
      if k>0;
         Xbar(Ind,1) = Ax(:,i);
      end;
      G(Ind,:) = H(:,:,DSGEModel.LastPeriod+i)'*Fexp;
      D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*B0;
      K(Ind,Ind2) = K1;
      Fi = F;
      for j=1:i-1;
         Ind1 = Ind1+q;
         Ind3 = Ind+(j*n);
         D(Ind,Ind1) = H(:,:,DSGEModel.LastPeriod+i)'*Fi*B0;
         if K2length==2;
            K(Ind3,Ind2) = K2;
         else;
            if j<=size(K2,3);
               K(Ind3,Ind2) = K2(:,:,j);
            end;
         end;
         Fi = F*Fi;
      end;
      ZTg(Ind2,1) = Z(:,i);
      UT(Ind2,1) = U(:,g);
      Fexp = F*Fexp;
   end;
end;
%
% create a vector with positions for the conditioning assumptions that should be used
%
PosVec = (1:m*g)' .* ~isnan(ZTg);
PosVec = PosVec(PosVec>0);
%
% adjust for missing restrictions
%
K = K(:,PosVec);
ZTg = ZTg(PosVec,1);
UT = UT(PosVec,1);
%
% speed-up code suggested by Ingvar Strid
%
KpD = K'*D;
KpDDpK = KpD*KpD';
DpKInvKpDDpK = KpD'/KpDDpK;
%
% check the rank of K'D
%
if rank(KpD)<length(PosVec);
   txt = ['Some of the conditioning assumptions are linearly dependent. The rank of K''*D is ' int2str(rank(KpD)) ', while the number of conditioning assumptions is ' int2str(length(PosVec)) '. YADA is therefore unable to perform the conditional forecasts for the Waggoner-Zha approach.'];
   About(txt,'error','Error - Conditioning Assumptions',200,500,CurrINI);
   PredPaths = [];
   return;
end;
%
% compute the conditioning relation when state vector and measurement errors are at mean
% and the economic shocks satisfy the observed data over the forecast horizon
%
vecNtg = vec(fliplr(etaTTh(:,1:g)));
kbarTg = ZTg-UT-(K'*(Xbar+(G*KsiTTz)));
%
% compute the mean of the shocks when state vector and measurement errors are at mean
%
mubarTg = DpKInvKpDDpK*kbarTg;
%
% compute mutildeTg
%
ktildeTg = ZTg-UT-(KpD*(vecNtg-mubarTg))-(K'*(Xbar+WbarTg+(G*KsiTTh)));
mutildeTg = DpKInvKpDDpK*ktildeTg;
%
% compute the idempotent covariance matrix for the shocks
%
SigmaTg = eye(q*g)-(DpKInvKpDDpK*KpD);
SigmaTg = diag(abs(diag(SigmaTg)))+tril(SigmaTg,-1)+tril(SigmaTg,-1)';
[SE,LE] = schur(SigmaTg);
SigmaTgStd = SE*sqrt(abs(LE));
%
% add info to the PredData structure
%
PredData.KernelX = [];
PredData.KernelY = [];
PredData.X = Xinput;
PredData.A = A;
if length(size(H))==2;
   PredData.H = H;
else;
   PredData.H = H(:,:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod+h);
end;
PredData.R = R;
PredData.B0 = B0;
PredData.F = F;
%
% compute the mean shock values when the ZLB is not binding
%
etaMean = etaTTh;
etaMean(:,1:g) = fliplr(reshape(mutildeTg+vecNtg-mubarTg,q,g));
%
% compute the mean predictions
%
ZLBKsiMean = zeros(r,h);
ZLBYMean = zeros(n,h);
KsiTMean = KsiTTh;
ZPos = ~isnan(Z);
%
At = zeros(DSGEModel.ZLBSampleT+1,1);
Xt = [etaMean(:,1);KsiTMean;KsiTMean(DSGEModel.RtildePosition)];
for t=1:h;
   [At,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
   if status==1;
      Ksit = P(1:r,:)*[Xt;At];
      rhat = P(r+1,:)*[Xt;At];
      ZLBKsiMean(:,t) = Ksit;
      if length(size(H))==2;
         ZLBYMean(:,t) = Ax(:,t)+(H'*Ksit)+wTTh(:,t);
      else;
         ZLBYMean(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit)+wTTh(:,t);
      end;
      if t+1<=h;
         At = M*At;
         Xt = [etaMean(:,t+1);Ksit;rhat];
      end;
   else;
      break;
   end;
end;
%
% the actual data; the model projection conditional on actual data; the conditional forecast; the conditional forecast using the model projection correction
%
%disp(YData)
%disp(ZLBYMean)
%disp([vec(fliplr(YData(:,1:g))) Xbar+(G*KsiTTh)+(D*vecNtg) Xbar+(G*KsiTTz)+(D*mubarTg) Xbar+(G*KsiTTh)+(D*(mutildeTg+vecNtg-mubarTg))])
%disp([ZTg K'*(Xbar+(G*KsiTTh)+(D*vecNtg))+UT])
%
% check if the mean shocks and state failed to find the anticipated shocks.
%
if status==0;
   if AlgorithmInformation(1)==1;
      AddTxt = 'YADA has recorded an invertibility problem when attempting to solve for the anticipated shocks.';
   elseif AlgorithmInformation(2)==1;
      AddTxt = 'YADA has recorded that a solution cannot be found, i.e., Step 3 (ii) holds (see the YADA Manual).';
   elseif AlgorithmInformation(3)==1;
      AddTxt = 'YADA has recorded that the anticipated shocks are nonnegative, that the restricted policy rate satisfies the zero lower bound, but the complementary slackness condition is violated.';
   else;
      AddTxt = ['YADA has recorded that the maximum number of attempts (' int2str(AlgorithmInformation(5)) ') to find the anticipated shocks was reached.'];
   end;
   txt = ['The forward-back shooting algorithm failed to solve for the anticipated shocks for period ' int2str(t) ' using the population means of the structural shocks and the state variables in the last historical period. ' AddTxt ' Are you sure you want to continue?'];
   answer = Query(txt,'question',300,['ZLB Predictions - ' NameStr],600,'no',CurrINI);
   if strcmp(lower(answer),'no')==1;
      return;
   end;
end;
%
% compute a decomposition of PTT in order to draw random values of
% the state at T, the end of the historical data
%
[S,L] = SchurDecomposition(PTT);
if rank(R)>0;
   R(:,DSGEModel.ZLBPosition) = 0;
   R(DSGEModel.ZLBPosition,:) = 0;
   [Rm,Lm] = SchurDecomposition(R);
   Rmeasure = Rm*sqrt(Lm);
   NumMeasure = size(Lm,1);
else;
   Rmeasure = [];
   wTh = zeros(n,h);
   NumMeasure = 0;
end;
%
% check if there are missing observations and if annualized data or data transformations should
% be applied to PredictY
%
if AnnualizeData==1;
   if maxYinit==1;
      %
      % take into account that there are missing obs for the
      % historical data which can affect annualization
      %
      YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
      nanpos = isnan(YSmooth);
      [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      if length(size(H))==2;
         if k>0;
            YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*KsitT);
         else;
            YHistPred = H'*KsitT;
         end;
      else;
         if k>0;
            YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
            for t=1:size(KsitT,2);
               YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
            end;
         else;
            YHistPred = zeros(n,size(KsitT,2));
            for t=1:size(KsitT,2);
               YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t);
            end;
         end;
      end;
      YSmooth(nanpos) = YHistPred(nanpos);
      if DSGEModel.FirstPeriod>1;
         YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod-1) YSmooth];
      end;
      if strcmp(DSGEModel.DataFrequency,'q')==1;
         Yinit = YSmooth(:,DSGEModel.LastPeriod-2:DSGEModel.LastPeriod);
      elseif strcmp(DSGEModel.DataFrequency,'m')==1;
         Yinit = YSmooth(:,DSGEModel.LastPeriod-10:DSGEModel.LastPeriod);
      else;
         Yinit = [];
      end;
   end;
   ZLBYMean = Annualize([Yinit ZLBYMean],DSGEModel.annual,DSGEModel.annualscale);
elseif TransData==1;
   if maxYinitial==1;
      YSmooth = DSGEModel.Y(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
      nanpos = isnan(YSmooth);
      [Ksitt,Ptt,KsitT,PtT,rttMat] = RunKalmanSmoother(DSGEModel.Y,DSGEModel.FirstPeriod,DSGEModel.LastPeriod,Yhat,Ksitt1,Ptt1,H,F,R,B0,Yerror,SigmaSqRoot,InvMSEY,KalmanGain,DSGEModel.YNaNs,DSGEModel.KalmanAlgorithm,DSGEModel.UseDoublingAlgorithm==4);
      if length(size(H))==2;
         if k>0;
            YHistPred = (A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod))+(H'*KsitT);
         else;
            YHistPred = H'*KsitT;
         end;
      else;
         if k>0;
            YHistPred = A'*DSGEModel.X(:,DSGEModel.FirstPeriod:DSGEModel.LastPeriod);
            for t=1:size(KsitT,2);
               YHistPred(:,t) = YHistPred(:,t)+(H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t));
            end;
         else;
            YHistPred = zeros(n,size(KsitT,2));
            for t=1:size(KsitT,2);
               YHistPred(:,t) = H(:,:,DSGEModel.FirstPeriod+t-1)'*KsitT(:,t);
            end;
         end;
      end;
      YSmooth(nanpos) = YHistPred(nanpos);
      if DSGEModel.FirstPeriod>1;
         YSmooth = [DSGEModel.Y(:,1:DSGEModel.FirstPeriod-1) YSmooth];
      end;
      Yinitial = YSmooth(:,1:DSGEModel.LastPeriod);
   end;
   YTmp = [Yinitial ZLBYMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   ZLBYMean = YData(Tt-h+1:Tt,:)';
end;
%
% check if we should run parallel computations
%
if CurrINI.NumWorkers<2;
   %
   % check if we should setup a wait handle
   %
   if DSGEModel.ShowProgress==1;
      ProgressStructure.title = ['Progress for ' int2str(NumPaths) ' Prediction Paths'];
      ProgressStructure.facecolor = CurrINI.progress_facecolor;
      ProgressStructure.startfacecolor = CurrINI.progress_startfacecolor;
      ProgressStructure.edgecolor = CurrINI.progress_edgecolor;
      ProgressStructure.bgcolor = CurrINI.progress_bgcolor;
      ProgressStructure.stop = 0;
      ProgressStructure.clock = DSGEModel.ShowProgressClock;
      ProgressStructure.label = 'Path Number:';
      %
      ProgressStructure.name = ['ZLB Cond. Predictive Distributions - ' TypeStr ' Data'];
      ProgressStructure.CurrINI = CurrINI;
      WaitHandle = ProgressDLG(0,ProgressStructure);
      set(WaitHandle,'Color',get(CurrINI.GraphicsRoot,'defaultuicontrolbackgroundcolor'));
      drawnow;
   elseif DSGEModel.RunDriver==0;
      txt = ['Please wait while YADA computes predictive distributions subject to the zero (effective) lower bound for the ' lower(TypeStr) ' data. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['ZLB Cond. Predictive Distributions - ' TypeStr ' Data'],600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % start the simulations
   %
   abort = '';
   StopEstimation = 0;
   ZLBYPaths = zeros(n,h,NumPaths);
   AtMean = zeros(DSGEModel.ZLBSampleT+1,h);
   XtMean = zeros(q+r+1,h);
   %
   NumBadValues = 0;
   %NumPaths = 1;
   if length(size(H))==2;
      for it=1:NumPaths;
         %
         % check if we should update the progress dialog
         %
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
               ZLBYPaths = [];
               ZLBKsiPaths = [];
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([it/NumPaths it]);
            end;
         end;
         %
         % loop over all g horizons
         %
         RungLoop = 1;
         while RungLoop==1;
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               wTh(DSGEModel.ZLBPosition,:) = 0;
               WTg = vec(fliplr(wTh(:,1:g)));
            end;
            %
            % draw initial state
            %
            KsiT = KsiTTh+(S*sqrt(L)*randn(size(L,2),1));
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(KpD*(vecNtg-mubarTg))-(K'*(Xbar+(G*KsiT)+WTg+WbarTg));
            muTg = DpKInvKpDDpK*kTg;
            %
            % initialize
            %
            PredictY = zeros(n,h);
            etaTh = zeros(q,h);
            K2sum = zeros(m,1);
            At = zeros(DSGEModel.ZLBSampleT+1,1);
            Xt = [zeros(q,1);KsiT;KsiT(DSGEModel.RtildePosition)];
            %
            % draw shocks
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw)+vecNtg-mubarTg;
            %
            % check
            %
            CheckCond = ZTg-(K'*(Xbar+(G*KsiT)+(D*etaTg)+WTg+WbarTg))-UT;
            if max(abs(CheckCond))<=1.0e-6;
               t = 1;
               while t<=g;
                  %
                  % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
                  %
                  Ind1 = (1+((g-t)*q):(g-t+1)*q);
                  etaTh(:,t) = etaTg(Ind1);
                  ZPosi = ZPos(:,t);
                  m1 = sum(ZPosi);
                  %
                  % add shocks to Xt
                  %
                  Xt(1:q) = etaTh(:,t);
                  [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                  if status==1;
                     %
                     % compute the state vector at forecast horizon t
                     %
                     Ksit = P(1:r,:)*[Xt;AtNew];
                     %
                     % check if we have non-zero anticipated shocks
                     %
                     if (sum(AtNew)>0)&(m1>0);
                        %
                        % check if the conditioning assumptions are satisfied
                        %
                        CA = Z(ZPosi,t) - (K1(:,ZPosi)'*Ax(:,t)) - (K1(:,ZPosi)'*(wTh(:,t)+wTTh(:,t))) - (K1(:,ZPosi)'*H'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        %
                        % check if we should recompute the 
                        %
                        if max(abs(CA))>=1.0e-6;
                           NumBadValues = NumBadValues+1;
                           status = 0;
                        end;
                     end;
                     if status==1;
                        At = AtNew;
                        AtMean(:,t) = AtMean(:,t)+((1/NumPaths)*At);
                        XtMean(:,t) = XtMean(:,t) + ((1/NumPaths)*Xt);
                        rhat = P(r+1,:)*[Xt;At];
                        ZLBKsiPaths(:,t,it) = Ksit;
                        KsiT = Ksit;
                        PredictY(:,t) = Ax(:,t)+(H'*Ksit)+wTh(:,t)+wTTh(:,t);
                        if t+1<=h;
                           At = M*At;
                           Xt = [zeros(q,1);Ksit;rhat];
                        end;
                        %
                        % update the K2sum measure
                        %
                        if K2length==2;
                           K2sum = K2sum+(K2'*PredictY(:,t));
                        else;
                           K2sum = zeros(m,1);
                           for j=1:t;
                              if j<=size(K2,3);
                                 K2sum = K2sum+(K2(:,:,j)'*PredictY(:,t+1-j));
                              end;
                           end;
                        end;
                        t = t+1;
                        if t==g+1;
                           RungLoop = 0;
                        end;
                     else;
                        t = g+1;
                        break
                     end;
                  else;
                     t = g+1;
                     break;
                  end;
               end;
            else;
               NumBadValues = NumBadValues+1;
            end;
         end;
         %
         % deal with the remaining horizons
         %
         while t<=h;
            etaTh(:,t) = etaTTh(:,t)+randn(q,1);
            Xt(1:q) = etaTh(:,t);
            [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
            if status==1;
               At = AtNew;
               AtMean(:,t) = AtMean(:,t)+((1/NumPaths)*At);
               XtMean(:,t) = XtMean(:,t) + ((1/NumPaths)*Xt);
               Ksit = P(1:r,:)*[Xt;At];
               rhat = P(r+1,:)*[Xt;At];
               ZLBKsiPaths(:,t,it) = Ksit;
               KsiT = Ksit;
               PredictY(:,t) = Ax(:,t)+(H'*Ksit)+wTh(:,t)+wTTh(:,t);
               if t+1<=h;
                  At = M*At;
                  Xt = [zeros(q,1);Ksit;rhat];
               end;
               t = t+1;
            end;
         end;
         %
         % check if we should annualize or transform
         %
         if AnnualizeData==1;
            PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
         elseif TransData==1;
            YTmp = [Yinitial PredictY];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredictY = YData(Tt-h+1:Tt,:)';
         end;
         %
         % add the projected path to the variable to be saved
         %
         ZLBYPaths(:,:,it) = PredictY;
         drawnow;
      end;
   else;
      for it=1:NumPaths;
         %
         % check if we should update the progress dialog
         %
         if DSGEModel.ShowProgress==1;
            abort = get(WaitHandle,'UserData');
            if (strcmp(abort,'cancel')==1)|(strcmp(abort,'done')==1);
               ZLBYPaths = [];
               ZLBKsiPaths = [];
               StopEstimation = 1;
               break;
            else;
               ProgressDLG([it/NumPaths it]);
            end;
         end;
         %
         % loop over all g horizons
         %
         RungLoop = 1;
         while RungLoop==1;
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               wTh(DSGEModel.ZLBPosition,:) = 0;
               WTg = vec(fliplr(wTh(:,1:g)));
            end;
            %
            % draw initial state
            %
            KsiT = KsiTTh+(S*sqrt(L)*randn(size(L,2),1));
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(KpD*(vecNtg-mubarTg))-(K'*(Xbar+(G*KsiT)+WTg+WbarTg));
            muTg = DpKInvKpDDpK*kTg;
            %
            % initialize
            %
            PredictY = zeros(n,h);
            etaTh = zeros(q,h);
            K2sum = zeros(m,1);
            At = zeros(DSGEModel.ZLBSampleT+1,1);
            Xt = [zeros(q,1);KsiT;KsiT(DSGEModel.RtildePosition)];
            %
            % draw shocks
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw)+vecNtg-mubarTg;
            %
            % check
            %
            CheckCond = ZTg-(K'*(Xbar+(G*KsiT)+(D*etaTg)+WTg+WbarTg))-UT;
            if max(abs(CheckCond))<=1.0e-6;
               t = 1;
               while t<=g;
                  %
                  % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
                  %
                  Ind1 = (1+((g-t)*q):(g-t+1)*q);
                  etaTh(:,t) = etaTg(Ind1);
                  ZPosi = ZPos(:,t);
                  m1 = sum(ZPosi);
                  %
                  % add shocks to Xt
                  %
                  Xt(1:q) = etaTh(:,t);
                  [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                  if status==1;
                     %
                     % compute the state vector at forecast horizon t
                     %
                     Ksit = P(1:r,:)*[Xt;AtNew];
                     %
                     % check if we have non-zero anticipated shocks
                     %
                     if (sum(AtNew)>0)&(m1>0);
                        %
                        % check if the conditioning assumptions are satisfied
                        %
                        CA = Z(ZPosi,t) - (K1(:,ZPosi)'*Ax(:,t)) - (K1(:,ZPosi)'*(wTh(:,t)+wTTh(:,t))) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+t)'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        %
                        % check if we should recompute the 
                        %
                        if max(abs(CA))>=1.0e-6;
                           NumBadValues = NumBadValues+1;
                           status = 0;
                        end;
                     end;
                     if status==1;
                        At = AtNew;
                        AtMean(:,t) = AtMean(:,t)+((1/NumPaths)*At);
                        XtMean(:,t) = XtMean(:,t) + ((1/NumPaths)*Xt);
                        rhat = P(r+1,:)*[Xt;At];
                        ZLBKsiPaths(:,t,it) = Ksit;
                        KsiT = Ksit;
                        PredictY(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit)+wTh(:,t)+wTTh(:,t);
                        if t+1<=h;
                           At = M*At;
                           Xt = [zeros(q,1);Ksit;rhat];
                        end;
                        %
                        % update the K2sum measure
                        %
                        if K2length==2;
                           K2sum = K2sum+(K2'*PredictY(:,t));
                        else;
                           K2sum = zeros(m,1);
                           for j=1:t;
                              if j<=size(K2,3);
                                 K2sum = K2sum+(K2(:,:,j)'*PredictY(:,t+1-j));
                              end;
                           end;
                        end;
                        t = t+1;
                        if t==g+1;
                           RungLoop = 0;
                        end;
                     else;
                        t = g+1;
                        break
                     end;
                  else;
                     t = g+1;
                     break;
                  end;
               end;
            else;
               NumBadValues = NumBadValues+1;
            end;
         end;
         %
         % deal with the remaining horizons
         %
         while t<=h;
            etaTh(:,t) = etaTTh(:,t)+randn(q,1);
            Xt(1:q) = etaTh(:,t);
            [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
            if status==1;
               At = AtNew;
               AtMean(:,t) = AtMean(:,t)+((1/NumPaths)*At);
               XtMean(:,t) = XtMean(:,t) + ((1/NumPaths)*Xt);
               Ksit = P(1:r,:)*[Xt;At];
               rhat = P(r+1,:)*[Xt;At];
               ZLBKsiPaths(:,t,it) = Ksit;
               KsiT = Ksit;
               PredictY(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit)+wTh(:,t)+wTTh(:,t);
               if t+1<=h;
                  At = M*At;
                  Xt = [zeros(q,1);Ksit;rhat];
               end;
               t = t+1;
            end;
         end;
         %
         % check if we should annualize or transform
         %
         if AnnualizeData==1;
            PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
         elseif TransData==1;
            YTmp = [Yinitial PredictY];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredictY = YData(Tt-h+1:Tt,:)';
         end;
         %
         % add the projected path to the variable to be saved
         %
         ZLBYPaths(:,:,it) = PredictY;
         drawnow;
      end;
   end;
   %
   % close any wait dialog
   %
   if DSGEModel.ShowProgress==1;
      set(WaitHandle,'UserData','done');
      close(WaitHandle);
      drawnow;
      pause(0.02);
   elseif DSGEModel.RunDriver==0;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
else;
   %
   % we can do parallel computations over the parameter draws
   %
   if DSGEModel.RunDriver==0;
      txt = ['Please wait while YADA computes predictive distributions for the ' lower(TypeStr) ' data using parallel computations with ' int2str(CurrINI.NumWorkers) ' workers. Computations started at: ' StringTrim(datestr(now,14))];
      WaitHandle = WaitDLG(txt,'information',['Predictive Distribution' NameStr],600,150,CurrINI,0);
      WaitControls = get(WaitHandle,'UserData');
   end;
   %
   % initialize
   %
   abort = '';
   StopEstimation = 0;
   ZLBYPaths = zeros(n,h,NumPaths);
   ZLBYPaths = zeros(n,h,NumPaths);
   AtPaths = zeros(DSGEModel.ZLBSampleT+1,h,NumPaths);
   XtPaths = zeros(q+r+1,h,NumPaths);
   %
   NumBadValues = 0;
   NumBindingValues = 0;
   %
   if DSGEModel.RandomNumberValue==1;
      %
      % without eval statements the spmd-code gives an error in older matlab systems
      %
      eval('spmd; rng(0,''combRecursive''); end;');
   else;
      eval('spmd; rng(''default''); end;');
   end;
   %
   if length(size(H))==2;
      parfor it=1:NumPaths;
         %
         % loop over all g horizons
         %
         RungLoop = 1;
         while RungLoop==1;
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               wTh(DSGEModel.ZLBPosition,:) = 0;
               WTg = vec(fliplr(wTh(:,1:g)));
            else;
               wTh = zeros(n,h);
               WTg = 0;
            end;
            %
            % draw initial state
            %
            KsiT = KsiTTh+(S*sqrt(L)*randn(size(L,2),1));
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(KpD*(vecNtg-mubarTg))-(K'*(Xbar+(G*KsiT)+WTg+WbarTg));
            muTg = DpKInvKpDDpK*kTg;
            %
            % initialize
            %
            PredictY = zeros(n,h);
            etaTh = zeros(q,h);
            K2sum = zeros(m,1);
            At = zeros(DSGEModel.ZLBSampleT+1,1);
            Xt = [zeros(q,1);KsiT;KsiT(DSGEModel.RtildePosition)];
            %
            % draw shocks
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw)+vecNtg-mubarTg;
            %
            % check
            %
            CheckCond = ZTg-(K'*(Xbar+(G*KsiT)+(D*etaTg)+WTg+WbarTg))-UT;
            if max(abs(CheckCond))<=1.0e-6;
               t = 1;
               AtPath = zeros(DSGEModel.ZLBSampleT+1,h);
               XtPath = zeros(q+r+1,h);
               ZLBKsiPath = zeros(r,h);
               while t<=g;
                  %
                  % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
                  %
                  Ind1 = (1+((g-t)*q):(g-t+1)*q);
                  etaTh(:,t) = etaTg(Ind1);
                  ZPosi = ZPos(:,t);
                  m1 = sum(ZPosi);
                  %
                  % add shocks to Xt
                  %
                  Xt(1:q) = etaTh(:,t);
                  [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                  if status==1;
                     %
                     % compute the state vector at forecast horizon t
                     %
                     Ksit = P(1:r,:)*[Xt;AtNew];
                     %
                     % check if we have non-zero anticipated shocks
                     %
                     if (sum(AtNew)>0)&(m1>0);
                        %
                        % check if the conditioning assumptions are satisfied
                        %
                        CA = Z(ZPosi,t) - (K1(:,ZPosi)'*Ax(:,t)) - (K1(:,ZPosi)'*(wTh(:,t)+wTTh(:,t))) - (K1(:,ZPosi)'*H'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        %
                        % check if we should recompute the 
                        %
                        if max(abs(CA))>=1.0e-6;
                           NumBadValues = NumBadValues+1;
                           status = 0;
                        end;
                     end;
                     if status==1;
                        At = AtNew;
                        AtPath(:,t) = At;
                        XtPath(:,t) = Xt;
                        rhat = P(r+1,:)*[Xt;At];
                        ZLBKsiPath(:,t) = Ksit;
                        KsiT = Ksit;
                        PredictY(:,t) = Ax(:,t)+(H'*Ksit)+wTh(:,t)+wTTh(:,t);
                        if t+1<=h;
                           At = M*At;
                           Xt = [zeros(q,1);Ksit;rhat];
                        end;
                        %
                        % update the K2sum measure
                        %
                        if K2length==2;
                           K2sum = K2sum+(K2'*PredictY(:,t));
                        else;
                           K2sum = zeros(m,1);
                           for j=1:t;
                              if j<=size(K2,3);
                                 K2sum = K2sum+(K2(:,:,j)'*PredictY(:,t+1-j));
                              end;
                           end;
                        end;
                        t = t+1;
                        if t==g+1;
                           RungLoop = 0;
                        end;
                     else;
                        t = g+1;
                        break
                     end;
                  else;
                     t = g+1;
                     break;
                  end;
               end;
            else;
               NumBadValues = NumBadValues+1;
            end;
         end;
         %
         % deal with the remaining horizons
         %
         while t<=h;
            etaTh(:,t) = etaTTh(:,t)+randn(q,1);
            Xt(1:q) = etaTh(:,t);
            [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
            if status==1;
               At = AtNew;
               AtPath(:,t) = At;
               XtPath(:,t) = Xt;
               Ksit = P(1:r,:)*[Xt;At];
               rhat = P(r+1,:)*[Xt;At];
               ZLBKsiPath(:,t) = Ksit;
               KsiT = Ksit;
               PredictY(:,t) = Ax(:,t)+(H'*Ksit)+wTh(:,t)+wTTh(:,t);
               if t+1<=h;
                  At = M*At;
                  Xt = [zeros(q,1);Ksit;rhat];
               end;
               t = t+1;
            end;
         end;
         ZLBKsiPaths(:,:,it) = ZLBKsiPath;
         AtPaths(:,:,it) = AtPath;
         XtPaths(:,:,it) = XtPath;
         %
         % check if we should annualize or transform
         %
         if AnnualizeData==1;
            PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
         elseif TransData==1;
            YTmp = [Yinitial PredictY];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredictY = YData(Tt-h+1:Tt,:)';
         end;
         %
         % add the projected path to the variable to be saved
         %
         ZLBYPaths(:,:,it) = PredictY;
         drawnow;
      end;
   else;
      parfor it=1:NumPaths;
         %
         % loop over all g horizons
         %
         RungLoop = 1;
         while RungLoop==1;
            %
            % Draw a sequence of measurement errors
            %
            if isempty(Rmeasure)==0;
               wTh = Rmeasure*randn(NumMeasure,h);
               wTh(DSGEModel.ZLBPosition,:) = 0;
               WTg = vec(fliplr(wTh(:,1:g)));
            else;
               wTh = zeros(n,h);
               WTg = 0;
            end;
            %
            % draw initial state
            %
            KsiT = KsiTTh+(S*sqrt(L)*randn(size(L,2),1));
            %
            % setut the mean for the shock distribution
            %
            kTg = ZTg-UT-(KpD*(vecNtg-mubarTg))-(K'*(Xbar+(G*KsiT)+WTg+WbarTg));
            muTg = DpKInvKpDDpK*kTg;
            %
            % initialize
            %
            PredictY = zeros(n,h);
            etaTh = zeros(q,h);
            K2sum = zeros(m,1);
            At = zeros(DSGEModel.ZLBSampleT+1,1);
            Xt = [zeros(q,1);KsiT;KsiT(DSGEModel.RtildePosition)];
            %
            % draw shocks
            %
            ShockDraw = randn(q*g,1);
            etaTg = muTg+(SigmaTgStd*ShockDraw)+vecNtg-mubarTg;
            %
            % check
            %
            CheckCond = ZTg-(K'*(Xbar+(G*KsiT)+(D*etaTg)+WTg+WbarTg))-UT;
            if max(abs(CheckCond))<=1.0e-6;
               t = 1;
               AtPath = zeros(DSGEModel.ZLBSampleT+1,h);
               XtPath = zeros(q+r+1,h);
               ZLBKsiPath = zeros(r,h);
               while t<=g;
                  %
                  % draw economic shocks from N(muTg,SigmaTg) for period 1 to g
                  %
                  Ind1 = (1+((g-t)*q):(g-t+1)*q);
                  etaTh(:,t) = etaTg(Ind1);
                  ZPosi = ZPos(:,t);
                  m1 = sum(ZPosi);
                  %
                  % add shocks to Xt
                  %
                  Xt(1:q) = etaTh(:,t);
                  [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
                  if status==1;
                     %
                     % compute the state vector at forecast horizon t
                     %
                     Ksit = P(1:r,:)*[Xt;AtNew];
                     %
                     % check if we have non-zero anticipated shocks
                     %
                     if (sum(AtNew)>0)&(m1>0);
                        %
                        % check if the conditioning assumptions are satisfied
                        %
                        CA = Z(ZPosi,t) - (K1(:,ZPosi)'*Ax(:,t)) - (K1(:,ZPosi)'*(wTh(:,t)+wTTh(:,t))) - (K1(:,ZPosi)'*H(:,:,DSGEModel.LastPeriod+t)'*Ksit) - K2sum(ZPosi) -U(ZPosi,g);
                        %
                        % check if we should recompute the 
                        %
                        if max(abs(CA))>=1.0e-6;
                           NumBadValues = NumBadValues+1;
                           status = 0;
                        end;
                     end;
                     if status==1;
                        At = AtNew;
                        AtPath(:,t) = At;
                        XtPath(:,t) = Xt;
                        rhat = P(r+1,:)*[Xt;At];
                        ZLBKsiPath(:,t) = Ksit;
                        KsiT = Ksit;
                        PredictY(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit)+wTh(:,t)+wTTh(:,t);
                        if t+1<=h;
                           At = M*At;
                           Xt = [zeros(q,1);Ksit;rhat];
                        end;
                        %
                        % update the K2sum measure
                        %
                        if K2length==2;
                           K2sum = K2sum+(K2'*PredictY(:,t));
                        else;
                           K2sum = zeros(m,1);
                           for j=1:t;
                              if j<=size(K2,3);
                                 K2sum = K2sum+(K2(:,:,j)'*PredictY(:,t+1-j));
                              end;
                           end;
                        end;
                        t = t+1;
                        if t==g+1;
                           RungLoop = 0;
                        end;
                     else;
                        t = g+1;
                        break
                     end;
                  else;
                     t = g+1;
                     break;
                  end;
               end;
            else;
               NumBadValues = NumBadValues+1;
            end;
         end;
         %
         % deal with the remaining horizons
         %
         while t<=h;
            etaTh(:,t) = etaTTh(:,t)+randn(q,1);
            Xt(1:q) = etaTh(:,t);
            [AtNew,status,AlgorithmInformation] = ForwardBackShootingAlgorithm(Xt,At,ePG,Rzlb(t,:),Rbar,DSGEModel.AIMTolerance);
            if status==1;
               At = AtNew;
               AtPath(:,t) = At;
               XtPath(:,t) = Xt;
               Ksit = P(1:r,:)*[Xt;At];
               rhat = P(r+1,:)*[Xt;At];
               ZLBKsiPath(:,t) = Ksit;
               KsiT = Ksit;
               PredictY(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit)+wTh(:,t)+wTTh(:,t);
               if t+1<=h;
                  At = M*At;
                  Xt = [zeros(q,1);Ksit;rhat];
               end;
               t = t+1;
            end;
         end;
         ZLBKsiPaths(:,:,it) = ZLBKsiPath;
         AtPaths(:,:,it) = AtPath;
         XtPaths(:,:,it) = XtPath;
         %
         % check if we should annualize or transform
         %
         if AnnualizeData==1;
            PredictY = Annualize([Yinit PredictY],DSGEModel.annual,DSGEModel.annualscale);
         elseif TransData==1;
            YTmp = [Yinitial PredictY];
            [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            PredictY = YData(Tt-h+1:Tt,:)';
         end;
         %
         % add the projected path to the variable to be saved
         %
         ZLBYPaths(:,:,it) = PredictY;
         drawnow;
      end;
   end;
   %
   % compute the mean values of Xt and At
   %
   AtMean = (1/NumPaths)*sum(AtPaths,3);
   XtMean = (1/NumPaths)*sum(XtPaths,3);
   %
   % close the wait dialog
   %
   if DSGEModel.RunDriver==0;
      set(WaitControls.text,'UserData','done');
      delete(WaitHandle);
      drawnow;
      pause(0.02);
   end;
end;
%
if StopEstimation==1;
   return;
end;
%
% now compute the mean projections using AtMean and XtMean
%
for t=1:h;
   Ksit = P(1:r,:)*[XtMean(:,t);AtMean(:,t)];
   rhat = P(r+1,:)*[XtMean(:,t);AtMean(:,t)];
   ZLBKsiMean(:,t) = Ksit;
   if length(size(H))==2;
      ZLBYMean(:,t) = Ax(:,t)+(H'*Ksit);
   else;
      ZLBYMean(:,t) = Ax(:,t)+(H(:,:,DSGEModel.LastPeriod+t)'*Ksit);
   end;
end;
if AnnualizeData==1;
   ZLBYMean = Annualize([Yinit ZLBYMean],DSGEModel.annual,DSGEModel.annualscale);
elseif TransData==1;
   YTmp = [Yinitial ZLBYMean];
   [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,YTmp',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
   ZLBYMean = YData(Tt-h+1:Tt,:)';
end;
%
% update the mean value
%
PredData.ZLBYMean = ZLBYMean;
PredData.ZLBKsiMean = ZLBKsiMean;
if DSGEModel.RunPredictionEvent==1;
   %
   % compute the prediction events results
   %
   PredEventData = CalculatePredictionEvents(ZLBYPaths,DSGEModel.PredictionEvent,1);
end;
if NumPaths>=500;
   [PredData.KernelX,PredData.KernelY] = PredictionKernelDensity(ZLBYPaths);
else;
   PredData.KernelX = [];
   PredData.KernelY = [];
end;
%
% save results to file
%
PredictionEvent = DSGEModel.PredictionEvent;
file = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-' strrep(NameStr,' ','') '-1-' int2str(DSGEModel.FirstPeriod) '.' int2str(DSGEModel.LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(h) '-' int2str(NumPaths) '.mat']);
save(file,'ZLBYPaths','ZLBKsiPaths','ZLBYMean','ZLBKsiMean','KsitTh','PredData','PredEventData','PredictionEvent','SelectedYData');
Status = 1;

%
% end of ZLBDSGECenteredCondPredictionPathsThetaWZ.m
%
