function [LogLikeValue,BeliefStatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,Y,X,FirstPeriod,LastPeriod,xbar,A,H,R,InitP,c,MaxIter,Tolerance,StartPeriod,AllowNaNs,NoSmoothedStatesForBeliefs)
% AdaptiveLearningKalmanFilter: Computes the joint Kalman filter output for state variables, belief coefficients,
%                               DSGE model solution matrices and log-likelihood.
%
% USAGE:
%
%       [LogLikeValue,BeliefStatus,lnLt,Yerror,Yhat,MSEY,InvMSEY,Ksitt1,Ptt1,Ksitt,Ptt,betatt,Zlag,...
%                     Mtilde,Ftilde,Btilde,LastSolution] = AdaptiveLearningKalmanFilter(m,SMatrix,betavec,Sigmau,R10,Sigmaeps,...
%                                                                  rhoAL,Hstar0,Hstarlag,Hstar1f,D,Mt1,Ft1,Bt1,Gtilde,Kgf,Y,X,...
%                                                                  FirstPeriod,LastPeriod,xbar,A,H,R,InitP,c,MaxIter,,...
%                                                                  Tolerance,StartPeriod,AllowNaNs,NoSmoothedStatesForBeliefs)
%
% REQUIRED INPUT:  m (integer) with the maximum number of lags of the state variables in the belief equations.
%
%                  SMatrix (matrix) of dimension r x f which selects the forward looking variables among the
%                          state variables.
%
%                  betavec (vector) of dimension g with the initial values for the beta-vector of the belief equations.
%
%                  Sigmau (matrix) of dimension f x f with the forecast error covariance matrix of the belief
%                          equations.
%
%                  R10 (matrix) of dimension g x g with the initial value of the covariance matrix of the
%                          forecasted belief coefficients.
%
%                  Sigmaeps (matrix) of dimension g x g with the covariance matrix of the innovation to the
%                          transition equation of the belief coefficients.
%
%                  rhoAL (scalar) with the value of the rho-parameter in the state-equation for the
%                          belief parameters.
%
%                  Hstarlag (matrix) of dimension r x r with the coefficient values on the lagged state variables.
%
%                  Hstar0 (matrix) of dimension r x r with the coefficient values on the contemporaneosu state variables.
%
%                  Hstar1f (matrix) of dimension r x f with the coefficient values on the expectated future values
%                          of the forward looking variables.
%
%                  D (matrix) of dimension r x q with the coefficient values on the structural shocks.
%
%                  Mt1 (matrix) of dimension r x k with the coefficients on the deterministic
%                          variables in the solution equation.
%
%                  Ft1 (matrix) of dimension r x r with the coefficients on lagged state variables.
%
%                  Bt1 (matrix) of dimension r x q with the coefficients on the structural shocks.
%
%                  Gtilde (matrix) of dimension gf x k+(r*m) which links vec of qlag (g x f) to a stacking
%                          of the deterministic variables and m lags of the state variables.
%
%                  Kgf (matrix) with the gf x gf commutation matrix such that vec(A') = Kgf*vec(A) when
%                          A is a g x f matrix.
%
%                  Y (matrix) of dimension nxT1 with observed data.
%
%                  X (matrix) of dimension kxT1 matrix with exogenous of predetermined data.
%
%                  FirstPeriod (integer) with the first period to use for the data.
%
%                  LastPeriod (integer) with the last period to use for the data.
%
%                  xbar (vector) of dimension k with unit element for a constant and zeros elsewhere.
%
%                  A (matrix) of dimension kxn with parameter values on the k exogenous variables in the measurement
%                          equation.
%
%                  H (matrix) of dimension rxn (or rxnxT1) with parameter values on the r state variables (ksi) in the
%                          measurement equation.
%
%                  R (matrix) of dimension nxn with covariances for the innovations in the measurement equation.
%
%                  InitP (integer) indicating method of determining initial value for P(1|0), the MSE matrix for
%                          ksi(1|0). If InitP=1 (default), the P(1|0) is given by the unconditional covariance matrix of ksi,
%                          calculated through the vec operator. Since this operation can be excessively slow for large systems
%                          (when r^2 is a large number) the Doubling Algorthm can be chosen to approximate the unconditional
%                          variance. This algorithm is called if initP=2. Finally, if InitP=3, then P(1|0) = c*eye(r*r).
%
%                  c (positive scalar) indicating the variance of P(1|0) when initP=3. The default value is 1000.
%
%                  MaxIter (integer), the maximum number of iterations to use with the doubling algorithm
%                          divided by 100. Must be multiplied by 100 when calling DoublingAlgorithmLyapunov.
%
%                  Tolerance (integer), determines the tolerance value for the doubling algorithm.
%
%                  StartPeriod (integer) indicating first observation to use when calculating the log-likelihood value. The
%                          default value is 1.
%
%                  AllowNaNs (boolean) that takes the value 1 if NaNs among the observed variables should be allowed
%                          or not.
%
%                  NoSmoothedStatesForBeliefs (boolean) that takes the value 1 if smoothed states for Ksi(t-2|t-1) should be
%                          replaced with Ksi(t-2|t-2), and 0 otherwise.
%
% REQUIRED OUTPUT: LogLikeValue (scalar) with the value of the log-likelihood function.
%
%                  BeliefStatus (boolean) which is 1 if the Kalman filter of the belief system runs without
%                          error, and 0 otherwise.
%
% OPTIONAL OUTPUT: lnLt (vector) of dimension 1xT with the time t values of the log-densities that sum to the value lnL.
%
%                  Yerror (matrix) of dimension nxT with the 1-step ahead forecast errors of the observed variables.
%
%                  Yhat (matrix) of dimension nxT with the predictions [y(StartPeriod|StartPeriod-1) ... y(T|T-1).
%
%                  MSEY (matrix) of dimension nxnxT with the mean squared error matrix for y(t|t-1).
%
%                  InvMSEY (matrix) of dimension nxnxT with the inverted 1-step ahead forecast error covariances of
%                          the observed variables.
%
%                  Ksitt1 (matrix) of dimension rxT with the state variable predictions.
%
%                  Ptt1 (matrix) of dimension rxrxT with covariances for the state variable prediction errors.
%
%                  Ksitt (matrix) of dimension r x T with the state variable update estimates.
%
%                  Ptt (matrix) of dimension r x r x T with the covariances of the update estimates.
%
%                  betatt (matrix) of dimension g x T with the one-step-ahead estimates of the
%                          belief coefficients (with the first m obs being equal betavec).
%
%                  Zlag (matrix) of dimension (k+r*m) x T with the estimated explanatory variables
%                          for the Kalman filter of the belief coefficients. The first m columns are
%                          zeros except for the deterministic variables.
%
%                  Mtilde (matrix) of dimension r x k x T with the coefficients on the deterministic
%                          variables in the solution equation.
%
%                  Ftilde (matrix) of dimension r x r x T with the coefficients on lagged state variables.
%
%                  Btilde (matrix) of dimension r x q x T with the coefficients on the structural shocks.
%
%                  LastSolution (structure) with fields betaTT, MTT, FTT, BTT, RTT, betavec and M with the
%                          belief coefficients and solution for period T.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: March 24, 2020.
%                        This version: March 1, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 25-03-2020: Continued the function after the initialization part.
%
% * 26-03-2020: Continued the function after the code for the main case (fixed H, no missing obs,
%               full rank shocks and measurement errors, and more than 2 output variables.
%
% * 27-03-2020: Finished the function from nargout being 2 for missing observations.
%
% * 31-03-2020: Added the input variables "Mt1", "Ft1" and "Bt1" to avoid computing initial
%               solution values twice.
%
% * 01-04-2020: Changed the name ending of adaptive learning parameters.
%
% * 15-04-2020: Rewrote the filter such that ksitt1 is initialized instead of ksitt as well
%               as for the covariance matrices Ptt1 and Ptt. Moreover, the beta filter is
%               initialized for beta(1|0) and R(1|0) instead of beta(0|0) and R(0|0). The
%               forecast to t+1|t are now at the end of each iteration before the model is
%               solved again.
%
% * 28-04-2020: The function now takes the input variables InitP.
%
% * 30-04-2020: The function now ensures that Ptt1Last is equal to c times the identity
%               when InitP is 3.
%
% * 04-05-2020: Simplified the initial value of Ptt1Last by avoiding PttLast. Added the output
%               LastSolution with fields betattLast, Mt1, Ft1 and Bt1 for period T.
%
% * 11-05-2020: Added updating and forecasting of the belief coefficients for periods t=1 and
%               t=2 (when m=2) as done by Slobodyan and Wouters. Corrected bugs where ztf was
%               using KsittLast as input rather than KsittTmp(:,t). Similarly, betatt1 was
%               forecasted rather than betatt1Last. Added the input variable "NoSmoothedStatesForBeliefs".
%
% * 12-05-2020: Made sure that "NoSmoothedStatesForBeliefs" is only checked once by the function and not
%               for each t.
%
% * 07-07-2020: Added the fields betavec and M to LastSolution.
%
% * 22-01-2021: Updated the documentation.
%
% * 27-04-2021: Updated the documentation.
%
% * 05-05-2021: Fixed a bug where the R matrix did not take missing data into account when updating
%               the forecast covariance matrix.
%
% * 11-05-2021: Added the field RTT to the LastSolution output variable.
%
% * 12-05-2021: Updated the documentation.
%
% * 18-05-2021: Updated the documentation.
%
% * 03-01-2022: Updated the documentation.
%
% * 02-01-2023: Updated the documentation.
%
% * 08-01-2024: Updated the documentation.
%
% * 29-02-2024: The filter now also provides the solution matrix on the anticipated shocks. It is available
%               in the LastSolution output variable only.
%
% * 01-03-2024: Made sure that the relevant matrix for the anticipated shocks is available for T-1 and T.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% initialize
%
BeliefStatus = 1;
%
[r,f] = size(SMatrix);
g = length(betavec);
Y = Y(:,FirstPeriod:LastPeriod);
[n,T] = size(Y);
if isempty(X)==0;
   X = X(:,FirstPeriod:LastPeriod);
end;
if length(size(H))==3;
   H = H(:,:,FirstPeriod:LastPeriod);
end;
k = size(X,1);
q = size(D,2);
In = eye(n);
If = eye(f);
lengthH = length(size(H));
if lengthH==2;
   HT = H';
end;
betatt1Last = betavec;
betattLast = betavec;
Rtt1Last = R10;
M = rhoAL*eye(g);
%
% Compute the initial value for the state variables
%
KsittLast = (eye(r)-Ft1)\(Mt1*xbar);
%
% Compute the initial value of P(1|0) based on selection,
% noting that only the 3rd option is available when the
% solution is nonstationary
%
if max(abs(eig(Ft1)))>=1;
   InitP = 3;
end;
InitP = min(InitP,3);
if InitP==1;
   Ptt1Last = reshape((eye(r*r)-kron(Ft1,Ft1))\vec(Bt1*Bt1'),r,r);
elseif InitP==2;
   %
   % Compute the initial value for P(1|0)
   %
   [Ptt1Last,stat] = DoublingAlgorithmLyapunov(Ft1,Bt1*Bt1',MaxIter*100,10^(-(Tolerance+1)));
   %Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
   %
   % if the algorithm didn't converge we simply try the vectorization solution
   %
   if stat==1;
      Ptt1Last = reshape((eye(r*r)-kron(Ft1,Ft1))\vec(Bt1*Bt1'),r,r);
   end;
else;
   if c<=0;
      c = 10;
   end;
   Ptt1Last = c*eye(r);
end;
%
% initialize the state variable forecasts
%
if k>0;
   Ksitt1Last = (Mt1*X(:,1))+(Ft1*KsittLast);
   Ax = A'*X;
else;
   Ksitt1Last = Ft1*KsittLast;
   Ax = zeros(n,T);
end;
%
% Initialize temporary matrices
%
if nargout>2;
   YhatTmp = zeros(n,T);
   MSEYTmp = zeros(n,n,T);
   PttTmp = zeros(r,r,T);
   betattTmp = zeros(g,T);
   MtildeTmp = zeros(r,k,T);
   BtildeTmp = zeros(r,q,T);
end;
Ksitt1Tmp = zeros(r,T);
Ptt1Tmp = zeros(r,r,T);
YerrorTmp = zeros(n,T);
InvMSEYTmp = zeros(n,n,T);
KsittTmp = zeros(r,T);
FtildeTmp = zeros(r,r,T);
ZlagTmp = zeros(k+(r*m),T);
if k>0;
   ZlagTmp(1:k,:) = X;
end;
lnLtTmp = zeros(1,T);
%
% Now we can compute the Kalman filtered series
%
lnLConst = -n*log(2*pi);
if NoSmoothedStatesForBeliefs==0;
   if lengthH==2;
      if AllowNaNs==0;
         %
         % no missing observations
         %
         if rank(Bt1*Bt1')+rank(R)>=n;
            %
            % MSEY should be full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In/SigmaYtt1;
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  YhatTmp(:,t) = Ytt1;
                  MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        rt1 = H*InvMSEYTmp(:,:,t-1)*YerrorTmp(:,t-1);
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H*InvMSEYTmp(:,:,t-2);
                        %
                        % compute r(t-2|t-1)
                        %
                        rt2 = (H*InvMSEYTmp(:,:,t-2)*YerrorTmp(:,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H')'*rt1);
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In/SigmaYtt1;
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  %PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  %YhatTmp(:,t) = Ytt1;
                  %MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        rt1 = H*InvMSEYTmp(:,:,t-1)*YerrorTmp(:,t-1);
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H*InvMSEYTmp(:,:,t-2);
                        %
                        % compute r(t-2|t-1)
                        %
                        rt2 = (H*InvMSEYTmp(:,:,t-2)*YerrorTmp(:,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H')'*rt1);
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         else;
            %
            % MSEY is not full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1);
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  YhatTmp(:,t) = Ytt1;
                  MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        rt1 = H*InvMSEYTmp(:,:,t-1)*YerrorTmp(:,t-1);
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H*InvMSEYTmp(:,:,t-2);
                        %
                        % compute r(t-2|t-1)
                        %
                        rt2 = (H*InvMSEYTmp(:,:,t-2)*YerrorTmp(:,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H')'*rt1);
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1);
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  %PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  %YhatTmp(:,t) = Ytt1;
                  %MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        rt1 = H*InvMSEYTmp(:,:,t-1)*YerrorTmp(:,t-1);
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H*InvMSEYTmp(:,:,t-2);
                        %
                        % compute r(t-2|t-1)
                        %
                        rt2 = (H*InvMSEYTmp(:,:,t-2)*YerrorTmp(:,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H')'*rt1);
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         end;
      else;
         %
         % missing observations
         %
         ln2pi = -log(2*pi);
         if nargout>2;
            YhatTmp = NaN*ones(n,T);
            MSEYTmp = NaN*ones(n,n,T);
         end;
         InvMSEYTmp = NaN*ones(n,n,T);
         YerrorTmp = NaN*ones(n,T);
         if rank(Bt1*Bt1')+rank(R)>=n;
            %
            % MSEY should be full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In(pos,pos)/SigmaYtt1(pos,pos);
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     YhatTmp(pos,t) = Ytt1(pos,1);
                     MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        pos1 = (~isnan(Y(:,t-1)));
                        nt1 = sum(pos1);
                        pos2 = (~isnan(Y(:,t-2)));
                        nt2 = sum(pos2);
                        if nt1>0;
                           rt1 = H(:,pos1)*InvMSEYTmp(pos1,pos1,t-1)*YerrorTmp(pos1,t-1);
                        else;
                           rt1 = zeros(r,1);
                        end;
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        if nt2>0;
                           Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,pos2)*InvMSEYTmp(pos2,pos2,t-2);
                           %
                           % compute r(t-2|t-1)
                           %
                           rt2 = (H(:,pos2)*InvMSEYTmp(pos2,pos2,t-2)*YerrorTmp(pos2,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,pos2)')'*rt1);
                        else;
                           rt2 = FtildeTmp(:,:,t-2)'*rt1;
                        end;
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In(pos,pos)/SigmaYtt1(pos,pos);
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     %PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     %YhatTmp(pos,t) = Ytt1(pos,1);
                     %MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        pos1 = (~isnan(Y(:,t-1)));
                        nt1 = sum(pos1);
                        pos2 = (~isnan(Y(:,t-2)));
                        nt2 = sum(pos2);
                        if nt1>0;
                           rt1 = H(:,pos1)*InvMSEYTmp(pos1,pos1,t-1)*YerrorTmp(pos1,t-1);
                        else;
                           rt1 = zeros(r,1);
                        end;
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        if nt2>0;
                           Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,pos2)*InvMSEYTmp(pos2,pos2,t-2);
                           %
                           % compute r(t-2|t-1)
                           %
                           rt2 = (H(:,pos2)*InvMSEYTmp(pos2,pos2,t-2)*YerrorTmp(pos2,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,pos2)')'*rt1);
                        else;
                           rt2 = FtildeTmp(:,:,t-2)'*rt1;
                        end;
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         else;
            %
            % MSEY is not full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1(pos,pos));
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     YhatTmp(pos,t) = Ytt1(pos,1);
                     MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        pos1 = (~isnan(Y(:,t-1)));
                        nt1 = sum(pos1);
                        pos2 = (~isnan(Y(:,t-2)));
                        nt2 = sum(pos2);
                        if nt1>0;
                           rt1 = H(:,pos1)*InvMSEYTmp(pos1,pos1,t-1)*YerrorTmp(pos1,t-1);
                        else;
                           rt1 = zeros(r,1);
                        end;
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        if nt2>0;
                           Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,pos2)*InvMSEYTmp(pos2,pos2,t-2);
                           %
                           % compute r(t-2|t-1)
                           %
                           rt2 = (H(:,pos2)*InvMSEYTmp(pos2,pos2,t-2)*YerrorTmp(pos2,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,pos2)')'*rt1);
                        else;
                           rt2 = FtildeTmp(:,:,t-2)'*rt1;
                        end;
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1(pos,pos));
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     %PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     %YhatTmp(pos,t) = Ytt1(pos,1);
                     %MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        pos1 = (~isnan(Y(:,t-1)));
                        nt1 = sum(pos1);
                        pos2 = (~isnan(Y(:,t-2)));
                        nt2 = sum(pos2);
                        if nt1>0;
                           rt1 = H(:,pos1)*InvMSEYTmp(pos1,pos1,t-1)*YerrorTmp(pos1,t-1);
                        else;
                           rt1 = zeros(r,1);
                        end;
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        if nt2>0;
                           Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,pos2)*InvMSEYTmp(pos2,pos2,t-2);
                           %
                           % compute r(t-2|t-1)
                           %
                           rt2 = (H(:,pos2)*InvMSEYTmp(pos2,pos2,t-2)*YerrorTmp(pos2,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,pos2)')'*rt1);
                        else;
                           rt2 = FtildeTmp(:,:,t-2)'*rt1;
                        end;
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         end;
      end;
   else;
      %
      % time-varying H-matrix
      %
      if AllowNaNs==0;
         %
         % no missing observations
         %
         if rank(Bt1*Bt1')+rank(R)>=n;
            %
            % MSEY should be full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In/SigmaYtt1;
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  YhatTmp(:,t) = Ytt1;
                  MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        rt1 = H(:,:,t-1)*InvMSEYTmp(:,:,t-1)*YerrorTmp(:,t-1);
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,:,t-2)*InvMSEYTmp(:,:,t-2);
                        %
                        % compute r(t-2|t-1)
                        %
                        rt2 = (H(:,:,t-2)*InvMSEYTmp(:,:,t-2)*YerrorTmp(:,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,:,t-2)')'*rt1);
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In/SigmaYtt1;
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  %PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  %YhatTmp(:,t) = Ytt1;
                  %MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        rt1 = H(:,:,t-1)*InvMSEYTmp(:,:,t-1)*YerrorTmp(:,t-1);
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,:,t-2)*InvMSEYTmp(:,:,t-2);
                        %
                        % compute r(t-2|t-1)
                        %
                        rt2 = (H(:,:,t-2)*InvMSEYTmp(:,:,t-2)*YerrorTmp(:,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,:,t-2)')'*rt1);
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         else;
            %
            % MSEY is not full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1);
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  YhatTmp(:,t) = Ytt1;
                  MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        rt1 = H(:,:,t-1)*InvMSEYTmp(:,:,t-1)*YerrorTmp(:,t-1);
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,:,t-2)*InvMSEYTmp(:,:,t-2);
                        %
                        % compute r(t-2|t-1)
                        %
                        rt2 = (H(:,:,t-2)*InvMSEYTmp(:,:,t-2)*YerrorTmp(:,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,:,t-2)')'*rt1);
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1);
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  %PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  %YhatTmp(:,t) = Ytt1;
                  %MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        rt1 = H(:,:,t-1)*InvMSEYTmp(:,:,t-1)*YerrorTmp(:,t-1);
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,:,t-2)*InvMSEYTmp(:,:,t-2);
                        %
                        % compute r(t-2|t-1)
                        %
                        rt2 = (H(:,:,t-2)*InvMSEYTmp(:,:,t-2)*YerrorTmp(:,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,:,t-2)')'*rt1);
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         end;
      else;
         %
         % missing observations
         %
         ln2pi = -log(2*pi);
         if nargout>2;
            YhatTmp = NaN*ones(n,T);
            MSEYTmp = NaN*ones(n,n,T);
         end;
         InvMSEYTmp = NaN*ones(n,n,T);
         YerrorTmp = NaN*ones(n,T);
         %
         if rank(Bt1*Bt1')+rank(R)>=n;
            %
            % MSEY should be full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In(pos,pos)/SigmaYtt1(pos,pos);
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     YhatTmp(pos,t) = Ytt1(pos,1);
                     MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        pos1 = (~isnan(Y(:,t-1)));
                        nt1 = sum(pos1);
                        pos2 = (~isnan(Y(:,t-2)));
                        nt2 = sum(pos2);
                        if nt1>0;
                           rt1 = H(:,pos1,t-1)*InvMSEYTmp(pos1,pos1,t-1)*YerrorTmp(pos1,t-1);
                        else;
                           rt1 = zeros(r,1);
                        end;
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        if nt2>0;
                           Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,pos2,t-2)*InvMSEYTmp(pos2,pos2,t-2);
                           %
                           % compute r(t-2|t-1)
                           %
                           rt2 = (H(:,pos2,t-2)*InvMSEYTmp(pos2,pos2,t-2)*YerrorTmp(pos2,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,pos2,t-2)')'*rt1);
                        else;
                           rt2 = FtildeTmp(:,:,t-2)'*rt1;
                        end;
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In(pos,pos)/SigmaYtt1(pos,pos);
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     %PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     %YhatTmp(pos,t) = Ytt1(pos,1);
                     %MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        pos1 = (~isnan(Y(:,t-1)));
                        nt1 = sum(pos1);
                        pos2 = (~isnan(Y(:,t-2)));
                        nt2 = sum(pos2);
                        if nt1>0;
                           rt1 = H(:,pos1,t-1)*InvMSEYTmp(pos1,pos1,t-1)*YerrorTmp(pos1,t-1);
                        else;
                           rt1 = zeros(r,1);
                        end;
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        if nt2>0;
                           Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,pos2,t-2)*InvMSEYTmp(pos2,pos2,t-2);
                           %
                           % compute r(t-2|t-1)
                           %
                           rt2 = (H(:,pos2,t-2)*InvMSEYTmp(pos2,pos2,t-2)*YerrorTmp(pos2,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,pos2,t-2)')'*rt1);
                        else;
                           rt2 = FtildeTmp(:,:,t-2)'*rt1;
                        end;
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         else;
            %
            % MSEY is not full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1(pos,pos));
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     YhatTmp(pos,t) = Ytt1(pos,1);
                     MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        pos1 = (~isnan(Y(:,t-1)));
                        nt1 = sum(pos1);
                        pos2 = (~isnan(Y(:,t-2)));
                        nt2 = sum(pos2);
                        if nt1>0;
                           rt1 = H(:,pos1,t-1)*InvMSEYTmp(pos1,pos1,t-1)*YerrorTmp(pos1,t-1);
                        else;
                           rt1 = zeros(r,1);
                        end;
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        if nt2>0;
                           Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,pos2,t-2)*InvMSEYTmp(pos2,pos2,t-2);
                           %
                           % compute r(t-2|t-1)
                           %
                           rt2 = (H(:,pos2,t-2)*InvMSEYTmp(pos2,pos2,t-2)*YerrorTmp(pos2,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,pos2,t-2)')'*rt1);
                        else;
                           rt2 = FtildeTmp(:,:,t-2)'*rt1;
                        end;
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1(pos,pos));
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     %PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     %YhatTmp(pos,t) = Ytt1(pos,1);
                     %MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        %
                        % compute r(t-1|t-1)
                        %
                        pos1 = (~isnan(Y(:,t-1)));
                        nt1 = sum(pos1);
                        pos2 = (~isnan(Y(:,t-2)));
                        nt2 = sum(pos2);
                        if nt1>0;
                           rt1 = H(:,pos1,t-1)*InvMSEYTmp(pos1,pos1,t-1)*YerrorTmp(pos1,t-1);
                        else;
                           rt1 = zeros(r,1);
                        end;
                        %
                        % compute the Kalman gain matrix for t-3 (t-1 lagged two periods)
                        %
                        if nt2>0;
                           Kt3 = FtildeTmp(:,:,t-2)*Ptt1Tmp(:,:,t-2)*H(:,pos2,t-2)*InvMSEYTmp(pos2,pos2,t-2);
                           %
                           % compute r(t-2|t-1)
                           %
                           rt2 = (H(:,pos2,t-2)*InvMSEYTmp(pos2,pos2,t-2)*YerrorTmp(pos2,t-2))+((FtildeTmp(:,:,t-2)-Kt3*H(:,pos2,t-2)')'*rt1);
                        else;
                           rt2 = FtildeTmp(:,:,t-2)'*rt1;
                        end;
                        %
                        % compute ksi(t-2|t-1)
                        %
                        ZlagTmp(k+r+1:k+(r*2),t) = Ksitt1Tmp(:,t-2)+(Ptt1Tmp(:,:,t-2)*rt2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         end;
      end;
   end;
else;
   if lengthH==2;
      if AllowNaNs==0;
         %
         % no missing observations
         %
         if rank(Bt1*Bt1')+rank(R)>=n;
            %
            % MSEY should be full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In/SigmaYtt1;
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  YhatTmp(:,t) = Ytt1;
                  MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In/SigmaYtt1;
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  %PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  %YhatTmp(:,t) = Ytt1;
                  %MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         else;
            %
            % MSEY is not full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1);
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  YhatTmp(:,t) = Ytt1;
                  MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1);
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  %PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  %YhatTmp(:,t) = Ytt1;
                  %MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         end;
      else;
         %
         % missing observations
         %
         ln2pi = -log(2*pi);
         if nargout>2;
            YhatTmp = NaN*ones(n,T);
            MSEYTmp = NaN*ones(n,n,T);
         end;
         InvMSEYTmp = NaN*ones(n,n,T);
         YerrorTmp = NaN*ones(n,T);
         if rank(Bt1*Bt1')+rank(R)>=n;
            %
            % MSEY should be full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In(pos,pos)/SigmaYtt1(pos,pos);
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     YhatTmp(pos,t) = Ytt1(pos,1);
                     MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In(pos,pos)/SigmaYtt1(pos,pos);
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     %PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     %YhatTmp(pos,t) = Ytt1(pos,1);
                     %MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         else;
            %
            % MSEY is not full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1(pos,pos));
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     YhatTmp(pos,t) = Ytt1(pos,1);
                     MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  PLastH = Ptt1Last*H;
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1(pos,pos));
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     %PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     %YhatTmp(pos,t) = Ytt1(pos,1);
                     %MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         end;
      end;
   else;
      %
      % time-varying H-matrix
      %
      if AllowNaNs==0;
         %
         % no missing observations
         %
         if rank(Bt1*Bt1')+rank(R)>=n;
            %
            % MSEY should be full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In/SigmaYtt1;
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  YhatTmp(:,t) = Ytt1;
                  MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In/SigmaYtt1;
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  %PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  %YhatTmp(:,t) = Ytt1;
                  %MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         else;
            %
            % MSEY is not full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1);
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  YhatTmp(:,t) = Ytt1;
                  MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1);
                  YerrorTmp(:,t) = Y(:,t)-Ytt1;
                  %
                  % compute the log-likelihood
                  %
                  lnLtTmp(1,t) = lnLConst-LogDet(SigmaYtt1)-(YerrorTmp(:,t)'*InvSigmaYtt1*YerrorTmp(:,t));
                  %
                  % update t|t
                  %
                  KsittTmp(:,t) = Ksitt1Last+(PLastH*InvSigmaYtt1*YerrorTmp(:,t));
                  %PttTmp(:,:,t) = Ptt1Last-(PLastH*InvSigmaYtt1*PLastH');
                  %
                  % collect forecast data
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;
                  %YhatTmp(:,t) = Ytt1;
                  %MSEYTmp(:,:,t) = SigmaYtt1;
                  InvMSEYTmp(:,:,t) = InvSigmaYtt1;
                  %
                  % now we update Ksitt1Last and Ptt1Last : t+1|t
                  %
                  if t<T;
                     KalmanGain = Ft1*PLastH*InvSigmaYtt1;
                     Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(:,t));
                     FDev = Ft1-(KalmanGain*HT);
                     Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R*KalmanGain')+(Bt1*Bt1');
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         end;
      else;
         %
         % missing observations
         %
         ln2pi = -log(2*pi);
         if nargout>2;
            YhatTmp = NaN*ones(n,T);
            MSEYTmp = NaN*ones(n,n,T);
         end;
         InvMSEYTmp = NaN*ones(n,n,T);
         YerrorTmp = NaN*ones(n,T);
         %
         if rank(Bt1*Bt1')+rank(R)>=n;
            %
            % MSEY should be full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In(pos,pos)/SigmaYtt1(pos,pos);
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     YhatTmp(pos,t) = Ytt1(pos,1);
                     MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = In(pos,pos)/SigmaYtt1(pos,pos);
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     %PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     %YhatTmp(pos,t) = Ytt1(pos,1);
                     %MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         else;
            %
            % MSEY is not full rank
            %
            if nargout>2;
               %
               % we need to store additional output variables
               %
               for t=1:T;
                  %
                  % forecast t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1(pos,pos));
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     YhatTmp(pos,t) = Ytt1(pos,1);
                     MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  betattTmp(:,t) = betattLast;
                  if k>0;
                     MtildeTmp(:,:,t) = Mt1;
                  end;
                  FtildeTmp(:,:,t) = Ft1;
                  BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if t>m;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1,InvMat] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
                  if t==T-1;
                     InvMatT1 = InvMat;
                  elseif t==T;
                     InvMatT = InvMat;
                  end;
               end;
            else;
               %
               % we only have 2 output variables
               %
               for t=1:T;
                  %
                  % forecast t|t-1
                  %
                  pos = (~isnan(Y(:,t)));
                  n1 = sum(pos);
                  %
                  % forecast observables: t|t-1
                  %
                  HT = H(:,:,t)';
                  PLastH = Ptt1Last*H(:,:,t);
                  SigmaYtt1 = (HT*PLastH)+R;
                  Ytt1 = Ax(:,t)+(HT*Ksitt1Last);
                  InvSigmaYtt1 = pinv(SigmaYtt1(pos,pos));
                  %
                  % collect state variable forecast data t|t-1
                  %
                  Ksitt1Tmp(:,t) = Ksitt1Last;
                  Ptt1Tmp(:,:,t) = Ptt1Last;               
                  %
                  if n1>0;
                     YerrorTmp(pos,t) = Y(pos,t)-Ytt1(pos,1);
                     %
                     % compute the log-likelihood
                     %
                     lnLtTmp(1,t) = (n1*ln2pi)-LogDet(SigmaYtt1(pos,pos))-(YerrorTmp(pos,t)'*InvSigmaYtt1*YerrorTmp(pos,t));
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last+(PLastH(:,pos)*InvSigmaYtt1*YerrorTmp(pos,t));
                     %PttTmp(:,:,t) = Ptt1Last-(PLastH(:,pos)*InvSigmaYtt1*PLastH(:,pos)');
                     %
                     % collect observed variable forecast data t|t-1
                     %
                     %YhatTmp(pos,t) = Ytt1(pos,1);
                     %MSEYTmp(pos,pos,t) = SigmaYtt1(pos,pos);
                     InvMSEYTmp(pos,pos,t) = InvSigmaYtt1;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        KalmanGain = Ft1*PLastH(:,pos)*InvSigmaYtt1;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last)+(KalmanGain*YerrorTmp(pos,t));
                        FDev = Ft1-(KalmanGain*HT(pos,:));
                        Ptt1Last = (FDev*Ptt1Last*FDev')+(KalmanGain*R(pos,pos)*KalmanGain')+(Bt1*Bt1');
                     end;
                  else;
                     %
                     % update t|t
                     %
                     KsittTmp(:,t) = Ksitt1Last;
                     PttTmp(:,:,t) = Ptt1Last;
                     %
                     % now we update Ksitt1Last and Ptt1Last : t+1|t
                     %
                     if t<T;
                        Ksitt1Last = (Mt1*X(:,t+1))+(Ft1*Ksitt1Last);
                        Ptt1Last = (Ft1*Ptt1Last*Ft1')+(Bt1*Bt1');
                     end;
                  end;
                  %
                  % collect data on belief coefficients and solution to the model
                  %
                  %betattTmp(:,t) = betattLast;
                  %if k>0;
                  %   MtildeTmp(:,:,t) = Mt1;
                  %end;
                  FtildeTmp(:,:,t) = Ft1;
                  %BtildeTmp(:,:,t) = Bt1;
                  %
                  % check if we should run the Kalman filter for the belief coefficients
                  %
                  if (t>m)&(t<T);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                     %
                     % if need be compute second lag of state variables
                     % conditional on information at t-1
                     %
                     if m==2;
                        ZlagTmp(k+r+1:k+(r*2),t) = KsittTmp(:,t-2);
                     end;
                  elseif t==1;
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);
                  elseif (t==2)&(m==2);
                     %
                     % compute forward variable update estimates at t
                     %
                     ztf = SMatrix'*KsittTmp(:,t);;
                     %
                     % collect lagged update state variables
                     %
                     ZlagTmp(k+1:k+r,t) = KsittTmp(:,t-1);
                  end;
                  %
                  % retrieve qt1 transpose from Zlag
                  %
                  qt1T = reshape(Kgf*Gtilde*ZlagTmp(:,t),f,g);
                  %
                  % update belief coefficients
                  %
                  Sigmazf = (qt1T*Rtt1Last*qt1T')+Sigmau;
                  if (rank(Sigmazf)==f)&(det(Sigmazf)>=1.0e-10);
                     InvSigmazf = Sigmazf\If;
                  else;
                     [S,Lambda] = SchurDecomposition(Sigmazf,1.0e-10);
                     f1 = size(Lambda,1);
                     InvLambda = diag(ones(f1,1)./diag(Lambda));
                     InvSigmazf = S*InvLambda*S';
                  end;
                  betattLast = betatt1Last+(Rtt1Last*qt1T'*InvSigmazf*(ztf-(qt1T*betattLast)));
                  RttLast = Rtt1Last-(Rtt1Last*qt1T'*InvSigmazf*qt1T*Rtt1Last);
                  %
                  % forecast
                  %
                  betatt1Last = betavec+(M*(betattLast-betavec));
                  Rtt1Last = (M*RttLast*M)+Sigmaeps;
                  %
                  % solve the model with betatt for use in the next period
                  %
                  [Mt1,Ft1,Bt1] = AdaptiveLearningSolveDSGEModel(Hstar0,Hstarlag,Hstar1f,D,Gtilde,Kgf,m,k,betattLast);
               end;
            end;
         end;
      end;
   end;
end;
%
% do the remaining calculations
%
LogLikeValue = (1/2)*sum(lnLtTmp(1,StartPeriod:T));
%
% test if the log-likelihood is real
%
if (isreal(LogLikeValue)==0)|(isnan(LogLikeValue)==1);
   BeliefStatus = 0;
end;
%
% Fix output to match StartPeriod
%
if nargout==3;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
elseif nargout==4;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
elseif nargout==5;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
elseif nargout==6;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
elseif nargout==7;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
elseif nargout==8;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
elseif nargout==9;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
elseif nargout==10;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   Ksitt = KsittTmp(:,StartPeriod:T);
elseif nargout==11;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   Ksitt = KsittTmp(:,StartPeriod:T);
   Ptt = PttTmp(:,:,StartPeriod:T);
elseif nargout==12;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   Ksitt = KsittTmp(:,StartPeriod:T);
   Ptt = PttTmp(:,:,StartPeriod:T);
   betatt = betattTmp(:,StartPeriod:T);
elseif nargout==13;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   Ksitt = KsittTmp(:,StartPeriod:T);
   Ptt = PttTmp(:,:,StartPeriod:T);
   betatt = betattTmp(:,StartPeriod:T);
   Zlag = ZlagTmp(:,StartPeriod:T);
elseif nargout==14;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   Ksitt = KsittTmp(:,StartPeriod:T);
   Ptt = PttTmp(:,:,StartPeriod:T);
   betatt = betattTmp(:,StartPeriod:T);
   Zlag = ZlagTmp(:,StartPeriod:T);
   Mtilde = MtildeTmp(:,:,StartPeriod:T);
elseif nargout==15;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   Ksitt = KsittTmp(:,StartPeriod:T);
   Ptt = PttTmp(:,:,StartPeriod:T);
   betatt = betattTmp(:,StartPeriod:T);
   Zlag = ZlagTmp(:,StartPeriod:T);
   Mtilde = MtildeTmp(:,:,StartPeriod:T);
   Ftilde = FtildeTmp(:,:,StartPeriod:T);
elseif nargout==16;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   Ksitt = KsittTmp(:,StartPeriod:T);
   Ptt = PttTmp(:,:,StartPeriod:T);
   betatt = betattTmp(:,StartPeriod:T);
   Zlag = ZlagTmp(:,StartPeriod:T);
   Mtilde = MtildeTmp(:,:,StartPeriod:T);
   Ftilde = FtildeTmp(:,:,StartPeriod:T);
   Btilde = BtildeTmp(:,:,StartPeriod:T);
elseif nargout==17;
   lnLt = (1/2)*lnLtTmp(1,StartPeriod:T);
   Yerror = YerrorTmp(:,StartPeriod:T);
   Yhat = YhatTmp(:,StartPeriod:T);
   MSEY = MSEYTmp(:,:,StartPeriod:T);
   InvMSEY = InvMSEYTmp(:,:,StartPeriod:T);
   Ksitt1 = Ksitt1Tmp(:,StartPeriod:T);
   Ptt1 = Ptt1Tmp(:,:,StartPeriod:T);
   Ksitt = KsittTmp(:,StartPeriod:T);
   Ptt = PttTmp(:,:,StartPeriod:T);
   betatt = betattTmp(:,StartPeriod:T);
   Zlag = ZlagTmp(:,StartPeriod:T);
   Mtilde = MtildeTmp(:,:,StartPeriod:T);
   Ftilde = FtildeTmp(:,:,StartPeriod:T);
   Btilde = BtildeTmp(:,:,StartPeriod:T);
   LastSolution.betaTT = betattLast;
   LastSolution.RTT = RttLast;
   LastSolution.MTT = Mt1;
   LastSolution.FTT = Ft1;
   LastSolution.BTT = Bt1;
   LastSolution.betavec = betavec;
   LastSolution.M = M;
   LastSolution.InvMatT1 = InvMatT1;
   LastSolution.InvMatT = InvMatT;
end;

%
% end of AdaptiveLearningKalmanFilter.m
%
