function CorrToolsMenuFunctions(selector,maingui)
% CorrToolsMenuFunctions: switch function for running correlation functions on the tools menu.
%
% USAGE:
%
%       CorrToolsMenuFunctions(selector,maingui)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
% OPTIONAL INPUT: maingui (handle) to the YADA GUI.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: May 10, 2010.
%                        This version: March 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 11-05-2010: Updated the documentation.
%
% * 12-05-2010: Wrote the conherence code for the intial values and the posterior mode estimates.
%
% * 14-05-2010: began work on the coherence code for the prior and the posterior distribution.
%
% * 16-06-2010: Changed the width of Query dialogs from 450 to 500 pixels.
%
% * 21-06-2010: Took into account that "CoherenceDistributionDLG" now accepts the "ModelStr"
%               input variable.
%
% * 22-06-2010: Updated the documentation.
%
% * 02-07-2010: It is now possible to save the results from population-based observed variable
%               correlations (time domain) to disk for the initial values and the posterior
%               mode values.
%
% * 08-07-2010: Changed the headers of About dialogs when "mcode" or "kalmanstatus" reports problems.
%
% * 14-10-2010: When writing the population covariance matrix of the economic shocks to file, the code
%               now checks which shocks can be observed from the data and the model parameters.
%
% * 15-10-2010: Took into account that "SVEExist" now takes the input variable "SimulSmooth".
%
% * 20-10-2010: The code now checks if the state variables can be observed via the model parameters and
%               the data or not by checking if the covariance matrix of the smooth estimates has zero
%               variances or not. This check is implemented for the initial values and the posterior mode
%               values when running the state variable correlations functions in the time domain.
%
% * 17-12-2010: The function now supports the Klein and Sims DSGE model solvers.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 17-03-2011: The measurement error correlations for fixed parameter values now take missing observations
%               into account.
%
% * 25-11-2011: Took into account that skewness statistics for the observed variables are now computed.
%
% * 02-12-2011: Took into account that kurtosis statistics for the observed variables are now also computed.
%
% * 02-01-2012: Updated the documentation.
%
% * 29-02-2012: Took into account that the number of simulated paths per parameter value can be unity.
%
% * 03-01-2013: Updated the documentation.
%
% * 23-05-2013: Added additional information when asking if data should be simulated.
%
% * 13-08-2013: Began work on posterior predictive checks for the one-step-ahead forecast error covariances.
%
% * 15-08-2013: Continued work on posterior predictive checks for the one-step-ahead stuff.
%
% * 19-08-2013: Finished the function for posterior predictive checks.
%
% * 20-08-2013: Began work on the prior predictive checks function for one-step-ahead forecast error
%               covariances.
%
% * 28-01-2014: Updated the documentation.
%
% * 07-03-2014: The user can now choose between the update and the smooth estimates of the structural shocks
%               at initial values and posterior mode when viewing the shocks correlations.
%
% * 10-03-2014: The same choice was added for the correlations of the measurement errors.
%
% * 20-03-2014: Began writing code for the parameter change effects on the observed variable population means and
%               covariances.
%
% * 21-03-2014: Finished coding the posterior mode based estimates of the effects from a parameter change on the
%               observed variable means, standard deviations, and correlations.
%
% * 24-03-2014: Finished coding the initial values based estimates of the effects from a parameters change on the
%               observed variable means, standard deviations, and correlations.
%
% * 25-03-2014: Finished coding the posterior draws estimates of the effects from a parameter change on the
%               observed variable means, standard deviations and correlations.
%
% * 26-03-2014: Added the cases with the effects on the means, standard deviations amd correlations of the observed
%               variables using the prior draws.
%
% * 14-01-2015: Updated the documentation.
%
% * 02-09-2015: Fixed the information displayed when the kalmanstatus is -1.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 23-08-2017: Took into account that LoadDSGEPosteriorDraws now provides the output variable "Weights".
%
% * 24-08-2017: Took into account that "DSGESVEDistribution", "DSGESVESimulatedDistribution",
%               "DSGECoherenceDistribution", and "DSGEMomentsResponseDistribution" now require the
%               input variable "Weights".
%
% * 25-08-2017: Took into account that "OneStepAheadPredictiveChecks" now requires the input variable
%               "Weights".
%
% * 15-09-2017: Updated the documentation.
%
% * 12-12-2017: Began work on ensuring that prior draws from the system prior allows for using a subset
%               of the draws for computing model features.
%
% * 13-12-2017: Continued work on the prior draws issue for the system prior.
%
% * 08-01-2018: Updated the documentation.
%
% * 10-01-2018: Made sure that shock correlations at the initial and posterior mode values are printed
%               to file.
%
% * 11-01-2018: Prepared some additional output from the shock correlations, namely the largest discrepancies
%               between the sample estimates and the theoretical correlations (asymptotic when able to compute).
%
% * 12-01-2018: Some further output from the shock correlations was prepared.
%
% * 01-08-2018: The function supports user written model solver function.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 12-09-2018: Fixed a bug with TheoStDev for measurement error correlations where the vector
%               instead of the diagonal matrix was used.
%
% * 25-10-2018: Made sure that the name of the prior draws file reflects using a system prior.
%
% * 23-01-2019: Updated the documentation.
%
% * 15-04-2019: Began work on parameter change effects on the state variable moments.
%
% * 17-04-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 26-01-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 22-02-2023: Added the optional input variable "maingui".
%
% * 18-01-2024: Updated the documentation.
%
% * 27-02-2024: Made sure that etatT and wtT exist after running "CalculateStateVariables".
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
if nargin==1;
   maingui = findobj('Type','figure','Tag','YADA');
end;
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
if MatlabNumber<7;
   lasterr = '';
end;
%
switch selector
   case 'CoherenceInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if we should compute coherency for the observed variables or the state variables
         %
         if gcbo==controls.obsvarcorrs.frequencydomain.initialvalues;
            VarStr = 'Observed';
            VariableNames = DSGEModel.VariableNames;
            VarType = 1;
         elseif gcbo==controls.statevarcorrs.frequencydomain.initialvalues;
            VarStr = 'State';
            VariableNames = DSGEModel.StateVariableNames;
            VarType = 0;
         end;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = ['Are you sure you want to view the coherence of the ' lower(VarStr) ' variables before running the posterior mode estimation routine?'];
            answer = Query(txt,'question',150,'Coherence',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               %
               % compute coherence
               %
               [SOmega,Omega] = DSGECoherenceTheta(theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI);
               if isempty(SOmega)==0;
                  %
                  % check if the user wants to save the results to disk
                  %
                  txt = ['Would you like to save the coherence results for the ' lower(VarStr) ' variables at initial values to file?'];
                  answer = Query(txt,'question',140,'Save - Coherence',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\' VarStr 'Variables-Coherence-' DSGEModel.NameOfModel '-InitialValues.mat']);
                     %
                     save(file,'VariableNames','SOmega','Omega');
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\' VarStr(1,1) 'VCoherenceInitVal.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the %s variables\n%%\n',lower(VarStr));
                     fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                     fprintf(fid,'%%\n%% matrix with coherency statistics ordered as (var1,var2),(var1,var3)\n%% and so on for the f different frequencies\n%%\n');
                     fprintf(fid,'SOmega = DataStructure.SOmega;\n');
                     fprintf(fid,'%%\n%% vector with the f different frequencies from 0 to pi\n%%\n');
                     fprintf(fid,'Omega = DataStructure.Omega;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The coherence results for the ' lower(VarStr) ' variables at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: VariableNames (string matrix with the names of the ' lower(VarStr) ' variables), SOmega (matrix with the unique coherence results for [variable 1 vs variable 2], [variable 1 vs variable 3], and so on, for each frequency [columns]), and Omega (vector with the different frequencies from 0 to pi). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Coherence - Initial Values',400,500,CurrINI);
                     drawnow;
                  end;
                  %
                  % display the results
                  %
                  CoherenceDistributionDLG('init',SOmega,Omega,[],VariableNames,VarStr,'Initial Values','',DSGEModel,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CoherencePriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if we should compute coherency for the observed variables or the state variables
         %
         if gcbo==controls.obsvarcorrs.frequencydomain.prior;
            VarStr = 'Observed';
            VariableNames = DSGEModel.VariableNames;
            VarType = 1;
         elseif gcbo==controls.statevarcorrs.frequencydomain.prior;
            VarStr = 'State';
            VariableNames = DSGEModel.StateVariableNames;
            VarType = 0;
         end;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,['Coherence For Prior - ' VarStr ' Variables'],500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Coherence - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check if calulcations have already been performed
            %
            if CoherenceExist(DSGEModel,VarType,NumPriorDraws,NumPriorDraws,1,0)==1;
               txt = ['YADA has located coherence results for the ' lower(VarStr) ' variables on disk. Would you like to use these results?'];
               answer = Query(txt,'question',150,['Coherence For Prior - ' VarStr ' Variables'],500,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               DoneCalc = 1;
            else;
               %
               % compute the distribution
               %
               DoneCalc = DSGECoherence(thetaDraws',theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI,NumPriorDraws,1,0);
            end;
            %
            % check if we should show any results
            %
            if DoneCalc==1;
               %
               % compute the distribution for the impulse responses
               %
               [SOmega,Omega,Quantiles] = DSGECoherenceDistribution(DSGEModel,CurrINI,VarStr,VariableNames,NumPriorDraws,NumPriorDraws,1,0,[]);
               %
               % ask if we should save results to disk
               %
               txt = ['Would you like to save the prior distributions of the coherence statistics for the ' lower(VarStr) ' variables to file?'];
               answer = Query(txt,'question',140,'Save - Coherence',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\' VarStr 'Variables-Coherence-' DSGEModel.NameOfModel '-Prior.mat']);
                  %
                  save(file,'VariableNames','SOmega','Omega','Quantiles');
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\' VarStr(1,1) 'VCoherencePrior.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the %s variables\n%%\n',lower(VarStr));
                  fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                  fprintf(fid,'%%\n%% matrix with mean coherency statistics ordered as (var1,var2),(var1,var3)\n%% and so on for the f different frequencies\n%%\n');
                  fprintf(fid,'SOmega = DataStructure.SOmega;\n');
                  fprintf(fid,'%%\n%% vector with the f different frequencies from 0 to pi\n%%\n');
                  fprintf(fid,'Omega = DataStructure.Omega;\n');
                  fprintf(fid,'%%\n%% vector structure with quantile data through the fields SOmega and percent\n%%\n');
                  fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  txt = ['The prior distribution of the coherence statistics for the ' lower(VarStr) ' variables have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: VariableNames (string matrix with the names of the ' lower(VarStr) ' variables), SOmega (matrix with the unique coherence results for [variable 1 vs variable 2], [variable 1 vs variable 3], and so on, for each frequency [columns]), Omega (vector with the different frequencies from 0 to pi), and Quantiles (vector structure with quantile data in the fields SOmega and percent). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Coherence - Prior Distribution',400,500,CurrINI);
                  drawnow;
               end;
               %
               % display the results
               %
               CoherenceDistributionDLG('init',SOmega,Omega,Quantiles,VariableNames,VarStr,'Prior Distribution','',DSGEModel,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CoherencePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the autocorrelations
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % check if we should compute coherence for the observed variables or the state variables
            %
            if gcbo==controls.obsvarcorrs.frequencydomain.postmode;
               VarStr = 'Observed';
               VariableNames = DSGEModel.VariableNames;
               VarType = 1;
            elseif gcbo==controls.statevarcorrs.frequencydomain.postmode;
               VarStr = 'State';
               VariableNames = DSGEModel.StateVariableNames;
               VarType = 0;
            end;
            %
            % compute coherence
            %
            [SOmega,Omega] = DSGECoherenceTheta(theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI);
            if isempty(SOmega)==0;
               %
               % check if the user wants to save the results to disk
               %
               txt = ['Would you like to save the coherence results for the ' lower(VarStr) ' variables at posterior mode to file?'];
               answer = Query(txt,'question',140,'Save - Coherence',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\' VarStr 'Variables-Coherence-' DSGEModel.NameOfModel '-PostMode.mat']);
                  %
                  save(file,'VariableNames','SOmega','Omega');
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\' VarStr(1,1) 'VCoherencePostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the %s variables\n%%\n',lower(VarStr));
                  fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                  fprintf(fid,'%%\n%% matrix with coherency statistics ordered as (var1,var2),(var1,var3)\n%% and so on for the f different frequencies\n%%\n');
                  fprintf(fid,'SOmega = DataStructure.SOmega;\n');
                  fprintf(fid,'%%\n%% vector with the f different frequencies from 0 to pi\n%%\n');
                  fprintf(fid,'Omega = DataStructure.Omega;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  txt = ['The coherence results for the ' lower(VarStr) ' variables at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: VariableNames (string matrix with the names of the ' lower(VarStr) ' variables), SOmega (matrix with the unique coherence results for [variable 1 vs variable 2], [variable 1 vs variable 3], and so on, for each frequency [columns]), and Omega (vector with the different frequencies from 0 to pi). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Coherence - Posterior Mode',400,500,CurrINI);
                  drawnow;
               end;
               %
               % display the results
               %
               CoherenceDistributionDLG('init',SOmega,Omega,[],VariableNames,VarStr,'Posterior Mode','',DSGEModel,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'CoherencePosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            theta = ModeData.thetaMode;
            %
            % check if we should compute coherency for the observed variables or the state variables
            %
            if gcbo==controls.obsvarcorrs.frequencydomain.posterior;
               VarStr = 'Observed';
               VariableNames = DSGEModel.VariableNames;
               VarType = 1;
            elseif gcbo==controls.statevarcorrs.frequencydomain.posterior;
               VarStr = 'State';
               VariableNames = DSGEModel.StateVariableNames;
               VarType = 0;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection',['Coherence - ' VarStr ' Variables'],'','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check if calculations have already been performed
                  %
                  if CoherenceExist(DSGEModel,VarType,UsedDraws,NumDraws,CurrChain,1)==1;
                     txt = ['YADA has located coherence results for the ' lower(VarStr) ' variables on disk. Would you like to use these results?'];
                     answer = Query(txt,'question',150,['Coherence For Posterior - ' VarStr ' Variables'],500,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the distribution
                     %
                     DoneCalc = DSGECoherence(thetaPostSample,theta,thetaPositions,ModelParameters,VarType,DSGEModel,CurrINI,NumDraws,CurrChain,1);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
                  %
                  % check if we should show any results
                  %
                  if DoneCalc==1;
                     %
                     % compute the distribution for the impulse responses
                     %
                     [SOmega,Omega,Quantiles] = DSGECoherenceDistribution(DSGEModel,CurrINI,VarStr,VariableNames,UsedDraws,NumDraws,CurrChain,1,Weights);
                     %
                     % ask if we should save results to disk
                     %
                     txt = ['Would you like to save the posterior distributions of the coherence statistics for the ' lower(VarStr) ' variables to file?'];
                     answer = Query(txt,'question',140,'Save - Coherence',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\' VarStr 'Variables-Coherence-' DSGEModel.NameOfModel '-Posterior.mat']);
                        %
                        save(file,'VariableNames','SOmega','Omega','Quantiles');
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\' VarStr(1,1) 'VCoherencePosterior.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the %s variables\n%%\n',lower(VarStr));
                        fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                        fprintf(fid,'%%\n%% matrix with mean coherency statistics ordered as (var1,var2),(var1,var3)\n%% and so on for the f different frequencies\n%%\n');
                        fprintf(fid,'SOmega = DataStructure.SOmega;\n');
                        fprintf(fid,'%%\n%% vector with the f different frequencies from 0 to pi\n%%\n');
                        fprintf(fid,'Omega = DataStructure.Omega;\n');
                        fprintf(fid,'%%\n%% vector structure with quantile data through the fields SOmega and percent\n%%\n');
                        fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The posterior distribution of the coherence statistics for the ' lower(VarStr) ' variables have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: VariableNames (string matrix with the names of the ' lower(VarStr) ' variables), SOmega (matrix with the unique coherence results for [variable 1 vs variable 2], [variable 1 vs variable 3], and so on, for each frequency [columns]), Omega (vector with the different frequencies from 0 to pi), and Quantiles (vector structure with quantile data in the fields SOmega and percent). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Coherence - Posterior Distribution',400,500,CurrINI);
                        drawnow;
                     end;
                     %
                     % display the results
                     %
                     CoherenceDistributionDLG('init',SOmega,Omega,Quantiles,VariableNames,VarStr,'Posterior Distribution','',DSGEModel,CurrINI);
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view correlations of the observed variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Observed Variable Correlations',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % ask if the user wants to simulate data
               %
               if NumPredPaths>1;
                  txt = 'Would you like to examine model based sample moments rather than model based population moments? Note: the model based sample moments require simulation of data.';
                  answer = Query(txt,'question',150,'Observed Variable Correlations - Initial Values',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'no')==1;
                  SimulateData = 0;
                  NumPredPaths = 0;
               else;
                  SimulateData = 1;
               end;
               if SimulateData==0;
                  [SVEs,mcode] = CalculateDSGECorrelations(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,FirstPeriod,LastPeriod);
               else;
                  [SVEs,mcode] = DSGECorrelationSimulationTheta(theta,thetaPositions,ModelParameters,NumPredPaths,'Initial Values',DSGEModel,CurrINI);
               end;
               %
               if mcode==1;
                  %
                  % check to make sure that CorrData is not empty
                  %
                  if isempty(SVEs)==0;
                     if SimulateData==0;
                        SVEs.Corr = SVEs.StateSpace.Corr;
                        SVEs.Mean = SVEs.StateSpace.Mean;
                        SVEs.Std = SVEs.StateSpace.Std;
                        SVEs.Cov = SVEs.StateSpace.Cov;
                        SVEs.Skew = SVEs.StateSpace.Skew;
                        SVEs.Kurt = SVEs.StateSpace.Kurt;
                        StateSpace = SVEs.StateSpace;
                        SVEs = rmfield(SVEs,'StateSpace');
                        AutoCorrType = 'Population-based';
                        DLGNameStr = 'Autocorrelation Functions - Initial Values';
                        BoxStr = 'Graphics - Population Autocorrelations';
                     else;
                        %
                        AutoCorrType = 'Sample-based';
                        DLGNameStr = 'Autocorrelation Distributions - Initial Values';
                        BoxStr = 'Graphics - Sample Autocorrelations';
                     end;
                     %
                     % ask if we should save the results to disk
                     %
                     txt = ['Would you like to save the ' lower(AutoCorrType) ' mean, standard deviation, correlations, skewness, kurtosis, and quantiles of the autocorrelation distributions for the initial parameter values to file?'];
                     answer = Query(txt,'question',140,'Save - Autocorrelation Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDist-' AutoCorrType '-' int2str(NumPredPaths) '-InitialValues.mat']);
                        VariableNames = DSGEModel.VariableNames;
                        save(file,'SVEs','VariableNames');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\ACDist' AutoCorrType(1) int2str(NumPredPaths) 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% structure with fields Data.Mean, Data.Std, Data.Corr, Data.Skew, Data.Kurt (data moments), Mean, Std, Corr, Skew, Kurt (model means), Quantiles, and AutoCovHorizon.\n%%\n');
                        fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The autocorrelation data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the data and model based mean, standard deviation, correlation, skewness, kurtosis, quantile data, and correlation points), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Autocorrelation Distribution - Initial Values',200,500,CurrINI);
                     end;
                     %
                     % display the autocorrelations
                     %
                     AutoCorrelationDistributionDLG('init',SVEs,AutoCorrType,DLGNameStr,BoxStr,DSGEModel,CurrINI);
                     %
                     % we can now present the results
                     %
                     if SimulateData==0;
                        SVEs.StateSpace = StateSpace;
                     end;
                     PrintAutoCorrelations(DSGEModel,SVEs,CurrINI,'initial',NumPredPaths);
                     if NumPredPaths==0;
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\InitialValueAutoCorr-' DSGEModel.NameOfModel '.txt']);
                     else;
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\InitialValueAutoCorr-' DSGEModel.NameOfModel '-' int2str(NumPredPaths) '.txt']);
                     end;
                     if FileExist(ResFile)==1;
                        txt = 'Would you like to view the data and model based mean and standard deviations of the observed variables?';
                        answer = Query(txt,'question',200,'Autocorrelation Distribution - Initial Values',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           TextGUI(ResFile,['Mean & Standard Deviation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                        end;
                     end;
                  end;
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Autocorrelation Distribution - Initial Values',200,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Autocorrelation Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Autocorrelation Distribution - Prior','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','Autocorrelation Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % ask if the user wants to simulate data
               %
               txt = 'Would you like to examine model based sample moments (prior predictive densities) rather than model based population moments? Note: the model based sample moments require simulation of data, while the population moments only use draws from the prior distribution of the structural parameters.';
               answer = Query(txt,'question',150,'Autocorrelation Distribution - Prior',500,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  SimulateData = 0;
                  NumPredPaths = 0;
               else;
                  SimulateData = 1;
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               end;
               %
               % First we check if the calculations have already been performed
               %
               if SVEExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,NumPredPaths,1,0,0)==1;
                  txt = 'YADA has located results for the autocorrelations on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'Autocorrelation Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % start estimation
                  %
                  if SimulateData==0;
                     DoneCalc = DSGEStateVariableEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'Autocorrelation',0,NumPriorDraws,1,DSGEModel,CurrINI);
                  else;
                     DoneCalc = DSGEStateVariableSimulation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'Autocorrelation',0,NumPriorDraws,1,NumPredPaths,DSGEModel,CurrINI);
                  end;
               end;
               %
               % check if we should show any results
               %
               if DoneCalc==1;
                  %
                  % compute the distribution for the 1-step ahead forecasts
                  %
                  if SimulateData==0;
                     SVEs = DSGESVEDistribution(DSGEModel,CurrINI,5,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                     AutoCorrType = 'Population-based';
                  else;
                     SVEs = DSGESVESimulatedDistribution(DSGEModel,CurrINI,5,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,NumPredPaths,0,[]);
                     AutoCorrType = 'Sample-based';
                  end;
                  SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                  %
                  % check if results should be written to file
                  %
                  txt = 'Would you like to save the mean, standard deviation, correlations, skewness, kurtosis, and quantiles of the autocorrelation distributions to file?';
                  answer = Query(txt,'question',140,'Save - Autocorrelation Distribution For Prior',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     if NumPredPaths==0;
                        file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDistPrior-' SelParamStr '-' int2str(NumPriorDraws) '.mat']);
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\ACDistPrior' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) '.m']);
                     else;
                        file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDistPrior-' SelParamStr '-' int2str(NumPriorDraws) '-' int2str(NumPredPaths) '.mat']);
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\ACDistPrior' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) int2str(NumPredPaths) '.m']);
                     end;
                     VariableNames = DSGEModel.VariableNames;
                     save(file,'SVEs','VariableNames');
                     %
                     % create a matlab file to read the data
                     %
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% structure with fields Data.Mean, Data.Std, Data.Corr, Data.Skew, Data.Kurt (data moments), Mean, Std, Corr, Skew, Kurt (model means), Quantiles, and AutoCovHorizon.\n%%\n');
                     fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The autocorrelation data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the data and model based mean, standard deviation, correlation, skewness, kurtosis, quantile data, and correlation points), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Autocorrelation Distribution - Prior',300,500,CurrINI);
                  end;
                  %
                  % display the autocorrelations
                  %
                  AutoCorrelationDistributionDLG('init',SVEs,AutoCorrType,'Autocorrelation Distribution - Prior','Graphics - Autocorrelation Distribution',DSGEModel,CurrINI);
                  %
                  % check if the user wants to view the mean and the standard deviations
                  %
                  txt = 'Would you like to view the data and model based mean and standard deviations of the observed variables?';
                  answer = Query(txt,'question',200,'Autocorrelation Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     if NumPredPaths==0;
                        file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDistPrior-' SelParamStr '-' int2str(NumPriorDraws) '.txt']);
                     else;
                        file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDistPrior-' SelParamStr '-' int2str(NumPriorDraws) '-' int2str(NumPredPaths) '.txt']);
                     end;
                     %
                     PrintStateVarDistribution(file,SVEs,DSGEModel,CurrINI,4,NumPriorDraws,NumPriorDraws,1,NumPredPaths,0);
                     %
                     % display the results
                     %
                     if FileExist(file)==1;
                        TextGUI(file,['Observed Variable Correlation Distributions For Prior - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Correlation Distributions',50,CurrINI,1,1);
                     end;
                  end;
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the autocorrelation distribution.';
               About(txt,'information','Autocorrelation Distribution - Prior',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the autocorrelations
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               if isfield(ModeData,'LastPeriod')==1;
%                  FirstPeriod = ModeData.FirstPeriod;
%                  LastPeriod = ModeData.LastPeriod;
%               else;
%                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               end;
%            else;
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % ask if the user wants to simulate data
            %
            if NumPredPaths>1;
               txt = 'Would you like to examine model based sample moments rather than model based population moments? Note: the model based sample moments require simulation of data.';
               answer = Query(txt,'question',150,'Observed Variable Correlations - Posterior Mode',500,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'no')==1;
               SimulateData = 0;
               NumPredPaths = 0;
            else;
               SimulateData = 1;
            end;
            if SimulateData==0;
               SVEs = CalculateDSGECorrelations(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,FirstPeriod,LastPeriod);
            else;
               SVEs = DSGECorrelationSimulationTheta(theta,thetaPositions,ModelParameters,NumPredPaths,'Posterior Mode',DSGEModel,CurrINI);
            end;
            if isempty(SVEs)==0;
               if SimulateData==0;
                  SVEs.Corr = SVEs.StateSpace.Corr;
                  SVEs.Mean = SVEs.StateSpace.Mean;
                  SVEs.Std = SVEs.StateSpace.Std;
                  SVEs.Cov = SVEs.StateSpace.Cov;
                  SVEs.Skew = SVEs.StateSpace.Skew;
                  SVEs.Kurt = SVEs.StateSpace.Kurt;
                  StateSpace = SVEs.StateSpace;
                  SVEs = rmfield(SVEs,'StateSpace');
                  AutoCorrType = 'Population-based';
                  DLGNameStr = 'Autocorrelation Functions - Posterior Mode';
                  BoxStr = 'Graphics - Population Autocorrelations';
               else;
                  %
                  AutoCorrType = 'Sample-based';
                  DLGNameStr = 'Autocorrelation Distributions - Posterior Mode';
                  BoxStr = 'Graphics - Sample Autocorrelations';
               end;
               %
               % ask if we should save the results to disk
               %
               txt = ['Would you like to save the ' lower(AutoCorrType) ' mean, standard deviation, correlations, skewness, kurtosis, and quantiles of the autocorrelation distributions for the posterior mode parameter estimate to file?'];
               answer = Query(txt,'question',140,'Save - Autocorrelation Distribution',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDist-' AutoCorrType '-' int2str(NumPredPaths) '-PosteriorMode.mat']);
                  VariableNames = DSGEModel.VariableNames;
                  save(file,'SVEs','VariableNames');
                  %
                  % create a matlab file to read the data
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\ACDist' AutoCorrType(1) int2str(NumPredPaths) 'PostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% structure with fields Data.Mean, Data.Std, Data.Corr, Data.Skew, Data.Kurt (data moments), Mean, Std, Corr, Skew, Kurt (model means), Quantiles, and AutoCovHorizon.\n%%\n');
                  fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  txt = ['The autocorrelation data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the data and model based mean, standard deviation, correlation, skewness, kurtosis, quantile data, and correlation points), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','Autocorrelation Distribution - Posterior Mode',200,500,CurrINI);
               end;
               %
               % display the autocorrelations
               %
               AutoCorrelationDistributionDLG('init',SVEs,AutoCorrType,DLGNameStr,BoxStr,DSGEModel,CurrINI);
               %
               % we can now present the results
               %
               if SimulateData==0;
                  SVEs.StateSpace = StateSpace;
               end;
               PrintAutoCorrelations(DSGEModel,SVEs,CurrINI,'posterior mode',NumPredPaths);
               if NumPredPaths==0;
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeAutoCorr-' DSGEModel.NameOfModel '.txt']);
               else;
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeAutoCorr-' DSGEModel.NameOfModel '-' int2str(NumPredPaths) '.txt']);
               end;
               if FileExist(ResFile)==1;
                  txt = 'Would you like to view the data and model based mean and standard deviations of the observed variables?';
                  answer = Query(txt,'question',200,'Autocorrelation Distribution - Posterior Mode',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     TextGUI(ResFile,['Mean & Standard Deviation - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Autocorrelation Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','Autocorrelation Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % ask if the user wants to simulate data
                     %
                     txt = 'Would you like to examine model based sample moments (posterior predictive densities) rather than model based population moments? Note: the model based sample moments require simulation of data, while the population moments only use draws from the posterior distribution of the structural parameters.';
                     answer = Query(txt,'question',150,'Autocorrelation Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        SimulateData = 0;
                        NumPredPaths = 0;
                     else;
                        SimulateData = 1;
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     end;
                     %
                     % First we check if the calculations have already been performed
                     %
                     if SVEExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,NumPredPaths,1,1,0)==1;
                        txt = 'YADA has located results for the autocorrelations on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'Autocorrelation Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % start estimation
                        %
                        if SimulateData==0;
                           DoneCalc = DSGEStateVariableEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'Autocorrelation',1,NumDraws,CurrChain,DSGEModel,CurrINI);
                        else;
                           DoneCalc = DSGEStateVariableSimulation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'Autocorrelation',1,NumDraws,CurrChain,NumPredPaths,DSGEModel,CurrINI);
                        end;
                     end;
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % compute the distribution for the 1-step ahead forecasts
                        %
                        if SimulateData==0;
                           SVEs = DSGESVEDistribution(DSGEModel,CurrINI,5,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights);
                           AutoCorrType = 'Population-based';
                        else;
                           SVEs = DSGESVESimulatedDistribution(DSGEModel,CurrINI,5,ScenarioParameters,UsedDraws,NumDraws,CurrChain,NumPredPaths,1,Weights);
                           AutoCorrType = 'Sample-based';
                        end;
                        %
                        % check if results should be written to file
                        %
                        txt = 'Would you like to save the mean, standard deviation, correlations, skewness, kurtosis, and quantiles of the autocorrelation distributions to file?';
                        answer = Query(txt,'question',140,'Save - Autocorrelation Distribution',500,'no',CurrINI);
                        SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           save(file,'SVEs','VariableNames');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\ACDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields Data.Mean, Data.Std, Data.Corr, Data.Skew, Data.Kurt (data moments), Mean, Std, Corr, Skew, Kurt (model means), Quantiles, and AutoCovHorizon.\n%%\n');
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The autocorrelation data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the data and model based mean, standard deviation, correlation, skewness, kurtosis, quantile data, and correlation points), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Autocorrelation Distribution',300,500,CurrINI);
                        end;
                        %
                        % display the autocorrelations
                        %
                        AutoCorrelationDistributionDLG('init',SVEs,AutoCorrType,'Autocorrelation Distribution','Graphics - Autocorrelation Distribution',DSGEModel,CurrINI);
                        %
                        % check if the user wants to view the mean and the standard deviations
                        %
                        txt = 'Would you like to view the data and model based mean and standard deviations of the observed variables?';
                        answer = Query(txt,'question',200,'Autocorrelation Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           if NumPredPaths==0;
                              file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.txt']);
                           else;
                              file = FixFilePath([DSGEModel.OutputDirectory '\AutoCorrelationDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '-' int2str(NumPredPaths) '.txt']);
                           end;
                           %
                           PrintStateVarDistribution(file,SVEs,DSGEModel,CurrINI,4,UsedDraws,NumDraws,CurrChain,NumPredPaths,1);
                           %
                           % display the results
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['Observed Variable Correlation Distributions - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Correlation Distributions',50,CurrINI,1,1);
                           end;
                        end;
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the autocorrelation distribution.';
                     About(txt,'information','Autocorrelation Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrParameterChangeInitVal'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view the effects on the means, standard deviations and correlations of the observed variables from a change in structural parameter before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Parameter Change - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     ParameterNames = ModeData.ParameterNames;
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               %
               % select which parameter to use
               %
               ParameterNames = ParameterNames.estimated;
               [action,ParamNumber] = SelectionDlg(ParameterNames,1,'Change parameter for gauging effects on population mean, st. dev., and correlations:','Parameter Selection','Parameter Change - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  %
               % compute the responses to the mean, standard deviation and correlations of the observed variables from an increase
               % of the selected parameter
               %
               [MeanResp,StdResp,CorrResp] = DSGEMomentsResponseTheta(theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,1,'Initial Values',DSGEModel,CurrINI);
               if isempty(StdResp)==0;
                  %
                  % write the results to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-ObsVarMomentsResponse-' StringTrim(ParameterNames(ParamNumber,:)) '-InitialValues.txt']);
                  %
                  PrintDSGEMomentsResponseTheta(ResFile,MeanResp,StdResp,CorrResp,ParameterNames,ParamNumber,1,'Initial Values',DSGEModel,CurrINI);
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Moment Responses To Parameter Change - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
                  end;
               end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrParameterChangePriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Parameter Change - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            ParameterNames = ParameterNames.estimated;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Parameter Change - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            [action,ParamNumber] = SelectionDlg(ParameterNames,1,'Change parameter for gauging effects on population mean, st. dev., and correlations:','Parameter Selection','Parameter Change - Prior Distribution','','',CurrINI);
            if strcmp(lower(action),'ok')==1;
               %
               % check if we already have results
               %
               if DSGEMomentsResponseExist(ParameterNames,ParamNumber,1,[],0,NumPriorDraws,NumPriorDraws,DSGEModel,CurrINI)==1;
                  txt = ['YADA has located results for the effects on the means, standard deviations and correlations of the observed variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" on disk. Would you like to use these results?'];
                  answer = Query(txt,'question',150,'Parameter Change - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % compute the responses to the mean, standard deviation and correlations of the observed variables from an increase
                  % of the selected parameter
                  %
                  DoneCalc = DSGEMomentsResponse(thetaDraws',theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,1,[],0,NumPriorDraws,DSGEModel,CurrINI);
               end;
               if DoneCalc==1;
                  %
                  % sort the results
                  %
                  MomentsResp = DSGEMomentsResponseDistribution(ParameterNames,ParamNumber,1,[],0,[],NumPriorDraws,NumPriorDraws,DSGEModel,CurrINI);
                  %
                  % ask if we should save results to disk
                  %
                  txt = ['Would you like to save the distributions for the effects on the means, standard deviations and correlations of the observed variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" to file?'];
                  answer = Query(txt,'question',140,'Save - Parameter Change Prior Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-ObsVarMomentsResponse-Prior-' StringTrim(ParameterNames(ParamNumber,:)) '-' int2str(NumPriorDraws) '.mat']);
                     save(MatFile,'MomentsResp');
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEOVMRPrior' strrep(StringTrim(ParameterNames(ParamNumber,:)),'-','') int2str(NumPriorDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                     fprintf(fid,'%%\n%% structure with fields MeanResp (if the model has exogenous variables),\n%% StdResp, CorrResp vector field Quantiles with subfields MeanResp, StdResp,\n%% CorrResp, and percent holding quantile data and quantile percent.\n%%\n');
                     fprintf(fid,'MomentsResp = DataStructure.MomentsResp;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The distributions of the effects on the means, standard deviations and correlations for the observed variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" have been saved to the file "' GetFilename(MatFile) '" in the directory "' GetPath(MatFile) '". The file contains 1 entry: MomentsResp (a structure with the effects on the means, standard deviations, correlations, and quantile data based on the prior distribution). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Parameter Change - Prior Distribution',200,500,CurrINI);
                  end;
                  %
                  % write the results to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-ObsVarMomentsResponse-Prior-' StringTrim(ParameterNames(ParamNumber,:)) '-' int2str(NumPriorDraws) '.txt']);
                  %
                  PrintDSGEMomentsResponse(ResFile,MomentsResp,ParameterNames,ParamNumber,1,[],0,NumPriorDraws,NumPriorDraws,DSGEModel,CurrINI);
                  %
                  % try to display it
                  %
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Parameter Change - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distributions',50,CurrINI,1,1);
                  end;
               end;
            end;
            clear('thetaDraws');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrParameterChangePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the autocorrelations
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % select which parameter to use
            %
            [action,ParamNumber] = SelectionDlg(ParameterNames,1,'Change parameter for gauging effects on population mean, st. dev., and correlations:','Parameter Selection','Parameter Change - Posterior Mode','','',CurrINI);
            if strcmp(lower(action),'ok')==1;
               %
               % compute the responses to the mean, standard deviation and correlations of the observed variables from an increase
               % of the selected parameter
               %
               [MeanResp,StdResp,CorrResp] = DSGEMomentsResponseTheta(theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,1,'Posterior Mode',DSGEModel,CurrINI);
               if isempty(StdResp)==0;
                  %
                  % write the results to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-ObsVarMomentsResponse-' StringTrim(ParameterNames(ParamNumber,:)) '-PostMode.txt']);
                  %
                  PrintDSGEMomentsResponseTheta(ResFile,MeanResp,StdResp,CorrResp,ParameterNames,ParamNumber,1,'Posterior Mode',DSGEModel,CurrINI);
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Moment Responses To Parameter Change - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode',50,CurrINI,1,1);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ObsVarCorrParameterChangePosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Parameter Change - Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  [action,ParamNumber] = SelectionDlg(ParameterNames,1,'Change parameter for gauging effects on population mean, st. dev., and correlations:','Parameter Selection','Parameter Change - Posterior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     %
                     % check if we already have results
                     %
                     if DSGEMomentsResponseExist(ParameterNames,ParamNumber,1,CurrChain,1,UsedDraws,NumDraws+NumBurnin,DSGEModel,CurrINI)==1;
                        txt = ['YADA has located results for the effects on the means, standard deviations and correlations of the observed variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" on disk. Would you like to use these results?'];
                        answer = Query(txt,'question',150,'Parameter Change - Posterior Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % compute the responses to the mean, standard deviation and correlations of the observed variables from an increase
                        % of the selected parameter
                        %
                        DoneCalc = DSGEMomentsResponse(thetaPostSample,theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,1,CurrChain,1,NumDraws+NumBurnin,DSGEModel,CurrINI);
                     end;
                     if DoneCalc==1;
                        %
                        % sort the results
                        %
                        MomentsResp = DSGEMomentsResponseDistribution(ParameterNames,ParamNumber,1,CurrChain,1,Weights,UsedDraws,NumDraws+NumBurnin,DSGEModel,CurrINI);
                        %
                        % ask if we should save results to disk
                        %
                        txt = ['Would you like to save the distributions for the effects on the means, standard deviations and correlations of the observed variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" to file?'];
                        answer = Query(txt,'question',140,'Save - Parameter Change Posterior Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-ObsVarMomentsResponse-' StringTrim(ParameterNames(ParamNumber,:)) '-' int2str(CurrChain) '-' int2str(UsedDraws) '.' int2str(NumDraws+NumBurnin) '.mat']);
                           save(MatFile,'MomentsResp');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEOVMR' strrep(StringTrim(ParameterNames(ParamNumber,:)),'-','') int2str(CurrChain) int2str(UsedDraws) int2str(NumDraws+NumBurnin) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                           fprintf(fid,'%%\n%% structure with fields MeanResp (if the model has exogenous variables),\n%% StdResp, CorrResp vector field Quantiles with subfields MeanResp, StdResp,\n%% CorrResp, and percent holding quantile data and quantile percent.\n%%\n');
                           fprintf(fid,'MomentsResp = DataStructure.MomentsResp;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The distributions of the effects on the means, standard deviations and correlations for the observed variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" have been saved to the file "' GetFilename(MatFile) '" in the directory "' GetPath(MatFile) '". The file contains 1 entry: MomentsResp (a structure with the effects on the means, standard deviations, correlations, and quantile data based on the posterior distribution). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Parameter Change - Posterior Distribution',200,500,CurrINI);
                        end;
                        %
                        % write the results to file
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-ObsVarMomentsResponse-' StringTrim(ParameterNames(ParamNumber,:)) '-' int2str(CurrChain) '-' int2str(UsedDraws) '.' int2str(NumDraws+NumBurnin) '.txt']);
                        %
                        PrintDSGEMomentsResponse(ResFile,MomentsResp,ParameterNames,ParamNumber,1,CurrChain,1,UsedDraws,NumDraws+NumBurnin,DSGEModel,CurrINI);
                        %
                        % try to display it
                        %
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Parameter Change - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,1);
                        end;
                     end;
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarCorrParameterChangeInitVal'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view the effects on the standard deviations and correlations of the state variables from a change in structural parameter before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Parameter Change - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     ParameterNames = ModeData.ParameterNames;
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               %
               % select which parameter to use
               %
               ParameterNames = ParameterNames.estimated;
               [action,ParamNumber] = SelectionDlg(ParameterNames,1,'Change parameter for gauging effects on population st. dev. and correlations:','Parameter Selection','Parameter Change - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  %
               % compute the responses to the mean, standard deviation and correlations of the observed variables from an increase
               % of the selected parameter
               %
               [MeanResp,StdResp,CorrResp] = DSGEMomentsResponseTheta(theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,0,'Initial Values',DSGEModel,CurrINI);
               if isempty(StdResp)==0;
                  %
                  % write the results to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-StateVarMomentsResponse-' StringTrim(ParameterNames(ParamNumber,:)) '-InitialValues.txt']);
                  %
                  PrintDSGEMomentsResponseTheta(ResFile,MeanResp,StdResp,CorrResp,ParameterNames,ParamNumber,0,'Initial Values',DSGEModel,CurrINI);
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Moment Responses To Parameter Change - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,1);
                  end;
               end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarCorrParameterChangePriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Parameter Change - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            ParameterNames = ParameterNames.estimated;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Parameter Change - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            [action,ParamNumber] = SelectionDlg(ParameterNames,1,'Change parameter for gauging effects on population st. dev. and correlations:','Parameter Selection','Parameter Change - Posterior Distribution','','',CurrINI);
            if strcmp(lower(action),'ok')==1;
               %
               % check if we already have results
               %
               if DSGEMomentsResponseExist(ParameterNames,ParamNumber,0,[],0,NumPriorDraws,NumPriorDraws,DSGEModel,CurrINI)==1;
                  txt = ['YADA has located results for the effects on the standard deviations and the correlations of the observed variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" on disk. Would you like to use these results?'];
                  answer = Query(txt,'question',150,'Parameter Change - Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % compute the responses to the mean, standard deviation and correlations of the observed variables from an increase
                  % of the selected parameter
                  %
                  DoneCalc = DSGEMomentsResponse(thetaDraws',theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,0,[],0,NumPriorDraws,DSGEModel,CurrINI);
               end;
               if DoneCalc==1;
                  %
                  % sort the results
                  %
                  MomentsResp = DSGEMomentsResponseDistribution(ParameterNames,ParamNumber,0,[],0,[],NumPriorDraws,NumPriorDraws,DSGEModel,CurrINI);
                  %
                  % ask if we should save results to disk
                  %
                  txt = ['Would you like to save the distributions for the effects on the standard deviations and correlations of the state variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" to file?'];
                  answer = Query(txt,'question',140,'Save - Parameter Change Prior Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-StateVarMomentsResponse-Prior-' StringTrim(ParameterNames(ParamNumber,:)) '-' int2str(NumPriorDraws) '.mat']);
                     save(MatFile,'MomentsResp');
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGESVMRPrior' strrep(StringTrim(ParameterNames(ParamNumber,:)),'-','') int2str(NumPriorDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                     fprintf(fid,'%%\n%% structure with fields StdResp, CorrResp vector field Quantiles with subfields\n%% StdResp and CorrResp, and percent holding quantile data and quantile percent.\n%%\n');
                     fprintf(fid,'MomentsResp = DataStructure.MomentsResp;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The distributions of the effects on the standard deviations and the correlations for the state variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" have been saved to the file "' GetFilename(MatFile) '" in the directory "' GetPath(MatFile) '". The file contains 1 entry: MomentsResp (a structure with the effects on the standard deviations, correlations, and quantile data based on the posterior distribution). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Parameter Change - Prior Distribution',200,500,CurrINI);
                  end;
                  %
                  % write the results to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-StateVarMomentsResponse-Prior-' StringTrim(ParameterNames(ParamNumber,:)) '-' int2str(NumPriorDraws) '.txt']);
                  %
                  PrintDSGEMomentsResponse(ResFile,MomentsResp,ParameterNames,ParamNumber,0,[],0,NumPriorDraws,NumPriorDraws,DSGEModel,CurrINI);
                  %
                  % try to display it
                  %
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Parameter Change - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distributions',50,CurrINI,1,1);
                  end;
               end;
            end;
            clear('thetaDraws');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarCorrParameterChangePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % File with the data needed to calculate the autocorrelations
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % select which parameter to use
            %
            [action,ParamNumber] = SelectionDlg(ParameterNames,1,'Change parameter for gauging effects on population st. dev. and correlations:','Parameter Selection','Parameter Change - Posterior Mode','','',CurrINI);
            if strcmp(lower(action),'ok')==1;
               %
               % compute the responses to the mean, standard deviation and correlations of the observed variables from an increase
               % of the selected parameter
               %
               [MeanResp,StdResp,CorrResp] = DSGEMomentsResponseTheta(theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,0,'Posterior Mode',DSGEModel,CurrINI);
               if isempty(StdResp)==0;
                  %
                  % write the results to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-StateVarMomentsResponse-' StringTrim(ParameterNames(ParamNumber,:)) '-PostMode.txt']);
                  %
                  PrintDSGEMomentsResponseTheta(ResFile,MeanResp,StdResp,CorrResp,ParameterNames,ParamNumber,0,'Posterior Mode',DSGEModel,CurrINI);
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Moment Responses To Parameter Change - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode',50,CurrINI,1,1);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarCorrParameterChangePosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Parameter Change - Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  [action,ParamNumber] = SelectionDlg(ParameterNames,1,'Change parameter for gauging effects on population st. dev. and correlations:','Parameter Selection','Parameter Change - Posterior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     %
                     % check if we already have results
                     %
                     if DSGEMomentsResponseExist(ParameterNames,ParamNumber,0,CurrChain,1,UsedDraws,NumDraws+NumBurnin,DSGEModel,CurrINI)==1;
                        txt = ['YADA has located results for the effects on the standard deviations and correlations of the state variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" on disk. Would you like to use these results?'];
                        answer = Query(txt,'question',150,'Parameter Change - Posterior Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % compute the responses to the mean, standard deviation and correlations of the observed variables from an increase
                        % of the selected parameter
                        %
                        DoneCalc = DSGEMomentsResponse(thetaPostSample,theta,thetaPositions,ModelParameters,ParameterNames,ParamNumber,0,CurrChain,1,NumDraws+NumBurnin,DSGEModel,CurrINI);
                     end;
                     if DoneCalc==1;
                        %
                        % sort the results
                        %
                        MomentsResp = DSGEMomentsResponseDistribution(ParameterNames,ParamNumber,0,CurrChain,1,Weights,UsedDraws,NumDraws+NumBurnin,DSGEModel,CurrINI);
                        %
                        % ask if we should save results to disk
                        %
                        txt = ['Would you like to save the distributions for the effects on the standard deviations and correlations of the state variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" to file?'];
                        answer = Query(txt,'question',140,'Save - Parameter Change Posterior Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-StateVarMomentsResponse-' StringTrim(ParameterNames(ParamNumber,:)) '-' int2str(CurrChain) '-' int2str(UsedDraws) '.' int2str(NumDraws+NumBurnin) '.mat']);
                           save(MatFile,'MomentsResp');
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGESVMR' strrep(StringTrim(ParameterNames(ParamNumber,:)),'-','') int2str(CurrChain) int2str(UsedDraws) int2str(NumDraws+NumBurnin) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                           fprintf(fid,'%%\n%% structure with fields StdResp, CorrResp vector field Quantiles with subfields\n%% StdResp and CorrResp, and percent holding quantile data and quantile percent.\n%%\n');
                           fprintf(fid,'MomentsResp = DataStructure.MomentsResp;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The distributions of the effects on the standard deviations and the correlations for the state variables from a change to the parameter "' StringTrim(ParameterNames(ParamNumber,:)) '" have been saved to the file "' GetFilename(MatFile) '" in the directory "' GetPath(MatFile) '". The file contains 1 entry: MomentsResp (a structure with the effects on the standard deviations, correlations, and quantile data based on the posterior distribution). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Parameter Change - Posterior Distribution',200,500,CurrINI);
                        end;
                        %
                        % write the results to file
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-StateVarMomentsResponse-' StringTrim(ParameterNames(ParamNumber,:)) '-' int2str(CurrChain) '-' int2str(UsedDraws) '.' int2str(NumDraws+NumBurnin) '.txt']);
                        %
                        PrintDSGEMomentsResponse(ResFile,MomentsResp,ParameterNames,ParamNumber,0,CurrChain,1,UsedDraws,NumDraws+NumBurnin,DSGEModel,CurrINI);
                        %
                        % try to display it
                        %
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Parameter Change - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distributions',50,CurrINI,1,1);
                        end;
                     end;
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondCorrInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view correlations of the observed variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Observed Variable Correlations',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % ask if the user wants to simulate data
               %
               if NumPredPaths>1;
                  txt = 'Would you like to examine model based sample correlations rather than model based population correlations? Note: the model based sample correlations require simulation of data.';
                  answer = Query(txt,'question',150,'Conditional Correlations - Initial Values',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'no')==1;
                  SimulateData = 0;
                  NumPredPaths = 0;
               else;
                  SimulateData = 1;
               end;
               [CondCorr,mcode] = DSGEConditionalCorrsTheta(theta,thetaPositions,ModelParameters,NumPredPaths,'Initial Values',DSGEModel,CurrINI,SimulateData,FirstPeriod,LastPeriod);
               if mcode==1;
                  %
                  % check to make sure that CondCorr is not empty
                  %
                  if isempty(CondCorr)==0;
                     if SimulateData==1;
                        %
                        % ask if we should save the results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the conditional correlation distribution at the initial parameter values to file?';
                        answer = Query(txt,'question',140,'Save - Conditional Correlation Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\CondCorrelationDist-' int2str(NumPredPaths) '-InitialValues.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           ShockNames = CondCorr(1).ShockNames;
                           for i=1:length(CondCorr);
                              ConditionalCorrs(i).Mean = CondCorr(i).Mean;
                              ConditionalCorrs(i).Quantiles = CondCorr(i).Quantiles;
                           end;
                           %
                           save(file,'VariableNames','ShockNames','ConditionalCorrs');
                           %
                           % write an m-file
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\CCDist' int2str(NumPredPaths) 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% vector structure with fields Mean and Quantiles.\n%% The length of the vector structure is equal to the number of state shocks\n%%\n');
                           fprintf(fid,'ConditionalCorrs = DataStructure.ConditionalCorrs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the non-zero state shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The model based sample conditional correlations data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ConditionalCorrs (a vector structure with the model based mean and quantiles of the sample conditional correlations), ShockNames (string matrix with the names of the nonzero state shocks), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Conditional Correlation Distribution - Initial Values',200,500,CurrINI);
                        end;
                     end;
                     %
                     % we can now present the results
                     %
                     if NumPredPaths==0;
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\InitialValueCondCorr-' DSGEModel.NameOfModel '.txt']);
                     else;
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\InitialValueCondCorr-' DSGEModel.NameOfModel '-' int2str(NumPredPaths) '.txt']);
                     end;
                     PrintCondCorrelations(ResFile,DSGEModel,CondCorr,CurrINI,'initial','DSGE',NumPredPaths,1);
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,['Conditional Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                     end;
                  end;
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Conditional Correlation Distribution - Initial Values',200,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondCorrPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Conditional Correlation Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Conditional Correlations Distribution - Prior','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','Conditional Correlation Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % ask if the user wants to simulate data
               %
               txt = 'Would you like to examine model based sample conditional correlations (prior predictive densities) rather than model based population conditional correlations? Note: the model based sample conditional correlations require simulation of data, while the population conditional correlations only use draws from the prior distribution of the structural parameters.';
               answer = Query(txt,'question',150,'Conditional Correlations Distribution - Prior',500,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  SimulateData = 0;
                  NumPredPaths = 0;
               else;
                  SimulateData = 1;
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               end;
               %
               % First we check if the calculations have already been performed
               %
               if CondCorrExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,NumPredPaths,0)==1;
                  txt = 'YADA has located results for the conditional correlations on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'Conditional Correlations Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % start estimation
                  %
                  if SimulateData==0;
                     DoneCalc = DSGECondCorrEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,0,NumPriorDraws,1,DSGEModel,CurrINI);
                  else;
                     DoneCalc = DSGECondCorrSimulation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,0,NumPriorDraws,1,NumPredPaths,DSGEModel,CurrINI);
                  end;
               end;
               if DoneCalc==1;
                  %
                  % determine the distribution for the correlations
                  %
                  CorrData = DSGECondCorrDistribution(DSGEModel,CurrINI,ScenarioParameters,0,[],NumPriorDraws,NumPriorDraws,1,NumPredPaths);
                  %
                  % ask if we should save the results to disk
                  %
                  if NumPredPaths==0;
                     SampleTypeStr = 'Population';
                     AddFileName = '';
                  else;
                     SampleTypeStr = 'Sample';
                     AddFileName = ['-' int2str(NumPredPaths)];
                  end;
                  %
                  txt = ['Would you like to save the mean and quantiles of the ' lower(SampleTypeStr) '-based conditional correlation distribution to file?'];
                  answer = Query(txt,'question',140,'Save - Conditional Correlation Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\CondCorrelationPrior' SampleTypeStr 'Dist' AddFileName '-' int2str(NumPriorDraws) '.mat']);
                     VariableNames = DSGEModel.VariableNames;
                     ShockNames = CorrData(1).ShockNames;
                     for i=1:length(CorrData);
                        ConditionalCorrs(i).Mean = CorrData(i).Mean;
                        ConditionalCorrs(i).Quantiles = CorrData(i).Quantiles;
                     end;
                     %
                     save(file,'VariableNames','ShockNames','ConditionalCorrs');
                     %
                     % write an m-file
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\CCP' SampleTypeStr 'Dist' strrep(AddFileName,'-','') int2str(NumPriorDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% vector structure with fields Mean and Quantiles.\n%% The length of the vector structure is equal to the number of state shocks\n%%\n');
                     fprintf(fid,'ConditionalCorrs = DataStructure.ConditionalCorrs;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the non-zero state shocks\n%%\n');
                     fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The model based ' lower(SampleTypeStr) ' conditional correlations data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ConditionalCorrs (a vector structure with the model based mean and quantiles of the ' lower(SampleTypeStr) ' conditional correlations), ShockNames (string matrix with the names of the nonzero state shocks), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Conditional Correlation Distribution - Prior',200,500,CurrINI);
                  end;
                  %
                  % print the output to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\PriorDistCondCorr-' DSGEModel.NameOfModel AddFileName '.' int2str(NumPriorDraws) '.txt']);
                  PrintCondCorrelations(ResFile,DSGEModel,CorrData,CurrINI,'prior distribution','DSGE',NumPredPaths,NumPriorDraws);
                  %
                  if FileExist(ResFile)==1;
                     TextGUI(ResFile,['Conditional Correlations For Prior - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution Results',50,CurrINI,1,1);
                  end;
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the conditional correlation distribution.';
               About(txt,'information','Conditional Correlation Distribution - Prior',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondCorrPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the autocorrelations
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               if isfield(ModeData,'LastPeriod')==1;
%                  FirstPeriod = ModeData.FirstPeriod;
%                  LastPeriod = ModeData.LastPeriod;
%               else;
%                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               end;
%            else;
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % ask if the user wants to simulate data
            %
            if NumPredPaths>1;
               txt = 'Would you like to examine model based sample correlations rather than model based population correlations? Note: the model based sample correlations require simulation of data.';
               answer = Query(txt,'question',150,'Conditional Correlations - Posterior Mode',500,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'no')==1;
               SimulateData = 0;
               NumPredPaths = 0;
            else;
               SimulateData = 1;
            end;
            CondCorr = DSGEConditionalCorrsTheta(theta,thetaPositions,ModelParameters,NumPredPaths,'Posterior Mode',DSGEModel,CurrINI,SimulateData,FirstPeriod,LastPeriod);
            if isempty(CondCorr)==0;
               if SimulateData==1;
                  %
                  % ask if we should save the results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the conditional correlation distribution at the posterior mode parameter estimate to file?';
                  answer = Query(txt,'question',140,'Save - Conditional Correlation Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\CondCorrelationDist-' int2str(NumPredPaths) '-PosteriorMode.mat']);
                     VariableNames = DSGEModel.VariableNames;
                     ShockNames = CondCorr(1).ShockNames;
                     for i=1:length(CondCorr);
                        ConditionalCorrs(i).Mean = CondCorr(i).Mean;
                        ConditionalCorrs(i).Quantiles = CondCorr(i).Quantiles;
                     end;
                     %
                     save(file,'VariableNames','ShockNames','ConditionalCorrs');
                     %
                     % write an m-file
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\CCDist' int2str(NumPredPaths) 'PostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% vector structure with fields Mean and Quantiles.\n%% The length of the vector structure is equal to the number of state shocks\n%%\n');
                     fprintf(fid,'ConditionalCorrs = DataStructure.ConditionalCorrs;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the non-zero state shocks\n%%\n');
                     fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The model based sample conditional correlations data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ConditionalCorrs (a vector structure with the model based mean and quantiles of the sample conditional correlations), ShockNames (string matrix with the names of the nonzero state shocks), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Conditional Correlation Distribution - Posterior Mode',200,500,CurrINI);
                  end;
               end;
               %
               % we can now present the results
               %
               if NumPredPaths==0;
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeCondCorr-' DSGEModel.NameOfModel '.txt']);
               else;
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeCondCorr-' DSGEModel.NameOfModel '-' int2str(NumPredPaths) '.txt']);
               end;
               PrintCondCorrelations(ResFile,DSGEModel,CondCorr,CurrINI,'posterior mode','DSGE',NumPredPaths,1);
               if FileExist(ResFile)==1;
                  TextGUI(ResFile,['Conditional Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'CondCorrPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Conditional Correlation Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','Conditional Correlation Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % ask if the user wants to simulate data
                     %
                     txt = 'Would you like to examine model based sample conditional correlations (posterior predictive densities) rather than model based population conditional correlations? Note: the model based sample conditional correlations require simulation of data, while the population conditional correlations only use draws from the posterior distribution of the structural parameters.';
                     answer = Query(txt,'question',150,'Conditional Correlations Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        SimulateData = 0;
                        NumPredPaths = 0;
                     else;
                        SimulateData = 1;
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     end;
                     %
                     % First we check if the calculations have already been performed
                     %
                     if CondCorrExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,NumPredPaths,1)==1;
                        txt = 'YADA has located results for the conditional correlations on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'Conditional Correlations Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % start estimation
                        %
                        if SimulateData==0;
                           DoneCalc = DSGECondCorrEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,1,NumDraws,CurrChain,DSGEModel,CurrINI);
                        else;
                           DoneCalc = DSGECondCorrSimulation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,1,NumDraws,CurrChain,NumPredPaths,DSGEModel,CurrINI);
                        end;
                     end;
                     if DoneCalc==1;
                        %
                        % determine the distribution for the correlations
                        %
                        CorrData = DSGECondCorrDistribution(DSGEModel,CurrINI,ScenarioParameters,1,Weights,UsedDraws,NumDraws,CurrChain,NumPredPaths);
                        %
                        % ask if we should save the results to disk
                        %
                        if NumPredPaths==0;
                           SampleTypeStr = 'Population';
                           AddFileName = '';
                        else;
                           SampleTypeStr = 'Sample';
                           AddFileName = ['-' int2str(NumPredPaths)];
                        end;
                        %
                        txt = ['Would you like to save the mean and quantiles of the ' lower(SampleTypeStr) '-based conditional correlation distribution to file?'];
                        answer = Query(txt,'question',140,'Save - Conditional Correlation Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\CondCorrelation' SampleTypeStr 'Dist' AddFileName '-' int2str(UsedDraws) '.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           ShockNames = CorrData(1).ShockNames;
                           for i=1:length(CorrData);
                              ConditionalCorrs(i).Mean = CorrData(i).Mean;
                              ConditionalCorrs(i).Quantiles = CorrData(i).Quantiles;
                           end;
                           %
                           save(file,'VariableNames','ShockNames','ConditionalCorrs');
                           %
                           % write an m-file
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\CC' SampleTypeStr 'Dist' strrep(AddFileName,'-','') int2str(UsedDraws) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% vector structure with fields Mean and Quantiles.\n%% The length of the vector structure is equal to the number of state shocks\n%%\n');
                           fprintf(fid,'ConditionalCorrs = DataStructure.ConditionalCorrs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the non-zero state shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The model based ' lower(SampleTypeStr) ' conditional correlations data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: ConditionalCorrs (a vector structure with the model based mean and quantiles of the ' lower(SampleTypeStr) ' conditional correlations), ShockNames (string matrix with the names of the nonzero state shocks), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Conditional Correlation Distribution',200,500,CurrINI);
                        end;
                        %
                        % print the output to file
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorDistCondCorr-' DSGEModel.NameOfModel AddFileName '.' int2str(UsedDraws) '.txt']);
                        PrintCondCorrelations(ResFile,DSGEModel,CorrData,CurrINI,'posterior distribution','DSGE',NumPredPaths,UsedDraws);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Conditional Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution Results',50,CurrINI,1,1);
                        end;
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the conditional correlation distribution.';
                     About(txt,'information','Conditional Correlation Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarCorrInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view correlations of the state variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Observed Variable Correlations',500,'no',CurrINI);
         end;
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               %
               % Try to calculate autocorrelations. First, however, the code needs to check if a unique convergent
               % solution exists.
               %
               [CorrData,mcode] = CalculateDSGECorrelations(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,FirstPeriod,LastPeriod);
               if mcode==1;
                  %
                  % check to make sure that CorrData is not empty
                  %
                  if isempty(CorrData)==0;
                     %
                     % compute the covariance matrix of the smooth estimates of the states to check if the state variables
                     % can be observed via the model and the data or not.
                     %
                     [StateVarStructure,status,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
                     if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'PtT')==1);
                        CorrData.PtT = abs(diag(StateVarStructure.PtT(:,:,size(StateVarStructure.PtT,3))));
                     else;
                        CorrData.PtT = [];
                     end;
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save the population standard deviations and correlations of the state variables at initial values to file?';
                     answer = Query(txt,'question',140,'Save - State Variable Correlations',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        StDev = CorrData.StateVars.StDev;
                        Correlations = CorrData.StateVars.Correlations;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCorrelations-InitialValues.mat']);
                        save(file,'StateVariableNames','StDev','Correlations');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SVCInitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% vector with the population standard deviations of the state variables\n%%\n');
                        fprintf(fid,'StDev = DataStructure.StDev;\n');
                        fprintf(fid,'%%\n%% matrix with the population correlations of the state variables\n%%\n');
                        fprintf(fid,'Correlations = DataStructure.Correlations;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The population standard deviations and correlations of the state variables at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: StateVariableNames (string matrix with the names of the state variables), StDev (vector with the population standard deviations), and Correlations (matrix with the population correlations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','State Variable Correlations - Initial Values',300,500,CurrINI);
                     end;
                     %
                     % we can now present the results
                     %
                     PrintStateCorrelations(DSGEModel,CorrData,CurrINI,'initial');
                     ResFile = FixFilePath([DSGEModel.OutputDirectory '\InitialValueStateCorr-' DSGEModel.NameOfModel '.txt']);
                     if FileExist(ResFile)==1;
                        TextGUI(ResFile,['State Variable Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                     end;
                  end;
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','State Variable Correlations - Initial Values',120,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarCorrPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'State Variable Correlation Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','State Variable Correlation Distribution - Prior','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','State Variable Correlation Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % ask if the user wants to simulate data
               %
               txt = 'Would you like to examine model based sample moments (prior predictive densities) rather than model based population moments? Note: the model based sample moments require simulation of data, while the population moments only use draws from the prior distribution of the structural parameters.';
               answer = Query(txt,'question',150,'State Variable Correlation Distribution - Prior',500,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  SimulateData = 0;
                  NumPredPaths = 0;
               else;
                  SimulateData = 1;
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               end;
               %
               % First we check if the calculations have already been performed
               %
               if SVEExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,NumPredPaths,0,0,0)==1;
                  txt = 'YADA has located results for the state variables correlations on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'State Variable Correlation Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % start estimation
                  %
                  if SimulateData==0;
                     DoneCalc = DSGEStateVariableEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'State Variable Correlation',0,NumPriorDraws,1,DSGEModel,CurrINI);
                  else;
                     DoneCalc = DSGEStateVariableSimulation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'State Variable Correlation',0,NumPriorDraws,1,NumPredPaths,DSGEModel,CurrINI);
                  end;
               end;
               %
               % check if we should show any results
               %
               if DoneCalc==1;
                  %
                  % compute the distribution for the state variables
                  %
                  if SimulateData==0;
                     SVEs = DSGESVEDistribution(DSGEModel,CurrINI,6,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                     MeanStr = '';
                  else;
                     SVEs = DSGESVESimulatedDistribution(DSGEModel,CurrINI,6,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,NumPredPaths,0,[]);
                     MeanStr = ' sample mean,';
                  end;
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the state variable correlations distributions to file?';
                  answer = Query(txt,'question',140,'Save - State Variable Correlation Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCorrPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '-' int2str(NumPredPaths) '.mat']);
                     StateVariableNames = DSGEModel.StateVariableNames;
                     save(file,'SVEs','StateVariableNames');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SVCPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) int2str(NumPredPaths) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% structure with fields StdKsi (mean standard deviation), CorrKsi (mean correlations), and Quantiles.\n%%\n');
                     fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The state variable correlation data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the mean of the' MeanStr ' standard deviation, the correlations, and quantile data), and StateVariableNames (string matrix with the names of the state variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','State Variable Correlation Distribution - Prior',200,500,CurrINI);
                  end;
                  %
                  % print the results to text file
                  %
                  if NumPredPaths==0;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCorrDistPrior-' strrep(num2str(ScenarioParameters),' ','') '-' int2str(NumPriorDraws) '.txt']);
                  else;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCorrDistPrior-' strrep(num2str(ScenarioParameters),' ','') '-' int2str(NumPriorDraws) '-' int2str(NumPredPaths) '.txt']);
                  end;
                  %
                  PrintStateVarDistribution(file,SVEs,DSGEModel,CurrINI,1,NumPriorDraws,NumPriorDraws,1,NumPredPaths,0);
                  %
                  % display the results
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['State Variable Correlation Distributions For Prior - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Correlation Distributions',50,CurrINI,1,1);
                  end;
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the state variable correlation distribution.';
               About(txt,'information','State Variable Correlation Distribution',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVarCorrPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the autocorrelations
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               if isfield(ModeData,'LastPeriod')==1;
%                  FirstPeriod = ModeData.FirstPeriod;
%                  LastPeriod = ModeData.LastPeriod;
%               else;
%                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               end;
%            else;
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % compute the correlations
            %
            CorrData = CalculateDSGECorrelations(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,FirstPeriod,LastPeriod);
            if isempty(CorrData)==0;
               %
               % compute the covariance matrix of the smooth estimates of the states to check if the state variables
               % can be observed via the model and the data or not.
               %
               StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
               if isempty(StateVarStructure)==0;
                  CorrData.PtT = abs(diag(StateVarStructure.PtT(:,:,size(StateVarStructure.PtT,3))));
               else;
                  CorrData.PtT = [];
               end;
               %
               % ask if we should save to disk
               %
               txt = 'Would you like to save the population standard deviations and correlations of the state variables at posterior mode to file?';
               answer = Query(txt,'question',140,'Save - State Variable Correlations',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  StateVariableNames = DSGEModel.StateVariableNames;
                  StDev = CorrData.StateVars.StDev;
                  Correlations = CorrData.StateVars.Correlations;
                  file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCorrelations-PosteriorMode.mat']);
                  save(file,'StateVariableNames','StDev','Correlations');
                  %
                  % create a matlab file to read the data
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\SVCPostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                  fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                  fprintf(fid,'%%\n%% vector with the population standard deviations of the state variables\n%%\n');
                  fprintf(fid,'StDev = DataStructure.StDev;\n');
                  fprintf(fid,'%%\n%% matrix with the population correlations of the state variables\n%%\n');
                  fprintf(fid,'Correlations = DataStructure.Correlations;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  txt = ['The population standard deviations and correlations of the state variables at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: StateVariableNames (string matrix with the names of the state variables), StDev (vector with the population standard deviations), and Correlations (matrix with the population correlations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','State Variable Correlations - Posterior Mode',300,500,CurrINI);
               end;
               %
               % we can now present the results
               %
               PrintStateCorrelations(DSGEModel,CorrData,CurrINI,'posterior mode');
               ResFile = FixFilePath([DSGEModel.OutputDirectory '\PosteriorModeStateCorr-' DSGEModel.NameOfModel '.txt']);
               if FileExist(ResFile)==1;
                  TextGUI(ResFile,['State Variable Correlations - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'StateVarCorrPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','State Variable Correlation Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','State Variable Correlation Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % ask if the user wants to simulate data
                     %
                     txt = 'Would you like to examine model based sample moments (posterior predictive densities) rather than model based population moments? Note: the model based sample moments require simulation of data, while the population moments only use draws from the posterior distribution of the structural parameters.';
                     answer = Query(txt,'question',150,'State Variable Correlation Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        SimulateData = 0;
                        NumPredPaths = 0;
                     else;
                        SimulateData = 1;
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     end;
                     %
                     % First we check if the calculations have already been performed
                     %
                     if SVEExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,NumPredPaths,0,1,0)==1;
                        txt = 'YADA has located results for the state variables correlations on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'State Variable Correlation Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % start estimation
                        %
                        if SimulateData==0;
                           DoneCalc = DSGEStateVariableEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'State Variable Correlation',1,NumDraws,CurrChain,DSGEModel,CurrINI);
                        else;
                           DoneCalc = DSGEStateVariableSimulation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'State Variable Correlation',1,NumDraws,CurrChain,NumPredPaths,DSGEModel,CurrINI);
                        end;
                     end;
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % compute the distribution for the state variables
                        %
                        if SimulateData==0;
                           SVEs = DSGESVEDistribution(DSGEModel,CurrINI,6,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights);
                           MeanStr = '';
                        else;
                           SVEs = DSGESVESimulatedDistribution(DSGEModel,CurrINI,6,ScenarioParameters,UsedDraws,NumDraws,CurrChain,NumPredPaths,1,Weights);
                           MeanStr = ' sample mean,';
                        end;
                        %
                        % ask if we should save results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the state variable correlations distributions to file?';
                        answer = Query(txt,'question',140,'Save - State Variable Correlation Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                           file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCorrDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                           StateVariableNames = DSGEModel.StateVariableNames;
                           save(file,'SVEs','StateVariableNames');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SVCDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields StdKsi (mean standard deviation), CorrKsi (mean correlations), and Quantiles.\n%%\n');
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The state variable correlation data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the mean of the' MeanStr ' standard deviation, the correlations, and quantile data), and StateVariableNames (string matrix with the names of the state variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','State Variable Correlation Distribution',200,500,CurrINI);
                        end;
                        %
                        % print the results to text file
                        %
                        if NumPredPaths==0;
                           file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCorrDist-' strrep(num2str(ScenarioParameters),' ','') '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.txt']);
                        else;
                           file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCorrDist-' strrep(num2str(ScenarioParameters),' ','') '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '-' int2str(NumPredPaths) '.txt']);
                        end;
                        %
                        PrintStateVarDistribution(file,SVEs,DSGEModel,CurrINI,1,UsedDraws,NumDraws,CurrChain,NumPredPaths,1);
                        %
                        % display the results
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['State Variable Correlation Distributions - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Correlation Distributions',50,CurrINI,1,1);
                        end;
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the state variable correlation distribution.';
                     About(txt,'information','State Variable Correlation Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShockCorrInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view correlations of the state shocks before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Project State Shocks at Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               %
               % Try to calculate state variables. First, however, the code needs to check if a unique convergent
               % solution exists.
               %
               [StateVarStructure,mcode,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
               if (mcode==1)&(kalmanstatus==0)&(isfield(StateVarStructure,'etatT')==1);
                  %
                  % compute the asymptotic shock covariances
                  %
                  [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
                  [AsSigmaUpdate,AsSigmaSmooth,status,RiccatiResults,UniquenessCheck] = DSGEAsymptoticShockCovariances(StateVarStructure.H,StateVarStructure.R,StateVarStructure.F,StateVarStructure.B0,DSGEModel.LastPeriod,size(StateVarStructure.etatt,2),DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.RiccatiMaxIterations,DSGEModel.RiccatiToleranceValue);
                  %
                  % compute the correlations
                  %
                  txt = 'Would you like to view the sample and population evidence on the correlations for the structural shocks based on the update estimates instead of the smooth estimates?';
                  answer = Query(txt,'question',250,'State Shock Correlations - Estimator',600,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     etatt = StateVarStructure.etatt;
                     T = size(etatt,2);
                     SampleMean = (1/T)*sum(etatt,2);
                     etatt = etatt-SampleMean*ones(1,T);
                     CovarianceMatrix = (1/T)*etatt*etatt';
                     NttAverage = (1/T)*sum(StateVarStructure.Ntt,3);
                     B0 = StateVarStructure.B0;
                     AvTheoCovarianceMatrix = B0'*NttAverage*B0;
                     AvTheoCovarianceMatrix = AvTheoCovarianceMatrix(StateVarStructure.KeepVar,StateVarStructure.KeepVar);
                     EstType = 'Update';
                     if status==0;
                        AsympCovarianceMatrix = AsSigmaUpdate;
                        AsympCovarianceMatrix = AsympCovarianceMatrix(StateVarStructure.KeepVar,StateVarStructure.KeepVar);
                     end;
                  else;
                     etatT = StateVarStructure.etatT;
                     T = size(etatT,2);
                     SampleMean = (1/T)*sum(etatT,2);
                     etatT = etatT-SampleMean*ones(1,T);
                     CovarianceMatrix = (1/T)*etatT*etatT';
                     NtMatAverage = (1/T)*sum(StateVarStructure.NtMat,3);
                     B0 = StateVarStructure.B0;
                     AvTheoCovarianceMatrix = B0'*NtMatAverage*B0;
                     AvTheoCovarianceMatrix = AvTheoCovarianceMatrix(StateVarStructure.KeepVar,StateVarStructure.KeepVar);
                     EstType = 'Smooth';
                     if status==0;
                        AsympCovarianceMatrix = (1/T)*sum(AsSigmaSmooth,3);
                        AsympCovarianceMatrix = AsympCovarianceMatrix(StateVarStructure.KeepVar,StateVarStructure.KeepVar);
                     end;
                  end;
                  Correlations = zeros(size(CovarianceMatrix));
                  StDev = sqrt(diag(diag(CovarianceMatrix)));
                  AvTheoCorrelations = Correlations;
                  AvTheoStd = sqrt(diag(diag(AvTheoCovarianceMatrix)));
                  if status==0;
                     AsympCorrelations = Correlations;
                     AsympStd = sqrt(diag(diag(AsympCovarianceMatrix)));
                  end;
                  %
                  StateShockNames = DSGEModel.StateShockNames(StateVarStructure.KeepVar,:);
                  %
                  % prepare output for display
                  %
                  prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                  prt_str = '%s';
                  for i=1:length(StateVarStructure.KeepVar);
                     prt_str = [prt_str '     ' prt_val];
                     for j=1:length(StateVarStructure.KeepVar);
                        Correlations(i,j) = CovarianceMatrix(i,j)/(StDev(i,i)*StDev(j,j));
                        AvTheoCorrelations(i,j) = AvTheoCovarianceMatrix(i,j)/(AvTheoStd(i,i)*AvTheoStd(j,j));
                        if status==0;
                           AsympCorrelations(i,j) = AsympCovarianceMatrix(i,j)/(AsympStd(i,i)*AsympStd(j,j));
                        end;
                     end;
                  end;
                  %
                  txt = ['Would you like to save to file the sample estimates of the first and second moments of the state shocks based on the ' lower(EstType) ' estimates at initial values?'];
                  answer = Query(txt,'question',140,'Save - State Shock Correlations',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     StDev = diag(StDev);
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateShockCorrelations-' EstType '-InitialValues.mat']);
                     if status==0;
                        save(file,'StateShockNames','SampleMean','StDev','Correlations','AvTheoCovarianceMatrix','AvTheoCorrelations','AsympCovarianceMatrix','AsympCorrelations');
                     else;
                        save(file,'StateShockNames','SampleMean','StDev','Correlations','AvTheoCovarianceMatrix','AvTheoCorrelations');
                     end;
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SSC' EstType 'InitVal.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the non-zero economic shocks\n%%\n');
                     fprintf(fid,'StateShockNames = DataStructure.StateShockNames;\n');
                     fprintf(fid,'%%\n%% vector with the sample mean for the smooth estimates\n%%\n');
                     fprintf(fid,'SampleMean = DataStructure.SampleMean;\n');
                     fprintf(fid,'%%\n%% vector with the sample standard deviation\n%%\n');
                     fprintf(fid,'StDev = DataStructure.StDev;\n');
                     fprintf(fid,'%%\n%% matrix with the sample correlations\n%%\n');
                     fprintf(fid,'Correlations = DataStructure.Correlations;\n');
                     fprintf(fid,'%%\n%% matrix with the average population covariances\n%%\n');
                     fprintf(fid,'AvTheoCovarianceMatrix = DataStructure.AvTheoCovarianceMatrix;\n');
                     fprintf(fid,'%%\n%% matrix with the average population correlations\n%%\n');
                     fprintf(fid,'AvTheoCorrelations = DataStructure.AvTheoCorrelations;\n');
                     if status==0;
                        fprintf(fid,'%%\n%% matrix with the average asymptotic population covariances\n%%\n');
                        fprintf(fid,'AsympCovarianceMatrix = DataStructure.AsympCovarianceMatrix;\n');
                        fprintf(fid,'%%\n%% matrix with the average asymptotic population correlations\n%%\n');
                        fprintf(fid,'AsympCorrelations = DataStructure.AsympCorrelations;\n');
                     end;
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     if status==0;
                        txt = ['The sample estimates of the first and second moments of the state shocks for the ' lower(EstType) ' estimator at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 8 entries: StateShockNames (string matrix with the names of the non-zero economic shocks), SampleMean (vector with the sample mean), StDev (vector with the sample standard deviation), Correlations (matrix with the sample correlations), AvTheoCovarianceMatrix (matrix with the average population covariances), AvTheoCorrelations (matrix with the average population correlations), AsympCovarianceMatrix (matrix with the average asymptotic population covariances), and AsympCorrelations (matrix with the average asymptotic population correlations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     else;
                        txt = ['The sample estimates of the first and second moments of the state shocks for the ' lower(EstType) ' estimator at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 6 entries: StateShockNames (string matrix with the names of the non-zero economic shocks), SampleMean (vector with the sample mean), StDev (vector with the sample standard deviation), Correlations (matrix with the sample correlations), AvTheoCovarianceMatrix (matrix with the average population covariances), and AvTheoCorrelations (matrix with the average population correlations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     end;
                     About(txt,'information','State Shock Correlations - Initial Values',250,600,CurrINI);
                  end;
                  %
                  txt = strvcat('Sample Mean matrix:','-------------------',' ');
                  for i=1:length(StateVarStructure.KeepVar);
                     txt = strvcat(txt,sprintf(['%s     ' prt_val],StateShockNames(i,:),SampleMean(i)));
                  end;
                  %
                  txt = strvcat(txt,' ','Sample Covariance matrix:','-------------------------',' ');
                  for i=1:length(StateVarStructure.KeepVar);
                     txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),CovarianceMatrix(:,i)));
                  end;
                  txt = strvcat(txt,' ','Average Population Covariance matrix:','-------------------------------------',' ');
                  ObsShocks = '';
                  for i=1:length(StateVarStructure.KeepVar);
                     txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),AvTheoCovarianceMatrix(:,i)));
                     if abs(AvTheoCovarianceMatrix(i,i)-1)<1.0e-10;;
                        ObsShocks = [ObsShocks StateShockNames(i,:) ', '];
                     end;
                  end;
                  if status==0;
                     txt = strvcat(txt,' ','Asymptotic Population Covariance matrix:','----------------------------------------',' ');
                     for i=1:length(StateVarStructure.KeepVar);
                        txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),AsympCovarianceMatrix(:,i)));
                     end;
                  end;
                  if isempty(ObsShocks)==0;
                     ObsShocks = ObsShocks(1:length(ObsShocks)-2);
                     txt = strvcat(txt,' ','NOTE: The following shocks can be observed given the data and the model parameters:',' ',['         ' ObsShocks],' ');
                  end;
                  txt = strvcat(txt,' ','Sample Correlation matrix:','--------------------------',' ');
                  for i=1:length(StateVarStructure.KeepVar);
                     txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),Correlations(:,i)));
                  end;
                  txt = strvcat(txt,' ','Average Population Correlation matrix:','--------------------------------------',' ');
                  for i=1:length(StateVarStructure.KeepVar);
                     txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),AvTheoCorrelations(:,i)));
                  end;
                  %
                  if status==0;
                     txt = strvcat(txt,' ','Asymptotic Population Correlation matrix:','-----------------------------------------',' ');
                     for i=1:length(StateVarStructure.KeepVar);
                        txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),AsympCorrelations(:,i)));
                     end;
                  end;
                  %
                  % add information about the largest deviations (greater than 0.3) between the sample estimates
                  % and the theoretical, if any such exist.
                  %
                  AddText = '';
                  if size(StateShockNames,2)<20;
                     AddSpace = SpaceStr(20-size(StateShockNames,2));
                  else;
                     AddSpace = '';
                  end;
                  if status==0;
                     for i=1:length(StateVarStructure.KeepVar);
                        for j=i+1:length(StateVarStructure.KeepVar);
                           if abs(Correlations(j,i)-AsympCorrelations(j,i))>=0.3;
                              if isempty(AddText)==1;
                                 AddText = strvcat('=================================================================================================','                 shock pair             sample and asymptotic correlations   absolute difference');
                              end;
                              AddText = strvcat(AddText,'-------------------------------------------------------------------------------------------------',sprintf(' %s%s, %s%s   %9.6f   %9.6f                %8.6f',AddSpace,StateShockNames(i,:),StateShockNames(j,:),AddSpace,Correlations(j,i),AsympCorrelations(j,i),abs(Correlations(j,i)-AsympCorrelations(j,i))));
                           end;
                        end;
                     end;
                     if isempty(AddText)==1;
                        AddText = strvcat('NOTE: The differences between the sample correlations and the asymptotic are all','      less than 0.3.');
                     else;
                        AddText = strvcat(AddText,'=================================================================================================',' ','NOTE: Only the absolute differences greater than or equal to 0.3 are shown.');
                     end;
                  end;
                  AddTextTwo = '';
                  for i=1:length(StateVarStructure.KeepVar);
                     for j=i+1:length(StateVarStructure.KeepVar);
                        if abs(Correlations(j,i)-AvTheoCorrelations(j,i))>=0.3;
                           if isempty(AddTextTwo)==1;
                              AddTextTwo = strvcat('=================================================================================================','                 shock pair           sample and average theoretical corrs   absolute difference');
                           end;
                           AddTextTwo = strvcat(AddTextTwo,'-------------------------------------------------------------------------------------------------',sprintf(' %s%s, %s%s   %9.6f   %9.6f                %8.6f',AddSpace,StateShockNames(i,:),StateShockNames(j,:),AddSpace,Correlations(j,i),AvTheoCorrelations(j,i),abs(Correlations(j,i)-AvTheoCorrelations(j,i))));
                        end;
                     end;
                  end;
                  if isempty(AddTextTwo)==1;
                     AddTextTwo = strvcat('NOTE: The differences between the sample correlations and the average theoretical','      are all less than 0.3.');
                  else;
                     AddTextTwo = strvcat(AddTextTwo,'=================================================================================================',' ','NOTE: Only the absolute differences greater than or equal to 0.3 are shown.');
                  end;
                  txt = strvcat(txt,' ',AddTextTwo);
                  if status==0;
                     txt = strvcat(txt,' ',AddText);
                  end;
                  %
                  % print to file
                  %
                  ResFile = FixFilePath([DSGEModel.OutputDirectory '\StateShockCorr-' EstType '-InitialValues.txt']);
                  fid = fopen(ResFile,'wt');
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*                  S A M P L E   A N D   P O P U L A T I O N                   *\n');
                  fprintf(fid,'*      S T A T E   S H O C K   M E A N S   A N D   C O V A R I A N C E S       *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  fprintf(fid,'NOTE: Using the %s estimates of the structural shocks at initial values.\n\n',lower(EstType));
                  %
                  for i=1:size(txt,1);
                     fprintf(fid,'%s\n',deblank(txt(i,:)));
                  end;
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'\n********************************************************************************\n\n');
                  fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  TextGUI(ResFile,['State Shocks - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                  %
               elseif (mcode==1)&(kalmanstatus~=0);
                  if kalmanstatus==1;
                     txt = 'The state variables are not stationary at the initial values.';
                  else;
                     txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                  end;
                  About(txt,'information','State Shock Correlations - Initial Values',200,500,CurrINI);
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','State Shock Correlations - Initial Values',200,500,CurrINI);
               elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==1);
                  txt = 'YADA couldn''t estimate the state shocks. Please make sure that your model has state shocks!';
                  About(txt,'information','YADA - State Shocks',120,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShockCorrPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'State Shock Correlation Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','State Shock Correlation Distribution - Prior','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','State Shock Correlation Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % First we check if the calculations have already been performed
               %
               if SVEExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,0,0,0)==1;
                  txt = 'YADA has located results for the state shock correlations on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'State Shock Correlation Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % start estimation
                  %
                  DoneCalc = DSGEStateVariableEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'State Shock Correlation',0,NumPriorDraws,1,DSGEModel,CurrINI);
               end;
               %
               % check if we should show any results
               %
               if DoneCalc==1;
                  %
                  % compute the distribution for the state shock correlations
                  %
                  SVEs = DSGESVEDistribution(DSGEModel,CurrINI,7,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the state shock correlation distributions to file?';
                  answer = Query(txt,'question',140,'Save - State Shock Correlation Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateShockCorrPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.mat']);
                     save(file,'SVEs');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SSCPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     if isfield(SVEs,'AvTheoStdeta')==1;
                        fprintf(fid,'%%\n%% structure with fields Meaneta (mean), Stdeta (sample standard deviation), Correta (matrix with sample\n%% correlations), AvTheoStdeta (average population standard deviation), AvTheoCorreta (matrix with\n%% average population correlations), Quantiles, and ShockNames.\n%%\n');
                     else;
                        fprintf(fid,'%%\n%% structure with fields Meaneta (mean), Stdeta (sample standard deviation), Correta (matrix with sample correlations), Quantiles, and ShockNames.\n%%\n');
                     end;
                     fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The state shock data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: SVEs (a structure with the average shocks, standard deviation, correlations, quantile data, and names of shocks). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','State Shock Correlation Distribution - Prior',200,500,CurrINI);
                  end;
                  %
                  % print the results to text file
                  %
                  file = FixFilePath([DSGEModel.OutputDirectory '\StateShockCorrDistPrior-' strrep(num2str(ScenarioParameters),' ','') '-' int2str(NumPriorDraws) '.txt']);
                  %
                  PrintStateVarDistribution(file,SVEs,DSGEModel,CurrINI,2,NumPriorDraws,NumPriorDraws,1,0,0);
                  %
                  % display the results
                  %
                  if FileExist(file)==1;
                     TextGUI(file,['State Shock Correlation Distributions For Prior - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Correlation Distributions',50,CurrINI,1,1);
                  end;
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the state shock distribution.';
               About(txt,'information','State Shock Correlation Distribution - Prior',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShockCorrPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % we can now calculate the state shocks
            %
            StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
            if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'etatT')==1);
               %
               % compute the asymptotic shock covariances
               %
               [DSGEModel.FirstPeriod,DSGEModel.LastPeriod] = CreateSubSample(DSGEModel);
               [AsSigmaUpdate,AsSigmaSmooth,status,RiccatiResults,UniquenessCheck] = DSGEAsymptoticShockCovariances(StateVarStructure.H,StateVarStructure.R,StateVarStructure.F,StateVarStructure.B0,DSGEModel.LastPeriod,size(StateVarStructure.etatt,2),DSGEModel.DAMaximumIterationsValue,DSGEModel.DAToleranceValue,DSGEModel.RiccatiMaxIterations,DSGEModel.RiccatiToleranceValue);
               %
               % compute the correlations
               %
               txt = 'Would you like to view the sample and population evidence on the correlations for the structural shocks based on the update estimates instead of the smooth estimates?';
               answer = Query(txt,'question',250,'State Shock Correlations - Estimator',600,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  etatt = StateVarStructure.etatt;
                  T = size(etatt,2);
                  SampleMean = (1/T)*sum(etatt,2);
                  etatt = etatt-SampleMean*ones(1,T);
                  CovarianceMatrix = (1/T)*etatt*etatt';
                  NttAverage = (1/T)*sum(StateVarStructure.Ntt,3);
                  B0 = StateVarStructure.B0;
                  AvTheoCovarianceMatrix = B0'*NttAverage*B0;
                  AvTheoCovarianceMatrix = AvTheoCovarianceMatrix(StateVarStructure.KeepVar,StateVarStructure.KeepVar);
                  EstType = 'Update';
                  if status==0;
                     AsympCovarianceMatrix = AsSigmaUpdate;
                     AsympCovarianceMatrix = AsympCovarianceMatrix(StateVarStructure.KeepVar,StateVarStructure.KeepVar);
                  end;
               else;
                  etatT = StateVarStructure.etatT;
                  T = size(etatT,2);
                  SampleMean = (1/T)*sum(etatT,2);
                  etatT = etatT-SampleMean*ones(1,T);
                  CovarianceMatrix = (1/T)*etatT*etatT';
                  NtMatAverage = (1/T)*sum(StateVarStructure.NtMat,3);
                  B0 = StateVarStructure.B0;
                  AvTheoCovarianceMatrix = B0'*NtMatAverage*B0;
                  AvTheoCovarianceMatrix = AvTheoCovarianceMatrix(StateVarStructure.KeepVar,StateVarStructure.KeepVar);
                  EstType = 'Smooth';
                  if status==0;
                     AsympCovarianceMatrix = (1/T)*sum(AsSigmaSmooth,3);
                     AsympCovarianceMatrix = AsympCovarianceMatrix(StateVarStructure.KeepVar,StateVarStructure.KeepVar);
                  end;
               end;
               Correlations = zeros(size(CovarianceMatrix));
               StDev = sqrt(diag(diag(CovarianceMatrix)));
               AvTheoCorrelations = Correlations;
               AvTheoStd = sqrt(diag(diag(AvTheoCovarianceMatrix)));
               if status==0;
                  AsympCorrelations = Correlations;
                  AsympStd = sqrt(diag(diag(AsympCovarianceMatrix)));
               end;
               %
               StateShockNames = DSGEModel.StateShockNames(StateVarStructure.KeepVar,:);
               %
               % prepare output for display
               %
               prt_val = ['%' int2str(6+CurrINI.decimals) '.' int2str(CurrINI.decimals) 'f'];
               prt_str = '%s';
               for i=1:length(StateVarStructure.KeepVar);
                  prt_str = [prt_str '     ' prt_val];
                  for j=1:length(StateVarStructure.KeepVar);
                     Correlations(i,j) = CovarianceMatrix(i,j)/(StDev(i,i)*StDev(j,j));
                     AvTheoCorrelations(i,j) = AvTheoCovarianceMatrix(i,j)/(AvTheoStd(i,i)*AvTheoStd(j,j));
                     if status==0;
                        AsympCorrelations(i,j) = AsympCovarianceMatrix(i,j)/(AsympStd(i,i)*AsympStd(j,j));
                     end;
                  end;
               end;
               %
               txt = ['Would you like to save to file the sample estimates of the first and second moments of the state shocks based on the ' lower(EstType) ' estimates at posterior mode?'];
               answer = Query(txt,'question',140,'Save - State Shock Correlations',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  StDev = diag(StDev);
                  file = FixFilePath([DSGEModel.OutputDirectory '\StateShockCorrelations-' EstType '-PosteriorMode.mat']);
                  if status==0;
                     save(file,'StateShockNames','SampleMean','StDev','Correlations','AvTheoCovarianceMatrix','AvTheoCorrelations','AsympCovarianceMatrix','AsympCorrelations');
                  else;
                     save(file,'StateShockNames','SampleMean','StDev','Correlations','AvTheoCovarianceMatrix','AvTheoCorrelations');
                  end;
                  %
                  % create a matlab file to read the data
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\SSC' EstType 'PostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the non-zero economic shocks\n%%\n');
                  fprintf(fid,'StateShockNames = DataStructure.StateShockNames;\n');
                  fprintf(fid,'%%\n%% vector with the sample mean for the %s estimates\n%%\n',lower(EstType));
                  fprintf(fid,'SampleMean = DataStructure.SampleMean;\n');
                  fprintf(fid,'%%\n%% vector with the sample standard deviation\n%%\n');
                  fprintf(fid,'StDev = DataStructure.StDev;\n');
                  fprintf(fid,'%%\n%% matrix with the sample correlations\n%%\n');
                  fprintf(fid,'Correlations = DataStructure.Correlations;\n');
                  fprintf(fid,'%%\n%% matrix with the average population covariances\n%%\n');
                  fprintf(fid,'AvTheoCovarianceMatrix = DataStructure.AvTheoCovarianceMatrix;\n');
                  fprintf(fid,'%%\n%% matrix with the average population correlations\n%%\n');
                  fprintf(fid,'AvTheoCorrelations = DataStructure.AvTheoCorrelations;\n');
                  if status==0;
                     fprintf(fid,'%%\n%% matrix with the average asymptotic population covariances\n%%\n');
                     fprintf(fid,'AsympCovarianceMatrix = DataStructure.AsympCovarianceMatrix;\n');
                     fprintf(fid,'%%\n%% matrix with the average asymptotic population correlations\n%%\n');
                     fprintf(fid,'AsympCorrelations = DataStructure.AsympCorrelations;\n');
                  end;
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  if status==0;
                     txt = ['The sample estimates of the first and second moments of the state shocks for the ' lower(EstType) ' estimator at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 8 entries: StateShockNames (string matrix with the names of the non-zero economic shocks), SampleMean (vector with the sample mean), StDev (vector with the sample standard deviation), Correlations (matrix with the sample correlations), AvTheoCovarianceMatrix (matrix with the average population covariances), AvTheoCorrelations (matrix with the average population correlations), AsympCovarianceMatrix (matrix with the average asymptotic population covariances), and AsympCorrelations (matrix with the average asymptotic population correlations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  else;
                     txt = ['The sample estimates of the first and second moments of the state shocks for the ' lower(EstType) ' estimator at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 6 entries: StateShockNames (string matrix with the names of the non-zero economic shocks), SampleMean (vector with the sample mean), StDev (vector with the sample standard deviation), Correlations (matrix with the sample correlations), AvTheoCovarianceMatrix (matrix with the average population covariances), and AvTheoCorrelations (matrix with the average population correlations). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  end;
                  About(txt,'information','State Shock Correlations - Posterior Mode',200,500,CurrINI);
               end;
               %
               txt = strvcat('Sample Mean matrix:','-------------------',' ');
               for i=1:length(StateVarStructure.KeepVar);
                  txt = strvcat(txt,sprintf(['%s     ' prt_val],StateShockNames(i,:),SampleMean(i)));
               end;
               %
               txt = strvcat(txt,' ','Sample Covariance matrix:','-------------------------',' ');
               for i=1:length(StateVarStructure.KeepVar);
                  txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),CovarianceMatrix(:,i)));
               end;
               txt = strvcat(txt,' ','Average Population Covariance matrix:','-------------------------------------',' ');
               ObsShocks = '';
               for i=1:length(StateVarStructure.KeepVar);
                  txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),AvTheoCovarianceMatrix(:,i)));
                  if abs(AvTheoCovarianceMatrix(i,i)-1)<1.0e-10;;
                     ObsShocks = [ObsShocks StateShockNames(i,:) ', '];
                  end;
               end;
               if status==0;
                  txt = strvcat(txt,' ','Asymptotic Population Covariance matrix:','----------------------------------------',' ');
                  for i=1:length(StateVarStructure.KeepVar);
                     txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),AsympCovarianceMatrix(:,i)));
                  end;
               end;
               if isempty(ObsShocks)==0;
                  ObsShocks = ObsShocks(1:length(ObsShocks)-2);
                  txt = strvcat(txt,' ','NOTE: The following shocks can be observed given the data and the model parameters:',' ',['         ' ObsShocks],' ');
               end;
               txt = strvcat(txt,' ','Sample Correlation matrix:','--------------------------',' ');
               for i=1:length(StateVarStructure.KeepVar);
                  txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),Correlations(:,i)));
               end;
               txt = strvcat(txt,' ','Average Population Correlation matrix:','--------------------------------------',' ');
               for i=1:length(StateVarStructure.KeepVar);
                  txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),AvTheoCorrelations(:,i)));
               end;
               %
               if status==0;
                  txt = strvcat(txt,' ','Asymptotic Population Correlation matrix:','-----------------------------------------',' ');
                  for i=1:length(StateVarStructure.KeepVar);
                     txt = strvcat(txt,sprintf(prt_str,StateShockNames(i,:),AsympCorrelations(:,i)));
                  end;
               end;
               %
               % add information about the largest deviations (greater than 0.3) between the sample estimates
               % and the theoretical, if any such exist.
               %
               AddText = '';
               if size(StateShockNames,2)<20;
                  AddSpace = SpaceStr(20-size(StateShockNames,2));
               else;
                  AddSpace = '';
               end;
               if status==0;
                  for i=1:length(StateVarStructure.KeepVar);
                     for j=i+1:length(StateVarStructure.KeepVar);
                        if abs(Correlations(j,i)-AsympCorrelations(j,i))>=0.3;
                           if isempty(AddText)==1;
                              AddText = strvcat('=================================================================================================','                 shock pair             sample and asymptotic correlations   absolute difference');
                           end;
                           AddText = strvcat(AddText,'-------------------------------------------------------------------------------------------------',sprintf(' %s%s, %s%s   %9.6f   %9.6f                %8.6f',AddSpace,StateShockNames(i,:),StateShockNames(j,:),AddSpace,Correlations(j,i),AsympCorrelations(j,i),abs(Correlations(j,i)-AsympCorrelations(j,i))));
                        end;
                     end;
                  end;
                  if isempty(AddText)==1;
                     AddText = strvcat('NOTE: The differences between the sample correlations and the asymptotic are all','      less than 0.3.');
                  else;
                     AddText = strvcat(AddText,'=================================================================================================',' ','NOTE: Only the absolute differences greater than or equal to 0.3 are shown.');
                  end;
               end;
               AddTextTwo = '';
               for i=1:length(StateVarStructure.KeepVar);
                  for j=i+1:length(StateVarStructure.KeepVar);
                     if abs(Correlations(j,i)-AvTheoCorrelations(j,i))>=0.3;
                        if isempty(AddTextTwo)==1;
                           AddTextTwo = strvcat('=================================================================================================','                 shock pair           sample and average theoretical corrs   absolute difference');
                        end;
                        AddTextTwo = strvcat(AddTextTwo,'-------------------------------------------------------------------------------------------------',sprintf(' %s%s, %s%s   %9.6f   %9.6f                %8.6f',AddSpace,StateShockNames(i,:),StateShockNames(j,:),AddSpace,Correlations(j,i),AvTheoCorrelations(j,i),abs(Correlations(j,i)-AvTheoCorrelations(j,i))));
                     end;
                  end;
               end;
               if isempty(AddTextTwo)==1;
                  AddTextTwo = strvcat('NOTE: The differences between the sample correlations and the average theoretical','      are all less than 0.3.');
               else;
                  AddTextTwo = strvcat(AddTextTwo,'=================================================================================================',' ','NOTE: Only the absolute differences greater than or equal to 0.3 are shown.');
               end;
               txt = strvcat(txt,' ',AddTextTwo);
               if status==0;
                  txt = strvcat(txt,' ',AddText);
               end;
               %
               % print to file
               %
               ResFile = FixFilePath([DSGEModel.OutputDirectory '\StateShockCorr-' EstType '-PostMode.txt']);
               fid = fopen(ResFile,'wt');
               %
               fprintf(fid,'********************************************************************************\n');
               fprintf(fid,'*                                                                              *\n');
               fprintf(fid,'*                  S A M P L E   A N D   P O P U L A T I O N                   *\n');
               fprintf(fid,'*      S T A T E   S H O C K   M E A N S   A N D   C O V A R I A N C E S       *\n');
               fprintf(fid,'*                                                                              *\n');
               fprintf(fid,'********************************************************************************\n\n');
               fprintf(fid,'NOTE: Using the %s estimates of the structural shocks at posterior mode.\n\n',lower(EstType));
               %
               for i=1:size(txt,1);
                  fprintf(fid,'%s\n',deblank(txt(i,:)));
               end;
               %
               % Some information about the filename and its location
               %
               fprintf(fid,'\n********************************************************************************\n\n');
               fprintf(fid,'Directory for file: %s\n',GetPath(ResFile));
               fprintf(fid,'Name of file:       %s\n',GetFilename(ResFile));
               fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
               fclose(fid);
               %
               TextGUI(ResFile,['State Shocks - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
            elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==1);
               txt = 'YADA couldn''t estimate the state shocks. Please make sure that your model has state shocks!';
               About(txt,'information','YADA - State Shocks',120,500,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'ShockCorrPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','State Shock Correlation Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','State Shock Correlation Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % First we check if the calculations have already been performed
                     %
                     if SVEExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,0,0,1,0)==1;
                        txt = 'YADA has located results for the state shock correlations on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'State Shock Correlation Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % start estimation
                        %
                        DoneCalc = DSGEStateVariableEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'State Shock Correlation',1,NumDraws,CurrChain,DSGEModel,CurrINI);
                     end;
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % compute the distribution for the state shock correlations
                        %
                        SVEs = DSGESVEDistribution(DSGEModel,CurrINI,7,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights);
                        %
                        % ask if we should save results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the state shock correlation distributions to file?';
                        answer = Query(txt,'question',140,'Save - State Shock Correlation Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                           file = FixFilePath([DSGEModel.OutputDirectory '\StateShockCorrDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                           save(file,'SVEs');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SSCDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           if isfield(SVEs,'AvTheoStdeta')==1;
                              fprintf(fid,'%%\n%% structure with fields Meaneta (mean), Stdeta (sample standard deviation), Correta (matrix with sample\n%% correlations), AvTheoStdeta (average population standard deviation), AvTheoCorreta (matrix with\n%% average population correlations), Quantiles, and ShockNames.\n%%\n');
                           else;
                              fprintf(fid,'%%\n%% structure with fields Meaneta (mean), Stdeta (sample standard deviation), Correta (matrix with sample correlations), Quantiles, and ShockNames.\n%%\n');
                           end;
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The state shock data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: SVEs (a structure with the average shocks, standard deviation, correlations, quantile data, and names of shocks). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','State Shock Correlation Distribution',200,500,CurrINI);
                        end;
                        %
                        % print the results to text file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateShockCorrDist-' strrep(num2str(ScenarioParameters),' ','') '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.txt']);
                        %
                        PrintStateVarDistribution(file,SVEs,DSGEModel,CurrINI,2,UsedDraws,NumDraws,CurrChain,0,1);
                        %
                        % display the results
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['State Shock Correlation Distributions - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Correlation Distributions',50,CurrINI,1,1);
                        end;
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the state shock distribution.';
                     About(txt,'information','State Shock Correlation Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MECorrInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view correlations of the measurement errors before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Project State Shocks at Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               %
               % Try to calculate state variables. First, however, the code needs to check if a unique convergent
               % solution exists.
               %
               [StateVarStructure,mcode,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
               if (mcode==1)&(kalmanstatus==0);
                  %
                  % we're OK
                  %
                  if isempty(StateVarStructure.wtT)==0;
                     txt = 'Would you like to view the sample and population evidence on the correlations for the measurement errors based on the update estimates instead of the smooth estimates?';
                     answer = Query(txt,'question',250,'Measurement Error Correlations - Estimator',600,'no',CurrINI);
                     %
                     % compute the correlations
                     %
                     T = size(StateVarStructure.wtT,2);
                     if strcmp(lower(answer),'no')==1;
                        if DSGEModel.YNaNs==0;
                           SampleMean = (1/T)*sum(StateVarStructure.wtT,2);
                           %
                           StateVarStructure.wtT = StateVarStructure.wtT-SampleMean*ones(1,T);
                           CovarianceMatrix = (1/T)*StateVarStructure.wtT*StateVarStructure.wtT';
                           StDev = sqrt(diag(diag(CovarianceMatrix)));
                        else;
                           Tvec = sum((~isnan(StateVarStructure.wtT)),2);
                           StateVarStructure.wtT(isnan(StateVarStructure.wtT)) = 0;
                           SampleMean = diag(1./Tvec)*sum(StateVarStructure.wtT,2);
                           %
                           StateVarStructure.wtT = StateVarStructure.wtT-SampleMean*ones(1,T);
                           CovarianceMatrix = (1/max(Tvec))*StateVarStructure.wtT*StateVarStructure.wtT';
                           StDev = sqrt(diag(diag(CovarianceMatrix)));
                        end;
                        AvTheoCovarianceMatrix = (1/T)*sum(StateVarStructure.MtT,3);
                        EstType = 'Smooth';
                     else;
                        if DSGEModel.YNaNs==0;
                           SampleMean = (1/T)*sum(StateVarStructure.wtt,2);
                           %
                           StateVarStructure.wtt = StateVarStructure.wtt-SampleMean*ones(1,T);
                           CovarianceMatrix = (1/T)*StateVarStructure.wtt*StateVarStructure.wtt';
                           StDev = sqrt(diag(diag(CovarianceMatrix)));
                        else;
                           Tvec = sum((~isnan(StateVarStructure.wtt)),2);
                           StateVarStructure.wtt(isnan(StateVarStructure.wtt)) = 0;
                           SampleMean = diag(1./Tvec)*sum(StateVarStructure.wtt,2);
                           %
                           StateVarStructure.wtt = StateVarStructure.wtt-SampleMean*ones(1,T);
                           CovarianceMatrix = (1/max(Tvec))*StateVarStructure.wtt*StateVarStructure.wtt';
                           StDev = sqrt(diag(diag(CovarianceMatrix)));
                        end;
                        AvTheoCovarianceMatrix = (1/T)*sum(StateVarStructure.Mtt,3);
                        EstType = 'Update';
                     end;
                     Correlations = zeros(size(CovarianceMatrix));
                     AvTheoCorrelations = zeros(size(AvTheoCovarianceMatrix));
                     TheoStDev = sqrt(diag(diag(AvTheoCovarianceMatrix)));
                     %
                     % prepare output for display
                     %
                     prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     prt_str = '%s';
                     for i=1:size(StateVarStructure.wtNames,1);
                        prt_str = [prt_str '     ' prt_val];
                        for j=1:size(StateVarStructure.wtNames,1);
                           Correlations(i,j) = CovarianceMatrix(i,j)/(StDev(i,i)*StDev(j,j));
                           AvTheoCorrelations(i,j) = AvTheoCovarianceMatrix(i,j)/(TheoStDev(i,i)*TheoStDev(j,j));
                        end;
                     end;
                     %
                     txt = ['Would you like to save to file the sample estimates of the first and second moments of the non-zero measurement errors based on the ' lower(EstType) ' estimates at initial values?'];
                     answer = Query(txt,'question',140,'Save - Measurement Error Correlations',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        StDev = diag(StDev);
                        MeasurementErrorNames = StateVarStructure.wtNames;
                        VariableNames = DSGEModel.VariableNames;
                        file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorCorrelations-' EstType '-InitialValues.mat']);
                        save(file,'MeasurementErrorNames','SampleMean','StDev','Correlations','AvTheoCovarianceMatrix');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\MEC' EstType 'InitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'MeasurementErrorNames = DataStructure.MeasurementErrorNames;\n');
                        fprintf(fid,'%%\n%% vector with the sample mean for the %s estimates\n%%\n',lower(EstType));
                        fprintf(fid,'SampleMean = DataStructure.SampleMean;\n');
                        fprintf(fid,'%%\n%% vector with the sample standard deviation\n%%\n');
                        fprintf(fid,'StDev = DataStructure.StDev;\n');
                        fprintf(fid,'%%\n%% matrix with the sample correlations\n%%\n');
                        fprintf(fid,'Correlations = DataStructure.Correlations;\n');
                        fprintf(fid,'%%\n%% matrix with the average population covariances\n%%\n');
                        fprintf(fid,'AvTheoCovarianceMatrix = DataStructure.AvTheoCovarianceMatrix;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The sample estimates of the first and second moments of the non-zero measurement errors for the smooth estimator at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: MeasurementErrorNames (string matrix with the names of the non-zero measurement errors), SampleMean (vector with the sample mean), StDev (vector with the sample standard deviation), Correlations (matrix with the sample correlations), and AvTheoCovarianceMatrix (matrix with the average population covariances). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Measurement Error Correlations - Initial Values',200,500,CurrINI);
                     end;
                     %
                     txt = strvcat(' ','MEASUREMENT ERRORS AT INITIAL VALUES','====================================',' ',['Using the ' lower(EstType) ' estimates of the measurement errors.'],' ');
                     %
                     txt = strvcat(txt,'Mean matrix:','------------',' ');
                     for i=1:size(StateVarStructure.wtNames,1);
                        txt = strvcat(txt,sprintf(['%s     ' prt_val],StateVarStructure.wtNames(i,:),SampleMean(i)));
                     end;
                     %
                     txt = strvcat(txt,' ','Covariance matrix:','------------------',' ');
                     for i=1:size(StateVarStructure.wtNames,1);
                        txt = strvcat(txt,sprintf(prt_str,StateVarStructure.wtNames(i,:),CovarianceMatrix(:,i)));
                     end;
                     %
                     txt = strvcat(txt,' ','Average population covariance matrix:','-------------------------------------',' ');
                     for i=1:size(StateVarStructure.wtNames,1);
                        txt = strvcat(txt,sprintf(prt_str,StateVarStructure.wtNames(i,:),AvTheoCovarianceMatrix(:,i)));
                     end;
                     %
                     if length(StDev)>1;
                        txt = strvcat(txt,' ','Correlation matrix:','-------------------',' ');
                        for i=1:size(StateVarStructure.wtNames,1);
                           txt = strvcat(txt,sprintf(prt_str,StateVarStructure.wtNames(i,:),Correlations(:,i)));
                        end;
                        %
                        txt = strvcat(txt,' ','Average population correlation matrix:','--------------------------------------',' ');
                        for i=1:size(StateVarStructure.wtNames,1);
                           txt = strvcat(txt,sprintf(prt_str,StateVarStructure.wtNames(i,:),AvTheoCorrelations(:,i)));
                        end;
                     end;
                     %
                     TextGUI(txt,['Measurement Errors - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                  else;
                     txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                     About(txt,'information','Measurement Errors',120,500,CurrINI);
                  end;
               elseif (mcode==1)&(kalmanstatus~=0);
                  if kalmanstatus==1;
                     txt = 'The state variables are not stationary at the initial values.';
                  else;
                     txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                  end;
                  About(txt,'information','Measurement Error Correlations - Initial Values',200,500,CurrINI);
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Measurement Error Correlations - Initial Values',200,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MECorrPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Measurement Error Correlation Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Measurement Error Correlation Distribution - Prior','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','Measurement Error Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % First we check if the calculations have already been performed
               %
               [Stat,MeasurementErrorStatus] = SVEExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,0,0,0);
               if (Stat==1)&(MeasurementErrorStatus==1);
                  txt = 'YADA has located results for the measurement errors on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'Measurement Error Correlation Distribution - Prior',500,'no',CurrINI);
                  DoCalc = 1;
               elseif (Stat==1)&(MeasurementErrorStatus==0);
                  txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                  About(txt,'information','Measurement Error Correlation Distribution - Prior',180,500,CurrINI);
                  DoCalc = 0;
               else;
                  answer = 'No';
                  DoCalc = 1;
               end;
               %
               %
               if DoCalc==1;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % start estimation
                     %
                     DoneCalc = DSGEStateVariableEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'Measurement Error Correlation',0,NumPriorDraws,1,DSGEModel,CurrINI);
                  end;
                  %
                  % check if we should show any results
                  %
                  if DoneCalc==1;
                     %
                     % compute the distribution for measurement error correlations
                     %
                     SVEs = DSGESVEDistribution(DSGEModel,CurrINI,8,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                     %
                     % ask if we should save results to disk
                     %
                     txt = 'Would you like to save the mean and quantiles of the measurement error correlation distributions to file?';
                     answer = Query(txt,'question',140,'Save - Measurement Error Correlation Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                        file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorCorrPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.mat']);
                        VariableNames = DSGEModel.VariableNames;
                        save(file,'SVEs','VariableNames');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\MECPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% structure with fields Meanw (mean), Stdw (standard deviation), Corrw (matrix with correlations), and ErrorIndicator.\n%%\n');
                        fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the variables\n%%\n');
                        fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The measurement error correlation data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the average measurement error, standard deviation, correlations, quantile data, and an indicator which functions have non-zero measurement errors), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Measurement Error Correlation Distribution - Prior',200,500,CurrINI);
                     end;
                     %
                     % print the results to text file
                     %
                     file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorCorrDistPrior-' strrep(num2str(ScenarioParameters),' ','') '-' int2str(NumPriorDraws) '.txt']);
                     %
                     PrintStateVarDistribution(file,SVEs,DSGEModel,CurrINI,3,NumPriorDraws,NumPriorDraws,1,0,0);
                     %
                     % display the results
                     %
                     if FileExist(file)==1;
                        TextGUI(file,['Measurement Error Correlation Distributions For Prior - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Correlation Distributions',50,CurrINI,1,1);
                     end;
                  else;
                     DSGEModel = OldDSGEModel;
                  end;
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the measurement error correlation distribution.';
               About(txt,'information','Measurement Error Correlation Distribution - Prior',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MECorrPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % we can now calculate the state shocks
            %
            StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
            if isempty(StateVarStructure)==0;
               if (isfield(StateVarStructure,'wtT')==1)&(isempty(StateVarStructure.wtT)==0);
                  txt = 'Would you like to view the sample and population evidence on the correlations for the measurement errors based on the update estimates instead of the smooth estimates?';
                  answer = Query(txt,'question',250,'Measurement Error Correlations - Estimator',600,'no',CurrINI);
                  %
                  % compute the correlations
                  %
                  T = size(StateVarStructure.wtT,2);
                  if strcmp(lower(answer),'no')==1;
                     if DSGEModel.YNaNs==0;
                        SampleMean = (1/T)*sum(StateVarStructure.wtT,2);
                        %
                        StateVarStructure.wtT = StateVarStructure.wtT-SampleMean*ones(1,T);
                        CovarianceMatrix = (1/T)*StateVarStructure.wtT*StateVarStructure.wtT';
                        StDev = sqrt(diag(diag(CovarianceMatrix)));
                     else;
                        Tvec = sum((~isnan(StateVarStructure.wtT)),2);
                        StateVarStructure.wtT(isnan(StateVarStructure.wtT)) = 0;
                        SampleMean = diag(1./Tvec)*sum(StateVarStructure.wtT,2);
                        %
                        StateVarStructure.wtT = StateVarStructure.wtT-SampleMean*ones(1,T);
                        CovarianceMatrix = (1/max(Tvec))*StateVarStructure.wtT*StateVarStructure.wtT';
                        StDev = sqrt(diag(diag(CovarianceMatrix)));
                     end;
                     AvTheoCovarianceMatrix = (1/T)*sum(StateVarStructure.MtT,3);
                     EstType = 'Smooth';
                  else;
                     if DSGEModel.YNaNs==0;
                        SampleMean = (1/T)*sum(StateVarStructure.wtt,2);
                        %
                        StateVarStructure.wtt = StateVarStructure.wtt-SampleMean*ones(1,T);
                        CovarianceMatrix = (1/T)*StateVarStructure.wtt*StateVarStructure.wtt';
                        StDev = sqrt(diag(diag(CovarianceMatrix)));
                     else;
                        Tvec = sum((~isnan(StateVarStructure.wtt)),2);
                        StateVarStructure.wtt(isnan(StateVarStructure.wtt)) = 0;
                        SampleMean = diag(1./Tvec)*sum(StateVarStructure.wtt,2);
                        %
                        StateVarStructure.wtt = StateVarStructure.wtt-SampleMean*ones(1,T);
                        CovarianceMatrix = (1/max(Tvec))*StateVarStructure.wtt*StateVarStructure.wtt';
                        StDev = sqrt(diag(diag(CovarianceMatrix)));
                     end;
                     AvTheoCovarianceMatrix = (1/T)*sum(StateVarStructure.Mtt,3);
                     EstType = 'Update';
                  end;
                  Correlations = zeros(size(CovarianceMatrix));
                  AvTheoCorrelations = zeros(size(AvTheoCovarianceMatrix));
                  TheoStDev = sqrt(diag(diag(AvTheoCovarianceMatrix)));
                  %
                  % prepare output for display
                  %
                  prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                  prt_str = '%s';
                  for i=1:size(StateVarStructure.wtNames,1);
                     prt_str = [prt_str '     ' prt_val];
                     for j=1:size(StateVarStructure.wtNames,1);
                        Correlations(i,j) = CovarianceMatrix(i,j)/(StDev(i,i)*StDev(j,j));
                        AvTheoCorrelations(i,j) = AvTheoCovarianceMatrix(i,j)/(TheoStDev(i,i)*TheoStDev(j,j));
                     end;
                  end;
                  %
                  txt = ['Would you like to save to file the sample estimates of the first and second moments of the non-zero measurement errors based on the ' lower(EstType) ' estimates at posterior mode?'];
                  answer = Query(txt,'question',140,'Save - Measurement Error Correlations',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     StDev = diag(StDev);
                     MeasurementErrorNames = StateVarStructure.wtNames;
                     VariableNames = DSGEModel.VariableNames;
                     file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorCorrelations-' EstType '-PosteriorMode.mat']);
                     save(file,'MeasurementErrorNames','SampleMean','StDev','Correlations','AvTheoCovarianceMatrix');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\MEC' EstType 'PostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% string matrix with the names of the non-zero measurement errors\n%%\n');
                     fprintf(fid,'MeasurementErrorNames = DataStructure.MeasurementErrorNames;\n');
                     fprintf(fid,'%%\n%% vector with the sample mean for the %s estimates\n%%\n',lower(EstType));
                     fprintf(fid,'SampleMean = DataStructure.SampleMean;\n');
                     fprintf(fid,'%%\n%% vector with the sample standard deviation\n%%\n');
                     fprintf(fid,'StDev = DataStructure.StDev;\n');
                     fprintf(fid,'%%\n%% matrix with the sample correlations\n%%\n');
                     fprintf(fid,'Correlations = DataStructure.Correlations;\n');
                     fprintf(fid,'%%\n%% matrix with the average population covariances\n%%\n');
                     fprintf(fid,'AvTheoCovarianceMatrix = DataStructure.AvTheoCovarianceMatrix;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The sample estimates of the first and second moments of the non-zero measurement errors for the smooth estimator at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: MeasurementErrorNames (string matrix with the names of the non-zero measurement errors), SampleMean (vector with the sample mean), StDev (vector with the sample standard deviation), Correlations (matrix with the sample correlations), and AvTheoCovarianceMatrix (matrix with the average population covariances). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Measurement Error Correlations - Posterior Mode',250,600,CurrINI);
                  end;
                  %
                  txt = strvcat(' ','MEASUREMENT ERRORS AT POSTERIOR MODE','====================================',' ',['Using the ' lower(EstType) ' estimates of the measurement errors.'],' ');
                  %
                  txt = strvcat(txt,'Mean matrix:','------------',' ');
                  for i=1:size(StateVarStructure.wtNames,1);
                     txt = strvcat(txt,sprintf(['%s     ' prt_val],StateVarStructure.wtNames(i,:),SampleMean(i)));
                  end;
                  %
                  txt = strvcat(txt,' ','Covariance matrix:','------------------',' ');
                  for i=1:size(StateVarStructure.wtNames,1);
                     txt = strvcat(txt,sprintf(prt_str,StateVarStructure.wtNames(i,:),CovarianceMatrix(:,i)));
                  end;
                  %
                  txt = strvcat(txt,' ','Average population covariance matrix:','-------------------------------------',' ');
                  for i=1:size(StateVarStructure.wtNames,1);
                     txt = strvcat(txt,sprintf(prt_str,StateVarStructure.wtNames(i,:),AvTheoCovarianceMatrix(:,i)));
                  end;
                  if length(StDev)>1;
                     txt = strvcat(txt,' ','Correlation matrix:','-------------------',' ');
                     for i=1:size(StateVarStructure.wtNames,1);
                        txt = strvcat(txt,sprintf(prt_str,StateVarStructure.wtNames(i,:),Correlations(:,i)));
                     end;
                     txt = strvcat(txt,' ','Average population correlation matrix:','--------------------------------------',' ');
                     for i=1:size(StateVarStructure.wtNames,1);
                        txt = strvcat(txt,sprintf(prt_str,StateVarStructure.wtNames(i,:),AvTheoCorrelations(:,i)));
                     end;
                  end;
                  %
                  TextGUI(txt,['Measurement Errors - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
               else;
                  txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                  About(txt,'information','Measurement Errors',120,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MECorrPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Measurement Error Correlation Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','Measurement Error Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % First we check if the calculations have already been performed
                     %
                     [Stat,MeasurementErrorStatus] = SVEExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,0,0,1,0);
                     if (Stat==1)&(MeasurementErrorStatus==1);
                        txt = 'YADA has located results for the measurement errors on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'Measurement Error Correlation Distribution',500,'no',CurrINI);
                        DoCalc = 1;
                     elseif (Stat==1)&(MeasurementErrorStatus==0);
                        txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                        About(txt,'information','Measurement Error Correlation Distribution',180,500,CurrINI);
                        DoCalc = 0;
                     else;
                        answer = 'No';
                        DoCalc = 1;
                     end;
                     %
                     %
                     if DoCalc==1;
                        if strcmp(lower(answer),'yes')==1;
                           DoneCalc = 1;
                        else;
                           %
                           % start estimation
                           %
                           DoneCalc = DSGEStateVariableEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'Measurement Error Correlation',1,NumDraws,CurrChain,DSGEModel,CurrINI);
                        end;
                        %
                        % check if we should show any results
                        %
                        if DoneCalc==1;
                           %
                           % compute the distribution for measurement error correlations
                           %
                           SVEs = DSGESVEDistribution(DSGEModel,CurrINI,8,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights);
                           %
                           % ask if we should save results to disk
                           %
                           txt = 'Would you like to save the mean and quantiles of the measurement error correlation distributions to file?';
                           answer = Query(txt,'question',140,'Save - Measurement Error Correlation Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                              file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorCorrDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                              VariableNames = DSGEModel.VariableNames;
                              save(file,'SVEs','VariableNames');
                              %
                              % create a matlab file to read the data
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\MECDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% structure with fields Meanw (mean), Stdw (standard deviation), Corrw (matrix with correlations), and ErrorIndicator.\n%%\n');
                              fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the variables\n%%\n');
                              fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              txt = ['The measurement error correlation data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the average measurement error, standard deviation, correlations, quantile data, and an indicator which functions have non-zero measurement errors), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','Measurement Error Correlation Distribution',200,500,CurrINI);
                           end;
                           %
                           % print the results to text file
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorCorrDist-' strrep(num2str(ScenarioParameters),' ','') '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.txt']);
                           %
                           PrintStateVarDistribution(file,SVEs,DSGEModel,CurrINI,3,UsedDraws,NumDraws,CurrChain,0,1);
                           %
                           % display the results
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['Measurement Error Correlation Distributions - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Correlation Distributions',50,CurrINI,1,1);
                           end;
                        end;
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the measurement error correlation distribution.';
                     About(txt,'information','Measurement Error Correlation Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'OneStepAheadPredCheckPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Prior Predictive Checks - One-Step-Ahead',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the posterior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Prior Predictive Checks - One-Step-Ahead','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','Prior Predictive Checks','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % check if results already exist
               %
               Status = PredCheckOneStepAheadExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0);
               if Status==1;
                  txt = 'YADA has located results for the prior predictive checks of the one-step-ahead forecast errors on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'Prior Predictive Checks - One-Step-Ahead',500,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               %
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  DoneCalc = DSGEOneStepAheadPredictiveChecks(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,0,NumPriorDraws,1,DSGEModel,CurrINI);
               end;
               %
               % check if we should show any results
               %
               if DoneCalc==1;
                  %
                  % collect the output and determine the posterior predictive p-value
                  %
                  [PValues,OmegaSim,OmegaObs] = OneStepAheadPredictiveChecks(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[],[]);
                  %
                  % ask if results sould be saved to file
                  %
                  txt = 'Would you like to save the p-values from the prior predictive checks for the one-step-ahead forecast error covariances to file?';
                  answer = Query(txt,'question',140,'Save - Prior Predictive Checks',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                     file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-PriorPC-OSA-' SelParamStr '-' int2str(NumPriorDraws) '.mat']);
                     VariableNames = DSGEModel.VariableNames;
                     Dn = DuplicationMatrix(DSGEModel.n);
                     PValuesMat = reshape(Dn*PValues',DSGEModel.n,DSGEModel.n);
                     save(file,'VariableNames','PValuesMat');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEPriorPCOSA' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% Matrix with probabilities that the covariance of the one-step-ahead forecast errors\n%% is greater than the observed\n%%\n');
                     fprintf(fid,'PValuesMat = DataStructure.PValuesMat;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the variables\n%%\n');
                     fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The p-values from the prior predictive checks for the one-step-ahead forecast error covariances have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: PValuesMat (a matrix with the probabilities that the forecast error covariances are greater than the observed), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Prior Predictive Checks - One-Step-Ahead',200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  PredictiveChecksOSAPlot('init',PValues,OmegaSim,OmegaObs,0,DSGEModel,CurrINI);
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the posterior predictive checks of the one-step-ahead forecast errors.';
               About(txt,'information','Prior Predictive Checks - One-Step-Ahead',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
            %
            % clear everything
            %
            clear('thetaDraws');
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'OneStepAheadPredCheckPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Posterior Predictive Checks - One-Step-Ahead','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','Posterior Predictive Checks','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % check if results already exist
                     %
                     Status = PredCheckOneStepAheadExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1);
                     if Status==1;
                        txt = 'YADA has located results for the posterior predictive checks of the one-step-ahead forecast errors on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'Posterior Predictive Checks - One-Step-Ahead',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     %
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        DoneCalc = DSGEOneStepAheadPredictiveChecks(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,1,NumDraws,CurrChain,DSGEModel,CurrINI);
                     end;
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % collect the output and determine the posterior predictive p-value
                        %
                        [PValues,OmegaSim,OmegaObs] = OneStepAheadPredictiveChecks(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights,[]);
                        %
                        % ask if results sould be saved to file
                        %
                        txt = 'Would you like to save the p-values from the posterior predictive checks for the one-step-ahead forecast error covariances to file?';
                        answer = Query(txt,'question',140,'Save - Posterior Predictive Checks',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGE-PostPC-OSA-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           Dn = DuplicationMatrix(DSGEModel.n);
                           PValuesMat = reshape(Dn*PValues',DSGEModel.n,DSGEModel.n);
                           save(file,'VariableNames','PValuesMat');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEPostPCOSA' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% Matrix with probabilities that the covariance of the one-step-ahead forecast errors\n%% is greater than the observed\n%%\n');
                           fprintf(fid,'PValuesMat = DataStructure.PValuesMat;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The p-values from the posterior predictive checks for the one-step-ahead forecast error covariances have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: PValuesMat (a matrix with the probabilities that the forecast error covariances are greater than the observed), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Posterior Predictive Checks - One-Step-Ahead',200,500,CurrINI);
                        end;
                        %
                        % display the results
                        %
                        PredictiveChecksOSAPlot('init',PValues,OmegaSim,OmegaObs,1,DSGEModel,CurrINI);
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the posterior predictive checks of the one-step-ahead forecast errors.';
                     About(txt,'information','Posterior Predictive Checks - One-Step-Ahead',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of CorrToolsMenuFunctions.m
%
