function DSGEVARForecastMenuFunctions(selector,maingui)
% DSGEVARForecastMenuFunctions: switch function for running functions on the DSGE-VAR menu.
%
% USAGE:
%
%       DSGEVARMenuFunctions(selector,maingui)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
% OPTIONAL INPUT: maingui (handle) to the YADA GUI.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: June 1, 2010.
%                        This version: March 18, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 02-06-2010: Continued work on the unconditional forecasts at posterior mode.
%
% * 03-06-2010: Did some more work on take-off-day.
%
% * 04-06-2010: Finished the unconditional forecasts at posterior mode and at initial values.
%
% * 07-06-2010: Began work on the unconditional forecasts for the posterior draws case.
%
% * 08-06-2010: Finished the unconditional forecasts for posterior draws.
%
% * 09-06-2010: Added the "Prediction Event" function, adn the unconditional forecast for the
%               prior draws.
%
% * 22-06-2010: Fixed bug from not initializing "ParamStr".
%
% * 23-07-2010: Added calls to "NormalDensityValues" for approximating the marginal predictive
%               densities with a normal.
%
% * 18-11-2010: Took into account that "PredEventData" and "YObsEventData" now have 7 columns.
%
% * 19-11-2010: And now that it has 8 columns.
%
% * 05-01-2011: Updated the documentation.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 21-01-2011: Ensured that the "YADA - Bad MAT File" dialogs can be up to 300 pixels in height.
%
% * 25-01-2011: Took into account that "CalcTransformData" takes the input variable "TransMatrix".
%
% * 18-02-2011: Started work on conditional forecasts for posterior mode values.
%
% * 21-02-2011: Continued work on the conditional forecasts at posterior mode values.
%
% * 24-02-2011: Fisnihed the conditional forecast at posterior mode values.
%
% * 25-02-2011: Added conditional forecasts at initial values and began work on the posterior draws.
%
% * 28-02-2011: Added code for dealing with the "DSGE Posterior Mode" cases.
%
% * 01-03-2011: Finished the conditional forecasts for the posterior draws, as well as the prediction
%               events and the prior draws.
%
% * 02-03-2011: Finished the conditional forecasts for the prior draws.
%
% * 28-03-2011: Began work on conditional forecasts at posterior mode using the distribution for a subset
%               of the shocks method.
%
% * 29-03-2011: Finished the conditional forecast at posterior mode using the subset shocks method. Added
%               the initial values case, the posterior draws, and the prediction events.
%
% * 30-03-2011: Added the prior draws case for the subset shocks conditioning method.
%
% * 02-01-2012: Updated the documentation.
%
% * 07-02-2012: Changed the conditions under which the subset method can be used.
%
% * 29-02-2012: Took into account that the number of simulated paths per parameter value can be unity.
%
% * 02-04-2012: Began work on the predictive likelihood functions.
%
% * 03-04-2012: Continued work on the posterior mode function for the predictive likelihood.
%
% * 04-04-2012: Continued working on the posterior mode case.
%
% * 05-04-2012: Finished the code for the posterior mode case.
%
% * 30-04-2012: Took into account that "PrintDSGEPredictiveLikelihood" has changed name to
%               "PrintDSGEPredictiveLikelihoodTheta".
%
% * 11-05-2012: Began work on the posterior distribution case for the predictive likelihood. Took into
%               account that "LoadVARDSGEPosteriorDraws" now provides "thetaPostSample" as an output
%               variable.
%
% * 14-05-2012: Continued work on the posterior distribution for the predictive likelihood. Took into
%               account that the above function now also provides "PostSample".
%
% * 15-05-2012: Took into account that "DSGEVARPredictiveLikelihoodParam" also provides the output
%               variables "MargPlugin" and "JointPlugin".
%
% * 21-08-2012: The predictive likelihood for posterior draws now uses the importance sampling estimator.
%
% * 29-08-2012: Took into account that "PredictionPathDistribution" now takes the input variable
%               "ConfidenceRegionMethod". Fixed plugin estimate issues for the log predictive
%               likelihood at posterior mode.
%
% * 30-08-2012: Made sure that the dimension of the Hessian matrix for the log posterior kernel is
%               mentioned in the question if the Laplace approximation should be computed for the
%               posterior mode values of the predictive likelihood.
%
% * 18-10-2012: Took into acount that "DSGEVARPredLikeImportanceSampling" now also provides StdErr for
%               the log predictive likelihood.
%
% * 23-10-2012: Took into account that the input variables for "DSGEVARPredictiveLikelihood" have changed.
%
% * 25-10-2012: Added the initial values and prior distribution cases for the predictive likelihood calculations.
%
% * 03-01-2013: Updated the documentation.
%
% * 28-01-2014: Updated the documentation.
%
% * 13-03-2014: Updated the documentation concerning lambda-values.
%
% * 14-01-2015: Updated the documentation.
%
% * 20-01-2015: Fixed a minor error mixing up NumParam with NumDraws for posterior sampling based computations of
%               the predictive likelihood.
%
% * 26-02-2015: Took into account that the function "DSGEVARPredLikeImportanceSampling" has had
%               its name changed to "DSGEVARPredLikeMCI", and that this function supports two
%               additional output variables "MinMaxCondLike" and "TestStd".
%
% * 15-06-2015: The predictive likelihood functions now allow for various actuals.
%
% * 27-01-2016: Remember Shoah.
%
% * 20-02-2017: Took into account that "PrintDSGEPredictiveLikelihoodTheta" takes two new input variables.
%
% * 17-03-2017: Took into account that "PrintDSGEPredictiveLikelihood" takes five new input variables.
%
% * 23-06-2017: Began work on the marginal predictive moments for prior and posterior draws.
%
% * 26-06-2017: Finished the prior and the posterior draws cases for the marginal predictive moments.
%
% * 17-08-2017: Removed command that checks if the data in the forecast sample are available or just NaNs
%               for the marginal predictive moments as this data is not used.
%
% * 11-09-2017: Took into account that "LoadVARDSGEPosteriorDraws" provides the output variable "Weights" and
%               that "DSGEVARPredLikeMCI", "DSGEVARPredictiveMoments", "DSGEVARPredictionPaths",
%               "DSGEVARPredictionDistribution", "LoadDSGEVARPredictionPaths", "DSGEVARCondPredictionPaths",
%               "DSGEVARCondPredictionPathsWZ", and "DSGEVARCondPredictionPathsMixed" require the input variable
%               "Weights".
%
% * 15-09-2017: Updated the documentation.
%
% * 24-10-2017: Took into account that "PrintPredictiveMoments" can now print back- and nowcast results,
%               while DSGE-VAR models are (currently) not compatible with ragged edge data.
%
% * 08-01-2018: Updated the documentation.
%
% * 15-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 23-01-2019: Updated the documentation.
%
% * 27-03-2019: Began work on the stationary predictive variance functions.
%
% * 28-03-2019: Finished the stationary predictive variance functions for the posterior and the
%               prior draws.
%
% * 10-01-2020: Updated the documentation.
%
% * 09-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 26-01-2021: Updated the documentation.
%
% * 12-05-2021: Took into account that "PrintDSGEPredictiveLikelihood" requires the IsAdaptiveLearning input
%               variable.
%
% * 18-05-2021: Took into account that "PrintPredictiveMoments" now requires the input variable
%               "IsAdaptiveLearning".
%
% * 06-01-2022: Updated the documentation.
%
% * 05-01-2023: Updated the documentation.
%
% * 22-02-2023: Added the optional input variable "maingui".
%
% * 05-07-2023: The function now tries to load the AIMData file directly.
%
% * 18-01-2024: Updated the documentation.
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
% * 18-03-2024: Made sure that the PredictionEvent field is not changed unless user selects it.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
if nargin==1;
   maingui = findobj('Type','figure','Tag','YADA');
end;
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
if MatlabNumber<7;
   lasterr = '';
end;
%
switch selector
   case 'UncondPredHeightInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==1;
            %
            % determine string with 1's and 0's for the selected shocks
            %
            SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
            SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
            %
            % check which data to compute unconditional forecasts
            %
            if gcbo==controls.dsgevartools.forecast.unconditional.original.height.initialvalues;
               DataType = 'Original';
               IsOriginal = 1;
            elseif gcbo==controls.dsgevartools.forecast.unconditional.annual.height.initialvalues;
               DataType = 'Annualized';
               IsOriginal = 0;
            end;
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               [lambda,T] = GetLambda(DSGEModel);
               LambdaValuesStr = '';
               for i=1:length(lambda);
                  LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
               end;
               [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Predictive Likelihood - Initial Values','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  lambdai = lambda(value);
                  %
                  % compute the prior mode of SigmaEpsilon
                  %
                  if isinf(lambdai)==0;
                     SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                  end;
                  if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                     %
                     % get the sample
                     %
%                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,HSample(2)+1:HSample(2)+MaxFH);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        YDataStr = 'Actuals from original sample';
                        action = 'ok';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - Initial Values','','',CurrINI);
                        if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        end;
                     end;
                     %
                     if strcmp(lower(action),'ok')==1;
                        MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        MaxFH = min(MaxFH,MaxPH);
                        YData = YData(:,1:MaxFH);
                        %
                        % matrix with exogenous variables
                        %
                        X = [];
                        if DSGEModel.k>0;
                           if NumPredX<MaxFH;
                              X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxFH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                           end;
                        end;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           %
                           % we may continue and store the new selection of prediction variables
                           %
                           DSGEModel.PredictionVariables = PredictionVariables;
                           %
                           % step length used for computing the Hessian with finite diffs
                           %
                           StepLengthStr = get(controls.optimize.steplength,'String');
                           StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                           %
                           % need to compute certain values
                           %
                           n = DSGEModel.n;
                           k = DSGEModel.k;
                           p = DSGEModel.BVARLags;
                           npk = (n*p)+k;
                           %
                           % setup the sample
                           %
                           [y,Y,HSample,xx,XX] = GetDSGEVARData(DSGEModel);
                           if k>0;
                              DetProductMoments = CalcDeterministicProductMoments(xx,XX);
                           else;
                              DetProductMoments = [];
                           end;
                           T = size(y,2);
                           GammaHatyy = (1/T)*y*y';
                           GammaHatyY = (1/T)*y*Y';
                           GammaHatYY = (1/T)*Y*Y';
                           if isinf(lambdai)==0;
                              logGPR = LogGammaProductRatio(n,((1+lambdai)*T)-npk,lambdai*T-npk);
                           else;
                              logGPR = [];
                           end;
                           %
                           % check if the user wants only the plugin estimate of the predictive likelihood
                           %
                           IsPlugin = 0;
                           if isinf(lambdai)==0;
                              NumParam = (n*npk)+(n*(n+1)/2)+length(theta);
                           else;
                              NumParam = length(theta);
                           end;
                           txt = strvcat('Would you like to compute the plugin estimate only and skip the Laplace approximation?',' ',['NOTE: The Laplace approximation requires that the Hessian matrix of the log posterior kernel is computed in addition to the plugin estimate. This matrix has dimension ' int2str(NumParam) 'x' int2str(NumParam) ' for the DSGE-VAR(' num2str(lambdai) ') model with ' int2str(p) ' lags. Moreover, the Laplace approximation is based on an optimal estimate of the parameters, i.e. that the gradient vector is (close to) zero. If you answer NO then YADA will compute both sets of values.']);
                           answer = Query(txt,'question',400,'Predictive Likelihood - Initial Values',600,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              IsPlugin = 1;
                           end;
                           %
                           % compute the joint and marginal predictive likelihood
                           %
                           tic;
                           [JointPDH,MargPDH,LaplaceMargLike,PredVars,MargPlugin,JointPlugin] = DSGEVARPredictiveLikelihoodParam(theta,thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,YData,X,IsOriginal,IsPlugin,StepLength,DSGEModel,CurrINI);
                           CompTime = toc;
                           %
                           % check if we have results
                           %
                           if isempty(JointPDH)==0;
                              %
                              % ask if we should the results to disk
                              %
                              %
                              SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                              SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                              %
                              PredictionSampleStrMat = CreateFullSample(DSGEModel);
                              PredictionSampleStrMat = PredictionSampleStrMat(HSample(2)+1:HSample(2)+MaxFH,:);
                              txt = ['Would you like to save the results at the initial parameter values from the DSGE-VAR(' num2str(lambdai) ') model with ' int2str(p) ' lags using the ' lower(DataType) ' data to disk?'];
                              answer = Query(txt,'question',200,'Save - Predictive Likelihood',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 %
                                 YNames = DSGEModel.VariableNames;
                                 file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPredictiveLikelihood-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataType '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-InitialValues.mat']);
                                 save(file,'JointPDH','MargPDH','LaplaceMargLike','MargPlugin','JointPlugin','PredVars','YNames','PredictionSampleStrMat');
                                 %
                                 % create a matlab m-file to read the output.
                                 %
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPLike' int2str(SelectedYData) '_' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(p) DataType SelectedVarStr strrep(PredictionSampleStrMat(1,:),':','_') strrep(PredictionSampleStrMat(MaxFH,:),':','_') 'InitVal.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 if IsPlugin==0;
                                    fprintf(fid,'%%\n%% Predictive likelihood of joint predictions \n%%\n');
                                    fprintf(fid,'JointPDH = DataStructure.JointPDH;\n');
                                    fprintf(fid,'%%\n%% Predictive likelihood of marginal predictions \n%%\n');
                                    fprintf(fid,'MargPDH = DataStructure.MargPDH;\n');
                                    fprintf(fid,'%%\n%% Laplace approximation of log marginal likelihood for historical sample \n%%\n');
                                    fprintf(fid,'LaplaceMargLike = DataStructure.LaplaceMargLike;\n');
                                    NumEntries = '8';
                                    AddText = 'JointPDH (vector with the values of the joint predictive likelihood), MargPDH (vector with the marginal predictive likelihood), LaplaceMargLike (scalar with the marginal likelihood for the historical sample), ';
                                 else;
                                    NumEntries = '5';
                                    AddText = '';
                                 end;
                                 fprintf(fid,'%%\n%% Plugin estimate of the joint predictive likelihood \n%%\n');
                                 fprintf(fid,'JointPlugin = DataStructure.JointPlugin;\n');
                                 fprintf(fid,'%%\n%% Plugin estimate of the marginal predictive likelihood \n%%\n');
                                 fprintf(fid,'MargPlugin = DataStructure.MargPlugin;\n');
                                 fprintf(fid,'%%\n%% positions of predicted variables in YNames \n%%\n');
                                 fprintf(fid,'PredVars = DataStructure.PredVars;\n');
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                                 fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The predictive likelihood results at the initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: ' AddText 'MargPlugin (vector with the plugin estimate of the marginal predictive likelihood), JointPlugin (vector with the plugin estimate of the joint predictive likelihood), PredVars (cell array with the locations of the predicted variables), YNames (string matrix with the names of the observed variables), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information','Predictive Likelihood - Initial Values',500,600,CurrINI);
                              end;
                              %
                              % write to file
                              %
                              ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPredictiveLikelihood-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataType '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-InitialValues.txt']);
                              %
                              PrintDSGEPredictiveLikelihoodTheta(ResFile,JointPDH,MargPDH,LaplaceMargLike,JointPlugin,MargPlugin,[],PredVars,PredictionSampleStrMat,'',YDataStr(SelectedYData,:),IsOriginal,IsPlugin,'initial values',lambdai,CompTime,DSGEModel,CurrINI);
                              %
                              % display the results
                              %
                              if FileExist(ResFile)==1;
                                 TextGUI(ResFile,['DSGE-VAR(' num2str(lambdai) ') Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,0);
                                 drawnow;
                              end;
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           txt = ['You didn''t select any variables to predict when computing the predictive likelihood of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA is therefore unable to help you!'];
                           About(txt,'information','Predictive Likelihood - Initial Values',150,500,CurrINI);
                        end;
                     end;
                  else;
                     txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive likelihood for the ' lower(DataType) ' data.'];
                     About(txt,'information','Predictive Likelihood - Initial Values',300,500,CurrINI);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'UncondPredHeightPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                        ModeData = [];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                        ModeData = [];
                     end;
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               UniformBounds = ModeData.UniformBounds;
               LowerBound = ModeData.LowerBound;
               thetaIndex = ModeData.thetaIndex;
            end;
         end;
         %
         if InitStatus==1;
            if gcbo==controls.dsgevartools.forecast.unconditional.original.height.prior;
               DataStr = 'Original';
               IsOriginal = 1;
            elseif gcbo==controls.dsgevartools.forecast.unconditional.annual.height.prior;
               DataStr = 'Annualized';
               IsOriginal = 0;
            end;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                  %
                  % determine which model to consider
                  %
                  [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Predictive Likelihood - Prior Distribution',DSGEModel,CurrINI);
                  if isempty(lambda)==0;
                     T = HSample(2)-HSample(1)+1;
                     if isinf(lambda)==0;
                        SigmaEpsilon = ((lambda*T)/((lambda*T)+DSGEModel.n+1))*SigmaEpsilon;
                     end;
                     %
                     % load the prior draws data
                     %
                     [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
                     UsedDraws = size(PhiDraws,1);
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws');
                     %
                     % determine string with 1's and 0's for the selected shocks
                     %
                     SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                     SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,HSample(2)+1:HSample(2)+MaxFH);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - Prior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        end;
                     end;
                     %
                     if strcmp(lower(DataAction),'ok')==1;
                        MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        MaxFH = min(MaxFH,MaxPH);
                        YData = YData(:,1:MaxFH);
                        %
                        % matrix with exogenous variables
                        %
                        X = [];
                        if DSGEModel.k>0;
                           if NumPredX<MaxFH;
                              X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxFH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                           end;
                        end;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           %
                           % we may continue and store the new selection of prediction variables
                           %
                           DSGEModel.PredictionVariables = PredictionVariables;
                           %
                           % need to compute certain values
                           %
                           n = DSGEModel.n;
                           p = DSGEModel.BVARLags;
                           npk = (n*p)+DSGEModel.k;
                           %
                           % check if the user wants to compute the marginal or the joint predictive likelihood
                           %
                           txt = ['Would you like to compute the marginal predictive likelikehood or the joint predictive likelihood for the ' lower(DataStr) ' data?'];
                           answer = Query(txt,'question',400,'Predictive Likelihood - Prior Distribution',500,'yes',CurrINI,{'Marginal' 'Joint'});
                           if strcmp(lower(answer),'marginal')==1;
                              IsMarginal = 1;
                              MargStr = 'Marginal';
                           else;
                              IsMarginal = 0;
                              MargStr = 'Joint';
                           end;
                           %
                           % check if results already exist on disk
                           %
                           if DSGEVARPredictiveLikelihoodExist(DSGEModel,SelectedYData,IsOriginal,IsMarginal,0,lambda,0,MaxFH,UsedDraws,0,controls)==1;
                              txt = ['YADA has located ' lower(MargStr) ' log posterior calculations of a DSGE-VAR with lambda=' num2str(lambda) ' and ' int2str(p) ' lags for the ' lower(DataStr) ' data for the predictive likelihood on disk. Would you like to use these results?'];
                              answer = Query(txt,'question',200,[MargStr ' Predictive Likelihood - Prior Distribution'],500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           tic;
                           if strcmp(lower(answer),'yes')==1;
                              DoneCalc = 1;
                           else;
                              %
                              % compute the predictive likelihood
                              %
                              DoneCalc = DSGEVARPredictiveLikelihood(PhiDraws,SigmaDraws,Phi,SigmaEpsilon,lambda,n,p,npk,HSample,YData,X,SelectedYData,IsOriginal,0,IsMarginal,0,0,DSGEModel,CurrINI,controls);
                           end;
                           clear('PhiDraws','SigmaDraws');
                           if DoneCalc==1;
                              %
                              % we can now do the remaining modified harmonic mean estimates.
                              %
                              [LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd] = DSGEVARPredLikeMCI(SelectedYData,UsedDraws,MaxFH,IsOriginal,IsMarginal,0,[],lambda,0,0,DSGEModel,controls);
                              CompTime = toc;
                              %
                              if isempty(LogPredLike)==0;
                                 %
                                 % ask the user if results should be saved to file
                                 %
                                 DrawsStr = get(controls.posterior.draws,'String');
                                 TotalDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                                 %
                                 BurninStr = get(controls.posterior.burnin,'String');
                                 NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
                                 %
                                 SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                                 SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                                 %
                                 PredictionSampleStrMat = CreateFullSample(DSGEModel);
                                 PredictionSampleStrMat = PredictionSampleStrMat(HSample(2)+1:HSample(2)+MaxFH,:);
                                 txt = ['Would you like to save the prior draws results of the DSGE-VAR model with lambda=' num2str(lambda) ' and ' int2str(p) ' lags for the ' lower(DataStr) ' data to disk?'];
                                 answer = Query(txt,'question',300,['Save - ' MargStr ' Predictive Likelihood'],500,'no',CurrINI);
                                 if strcmp(lower(answer),'yes')==1;
                                    YNames = DSGEModel.VariableNames;
                                    file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' MargStr 'PredictiveLikelihood-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(UsedDraws) '.mat']);
                                    save(file,'LogPredLike','StdErr','PredVars','YNames','PredictionSampleStrMat');
                                    %
                                    % create a matlab m-file to read the output.
                                    %
                                    mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' MargStr(1) 'PLikePrior' int2str(SelectedYData) '_' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(p) DataStr(1) SelectedVarStr strrep(PredictionSampleStrMat(1,:),':','_') strrep(PredictionSampleStrMat(MaxFH,:),':','_') int2str(UsedDraws) '.m']);
                                    fid = fopen(mfile,'wt');
                                    fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                    fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                    fprintf(fid,'%%\n%% Predictive likelihood of %s predictions \n%%\n',lower(MargStr));
                                    fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                                    fprintf(fid,'%%\n%% numerical standard errors of the log predictive likelihood of %s predictions \n%%\n',lower(MargStr));
                                    fprintf(fid,'StdErr = DataStructure.StdErr;\n');
                                    fprintf(fid,'%%\n%% positions of predicted variables in YNames \n%%\n');
                                    fprintf(fid,'PredVars = DataStructure.PredVars;\n');
                                    fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                    fprintf(fid,'YNames = DataStructure.YNames;\n');
                                    fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                                    fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                                    fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                    %
                                    % end the script file
                                    %
                                    fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                    fclose(fid);
                                    %
                                    txt = ['The ' lower(MargStr) ' predictive likelihood results for the prior draws of a DSGE-VAR model with lambda=' num2str(lambda) ' and ' int2str(p) ' lags have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: LogPredLike (vector of dimension h with the values of the ' lower(MargStr) ' log predictive likelihood), StdErr (vector of dimension h with the numerical standard errors of the log predictive likelihood), PredVars (cell array with the locations of the predicted variables), YNames (string matrix with the names of the observed variables), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                    About(txt,'information',[MargStr ' Predictive Likelihood - Prior Distribution'],600,650,CurrINI);
                                 end;
                                 %
                                 % write the results to text file
                                 %
                                 ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' MargStr 'PredictiveLikelihood-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(UsedDraws) '.txt']);
                                 %
                                 PrintDSGEPredictiveLikelihood(ResFile,LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,[],[],[],[],PredictionSampleStrMat,'',YDataStr(SelectedYData,:),IsOriginal,IsMarginal,0,0,UsedDraws,0,UsedDraws,lambda,CompTime,DSGEModel,CurrINI)
                                 %
                                 % display the results
                                 %
                                 if FileExist(ResFile)==1;
                                    TextGUI(ResFile,[MargStr ' Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],['Prior Distribution - DSGE-VAR(' num2str(lambda) ',' int2str(p) ')'],50,CurrINI,1,0);
                                 end;
                              end;
                           else;
                              CompTime = toc;
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           clear('PhiDraws','SigmaDraws');
                           txt = 'You didn''t select any variables to predict when computing the predictive likelihood. YADA is therefore unable to help you!';
                           About(txt,'information','Predictive Likelihood - Prior Distribution',150,500,CurrINI);
                        end;
                     end;
                  end;
               else;
                  txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive likelihood for the ' lower(DataStr) ' data.'];
                  About(txt,'information','Predictive Likelihood - Prior Distribution',300,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UncondPredHeightPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if (gcbo==controls.dsgevartools.forecast.unconditional.original.height.dsgemode)|(gcbo==controls.dsgevartools.forecast.unconditional.annual.height.dsgemode);
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
            ModeValue = 1;
            if gcbo==controls.dsgevartools.forecast.unconditional.original.height.dsgemode;
               DataType = 'Original';
               IsOriginal = 1;
            else;
               DataType = 'Annualized';
               IsOriginal = 0;
            end;
         elseif (gcbo==controls.dsgevartools.forecast.unconditional.original.height.margmode)|(gcbo==controls.dsgevartools.forecast.unconditional.annual.height.margmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            ModeValue = 2;
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            if gcbo==controls.dsgevartools.forecast.unconditional.original.height.margmode;
               DataType = 'Original';
               IsOriginal = 1;
            else;
               DataType = 'Annualized';
               IsOriginal = 0;
            end;
         elseif (gcbo==controls.dsgevartools.forecast.unconditional.original.height.jointmode)|(gcbo==controls.dsgevartools.forecast.unconditional.original.height.jointmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            ModeValue = 3;
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
            if gcbo==controls.dsgevartools.forecast.unconditional.original.height.jointmode;
               DataType = 'Original';
               IsOriginal = 1;
            else;
               DataType = 'Annualized';
               IsOriginal = 0;
            end;
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Predictive Likelihood - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            HeaderStr = 'YADA - Bad MAT File';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            %
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==1;
                  SigmaEpsilon = ModeData.SigmaModeEpsilon;
                  ModeStatus = 1;
               else;
                  [Phi,SigmaEpsilon,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               end;
               if ModeStatus==1;
                  if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                     %
                     % check predictive issues
                     %
                     theta = ModeData.thetaMode;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     LowerBound = ModeData.LowerBound;
                     UniformBounds = ModeData.UniformBounds;
                     ModelParameters = ModeData.ModelParameters;
                     %
                     % try to load the AIMData, other wise use ModeData
                     %
                     if MatlabNumber>=8.0;
                        try;
                           AIMData = load(DSGEModel.AIMDataFile);
                        catch ME;
                           AIMData = ModeData.AIMData;
                        end;
                     else;
                        try;
                           AIMData = load(DSGEModel.AIMDataFile);
                        catch;
                           AIMData = ModeData.AIMData;
                        end;
                     end;
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        YDataStr = 'Actuals from original sample';
                        action = 'ok';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection',['Predictive Likelihood - ' ModeType],'','',CurrINI);
                        if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        end;
                     end;
                     %
                     if strcmp(lower(action),'ok')==1;
                        MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        MaxFH = min(MaxFH,MaxPH);
                        YData = YData(:,1:MaxFH);
                        %
                        % matrix with exogenous variables
                        %
                        X = [];
                        if DSGEModel.k>0;
                           if NumPredX<MaxFH;
                              X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                           end;
                        end;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           %
                           % we may continue and store the new selection of prediction variables
                           %
                           DSGEModel.PredictionVariables = PredictionVariables;
                           %
                           % step length used for computing the Hessian with finite diffs
                           %
                           StepLengthStr = get(controls.optimize.steplength,'String');
                           StepLength = str2num(StringTrim(StepLengthStr(DSGEModel.StepLengthHessian,:)));
                           %
                           % need to compute certain values
                           %
                           n = DSGEModel.n;
                           k = DSGEModel.k;
                           p = DSGEModel.BVARLags;
                           npk = (n*p)+k;
                           %
                           %phi = ModeData.phiMode;
                           %
                           % setup the sample
                           %
                           [y,Y,HSample,xx,XX] = GetDSGEVARData(DSGEModel);
                           if k>0;
                              DetProductMoments = CalcDeterministicProductMoments(xx,XX);
                           else;
                              DetProductMoments = [];
                           end;
                           T = size(y,2);
                           GammaHatyy = (1/T)*y*y';
                           GammaHatyY = (1/T)*y*Y';
                           GammaHatYY = (1/T)*Y*Y';
                           if ModeValue<3;
                              if isinf(lambdai)==0;
                                 logGPR = LogGammaProductRatio(n,((1+lambdai)*T)-npk,lambdai*T-npk);
                              else;
                                 logGPR = [];
                              end;
                           else;
                              if isinf(lambdai)==0;
                                 logGPR = DSGEVARLogConcLikelihoodConstant(lambdai,T,n,npk);
                              else;
                                 logGPR = [];
                              end;
                           end;
                           %
                           % check if the user wants only the plugin estimate of the predictive likelihood
                           %
                           IsPlugin = 0;
                           if isinf(lambdai)==0;
                              NumParam = (n*npk)+(n*(n+1)/2)+length(theta);
                           else;
                              NumParam = length(theta);
                           end;
                           txt = strvcat('Would you like to compute the plugin estimate only and skip the Laplace approximation?',' ',['NOTE: The Laplace approximation requires that the Hessian matrix of the log posterior kernel is computed in addition to the plugin estimate. This matrix has dimension ' int2str(NumParam) 'x' int2str(NumParam) ' for the DSGE-VAR(' num2str(lambdai) ') model with ' int2str(p) ' lags. If you answer NO then YADA will compute both sets of values.']);
                           answer = Query(txt,'question',400,['Predictive Likelihood - ' ModeType],600,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              IsPlugin = 1;
                           end;
                           %
                           % compute the joint and marginal predictive likelihood
                           %
                           tic;
                           [JointPDH,MargPDH,LaplaceMargLike,PredVars,MargPlugin,JointPlugin] = DSGEVARPredictiveLikelihoodParam(theta,thetaPositions,thetaIndex,thetaDist,PriorDist,LowerBound,UniformBounds,ModelParameters,AIMData,lambdai,T,n,p,npk,GammaHatyy,GammaHatyY,GammaHatYY,DetProductMoments,HSample,logGPR,YData,X,IsOriginal,IsPlugin,StepLength,DSGEModel,CurrINI);
                           CompTime = toc;
                           %
                           % check if we have results
                           %
                           if isempty(JointPDH)==0;
                              %
                              % ask if we should the results to disk
                              %
                              %
                              SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                              SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                              %
                              PredictionSampleStrMat = CreateFullSample(DSGEModel);
                              PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                              txt = ['Would you like to save the ' lower(ModeType) ' results from the DSGE-VAR(' num2str(lambdai) ') model with ' int2str(p) ' lags using the ' lower(DataType) ' data to disk?'];
                              answer = Query(txt,'question',200,'Save - Predictive Likelihood',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 %
                                 YNames = DSGEModel.VariableNames;
                                 file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPredictiveLikelihood-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataType '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' ModeShort '.mat']);
                                 save(file,'JointPDH','MargPDH','LaplaceMargLike','MargPlugin','JointPlugin','PredVars','YNames','PredictionSampleStrMat');
                                 %
                                 % create a matlab m-file to read the output.
                                 %
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPLike' int2str(SelectedYData) '_' strrep(num2str(lambdai,'%0.4f'),'.','_') int2str(p) DataType SelectedVarStr strrep(PredictionSampleStrMat(1,:),':','_') strrep(PredictionSampleStrMat(MaxFH,:),':','_') ModeShort '.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 if IsPlugin==0;
                                    fprintf(fid,'%%\n%% Predictive likelihood of joint predictions \n%%\n');
                                    fprintf(fid,'JointPDH = DataStructure.JointPDH;\n');
                                    fprintf(fid,'%%\n%% Predictive likelihood of marginal predictions \n%%\n');
                                    fprintf(fid,'MargPDH = DataStructure.MargPDH;\n');
                                    fprintf(fid,'%%\n%% Laplace approximation of log marginal likelihood for historical sample \n%%\n');
                                    fprintf(fid,'LaplaceMargLike = DataStructure.LaplaceMargLike;\n');
                                    NumEntries = '8';
                                    AddText = 'JointPDH (vector with the values of the joint predictive likelihood), MargPDH (vector with the marginal predictive likelihood), LaplaceMargLike (scalar with the marginal likelihood for the historical sample), ';
                                 else;
                                    NumEntries = '5';
                                    AddText = '';
                                 end;
                                 fprintf(fid,'%%\n%% Plugin estimate of the joint predictive likelihood \n%%\n');
                                 fprintf(fid,'JointPlugin = DataStructure.JointPlugin;\n');
                                 fprintf(fid,'%%\n%% Plugin estimate of the marginal predictive likelihood \n%%\n');
                                 fprintf(fid,'MargPlugin = DataStructure.MargPlugin;\n');
                                 fprintf(fid,'%%\n%% positions of predicted variables in YNames \n%%\n');
                                 fprintf(fid,'PredVars = DataStructure.PredVars;\n');
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                                 fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The predictive likelihood results at ' lower(ModeType) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: ' AddText 'MargPlugin (vector with the plugin estimate of the marginal predictive likelihood), JointPlugin (vector with the plugin estimate of the joint predictive likelihood), PredVars (cell array with the locations of the predicted variables), YNames (string matrix with the names of the observed variables), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information',['Predictive Likelihood - ' ModeType],500,600,CurrINI);
                              end;
                              %
                              % write to file
                              %
                              ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPredictiveLikelihood-' num2str(lambdai,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataType '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' ModeShort '.txt']);
                              %
                              PrintDSGEPredictiveLikelihoodTheta(ResFile,JointPDH,MargPDH,LaplaceMargLike,JointPlugin,MargPlugin,[],PredVars,PredictionSampleStrMat,'',YDataStr(SelectedYData,:),IsOriginal,IsPlugin,ModeType,lambdai,CompTime,DSGEModel,CurrINI);
                              %
                              % display the results
                              %
                              if FileExist(ResFile)==1;
                                 TextGUI(ResFile,['DSGE-VAR(' num2str(lambdai) ') Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],[ModeType ' Parameter Values'],50,CurrINI,1,0);
                                 drawnow;
                              end;
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           txt = ['You didn''t select any variables to predict when computing the predictive likelihood of the DSGE-VAR model with lambda=' num2str(lambdai) '. YADA is therefore unable to help you!'];
                           About(txt,'information',['Predictive Likelihood - ' ModeType],150,500,CurrINI);
                        end;
                     end;
                  else;
                     txt = ['The ' lower(ModeType) ' estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive likelihood for the ' lower(DataType) ' data.'];
                     About(txt,'information',[DataType ' Predictive Likelihood - ' ModeType],300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UncondPredHeightPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         if gcbo==controls.dsgevartools.forecast.unconditional.original.height.posterior;
            IsOriginal = 1;
            DataStr = 'Original';
         elseif gcbo==controls.dsgevartools.forecast.unconditional.annual.height.posterior;
            IsOriginal = 0;
            DataStr = 'Annualized';
         end;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection([DataStr ' Predictive Likelihood - Posterior Distribution'],DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            UsedDraws = size(PhiDraws,1);
            [NumDraws,NumParam] = size(thetaPostSample);
            if UsedDraws>=100
               if IsPositiveDefinite(SigmaModeEpsilon)==1;
                  if NumDraws==UsedDraws;
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws','PostSample','thetaPostSample');
                     %
                     % determine string with 1's and 0's for the selected shocks
                     %
                     SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
                     SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
                     %
                     % determine the historical sample
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - Posterior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        end;
                     end;
                     %
                     if strcmp(lower(DataAction),'ok')==1;
                        MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        MaxFH = min(MaxFH,MaxPH);
                        YData = YData(:,1:MaxFH);
                        %
                        % matrix with exogenous variables
                        %
                        X = [];
                        if DSGEModel.k>0;
                           if NumPredX<MaxFH;
                              X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                           end;
                        end;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           %
                           % we may continue and store the new selection of prediction variables
                           %
                           DSGEModel.PredictionVariables = PredictionVariables;
                           %
                           % need to compute certain values
                           %
                           n = DSGEModel.n;
                           p = DSGEModel.BVARLags;
                           npk = (n*p)+DSGEModel.k;
                           %
                           % setup the sample
                           %
                           [y,Y,HSample,xx,XX] = GetDSGEVARData(DSGEModel);
                           %
                           % check if the user wants to compute the marginal or the joint predictive likelihood
                           %
                           txt = ['Would you like to compute the marginal predictive likelikehood or the joint predictive likelihood for the ' lower(DataStr) ' data?'];
                           answer = Query(txt,'question',400,'Predictive Likelihood - Posterior Distribution',500,'yes',CurrINI,{'Marginal' 'Joint'});
                           if strcmp(lower(answer),'marginal')==1;
                              IsMarginal = 1;
                              MargStr = 'Marginal';
                           else;
                              IsMarginal = 0;
                              MargStr = 'Joint';
                           end;
                           %
                           % check if results already exist on disk
                           %
                           if DSGEVARPredictiveLikelihoodExist(DSGEModel,SelectedYData,IsOriginal,IsMarginal,1,lambda,PME,MaxFH,UsedDraws,CurrChain,controls)==1;
                              txt = ['YADA has located ' lower(MargStr) ' log posterior calculations of a DSGE-VAR with lambda=' num2str(lambda) ' and ' int2str(p) ' lags for the ' lower(DataStr) ' data for the predictive likelihood on disk. Would you like to use these results?'];
                              answer = Query(txt,'question',200,[MargStr ' Predictive Likelihood - Posterior Distribution'],500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           tic;
                           if strcmp(lower(answer),'yes')==1;
                              DoneCalc = 1;
                           else;
                              %
                              % compute the predictive likelihood
                              %
                              DoneCalc = DSGEVARPredictiveLikelihood(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,lambda,n,p,npk,HSample,YData,X,SelectedYData,IsOriginal,1,IsMarginal,CurrChain,PME,DSGEModel,CurrINI,controls);
                           end;
                           clear('PhiDraws','SigmaDraws');
                           if DoneCalc==1;
                              %
                              % we can now do the remaining modified harmonic mean estimates.
                              %
                              [LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd] = DSGEVARPredLikeMCI(SelectedYData,NumDraws,MaxFH,IsOriginal,IsMarginal,1,Weights,lambda,PME,CurrChain,DSGEModel,controls);
                              CompTime = toc;
                              %
                              if isempty(LogPredLike)==0;
                                 %
                                 % ask the user if results should be saved to file
                                 %
                                 DrawsStr = get(controls.posterior.draws,'String');
                                 TotalDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                                 %
                                 BurninStr = get(controls.posterior.burnin,'String');
                                 NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
                                 %
                                 SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                                 SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                                 %
                                 PredictionSampleStrMat = CreateFullSample(DSGEModel);
                                 PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                                 txt = ['Would you like to save the posterior draws results of the DSGE-VAR model with lambda=' num2str(lambda) ' and ' int2str(p) ' lags for the ' lower(DataStr) ' data to disk?'];
                                 answer = Query(txt,'question',300,['Save - ' MargStr ' Predictive Likelihood'],500,'no',CurrINI);
                                 if strcmp(lower(answer),'yes')==1;
                                    YNames = DSGEModel.VariableNames;
                                    file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' MargStr 'PredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(UsedDraws) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.mat']);
                                    save(file,'LogPredLike','StdErr','PredVars','YNames','PredictionSampleStrMat');
                                    %
                                    % create a matlab m-file to read the output.
                                    %
                                    mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' MargStr(1) 'PLike' int2str(SelectedYData) '_' strrep(num2str(lambda,'%0.4f'),'.','_') int2str(p) int2str(PME) DataStr(1) SelectedVarStr strrep(PredictionSampleStrMat(1,:),':','_') strrep(PredictionSampleStrMat(MaxFH,:),':','_') int2str(UsedDraws) int2str(TotalDraws) int2str(CurrChain) '.m']);
                                    fid = fopen(mfile,'wt');
                                    fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                    fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                    fprintf(fid,'%%\n%% Predictive likelihood of %s predictions \n%%\n',lower(MargStr));
                                    fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                                    fprintf(fid,'%%\n%% numerical standard errors of the log predictive likelihood of %s predictions \n%%\n',lower(MargStr));
                                    fprintf(fid,'StdErr = DataStructure.StdErr;\n');
                                    fprintf(fid,'%%\n%% positions of predicted variables in YNames \n%%\n');
                                    fprintf(fid,'PredVars = DataStructure.PredVars;\n');
                                    fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                    fprintf(fid,'YNames = DataStructure.YNames;\n');
                                    fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                                    fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                                    fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                    %
                                    % end the script file
                                    %
                                    fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                    fclose(fid);
                                    %
                                    txt = ['The ' lower(MargStr) ' predictive likelihood results for the posterior draws of a DSGE-VAR model with lambda=' num2str(lambda) ' and ' int2str(p) ' lags have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: LogPredLike (vector of dimension h with the values of the ' lower(MargStr) ' log predictive likelihood), StdErr (vector of dimension h with the numerical standard errors of the log predictive likelihood), PredVars (cell array with the locations of the predicted variables), YNames (string matrix with the names of the observed variables), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                    About(txt,'information',[MargStr ' Predictive Likelihood - Posterior Distribution'],600,650,CurrINI);
                                 end;
                                 %
                                 % write the results to text file
                                 %
                                 ResFile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR' MargStr 'PredictiveLikelihood-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '-' DataStr '-' SelectedVarStr '-' strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.') '-' int2str(UsedDraws) '.' int2str(TotalDraws) '-' int2str(CurrChain) '.txt']);
                                 %
                                 PrintDSGEPredictiveLikelihood(ResFile,LogPredLike,StdErr,PredVars,MinMaxCondLike,TestStd,[],[],[],[],PredictionSampleStrMat,'',YDataStr(SelectedYData,:),IsOriginal,IsMarginal,1,0,UsedDraws,NumBurnin,TotalDraws,lambda,CompTime,DSGEModel,CurrINI)
                                 %
                                 % display the results
                                 %
                                 if FileExist(ResFile)==1;
                                    TextGUI(ResFile,[MargStr ' Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],['Posterior Distribution - DSGE-VAR(' num2str(lambda) ',' int2str(p) ')'],50,CurrINI,1,0);
                                 end;
                              end;
                           else;
                              CompTime = toc;
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           clear('PhiDraws','SigmaDraws');
                           txt = 'You didn''t select any variables to predict when computing the predictive likelihood. YADA is therefore unable to help you!';
                           About(txt,'information','Predictive Likelihood - Posterior Distribution',150,500,CurrINI);
                        end;
                     end;
                  else;
                     clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
                     txt = ['To compute the predictive likelihood YADA requires the same number of posterior draws of both the DSGE model parameters and the VAR parameters. YADA was only able to locate ' int2str(NumDraws) ' draws of the DSGE model parameters, while ' int2str(UsedDraws) ' draws are available of the VAR parameters with lambda=' num2str(lambda) '.'];
                     About(txt,'information','Predictive Likelihood - Posterior Distribution',300,500,CurrINI);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
                  txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive likelihood for the ' lower(DataStr) ' data.'];
                  About(txt,'information','Predictive Likelihood - Posterior Distribution',300,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute the predictive likelihood of the ' lower(DataStr) ' observed variables. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information','Predictive Likelihood - Posterior Distribution',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconMargPredMomentsPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                        ModeData = [];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                        ModeData = [];
                     end;
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               UniformBounds = ModeData.UniformBounds;
               LowerBound = ModeData.LowerBound;
               thetaIndex = ModeData.thetaIndex;
            end;
         end;
         %
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                  %
                  % determine which model to consider
                  %
                  [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Marginal Predictive Moments - Prior Distribution',DSGEModel,CurrINI);
                  if isempty(lambda)==0;
                     T = HSample(2)-HSample(1)+1;
                     if isinf(lambda)==0;
                        SigmaEpsilon = ((lambda*T)/((lambda*T)+DSGEModel.n+1))*SigmaEpsilon;
                     end;
                     %
                     % load the prior draws data
                     %
                     [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
                     UsedDraws = size(PhiDraws,1);
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws');
                     %
                     % determine the historical sample
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     %MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxFH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                        end;
                     end;
                     %
                     % need to compute certain values
                     %
                     n = DSGEModel.n;
                     p = DSGEModel.BVARLags;
                     %
                     % Determine the prediction sample
                     %
                     PredictionSampleStrMat = CreateFullSample(DSGEModel);
                     PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                     SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                     %
                     % check if older calculations exists given that such may be used
                     %
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-PredictiveMoments-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(MaxFH) '.' int2str(UsedDraws) '.mat']);
                     if FileExist(MatFile)==1;
                        txt = ['YADA has located prior draws based estimates of the marginal predictive moments on disk. Would you like to use these results?'];
                        answer = Query(txt,'question',200,'Marginal Predictive Moments - Prior Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     tic;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        DoneCalc = DSGEVARPredictiveMoments(PhiDraws,SigmaDraws,Phi,SigmaEpsilon,lambda,n,p,X,MaxFH,0,[],SmpStr,0,0,DSGEModel,CurrINI,controls);
                     end;
                     if DoneCalc==1;
                        %
                        % load the results
                        %
                        CompTime = toc;
                        MPM = load(MatFile);
                        PredY = MPM.MeanPredictY;
                        n = DSGEModel.n;
                        CovPred = zeros(n,n,MaxFH);
                        for i=1:MaxFH;
                           CovPred(:,:,i) = MPM.ShockCov(:,:,i)+MPM.ParameterCov(:,:,i);
                        end;
                        %
                        if isfield(MPM,'CovPred')==0;
                           save(MatFile,'CovPred','PredictionSampleStrMat','-APPEND');
                        end;
                        if isfield(MPM,'YData')==0;
                           save(MatFile,'YData','-APPEND');
                        end;
                        %
                        % ask if the user wants to have easy access to the results
                        %
                        txt = ['The marginal predictive moments using the prior draws have been saved to the file "' GetFilename(MatFile) '" in the "' GetPath(MatFile) '" directory. Would you like YADA to write an m-file to give you direct access to these results?'];
                        answer = Query(txt,'question',300,'Marginal Predictive Moments - Prior Distribution',600,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           YNames = DSGEModel.VariableNames;
                           if isfield(MPM,'YNames')==0;
                              save(MatFile,'YNames','-APPEND');
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\' strrep(strrep(GetName(MatFile),'-',''),'.','') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                           fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                           fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables \n%%\n');
                           fprintf(fid,'MeanPredictY = DataStructure.MeanPredictY;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the marginal prediction covariances \n%%\n');
                           fprintf(fid,'CovPred = DataStructure.CovPred;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to the shock uncertainty \n%%\n');
                           fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to parameter uncertainty \n%%\n');
                           fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                           fprintf(fid,'%%\n%% matrix with the actuals for the prediction sample \n%%\n');
                           fprintf(fid,'YData = DataStructure.YData;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['You can access the marginal predictive moments estimated from the prior distribution through the file: "' GetFilename(mfile) '", located in the "' GetPath(mfile) '" directory.'];
                           About(txt,'information','Marginal Predictive Moments - Prior Distribution',500,600,CurrINI);
                        end;
                        %
                        % write to a text file
                        %
                        ResFile = strrep(MatFile,'.mat','.txt');
                        PrintPredictiveMoments(ResFile,PredY,CovPred,PredictionSampleStrMat,[],[],[],lambda,UsedDraws,UsedDraws,0,0,0,CompTime,DSGEModel,CurrINI);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Marginal Predictive Moments - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],['Prior Distribution - DSGE-VAR(' num2str(lambda) ',' int2str(p) ')'],50,CurrINI,1,0);
                        end;
                     else;
                        CompTime = toc;
                     end;
                  end;
               else;
                  txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the marginal predictive moments for the observed variables.'];
                  About(txt,'information','Marginal Predictive Moments - Prior Distribution',300,500,CurrINI);
                  drawnow;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconMargPredMomentsPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Marginal Predictive Moments - Posterior Distribution',DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            UsedDraws = size(PhiDraws,1);
            [NumDraws,NumParam] = size(thetaPostSample);
            if UsedDraws>=100
               if IsPositiveDefinite(SigmaModeEpsilon)==1;
                  if NumDraws==UsedDraws;
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws','PostSample','thetaPostSample');
                     %
                     % determine the historical sample
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     %MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxFH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                        end;
                     end;
                     %
                     % need to compute certain values
                     %
                     n = DSGEModel.n;
                     p = DSGEModel.BVARLags;
                     DrawsStr = get(controls.posterior.draws,'String');
                     NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                     BurninStr = get(controls.posterior.burnin,'String');
                     NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
                     %
                     % Determine the prediction sample
                     %
                     PredictionSampleStrMat = CreateFullSample(DSGEModel);
                     PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                     SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                     %
                     % check if older calculations exists given that such may be used
                     %
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-PredictiveMoments-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' SmpStr '-' int2str(CurrChain) '-' int2str(MaxFH) '.' int2str(UsedDraws) '-' int2str(NumPostDraws) '.mat']);
                     if FileExist(MatFile)==1;
                        txt = ['YADA has located posterior draws based estimates of the marginal predictive moments on disk. Would you like to use these results?'];
                        answer = Query(txt,'question',200,'Marginal Predictive Moments - Posterior Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     tic;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        DoneCalc = DSGEVARPredictiveMoments(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,lambda,n,p,X,MaxFH,1,Weights,SmpStr,CurrChain,PME,DSGEModel,CurrINI,controls);
                     end;
                     if DoneCalc==1;
                        %
                        % load the results
                        %
                        CompTime = toc;
                        MPM = load(MatFile);
                        PredY = MPM.MeanPredictY;
                        n = DSGEModel.n;
                        CovPred = zeros(n,n,MaxFH);
                        for i=1:MaxFH;
                           CovPred(:,:,i) = MPM.ShockCov(:,:,i)+MPM.ParameterCov(:,:,i);
                        end;
                        %
                        if isfield(MPM,'CovPred')==0;
                           save(MatFile,'CovPred','PredictionSampleStrMat','-APPEND');
                        end;
                        if isfield(MPM,'YData')==0;
                           save(MatFile,'YData','-APPEND');
                        end;
                        %
                        % ask if the user wants to have easy access to the results
                        %
                        txt = ['The marginal predictive moments using the posterior draws have been saved to the file "' GetFilename(MatFile) '" in the "' GetPath(MatFile) '" directory. Would you like YADA to write an m-file to give you direct access to these results?'];
                        answer = Query(txt,'question',300,'Marginal Predictive Moments - Posterior Distribution',600,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           YNames = DSGEModel.VariableNames;
                           if isfield(MPM,'YNames')==0;
                              save(MatFile,'YNames','-APPEND');
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\' strrep(strrep(GetName(MatFile),'-',''),'.','') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                           fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                           fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables \n%%\n');
                           fprintf(fid,'MeanPredictY = DataStructure.MeanPredictY;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the marginal prediction covariances \n%%\n');
                           fprintf(fid,'CovPred = DataStructure.CovPred;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to the shock uncertainty \n%%\n');
                           fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to parameter uncertainty \n%%\n');
                           fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                           fprintf(fid,'%%\n%% matrix with the actuals for the prediction sample \n%%\n');
                           fprintf(fid,'YData = DataStructure.YData;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['You can access the marginal predictive moments estimated from the posterior distribution through the file: "' GetFilename(mfile) '", located in the "' GetPath(mfile) '" directory.'];
                           About(txt,'information','Marginal Predictive Moments - Posterior Distribution',500,600,CurrINI);
                        end;
                        %
                        % write to a text file
                        %
                        ResFile = strrep(MatFile,'.mat','.txt');
                        PrintPredictiveMoments(ResFile,PredY,CovPred,PredictionSampleStrMat,[],[],[],lambda,UsedDraws,NumPostDraws,NumBurnin,1,0,CompTime,DSGEModel,CurrINI);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Marginal Predictive Moments - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],['Posterior Distribution - DSGE-VAR(' num2str(lambda) ',' int2str(p) ')'],50,CurrINI,1,0);
                           drawnow;
                        end;
                     else;
                        CompTime = toc;
                     end;
                  else;
                     clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
                     txt = ['To compute the marginal predictive moments YADA requires the same number of posterior draws of both the DSGE model parameters and the VAR parameters. YADA was only able to locate ' int2str(NumDraws) ' draws of the DSGE model parameters, while ' int2str(UsedDraws) ' draws are available of the VAR parameters with lambda=' num2str(lambda) '.'];
                     About(txt,'information','Marginal Predictive Moments - Posterior Distribution',300,500,CurrINI);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
                  txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the marginal predictive moments of the observed variables.'];
                  About(txt,'information','Marginal Predictive Moments - Posterior Distribution',300,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute the marginal predictive moments of the observed variables. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information','Marginal Predictive Moments - Posterior Distribution',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconStationaryPredVariancePriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                        ModeData = [];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                        ModeData = [];
                     end;
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               UniformBounds = ModeData.UniformBounds;
               LowerBound = ModeData.LowerBound;
               thetaIndex = ModeData.thetaIndex;
            end;
         end;
         if InitStatus==1;
            %
            % check is we can determine the Phi and SigmaEpsilon parameters
            %
            [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
            if Status==1;
               if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                  %
                  % determine which model to consider
                  %
                  [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Stationary Predictive Variance - Prior Distribution',DSGEModel,CurrINI);
                  if isempty(lambda)==0;
                     T = HSample(2)-HSample(1)+1;
                     if isinf(lambda)==0;
                        SigmaEpsilon = ((lambda*T)/((lambda*T)+DSGEModel.n+1))*SigmaEpsilon;
                     end;
                     %
                     % load the prior draws data
                     %
                     [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
                     UsedDraws = size(PhiDraws,1);
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws');
                     %
                     % determine the historical sample
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     [MaxFH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxFH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                        end;
                        XT = DSGEModel.X(:,LastPeriod);
                     else;
                        XT = [];
                     end;
                     %
                     % need to compute certain values
                     %
                     n = DSGEModel.n;
                     p = DSGEModel.BVARLags;
                     %
                     % check if older calculations exists given that such may be used
                     %
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-StationaryPredictiveVariance-Prior-' num2str(lambda,'%0.4f') '-' int2str(p) '-' DSGEModel.NameOfModel '-' int2str(MaxFH) '.' int2str(UsedDraws) '.mat']);
                     if FileExist(MatFile)==1;
                        txt = ['YADA has located prior draws based estimates of the stationary predictive variances on disk. Would you like to use these results?'];
                        answer = Query(txt,'question',200,'Stationary Predictive Variance - Prior Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     tic;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        DoneCalc = DSGEVARStationaryPredictiveVariance(PhiDraws,SigmaDraws,Phi,SigmaEpsilon,lambda,n,p,X,XT,MaxFH,0,[],[],0,0,DSGEModel,CurrINI);
                     end;
                     CompTime = toc;
                     clear('PhiDraws','SigmaDraws');
                     if DoneCalc==1;
                        %
                        % load the results
                        %
                        SPC = load(MatFile);
                        ShockCov = SPC.ShockCov;
                        ParameterCov = SPC.ParameterCov;
                        TotalCov = SPC.TotalCov;
                        %
                        % ask if the user wants to have easy access to the results
                        %
                        txt = ['The stationary predictive variances using the prior draws have been saved to the file "' GetFilename(MatFile) '" in the "' GetPath(MatFile) '" directory. Would you like YADA to write an m-file to give you direct access to these results?'];
                        answer = Query(txt,'question',300,'Stationary Predictive Variance - Prior Distribution',600,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           YNames = DSGEModel.VariableNames;
                           if isfield(SPC,'YNames')==0;
                              save(MatFile,'YNames','-APPEND');
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\' strrep(strrep(GetName(MatFile),'-',''),'.','') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the marginal prediction covariances \n%%\n');
                           fprintf(fid,'TotalCov = DataStructure.TotalCov;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to the shock uncertainty \n%%\n');
                           fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to parameter uncertainty \n%%\n');
                           fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['You can access the stationary predictive variances estimated from the prior distribution through the file: "' GetFilename(mfile) '", located in the "' GetPath(mfile) '" directory.'];
                           About(txt,'information','Stationary Predictive Variance - Prior Distribution',500,600,CurrINI);
                        end;
                        %
                        % write to a text file
                        %
                        ResFile = strrep(MatFile,'.mat','.txt');
                        PrintStationaryPredictiveVariance(ResFile,[],ShockCov,[],ParameterCov,TotalCov,lambda,UsedDraws,UsedDraws,0,0,CompTime,DSGEModel,CurrINI);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Stationary Predictive Variance - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],['Prior Distribution - DSGE-VAR(' num2str(lambda) ',' int2str(p) ')'],50,CurrINI,1,0);
                           drawnow;
                        end;
                     end;
                  end;
               else;
                  txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the stationary predictive variances for the observed variables.'];
                  About(txt,'information','Stationary Predictive Variance - Prior Distribution',300,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconStationaryPredVariancePosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Stationary Predictive Variance - Posterior Distribution',DSGEModel,controls,CurrINI);
         if value>0;
            lambda = Lambda(value);
            %
            % load the posterior draws
            %
            [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,0);
            UsedDraws = size(PhiDraws,1);
            [NumDraws,NumParam] = size(thetaPostSample);
            if UsedDraws>=100
               if IsPositiveDefinite(SigmaModeEpsilon)==1;
                  if NumDraws==UsedDraws;
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws','PostSample','thetaPostSample');
                     %
                     % determine the historical sample
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     [MaxFH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxFH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxFH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxFH))];
                        end;
                        XT = DSGEModel.X(:,LastPeriod);
                     else;
                        XT = [];
                     end;
                     %
                     % need to compute certain values
                     %
                     n = DSGEModel.n;
                     p = DSGEModel.BVARLags;
                     DrawsStr = get(controls.posterior.draws,'String');
                     NumPostDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                     BurninStr = get(controls.posterior.burnin,'String');
                     NumBurnin = str2num(strrep(StringTrim(BurninStr(DSGEModel.BurnInValue,:)),',',''));
                     %
                     % check if older calculations exists given that such may be used
                     %
                     MatFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-VAR-StationaryPredictiveVariance-' num2str(lambda,'%0.4f') '-' int2str(p) '-' int2str(PME) '-' DSGEModel.NameOfModel '-' int2str(CurrChain) '-' int2str(MaxFH) '.' int2str(UsedDraws) '-' int2str(NumPostDraws) '.mat']);
                     if FileExist(MatFile)==1;
                        txt = ['YADA has located posterior draws based estimates of the stationary predictive variances on disk. Would you like to use these results?'];
                        answer = Query(txt,'question',200,'Stationary Predictive Variance - Posterior Distribution',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     tic;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        DoneCalc = DSGEVARStationaryPredictiveVariance(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,lambda,n,p,X,XT,MaxFH,1,Weights,NumPostDraws,CurrChain,PME,DSGEModel,CurrINI);
                     end;
                     CompTime = toc;
                     clear('PhiDraws','SigmaDraws');
                     if DoneCalc==1;
                        %
                        % load the results
                        %
                        SPC = load(MatFile);
                        ShockCov = SPC.ShockCov;
                        ParameterCov = SPC.ParameterCov;
                        TotalCov = SPC.TotalCov;
                        %
                        % ask if the user wants to have easy access to the results
                        %
                        txt = ['The stationary predictive variances using the posterior draws have been saved to the file "' GetFilename(MatFile) '" in the "' GetPath(MatFile) '" directory. Would you like YADA to write an m-file to give you direct access to these results?'];
                        answer = Query(txt,'question',300,'Stationary Predictive Variance - Posterior Distribution',600,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           YNames = DSGEModel.VariableNames;
                           if isfield(SPC,'YNames')==0;
                              save(MatFile,'YNames','-APPEND');
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\' strrep(strrep(GetName(MatFile),'-',''),'.','') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(MatFile) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(MatFile));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the marginal prediction covariances \n%%\n');
                           fprintf(fid,'TotalCov = DataStructure.TotalCov;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to the shock uncertainty \n%%\n');
                           fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                           fprintf(fid,'%%\n%% 3D matrix with the part due to parameter uncertainty \n%%\n');
                           fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['You can access the stationary predictive variances estimated from the posterior distribution through the file: "' GetFilename(mfile) '", located in the "' GetPath(mfile) '" directory.'];
                           About(txt,'information','Stationary Predictive Variance - Posterior Distribution',500,600,CurrINI);
                        end;
                        %
                        % write to a text file
                        %
                        ResFile = strrep(MatFile,'.mat','.txt');
                        PrintStationaryPredictiveVariance(ResFile,[],ShockCov,[],ParameterCov,TotalCov,lambda,UsedDraws,NumPostDraws,NumBurnin,1,CompTime,DSGEModel,CurrINI);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Stationary Predictive Variance - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],['Posterior Distribution - DSGE-VAR(' num2str(lambda) ',' int2str(p) ')'],50,CurrINI,1,0);
                           drawnow;
                        end;
                     end;
                  else;
                     clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
                     txt = ['To compute the marginal predictive moments YADA requires the same number of posterior draws of both the DSGE model parameters and the VAR parameters. YADA was only able to locate ' int2str(NumDraws) ' draws of the DSGE model parameters, while ' int2str(UsedDraws) ' draws are available of the VAR parameters with lambda=' num2str(lambda) '.'];
                     About(txt,'information','Stationary Predictive Variance - Posterior Distribution',300,500,CurrINI);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
                  txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the stationary predictive variances of the observed variables.'];
                  About(txt,'information','Stationary Predictive Variance - Posterior Distribution',300,500,CurrINI);
               end;
            else;
               clear('PhiDraws','SigmaDraws','A0Draws','thetaPostSample','PostSample');
               txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute the marginal predictive moments of the observed variables. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
               About(txt,'information','Stationary Predictive Variance - Posterior Distribution',200,500,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UncondInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                        ModeData = [];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                        ModeData = [];
                     end;
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check which data to compute unconditional forecasts
            %
            ErrorStr = '';
            if gcbo==controls.dsgevartools.forecast.unconditional.original.initialvalues;
               DataType = 'Original';
               ForecastType = 1;
            elseif gcbo==controls.dsgevartools.forecast.unconditional.annual.initialvalues;
               DataType = 'Annualized';
               ForecastType = 2;
            elseif gcbo==controls.dsgevartools.forecast.unconditional.transformed.initialvalues;
               DataType = 'Transformed';
               ForecastType = 3;
               %
               % check if we can properly transform the data
               %
               DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
               if DTStatus==1;
                  [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               else;
                  ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
                  HeaderStr = 'Missing Transformation Information';
               end;
            end;
            if isempty(ErrorStr)==1;
               %
               % check is we can determine the Phi and SigmaEpsilon parameters
               %
               [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
               if Status==1;
                  [lambda,T] = GetLambda(DSGEModel);
                  LambdaValuesStr = '';
                  for i=1:length(lambda);
                     LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
                  end;
                  [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Predictive Distributions - Initial Values'],'','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     lambdai = lambda(value);
                     %
                     % compute the prior mode of SigmaEpsilon
                     %
                     if isinf(lambdai)==0;
                        SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                     end;
                     if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                        %
                        % check how many time periods to use for predictive distributions
                        %
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        %
                        % setup the necessary data
                        %
                        [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                        DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                        DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                        SampleStrMat = CreateFullSample(DSGEModel);
                        SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                        %
                        X = [];
                        if DSGEModel.k>0;
                           Phi0 = Phi(:,1:DSGEModel.k);
                           Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                           %
                           % setup X
                           %
                           if NumPredX<MaxPH;
                              X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                           end;
                        else;
                           Phi0 = [];
                        end;
                        %
                        % determine how many prediction paths to use
                        %
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        if NumPredPaths>1;
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPh
                        %
                        if DSGEModel.RandomNumberValue==1;
                           randn('state',0);
                        else;
                           randn('state',sum(100*clock));
                        end;
                        %
                        % compute the unconditional forecasts
                        %
                        [PredPaths,PredData,PredEventData,YObsEventData] = DSGEVARPredictionPathsParam(Phi0,Phi,SigmaEpsilon,X,HSample(2),MaxPH,lambdai,NumPredPaths,'Initial Values',ForecastType,DSGEModel,CurrINI);
                        if isempty(PredPaths)==0;
                           %
                           % the computations went OK. Compute point estimates from predictive distribution and
                           % quantiles
                           %
                           [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                           %
                           % compute the mean error bias in percent
                           %
                           MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
                           if DSGEModel.AdjustPredictionPaths==0;
                              %
                              % ask if the prediction distribution should be corrected
                              %
                              txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust the conditional predictive distribution such that its mean matches the theoretical mean?'];
                              answer = Query(txt,'question',140,'Mean Error Bias - Predictive Distributions',500,'yes',CurrINI);
                           else;
                              answer = 'Yes';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              %
                              % adjust the quantiles to match
                              %
                              for i=1:length(Quantiles);
                                 OlVal = Quantiles(i).Quantile;
                                 Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                              end;
                              MeanPrediction = PredData.PredMean;
                           end;
                           %
                           % compute the forecast errors based on the mean predictions
                           %
                           PredictionErrors = CalculatePredictionErrors(MeanPrediction,ForecastType==2,ForecastType==3,'DSGE-VAR',DSGEModel);
                           %
                           % compute the observed variables and fix the DateVector and SampleStrMat
                           %
                           if ForecastType==1;
                              Y = DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           elseif ForecastType==2;
                              Y = Annualize(DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2))),DSGEModel.annual,DSGEModel.annualscale);
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           elseif ForecastType==3;
                              [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                              Y = YData';
                              if size(Y,2)==DSGEModel.T;
                                 Y = Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 T2 = size(Y,2);
                                 if (HSample(1)-(DSGEModel.T-T2))>=1;
                                    Y = Y(:,HSample(1)-(DSGEModel.T-T2):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    Y = Y(:,1:min((HSample(2)+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 end;
                                 if length(DateVector)>size(Y,2);
                                    SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 elseif length(DateVector)<size(Y,2);
                                    Y = Y(:,size(Y,2)-length(DateVector)+1:size(Y,2));
                                 end;
                              end;
                           end;
                           %
                           % check if the user wants to save the data
                           %
                           txt = ['Would you like to save the predictive distributions for the ' lower(DataType) ' data at initial values to file?'];
                           answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-' DataType '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' num2str(lambdai,'%0.4f') '-InitialValues.mat']);
                              YNames = DSGEModel.VariableNames;
                              PredMean = PredData.PredMean;
                              ShockPaths = PredData.Shocks;
                              KernelX = PredData.KernelX;
                              KernelY = PredData.KernelY;
                              if isempty(PredEventData)==1;
                                 save(file,'PredPaths','ShockPaths','MeanPrediction','PredMean','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames');
                              else;
                                 save(file,'PredPaths','ShockPaths','MeanPrediction','PredMean','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','PredictionEvent','PredEventData','YObsEventData');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPD' DataType int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod strrep(num2str(lambdai,'%0.4f'),'.','_') 'InitVal.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%%  3D matrix with all the prediction paths for the observed variables\n%%\n');
                              fprintf(fid,'PredPaths = DataStructure.PredPaths;\n');
                              fprintf(fid,'%%\n%% Underlying draws of the residuals for all paths\n%%\n');
                              fprintf(fid,'ShockPaths = DataStructure.ShockPaths;\n');
                              fprintf(fid,'%%\n%% matrix with the sample mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              if isempty(KernelX)==0;
                                 fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                 fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                                 fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                 fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                                 KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                                 KVal = 2;
                              else;
                                 KStr = '';
                                 KVal = 0;
                              end;
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              if isempty(PredEventData)==0;
                                 fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                 fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                 fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                                 NumEntries = int2str(13+KVal);
                                 ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true prediction events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with the number of true events for prediction paths whose mean is the observed data, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                              else;
                                 NumEntries = int2str(10+KVal);
                                 ExtraEntry = '';
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The predictive distributions for the ' lower(DataType) ' data at initial values for a DSGE-VAR with lambda=' num2str(lambdai) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), PredPaths (3D matrix with the prediction paths for the observed variables), ShockPaths (3D matrix with the underlying residuals for the prediction paths), MeanPrediction (matrix with the sample mean predictions), PredMean (matrix with the population mean predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information',[DataType ' Predictive Distributions - Initial Values'],700,600,CurrINI);
                           end;
                           %
                           if isempty(PredEventData)==0;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-' DataType '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' num2str(lambdai,'%0.4f') '-InitialValues.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['DSGE-VAR(' num2str(lambdai) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,0);
                              end;
                           end;
                           %
                           % Open a dialog for displaying predictive distributions
                           %
                           DSGEVARPredictiveDistributionDLG('init',DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,PredData.KernelX,PredData.KernelY,lambdai,'Initial Values',DataType,'Unconditional',[]);
                        end;
                     else;
                        txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive distributions for the ' lower(DataType) ' data.'];
                        About(txt,'information',[DataType ' Predictive Distribution - Initial Values'],300,500,CurrINI);
                     end;
                  end;
               end;
            else;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'CondInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                        ModeData = [];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                        ModeData = [];
                     end;
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         if InitStatus==1;
            %
            % check which data to compute unconditional forecasts
            %
            ErrorStr = '';
            if gcbo==controls.dsgevartools.forecast.conditional.original.initialvalues;
               DataType = 'Original';
               ForecastType = 1;
            elseif gcbo==controls.dsgevartools.forecast.conditional.annual.initialvalues;
               DataType = 'Annualized';
               ForecastType = 2;
            elseif gcbo==controls.dsgevartools.forecast.conditional.transformed.initialvalues;
               DataType = 'Transformed';
               ForecastType = 3;
               %
               % check if we can properly transform the data
               %
               DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
               if DTStatus==1;
                  [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               else;
                  ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
                  HeaderStr = 'Missing Transformation Information';
               end;
            end;
            if isempty(ErrorStr)==1;
               %
               % check is we can determine the Phi and SigmaEpsilon parameters
               %
               [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
               if Status==1;
                  [lambda,T] = GetLambda(DSGEModel);
                  LambdaValuesStr = '';
                  for i=1:length(lambda);
                     LambdaValuesStr = strvcat(LambdaValuesStr,['lambda = ' num2str(lambda(i),'%0.4f')]);
                  end;
                  [action,value] = SelectionDlg(LambdaValuesStr,length(lambda),'Select the value of the lambda hyperparameter:','Lambda Selection','Conditional Predictive Distributions - Initial Values','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     lambdai = lambda(value);
                     %
                     % compute the prior mode of SigmaEpsilon
                     %
                     if isinf(lambdai)==0;
                        SigmaEpsilon = ((lambdai*T)/((lambdai*T)+DSGEModel.n+1))*SigmaEpsilon;
                     end;
                     if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                        %
                        % check if we can identify the A0 matrix
                        %
                        [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                        %
                        % check which conditioning method to use
                        %
                        action = 'OK';
                        if isempty(A0)==0;
                           if strcmp(get(controls.forecast.shockcontrol,'Enable'),'off')==1;
                              %
                              % let the user choose
                              %
                              [action,ShockControlMethod] = SelectionDlg(strvcat('Values for shocks','Distribution of shocks (Waggoner-Zha)','Distribution of subset of shocks'),1,'Select method for controlling conditional forecasts:','Conditional Forecast Method Selection','Conditional Predictive Distributions - Initial Values','','',CurrINI);
                              if (strcmp(lower(action),'ok')==1)&(ShockControlMethod==3);
                                 %
                                 SetShocks = 0;
                                 MixedConditionalShocks = DSGEModel.MixedConditionalShocks;
                                 while SetShocks==0;
                                    [action,positions] = SelectCondVarShockDLG('init','Shocks','Subset Method Conditioning',DSGEModel.StateShockNames,MixedConditionalShocks,CurrINI);
                                    if strcmp(lower(action),'ok')==1;
                                       if isempty(positions)==0;
                                          if length(positions)>=length(DSGEModel.ConditionalShocks);
                                             %
                                             % we're duin OK
                                             %
                                             DSGEModel.MixedConditionalShocks = positions;
                                             SetShocks = 1;
                                          else;
                                             txt = ['The number of shocks for conditional forecasting with the distribution for a subset of the shocks method (' int2str(length(positions)) ') that you selected is less than the number of conditioning assumptions (' int2str(length(DSGEModel.ConditionalShocks)) '). This is not a valid choice in YADA. You must select at least as many shocks in the subset as there are conditioning assumptions.'];
                                             About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                          end;
                                       else;
                                          txt = 'You didn''t select any shocks for conditional forecasting with the distribution for a subset of the shocks method. This is not a valid choice in YADA. You must select at least one more shock in the subset than there are conditioning assumptions and fewer than the total number of shocks.';
                                          About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                       end;
                                    else;
                                       SetShocks = 1;
                                    end;
                                 end;
                              end;
                           else;
                              ShockControlMethod = DSGEModel.ShockControlMethod;
                           end;
                        else;
                           ShockControlMethod = 2;
                        end;
                        %
                        % check if we should continue
                        %
                        if strcmp(lower(action),'ok')==1;
                           %
                           % check how many time periods to use for predictive distributions
                           %
                           [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                           %
                           % setup the necessary data
                           %
                           [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                           DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                           DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                           SampleStrMat = CreateFullSample(DSGEModel);
                           SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                           %
                           X = [];
                           if DSGEModel.k>0;
                              Phi0 = Phi(:,1:DSGEModel.k);
                              Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                              %
                              % setup X
                              %
                              if NumPredX<MaxPH;
                                 X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                              end;
                              if NumPredX>0;
                                 X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                              end;
                           else;
                              Phi0 = [];
                           end;
                           %
                           % determine how many prediction paths to use
                           %
                           NumPredPathsStr = get(controls.forecast.predpaths,'String');
                           NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                           %
                           % check the length of the conditioning sample
                           %
                           [status,NumPeriods] = CheckConditioningData(DSGEModel);
                           %
                           % check if we should adjust the Z matrix
                           %
                           if MaxPH<NumPeriods;
                              NumPeriods = MaxPH;
                           end;
                           %
                           % collect the conditioning information
                           %
                           Z = DSGEModel.Z(DSGEModel.ConditionalVariables,HSample(2)+1:HSample(2)+NumPeriods);
                           U = DSGEModel.U(DSGEModel.ConditionalVariables,HSample(2));
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           if NumPredPaths>1;
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPh
                           %
                           if DSGEModel.RandomNumberValue==1;
                              randn('state',0);
                           else;
                              randn('state',sum(100*clock));
                           end;
                           %
                           % compute the forecasts
                           %
                           if ShockControlMethod==1;
                              %
                              % direct control of the shocks
                              %
                              [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ] = DSGEVARCondPredictionPathsParam(Phi0,Phi,A0,X,Z,U,HSample(2),MaxPH,lambdai,NumPredPaths,'Initial Values',ForecastType,DSGEModel.DSGEVARShocks,DSGEModel,CurrINI);
                           elseif ShockControlMethod==2;
                              %
                              % control of the distribution of shocks
                              %
                              [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ] = DSGEVARCondPredictionPathsParamWZ(Phi0,Phi,SigmaEpsilon,X,Z,U,HSample(2),MaxPH,lambdai,NumPredPaths,'Initial Values',ForecastType,DSGEModel,CurrINI);
                           else;
                              %
                              % control of the distribution of a subset of the shocks method
                              %
                              [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ] = DSGEVARCondPredictionPathsParamMixed(Phi0,Phi,A0,X,Z,U,HSample(2),MaxPH,lambdai,NumPredPaths,'Initial Values',ForecastType,DSGEModel.DSGEVARShocks,DSGEModel,CurrINI);
                           end;
                           if isempty(PredPaths)==0;
                              %
                              % the computations went OK. Compute point estimates from predictive distribution and
                              % quantiles
                              %
                              [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                              %
                              % compute the mean error bias in percent
                              %
                              MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
                              if DSGEModel.AdjustPredictionPaths==0;
                                 %
                                 % ask if the prediction distribution should be corrected
                                 %
                                 txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust the conditional predictive distribution such that its mean matches the theoretical mean?'];
                                 answer = Query(txt,'question',140,'Mean Error Bias - Conditional Predictive Distributions',500,'yes',CurrINI);
                              else;
                                 answer = 'Yes';
                              end;
                              if strcmp(lower(answer),'yes')==1;
                                 %
                                 % adjust the quantiles to match
                                 %
                                 for i=1:length(Quantiles);
                                    OlVal = Quantiles(i).Quantile;
                                    Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                                 end;
                                 MeanPrediction = PredData.PredMean;
                              end;
                              %
                              % compute the forecast errors based on the mean predictions
                              %
                              PredictionErrors = CalculatePredictionErrors(MeanPrediction,ForecastType==2,ForecastType==3,'DSGE-VAR',DSGEModel);
                              %
                              % check if we have modesty statistics to deal with
                              %
                              if ForecastType==1;
                                 %
                                 % check how often we have a greater value of column 1 than column 2 in MultiModestyStat
                                 %
                                 MultiPValue = sum(MultiModestyStat(:,1)<=MultiModestyStat(:,2))/NumPredPaths;
                                 MeanMultiModesty = (1/NumPredPaths)*[sum(MultiModestyStat(:,1)) sum(MultiModestyStat(:,2))];
                                 MeanUniModesty = (1/NumPredPaths)*sum(UniModestyStat);
                                 if exist('YADAchi2cdf')==2;
                                    ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                 else;
                                    if exist('chi2cdf')==2;
                                       ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                    else;
                                       ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                    end;
                                 end;
                                 PrintDSGEVARModestyStatistics(DSGEModel,CurrINI,'Initial Values',lambdai,1,MaxPH,NumPeriods,1,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,UniModestyStatLZ,PredData.epsShocks,ShockControlMethod);
                              else;
                                 MultiPValue = [];
                                 MeanMultiModesty = [];
                                 MeanUniModesty = [];
                                 ChiSqPValue = [];
                              end;
                              %
                              % compute the observed variables and fix the DateVector and SampleStrMat
                              %
                              if ForecastType==1;
                                 Y = DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                 if length(DateVector)>size(Y,2);
                                    SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 end;
                              elseif ForecastType==2;
                                 Y = Annualize(DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2))),DSGEModel.annual,DSGEModel.annualscale);
                                 if length(DateVector)>size(Y,2);
                                    SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 end;
                              elseif ForecastType==3;
                                 [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                                 Y = YData';
                                 if size(Y,2)==DSGEModel.T;
                                    Y = Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    T2 = size(Y,2);
                                    if (HSample(1)-(DSGEModel.T-T2))>=1;
                                       Y = Y(:,HSample(1)-(DSGEModel.T-T2):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                    else;
                                       Y = Y(:,1:min((HSample(2)+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                    end;
                                    if length(DateVector)>size(Y,2);
                                       SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                       DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                    elseif length(DateVector)<size(Y,2);
                                       Y = Y(:,size(Y,2)-length(DateVector)+1:size(Y,2));
                                    end;
                                 end;
                              end;
                              %
                              % check if the user wants to save the data
                              %
                              txt = ['Would you like to save the conditional predictive distributions for the ' lower(DataType) ' data at initial values to file?'];
                              answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                                 file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-' DataType '-' int2str(ShockControlMethod) '.' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' num2str(lambdai,'%0.4f') '-InitialValues.mat']);
                                 YNames = DSGEModel.VariableNames;
                                 PredMean = PredData.PredMean;
                                 ShockPaths = PredData.Shocks;
                                 KernelX = PredData.KernelX;
                                 KernelY = PredData.KernelY;
                                 MeanUniModestyLZ = UniModestyStatLZ;
                                 if isempty(PredEventData)==1;
                                    save(file,'PredPaths','ShockPaths','MeanPrediction','PredMean','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ');
                                 else;
                                    save(file,'PredPaths','ShockPaths','MeanPrediction','PredMean','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','PredictionEvent','PredEventData','YObsEventData');
                                 end;
                                 %
                                 % create a matlab m-file to read the output.
                                 %
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCPD' DataType int2str(ShockControlMethod) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod strrep(num2str(lambdai,'%0.4f'),'.','_') 'InitVal.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%%  3D matrix with all the prediction paths for the observed variables\n%%\n');
                                 fprintf(fid,'PredPaths = DataStructure.PredPaths;\n');
                                 fprintf(fid,'%%\n%% Underlying draws of the residuals for all paths\n%%\n');
                                 fprintf(fid,'ShockPaths = DataStructure.ShockPaths;\n');
                                 fprintf(fid,'%%\n%% matrix with the sample mean predictions of the observed variables\n%%\n');
                                 fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                                 fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the observed variables\n%%\n');
                                 fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                                 fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                                 fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                                 fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                                 fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                                 if isempty(KernelX)==0;
                                    fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                    fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                                    fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                    fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                                    KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                                    KVal = 2;
                                 else;
                                    KStr = '';
                                    KVal = 0;
                                 end;
                                 fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                 fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                 fprintf(fid,'%%\n%% observed variables\n%%\n');
                                 fprintf(fid,'Y = DataStructure.Y;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                                 fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                                 if isempty(PredEventData)==0;
                                    fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                    fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                    fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                    fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                    fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                    fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                                    NumEntries = int2str(13+KVal);
                                    ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true prediction events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with the number of true events for prediction paths whose mean is the observed data, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                                 else;
                                    NumEntries = int2str(10+KVal);
                                    ExtraEntry = '';
                                 end;
                                 if ForecastType==1;
                                    fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                                    fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                                    fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                                    fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                                    fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                                    fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                                    fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                                    fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                                    fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                                    fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                                    NumEntries = num2str(str2num(NumEntries)+5,'%0.0f');
                                    ExtraEntry = [ExtraEntry 'MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), '];
                                 end;
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The predictive distributions for the ' lower(DataType) ' data at initial values for a DSGE-VAR with lambda=' num2str(lambdai) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), PredPaths (3D matrix with the prediction paths for the observed variables), ShockPaths (3D matrix with the underlying residuals for the prediction paths), MeanPrediction (matrix with the sample mean predictions), PredMean (matrix with the population mean predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information','Conditional Predictive Distributions - Initial Values',700,600,CurrINI);
                              end;
                              %
                              if isempty(PredEventData)==0;
                                 %
                                 % write prediction event results to file
                                 %
                                 file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-' DataType '-' int2str(ShockControlMethod) '.' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' num2str(lambdai,'%0.4f') '-InitialValues.txt']);
                                 [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                                 PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                                 %
                                 % displays the file if it exists
                                 %
                                 if FileExist(file)==1;
                                    TextGUI(file,['DSGE-VAR(' num2str(lambdai) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,0);
                                 end;
                              end;
                              %
                              % Open a dialog for displaying predictive distributions
                              %
                              DSGEVARPredictiveDistributionDLG('init',DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,PredData.KernelX,PredData.KernelY,lambdai,'Initial Values',DataType,'Conditional',[]);
                           end;
                        end;
                     else;
                        txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the conditional predictive distributions for the ' lower(DataType) ' data.'];
                        About(txt,'information','Conditional Predictive Distribution - Initial Values',300,500,CurrINI);
                     end;
                  end;
               end;
            else;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'UncondPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                        ModeData = [];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                        ModeData = [];
                     end;
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         %
         if InitStatus==1;
            ErrorStr = '';
            if gcbo==controls.dsgevartools.forecast.unconditional.original.prior;
               DataType = 'Original';
               ForecastType = 1;
            elseif gcbo==controls.dsgevartools.forecast.unconditional.annual.prior;
               DataType = 'Annualized';
               ForecastType = 2;
            elseif gcbo==controls.dsgevartools.forecast.unconditional.transformed.prior;
               DataType = 'Transformed';
               ForecastType = 3;
               %
               % check if the transformation functions work
               %
               DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
               if DTStatus==1;
                  [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               else;
                  ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
                  HeaderStr = 'Missing Transformation Information';
               end;
            end;
            %
            if isempty(ErrorStr)==1;
               %
               % check is we can determine the Phi and SigmaEpsilon parameters
               %
               [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
               if Status==1;
                  if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                     %
                     % determine which model to consider
                     %
                     [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws([DataType ' Predictive Distribution - Prior'],DSGEModel,CurrINI);
                     if isempty(lambda)==0;
                        T = HSample(2)-HSample(1)+1;
                        if isinf(lambda)==0;
                           SigmaEpsilon = ((lambda*T)/((lambda*T)+DSGEModel.n+1))*SigmaEpsilon;
                        end;
                        %
                        % load the prior draws data
                        %
                        [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
                        UsedDraws = size(PhiDraws,1);
                        %
                        % we don't need the posterior draws of A0
                        %
                        clear('A0Draws');
                        %
                        % check how many time periods to use for predictive distributions
                        %
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        %
                        % setup the necessary data
                        %
                        [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                        DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                        DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                        SampleStrMat = CreateFullSample(DSGEModel);
                        SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                        %
                        X = [];
                        if DSGEModel.k>0;
                           %
                           % setup X
                           %
                           if NumPredX<MaxPH;
                              X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                           end;
                        end;
                        %
                        % determine how many prediction paths to use
                        %
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPh
                        %
                        if DSGEModel.RandomNumberValue==1;
                           randn('state',0);
                        else;
                           randn('state',sum(100*clock));
                        end;
                        %
                        % check if the results already exist on disk
                        %
                        if DSGEVARPredictionsExist(UsedDraws,MaxPH,0,'',DataType,lambda,[],[],0,NumPredPaths,DSGEModel,controls)==1;
                           txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,[DataType ' Predictive Distribution - Prior'],500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        %
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % collect results
                           %
                           [PredEventData,PredictionEvent,ShockMean,ShockNames,ShockCov,ParameterCov] = LoadDSGEVARPredictionUncertainty(UsedDraws,MaxPH,0,'',DataType,lambda,[],[],0,NumPredPaths,DSGEModel,controls);
                           DoneCalc = 1;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % compute the predictive distribution
                           %
                           [DoneCalc,PredEventData,ShockCov,ParameterCov] = DSGEVARPredictionPaths(PhiDraws,SigmaDraws,Phi,SigmaEpsilon,X,HSample(2),MaxPH,ForecastType,[],0,[],[],lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                        end;
                        %
                        % clear everything
                        %
                        clear('PhiDraws','SigmaDraws');
                        %
                        % check if we should show any results
                        %
                        if DoneCalc==1;
                           %
                           % calculate the distribution properties
                           %
                           [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEVARPredictionDistribution(UsedDraws,MaxPH,0,'',DataType,lambda,[],[],0,[],NumPredPaths,DSGEModel,CurrINI,controls);
                           %
                           % compute the forecast errors based on the mean predictions
                           %
                           PredictionErrors = CalculatePredictionErrors(MeanPrediction,ForecastType==2,ForecastType==3,'DSGE-VAR',DSGEModel);
                           %
                           % compute the observed variables and fix the DateVector and SampleStrMat
                           %
                           if ForecastType==1;
                              Y = DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           elseif ForecastType==2;
                              Y = Annualize(DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2))),DSGEModel.annual,DSGEModel.annualscale);
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           elseif ForecastType==3;
                              [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                              Y = YData';
                              if size(Y,2)==DSGEModel.T;
                                 Y = Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 T2 = size(Y,2);
                                 if (HSample(1)-(DSGEModel.T-T2))>=1;
                                    Y = Y(:,HSample(1)-(DSGEModel.T-T2):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    Y = Y(:,1:min((HSample(2)+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 end;
                                 if length(DateVector)>size(Y,2);
                                    SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 elseif length(DateVector)<size(Y,2);
                                    Y = Y(:,size(Y,2)-length(DateVector)+1:size(Y,2));
                                 end;
                              end;
                           end;
                           %
                           % check if the user wants to save the data
                           %
                           txt = ['Would you like to save the predictive distributions for the ' lower(DataType) ' data at to file?'];
                           answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsedDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                              YNames = DSGEModel.VariableNames;
                              if isempty(PredEventData)==1;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','ShockCov','ParameterCov');
                              else;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','ShockCov','ParameterCov','PredictionEvent','PredEventData');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPDPrior' DataType(1,1:4) strrep(num2str(lambda,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(UsedDraws) 'x' int2str(NumPredPaths) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              if ForecastType==1;
                                 fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to residual/shock uncertainty\n%%\n');
                                 fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                                 fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                                 fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                                 CovDecompStr = 'ShockCov (3D error covariance matrix with part due to shock uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), ';
                              else;
                                 CovDecompStr = '';
                              end;
                              if isempty(PredEventData)==0;
                                 fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                 fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                 fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 if ForecastType==1;
                                    NumEntries = '14';
                                 else;
                                    NumEntries = '12';
                                 end;
                                 ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                              else;
                                 if ForecastType==1;
                                    NumEntries = '12';
                                 else;
                                    NumEntries = '10';
                                 end;
                                 ExtraEntry = '';
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information',[DataType ' Predictive Distributions - Prior'],500,600,CurrINI);
                           end;
                           %
                           if isempty(PredEventData)==0;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-Prior-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsedDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,UsedDraws,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['DSGE-VAR(' num2str(lambda) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                              end;
                           end;
                           %
                           % summarize the information in the uncertainty variables more conveniently
                           %
                           if isempty(ShockCov)==0;
                              [ShockVar,ParameterVar,TotalVar] = SummarizeDSGEVARPredictions(ShockCov,ParameterCov);
                              %
                              % Open a dialog for displaying predictive distributions
                              %
                              UData.ShockVar = ShockVar;
                              UData.ParameterVar = ParameterVar;
                              [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                           else;
                              UData = [];
                           end;
                           %
                           % Open a dialog for displaying predictive distributions
                           %
                           DSGEVARPredictiveDistributionDLG('init',DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,KernelX,KernelY,lambda,'Prior Distribution',DataType,'Unconditional',UData);
                        end;
                     end;
                  else;
                     txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive distributions for the ' lower(DataType) ' data.'];
                     About(txt,'information',[DataType ' Predictive Distribution - Prior'],300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % retrieve data on initial values for the DSGE model
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if InitStatus==0;
            %
            % check if we can load the data somehow else
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               try;
                  ModeData = load(ModeFile);
                  InitStatus = 1;
                  Status = 1;
               catch;
                  [Status,Lambda,Files] = VerifyDSGEVARMarginalMode(DSGEModel);
                  if Status==0;
                     [Status,Lambda,Files] = VerifyDSGEVARJointMode(DSGEModel);
                  end;
                  ModeData = [];
               end;
            end;
            if (isempty(ModeData)==1)&(Status==1);
               ErrorStr = '';
               if isempty(ModeData)==1;
                  ModeFile = StringTrim(Files(1,:));
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                        ModeData = [];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                        InitStatus = 1;
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                        ModeData = [];
                     end;
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
            if isempty(ModeData)==0;
               InitStatus = 1;
               theta = ModeData.theta;
               thetaPositions = ModeData.thetaPositions;
               ModelParameters = ModeData.ModelParameters;
               thetaDist = ModeData.thetaDist;
               PriorDist = ModeData.PriorDist;
               LowerBound = ModeData.LowerBound;
            end;
         end;
         %
         if InitStatus==1;
            ErrorStr = '';
            if gcbo==controls.dsgevartools.forecast.conditional.original.prior;
               DataType = 'Original';
               ForecastType = 1;
            elseif gcbo==controls.dsgevartools.forecast.conditional.annual.prior;
               DataType = 'Annualized';
               ForecastType = 2;
            elseif gcbo==controls.dsgevartools.forecast.conditional.transformed.prior;
               DataType = 'Transformed';
               ForecastType = 3;
               %
               % check if the transformation functions work
               %
               DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
               if DTStatus==1;
                  [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               else;
                  ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
                  HeaderStr = 'Missing Transformation Information';
               end;
            end;
            %
            if isempty(ErrorStr)==1;
               %
               % check is we can determine the Phi and SigmaEpsilon parameters
               %
               [Phi,SigmaEpsilon,A,H,R,F,B0,AIMData,DetProductMoments,HSample,Status] = GetDSGEVARPriorParameters(theta,thetaPositions,ModelParameters,PriorDist,thetaDist,LowerBound,DSGEModel,CurrINI);
               if Status==1;
                  if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                     %
                     % determine which model to consider
                     %
                     [lambda,PriorDrawsFile] = GetVARDSGEPriorDraws('Conditional Predictive Distribution - Prior',DSGEModel,CurrINI);
                     if isempty(lambda)==0;
                        T = HSample(2)-HSample(1)+1;
                        if isinf(lambda)==0;
                           SigmaEpsilon = ((lambda*T)/((lambda*T)+DSGEModel.n+1))*SigmaEpsilon;
                        end;
                        %
                        % determine A0 just in case we need it
                        %
                        [A0,IdentifyStatus] = DSGEVARIdentifyShocks(SigmaEpsilon,H,B0,DSGEModel.DSGEVARShocks,HSample);
                        %
                        % load the prior draws data
                        %
                        [PhiDraws,SigmaDraws,A0Draws] = LoadVARDSGEPriorDraws(DSGEModel,PriorDrawsFile);
                        UsedDraws = size(PhiDraws,1);
                        %
                        % check which conditioning method to use
                        %
                        action = 'OK';
                        if isempty(A0)==0;
                           if strcmp(get(controls.forecast.shockcontrol,'Enable'),'off')==1;
                              %
                              % let the user choose
                              %
                              [action,ShockControlMethod] = SelectionDlg(strvcat('Values for shocks','Distribution of shocks (Waggoner-Zha)','Distribution of subset of shocks'),1,'Select method for controlling conditional forecasts:','Conditional Forecast Method Selection','Conditional Predictive Distribution - Prior','','',CurrINI);
                              if (strcmp(lower(action),'ok')==1)&(ShockControlMethod==3);
                                 %
                                 SetShocks = 0;
                                 MixedConditionalShocks = DSGEModel.MixedConditionalShocks;
                                 while SetShocks==0;
                                    [action,positions] = SelectCondVarShockDLG('init','Shocks','Subset Method Conditioning',DSGEModel.StateShockNames,MixedConditionalShocks,CurrINI);
                                    if strcmp(lower(action),'ok')==1;
                                       if isempty(positions)==0;
                                          if length(positions)>=length(DSGEModel.ConditionalShocks);
                                             %
                                             % we're duin OK
                                             %
                                             DSGEModel.MixedConditionalShocks = positions;
                                             SetShocks = 1;
                                          else;
                                             txt = ['The number of shocks for conditional forecasting with the distribution for a subset of the shocks method (' int2str(length(positions)) ') that you selected is less than the number of conditioning assumptions (' int2str(length(DSGEModel.ConditionalShocks)) '). This is not a valid choice in YADA. You must select at least as many shocks in the subset as there are conditioning assumptions.'];
                                             About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                          end;
                                       else;
                                          txt = 'You didn''t select any shocks for conditional forecasting with the distribution for a subset of the shocks method. This is not a valid choice in YADA. You must select at least one more shock in the subset than there are conditioning assumptions and fewer than the total number of shocks.';
                                          About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                       end;
                                    else;
                                       SetShocks = 1;
                                    end;
                                 end;
                              end;
                           else;
                              ShockControlMethod = DSGEModel.ShockControlMethod;
                           end;
                        else;
                           ShockControlMethod = 2;
                        end;
                        %
                        if strcmp(lower(action),'ok')==1;
                           %
                           % check which draws we can clear from memory
                           %
                           if ShockControlMethod==1;
                              clear('SigmaDraws');
                              ShockMethod = 'Direct-';
                           elseif ShockControlMethod==2;
                              clear('A0Draws');
                              ShockMethod = 'WaggonerZha-';
                           else;
                              clear('SigmaDraws');
                              ShockMethod = 'Subset-';
                           end;
                           %
                           % check how many time periods to use for predictive distributions
                           %
                           [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                           %
                           % setup the necessary data
                           %
                           [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                           DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                           DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                           SampleStrMat = CreateFullSample(DSGEModel);
                           SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                           %
                           X = [];
                           if DSGEModel.k>0;
                              %
                              % setup X
                              %
                              if NumPredX<MaxPH;
                                 X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                              end;
                              if NumPredX>0;
                                 X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                              end;
                           end;
                           %
                           % determine how many prediction paths to use
                           %
                           NumPredPathsStr = get(controls.forecast.predpaths,'String');
                           NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                           %
                           % check the length of the conditioning sample
                           %
                           [status,NumPeriods] = CheckConditioningData(DSGEModel);
                           %
                           % check if we should adjust the Z matrix
                           %
                           if MaxPH<NumPeriods;
                              NumPeriods = MaxPH;
                           end;
                           %
                           % collect the conditioning information
                           %
                           Z = DSGEModel.Z(DSGEModel.ConditionalVariables,HSample(2)+1:HSample(2)+NumPeriods);
                           U = DSGEModel.U(DSGEModel.ConditionalVariables,HSample(2));
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPh
                           %
                           if DSGEModel.RandomNumberValue==1;
                              randn('state',0);
                           else;
                              randn('state',sum(100*clock));
                           end;
                           %
                           % check if the results already exist on disk
                           %
                           if DSGEVARPredictionsExist(UsedDraws,MaxPH,1,ShockMethod,DataType,lambda,[],[],0,NumPredPaths,DSGEModel,controls)==1;
                              txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                              answer = Query(txt,'question',150,'Conditional Predictive Distribution - Prior',500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           %
                           if strcmp(lower(answer),'yes')==1;
                              %
                              % collect results
                              %
                              [PredEventData,PredictionEvent,ShockMean,ShockNames,ShockCov,ParameterCov] = LoadDSGEVARPredictionUncertainty(UsedDraws,MaxPH,1,ShockMethod,DataType,lambda,[],[],0,NumPredPaths,DSGEModel,controls);
                              DoneCalc = 1;
                           else;
                              %
                              % ask if we should conduct prediction event calculations
                              %
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                    DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                 end;
                                 %
                                 % make sure that the prediction event length does not exceed the maximum prediction horizon
                                 %
                                 for i=1:DSGEModel.n;
                                    if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                       DSGEModel.PredictionEvent(i,3) = MaxPH;
                                    end;
                                 end;
                                 %
                                 % display a dialog for setting up an prediction event
                                 %
                                 [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    DSGEModel.PredictionEvent = PredictionEvent;
                                    DSGEModel.RunPredictionEvent = 1;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                              %
                              % compute the predictive distribution
                              %
                              if ShockControlMethod==1;
                                 [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPaths(PhiDraws,A0Draws,Phi,A0,X,Z,U,HSample(2),MaxPH,ForecastType,[],0,[],[],lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                              elseif ShockControlMethod==2;
                                 [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPathsWZ(PhiDraws,SigmaDraws,Phi,SigmaEpsilon,X,Z,U,HSample(2),MaxPH,ForecastType,[],0,[],[],lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                              else;
                                 [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPathsMixed(PhiDraws,A0Draws,Phi,A0,X,Z,U,HSample(2),MaxPH,ForecastType,[],0,[],[],lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                              end;
                           end;
                           %
                           % clear all draws
                           %
                           if ShockControlMethod==1;
                              clear('PhiDraws','A0Draws');
                           elseif ShockControlMethod==2;
                              clear('PhiDraws','SigmaDraws');
                           else;
                              clear('PhiDraws','A0Draws');
                           end;
                           if DoneCalc==1;
                              %
                              % calculate the distribution properties
                              %
                              [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEVARPredictionDistribution(UsedDraws,MaxPH,1,ShockMethod,DataType,lambda,[],[],0,[],NumPredPaths,DSGEModel,CurrINI,controls);
                              %
                              % compute the forecast errors based on the mean predictions
                              %
                              PredictionErrors = CalculatePredictionErrors(MeanPrediction,ForecastType==2,ForecastType==3,'DSGE-VAR',DSGEModel);
                              %
                              % compute the observed variables and fix the DateVector and SampleStrMat
                              %
                              if ForecastType==1;
                                 Y = DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                 if length(DateVector)>size(Y,2);
                                    SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 end;
                                 %
                                 % deal with modesty analysis
                                 %
                                 [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = DSGEVARModestyStatistics(DSGEModel.OutputDirectory,ShockMethod,DSGEModel.NameOfModel,MaxPH,UsedDraws,UsedDraws,NumPredPaths,lambda,[],[],0,DSGEModel.n,DSGEModel.BVARLags);
                                 if exist('YADAchi2cdf')==2;
                                    ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                 else;
                                    if exist('chi2cdf')==2;
                                       ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                    else;
                                       ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                    end;
                                 end;
                                 %
                                 % print results to file
                                 %
                                 PrintDSGEVARModestyStatistics(DSGEModel,CurrINI,[],lambda,0,MaxPH,NumPeriods,UsedDraws,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,ShockMean,ShockControlMethod);
                              elseif ForecastType==2;
                                 Y = Annualize(DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2))),DSGEModel.annual,DSGEModel.annualscale);
                                 if length(DateVector)>size(Y,2);
                                    SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 end;
                                 ChiSqPValue = [];
                                 MultiPValue = [];
                                 MeanMultiModesty = [];
                                 MeanUniModesty = [];
                                 MeanUniModestyLZ = [];
                              elseif ForecastType==3;
                                 [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                                 Y = YData';
                                 if size(Y,2)==DSGEModel.T;
                                    Y = Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    T2 = size(Y,2);
                                    if (HSample(1)-(DSGEModel.T-T2))>=1;
                                       Y = Y(:,HSample(1)-(DSGEModel.T-T2):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                    else;
                                       Y = Y(:,1:min((HSample(2)+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                    end;
                                    if length(DateVector)>size(Y,2);
                                       SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                       DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                    elseif length(DateVector)<size(Y,2);
                                       Y = Y(:,size(Y,2)-length(DateVector)+1:size(Y,2));
                                    end;
                                 end;
                                 ChiSqPValue = [];
                                 MultiPValue = [];
                                 MeanMultiModesty = [];
                                 MeanUniModesty = [];
                                 MeanUniModestyLZ = [];
                              end;
                              %
                              % check if the user wants to save the data
                              %
                              txt = ['Would you like to save the conditional predictive distributions for the ' lower(DataType) ' data at to file?'];
                              answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                                 %
                                 file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-Prior-' int2str(ShockControlMethod) '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsedDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                                 YNames = DSGEModel.VariableNames;
                                 if isempty(PredEventData)==1;
                                    save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','ShockCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ');
                                 else;
                                    save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','ShockCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','PredictionEvent','PredEventData');
                                 end;
                                 %
                                 % create a matlab m-file to read the output.
                                 %
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCPDPrior' int2str(ShockControlMethod) DataType(1,1:4) strrep(num2str(lambda,'%0.4f'),'.','_') int2str(DSGEModel.BVARLags) int2str(UsedDraws) 'x' int2str(NumPredPaths) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                                 fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                                 fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                                 fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                                 fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                                 fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                                 fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                                 fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                                 fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                 fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                                 fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                 fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                 fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                 fprintf(fid,'%%\n%% observed variables\n%%\n');
                                 fprintf(fid,'Y = DataStructure.Y;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                                 fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                                 if ForecastType==1;
                                    fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to residual/shock uncertainty\n%%\n');
                                    fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                                    fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                                    fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                                    fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                                    fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                                    fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                                    fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                                    fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                                    fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                                    fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                                    fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                                    fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                                    fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                                    CovDecompStr = 'ShockCov (3D error covariance matrix with part due to shock uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), ';
                                 else;
                                    CovDecompStr = '';
                                 end;
                                 if isempty(PredEventData)==0;
                                    fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                    fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                    fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                    fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                    if ForecastType==1;
                                       NumEntries = '19';
                                    else;
                                       NumEntries = '12';
                                    end;
                                    ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                                 else;
                                    if ForecastType==1;
                                       NumEntries = '17';
                                    else;
                                       NumEntries = '10';
                                    end;
                                    ExtraEntry = '';
                                 end;
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The conditional predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information','Conditional Predictive Distributions - Prior',500,600,CurrINI);
                              end;
                              %
                              if isempty(PredEventData)==0;
                                 %
                                 % write prediction event results to file
                                 %
                                 file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-Prior-' int2str(ShockControlMethod) '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(DSGEModel.BVARLags) '-' int2str(UsedDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                                 [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                                 PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,UsedDraws,CurrINI);
                                 %
                                 % displays the file if it exists
                                 %
                                 if FileExist(file)==1;
                                    TextGUI(file,['DSGE-VAR(' num2str(lambda) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution - Conditional Predictions',50,CurrINI,1,0);
                                    drawnow;
                                    pause(0.02);
                                 end;
                              end;
                              %
                              % summarize the information in the uncertainty variables more conveniently
                              %
                              if isempty(ShockCov)==0;
                                 [ShockVar,ParameterVar,TotalVar] = SummarizeDSGEVARPredictions(ShockCov,ParameterCov);
                                 %
                                 % Open a dialog for displaying predictive distributions
                                 %
                                 UData.ShockVar = ShockVar;
                                 UData.ParameterVar = ParameterVar;
                                 [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                              else;
                                 UData = [];
                              end;
                              %
                              % Open a dialog for displaying predictive distributions
                              %
                              DSGEVARPredictiveDistributionDLG('init',DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,KernelX,KernelY,lambda,'Prior Distribution',DataType,'Conditional',UData);
                           end;
                        else;
                           clear('PhiDraws','A0Draws','SigmaDraws');
                        end;
                     end;
                  else;
                     txt = ['The initial values estimate of the residual covariance matrix of the DSGE-VAR model is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the conditional predictive distributions for the ' lower(DataType) ' data.'];
                     About(txt,'information','Conditional Predictive Distribution - Prior',300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UncondPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if (gcbo==controls.dsgevartools.forecast.unconditional.original.margmode)|(gcbo==controls.dsgevartools.forecast.unconditional.annual.margmode)|(gcbo==controls.dsgevartools.forecast.unconditional.transformed.margmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.forecast.unconditional.original.jointmode)|(gcbo==controls.dsgevartools.forecast.unconditional.annual.jointmode)|(gcbo==controls.dsgevartools.forecast.unconditional.transformed.jointmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.forecast.unconditional.original.dsgemode)|(gcbo==controls.dsgevartools.forecast.unconditional.annual.dsgemode)|(gcbo==controls.dsgevartools.forecast.unconditional.transformed.dsgemode);
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % check which data to compute unconditional forecasts
         %
         if (gcbo==controls.dsgevartools.forecast.unconditional.original.margmode)|(gcbo==controls.dsgevartools.forecast.unconditional.original.jointmode)|(gcbo==controls.dsgevartools.forecast.unconditional.original.dsgemode);
            DataType = 'Original';
            ForecastType = 1;
         elseif (gcbo==controls.dsgevartools.forecast.unconditional.annual.margmode)|(gcbo==controls.dsgevartools.forecast.unconditional.annual.jointmode)|(gcbo==controls.dsgevartools.forecast.unconditional.annual.dsgemode);
            DataType = 'Annualized';
            ForecastType = 2;
         elseif (gcbo==controls.dsgevartools.forecast.unconditional.transformed.margmode)|(gcbo==controls.dsgevartools.forecast.unconditional.transformed.jointmode)|(gcbo==controls.dsgevartools.forecast.unconditional.transformed.dsgemode);
            DataType = 'Transformed';
            ForecastType = 3;
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',[DataType ' Predictive Distribution - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            HeaderStr = 'YADA - Bad MAT File';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            %
            % check if we should check transformations functions
            %
            if (isempty(ErrorStr)==1)&(ForecastType==3);
               DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
               if DTStatus==1;
                  [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               else;
                  ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
                  HeaderStr = 'Missing Transformation Information';
               end;
            end;
            %
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==1;
                  Phi = ModeData.PhiMode;
                  SigmaEpsilon = ModeData.SigmaModeEpsilon;
                  ModeStatus = 1;
               else;
                  [Phi,SigmaEpsilon,A0,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
               end;
               if ModeStatus==1;
                  if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     %
                     % setup the necessary data
                     %
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     X = [];
                     if DSGEModel.k>0;
                        Phi0 = Phi(:,1:DSGEModel.k);
                        Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                        %
                        % setup X
                        %
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                        end;
                     else;
                        Phi0 = [];
                     end;
                     %
                     % determine how many prediction paths to use
                     %
                     NumPredPathsStr = get(controls.forecast.predpaths,'String');
                     NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     %
                     % ask if we should conduct prediction event calculations
                     %
                     if NumPredPaths>1;
                        txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                        answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                           DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                        end;
                        %
                        % make sure that the prediction event length does not exceed the maximum prediction horizon
                        %
                        for i=1:DSGEModel.n;
                           if DSGEModel.PredictionEvent(i,3)>MaxPH;
                              DSGEModel.PredictionEvent(i,3) = MaxPH;
                           end;
                        end;
                        %
                        % display a dialog for setting up an prediction event
                        %
                        [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                        if strcmp(lower(action),'ok')==1;
                           DSGEModel.PredictionEvent = PredictionEvent;
                           DSGEModel.RunPredictionEvent = 1;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                     %
                     % we can now compute the predictive distribution for the endogenous variables
                     % up to period MaxPh
                     %
                     if DSGEModel.RandomNumberValue==1;
                        randn('state',0);
                     else;
                        randn('state',sum(100*clock));
                     end;
                     %
                     % compute the unconditional forecasts
                     %
                     [PredPaths,PredData,PredEventData,YObsEventData] = DSGEVARPredictionPathsParam(Phi0,Phi,SigmaEpsilon,X,HSample(2),MaxPH,lambdai,NumPredPaths,ModeType,ForecastType,DSGEModel,CurrINI);
                     if isempty(PredPaths)==0;
                        %
                        % the computations went OK. Compute point estimates from predictive distribution and
                        % quantiles
                        %
                        [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                        %
                        % compute the mean error bias in percent
                        %
                        MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
                        if DSGEModel.AdjustPredictionPaths==0;
                           %
                           % ask if the prediction distribution should be corrected
                           %
                           txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust the conditional predictive distribution such that its mean matches the theoretical mean?'];
                           answer = Query(txt,'question',140,'Mean Error Bias - Predictive Distributions',500,'yes',CurrINI);
                        else;
                           answer = 'Yes';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % adjust the quantiles to match
                           %
                           for i=1:length(Quantiles);
                              OlVal = Quantiles(i).Quantile;
                              Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                           end;
                           MeanPrediction = PredData.PredMean;
                        end;
                        %
                        % compute the forecast errors based on the mean predictions
                        %
                        PredictionErrors = CalculatePredictionErrors(MeanPrediction,ForecastType==2,ForecastType==3,'DSGE-VAR',DSGEModel);
                        %
                        % compute the observed variables and fix the DateVector and SampleStrMat
                        %
                        if ForecastType==1;
                           Y = DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                           if length(DateVector)>size(Y,2);
                              SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                        elseif ForecastType==2;
                           Y = Annualize(DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2))),DSGEModel.annual,DSGEModel.annualscale);
                           if length(DateVector)>size(Y,2);
                              SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                        elseif ForecastType==3;
                           [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                           Y = YData';
                           if size(Y,2)==DSGEModel.T;
                              Y = Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              T2 = size(Y,2);
                              if (HSample(1)-(DSGEModel.T-T2))>=1;
                                 Y = Y(:,HSample(1)-(DSGEModel.T-T2):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 Y = Y(:,1:min((HSample(2)+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                              end;
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              elseif length(DateVector)<size(Y,2);
                                 Y = Y(:,size(Y,2)-length(DateVector)+1:size(Y,2));
                              end;
                           end;
                        end;
                        %
                        % check if the user wants to save the data
                        %
                        txt = ['Would you like to save the predictive distributions for the ' lower(DataType) ' data at ' lower(ModeType) ' to file?'];
                        answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-' DataType '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' num2str(lambdai,'%0.4f') '-' ModeShort '.mat']);
                           YNames = DSGEModel.VariableNames;
                           PredMean = PredData.PredMean;
                           ShockPaths = PredData.Shocks;
                           KernelX = PredData.KernelX;
                           KernelY = PredData.KernelY;
                           if isempty(PredEventData)==1;
                              save(file,'PredPaths','ShockPaths','MeanPrediction','PredMean','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames');
                           else;
                              save(file,'PredPaths','ShockPaths','MeanPrediction','PredMean','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','PredictionEvent','PredEventData','YObsEventData');
                           end;
                           %
                           % create a matlab m-file to read the output.
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPD' DataType int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod strrep(num2str(lambdai,'%0.4f'),'.','_') ModeShort '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%%  3D matrix with all the prediction paths for the observed variables\n%%\n');
                           fprintf(fid,'PredPaths = DataStructure.PredPaths;\n');
                           fprintf(fid,'%%\n%% Underlying draws of the residuals for all paths\n%%\n');
                           fprintf(fid,'ShockPaths = DataStructure.ShockPaths;\n');
                           fprintf(fid,'%%\n%% matrix with the sample mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                           fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the observed variables\n%%\n');
                           fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                           fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                           fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                           fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                           if isempty(KernelX)==0;
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                              KVal = 2;
                           else;
                              KStr = '';
                              KVal = 0;
                           end;
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           if isempty(PredEventData)==0;
                              fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                              fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                              fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                              fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                              fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                              fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                              NumEntries = int2str(13+KVal);
                              ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true prediction events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with the number of true events for prediction paths whose mean is the observed data, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                           else;
                              NumEntries = int2str(10+KVal);
                              ExtraEntry = '';
                           end;
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The predictive distributions for the ' lower(DataType) ' data at ' lower(ModeType) ' for a DSGE-VAR with lambda=' num2str(lambdai) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), PredPaths (3D matrix with the prediction paths for the observed variables), ShockPaths (3D matrix with the underlying residuals for the prediction paths), MeanPrediction (matrix with the sample mean predictions), PredMean (matrix with the population mean predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information',[DataType ' Predictive Distributions - ' ModeType],700,600,CurrINI);
                        end;
                        %
                        if isempty(PredEventData)==0;
                           %
                           % write prediction event results to file
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-' DataType '-' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' num2str(lambdai,'%0.4f') '-' ModeShort '.txt']);
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                           %
                           % displays the file if it exists
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['DSGE-VAR(' num2str(lambdai) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],[ModeType ' Parameter Values'],50,CurrINI,1,0);
                           end;
                        end;
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        DSGEVARPredictiveDistributionDLG('init',DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,PredData.KernelX,PredData.KernelY,lambdai,ModeType,DataType,'Unconditional',[]);
                     end;
                  else;
                     txt = ['The ' lower(ModeType) ' estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive distributions for the ' lower(DataType) ' data.'];
                     About(txt,'information',[DataType ' Predictive Distribution - ' ModeType],300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % determine string with 1's and 0's for the selected shocks
         %
         SelectedShockStr = ismember((1:length(DSGEModel.StateShockPositions)),DSGEModel.DSGEVARShocks);
         SelectedShockStr = strrep(num2str(SelectedShockStr),' ','');
         %
         if (gcbo==controls.dsgevartools.forecast.conditional.original.margmode)|(gcbo==controls.dsgevartools.forecast.conditional.annual.margmode)|(gcbo==controls.dsgevartools.forecast.conditional.transformed.margmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARMarginalMode(DSGEModel);
            ModeType = 'Marginal Posterior Mode';
            ModeShort = 'MargMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\MarginalPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.forecast.conditional.original.jointmode)|(gcbo==controls.dsgevartools.forecast.conditional.annual.jointmode)|(gcbo==controls.dsgevartools.forecast.conditional.transformed.jointmode);
            [Status,lambda,ModeFiles] = VerifyDSGEVARJointMode(DSGEModel);
            ModeType = 'Joint Posterior Mode';
            ModeShort = 'JointMode';
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\JointPosteriorMode-DSGEVAR-MaxMargLike-' int2str(DSGEModel.BVARLags) '-' SelectedShockStr '-' DSGEModel.NameOfModel '.mat']);
         elseif (gcbo==controls.dsgevartools.forecast.conditional.original.dsgemode)|(gcbo==controls.dsgevartools.forecast.conditional.annual.dsgemode)|(gcbo==controls.dsgevartools.forecast.conditional.transformed.dsgemode);
            Status = 1;
            lambda = GetLambda(DSGEModel);
            MatFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            ModeFiles = '';
            for i=1:length(lambda);
               ModeFiles = strvcat(ModeFiles,MatFile);
            end;
            ModeType = 'DSGE Posterior Mode';
            ModeShort = 'DSGEMode';
         end;
         %
         % check which data to compute unconditional forecasts
         %
         if (gcbo==controls.dsgevartools.forecast.conditional.original.margmode)|(gcbo==controls.dsgevartools.forecast.conditional.original.jointmode)|(gcbo==controls.dsgevartools.forecast.conditional.original.dsgemode);
            DataType = 'Original';
            ForecastType = 1;
         elseif (gcbo==controls.dsgevartools.forecast.conditional.annual.margmode)|(gcbo==controls.dsgevartools.forecast.conditional.annual.jointmode)|(gcbo==controls.dsgevartools.forecast.conditional.annual.dsgemode);
            DataType = 'Annualized';
            ForecastType = 2;
         elseif (gcbo==controls.dsgevartools.forecast.conditional.transformed.margmode)|(gcbo==controls.dsgevartools.forecast.conditional.transformed.jointmode)|(gcbo==controls.dsgevartools.forecast.conditional.transformed.dsgemode);
            DataType = 'Transformed';
            ForecastType = 3;
         end;
         %
         % get suitable data for the selection dialog
         %
         if length(lambda)>1;
            [LambdaValuesStr,SuggestedLambdaPosition] = GetLambdaSelectionData(MatFile,lambda);
            [action,value] = SelectionDlg(LambdaValuesStr,SuggestedLambdaPosition,'Select the value of the lambda hyperparameter:','Lambda Selection',['Conditional Predictive Distribution - ' ModeType],'','',CurrINI);
         else;
            action = 'OK';
            value = 1;
         end;
         if strcmp(lower(action),'ok')==1;
            lambdai = lambda(value);
            %
            % try to load the mode-file
            %
            ModeFile = StringTrim(ModeFiles(value,:));
            ErrorStr = '';
            HeaderStr = 'YADA - Bad MAT File';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            %
            % check if we should check transformations functions
            %
            if (isempty(ErrorStr)==1)&(ForecastType==3);
               DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
               if DTStatus==1;
                  [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
               else;
                  ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
                  HeaderStr = 'Missing Transformation Information';
               end;
            end;
            %
            if isempty(ErrorStr)==1;
               if isfield(ModeData,'PhiMode')==1;
                  Phi = ModeData.PhiMode;
                  SigmaEpsilon = ModeData.SigmaModeEpsilon;
                  ModeStatus = 1;
                  DSGEVARShocks = ModeData.DSGEModel.DSGEVARShocks;
               else;
                  [Phi,SigmaEpsilon,ModeData.A0Mode,IdentifyStatus,ModeStatus] = GetDSGEVARParameters(ModeData.thetaMode,ModeData.thetaPositions,ModeData.ModelParameters,ModeData.PriorDist,ModeData.thetaDist,ModeData.LowerBound,lambdai,DSGEModel,CurrINI);
                  DSGEVARShocks = DSGEModel.DSGEVARShocks;
               end;
               if ModeStatus==1;
                  if IsPositiveSemiDefinite(SigmaEpsilon)==1;
                     if isfield(ModeData,'A0Mode')==1;
                        A0 = ModeData.A0Mode;
                     else;
                        A0 = [];
                     end;
                     %
                     % check which conditioning method to use
                     %
                     action = 'OK';
                     if isempty(A0)==0;
                        if strcmp(get(controls.forecast.shockcontrol,'Enable'),'off')==1;
                           %
                           % let the user choose
                           %
                           [action,ShockControlMethod] = SelectionDlg(strvcat('Values for shocks','Distribution of shocks (Waggoner-Zha)','Distribution of subset of shocks'),1,'Select method for controlling conditional forecasts:','Conditional Forecast Method Selection',['Conditional Predictive Distributions - ' ModeType],'','',CurrINI);
                           if (strcmp(lower(action),'ok')==1)&(ShockControlMethod==3);
                              %
                              SetShocks = 0;
                              MixedConditionalShocks = DSGEModel.MixedConditionalShocks;
                              while SetShocks==0;
                                 [action,positions] = SelectCondVarShockDLG('init','Shocks','Subset Method Conditioning',DSGEModel.StateShockNames,MixedConditionalShocks,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    if isempty(positions)==0;
                                       if length(positions)>=length(DSGEModel.ConditionalShocks);
                                          %
                                          % we're duin OK
                                          %
                                          DSGEModel.MixedConditionalShocks = positions;
                                          SetShocks = 1;
                                       else;
                                          txt = ['The number of shocks for conditional forecasting with the distribution for a subset of the shocks method (' int2str(length(positions)) ') that you selected is less than the number of conditioning assumptions (' int2str(length(DSGEModel.ConditionalShocks)) '). This is not a valid choice in YADA. You must select at least as many shocks in the subset as there are conditioning assumptions.'];
                                          About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                       end;
                                    else;
                                       txt = 'You didn''t select any shocks for conditional forecasting with the distribution for a subset of the shocks method. This is not a valid choice in YADA. You must select at least one more shock in the subset than there are conditioning assumptions and fewer than the total number of shocks.';
                                       About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                    end;
                                 else;
                                    SetShocks = 1;
                                 end;
                              end;
                           end;
                        else;
                           ShockControlMethod = DSGEModel.ShockControlMethod;
                        end;
                     else;
                        ShockControlMethod = 2;
                     end;
                     %
                     % check if we should continue
                     %
                     if strcmp(lower(action),'ok')==1;
                        %
                        % check how many time periods to use for predictive distributions
                        %
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        %
                        % setup the necessary data
                        %
                        [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                        DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                        DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                        SampleStrMat = CreateFullSample(DSGEModel);
                        SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                        %
                        X = [];
                        if DSGEModel.k>0;
                           Phi0 = Phi(:,1:DSGEModel.k);
                           Phi = Phi(:,DSGEModel.k+1:size(Phi,2));
                           %
                           % setup X
                           %
                           if NumPredX<MaxPH;
                              X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                           end;
                        else;
                           Phi0 = [];
                        end;
                        %
                        % determine how many prediction paths to use
                        %
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                        %
                        % check the length of the conditioning sample
                        %
                        [status,NumPeriods] = CheckConditioningData(DSGEModel);
                        %
                        % check if we should adjust the Z matrix
                        %
                        if MaxPH<NumPeriods;
                           NumPeriods = MaxPH;
                        end;
                        %
                        % collect the conditioning information
                        %
                        Z = DSGEModel.Z(DSGEModel.ConditionalVariables,HSample(2)+1:HSample(2)+NumPeriods);
                        U = DSGEModel.U(DSGEModel.ConditionalVariables,HSample(2));
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        if NumPredPaths>1;
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPh
                        %
                        if DSGEModel.RandomNumberValue==1;
                           randn('state',0);
                        else;
                           randn('state',sum(100*clock));
                        end;
                        %
                        % compute the forecasts
                        %
                        if ShockControlMethod==1;
                           %
                           % direct control of the shocks
                           %
                           [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ] = DSGEVARCondPredictionPathsParam(Phi0,Phi,A0,X,Z,U,HSample(2),MaxPH,lambdai,NumPredPaths,ModeType,ForecastType,DSGEVARShocks,DSGEModel,CurrINI);
                        elseif ShockControlMethod==2;
                           %
                           % control of the distribution of shocks
                           %
                           [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ] = DSGEVARCondPredictionPathsParamWZ(Phi0,Phi,SigmaEpsilon,X,Z,U,HSample(2),MaxPH,lambdai,NumPredPaths,ModeType,ForecastType,DSGEModel,CurrINI);
                        else;
                           %
                           % control of the distribution of a subset of the shocks method
                           %
                           [PredPaths,PredData,PredEventData,YObsEventData,MultiModestyStat,UniModestyStat,UniModestyStatLZ] = DSGEVARCondPredictionPathsParamMixed(Phi0,Phi,A0,X,Z,U,HSample(2),MaxPH,lambdai,NumPredPaths,ModeType,ForecastType,DSGEVARShocks,DSGEModel,CurrINI);
                        end;
                        if isempty(PredPaths)==0;
                           %
                           % the computations went OK. Compute point estimates from predictive distribution and
                           % quantiles
                           %
                           [MeanPrediction,Quantiles] = PredictionPathDistribution(PredPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                           %
                           % compute the mean error bias in percent
                           %
                           MeanErrorBias = max(max(abs((PredData.PredMean-MeanPrediction)./PredData.PredMean)));
                           if DSGEModel.AdjustPredictionPaths==0;
                              %
                              % ask if the prediction distribution should be corrected
                              %
                              txt = ['The absolute mean error bias from using the simulated draws is ' num2str(MeanErrorBias,'%0.4f') ' percent relative to the theoretical mean. Would you like to adjust the conditional predictive distribution such that its mean matches the theoretical mean?'];
                              answer = Query(txt,'question',140,'Mean Error Bias - Conditional Predictive Distributions',500,'yes',CurrINI);
                           else;
                              answer = 'Yes';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              %
                              % adjust the quantiles to match
                              %
                              for i=1:length(Quantiles);
                                 OlVal = Quantiles(i).Quantile;
                                 Quantiles(i).Quantile = Quantiles(i).Quantile+(PredData.PredMean-MeanPrediction);
                              end;
                              MeanPrediction = PredData.PredMean;
                           end;
                           %
                           % compute the forecast errors based on the mean predictions
                           %
                           PredictionErrors = CalculatePredictionErrors(MeanPrediction,ForecastType==2,ForecastType==3,'DSGE-VAR',DSGEModel);
                           %
                           % check if we have modesty statistics to deal with
                           %
                           if ForecastType==1;
                              %
                              % check how often we have a greater value of column 1 than column 2 in MultiModestyStat
                              %
                              MultiPValue = sum(MultiModestyStat(:,1)<=MultiModestyStat(:,2))/NumPredPaths;
                              MeanMultiModesty = (1/NumPredPaths)*[sum(MultiModestyStat(:,1)) sum(MultiModestyStat(:,2))];
                              MeanUniModesty = (1/NumPredPaths)*sum(UniModestyStat);
                              if exist('YADAchi2cdf')==2;
                                 ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                              else;
                                 if exist('chi2cdf')==2;
                                    ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                 else;
                                    ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                 end;
                              end;
                              PrintDSGEVARModestyStatistics(DSGEModel,CurrINI,ModeType,lambdai,1,MaxPH,NumPeriods,1,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,UniModestyStatLZ,PredData.epsShocks,ShockControlMethod);
                           else;
                              MultiPValue = [];
                              MeanMultiModesty = [];
                              MeanUniModesty = [];
                              ChiSqPValue = [];
                           end;
                           %
                           % compute the observed variables and fix the DateVector and SampleStrMat
                           %
                           if ForecastType==1;
                              Y = DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           elseif ForecastType==2;
                              Y = Annualize(DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2))),DSGEModel.annual,DSGEModel.annualscale);
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                           elseif ForecastType==3;
                              [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                              Y = YData';
                              if size(Y,2)==DSGEModel.T;
                                 Y = Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 T2 = size(Y,2);
                                 if (HSample(1)-(DSGEModel.T-T2))>=1;
                                    Y = Y(:,HSample(1)-(DSGEModel.T-T2):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    Y = Y(:,1:min((HSample(2)+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 end;
                                 if length(DateVector)>size(Y,2);
                                    SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 elseif length(DateVector)<size(Y,2);
                                    Y = Y(:,size(Y,2)-length(DateVector)+1:size(Y,2));
                                 end;
                              end;
                           end;
                           %
                           % check if the user wants to save the data
                           %
                           txt = ['Would you like to save the conditional predictive distributions for the ' lower(DataType) ' data at ' lower(ModeType) ' to file?'];
                           answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-' DataType '-' int2str(ShockControlMethod) '.' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' num2str(lambdai,'%0.4f') '-' ModeShort '.mat']);
                              YNames = DSGEModel.VariableNames;
                              PredMean = PredData.PredMean;
                              ShockPaths = PredData.Shocks;
                              KernelX = PredData.KernelX;
                              KernelY = PredData.KernelY;
                              MeanUniModestyLZ = UniModestyStatLZ;
                              if isempty(PredEventData)==1;
                                 save(file,'PredPaths','ShockPaths','MeanPrediction','PredMean','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ');
                              else;
                                 save(file,'PredPaths','ShockPaths','MeanPrediction','PredMean','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','PredictionEvent','PredEventData','YObsEventData');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCPD' DataType int2str(ShockControlMethod) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod strrep(num2str(lambdai,'%0.4f'),'.','_') ModeShort '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%%  3D matrix with all the prediction paths for the observed variables\n%%\n');
                              fprintf(fid,'PredPaths = DataStructure.PredPaths;\n');
                              fprintf(fid,'%%\n%% Underlying draws of the residuals for all paths\n%%\n');
                              fprintf(fid,'ShockPaths = DataStructure.ShockPaths;\n');
                              fprintf(fid,'%%\n%% matrix with the sample mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the population mean of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredMean = DataStructure.PredMean;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              if isempty(KernelX)==0;
                                 fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                 fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                                 fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                 fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                                 KStr = 'KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), ';
                                 KVal = 2;
                              else;
                                 KStr = '';
                                 KVal = 0;
                              end;
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              if isempty(PredEventData)==0;
                                 fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                 fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                 fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 fprintf(fid,'%%\n%% matrix with observed event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'YObsEventData = DataStructure.YObsEventData;\n');
                                 NumEntries = int2str(13+KVal);
                                 ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true prediction events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), YObsEventData (matrix with the number of true events for prediction paths whose mean is the observed data, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                              else;
                                 NumEntries = int2str(10+KVal);
                                 ExtraEntry = '';
                              end;
                              if ForecastType==1;
                                 fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                                 fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                                 fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                                 fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                                 fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                                 fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                                 fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                                 fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                                 fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                                 fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                                 NumEntries = num2str(str2num(NumEntries)+5,'%0.0f');
                                 ExtraEntry = [ExtraEntry 'MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), '];
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The predictive distributions for the ' lower(DataType) ' data at ' lower(ModeType) ' for a DSGE-VAR with lambda=' num2str(lambdai) ' have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), PredPaths (3D matrix with the prediction paths for the observed variables), ShockPaths (3D matrix with the underlying residuals for the prediction paths), MeanPrediction (matrix with the sample mean predictions), PredMean (matrix with the population mean predictions), PredictionErrors (matrix with errors for the mean predictions), Quantiles (vector structure with 2 fields: Quantile and percent), ' KStr 'PredDateVector (vector with dates for the predictions), Y (matrix) with the observed variables, ' ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information',['Conditional Predictive Distributions - ' ModeType],700,600,CurrINI);
                           end;
                           %
                           if isempty(PredEventData)==0;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-' DataType '-' int2str(ShockControlMethod) '.' num2str(MaxPH,'%0.0f') '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '-' num2str(lambdai,'%0.4f') '-' ModeShort '.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,YObsEventData,DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['DSGE-VAR(' num2str(lambdai) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],[ModeType ' Parameter Values'],50,CurrINI,1,0);
                              end;
                           end;
                           %
                           % Open a dialog for displaying predictive distributions
                           %
                           DSGEVARPredictiveDistributionDLG('init',DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,PredData.KernelX,PredData.KernelY,lambdai,ModeType,DataType,'Conditional',[]);
                        end;
                     end;
                  else;
                     txt = ['The ' lower(ModeType) ' estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambdai) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive distributions for the ' lower(DataType) ' data.'];
                     About(txt,'information',['Conditional Predictive Distribution - ' ModeType],300,500,CurrINI);
                  end;
               end;
            else;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UncondPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ErrorStr = '';
         if gcbo==controls.dsgevartools.forecast.unconditional.original.posterior;
            DataType = 'Original';
            ForecastType = 1;
         elseif gcbo==controls.dsgevartools.forecast.unconditional.annual.posterior;
            DataType = 'Annualized';
            ForecastType = 2;
         elseif gcbo==controls.dsgevartools.forecast.unconditional.transformed.posterior;
            DataType = 'Transformed';
            ForecastType = 3;
            %
            % check if the transformation functions work
            %
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection([DataType ' Predictive Distribution - Posterior'],DSGEModel,controls,CurrINI);
            if value>0;
               lambda = Lambda(value);
               %
               % load the posterior draws
               %
               [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,1);
               clear('thetaPostSample','PostSample');
               UsedDraws = size(PhiDraws,1);
               if UsedDraws>=100;
                  if IsPositiveSemiDefinite(SigmaModeEpsilon)==1;
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws');
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     %
                     % setup the necessary data
                     %
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     X = [];
                     if DSGEModel.k>0;
                        %
                        % setup X
                        %
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                        end;
                     end;
                     %
                     % determine how many prediction paths to use
                     %
                     NumPredPathsStr = get(controls.forecast.predpaths,'String');
                     NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     %
                     % we can now compute the predictive distribution for the endogenous variables
                     % up to period MaxPh
                     %
                     if DSGEModel.RandomNumberValue==1;
                        randn('state',0);
                     else;
                        randn('state',sum(100*clock));
                     end;
                     %
                     % check if the results already exist on disk
                     %
                     if DSGEVARPredictionsExist(UsedDraws,MaxPH,0,'',DataType,lambda,PME,CurrChain,1,NumPredPaths,DSGEModel,controls)==1;
                        txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,[DataType ' Predictive Distribution - Posterior'],500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     %
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % collect results
                        %
                        [PredEventData,PredictionEvent,ShockMean,ShockNames,ShockCov,ParameterCov] = LoadDSGEVARPredictionUncertainty(UsedDraws,MaxPH,0,'',DataType,lambda,PME,CurrChain,1,NumPredPaths,DSGEModel,controls);
                        DoneCalc = 1;
                     else;
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                        answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % compute the predictive distribution
                        %
                        [DoneCalc,PredEventData,ShockCov,ParameterCov] = DSGEVARPredictionPaths(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,X,HSample(2),MaxPH,ForecastType,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                     end;
                     %
                     % clear everything
                     %
                     clear('PhiDraws','SigmaDraws');
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % calculate the distribution properties
                        %
                        [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEVARPredictionDistribution(UsedDraws,MaxPH,0,'',DataType,lambda,PME,CurrChain,1,Weights,NumPredPaths,DSGEModel,CurrINI,controls);
                        %
                        % compute the forecast errors based on the mean predictions
                        %
                        PredictionErrors = CalculatePredictionErrors(MeanPrediction,ForecastType==2,ForecastType==3,'DSGE-VAR',DSGEModel);
                        %
                        % compute the observed variables and fix the DateVector and SampleStrMat
                        %
                        if ForecastType==1;
                           Y = DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                           if length(DateVector)>size(Y,2);
                              SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                        elseif ForecastType==2;
                           Y = Annualize(DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2))),DSGEModel.annual,DSGEModel.annualscale);
                           if length(DateVector)>size(Y,2);
                              SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                        elseif ForecastType==3;
                           [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                           Y = YData';
                           if size(Y,2)==DSGEModel.T;
                              Y = Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                           else;
                              T2 = size(Y,2);
                              if (HSample(1)-(DSGEModel.T-T2))>=1;
                                 Y = Y(:,HSample(1)-(DSGEModel.T-T2):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 Y = Y(:,1:min((HSample(2)+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                              end;
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              elseif length(DateVector)<size(Y,2);
                                 Y = Y(:,size(Y,2)-length(DateVector)+1:size(Y,2));
                              end;
                           end;
                        end;
                        %
                        % check if the user wants to save the data
                        %
                        txt = ['Would you like to save the predictive distributions for the ' lower(DataType) ' data at to file?'];
                        answer = Query(txt,'question',140,'Save - Predictive Distributions',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                           DrawsStr = get(controls.posterior.draws,'String');
                           NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(PME) '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                           YNames = DSGEModel.VariableNames;
                           if isempty(PredEventData)==1;
                              save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','ShockCov','ParameterCov');
                           else;
                              save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','ShockCov','ParameterCov','PredictionEvent','PredEventData');
                           end;
                           %
                           % create a matlab m-file to read the output.
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPD' DataType(1,1:4) strrep(num2str(lambda,'%0.4f'),'.','_') int2str(PME) int2str(DSGEModel.BVARLags) int2str(UsedDraws) 'x' int2str(NumPredPaths) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                           fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                           fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                           fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                           fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                           fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                           fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                           fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                           fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                           fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           if ForecastType==1;
                              fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to residual/shock uncertainty\n%%\n');
                              fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                              fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                              fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                              CovDecompStr = 'ShockCov (3D error covariance matrix with part due to shock uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), ';
                           else;
                              CovDecompStr = '';
                           end;
                           if isempty(PredEventData)==0;
                              fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                              fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                              fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                              fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                              if ForecastType==1;
                                 NumEntries = '14';
                              else;
                                 NumEntries = '12';
                              end;
                              ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                           else;
                              if ForecastType==1;
                                 NumEntries = '12';
                              else;
                                 NumEntries = '10';
                              end;
                              ExtraEntry = '';
                           end;
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information',[DataType ' Predictive Distributions - Posterior'],500,600,CurrINI);
                        end;
                        %
                        if isempty(PredEventData)==0;
                           %
                           % write prediction event results to file
                           %
                           DrawsStr = get(controls.posterior.draws,'String');
                           NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(PME) '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,UsedDraws,CurrINI);
                           %
                           % displays the file if it exists
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['DSGE-VAR(' num2str(lambda) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                           end;
                        end;
                        %
                        % summarize the information in the uncertainty variables more conveniently
                        %
                        if isempty(ShockCov)==0;
                           [ShockVar,ParameterVar,TotalVar] = SummarizeDSGEVARPredictions(ShockCov,ParameterCov);
                           %
                           % Open a dialog for displaying predictive distributions
                           %
                           UData.ShockVar = ShockVar;
                           UData.ParameterVar = ParameterVar;
                           [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                        else;
                           UData = [];
                        end;
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        DSGEVARPredictiveDistributionDLG('init',DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,KernelX,KernelY,lambda,'Posterior Distribution',DataType,'Unconditional',UData);
                     end;
                  else;
                     clear('PhiDraws','SigmaDraws','A0Draws');
                     txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the predictive distributions for the ' lower(DataType) ' data.'];
                     About(txt,'information',[DataType ' Predictive Distribution - Posterior'],300,500,CurrINI);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws');
                  txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute the predictive distribution of the ' lower(DataType) ' observed variables. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
                  About(txt,'information',[DataType ' Predictive Distribution - Posterior'],200,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ErrorStr = '';
         if gcbo==controls.dsgevartools.forecast.conditional.original.posterior;
            DataType = 'Original';
            ForecastType = 1;
         elseif gcbo==controls.dsgevartools.forecast.conditional.annual.posterior;
            DataType = 'Annualized';
            ForecastType = 2;
         elseif gcbo==controls.dsgevartools.forecast.conditional.transformed.posterior;
            DataType = 'Transformed';
            ForecastType = 3;
            %
            % check if the transformation functions work
            %
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Conditional Predictive Distribution - Posterior',DSGEModel,controls,CurrINI);
            if value>0;
               lambda = Lambda(value);
               %
               % load the posterior draws
               %
               [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,1);
               clear('thetaPostSample','PostSample');
               UsedDraws = size(PhiDraws,1);
               if UsedDraws>=100;
                  if IsPositiveSemiDefinite(SigmaModeEpsilon)==1;
                     %
                     % check which conditioning method to use
                     %
                     action = 'OK';
                     if isempty(A0Mode)==0;
                        if strcmp(get(controls.forecast.shockcontrol,'Enable'),'off')==1;
                           %
                           % let the user choose
                           %
                           [action,ShockControlMethod] = SelectionDlg(strvcat('Values for shocks','Distribution of shocks (Waggoner-Zha)','Distribution of subset of shocks'),1,'Select method for controlling conditional forecasts:','Conditional Forecast Method Selection','Conditional Predictive Distribution - Posterior','','',CurrINI);
                           if (strcmp(lower(action),'ok')==1)&(ShockControlMethod==3);
                              %
                              SetShocks = 0;
                              MixedConditionalShocks = DSGEModel.MixedConditionalShocks;
                              while SetShocks==0;
                                 [action,positions] = SelectCondVarShockDLG('init','Shocks','Subset Method Conditioning',DSGEModel.StateShockNames,MixedConditionalShocks,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    if isempty(positions)==0;
                                       if length(positions)>=length(DSGEModel.ConditionalShocks);
                                          %
                                          % we're duin OK
                                          %
                                          DSGEModel.MixedConditionalShocks = positions;
                                          SetShocks = 1;
                                       else;
                                          txt = ['The number of shocks for conditional forecasting with the distribution for a subset of the shocks method (' int2str(length(positions)) ') that you selected is less than the number of conditioning assumptions (' int2str(length(DSGEModel.ConditionalShocks)) '). This is not a valid choice in YADA. You must select at least as many shocks in the subset as there are conditioning assumptions.'];
                                          About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                       end;
                                    else;
                                       txt = 'You didn''t select any shocks for conditional forecasting with the distribution for a subset of the shocks method. This is not a valid choice in YADA. You must select at least one more shock in the subset than there are conditioning assumptions and fewer than the total number of shocks.';
                                       About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                    end;
                                 else;
                                    SetShocks = 1;
                                 end;
                              end;
                           end;
                        else;
                           ShockControlMethod = DSGEModel.ShockControlMethod;
                        end;
                     else;
                        ShockControlMethod = 2;
                     end;
                     %
                     % check if we should continue
                     %
                     if strcmp(lower(action),'ok')==1;
                        %
                        % check which draws we can clear from memory
                        %
                        if ShockControlMethod==1;
                           clear('SigmaDraws');
                           ShockMethod = 'Direct-';
                        elseif ShockControlMethod==2;
                           clear('A0Draws');
                           ShockMethod = 'WaggonerZha-';
                        else;
                           clear('SigmaDraws');
                           ShockMethod = 'Subset-';
                        end;
                        %
                        % check how many time periods to use for predictive distributions
                        %
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        %
                        % setup the necessary data
                        %
                        [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                        DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                        DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                        SampleStrMat = CreateFullSample(DSGEModel);
                        SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                        %
                        X = [];
                        if DSGEModel.k>0;
                           %
                           % setup X
                           %
                           if NumPredX<MaxPH;
                              X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                           end;
                        end;
                        %
                        % determine how many prediction paths to use
                        %
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                        %
                        % determine the total number of available draws
                        %
                        DrawsStr = get(controls.posterior.draws,'String');
                        NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                        %
                        % check the length of the conditioning sample
                        %
                        [status,NumPeriods] = CheckConditioningData(DSGEModel);
                        %
                        % check if we should adjust the Z matrix
                        %
                        if MaxPH<NumPeriods;
                           NumPeriods = MaxPH;
                        end;
                        %
                        % collect the conditioning information
                        %
                        Z = DSGEModel.Z(DSGEModel.ConditionalVariables,HSample(2)+1:HSample(2)+NumPeriods);
                        U = DSGEModel.U(DSGEModel.ConditionalVariables,HSample(2));
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPh
                        %
                        if DSGEModel.RandomNumberValue==1;
                           randn('state',0);
                        else;
                           randn('state',sum(100*clock));
                        end;
                        %
                        % check if the results already exist on disk
                        %
                        if DSGEVARPredictionsExist(UsedDraws,MaxPH,1,ShockMethod,DataType,lambda,PME,CurrChain,1,NumPredPaths,DSGEModel,controls)==1;
                           txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,'Conditional Predictive Distribution - Posterior',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        %
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % collect results
                           %
                           [PredEventData,PredictionEvent,ShockMean,ShockNames,ShockCov,ParameterCov] = LoadDSGEVARPredictionUncertainty(UsedDraws,MaxPH,1,ShockMethod,DataType,lambda,PME,CurrChain,1,NumPredPaths,DSGEModel,controls);
                           DoneCalc = 1;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % compute the predictive distribution
                           %
                           if ShockControlMethod==1;
                              [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPaths(PhiDraws,A0Draws,PhiMode,A0Mode,X,Z,U,HSample(2),MaxPH,ForecastType,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                           elseif ShockControlMethod==2;
                              [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPathsWZ(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,X,Z,U,HSample(2),MaxPH,ForecastType,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                           else;
                              [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPathsMixed(PhiDraws,A0Draws,PhiMode,A0Mode,X,Z,U,HSample(2),MaxPH,ForecastType,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                           end;
                        end;
                        %
                        % clear everything
                        %
                        if ShockControlMethod==1;
                           clear('PhiDraws','A0Draws');
                        elseif ShockControlMethod==2;
                           clear('PhiDraws','SigmaDraws');
                        else;
                           clear('PhiDraws','A0Draws');
                        end;
                        if DoneCalc==1;
                           %
                           % calculate the distribution properties
                           %
                           [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY] = DSGEVARPredictionDistribution(UsedDraws,MaxPH,1,ShockMethod,DataType,lambda,PME,CurrChain,1,Weights,NumPredPaths,DSGEModel,CurrINI,controls);
                           %
                           % compute the forecast errors based on the mean predictions
                           %
                           PredictionErrors = CalculatePredictionErrors(MeanPrediction,ForecastType==2,ForecastType==3,'DSGE-VAR',DSGEModel);
                           %
                           % compute the observed variables and fix the DateVector and SampleStrMat
                           %
                           if ForecastType==1;
                              Y = DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                              %
                              % deal with modesty analysis
                              %
                              [MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ] = DSGEVARModestyStatistics(DSGEModel.OutputDirectory,ShockMethod,DSGEModel.NameOfModel,MaxPH,UsedDraws,NumDraws,NumPredPaths,lambda,PME,CurrChain,1,DSGEModel.n,DSGEModel.BVARLags);
                              if exist('YADAchi2cdf')==2;
                                 ChiSqPValue = 1-YADAchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                              else;
                                 if exist('chi2cdf')==2;
                                    ChiSqPValue = 1-chi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                 else;
                                    ChiSqPValue = 1-YADACheapchi2cdf(MeanMultiModesty(1),DSGEModel.n);
                                 end;
                              end;
                              %
                              % print results to file
                              %
                              PrintDSGEVARModestyStatistics(DSGEModel,CurrINI,CurrChain,lambda,1,MaxPH,NumPeriods,UsedDraws,NumPredPaths,MultiPValue,MeanMultiModesty,MeanUniModesty,MeanUniModestyLZ,ShockMean,ShockControlMethod);
                           elseif ForecastType==2;
                              Y = Annualize(DSGEModel.Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2))),DSGEModel.annual,DSGEModel.annualscale);
                              if length(DateVector)>size(Y,2);
                                 SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                              ChiSqPValue = [];
                              MultiPValue = [];
                              MeanMultiModesty = [];
                              MeanUniModesty = [];
                              MeanUniModestyLZ = [];
                           elseif ForecastType==3;
                              [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                              Y = YData';
                              if size(Y,2)==DSGEModel.T;
                                 Y = Y(:,HSample(1):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 T2 = size(Y,2);
                                 if (HSample(1)-(DSGEModel.T-T2))>=1;
                                    Y = Y(:,HSample(1)-(DSGEModel.T-T2):min((HSample(2)+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    Y = Y(:,1:min((HSample(2)+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                 end;
                                 if length(DateVector)>size(Y,2);
                                    SampleStrMat = SampleStrMat(length(DateVector)-size(Y,2)+1:size(SampleStrMat,1),:);
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 elseif length(DateVector)<size(Y,2);
                                    Y = Y(:,size(Y,2)-length(DateVector)+1:size(Y,2));
                                 end;
                              end;
                              ChiSqPValue = [];
                              MultiPValue = [];
                              MeanMultiModesty = [];
                              MeanUniModesty = [];
                              MeanUniModestyLZ = [];
                           end;
                           %
                           % check if the user wants to save the data
                           %
                           txt = ['Would you like to save the conditional predictive distributions for the ' lower(DataType) ' data at to file?'];
                           answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-' int2str(ShockControlMethod) '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(PME) '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                              YNames = DSGEModel.VariableNames;
                              if isempty(PredEventData)==1;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','ShockCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ');
                              else;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','ShockCov','ParameterCov','MeanMultiModesty','MultiPValue','ChiSqPValue','MeanUniModesty','MeanUniModestyLZ','PredictionEvent','PredEventData');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCPD' int2str(ShockControlMethod) DataType(1,1:4) strrep(num2str(lambda,'%0.4f'),'.','_') int2str(PME) int2str(DSGEModel.BVARLags) int2str(UsedDraws) 'x' int2str(NumPredPaths) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              if ForecastType==1;
                                 fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to residual/shock uncertainty\n%%\n');
                                 fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                                 fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                                 fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                                 fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                                 fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                                 fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                                 fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                                 fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                                 fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                                 fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                                 fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                                 fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                                 fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                                 CovDecompStr = 'ShockCov (3D error covariance matrix with part due to shock uncertainty), ParameterCov (3D covariance matrix with part due to parameter uncertainty), MeanMultiModesty (1x2 vector with the mean of the multivariate modesty statistic and the mean of the multivariate reference modesty statistic), MultiPValue (scalar with the tail probability of the modesty statistic), ChiSqPValue (scalar with the tail probability of the modesty statistic using the chi-square distribution), MeanUniModesty (vector with the univariate modesty statistic from ALLV), MeanUniModestyLZ (vector with the univariate Leeper-Zha modesty statistic), ';
                              else;
                                 CovDecompStr = '';
                              end;
                              if isempty(PredEventData)==0;
                                 fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                 fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                 fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 if ForecastType==1;
                                    NumEntries = '19';
                                 else;
                                    NumEntries = '12';
                                 end;
                                 ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                              else;
                                 if ForecastType==1;
                                    NumEntries = '17';
                                 else;
                                    NumEntries = '10';
                                 end;
                                 ExtraEntry = '';
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The conditional predictive distributions have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','Conditional Predictive Distributions - Posterior',500,600,CurrINI);
                           end;
                           %
                           if isempty(PredEventData)==0;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-' int2str(ShockControlMethod) '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(PME) '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,UsedDraws,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['DSGE-VAR(' num2str(lambda) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Conditional Predictions',50,CurrINI,1,0);
                                 drawnow;
                                 pause(0.02);
                              end;
                           end;
                           %
                           % summarize the information in the uncertainty variables more conveniently
                           %
                           if isempty(ShockCov)==0;
                              [ShockVar,ParameterVar,TotalVar] = SummarizeDSGEVARPredictions(ShockCov,ParameterCov);
                              %
                              % Open a dialog for displaying predictive distributions
                              %
                              UData.ShockVar = ShockVar;
                              UData.ParameterVar = ParameterVar;
                              [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                           else;
                              UData = [];
                           end;
                           %
                           % Open a dialog for displaying predictive distributions
                           %
                           DSGEVARPredictiveDistributionDLG('init',DSGEModel,CurrINI,Y,DateVector,SampleStrMat,MeanPrediction,Quantiles,KernelX,KernelY,lambda,'Posterior Distribution',DataType,'Conditional',UData);
                        end;
                     else;
                        clear('PhiDraws','A0Draws','SigmaDraws');
                     end;
                  else;
                     clear('PhiDraws','SigmaDraws','A0Draws');
                     txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute the conditional predictive distributions for the ' lower(DataType) ' data.'];
                     About(txt,'information','Conditional Predictive Distribution - Posterior',300,500,CurrINI);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws');
                  txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute the conditional predictive distribution of the ' lower(DataType) ' observed variables. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
                  About(txt,'information','Conditional Predictive Distribution - Posterior',200,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UncondPredictionEvent'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ErrorStr = '';
         if gcbo==controls.dsgevartools.forecast.unconditional.original.predevent;
            DataType = 'Original';
            ForecastType = 1;
         elseif gcbo==controls.dsgevartools.forecast.unconditional.annual.predevent;
            DataType = 'Annualized';
            ForecastType = 2;
         elseif gcbo==controls.dsgevartools.forecast.unconditional.transformed.predevent;
            DataType = 'Transformed';
            ForecastType = 3;
            %
            % check if the transformation functions work
            %
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection([DataType ' Predictive Distribution - Posterior'],DSGEModel,controls,CurrINI);
            if value>0;
               lambda = Lambda(value);
               %
               % load the posterior draws
               %
               [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,1);
               clear('thetaPostSample','PostSample');
               UsedDraws = size(PhiDraws,1);
               if UsedDraws>=100;
                  if IsPositiveSemiDefinite(SigmaModeEpsilon)==1;
                     %
                     % we don't need the posterior draws of A0
                     %
                     clear('A0Draws');
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     %
                     % setup the necessary data
                     %
                     [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                     DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                     DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                     SampleStrMat = CreateFullSample(DSGEModel);
                     SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                     %
                     X = [];
                     if DSGEModel.k>0;
                        %
                        % setup X
                        %
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                        end;
                     end;
                     %
                     % determine how many prediction paths to use
                     %
                     NumPredPathsStr = get(controls.forecast.predpaths,'String');
                     NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                     %
                     % we can now compute the predictive distribution for the endogenous variables
                     % up to period MaxPh
                     %
                     if DSGEModel.RandomNumberValue==1;
                        randn('state',0);
                     else;
                        randn('state',sum(100*clock));
                     end;
                     %
                     % prediction event calculations
                     %
                     if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                        DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                     end;
                     %
                     % make sure that the prediction event length does not exceed the maximum prediction horizon
                     %
                     for i=1:DSGEModel.n;
                        if DSGEModel.PredictionEvent(i,3)>MaxPH;
                           DSGEModel.PredictionEvent(i,3) = MaxPH;
                        end;
                     end;
                     %
                     % display a dialog for setting up an prediction event
                     %
                     [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                     %
                     % check if user cancelled or not
                     %
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PredictionEvent = PredictionEvent;
                        DSGEModel.RunPredictionEvent = 1;
                        %
                        % check if we have already stored results to disk
                        %
                        if DSGEVARPredictionsExist(UsedDraws,MaxPH,0,'',DataType,lambda,PME,CurrChain,1,NumPredPaths,DSGEModel,controls)==1;
                           txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,[DataType ' Predictive Distribution - Posterior'],500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % compute the prediction event results
                           %
                           PredEventData = LoadDSGEVARPredictionPaths(PredictionEvent,UsedDraws,MaxPH,0,'',DataType,Weights,lambda,PME,CurrChain,NumPredPaths,DSGEModel,CurrINI,controls);
                           DoneCalc = 1;
                        else;
                           [DoneCalc,PredEventData,ShockCov,ParameterCov] = DSGEVARPredictionPaths(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,X,HSample(2),MaxPH,ForecastType,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                        end;
                        %
                        % clear everything
                        %
                        clear('PhiDraws','SigmaDraws');
                        %
                        % check if we should show any results
                        %
                        if DoneCalc==1;
                           %
                           % check if summary results from the predictive distribution have been saved to mat-file
                           %
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           DrawsStr = get(controls.posterior.draws,'String');
                           NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(PME) '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                           if FileExist(file)==1;
                              save(file,'PredictionEvent','PredEventData','-APPEND');
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARPD' DataType(1,1:4) strrep(num2str(lambda,'%0.4f'),'.','_') int2str(PME) int2str(DSGEModel.BVARLags) int2str(UsedDraws) 'x' int2str(NumPredPaths) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(DSGEModel.Percentiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              if ForecastType==1;
                                 fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to residual/shock uncertainty\n%%\n');
                                 fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                                 fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                                 fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                              end;
                              fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                              fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                              fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                              fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                           end;
                           %
                           % write prediction event results to file
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-PredictiveDist-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(PME) '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                           PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,UsedDraws,CurrINI);
                           %
                           % displays the file if it exists
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['DSGE-VAR(' num2str(lambda) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                           end;
                        else;
                           clear('PhiDraws','SigmaDraws');
                        end;
                     else;
                        %
                        % make sure that the draws are cleared from memory
                        %
                        clear('PhiDraws','SigmaDraws');
                     end;
                  else;
                     clear('PhiDraws','SigmaDraws','A0Draws');
                     txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute prediction event probabilities using the predictive distributions for the ' lower(DataType) ' data.'];
                     About(txt,'information',[DataType ' Predictive Distribution - Posterior'],300,500,CurrINI);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws');
                  txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute prediction event probabilities using the predictive distribution of the ' lower(DataType) ' observed variables. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
                  About(txt,'information',[DataType ' Predictive Distribution - Posterior'],200,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'CondPredictionEvent'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         ErrorStr = '';
         if gcbo==controls.dsgevartools.forecast.conditional.original.predevent;
            DataType = 'Original';
            ForecastType = 1;
         elseif gcbo==controls.dsgevartools.forecast.conditional.annual.predevent;
            DataType = 'Annualized';
            ForecastType = 2;
         elseif gcbo==controls.dsgevartools.forecast.conditional.transformed.predevent;
            DataType = 'Transformed';
            ForecastType = 3;
            %
            % check if the transformation functions work
            %
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            [value,CurrChain,PME,Lambda] = GetVARDSGEPostDrawsSelection('Conditional Predictive Distribution - Posterior',DSGEModel,controls,CurrINI);
            if value>0;
               lambda = Lambda(value);
               %
               % load the posterior draws
               %
               [PhiDraws,SigmaDraws,A0Draws,thetaPostSample,PostSample,Weights,PhiMode,SigmaModeEpsilon,A0Mode] = LoadVARDSGEPosteriorDraws(DSGEModel,CurrINI,lambda,PME,CurrChain,1);
               clear('thetaPostSample','PostSample');
               UsedDraws = size(PhiDraws,1);
               if UsedDraws>=100;
                  if IsPositiveSemiDefinite(SigmaModeEpsilon)==1;
                     %
                     % check which conditioning method to use
                     %
                     action = 'OK';
                     if isempty(A0Mode)==0;
                        if strcmp(get(controls.forecast.shockcontrol,'Enable'),'off')==1;
                           %
                           % let the user choose
                           %
                           [action,ShockControlMethod] = SelectionDlg(strvcat('Values for shocks','Distribution of shocks (Waggoner-Zha)','Distribution of subset of shocks'),1,'Select method for controlling conditional forecasts:','Conditional Forecast Method Selection','Conditional Predictive Distribution - Posterior','','',CurrINI);
                           if (strcmp(lower(action),'ok')==1)&(ShockControlMethod==3);
                              %
                              SetShocks = 0;
                              MixedConditionalShocks = DSGEModel.MixedConditionalShocks;
                              while SetShocks==0;
                                 [action,positions] = SelectCondVarShockDLG('init','Shocks','Subset Method Conditioning',DSGEModel.StateShockNames,MixedConditionalShocks,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    if isempty(positions)==0;
                                       if length(positions)>=length(DSGEModel.ConditionalShocks);
                                          %
                                          % we're duin OK
                                          %
                                          DSGEModel.MixedConditionalShocks = positions;
                                          SetShocks = 1;
                                       else;
                                          txt = ['The number of shocks for conditional forecasting with the distribution for a subset of the shocks method (' int2str(length(positions)) ') that you selected is less than the number of conditioning assumptions (' int2str(length(DSGEModel.ConditionalShocks)) '). This is not a valid choice in YADA. You must select at least as many shocks in the subset as there are conditioning assumptions.'];
                                          About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                       end;
                                    else;
                                       txt = 'You didn''t select any shocks for conditional forecasting with the distribution for a subset of the shocks method. This is not a valid choice in YADA. You must select at least one more shock in the subset than there are conditioning assumptions and fewer than the total number of shocks.';
                                       About(txt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                    end;
                                 else;
                                    SetShocks = 1;
                                 end;
                              end;
                           end;
                        else;
                           ShockControlMethod = DSGEModel.ShockControlMethod;
                        end;
                     else;
                        ShockControlMethod = 2;
                     end;
                     %
                     % check if we should continue
                     %
                     if strcmp(lower(action),'ok')==1;
                        %
                        % check which draws we can clear from memory
                        %
                        if ShockControlMethod==1;
                           clear('SigmaDraws');
                           ShockMethod = 'Direct-';
                        elseif ShockControlMethod==2;
                           clear('A0Draws');
                           ShockMethod = 'WaggonerZha-';
                        else;
                           clear('SigmaDraws');
                           ShockMethod = 'Subset-';
                        end;
                        %
                        % check how many time periods to use for predictive distributions
                        %
                        [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                        %
                        % setup the necessary data
                        %
                        [y,Y,HSample] = GetDSGEVARData(DSGEModel);
                        DateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                        DateVector = DateVector(HSample(1):min(HSample(2)+MaxPH,size(DSGEModel.Y,2)));
                        SampleStrMat = CreateFullSample(DSGEModel);
                        SampleStrMat = SampleStrMat(HSample(1):HSample(2),:);
                        %
                        X = [];
                        if DSGEModel.k>0;
                           %
                           % setup X
                           %
                           if NumPredX<MaxPH;
                              X = DSGEModel.X(:,HSample(2)+1:HSample(2)+(MaxPH-NumPredX));
                           end;
                           if NumPredX>0;
                              X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                           end;
                        end;
                        %
                        % determine how many prediction paths to use
                        %
                        NumPredPathsStr = get(controls.forecast.predpaths,'String');
                        NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                        %
                        % determine the total number of available draws
                        %
                        DrawsStr = get(controls.posterior.draws,'String');
                        NumDraws = str2num(StringTrim(strrep(DrawsStr(DSGEModel.PosteriorDrawsValue,:),',','')));
                        %
                        % check the length of the conditioning sample
                        %
                        [status,NumPeriods] = CheckConditioningData(DSGEModel);
                        %
                        % check if we should adjust the Z matrix
                        %
                        if MaxPH<NumPeriods;
                           NumPeriods = MaxPH;
                        end;
                        %
                        % collect the conditioning information
                        %
                        Z = DSGEModel.Z(DSGEModel.ConditionalVariables,HSample(2)+1:HSample(2)+NumPeriods);
                        U = DSGEModel.U(DSGEModel.ConditionalVariables,HSample(2));
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPh
                        %
                        if DSGEModel.RandomNumberValue==1;
                           randn('state',0);
                        else;
                           randn('state',sum(100*clock));
                        end;
                        %
                        % prediction event calculations
                        %
                        if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                           DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                        end;
                        %
                        % make sure that the prediction event length does not exceed the maximum prediction horizon
                        %
                        for i=1:DSGEModel.n;
                           if DSGEModel.PredictionEvent(i,3)>MaxPH;
                              DSGEModel.PredictionEvent(i,3) = MaxPH;
                           end;
                        end;
                        %
                        % display a dialog for setting up an prediction event
                        %
                        [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                        %
                        % check if user cancelled or not
                        %
                        if strcmp(lower(action),'ok')==1;
                           DSGEModel.PredictionEvent = PredictionEvent;
                           DSGEModel.RunPredictionEvent = 1;
                           %
                           % check if we have already stored results to disk
                           %
                           if DSGEVARPredictionsExist(UsedDraws,MaxPH,1,ShockMethod,DataType,lambda,PME,CurrChain,1,NumPredPaths,DSGEModel,controls)==1;
                              txt = 'YADA has located results for the predictive distribution on disk. Would you like to use these results?';
                              answer = Query(txt,'question',150,'Conditional Predictive Distribution - Posterior',500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              %
                              % compute the prediction event results
                              %
                              PredEventData = LoadDSGEVARPredictionPaths(PredictionEvent,UsedDraws,MaxPH,1,ShockMethod,DataType,Weights,lambda,PME,CurrChain,NumPredPaths,DSGEModel,CurrINI,controls);
                              DoneCalc = 1;
                           else;
                              %
                              % compute the predictive distribution
                              %
                              if ShockControlMethod==1;
                                 [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPaths(PhiDraws,A0Draws,PhiMode,A0Mode,X,Z,U,HSample(2),MaxPH,ForecastType,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                              elseif ShockControlMethod==2;
                                 [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPathsWZ(PhiDraws,SigmaDraws,PhiMode,SigmaModeEpsilon,X,Z,U,HSample(2),MaxPH,ForecastType,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                              else;
                                 [DoneCalc,PredEventData,ShockMean,ShockNames,ShockCov,ParameterCov] = DSGEVARCondPredictionPathsMixed(PhiDraws,A0Draws,PhiMode,A0Mode,X,Z,U,HSample(2),MaxPH,ForecastType,CurrChain,1,Weights,PME,lambda,NumPredPaths,DSGEModel,CurrINI,controls);
                              end;
                           end;
                           %
                           % clear everything
                           %
                           if ShockControlMethod==1;
                              clear('PhiDraws','A0Draws');
                           elseif ShockControlMethod==2;
                              clear('PhiDraws','SigmaDraws');
                           else;
                              clear('PhiDraws','A0Draws');
                           end;
                           if DoneCalc==1;
                              %
                              % check if summary results from the predictive distribution have been saved to mat-file
                              %
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-' int2str(ShockControlMethod) '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(PME) '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                              if FileExist(file)==1;
                                 save(file,'PredictionEvent','PredEventData','-APPEND');
                                 %
                                 % create a matlab m-file to read the output.
                                 %
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\DSGEVARCPD' int2str(ShockControlMethod) DataType(1,1:4) strrep(num2str(lambda,'%0.4f'),'.','_') int2str(PME) int2str(DSGEModel.BVARLags) int2str(UsedDraws) 'x' int2str(NumPredPaths) int2str(MaxPH) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                                 fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                                 fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                                 fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                                 fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                                 fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                                 fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(DSGEModel.Percentiles));
                                 fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                                 fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                 fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                                 fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                                 fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                 fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                 fprintf(fid,'%%\n%% observed variables\n%%\n');
                                 fprintf(fid,'Y = DataStructure.Y;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                                 fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                                 if ForecastType==1;
                                    fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to residual/shock uncertainty\n%%\n');
                                    fprintf(fid,'ShockCov = DataStructure.ShockCov;\n');
                                    fprintf(fid,'%%\n%% 3D Error covariance matrix (nxnxh) part due to parameter uncertainty\n%%\n');
                                    fprintf(fid,'ParameterCov = DataStructure.ParameterCov;\n');
                                    fprintf(fid,'%%\n%% vector with the mean of the multivariate modesty statistic and its reference statistic\n%%\n');
                                    fprintf(fid,'MeanMultiModesty = DataStructure.MeanMultiModesty;\n');
                                    fprintf(fid,'%%\n%% tail probability based on the reference statistic\n%%\n');
                                    fprintf(fid,'MultiPValue = DataStructure.MultiPValue;\n');
                                    fprintf(fid,'%%\n%% tail probability based on the chi-square distribution\n%%\n');
                                    fprintf(fid,'ChiSqPValue = DataStructure.ChiSqPValue;\n');
                                    fprintf(fid,'%%\n%% Adolfson, Lasen, Lind and Villani univariate modesty statistic\n%%\n');
                                    fprintf(fid,'MeanUniModesty = DataStructure.MeanUniModesty;\n');
                                    fprintf(fid,'%%\n%% Leeper and Zha univariate modesty statistic\n%%\n');
                                    fprintf(fid,'MeanUniModestyLZ = DataStructure.MeanUniModestyLZ;\n');
                                 end;
                                 if isempty(PredEventData)==0;
                                    fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                    fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                    fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                    fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 end;
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                              end;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\DSGEVAR-CondPredictiveDist-' int2str(ShockControlMethod) '-' DataType '-' num2str(lambda,'%0.4f') '-' int2str(PME) '-' int2str(DSGEModel.BVARLags) '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' int2str(MaxPH) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,UsedDraws,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['DSGE-VAR(' num2str(lambda) ') Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Conditional Predictions',50,CurrINI,1,0);
                                 drawnow;
                                 pause(0.02);
                              end;
                           end;
                        else;
                           if ShockControlMethod==1;
                              clear('PhiDraws','A0Draws');
                           else;
                              clear('PhiDraws','SigmaDraws');
                           end;
                        end;
                     else;
                        clear('PhiDraws','A0Draws','SigmaDraws');
                     end;
                  else;
                     clear('PhiDraws','SigmaDraws','A0Draws');
                     txt = ['The posterior mode estimate of the residual covariance matrix of the DSGE-VAR model with lambda=' num2str(lambda) ' is not positive semidefinite with at least rank 1. YADA is therefore unable to compute prediction event probabilities using the conditional predictive distributions for the ' lower(DataType) ' data.'];
                     About(txt,'information','Conditional Predictive Distribution - Posterior',300,500,CurrINI);
                  end;
               else;
                  clear('PhiDraws','SigmaDraws','A0Draws');
                  txt = ['YADA requires at least 100 posterior draws of the VAR model parameters before it will compute prediction event probabilities using the conditional predictive distribution of the ' lower(DataType) ' observed variables. You currently only have ' int2str(UsedDraws) ' posterior draws of the DSGE-VAR parameters with lambda=' num2str(lambda) '.'];
                  About(txt,'information','Conditional Predictive Distribution - Posterior',200,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of DSGEVARForecastMenuFunctions.m
%
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                  