function ToolsMenuFunctions(selector,maingui)
% ToolsMenuFunctions: switch function for running some of the function on the tools menu.
%
% USAGE:
%
%       ToolsMenuFunctions(selector,maingui)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
% OPTIONAL INPUT: maingui (handle) to the YADA GUI.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: February 15, 2010.
%                        This version: March 15, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 16-02-2010: Changed the order of the input variables for "YADAEnableControls".
%
% * 18-02-2010: Took into account that the prior and posterior distributions of the state shock
%               correlations can now handle the sample as well as the average population moments.
%
% * 22-03-2010: The parameter scenario can now be run with a combination of state and observed
%               variable scenarios provided that the needed functions exist.
%
% * 15-04-2010: Fixed a display text issue for the impulse responses when having selected shock
%               groups instead of the individual shocks.
%
% * 26-04-2010: The "ShockNames" variable is now always stored to the mat-file when saving conditional
%               prediction results to disk for the initial values.
%
% * 05-05-2010: Changed names for the information matrix cases when estimating it in the time domain.
%               Began work on the information matrix cases for the frequency domain.
%
% * 06-05-2010: Finished the information matrix functions for the frequency domain cases.
%
% * 09-05-2010: Changed names for the observed variable correlations when estimating them in the
%               time domain (compared to the coherence statistics in the frequency domain).
%
% * 10-05-2010: The function has been split into several parts to allow faster loading of the individual
%               cases and thereby speed up YADA.
%
% * 12-05-2010: The function "DSGEIdentificationPatterns" now always receives the information matrix
%               rather than a correlation version based on its inverse.
%
% * 02-06-2010: Took into account that OVSTheta accepts some additional input variables.
%
% * 08-07-2010: Changed headers on About dialogs when "mcode" or "kalmanstatus" reports problems.
%
% * 03-08-2010: Took into account that "StateVarDistributionDLG" now accepts the "IsConstrained"
%               and "StartPeriod" input variables.
%
% * 18-08-2010: Made sure that parameter functions are executed before the model is solved when computing
%               the eigenvalues.
%
% * 30-08-2010: Checks for infinite values for the log-likelihood function are now in place when preparing
%               displaying the function values.
%
% * 07-10-2010: Began work on using simulation smoothing for the estimated structural shocks under the
%               posterior mode estimator.
%
% * 08-10-2010: Updated the input variables for the "DSGESimulationSmootherTheta" function.
%
% * 14-10-2010: Finished the simulation smoothing code for the state shocks at posterior mode. Added code
%               for simulation smoothing of the state shocks at initial values, simulation smoothing of the
%               state variables at posterior mode and at initial values, and finally simulation smoothing for
%               the measurement errors at posterior mode and at the initial values.
%
% * 15-10-2010: Began work on simulation smoothing for the state variables based on the posterior draws of
%               the parameters.
%
% * 26-10-2010: It is now optional to save the covariance matrices of the state variables when saving the
%               point estimates.
%
% * 28-10-2010: Finished the code for running the simulation smoother for state variables using the posterior
%               draws. Added the code for the state variables with prior draws.
%
% * 29-10-2010: Added code for using the simulation smoother when estimating the economic shocks and measurement
%               errors using prior and posterior draws.
%
% * 03-11-2010: Took into account that "GraphSimulatedData" now takes the "lambda" input variable.
%
% * 15-12-2010: Started work on integrating the Klein and Sims QZ decomposition based model solvers.
%
% * 17-12-2010: Took into account that "SolveDSGEModel" takes the input variable "OrderQZ" from the CurrINI
%               structure.
%
% * 22-12-2010: Added code for computing the log-likelihood with the square root filter.
%
% * 05-01-2011: Updated the documentation.
%
% * 10-01-2011: Changed all dos copy calls to "CopyFile" calls.
%
% * 11-01-2011: Made sure that "ModeFile" supports unix and mac paths.
%
% * 12-01-2011: Added support for unix and mac paths.
%
% * 17-06-2011: Removed unused output variables from calls to "InitializeDSGEModelSimulation".
%
% * 20-06-2011: Made sure that the "DSGEEigenvaluesInitialValues" case collects the "ParameterNames"
%               structure from "InitializeDSGEModelSimulation".
%
% * 21-06-2011: The DSGE model eigenvalues can now be displayed also when the model doesn't have a
%               unique convergent solution.
%
% * 22-06-2011: Fixed the "="-character line when writing eigenvalue results to file.
%
% * 04-10-2011: Fixed an issue with calling "CalculateDSGESquareRootFilter".
%
% * 06-12-2011: Made sure that "ExcludedParameterNames" is written correctly to file.
%
% * 14-12-2011: Began work on the Monte Carlo Filtering function.
%
% * 15-12-2011: Continued writing Monte Carlo filtering code.
%
% * 16-12-2011: Continued working on the Monte Carlo filtering code.
%
% * 19-12-2011: Finished the Monte Carlo filtering code.
%
% * 22-12-2011: Took into account that graphing of the log-likelihood is not undertaken when
%               the output vector is empty (new feature).
%
% * 02-01-2012: Updated the documentation.
%
% * 29-02-2012: Took into account that the number of simulated paths per parameter value can be unity.
%
% * 07-03-2012: Removed an unnecessary call to "SetLogText" when computing the log-likelihood for the
%               initial values.
%
% * 08-03-2012: The 1-step ahead forecasts of the observed variables with fixed parameter values can now
%               store smoothly projected values of the observed variables when the data contains missing
%               observations. The projected values only cover the estimation sample.
%
% * 20-03-2012: Fixed a bug where fields can be missing from "StateVarStructure" when running the
%               "CalculateDSGEStateVariables" function under diffuse initialization of the Kalman filter.
%
% * 07-05-2012: The simulation smoother for prior and posterior draws can now support 1 path per parameter
%               draw by converting it to 4.
%
% * 05-06-2012: Took into account that the model solvers now take the "StateEquationPositions" input vector.
%
% * 16-11-2012: The log-likelihood function calculations now take the computational time "KalmanTime" into account.
%
% * 02-01-2013: Took into account that "SolveDSGEModel" now supports "ErrMsg".
%
% * 05-02-2013: Took into account that "DSGEtoVARModel" now also checks if the poor man's invertibility condition
%               may be necessary as well as sufficient.
%
% * 28-01-2014: Updated the documentation.
%
% * 13-02-2014: Made sure that the model eigenvalues functions properly checks that the eigenvalues have actually
%               been computed.
%
% * 13-03-2014: Took into account that "GraphLogLikelihood" now takes the input variable "ModelStr".
%
% * 17-12-2014: Added cases for computing the state variables impulse responses using prior and posterior
%               parameter draws. Added the posterior draws case.
%
% * 18-12-2014: Added state variable impulse responses for the prior draws.
%
% * 14-01-2015: Updated the documentation.
%
% * 02-09-2015: Fixed the information displayed when the kalmanstatus is -1.
%
% * 27-01-2016: Remember Shoah.
%
% * 06-12-2016: Took into acccount that the state space matrices are now provided as output from the
%               parameter scenarios function.
%
% * 21-03-2017: Updated the documentation.
%
% * 09-06-2017: The Monte Carlo filtering function now deals with a system prior, i.e. this prior implies
%               that Monte Carlo filtering is pointless since all draws will be connected with a unique
%               and convergent solution to the DSGE model.
%
% * 23-08-2017: Took into account that "LoadDSGEPosteriorDraws" now provides the output variable "Weights".
%
% * 24-08-2017: Took into account that "DSGESVEDistribution" and "DSGESimSmoothDistribution" now require
%               the input variable "Weights".
%
% * 15-09-2017: Updated the documentation.
%
% * 13-12-2017: Made sure that a subset of prior draws can be used when the model includes a system prior.
%
% * 08-01-2018: Updated the documentation.
%
% * 30-07-2018: Began work on supporting user written model solver function.
%
% * 01-08-2018: The function supports user written model solver function.
%
% * 03-08-2018: Added copy call for system prior file.
%
% * 16-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 25-10-2018: Made sure that the name of the prior draws file reflects using a system prior.
%
% * 23-01-2019: Updated the documentation.
%
% * 26-03-2019: Adapted the parameter scenarios to allow for the calibrated parameters to be changed
%               as well as the estimated.
%
% * 10-12-2019: Took into account that the user defined DSGEModel.ModelSolverFile now accepts the
%               additional three input variables "StateVariableNames", "StateEquationNames"
%               and "StateShockNames", which are all covered by fields of the structure DSGEModel.
%
% * 10-01-2020: Updated the documentation.
%
% * 25-05-2020: Took into account that "DSGEPriorSampling" and "DSGESystemPriorSampling" take the input variable
%               "IsAdaptiveLearning".
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 26-01-2021: Updated the documentation.
%
% * 22-02-2021: Began work on the IRFs for the permanent shock at initial values and at posterior mode values.
%
% * 23-02-2021: Continued with the IRFs at posterior mode.
%
% * 11-03-2021: Added support for "ResponseDenominatorPosition" when computing the permanent shock responses.
%               This makes it possible to measure the responses relative to (divided by) the responses of one
%               of the observed variables.
%
% * 12-03-2021: Finished the permanent shock function.
%
% * 28-04-2021: Updated the documentation.
%
% * 06-01-2022: Updated the documentation.
%
% * 18-02-2022: Changed eval statements for feval to speed up the function.
%
% * 21-02-2022: Changed eval calls to feval.
%
% * 05-01-2023: Updated the documentation.
%
% * 24-01-2023: Added driver support to "StateVariablesPostMode" and "ShocksPostMode".
%
% * 06-02-2023: The impulse responses for the original data, annualized data and levels
%               data now support the driver functionality.
%
% * 21-02-2023: Added the "maingui" input variable.
%
% * 13-06-2023: Updated the documentation.
%
% * 05-10-2023: Updated the documentation.
%
% * 06-10-2023: Made sure that the output directory exists for storing prior draws.
%
% * 02-11-2023: Added the call to "GraphMonteCarloFilteringDensity" when running the Monte Carlo
%               filtering function.
%
% * 18-01-2024: Updated the documentation.
%
% * 23-02-2024: Made sure that StateVarStructure has a field "etatT" when estimating the shocks.
%
% * 26-02-2024: Updated the documentation.
%
% * 27-02-2024: Made sure that etatT and wtT exist after running "CalculateStateVariables".
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
if nargin==1;
   maingui = findobj('Type','figure','Tag','YADA');
end;
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
if MatlabNumber<7;
   lasterr = '';
end;
%
switch selector
   case 'DSGEEigenvaluesInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check the status of the DSGE model files
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames] = InitializeDSGEModelSimulation(DSGEModel,CurrINI,0);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  ParameterNames = ModeData.ParameterNames;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         if InitStatus==1;
            %
            % run AiMSolver to check the eigenvalues of the DSGE model.
            %
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     if DSGEModel.ModelSolver==1;
                        [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
                     elseif DSGEModel.ModelSolver==2;
                        [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
                     elseif DSGEModel.ModelSolver==3;
                        [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
                     elseif DSGEModel.ModelSolver==4;
                        [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                     end;
                  catch ME;
                     ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     if DSGEModel.ModelSolver==1;
                        [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
                     elseif DSGEModel.ModelSolver==2;
                        [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
                     elseif DSGEModel.ModelSolver==3;
                        [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
                     elseif DSGEModel.ModelSolver==4;
                        [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
                     end;
                  catch;
                     ErrorStr = ['There was an error running AiM. YADA received the following message: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  %
                  % Some generic model information
                  %
                  txt = strvcat(' ',['AiM Model File:                            ' GetFilename(DSGEModel.AIMFile)],' ');
                  if DSGEModel.ModelSolver==1;
                     txt = strvcat(txt,['Number of equations:                       ' int2str(AIMData.neq)]);
                  else;
                     txt = strvcat(txt,['Number of equations:                       ' int2str(length(DSGEModel.StateVariablePositions))]);
                  end;
                  txt = strvcat(txt,['Number of lags:                            ' int2str(AIMData.nlag)]);
                  txt = strvcat(txt,['Number of leads:                           ' int2str(AIMData.nlead)]);
                  if isempty(ModelEigenvalues)==0;
                     if DSGEModel.ModelSolver==1;
                        txt = strvcat(txt,['Number of exact shiftrights:               ' int2str(ModelEigenvalues.NumExact)]);
                        txt = strvcat(txt,['Number of numeric shiftrights:             ' int2str(ModelEigenvalues.NumNumeric)]);
                     end;
                     txt = strvcat(txt,['Number of large roots:                     ' int2str(ModelEigenvalues.LargeRoots)]);
                     if DSGEModel.ModelSolver==1;
                        txt = strvcat(txt,['Number of stability conditions:            ' int2str(ModelEigenvalues.NumExact+ModelEigenvalues.NumNumeric+ModelEigenvalues.LargeRoots)]);
                        txt = strvcat(txt,['Number of required stability conditions:   ' int2str(AIMData.neq*AIMData.nlead)]);
                        txt = strvcat(txt,['Difference:                                ' int2str(ModelEigenvalues.DimStability)]);
                     else;
                        txt = strvcat(txt,['Number of required large roots:            ' int2str(length(ModelEigenvalues.Roots)-length(DSGEModel.StateVariablePositions))]);
                        txt = strvcat(txt,['Number of stability conditions:            ' int2str(length(ModelEigenvalues.Roots)-ModelEigenvalues.LargeRoots)]);
                        txt = strvcat(txt,['Number of required stability conditions:   ' int2str(length(DSGEModel.StateVariablePositions))]);
                     end;
                     txt = strvcat(txt,['Dimension of DSGE state transition matrix: ' int2str(ModelEigenvalues.DimCompanion)]);
                     txt = strvcat(txt,['Number of parameters to estimate:          ' int2str(length(theta))]);
                     txt = strvcat(txt,['Total number of parameters:                ' int2str(size(ParameterNames.all,1))]);
                  end;
                  %
                  if mcode==1;
                     mcodeStr = 'Unique convergent solution.';
                  elseif mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'The companion matrix a has Inf or NaN entries.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     txt = strvcat(txt,['AiM return code:                           ' mcodeStr]);
                  elseif DSGEModel.ModelSolver==2;
                     txt = strvcat(txt,['Klein''s solver return code:                ' mcodeStr]);
                  elseif DSGEModel.ModelSolver==3;
                     txt = strvcat(txt,['Gensys return code:                        ' mcodeStr]);
                  elseif DSGEModel.ModelSolver==4;
                     txt = strvcat(txt,['Own algorithm return code:                 ' mcodeStr]);
                  end;
                  if mcode==1;
                     txt = strvcat(txt,['Maximum absolute error:                    ' num2str(MaxAbsError)]);
                  end;
                  if (mcode~=7)&(mcode~=8)&(isempty(ModelEigenvalues)==0);
                     txt = strvcat(txt,' ','                           Eigenvalues','Number       Real          Imaginary      Amplitude         Period');
                     %
                     prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
                     prt_str = [' %3.0f     ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val];
                     for i=1:length(ModelEigenvalues.Roots);
                        if i<=length(ModelEigenvalues.Amplitude);
                           txti = sprintf(prt_str,i,real(ModelEigenvalues.Roots(i)),imag(ModelEigenvalues.Roots(i)),ModelEigenvalues.Amplitude(i),ModelEigenvalues.Period(i));
                        else;
                           txti = sprintf(prt_str,i,real(ModelEigenvalues.Roots(i)),imag(ModelEigenvalues.Roots(i)),0,NaN);
                        end;
                        txt = strvcat(txt,txti);
                     end;
                  end;
                  %
                  txt = strvcat(txt,' ');
                  %
                  % write to file
                  %
                  EigenValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Eigenvalues-' DSGEModel.NameOfModel '-InitialValues.txt']);
                  fid = fopen(EigenValueFile,'wt');
                  for i=1:size(txt,1);
                     fprintf(fid,'%s\n',txt(i,:));
                  end;
                  fprintf(fid,'================================================================================\n\n');
                  %
                  % bottom file data
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(EigenValueFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(EigenValueFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  % Display information
                  %
                  if FileExist(EigenValueFile)==1;
                     TextGUI(EigenValueFile,['DSGE Model Eigenvalues - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                  end;
               else;
                  About(ErrorStr,'error','AiM Error',CurrINI.scrsz(4)-50,600,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'DSGEEigenvaluesPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % load data from file
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            %
            % run AiMSolver to check the eigenvalues of the DSGE model.
            %
            AIMData = load(DSGEModel.AIMDataFile);
            %
            % Make sure that compute_aim_matrices is copied to the tmp directory
            %
            [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
            %
            % check if the update parameter function exists
            %
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
            end;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
            end;
            %
            % check if the model solver file should be copied
            %
            if DSGEModel.ModelSolver==4;
               if FileExist(DSGEModel.ModelSolverFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
               end;
            end;
            %
            % make sure the ModelParameters structure gets the correct values
            %
            ModelParameters = ThetaToModelParameters(ModeData.ModelParameters,ModeData.thetaMode,ModeData.thetaPositions);
            %
            % check if we update parameter should be executed
            %
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               ModelParameters = feval(GetName(DSGEModel.UpdateParameterFile),ModelParameters);
            end;
            %
            if DSGEModel.ModelSolver==1;
               [mcode,MaxAbsError,BMatrix,SMatrix,ModelEigenvalues] = AiMSolver(ModelParameters,AIMData.neq,AIMData.nlag,AIMData.nlead,DSGEModel.AIMTolerance);
            elseif DSGEModel.ModelSolver==2;
               [F,B0,mcode,MaxAbsError,ModelEigenvalues] = KleinSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
            elseif DSGEModel.ModelSolver==3;
               [F,B0,mcode,MaxAbsError,ModelEigenvalues] = SimsSolver(ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ);
            elseif DSGEModel.ModelSolver==4;
               [F,B0,mcode,MaxAbsError,ModelEigenvalues] = feval(GetName(DSGEModel.ModelSolverFile),ModelParameters,AIMData.nlead,DSGEModel.StateVariablePositions,DSGEModel.StateEquationPositions,DSGEModel.StateShockPositions,DSGEModel.AIMTolerance,CurrINI.OrderQZ,DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.StateShockNames);
            end;
            %
            % Some generic model information
            %
            txt = strvcat(' ',['AiM Model File:                            ' GetFilename(DSGEModel.AIMFile)],' ');
            if DSGEModel.ModelSolver==1;
               txt = strvcat(txt,['Number of equations:                       ' int2str(AIMData.neq)]);
            else;
               txt = strvcat(txt,['Number of equations:                       ' int2str(length(DSGEModel.StateVariablePositions))]);
            end;
            txt = strvcat(txt,['Number of lags:                            ' int2str(AIMData.nlag)]);
            txt = strvcat(txt,['Number of leads:                           ' int2str(AIMData.nlead)]);
            if isempty(ModelEigenvalues)==0;
               if DSGEModel.ModelSolver==1;
                  txt = strvcat(txt,['Number of exact shiftrights:               ' int2str(ModelEigenvalues.NumExact)]);
                  txt = strvcat(txt,['Number of numeric shiftrights:             ' int2str(ModelEigenvalues.NumNumeric)]);
               end;
               txt = strvcat(txt,['Number of large roots:                     ' int2str(ModelEigenvalues.LargeRoots)]);
               if DSGEModel.ModelSolver==1;
                  txt = strvcat(txt,['Number of stability conditions:            ' int2str(ModelEigenvalues.NumExact+ModelEigenvalues.NumNumeric+ModelEigenvalues.LargeRoots)]);
                  txt = strvcat(txt,['Number of required stability conditions:   ' int2str(AIMData.neq*AIMData.nlead)]);
                  txt = strvcat(txt,['Difference:                                ' int2str(ModelEigenvalues.DimStability)]);
               else;
                  txt = strvcat(txt,['Number of required large roots:            ' int2str(length(ModelEigenvalues.Roots)-length(DSGEModel.StateVariablePositions))]);
                  txt = strvcat(txt,['Number of stability conditions:            ' int2str(length(ModelEigenvalues.Roots)-ModelEigenvalues.LargeRoots)]);
                  txt = strvcat(txt,['Number of required stability conditions:   ' int2str(length(DSGEModel.StateVariablePositions))]);
               end;
               txt = strvcat(txt,['Dimension of DSGE state transition matrix: ' int2str(ModelEigenvalues.DimCompanion)]);
               txt = strvcat(txt,['Number of parameters to estimate:          ' int2str(length(ModeData.theta))]);
               txt = strvcat(txt,['Total number of parameters:                ' int2str(size(ModeData.ParameterNames.estimated,1)+size(ModeData.ParameterNames.calibrated,1)+size(ModeData.ParameterNames.additional,1))]);
            end;
            if mcode==1;
               mcodeStr = 'Unique convergent solution.';
            elseif mcode==2;
               mcodeStr = 'Roots are not correctly computed by real_schur.'
            elseif mcode==3;
               mcodeStr = 'Too many big roots.';
            elseif mcode==35;
               mcodeStr = 'Too many big roots, and q(:,right) is singular.';
            elseif mcode==4;
               mcodeStr = 'Too few big roots.';
            elseif mcode==45;
               mcodeStr = 'Too few big roots, and q(:,right) is singular.';
            elseif mcode==5;
               mcodeStr = 'q(:,right) is singular.';
            elseif mcode==61;
               mcodeStr = 'Too many exact shiftrights.';
            elseif mcode==62;
               mcodeStr = 'Too many numeric shiftrights.';
            elseif mcode==7;
               mcodeStr = 'The companion matrix a has Inf or NaN entries.';
            elseif mcode==8;
               mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
            elseif mcode==-1;
               mcodeStr = 'No stable solution.';
            elseif mcode==-2;
               mcodeStr = 'Too many large generalized eigenvalues.';
            elseif mcode==-3;
               mcodeStr = 'Too few large generalized eigenvalues.';
            else;
               mcodeStr = 'Return code not properly specified.';
            end;
            if DSGEModel.ModelSolver==1;
               txt = strvcat(txt,['AiM return code:                           ' mcodeStr]);
            elseif DSGEModel.ModelSolver==2;
               txt = strvcat(txt,['Klein''s solver return code:                ' mcodeStr]);
            elseif DSGEModel.ModelSolver==3;
               txt = strvcat(txt,['Gensys return code:                        ' mcodeStr]);
            elseif DSGEModel.ModelSolver==4;
               txt = strvcat(txt,['Own algorithm return code:                 ' mcodeStr]);
            end;
            if mcode==1;
               txt = strvcat(txt,['Maximum absolute error:                    ' num2str(MaxAbsError)]);
            end;
            if (mcode~=7)&(mcode~=8)&(isempty(ModelEigenvalues)==0);
               txt = strvcat(txt,' ','                           Eigenvalues','Number       Real          Imaginary      Amplitude         Period');
               %
               prt_val = ['%' num2str(6+CurrINI.decimals,'%0.0f') '.' num2str(CurrINI.decimals,'%0.0f') 'f'];
               prt_str = [' %3.0f     ' prt_val '   ' prt_val '   ' prt_val '   ' prt_val];
               for i=1:length(ModelEigenvalues.Roots);
                  if i<=length(ModelEigenvalues.Amplitude);
                     txti = sprintf(prt_str,i,real(ModelEigenvalues.Roots(i)),imag(ModelEigenvalues.Roots(i)),ModelEigenvalues.Amplitude(i),ModelEigenvalues.Period(i));
                  else;
                     txti = sprintf(prt_str,i,real(ModelEigenvalues.Roots(i)),imag(ModelEigenvalues.Roots(i)),0,NaN);
                  end;
                  txt = strvcat(txt,txti);
               end;
            end;
            %
            txt = strvcat(txt,' ');
            %
            % write to file
            %
            EigenValueFile = FixFilePath([DSGEModel.OutputDirectory '\DSGE-Eigenvalues-' DSGEModel.NameOfModel '-PostMode.txt']);
            fid = fopen(EigenValueFile,'wt');
            for i=1:size(txt,1);
               fprintf(fid,'%s\n',txt(i,:));
            end;
            fprintf(fid,'================================================================================\n\n');
            %
            % bottom file data
            %
            fprintf(fid,'Directory for file: %s\n',GetPath(EigenValueFile));
            fprintf(fid,'Name of file:       %s\n',GetFilename(EigenValueFile));
            fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
            fclose(fid);
            %
            % Display information
            %
            if FileExist(EigenValueFile)==1;
               TextGUI(EigenValueFile,['DSGE Model Eigenvalues - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MonteCarloFiltering'
       if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the user has a system prior
         %
         if strcmp(lower(DSGEModel.SystemPriorFile),lower(pwd))==1;
            IsSystemPrior = 0;
         else;
            IsSystemPrior = 1;
         end;
         %
         % there's no point in using Monte Carlo filtering if there's a system prior since
         % prior sampling will always give a unique convergent solution at each prior draw
         % due to the MCMC nature of the sampler (Metropolis-Hastings)
         %
         if IsSystemPrior==0;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if FileExist(DSGEModel.PriorFile)==1;
                     txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                     answer = Query(txt,'question',200,'Monte Carlo Filtering',500,'no',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     InitStatus = 1;
                     theta = ModeData.theta;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     LowerBound = ModeData.LowerBound;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                  else;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  end;
               else;
                  if FileExist(DSGEModel.PriorFile)==1;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  else;
                     InitStatus = 0;
                  end;
                  if InitStatus==0;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            else;
               [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            end;
            if (InitStatus==1)&(isempty(ParameterNames.estimated)==0);
               %
               % load the AIMData file
               %
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  set(controls.about,'UserData',0);
                  return;
               end;
               %
               % suggest how many prior draws to compute
               %
               PriorDrawsStr = Str2Matrix('1,000|1,500|2,000|2,500|3,000|3,500|4,000|4,500|5,000|6,000|7,000|8,000|9,000|10,000|11,000|12,000|13,000|14,000|15,000|16,000|17,000|18,000|19,000|20,000|25,000|30,000|35,000|40,000|45,000|50,000|55,000|60,000|65,000|70,000|75,000|80,000|85,000|90,000|95,000|100,000|110,000|120,000|130,000|140,000|150,000|160,000|170,000|180,000|190,000|200,000|250,000|300,000|350,000|400,000|450,000|500,000','|');
               %
               % open a selection dialog
               %
               [action,PriorDrawsValue] = SelectionDlg(PriorDrawsStr,DSGEModel.MonteCarloFilterDraws,'Select the number of draws from the prior distribution:','Sampling Selection','Monte Carlo Filtering','','',CurrINI);
               if strcmp(lower(action),'ok')==1;
                  NumPriorDraws = str2num(strrep(StringTrim(PriorDrawsStr(PriorDrawsValue,:)),',',''));
%                  if FileExist(DSGEModel.SystemPriorFile)==0;
                     PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' int2str(NumPriorDraws) '-' DSGEModel.NameOfModel '.mat']);
%                  else;
%                     PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' int2str(NumPriorDraws) '-' DSGEModel.NameOfModel '.mat']);
%                  end;
                  %
                  % create output folder if necessary
                  %
                  stat = MakeDir(DSGEModel.OutputDirectory,'priordraws');
                  %
                  % check if we should run the prior sampling function
                  %
                  if (FileExist(PriorDrawsFile)==1)&(DSGEModel.OverwriteDraws==0);
                     PD = load(PriorDrawsFile);
                     thetaDraws = PD.thetaDraws;
                     if isfield(PD,'SolutionCode')==1;
                        SolutionCode = PD.SolutionCode;
                     else;
                        SolutionCode = [];
                     end;
                     SaveDraws = 0;
                  else;
                     thetaDraws = DSGEPriorSampling(PriorDist,thetaDist,LowerBound,NumPriorDraws,0,'Monte Carlo Filtering',DSGEModel,CurrINI);
                     SolutionCode = [];
                     SaveDraws = 1;
                  end;
                  if isempty(thetaDraws)==0;
                     %
                     % check if we should save the draws to file
                     %
                     if SaveDraws==1;
                        save(PriorDrawsFile,'thetaDraws');
                     end;
                     %
                     % now run the Monte Carlo Filtering function
                     %
                     if isempty(SolutionCode)==1;
                        SolutionCode = MonteCarloFiltering(thetaDraws,thetaPositions,ModelParameters,AIMData,DSGEModel,CurrINI);
                        if isempty(SolutionCode)==0;
                           NumUnique = sum(SolutionCode==1);
                           if (NumUnique>=200)&(NumPriorDraws-NumUnique>=200);
                              SaveSolutionCode = 1;
                           elseif (NumUnique==0)|(NumUnique==NumPriorDraws);
                              SaveSolutionCode = 0;
                              %
                              % a really bad case
                              %
                              txt = ['The number of parameter draws with a unique and convergent solution is equal to ' int2str(NumUnique) ' while with number of cases with indeterminacy and no solution are ' int2str(sum(SolutionCode==2)) ' and ' int2str(sum(SolutionCode==0)) ', respectively. Monte Carlo filtering requires that both the number of unique and convergent solutions as well as the sum of the number of cases with indeterminacy and no solution are greater than zero and preferably at least equal to 200. YADA has therefore aborted from the exercise.'];
                              About(txt,'information','Monte Carlo Filtering',300,500,CurrINI);
                           else;
                              if NumUnique>=200;
                                 RecNumDraws = ceil((200/(NumPriorDraws-NumUnique))*NumPriorDraws);
                              else;
                                 RecNumDraws = ceil((200/NumUnique)*NumPriorDraws);
                              end;
                              txt = ['The number of parameter draws with a unique and convergent solution is equal to ' int2str(NumUnique) ' while with number of cases with indeterminacy and no solution are ' int2str(sum(SolutionCode==2)) ' and ' int2str(sum(SolutionCode==0)) ', respectively. It is recommended that both the number of unique and convergent solutions and the sum of the other two cases are at least 200. To achieve this you should increase the number of draws from the prior distribution to at least ' int2str(RecNumDraws) '. Are you sure that you would like to keep these results?'];
                              answer = Query(txt,'question',300,'Monte Carlo Filtering',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 SaveSolutionCode = 1;
                              else;
                                 SaveSolutionCode = 0;
                              end;
                           end;
                           if SaveSolutionCode==1;
                              save(PriorDrawsFile,'SolutionCode','-APPEND');
                           else;
                              delete(PriorDrawsFile);
                              SolutionCode = [];
                           end;
                        end;
                     end;
                     %
                     if isempty(SolutionCode)==0;
                        %
                        % keep the new value
                        %
                        DSGEModel.MonteCarloFilterDraws = PriorDrawsValue;
                        %
                        % process results
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\MonteCarloFiltering-' DSGEModel.NameOfModel '-' int2str(NumPriorDraws) '.txt']);
                        PrintMonteCarloFiltering(ResFile,SolutionCode,thetaDraws,ParameterNames.estimated,DSGEModel,CurrINI);
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['Monte Carlo Filtering - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Kolmogorov-Smirnov Tests',50,CurrINI,1,0);
                        end;
                        %
                        % plot the Monte Carlo filtering densities
                        %
                        GraphMonteCarloFilteringDensity(SolutionCode,thetaDraws,ParameterNames.estimated,0,DSGEModel,CurrINI);
                        %
                        % now show graphs
                        %
                        ScatterPlotMonteCarloFiltering('init',SolutionCode,thetaDraws,ParameterNames.estimated,DSGEModel,CurrINI);
                     end;
                  end;
               end;
            elseif (InitStatus==1)&(isempty(ParameterNames.estimated)==1);
               txt = 'All parameters of your DSGE model have status calibrated. Hence, there is no prior distribution from which YADA can sample.';
               About(txt,'information','Prior Sampling - Calibrated DSGE Model',200,500,CurrINI);
            end;
         else;
            txt = 'The model has a system prior and YADA has therefore skipped Monte Carlo filtering. Prior sampling with a system prior is achieved with a Metropolis-Hastings algorithm, with the effect that all prior draws imply a unique and convergent solution. This is not consistent with Monte Carlo filtering, which allows some draws to imply indeterminacy or no solution.';
            About(txt,'information','Monte Carlo Filtering',350,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PoorManInverseInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
         %
         [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         if InitStatus==1;
            %
            % load the AIMData file
            %
            AIMData = load(DSGEModel.AIMDataFile);
            %
            % check if we can solve the model
            %
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                  if isempty(ErrMsg)==0;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                  end;
               catch ME;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                  if isempty(ErrMsg)==0;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                  end;
               catch;
                  ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if mcode==1;
                  if length(size(H))==3;
                     %
                     % check if H has the required size
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     if size(H,3)>=LastPeriod;
                        %
                        % pick a certain period for the H matrix
                        %
                        SubSampleMat = get(controls.dsge.initializekalman,'String');
                        SubSampleMat = SubSampleMat(DSGEModel.KalmanFirstObservation:size(SubSampleMat,1),:);
                        [action,SelectedPosition] = SelectionDlg(SubSampleMat,size(SubSampleMat,1),'Select time period for the measurement matrix H','Measurement Matrix','DSGE Model To VAR Model - Initial Values','','',CurrINI);
                        %
                        SelPos = FirstPeriod+DSGEModel.KalmanFirstObservation-1+(SelectedPosition-1);
                        H = H(:,:,SelPos);
                     else;
                        action = 'Cancel';
                        ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(LastPeriod) '). YADA has therefore aborted from checking the poor man''s invertibility condition.'];
                        About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
                     end;
                  else;
                     action = 'OK';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     %
                     % now we check the eigenvalue condition
                     %
                     [status,EigenValues,Fstatus] = DSGEtoVARModel(H,R,F,B0);
                     if status==1;
                        if Fstatus==1;
                           Ftest = 'Since the largest eigenvalue of the state variable transition matrix is less than unity, the poor man''s invertibility condition is not only sufficient, but also necessary.';
                        else;
                           Ftest = 'Since the largest eigenvalue of the state variable transition matrix is unstable, the poor man''s invertibility condition, while sufficient, need not be necessary (unles certain other more complex conditions are met).';
                        end;
                        StatusStr = ['The "poor man''s invertibility condition" (see Fernndez-Villaverde, Rubio-Ramrez, Sargent and Watson, 2007, AER) is satisfied, with the largest eigenvalue being equal to ' num2str(max(EigenValues)) ' and is therefore less than unity. The DSGE model can therefore be rewritten as an infinite order VAR model. ' Ftest];
                     elseif status==0;
                        if Fstatus==1;
                           Ftest = 'Since the largest eigenvalue of the state variable transition matrix is less than unity, the poor man''s invertibility condition is not only sufficient, but also necessary. The DSGE model can therefore not be rewritten as an infinite order VAR model.';
                        else;
                           Ftest = 'Since the largest eigenvalue of the state variable transition matrix is unstable, the poor man''s invertibility condition, while sufficient, need not be necssary (unles certain other more complex conditions are met). It is therefore not known from the undertaken eigenvalue tests if the DSGE model can nevertheless be rewritten as in infinite order VAR model.';
                        end;
                        StatusStr = ['The "poor man''s invertibility condition" (see Fernndez-Villaverde, Rubio-Ramrez, Sargent and Watson, 2007, AER) is not satisfied. The largest eigenvalue is equal to ' num2str(max(EigenValues)) ' and is therefore not less than unity. ' Ftest];
                     elseif status==-1;
                        StatusStr = 'The DSGE model has too many economic shocks and unique measurement errors relative to the number of observed variables. It can therefore not be rewitten as an infinite order VAR model.';
                     else;
                        StatusStr = 'The DSGE model has too few economic shocks and unique measurement errors relative to the number of observed variables. It can therefore not be rewitten as an infinite order VAR model.';
                     end;
                     About(StatusStr,'information','DSGE Model To VAR Model - Initial Values',300,600,CurrINI);
                  end;
               else;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                  About(txt,'information','DSGE Model To VAR Model - Initial Values',120,500,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'PoorManInversePostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % copy files to the tmp folder
            %
            [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
            [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
            end;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
            end;
            %
            % check if the model solver file should be copied
            %
            if DSGEModel.ModelSolver==4;
               if FileExist(DSGEModel.ModelSolverFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
               end;
            end;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                     if isempty(ErrMsg)==0;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                     end;
                  catch ME;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                     if isempty(ErrMsg)==0;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                     end;
                  catch;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if mcode==1;
                     if length(size(H))==3;
                        %
                        % check if H has the required size
                        %
                        [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                        if size(H,3)>=LastPeriod;
                           %
                           % pick a certain period for the H matrix
                           %
                           SubSampleMat = get(controls.dsge.initializekalman,'String');
                           SubSampleMat = SubSampleMat(DSGEModel.KalmanFirstObservation:size(SubSampleMat,1),:);
                           [action,SelectedPosition] = SelectionDlg(SubSampleMat,size(SubSampleMat,1),'Select time period for the measurement matrix H','Measurement Matrix','DSGE Model To VAR Model - Posterior Mode','','',CurrINI);
                           %
                           SelPos = FirstPeriod+DSGEModel.KalmanFirstObservation-1+(SelectedPosition-1);
                           H = H(:,:,SelPos);
                        else;
                           action = 'cancel';
                           ErrorStr = ['The number of time periods for the time-varying measurement matrix H (' int2str(size(H,3)) ') is lower than the number of observations (T = ' int2str(LastPeriod) '). YADA has therefore aborted from checking the poor man''s invertibility condition.'];
                           About(ErrorStr,'error','Error - Measurement Equation',500,600,CurrINI);
                        end;
                     else;
                        action = 'OK';
                     end;
                     if strcmp(lower(action),'ok')==1;
                        %
                        % now we check the eigenvalue condition
                        %
                        [status,EigenValues,Fstatus] = DSGEtoVARModel(H,R,F,B0);
                        if status==1;
                           if Fstatus==1;
                              Ftest = 'Since the largest eigenvalue of the state variable transition matrix is less than unity, the poor man''s invertibility condition is not only sufficient, but also necessary.';
                           else;
                              Ftest = 'Since the largest eigenvalue of the state variable transition matrix is unstable, the poor man''s invertibility condition, while sufficient, need not be necessary (unles certain other more complex conditions are met).';
                           end;
                           StatusStr = ['The "poor man''s invertibility condition" (see Fernndez-Villaverde, Rubio-Ramrez, Sargent and Watson, 2007, AER) is satisfied, with the largest eigenvalue being equal to ' num2str(max(EigenValues)) ' and is therefore less than unity. The DSGE model can therefore be rewritten as an infinite order VAR model. ' Ftest];
                        elseif status==0;
                           if Fstatus==1;
                              Ftest = 'Since the largest eigenvalue of the state variable transition matrix is less than unity, the poor man''s invertibility condition is not only sufficient, but also necessary. The DSGE model can therefore not be rewritten as an infinite order VAR model.';
                           else;
                              Ftest = 'Since the largest eigenvalue of the state variable transition matrix is unstable, the poor man''s invertibility condition, while sufficient, need not be necssary (unles certain other more complex conditions are met). It is therefore not known from the undertaken eigenvalue tests if the DSGE model can nevertheless be rewritten as in infinite order VAR model.';
                           end;
                           StatusStr = ['The "poor man''s invertibility condition" (see Fernndez-Villaverde, Rubio-Ramrez, Sargent and Watson, 2007, AER) is not satisfied. The largest eigenvalue is equal to ' num2str(max(EigenValues)) ' and is therefore not less than unity. ' Ftest];
                        elseif status==-1;
                           StatusStr = 'The DSGE model has too many economic shocks and unique measurement errors relative to the number of observed variables. It can therefore not be rewitten as an infinite order VAR model.';
                        else;
                           StatusStr = 'The DSGE model has too few economic shocks and unique measurement errors relative to the number of observed variables. It can therefore not be rewitten as an infinite order VAR model.';
                        end;
                        About(StatusStr,'information','DSGE Model To VAR Model - Posterior Mode',300,600,CurrINI);
                     end;
                  else;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     About(txt,'information','DSGE Model To VAR Model - Posterior Mode',500,600,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'InfMatFreqDomainInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % get the parameter data
         %
         [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  ParameterNames = ModeData.ParameterNames;
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         if InitStatus==1;
            %
            % determine the parameter names
            %
            ParameterNames = ParameterNames.estimated;
            %
            % calculate Fisher's information matrix in the frequency domain for the DSGE model
            %
            [InformationMatrix,ParamIncluded,ParamExcluded] = DSGEFrequencyDomainInfMat(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
            if isempty(InformationMatrix)==0;
               %
               % determine which parameters are included and which are excluded from the analysis
               %
               IncludedParameterNames = ParameterNames(ParamIncluded,:);
               ExcludedParameterNames = ParameterNames(ParamExcluded,:);
               %
               % compute the inverse of Fisher's information if possible
               %
               if (rank(InformationMatrix,1.02-010)==length(InformationMatrix))&(IsPositiveDefinite(InformationMatrix)==1);
                  invIMatrix = eye(length(InformationMatrix))/InformationMatrix;
                  stdvec = sqrt(diag(invIMatrix));
                  CorrMatrix = invIMatrix./(stdvec*stdvec');
                  %
                  % compute eigenvectors and eigenvalues of the correlation amtrix
                  %
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  % check which parameters have an absolute correlation greater than or equal 0.7
                  %
                  ExtremeCorrBound = 0.7;
                  CorrMatrix = tril(CorrMatrix,-1);
                  [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                  if isempty(RowIndex)==0;
                     ExtremeCorrParameters = cell(length(RowIndex),3);
                     for i=1:length(RowIndex)
                        ExtremeCorrParameters{i,1} = IncludedParameterNames(RowIndex(i),:);
                        ExtremeCorrParameters{i,2} = IncludedParameterNames(ColIndex(i),:);
                        ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                     end;
                  else;
                     ExtremeCorrParameters = [];
                  end;
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-InitialValues-' DSGEModel.NameOfModel '.txt']);
                  %
                  % write the ExtremeCorrParameters to file
                  %
                  WriteString = 'wt';
                  if isempty(ExtremeCorrParameters)==0;
                     WriteString = 'at';
                     HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *');
                     PrintExtremeCorrelations(TextFile,HeaderStr,'Frequency',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),size(IncludedParameterNames,1),CurrINI);
                  end;
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,WriteString,CurrINI);
                  fid = fopen(TextFile,'at');
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(ParamIncluded);
                     fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % print which parameters were excluded from the analysis
                  %
                  if isempty(ParamExcluded)==0;
                     fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                     ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                     fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                     fprintf(fid,'================================================================================\n\n');
                  end;
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               else;
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-InitialValues-' DSGEModel.NameOfModel '.txt']);
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,'wt',CurrINI);
                  fid = fopen(TextFile,'at');
                  fprintf(fid,'Rank of information matrix:    %0.0f\n',rank(InformationMatrix));
                  fprintf(fid,'Number of included parameters: %0.0f\n',length(ParamIncluded));
                  fprintf(fid,'Number of excluded parameters: %0.0f\n\n',length(ParamExcluded));
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(ParamIncluded);
                     fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % print which parameters were excluded from the analysis
                  %
                  if isempty(ParamExcluded)==0;
                     fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                     ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                     fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                     fprintf(fid,'================================================================================\n\n');
                  end;
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               end;
            end;
         end;
         set(controls.about,'UserData',0);
      end;
   case 'InfMatFreqDomainPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the autocorrelations
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % calculate Fisher's information matrix in the frequency domain for the DSGE model
            %
            [InformationMatrix,ParamIncluded,ParamExcluded] = DSGEFrequencyDomainInfMat(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
            if isempty(InformationMatrix)==0;
               %
               % determine which parameters are included and which are excluded from the analysis
               %
               IncludedParameterNames = ParameterNames(ParamIncluded,:);
               ExcludedParameterNames = ParameterNames(ParamExcluded,:);
               %
               % compute the inverse of Fisher's information if possible
               %
               if (rank(InformationMatrix,1.02-010)==length(InformationMatrix))&(IsPositiveDefinite(InformationMatrix)==1);
                  invIMatrix = eye(length(InformationMatrix))/InformationMatrix;
                  stdvec = sqrt(diag(invIMatrix));
                  CorrMatrix = invIMatrix./(stdvec*stdvec');
                  %
                  % compute eigenvectors and eigenvalues of the correlation amtrix
                  %
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  % check which parameters have an absolute correlation greater than or equal 0.7
                  %
                  ExtremeCorrBound = 0.7;
                  CorrMatrix = tril(CorrMatrix,-1);
                  [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                  if isempty(RowIndex)==0;
                     ExtremeCorrParameters = cell(length(RowIndex),3);
                     for i=1:length(RowIndex)
                        ExtremeCorrParameters{i,1} = IncludedParameterNames(RowIndex(i),:);
                        ExtremeCorrParameters{i,2} = IncludedParameterNames(ColIndex(i),:);
                        ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                     end;
                  else;
                     ExtremeCorrParameters = [];
                  end;
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-PosteriorMode-' DSGEModel.NameOfModel '.txt']);
                  %
                  % write the ExtremeCorrParameters to file
                  %
                  WriteString = 'wt';
                  if isempty(ExtremeCorrParameters)==0;
                     WriteString = 'at';
                     HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *');
                     PrintExtremeCorrelations(TextFile,HeaderStr,'Frequency',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),size(IncludedParameterNames,1),CurrINI);
                  end;
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,WriteString,CurrINI);
                  fid = fopen(TextFile,'at');
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(ParamIncluded);
                     fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % print which parameters were excluded from the analysis
                  %
                  if isempty(ParamExcluded)==0;
                     fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                     ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                     fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                     fprintf(fid,'================================================================================\n\n');
                  end;
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               else;
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-FrequencyDomain-PosteriorMode-' DSGEModel.NameOfModel '.txt']);
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,IncludedParameterNames,'wt',CurrINI);
                  fid = fopen(TextFile,'at');
                  fprintf(fid,'Rank of information matrix:    %0.0f\n',rank(InformationMatrix));
                  fprintf(fid,'Number of included parameters: %0.0f\n',length(ParamIncluded));
                  fprintf(fid,'Number of excluded parameters: %0.0f\n\n',length(ParamExcluded));
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(ParamIncluded);
                     fprintf(fid,' %s\n',IncludedParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % print which parameters were excluded from the analysis
                  %
                  if isempty(ParamExcluded)==0;
                     fprintf(fid,'NOTE: The following parameters are excluded from the analysis\n      since they don''t affect the population spectrum:\n\n');
                     ExcludedParameterNames = vecstr(ExcludedParameterNames,', ');
                     fprintf(fid,'      %s.\n\n',ExcludedParameterNames(1,1:length(ExcludedParameterNames)-1));
                     fprintf(fid,'================================================================================\n\n');
                  end;
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'InfMatTimeDomainInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to check the information matrix before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Information Matrix - Time Domain',450,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               ParameterNames = ParameterNames.estimated;
               %
               % calculate Fisher's information matrix for the DSGE model
               %
               InformationMatrix = DSGEInformationMatrix(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
               %
               % compute the inverse of Fisher's information
               %
               if rank(InformationMatrix,1.02-010)==length(theta);
                  invIMatrix = eye(length(theta))/InformationMatrix;
                  stdvec = sqrt(diag(invIMatrix));
                  CorrMatrix = invIMatrix./(stdvec*stdvec');
                  %
                  % compute eigenvectors and eigenvalues of the correlation amtrix
                  %
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  % check which parameters have an absolute correlation greater than or equal 0.7
                  %
                  ExtremeCorrBound = 0.7;
                  CorrMatrix = tril(CorrMatrix,-1);
                  [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                  if isempty(RowIndex)==0;
                     ExtremeCorrParameters = cell(length(RowIndex),3);
                     for i=1:length(RowIndex)
                        ExtremeCorrParameters{i,1} = ParameterNames(RowIndex(i),:);
                        ExtremeCorrParameters{i,2} = ParameterNames(ColIndex(i),:);
                        ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                     end;
                  else;
                     ExtremeCorrParameters = [];
                  end;
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-InitialValues-' DSGEModel.NameOfModel '.txt']);
                  %
                  % write the ExtremeCorrParameters to file
                  %
                  WriteString = 'wt';
                  if isempty(ExtremeCorrParameters)==0;
                     WriteString = 'at';
                     HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *');
                     PrintExtremeCorrelations(TextFile,HeaderStr,'Time',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),length(theta),CurrINI);
                  end;
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,WriteString,CurrINI);
                  fid = fopen(TextFile,'at');
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(theta);
                     fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Parameter Values',50,CurrINI,1,1);
                  end;
               else;
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-InitialValues-' DSGEModel.NameOfModel '.txt']);
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,'wt',CurrINI);
                  fid = fopen(TextFile,'at');
                  fprintf(fid,'Rank of information matrix: %0.0f\n',rank(InformationMatrix));
                  fprintf(fid,'Number of parameters:       %0.0f\n\n',length(theta));
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(theta);
                     fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'InfMatTimeDomainPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the autocorrelations
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
            %
            % calculate Fisher's information matrix for the DSGE model
            %
            InformationMatrix = DSGEInformationMatrix(theta,thetaPositions,ModelParameters,ParameterNames,DSGEModel,CurrINI);
            if isempty(InformationMatrix)==0;
               %
               % compute the inverse of Fisher's information
               %
               if rank(InformationMatrix,1.02-010)==length(theta);
                  invIMatrix = eye(length(theta))/InformationMatrix;
                  stdvec = sqrt(diag(invIMatrix));
                  CorrMatrix = invIMatrix./(stdvec*stdvec');
                  %
                  % compute eigenvectors and eigenvalues of the correlation amtrix
                  %
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  % check which parameters have an absolute correlation greater than or equal 0.7
                  %
                  ExtremeCorrBound = 0.7;
                  CorrMatrix = tril(CorrMatrix,-1);
                  [RowIndex,ColIndex] = find(abs(CorrMatrix)>=ExtremeCorrBound);
                  if isempty(RowIndex)==0;
                     ExtremeCorrParameters = cell(length(RowIndex),3);
                     for i=1:length(RowIndex)
                        ExtremeCorrParameters{i,1} = ParameterNames(RowIndex(i),:);
                        ExtremeCorrParameters{i,2} = ParameterNames(ColIndex(i),:);
                        ExtremeCorrParameters{i,3} = CorrMatrix(RowIndex(i),ColIndex(i));
                     end;
                  else;
                     ExtremeCorrParameters = [];
                  end;
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-PosteriorMode-' DSGEModel.NameOfModel '.txt']);
                  %
                  % write the ExtremeCorrParameters to file
                  %
                  WriteString = 'wt';
                  if isempty(ExtremeCorrParameters)==0;
                     WriteString = 'at';
                     HeaderStr = strvcat('*       P A R A M E T E R   C O R R E L A T I O N S   U S I N G   T H E        *','* I N V E R S E   O F   F I S H E R '' S   I N F O R M A T I O N   M A T R I X  *');
                     PrintExtremeCorrelations(TextFile,HeaderStr,'Time',ExtremeCorrParameters,num2str(ExtremeCorrBound,'%0.2f'),rank(InformationMatrix),length(theta),CurrINI);
                  end;
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,WriteString,CurrINI);
                  fid = fopen(TextFile,'at');
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(theta);
                     fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  %
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               else;
                  [EigenVectorMatrix,EigenValues,ParameterOrdering] = DSGEIdentificationPatterns(InformationMatrix);
                  %
                  TextFile = FixFilePath([DSGEModel.OutputDirectory '\InformationMatrixProperties-TimeDomain-PosteriorMode-' DSGEModel.NameOfModel '.txt']);
                  PrintSmallEigenValues(TextFile,EigenValues,EigenVectorMatrix,ParameterNames,'wt',CurrINI);
                  fid = fopen(TextFile,'at');
                  fprintf(fid,'Rank of information matrix: %0.0f\n',rank(InformationMatrix));
                  fprintf(fid,'Number of parameters:       %0.0f\n\n',length(theta));
                  %
                  % print data on parameter ordering
                  %
                  fprintf(fid,'********************************************************************************\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'*            P A R A M E T E R   O R D E R I N G   B A S E D   O N             *\n');
                  fprintf(fid,'*  Q R - F A C T O R I Z A T I O N   W I T H   C O L U M N   P I V O T I N G   *\n');
                  fprintf(fid,'*              O F   T H E   I N F O R M A T I O N   M A T R I X               *\n');
                  fprintf(fid,'*                                                                              *\n');
                  fprintf(fid,'********************************************************************************\n\n');
                  %
                  fprintf(fid,' Parameters are ordered from the most to the least identified\n');
                  fprintf(fid,'==============================================================\n');
                  for i=1:length(theta);
                     fprintf(fid,' %s\n',ParameterNames(ParameterOrdering(i,2),:));
                  end;
                  fprintf(fid,'==============================================================\n\n');
                  %
                  % Some information about the filename and its location
                  %
                  fprintf(fid,'Directory for file: %s\n',GetPath(TextFile));
                  fprintf(fid,'Name of file:       %s\n',GetFilename(TextFile));
                  fprintf(fid,'Output created on:  %s\n\n',datestr(now,0));
                  fclose(fid);
                  if FileExist(TextFile)==1;
                     TextGUI(TextFile,['Fisher''s Information Matrix - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,1);
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'DSGESimulateDataInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % get the parameter data
         %
         [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if isempty(ErrorStr)==1;
                  theta = ModeData.theta;
                  thetaPositions = ModeData.thetaPositions;
                  ModelParameters = ModeData.ModelParameters;
                  ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                  InitStatus = 1;
               else;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            end;
         end;
         %
         if InitStatus==1;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                     if isempty(ErrMsg)==0;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                     end;
                  catch ME;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                     if isempty(ErrMsg)==0;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                     end;
                  catch;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if mcode==1;
                     %
                     % select the number of observations for the simulations
                     %
                     NumObs = (50:25:10000);
                     Tpos = (NumObs==DSGEModel.T) .* (1:length(NumObs));
                     Tpos = Tpos(Tpos>0);
                     if isempty(Tpos)==1;
                        Tpos = (NumObs>DSGEModel.T+1) .* (1:length(NumObs));
                        Tpos = Tpos(Tpos>0);
                        if isempty(Tpos)==0;
                           Tpos = Tpos(1);
                        end;
                        if Tpos==1;
                           NumObs = [DSGEModel.T NumObs];
                        elseif isempty(Tpos)==1;
                           NumObs = [NumObs DSGEModel.T];
                           Tpos = length(NumObs);
                        else;
                           NumObs = [NumObs(1:Tpos-1) DSGEModel.T NumObs(Tpos+1:length(NumObs))];
                        end;
                     end;
                     %
                     % create the string matrix with the number of observation
                     %
                     ObsStrMat = '';
                     for i=1:length(NumObs);
                        ObsStrMat = strvcat(ObsStrMat,[int2str(NumObs(i)) ' observations']);
                     end;
                     %
                     % display the selection dialog
                     %
                     [action,SelectedOption] = SelectionDlg(ObsStrMat,Tpos,'Select the number of observations to simulate','Simulate Data Selection','Simulate Data - Initial Values','','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        %
                        % simulate the data
                        %
                        [SimulatedData,KeepShocks] = DSGESimulateVariables(A,H,R,F,B0,NumObs(SelectedOption),DSGEModel,CurrINI);
                        %
                        % ask if simulated data should be saved to file
                        %
                        txt = 'Would you like to save the simulated data to file?';
                        answer = Query(txt,'question',140,'Save - Simulate Data',500,'no',CurrINI);
                        drawnow;
                        if strcmp(lower(answer),'yes')==1;
                           Y = SimulatedData.Y;
                           X = SimulatedData.X;
                           ksi = SimulatedData.ksi;
                           eta = SimulatedData.eta;
                           w = SimulatedData.w;
                           H = SimulatedData.Parameters.H;
                           B0 = SimulatedData.Parameters.B0;
                           YNames = DSGEModel.VariableNames;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
                           file = FixFilePath([DSGEModel.OutputDirectory '\SimulatedData-' DSGEModel.NameOfModel '-InitialValues.mat']);
                           if isempty(DSGEModel.X)==0;
                              XNames = DSGEModel.XVariableNames;
                              save(file,'Y','X','ksi','eta','w','A','H','R','F','B0','YNames','XNames','StateVariableNames','ShockNames');
                              NumEntries = '14';
                           else;
                              save(file,'Y','ksi','eta','w','H','R','F','B0','YNames','StateVariableNames','ShockNames');
                              NumEntries = '11';
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SimuledDataInitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% string matrix with the names of the exogenous variables\n%%\n');
                              fprintf(fid,'XNames = DataStructure.XNames;\n');
                           end;
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the exonomic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% matrix with the simulated observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% matrix with the generated exogenous variables\n%%\n');
                              fprintf(fid,'X = DataStructure.X;\n');
                           end;
                           fprintf(fid,'%%\n%% matrix with the simulated state variables\n%%\n');
                           fprintf(fid,'ksi = DataStructure.ksi;\n');
                           fprintf(fid,'%%\n%% matrix with the simulated standardized economic shocks\n%%\n');
                           fprintf(fid,'eta = DataStructure.eta;\n');
                           fprintf(fid,'%%\n%% matrix with the simulated measurement errors\n%%\n');
                           fprintf(fid,'w = DataStructure.w;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% A-matrix for the measurement equation\n%%\n');
                              fprintf(fid,'A = DataStructure.A;\n');
                              XNameStr = 'XNames (string matrix with names of the exogenous variables), ';
                              XStr = 'X (matrix with generated data for the exogenous variables), ';
                              AStr = 'A (matrix with parameters on the exogenous variables in the measurement equation), ';
                           else;
                              XNameStr = '';
                              XStr = '';
                              AStr = '';
                           end;
                           fprintf(fid,'%%\n%% H-matrix for the measurement equation\n%%\n');
                           fprintf(fid,'H = DataStructure.H;\n');
                           fprintf(fid,'%%\n%% covariance matrix for the measurement errors\n%%\n');
                           fprintf(fid,'R = DataStructure.R;\n');
                           fprintf(fid,'%%\n%% F-matrix for the state equation\n%%\n');
                           fprintf(fid,'F = DataStructure.F;\n');
                           fprintf(fid,'%%\n%% B0-matrix for the state equation\n%%\n');
                           fprintf(fid,'B0 = DataStructure.B0;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The simulated data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), ' XNameStr 'StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), Y (matrix with simulated data for the observed variables), ' XStr 'ksi (matrix with simulated data for the state variables), eta (matrix with simulated standardized economic shocks), w (matrix with simulated measurement errors), ' AStr 'H (measurement matrix for the state variables), R (covariance matrix for the measurement errors), F (state transition matrix), and B0 (coefficient matrix on the economic shocks in the state equation). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Simulated Data - Initial Values',400,500,CurrINI);
                           drawnow;
                        end;
                        %
                        % display the generated data
                        %
                        GraphSimulatedData('init',CurrINI,DSGEModel,SimulatedData.Y,[],'Initial Values');
                     end;
                  else;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     About(txt,'information','Simulated Data - Initial Values',120,500,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'DSGESimulateDataPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % copy files to the tmp folder
            %
            [stat,msg] = CopyFile([GetPath(DSGEModel.AIMDataFile) 'compute_aim_matrices.m'],[pwd '\tmp']);
            [stat,msg] = CopyFile(DSGEModel.MeasurementEquationFile,[pwd '\tmp']);
            if FileExist(DSGEModel.UpdateParameterFile)==1;
               [stat,msg] = CopyFile(DSGEModel.UpdateParameterFile,[pwd '\tmp']);
            end;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               [stat,msg] = CopyFile(DSGEModel.SystemPriorFile,[pwd '\tmp']);
            end;
            %
            % check if we should copy model solver file
            %
            if DSGEModel.ModelSolver==4;
               if FileExist(DSGEModel.ModelSolverFile)==1;
                  [stat,msg] = CopyFile(DSGEModel.ModelSolverFile,[pwd '\tmp']);
               end;
            end;
            %
            % load the AIMData file
            %
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
            if isempty(ErrorStr)==1;
               %
               % check if we can solve the model
               %
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                     if isempty(ErrMsg)==0;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                     end;
                  catch ME;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     [A,H,R,F,B0,mcode,ErrMsg] = SolveDSGEModel(DSGEModel,theta,thetaPositions,ModelParameters,AIMData,CurrINI.OrderQZ);
                     if isempty(ErrMsg)==0;
                        ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' ErrMsg];
                     end;
                  catch;
                     ErrorStr = ['YADA caught an error when trying to solve the DSGE model. Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if mcode==1;
                     %
                     % select the number of observations for the simulations
                     %
                     NumObs = (50:25:10000);
                     Tpos = (NumObs==DSGEModel.T) .* (1:length(NumObs));
                     Tpos = Tpos(Tpos>0);
                     if isempty(Tpos)==1;
                        Tpos = (NumObs>DSGEModel.T+1) .* (1:length(NumObs));
                        Tpos = Tpos(Tpos>0);
                        if isempty(Tpos)==0;
                           Tpos = Tpos(1);
                        end;
                        if Tpos==1;
                           NumObs = [DSGEModel.T NumObs];
                        elseif isempty(Tpos)==1;
                           NumObs = [NumObs DSGEModel.T];
                           Tpos = length(NumObs);
                        else;
                           NumObs = [NumObs(1:Tpos-1) DSGEModel.T NumObs(Tpos+1:length(NumObs))];
                        end;
                     end;
                     %
                     % create the string matrix with the number of observation
                     %
                     ObsStrMat = '';
                     for i=1:length(NumObs);
                        ObsStrMat = strvcat(ObsStrMat,[int2str(NumObs(i)) ' observations']);
                     end;
                     %
                     % display the selection dialog
                     %
                     [action,SelectedOption] = SelectionDlg(ObsStrMat,Tpos,'Select the number of observations to simulate','Simulate Data Selection','Simulate Data - Posterior Mode','','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        %
                        % simulate the data
                        %
                        [SimulatedData,KeepShocks] = DSGESimulateVariables(A,H,R,F,B0,NumObs(SelectedOption),DSGEModel,CurrINI);
                        %
                        % ask if simulated data should be saved to file
                        %
                        txt = 'Would you like to save the simulated data to file?';
                        answer = Query(txt,'question',140,'Save - Simulate Data',500,'no',CurrINI);
                        drawnow;
                        if strcmp(lower(answer),'yes')==1;
                           Y = SimulatedData.Y;
                           X = SimulatedData.X;
                           ksi = SimulatedData.ksi;
                           eta = SimulatedData.eta;
                           w = SimulatedData.w;
                           H = SimulatedData.Parameters.H;
                           B0 = SimulatedData.Parameters.B0;
                           YNames = DSGEModel.VariableNames;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
                           file = FixFilePath([DSGEModel.OutputDirectory '\SimulatedData-' DSGEModel.NameOfModel '-PostMode.mat']);
                           if isempty(DSGEModel.X)==0;
                              XNames = DSGEModel.XVariableNames;
                              save(file,'Y','X','ksi','eta','w','A','H','R','F','B0','YNames','XNames','StateVariableNames','ShockNames');
                              NumEntries = '14';
                           else;
                              save(file,'Y','ksi','eta','w','H','R','F','B0','YNames','StateVariableNames','ShockNames');
                              NumEntries = '11';
                           end;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SimuledDataPostMode.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% string matrix with the names of the exogenous variables\n%%\n');
                              fprintf(fid,'XNames = DataStructure.XNames;\n');
                           end;
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the exonomic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           fprintf(fid,'%%\n%% matrix with the simulated observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% matrix with the generated exogenous variables\n%%\n');
                              fprintf(fid,'X = DataStructure.X;\n');
                           end;
                           fprintf(fid,'%%\n%% matrix with the simulated state variables\n%%\n');
                           fprintf(fid,'ksi = DataStructure.ksi;\n');
                           fprintf(fid,'%%\n%% matrix with the simulated standardized economic shocks\n%%\n');
                           fprintf(fid,'eta = DataStructure.eta;\n');
                           fprintf(fid,'%%\n%% matrix with the simulated measurement errors\n%%\n');
                           fprintf(fid,'w = DataStructure.w;\n');
                           if isempty(DSGEModel.X)==0;
                              fprintf(fid,'%%\n%% A-matrix for the measurement equation\n%%\n');
                              fprintf(fid,'A = DataStructure.A;\n');
                              XNameStr = 'XNames (string matrix with names of the exogenous variables), ';
                              XStr = 'X (matrix with generated data for the exogenous variables), ';
                              AStr = 'A (matrix with parameters on the exogenous variables in the measurement equation), ';
                           else;
                              XNameStr = '';
                              XStr = '';
                              AStr = '';
                           end;
                           fprintf(fid,'%%\n%% H-matrix for the measurement equation\n%%\n');
                           fprintf(fid,'H = DataStructure.H;\n');
                           fprintf(fid,'%%\n%% covariance matrix for the measurement errors\n%%\n');
                           fprintf(fid,'R = DataStructure.R;\n');
                           fprintf(fid,'%%\n%% F-matrix for the state equation\n%%\n');
                           fprintf(fid,'F = DataStructure.F;\n');
                           fprintf(fid,'%%\n%% B0-matrix for the state equation\n%%\n');
                           fprintf(fid,'B0 = DataStructure.B0;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The simulated data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), ' XNameStr 'StateVariableNames (string matrix with the names of the state variables), ShockNames (string matrix with the names of the economic shocks), Y (matrix with simulated data for the observed variables), ' XStr 'ksi (matrix with simulated data for the state variables), eta (matrix with simulated standardized economic shocks), w (matrix with simulated measurement errors), ' AStr 'H (measurement matrix for the state variables), R (covariance matrix for the measurement errors), F (state transition matrix), and B0 (coefficient matrix on the economic shocks in the state equation). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Simulated Data - Posterior Mode',400,500,CurrINI);
                           drawnow;
                        end;
                        %
                        % display the generated data
                        %
                        GraphSimulatedData('init',CurrINI,DSGEModel,SimulatedData.Y,[],'Posterior Mode');
                     end;
                  else;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '"'];
                     About(txt,'information','Simulated Data - Posterior Mode',120,500,CurrINI);
                  end;
               else;
                  About(ErrorStr,'information','YADA - Error Message',CurrINI.scrsz(4)-50,600,CurrINI);
               end;
            else;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'OSAForecastInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the forecasted observed variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'1-Step Ahead Forecasts - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               %
               % Try to calculate state variables. First, however, the code needs to check if a unique convergent
               % solution exists.
               %
               [StateVarStructure,mcode,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
               if (mcode==1)&(kalmanstatus==0);
                  %
                  % we're OK
                  %
                  if isempty(StateVarStructure)==0;
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save the observed variables and 1-step ahead forecasts at initial parameter values to file?';
                     answer = Query(txt,'question',140,'Save - 1-Step Ahead Forecasts',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        Y = StateVarStructure.Y;
                        Yhat = StateVarStructure.Yhat;
                        ProjectedY = StateVarStructure.ProjectedY;
                        YNames = DSGEModel.VariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                        if length(DateVector)>size(Y,2);
                           DateVector = DateVector(1,length(DateVector)-length(Y)+1:length(DateVector));
                        end;
                        FullDateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                        file = FixFilePath([DSGEModel.OutputDirectory '\ObsVarForecast-InitialValues.mat']);
                        if DSGEModel.YNaNs==0;
                           save(file,'Y','Yhat','YNames','DateVector');
                        else;
                           save(file,'Y','Yhat','YNames','DateVector','FullDateVector','ProjectedY');
                        end;
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\OVFInitVal.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% matrix with the observed variables over the estimation sample\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        fprintf(fid,'%%\n%% matrix with the 1-step ahead forecasts of the observed variables\n%%\n');
                        fprintf(fid,'Yhat = DataStructure.Yhat;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        if DSGEModel.YNaNs==0;
                           TotalEntries = '4';
                           ExtraEntryStr = '';
                        else;
                           TotalEntries = '6';
                           ExtraEntryStr = ['ProjectedY (' int2str(size(ProjectedY,1)) 'x' int2str(size(ProjectedY,2)) ' matrix with smoothly projected values for the observed variables over the full sample), FullDateVector (1x' int2str(size(ProjectedY,2)) ' vector with dates for the full sample), '];
                           fprintf(fid,'%%\n%% matrix with the smoothly projected estimates of the observed variables for full sample\n%%\n');
                           fprintf(fid,'ProjectedY = DataStructure.ProjectedY;\n');
                           fprintf(fid,'%%\n%% vector with the dates for full sample\n%%\n');
                           fprintf(fid,'FullDateVector = DataStructure.FullDateVector;\n');
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The observed variables and the 1-step ahead forecasts at initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' TotalEntries ' entries: Y (' num2str(size(Y,1),'%0.0f') 'x' num2str(size(Y,2),'%0.0f') ' matrix with the observed variables over the estimation sample), Yhat (' num2str(size(Y,1),'%0.0f') 'x' num2str(size(Y,2),'%0.0f') ' matrix with the 1-step ahead forecasts of the observed variables), YNames (string matrix with the names of the oberved variables), ' ExtraEntryStr 'and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','1-Step Ahead Forecasts - Initial Values',200,500,CurrINI);
                     end;
                     %
                     % display the graphs
                     %
                     StateGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Initial Values','Observed');
                  end;
               elseif (mcode==1)&(kalmanstatus~=0);
                  if kalmanstatus==1;
                     txt = 'The state variables are not stationary at the initial values.';
                  else;
                     txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                  end;
                  About(txt,'information','1-Step Ahead Forecasts - Initial Values',200,500,CurrINI);
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','1-Step Ahead Forecasts - Initial Values',120,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'OSAForecastPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'1-Step Ahead Forecast Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','1-Step Ahead Forecast Distribution - Prior','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','1-Step Ahead Forecast Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % First we check if the calculations have already been performed
               %
               if SVEExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,0,0,0)==1;
                  txt = 'YADA has located results for the 1-step ahead forecasts on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'1-Step Ahead Forecast Distribution - Prior',450,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % start estimation
                  %
                  DoneCalc = DSGEStateVariableEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'1-Step Ahead Forecast',0,NumPriorDraws,1,DSGEModel,CurrINI);
               end;
               %
               % check if we should show any results
               %
               if DoneCalc==1;
                  %
                  % compute the distribution for the 1-step ahead forecasts
                  %
                  SVEs = DSGESVEDistribution(DSGEModel,CurrINI,1,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the 1-step ahead forecast distributions to file?';
                  answer = Query(txt,'question',140,'Save - 1-Step Ahead Forecast Distribution - Prior',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                     file = FixFilePath([DSGEModel.OutputDirectory '\OneStepForecastPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.mat']);
                     VariableNames = DSGEModel.VariableNames;
                     save(file,'SVEs','VariableNames');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\OSFPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) '.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% structure with fields Y (observed variables), Yhat (1-step ahead forecasts), Quantiles, and DateVector.\n%%\n');
                     fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                     fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The 1-step ahead forecast data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the mean for the 1-step ahead forecast and the observed values for the variables, quantile data, and numeric dates), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','1-Step Ahead Forecast Distribution - Prior',200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  StateVarDistributionDLG('init',SVEs,1,0,0,[],DSGEModel,CurrINI);
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the 1-step ahead forecast distribution.';
               About(txt,'information','1-Step Ahead Forecast Distribution - Prior',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'OSAForecastPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate forecasts
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % we can now compute the state variables
            %
            StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
            %
            if isempty(StateVarStructure)==0;
               %
               % ask if we should save to disk
               %
               txt = 'Would you like to save the observed variables and 1-step ahead forecasts at posterior mode to file?';
               answer = Query(txt,'question',140,'Save - 1-Step Ahead Forecasts',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  Y = StateVarStructure.Y;
                  Yhat = StateVarStructure.Yhat;
                  ProjectedY = StateVarStructure.ProjectedY;
                  YNames = DSGEModel.VariableNames;
                  DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                  if length(DateVector)>size(Y,2);
                     DateVector = DateVector(1,length(DateVector)-size(Y,2)+1:length(DateVector));
                  end;
                  FullDateVector = MakeDateVector(DSGEModel.BeginYear,DSGEModel.BeginPeriod,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.DataFrequency);
                  file = FixFilePath([DSGEModel.OutputDirectory '\ObsVarForecast-PosteriorMode.mat']);
                  if DSGEModel.YNaNs==0;
                     save(file,'Y','Yhat','YNames','DateVector');
                  else;
                     save(file,'Y','Yhat','YNames','DateVector','FullDateVector','ProjectedY');
                  end;
                  %
                  % create a matlab file to read the data
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\OVFPostMode.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% matrix with the observed variables over the estimation sample\n%%\n');
                  fprintf(fid,'Y = DataStructure.Y;\n');
                  fprintf(fid,'%%\n%% matrix with the 1-step ahead forecasts of the observed variables\n%%\n');
                  fprintf(fid,'Yhat = DataStructure.Yhat;\n');
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'YNames = DataStructure.YNames;\n');
                  fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                  fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                  if DSGEModel.YNaNs==0;
                     TotalEntries = '4';
                     ExtraEntryStr = '';
                  else;
                     TotalEntries = '6';
                     ExtraEntryStr = ['ProjectedY (' int2str(size(ProjectedY,1)) 'x' int2str(size(ProjectedY,2)) ' matrix with smoothly projected values for the observed variables over the full sample), FullDateVector (1x' int2str(size(ProjectedY,2)) ' vector with dates for the full sample), '];
                     fprintf(fid,'%%\n%% matrix with the smoothly projected estimates of the observed variables for full sample\n%%\n');
                     fprintf(fid,'ProjectedY = DataStructure.ProjectedY;\n');
                     fprintf(fid,'%%\n%% vector with the dates for full sample\n%%\n');
                     fprintf(fid,'FullDateVector = DataStructure.FullDateVector;\n');
                  end;
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  txt = ['The observed variables and the 1-step ahead forecasts at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' TotalEntries ' entries: Y (' num2str(size(Y,1),'%0.0f') 'x' num2str(size(Y,2),'%0.0f') ' matrix with the observed variables over the estimation sample), Yhat (' num2str(size(Y,1),'%0.0f') 'x' num2str(size(Y,2),'%0.0f') ' matrix with the 1-step ahead forecasts of the observed variables), YNames (string matrix with the names of the observed variables), ' ExtraEntryStr 'and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information','1-Step Ahead Forecasts - Posterior Mode',200,500,CurrINI);
               end;
               %
               % Display the graphs
               %
               StateGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Posterior Mode','Observed');
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'OSAForecastPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','1-Step Ahead Forecast Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','1-Step Ahead Forecast Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % First we check if the calculations have already been performed
                     %
                     if SVEExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,0,0,1,0)==1;
                        txt = 'YADA has located results for the 1-step ahead forecasts on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'1-Step Ahead Forecast Distribution',450,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % start estimation
                        %
                        DoneCalc = DSGEStateVariableEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'1-Step Ahead Forecast',1,NumDraws,CurrChain,DSGEModel,CurrINI);
                     end;
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % compute the distribution for the 1-step ahead forecasts
                        %
                        SVEs = DSGESVEDistribution(DSGEModel,CurrINI,1,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights);
                        %
                        % ask if we should save results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the 1-step ahead forecast distributions to file?';
                        answer = Query(txt,'question',140,'Save - 1-Step Ahead Forecast Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                           file = FixFilePath([DSGEModel.OutputDirectory '\OneStepForecastDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                           VariableNames = DSGEModel.VariableNames;
                           save(file,'SVEs','VariableNames');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\OSFDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields Y (observed variables), Yhat (1-step ahead forecasts), Quantiles, and DateVector.\n%%\n');
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The 1-step ahead forecast data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the mean for the 1-step ahead forecast and the observed values for the variables, quantile data, and numeric dates), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','1-Step Ahead Forecast Distribution',200,500,CurrINI);
                        end;
                        %
                        % display the results
                        %
                        StateVarDistributionDLG('init',SVEs,1,1,0,[],DSGEModel,CurrINI);
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the 1-step ahead forecast distribution.';
                     About(txt,'information','1-Step Ahead Forecast Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',500,600,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVariablesInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of projections of the state variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'State Variables - Initial Values',450,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               NumPathsStr = get(controls.forecast.predpaths,'String');
               TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % check if we should use simulation smoother
               %
               if TotalPaths>1;
                  txt = strvcat('Would you like to compute the distribution of the state variables conditional on the data and the initial parameter values (simulation smoothing) instead of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state variables conditional on all the data, and doesn''t cover the update and the forecast estimators.');
                  answer = Query(txt,'question',300,'State Variables - Initial Values',600,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  SimulSmooth = 1;
               else;
                  SimulSmooth = 0;
               end;
               %
               if SimulSmooth==0;
                  %
                  % check if we should run the smooth estimates recursively
                  %
                  txt = 'Would you also like to compute the smooth estimates recursively over the full estimation sample?';
                  answer = Query(txt,'question',200,'State Variables - Initial Values',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     RecEst = 1;
                  else;
                     RecEst = 0;
                  end;
                  %
                  % Try to calculate state variables. First, however, the code needs to check if a unique convergent
                  % solution exists.
                  %
                  [StateVarStructure,mcode,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,RecEst,'State Variables - Initial Values');
                  if (mcode==1)&(kalmanstatus==0);
                     %
                     % we're OK
                     %
                     if isempty(StateVarStructure)==0;
                        %
                        % ask if we should save to disk
                        %
                        txt = 'Would you like to save estimates of the state variables at initial parameter values to file?';
                        answer = Query(txt,'question',140,'Save - State Variables',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           StateVariableForecast = StateVarStructure.Ksitt1;
                           StateVariableUpdate = StateVarStructure.Ksitt;
                           StateVariableSmooth = StateVarStructure.KsitT;
                           %
                           txt = 'Would you also like to save the covariance matrices of the state variables based on the three estimators?';
                           covanswer = Query(txt,'question',140,'Save - State Variable Covariance Matrices',500,'no',CurrINI);
                           if strcmp(lower(covanswer),'yes')==1;
                              ForecastCovMatrix = StateVarStructure.Ptt1;
                              UpdateCovMatrix = StateVarStructure.Ptt;
                              SmoothCovMatrix = StateVarStructure.PtT;
                              CovStr = 'ForecastCovMatrix (covariance matrix based on the forecasted state variables), UpdateCovMatrix (covariance matrix based on the update estimates), SmoothCovMatrix (covariance matrix based on the smooth estimates), ';
                              AddEntry = 3;
                           else;
                              CovStr = '';
                              AddEntry = 0;
                           end;
                           %
                           if RecEst==1;
                              for i=1:length(StateVarStructure.recursive);
                                 RecursiveSmoothStateVariable(i).SmoothStateVariable = StateVarStructure.recursive(i).KsitT;
                              end;
                              InfoStr = 'RecursiveSmoothStateVariable (vector structure with recursive smooth estimates of the state variables), ';
                              NumEntries = 6+AddEntry;
                           else;
                              InfoStr = '';
                              NumEntries = 5+AddEntry;
                           end;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                           if length(DateVector)>size(StateVariableForecast,2);
                              DateVector = DateVector(1,length(DateVector)-size(StateVariableForecast,2)+1:length(DateVector));
                           end;
                           file = FixFilePath([DSGEModel.OutputDirectory '\StateVariables-InitialValues.mat']);
                           if AddEntry==0;
                              if RecEst==1;
                                 save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','RecursiveSmoothStateVariable','StateVariableNames','DateVector');
                              else;
                                 save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','StateVariableNames','DateVector');
                              end;
                           else;
                              if RecEst==1;
                                 save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','RecursiveSmoothStateVariable','ForecastCovMatrix','UpdateCovMatrix','SmoothCovMatrix','StateVariableNames','DateVector');
                              else;
                                 save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','ForecastCovMatrix','UpdateCovMatrix','SmoothCovMatrix','StateVariableNames','DateVector');
                              end;
                           end;
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SVInitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% matrix with estimates of the forecasted state variables\n%%\n');
                           fprintf(fid,'StateVariableForecast = DataStructure.StateVariableForecast;\n');
                           fprintf(fid,'%%\n%% matrix with estimates of the updated state variable projections\n%%\n');
                           fprintf(fid,'StateVariableUpdate = DataStructure.StateVariableUpdate;\n');
                           fprintf(fid,'%%\n%% matrix with estimates of the smoothed state variable projections\n%%\n');
                           fprintf(fid,'StateVariableSmooth = DataStructure.StateVariableSmooth;\n');
                           if RecEst==1;
                              fprintf(fid,'%%\n%% vector structure with recursive estimates of the smoothed state variable projections\n%%\n');
                              fprintf(fid,'RecursiveSmoothStateVariable = DataStructure.RecursiveSmoothStateVariable;\n');
                           end;
                           if AddEntry==3;
                              fprintf(fid,'%%\n%% matrix with covariances of the estimates of the forecasted state variables\n%%\n');
                              fprintf(fid,'ForecastCovMatrix = DataStructure.ForecastCovMatrix;\n');
                              fprintf(fid,'%%\n%% matrix with covariances of the estimates of the updated state variable projections\n%%\n');
                              fprintf(fid,'UpdateCovMatrix = DataStructure.UpdateCovMatrix;\n');
                              fprintf(fid,'%%\n%% matrix with covariances of the estimates of the smoothed state variable projections\n%%\n');
                              fprintf(fid,'SmoothCovMatrix = DataStructure.SmoothCovMatrix;\n');
                           end;
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['Estimates of the state variables at initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: StateVariableForecast (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the forecasted state variables), StateVariableUpdate (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the updated state variable projections), StateVariableSmooth (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the smoothed state variable projections), ' InfoStr CovStr 'StateVariableNames (string matrix with the names of the state variables), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','State Variables - Initial Values',500,600,CurrINI);
                        end;
                        %
                        % display the graphs
                        %
                        StateGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Initial Values','State');
                     end;
                  elseif (mcode==1)&(kalmanstatus~=0);
                     if kalmanstatus==1;
                        txt = 'The state variables are not stationary at the initial values.';
                     else;
                        txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                     end;
                     About(txt,'information','State Variables - Initial Values',200,500,CurrINI);
                  elseif mcode~=1;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                     About(txt,'information','State Variables - Initial Values',120,500,CurrINI);
                  end;
               else;
                  %
                  % run the simulation smoother
                  %
                  [StateVarStructure,TotalPaths,mcode,kalmanstatus] = DSGESimulationSmootherTheta(theta,thetaPositions,ModelParameters,1,'Initial Values',DSGEModel,controls,CurrINI);
                  if (mcode==1)&(kalmanstatus==0);
                     %
                     % we're OK
                     %
                     if isempty(StateVarStructure)==0;
                        %
                        % ask if we should save results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the state variable conditional distributions at initial values to file?';
                        answer = Query(txt,'question',140,'Save - State Variable Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCondDist-' int2str(TotalPaths) '-InitialValues.mat']);
                           save(file,'StateVarStructure');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SVCondDist' int2str(TotalPaths) 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields KsitT (smooth mean), PtT (population covariance matrix for smoother), Quantiles, DateVector and StateVariableNames.\n%%\n');
                           fprintf(fid,'StateVarStructure = DataStructure.StateVarStructure;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The state variable data at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: StateVarStructure (a structure with the mean and covariance matrix for the smooth estimator, quantile data, names of state variables, and numeric dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','State Variable Distribution',200,500,CurrINI);
                        end;
                        %
                        % display the results
                        %
                        StateVarDistributionDLG('init',StateVarStructure,2,1,0,[],DSGEModel,CurrINI);
                     end;
                  elseif (mcode==1)&(kalmanstatus~=0);
                     if kalmanstatus==1;
                        txt = 'The state variables are not stationary at the initial values.';
                     else;
                        txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                     end;
                     About(txt,'information','State Variable Distribution - Initial Values',200,500,CurrINI);
                  elseif mcode~=1;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                     About(txt,'information','State Variable Distribution - Initial Values',120,500,CurrINI);
                  end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVariablesPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'State Variable Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            txt = strvcat('Would you like to compute the distribution of the state variables conditional on the data (simulation smoothing) instead of just the distribution of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state variables conditional on all the data, and doesn''t cover the update and the forecast estimators.');
            answer = Query(txt,'question',300,'State Variables - Prior Distribution',600,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
               TotalPaths = max(TotalPaths,4);
            else;
               SimulSmooth = 0;
               TotalPaths = 0;
            end;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','State Variables - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','State Variable Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % First we check if the calculations have already been performed
               %
               if SVEExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,TotalPaths,0,0,SimulSmooth)==1;
                  txt = 'YADA has located results for the state variables on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'State Variable Distribution - Prior',450,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % start estimation
                  %
                  if SimulSmooth==0;
                     DoneCalc = DSGEStateVariableEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'State Variable',0,NumPriorDraws,1,DSGEModel,CurrINI);
                  else;
                     DoneCalc = DSGESimulationSmoother(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'State Variable',0,TotalPaths,NumPriorDraws,1,DSGEModel,controls,CurrINI);
                  end;
               end;
               %
               % check if we should show any results
               %
               if DoneCalc==1;
                  %
                  % compute the distribution for the state variables
                  %
                  if SimulSmooth==0;
                     SVEs = DSGESVEDistribution(DSGEModel,CurrINI,2,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                  else;
                     SVEs = DSGESimSmoothDistribution(DSGEModel,CurrINI,1,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,TotalPaths,0,[]);
                  end;
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the state variable distributions to file?';
                  answer = Query(txt,'question',140,'Save - State Variable Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                     if SimulSmooth==0;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateVariablePriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.mat']);
                     else;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableSimulSmoothPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.' int2str(TotalPaths) '.mat']);
                     end;
                     StateVariableNames = DSGEModel.StateVariableNames;
                     save(file,'SVEs','StateVariableNames');
                     %
                     % create a matlab file to read the data
                     %
                     if SimulSmooth==0;
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SVPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) '.m']);
                     else;
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SVSimSmoothPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) int2str(TotalPaths) '.m']);
                     end;
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     if SimulSmooth==0;
                        fprintf(fid,'%%\n%% structure with fields KsitT (smooth mean), Ksitt (update mean), Ksitt1 (forecast mean), Quantiles, and DateVector.\n%%\n');
                        SStr = 'forecast, the update and the smooth estimators';
                     else;
                        fprintf(fid,'%%\n%% structure with fields KsitT (smooth mean), Quantiles, and DateVector.\n%%\n');
                        SStr = 'smooth estimator';
                     end;
                     fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                     fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The state variables data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the mean of the ' SStr ', quantile data, and numeric dates), and StateVariableNames (string matrix with the names of the state variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','State Variable Distribution - Prior',200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  StateVarDistributionDLG('init',SVEs,2,0,0,[],DSGEModel,CurrINI);
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the state variable distribution.';
               About(txt,'information','State Variable Distribution - Prior',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'StateVariablesPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            if DSGEModel.RunDriver==0;
               if TotalPaths>1;
                  txt = strvcat('Would you like to compute the distribution of the state variables conditional on the data and the posterior mode parameter values (simulation smoothing) instead of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state variables conditional on all the data, and doesn''t cover the update and the forecast estimators.');
                  answer = Query(txt,'question',300,'State Variables - Posterior Mode',600,'no',CurrINI);
               else;
                  answer = 'No';
               end;
            else;
               if TotalPaths>1;
                  answer = DSGEModel.answers{1};
               else;
                  answer = 'No';
               end;
            end;
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
            else;
               SimulSmooth = 0;
            end;
            %
            if SimulSmooth==0;
               %
               % check if we should run recursive estimates
               %
               if DSGEModel.RunDriver==0;
                  txt = 'Would you also like to compute the smooth estimates recursively over the full estimation sample?';
                  answer = Query(txt,'question',200,'State Variables - Posterior Mode',500,'no',CurrINI);
               else;
                  answer = DSGEModel.answers{2};
               end;
               if strcmp(lower(answer),'yes')==1;
                  RecEst = 1;
               else;
                  RecEst = 0;
               end;
               %
               % we can now compute the state variables
               %
               StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,RecEst,'State Variables - Posterior Mode');
               if isempty(StateVarStructure)==0;
                  %
                  % ask if we should save to disk
                  %
                  if DSGEModel.RunDriver==0;
                     txt = 'Would you like to save estimates of the state variables at posterior mode to file?';
                     answer = Query(txt,'question',140,'Save - State Variables',500,'no',CurrINI);
                  else;
                     answer = DSGEModel.answers{3};
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     if DSGEModel.RunDriver==1;
                        SetLogText(strvcat('* Saving results to: ',['     ' DSGEModel.OutputDirectory]));
                     end;
                     StateVariableForecast = StateVarStructure.Ksitt1;
                     StateVariableUpdate = StateVarStructure.Ksitt;
                     StateVariableSmooth = StateVarStructure.KsitT;
                     %
                     if DSGEModel.RunDriver==0;
                        txt = 'Would you also like to save the covariance matrices of the state variables based on the three estimators?';
                        covanswer = Query(txt,'question',140,'Save - State Variable Covariance Matrices',500,'no',CurrINI);
                     else;
                        covanswer = DSGEModel.answers{4};
                     end;
                     if strcmp(lower(covanswer),'yes')==1;
                        ForecastCovMatrix = StateVarStructure.Ptt1;
                        UpdateCovMatrix = StateVarStructure.Ptt;
                        SmoothCovMatrix = StateVarStructure.PtT;
                        CovStr = 'ForecastCovMatrix (covariance matrix based on the forecasted state variables), UpdateCovMatrix (covariance matrix based on the update estimates), SmoothCovMatrix (covariance matrix based on the smooth estimates), ';
                        AddEntry = 3;
                     else;
                        CovStr = '';
                        AddEntry = 0;
                     end;
                     if RecEst==1;
                        for i=1:length(StateVarStructure.recursive);
                           RecursiveSmoothStateVariable(i).SmoothStateVariable = StateVarStructure.recursive(i).KsitT;
                        end;
                        InfoStr = 'RecursiveSmoothStateVariable (vector structure with recursive smooth estimates of the state variables), ';
                        NumEntries = 6+AddEntry;
                     else;
                        InfoStr = '';
                        NumEntries = 5+AddEntry;
                     end;
                     StateVariableNames = DSGEModel.StateVariableNames;
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     if length(DateVector)>size(StateVariableForecast,2);
                        DateVector = DateVector(1,length(DateVector)-size(StateVariableForecast,2)+1:length(DateVector));
                     end;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateVariables-PosteriorMode.mat']);
                     if AddEntry==0;
                        if RecEst==1;
                           save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','RecursiveSmoothStateVariable','StateVariableNames','DateVector');
                        else;
                           save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','StateVariableNames','DateVector');
                        end;
                     else;
                        if RecEst==1;
                           save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','RecursiveSmoothStateVariable','ForecastCovMatrix','UpdateCovMatrix','SmoothCovMatrix','StateVariableNames','DateVector');
                        else;
                           save(file,'StateVariableForecast','StateVariableUpdate','StateVariableSmooth','ForecastCovMatrix','UpdateCovMatrix','SmoothCovMatrix','StateVariableNames','DateVector');
                        end;
                     end;
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SVPostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% matrix with estimates of the forecasted state variables\n%%\n');
                     fprintf(fid,'StateVariableForecast = DataStructure.StateVariableForecast;\n');
                     fprintf(fid,'%%\n%% matrix with estimates of the updated state variable projections\n%%\n');
                     fprintf(fid,'StateVariableUpdate = DataStructure.StateVariableUpdate;\n');
                     fprintf(fid,'%%\n%% matrix with estimates of the smoothed state variable projections\n%%\n');
                     fprintf(fid,'StateVariableSmooth = DataStructure.StateVariableSmooth;\n');
                     if RecEst==1;
                        fprintf(fid,'%%\n%% vector structure with recursive estimates of the smoothed state variable projections\n%%\n');
                        fprintf(fid,'RecursiveSmoothStateVariable = DataStructure.RecursiveSmoothStateVariable;\n');
                     end;
                     if AddEntry==3;
                        fprintf(fid,'%%\n%% matrix with covariances of the estimates of the forecasted state variables\n%%\n');
                        fprintf(fid,'ForecastCovMatrix = DataStructure.ForecastCovMatrix;\n');
                        fprintf(fid,'%%\n%% matrix with covariances of the estimates of the updated state variable projections\n%%\n');
                        fprintf(fid,'UpdateCovMatrix = DataStructure.UpdateCovMatrix;\n');
                        fprintf(fid,'%%\n%% matrix with covariances of the estimates of the smoothed state variable projections\n%%\n');
                        fprintf(fid,'SmoothCovMatrix = DataStructure.SmoothCovMatrix;\n');
                     end;
                     fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                     fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     if DSGEModel.RunDriver==0;
                        txt = ['Estimates of the state variables at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: StateVariableForecast (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the forecasted state variables), StateVariableUpdate (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the updated state variable projections), StateVariableSmooth (' num2str(size(StateVariableForecast,1),'%0.0f') 'x' num2str(size(StateVariableForecast,2),'%0.0f') ' matrix with estimates of the smoothed state variable projections), ' InfoStr CovStr 'StateVariableNames (string matrix with the names of the state variables), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','State Variables - Posterior Mode',500,600,CurrINI);
                     end;
                  end;
                  %
                  % Display the graphs
                  %
                  if DSGEModel.RunDriver==0;
                     StateGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Posterior Mode','State');
                  else;
                     SetLogText(strvcat('* Finished calculations.',' '));
                  end;
               end;
            else;
               %
               % run the simulation smoother for state shocks
               %
               [StateVarStructure,TotalPaths] = DSGESimulationSmootherTheta(theta,thetaPositions,ModelParameters,1,'Posterior Mode',DSGEModel,controls,CurrINI);
               if isempty(StateVarStructure)==0;
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the state variable conditional distributions at posterior mode to file?';
                  answer = Query(txt,'question',140,'Save - State Variable Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableCondDist-' int2str(TotalPaths) '-PosteriorMode.mat']);
                     save(file,'StateVarStructure');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SVCondDist' int2str(TotalPaths) 'PostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% structure with fields KsitT (smooth mean), PtT (population covariance matrix for smoother), Quantiles, DateVector and StateVariableNames.\n%%\n');
                     fprintf(fid,'StateVarStructure = DataStructure.StateVarStructure;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The state variable data at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: StateVarStructure (a structure with the mean and covariance matrix for the smooth estimator, quantile data, names of state variables, and numeric dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','State Variable Distribution',200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  StateVarDistributionDLG('init',StateVarStructure,2,1,0,[],DSGEModel,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'StateVariablesPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            txt = strvcat('Would you like to compute the distribution of the state variables conditional on the data (simulation smoothing) instead of just the distribution of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state variables conditional on all the data, and doesn''t cover the update and the forecast estimators.');
            answer = Query(txt,'question',300,'State Variables - Posterior Distribution',600,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
               TotalPaths = max(TotalPaths,4);
            else;
               SimulSmooth = 0;
               TotalPaths = 0;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','State Variable Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     if TotalPaths==0;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        if length(UseDraws)<NumDraws;
                           thetaPostSample = thetaPostSample(UseDraws,:);
                           UsedDraws = length(UseDraws);
                        else;
                           UsedDraws = NumDraws;
                        end;
                     else;
                        %
                        % check how many draws we should pick among these
                        %
                        PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                        NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                        %
                        % get the draws to use
                        %
                        if DSGEModel.RandomizeDraws==0;
                           DrawFreq = round(NumDraws/NumPostDrawsUsage);
                           SelectDraws = (1:DrawFreq:NumDraws);
                           if length(SelectDraws)+1==NumPostDrawsUsage;
                              SelectDraws = [SelectDraws NumDraws];
                           elseif length(SelectDraws)>NumPostDrawsUsage;
                              SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                           end;
                           thetaPostSample = thetaPostSample(SelectDraws,:);
                        else;
                           thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                        end;
                        UsedDraws = size(thetaPostSample,1);
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','State Variable Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % First we check if the calculations have already been performed
                     %
                     if SVEExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,TotalPaths,0,1,SimulSmooth)==1;
                        txt = 'YADA has located results for the state variables on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'State Variable Distribution',450,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % start estimation
                        %
                        if SimulSmooth==0;
                           DoneCalc = DSGEStateVariableEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'State Variable',1,NumDraws,CurrChain,DSGEModel,CurrINI);
                        else;
                           DoneCalc = DSGESimulationSmoother(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'State Variable',1,TotalPaths,NumDraws,CurrChain,DSGEModel,controls,CurrINI);
                        end;
                     end;
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % compute the distribution for the state variables
                        %
                        if SimulSmooth==0;
                           SVEs = DSGESVEDistribution(DSGEModel,CurrINI,2,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights);
                        else;
                           SVEs = DSGESimSmoothDistribution(DSGEModel,CurrINI,1,ScenarioParameters,UsedDraws,NumDraws,CurrChain,TotalPaths,1,Weights);
                        end;
                        %
                        % ask if we should save results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the state variable distributions to file?';
                        answer = Query(txt,'question',140,'Save - State Variable Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                           if SimulSmooth==0;
                              file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                           else;
                              file = FixFilePath([DSGEModel.OutputDirectory '\StateVariableSimSmoothDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(TotalPaths) '-' int2str(CurrChain) '.mat']);
                           end;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           save(file,'SVEs','StateVariableNames');
                           %
                           % create a matlab file to read the data
                           %
                           if SimulSmooth==0;
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\SVDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                           else;
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\SVSimSmoothDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(TotalPaths) int2str(CurrChain) '.m']);
                           end;
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           if SimulSmooth==0;
                              fprintf(fid,'%%\n%% structure with fields KsitT (smooth mean), Ksitt (update mean), Ksitt1 (forecast mean), Quantiles, and DateVector.\n%%\n');
                              SStr = 'forecast, the update and the smooth estimators';
                           else;
                              fprintf(fid,'%%\n%% structure with fields KsitT (smooth mean), Quantiles, and DateVector.\n%%\n');
                              SStr = 'smooth estimator';
                           end;
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The state variables data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the mean of the ' SStr ', quantile data, and numeric dates), and StateVariableNames (string matrix with the names of the state variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','State Variable Distribution',200,500,CurrINI);
                        end;
                        %
                        % display the results
                        %
                        StateVarDistributionDLG('init',SVEs,2,1,0,[],DSGEModel,CurrINI);
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the state variable distribution.';
                     About(txt,'information','State Variable Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShocksInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the state shocks before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'State Shocks - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               NumPathsStr = get(controls.forecast.predpaths,'String');
               TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % check if we should use simulation smoother
               %
               if TotalPaths>1;
                  txt = strvcat('Would you like to compute the distribution of the state shocks conditional on the data and the initial parameter values (simulation smoothing) instead of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state shocks conditional on all the data, and doesn''t cover the update estimator.');
                  answer = Query(txt,'question',300,'State Shocks - Initial Values',600,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  SimulSmooth = 1;
               else;
                  SimulSmooth = 0;
               end;
               %
               if SimulSmooth==0;
                  %
                  % check if we should run the smooth estimates recursively
                  %
                  txt = 'Would you also like to compute the smooth estimates recursively over the full estimation sample?';
                  answer = Query(txt,'question',200,'State Shocks - Initial Values',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     RecEst = 1;
                  else;
                     RecEst = 0;
                  end;
                  %
                  % Try to calculate state variables. First, however, the code needs to check if a unique convergent
                  % solution exists.
                  %
                  [StateVarStructure,mcode,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,RecEst,'State Shocks - Initial Values');
                  if (mcode==1)&(kalmanstatus==0);
                     %
                     % we're OK
                     %
                     if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'etatT')==1)&(isempty(StateVarStructure.etatT)==0);
                        %
                        % ask if we should save to disk
                        %
                        txt = 'Would you like to save the smooth and update estimates of the state shocks at initial parameter values to file?';
                        answer = Query(txt,'question',140,'Save - State Shocks',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SmoothStateShocks = StateVarStructure.etatT;
                           UpdateStateShocks = StateVarStructure.etatt;
                           if RecEst==1;
                              for i=1:length(StateVarStructure.recursive);
                                 RecursiveSmoothStateShocks(i).SmoothStateShocks = StateVarStructure.recursive(i).etatT;
                              end;
                              InfoStr = 'RecursiveSmoothStateShocks (vector structure with recursive smooth estimates of the non-zero economic shocks), ';
                              NumEntries = 5;
                           else;
                              InfoStr = '';
                              NumEntries = 4;
                           end;
                           StateShockNames = DSGEModel.StateShockNames(StateVarStructure.KeepVar,:);
                           DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                           if length(DateVector)>size(SmoothStateShocks,2);
                              DateVector = DateVector(1,length(DateVector)-size(SmoothStateShocks,2)+1:length(DateVector));
                           end;
                           file = FixFilePath([DSGEModel.OutputDirectory '\StateShocks-InitialValues.mat']);
                           if RecEst==1;
                              save(file,'SmoothStateShocks','UpdateStateShocks','RecursiveSmoothStateShocks','StateShockNames','DateVector');
                           else;
                              save(file,'SmoothStateShocks','UpdateStateShocks','StateShockNames','DateVector');
                           end;
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SSInitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% matrix with the smooth estimates of the non-zero economic shocks\n%%\n');
                           fprintf(fid,'SmoothStateShocks = DataStructure.SmoothStateShocks;\n');
                           fprintf(fid,'%%\n%% matrix with the update estimates of the non-zero economic shocks\n%%\n');
                           fprintf(fid,'UpdateStateShocks = DataStructure.UpdateStateShocks;\n');
                           if RecEst==1;
                              fprintf(fid,'%%\n%% vector structure with recursive estimates of the smoothed state shocks\n%%\n');
                              fprintf(fid,'RecursiveSmoothStateShocks = DataStructure.RecursiveSmoothStateShocks;\n');
                           end;
                           fprintf(fid,'%%\n%% string matrix with the names of the non-zero economic shocks\n%%\n');
                           fprintf(fid,'StateShockNames = DataStructure.StateShockNames;\n');
                           fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The smooth and update estimates of the state shocks at initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: SmoothStateShocks and UpdateStateShocks (' num2str(size(SmoothStateShocks,1),'%0.0f') 'x' num2str(size(SmoothStateShocks,2),'%0.0f') ' matrix with the smooth and update estimates of the non-zero economic shocks), ' InfoStr 'StateShockNames (string matrix with the names of the non-zero economic shocks), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','State Shocks - Initial Values',200,500,CurrINI);
                        end;
                        %
                        % display the graphs
                        %
                        ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Initial Values','State Shock');
                     elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==1);
                        txt = 'YADA couldn''t estimate the state shocks. Please make sure that your model has state shocks!';
                        About(txt,'information','YADA - State Shocks',120,500,CurrINI);
                     end;
                  elseif (mcode==1)&(kalmanstatus~=0);
                     if kalmanstatus==1;
                        txt = 'The state variables are not stationary at the initial values.';
                     else;
                        txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                     end;
                     About(txt,'information','State Shocks - Initial Values',200,500,CurrINI);
                  elseif mcode~=1;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                     About(txt,'information','State Shocks - Initial Values',200,500,CurrINI);
                  end;
               else;
                  %
                  % run the simulation smoother for state shocks
                  %
                  [StateVarStructure,TotalPaths,mcode,kalmanstatus] = DSGESimulationSmootherTheta(theta,thetaPositions,ModelParameters,2,'Initial Values',DSGEModel,controls,CurrINI);
                  if (mcode==1)&(kalmanstatus==0);
                     %
                     % we're OK
                     %
                     if isempty(StateVarStructure)==0;
                        %
                        % ask if we should save results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the state shock conditional distributions at initial values to file?';
                        answer = Query(txt,'question',140,'Save - State Shock Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\StateShockCondDist-' int2str(TotalPaths) '-InitialValues.mat']);
                           save(file,'StateVarStructure');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\SSCondDist' int2str(TotalPaths) 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields etatT (smooth mean), Quantiles, DateVector and ShockNames.\n%%\n');
                           fprintf(fid,'StateVarStructure = DataStructure.StateVarStructure;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The state shock data at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: StateVarStructure (a structure with the mean for the smooth estimator, quantile data, names of shocks, and numeric dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','State Shock Distribution',200,500,CurrINI);
                        end;
                        %
                        % display the results
                        %
                        StateVarDistributionDLG('init',StateVarStructure,3,1,0,[],DSGEModel,CurrINI);
                     end;
                  elseif (mcode==1)&(kalmanstatus~=0);
                     if kalmanstatus==1;
                        txt = 'The state variables are not stationary at the initial values.';
                     else;
                        txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                     end;
                     About(txt,'information','State Shock Distribution - Initial Values',200,500,CurrINI);
                  elseif mcode~=1;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                     About(txt,'information','State Shock Distribution - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShocksPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'State Shock Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            txt = strvcat('Would you like to compute the distribution of the state shocks conditional on the data (simulation smoothing) instead of just the distribution of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state shocks conditional on all the data, and doesn''t cover the update estimator.');
            answer = Query(txt,'question',300,'State Shocks - Prior Distribution',600,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
               TotalPaths = max(TotalPaths,4);
            else;
               SimulSmooth = 0;
               TotalPaths = 0;
            end;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','State Shocks - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','State Shock Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % First we check if the calculations have already been performed
               %
               if SVEExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,TotalPaths,0,0,SimulSmooth)==1;
                  txt = 'YADA has located results for the state shocks on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'State Shock Distribution - Prior',450,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  DoneCalc = 1;
               else;
                  %
                  % start estimation
                  %
                  if SimulSmooth==0;
                     DoneCalc = DSGEStateVariableEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'State Shock',0,NumPriorDraws,1,DSGEModel,CurrINI);
                  else;
                     DoneCalc = DSGESimulationSmoother(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'State Shock',0,TotalPaths,NumPriorDraws,1,DSGEModel,controls,CurrINI);
                  end;
               end;
               %
               % check if we should show any results
               %
               if DoneCalc==1;
                  %
                  % compute the distribution for the state shocks
                  %
                  if SimulSmooth==0;
                     SVEs = DSGESVEDistribution(DSGEModel,CurrINI,3,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                  else;
                     SVEs = DSGESimSmoothDistribution(DSGEModel,CurrINI,2,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,TotalPaths,0,[]);
                  end;
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the state shock distributions to file?';
                  answer = Query(txt,'question',140,'Save - State Shock Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                     if SimulSmooth==0;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateShockPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.mat']);
                     else;
                        file = FixFilePath([DSGEModel.OutputDirectory '\StateShockSimulSmoothPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.' int2str(TotalPaths) '.mat']);
                     end;
                     save(file,'SVEs');
                     %
                     % create a matlab file to read the data
                     %
                     if SimulSmooth==0;
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SSPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) '.m']);
                     else;
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\SSSimSmoothPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) int2str(TotalPaths) '.m']);
                     end;
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     if SimulSmooth==0;
                        fprintf(fid,'%%\n%% structure with fields etatT (smooth mean), etatt (update mean), Quantiles, DateVector and ShockNames.\n%%\n');
                        SStr = 'update and the smooth estimators';
                     else;
                        fprintf(fid,'%%\n%% structure with fields etatT (smooth mean), Quantiles, DateVector and ShockNames.\n%%\n');
                        SStr = 'smooth estimator';
                     end;
                     fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The state shock data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: SVEs (a structure with the mean of the ' SStr ', quantile data, names of shocks, and numeric dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','State Shock Distribution - Prior',200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  StateVarDistributionDLG('init',SVEs,3,0,0,[],DSGEModel,CurrINI);
               else;
                  DSGEModel = OldDSGEModel;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the state shock distribution.';
               About(txt,'information','State Shock Distribution - Prior',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShocksPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            if DSGEModel.RunDriver==0;
               if TotalPaths>1;
                  txt = strvcat('Would you like to compute the distribution of the state shocks conditional on the data and the posterior mode parameter values (simulation smoothing) instead of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state shocks conditional on all the data, and doesn''t cover the update estimator.');
                  answer = Query(txt,'question',300,'State Shocks - Posterior Mode',600,'no',CurrINI);
               else;
                  answer = 'No';
               end;
            else;
               if TotalPaths>1;
                  answer = DSGEModel.answers{1};
               else;
                  answer = 'No';
               end;
            end;
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
            else;
               SimulSmooth = 0;
            end;
            %
            % we can now calculate the state shocks
            %
            if SimulSmooth==0;
               %
               % check if we should run recursive estimates
               %
               if DSGEModel.RunDriver==0;
                  txt = 'Would you also like to compute the smooth estimates recursively over the full estimation sample?';
                  answer = Query(txt,'question',200,'State Shocks - Posterior Mode',500,'no',CurrINI);
               else;
                  answer = DSGEModel.answers{2};
               end;
               if strcmp(lower(answer),'yes')==1;
                  RecEst = 1;
               else;
                  RecEst = 0;
               end;
               StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,RecEst,'State Shocks - Posterior Mode');
               if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'etatT')==1)&(isempty(StateVarStructure.etatT)==0);
                  %
                  % ask if we should save to disk
                  %
                  if DSGEModel.RunDriver==0;
                     txt = 'Would you like to save the smooth and update estimates of the state shocks at posterior mode to file?';
                     answer = Query(txt,'question',140,'Save - State Shocks',500,'no',CurrINI);
                  else;
                     answer = DSGEModel.answers{3};
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     if DSGEModel.RunDriver==1;
                        SetLogText(strvcat('* Saving results to: ',['     ' DSGEModel.OutputDirectory]));
                     end;
                     SmoothStateShocks = StateVarStructure.etatT;
                     UpdateStateShocks = StateVarStructure.etatt;
                     if RecEst==1;
                        for i=1:length(StateVarStructure.recursive);
                           RecursiveSmoothStateShocks(i).SmoothStateShocks = StateVarStructure.recursive(i).etatT;
                        end;
                        InfoStr = 'RecursiveSmoothStateShocks (vector structure with recursive smooth estimates of the non-zero economic shocks), ';
                        NumEntries = 5;
                     else;
                        InfoStr = '';
                        NumEntries = 4;
                     end;
                     StateShockNames = DSGEModel.StateShockNames(StateVarStructure.KeepVar,:);
                     DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                     if length(DateVector)>size(SmoothStateShocks,2);
                        DateVector = DateVector(1,length(DateVector)-size(SmoothStateShocks,2)+1:length(DateVector));
                     end;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateShocks-PosteriorMode.mat']);
                     if RecEst==1;
                        save(file,'SmoothStateShocks','UpdateStateShocks','RecursiveSmoothStateShocks','StateShockNames','DateVector');
                     else;
                        save(file,'SmoothStateShocks','UpdateStateShocks','StateShockNames','DateVector');
                     end;
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SSPostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% matrix with the smooth estimates of the non-zero economic shocks\n%%\n');
                     fprintf(fid,'SmoothStateShocks = DataStructure.SmoothStateShocks;\n');
                     fprintf(fid,'%%\n%% matrix with the update estimates of the non-zero economic shocks\n%%\n');
                     fprintf(fid,'UpdateStateShocks = DataStructure.UpdateStateShocks;\n');
                     if RecEst==1;
                        fprintf(fid,'%%\n%% vector structure with recursive estimates of the smoothed state shocks\n%%\n');
                        fprintf(fid,'RecursiveSmoothStateShocks = DataStructure.RecursiveSmoothStateShocks;\n');
                     end;
                     fprintf(fid,'%%\n%% string matrix with the names of the non-zero economic shocks\n%%\n');
                     fprintf(fid,'StateShockNames = DataStructure.StateShockNames;\n');
                     fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                     fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     if DSGEModel.RunDriver==0;
                        txt = ['The smooth and update estimates of the state shocks at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: SmoothStateShocks and UpdateStateShocks (' num2str(size(SmoothStateShocks,1),'%0.0f') 'x' num2str(size(SmoothStateShocks,2),'%0.0f') ' matrix with the smooth and update estimates of the non-zero economic shocks), ' InfoStr 'StateShockNames (string matrix with the names of the non-zero economic shocks), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','State Shocks - Posterior Mode',200,500,CurrINI);
                     end;
                  end;
                  %
                  % Display the graphs
                  %
                  if DSGEModel.RunDriver==0;
                     ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Posterior Mode','State Shock');
                  else;
                     SetLogText(strvcat('* Finished calculations.',' '));
                  end;
               elseif (isempty(StateVarStructure)==0)&(isempty(StateVarStructure.etatT)==1);
                  txt = 'YADA couldn''t estimate the state shocks. Please make sure that your model has state shocks!';
                  About(txt,'information','YADA - State Shocks',120,500,CurrINI);
               end;
            else;
               %
               % run the simulation smoother for state shocks
               %
               [StateVarStructure,TotalPaths] = DSGESimulationSmootherTheta(theta,thetaPositions,ModelParameters,2,'Posterior Mode',DSGEModel,controls,CurrINI);
               if isempty(StateVarStructure)==0;
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the state shock conditional distributions at posterior mode to file?';
                  answer = Query(txt,'question',140,'Save - State Shock Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\StateShockCondDist-' int2str(TotalPaths) '-PosteriorMode.mat']);
                     save(file,'StateVarStructure');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\SSCondDist' int2str(TotalPaths) 'PostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% structure with fields etatT (smooth mean), Quantiles, DateVector and ShockNames.\n%%\n');
                     fprintf(fid,'StateVarStructure = DataStructure.StateVarStructure;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The state shock data at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: StateVarStructure (a structure with the mean for the smooth estimator, quantile data, names of shocks, and numeric dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','State Shock Distribution',200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  StateVarDistributionDLG('init',StateVarStructure,3,1,0,[],DSGEModel,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ShocksPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            txt = strvcat('Would you like to compute the distribution of the state shocks conditional on the data (simulation smoothing) instead of just the distribution of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state shocks conditional on all the data, and doesn''t cover the update estimator.');
            answer = Query(txt,'question',300,'State Shocks - Posterior Distribution',600,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
               TotalPaths = max(TotalPaths,4);
            else;
               SimulSmooth = 0;
               TotalPaths = 0;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','State Shock Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     if TotalPaths==0;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        if length(UseDraws)<NumDraws;
                           thetaPostSample = thetaPostSample(UseDraws,:);
                           UsedDraws = length(UseDraws);
                        else;
                           UsedDraws = NumDraws;
                        end;
                     else;
                        %
                        % check how many draws we should pick among these
                        %
                        PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                        NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                        %
                        % get the draws to use
                        %
                        if DSGEModel.RandomizeDraws==0;
                           DrawFreq = round(NumDraws/NumPostDrawsUsage);
                           SelectDraws = (1:DrawFreq:NumDraws);
                           if length(SelectDraws)+1==NumPostDrawsUsage;
                              SelectDraws = [SelectDraws NumDraws];
                           elseif length(SelectDraws)>NumPostDrawsUsage;
                              SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                           end;
                           thetaPostSample = thetaPostSample(SelectDraws,:);
                        else;
                           thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                        end;
                        UsedDraws = size(thetaPostSample,1);
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','State Shock Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % First we check if the calculations have already been performed
                     %
                     if SVEExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,TotalPaths,0,1,SimulSmooth)==1;
                        txt = 'YADA has located results for the state shocks on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'State Shock Distribution',450,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DoneCalc = 1;
                     else;
                        %
                        % start estimation
                        %
                        if SimulSmooth==0;
                           DoneCalc = DSGEStateVariableEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'State Shock',1,NumDraws,CurrChain,DSGEModel,CurrINI);
                        else;
                           DoneCalc = DSGESimulationSmoother(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'State Shock',1,TotalPaths,NumDraws,CurrChain,DSGEModel,controls,CurrINI);
                        end;
                     end;
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % compute the distribution for the state shocks
                        %
                        if SimulSmooth==0;
                           SVEs = DSGESVEDistribution(DSGEModel,CurrINI,3,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights);
                        else;
                           SVEs = DSGESimSmoothDistribution(DSGEModel,CurrINI,2,ScenarioParameters,UsedDraws,NumDraws,CurrChain,TotalPaths,1,Weights);
                        end;
                        %
                        % ask if we should save results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the state shock distributions to file?';
                        answer = Query(txt,'question',140,'Save - State Shock Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                           if SimulSmooth==0;
                              file = FixFilePath([DSGEModel.OutputDirectory '\StateShockDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                           else;
                              file = FixFilePath([DSGEModel.OutputDirectory '\StateShockSimulSmoothDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(TotalPaths) '-' int2str(CurrChain) '.mat']);
                           end;
                           save(file,'SVEs');
                           %
                           % create a matlab file to read the data
                           %
                           if SimulSmooth==0;
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\SSDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                           else;
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\SSSimSmoothDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(TotalPaths) int2str(CurrChain) '.m']);
                           end;
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           if SimulSmooth==0;
                              fprintf(fid,'%%\n%% structure with fields etatT (smooth mean), etatt (update mean), Quantiles, DateVector and ShockNames.\n%%\n');
                              SStr = 'update and the smooth estimators';
                           else;
                              fprintf(fid,'%%\n%% structure with fields etatT (smooth mean), Quantiles, DateVector and ShockNames.\n%%\n');
                              SStr = 'smooth estimator';
                           end;
                           fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The state shock data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: SVEs (a structure with the mean of the ' SStr ', quantile data, names of shocks, and numeric dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','State Shock Distribution',200,500,CurrINI);
                        end;
                        %
                        % display the results
                        %
                        StateVarDistributionDLG('init',SVEs,3,1,0,[],DSGEModel,CurrINI);
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the state shock distribution.';
                     About(txt,'information','State Shock Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MEInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the measurement errors before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Measurement Errors - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               NumPathsStr = get(controls.forecast.predpaths,'String');
               TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               %
               % check if we should use simulation smoother
               %
               if TotalPaths>1;
                  txt = strvcat('Would you like to compute the distribution of the measurement errors conditional on the data and the initial parameter values (simulation smoothing) instead of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the measurement errors conditional on all the data, and doesn''t cover the update estimator.');
                  answer = Query(txt,'question',300,'Measurement Errors - Initial Values',600,'no',CurrINI);
               else;
                  answer = 'No';
               end;
               if strcmp(lower(answer),'yes')==1;
                  SimulSmooth = 1;
               else;
                  SimulSmooth = 0;
               end;
               %
               if SimulSmooth==0;
                  %
                  % check if we should run recursive estimates
                  %
                  txt = 'Would you also like to compute the smooth estimates recursively over the full estimation sample?';
                  answer = Query(txt,'question',200,'Measurement Errors - Initial Values',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     RecEst = 1;
                  else;
                     RecEst = 0;
                  end;
                  %
                  % Try to calculate state variables. First, however, the code needs to check if a unique convergent
                  % solution exists.
                  %
                  [StateVarStructure,mcode,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,RecEst,'Measurement Errors - Initial Values');
                  if (mcode==1)&(kalmanstatus==0);
                     %
                     % we're OK
                     %
                     if (isfield(StateVarStructure,'wtT')==1)&(isempty(StateVarStructure.wtT)==0);
                        %
                        % ask if we should save to disk
                        %
                        txt = 'Would you like to save smooth and update estimates of the measurement errors at initial parameter values to file?';
                        answer = Query(txt,'question',140,'Save - Measurement Errors',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           SmoothMeasurementError = StateVarStructure.wtT;
                           UpdateMeasurementError = StateVarStructure.wtt;
                           if RecEst==1;
                              for i=1:length(StateVarStructure.recursive);
                                 RecursiveSmoothMeasurementError(i).SmoothMeasurementError = StateVarStructure.recursive(i).etatT;
                              end;
                              InfoStr = 'RecursiveSmoothMeasurementError (vector structure with recursive smooth estimates of the measurement errors), ';
                              NumEntries = 5;
                           else;
                              InfoStr = '';
                              NumEntries = 4;
                           end;
                           MeasurementErrorNames = StateVarStructure.wtNames;
                           DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                           if length(DateVector)>size(SmoothMeasurementError,2);
                              DateVector = DateVector(1,length(DateVector)-size(SmoothMeasurementError,2)+1:length(DateVector));
                           end;
                           file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrors-InitialValues.mat']);
                           if RecEst==1;
                              save(file,'SmoothMeasurementError','UpdateMeasurementError','RecursiveSmoothMeasurementError','MeasurementErrorNames','DateVector');
                           else;
                              save(file,'SmoothMeasurementError','UpdateMeasurementError','MeasurementErrorNames','DateVector');
                           end;
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\MEInitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% smoothed estimates of the non-zero measurement errors\n%%\n');
                           fprintf(fid,'SmoothMeasurementError = DataStructure.SmoothMeasurementError;\n');
                           fprintf(fid,'%%\n%% update estimates of the non-zero measurement errors\n%%\n');
                           fprintf(fid,'UpdateMeasurementError = DataStructure.UpdateMeasurementError;\n');
                           if RecEst==1;
                              fprintf(fid,'%%\n%% vector structure with recursive estimates of the smoothed measurement errors\n%%\n');
                              fprintf(fid,'RecursiveSmoothMeasurementError = DataStructure.RecursiveSmoothMeasurementError;\n');
                           end;
                           fprintf(fid,'%%\n%% string matrix with the names of the non-zero measurement errors\n%%\n');
                           fprintf(fid,'MeasurementErrorNames = DataStructure.MeasurementErrorNames;\n');
                           fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['Smooth and update estimates of the measurement errors at initial parameter values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: SmoothMeasurementError and UpdateMeasurementError (' num2str(size(SmoothMeasurementError,1),'%0.0f') 'x' num2str(size(SmoothMeasurementError,2),'%0.0f') ' matrix with smoothed and update estimates of the non-zero measurement errors), ' InfoStr 'MeasurementErrorNames (string matrix with the names of the non-zero measurement errors), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Measurement Errors - Initial Values',200,500,CurrINI);
                        end;
                        %
                        % Display the graphs
                        %
                        ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Initial Values','Measurement Error');
                     else;
                        txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                        About(txt,'information','Measurement Errors - Initial Values',120,500,CurrINI);
                     end;
                  elseif (mcode==1)&(kalmanstatus~=0);
                     if kalmanstatus==1;
                        txt = 'The state variables are not stationary at the initial values.';
                     else;
                        txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                     end;
                     About(txt,'information','Measurement Errors - Initial Values',200,500,CurrINI);
                  elseif mcode~=1;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                     About(txt,'information','Measurement Errors - Initial Values',200,500,CurrINI);
                  end;
               else;
                  %
                  % run the simulation smoother
                  %
                  [StateVarStructure,TotalPaths,mcode,kalmanstatus] = DSGESimulationSmootherTheta(theta,thetaPositions,ModelParameters,3,'Initial Values',DSGEModel,controls,CurrINI);
                  if (mcode==1)&(kalmanstatus==0);
                     %
                     % we're OK
                     %
                     if isempty(StateVarStructure)==0;
                        %
                        % ask if we should save results to disk
                        %
                        txt = 'Would you like to save the mean and quantiles of the conditional distributions of the measurement errors at initial values to file?';
                        answer = Query(txt,'question',140,'Save - Measurement Error Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           file = FixFilePath([DSGEModel.OutputDirectory '\MeaxsureErrorCondDist-' int2str(TotalPaths) '-InitialValues.mat']);
                           save(file,'StateVarStructure');
                           %
                           % create a matlab file to read the data
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\MECondDist' int2str(TotalPaths) 'InitVal.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% structure with fields wtT (smooth mean), Quantiles, DateVector, VariableNames and ErrorIndicator.\n%%\n');
                           fprintf(fid,'StateVarStructure = DataStructure.StateVarStructure;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           txt = ['The measurement error data at initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: StateVarStructure (a structure with the mean for the smooth estimator, quantile data, indicator of equations with measurement errors, names of observed variables, and numeric dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Measurement Error Distribution',200,500,CurrINI);
                        end;
                        %
                        % display the results
                        %
                        StateVarDistributionDLG('init',StateVarStructure,4,1,0,[],DSGEModel,CurrINI);
                     end;
                  elseif (mcode==1)&(kalmanstatus~=0);
                     if kalmanstatus==1;
                        txt = 'The state variables are not stationary at the initial values.';
                     else;
                        txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                     end;
                     About(txt,'information','Measurement Error Distribution - Initial Values',200,500,CurrINI);
                  elseif mcode~=1;
                     if mcode==2;
                        mcodeStr = 'Roots are not correctly computed by real_schur.'
                     elseif mcode==3;
                        mcodeStr = 'Too many big roots.';
                     elseif mcode==35;
                        mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                     elseif mcode==4;
                        mcodeStr = 'Too few big roots.';
                     elseif mcode==45;
                        mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                     elseif mcode==5;
                        mcodeStr = 'q(:,right) is singular.';
                     elseif mcode==61;
                        mcodeStr = 'Too many exact shiftrights.';
                     elseif mcode==62;
                        mcodeStr = 'Too many numeric shiftrights.';
                     elseif mcode==7;
                        mcodeStr = 'Infinite or NaN values detected.';
                     elseif mcode==8;
                        mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                     elseif mcode==-1;
                        mcodeStr = 'No stable solution.';
                     elseif mcode==-2;
                        mcodeStr = 'Too many large generalized eigenvalues.';
                     elseif mcode==-3;
                        mcodeStr = 'Too few large generalized eigenvalues.';
                     else;
                        mcodeStr = 'Return code not properly specified.';
                     end;
                     if DSGEModel.ModelSolver==1;
                        SolverStr = 'AiM';
                     elseif DSGEModel.ModelSolver==2;
                        SolverStr = 'Klein';
                     elseif DSGEModel.ModelSolver==3;
                        SolverStr = 'Gensys';
                     elseif DSGEModel.ModelSolver==4;
                        SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                     end;
                     %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                     txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                     About(txt,'information','Measurement Error Distribution - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MEPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Measurement Error Distribution - Prior',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            txt = strvcat('Would you like to compute the distribution of the measurement errors conditional on the data (simulation smoothing) instead of just the distribution of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the state shocks conditional on all the data, and doesn''t cover the update estimator.');
            answer = Query(txt,'question',300,'Measurement Errors - Prior Distribution',600,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
               TotalPaths = max(TotalPaths,4);
            else;
               SimulSmooth = 0;
               TotalPaths = 0;
            end;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','Measurement Errors - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % check which parameters should be changed
            %
            if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
               ScenarioParameters = ones(1,length(theta));
            else;
               ScenarioParameters = DSGEModel.ScenarioParameters;
            end;
            positions = ScenarioParameters .* (1:length(theta));
            %
            % remove all zeros from positions
            %
            positions = positions(positions>0);
            %
            [PosAction,positions] = SelectCondVarShockDLG('init','Measurement Error Distribution','Parameters For',ParameterNames.estimated,positions,CurrINI);
            if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
               %
               % rewrite positions into DSGEModel.ScenarioParameters
               %
               ScenarioParameters = zeros(1,length(theta));
               ScenarioParameters(positions) = ones(1,length(positions));
               %
               % First we check if the calculations have already been performed
               %
               [Stat,MeasurementErrorStatus] = SVEExist(DSGEModel,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,TotalPaths,0,0,SimulSmooth);
               if (Stat==1)&(MeasurementErrorStatus==1);
                  txt = 'YADA has located results for the measurement errors on disk. Would you like to use these results?';
                  answer = Query(txt,'question',150,'Measurement Error Distribution - Prior',450,'no',CurrINI);
                  DoCalc = 1;
               elseif (Stat==1)&(MeasurementErrorStatus==0);
                  txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                  About(txt,'information','Measurement Error Distribution - Prior',180,500,CurrINI);
                  DoCalc = 0;
               else;
                  answer = 'No';
                  DoCalc = 1;
               end;
               %
               if DoCalc==1;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % start estimation
                     %
                     if SimulSmooth==0;
                        DoneCalc = DSGEStateVariableEstimation(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'Measurement Error',0,NumPriorDraws,1,DSGEModel,CurrINI);
                     else;
                        DoneCalc = DSGESimulationSmoother(theta,thetaDraws',thetaPositions,ModelParameters,ScenarioParameters,'Measurement Error',0,TotalPaths,NumPriorDraws,1,DSGEModel,controls,CurrINI);
                     end;
                  end;
                  %
                  % check if we should show any results
                  %
                  if DoneCalc==1;
                     %
                     % compute the distribution for measurement errors
                     %
                     if SimulSmooth==0;
                        SVEs = DSGESVEDistribution(DSGEModel,CurrINI,4,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,0,[]);
                     else;
                        SVEs = DSGESimSmoothDistribution(DSGEModel,CurrINI,3,ScenarioParameters,NumPriorDraws,NumPriorDraws,1,TotalPaths,0,[]);
                     end;
                     %
                     % ask if we should save results to disk
                     %
                     txt = 'Would you like to save the mean and quantiles of the measurement error distributions to file?';
                     answer = Query(txt,'question',140,'Save - Measurement Error Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                        if SimulSmooth==0;
                           file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.mat']);
                        else;
                           file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorSimulSmoothPriorDist-' SelParamStr '-' int2str(NumPriorDraws) '.' int2str(TotalPaths) '.mat']);
                        end;
                        VariableNames = DSGEModel.VariableNames;
                        save(file,'SVEs','VariableNames');
                        %
                        % create a matlab file to read the data
                        %
                        if SimulSmooth==0;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\MEPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) '.m']);
                        else;
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\MESimSmoothPriorDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(NumPriorDraws) int2str(TotalPaths) '.m']);
                        end;
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        if SimulSmooth==0;
                           fprintf(fid,'%%\n%% structure with fields wtT (smooth mean), wtt (update mean), Quantiles, DateVector and ErrorIndicator.\n%%\n');
                           SStr = 'update and the smooth estimators';
                        else;
                           fprintf(fid,'%%\n%% structure with fields wtT (smooth mean), Quantiles, DateVector and ErrorIndicator.\n%%\n');
                           SStr = 'smooth estimator';
                        end;
                        fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the variables\n%%\n');
                        fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The measurement error data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the mean of the ' SStr ', quantile data, indicator which functions have non-zero measurement errors, and numeric dates), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Measurement Error Distribution - Prior',200,500,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     StateVarDistributionDLG('init',SVEs,4,0,0,[],DSGEModel,CurrINI);
                  else;
                     DSGEModel = OldDSGEModel;
                  end;
               end;
            elseif isempty(positions)==1;
               txt = 'You didn''t select any parameters that should be varied for the estimation of the measurement error distribution.';
               About(txt,'information','Measurement Error Distribution - Prior',150,500,CurrINI);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MEPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            if TotalPaths>1;
               txt = strvcat('Would you like to compute the distribution of the measurement errors conditional on the data and the posterior mode parameter values (simulation smoothing) instead of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the measurement errors conditional on all the data, and doesn''t cover the update estimator.');
               answer = Query(txt,'question',300,'Measurement Errors - Posterior Mode',600,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
            else;
               SimulSmooth = 0;
            end;
            %
            if SimulSmooth==0;
               %
               % check if we should run recursive estimates
               %
               txt = 'Would you also like to compute the smooth estimates recursively over the full estimation sample?';
               answer = Query(txt,'question',200,'Measurement Errors - Posterior Mode',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  RecEst = 1;
               else;
                  RecEst = 0;
               end;
               %
               % we can now calculate the measurement errors
               %
               StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,RecEst,'Measurement Errors - Posterior Mode');
               if isempty(StateVarStructure)==0;
                  if (isfield(StateVarStructure,'wtT')==1)&(isempty(StateVarStructure.wtT)==0);
                     %
                     % ask if we should save to disk
                     %
                     txt = 'Would you like to save smooth and update estimates of the measurement errors at posterior mode to file?';
                     answer = Query(txt,'question',140,'Save - Measurement Errors',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        SmoothMeasurementError = StateVarStructure.wtT;
                        UpdateMeasurementError = StateVarStructure.wtt;
                        if RecEst==1;
                           for i=1:length(StateVarStructure.recursive);
                              RecursiveSmoothMeasurementError(i).SmoothMeasurementError = StateVarStructure.recursive(i).etatT;
                           end;
                           InfoStr = 'RecursiveSmoothMeasurementError (vector structure with recursive smooth estimates of the measurement errors), ';
                           NumEntries = 5;
                        else;
                           InfoStr = '';
                           NumEntries = 4;
                        end;
                        MeasurementErrorNames = StateVarStructure.wtNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                        if length(DateVector)>size(SmoothMeasurementError,2);
                           DateVector = DateVector(1,length(DateVector)-size(SmoothMeasurementError,2)+1:length(DateVector));
                        end;
                        file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrors-PosteriorMode.mat']);
                        if RecEst==1;
                           save(file,'SmoothMeasurementError','UpdateMeasurementError','RecursiveSmoothMeasurementError','MeasurementErrorNames','DateVector');
                        else;
                           save(file,'SmoothMeasurementError','UpdateMeasurementError','MeasurementErrorNames','DateVector');
                        end;
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\MEPostMode.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% smoothed estimates of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'SmoothMeasurementError = DataStructure.SmoothMeasurementError;\n');
                        fprintf(fid,'%%\n%% update estimates of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'UpdateMeasurementError = DataStructure.UpdateMeasurementError;\n');
                        if RecEst==1;
                           fprintf(fid,'%%\n%% vector structure with recursive estimates of the smoothed measurement errors\n%%\n');
                           fprintf(fid,'RecursiveSmoothMeasurementError = DataStructure.RecursiveSmoothMeasurementError;\n');
                        end;
                        fprintf(fid,'%%\n%% string matrix with the names of the non-zero measurement errors\n%%\n');
                        fprintf(fid,'MeasurementErrorNames = DataStructure.MeasurementErrorNames;\n');
                        fprintf(fid,'%%\n%% vector with the dates\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['Smooth and update estimates of the measurement errors at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' int2str(NumEntries) ' entries: SmoothMeasurementError and UpdateMeasurementError (' num2str(size(SmoothMeasurementError,1),'%0.0f') 'x' num2str(size(SmoothMeasurementError,2),'%0.0f') ' matrix with smoothed and update estimates of the non-zero measurement errors), ' InfoStr 'MeasurementErrorNames (string matrix with the names of the non-zero measurement errors), and DateVector (1x' num2str(length(DateVector),'%0.0f') ' vector with the dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Measurement Errors - Posterior Mode',200,500,CurrINI);
                     end;
                     %
                     % Display the graphs
                     %
                     ShockGraphDLG('init',CurrINI,DSGEModel,StateVarStructure,'Posterior Mode','Measurement Error');
                  else;
                     txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                     About(txt,'information','Measurement Errors - Posterior Mode',120,500,CurrINI);
                  end;
               end;
            else;
               %
               % run the simulation smoother
               %
               [StateVarStructure,TotalPaths] = DSGESimulationSmootherTheta(theta,thetaPositions,ModelParameters,3,'Posterior Mode',DSGEModel,controls,CurrINI);
               if isempty(StateVarStructure)==0;
                  %
                  % ask if we should save results to disk
                  %
                  txt = 'Would you like to save the mean and quantiles of the conditional distributions of the measurement errors at posterior mode to file?';
                  answer = Query(txt,'question',140,'Save - Measurement Error Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     file = FixFilePath([DSGEModel.OutputDirectory '\MeaxsureErrorCondDist-' int2str(TotalPaths) '-PosteriorMode.mat']);
                     save(file,'StateVarStructure');
                     %
                     % create a matlab file to read the data
                     %
                     mfile = FixFilePath([DSGEModel.OutputDirectory '\MECondDist' int2str(TotalPaths) 'PostMode.m']);
                     fid = fopen(mfile,'wt');
                     fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                     fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                     fprintf(fid,'%%\n%% structure with fields wtT (smooth mean), Quantiles, DateVector, VariableNames and ErrorIndicator.\n%%\n');
                     fprintf(fid,'StateVarStructure = DataStructure.StateVarStructure;\n');
                     fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                     %
                     % end the script file
                     %
                     fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                     fclose(fid);
                     txt = ['The measurement error data at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 1 entry: StateVarStructure (a structure with the mean for the smooth estimator, quantile data, indicator of equations with measurement errors, names of observed variables, and numeric dates). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                     About(txt,'information','Measurement Error Distribution',200,500,CurrINI);
                  end;
                  %
                  % display the results
                  %
                  StateVarDistributionDLG('init',StateVarStructure,4,1,0,[],DSGEModel,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'MEPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = ModeData.ParameterNames.estimated;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            NumPathsStr = get(controls.forecast.predpaths,'String');
            TotalPaths = str2num(strrep(strrep(strrep(StringTrim(NumPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            %
            % check if we should use simulation smoother
            %
            txt = strvcat('Would you like to compute the distribution of the measurement errors conditional on the data (simulation smoothing) instead of just the distribution of the point estimators?',' ','NOTE: Simulation smoothing considers the distribution of the measurement errors conditional on all the data, and doesn''t cover the update estimator.');
            answer = Query(txt,'question',300,'Measurement Errors - Posterior Distribution',600,'no',CurrINI);
            if strcmp(lower(answer),'yes')==1;
               SimulSmooth = 1;
               TotalPaths = max(TotalPaths,4);
            else;
               SimulSmooth = 0;
               TotalPaths = 0;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Measurement Error Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     if TotalPaths==0;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        if length(UseDraws)<NumDraws;
                           thetaPostSample = thetaPostSample(UseDraws,:);
                           UsedDraws = length(UseDraws);
                        else;
                           UsedDraws = NumDraws;
                        end;
                     else;
                        %
                        % check how many draws we should pick among these
                        %
                        PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                        NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                        %
                        % get the draws to use
                        %
                        if DSGEModel.RandomizeDraws==0;
                           DrawFreq = round(NumDraws/NumPostDrawsUsage);
                           SelectDraws = (1:DrawFreq:NumDraws);
                           if length(SelectDraws)+1==NumPostDrawsUsage;
                              SelectDraws = [SelectDraws NumDraws];
                           elseif length(SelectDraws)>NumPostDrawsUsage;
                              SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                           end;
                           thetaPostSample = thetaPostSample(SelectDraws,:);
                        else;
                           thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                        end;
                        UsedDraws = size(thetaPostSample,1);
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % check which parameters should be changed
                  %
                  if (isempty(DSGEModel.ScenarioParameters)==1)|(length(DSGEModel.ScenarioParameters)~=length(theta));
                     ScenarioParameters = ones(1,length(theta));
                  else;
                     ScenarioParameters = DSGEModel.ScenarioParameters;
                  end;
                  positions = ScenarioParameters .* (1:length(theta));
                  %
                  % remove all zeros from positions
                  %
                  positions = positions(positions>0);
                  %
                  [PosAction,positions] = SelectCondVarShockDLG('init','Measurement Error Distribution','Parameters For',ParameterNames,positions,CurrINI);
                  if (strcmp(lower(PosAction),'ok')==1)&(isempty(positions)==0);
                     %
                     % rewrite positions into DSGEModel.ScenarioParameters
                     %
                     ScenarioParameters = zeros(1,length(theta));
                     ScenarioParameters(positions) = ones(1,length(positions));
                     %
                     % First we check if the calculations have already been performed
                     %
                     [Stat,MeasurementErrorStatus] = SVEExist(DSGEModel,ScenarioParameters,UsedDraws,NumDraws,CurrChain,TotalPaths,0,1,SimulSmooth);
                     if (Stat==1)&(MeasurementErrorStatus==1);
                        txt = 'YADA has located results for the measurement errors on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,'Measurement Error Distribution',450,'no',CurrINI);
                        DoCalc = 1;
                     elseif (Stat==1)&(MeasurementErrorStatus==0);
                        txt = ['The DSGE model "' DSGEModel.NameOfModel '" doesn''t have any measurement errors.'];
                        About(txt,'information','Measurement Error Distribution',180,500,CurrINI);
                        DoCalc = 0;
                     else;
                        answer = 'No';
                        DoCalc = 1;
                     end;
                     %
                     if DoCalc==1;
                        if strcmp(lower(answer),'yes')==1;
                           DoneCalc = 1;
                        else;
                           %
                           % start estimation
                           %
                           if SimulSmooth==0;
                              DoneCalc = DSGEStateVariableEstimation(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'Measurement Error',1,NumDraws,CurrChain,DSGEModel,CurrINI);
                           else;
                              DoneCalc = DSGESimulationSmoother(theta,thetaPostSample,thetaPositions,ModelParameters,ScenarioParameters,'Measurement Error',1,TotalPaths,NumDraws,CurrChain,DSGEModel,controls,CurrINI);
                           end;
                        end;
                        %
                        % check if we should show any results
                        %
                        if DoneCalc==1;
                           %
                           % compute the distribution for measurement errors
                           %
                           if SimulSmooth==0;
                              SVEs = DSGESVEDistribution(DSGEModel,CurrINI,4,ScenarioParameters,UsedDraws,NumDraws,CurrChain,1,Weights);
                           else;
                              SVEs = DSGESimSmoothDistribution(DSGEModel,CurrINI,3,ScenarioParameters,UsedDraws,NumDraws,CurrChain,TotalPaths,1,Weights);
                           end;
                           %
                           % ask if we should save results to disk
                           %
                           txt = 'Would you like to save the mean and quantiles of the measurement error distributions to file?';
                           answer = Query(txt,'question',140,'Save - Measurement Error Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              SelParamStr = strrep(num2str(ScenarioParameters),' ','');
                              if SimulSmooth==0;
                                 file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '-' int2str(CurrChain) '.mat']);
                              else;
                                 file = FixFilePath([DSGEModel.OutputDirectory '\MeasurementErrorSimulSmoothDist-' SelParamStr '-' int2str(UsedDraws) '-' int2str(NumDraws) '.' int2str(TotalPaths) '-' int2str(CurrChain) '.mat']);
                              end;
                              VariableNames = DSGEModel.VariableNames;
                              save(file,'SVEs','VariableNames');
                              %
                              % create a matlab file to read the data
                              %
                              if SimulSmooth==0;
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\MEDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(CurrChain) '.m']);
                              else;
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\MESimSmoothDist' int2str(sum(ScenarioParameters)) 'x' int2str(length(ScenarioParameters)-sum(ScenarioParameters)) int2str(UsedDraws) int2str(NumDraws) int2str(TotalPaths) int2str(CurrChain) '.m']);
                              end;
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              if SimulSmooth==0;
                                 fprintf(fid,'%%\n%% structure with fields wtT (smooth mean), wtt (update mean), Quantiles, DateVector and ErrorIndicator.\n%%\n');
                                 SStr = 'update and the smooth estimators';
                              else;
                                 fprintf(fid,'%%\n%% structure with fields wtT (smooth mean), Quantiles, DateVector and ErrorIndicator.\n%%\n');
                                 SStr = 'smooth estimator';
                              end;
                              fprintf(fid,'SVEs = DataStructure.SVEs;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the variables\n%%\n');
                              fprintf(fid,'VariableNames = DataStructure.VariableNames;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              txt = ['The measurement error data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 2 entries: SVEs (a structure with the mean of the ' SStr ', quantile data, indicator which functions have non-zero measurement errors, and numeric dates), and VariableNames (string matrix with the names of the observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','Measurement Error Distribution',200,500,CurrINI);
                           end;
                           %
                           % display the results
                           %
                           StateVarDistributionDLG('init',SVEs,4,1,0,[],DSGEModel,CurrINI);
                        end;
                     end;
                  elseif isempty(positions)==1;
                     txt = 'You didn''t select any parameters that should be varied for the estimation of the measurement error distribution.';
                     About(txt,'information','Measurement Error Distribution',150,500,CurrINI);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LogLikelihoodInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view a graph of the log-likelihood function before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Log-Likelihood Function - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
               %
               % Try to calculate state variables. First, however, the code needs to check if a unique convergent
               % solution exists.
               %
               if DSGEModel.KalmanAlgorithm~=2;
                  [StateVarStructure,mcode,kalmanstatus] = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
               else;
                  [StateVarStructure.lnLt,StateVarStructure.nonsingular,StateVarStructure.KalmanTime,mcode,kalmanstatus] = CalculateDSGESquareRootFilter(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
               end;
               if (mcode==1)&(kalmanstatus==0);
                  %
                  % we're OK
                  %
                  if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'lnLt')==1);
                     if (isempty(StateVarStructure.lnLt)==0)&(sum(isinf(StateVarStructure.lnLt))==0);
                        if StateVarStructure.nonsingular==1;
                           GraphLogLikelihood(StateVarStructure.lnLt,StateVarStructure.KalmanTime,DSGEModel,CurrINI,'','Initial Values');
                        else;
                           txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                           About(txt,'information','Log-Likelihood Function - Initial Values',300,500,CurrINI);
                        end;
                     elseif sum(isinf(StateVarStructure.lnLt))>0;
                        if StateVarStructure.nonsingular==1;
                           txt = ['There are ' int2str(sum(isinf(StateVarStructure.lnLt))) ' infinite values when computing the log-likelihood function. The most likely reason is that the 1-step ahead forecast error covariance matrix of the observed variables is singular.'];
                        else;
                           txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                        end;
                        About(txt,'information','Log-Likelihood Function - Initial Values',300,500,CurrINI);
                     end;
                  end;
               elseif (mcode==1)&(kalmanstatus~=0);
                  if kalmanstatus==1;
                     txt = 'The state variables are not stationary at the initial values.';
                  else;
                     if StateVarStructure.nonsingular==1;
                        txt = 'The covariance matrix of the forecast of the observed variables in period t conditional on data until t-1 is not positive definite for some time period t. This implies that the log-likelihood value is a NaN (or a complex number) in Matlab.';
                     else;
                        txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                     end;
                  end;
                  About(txt,'information','Log-Likelihood Function - Initial Values',200,500,CurrINI);
               elseif mcode~=1;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  txt = ['The ' SolverStr ' solver provided the return code: ' int2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Log-Likelihood Function - Initial Values',200,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'LogLikelihoodPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % we can now calculate the state shocks
            %
            if DSGEModel.KalmanAlgorithm~=2;
               StateVarStructure = CalculateDSGEStateVariables(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0,0,'');
            else;
               [StateVarStructure.lnLt,StateVarStructure.nonsingular,StateVarStructure.KalmanTime] = CalculateDSGESquareRootFilter(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
            end;
            if (isempty(StateVarStructure)==0)&(isfield(StateVarStructure,'lnLt')==1);
               if (isempty(StateVarStructure.lnLt)==0)&(sum(isinf(StateVarStructure.lnLt))==0);
                  if StateVarStructure.nonsingular==1;
                     GraphLogLikelihood(StateVarStructure.lnLt,StateVarStructure.KalmanTime,DSGEModel,CurrINI,'','Posterior Mode');
                  else;
                     txt = 'The model has fewer structural shocks and measurement errors than observed variables. The log-likelihood function can therefore not be computed.';
                     About(txt,'information','Log-Likelihood Function - Posterior Mode',300,500,CurrINI);
                  end;
               elseif sum(isinf(StateVarStructure.lnLt))>0;
                  txt = ['There are ' int2str(sum(isinf(StateVarStructure.lnLt))) ' infinite values when computing the log-likelihood function. The most likely reason is that the 1-step ahead forecast error covariance matrix of the observed variables is singular.'];
                  About(txt,'information','Log-Likelihood Function - Posterior Mode',300,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ParameterScenariosInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the parameter scenario before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Parameter Scenario at Initial Values',450,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     thetaIndex = ModeData.thetaIndex;
                     UniformBounds = ModeData.UniformBounds;
                     LowerBound = ModeData.LowerBound;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
               [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
               %
               % selection dialog for the parameters to change under the secnario
               %
               CalibratedParameterNames = ParameterNames.calibrated;
               ParameterNames = strvcat(ParameterNames.estimated,ParameterNames.calibrated);
               if isempty(DSGEModel.ParameterScenario)==1;
                  ParameterScenario = zeros(1,size(ParameterNames,1));
               else;
                  ParameterScenario = DSGEModel.ParameterScenario;
                  if length(ParameterScenario)<size(ParameterNames,1);
                     ParameterScenario = [ParameterScenario zeros(1,size(ParameterNames,1)-length(ParameterScenario))];
                  end;
               end;
               %
               % collect all the calibrated parameters
               %
               if length(ParameterScenario)>length(theta);
                  CalibratedParams = GetCalibratedParameters(CalibratedParameterNames,ModelParameters);
               else;
                  CalibratedParams = [];
               end;
               ScenarioParams = ParameterScenario.*(1:length(theta)+length(CalibratedParams));
               ScenarioParams = ScenarioParams(ScenarioParams>0);
               OldScenarioParams = ScenarioParams;
               [PosAction,ScenarioParams] = SelectCondVarShockDLG('init','Parameters','Scenario',ParameterNames,ScenarioParams,CurrINI);
               if strcmp(PosAction,'OK')==1;
                  if length(ScenarioParams)>0;
                     ParameterScenario = zeros(1,length(theta)+length(CalibratedParams));
                     ParameterScenario(ScenarioParams) = 1;
                     %
                     % we have now selected at least one parameter. now we need to choose new values for these parameters
                     %
                     phi = ThetaToPhi(theta,thetaIndex,UniformBounds,LowerBound);
                     phiTest = (phi==0);
                     BaseParameterValue = PhiToTheta(1.05*(phi+((0.05/1.05)*phiTest)),thetaIndex,UniformBounds,LowerBound);
                     if length(CalibratedParams)>0;
                        BaseParameterValue = [BaseParameterValue;1.1*CalibratedParams];
                     end;
                     %
                     ParameterFile = FixFilePath([DSGEModel.OutputDirectory '\parameterscenario\ParameterScenarioValues-' DSGEModel.NameOfModel '-InitialValues.mat']);
                     if isequal(ScenarioParams,OldScenarioParams)==1;
                        if FileExist(ParameterFile)==1;
                           ParameterData = load(ParameterFile);
                           if isfield(ParameterData,'ParameterScenarioValues')==1;
                              ParameterScenarioValues = ParameterData.ParameterScenarioValues;
                              if length(ParameterScenarioValues)>(length(theta)+length(CalibratedParams));
                                 ParameterScenarioValues = BaseParameterValue;
                              elseif length(ParameterScenarioValues)==length(theta);
                                 ParameterScenarioValues = [ParameterScenarioValues;1.1*CalibratedParams];
                              elseif length(ParameterScenarioValues)<length(theta);
                                 ParameterScenarioValues = BaseParameterValue;
                              end;
                           else;
                              ParameterScenarioValues = BaseParameterValue;
                           end;
                        else;
                           ParameterScenarioValues = BaseParameterValue;
                        end;
                     else;
                        ParameterScenarioValues = BaseParameterValue;
                     end;
                     ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
                     %
                     % make sure that all parameter values satisfy the bounds
                     %
                     ParameterScenarioValues(1:length(theta)) = CheckBounds(ParameterScenarioValues(1:length(theta)),ParameterBounds);
                     %
                     % we now have the last selected alternative values for the parameters
                     %
                     ParameterOriginalValues = [theta;CalibratedParams];
                     if length(CalibratedParams)>0;
                        ParameterBounds = [ParameterBounds;[-Inf*ones(length(CalibratedParams),1) Inf*ones(length(CalibratedParams),1) zeros(length(CalibratedParams),1)]];
                     end;
                     [action,NewValues] = ParameterScenarioDLG('init',ParameterOriginalValues(ScenarioParams),ParameterScenarioValues(ScenarioParams),ParameterBounds(ScenarioParams,:),ParameterNames(ScenarioParams,:),'Initial Values',CurrINI);
                     if strcmp(action,'OK')==1;
                        %
                        % we have an alternative vector of values
                        %
                        ParameterScenarioValues = ParameterOriginalValues;
                        ParameterScenarioValues(ScenarioParams) = NewValues;
                        if max(abs(ParameterOriginalValues-ParameterScenarioValues))>1.0e-012;
                           %
                           % select the period when the parameters change
                           %
                           if FileExist(FixFilePath([pwd '\addons\ovs\OVSTheta.m']))==1;
                              ParameterScenarioValue = DSGEModel.ParameterScenarioValue;
                              SubSampleStrMat = SubSampleStrMat(DSGEModel.KalmanFirstObservation:size(SubSampleStrMat,1),:);
                              %
                              % check if ParameterScenarioValue is valid
                              %
                              if size(SubSampleStrMat,1)<ParameterScenarioValue;
                                 ParameterScenarioValue = 1;
                              end;
                              [action,BreakPeriod] = SelectionDlg(SubSampleStrMat,ParameterScenarioValue,'Select scenario start period:','Parameter Scenario','Initial Parameter Values','','',CurrINI);
                              if BreakPeriod~=ParameterScenarioValue;
                                 DSGEModel.ParameterScenarioValue = BreakPeriod;
                              end;
                           else;
                              action = 'OK';
                              BreakPeriod = 1;
                           end;
                           if strcmp(lower(action),'ok')==1;
                              %
                              % we can now try to solve the model for these two parameters values and check
                              % the impact th new values has on the data (assuming the shocks are unaffected).
                              %
                              [Status,Y,YScenario,OriginalShocks,ScenarioShocks,OriginalStates,ScenarioStates,KeepShocks,Params] = DSGEParameterScenariosTheta(DSGEModel,theta,ParameterScenarioValues,CalibratedParameterNames,thetaPositions,ModelParameters,FirstPeriod,LastPeriod,BreakPeriod,0,CurrINI);
                              if Status==1;
                                 %
                                 % we first save the changes to both the DSGEModel structure and the parameter scenario data
                                 %
                                 DSGEModel.ParameterScenario = ParameterScenario;
                                 if DirExist(FixFilePath([DSGEModel.OutputDirectory '\parameterscenario']))==0;
                                    stat = MakeDir(DSGEModel.OutputDirectory,'parameterscenario');
                                 end;
                                 save(ParameterFile,'ParameterScenarioValues');
                                 %
                                 % check
                                 %
                                 if (FileExist(FixFilePath([pwd '\addons\ovs\OVSTheta.m']))==1)&(BreakPeriod>=2)&((CheckScenarioData(DSGEModel)==1)|(CheckStateScenarioData(DSGEModel)==1));
                                    %
                                    % make sure we take changes to calibrated parameters into account
                                    %
                                    NewModelParameters = ModelParameters;
                                    if length(CalibratedParams)>0;
                                       for i=1:length(CalibratedParams);
                                          NewModelParameters = setfield(NewModelParameters,StringTrim(CalibratedParameterNames(i,:)),ParameterScenarioValues(length(theta)+i));
                                       end;
                                    end;
                                    [Status,YScenario,ScenarioShocks,ScenarioStates,ManShocks,ManStates] = OVSTheta(YScenario,ScenarioShocks,ScenarioStates,OriginalShocks,OriginalStates,ParameterScenarioValues(1:length(theta)),thetaPositions,NewModelParameters,1,FirstPeriod,LastPeriod,BreakPeriod,'Initial Values',DSGEModel,CurrINI);
                                 else;
                                    Status = 1;
                                    ManShocks = ScenarioShocks;
                                    ManStates = ScenarioStates;
                                 end;
                                 %
                                 if Status==1;
                                    %
                                    % create additional inputs for displaying results
                                    %
                                    DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                                    DateVector = DateVector(DSGEModel.KalmanFirstObservation:length(DateVector));
                                    ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
                                    %
                                    if gcbo==controls.ps.initialvalues;
                                       AnnualizeData = 0;
                                       AnnualStr = 'Original';
                                    else;
                                       AnnualizeData = 1;
                                       AnnualStr = 'Annualized';
                                       Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                                       YScenario = Annualize(YScenario,DSGEModel.annual,DSGEModel.annualscale);
                                    end;
                                    %
                                    % check if the user wants to save the results to disk
                                    %
                                    txt = 'Would you like to save the parameter scenario results at initial values to disk?';
                                    answer = Query(txt,'question',140,'Save - Parameter Scenario',500,'no',CurrINI);
                                    OrigDateVector = DateVector;
                                    if strcmp(lower(answer),'yes')==1;
                                       ShockDateVector = DateVector;
                                       YNames = DSGEModel.VariableNames;
                                       StateVariableNames = DSGEModel.StateVariableNames;
                                       OriginalTheta = theta;
                                       ParameterScenarioTheta = ParameterScenarioValues(1:length(theta));
                                       if length(CalibratedParams)>0;
                                          ParameterScenarioCalibratedParams = ParameterScenarioValues(1+length(theta):length(ParameterScenarioValues));
                                       else;
                                          ParameterScenarioCalibratedParams = [];
                                       end;
                                       %
                                       A = Params.A;
                                       H = Params.H;
                                       R = Params.R;
                                       F = Params.F;
                                       B0 = Params.B0;
                                       AScenario = Params.AScenario;
                                       HScenario = Params.HScenario;
                                       RScenario = Params.RScenario;
                                       FScenario = Params.FScenario;
                                       B0Scenario = Params.B0Scenario;
                                       %
                                       if AnnualizeData==0;
                                          file = FixFilePath([DSGEModel.OutputDirectory '\ParameterScenario-Original-' int2str(length(ScenarioParams)) '-InitialValues.mat']);
                                          mfile = FixFilePath([DSGEModel.OutputDirectory '\ParaScenOrig' int2str(length(ScenarioParams)) 'InitVal.m']);
                                       else;
                                          DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                          file = FixFilePath([DSGEModel.OutputDirectory '\ParameterScenario-Annual-' int2str(length(ScenarioParams)) '-InitialValues.mat']);
                                          mfile = FixFilePath([DSGEModel.OutputDirectory '\ParaScenAnn' int2str(length(ScenarioParams)) 'InitVal.m']);
                                       end;
                                       %
                                       if FileExist(FixFilePath([pwd '\addons\ovs\OVSTheta.m']))==1;
                                          save(file,'ParameterNames','OriginalTheta','ParameterScenarioTheta','CalibratedParams','ParameterScenarioCalibratedParams','A','H','R','F','B0','AScenario','HScenario','RScenario','FScenario','B0Scenario','YNames','Y','YScenario','DateVector','ShockNames','OriginalShocks','ScenarioShocks','StateVariableNames','OriginalStates','ScenarioStates','ManShocks','ManStates','ShockDateVector','BreakPeriod');
                                       else;
                                          save(file,'ParameterNames','OriginalTheta','ParameterScenarioTheta','CalibratedParams','ParameterScenarioCalibratedParams','A','H','R','F','B0','AScenario','HScenario','RScenario','FScenario','B0Scenario','YNames','Y','YScenario','DateVector','ShockNames','OriginalShocks','ScenarioShocks','StateVariableNames','OriginalStates','ScenarioStates','ManShocks','ManStates','ShockDateVector');
                                       end;
                                       %
                                       % create the m-file to access the data from
                                       %
                                       fid = fopen(mfile,'wt');
                                       fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                       fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                       %
                                       fprintf(fid,'%%\n%% M E A S U R E M E N T   E Q U A T I O N\n');
                                       fprintf(fid,'%%\n%% matrix with original parameters on exogenous variables\n%%\n');
                                       fprintf(fid,'A = DataStructure.A;\n');
                                       fprintf(fid,'%%\n%% matrix with original parameters on state variables\n%%\n');
                                       fprintf(fid,'H = DataStructure.H;\n');
                                       fprintf(fid,'%%\n%% matrix with original measurement error covariances\n%%\n');
                                       fprintf(fid,'R = DataStructure.R;\n');
                                       fprintf(fid,'%%\n%% S T A T E   E Q U A T I O N\n');
                                       fprintf(fid,'%%\n%% matrix with original state transition parameters\n%%\n');
                                       fprintf(fid,'F = DataStructure.F;\n');
                                       fprintf(fid,'%%\n%% matrix with original shock parameters\n%%\n');
                                       fprintf(fid,'B0 = DataStructure.B0;\n');
                                       %
                                       fprintf(fid,'%%\n%% matrix with scenario parameters on exogenous variables\n%%\n');
                                       fprintf(fid,'AScenario = DataStructure.AScenario;\n');
                                       fprintf(fid,'%%\n%% matrix with scenario parameters on state variables\n%%\n');
                                       fprintf(fid,'HScenario = DataStructure.HScenario;\n');
                                       fprintf(fid,'%%\n%% matrix with scenario measurement error covariances\n%%\n');
                                       fprintf(fid,'RScenario = DataStructure.RScenario;\n');
                                       fprintf(fid,'%%\n%% matrix with scenario state transition parameters\n%%\n');
                                       fprintf(fid,'FScenario = DataStructure.FScenario;\n');
                                       fprintf(fid,'%%\n%% matrix with scenario shock parameters\n%%\n');
                                       fprintf(fid,'B0Scenario = DataStructure.B0Scenario;\n');
                                       %
                                       fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                       fprintf(fid,'YNames = DataStructure.YNames;\n');
                                       fprintf(fid,'%%\n%% matrix with data for the observed variables\n%%\n');
                                       fprintf(fid,'Y = DataStructure.Y;\n');
                                       fprintf(fid,'%%\n%% The observed variables for unchanged economic shocks but with scenario parameters\n%%\n');
                                       fprintf(fid,'YScenario = DataStructure.YScenario;\n');
                                       fprintf(fid,'%%\n%% vector of dates for the observed variables\n%%\n');
                                       fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                                       fprintf(fid,'%%\n%% string matrix with names of the state variables\n%%\n');
                                       fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                                       fprintf(fid,'%%\n%% matrix with smooth estimates of the state variables using the original parameters\n%%\n');
                                       fprintf(fid,'OriginalStates = DataStructure.OriginalStates;\n');
                                       fprintf(fid,'%%\n%% matrix with state variables for the scenario parameters\n%%\n');
                                       fprintf(fid,'ScenarioStates = DataStructure.ScenarioStates;\n');
                                       fprintf(fid,'%%\n%% string matrix with names of the economic shocks\n%%\n');
                                       fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                                       fprintf(fid,'%%\n%% matrix with smooth estimates of the economic shocks using the original parameters\n%%\n');
                                       fprintf(fid,'OriginalShocks = DataStructure.OriginalShocks;\n');
                                       fprintf(fid,'%%\n%% matrix with smooth estimates of the economic shocks using the scenario parameters\n%%\n');
                                       fprintf(fid,'ScenarioShocks = DataStructure.ScenarioShocks;\n');
                                       if FileExist(FixFilePath([pwd '\addons\ovs\OVSTheta.m']))==1;
                                          fprintf(fid,'%%\n%% The position in the data when parameters change\n%%\n');
                                          fprintf(fid,'BreakPeriod = DataStructure.BreakPeriod;\n');
                                          NumEntries = '29';
                                          BreakEntry = ' BreakPeriod (position in the sample when the parameters change),';
                                       else;
                                          NumEntries = '28';
                                          BreakEntry = '';
                                       end;
                                       fprintf(fid,'%%\n%% matrix with the manipulated values of the economic shocks\n%%\n');
                                       fprintf(fid,'ManShocks = DataStructure.ManShocks;\n');
                                       fprintf(fid,'%%\n%% matrix with the manipulated values of the state variables\n%%\n');
                                       fprintf(fid,'ManStates = DataStructure.ManStates;\n');
                                       fprintf(fid,'%%\n%% vector of dates for the estimated economic shocks\n%%\n');
                                       fprintf(fid,'ShockDateVector = DataStructure.ShockDateVector;\n');
                                       fprintf(fid,'%%\n%% string matrix with names of the estimated and calibrated parameters\n%%\n');
                                       fprintf(fid,'ParameterNames = DataStructure.ParameterNames;\n');
                                       fprintf(fid,'%%\n%% vector with the original values of the estimated parameters\n%%\n');
                                       fprintf(fid,'OriginalTheta = DataStructure.OriginalTheta;\n');
                                       fprintf(fid,'%%\n%% vector with the scenario values of the estimated parameters\n%%\n');
                                       fprintf(fid,'ParameterScenarioTheta = DataStructure.ParameterScenarioTheta;\n');
                                       fprintf(fid,'%%\n%% vector with the original values of the calibrated parameters\n%%\n');
                                       fprintf(fid,'CalibratedParams = DataStructure.CalibratedParams;\n');
                                       fprintf(fid,'%%\n%% vector with the scenario values of the calibrated parameters\n%%\n');
                                       fprintf(fid,'ParameterScenarioCalibratedParams = DataStructure.ParameterScenarioCalibratedParams;\n');
                                       fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                       %
                                       % end the script file
                                       %
                                       fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                       fclose(fid);
                                       %
                                       txt = ['The parameter scenario data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: the 5 original and the 5 scenario state space matrices (A, H, R, F, B0, AScenario, HScenario, RScenario, FScenario, B0Scenario), YNames (string matrix with the names of the observed variables), Y (matrix with the observed variables), YScenario (matrix with the observed variables under the parameter scenario with unchanged shocks), DateVector (vector with dates fvor the observed variables), StateVariableNames (string matrix with the names of the state variables), OriginalStates (matrix with smooth estimates of the states for the original parameters), ScenarioStates (matrix with state variables under the parameter scenario), ShockNames (string matrix with the names of the economic shocks), OriginalShocks (matrix with the smooth estimates of the economic shocks using the original parameters), ScenarioShocks (matrix with smooth estimates of the economic shocks using the scenario parameters),' BreakEntry ' ManShocks (matrix with the manipulated values of the economic shocks under the extended scenario), ManStates (matrix with the manipulated values of the state variables under the extended scenario), ShockDateVector (vector with dates for the shocks), ParameterNames (string matrix with the names of the estimated and the calibrated parameters), OriginalTheta (vector with the original values of the estimated parameters), ParameterScenarioTheta (vector with the scenario values of the estimated parameters), CalibratedParams (vector with the original values of the calibrated parameters), and ParameterScenarioCalibratedParams (vector with the scenario values of the calibrated parameters). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                       About(txt,'information','Parameter Scenario',400,500,CurrINI);
                                    end;
                                    %
                                    % time to show the results
                                    %
                                    GraphParameterScenarioDLG('init',Y,YScenario,OriginalShocks,ScenarioShocks,ManShocks,ShockNames,OriginalStates,ScenarioStates,ManStates,OrigDateVector,BreakPeriod,AnnualizeData,AnnualStr,'Initial Values',DSGEModel,CurrINI);
                                 end;
                              end;
                           end;
                        else;
                           txt = 'The values for all parameters under the scenario are equal to the initial parameter values and any original calibrated values. This makes the scenario pointless and YADA has therefore aborted.';
                           About(txt,'information','Parameter Scenario - Initial Value',200,500,CurrINI);
                        end;
                     end;
                  else;
                     txt = 'You didn''t select any parameters to change for the "Parameter Scenario". This tool requires that you select at least one parameter to change.';
                     About(txt,'information','Parameter Scenario - Initial Values',200,500,CurrINI);
                  end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ParameterScenariosPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            phi = ModeData.phiMode;
            InverseHessian = ModeData.InverseHessian;
            UniformBounds = ModeData.UniformBounds;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            ParameterNames = strvcat(ModeData.ParameterNames.estimated,ModeData.ParameterNames.calibrated);
            thetaIndex = ModeData.thetaIndex;
            LowerBound = ModeData.LowerBound;
            phiFittedStd = ModeData.phiFittedStd;
            thetaDist = ModeData.thetaDist;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%            else;
               [FirstPeriod,LastPeriod,SubSampleStrMat] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % selection dialog for the parameters to change under the scenario
            %
            if isempty(DSGEModel.ParameterScenario)==1;
               ParameterScenario = zeros(1,size(ParameterNames,1));
            else;
               ParameterScenario = DSGEModel.ParameterScenario;
               if length(ParameterScenario)<size(ParameterNames,1);
                  ParameterScenario = [ParameterScenario zeros(1,size(ParameterNames,1)-length(ParameterScenario))];
               end;
            end;
            %
            % collect all the calibrated parameters
            %
            if length(ParameterScenario)>length(theta);
               CalibratedParams = GetCalibratedParameters(ModeData.ParameterNames.calibrated,ModelParameters);
            else;
               CalibratedParams = [];
            end;
            ScenarioParams = ParameterScenario.*(1:length(theta)+length(CalibratedParams));
            ScenarioParams = ScenarioParams(ScenarioParams>0);
            OldScenarioParams = ScenarioParams;
            [PosAction,ScenarioParams] = SelectCondVarShockDLG('init','Parameters','Scenario',ParameterNames,ScenarioParams,CurrINI);
            if strcmp(PosAction,'OK')==1;
               if length(ScenarioParams)>0;
                  ParameterScenario = zeros(1,length(theta)+length(CalibratedParams));
                  ParameterScenario(ScenarioParams) = 1;
                  %
                  % we have now selected at least one parameter. now we need to choose new values for these parameters
                  %
                  if (DSGEModel.InverseHessianEstimator==2)&(isempty(phiFittedStd)==0);
                     BaseParameterValue = PhiToTheta(phi+phiFittedStd,thetaIndex,UniformBounds,LowerBound);
                  else;
                     BaseParameterValue = PhiToTheta(phi+sqrt(diag(InverseHessian)),thetaIndex,UniformBounds,LowerBound);
                  end;
                  if length(CalibratedParams)>0;
                     BaseParameterValue = [BaseParameterValue;1.1*CalibratedParams];
                  end;
                  %
                  % check if we have the same selected scenario params as before
                  %
                  ParameterFile = FixFilePath([DSGEModel.OutputDirectory '\parameterscenario\ParameterScenarioValues-' DSGEModel.NameOfModel '-PostMode.mat']);
                  if isequal(ScenarioParams,OldScenarioParams)==1;
                     if FileExist(ParameterFile)==1;
                        ParameterData = load(ParameterFile);
                        if isfield(ParameterData,'ParameterScenarioValues')==1;
                           ParameterScenarioValues = ParameterData.ParameterScenarioValues;
                           if length(ParameterScenarioValues)>(length(theta)+length(CalibratedParams));
                              ParameterScenarioValues = BaseParameterValue;
                           elseif length(ParameterScenarioValues)==length(theta);
                              ParameterScenarioValues = [ParameterScenarioValues;1.1*CalibratedParams];
                           elseif length(ParameterScenarioValues)<length(theta);
                              ParameterScenarioValues = BaseParameterValue;
                           end;
                        else;
                           ParameterScenarioValues = BaseParameterValue;
                        end;
                     else;
                        ParameterScenarioValues = BaseParameterValue;
                     end;
                  else;
                     ParameterScenarioValues = BaseParameterValue;
                  end;
                  ParameterBounds = CreateBounds(thetaDist,LowerBound,UniformBounds);
                  %
                  % make sure that all parameter values satisfy the bounds except for the calibrated ones
                  %
                  ParameterScenarioValues(1:length(theta)) = CheckBounds(ParameterScenarioValues(1:length(theta)),ParameterBounds);
                  %
                  % we now have the last selected alternative values for the parameters
                  %
                  ParameterOriginalValues = [theta;CalibratedParams];
                  if length(CalibratedParams)>0;
                     ParameterBounds = [ParameterBounds;[-Inf*ones(length(CalibratedParams),1) Inf*ones(length(CalibratedParams),1) zeros(length(CalibratedParams),1)]];
                  end;
                  [action,NewValues] = ParameterScenarioDLG('init',ParameterOriginalValues(ScenarioParams),ParameterScenarioValues(ScenarioParams),ParameterBounds(ScenarioParams,:),ParameterNames(ScenarioParams,:),'Posterior Mode',CurrINI);
                  if strcmp(action,'OK')==1;
                     %
                     % we have an alternative vector of values
                     %
                     ParameterScenarioValues = ParameterOriginalValues;
                     ParameterScenarioValues(ScenarioParams) = NewValues;
                     if max(abs(ParameterOriginalValues-ParameterScenarioValues))>1.0e-012;
                        %
                        % select the period when the parameters change
                        %
                        if FileExist(FixFilePath([pwd '\addons\ovs\OVSTheta.m']))==1;
                           ParameterScenarioValue = DSGEModel.ParameterScenarioValue;
                           SubSampleStrMat = SubSampleStrMat(DSGEModel.KalmanFirstObservation:size(SubSampleStrMat,1),:);
                           %
                           % check if ParameterScenarioValue is valid
                           %
                           if size(SubSampleStrMat,1)<ParameterScenarioValue;
                              ParameterScenarioValue = 1;
                           end;
                           [action,BreakPeriod] = SelectionDlg(SubSampleStrMat,ParameterScenarioValue,'Select scenario start period:','Parameter Scenario','Posterior Mode Parameter Values','','',CurrINI);
                           if BreakPeriod~=ParameterScenarioValue;
                              DSGEModel.ParameterScenarioValue = BreakPeriod;
                           end;
                        else;
                           action = 'OK';
                           BreakPeriod = 1;
                        end;
                        if strcmp(lower(action),'ok')==1;
                           %
                           % we can now try to solve the model for these two parameters values and check
                           % the impact th new values has on the data (assuming the shocks are unaffected).
                           %
                           [Status,Y,YScenario,OriginalShocks,ScenarioShocks,OriginalStates,ScenarioStates,KeepShocks,Params] = DSGEParameterScenariosTheta(DSGEModel,theta,ParameterScenarioValues,ModeData.ParameterNames.calibrated,thetaPositions,ModelParameters,FirstPeriod,LastPeriod,BreakPeriod,1,CurrINI);
                           if Status==1;
                              %
                              % we first save the changes to both the DSGEModel structure and the parameter scenario data
                              %
                              DSGEModel.ParameterScenario = ParameterScenario;
                              if DirExist(FixFilePath([DSGEModel.OutputDirectory '\parameterscenario']))==0;
                                 stat = MakeDir(DSGEModel.OutputDirectory,'parameterscenario');
                              end;
                              save(ParameterFile,'ParameterScenarioValues');
                              %
                              % check
                              %
                              if (FileExist(FixFilePath([pwd '\addons\ovs\OVSTheta.m']))==1)&(BreakPeriod>=2)&((CheckScenarioData(DSGEModel)==1)|(CheckStateScenarioData(DSGEModel)==1));
                                 %
                                 % make sure we take changes to calibrated parameters into account
                                 %
                                 NewModelParameters = ModelParameters;
                                 if length(CalibratedParams)>0;
                                    for i=1:length(CalibratedParams);
                                       NewModelParameters = setfield(NewModelParameters,StringTrim(ModeData.ParameterNames.calibrated(i,:)),ParameterScenarioValues(length(theta)+i));
                                    end;
                                 end;
                                 [Status,YScenario,ScenarioShocks,ScenarioStates,ManShocks,ManStates] = OVSTheta(YScenario,ScenarioShocks,ScenarioStates,OriginalShocks,OriginalStates,ParameterScenarioValues(1:length(theta)),thetaPositions,NewModelParameters,0,FirstPeriod,LastPeriod,BreakPeriod,'Posterior Mode',DSGEModel,CurrINI);
                              else;
                                 Status = 1;
                                 ManShocks = ScenarioShocks;
                                 ManStates = ScenarioStates;
                              end;
                              %
                              if Status==1;
                                 %
                                 % create additional inputs for displaying results
                                 %
                                 DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency);
                                 DateVector = DateVector(DSGEModel.KalmanFirstObservation:length(DateVector));
                                 ShockNames = DSGEModel.StateShockNames(KeepShocks,:);
                                 %
                                 if gcbo==controls.ps.postmode;
                                    AnnualizeData = 0;
                                    AnnualStr = 'Original';
                                 else;
                                    AnnualizeData = 1;
                                    AnnualStr = 'Annualized';
                                    Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                                    YScenario = Annualize(YScenario,DSGEModel.annual,DSGEModel.annualscale);
                                 end;
                                 %
                                 % check if the user wants to save the results to disk
                                 %
                                 txt = 'Would you like to save the parameter scenario results at posterior mode to disk?';
                                 answer = Query(txt,'question',140,'Save - Parameter Scenario',500,'no',CurrINI);
                                 OrigDateVector = DateVector;
                                 if strcmp(lower(answer),'yes')==1;
                                    ShockDateVector = DateVector;
                                    YNames = DSGEModel.VariableNames;
                                    StateVariableNames = DSGEModel.StateVariableNames;
                                    OriginalTheta = theta;
                                    ParameterScenarioTheta = ParameterScenarioValues(1:length(theta));
                                    if length(CalibratedParams)>0;
                                       ParameterScenarioCalibratedParams = ParameterScenarioValues(1+length(theta):length(ParameterScenarioValues));
                                    else;
                                       ParameterScenarioCalibratedParams = [];
                                    end;
                                    %
                                    A = Params.A;
                                    H = Params.H;
                                    R = Params.R;
                                    F = Params.F;
                                    B0 = Params.B0;
                                    AScenario = Params.AScenario;
                                    HScenario = Params.HScenario;
                                    RScenario = Params.RScenario;
                                    FScenario = Params.FScenario;
                                    B0Scenario = Params.B0Scenario;
                                    %
                                    if AnnualizeData==0;
                                       file = FixFilePath([DSGEModel.OutputDirectory '\ParameterScenario-Original-' int2str(length(ScenarioParams)) '-PosteriorMode.mat']);
                                       mfile = FixFilePath([DSGEModel.OutputDirectory '\ParaScenOrig' int2str(length(ScenarioParams)) 'PostMode.m']);
                                    else;
                                       DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                       file = FixFilePath([DSGEModel.OutputDirectory '\ParameterScenario-Annual-' int2str(length(ScenarioParams)) '-PosteriorMode.mat']);
                                       mfile = FixFilePath([DSGEModel.OutputDirectory '\ParaScenAnn' int2str(length(ScenarioParams)) 'PostMode.m']);
                                    end;
                                    %
                                    if FileExist(FixFilePath([pwd '\addons\ovs\OVSTheta.m']))==1;
                                       save(file,'ParameterNames','OriginalTheta','ParameterScenarioTheta','CalibratedParams','ParameterScenarioCalibratedParams','A','H','R','F','B0','AScenario','HScenario','RScenario','FScenario','B0Scenario','YNames','Y','YScenario','DateVector','ShockNames','OriginalShocks','ScenarioShocks','StateVariableNames','OriginalStates','ScenarioStates','ManShocks','ManStates','ShockDateVector','BreakPeriod');
                                    else;
                                       save(file,'ParameterNames','OriginalTheta','ParameterScenarioTheta','CalibratedParams','ParameterScenarioCalibratedParams','A','H','R','F','B0','AScenario','HScenario','RScenario','FScenario','B0Scenario','YNames','Y','YScenario','DateVector','ShockNames','OriginalShocks','ScenarioShocks','StateVariableNames','OriginalStates','ScenarioStates','ManShocks','ManStates','ShockDateVector');
                                    end;
                                    %
                                    % create the m-file to access the data from
                                    %
                                    fid = fopen(mfile,'wt');
                                    fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                    fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                    %
                                    fprintf(fid,'%%\n%% M E A S U R E M E N T   E Q U A T I O N\n');
                                    fprintf(fid,'%%\n%% matrix with original parameters on exogenous variables\n%%\n');
                                    fprintf(fid,'A = DataStructure.A;\n');
                                    fprintf(fid,'%%\n%% matrix with original parameters on state variables\n%%\n');
                                    fprintf(fid,'H = DataStructure.H;\n');
                                    fprintf(fid,'%%\n%% matrix with original measurement error covariances\n%%\n');
                                    fprintf(fid,'R = DataStructure.R;\n');
                                    fprintf(fid,'%%\n%% S T A T E   E Q U A T I O N\n');
                                    fprintf(fid,'%%\n%% matrix with original state transition parameters\n%%\n');
                                    fprintf(fid,'F = DataStructure.F;\n');
                                    fprintf(fid,'%%\n%% matrix with original shock parameters\n%%\n');
                                    fprintf(fid,'B0 = DataStructure.B0;\n');
                                    %
                                    fprintf(fid,'%%\n%% matrix with scenario parameters on exogenous variables\n%%\n');
                                    fprintf(fid,'AScenario = DataStructure.AScenario;\n');
                                    fprintf(fid,'%%\n%% matrix with scenario parameters on state variables\n%%\n');
                                    fprintf(fid,'HScenario = DataStructure.HScenario;\n');
                                    fprintf(fid,'%%\n%% matrix with scenario measurement error covariances\n%%\n');
                                    fprintf(fid,'RScenario = DataStructure.RScenario;\n');
                                    fprintf(fid,'%%\n%% matrix with scenario state transition parameters\n%%\n');
                                    fprintf(fid,'FScenario = DataStructure.FScenario;\n');
                                    fprintf(fid,'%%\n%% matrix with scenario shock parameters\n%%\n');
                                    fprintf(fid,'B0Scenario = DataStructure.B0Scenario;\n');
                                    %
                                    fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                    fprintf(fid,'YNames = DataStructure.YNames;\n');
                                    fprintf(fid,'%%\n%% matrix with data for the observed variables\n%%\n');
                                    fprintf(fid,'Y = DataStructure.Y;\n');
                                    fprintf(fid,'%%\n%% The observed variables for unchanged economic shocks but with scenario parameters\n%%\n');
                                    fprintf(fid,'YScenario = DataStructure.YScenario;\n');
                                    fprintf(fid,'%%\n%% vector of dates for the observed variables\n%%\n');
                                    fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                                    fprintf(fid,'%%\n%% string matrix with names of the state variables\n%%\n');
                                    fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                                    fprintf(fid,'%%\n%% matrix with smooth estimates of the state variables using the original parameters\n%%\n');
                                    fprintf(fid,'OriginalStates = DataStructure.OriginalStates;\n');
                                    fprintf(fid,'%%\n%% matrix with state variables for the scenario parameters\n%%\n');
                                    fprintf(fid,'ScenarioStates = DataStructure.ScenarioStates;\n');
                                    fprintf(fid,'%%\n%% string matrix with names of the economic shocks\n%%\n');
                                    fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                                    fprintf(fid,'%%\n%% matrix with smooth estimates of the economic shocks using the original parameters\n%%\n');
                                    fprintf(fid,'OriginalShocks = DataStructure.OriginalShocks;\n');
                                    fprintf(fid,'%%\n%% matrix with smooth estimates of the economic shocks using the scenario parameters\n%%\n');
                                    fprintf(fid,'ScenarioShocks = DataStructure.ScenarioShocks;\n');
                                    if FileExist(FixFilePath([pwd '\addons\ovs\OVSTheta.m']))==1;
                                       fprintf(fid,'%%\n%% The position in the data when parameters change\n%%\n');
                                       fprintf(fid,'BreakPeriod = DataStructure.BreakPeriod;\n');
                                       NumEntries = '29';
                                       BreakEntry = ' BreakPeriod (position in the sample when the parameters change),';
                                    else;
                                       NumEntries = '28';
                                       BreakEntry = '';
                                    end;
                                    fprintf(fid,'%%\n%% matrix with the manipulated values of the economic shocks\n%%\n');
                                    fprintf(fid,'ManShocks = DataStructure.ManShocks;\n');
                                    fprintf(fid,'%%\n%% matrix with the manipulated values of the state variables\n%%\n');
                                    fprintf(fid,'ManStates = DataStructure.ManStates;\n');
                                    fprintf(fid,'%%\n%% vector of dates for the estimated economic shocks\n%%\n');
                                    fprintf(fid,'ShockDateVector = DataStructure.ShockDateVector;\n');
                                    fprintf(fid,'%%\n%% string matrix with names of the estimated and calibrated parameters\n%%\n');
                                    fprintf(fid,'ParameterNames = DataStructure.ParameterNames;\n');
                                    fprintf(fid,'%%\n%% vector with the original values of the estimated parameters\n%%\n');
                                    fprintf(fid,'OriginalTheta = DataStructure.OriginalTheta;\n');
                                    fprintf(fid,'%%\n%% vector with the scenario values of the estimated parameters\n%%\n');
                                    fprintf(fid,'ParameterScenarioTheta = DataStructure.ParameterScenarioTheta;\n');
                                    fprintf(fid,'%%\n%% vector with the original values of the calibrated parameters\n%%\n');
                                    fprintf(fid,'CalibratedParams = DataStructure.CalibratedParams;\n');
                                    fprintf(fid,'%%\n%% vector with the scenario values of the calibrated parameters\n%%\n');
                                    fprintf(fid,'ParameterScenarioCalibratedParams = DataStructure.ParameterScenarioCalibratedParams;\n');
                                    fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                    %
                                    % end the script file
                                    %
                                    fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                    fclose(fid);
                                    %
                                    txt = ['The parameter scenario data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: the 5 original and the 5 scenario state space matrices (A, H, R, F, B0, AScenario, HScenario, RScenario, FScenario, B0Scenario), YNames (string matrix with the names of the observed variables), Y (matrix with the observed variables), YScenario (matrix with the observed variables under the parameter scenario with unchanged shocks), DateVector (vector with dates fvor the observed variables), StateVariableNames (string matrix with the names of the state variables), OriginalStates (matrix with smooth estimates of the states for the original parameters), ScenarioStates (matrix with state variables under the parameter scenario), ShockNames (string matrix with the names of the economic shocks), OriginalShocks (matrix with the smooth estimates of the economic shocks using the original parameters), ScenarioShocks (matrix with smooth estimates of the economic shocks using the scenario parameters),' BreakEntry ' ManShocks (matrix with the manipulated values of the economic shocks under the extended scenario), ManStates (matrix with the manipulated values of the state variables under the extended scenario), ShockDateVector (vector with dates for the shocks), ParameterNames (string matrix with the names of the estimated and the calibrated parameters), OriginalTheta (vector with the original values of the estimated parameters), ParameterScenarioTheta (vector with the scenario values of the estimated parameters), CalibratedParams (vector with the original values of the calibrated parameters), and ParameterScenarioCalibratedParams (vector with the scenario values of the calibrated parameters). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                    About(txt,'information','Parameter Scenario',500,600,CurrINI);
                                 end;
                                 %
                                 % time to show the results
                                 %
                                 GraphParameterScenarioDLG('init',Y,YScenario,OriginalShocks,ScenarioShocks,ManShocks,ShockNames,OriginalStates,ScenarioStates,ManStates,OrigDateVector,BreakPeriod,AnnualizeData,AnnualStr,'Posterior Mode',DSGEModel,CurrINI);
                              end;
                           end;
                        end;
                     else;
                        txt = 'The values for all parameters under the scenario are equal to the posterior mode values and any original calibrated values. This makes the scenario pointless and YADA has therefore aborted.';
                        About(txt,'information','Parameter Scenario - Posterior Mode',200,500,CurrINI);
                     end;
                  end;
               else;
                  txt = 'You didn''t select any parameters to change for the "Parameter Scenario". This tool requires that you select at least one parameter to change.';
                  About(txt,'information','Parameter Scenario - Posterior Mode',200,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the impulse responses before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Impulse Responses - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               %
               % Try to calculate impulse responses. First, however, the code needs to check if a unique convergent
               % solution exists.
               %
               [IRStructure,mcode] = CalculateDSGEImpulseResponse(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0);
               %
               % If status~=1, the model doesn't have a unique convergent solutions
               %
               if mcode==1;
                  ShockType = 'individual';
                  if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                     txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                     answer = Query(txt,'question',140,'Original Impulse Responses - Initial Values',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups);
                        Ksi = zeros(size(IRStructure.Ksi,1),size(IMatrix,2),size(IRStructure.Ksi,3));
                        Y = zeros(size(IRStructure.Y,1),size(IMatrix,2),size(IRStructure.Y,3));
                        for i=1:size(IRStructure.Y,3);
                           Ksi(:,:,i) = IRStructure.Ksi(:,:,i)*IMatrix;
                           Y(:,:,i) = IRStructure.Y(:,:,i)*IMatrix;
                        end;
                        ShockType = 'group';
                        IRStructure.Ksi = Ksi;
                        IRStructure.Y = Y;
                     end;
                  end;
                  %
                  % Display the impulse responses
                  %
                  IRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Initial Values','Original',ShockType);
               else;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Impulse Responses - Initial Values',200,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRAnnualInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the impulse responses before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Impulse Responses - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               %
               % Try to calculate impulse responses. First, however, the code needs to check if a unique convergent
               % solution exists.
               %
               [IRStructure,mcode] = CalculateDSGEImpulseResponse(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,2);
               %
               % If status~=1, the model doesn't have a unique convergent solutions
               %
               if mcode==1;
                  ShockType = 'individual';
                  if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                     txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                     answer = Query(txt,'question',140,'Annualized Impulse Responses - Initial Values',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups);
                        Ksi = zeros(size(IRStructure.Ksi,1),size(IMatrix,2),size(IRStructure.Ksi,3));
                        Y = zeros(size(IRStructure.Y,1),size(IMatrix,2),size(IRStructure.Y,3));
                        for i=1:size(IRStructure.Y,3);
                           Ksi(:,:,i) = IRStructure.Ksi(:,:,i)*IMatrix;
                           Y(:,:,i) = IRStructure.Y(:,:,i)*IMatrix;
                        end;
                        ShockType = 'group';
                        IRStructure.Ksi = Ksi;
                        IRStructure.Y = Y;
                     end;
                  end;
                  %
                  % Display the impulse responses
                  %
                  IRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Initial Values','Annualized',ShockType);
               else;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Impulse Responses - Initial Values',200,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRLevelsInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the impulse responses of the levels data before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Impulse Responses - Initial Values',500,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               %
               % Try to calculate impulse responses. First, however, the code needs to check if a unique convergent
               % solution exists.
               %
               [IRStructure,mcode] = CalculateDSGEImpulseResponse(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,1);
               %
               % If status~=1, the model doesn't have a unique convergent solutions
               %
               if mcode==1;
                  %
                  % Display the impulse responses
                  %
                  ShockType = 'individual';
                  if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                     txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                     answer = Query(txt,'question',140,'Levels Impulse Responses - Initial Values',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups);
                        Ksi = zeros(size(IRStructure.Ksi,1),size(IMatrix,2),size(IRStructure.Ksi,3));
                        Y = zeros(size(IRStructure.Y,1),size(IMatrix,2),size(IRStructure.Y,3));
                        for i=1:size(IRStructure.Y,3);
                           Ksi(:,:,i) = IRStructure.Ksi(:,:,i)*IMatrix;
                           Y(:,:,i) = IRStructure.Y(:,:,i)*IMatrix;
                        end;
                        ShockType = 'group';
                        IRStructure.Ksi = Ksi;
                        IRStructure.Y = Y;
                     end;
                  end;
                  IRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Initial Values','Levels',ShockType);
               else;
                  if mcode==2;
                     mcodeStr = 'Roots are not correctly computed by real_schur.'
                  elseif mcode==3;
                     mcodeStr = 'Too many big roots.';
                  elseif mcode==35;
                     mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                  elseif mcode==4;
                     mcodeStr = 'Too few big roots.';
                  elseif mcode==45;
                     mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                  elseif mcode==5;
                     mcodeStr = 'q(:,right) is singular.';
                  elseif mcode==61;
                     mcodeStr = 'Too many exact shiftrights.';
                  elseif mcode==62;
                     mcodeStr = 'Too many numeric shiftrights.';
                  elseif mcode==7;
                     mcodeStr = 'Infinite or NaN values detected.';
                  elseif mcode==8;
                     mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                  elseif mcode==-1;
                     mcodeStr = 'No stable solution.';
                  elseif mcode==-2;
                     mcodeStr = 'Too many large generalized eigenvalues.';
                  elseif mcode==-3;
                     mcodeStr = 'Too few large generalized eigenvalues.';
                  else;
                     mcodeStr = 'Return code not properly specified.';
                  end;
                  if DSGEModel.ModelSolver==1;
                     SolverStr = 'AiM';
                  elseif DSGEModel.ModelSolver==2;
                     SolverStr = 'Klein';
                  elseif DSGEModel.ModelSolver==3;
                     SolverStr = 'Gensys';
                  elseif DSGEModel.ModelSolver==4;
                     SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                  end;
                  %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                  txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                  About(txt,'information','Impulse Responses - Initial Values',200,500,CurrINI);
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % are we using annualized data, levels, or the original
         %
         if gcbo==controls.irfunctions.prior;
            AnnualizeData = 0;
            LevelsData = 0;
            QueryHead = 'Impulse Responses For Prior - Original Data';
            VarType = 'Original';
         elseif gcbo==controls.irfunctions.lev_prior;
            AnnualizeData = 0;
            LevelsData = 1;
            QueryHead = 'Impulse Responses For Prior - Levels Data';
            VarType = 'Levels';
         else;
            AnnualizeData = 1;
            LevelsData = 0;
            QueryHead = 'Impulse Responses For Prior - Annualized Data';
            VarType = 'Annualized';
         end;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection',QueryHead,'','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % First we check if the calculations have already been performed
            %
            if IRFsExist(DSGEModel,AnnualizeData,LevelsData,NumPriorDraws,NumPriorDraws,1,0,0)==1;
               txt = 'YADA has located results for the impulse responses on disk. Would you like to use these results?';
               answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               DoneCalc = 1;
            else;
               %
               % compute the impulse responses for all the posterior draws
               %
               DoneCalc = DSGEImpulseResponses(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,AnnualizeData,LevelsData,NumPriorDraws,1,0);
            end;
            %
            if DoneCalc==1;
               ShockType = 'Individual';
               if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                  txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                  answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     ShockType = 'Group';
                  end;
               end;
               %
               % compute the distribution for the impulse responses
               %
               IRFs = DSGEIRFsDistribution(DSGEModel,CurrINI,AnnualizeData,LevelsData,lower(ShockType),NumPriorDraws,NumPriorDraws,1,0,[]);
               %
               % ask if we should save results to disk
               %
               txt = 'Would you like to save the impulse response functions to file?';
               answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  YNames = DSGEModel.VariableNames;
                  file = FixFilePath([DSGEModel.OutputDirectory '\ImpulseResponsePriorDist-' VarType '-' ShockType '-' int2str(NumPriorDraws) '.mat']);
                  if strcmp(lower(ShockType),'individual')==1;
                     ShockNames = DSGEModel.StateShockNames;
                     save(file,'YNames','ShockNames','IRFs');
                  else;
                     ShockGroupNames = DSGEModel.ShockGroupNames;
                     save(file,'YNames','ShockGroupNames','IRFs');
                  end;
                  %
                  % create a matlab file to read the data
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\IRPriorDist' VarType ShockType int2str(NumPriorDraws) '.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                  fprintf(fid,'YNames = DataStructure.YNames;\n');
                  if strcmp(lower(ShockType),'individual')==1;
                     fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                     fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                     ShockStr = 'ShockNames (string matrix with the names of the economic shocks)';
                  else;
                     fprintf(fid,'%%\n%% string matrix with the names of the shock groups\n%%\n');
                     fprintf(fid,'ShockGroupNames = DataStructure.ShockGroupNames;\n');
                     ShockStr = 'ShockGroupNames (string matrix with the names of the shock groups)';
                  end;
                  fprintf(fid,'%%\n%% structure with fields Y containing the prior mean\n%% impulse responses, and vector field Quantiles\n%% with subfields Y and percent holding quantile data and quantile percent.\n%%\n');
                  fprintf(fid,'IRFs = DataStructure.IRFs;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  txt = ['The impulse response data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), ' ShockStr ', and IRFs (a structure with prior mean and quantile data for the impulse responses). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information',QueryHead,200,500,CurrINI);
               end;
               %
               % display the results
               %
               IRDistributionGraphDLG('init',DSGEModel,CurrINI,IRFs,VarType,lower(ShockType),0,1,[]);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % we can now compute the impulse response functions up to a given horizon
            %
            IRStructure = CalculateDSGEImpulseResponse(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,0);
            %
            % Display the impulse responses
            %
            if (isempty(IRStructure)==0)&(DSGEModel.RunDriver==0);
               %
               % check if the user would like to plot shocks or shock groups
               %
               ShockType = 'individual';
               if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                  txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                  answer = Query(txt,'question',140,'Original Impulse Responses - Posterior Mode',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups);
                     Ksi = zeros(size(IRStructure.Ksi,1),size(IMatrix,2),size(IRStructure.Ksi,3));
                     Y = zeros(size(IRStructure.Y,1),size(IMatrix,2),size(IRStructure.Y,3));
                     for i=1:size(IRStructure.Y,3);
                        Ksi(:,:,i) = IRStructure.Ksi(:,:,i)*IMatrix;
                        Y(:,:,i) = IRStructure.Y(:,:,i)*IMatrix;
                     end;
                     ShockType = 'group';
                     IRStructure.Ksi = Ksi;
                     IRStructure.Y = Y;
                  end;
               end;
               IRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Posterior Mode','Original',ShockType);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'IRAnnualPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % we can now compute the impulse response functions up to a given horizon
            %
            IRStructure = CalculateDSGEImpulseResponse(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,2);
            %
            % Display the impulse responses
            %
            if (isempty(IRStructure)==0)&(DSGEModel.RunDriver==0);
               ShockType = 'individual';
               if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                  txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                  answer = Query(txt,'question',140,'Annualized Impulse Responses - Posterior Mode',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups);
                     Ksi = zeros(size(IRStructure.Ksi,1),size(IMatrix,2),size(IRStructure.Ksi,3));
                     Y = zeros(size(IRStructure.Y,1),size(IMatrix,2),size(IRStructure.Y,3));
                     for i=1:size(IRStructure.Y,3);
                        Ksi(:,:,i) = IRStructure.Ksi(:,:,i)*IMatrix;
                        Y(:,:,i) = IRStructure.Y(:,:,i)*IMatrix;
                     end;
                     ShockType = 'group';
                     IRStructure.Ksi = Ksi;
                     IRStructure.Y = Y;
                  end;
               end;
               IRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Posterior Mode','Annualized',ShockType);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'IRLevelsPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % we can now compute the impulse response functions up to a given horizon
            %
            IRStructure = CalculateDSGEImpulseResponse(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,1);
            %
            % Display the impulse responses
            %
            if (isempty(IRStructure)==0)&(DSGEModel.RunDriver==0);
               ShockType = 'individual';
               if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                  txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                  answer = Query(txt,'question',140,'Levels Impulse Responses - Posterior Mode',500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     IMatrix = SubIdentityMatrix(DSGEModel.ShockGroups);
                     Ksi = zeros(size(IRStructure.Ksi,1),size(IMatrix,2),size(IRStructure.Ksi,3));
                     Y = zeros(size(IRStructure.Y,1),size(IMatrix,2),size(IRStructure.Y,3));
                     for i=1:size(IRStructure.Y,3);
                        Ksi(:,:,i) = IRStructure.Ksi(:,:,i)*IMatrix;
                        Y(:,:,i) = IRStructure.Y(:,:,i)*IMatrix;
                     end;
                     ShockType = 'group';
                     IRStructure.Ksi = Ksi;
                     IRStructure.Y = Y;
                  end;
               end;
               IRGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Posterior Mode','Levels',ShockType);
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         set(controls.about,'UserData',0);
      end;
   case 'IRPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            theta = ModeData.thetaMode;
%            if isfield(ModeData,'KalmanFirstObservation')==1;
%               DSGEModel.KalmanFirstObservation = ModeData.KalmanFirstObservation;
%               FirstPeriod = ModeData.DSGEModel.FirstPeriod;
%               LastPeriod = ModeData.DSGEModel.LastPeriod;
%            else;
%               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
%               DSGEModel.KalmanFirstObservation = round(((DSGEModel.InitializeKalman-1)/100)*(LastPeriod-FirstPeriod+1))+1;
%            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Impulse Responses','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % are we using annualized data, levels, or the original
                  %
                  if gcbo==controls.irfunctions.posterior;
                     AnnualizeData = 0;
                     LevelsData = 0;
                     QueryHead = 'Impulse Responses - Original Data';
                     VarType = 'Original';
                  elseif gcbo==controls.irfunctions.lev_posterior;
                     AnnualizeData = 0;
                     LevelsData = 1;
                     QueryHead = 'Impulse Responses - Levels Data';
                     VarType = 'Levels';
                  else;
                     AnnualizeData = 1;
                     LevelsData = 0;
                     QueryHead = 'Impulse Responses - Annualized Data';
                     VarType = 'Annualized';
                  end;
                  %
                  % First we check if the calculations have already been performed
                  %
                  if IRFsExist(DSGEModel,AnnualizeData,LevelsData,UsedDraws,NumDraws,CurrChain,1,0)==1;
                     txt = 'YADA has located results for the impulse responses on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the impulse responses for all the posterior draws
                     %
                     DoneCalc = DSGEImpulseResponses(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,AnnualizeData,LevelsData,NumDraws,CurrChain,1);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
                  %
                  % check if we should show any results
                  %
                  if DoneCalc==1;
                     ShockType = 'Individual';
                     if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                        txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           ShockType = 'Group';
                        end;
                     end;
                     %
                     % compute the distribution for the impulse responses
                     %
                     IRFs = DSGEIRFsDistribution(DSGEModel,CurrINI,AnnualizeData,LevelsData,lower(ShockType),UsedDraws,NumDraws,CurrChain,1,Weights);
                     %
                     % ask if we should save results to disk
                     %
                     txt = 'Would you like to save the impulse response functions to file?';
                     answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        YNames = DSGEModel.VariableNames;
                        file = FixFilePath([DSGEModel.OutputDirectory '\ImpulseResponseDist-' VarType '-' ShockType '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
                        if strcmp(lower(ShockType),'individual')==1;
                           ShockNames = DSGEModel.StateShockNames;
                           save(file,'YNames','ShockNames','IRFs');
                        else;
                           ShockGroupNames = DSGEModel.ShockGroupNames;
                           save(file,'YNames','ShockGroupNames','IRFs');
                        end;
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\IRDist' VarType ShockType int2str(CurrChain) int2str(UsedDraws) int2str(NumDraws) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        if strcmp(lower(ShockType),'individual')==1;
                           fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           ShockStr = 'ShockNames (string matrix with the names of the economic shocks)';
                        else;
                           fprintf(fid,'%%\n%% string matrix with the names of the shock groups\n%%\n');
                           fprintf(fid,'ShockGroupNames = DataStructure.ShockGroupNames;\n');
                           ShockStr = 'ShockGroupNames (string matrix with the names of the shock groups)';
                        end;
                        fprintf(fid,'%%\n%% structure with fields Y containing the posterior mean\n%% impulse responses, and vector field Quantiles\n%% with subfields Y and percent holding quantile data and quantile percent.\n%%\n');
                        fprintf(fid,'IRFs = DataStructure.IRFs;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The impulse response data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), ' ShockStr ', and IRFs (a structure with posterior mean and quantile data for the impulse responses). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',QueryHead,200,500,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     IRDistributionGraphDLG('init',DSGEModel,CurrINI,IRFs,VarType,lower(ShockType),1,1,[]);
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRStateVarsPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         AnnualizeData = 0;
         LevelsData = 0;
         QueryHead = 'Impulse Responses For Prior - State Variables';
         VarType = 'Original';
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection',QueryHead,'','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            %
            % First we check if the calculations have already been performed
            %
            if IRFsExist(DSGEModel,AnnualizeData,LevelsData,NumPriorDraws,NumPriorDraws,1,0,1)==1;
               txt = 'YADA has located results for the impulse responses on disk. Would you like to use these results?';
               answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
            else;
               answer = 'No';
            end;
            if strcmp(lower(answer),'yes')==1;
               DoneCalc = 1;
            else;
               %
               % compute the impulse responses for all the posterior draws
               %
               DoneCalc = DSGEImpulseResponses(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,AnnualizeData,LevelsData,NumPriorDraws,1,0);
            end;
            %
            % check if we should show any results
            %
            if DoneCalc==1;
               ShockType = 'Individual';
               if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                  txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                  answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  if strcmp(lower(answer),'yes')==1;
                     ShockType = 'Group';
                  end;
               end;
               %
               % compute the distribution for the impulse responses
               %
               IRFs = StateVarIRFsDistribution(DSGEModel,CurrINI,lower(ShockType),NumPriorDraws,NumPriorDraws,1,0,[]);
               %
               % ask if we should save results to disk
               %
               txt = 'Would you like to save the impulse response functions for the state variables to file?';
               answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
               if strcmp(lower(answer),'yes')==1;
                  StateVariableNames = DSGEModel.StateVariableNames;
                  file = FixFilePath([DSGEModel.OutputDirectory '\ImpulseResponsePriorDist-' VarType '-' ShockType '-' int2str(NumPriorDraws) '.mat']);
                  if strcmp(lower(ShockType),'individual')==1;
                     ShockNames = DSGEModel.StateShockNames;
                     save(file,'StateVariableNames','ShockNames','IRFs');
                  else;
                     ShockGroupNames = DSGEModel.ShockGroupNames;
                     save(file,'StateVariableNames','ShockGroupNames','IRFs');
                  end;
                  %
                  % create a matlab file to read the data
                  %
                  mfile = FixFilePath([DSGEModel.OutputDirectory '\IRPriorDist' VarType ShockType int2str(NumPriorDraws) '.m']);
                  fid = fopen(mfile,'wt');
                  fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                  fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                  fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                  fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                  if strcmp(lower(ShockType),'individual')==1;
                     fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                     fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                     ShockStr = 'ShockNames (string matrix with the names of the economic shocks)';
                  else;
                     fprintf(fid,'%%\n%% string matrix with the names of the shock groups\n%%\n');
                     fprintf(fid,'ShockGroupNames = DataStructure.ShockGroupNames;\n');
                     ShockStr = 'ShockGroupNames (string matrix with the names of the shock groups)';
                  end;
                  fprintf(fid,'%%\n%% structure with fields Y containing the prior mean\n%% impulse responses, and vector field Quantiles\n%% with subfields Y and percent holding quantile data and quantile percent.\n%%\n');
                  fprintf(fid,'IRFs = DataStructure.IRFs;\n');
                  fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                  %
                  % end the script file
                  %
                  fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                  fclose(fid);
                  txt = ['The impulse response data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: StateVariableNames (string matrix with the names of the state variables), ' ShockStr ', and IRFs (a structure with prior mean and quantile data for the impulse responses). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                  About(txt,'information',QueryHead,200,500,CurrINI);
               end;
               %
               % display the results
               %
               StateVarIRDistributionGraphDLG('init',DSGEModel,CurrINI,IRFs,lower(ShockType),0);
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRStateVarsPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            theta = ModeData.thetaMode;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Impulse Responses','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check which draws we should use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                     if length(UseDraws)<NumDraws;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        UsedDraws = length(UseDraws);
                     else;
                        UsedDraws = NumDraws;
                     end;
                  else;
                     UsedDraws = NumDraws;
                  end;
                  %
                  % are we using annualized data, levels, or the original
                  %
                  AnnualizeData = 0;
                  LevelsData = 0;
                  QueryHead = 'Impulse Responses - State Variables';
                  VarType = 'StateVars';
                  %
                  % First we check if the calculations have already been performed
                  %
                  if IRFsExist(DSGEModel,AnnualizeData,LevelsData,UsedDraws,NumDraws,CurrChain,1,1)==1;
                     txt = 'YADA has located results for the impulse responses on disk. Would you like to use these results?';
                     answer = Query(txt,'question',150,QueryHead,450,'no',CurrINI);
                  else;
                     answer = 'No';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     DoneCalc = 1;
                  else;
                     %
                     % compute the impulse responses for all the posterior draws
                     %
                     DoneCalc = DSGEImpulseResponses(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,AnnualizeData,LevelsData,NumDraws,CurrChain,1);
                  end;
                  %
                  % clear everything
                  %
                  clear('thetaPostSample');
                  %
                  % check if we should show any results
                  %
                  if DoneCalc==1;
                     ShockType = 'Individual';
                     if length(DSGEModel.ShockGroups)>max(DSGEModel.ShockGroups);
                        txt = 'Would you like to view plots of impulse responses for shock groups instead of for the individual shocks?';
                        answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           ShockType = 'Group';
                        end;
                     end;
                     %
                     % compute the distribution for the impulse responses
                     %
                     IRFs = StateVarIRFsDistribution(DSGEModel,CurrINI,lower(ShockType),UsedDraws,NumDraws,CurrChain,1,Weights);
                     %
                     % ask if we should save results to disk
                     %
                     txt = 'Would you like to save the impulse response functions for the state variables to file?';
                     answer = Query(txt,'question',140,'Save - Impulse Responses',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        file = FixFilePath([DSGEModel.OutputDirectory '\ImpulseResponseDist-' VarType '-' ShockType '-' int2str(CurrChain) '-' int2str(UsedDraws) '-' int2str(NumDraws) '.mat']);
                        if strcmp(lower(ShockType),'individual')==1;
                           ShockNames = DSGEModel.StateShockNames;
                           save(file,'StateVariableNames','ShockNames','IRFs');
                        else;
                           ShockGroupNames = DSGEModel.ShockGroupNames;
                           save(file,'StateVariableNames','ShockGroupNames','IRFs');
                        end;
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\IRDist' VarType ShockType int2str(CurrChain) int2str(UsedDraws) int2str(NumDraws) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        if strcmp(lower(ShockType),'individual')==1;
                           fprintf(fid,'%%\n%% string matrix with the names of the economic shocks\n%%\n');
                           fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                           ShockStr = 'ShockNames (string matrix with the names of the economic shocks)';
                        else;
                           fprintf(fid,'%%\n%% string matrix with the names of the shock groups\n%%\n');
                           fprintf(fid,'ShockGroupNames = DataStructure.ShockGroupNames;\n');
                           ShockStr = 'ShockGroupNames (string matrix with the names of the shock groups)';
                        end;
                        fprintf(fid,'%%\n%% structure with fields Ksi containing the posterior mean\n%% impulse responses, and vector field Quantiles\n%% with subfields Ksi and percent holding quantile data and quantile percent.\n%%\n');
                        fprintf(fid,'IRFs = DataStructure.IRFs;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The impulse response data have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: StateVariableNames (string matrix with the names of the state variables), ' ShockStr ', and IRFs (a structure with posterior mean and quantile data for the impulse responses). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information',QueryHead,200,500,CurrINI);
                     end;
                     %
                     % display the results
                     %
                     StateVarIRDistributionGraphDLG('init',DSGEModel,CurrINI,IRFs,lower(ShockType),1);
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;

         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRPermanentShockInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the forecast error decompositions before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Forecast Error Decompositions - Initial Values',450,'no',CurrINI);
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  ErrorStr = '';
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            %
            if InitStatus==1;
               %
               % now we need to select the model variable to target, the target (policy) equation
               % and the permanent shock value
               %
               [action,TargetVariablePosition,TargetEqPosition,TargetShockValue,ResponseDenominatorPosition] = PermanentShockSelectionDlg('init',DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.TargetVariablePosition,DSGEModel.TargetEqPosition,DSGEModel.TargetShockValue,strvcat(DSGEModel.VariableNames,'None'),DSGEModel.ResponseDenominatorPosition,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  %
                  % check to make sure that TargetShockValue is valid
                  %
                  if (isempty(TargetShockValue)==0)&(isinf(TargetShockValue)==0)&(isnan(TargetShockValue)==0);
                     DSGEModel.TargetVariablePosition = TargetVariablePosition;
                     DSGEModel.TargetEqPosition = TargetEqPosition;
                     DSGEModel.TargetShockValue = TargetShockValue;
                     DSGEModel.ResponseDenominatorPosition = ResponseDenominatorPosition;
                     %
                     % we can now compute the IRFs for the permanent shock
                     %
                     [IRStructure,mcode] = CalculateDSGEPermanentIRFs(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
                     %
                     % checkl if the model solution is unique and covergent
                     %
                     if mcode==1;
                        %
                        % we're OK
                        %
                        if isempty(IRStructure)==0;
                           %
                           % check if the user would like to save the results to file
                           %
                           txt = ['Would you like to save the responses to a permanent shock to the target variable for ' StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:)) ' at the initial parameter values to file for the state and observed variables?'];
                           answer = Query(txt,'question',140,'Save - Permanent Shock Responses',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              file = FixFilePath([DSGEModel.OutputDirectory '\PermanentShockResponses-' int2str(IRStructure.RelativeResponse) '-' int2str(DSGEModel.TargetVariablePosition) '.' int2str(DSGEModel.TargetEqPosition) '-InitialValues.mat']);
                              YNames = DSGEModel.VariableNames;
                              StateVariableNames = DSGEModel.StateVariableNames;
                              save(file,'IRStructure','YNames','StateVariableNames');
                              %
                              % create a matlab file to read the data
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\PermShockResp' int2str(IRStructure.RelativeResponse) int2str(DSGEModel.TargetVariablePosition) int2str(DSGEModel.TargetEqPosition) 'InitVal.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                              fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                              if IRStructure.RelativeResponse==0;
                                 fprintf(fid,'%%\n%% structure with fields Ksi, Y and AccY with the responses\n%% to the state variables, the observed and the\n%% accumulated responses to the observed variables \n%%\n%% All responses are measured relative to the last period response of %s.\n',StringTrim(DSGEModel.VariableNames(DSGEModel.ResponseDenominatorPosition,:)));
                              else;
                                 fprintf(fid,'%%\n%% structure with fields Ksi, Y and AccY with the responses\n%% to the state variables, the observed and the\n%% accumulated responses to the observed variables \n%%\n');
                              end;
                              fprintf(fid,'IRStructure = DataStructure.IRStructure;\n');
                              fprintf(fid,'%%\n%% The permanent response concerns the target variable for %s\n',StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:)));
                              fprintf(fid,'%% The size of the shock is %0.4f\n',DSGEModel.TargetShockValue);
                              if IRStructure.RelativeResponse==1;
                                 fprintf(fid,'%% The responses are measured relative to the h+1 (last) response to %s\n',StringTrim(DSGEModel.VariableNames(DSGEModel.ResponseDenominatorPosition,:)));
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              txt = ['The permanent shock responses to the state variables and the observed variables at the initial values have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the variables), and IRStructure (a structure with fields Ksi a matrix of dimension rxh+1 for the state variables, Y a matrix of dimension nxh+1 for the observed, and AccY a matrix of dimension nxh+1 for the accumulated observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','Permanent Shock Responses - Initial Values',200,500,CurrINI);
                           end;
                           %
                           % display the responses
                           %
                           PermanentShockGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Initial Values');
                        end;
                     else;
                        if mcode==2;
                           mcodeStr = 'Roots are not correctly computed by real_schur.'
                        elseif mcode==3;
                           mcodeStr = 'Too many big roots.';
                        elseif mcode==35;
                           mcodeStr = 'Too many big roots, and q(:,right) is singular.';
                        elseif mcode==4;
                           mcodeStr = 'Too few big roots.';
                        elseif mcode==45;
                           mcodeStr = 'Too few big roots, and q(:,right) is singular.';
                        elseif mcode==5;
                           mcodeStr = 'q(:,right) is singular.';
                        elseif mcode==61;
                           mcodeStr = 'Too many exact shiftrights.';
                        elseif mcode==62;
                           mcodeStr = 'Too many numeric shiftrights.';
                        elseif mcode==7;
                           mcodeStr = 'Infinite or NaN values detected.';
                        elseif mcode==8;
                           mcodeStr = 'The function "compute_aim_matrices" returns complex numbers.';
                        elseif mcode==-1;
                           mcodeStr = 'No stable solution.';
                        elseif mcode==-2;
                           mcodeStr = 'Too many large generalized eigenvalues.';
                        elseif mcode==-3;
                           mcodeStr = 'Too few large generalized eigenvalues.';
                        else;
                           mcodeStr = 'Return code not properly specified.';
                        end;
                        if DSGEModel.ModelSolver==1;
                           SolverStr = 'AiM';
                        elseif DSGEModel.ModelSolver==2;
                           SolverStr = 'Klein';
                        elseif DSGEModel.ModelSolver==3;
                           SolverStr = 'Gensys';
                        elseif DSGEModel.ModelSolver==4;
                           SolverStr = ['Own algorithm (' GetName(DSGEModel.ModelSolverFile) ')'];
                        end;
                        %SetLogText(strvcat(['   STATUS: ' mcodeStr],' '));
                        txt = ['The ' SolverStr ' solver provided the return code: ' num2str(mcode) ', i.e., "' mcodeStr '" You have to change your initial parameter values.'];
                        About(txt,'information','Forecast Error Decompositions - Initial Values',120,500,CurrINI);
                     end;
                  else;
                     txt = ['The value you selected for the permanent shock (' str2num(TargetShockValue) ') is not valid.'];
                     About(txt,'information','Permanent Shocks - Bad Value',500,600,CurrINI);
                  end;
               end;
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'IRPermanentShockPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % File with the data needed to calculate the impulse responses
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            %
            % now we need to select the model variable to target, the target (policy) equation
            % and the permanent shock value
            %
            [action,TargetVariablePosition,TargetEqPosition,TargetShockValue,ResponseDenominatorPosition] = PermanentShockSelectionDlg('init',DSGEModel.StateVariableNames,DSGEModel.StateEquationNames,DSGEModel.TargetVariablePosition,DSGEModel.TargetEqPosition,DSGEModel.TargetShockValue,strvcat(DSGEModel.VariableNames,'None'),DSGEModel.ResponseDenominatorPosition,CurrINI);
            if strcmp(lower(action),'ok')==1;
               %
               % check to make sure that TargetShockValue is valid
               %
               if (isempty(TargetShockValue)==0)&(isinf(TargetShockValue)==0)&(isnan(TargetShockValue)==0);
                  DSGEModel.TargetVariablePosition = TargetVariablePosition;
                  DSGEModel.TargetEqPosition = TargetEqPosition;
                  DSGEModel.TargetShockValue = TargetShockValue;
                  DSGEModel.ResponseDenominatorPosition = ResponseDenominatorPosition;
                  %
                  % we can now compute the IRFs for the permanent shock
                  %
                  IRStructure = CalculateDSGEPermanentIRFs(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI);
                  %
                  % Display the impulse responses
                  %
                  if isempty(IRStructure)==0;
                     %
                     % check if the user would like to save the results to file
                     %
                     txt = ['Would you like to save the responses to a permanent shock to the target variable for ' StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:)) ' at posterior mode to file for the state and observed variables?'];
                     answer = Query(txt,'question',140,'Save - Permanent Shock Responses',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        file = FixFilePath([DSGEModel.OutputDirectory '\PermanentShockResponses-' int2str(IRStructure.RelativeResponse) '-' int2str(DSGEModel.TargetVariablePosition) '.' int2str(DSGEModel.TargetEqPosition) '-PosteriorMode.mat']);
                        YNames = DSGEModel.VariableNames;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        save(file,'IRStructure','YNames','StateVariableNames');
                        %
                        % create a matlab file to read the data
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\PermShockResp' int2str(IRStructure.RelativeResponse) int2str(DSGEModel.TargetVariablePosition) int2str(DSGEModel.TargetEqPosition) 'PostMode.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the state variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        if IRStructure.RelativeResponse==0;
                           fprintf(fid,'%%\n%% structure with fields Ksi, Y and AccY with the responses\n%% to the state variables, the observed and the\n%% accumulated responses to the observed variables \n%%\n%% All responses are measured relative to the last period response of %s.\n',StringTrim(DSGEModel.VariableNames(DSGEModel.ResponseDenominatorPosition,:)));
                        else;
                           fprintf(fid,'%%\n%% structure with fields Ksi, Y and AccY with the responses\n%% to the state variables, the observed and the\n%% accumulated responses to the observed variables \n%%\n');
                        end;
                        fprintf(fid,'IRStructure = DataStructure.IRStructure;\n');
                        fprintf(fid,'%%\n%% The permanent response concerns the target variable for %s\n',StringTrim(DSGEModel.StateVariableNames(DSGEModel.TargetVariablePosition,:)));
                        fprintf(fid,'%% The size of the shock is %0.4f\n',DSGEModel.TargetShockValue);
                        if IRStructure.RelativeResponse==1;
                           fprintf(fid,'%% The responses are measured relative to the h+1 (last) response to %s\n',StringTrim(DSGEModel.VariableNames(DSGEModel.ResponseDenominatorPosition,:)));
                        end;
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        txt = ['The permanent shock responses to the state variables and the observed variables at posterior mode have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 3 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the variables), and IRStructure (a structure with fields Ksi a matrix of dimension rxh+1 for the state variables, Y a matrix of dimension nxh+1 for the observed, and AccY a matrix of dimension nxh+1 for the accumulated observed variables). To access this data you may run the matlab script file "' GetFilename(mfile) '".'];
                        About(txt,'information','Permanent Shock Responses - Posterior Mode',200,500,CurrINI);
                     end;
                     %
                     % display the responses
                     %
                     PermanentShockGraphDLG('init',CurrINI,DSGEModel,IRStructure,'Posterior Mode');
                  end;
               else;
                  txt = ['The value you selected for the permanent shock (' str2num(TargetShockValue) ') is not valid.'];
                  About(txt,'information','Permanent Shocks - Bad Value',500,600,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of ToolsMenuFunctions.m
%
