function ZLBForecastToolsMenuFunctions(selector,maingui)
% ZLBForecastToolsMenuFunctions: switch function for running prediction functions on the tools menu that are
%                                subject to the zero lower bound.
%
% USAGE:
%
%       ZLBForecastToolsMenuFunctions(selector,maingui)
%
% REQUIRED INPUT: selector (string) that determines which sub-function should be applied.
%
% OPTIONAL INPUT: maingui (handle) to the YADA GUI.
%
%
%                       Written by: Anders Warne
%                                   New Area Wide Model Project
%                                   Forecasting and Policy Modelling Division
%                                   European Central Bank (ECB)
%                                   Email: anders.warne@ecb.europa.eu
%                                   Copyright  2006-2024 European Central Bank.
%
%                       First version: August 18, 2014.
%                        This version: March 18, 2024.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% LICENSE INFORMATION:
%
%      YADA is free software: you can redistribute it and/or modify
%      it under the terms of the GNU General Public License as published by
%      the Free Software Foundation, either version 3 of the License, or
%      (at your option) any later version.
%
%      This program is distributed in the hope that it will be useful,
%      but WITHOUT ANY WARRANTY; without even the implied warranty of
%      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%      GNU General Public License for more details.
%
%      You should have received a copy of the GNU General Public License
%      along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%      YADA is released under the GNU General Public License, Version 3,
%      29 June 2007 <http://www.gnu.org/licenses/>. The current release of
%      the program was last modified by the ECB on the "This version" date
%      above.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% CHANGELOG:
%
% * 19-08-2014: Continued work on the posterior mode function for conditional forecasts.
%
% * 20-08-2014: Continued work on the posterior mode function for conditional forecasts.
%
% * 21-08-2014: Continued work on the posterior mode function for conditional forecasts.
%
% * 22-08-2014: Continued work on the posterior mode function for conditional forecasts.
%
% * 26-08-2014: Took into account that "ZLBSelectionDLG" now includes the input and output variable "REqPosition".
%
% * 03-09-2014: Took into account that "ZLBDSGEPredictionPathsTheta" now includes the "IsConditional" input variable.
%
% * 04-09-2014: Took into account that "ZLBDSGEPredictionPathsTheta" also takes the input variables "etaMean" and
%               "KsiTMean".
%
% * 05-09-2014: Continued work on the conditional forecast from the part where data are transformed.
%
% * 10-09-2014: Continued with the function from the point of calculating and presenting the prediction events.
%
% * 11-09-2014: Made sure that the filename for the results of the ZLB calculations for the posterior mode case
%               depends on the selected sample. Changed the field "RhatPosition" to "RtildePosition".
%
% * 16-09-2014: Took into account that the function "ZLBDSGEPredictionPathsTheta" now supports the output variable
%               "Data" with data on the suggested shocks, initial state, and measurement errors as well as
%               the used ones.
%
% * 22-10-2014: Made sure that "AIMData" is read from it's own mat-file rather than from the posterior mode mat-file.
%
% * 16-12-2014: Added the initial values case fpr the ZLB calculations subject to conditioning restrictions.
%
% * 14-01-2015: Updated the documentation.
%
% * 27-01-2016: Remember Shoah.
%
% * 21-03-2017: Updated the documentation.
%
% * 15-09-2017: Updated the documentation.
%
% * 08-01-2018: Updated the documentation.
%
% * 16-08-2018: Took into account the new error handling via the function YADALastError.
%
% * 29-08-2018: Updated the documentation.
%
% * 23-01-2019: Updated the documentation.
%
% * 10-01-2020: Updated the documentation.
%
% * 10-12-2020: Increased the size of the About dialog for ErrorStr.
%
% * 26-01-2021: Updated the documentation.
%
% * 22-02-2021: Made sure that NumPredPaths is not equal to 1 for the ZLB calculations.
%
% * 06-01-2022: Updated the documentation.
%
% * 24-08-2022: Began work on unconditional forecasts subject to the ZLB for the initial
%               values and the posterior mode values.
%
% * 25-08-2022: Finished work on the unconditional forecasts for the posterior mode and
%               initial values cases. Began work on the unconditional posterior distribution
%               case.
%
% * 29-08-2022: Continued work on the unconditional posterior distribution case.
%
% * 30-08-2022: Continued work on the unconditional posterior distribution case.
%
% * 01-09-2022: Continued work on the unconditional posterior predictive distribution.
%
% * 02-09-2022: Continued work on the above function.
%
% * 05-09-2022: Added the normal approximation for the predictive distribution using the
%               posterior draws.
%
% * 08-09-2022: Finished the unconditional posterior predictive distribution.
%
% * 09-09-2022: The unconditional posterior predictive distribution no longer uses the
%               AddionalDraws variable.
%
% * 05-10-2022: Recoded the unconditional predictive distribution for the initial values case.
%
% * 27-10-2022: Took into account that "ZLBPredictiveDistributionDLG" takes an additional
%               input variable.
%
% * 28-20-2022: Closed a command.
%
% * 05-01-2023: Updated the documentation.
%
% * 09-02-2023: Added driver support for the posterior mode case.
%
% * 17-02-2023: Removed all use of "AdditionalDraws".
%
% * 21-02-2023: Added the "maingui" input variable.
%
% * 23-02-2023: Returned to the ZLB conditional forecasts at posterior mode.
%
% * 24-02-2023: Continued work on the above function.
%
% * 27-02-2023: Continued work on the same function.
%
% * 03-03-2023: Continued work on the same function for the mixed case.
%
% * 14-03-2023: Added the initial values case for the ZLB conditional forecasts.
%
% * 15-03-2023: Updated the documentation, fixed some erroneous window names.
%
% * 16-03-2023: Began work on the posterior draws case for the ZLB conditional forecasts.
%
% * 17-03-2023: Added code for displaying the results for the ZLB conditional forecasts using
%               the posterior draws.
%
% * 20-03-2023: The direct control method is now fully supported.
%
% * 21-03-2023: Added call to Waggoner-Zha approach centered and not for the ZLB conditional forecasts
%               using the posterior draws. Also added the mixed case.
%
% * 22-03-2023: Added code for showing the results for the conditional forecasts using prior draws.
%
% * 23-03-2023: Finished the prior draws code for the conditional forecasts.
%
% * 21-11-2023: Added code for saving average shock values when the ZLB is binding for the posterior draws.
%
% * 23-11-2023: Took into account that "ZLBSelectionDLG" requires the "ZLBRuleType" input and output variable.
%               The mat-file names now also depend on the "ZLBRuleType" variable.
%
% * 16-01-2024: Added the function cases "UnconCRPSPriorDist" and "UnconCRPSPosteriorDist" for computing the
%               CRPS and ES using prior or posterior draws and the prediction paths subject to the ZLB.
%
% * 17-01-2024: Completed work on the posterior draws case for the CRPS and ES calculations using the unconditional
%               prediction paths.
%
% * 22-01-2024: Began work on the prior draws case for the CRPS and ES calculations using the unconditional
%               prediction paths.
%
% * 26-01-2024: Continued work on the prior draws function for the CRPS and ES calculations.
%
% * 29-01-2024: Finished the prior draws function for the CRPS and ES calculations. Took into account that
%               "PrintCRPS" now takes the IsZLB input variable and that "ComputeCRPSZLBDSGEPaths" doesn't
%               take the "Weights" input variable. Added the function cases "UnconPredHeightPriorDist" and
%               "UnconPredHeightPosteriorDist" for computing the predictive likelihood using prior or
%               posterior draws and the prediction paths subject to the ZLB.
%
% * 30-01-2024: Continued work on the posterior draws case for computing the predictive likelihood from
%               prediction paths subject to the ZLB.
%
% * 31-01-2024: Finished the posterior draws case.
%
% * 02-02-2024: Began and finished work on the prior draws case.
%
% * 06-02-2044: Added function cases for computing the predictive likelihood for the conditional forecasts
%               subject to the ZLB using prior or posterior draws. Finished the latter case.
%
% * 07-02-2024: Finished the prior draws case for the predictive likelihood with conditional forecasts
%               subject to the ZLB. Added function cases for computing the CRPS and ES for the conditional
%               forecasts subject to the ZLB using prior or posterior draws. Finished both cases.
%
% * 08-02-2024: Updated the documentation and fixed some issue with filenames of files that are saved on disk.
%
% * 22-02-2024: Added the function case for computing the unconditional predictive distribution under adaptive
%               learning and subject to the ZLB.
%
% * 23-02-2024: Continued work on the posterior draws case for adaptive learning.
%
% * 28-02-2024: Continued work on the posterior draws case for unconditional predictive distributions under
%               adaptive learning.
%
% * 05-03-2024: Continued work on the posterior draws case as the adaptive learning function is now ready for action.
%
% * 06-03-2024: Added function cases for the predictive likelihood and CRPS/ES computations based on the
%               unconditional predictive distributions under adaptive learning and subject to the ZLB using
%               the posterior draws.
%
% * 08-03-2024: Added the function cases for the predictive likelihood and CRPS/ES computations based on
%               the unconditional predictive distributions under adaptive learning and subject to the ZLB
%               using the prior draws.
%
% * 15-03-2024: Changed the lower vertical position of the TextGUI function from 32 pixels to 50.
%
% * 18-03-2024: Made sure that the PredictionEvent field is not changed unless user selects it.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
% collect some inportant data structures
%
if nargin==1;
   maingui = findobj('Type','figure','Tag','YADA');
end;
controls = get(maingui,'UserData');
CurrINI = get(controls.filemenu,'UserData');
DSGEModel = get(controls.open,'UserData');
OrigDSGEModel = get(controls.openmenu,'UserData');
if MatlabNumber<7;
   lasterr = '';
end;
%
switch selector
   case 'UnconPredInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         ErrorStr = '';
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the conditional predictive distributions of the observed variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Zero Lower Bound Forecasts - Initial Values',500,'no',CurrINI);
         end;
         %
         % check if we should check transformations functions
         %
         if gcbo==controls.predict.unconditional.transformed.zlb.initialvalues;
            ErrorStr = '';
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
            if isempty(ErrorStr)==0;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
               answer = 'no';
            end;
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if MatlabNumber>=8.0;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                  end;
               end;
            end;
            %
            if (InitStatus==1)&(strcmp(ErrorStr,'')==1);
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % determine the state equation names
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                  end;
               end;
               %
               % check if we should annualize or not
               %
               if gcbo==controls.predict.unconditional.original.zlb.initialvalues;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'Original';
               elseif gcbo==controls.predict.unconditional.annual.zlb.initialvalues;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'Annual';
               else;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'Transformed';
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               if NumPredPaths==1;
                  NumPredPaths = 100;
                  DSGEModel.NumPredPathsValue = 1;
               end;
               %
               % Determine the position of the monetary policy rate and select sample length
               % when the zero zero lower bound may be binding.
               %
               [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.RtildePosition = RtildePosition;
                  DSGEModel.REqPosition = REqPosition;
                  DSGEModel.ZLBRuleType = ZLBRuleType;
                  DSGEModel.ZLBSampleT = ZLBSampleT;
                  %
                  % ask if the user wants to center the original forecasts from the linear model on the actual data
                  %
                  txt = 'Would you like to recenter the structural shocks such that the predictive distribution of the DSGE model without anticipated shocks (zero values) is centered on the observed data?';
                  answer = Query(txt,'question',300,'Zero Lower Bound Forecasts - Initial Values',600,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     IsCentered = 0;
                     action = 'ok';
                     YDataStr = '';
                     SelectedYData = [];
                  else;
                     IsCentered = 1;
                     %
                     % check which actuals data to use
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check which data to use
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        YDataStr = 'Actuals from original sample';
                        action = 'ok';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Zero Lower Bound Forecasts - Initial Values','','',CurrINI);
                        if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        end;
                     end;
                     if strcmp(lower(action),'ok')==1;
                        MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                        MaxFH = min(MaxFH,MaxPH);
                        YData = YData(:,1:MaxFH);
                        MaxPH = MaxFH;
                        YDataStr = ['-' int2str(SelectedYData)];
                        if MaxFH==0;
                           txt = ['There are no data points for the observed variables over the forecast sample. YADA will therefore not be able to re-center the structural shocks. YADA will therefore abort the calculations.'];
                           About(txt,'information','Zero Lower Bound Forecasts - Initial Values',150,600,CurrINI);
                           action = 'cancel';
                        else;
                           X = X(:,1:MaxPH);
                        end;
                     end;
                  end;
                  if strcmp(lower(action),'ok')==1;
                     if IsCentered==1;
                        FileCentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-Prediction-' DSGEModel.NameOfModel '-InitialValues-1-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                        file = FileCentered;
                     else;
                        FileUncentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-Prediction-' DSGEModel.NameOfModel '-InitialValues-0-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                        file = FileUncentered;
                     end;
                     if FileExist(file)==1;
                        txt = 'YADA has located results for the unconditional predictive distribution on disk. Would you like to use these results?';
                        answer = Query(txt,'question',250,'Zero Lower Bound Forecasts - Initial Values',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        ZLBData = load(file);
                        Status = 1;
                        ZLBYPaths = ZLBData.ZLBYPaths;
                        ZLBKsiPaths = ZLBData.ZLBKsiPaths;
                        PredData = ZLBData.PredData;
                        KsitTh = ZLBData.KsitTh;
                        PredEventData = ZLBData.PredEventData;
                        if isempty(PredEventData)==0;
                           DSGEModel.PredictionEvent = ZLBData.PredictionEvent;
                           DSGEModel.RunPredictionEvent = 1;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                     else;
                        if NumPredPaths>1;
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Zero Lower Bound Forecasts - Initial Values',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        if DSGEModel.RandomNumberValue==1;
                           randn('state',0);
                        else;
                           randn('state',sum(100*clock));
                        end;
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPH
                        %
                        if IsCentered==0;
                           %
                           % simpler calculations when not having centered unrestricted predictive distribution
                           %
                           [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,'Initial Values');
                        else;
                           %
                           % more complex calculations which need to run the unconditional predictions first, also using
                           % the additional draws
                           %
                           [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,MaxPH,NumPredPaths,AnnualizeData,TransData,'Initial Values',YDataStr);
                        end;
                     end;
                     %
                     if Status==1;
                        %
                        % compute point estimates and quantile values
                        %
                        % 1. for the observables
                        %
                        [ZLBYMeanPrediction,ZLBYQuantiles] = PredictionPathDistribution(ZLBYPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                        %
                        % 2. for the state variables
                        %
                        [ZLBKsiMeanPrediction,ZLBKsiQuantiles] = PredictionPathDistribution(ZLBKsiPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                        %
                        % check if we should compute prediction event results
                        %
                        if DSGEModel.RunPredictionEvent==1;
                           PredEventData = CalculatePredictionEvents(ZLBYPaths,DSGEModel.PredictionEvent);
                           %
                           % write the results to text file
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-PredictionEvent-' DSGEModel.NameOfModel '-InitialValues-' int2str(IsCentered) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.txt']);
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                           %
                           % displays the file if it exists
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['Zero Lower Bound Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,0);
                           end;
                        end;
                        %
                        % check if the user wants to save the data
                        %
                        txt = ['Would you like to save the unconditional predictive distributions for the ' lower(AnnualStr) ' data, subject to the zero lower bound, at initial values to file?'];
                        answer = Query(txt,'question',140,'Save - Zero Lower Bound Forecasts',500,'no',CurrINI);
                        drawnow;
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(ZLBYMeanPrediction,2));
                           file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-PredictionResults-' DSGEModel.NameOfModel '-' AnnualStr '-InitialValues-' int2str(IsCentered) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                           YNames = DSGEModel.VariableNames;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                           Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           if (length(DateVector)>size(Y,2))&(TransData==0);
                              DateVector = DateVector(1,1:size(Y,2));
                           end;
                           save(file,'YNames','StateVariableNames','DateVector','Y','ZLBYMeanPrediction','ZLBYQuantiles','KsitTh','ZLBKsiMeanPrediction','ZLBKsiQuantiles','ZLBYPaths','ZLBKsiPaths','PredData');
                           %
                           % create a matlab m-file to read the output.
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBPRes' AnnualStr 'InitVal' int2str(IsCentered) int2str(FirstPeriod) int2str(LastPeriod) int2str(DSGEModel.ZLBSampleT) int2str(DSGEModel.RtildePosition) int2str(DSGEModel.REqPosition) int2str(DSGEModel.ZLBRuleType) int2str(MaxPH) int2str(NumPredPaths) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the observed variables\n%%\n');
                           fprintf(fid,'ZLBYPaths = DataStructure.ZLBYPaths;\n');
                           fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the state variables\n%%\n');
                           fprintf(fid,'ZLBKsiPaths = DataStructure.ZLBKsiPaths;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the state variable variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           if IsCentered==0;
                              fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB\n%%\n');
                           else;
                              fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB with recentered shocks\n%%\n');
                           end;
                           fprintf(fid,'%%\n%% smooth estimates of the state variables\n%%\n');
                           fprintf(fid,'KsitTh = DataStructure.KsitTh;\n');
                           fprintf(fid,'ZLBYMeanPrediction = DataStructure.ZLBYMeanPrediction;\n');
                           if IsCentered==0;
                              fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB\n%%\n');
                           else;
                              fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB with recentered shocks\n%%\n');
                           end;
                           fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Y\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBYQuantiles));
                           fprintf(fid,'ZLBYQuantiles = DataStructure.ZLBYQuantiles;\n');
                           fprintf(fid,'ZLBKsiMeanPrediction = DataStructure.ZLBKsiMeanPrediction;\n');
                           fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Ksi\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBKsiQuantiles));
                           fprintf(fid,'ZLBKsiQuantiles = DataStructure.ZLBKsiQuantiles;\n');
                           fprintf(fid,'%%\n%% structure with data on the suggested and the used shocks, initial states, and\n%% measurement errors\n%%\n');
                           fprintf(fid,'PredData = DataStructure.PredData;\n');
                           fprintf(fid,'%%\n%% vector with dates for the observed and the state variables\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The unconditional predictive distributions for the observed and the state variables, subject to the zero lower bound and at the initial values, have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 12 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), Y (matrix with the observed variables), ZLBYMeanPrediction (matrix with mean predictions for the observed variables), ZLBYQuantiles (vector structure with 2 fields: Quantile and percent for the observed variables), ZLBKsiMeanPredictions (matrix with mean predictions for the state variables), ZLBKsiQuantiles (vector structure with 2 fields: Quantile and percent for the state variables), ZLBYPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the observed variables), ZLBKsiPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the state variables), Data (structure with fields containing the original and used draws of structural shocks, initial states, and measurement errors), and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Zero Lower Bound Forecasts - Initial Values',800,700,CurrINI);
                        end;
                        %
                        if size(PredData.A,1)==1;
                           ObsVarMean = PredData.A';
                        else;
                           ObsVarMean = [];
                        end;
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,ZLBYMeanPrediction,ObsVarMean,ZLBYQuantiles,PredData.KernelX,PredData.KernelY,KsitTh,ZLBKsiMeanPrediction,ZLBKsiQuantiles,'Initial Values',AnnualStr,0,0,IsCentered,[],SelectedYData);
                     end;
                  end;
               end;
            elseif strcmp(ErrorStr,'')==0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.predict.unconditional.transformed.zlb.postmode);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         %
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            % StateEqNames = ModeData.AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % determine the state equation names
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % check if we should annualize or not
            %
            if DSGEModel.RunDriver==0;
               if gcbo==controls.predict.unconditional.original.zlb.postmode;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'Original';
               elseif gcbo==controls.predict.unconditional.annual.zlb.postmode;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'Annual';
               elseif gcbo==controls.predict.unconditional.transformed.zlb.postmode;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'Transformed';
               end;
            else;
               if strcmp(lower(DSGEModel.answers{1}),'original')==1;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'original';
               elseif strcmp(lower(DSGEModel.answers{1}),'annual')==1;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'annual';
               else;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'transformed';
               end;
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            if NumPredPaths==1;
               NumPredPaths = 100;
               DSGEModel.NumPredPathsValue = 1;
            end;
            %
            % Determine the position of the monetary policy rate and select sample length
            % when the zero zero lower bound may be binding.
            %
            if DSGEModel.RunDriver==0;
               [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
            else;
               action = DSGEModel.answers{2};
               RtildePosition = DSGEModel.RtildePosition;
               REqPosition = DSGEModel.REqPosition;
               ZLBRuleType = DSGEModel.ZLBRuleType;
               ZLBSampleT = DSGEModel.ZLBSampleT;
            end;
            if (strcmp(lower(action),'ok')==1)&(sum([isempty(RtildePosition) isempty(REqPosition)])==0);
               DSGEModel.RtildePosition = RtildePosition;
               DSGEModel.REqPosition = REqPosition;
               DSGEModel.ZLBSampleT = ZLBSampleT;
               DSGEModel.ZLBRuleType = ZLBRuleType;
               %
               % ask if the user wants to center the original forecasts from the linear model on the actual data
               %
               if DSGEModel.RunDriver==0;
                  txt = 'Would you like to recenter the structural shocks such that the predictive distribution of the DSGE model without anticipated shocks (zero values) is centered on the observed data?';
                  answer = Query(txt,'question',300,'Zero Lower Bound Forecasts - Posterior Mode',600,'no',CurrINI);
               else;
                  answer = DSGEModel.answers{3};
               end;
               if strcmp(lower(answer),'no')==1;
                  IsCentered = 0;
                  action = 'ok';
                  YDataStr = '';
                  SelectedYData = [];
               else;
                  IsCentered = 1;
                  %
                  % check which actuals data to use
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check which data to use
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  if (isempty(DSGEModel.Actuals)==1)|(DSGEModel.RunDriver==1);
                     SelectedYData = 1;
                     YDataStr = 'Actuals from original sample';
                     action = 'ok';
                  else;
                     YDataStr = 'Actuals from original sample';
                     m = length(DSGEModel.Actuals);
                     for i=1:m;
                        YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                     end;
                     [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Zero Lower Bound Forecasts - Posterior Mode','','',CurrINI);
                     if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                        YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     end;
                  end;
                  if strcmp(lower(action),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     MaxPH = MaxFH;
                     YDataStr = ['-' int2str(SelectedYData)];
                     if MaxFH==0;
                        txt = ['There are no data points for the observed variables over the forecast sample. YADA will therefore not be able to re-center the structural shocks. YADA will therefore abort the calculations.'];
                        About(txt,'information','Zero Lower Bound Forecasts - Posterior Mode',150,600,CurrINI);
                        action = 'cancel';
                     else;
                        X = X(:,1:MaxPH);
                     end;
                  end;
               end;
               %
               if strcmp(lower(action),'ok')==1;
                  if IsCentered==1;
                     FileCentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-Prediction-' DSGEModel.NameOfModel '-PosteriorMode-1-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                     file = FileCentered;
                  else;
                     FileUncentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-Prediction-' DSGEModel.NameOfModel '-PosteriorMode-0-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                     file = FileUncentered;
                  end;
                  if DSGEModel.RunDriver==0;
                     if FileExist(file)==1;
                        txt = 'YADA has located results for the unconditional predictive distribution on disk. Would you like to use these results?';
                        answer = Query(txt,'question',250,'Zero Lower Bound Forecasts - Posterior Mode',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                  else;
                     answer = DSGEModel.answers{4};
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     ZLBData = load(file);
                     Status = 1;
                     ZLBYPaths = ZLBData.ZLBYPaths;
                     ZLBKsiPaths = ZLBData.ZLBKsiPaths;
                     PredData = ZLBData.PredData;
                     KsitTh = ZLBData.KsitTh;
                     PredEventData = ZLBData.PredEventData;
                     if isempty(PredEventData)==0;
                        DSGEModel.PredictionEvent = ZLBData.PredictionEvent;
                        DSGEModel.RunPredictionEvent = 1;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                  else;
                     if DSGEModel.RunDriver==0;
                        if NumPredPaths>1;
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Zero Lower Bound Forecasts - Posterior Mode',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                     else;
                        answer = DSGEModel.answers{5};
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                           DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                        end;
                        %
                        % make sure that the prediction event length does not exceed the maximum prediction horizon
                        %
                        for i=1:DSGEModel.n;
                           if DSGEModel.PredictionEvent(i,3)>MaxPH;
                              DSGEModel.PredictionEvent(i,3) = MaxPH;
                           end;
                        end;
                        %
                        % display a dialog for setting up an prediction event
                        %
                        [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                        if strcmp(lower(action),'ok')==1;
                           DSGEModel.PredictionEvent = PredictionEvent;
                           DSGEModel.RunPredictionEvent = 1;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                     %
                     if DSGEModel.RandomNumberValue==1;
                        randn('state',0);
                     else;
                        randn('state',sum(100*clock));
                     end;
                     %
                     % we can now compute the predictive distribution for the endogenous variables
                     % up to period MaxPH
                     %
                     if IsCentered==0;
                        %
                        % simpler calculations when not having centered unrestricted predictive distribution
                        %
                        [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGEPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,'Posterior Mode');
                     else;
                        %
                        % more complex calculations which need to run the unconditional predictions first, also using
                        % the additional draws
                        %
                        [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,MaxPH,NumPredPaths,AnnualizeData,TransData,'Posterior Mode',YDataStr);
                     end;
                  end;
                  %
                  if Status==1;
                     %
                     % compute point estimates and quantile values
                     %
                     % 1. for the observables
                     %
                     [ZLBYMeanPrediction,ZLBYQuantiles] = PredictionPathDistribution(ZLBYPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                     %
                     % 2. for the state variables
                     %
                     [ZLBKsiMeanPrediction,ZLBKsiQuantiles] = PredictionPathDistribution(ZLBKsiPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                     %
                     % check if we should compute prediction event results
                     %
                     if DSGEModel.RunPredictionEvent==1;
                        PredEventData = CalculatePredictionEvents(ZLBYPaths,DSGEModel.PredictionEvent);
                        %
                        % write the results to text file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-PredictionEvent-' DSGEModel.NameOfModel '-PosteriorMode-' int2str(IsCentered) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Zero Lower Bound Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,0);
                        end;
                     end;
                     %
                     % check if the user wants to save the data
                     %
                     if DSGEModel.RunDriver==0;
                        txt = ['Would you like to save the unconditional predictive distributions for the ' lower(AnnualStr) ' data, subject to the zero lower bound, at posterior mode to file?'];
                        answer = Query(txt,'question',140,'Save - Zero Lower Bound Forecasts',500,'no',CurrINI);
                        drawnow;
                     else;
                        answer = DSGEModel.answers{6};
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        if DSGEModel.RunDriver==1;
                           SetLogText(strvcat('* Saving results to: ',['     ' DSGEModel.OutputDirectory]));
                        end;
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(ZLBYMeanPrediction,2));
                        file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-PredictionResults-' DSGEModel.NameOfModel '-' AnnualStr '-PosteriorMode-' int2str(IsCentered) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                        YNames = DSGEModel.VariableNames;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                        Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        if (length(DateVector)>size(Y,2))&(TransData==0);
                           DateVector = DateVector(1,1:size(Y,2));
                        end;
                        save(file,'YNames','StateVariableNames','DateVector','Y','ZLBYMeanPrediction','ZLBYQuantiles','KsitTh','ZLBKsiMeanPrediction','ZLBKsiQuantiles','ZLBYPaths','ZLBKsiPaths','PredData');
                        %
                        % create a matlab m-file to read the output.
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBPRes' AnnualStr 'PostMode' int2str(IsCentered) int2str(FirstPeriod) int2str(LastPeriod) int2str(DSGEModel.ZLBSampleT) int2str(DSGEModel.RtildePosition) int2str(DSGEModel.REqPosition) int2str(DSGEModel.ZLBRuleType) int2str(MaxPH) int2str(NumPredPaths) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the observed variables\n%%\n');
                        fprintf(fid,'ZLBYPaths = DataStructure.ZLBYPaths;\n');
                        fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the state variables\n%%\n');
                        fprintf(fid,'ZLBKsiPaths = DataStructure.ZLBKsiPaths;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the state variable variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% observed variables\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        if IsCentered==0;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB\n%%\n');
                        else;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB with recentered shocks\n%%\n');
                        end;
                        fprintf(fid,'%%\n%% smooth estimates of the state variables\n%%\n');
                        fprintf(fid,'KsitTh = DataStructure.KsitTh;\n');
                        fprintf(fid,'ZLBYMeanPrediction = DataStructure.ZLBYMeanPrediction;\n');
                        if IsCentered==0;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB\n%%\n');
                        else;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB with recentered shocks\n%%\n');
                        end;
                        fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Y\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBYQuantiles));
                        fprintf(fid,'ZLBYQuantiles = DataStructure.ZLBYQuantiles;\n');
                        fprintf(fid,'ZLBKsiMeanPrediction = DataStructure.ZLBKsiMeanPrediction;\n');
                        fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Ksi\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBKsiQuantiles));
                        fprintf(fid,'ZLBKsiQuantiles = DataStructure.ZLBKsiQuantiles;\n');
                        fprintf(fid,'%%\n%% structure with data on the suggested and the used shocks, initial states, and\n%% measurement errors\n%%\n');
                        fprintf(fid,'PredData = DataStructure.PredData;\n');
                        fprintf(fid,'%%\n%% vector with dates for the observed and the state variables\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        if DSGEModel.RunDriver==0;
                           txt = ['The unconditional predictive distributions for the observed and the state variables, subject to the zero lower bound and at the posterior mode estimates, have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 12 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), Y (matrix with the observed variables), ZLBYMeanPrediction (matrix with mean predictions for the observed variables), ZLBYQuantiles (vector structure with 2 fields: Quantile and percent for the observed variables), ZLBKsiMeanPredictions (matrix with mean predictions for the state variables), ZLBKsiQuantiles (vector structure with 2 fields: Quantile and percent for the state variables), ZLBYPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the observed variables), ZLBKsiPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the state variables), Data (structure with fields containing the original and used draws of structural shocks, initial states, and measurement errors), and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Zero Lower Bound Forecasts - Posterior Mode',800,700,CurrINI);
                        end;
                     end;
                     %
                     if DSGEModel.RunDriver==0;
                        if size(PredData.A,1)==1;
                           ObsVarMean = PredData.A';
                        else;
                           ObsVarMean = [];
                        end;
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,ZLBYMeanPrediction,ObsVarMean,ZLBYQuantiles,PredData.KernelX,PredData.KernelY,KsitTh,ZLBKsiMeanPrediction,ZLBKsiQuantiles,'Posterior Mode',AnnualStr,1,0,IsCentered,[],SelectedYData);
                     else;
                        SetLogText(strvcat('* Finished calculations.',' '));
                     end;
                  end;
               end;
            else;
               if DSGEModel.RunDriver==1;
                  SetLogText(strvcat('You haven''t determined the policy rate or the policy equation.','Unable to run the function.'))
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         % check if we should check transformations functions
         %
         ErrorStr = '';
         if gcbo==controls.predict.unconditional.transformed.zlb.prior;
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % are we using annualized data, transformed or the original
            %
            if gcbo==controls.predict.unconditional.original.zlb.prior;
               TransData = 0;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'ZLB Predictive Distributions - Original Data';
            elseif gcbo==controls.predict.unconditional.annual.zlb.prior;
               TransData = 0;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'ZLB Predictive Distributions - Annualized Data';
            else;
               TransData = 1;
               AnnualizeData = 0;
               AnnualStr = 'transformed';
               QueryHead = 'ZLB Predictive Distributions - Transformed Data';
            end;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if FileExist(DSGEModel.PriorFile)==1;
                     txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                     answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     InitStatus = 1;
                     theta = ModeData.theta;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     LowerBound = ModeData.LowerBound;
                     ModelParameters = ModeData.ModelParameters;
                     thetaPositions = ModeData.thetaPositions;
                  else;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  end;
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            else;
               [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            end;
            if InitStatus==1;
               %
               % load the data
               %
               if FileExist(DSGEModel.SystemPriorFile)==0;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
               else;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
               end;
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % check if we have a system prior and if so, use a subset of the prior draws
               %
               OldDSGEModel = DSGEModel;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  if DSGEModel.PosteriorSampler>7;
                     [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection',QueryHead,'','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PostDrawsPercentValue = SelectedOption;
                     end;
                  else;
                     action = 'ok';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                     if length(UseDraws)<NumPriorDraws;
                        thetaDraws = thetaDraws(:,UseDraws);
                        NumPriorDraws = length(UseDraws);
                     end;
                  end;
               end;
               %
               % initialize the random number generator
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
                  rand('state',0);
               else;
                  randn('state',sum(100*clock));
                  rand('state',sum(100*clock));
               end;
               %
               % determine the state equation names
               %
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % get the sample dates
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                  end;
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               if NumPredPaths==1;
                  NumPredPaths = 100;
                  DSGEModel.NumPredPathsValue = 1;
               end;
               %
               % ask if the user wants to center the original forecasts from the linear model on the actual data
               %
               txt = 'Would you like to recenter the structural shocks such that the predictive distribution of the DSGE model without anticipated shocks (zero values) is centered on the observed data?';
               answer = Query(txt,'question',300,QueryHead,600,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  IsCentered = 0;
                  action = 'ok';
                  YDataStr = '';
                  SelectedYData = [];
               else;
                  IsCentered = 1;
                  %
                  % check which actuals data to use
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check which data to use
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  if isempty(DSGEModel.Actuals)==1;
                     SelectedYData = 1;
                     YDataStr = 'Actuals from original sample';
                     action = 'ok';
                  else;
                     YDataStr = 'Actuals from original sample';
                     m = length(DSGEModel.Actuals);
                     for i=1:m;
                        YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                     end;
                     [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection',QueryHead,'','',CurrINI);
                     if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                        YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     end;
                  end;
                  if strcmp(lower(action),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     MaxPH = MaxFH;
                     YDataStr = ['-' int2str(SelectedYData)];
                     if MaxFH==0;
                        txt = ['There are no data points for the observed variables over the forecast sample. YADA will therefore not be able to re-center the structural shocks. YADA will therefore abort the calculations.'];
                        About(txt,'information',QueryHead,150,600,CurrINI);
                        action = 'cancel';
                     else;
                        X = X(:,1:MaxPH);
                     end;
                  end;
               end;
               %
               if strcmp(lower(action),'ok')==1;
                  %
                  % Determine the position of the monetary policy rate and select sample length
                  % when the zero zero lower bound may be binding.
                  %
                  [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.RtildePosition = RtildePosition;
                     DSGEModel.REqPosition = REqPosition;
                     DSGEModel.ZLBRuleType = ZLBRuleType;
                     DSGEModel.ZLBSampleT = ZLBSampleT;
                     %
                     % check if we have already stored results to disk
                     %
                     if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0,IsCentered,YDataStr)==1;
                        txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % collect certain results
                        %
                        [PredEventData,PredictionEvent,ObsVarMean] = LoadZLBDSGEPredictionData(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0,IsCentered,YDataStr);
                        DoneCalc = 1;
                     else;
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        if NumPredPaths>1;
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % transpose the prior draws matrix
                        %
                        thetaDraws = thetaDraws';
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPH
                        %
                        if IsCentered==0;
                           %
                           % simpler calculations when not having centered unrestricted predictive distribution
                           %
                           [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGEPredictionPaths(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,[],0,[]);
                        else;
                           %
                           % more complex calculations which need to run the unconditional predictions first, also using
                           % the additional draws
                           %
                           [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECenteredPredictionPaths(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,MaxPH,NumPredPaths,AnnualizeData,TransData,[],0,[],YDataStr);
                        end;
                     end;
                     %
                     % clear everything
                     %
                     clear('thetaDraws');
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % yes, now we need to calculate some distribution properties
                        %
                        [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY,AverageZLBShocks,NumZLBTimes] = ZLBDSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0,IsCentered,YDataStr,[],DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                        %
                        % compute the forecast errors based on the mean predictions
                        %
                        PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                        %
                        % check if the user wants to save the data
                        %
                        txt = ['Would you like to save the predictive distributions subject to the zero lower bound for the ' AnnualStr ' data to file?'];
                        answer = Query(txt,'question',140,'Save - ZLB Predictive Distributions',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                           file = FixFilePath([DSGEModel.OutputDirectory '\ZLBPredictivePriorDist-' int2str(IsCentered) YDataStr '-' AnnualStr '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                           YNames = DSGEModel.VariableNames;
                           DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                           Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           if (length(DateVector)>size(Y,2))&(TransData==0);
                              DateVector = DateVector(1,1:size(Y,2));
                           end;
                           EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                           EstimationStartPeriodOrig = EstimationStartPeriod;
                           YOrig = Y;
                           DateVectorOrig = DateVector;
                           if AnnualizeData==1;
                              Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                              EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                           if TransData==1;
                              [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                              Y = YData';
                              if size(Y,2)==DSGEModel.T;
                                 Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 T2 = size(Y,2);
                                 if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                    Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 end;
                              end;
                           end;
                           if isempty(PredEventData)==1;
                              save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                           else;
                              save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                           end;
                           if isempty(AverageZLBShocks)==0;
                              save(file,'AverageZLBShocks','NumZLBTimes','-APPEND');
                           end;
                           %
                           % create a matlab m-file to read the output.
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBPPD' int2str(IsCentered) strrep(YDataStr,'-','') AnnualStr int2str(ZLBSampleT) int2str(RtildePosition) int2str(REqPosition) int2str(ZLBRuleType) int2str(MaxPH) int2str(NumPriorDraws) int2str(NumPredPaths) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                           fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                           fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                           fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                           fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                           fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                           fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                           fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                           fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                           fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           CovDecompStr = '';
                           if isempty(PredEventData)==0;
                              fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                              fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                              fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                              fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                              NumEntries = '12';
                              ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                           else;
                              NumEntries = '10';
                              ExtraEntry = '';
                           end;
                           if isempty(AverageZLBShocks)==0;
                              fprintf(fid,'%%\n%% the names of the structural shocks\n%%\n');
                              fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                              fprintf(fid,'%%\n%% the average shock values when the ZLB is binding\n%%\n');
                              fprintf(fid,'AverageZLBShocks = DataStructure.AverageZLBShocks;\n');
                              fprintf(fid,'%%\n%% the number of times the ZLB is binding\n%%\n');
                              fprintf(fid,'NumZLBTimes = DataStructure.NumZLBTimes;\n');
                              NumEntries = int2str(str2num(NumEntries)+3);
                              ExtraEntry = [ExtraEntry 'AverageZLBShocks (matrix with the average shock values when the ZLB is binding), NumZLBTimes (number of times the ZLB is binding), '];
                           end;
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The predictive distributions using the prior draws and subject to the zero lower bound have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','ZLB Predictive Distributions',500,600,CurrINI);
                        end;
                        if isempty(PredEventData)==0;
                           %
                           % write prediction event results to file
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\ZLBPriorPredictionEvent-' int2str(IsCentered) '-' AnnualStr '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPriorDraws,CurrINI);
                           %
                           % displays the file if it exists
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                           end;
                        end;
                        %
                        % summarize the information in the uncertainty variables more conveniently
                        %
                        if (AnnualizeData==0)&(TransData==0);
                           %
                           % need to compute a nxh matrix with predictive variances from CovPrediction
                           %
                           [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(zeros(DSGEModel.n,DSGEModel.n,MaxPH),CovPrediction,zeros(DSGEModel.n,MaxPH),zeros(DSGEModel.n,DSGEModel.n,MaxPH));
                           [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                        else;
                           UData = [];
                        end;
                        %
                        % display the predictive distributions
                        %
                        ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,KernelX,KernelY,[],[],[],'Prior Distribution',AnnualStr,0,0,IsCentered,UData,SelectedYData);
                     end;
                  end;
               else;
                  clear('thetaDraws');
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.predict.unconditional.transformed.zlb.posterior);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            thetaMode = ModeData.thetaMode;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % get the sample dates
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','ZLB Predictive Distributions - Posterior','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % initialize the random number generator
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                     rand('state',0);
                  else;
                     randn('state',sum(100*clock));
                     rand('state',sum(100*clock));
                  end;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check how many draws we should pick among these
                  %
                  PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                  NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                  %
                  % get the draws to use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     if DSGEModel.RandomizeDraws==0;
                        DrawFreq = round(NumDraws/NumPostDrawsUsage);
                        SelectDraws = (1:DrawFreq:NumDraws);
                        if length(SelectDraws)+1==NumPostDrawsUsage;
                           SelectDraws = [SelectDraws NumDraws];
                        elseif length(SelectDraws)>NumPostDrawsUsage;
                           SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                        end;
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                     else;
                        thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                     end;
                  else;
                     txt = 'Would you like to use a sub-sample of the posterior draws when computing the predictive distribution subject to the zero lower bound?';
                     answer = Query(txt,'question',400,'ZLB Predictive Distributions - Posterior',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % select how many draws to use
                        %
                        SelectDraws = SampledPosteriorDraws(DSGEModel,CurrINI,'ZLB Predictive Distributions - Posterior',Weights,NumDraws,[]);
                        if isempty(SelectDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                        NumPostDrawsUsage = length(SelectDraws);
                        if DSGEModel.PosteriorSampler~=9;
                           %
                           % fix the weights
                           %
                           Weights = Weights(SelectDraws,1);
                           Weights = Weights*length(Weights)/sum(Weights);
                        else;
                           if isempty(Weights)==0;
                              Weights = Weights(SelectDraws);
                           end;
                        end;
                     end;
                  end;
                  %
                  % are we using annualized data, transformed or the original
                  %
                  if gcbo==controls.predict.unconditional.original.zlb.posterior;
                     TransData = 0;
                     AnnualizeData = 0;
                     AnnualStr = 'original';
                     QueryHead = 'ZLB Predictive Distributions - Original Data';
                  elseif gcbo==controls.predict.unconditional.annual.zlb.posterior;
                     TransData = 0;
                     AnnualizeData = 1;
                     AnnualStr = 'annual';
                     QueryHead = 'ZLB Predictive Distributions - Annualized Data';
                  else;
                     TransData = 1;
                     AnnualizeData = 0;
                     AnnualStr = 'transformed';
                     QueryHead = 'ZLB Predictive Distributions - Transformed Data';
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  %
                  % ask if the user wants to center the original forecasts from the linear model on the actual data
                  %
                  txt = 'Would you like to recenter the structural shocks such that the predictive distribution of the DSGE model without anticipated shocks (zero values) is centered on the observed data?';
                  answer = Query(txt,'question',300,QueryHead,600,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     IsCentered = 0;
                     action = 'ok';
                     YDataStr = '';
                     SelectedYData = [];
                  else;
                     IsCentered = 1;
                     %
                     % check which actuals data to use
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check which data to use
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        YDataStr = 'Actuals from original sample';
                        action = 'ok';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection',QueryHead,'','',CurrINI);
                        if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        end;
                     end;
                     if strcmp(lower(action),'ok')==1;
                        MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                        MaxFH = min(MaxFH,MaxPH);
                        YData = YData(:,1:MaxFH);
                        MaxPH = MaxFH;
                        YDataStr = ['-' int2str(SelectedYData)];
                        if MaxFH==0;
                           txt = ['There are no data points for the observed variables over the forecast sample. YADA will therefore not be able to re-center the structural shocks. YADA will therefore abort the calculations.'];
                           About(txt,'information',QueryHead,150,600,CurrINI);
                           action = 'cancel';
                        else;
                           X = X(:,1:MaxPH);
                        end;
                     end;
                  end;
                  %
                  if strcmp(lower(action),'ok')==1;
                     %
                     % Determine the position of the monetary policy rate and select sample length
                     % when the zero zero lower bound may be binding.
                     %
                     [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % check if we have already stored results to disk
                        %
                        if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,IsCentered,YDataStr)==1;
                           txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % collect certain results
                           %
                           [PredEventData,PredictionEvent,ObsVarMean] = LoadZLBDSGEPredictionData(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,IsCentered,YDataStr);
                           DoneCalc = 1;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           if NumPredPaths>1;
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           if IsCentered==0;
                              %
                              % simpler calculations
                              %
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                           else;
                              %
                              % more complex calculations
                              %
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECenteredPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,YDataStr);
                           end;
                        end;
                        %
                        % clear everything
                        %
                        clear('thetaPostSample');
                        %
                        % check if we should show any results
                        %
                        if DoneCalc==1;
                           %
                           % yes, now we need to calculate some distribution properties
                           %
                           [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY,AverageZLBShocks,NumZLBTimes] = ZLBDSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,IsCentered,YDataStr,Weights,DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                           %
                           % compute the forecast errors based on the mean predictions
                           %
                           PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                           %
                           % check if the user wants to save the data
                           %
                           txt = ['Would you like to save the predictive distributions subject to the zero lower bound for the ' AnnualStr ' data to file?'];
                           answer = Query(txt,'question',140,'Save - ZLB Predictive Distributions',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                              file = FixFilePath([DSGEModel.OutputDirectory '\ZLBPredictiveDist-' int2str(IsCentered) YDataStr '-' AnnualStr '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                              YNames = DSGEModel.VariableNames;
                              DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                              Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              if (length(DateVector)>size(Y,2))&(TransData==0);
                                 DateVector = DateVector(1,1:size(Y,2));
                              end;
                              EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                              EstimationStartPeriodOrig = EstimationStartPeriod;
                              YOrig = Y;
                              DateVectorOrig = DateVector;
                              if AnnualizeData==1;
                                 Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                                 EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                              if TransData==1;
                                 [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                                 Y = YData';
                                 if size(Y,2)==DSGEModel.T;
                                    Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    T2 = size(Y,2);
                                    if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                       Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                    else;
                                       Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                       DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                    end;
                                 end;
                              end;
                              if isempty(PredEventData)==1;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                              else;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                              end;
                              if isempty(AverageZLBShocks)==0;
                                 ShockNames = DSGEModel.StateShockNames;
                                 save(file,'ShockNames','AverageZLBShocks','NumZLBTimes','-APPEND');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBPD' int2str(IsCentered) strrep(YDataStr,'-','') AnnualStr int2str(CurrChain) int2str(ZLBSampleT) int2str(RtildePosition) int2str(REqPosition) int2str(ZLBRuleType) int2str(MaxPH) int2str(NumPostDrawsUsage) int2str(NumPredPaths) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              CovDecompStr = '';
                              if isempty(PredEventData)==0;
                                 fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                 fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                 fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 NumEntries = '12';
                                 ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                              else;
                                 NumEntries = '10';
                                 ExtraEntry = '';
                              end;
                              if isempty(AverageZLBShocks)==0;
                                 fprintf(fid,'%%\n%% the names of the structural shocks\n%%\n');
                                 fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                                 fprintf(fid,'%%\n%% the average shock values when the ZLB is binding\n%%\n');
                                 fprintf(fid,'AverageZLBShocks = DataStructure.AverageZLBShocks;\n');
                                 fprintf(fid,'%%\n%% the number of times the ZLB is binding\n%%\n');
                                 fprintf(fid,'NumZLBTimes = DataStructure.NumZLBTimes;\n');
                                 NumEntries = int2str(str2num(NumEntries)+3);
                                 ExtraEntry = [ExtraEntry 'AverageZLBShocks (matrix with the average shock values when the ZLB is binding), NumZLBTimes (number of times the ZLB is binding), '];
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The predictive distributions subject to the zero lower bound have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','ZLB Predictive Distributions',500,600,CurrINI);
                           end;
                           if isempty(PredEventData)==0;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\ZLBPredictionEvent-' int2str(IsCentered) '-' AnnualStr '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                              end;
                           end;
                           %
                           % summarize the information in the uncertainty variables more conveniently
                           %
                           if (AnnualizeData==0)&(TransData==0);
                              %
                              % need to compute a nxh matrix with predictive variances from CovPrediction
                              %
                              [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(zeros(DSGEModel.n,DSGEModel.n,MaxPH),CovPrediction,zeros(DSGEModel.n,MaxPH),zeros(DSGEModel.n,DSGEModel.n,MaxPH));
                              [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                           else;
                              UData = [];
                           end;
                           %
                           % display the predictive distributions
                           %
                           ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,KernelX,KernelY,[],[],[],'Posterior Distribution',AnnualStr,1,0,IsCentered,UData,SelectedYData);
                        end;
                     end;
                  else;
                     clear('thetaPostSample');
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;         
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ZLBUnconPredPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         % check if we should check transformations functions
         %
         ErrorStr = '';
         if gcbo==controls.learningtools.predictivedist.unconditional.transformed.zlb.prior;
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % are we using annualized data, transformed or the original
            %
            if gcbo==controls.learningtools.predictivedist.unconditional.original.zlb.prior;
               TransData = 0;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'ZLB Predictive Distributions - Original Data';
            elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.zlb.prior;
               TransData = 0;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'ZLB Predictive Distributions - Annualized Data';
            elseif gcbo==controls.learningtools.predictivedist.unconditional.transformed.zlb.prior;
               TransData = 1;
               AnnualizeData = 0;
               AnnualStr = 'transformed';
               QueryHead = 'ZLB Predictive Distributions - Transformed Data';
            end;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if FileExist(DSGEModel.PriorFile)==1;
                     txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                     answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     InitStatus = 1;
                     theta = ModeData.theta;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     LowerBound = ModeData.LowerBound;
                     ModelParameters = ModeData.ModelParameters;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     UniformBounds = ModeData.UniformBounds;
                  else;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  end;
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            else;
               [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            end;
            if InitStatus==1;
               %
               % get the necessary information about the adaptive learning parameters
               %
               [status,thetaAL,thetaIndexAL,thetaDistAL,LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL] = VerifyAdaptiveLearningPriorData(DSGEModel,CurrINI);
               if status==1;
                  %
                  % we are able to read the belief prior file and collect initial parameter values for them,. Now combine this data with the
                  % data from the DSGE model with its posterior mode estimates
                  %
                  [theta,thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds] = AdaptiveLearningExtendPriorFields(theta, ...
                                   thetaIndex,thetaDist,LowerBound,ModelParameters,thetaPositions,PriorDist,ParameterNames,UniformBounds,thetaAL,thetaIndexAL,thetaDistAL, ...
                                   LowerBoundAL,ModelParametersAL,thetaPositionsAL,PriorDistAL,ParameterNamesAL,UniformBoundsAL);
                  %
                  % load the data
                  %
                  if FileExist(DSGEModel.SystemPriorFile)==0;
                     PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
                  else;
                     PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
                  end;
                  PriorData = load(PriorDrawsFile);
                  thetaDraws = PriorData.thetaDraws;
                  NumPriorDraws = size(thetaDraws,2);
                  %
                  % check if we have a system prior and if so, use a subset of the prior draws
                  %
                  OldDSGEModel = DSGEModel;
                  if FileExist(DSGEModel.SystemPriorFile)==1;
                     if DSGEModel.PosteriorSampler>7;
                        [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection',QueryHead,'','',CurrINI);
                        if strcmp(lower(action),'ok')==1;
                           DSGEModel.PostDrawsPercentValue = SelectedOption;
                        end;
                     else;
                        action = 'ok';
                     end;
                     if strcmp(lower(action),'ok')==1;
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                        if length(UseDraws)<NumPriorDraws;
                           thetaDraws = thetaDraws(:,UseDraws);
                           NumPriorDraws = length(UseDraws);
                        end;
                     end;
                  end;
                  %
                  % initialize the random number generator
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                     rand('state',0);
                  else;
                     randn('state',sum(100*clock));
                     rand('state',sum(100*clock));
                  end;
                  %
                  % determine the state equation names
                  %
                  StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
                  StateEquationNames = '';
                  for s=1:length(DSGEModel.StateEquationPositions);
                     StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
                  end;
                  %
                  % get the sample dates
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxPH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  %
                  % Determine the position of the monetary policy rate and select sample length
                  % when the zero zero lower bound may be binding.
                  %
                  [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     %
                     % check if RtildePosition matches a forward looking variable
                     %
                     if max(DSGEModel.ForwardVariablePositions==RtildePosition)==0;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % check if we have already stored results to disk
                        %
                        if ZLBPredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0,0,'')==1;
                           txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        %
                        if strcmp(lower(answer),'yes')==1;
                           [PredEventData,PredictionEvent,ObsVarMean] = LoadZLBDSGEPredictionData(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0,0,'');
                           DoneCalc = 1;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGEPredictionPaths(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,[],0,[]);
                        end;
                        %
                        % clear everything
                        %
                        clear('thetaDraws');
                        if DoneCalc==1;
                           %
                           % yes, now we need to calculate some distribution properties
                           %
                           [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY,AverageZLBShocks,NumZLBTimes] = ZLBDSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,[],0,0,'',[],DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                           %
                           % compute the forecast errors based on the mean predictions
                           %
                           PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                           %
                           % check if the user wants to save the data
                           %
                           txt = ['Would you like to save the predictive distributions subject to the zero lower bound for the ' AnnualStr ' data to file?'];
                           answer = Query(txt,'question',140,'Save - ZLB Predictive Distributions',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                              file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLBPriorPredictiveDist-0-' AnnualStr '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                              YNames = DSGEModel.VariableNames;
                              DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                              Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              if (length(DateVector)>size(Y,2))&(TransData==0);
                                 DateVector = DateVector(1,1:size(Y,2));
                              end;
                              EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                              EstimationStartPeriodOrig = EstimationStartPeriod;
                              YOrig = Y;
                              DateVectorOrig = DateVector;
                              if AnnualizeData==1;
                                 Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                                 EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                              if TransData==1;
                                 [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                                 Y = YData';
                                 if size(Y,2)==DSGEModel.T;
                                    Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    T2 = size(Y,2);
                                    if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                       Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                    else;
                                       Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                       DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                    end;
                                 end;
                              end;
                              if isempty(PredEventData)==1;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                              else;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                              end;
                              if isempty(AverageZLBShocks)==0;
                                 ShockNames = DSGEModel.StateShockNames;
                                 save(file,'ShockNames','AverageZLBShocks','NumZLBTimes','-APPEND');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\AL' int2str(DSGEModel.FixedBeliefsForProj) 'ZLBPriorPD0' AnnualStr int2str(ZLBSampleT) int2str(RtildePosition) int2str(REqPosition) int2str(ZLBRuleType) int2str(MaxPH) int2str(NumPriorDraws) int2str(NumPredPaths) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              CovDecompStr = '';
                              if isempty(PredEventData)==0;
                                 fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                 fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                 fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 NumEntries = '12';
                                 ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                              else;
                                 NumEntries = '10';
                                 ExtraEntry = '';
                              end;
                              if isempty(AverageZLBShocks)==0;
                                 fprintf(fid,'%%\n%% the names of the structural shocks\n%%\n');
                                 fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                                 fprintf(fid,'%%\n%% the average shock values when the ZLB is binding\n%%\n');
                                 fprintf(fid,'AverageZLBShocks = DataStructure.AverageZLBShocks;\n');
                                 fprintf(fid,'%%\n%% the number of times the ZLB is binding\n%%\n');
                                 fprintf(fid,'NumZLBTimes = DataStructure.NumZLBTimes;\n');
                                 NumEntries = int2str(str2num(NumEntries)+3);
                                 ExtraEntry = [ExtraEntry 'AverageZLBShocks (matrix with the average shock values when the ZLB is binding), NumZLBTimes (number of times the ZLB is binding), '];
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The predictive distributions subject to the zero lower bound and based on adaptive learning have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','ZLB Predictive Distributions',500,600,CurrINI);
                           end;
                           if isempty(PredEventData)==0;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLBPriorPredictionEvent-0-' AnnualStr '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPriorDraws,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'ZLB Prior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                              end;
                           end;
                           %
                           % summarize the information in the uncertainty variables more conveniently
                           %
                           if (AnnualizeData==0)&(TransData==0);
                              %
                              % need to compute a nxh matrix with predictive variances from CovPrediction
                              %
                              [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(zeros(DSGEModel.n,DSGEModel.n,MaxPH),CovPrediction,zeros(DSGEModel.n,MaxPH),zeros(DSGEModel.n,DSGEModel.n,MaxPH));
                              [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                           else;
                              UData = [];
                           end;
                           %
                           % display the predictive distributions
                           %
                           ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,KernelX,KernelY,[],[],[],'Prior Distribution',AnnualStr,0,0,0,UData,1);
                        end;
                     else;
                        txt = ['One of the forward looking variables is also selected as the monetary policy rate variable. This is not allowed as it is likely incompatible with adaptive learning.'];
                        About(txt,'information',QueryHead,200,600,CurrINI);
                     end;
                  end;
               else;
                   txt = ['YADA was unable to locate a belief system prior file, not even the default file "DefaultBeliefSystemPrior.xls" in the directory "' FixFilePath([pwd '\learning\data\']) '". YADA is therefore unable to continue with the predictive distributions based on the prior draws.'];
                   About(txt,'information',QueryHead,500,600,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ZLBUnconPredPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % function for adaptive learning
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.learningtools.predictivedist.unconditional.transformed.posterior);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            thetaMode = ModeData.thetaMode;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % get the sample dates
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' int2str(i)]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','ZLB Predictive Distributions','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadAdaptiveLearningDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  %
                  % get the number of draws and estimated parameters
                  %
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % initialize the random number generator
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                     rand('state',0);
                  else;
                     randn('state',sum(100*clock));
                     rand('state',sum(100*clock));
                  end;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check how many draws we should pick among these
                  %
                  PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                  NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                  %
                  % get the draws to use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     if DSGEModel.RandomizeDraws==0;
                        DrawFreq = round(NumDraws/NumPostDrawsUsage);
                        SelectDraws = (1:DrawFreq:NumDraws);
                        if length(SelectDraws)+1==NumPostDrawsUsage;
                           SelectDraws = [SelectDraws NumDraws];
                        elseif length(SelectDraws)>NumPostDrawsUsage;
                           SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                        end;
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                     else;
                        thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                     end;
                  else;
                     txt = 'Would you like to use a sub-sample of the posterior draws when computing the predictive distribution subject to the zero lower bound?';
                     answer = Query(txt,'question',400,'ZLB Predictive Distributions - Posterior',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % select how many draws to use
                        %
                        SelectDraws = SampledPosteriorDraws(DSGEModel,CurrINI,'ZLB Predictive Distributions - Posterior',Weights,NumDraws,[]);
                        if isempty(SelectDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                        NumPostDrawsUsage = length(SelectDraws);
                        if DSGEModel.PosteriorSampler~=9;
                           %
                           % fix the weights
                           %
                           Weights = Weights(SelectDraws,1);
                           Weights = Weights*length(Weights)/sum(Weights);
                        else;
                           if isempty(Weights)==0;
                              Weights = Weights(SelectDraws);
                           end;
                        end;
                     end;
                  end;
                  %
                  % are we using annualized data, transformed or the original
                  %
                  if gcbo==controls.learningtools.predictivedist.unconditional.original.zlb.posterior;
                     TransData = 0;
                     AnnualizeData = 0;
                     AnnualStr = 'original';
                     QueryHead = 'ZLB Predictive Distributions - Original Data';
                  elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.zlb.posterior;
                     TransData = 0;
                     AnnualizeData = 1;
                     AnnualStr = 'annual';
                     QueryHead = 'ZLB Predictive Distributions - Annualized Data';
                  elseif gcbo==controls.learningtools.predictivedist.unconditional.transformed.zlb.posterior;
                     TransData = 1;
                     AnnualizeData = 0;
                     AnnualStr = 'transformed';
                     QueryHead = 'ZLB Predictive Distributions - Transformed Data';
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  %
                  % Determine the position of the monetary policy rate and select sample length
                  % when the zero zero lower bound may be binding.
                  %
                  [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     %
                     % check if RtildePosition matches a forward looking variable
                     %
                     if max(DSGEModel.ForwardVariablePositions==RtildePosition)==0;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % check if we have already stored results to disk
                        %
                        if ZLBPredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,0,'')==1;
                           txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        %
                        if strcmp(lower(answer),'yes')==1;
                           [PredEventData,PredictionEvent,ObsVarMean] = LoadZLBDSGEPredictionData(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,0,'');
                           DoneCalc = 1;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                        end;
                        %
                        % clear everything
                        %
                        clear('thetaPostSample');
                        %
                        % check if we should show any results
                        %
                        if DoneCalc==1;
                           %
                           % yes, now we need to calculate some distribution properties
                           %
                           [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY,AverageZLBShocks,NumZLBTimes] = ZLBDSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,0,'',Weights,DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                           %
                           % compute the forecast errors based on the mean predictions
                           %
                           PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                           %
                           % check if the user wants to save the data
                           %
                           txt = ['Would you like to save the predictive distributions subject to the zero lower bound for the ' AnnualStr ' data to file?'];
                           answer = Query(txt,'question',140,'Save - ZLB Predictive Distributions',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                              file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLBPredictiveDist-0-' AnnualStr '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                              YNames = DSGEModel.VariableNames;
                              DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                              Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              if (length(DateVector)>size(Y,2))&(TransData==0);
                                 DateVector = DateVector(1,1:size(Y,2));
                              end;
                              EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                              EstimationStartPeriodOrig = EstimationStartPeriod;
                              YOrig = Y;
                              DateVectorOrig = DateVector;
                              if AnnualizeData==1;
                                 Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                                 EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                              if TransData==1;
                                 [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                                 Y = YData';
                                 if size(Y,2)==DSGEModel.T;
                                    Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    T2 = size(Y,2);
                                    if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                       Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                    else;
                                       Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                       DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                    end;
                                 end;
                              end;
                              if isempty(PredEventData)==1;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                              else;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                              end;
                              if isempty(AverageZLBShocks)==0;
                                 ShockNames = DSGEModel.StateShockNames;
                                 save(file,'ShockNames','AverageZLBShocks','NumZLBTimes','-APPEND');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\AL' int2str(DSGEModel.FixedBeliefsForProj) 'ZLBPD0' AnnualStr int2str(CurrChain) int2str(ZLBSampleT) int2str(RtildePosition) int2str(REqPosition) int2str(ZLBRuleType) int2str(MaxPH) int2str(NumPostDrawsUsage) int2str(NumPredPaths) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              CovDecompStr = '';
                              if isempty(PredEventData)==0;
                                 fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                 fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                 fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 NumEntries = '12';
                                 ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                              else;
                                 NumEntries = '10';
                                 ExtraEntry = '';
                              end;
                              if isempty(AverageZLBShocks)==0;
                                 fprintf(fid,'%%\n%% the names of the structural shocks\n%%\n');
                                 fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                                 fprintf(fid,'%%\n%% the average shock values when the ZLB is binding\n%%\n');
                                 fprintf(fid,'AverageZLBShocks = DataStructure.AverageZLBShocks;\n');
                                 fprintf(fid,'%%\n%% the number of times the ZLB is binding\n%%\n');
                                 fprintf(fid,'NumZLBTimes = DataStructure.NumZLBTimes;\n');
                                 NumEntries = int2str(str2num(NumEntries)+3);
                                 ExtraEntry = [ExtraEntry 'AverageZLBShocks (matrix with the average shock values when the ZLB is binding), NumZLBTimes (number of times the ZLB is binding), '];
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The predictive distributions subject to the zero lower bound and based on adaptive learning have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','ZLB Predictive Distributions',500,600,CurrINI);
                           end;
                           if isempty(PredEventData)==0;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-ZLBPredictionEvent-0-' AnnualStr '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'ZLB Posterior Distribution - Unconditional Predictions',50,CurrINI,1,0);
                              end;
                           end;
                           %
                           % summarize the information in the uncertainty variables more conveniently
                           %
                           if (AnnualizeData==0)&(TransData==0);
                              %
                              % need to compute a nxh matrix with predictive variances from CovPrediction
                              %
                              [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(zeros(DSGEModel.n,DSGEModel.n,MaxPH),CovPrediction,zeros(DSGEModel.n,MaxPH),zeros(DSGEModel.n,DSGEModel.n,MaxPH));
                              [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                           else;
                              UData = [];
                           end;
                           %
                           % display the predictive distributions
                           %
                           ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,KernelX,KernelY,[],[],[],'Posterior Distribution',AnnualStr,1,0,0,UData,1);
                        end;
                     else;
                        txt = ['One of the forward looking variables is also selected as the monetary policy rate variable. This is not allowed as it is likely incompatible with adaptive learning.'];
                        About(txt,'information',QueryHead,200,600,CurrINI);
                     end;
                  else;
                     clear('thetaPostSample');
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' int2str(CurrChain) ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconCRPSPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         if MatlabNumber>=8.0;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'CRPS - ZLB Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  UniformBounds = ModeData.UniformBounds;
                  thetaIndex = ModeData.thetaIndex;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the prior draws data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','CRPS - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            clear('PriorData');
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            UsePaths = 1;
            if NumPredPaths*NumPriorDraws>=100000;
               txt = ['The total number of paths that the CRPS calculations will use is equal ' int2str(NumPredPaths*NumPriorDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
               answer = Query(txt,'question',400,'CRPS - Prior Distribution',500,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  UsePaths = 0;
               end;
            end;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            if UsePaths==1;
               %
               % determine the sample
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               DSGEModel.FirstPeriod = FirstPeriod;
               DSGEModel.LastPeriod = LastPeriod;
               %
               % check how many time periods to use for predictive distributions
               %
               MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
               %
               % check the maximum when we take NaNs into account
               %
               YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
               BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
               if isempty(DSGEModel.Actuals)==1;
                  SelectedYData = 1;
                  DataAction = 'ok';
                  YDataStr = 'Actuals from original sample';
               else;
                  YDataStr = 'Actuals from original sample';
                  m = length(DSGEModel.Actuals);
                  for i=1:m;
                     YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                  end;
                  [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - Prior Distribution','','',CurrINI);
                  if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                     YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                  end;
               end;
            else;
               DataAction = 'cancel';
            end;
            %
            if strcmp(lower(DataAction),'ok')==1;
               MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               MaxFH = min(MaxFH,MaxPH);
               YData = YData(:,1:MaxFH);
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                  end;
               end;
               %
               % Determine the position of the monetary policy rate and select sample length
               % when the zero zero lower bound may be binding.
               %
               [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.RtildePosition = RtildePosition;
                  DSGEModel.REqPosition = REqPosition;
                  DSGEModel.ZLBRuleType = ZLBRuleType;
                  DSGEModel.ZLBSampleT = ZLBSampleT;
                  %
                  % selection of variable 
                  %
                  if isempty(DSGEModel.PredictionVariables)==1;
                     PredictionVariables = ones(1,DSGEModel.n);
                     PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                  else;
                     PredictionVariables = DSGEModel.PredictionVariables;
                  end;
                  [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                  if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                     DSGEModel.PredictionVariables = PredictionVariables;
                     %
                     % Determine the prediction sample
                     %
                     SampleStrMat = CreateFullSample(DSGEModel);
                     PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                     SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                     %
                     % are we using annualized data, transformed or the original
                     %
                     if gcbo==controls.predict.unconditional.original.zlb.crps.prior;
                        AnnualizeData = 0;
                        AnnualStr = 'original';
                        QueryHead = 'ZLB Predictive Distributions - Original Data';
                     elseif gcbo==controls.predict.unconditional.annual.zlb.crps.prior;
                        AnnualizeData = 1;
                        AnnualStr = 'annual';
                        QueryHead = 'ZLB Predictive Distributions - Annualized Data';
                     end;
                     if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,0,[],0,0,'')==1;
                        txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % collect certain results
                        %
                        DoneCalc = 1;
                        tic;
                     else;
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        if NumPredPaths>1;
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % transpose the prior draws matrix
                        %
                        thetaDraws = thetaDraws';
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPH
                        %
                        tic;
                        [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGEPredictionPaths(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,0,[],0,[]);
                     end;
                     clear('thetaDraws');
                     if DoneCalc==1;
                        %
                        % we can now compute the continuous ranked probability scores
                        %
                        CRPSMatrix = ComputeCRPSZLBDSGEPaths(YData,PredictionVariables,NumPriorDraws,NumPredPaths,MaxPH,AnnualizeData,[],0,'DSGE','','',DSGEModel,CurrINI);
                        CompTime = toc;
                        %
                        % check if the user wants to store the results to disk
                        %
                        txt = 'Would you like to save the continuous ranked probability score (CRPS) and energy score (ES) estimates to file?';
                        answer = Query(txt,'question',300,'CRPS - ZLB Prior Distribution',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           file = FixFilePath([DSGEModel.OutputDirectory '\CRPS-ZLB-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                           YNames = DSGEModel.VariableNames;
                           save(file,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\CRPSZLBPrior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(AnnualizeData) '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumPriorDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                           fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                           fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%% for each forecast horizon.\n%%\n');
                           fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The continuous probability score (CRPS) estimates and the energy scores for the jointly predicted variables based on the prior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','CRPS - ZLB Prior Distribution',500,600,CurrINI);
                        end;
                        %
                        % write the results to a text file
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\CRPS-ZLB-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                        PrintCRPS(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,0,0,0,1,NumPriorDraws,[],NumPriorDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                        %
                        % check if file exists and display it then
                        %
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['CRPS & ES - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'ZLB Prior Distribution',50,CurrINI,1,0);
                        end;
                     else;
                        CompTime = toc;
                     end;
                  elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                     clear('thetaPostSample');
                     txt = 'You didn''t select any variables to predict when computing the continuous ranked probability score. YADA is therefore unable to help you!';
                     About(txt,'information','CRPS - ZLB Prior Distribution',150,500,CurrINI);
                     DSGEModel = OldDSGEModel;
                  end;
               else;
                  DSGEModel = OldDSGEModel;
               end;
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconCRPSPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         %
         if strcmp(ErrorStr,'')==1;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','CRPS - ZLB Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the continuous ranked probability score (CRPS)?';
                  answer = Query(txt,'question',400,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'CRPS - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  UsePaths = 1;
                  if NumPredPaths*NumDraws>100000;
                     txt = ['The total number of paths that the CRPS calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                     answer = Query(txt,'question',400,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        UsePaths = 0;
                     end;
                  end;
                  %
                  % check if we should continue
                  %
                  if UsePaths==1;
                     %
                     % collect some required data
                     %
                     theta = ModeData.thetaMode;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     LowerBound = ModeData.LowerBound;
                     UniformBounds = ModeData.UniformBounds;
                     ModelParameters = ModeData.ModelParameters;
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - ZLB Posterior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                           BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                        end;
                     end;
                  else;
                     DataAction = 'cancel';
                  end;
                  %
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                        end;
                     end;
                     %
                     % Determine the position of the monetary policy rate and select sample length
                     % when the zero zero lower bound may be binding.
                     %
                     [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           DSGEModel.PredictionVariables = PredictionVariables;
                           %
                           % Determine the prediction sample
                           %
                           SampleStrMat = CreateFullSample(DSGEModel);
                           PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                           SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                           %
                           if gcbo==controls.predict.unconditional.original.zlb.crps.posterior;
                              AnnualizeData = 0;
                              AnnualStr = 'original';
                              QueryHead = 'ZLB Predictive Distributions (CRPS) - Original Data';
                           elseif gcbo==controls.predict.unconditional.annual.zlb.crps.posterior;
                              AnnualizeData = 1;
                              AnnualStr = 'annual';
                              QueryHead = 'ZLB Predictive Distributions (CRPS) - Annualized Data';
                           end;
                           %
                           % check if we have already stored results to disk
                           %
                           if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumDraws,NumPredPaths,AnnualizeData,0,CurrChain,1,0,'')==1;
                              txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                              answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           %
                           % check if we shall recompute
                           %
                           if strcmp(lower(answer),'yes')==1;
                              DoneCalc = 1;
                              tic;
                           else;
                              %
                              % ask if we should conduct prediction event calculations
                              %
                              txt = strvcat('Would you like YADA to calculate prediction event probabilities and perform risk analysis?',' ','NOTE: The results will only be stored to disk and will not be displayed.');
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                    DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                 end;
                                 %
                                 % make sure that the prediction event length does not exceed the maximum prediction horizon
                                 %
                                 for i=1:DSGEModel.n;
                                    if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                       DSGEModel.PredictionEvent(i,3) = MaxPH;
                                    end;
                                 end;
                                 %
                                 % display a dialog for setting up an prediction event
                                 %
                                 [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    DSGEModel.PredictionEvent = PredictionEvent;
                                    DSGEModel.RunPredictionEvent = 1;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                              %
                              % we can now compute the predictive distribution for the endogenous variables
                              % up to period MaxPH
                              %
                              tic;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGEPredictionPaths(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
                           end;
                           clear('thetaPostSample');
                           if DoneCalc==1;
                              %
                              % we can now compute the continuous ranked probability scores
                              %
                              CRPSMatrix = ComputeCRPSZLBDSGEPaths(YData,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,1,'DSGE','','',DSGEModel,CurrINI);
                              CompTime = toc;
                              %
                              % check if the user wants to store the results to disk
                              %
                              txt = 'Would you like to save the continuous ranked probability score (CRPS) and energy score (ES) estimates to file?';
                              answer = Query(txt,'question',300,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                                 file = FixFilePath([DSGEModel.OutputDirectory '\CRPS-ZLB-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                                 YNames = DSGEModel.VariableNames;
                                 save(file,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\CRPSZLBPosterior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(AnnualizeData) '_' int2str(CurrChain) '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                                 fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                                 fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%% for each forecast horizon.\n%%\n');
                                 fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                 fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The continuous probability score (CRPS) estimates and the energy scores for the jointly predicted variables based on the posterior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information','CRPS - ZLB Posterior Distribution',500,600,CurrINI);
                              end;
                              %
                              % write the results to a text file
                              %
                              ResFile = FixFilePath([DSGEModel.OutputDirectory '\CRPS-ZLB-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                              PrintCRPS(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,1,0,0,1,NumDraws,NumBurnin,NumPostDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                              %
                              % check if file exists and display it then
                              %
                              if FileExist(ResFile)==1;
                                 TextGUI(ResFile,['CRPS & ES - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'ZLB Posterior Distribution',50,CurrINI,1,0);
                              end;
                           else;
                              CompTime = toc;
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           clear('thetaPostSample');
                           txt = 'You didn''t select any variables to predict when computing the continuous ranked probability score. YADA is therefore unable to help you!';
                           About(txt,'information','CRPS - ZLB Posterior Distribution',150,500,CurrINI);
                        end;
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;         
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredHeightPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         if MatlabNumber>=8.0;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Predictive Likelihood - ZLB Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  UniformBounds = ModeData.UniformBounds;
                  thetaIndex = ModeData.thetaIndex;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the prior draws data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','CRPS - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            clear('PriorData');
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            UsePaths = 1;
            if NumPredPaths*NumPriorDraws>=10000000;
               txt = ['The total number of paths that the predictive likelihood calculations will use is equal ' int2str(NumPredPaths*NumPriorDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
               answer = Query(txt,'question',400,'Predictive Likelihood - Prior Distribution',500,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  UsePaths = 0;
               end;
            end;
            %
            if UsePaths==1;
               %
               % determine the sample
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               DSGEModel.FirstPeriod = FirstPeriod;
               DSGEModel.LastPeriod = LastPeriod;
               %
               % check how many time periods to use for predictive distributions
               %
               MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
               %
               % check the maximum when we take NaNs into account
               %
               YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
               BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
               if isempty(DSGEModel.Actuals)==1;
                  SelectedYData = 1;
                  DataAction = 'ok';
                  YDataStr = 'Actuals from original sample';
               else;
                  YDataStr = 'Actuals from original sample';
                  m = length(DSGEModel.Actuals);
                  for i=1:m;
                     YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                  end;
                  [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - Prior Distribution','','',CurrINI);
                  if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                     YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                  end;
               end;
            else;
               DataAction = 'cancel';
            end;
            %
            if strcmp(lower(DataAction),'ok')==1;
               MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               MaxFH = min(MaxFH,MaxPH);
               YData = YData(:,1:MaxFH);
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                  end;
               end;
               %
               % determine the state equation names
               %
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % Determine the position of the monetary policy rate and select sample length
               % when the zero zero lower bound may be binding.
               %
               [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.RtildePosition = RtildePosition;
                  DSGEModel.REqPosition = REqPosition;
                  DSGEModel.ZLBRuleType = ZLBRuleType;
                  DSGEModel.ZLBSampleT = ZLBSampleT;
                  %
                  % selection of variable 
                  %
                  if isempty(DSGEModel.PredictionVariables)==1;
                     PredictionVariables = ones(1,DSGEModel.n);
                     PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                  else;
                     PredictionVariables = DSGEModel.PredictionVariables;
                  end;
                  %
                  [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                  if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                     DSGEModel.PredictionVariables = PredictionVariables;
                     %
                     % Determine the prediction sample
                     %
                     SampleStrMat = CreateFullSample(DSGEModel);
                     PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                     SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                     %
                     %
                     if gcbo==controls.predict.unconditional.original.zlb.height.prior;
                        AnnualizeData = 0;
                        AnnualStr = 'original';
                        QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Original Data';
                     elseif gcbo==controls.predict.unconditional.annual.zlb.height.prior;
                        AnnualizeData = 1;
                        AnnualStr = 'annual';
                        QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Annualized Data';
                     end;
                     if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,0,[],0,0,'')==1;
                        txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % collect certain results
                        %
                        DoneCalc = 1;
                        tic;
                     else;
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        if NumPredPaths>1;
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % transpose the prior draws matrix
                        %
                        thetaDraws = thetaDraws';
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPH
                        %
                        tic;
                        [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGEPredictionPaths(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,0,[],0,[]);
                     end;
                     clear('thetaDraws');
                     if DoneCalc==1;
                        %
                        % now compute the predictive likelihood with kernel density estimation for the predicted variables
                        %
                        LogPredLike = PredLikeDSGEPathsKernelDensity(YData,PredictionVariables,NumPriorDraws,NumPredPaths,MaxPH,AnnualizeData,[],0,'DSGE','','',DSGEModel,CurrINI);
                        CompTime = toc;
                        %
                        % ask the user if results should be saved to file
                        %
                        SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                        SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                        %
                        PredictionSampleStrMat = CreateFullSample(DSGEModel);
                        PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                        txt = ['Would you like to save the prior draws results for the ' lower(AnnualStr) ' predictions to disk?'];
                        answer = Query(txt,'question',200,'Save - ZLB Predictive Likelihood',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveLikelihood-ZLB-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                           YNames = DSGEModel.VariableNames;
                           save(file,'YNames','PredictionVariables','LogPredLike','PredDateVector','PredictionSampleStrMat');
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\PLikeZLBPrior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' AnnualStr '_' SelectedVarStr '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumPriorDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                           fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                           fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable log predictive likelihoods\n%% and the joint predictive likelihood for each forecast horizon.\n%%\n');
                           fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                           fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The predictive likelihood estimates for the individual and the selected jointly predicted variables based on the prior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), PredDateVector (vector with dates for the forecast horizon), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Predictive Likelihood - ZLB Posterior Distribution',500,600,CurrINI);
                        end;
                        %
                        % write the results to a text file
                        %
                        ResFile = FixFilePath([DSGEModel.OutputDirectory '\PredictiveLikelihood-ZLB-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                        %
                        PrintZLBPredictiveLikelihood(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),LogPredLike,AnnualizeData,0,0,0,1,NumPriorDraws,[],NumPriorDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                        %
                        % display the results
                        %
                        if FileExist(ResFile)==1;
                           TextGUI(ResFile,['ZLB Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,0);
                        end;
                     else;
                        CompTime = toc;
                     end;
                  elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                     clear('thetaPostSample');
                     txt = 'You didn''t select any variables to predict when computing the log predictive likelihood. YADA is therefore unable to help you!';
                     About(txt,'information','Predictive Likelihood - ZLB Prior Distribution',150,500,CurrINI);
                     DSGEModel = OldDSGEModel;
                  end;
               else;
                  DSGEModel = OldDSGEModel;
               end;
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'UnconPredHeightPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the predictive distribution
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Predictive Likelihood - ZLB Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the predictive likelihood?';
                  answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'Predictive Likelihood - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  UsePaths = 1;
                  if NumPredPaths*NumDraws>=10000000;
                     txt = ['The total number of paths that the predictive likelihood calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                     answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Posterior Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        UsePaths = 0;
                     end;
                  end;
                  %
                  % check if we should continue
                  %
                  if UsePaths==1;
                     %
                     % collect some required data
                     %
                     theta = ModeData.thetaMode;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     LowerBound = ModeData.LowerBound;
                     UniformBounds = ModeData.UniformBounds;
                     ModelParameters = ModeData.ModelParameters;
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - ZLB Posterior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                           BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                        end;
                     end;
                  else;
                     DataAction = 'cancel';
                  end;
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                        end;
                     end;
                     %
                     % Determine the position of the monetary policy rate and select sample length
                     % when the zero zero lower bound may be binding.
                     %
                     [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           DSGEModel.PredictionVariables = PredictionVariables;
                           %
                           % Determine the prediction sample
                           %
                           SampleStrMat = CreateFullSample(DSGEModel);
                           PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                           SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                           %
                           %
                           if gcbo==controls.predict.unconditional.original.zlb.height.posterior;
                              AnnualizeData = 0;
                              AnnualStr = 'original';
                              QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Original Data';
                           elseif gcbo==controls.predict.unconditional.annual.zlb.height.posterior;
                              AnnualizeData = 1;
                              AnnualStr = 'annual';
                              QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Annualized Data';
                           end;
                           %
                           % check if we have already stored results to disk
                           %
                           if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumDraws,NumPredPaths,AnnualizeData,0,CurrChain,1,0,'')==1;
                              txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                              answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           %
                           % check if we shall recompute
                           %
                           if strcmp(lower(answer),'yes')==1;
                              DoneCalc = 1;
                              tic;
                           else;
                              %
                              % ask if we should conduct prediction event calculations
                              %
                              txt = strvcat('Would you like YADA to calculate prediction event probabilities and perform risk analysis?',' ','NOTE: The results will only be stored to disk and will not be displayed.');
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                    DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                 end;
                                 %
                                 % make sure that the prediction event length does not exceed the maximum prediction horizon
                                 %
                                 for i=1:DSGEModel.n;
                                    if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                       DSGEModel.PredictionEvent(i,3) = MaxPH;
                                    end;
                                 end;
                                 %
                                 % display a dialog for setting up an prediction event
                                 %
                                 [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    DSGEModel.PredictionEvent = PredictionEvent;
                                    DSGEModel.RunPredictionEvent = 1;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                              %
                              % we can now compute the predictive distribution for the endogenous variables
                              % up to period MaxPH
                              %
                              tic;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGEPredictionPaths(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
                           end;
                           clear('thetaPostSample');
                           if DoneCalc==1;
                              %
                              % now compute the predictive likelihood with kernel density estimation for the predicted variables
                              %
                              LogPredLike = PredLikeDSGEPathsKernelDensity(YData,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,1,'DSGE','','',DSGEModel,CurrINI);
                              CompTime = toc;
                              %
                              % ask the user if results should be saved to file
                              %
                              SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                              SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                              %
                              PredictionSampleStrMat = CreateFullSample(DSGEModel);
                              PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                              txt = ['Would you like to save the posterior draws results for the ' lower(AnnualStr) ' predictions to disk?'];
                              answer = Query(txt,'question',200,'Save - ZLB Predictive Likelihood',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                                 file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveLikelihood-ZLB-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                                 YNames = DSGEModel.VariableNames;
                                 save(file,'YNames','PredictionVariables','LogPredLike','PredDateVector','PredictionSampleStrMat');
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\PLikeZLBPosterior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' AnnualStr '_' SelectedVarStr '_' int2str(CurrChain) '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                                 fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                                 fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable log predictive likelihoods\n%% and the joint predictive likelihood for each forecast horizon.\n%%\n');
                                 fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                 fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                 fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                                 fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The predictive likelihood estimates for the individual and the selected jointly predicted variables based on the posterior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), PredDateVector (vector with dates for the forecast horizon), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information','Predictive Likelihood - ZLB Posterior Distribution',500,600,CurrINI);
                              end;
                              %
                              % write the results to a text file
                              %
                              ResFile = FixFilePath([DSGEModel.OutputDirectory '\PredictiveLikelihood-ZLB-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                              %
                              PrintZLBPredictiveLikelihood(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),LogPredLike,AnnualizeData,1,0,0,1,NumDraws,NumBurnin,NumPostDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                              %
                              % display the results
                              %
                              if FileExist(ResFile)==1;
                                 TextGUI(ResFile,['ZLB Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,0);
                              end;
                           else;
                              CompTime = toc;
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           clear('thetaPostSample');
                           txt = 'You didn''t select any variables to predict when computing the log predictive likelihood. YADA is therefore unable to help you!';
                           About(txt,'information','Predictive Likelihood - ZLB Posterior Distribution',150,500,CurrINI);
                        end;
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ZLBUnconPredHeightPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % are we using annualized data, transformed or the original
            %
            if gcbo==controls.learningtools.predictivedist.unconditional.original.zlb.height.prior;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Original Data';
            elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.zlb.height.prior;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Annualized Data';
            end;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if FileExist(DSGEModel.PriorFile)==1;
                     txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                     answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     InitStatus = 1;
                     theta = ModeData.theta;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     LowerBound = ModeData.LowerBound;
                     ModelParameters = ModeData.ModelParameters;
                     thetaPositions = ModeData.thetaPositions;
                  else;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  end;
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            else;
               [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            end;
            if InitStatus==1;
               %
               % load the data
               %
               if FileExist(DSGEModel.SystemPriorFile)==0;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               else;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               end;
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % check if we have a system prior and if so, use a subset of the prior draws
               %
               OldDSGEModel = DSGEModel;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  if DSGEModel.PosteriorSampler>7;
                     [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection',QueryHead,'','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PostDrawsPercentValue = SelectedOption;
                     end;
                  else;
                     action = 'ok';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                     if length(UseDraws)<NumPriorDraws;
                        thetaDraws = thetaDraws(:,UseDraws);
                        NumPriorDraws = length(UseDraws);
                     end;
                  end;
               end;
               %
               % initialize the random number generator
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
                  rand('state',0);
               else;
                  randn('state',sum(100*clock));
                  rand('state',sum(100*clock));
               end;
               %
               % determine the state equation names
               %
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               if NumPredPaths==1;
                  NumPredPaths = 100;
                  DSGEModel.NumPredPathsValue = 1;
               end;
               %
               UsePaths = 1;
               if NumPredPaths*NumPriorDraws>=10000000;
                  txt = ['The total number of paths that the predictive likelihood calculations will use is equal ' int2str(NumPredPaths*NumPriorDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                  answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Prior Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     UsePaths = 0;
                  end;
               end;
               %
               if UsePaths==1;
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  DSGEModel.FirstPeriod = FirstPeriod;
                  DSGEModel.LastPeriod = LastPeriod;
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check the maximum when we take NaNs into account
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                  if isempty(DSGEModel.Actuals)==1;
                     SelectedYData = 1;
                     DataAction = 'ok';
                     YDataStr = 'Actuals from original sample';
                  else;
                     YDataStr = 'Actuals from original sample';
                     m = length(DSGEModel.Actuals);
                     for i=1:m;
                        YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                     end;
                     [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - ZLB Prior Distribution','','',CurrINI);
                     if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                        YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                     end;
                  end;
               else;
                  DataAction = 'cancel';
               end;
               if strcmp(lower(DataAction),'ok')==1;
                  MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  MaxFH = min(MaxFH,MaxPH);
                  YData = YData(:,1:MaxFH);
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxPH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                     end;
                  end;
                  %
                  % Determine the position of the monetary policy rate and select sample length
                  % when the zero zero lower bound may be binding.
                  %
                  [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     %
                     % check if RtildePosition matches a forward looking variable
                     %
                     DSGEModel.RtildePosition = RtildePosition;
                     DSGEModel.REqPosition = REqPosition;
                     DSGEModel.ZLBRuleType = ZLBRuleType;
                     DSGEModel.ZLBSampleT = ZLBSampleT;
                     %
                     % selection of variable 
                     %
                     if isempty(DSGEModel.PredictionVariables)==1;
                        PredictionVariables = ones(1,DSGEModel.n);
                        PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                     else;
                        PredictionVariables = DSGEModel.PredictionVariables;
                     end;
                     %
                     [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                     if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                        DSGEModel.PredictionVariables = PredictionVariables;
                        %
                        % Determine the prediction sample
                        %
                        SampleStrMat = CreateFullSample(DSGEModel);
                        PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                        SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                        %
                        if ZLBPredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,0,[],0,0,'')==1;
                           txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           DoneCalc = 1;
                           tic;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           tic;
                           [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGEPredictionPaths(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,0,[],0,[]);
                        end;
                        %
                        % clear everything
                        %
                        if DoneCalc==1;
                           %
                           % now compute the predictive likelihood with kernel density estimation for the predicted variables
                           %
                           LogPredLike = PredLikeDSGEPathsKernelDensity(YData,PredictionVariables,NumPriorDraws,NumPredPaths,MaxPH,AnnualizeData,[],0,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel,CurrINI);
                           CompTime = toc;
                           %
                           % ask the user if results should be saved to file
                           %
                           SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                           SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                           %
                           PredictionSampleStrMat = CreateFullSample(DSGEModel);
                           PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                           txt = ['Would you like to save the prior draws results for the ' lower(AnnualStr) ' predictions to disk?'];
                           answer = Query(txt,'question',200,'Save - ZLB Predictive Likelihood',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-PredictiveLikelihood-ZLB-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                              YNames = DSGEModel.VariableNames;
                              save(file,'YNames','PredictionVariables','LogPredLike','PredDateVector','PredictionSampleStrMat');
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\AL' int2str(DSGEModel.FixedBeliefsForProj) 'PLikeZLBPrior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' AnnualStr '_' SelectedVarStr '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumPriorDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                              fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                              fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable log predictive likelihoods\n%% and the joint predictive likelihood for each forecast horizon.\n%%\n');
                              fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                              fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The predictive likelihood estimates for the individual and the selected jointly predicted variables based on the prior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), PredDateVector (vector with dates for the forecast horizon), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','Predictive Likelihood - ZLB Prior Distribution',500,600,CurrINI);
                           end;
                           %
                           % make sure that the belief system information is sent to the print function.
                           %
                           if DSGEModel.BeliefSystem==1;
                              DSGEModel.BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           else;
                              if DSGEModel.BeliefSystem==2;
                                 [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                              end;
                              DSGEModel.BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           end;
                           %
                           % write the results to a text file
                           %
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-PredictiveLikelihood-ZLB-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                           %
                           PrintZLBPredictiveLikelihood(ResFile,'DSGE with adaptive learning',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),LogPredLike,AnnualizeData,0,0,1,1,NumPriorDraws,0,NumPriorDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                           %
                           % display the results
                           %
                           if FileExist(ResFile)==1;
                              TextGUI(ResFile,['ZLB Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,0);
                           end;
                        else;
                           CompTime = toc;
                        end;
                     elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                        txt = 'You didn''t select any variables to predict when computing the log predictive likelihood. YADA is therefore unable to help you!';
                        About(txt,'information','Predictive Likelihood - ZLB Prior Distribution',150,500,CurrINI);
                     end;
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ZLBUnconPredHeightPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the predictive distribution
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Predictive Likelihood - ZLB Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the predictive likelihood?';
                  answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'Predictive Likelihood - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  UsePaths = 1;
                  if NumPredPaths*NumDraws>=10000000;
                     txt = ['The total number of paths that the predictive likelihood calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                     answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Posterior Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        UsePaths = 0;
                     end;
                  end;
                  %
                  % check if we should continue
                  %
                  if UsePaths==1;
                     %
                     % collect some required data
                     %
                     theta = ModeData.thetaMode;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     LowerBound = ModeData.LowerBound;
                     UniformBounds = ModeData.UniformBounds;
                     ModelParameters = ModeData.ModelParameters;
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - ZLB Posterior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                           BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                        end;
                     end;
                  else;
                     DataAction = 'cancel';
                  end;
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                        end;
                     end;
                     %
                     % Determine the position of the monetary policy rate and select sample length
                     % when the zero zero lower bound may be binding.
                     %
                     [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           DSGEModel.PredictionVariables = PredictionVariables;
                           %
                           % Determine the prediction sample
                           %
                           SampleStrMat = CreateFullSample(DSGEModel);
                           PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                           SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                           %
                           %
                           if gcbo==controls.learningtools.predictivedist.unconditional.original.zlb.height.posterior;
                              AnnualizeData = 0;
                              AnnualStr = 'original';
                              QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Original Data';
                           elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.zlb.height.posterior;
                              AnnualizeData = 1;
                              AnnualStr = 'annual';
                              QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Annualized Data';
                           end;
                           %
                           % check if we have already stored results to disk
                           %
                           if ZLBPredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumDraws,NumPredPaths,AnnualizeData,0,CurrChain,1,0,'')==1;
                              txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                              answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           %
                           if strcmp(lower(answer),'yes')==1;
                              DoneCalc = 1;
                              tic;
                           else;
                              %
                              % ask if we should conduct prediction event calculations
                              %
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                    DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                 end;
                                 %
                                 % make sure that the prediction event length does not exceed the maximum prediction horizon
                                 %
                                 for i=1:DSGEModel.n;
                                    if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                       DSGEModel.PredictionEvent(i,3) = MaxPH;
                                    end;
                                 end;
                                 %
                                 % display a dialog for setting up an prediction event
                                 %
                                 [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    DSGEModel.PredictionEvent = PredictionEvent;
                                    DSGEModel.RunPredictionEvent = 1;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                              %
                              % we can now compute the predictive distribution for the endogenous variables
                              % up to period MaxPH
                              %
                              tic;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                           end;
                           %
                           % clear everything
                           %
                           clear('thetaPostSample');
                           if DoneCalc==1;
                              %
                              % now compute the predictive likelihood with kernel density estimation for the predicted variables
                              %
                              LogPredLike = PredLikeDSGEPathsKernelDensity(YData,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,1,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel,CurrINI);
                              CompTime = toc;
                              %
                              % ask the user if results should be saved to file
                              %
                              SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                              SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                              %
                              PredictionSampleStrMat = CreateFullSample(DSGEModel);
                              PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                              txt = ['Would you like to save the posterior draws results for the ' lower(AnnualStr) ' predictions to disk?'];
                              answer = Query(txt,'question',200,'Save - ZLB Predictive Likelihood',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                                 file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-PredictiveLikelihood-ZLB-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                                 YNames = DSGEModel.VariableNames;
                                 save(file,'YNames','PredictionVariables','LogPredLike','PredDateVector','PredictionSampleStrMat');
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\AL' int2str(DSGEModel.FixedBeliefsForProj) 'PLikeZLBPosterior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' AnnualStr '_' SelectedVarStr '_' int2str(CurrChain) '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                                 fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                                 fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable log predictive likelihoods\n%% and the joint predictive likelihood for each forecast horizon.\n%%\n');
                                 fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                 fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                 fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                                 fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The predictive likelihood estimates for the individual and the selected jointly predicted variables based on the posterior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), PredDateVector (vector with dates for the forecast horizon), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information','Predictive Likelihood - ZLB Posterior Distribution',500,600,CurrINI);
                              end;
                              %
                              % make sure that the belief system information is sent to the print function.
                              %
                              if DSGEModel.BeliefSystem==1;
                                 DSGEModel.BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                              else;
                                 if DSGEModel.BeliefSystem==2;
                                    [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                                 end;
                                 DSGEModel.BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                              end;
                              %
                              % write the results to a text file
                              %
                              ResFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-PredictiveLikelihood-ZLB-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                              %
                              PrintZLBPredictiveLikelihood(ResFile,'DSGE with adaptive learning',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),LogPredLike,AnnualizeData,1,0,1,1,NumDraws,NumBurnin,NumPostDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                              %
                              % display the results
                              %
                              if FileExist(ResFile)==1;
                                 TextGUI(ResFile,['ZLB Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,0);
                              end;
                           else;
                              CompTime = toc;
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           clear('thetaPostSample');
                           txt = 'You didn''t select any variables to predict when computing the log predictive likelihood. YADA is therefore unable to help you!';
                           About(txt,'information','Predictive Likelihood - ZLB Posterior Distribution',150,500,CurrINI);
                        end;
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ZLBUnconCRPSPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         ErrorStr = '';
         if MatlabNumber>=8.0;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % are we using annualized data, transformed or the original
            %
            if gcbo==controls.learningtools.predictivedist.unconditional.original.zlb.crps.prior;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'ZLB Predictive Distributions (CRPS) - Original Data';
            elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.zlb.crps.prior;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'ZLB Predictive Distributions (CRPS) - Annualized Data';
            end;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if FileExist(DSGEModel.PriorFile)==1;
                     txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                     answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     InitStatus = 1;
                     theta = ModeData.theta;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     LowerBound = ModeData.LowerBound;
                     ModelParameters = ModeData.ModelParameters;
                     thetaPositions = ModeData.thetaPositions;
                  else;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  end;
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            else;
               [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            end;
            if InitStatus==1;
               %
               % load the data
               %
               if FileExist(DSGEModel.SystemPriorFile)==0;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               else;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
               end;
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % check if we have a system prior and if so, use a subset of the prior draws
               %
               OldDSGEModel = DSGEModel;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  if DSGEModel.PosteriorSampler>7;
                     [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection',QueryHead,'','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PostDrawsPercentValue = SelectedOption;
                     end;
                  else;
                     action = 'ok';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                     if length(UseDraws)<NumPriorDraws;
                        thetaDraws = thetaDraws(:,UseDraws);
                        NumPriorDraws = length(UseDraws);
                     end;
                  end;
               end;
               %
               % initialize the random number generator
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
                  rand('state',0);
               else;
                  randn('state',sum(100*clock));
                  rand('state',sum(100*clock));
               end;
               %
               % determine the state equation names
               %
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               if NumPredPaths==1;
                  NumPredPaths = 100;
                  DSGEModel.NumPredPathsValue = 1;
               end;
               %
               UsePaths = 1;
               if NumPredPaths*NumPriorDraws>=10000000;
                  txt = ['The total number of paths that the predictive likelihood calculations will use is equal ' int2str(NumPredPaths*NumPriorDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                  answer = Query(txt,'question',400,'CRPS - ZLB Prior Distribution',500,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     UsePaths = 0;
                  end;
               end;
               %
               if UsePaths==1;
                  %
                  [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                  DSGEModel.FirstPeriod = FirstPeriod;
                  DSGEModel.LastPeriod = LastPeriod;
                  %
                  % check how many time periods to use for predictive distributions
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check the maximum when we take NaNs into account
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                  if isempty(DSGEModel.Actuals)==1;
                     SelectedYData = 1;
                     DataAction = 'ok';
                     YDataStr = 'Actuals from original sample';
                  else;
                     YDataStr = 'Actuals from original sample';
                     m = length(DSGEModel.Actuals);
                     for i=1:m;
                        YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                     end;
                     [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - ZLB Prior Distribution','','',CurrINI);
                     if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                        YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                     end;
                  end;
               else;
                  DataAction = 'cancel';
               end;
               if strcmp(lower(DataAction),'ok')==1;
                  MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                  [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                  MaxFH = min(MaxFH,MaxPH);
                  YData = YData(:,1:MaxFH);
                  %
                  % matrix with exogenous variables
                  %
                  X = [];
                  if DSGEModel.k>0;
                     if NumPredX<MaxPH;
                        X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                     end;
                     if NumPredX>0;
                        X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                     end;
                  end;
                  %
                  % Determine the position of the monetary policy rate and select sample length
                  % when the zero zero lower bound may be binding.
                  %
                  [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     %
                     % check if RtildePosition matches a forward looking variable
                     %
                     DSGEModel.RtildePosition = RtildePosition;
                     DSGEModel.REqPosition = REqPosition;
                     DSGEModel.ZLBRuleType = ZLBRuleType;
                     DSGEModel.ZLBSampleT = ZLBSampleT;
                     %
                     % selection of variable 
                     %
                     if isempty(DSGEModel.PredictionVariables)==1;
                        PredictionVariables = ones(1,DSGEModel.n);
                        PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                     else;
                        PredictionVariables = DSGEModel.PredictionVariables;
                     end;
                     %
                     [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                     if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                        DSGEModel.PredictionVariables = PredictionVariables;
                        %
                        % Determine the prediction sample
                        %
                        SampleStrMat = CreateFullSample(DSGEModel);
                        PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                        SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                        %
                        if ZLBPredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,0,[],0,0,'')==1;
                           txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           DoneCalc = 1;
                           tic;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           tic;
                           [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGEPredictionPaths(thetaDraws',theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,0,[],0,[]);
                        end;
                        %
                        % clear everything
                        %
                        if DoneCalc==1;
                           %
                           CRPSMatrix = ComputeCRPSZLBDSGEPaths(YData,PredictionVariables,NumPriorDraws,NumPredPaths,MaxPH,AnnualizeData,[],0,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel,CurrINI);
                           CompTime = toc;
                           %
                           % check if the user wants to store the results to disk
                           %
                           txt = 'Would you like to save the continuous ranked probability score (CRPS) and energy score (ES) estimates to file?';
                           answer = Query(txt,'question',300,'CRPS - ZLB Prior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-CRPS-ZLB-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                              YNames = DSGEModel.VariableNames;
                              save(file,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\AL' int2str(DSGEModel.FixedBeliefsForProj) 'CRPSZLBPrior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(AnnualizeData) '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumPriorDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                              fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                              fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%% for each forecast horizon.\n%%\n');
                              fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The continuous probability score (CRPS) estimates and the energy scores for the jointly predicted variables based on the prior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','CRPS - ZLB Prior Distribution',500,600,CurrINI);
                           end;
                           %
                           % make sure that the belief system information is sent to the print function.
                           %
                           if DSGEModel.BeliefSystem==1;
                              DSGEModel.BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           else;
                              if DSGEModel.BeliefSystem==2;
                                 [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                              end;
                              DSGEModel.BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                           end;
                           %
                           % write the results to a text file
                           %
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-CRPS-ZLB-Prior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                           PrintCRPS(ResFile,'DSGE with adaptive learning',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,0,0,1,1,NumPriorDraws,0,NumPriorDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                           %
                           % check if file exists and display it then
                           %
                           if FileExist(ResFile)==1;
                              TextGUI(ResFile,['CRPS & ES - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'ZLB Prior Distribution',50,CurrINI,1,0);
                           end;
                        else;
                           CompTime = toc;
                        end;
                     elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                        txt = 'You didn''t select any variables to predict when computing the log predictive likelihood. YADA is therefore unable to help you!';
                        About(txt,'information','Predictive Likelihood - ZLB Prior Distribution',150,500,CurrINI);
                     end;
                  end;
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ZLBUnconCRPSPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-AdaptiveLearning-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','CRPS - ZLB Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the continuous ranked probability score (CRPS)?';
                  answer = Query(txt,'question',400,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'CRPS - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  UsePaths = 1;
                  if NumPredPaths*NumDraws>100000;
                     txt = ['The total number of paths that the CRPS calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                     answer = Query(txt,'question',400,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        UsePaths = 0;
                     end;
                  end;
                  %
                  % check if we should continue
                  %
                  if UsePaths==1;
                     %
                     % collect some required data
                     %
                     theta = ModeData.thetaMode;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     LowerBound = ModeData.LowerBound;
                     UniformBounds = ModeData.UniformBounds;
                     ModelParameters = ModeData.ModelParameters;
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - ZLB Posterior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                           BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                        end;
                     end;
                  else;
                     DataAction = 'cancel';
                  end;
                  %
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                        end;
                     end;
                     %
                     % Determine the position of the monetary policy rate and select sample length
                     % when the zero zero lower bound may be binding.
                     %
                     [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           DSGEModel.PredictionVariables = PredictionVariables;
                           %
                           % Determine the prediction sample
                           %
                           %
                           % Determine the prediction sample
                           %
                           SampleStrMat = CreateFullSample(DSGEModel);
                           PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                           SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                           %
                           if gcbo==controls.learningtools.predictivedist.unconditional.original.zlb.crps.posterior;
                              AnnualizeData = 0;
                              AnnualStr = 'original';
                              QueryHead = 'ZLB Predictive Distributions (CRPS) - Original Data';
                           elseif gcbo==controls.learningtools.predictivedist.unconditional.annual.zlb.crps.posterior;
                              AnnualizeData = 1;
                              AnnualStr = 'annual';
                              QueryHead = 'ZLB Predictive Distributions (CRPS) - Annualized Data';
                           end;
                           %
                           % check if we have already stored results to disk
                           %
                           if ZLBPredictionsExist(DSGEModel.OutputDirectory,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumDraws,NumPredPaths,AnnualizeData,0,CurrChain,1,0,'')==1;
                              txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                              answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           %
                           if strcmp(lower(answer),'yes')==1;
                              DoneCalc = 1;
                              tic;
                           else;
                              %
                              % ask if we should conduct prediction event calculations
                              %
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                    DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                 end;
                                 %
                                 % make sure that the prediction event length does not exceed the maximum prediction horizon
                                 %
                                 for i=1:DSGEModel.n;
                                    if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                       DSGEModel.PredictionEvent(i,3) = MaxPH;
                                    end;
                                 end;
                                 %
                                 % display a dialog for setting up an prediction event
                                 %
                                 [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                 if strcmp(lower(action),'ok')==1;
                                    DSGEModel.PredictionEvent = PredictionEvent;
                                    DSGEModel.RunPredictionEvent = 1;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                              %
                              % we can now compute the predictive distribution for the endogenous variables
                              % up to period MaxPH
                              %
                              tic;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBAdaptiveLearningDSGEPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                           end;
                           %
                           % clear everything
                           %
                           clear('thetaPostSample');
                           if DoneCalc==1;
                              %
                              % now compute the predictive likelihood with kernel density estimation for the predicted variables
                              %
                              CRPSMatrix = ComputeCRPSZLBDSGEPaths(YData,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,1,['AdaptiveLearningDSGE-' int2str(DSGEModel.FixedBeliefsForProj)],'','',DSGEModel,CurrINI);
                              CompTime = toc;
                              %
                              % check if the user wants to store the results to disk
                              %
                              txt = 'Would you like to save the continuous ranked probability score (CRPS) and energy score (ES) estimates to file?';
                              answer = Query(txt,'question',300,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                              if strcmp(lower(answer),'yes')==1;
                                 [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                                 file = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-CRPS-ZLB-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                                 YNames = DSGEModel.VariableNames;
                                 save(file,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
                                 mfile = FixFilePath([DSGEModel.OutputDirectory '\AL' int2str(DSGEModel.FixedBeliefsForProj) 'CRPSZLBPosterior_' DSGEModel.NameOfModel '_' int2str(SelectedYData) '_' int2str(AnnualizeData) '_' int2str(CurrChain) '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                                 fid = fopen(mfile,'wt');
                                 fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                 fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                 fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                 fprintf(fid,'YNames = DataStructure.YNames;\n');
                                 fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                                 fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                                 fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%% for each forecast horizon.\n%%\n');
                                 fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
                                 fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                 fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                 fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                 %
                                 % end the script file
                                 %
                                 fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                 fclose(fid);
                                 %
                                 txt = ['The continuous probability score (CRPS) estimates and the energy scores for the jointly predicted variables based on the posterior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                 About(txt,'information','CRPS - ZLB Posterior Distribution',500,600,CurrINI);
                              end;
                              %
                              % make sure that the belief system information is sent to the print function.
                              %
                              if DSGEModel.BeliefSystem==1;
                                 DSGEModel.BeliefSys = YADADefaultBeliefSystem(DSGEModel.X,DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                              else;
                                 if DSGEModel.BeliefSystem==2;
                                    [stat,msg] = CopyFile(DSGEModel.BeliefSystemFile,[pwd '\tmp']);
                                 end;
                                 DSGEModel.BeliefSys = feval(GetName(DSGEModel.BeliefSystemFile),DSGEModel.XVariableNames,DSGEModel.StateVariableNames,DSGEModel.ForwardVariableNames);
                              end;
                              %
                              % write the results to a text file
                              %
                              ResFile = FixFilePath([DSGEModel.OutputDirectory '\AdaptiveLearning-' int2str(DSGEModel.FixedBeliefsForProj) '-CRPS-ZLB-Posterior-' DSGEModel.NameOfModel '-' int2str(SelectedYData) '.' int2str(AnnualizeData) '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                              PrintCRPS(ResFile,'DSGE with adaptive learning',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,1,0,1,1,NumDraws,NumBurnin,NumPostDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                              %
                              % check if file exists and display it then
                              %
                              if FileExist(ResFile)==1;
                                 TextGUI(ResFile,['CRPS & ES - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'ZLB Posterior Distribution',50,CurrINI,1,0);
                              end;
                           else;
                              CompTime = toc;
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           clear('thetaPostSample');
                           txt = 'You didn''t select any variables to predict when computing the continuous ranked probability score. YADA is therefore unable to help you!';
                           About(txt,'information','CRPS - ZLB Posterior Distribution',150,500,CurrINI);
                        end;
                     end;
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConPredHeightPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         if MatlabNumber>=8.0;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'Predictive Likelihood - ZLB Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  UniformBounds = ModeData.UniformBounds;
                  thetaIndex = ModeData.thetaIndex;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the prior draws data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','CRPS - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            clear('PriorData');
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            UsePaths = 1;
            if NumPredPaths*NumPriorDraws>=10000000;
               txt = ['The total number of paths that the predictive likelihood calculations will use is equal ' int2str(NumPredPaths*NumPriorDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
               answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Prior Distribution',500,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  UsePaths = 0;
               end;
            end;
            %
            if UsePaths==1;
               %
               % determine the sample
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               DSGEModel.FirstPeriod = FirstPeriod;
               DSGEModel.LastPeriod = LastPeriod;
               %
               % check how many time periods to use for predictive distributions
               %
               MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
               %
               % check the maximum when we take NaNs into account
               %
               YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
               BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
               if isempty(DSGEModel.Actuals)==1;
                  SelectedYData = 1;
                  DataAction = 'ok';
                  YDataStr = 'Actuals from original sample';
               else;
                  YDataStr = 'Actuals from original sample';
                  m = length(DSGEModel.Actuals);
                  for i=1:m;
                     YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                  end;
                  [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - Prior Distribution','','',CurrINI);
                  if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                     YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                  end;
               end;
            else;
               DataAction = 'cancel';
            end;
            %
            if strcmp(lower(DataAction),'ok')==1;
               MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               MaxFH = min(MaxFH,MaxPH);
               YData = YData(:,1:MaxFH);
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                  end;
               end;
               %
               % check the length of the conditioning sample
               %
               [status,NumPeriods] = CheckConditioningData(DSGEModel);
               %
               % check if we should adjust the X matrix
               %
               if MaxPH<NumPeriods;
                  NumPeriods = MaxPH;
               end;
               %
               % collect the conditional variables and the selected conditioning sample
               %
               Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
               if LastPeriod>=NumPeriods;
                  U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
               else;
                  %
                  % we have to create artifical initial conditions for the conditioning variables
                  %
                  U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
               end;
               %
               % determine the state equation names
               %
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % Determine the position of the monetary policy rate and select sample length
               % when the zero zero lower bound may be binding.
               %
               [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.RtildePosition = RtildePosition;
                  DSGEModel.REqPosition = REqPosition;
                  DSGEModel.ZLBRuleType = ZLBRuleType;
                  DSGEModel.ZLBSampleT = ZLBSampleT;
                  %
                  % selection of variable 
                  %
                  if isempty(DSGEModel.PredictionVariables)==1;
                     PredictionVariables = ones(1,DSGEModel.n);
                     PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                  else;
                     PredictionVariables = DSGEModel.PredictionVariables;
                  end;
                  %
                  [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                  if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                     %
                     % check if there are any matches between conditional variable names and predicted variable names
                     %
                     [status,VarStrMat] = CompareVariablesStrMat(DSGEModel.ZVariableNames(DSGEModel.ConditionalVariables,:),DSGEModel.VariableNames(PredictionVariables,:));
                     %
                     if status==1;
                        VarStr = vecstr(VarStrMat,',');
                        VarStr = VarStr(1:length(VarStr)-1);
                        txt = ['Some of the conditioning variables have the same name as the selected predicted variables. These variables are: ' VarStr '. Are you sure you wish to continue?'];
                        answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Prior Distribution',600,'no',CurrINI);
                     else;
                        answer = 'yes';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DSGEModel.PredictionVariables = PredictionVariables;
                        %
                        % Determine the prediction sample
                        %
                        SampleStrMat = CreateFullSample(DSGEModel);
                        PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                        SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                        %
                        %
                        if gcbo==controls.predict.conditional.original.zlb.height.prior;
                           AnnualizeData = 0;
                           AnnualStr = 'original';
                           QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Original Data';
                        elseif gcbo==controls.predict.conditional.annual.zlb.height.prior;
                           AnnualizeData = 1;
                           AnnualStr = 'annual';
                           QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Annualized Data';
                        end;
                        %
                        % check which method is used for dealing with the conditioning assumptions
                        %
                        if DSGEModel.ShockControlMethod==1;
                           ShockMethod = 'Direct-';
                        elseif DSGEModel.ShockControlMethod==2;
                           ShockMethod = 'WaggonerZha-';
                        else;
                           ShockMethod = 'Subset-';
                        end;
                        %
                        % check if we have already stored results to disk
                        %
                        if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,0,[],0,0,'')==1;
                           txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           tic;
                           DoneCalc = 1;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           if NumPredPaths>1;
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % transpose the prior draws matrix
                           %
                           thetaDraws = thetaDraws';
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           tic;
                           if DSGEModel.ShockControlMethod==1;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPaths(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,0,0,[]);
                           elseif DSGEModel.ShockControlMethod==2;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsWZ(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,0,0,[]);
                           else;
                              %
                              % check if the selected method can actually be used
                              %
                              CondTxt = '';
                              if isempty(DSGEModel.MixedConditionalShocks)==1;
                                 CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                              elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                                 CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                              end;
                              if isempty(CondTxt)==1;
                                 [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsMixed(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,0,0,[]);
                              else;
                                 About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                 DoneCalc = 0;
                              end;
                           end;
                        end;
                        %
                        % clear everything
                        %
                        clear('thetaDraws');
                        if DoneCalc==1;
                           %
                           % now compute the predictive likelihood with kernel density estimation for the predicted variables
                           %
                           LogPredLike = PredLikeDSGEPathsKernelDensity(YData,PredictionVariables,NumPriorDraws,NumPredPaths,MaxPH,AnnualizeData,[],0,'DSGE','Cond',ShockMethod,DSGEModel,CurrINI);
                           CompTime = toc;
                           %
                           % ask the user if results should be saved to file
                           %
                           SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                           SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                           %
                           PredictionSampleStrMat = CreateFullSample(DSGEModel);
                           PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                           txt = ['Would you like to save the prior draws results for the ' lower(AnnualStr) ' predictions to disk?'];
                           answer = Query(txt,'question',200,'Save - ZLB Predictive Likelihood',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveLikelihood-ZLB-Cond-Prior-' DSGEModel.NameOfModel '-' ShockMethod int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                              YNames = DSGEModel.VariableNames;
                              save(file,'YNames','PredictionVariables','LogPredLike','PredDateVector','PredictionSampleStrMat');
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\PLikeZLBCondPrior_' DSGEModel.NameOfModel '_' strrep(ShockMethod,'-','') '_' int2str(SelectedYData) '_' AnnualStr '_' SelectedVarStr '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumPriorDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                              fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                              fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable log predictive likelihoods\n%% and the joint predictive likelihood for each forecast horizon.\n%%\n');
                              fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                              fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The predictive likelihood estimates for the individual and the selected jointly predicted variables based on the prior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), LogPredLike (matrix with the individual variable log predictive likelihood values and the joint value for each forecast horizon), PredDateVector (vector with dates for the forecast horizon), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','Predictive Likelihood - ZLB Prior Distribution',500,600,CurrINI);
                           end;
                           %
                           % write the results to a text file
                           %
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\PredictiveLikelihood-ZLB-Cond-Prior-' DSGEModel.NameOfModel '-' ShockMethod int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                           %
                           PrintZLBPredictiveLikelihood(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),LogPredLike,AnnualizeData,0,1,0,1,NumPriorDraws,[],NumPriorDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                           %
                           % display the results
                           %
                           if FileExist(ResFile)==1;
                              TextGUI(ResFile,['ZLB Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution',50,CurrINI,1,0);
                           end;
                        else;
                           CompTime = toc;
                        end;
                     end;
                  elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                     txt = 'You didn''t select any variables to predict when computing the log predictive likelihood. YADA is therefore unable to help you!';
                     About(txt,'information','Predictive Likelihood - ZLB Prior Distribution',150,500,CurrINI);
                  end;
               end;
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConPredHeightPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','Predictive Likelihood - ZLB Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the predictive likelihood?';
                  answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'Predictive Likelihood - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  UsePaths = 1;
                  if NumPredPaths*NumDraws>=10000000;
                     txt = ['The total number of paths that the predictive likelihood calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                     answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Posterior Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        UsePaths = 0;
                     end;
                  end;
                  %
                  % check if we should continue
                  %
                  if UsePaths==1;
                     %
                     % collect some required data
                     %
                     theta = ModeData.thetaMode;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     LowerBound = ModeData.LowerBound;
                     UniformBounds = ModeData.UniformBounds;
                     ModelParameters = ModeData.ModelParameters;
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Predictive Likelihood - ZLB Posterior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                           BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                        end;
                     end;
                  else;
                     DataAction = 'cancel';
                  end;
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                        end;
                     end;
                     %
                     % check the length of the conditioning sample
                     %
                     [status,NumPeriods] = CheckConditioningData(DSGEModel);
                     %
                     % check if we should adjust the X matrix
                     %
                     if MaxPH<NumPeriods;
                        NumPeriods = MaxPH;
                     end;
                     %
                     % collect the conditional variables and the selected conditioning sample
                     %
                     Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
                     if LastPeriod>=NumPeriods;
                        U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
                     else;
                        %
                        % we have to create artifical initial conditions for the conditioning variables
                        %
                        U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
                     end;
                     %
                     % Determine the position of the monetary policy rate and select sample length
                     % when the zero zero lower bound may be binding.
                     %
                     [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           %
                           % check if there are any matches between conditional variable names and predicted variable names
                           %
                           [status,VarStrMat] = CompareVariablesStrMat(DSGEModel.ZVariableNames(DSGEModel.ConditionalVariables,:),DSGEModel.VariableNames(PredictionVariables,:));
                           %
                           if status==1;
                              VarStr = vecstr(VarStrMat,',');
                              VarStr = VarStr(1:length(VarStr)-1);
                              txt = ['Some of the conditioning variables have the same name as the selected predicted variables. These variables are: ' VarStr '. Are you sure you wish to continue?'];
                              answer = Query(txt,'question',400,'Predictive Likelihood - ZLB Posterior Distribution',600,'no',CurrINI);
                           else;
                              answer = 'yes';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              DSGEModel.PredictionVariables = PredictionVariables;
                              %
                              % Determine the prediction sample
                              %
                              SampleStrMat = CreateFullSample(DSGEModel);
                              PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                              SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                              %
                              %
                              if gcbo==controls.predict.conditional.original.zlb.height.posterior;
                                 AnnualizeData = 0;
                                 AnnualStr = 'original';
                                 QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Original Data';
                              elseif gcbo==controls.predict.conditional.annual.zlb.height.posterior;
                                 AnnualizeData = 1;
                                 AnnualStr = 'annual';
                                 QueryHead = 'ZLB Predictive Distributions (Predictive Likelihood) - Annualized Data';
                              end;
                              %
                              % check which method is used for dealing with the conditioning assumptions
                              %
                              if DSGEModel.ShockControlMethod==1;
                                 ShockMethod = 'Direct-';
                              elseif DSGEModel.ShockControlMethod==2;
                                 ShockMethod = 'WaggonerZha-';
                              else;
                                 ShockMethod = 'Subset-';
                              end;
                              %
                              % check if we have already stored results to disk
                              %
                              if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumDraws,NumPredPaths,AnnualizeData,0,CurrChain,1,0,'')==1;
                                 txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                                 answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                              else;
                                 answer = 'No';
                              end;
                              if strcmp(lower(answer),'yes')==1;
                                 %
                                 % collect certain results
                                 %
                                 tic;
                                 DoneCalc = 1;
                              else;
                                 %
                                 % ask if we should conduct prediction event calculations
                                 %
                                 if NumPredPaths>1;
                                    txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                                    answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                                 else;
                                    answer = 'No';
                                 end;
                                 if strcmp(lower(answer),'yes')==1;
                                    if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                       DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                    end;
                                    %
                                    % make sure that the prediction event length does not exceed the maximum prediction horizon
                                    %
                                    for i=1:DSGEModel.n;
                                       if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                          DSGEModel.PredictionEvent(i,3) = MaxPH;
                                       end;
                                    end;
                                    %
                                    % display a dialog for setting up an prediction event
                                    %
                                    [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                    if strcmp(lower(action),'ok')==1;
                                       DSGEModel.PredictionEvent = PredictionEvent;
                                       DSGEModel.RunPredictionEvent = 1;
                                    else;
                                       DSGEModel.RunPredictionEvent = 0;
                                    end;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                                 %
                                 % we can now compute the predictive distribution for the endogenous variables
                                 % up to period MaxPH
                                 %
                                 tic;
                                 if DSGEModel.ShockControlMethod==1;
                                    [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPaths(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
                                 elseif DSGEModel.ShockControlMethod==2;
                                    [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsWZ(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
                                 else;
                                    %
                                    % check if the selected method can actually be used
                                    %
                                    CondTxt = '';
                                    if isempty(DSGEModel.MixedConditionalShocks)==1;
                                       CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                                    elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                                       CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                                    end;
                                    if isempty(CondTxt)==1;
                                       [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsMixed(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
                                    else;
                                       About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                       DoneCalc = 0;
                                    end;
                                 end;
                              end;
                              %
                              % clear everything
                              %
                              clear('thetaPostSample');
                              if DoneCalc==1;
                                 %
                                 % now compute the predictive likelihood with kernel density estimation for the predicted variables
                                 %
                                 LogPredLike = PredLikeDSGEPathsKernelDensity(YData,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,1,'DSGE','Cond',ShockMethod,DSGEModel,CurrINI);
                                 CompTime = toc;
                                 %
                                 % ask the user if results should be saved to file
                                 %
                                 SelectedVarStr = ismember((1:DSGEModel.n),DSGEModel.PredictionVariables);
                                 SelectedVarStr = strrep(num2str(SelectedVarStr),' ','');
                                 %
                                 PredictionSampleStrMat = CreateFullSample(DSGEModel);
                                 PredictionSampleStrMat = PredictionSampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                                 txt = ['Would you like to save the posterior draws results for the ' lower(AnnualStr) ' predictions to disk?'];
                                 answer = Query(txt,'question',200,'Save - ZLB Predictive Likelihood',500,'no',CurrINI);
                                 if strcmp(lower(answer),'yes')==1;
                                    [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                                    file = FixFilePath([DSGEModel.OutputDirectory '\PredictiveLikelihood-ZLB-Cond-Posterior-' DSGEModel.NameOfModel '-' ShockMethod int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                                    YNames = DSGEModel.VariableNames;
                                    save(file,'YNames','PredictionVariables','LogPredLike','PredDateVector','PredictionSampleStrMat');
                                    mfile = FixFilePath([DSGEModel.OutputDirectory '\PLikeZLBCondPosterior_' DSGEModel.NameOfModel '_' strrep(ShockMethod,'-','') '_' int2str(SelectedYData) '_' AnnualStr '_' SelectedVarStr '_' int2str(CurrChain) '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                                    fid = fopen(mfile,'wt');
                                    fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                    fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                    fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                    fprintf(fid,'YNames = DataStructure.YNames;\n');
                                    fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                                    fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                                    fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable log predictive likelihoods\n%% and the joint predictive likelihood for each forecast horizon.\n%%\n');
                                    fprintf(fid,'LogPredLike = DataStructure.LogPredLike;\n');
                                    fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                    fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                    fprintf(fid,'%%\n%% string matrix with prediction sample dates \n%%\n');
                                    fprintf(fid,'PredictionSampleStrMat = DataStructure.PredictionSampleStrMat;\n');
                                    fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                    %
                                    % end the script file
                                    %
                                    fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                    fclose(fid);
                                    %
                                    txt = ['The predictive likelihood estimates for the individual and the selected jointly predicted variables based on the posterior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 5 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), LogPredLike (matrix with the individual variable log predictive likelihood values and the joint value for each forecast horizon), PredDateVector (vector with dates for the forecast horizon), and PredictionSampleStrMat (string matrix with the prediction sample dates). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                    About(txt,'information','Predictive Likelihood - ZLB Posterior Distribution',500,600,CurrINI);
                                 end;
                                 %
                                 % write the results to a text file
                                 %
                                 ResFile = FixFilePath([DSGEModel.OutputDirectory '\PredictiveLikelihood-ZLB-Cond-Posterior-' DSGEModel.NameOfModel '-' ShockMethod int2str(SelectedYData) '.' AnnualStr '-' SelectedVarStr '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                                 %
                                 PrintZLBPredictiveLikelihood(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),LogPredLike,AnnualizeData,1,1,0,1,NumDraws,NumBurnin,NumPostDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                                 %
                                 % display the results
                                 %
                                 if FileExist(ResFile)==1;
                                    TextGUI(ResFile,['ZLB Predictive Likelihood - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution',50,CurrINI,1,0);
                                 end;
                              else;
                                 CompTime = toc;
                              end;
                           else;
                              clear('thetaPostSample');
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           clear('thetaPostSample');
                           txt = 'You didn''t select any variables to predict when computing the log predictive likelihood. YADA is therefore unable to help you!';
                           About(txt,'information','Predictive Likelihood - ZLB Posterior Distribution',150,500,CurrINI);
                        else;
                           clear('thetaPostSample');
                        end;
                     else;
                        clear('thetaPostSample');
                     end;
                  else;
                     clear('thetaPostSample');
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConCRPSPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         if MatlabNumber>=8.0;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               AIMData = load(DSGEModel.AIMDataFile);
            catch;
               ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
            end;
         end;
         %
         % check if the posterior mode file exists
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            ErrorStr = '';
            if MatlabNumber>=8.0;
               try;
                  ModeData = load(ModeFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  ModeData = load(ModeFile);
               catch;
                  ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if FileExist(DSGEModel.PriorFile)==1;
                  txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                  answer = Query(txt,'question',200,'CRPS - ZLB Prior Distribution',500,'no',CurrINI);
               else;
                  answer = 'Yes';
               end;
               if strcmp(lower(answer),'yes')==1;
                  InitStatus = 1;
                  theta = ModeData.theta;
                  thetaDist = ModeData.thetaDist;
                  PriorDist = ModeData.PriorDist;
                  ParameterNames = ModeData.ParameterNames;
                  LowerBound = ModeData.LowerBound;
                  ModelParameters = ModeData.ModelParameters;
                  thetaPositions = ModeData.thetaPositions;
                  UniformBounds = ModeData.UniformBounds;
                  thetaIndex = ModeData.thetaIndex;
               else;
                  [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
               end;
            else;
               InitStatus = 0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         else;
            [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
         end;
         if InitStatus==1;
            %
            % load the prior draws data
            %
            if FileExist(DSGEModel.SystemPriorFile)==0;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
            else;
               PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
            end;
            PriorData = load(PriorDrawsFile);
            thetaDraws = PriorData.thetaDraws;
            NumPriorDraws = size(thetaDraws,2);
            %
            % check if we have a system prior and if so, use a subset of the prior draws
            %
            OldDSGEModel = DSGEModel;
            if FileExist(DSGEModel.SystemPriorFile)==1;
               if DSGEModel.PosteriorSampler>7;
                  [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection','CRPS - Prior Distribution','','',CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.PostDrawsPercentValue = SelectedOption;
                  end;
               else;
                  action = 'ok';
               end;
               if strcmp(lower(action),'ok')==1;
                  UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                  if length(UseDraws)<NumPriorDraws;
                     thetaDraws = thetaDraws(:,UseDraws);
                     NumPriorDraws = length(UseDraws);
                  end;
               end;
            end;
            clear('PriorData');
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            UsePaths = 1;
            if NumPredPaths*NumPriorDraws>100000;
               txt = ['The total number of paths that the CRPS and ES calculations will use is equal ' int2str(NumPredPaths*NumPriorDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
               answer = Query(txt,'question',400,'CRPS - Prior Distribution',500,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  UsePaths = 0;
               end;
            end;
            %
            if UsePaths==1;
               %
               % determine the sample
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               DSGEModel.FirstPeriod = FirstPeriod;
               DSGEModel.LastPeriod = LastPeriod;
               %
               % check how many time periods to use for predictive distributions
               %
               MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
               %
               % check the maximum when we take NaNs into account
               %
               YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
               BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
               if isempty(DSGEModel.Actuals)==1;
                  SelectedYData = 1;
                  DataAction = 'ok';
                  YDataStr = 'Actuals from original sample';
               else;
                  YDataStr = 'Actuals from original sample';
                  m = length(DSGEModel.Actuals);
                  for i=1:m;
                     YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                  end;
                  [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - Prior Distribution','','',CurrINI);
                  if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                     YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                  end;
               end;
            else;
               DataAction = 'cancel';
            end;
            %
            if strcmp(lower(DataAction),'ok')==1;
               MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               MaxFH = min(MaxFH,MaxPH);
               YData = YData(:,1:MaxFH);
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                  end;
               end;
               %
               % check the length of the conditioning sample
               %
               [status,NumPeriods] = CheckConditioningData(DSGEModel);
               %
               % check if we should adjust the X matrix
               %
               if MaxPH<NumPeriods;
                  NumPeriods = MaxPH;
               end;
               %
               % collect the conditional variables and the selected conditioning sample
               %
               Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
               if LastPeriod>=NumPeriods;
                  U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
               else;
                  %
                  % we have to create artifical initial conditions for the conditioning variables
                  %
                  U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
               end;
               %
               % determine the state equation names
               %
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % Determine the position of the monetary policy rate and select sample length
               % when the zero zero lower bound may be binding.
               %
               [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.RtildePosition = RtildePosition;
                  DSGEModel.REqPosition = REqPosition;
                  DSGEModel.ZLBRuleType = ZLBRuleType;
                  DSGEModel.ZLBSampleT = ZLBSampleT;
                  %
                  % selection of variable 
                  %
                  if isempty(DSGEModel.PredictionVariables)==1;
                     PredictionVariables = ones(1,DSGEModel.n);
                     PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                  else;
                     PredictionVariables = DSGEModel.PredictionVariables;
                  end;
                  %
                  [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                  if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                     %
                     % check if there are any matches between conditional variable names and predicted variable names
                     %
                     [status,VarStrMat] = CompareVariablesStrMat(DSGEModel.ZVariableNames(DSGEModel.ConditionalVariables,:),DSGEModel.VariableNames(PredictionVariables,:));
                     %
                     if status==1;
                        VarStr = vecstr(VarStrMat,',');
                        VarStr = VarStr(1:length(VarStr)-1);
                        txt = ['Some of the conditioning variables have the same name as the selected predicted variables. These variables are: ' VarStr '. Are you sure you wish to continue?'];
                        answer = Query(txt,'question',400,'CRPS - ZLB Prior Distribution',600,'no',CurrINI);
                     else;
                        answer = 'yes';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        DSGEModel.PredictionVariables = PredictionVariables;
                        %
                        % Determine the prediction sample
                        %
                        SampleStrMat = CreateFullSample(DSGEModel);
                        PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                        SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                        %
                        %
                        if gcbo==controls.predict.conditional.original.zlb.crps.prior;
                           AnnualizeData = 0;
                           AnnualStr = 'original';
                           QueryHead = 'ZLB Predictive Distributions (CRPS & ES) - Original Data';
                        elseif gcbo==controls.predict.conditional.annual.zlb.crps.prior;
                           AnnualizeData = 1;
                           AnnualStr = 'annual';
                           QueryHead = 'ZLB Predictive Distributions (CRPS & ES) - Annualized Data';
                        end;
                        %
                        % check which method is used for dealing with the conditioning assumptions
                        %
                        if DSGEModel.ShockControlMethod==1;
                           ShockMethod = 'Direct-';
                        elseif DSGEModel.ShockControlMethod==2;
                           ShockMethod = 'WaggonerZha-';
                        else;
                           ShockMethod = 'Subset-';
                        end;
                        %
                        % check if we have already stored results to disk
                        %
                        if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,0,[],0,0,'')==1;
                           txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           tic;
                           DoneCalc = 1;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           if NumPredPaths>1;
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % transpose the prior draws matrix
                           %
                           thetaDraws = thetaDraws';
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           tic;
                           if DSGEModel.ShockControlMethod==1;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPaths(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,0,0,[]);
                           elseif DSGEModel.ShockControlMethod==2;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsWZ(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,0,0,[]);
                           else;
                              %
                              % check if the selected method can actually be used
                              %
                              CondTxt = '';
                              if isempty(DSGEModel.MixedConditionalShocks)==1;
                                 CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                              elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                                 CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                              end;
                              if isempty(CondTxt)==1;
                                 [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsMixed(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,0,0,[]);
                              else;
                                 About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                 DoneCalc = 0;
                              end;
                           end;
                        end;
                        %
                        % clear everything
                        %
                        clear('thetaDraws');
                        if DoneCalc==1;
                           %
                           % now compute the predictive likelihood with kernel density estimation for the predicted variables
                           %
                           CRPSMatrix = ComputeCRPSZLBDSGEPaths(YData,PredictionVariables,NumPriorDraws,NumPredPaths,MaxPH,AnnualizeData,[],0,'DSGE','Cond',ShockMethod,DSGEModel,CurrINI);
                           CompTime = toc;
                           %
                           % check if the user wants to store the results to disk
                           %
                           txt = 'Would you like to save the continuous ranked probability score (CRPS) and energy score (ES) estimates to file?';
                           answer = Query(txt,'question',300,'CRPS - ZLB Prior Distribution',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              file = FixFilePath([DSGEModel.OutputDirectory '\CRPS-ZLB-Cond-Prior-' DSGEModel.NameOfModel '-' ShockMethod int2str(SelectedYData) '.' AnnualStr '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                              YNames = DSGEModel.VariableNames;
                              save(file,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\CRPSZLBCondPrior_' DSGEModel.NameOfModel '_' strrep(ShockMethod,'-','') '_' int2str(SelectedYData) '_' AnnualStr '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumPriorDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                              fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                              fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%% for each forecast horizon.\n%%\n');
                              fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The continuous probability score (CRPS) estimates and the energy scores for the jointly predicted variables based on the prior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','CRPS - ZLB Prior Distribution',500,600,CurrINI);
                           end;
                           %
                           % write the results to a text file
                           %
                           ResFile = FixFilePath([DSGEModel.OutputDirectory '\CRPS-ZLB-Cond-Prior-' DSGEModel.NameOfModel '-' ShockMethod int2str(SelectedYData) '.' AnnualStr '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                           PrintCRPS(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,0,1,0,1,NumPriorDraws,[],NumPriorDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                           %
                           % check if file exists and display it then
                           %
                           if FileExist(ResFile)==1;
                              TextGUI(ResFile,['CRPS & ES - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'ZLB Prior Distribution',50,CurrINI,1,0);
                           end;
                        else;
                           CompTime = toc;
                        end;
                     end;
                  elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                     txt = 'You didn''t select any variables to predict when computing the CRPS and ES. YADA is therefore unable to help you!';
                     About(txt,'information','CRPS - ZLB Prior Distribution',150,500,CurrINI);
                  end;
               end;
            else;
               DSGEModel = OldDSGEModel;
            end;
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConCRPSPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','CRPS - ZLB Posterior Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  clear('LogPost','AcceptedDraws','PostSample');
                  [NumPostDraws,NumParam] = size(thetaPostSample);
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumPostDraws,:);
                  NumDraws = NumPostDraws-NumBurnin;
                  %
                  % check if the user wishes a sub-sample
                  %
                  txt = 'Would you like to use a sub-sample of the posterior draws when estimating the CRPS and the ES?';
                  answer = Query(txt,'question',400,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                  if (DSGEModel.PosteriorSampler<=7)|(DSGEModel.PosteriorSampler==9);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumDraws);
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        NumDraws = length(UseDraws);
                     end;
                  else;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % check which draws we should use
                        %
                        UseDraws = WeightBasedPosteriorDraws(DSGEModel,CurrINI,'CRPS - Posterior Distribution',Weights,[]);
                        if isempty(UseDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        thetaPostSample = thetaPostSample(UseDraws,:);
                        Weights = Weights(UseDraws,1);
                        Weights = Weights*length(Weights)/sum(Weights);
                        NumDraws = length(UseDraws);
                     end;
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  UsePaths = 1;
                  if NumPredPaths*NumDraws>100000;
                     txt = ['The total number of paths that the CRPS and ES calculations will use is equal ' int2str(NumPredPaths*NumDraws) ' and it may take a very long time to finish them. Are you sure you wish to continue?'];
                     answer = Query(txt,'question',400,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                     if strcmp(lower(answer),'no')==1;
                        UsePaths = 0;
                     end;
                  end;
                  %
                  % check if we should continue
                  %
                  if UsePaths==1;
                     %
                     % collect some required data
                     %
                     theta = ModeData.thetaMode;
                     thetaPositions = ModeData.thetaPositions;
                     thetaIndex = ModeData.thetaIndex;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     LowerBound = ModeData.LowerBound;
                     UniformBounds = ModeData.UniformBounds;
                     ModelParameters = ModeData.ModelParameters;
                     %
                     [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
                     DSGEModel.FirstPeriod = FirstPeriod;
                     DSGEModel.LastPeriod = LastPeriod;
                     %
                     % check how many time periods to use for predictive distributions
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check the maximum when we take NaNs into account
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     BackYData = DSGEModel.Y(:,LastPeriod-1:LastPeriod);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        DataAction = 'ok';
                        YDataStr = 'Actuals from original sample';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [DataAction,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','CRPS - ZLB Posterior Distribution','','',CurrINI);
                        if (strcmp(lower(DataAction),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                           BackYData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod-1:LastPeriod);
                        end;
                     end;
                  else;
                     DataAction = 'cancel';
                  end;
                  if strcmp(lower(DataAction),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     %
                     % matrix with exogenous variables
                     %
                     X = [];
                     if DSGEModel.k>0;
                        if NumPredX<MaxPH;
                           X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                        end;
                        if NumPredX>0;
                           X = [X DSGEModel.PredictedX(:,1:min(NumPredX,MaxPH))];
                        end;
                     end;
                     %
                     % check the length of the conditioning sample
                     %
                     [status,NumPeriods] = CheckConditioningData(DSGEModel);
                     %
                     % check if we should adjust the X matrix
                     %
                     if MaxPH<NumPeriods;
                        NumPeriods = MaxPH;
                     end;
                     %
                     % collect the conditional variables and the selected conditioning sample
                     %
                     Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
                     if LastPeriod>=NumPeriods;
                        U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
                     else;
                        %
                        % we have to create artifical initial conditions for the conditioning variables
                        %
                        U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
                     end;
                     %
                     % Determine the position of the monetary policy rate and select sample length
                     % when the zero zero lower bound may be binding.
                     %
                     [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % selection of variable 
                        %
                        if isempty(DSGEModel.PredictionVariables)==1;
                           PredictionVariables = ones(1,DSGEModel.n);
                           PredictionVariables = PredictionVariables .* (1:DSGEModel.n);
                        else;
                           PredictionVariables = DSGEModel.PredictionVariables;
                        end;
                        %
                        [Action,PredictionVariables] = SelectCondVarShockDLG('init','Prediction Variables','',DSGEModel.VariableNames,PredictionVariables,CurrINI);
                        if (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==0);
                           %
                           % check if there are any matches between conditional variable names and predicted variable names
                           %
                           [status,VarStrMat] = CompareVariablesStrMat(DSGEModel.ZVariableNames(DSGEModel.ConditionalVariables,:),DSGEModel.VariableNames(PredictionVariables,:));
                           %
                           if status==1;
                              VarStr = vecstr(VarStrMat,',');
                              VarStr = VarStr(1:length(VarStr)-1);
                              txt = ['Some of the conditioning variables have the same name as the selected predicted variables. These variables are: ' VarStr '. Are you sure you wish to continue?'];
                              answer = Query(txt,'question',400,'CRPS - ZLB Posterior Distribution',600,'no',CurrINI);
                           else;
                              answer = 'yes';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              DSGEModel.PredictionVariables = PredictionVariables;
                              %
                              % Determine the prediction sample
                              %
                              SampleStrMat = CreateFullSample(DSGEModel);
                              PredictionSampleStrMat = SampleStrMat(LastPeriod+1:LastPeriod+MaxFH,:);
                              SmpStr = [strrep(PredictionSampleStrMat(1,:),':','.') '.' strrep(PredictionSampleStrMat(MaxFH,:),':','.')];
                              %
                              %
                              if gcbo==controls.predict.conditional.original.zlb.crps.posterior;
                                 AnnualizeData = 0;
                                 AnnualStr = 'original';
                                 QueryHead = 'ZLB Predictive Distributions (CRPS & ES) - Original Data';
                              elseif gcbo==controls.predict.conditional.annual.zlb.crps.posterior;
                                 AnnualizeData = 1;
                                 AnnualStr = 'annual';
                                 QueryHead = 'ZLB Predictive Distributions (CRPS & ES) - Annualized Data';
                              end;
                              %
                              % check which method is used for dealing with the conditioning assumptions
                              %
                              if DSGEModel.ShockControlMethod==1;
                                 ShockMethod = 'Direct-';
                              elseif DSGEModel.ShockControlMethod==2;
                                 ShockMethod = 'WaggonerZha-';
                              else;
                                 ShockMethod = 'Subset-';
                              end;
                              %
                              % check if we have already stored results to disk
                              %
                              if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumDraws,NumPredPaths,AnnualizeData,0,CurrChain,1,0,'')==1;
                                 txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                                 answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                              else;
                                 answer = 'No';
                              end;
                              if strcmp(lower(answer),'yes')==1;
                                 %
                                 % collect certain results
                                 %
                                 tic;
                                 DoneCalc = 1;
                              else;
                                 %
                                 % ask if we should conduct prediction event calculations
                                 %
                                 if NumPredPaths>1;
                                    txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                                    answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                                 else;
                                    answer = 'No';
                                 end;
                                 if strcmp(lower(answer),'yes')==1;
                                    if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                       DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                                    end;
                                    %
                                    % make sure that the prediction event length does not exceed the maximum prediction horizon
                                    %
                                    for i=1:DSGEModel.n;
                                       if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                          DSGEModel.PredictionEvent(i,3) = MaxPH;
                                       end;
                                    end;
                                    %
                                    % display a dialog for setting up an prediction event
                                    %
                                    [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                                    if strcmp(lower(action),'ok')==1;
                                       DSGEModel.PredictionEvent = PredictionEvent;
                                       DSGEModel.RunPredictionEvent = 1;
                                    else;
                                       DSGEModel.RunPredictionEvent = 0;
                                    end;
                                 else;
                                    DSGEModel.RunPredictionEvent = 0;
                                 end;
                                 %
                                 % we can now compute the predictive distribution for the endogenous variables
                                 % up to period MaxPH
                                 %
                                 tic;
                                 if DSGEModel.ShockControlMethod==1;
                                    [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPaths(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
                                 elseif DSGEModel.ShockControlMethod==2;
                                    [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsWZ(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
                                 else;
                                    %
                                    % check if the selected method can actually be used
                                    %
                                    CondTxt = '';
                                    if isempty(DSGEModel.MixedConditionalShocks)==1;
                                       CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                                    elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                                       CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                                    end;
                                    if isempty(CondTxt)==1;
                                       [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsMixed(thetaPostSample,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,0,CurrChain,1,Weights);
                                    else;
                                       About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                       DoneCalc = 0;
                                    end;
                                 end;
                              end;
                              %
                              % clear everything
                              %
                              clear('thetaPostSample');
                              if DoneCalc==1;
                                 %
                                 % now compute the predictive likelihood with kernel density estimation for the predicted variables
                                 %
                                 CRPSMatrix = ComputeCRPSZLBDSGEPaths(YData,PredictionVariables,NumDraws,NumPredPaths,MaxPH,AnnualizeData,CurrChain,1,'DSGE','Cond',ShockMethod,DSGEModel,CurrINI);
                                 CompTime = toc;
                                 %
                                 % ask the user if results should be saved to file
                                 %
                                 txt = 'Would you like to save the continuous ranked probability score (CRPS) and energy score (ES) estimates to file?';
                                 answer = Query(txt,'question',300,'CRPS - ZLB Posterior Distribution',500,'no',CurrINI);
                                 if strcmp(lower(answer),'yes')==1;
                                    [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                                    file = FixFilePath([DSGEModel.OutputDirectory '\CRPS-ZLB-Cond-Posterior-' DSGEModel.NameOfModel '-' ShockMethod int2str(SelectedYData) '.' AnnualStr '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.mat']);
                                    YNames = DSGEModel.VariableNames;
                                    save(file,'YNames','PredictionVariables','CRPSMatrix','PredDateVector');
                                    mfile = FixFilePath([DSGEModel.OutputDirectory '\CRPSZLBCondPosterior_' DSGEModel.NameOfModel '_' strrep(ShockMethod,'-','') '_' int2str(SelectedYData) '_' AnnualStr '_' int2str(CurrChain) '_' int2str(DSGEModel.ZLBSampleT) '_' int2str(DSGEModel.RtildePosition) '_' int2str(DSGEModel.REqPosition) '_' int2str(DSGEModel.ZLBRuleType) '_' int2str(MaxPH) '_' int2str(NumDraws) 'x' int2str(NumPredPaths) '_' strrep(SmpStr,'.','_') '.m']);
                                    fid = fopen(mfile,'wt');
                                    fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                                    fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                                    fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                                    fprintf(fid,'YNames = DataStructure.YNames;\n');
                                    fprintf(fid,'%%\n%% Selected group of joint prediction variables\n%%\n');
                                    fprintf(fid,'PredictionVariables = DataStructure.PredictionVariables;\n');
                                    fprintf(fid,'%%\n%% n+1 x h matrix with the individual variable CRPS and the joint energy score\n%% for each forecast horizon.\n%%\n');
                                    fprintf(fid,'CRPSMatrix = DataStructure.CRPSMatrix;\n');
                                    fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                                    fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                                    fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                                    %
                                    % end the script file
                                    %
                                    fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                                    fclose(fid);
                                    %
                                    txt = ['The continuous probability score (CRPS) estimates and the energy scores for the jointly predicted variables based on the posterior draws have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 4 entries: YNames (string matrix with the names of the observed variables), PredictionVariables (vector with integer values for the jointly predicted variables), CRPSMatrix (matrix with the individual variable CRPS values and the joint energy score for each forecast horizon), and PredDateVector (vector with dates for the forecast horizon). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                                    About(txt,'information','CRPS - ZLB Posterior Distribution',500,600,CurrINI);
                                 end;
                                 %
                                 % write the results to a text file
                                 %
                                 ResFile = FixFilePath([DSGEModel.OutputDirectory '\CRPS-ZLB-Cond-Posterior-' DSGEModel.NameOfModel '-' ShockMethod int2str(SelectedYData) '.' AnnualStr '-' int2str(CurrChain) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumDraws) '.' int2str(NumPredPaths) '-' SmpStr '.txt']);
                                 PrintCRPS(ResFile,'DSGE',PredictionVariables,PredictionSampleStrMat,YDataStr(SelectedYData,:),CRPSMatrix,AnnualizeData,1,1,0,1,NumDraws,NumBurnin,NumPostDraws,NumPredPaths,[],CompTime,DSGEModel,CurrINI);
                                 %
                                 % check if file exists and display it then
                                 %
                                 if FileExist(ResFile)==1;
                                    TextGUI(ResFile,['CRPS & ES - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'ZLB Posterior Distribution',50,CurrINI,1,0);
                                 end;
                              else;
                                 CompTime = toc;
                              end;
                           else;
                              clear('thetaPostSample');
                           end;
                        elseif (strcmp(lower(Action),'ok')==1)&(isempty(PredictionVariables)==1);
                           clear('thetaPostSample');
                           txt = 'You didn''t select any variables to predict when computing the log predictive likelihood. YADA is therefore unable to help you!';
                           About(txt,'information','Predictive Likelihood - ZLB Posterior Distribution',150,500,CurrINI);
                        end;
                     else;
                        clear('thetaPostSample');
                     end;
                  else;
                     clear('thetaPostSample');
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConPredInitialValues'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         drawnow;
         %
         % check first if posterior mode results exists. If they do we load initial values from this file.
         % Otherwise we have to do a bunch of checking
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         ErrorStr = '';
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         if FileExist(ModeFile)==1;
            answer = 'Yes';
         else;
            %
            % Ask if the user really wants to look at these before estimating the posterior mode
            %
            txt = 'Are you sure you want to view graphs of the conditional predictive distributions of the observed variables before running the posterior mode estimation routine?';
            answer = Query(txt,'question',150,'Zero Lower Bound Forecasts - Initial Values',500,'no',CurrINI);
         end;
         %
         % check if we should check transformations functions
         %
         if gcbo==controls.predict.unconditional.transformed.zlb.initialvalues;
            ErrorStr = '';
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
            if isempty(ErrorStr)==0;
               About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
               answer = 'no';
            end;
         end;
         %
         if strcmp(lower(answer),'yes')==1;
            %
            % posterior mode estimation has not been completed yet. We therefore need to check all the imput files
            %
            [InitStatus,theta,thetaPositions,ModelParameters] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            if (InitStatus==0)&(FileExist(DSGEModel.PriorFile)==0);
               if FileExist(ModeFile)==1;
                  if MatlabNumber>=8.0;
                     try;
                        ModeData = load(ModeFile);
                     catch ME;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                     end;
                  else;
                     try;
                        ModeData = load(ModeFile);
                     catch;
                        ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                     end;
                  end;
                  if isempty(ErrorStr)==1;
                     theta = ModeData.theta;
                     thetaPositions = ModeData.thetaPositions;
                     ModelParameters = ModeData.ModelParameters;
                     ModelParameters = ThetaToModelParameters(ModelParameters,theta,thetaPositions);
                     InitStatus = 1;
                  else;
                     About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
                  end;
               end;
            end;
            if strcmp(ErrorStr,'')==1;
               if MatlabNumber>=8.0;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     AIMData = load(DSGEModel.AIMDataFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
                  end;
               end;
            end;
            %
            if (InitStatus==1)&(strcmp(ErrorStr,'')==1);
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % determine the state equation names
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                  end;
               end;
               %
               % check the length of the conditioning sample
               %
               [status,NumPeriods] = CheckConditioningData(DSGEModel);
               %
               % check if we should adjust the Z matrix
               %
               if MaxPH<NumPeriods;
                  NumPeriods = MaxPH;
               end;
               %
               % collect the conditional variables and the selected conditioning sample
               %
               Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
               if LastPeriod>=NumPeriods;
                  U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
               else;
                  %
                  % we have to create artifical initial conditions for the conditioning variables
                  %
                  U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
               end;
               %
               % check if we should annualize or not
               %
               if gcbo==controls.predict.conditional.original.zlb.initialvalues;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'Original';
               elseif gcbo==controls.predict.conditional.annual.zlb.initialvalues;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'Annual';
               else;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'Transformed';
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               if NumPredPaths==1;
                  NumPredPaths = 100;
                  DSGEModel.NumPredPathsValue = 1;
               end;
               %
               % Determine the position of the monetary policy rate and select sample length
               % when the zero zero lower bound may be binding.
               %
               [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
               if strcmp(lower(action),'ok')==1;
                  DSGEModel.RtildePosition = RtildePosition;
                  DSGEModel.REqPosition = REqPosition;
                  DSGEModel.ZLBRuleType = ZLBRuleType;
                  DSGEModel.ZLBSampleT = ZLBSampleT;
                  %
                  txt = 'Would you like to recenter the structural shocks such that the predictive distribution of the DSGE model without anticipated shocks (zero values) is centered on the observed data?';
                  answer = Query(txt,'question',300,'Zero Lower Bound Forecasts - Initial Values',600,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     IsCentered = 0;
                     action = 'ok';
                     YDataStr = '';
                     SelectedYData = [];
                  else;
                     IsCentered = 1;
                  end;
                  %
                  if strcmp(lower(action),'ok')==1;
                     if IsCentered==1;
                        FileCentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-InitialValues-1-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                        file = FileCentered;
                     else;
                        FileUncentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-InitialValues-0-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                        file = FileUncentered;
                     end;
                     if FileExist(file)==1;
                        txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
                        answer = Query(txt,'question',250,'Zero Lower Bound Forecasts - Initial Values',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     %
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % retrieve data from file
                        %
                        ZLBData = load(file);
                        status = 1;
                        ZLBYPaths = ZLBData.ZLBYPaths;
                        ZLBKsiPaths = ZLBData.ZLBKsiPaths;
                        KsitTh = ZLBData.KsitTh;
                        PredData = ZLBData.PredData;
                        Status = 1;
                        PredEventData = ZLBData.PredEventData;
                        if isempty(PredEventData)==0;
                           DSGEModel.PredictionEvent = ZLBData.PredictionEvent;
                           DSGEModel.RunPredictionEvent = 1;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        if IsCentered==1;
                           SelectedYData = ZLBData.SelectedYData;
                        else;
                           SelectedYData = [];
                        end;
                     else;
                        if IsCentered==1;
                           %
                           % check which actuals data to use
                           %
                           MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                           %
                           % check which data to use
                           %
                           YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                           if isempty(DSGEModel.Actuals)==1;
                              SelectedYData = 1;
                              YDataStr = 'Actuals from original sample';
                              action = 'ok';
                           else;
                              YDataStr = 'Actuals from original sample';
                              m = length(DSGEModel.Actuals);
                              for i=1:m;
                                 YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                              end;
                              [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Zero Lower Bound Forecasts - Initial Values','','',CurrINI);
                              if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                                 YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                              else;
                                 Status = 0;
                              end;
                           end;
                           if strcmp(lower(action),'ok')==1;
                              MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                              MaxFH = min(MaxFH,MaxPH);
                              YData = YData(:,1:MaxFH);
                              MaxPH = MaxFH;
                              YDataStr = ['-' int2str(SelectedYData)];
                              if MaxFH==0;
                                 txt = ['There are no data points for the observed variables over the forecast sample. YADA will therefore not be able to re-center the structural shocks. YADA will therefore abort the calculations.'];
                                 About(txt,'information','Zero Lower Bound Forecasts - Initial Values',150,600,CurrINI);
                                 action = 'cancel';
                              else;
                                 X = X(:,1:MaxPH);
                              end;
                           end;
                        else;
                           action = 'ok';
                        end;
                        if strcmp(lower(action),'ok')==1;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           if NumPredPaths>1;
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer2 = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           else;
                              answer2 = 'No';
                           end;
                           if strcmp(lower(answer2),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPh
                           %
                           if DSGEModel.RandomNumberValue==1;
                              randn('state',0);
                           else;
                              randn('state',sum(100*clock));
                           end;
                           %
                           Status = 0;
                           if strcmp(lower(answer),'no')==1;
                              if DSGEModel.ShockControlMethod==1;
                                 if IsCentered==0;
                                    %
                                    % simpler calculations when not having centered unrestricted conditional predictive distribution
                                    %
                                    [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECondPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Initial Values');
                                 else;
                                    %
                                    % more complex calculations which need to run the unconditional predictions first, also using
                                    % the additional draws
                                    %
                                    [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredCondPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Initial Values',SelectedYData);
                                 end;
                              elseif DSGEModel.ShockControlMethod==2;
                                 if IsCentered==0;
                                    %
                                    % simpler calculations when not having centered unrestricted conditional predictive distribution
                                    %
                                    [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Initial Values');
                                 else;
                                    %
                                    % more complex calculations which need to run the unconditional predictions first, also using
                                    % the additional draws
                                    %
                                    [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredCondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Initial Values',SelectedYData);
                                 end;
                              else;
                                 %
                                 % check if the selected method can actually be used
                                 %
                                 CondTxt = '';
                                 if isempty(DSGEModel.MixedConditionalShocks)==1;
                                    CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                                 elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                                    CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                                 end;
                                 if isempty(CondTxt)==1;
                                    if IsCentered==0;
                                       %
                                       % simpler calculations when not having centered unrestricted conditional predictive distribution
                                       %
                                       [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECondPredictionPathsThetaMixed(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Initial Values');
                                    else;
                                       %
                                       % more complex calculations which need to run the unconditional predictions first, also using
                                       % the additional draws
                                       %
                                       [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredCondPredictionPathsThetaMixed(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Initial Values',SelectedYData);
                                    end;
                                 else;
                                    About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                    PredPaths = [];
                                 end;
                              end;
                           end;
                        end;
                     end;
                     %
                     % check if we should continues
                     %
                     if Status==1;
                        %
                        % compute point estimates and quantile values
                        %
                        % 1. for the observables
                        %
                        [ZLBYMeanPrediction,ZLBYQuantiles] = PredictionPathDistribution(ZLBYPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                        %
                        % 2. for the state variables
                        %
                        [ZLBKsiMeanPrediction,ZLBKsiQuantiles] = PredictionPathDistribution(ZLBKsiPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                        %
                        % check if we should compute prediction event results
                        %
                        if DSGEModel.RunPredictionEvent==1;
                           PredEventData = CalculatePredictionEvents(ZLBYPaths,DSGEModel.PredictionEvent);
                           %
                           % write the results to text file
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionEvent-' DSGEModel.NameOfModel '-InitialValues-' int2str(IsCentered) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.txt']);
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                           %
                           % displays the file if it exists
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['Zero Lower Bound Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Initial Values',50,CurrINI,1,0);
                           end;
                        end;
                        %
                        % check if the user wants to save the data
                        %
                        txt = ['Would you like to save the conditional predictive distributions for the ' lower(AnnualStr) ' data, subject to the zero lower bound, at initial values to file?'];
                        answer = Query(txt,'question',140,'Save - Zero Lower Bound Forecasts',500,'no',CurrINI);
                        drawnow;
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(ZLBYMeanPrediction,2));
                           file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionResults-' DSGEModel.NameOfModel '-' AnnualStr '-InitialValues-' int2str(IsCentered) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                           YNames = DSGEModel.VariableNames;
                           StateVariableNames = DSGEModel.StateVariableNames;
                           DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                           Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           if (length(DateVector)>size(Y,2))&(TransData==0);
                              DateVector = DateVector(1,1:size(Y,2));
                           end;
                           save(file,'YNames','StateVariableNames','DateVector','Y','ZLBYMeanPrediction','ZLBYQuantiles','KsitTh','ZLBKsiMeanPrediction','ZLBKsiQuantiles','ZLBYPaths','ZLBKsiPaths','PredData');
                           %
                           % create a matlab m-file to read the output.
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBCPRes' AnnualStr 'InitVal' int2str(IsCentered) int2str(FirstPeriod) int2str(LastPeriod) int2str(DSGEModel.ZLBSampleT) int2str(DSGEModel.RtildePosition) int2str(DSGEModel.REqPosition) int2str(DSGEModel.ZLBRuleType) int2str(DSGEModel.ShockControlMethod) int2str(MaxPH) int2str(NumPredPaths) '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the observed variables\n%%\n');
                           fprintf(fid,'ZLBYPaths = DataStructure.ZLBYPaths;\n');
                           fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the state variables\n%%\n');
                           fprintf(fid,'ZLBKsiPaths = DataStructure.ZLBKsiPaths;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% string matrix with the names of the state variable variables\n%%\n');
                           fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                           fprintf(fid,'%%\n%% observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           if IsCentered==0;
                              fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB\n%%\n');
                           else;
                              fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB with recentered shocks\n%%\n');
                           end;
                           fprintf(fid,'%%\n%% smooth estimates of the state variables\n%%\n');
                           fprintf(fid,'KsitTh = DataStructure.KsitTh;\n');
                           fprintf(fid,'ZLBYMeanPrediction = DataStructure.ZLBYMeanPrediction;\n');
                           if IsCentered==0;
                              fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB\n%%\n');
                           else;
                              fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB with recentered shocks\n%%\n');
                           end;
                           fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Y\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBYQuantiles));
                           fprintf(fid,'ZLBYQuantiles = DataStructure.ZLBYQuantiles;\n');
                           fprintf(fid,'ZLBKsiMeanPrediction = DataStructure.ZLBKsiMeanPrediction;\n');
                           fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Ksi\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBKsiQuantiles));
                           fprintf(fid,'ZLBKsiQuantiles = DataStructure.ZLBKsiQuantiles;\n');
                           fprintf(fid,'%%\n%% structure with data on the suggested and the used shocks, initial states, and\n%% measurement errors\n%%\n');
                           fprintf(fid,'PredData = DataStructure.PredData;\n');
                           fprintf(fid,'%%\n%% vector with dates for the observed and the state variables\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The conditional predictive distributions for the observed and the state variables, subject to the zero lower bound and at the initial values, have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 12 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), Y (matrix with the observed variables), ZLBYMeanPrediction (matrix with mean predictions for the observed variables), ZLBYQuantiles (vector structure with 2 fields: Quantile and percent for the observed variables), ZLBKsiMeanPredictions (matrix with mean predictions for the state variables), ZLBKsiQuantiles (vector structure with 2 fields: Quantile and percent for the state variables), ZLBYPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the observed variables), ZLBKsiPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the state variables), Data (structure with fields containing the original and used draws of structural shocks, initial states, and measurement errors), and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Zero Lower Bound Forecasts - Initial Values',800,700,CurrINI);
                        end;
                        %
                        if size(PredData.A,1)==1;
                           ObsVarMean = PredData.A';
                        else;
                           ObsVarMean = [];
                        end;
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,ZLBYMeanPrediction,ObsVarMean,ZLBYQuantiles,PredData.KernelX,PredData.KernelY,KsitTh,ZLBKsiMeanPrediction,ZLBKsiQuantiles,'Initial Values',AnnualStr,0,1,IsCentered,[],SelectedYData);
                     end;
                  end;
               end;
            elseif strcmp(ErrorStr,'')==0;
               About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
            end;
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConPredPriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         %
         % check if we should check transformations functions
         %
         ErrorStr = '';
         if gcbo==controls.predict.conditional.transformed.zlb.prior;
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         if isempty(ErrorStr)==1;
            %
            % are we using annualized data or the original
            %
            if gcbo==controls.predict.conditional.original.zlb.prior;
               TransData = 0;
               AnnualizeData = 0;
               AnnualStr = 'original';
               QueryHead = 'ZLB Cond. Predictive Distributions - Original Data';
            elseif gcbo==controls.predict.conditional.annual.zlb.prior;
               TransData = 0;
               AnnualizeData = 1;
               AnnualStr = 'annual';
               QueryHead = 'ZLB Cond. Predictive Distributions - Annualized Data';
            else;
               TransData = 1;
               AnnualizeData = 0;
               AnnualStr = 'transformed';
               QueryHead = 'ZLB Cond. Predictive Distributions - Transformed Data';
            end;
            %
            % check if the posterior mode file exists
            %
            ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
            if FileExist(ModeFile)==1;
               ErrorStr = '';
               if MatlabNumber>=8.0;
                  try;
                     ModeData = load(ModeFile);
                  catch ME;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
                  end;
               else;
                  try;
                     ModeData = load(ModeFile);
                  catch;
                     ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
                  end;
               end;
               if strcmp(ErrorStr,'')==1;
                  if FileExist(DSGEModel.PriorFile)==1;
                     txt = ['Would you like to use the prior information stored in the posterior mode results file "' GetFilename(ModeFile) '"? If you answer ''No'', YADA will reread the prior file "' GetFilename(DSGEModel.PriorFile) '".'];
                     answer = Query(txt,'question',200,QueryHead,500,'no',CurrINI);
                  else;
                     answer = 'Yes';
                  end;
                  if strcmp(lower(answer),'yes')==1;
                     InitStatus = 1;
                     theta = ModeData.theta;
                     thetaDist = ModeData.thetaDist;
                     PriorDist = ModeData.PriorDist;
                     ParameterNames = ModeData.ParameterNames;
                     LowerBound = ModeData.LowerBound;
                     ModelParameters = ModeData.ModelParameters;
                     thetaPositions = ModeData.thetaPositions;
                  else;
                     [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
                  end;
               else;
                  InitStatus = 0;
                  About(ErrorStr,'information','YADA - Bad MAT File',500,600,CurrINI);
               end;
            else;
               [InitStatus,theta,thetaPositions,ModelParameters,thetaDist,PriorDist,ParameterNames,thetaIndex,UniformBounds,LowerBound] = InitializeDSGEModelSimulation(DSGEModel,CurrINI);
            end;
            if InitStatus==1;
               %
               % load the data
               %
               if FileExist(DSGEModel.SystemPriorFile)==0;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-' DSGEModel.NameOfModel '.mat']);
               else;
                  PriorDrawsFile = FixFilePath([DSGEModel.OutputDirectory '\priordraws\PriorDraws-SystemPrior-' DSGEModel.NameOfModel '.mat']);
               end;
               PriorData = load(PriorDrawsFile);
               thetaDraws = PriorData.thetaDraws;
               NumPriorDraws = size(thetaDraws,2);
               %
               % check if we have a system prior and if so, use a subset of the prior draws
               %
               OldDSGEModel = DSGEModel;
               if FileExist(DSGEModel.SystemPriorFile)==1;
                  if DSGEModel.PosteriorSampler>7;
                     [action,SelectedOption] = SelectionDlg(get(controls.posterior.usepostdraws,'String'),DSGEModel.PostDrawsPercentValue,'Select the percent of prior draws to use (cancel means all draws):','Prior Draws Selection',QueryHead,'','',CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.PostDrawsPercentValue = SelectedOption;
                     end;
                  else;
                     action = 'ok';
                  end;
                  if strcmp(lower(action),'ok')==1;
                     UseDraws = KeepPosteriorDraws(DSGEModel,controls.posterior.usepostdraws,NumPriorDraws);
                     if length(UseDraws)<NumPriorDraws;
                        thetaDraws = thetaDraws(:,UseDraws);
                        NumPriorDraws = length(UseDraws);
                     end;
                  end;
               end;
               %
               % initialize the random number generator
               %
               if DSGEModel.RandomNumberValue==1;
                  randn('state',0);
                  rand('state',0);
               else;
                  randn('state',sum(100*clock));
                  rand('state',sum(100*clock));
               end;
               %
               % determine the state equation names
               %
               StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
               StateEquationNames = '';
               for s=1:length(DSGEModel.StateEquationPositions);
                  StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
               end;
               %
               % get the sample dates
               %
               [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
               %
               % check how many time periods to use for predictive distributions
               %
               [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
               %
               % matrix with exogenous variables
               %
               X = [];
               if DSGEModel.k>0;
                  if NumPredX<MaxPH;
                     X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
                  end;
                  if NumPredX>0;
                     X = [X DSGEModel.PredictedX(:,1:NumPredX)];
                  end;
               end;
               %
               % check the length of the conditioning sample
               %
               [status,NumPeriods] = CheckConditioningData(DSGEModel);
               %
               % check if we should adjust the X matrix
               %
               if MaxPH<NumPeriods;
                  NumPeriods = MaxPH;
               end;
               %
               % collect the conditional variables and the selected conditioning sample
               %
               Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
               if LastPeriod>=NumPeriods;
                  U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
               else;
                  %
                  % we have to create artifical initial conditions for the conditioning variables
                  %
                  U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
               end;
               %
               % check which method is used for dealing with the conditioning assumptions
               %
               if DSGEModel.ShockControlMethod==1;
                  ShockMethod = 'Direct-';
               elseif DSGEModel.ShockControlMethod==2;
                  ShockMethod = 'WaggonerZha-';
               else;
                  ShockMethod = 'Subset-';
               end;
               %
               % determine how many prediction paths to use
               %
               NumPredPathsStr = get(controls.forecast.predpaths,'String');
               NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
               if NumPredPaths==1;
                  NumPredPaths = 100;
                  DSGEModel.NumPredPathsValue = 1;
               end;
               %
               % ask if the user wants to center the original forecasts from the linear model on the actual data
               %
               txt = 'Would you like to recenter the structural shocks such that the conditional predictive distribution of the DSGE model without anticipated shocks (zero values) is centered on the observed data?';
               answer = Query(txt,'question',300,QueryHead,600,'no',CurrINI);
               if strcmp(lower(answer),'no')==1;
                  IsCentered = 0;
                  action = 'ok';
                  YDataStr = '';
                  SelectedYData = [];
               else;
                  IsCentered = 1;
                  %
                  % check which actuals data to use
                  %
                  MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                  %
                  % check which data to use
                  %
                  YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                  if isempty(DSGEModel.Actuals)==1;
                     SelectedYData = 1;
                     YDataStr = 'Actuals from original sample';
                     action = 'ok';
                  else;
                     YDataStr = 'Actuals from original sample';
                     m = length(DSGEModel.Actuals);
                     for i=1:m;
                        YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                     end;
                     [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection',QueryHead,'','',CurrINI);
                     if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                        YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                     end;
                  end;
                  if strcmp(lower(action),'ok')==1;
                     MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                     MaxFH = min(MaxFH,MaxPH);
                     YData = YData(:,1:MaxFH);
                     MaxPH = MaxFH;
                     YDataStr = ['-' int2str(SelectedYData)];
                     if MaxFH==0;
                        txt = ['There are no data points for the observed variables over the forecast sample. YADA will therefore not be able to re-center the structural shocks. YADA will therefore abort the calculations.'];
                        About(txt,'information',QueryHead,150,600,CurrINI);
                        action = 'cancel';
                     else;
                        X = X(:,1:MaxPH);
                     end;
                  end;
               end;
               %
               if strcmp(lower(action),'ok')==1;
                  %
                  % Determine the position of the monetary policy rate and select sample length
                  % when the zero zero lower bound may be binding.
                  %
                  [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEMoel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                  if strcmp(lower(action),'ok')==1;
                     DSGEModel.RtildePosition = RtildePosition;
                     DSGEModel.REqPosition = REqPosition;
                     DSGEModel.ZLBRuleType = ZLBRuleType;
                     DSGEModel.ZLBSampleT = ZLBSampleT;
                     %
                     % check if we have already stored results to disk
                     %
                     if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,0,0,IsCentered,YDataStr)==1;
                        txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                        answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % collect certain results
                        %
                        [PredEventData,PredictionEvent,ObsVarMean] = LoadZLBDSGEPredictionData(DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,0,0,IsCentered,YDataStr);
                        DoneCalc = 1;
                     else;
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        if NumPredPaths>1;
                           txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                           answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % transpose the prior draws matrix
                        %
                        thetaDraws = thetaDraws';
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPH
                        %
                        if DSGEModel.ShockControlMethod==1;
                           if IsCentered==0;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPaths(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,0,0,[]);
                           else;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECenteredCondPredictionPaths(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,0,0,[],YDataStr);
                           end;
                        elseif DSGEModel.ShockControlMethod==2;
                           if IsCentered==0;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsWZ(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,0,0,[]);
                           else;
                              [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECenteredCondPredictionPathsWZ(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,0,0,[],YDataStr);
                           end;
                        else;
                           %
                           % check if the selected method can actually be used
                           %
                           CondTxt = '';
                           if isempty(DSGEModel.MixedConditionalShocks)==1;
                              CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                           elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                              CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                           end;
                           if isempty(CondTxt)==1;
                              if IsCentered==0;
                                 [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsMixed(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,0,0,[]);
                              else;
                                 [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECenteredCondPredictionPathsMixed(thetaDraws,theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,0,0,[],YDataStr);
                              end;
                           else;
                              About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                              DoneCalc = 0;
                           end;
                        end;
                     end;
                     %
                     % clear everything
                     %
                     clear('thetaDraws');
                     %
                     % check if we should show any results
                     %
                     if DoneCalc==1;
                        %
                        % yes, now we need to calculate some distribution properties
                        %
                        [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY,AverageZLBShocks,NumZLBTimes] = ZLBDSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPriorDraws,NumPredPaths,AnnualizeData,TransData,0,0,IsCentered,YDataStr,[],DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                        %
                        % compute the forecast errors based on the mean predictions
                        %
                        PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                        %
                        % check if the user wants to save the data
                        %
                        txt = ['Would you like to save the conditional predictive distributions for the ' AnnualStr ' data to file?'];
                        answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                        if strcmp(lower(answer),'yes')==1;
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                           file = FixFilePath([DSGEModel.OutputDirectory '\ZLBCondPredictivePriorDist-' int2str(IsCentered) YDataStr '-' ShockMethod AnnualStr '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                           YNames = DSGEModel.VariableNames;
                           DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                           Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                           if (length(DateVector)>size(Y,2))&(TransData==0);
                              DateVector = DateVector(1,1:size(Y,2));
                           end;
                           EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                           EstimationStartPeriodOrig = EstimationStartPeriod;
                           YOrig = Y;
                           DateVectorOrig = DateVector;
                           if AnnualizeData==1;
                              Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                              EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                              DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                           end;
                           if TransData==1;
                              [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                              Y = YData';
                              if size(Y,2)==DSGEModel.T;
                                 Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              else;
                                 T2 = size(Y,2);
                                 if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                    Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                    DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                 end;
                              end;
                           end;
                           if isempty(PredEventData)==1;
                              save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                           else;
                              save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                           end;
                           if isempty(AverageZLBShocks)==0;
                              save(file,'AverageZLBShocks','NumZLBTimes','-APPEND');
                           end;
                           %
                           % create a matlab m-file to read the output.
                           %
                           mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBCondPPD' int2str(IsCentered) strrep(YDataStr,'-','') strrep(ShockMethod,'-','') AnnualStr int2str(CurrChain) int2str(ZLBSampleT) int2str(RtildePosition) int2str(REqPosition) int2str(ZLBRuleType) int2str(MaxPH) int2str(NumPriorDraws) int2str(NumPredPaths) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                           fid = fopen(mfile,'wt');
                           fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                           fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                           fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                           fprintf(fid,'YNames = DataStructure.YNames;\n');
                           fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                           fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                           fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                           fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                           fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                           fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                           fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                           fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                           fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                           fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                           fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                           fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                           fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                           fprintf(fid,'%%\n%% observed variables\n%%\n');
                           fprintf(fid,'Y = DataStructure.Y;\n');
                           fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                           fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                           CovDecompStr = '';
                           if isempty(PredEventData)==0;
                              fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                              fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                              fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                              fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                              NumEntries = '12';
                              ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                           else;
                              NumEntries = '10';
                              ExtraEntry = '';
                           end;
                           if isempty(AverageZLBShocks)==0;
                              fprintf(fid,'%%\n%% the names of the structural shocks\n%%\n');
                              fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                              fprintf(fid,'%%\n%% the average shock values when the ZLB is binding\n%%\n');
                              fprintf(fid,'AverageZLBShocks = DataStructure.AverageZLBShocks;\n');
                              fprintf(fid,'%%\n%% the number of times the ZLB is binding\n%%\n');
                              fprintf(fid,'NumZLBTimes = DataStructure.NumZLBTimes;\n');
                              NumEntries = int2str(str2num(NumEntries)+3);
                              ExtraEntry = [ExtraEntry 'AverageZLBShocks (matrix with the average shock values when the ZLB is binding), NumZLBTimes (number of times the ZLB is binding), '];
                           end;
                           fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                           %
                           % end the script file
                           %
                           fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                           fclose(fid);
                           %
                           txt = ['The conditional predictive distributions subject to the zero lower bound have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','ZLB Cond. Predictive Distributions',500,600,CurrINI);
                        end;
                        if isempty(PredEventData)==0;
                           %
                           % write prediction event results to file
                           %
                           file = FixFilePath([DSGEModel.OutputDirectory '\ZLBCondPriorPredictionEvent-' int2str(IsCentered) '-' AnnualStr '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPriorDraws) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                           [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                           PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                           %
                           % displays the file if it exists
                           %
                           if FileExist(file)==1;
                              TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Prior Distribution - Conditional Predictions',50,CurrINI,1,0);
                           end;
                        end;
                        %
                        % summarize the information in the uncertainty variables more conveniently
                        %
                        if (AnnualizeData==0)&(TransData==0);
                           %
                           % need to compute a nxh matrix with predictive variances from CovPrediction
                           %
                           [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(zeros(DSGEModel.n,DSGEModel.n,MaxPH),CovPrediction,zeros(DSGEModel.n,MaxPH),zeros(DSGEModel.n,DSGEModel.n,MaxPH));
                           [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                        else;
                           UData = [];
                        end;
                        %
                        % display the predictive distributions
                        %
                        ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,KernelX,KernelY,[],[],[],'Prior Distribution',AnnualStr,1,0,IsCentered,UData,SelectedYData);
                     end;
                  else;
                     clear('thetaDraws');
                  end;
               else;
                  clear('thetaDraws');
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConPredPostMode'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % File with the data needed to calculate the predictive distr
         %
         OriginalNumPredPathsValue = DSGEModel.NumPredPathsValue;
         %
         drawnow;
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.predict.conditional.transformed.zlb.postmode);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            theta = ModeData.thetaMode;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            % StateEqNames = ModeData.AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % determine the state equation names
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckConditioningData(DSGEModel);
            %
            % check if we should adjust the Z matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            if LastPeriod>=NumPeriods;
               U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
            else;
               %
               % we have to create artifical initial conditions for the conditioning variables
               %
               U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
            end;
            %
            % check if we should annualize or not
            %
            if DSGEModel.RunDriver==0;
               if gcbo==controls.predict.conditional.original.zlb.postmode;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'Original';
               elseif gcbo==controls.predict.conditional.annual.zlb.postmode;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'Annual';
               else;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'Transformed';
               end;
            else;
               if strcmp(lower(DSGEModel.answers{1}),'original')==1;
                  TransData = 0;
                  AnnualizeData = 0;
                  AnnualStr = 'Original';
               elseif strcmp(lower(DSGEModel.answers{1}),'annual')==1;
                  TransData = 0;
                  AnnualizeData = 1;
                  AnnualStr = 'Annual';
               else;
                  TransData = 1;
                  AnnualizeData = 0;
                  AnnualStr = 'Transformed';
               end;
            end;
            %
            % determine how many prediction paths to use
            %
            NumPredPathsStr = get(controls.forecast.predpaths,'String');
            NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
            if NumPredPaths==1;
               NumPredPaths = 100;
               DSGEModel.NumPredPathsValue = 1;
            end;
            %
            % Determine the position of the monetary policy rate and select sample length
            % when the zero zero lower bound may be binding.
            %
            if DSGEModel.RunDriver==0;
               [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
            else;
               action = DSGEModel.answers{2};
               RtildePosition = DSGEModel.RtildePosition;
               REqPosition = DSGEModel.REqPosition;
               ZLBRuleType = DSGEModel.ZLBRuleType;
               ZLBSampleT = DSGEModel.ZLBSampleT;
            end;
            if (strcmp(lower(action),'ok')==1)&(sum([isempty(RtildePosition) isempty(REqPosition)])==0);
               DSGEModel.RtildePosition = RtildePosition;
               DSGEModel.REqPosition = REqPosition;
               DSGEModel.ZLBRuleType = ZLBRuleType;
               DSGEModel.ZLBSampleT = ZLBSampleT;
               %
               if DSGEModel.RunDriver==0;
                  txt = 'Would you like to recenter the structural shocks such that the predictive distribution of the DSGE model without anticipated shocks (zero values) is centered on the observed data?';
                  answer = Query(txt,'question',300,'Zero Lower Bound Forecasts - Posterior Mode',600,'no',CurrINI);
               else;
                  answer = DSGEModel.answers{3};
               end;
               if strcmp(lower(answer),'no')==1;
                  IsCentered = 0;
                  action = 'ok';
                  YDataStr = '';
                  SelectedYData = [];
               else;
                  IsCentered = 1;
               end;
               %
               if strcmp(lower(action),'ok')==1;
                  if IsCentered==1;
                     FileCentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-PosteriorMode-1-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                     file = FileCentered;
                  else;
                     FileUncentered = FixFilePath([DSGEModel.OutputDirectory '\predictions\ZLB-CondPrediction-' DSGEModel.NameOfModel '-PosteriorMode-0-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                     file = FileUncentered;
                  end;
                  %
                  if DSGEModel.RunDriver==0;
                     if FileExist(file)==1;
                        txt = 'YADA has located results for the conditional predictive distribution on disk. Would you like to use these results?';
                        answer = Query(txt,'question',250,'Zero Lower Bound Forecasts - Posterior Mode',500,'no',CurrINI);
                     else;
                        answer = 'No';
                     end;
                  else;
                     answer = DSGEModel.answers{4};
                  end;
                  %
                  if strcmp(lower(answer),'yes')==1;
                     %
                     % retrieve data from file
                     %
                     ZLBData = load(file);
                     status = 1;
                     ZLBYPaths = ZLBData.ZLBYPaths;
                     ZLBKsiPaths = ZLBData.ZLBKsiPaths;
                     KsitTh = ZLBData.KsitTh;
                     PredData = ZLBData.PredData;
                     Status = 1;
                     PredEventData = ZLBData.PredEventData;
                     if isempty(PredEventData)==0;
                        DSGEModel.PredictionEvent = ZLBData.PredictionEvent;
                        DSGEModel.RunPredictionEvent = 1;
                     else;
                        DSGEModel.RunPredictionEvent = 0;
                     end;
                     if IsCentered==1;
                        SelectedYData = ZLBData.SelectedYData;
                     else;
                        SelectedYData = [];
                     end;
                  else;
                     if IsCentered==1;
                        %
                        % check which actuals data to use
                        %
                        MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                        %
                        % check which data to use
                        %
                        YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                        if (isempty(DSGEModel.Actuals)==1)|(DSGEModel.RunDriver==1);
                           SelectedYData = 1;
                           YDataStr = 'Actuals from original sample';
                           action = 'ok';
                        else;
                           YDataStr = 'Actuals from original sample';
                           m = length(DSGEModel.Actuals);
                           for i=1:m;
                              YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                           end;
                           [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection','Zero Lower Bound Forecasts - Posterior Mode','','',CurrINI);
                           if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                              YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                           else;
                              Status = 0;
                           end;
                        end;
                        if strcmp(lower(action),'ok')==1;
                           MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                           MaxFH = min(MaxFH,MaxPH);
                           YData = YData(:,1:MaxFH);
                           MaxPH = MaxFH;
                           YDataStr = ['-' int2str(SelectedYData)];
                           if MaxFH==0;
                              txt = ['There are no data points for the observed variables over the forecast sample. YADA will therefore not be able to re-center the structural shocks. YADA will therefore abort the calculations.'];
                              About(txt,'information','Zero Lower Bound Forecasts - Posterior Mode',150,600,CurrINI);
                              action = 'cancel';
                           else;
                              X = X(:,1:MaxPH);
                           end;
                        end;
                     else;
                        action = 'ok';
                     end;
                     if strcmp(lower(action),'ok')==1;
                        %
                        % ask if we should conduct prediction event calculations
                        %
                        if NumPredPaths>1;
                           if DSGEModel.RunDriver==0;
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer2 = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           else;
                              answer2 = DSGEModel.answers{5};
                           end;
                        else;
                           answer2 = 'No';
                        end;
                        if strcmp(lower(answer2),'yes')==1;
                           if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                              DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                           end;
                           %
                           % make sure that the prediction event length does not exceed the maximum prediction horizon
                           %
                           for i=1:DSGEModel.n;
                              if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                 DSGEModel.PredictionEvent(i,3) = MaxPH;
                              end;
                           end;
                           %
                           % display a dialog for setting up an prediction event
                           %
                           [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                           if strcmp(lower(action),'ok')==1;
                              DSGEModel.PredictionEvent = PredictionEvent;
                              DSGEModel.RunPredictionEvent = 1;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                        else;
                           DSGEModel.RunPredictionEvent = 0;
                        end;
                        %
                        % we can now compute the predictive distribution for the endogenous variables
                        % up to period MaxPh
                        %
                        if DSGEModel.RandomNumberValue==1;
                           randn('state',0);
                        else;
                           randn('state',sum(100*clock));
                        end;
                        %
                        Status = 0;
                        if strcmp(lower(answer),'no')==1;
                           if DSGEModel.ShockControlMethod==1;
                              if IsCentered==0;
                                 %
                                 % simpler calculations when not having centered unrestricted conditional predictive distribution
                                 %
                                 [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECondPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Posterior Mode');
                              else;
                                 %
                                 % more complex calculations which need to run the unconditional predictions first, also using
                                 % the additional draws
                                 %
                                 [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredCondPredictionPathsTheta(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Posterior Mode',SelectedYData);
                              end;
                           elseif DSGEModel.ShockControlMethod==2;
                              if IsCentered==0;
                                 %
                                 % simpler calculations when not having centered unrestricted conditional predictive distribution
                                 %
                                 [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Posterior Mode');
                              else;
                                 %
                                 % more complex calculations which need to run the unconditional predictions first, also using
                                 % the additional draws
                                 %
                                 [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredCondPredictionPathsThetaWZ(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Posterior Mode',SelectedYData);
                              end;
                           else;
                              %
                              % check if the selected method can actually be used
                              %
                              CondTxt = '';
                              if isempty(DSGEModel.MixedConditionalShocks)==1;
                                 CondTxt = 'You haven''t selected any shocks for use with the distribution of a subset of the shocks conditioning method.';
                                 LogText = strvcat('You haven''t selected any shocks for use with the distribution','of a subset of the shocks conditioning method.');
                              elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                                 CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                                 LogText = strvcat('The number of shocks you have selected to use to ensure that the','conditioning assumptions are satisfied under the distribution of','a subset of the shocks conditioning method is not sufficient.',['You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.']);
                              end;
                              if isempty(CondTxt)==1;
                                 if IsCentered==0;
                                    %
                                    % simpler calculations when not having centered unrestricted conditional predictive distribution
                                    %
                                    [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECondPredictionPathsThetaMixed(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Posterior Mode');
                                 else;
                                    %
                                    % more complex calculations which need to run the unconditional predictions first, also using
                                    % the additional draws
                                    %
                                    [Status,ZLBYPaths,ZLBKsiPaths,KsitTh,PredData,PredEventData] = ZLBDSGECenteredCondPredictionPathsThetaMixed(theta,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,'Posterior Mode',SelectedYData);
                                 end;
                              else;
                                 if DSGEModel.RunDriver==0;
                                    About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                 else;
                                    SetLogText(LogText);
                                 end;
                                 PredPaths = [];
                              end;
                           end;
                        end;
                     end;
                  end;
                  %
                  % check if we should continues
                  %
                  if Status==1;
                     %
                     % compute point estimates and quantile values
                     %
                     % 1. for the observables
                     %
                     [ZLBYMeanPrediction,ZLBYQuantiles] = PredictionPathDistribution(ZLBYPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                     %
                     % 2. for the state variables
                     %
                     [ZLBKsiMeanPrediction,ZLBKsiQuantiles] = PredictionPathDistribution(ZLBKsiPaths,DSGEModel.Percentiles,DSGEModel.ConfidenceRegionMethod);
                     %
                     % check if we should compute prediction event results
                     %
                     if DSGEModel.RunPredictionEvent==1;
                        PredEventData = CalculatePredictionEvents(ZLBYPaths,DSGEModel.PredictionEvent);
                        %
                        % write the results to text file
                        %
                        file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionEvent-' DSGEModel.NameOfModel '-PosteriorMode-' int2str(IsCentered) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.txt']);
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                        PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],DSGEModel.PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,1,CurrINI);
                        %
                        % displays the file if it exists
                        %
                        if FileExist(file)==1;
                           TextGUI(file,['Zero Lower Bound Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Mode Parameter Values',50,CurrINI,1,0);
                        end;
                     end;
                     %
                     % check if the user wants to save the data
                     %
                     if DSGEModel.RunDriver==0;
                        txt = ['Would you like to save the conditional predictive distributions for the ' lower(AnnualStr) ' data, subject to the zero lower bound, at posterior mode to file?'];
                        answer = Query(txt,'question',140,'Save - Zero Lower Bound Forecasts',500,'no',CurrINI);
                        drawnow;
                     else;
                        answer = DSGEModel.answers{6};
                     end;
                     if strcmp(lower(answer),'yes')==1;
                        if DSGEModel.RunDriver==1;
                           SetLogText(strvcat('* Saving results to: ',['     ' DSGEModel.OutputDirectory]));
                        end;
                        [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(ZLBYMeanPrediction,2));
                        file = FixFilePath([DSGEModel.OutputDirectory '\ZLB-CondPredictionResults-' DSGEModel.NameOfModel '-' AnnualStr '-PosteriorMode-' int2str(IsCentered) '-' int2str(FirstPeriod) '.' int2str(LastPeriod) '-' int2str(DSGEModel.ZLBSampleT) '.' int2str(DSGEModel.RtildePosition) '.' int2str(DSGEModel.REqPosition) '.' int2str(DSGEModel.ZLBRuleType) '-' int2str(DSGEModel.ShockControlMethod) '-' int2str(MaxPH) '-' int2str(NumPredPaths) '.mat']);
                        YNames = DSGEModel.VariableNames;
                        StateVariableNames = DSGEModel.StateVariableNames;
                        DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                        Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                        if (length(DateVector)>size(Y,2))&(TransData==0);
                           DateVector = DateVector(1,1:size(Y,2));
                        end;
                        save(file,'YNames','StateVariableNames','DateVector','Y','ZLBYMeanPrediction','ZLBYQuantiles','KsitTh','ZLBKsiMeanPrediction','ZLBKsiQuantiles','ZLBYPaths','ZLBKsiPaths','PredData');
                        %
                        % create a matlab m-file to read the output.
                        %
                        mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBCPRes' AnnualStr 'PostMode' int2str(IsCentered) int2str(FirstPeriod) int2str(LastPeriod) int2str(DSGEModel.ZLBSampleT) int2str(DSGEModel.RtildePosition) int2str(DSGEModel.REqPosition) int2str(DSGEModel.ZLBRuleType) int2str(DSGEModel.ShockControlMethod) int2str(MaxPH) int2str(NumPredPaths) '.m']);
                        fid = fopen(mfile,'wt');
                        fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                        fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                        fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the observed variables\n%%\n');
                        fprintf(fid,'ZLBYPaths = DataStructure.ZLBYPaths;\n');
                        fprintf(fid,'%%\n%% 3D matrix with all the prediction paths for the state variables\n%%\n');
                        fprintf(fid,'ZLBKsiPaths = DataStructure.ZLBKsiPaths;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                        fprintf(fid,'YNames = DataStructure.YNames;\n');
                        fprintf(fid,'%%\n%% string matrix with the names of the state variable variables\n%%\n');
                        fprintf(fid,'StateVariableNames = DataStructure.StateVariableNames;\n');
                        fprintf(fid,'%%\n%% observed variables\n%%\n');
                        fprintf(fid,'Y = DataStructure.Y;\n');
                        if IsCentered==0;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB\n%%\n');
                        else;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the observed variables (Y) subject to the ZLB with recentered shocks\n%%\n');
                        end;
                        fprintf(fid,'%%\n%% smooth estimates of the state variables\n%%\n');
                        fprintf(fid,'KsitTh = DataStructure.KsitTh;\n');
                        fprintf(fid,'ZLBYMeanPrediction = DataStructure.ZLBYMeanPrediction;\n');
                        if IsCentered==0;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB\n%%\n');
                        else;
                           fprintf(fid,'%%\n%% matrix with the mean of the simulated predictions of the state variables (Ksi) subject to the ZLB with recentered shocks\n%%\n');
                        end;
                        fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Y\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBYQuantiles));
                        fprintf(fid,'ZLBYQuantiles = DataStructure.ZLBYQuantiles;\n');
                        fprintf(fid,'ZLBKsiMeanPrediction = DataStructure.ZLBKsiMeanPrediction;\n');
                        fprintf(fid,'%%\n%% vector structure with quantiles from the conditional predictive distribution of Ksi\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(ZLBKsiQuantiles));
                        fprintf(fid,'ZLBKsiQuantiles = DataStructure.ZLBKsiQuantiles;\n');
                        fprintf(fid,'%%\n%% structure with data on the suggested and the used shocks, initial states, and\n%% measurement errors\n%%\n');
                        fprintf(fid,'PredData = DataStructure.PredData;\n');
                        fprintf(fid,'%%\n%% vector with dates for the observed and the state variables\n%%\n');
                        fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                        fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                        %
                        % end the script file
                        %
                        fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                        fclose(fid);
                        %
                        if DSGEModel.RunDriver==0;
                           txt = ['The conditional predictive distributions for the observed and the state variables, subject to the zero lower bound and at the posterior mode estimates, have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains 12 entries: YNames (string matrix with the names of the observed variables), StateVariableNames (string matrix with the names of the state variables), Y (matrix with the observed variables), ZLBYMeanPrediction (matrix with mean predictions for the observed variables), ZLBYQuantiles (vector structure with 2 fields: Quantile and percent for the observed variables), ZLBKsiMeanPredictions (matrix with mean predictions for the state variables), ZLBKsiQuantiles (vector structure with 2 fields: Quantile and percent for the state variables), ZLBYPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the observed variables), ZLBKsiPaths (3D matrix of dimension nxhxNumPaths with prediction paths for the state variables), Data (structure with fields containing the original and used draws of structural shocks, initial states, and measurement errors), and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                           About(txt,'information','Zero Lower Bound Forecasts - Posterior Mode',800,700,CurrINI);
                        end;
                     end;
                     %
                     if DSGEModel.RunDriver==0;
                        if size(PredData.A,1)==1;
                           ObsVarMean = PredData.A';
                        else;
                           ObsVarMean = [];
                        end;
                        %
                        % Open a dialog for displaying predictive distributions
                        %
                        ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,ZLBYMeanPrediction,ObsVarMean,ZLBYQuantiles,PredData.KernelX,PredData.KernelY,KsitTh,ZLBKsiMeanPrediction,ZLBKsiQuantiles,'Posterior Mode',AnnualStr,1,1,IsCentered,[],SelectedYData);
                     else;
                        SetLogText(strvcat('* Finished calculations.',' '));
                     end;
                  end;
               end;
            else;
               if DSGEModel.RunDriver==1;
                  SetLogText(strvcat('You haven''t determined the policy rate or the policy equation.','Unable to run the function.'))
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         % reset in case it was temporarily changed
         %
         DSGEModel.NumPredPathsValue = OriginalNumPredPathsValue;
         %
         set(controls.about,'UserData',0);
      end;
   case 'ConPredPosteriorDist'
      if get(controls.about,'UserData')==0;
         set(controls.about,'UserData',1);
         %
         % first attempt to load some posterior mode data
         %
         ModeFile = FixFilePath([DSGEModel.OutputDirectory '\mode\PosteriorMode-' DSGEModel.NameOfModel '.mat']);
         ErrorStr = '';
         HeaderStr = 'YADA - Bad MAT File';
         if MatlabNumber>=8.0;
            try;
               ModeData = load(ModeFile);
            catch ME;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError(ME)];
            end;
         else;
            try;
               ModeData = load(ModeFile);
            catch;
               ErrorStr = ['Unable to load the file "' ModeFile '". Message caught is: ' YADALastError];
            end;
         end;
         if strcmp(ErrorStr,'')==1;
            if MatlabNumber>=8.0;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch ME;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError(ME)];
               end;
            else;
               try;
                  AIMData = load(DSGEModel.AIMDataFile);
               catch;
                  ErrorStr = ['Unable to load the file "' DSGEModel.AIMDataFile '". Message caught is: ' YADALastError];
               end;
            end;
         end;
         %
         % check if we should check transformations functions
         %
         if (isempty(ErrorStr)==1)&(gcbo==controls.predict.conditional.transformed.zlb.posterior);
            DTStatus = VerifyDataTransformation(DSGEModel.VariableNames,DSGEModel.YTransformation,0);
            if DTStatus==1;
               [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
            else;
               ErrorStr = 'You haven''t specified all the necessary fields for computing transformations of the observed variables.';
               HeaderStr = 'Missing Transformation Information';
            end;
         end;
         %
         if strcmp(ErrorStr,'')==1;
            thetaPositions = ModeData.thetaPositions;
            ModelParameters = ModeData.ModelParameters;
            thetaMode = ModeData.thetaMode;
            %
            % determine the state equation names
            %
            StateEqNames = AIMData.eqname(DSGEModel.StateEquationPositions,:);
            StateEquationNames = '';
            for s=1:length(DSGEModel.StateEquationPositions);
               StateEquationNames = strvcat(StateEquationNames,StringTrim(StateEqNames(s,:)));
            end;
            %
            % get the sample dates
            %
            [FirstPeriod,LastPeriod] = CreateSubSample(DSGEModel);
            %
            % check how many time periods to use for predictive distributions
            %
            [MaxPH,NumPredX] = CalcMaxPredictionHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,size(DSGEModel.PredictedX,2));
            %
            % matrix with exogenous variables
            %
            X = [];
            if DSGEModel.k>0;
               if NumPredX<MaxPH;
                  X = DSGEModel.X(:,LastPeriod+1:LastPeriod+(MaxPH-NumPredX));
               end;
               if NumPredX>0;
                  X = [X DSGEModel.PredictedX(:,1:NumPredX)];
               end;
            end;
            %
            % check the length of the conditioning sample
            %
            [status,NumPeriods] = CheckConditioningData(DSGEModel);
            %
            % check if we should adjust the X matrix
            %
            if MaxPH<NumPeriods;
               NumPeriods = MaxPH;
            end;
            %
            % collect the conditional variables and the selected conditioning sample
            %
            Z = DSGEModel.Z(DSGEModel.ConditionalVariables,LastPeriod+1:LastPeriod+NumPeriods);
            if LastPeriod>=NumPeriods;
               U = DSGEModel.U(DSGEModel.ConditionalVariables,LastPeriod-NumPeriods+1:LastPeriod);
            else;
               %
               % we have to create artifical initial conditions for the conditioning variables
               %
               U = [DSGEModel.U(DSGEModel.ConditionalVariables,1)*ones(1,NumPeriods-LastPeriod) DSGEModel.U(DSGEModel.ConditionalVariables,1:LastPeriod)];
            end;
            %
            % if there are multiple chains, ask which one to load
            %
            ChainsStr = get(controls.posterior.chains,'String');
            NumChains = str2num(StringTrim(ChainsStr(DSGEModel.ParallelChainsValue,:)));
            %
            if NumChains>1;
               %
               % ask which chain to load
               %
               ChainsStr = '';
               for i=1:NumChains;
                  ChainsStr = strvcat(ChainsStr,['Posterior chain number ' num2str(i,'%0.0f')]);
               end;
               [action,CurrChain] = SelectionDlg(ChainsStr,1,'Select the posterior draws chain to load','MCMC Chain Selection','ZLB Conditional Predictive Distribution','','',CurrINI);
            else;
               CurrChain = 1;
               action = 'ok';
            end;
            %
            %
            if strcmp(lower(action),'cancel')==0;
               %
               % check if the data actually exists
               %
               if VerifyDSGEPosteriorDraws(DSGEModel,CurrChain,controls)==1;
                  %
                  % load the data
                  %
                  [PostSample,thetaPostSample,LogPost,AcceptedDraws,NumBurnin,Weights] = LoadDSGEPosteriorDraws(DSGEModel,CurrINI,CurrChain);
                  NumDraws = length(LogPost);
                  clear('PostSample','LogPost','AcceptedDraws');
                  drawnow;
                  %
                  % initialize the random number generator
                  %
                  if DSGEModel.RandomNumberValue==1;
                     randn('state',0);
                     rand('state',0);
                  else;
                     randn('state',sum(100*clock));
                     rand('state',sum(100*clock));
                  end;
                  %
                  % select which posterior draws to use
                  %
                  thetaPostSample = thetaPostSample(NumBurnin+1:NumDraws,:);
                  NumDraws = NumDraws-NumBurnin;
                  %
                  % check how many draws we should pick among these
                  %
                  PostDrawsUsageStr = get(controls.posterior.postdrawsusage,'String');
                  NumPostDrawsUsage = min(str2num(strrep(StringTrim(PostDrawsUsageStr(DSGEModel.PostDrawsUsageValue,:)),',','')),NumDraws);
                  %
                  % get the draws to use
                  %
                  if DSGEModel.PosteriorSampler<=7;
                     if DSGEModel.RandomizeDraws==0;
                        DrawFreq = round(NumDraws/NumPostDrawsUsage);
                        SelectDraws = (1:DrawFreq:NumDraws);
                        if length(SelectDraws)+1==NumPostDrawsUsage;
                           SelectDraws = [SelectDraws NumDraws];
                        elseif length(SelectDraws)>NumPostDrawsUsage;
                           SelectDraws = SelectDraws(1:NumPostDrawsUsage);
                        end;
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                     else;
                        thetaPostSample = thetaPostSample(ceil(rand(NumPostDrawsUsage,1)*NumDraws),:);
                     end;
                  else;
                     txt = 'Would you like to use a sub-sample of the posterior draws when computing the predictive distribution?';
                     answer = Query(txt,'question',400,'ZLB Predictive Distributions - Posterior',500,'no',CurrINI);
                     if strcmp(lower(answer),'yes')==1;
                        %
                        % select how many draws to use
                        %
                        SelectDraws = SampledPosteriorDraws(DSGEModel,CurrINI,'ZLB Predictive Distributions - Posterior',Weights,NumDraws,[]);
                        if isempty(SelectDraws)==1;
                           set(controls.about,'UserData',0);
                           return;
                        end;
                        NumPostDrawsUsage = length(SelectDraws);
                        thetaPostSample = thetaPostSample(SelectDraws,:);
                        if DSGEModel.PosteriorSampler~=9;
                           %
                           % fix the weights
                           %
                           Weights = Weights(SelectDraws,1);
                           Weights = Weights*length(Weights)/sum(Weights);
                        else;
                           if isempty(Weights)==0;
                              Weights = Weights(SelectDraws);
                           end;
                        end;
                     end;
                  end;
                  %
                  % are we using annualized data, transformed or the original
                  %
                  if gcbo==controls.predict.conditional.original.zlb.posterior;
                     TransData = 0;
                     AnnualizeData = 0;
                     AnnualStr = 'original';
                     QueryHead = 'ZLB Cond. Predictive Distributions - Original Data';
                  elseif gcbo==controls.predict.conditional.annual.zlb.posterior;
                     TransData = 0;
                     AnnualizeData = 1;
                     AnnualStr = 'annual';
                     QueryHead = 'ZLB Cond. Predictive Distributions - Annualized Data';
                  else;
                     TransData = 1;
                     AnnualizeData = 0;
                     AnnualStr = 'transformed';
                     QueryHead = 'ZLB Cond. Predictive Distributions - Transformed Data';
                  end;
                  %
                  % check which method is used for dealing with the conditioning assumptions
                  %
                  if DSGEModel.ShockControlMethod==1;
                     ShockMethod = 'Direct-';
                  elseif DSGEModel.ShockControlMethod==2;
                     ShockMethod = 'WaggonerZha-';
                  else;
                     ShockMethod = 'Subset-';
                  end;
                  %
                  % determine how many prediction paths to use
                  %
                  NumPredPathsStr = get(controls.forecast.predpaths,'String');
                  NumPredPaths = str2num(strrep(strrep(strrep(StringTrim(NumPredPathsStr(DSGEModel.NumPredPathsValue,:)),'path',''),'s',''),',',''));
                  if NumPredPaths==1;
                     NumPredPaths = 100;
                     DSGEModel.NumPredPathsValue = 1;
                  end;
                  %
                  % ask if the user wants to center the original forecasts from the linear model on the actual data
                  %
                  txt = 'Would you like to recenter the structural shocks such that the conditional predictive distribution of the DSGE model without anticipated shocks (zero values) is centered on the observed data?';
                  answer = Query(txt,'question',300,QueryHead,600,'no',CurrINI);
                  if strcmp(lower(answer),'no')==1;
                     IsCentered = 0;
                     action = 'ok';
                     YDataStr = '';
                     SelectedYData = [];
                  else;
                     IsCentered = 1;
                     %
                     % check which actuals data to use
                     %
                     MaxFH = CalcMaxForecastHorizon(DSGEModel.MaxForecastHorizon,DSGEModel.DataFrequency,DSGEModel.EndYear,DSGEModel.EndPeriod,DSGEModel.SubEndYear,DSGEModel.SubEndPeriod);
                     %
                     % check which data to use
                     %
                     YData = DSGEModel.Y(:,LastPeriod+1:LastPeriod+MaxFH);
                     if isempty(DSGEModel.Actuals)==1;
                        SelectedYData = 1;
                        YDataStr = 'Actuals from original sample';
                        action = 'ok';
                     else;
                        YDataStr = 'Actuals from original sample';
                        m = length(DSGEModel.Actuals);
                        for i=1:m;
                           YDataStr = strvcat(YDataStr,DSGEModel.Actuals(i).title);
                        end;
                        [action,SelectedYData] = SelectionDlg(YDataStr,1,'Select the actuals dataset to use','Data Selection',QueryHead,'','',CurrINI);
                        if (strcmp(lower(action),'ok')==1)&(SelectedYData>1);
                           YData = DSGEModel.Actuals(SelectedYData-1).data(:,LastPeriod+1:LastPeriod+MaxFH);
                        end;
                     end;
                     if strcmp(lower(action),'ok')==1;
                        MaxFH = max((sum(~isnan(YData),1)>0).*(1:MaxFH));
                        MaxFH = min(MaxFH,MaxPH);
                        YData = YData(:,1:MaxFH);
                        MaxPH = MaxFH;
                        YDataStr = ['-' int2str(SelectedYData)];
                        if MaxFH==0;
                           txt = ['There are no data points for the observed variables over the forecast sample. YADA will therefore not be able to re-center the structural shocks. YADA will therefore abort the calculations.'];
                           About(txt,'information',QueryHead,150,600,CurrINI);
                           action = 'cancel';
                        else;
                           X = X(:,1:MaxPH);
                        end;
                     end;
                  end;
                  %
                  if strcmp(lower(action),'ok')==1;
                     %
                     % Determine the position of the monetary policy rate and select sample length
                     % when the zero zero lower bound may be binding.
                     %
                     [action,RtildePosition,REqPosition,ZLBRuleType,ZLBSampleT] = ZLBSelectionDLG('init',DSGEModel.StateVariableNames,StateEquationNames,DSGEModel.DataFrequency,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,DSGEModel.ZLBSampleT,CurrINI);
                     if strcmp(lower(action),'ok')==1;
                        DSGEModel.RtildePosition = RtildePosition;
                        DSGEModel.REqPosition = REqPosition;
                        DSGEModel.ZLBRuleType = ZLBRuleType;
                        DSGEModel.ZLBSampleT = ZLBSampleT;
                        %
                        % check if we have already stored results to disk
                        %
                        if ZLBPredictionsExist(DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,IsCentered,YDataStr)==1;
                           txt = 'YADA has located results for the predictive distribution subject to the zero lower bound on disk. Would you like to use these results?';
                           answer = Query(txt,'question',150,QueryHead,600,'no',CurrINI);
                        else;
                           answer = 'No';
                        end;
                        if strcmp(lower(answer),'yes')==1;
                           %
                           % collect certain results
                           %
                           [PredEventData,PredictionEvent,ObsVarMean] = LoadZLBDSGEPredictionData(DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,IsCentered,YDataStr);
                           DoneCalc = 1;
                        else;
                           %
                           % ask if we should conduct prediction event calculations
                           %
                           if NumPredPaths>1;
                              txt = 'Would you like YADA to calculate prediction event probabilities and perform risk analysis?';
                              answer = Query(txt,'question',140,'Prediction Events',500,'no',CurrINI);
                           else;
                              answer = 'No';
                           end;
                           if strcmp(lower(answer),'yes')==1;
                              if (isfield(DSGEModel,'PredictionEvent')==0)|(isempty(DSGEModel.PredictionEvent)==1)|(size(DSGEModel.PredictionEvent,1)~=DSGEModel.n)|(size(DSGEModel.PredictionEvent,2)~=3);
                                 DSGEModel.PredictionEvent = [Inf*ones(DSGEModel.n,1) zeros(DSGEModel.n,1) ones(DSGEModel.n,1)];
                              end;
                              %
                              % make sure that the prediction event length does not exceed the maximum prediction horizon
                              %
                              for i=1:DSGEModel.n;
                                 if DSGEModel.PredictionEvent(i,3)>MaxPH;
                                    DSGEModel.PredictionEvent(i,3) = MaxPH;
                                 end;
                              end;
                              %
                              % display a dialog for setting up an prediction event
                              %
                              [action,PredictionEvent] = PredictionEventDLG('init',DSGEModel.VariableNames,DSGEModel.PredictionEvent,MaxPH,CurrINI);
                              if strcmp(lower(action),'ok')==1;
                                 DSGEModel.PredictionEvent = PredictionEvent;
                                 DSGEModel.RunPredictionEvent = 1;
                              else;
                                 DSGEModel.RunPredictionEvent = 0;
                              end;
                           else;
                              DSGEModel.RunPredictionEvent = 0;
                           end;
                           %
                           % we can now compute the predictive distribution for the endogenous variables
                           % up to period MaxPH
                           %
                           if DSGEModel.ShockControlMethod==1;
                              if IsCentered==0;
                                 [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                              else;
                                 [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECenteredCondPredictionPaths(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,YDataStr);
                              end;
                           elseif DSGEModel.ShockControlMethod==2;
                              if IsCentered==0;
                                 [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsWZ(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                              else;
                                 [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECenteredCondPredictionPathsWZ(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,YDataStr);
                              end;
                           else;
                              %
                              % check if the selected method can actually be used
                              %
                              CondTxt = '';
                              if isempty(DSGEModel.MixedConditionalShocks)==1;
                                 CondTxt = 'You haven''t selected any shocks for use which the distribution of a subset of the shocks conditioning method.';
                              elseif length(DSGEModel.MixedConditionalShocks)<length(DSGEModel.ConditionalVariables);
                                 CondTxt = ['The number of shocks you have selected to use to ensure that the conditioning assumptions are satisfied under the distribution of a subset of the shocks conditioning method is not sufficient. You need at least ' int2str(length(DSGEModel.ConditionalVariables)) ' such shocks.'];
                              end;
                              if isempty(CondTxt)==1;
                                 if IsCentered==0;
                                    [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECondPredictionPathsMixed(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights);
                                 else;
                                    [DoneCalc,PredEventData,ObsVarMean] = ZLBDSGECenteredCondPredictionPathsMixed(thetaPostSample,thetaMode,thetaPositions,ModelParameters,DSGEModel,CurrINI,YData,X,Z,U,MaxPH,NumPredPaths,AnnualizeData,TransData,CurrChain,1,Weights,YDataStr);
                                 end;
                              else;
                                 About(CondTxt,'information','Bad Shocks Selection - Distribution of Subset Shocks Method',300,550,CurrINI);
                                 DoneCalc = 0;
                              end;
                           end;
                        end;
                        %
                        % clear everything
                        %
                        clear('thetaPostSample');
                        %
                        % check if we should show any results
                        %
                        if DoneCalc==1;
                           %
                           % yes, now we need to calculate some distribution properties
                           %
                           [MeanPrediction,CovPrediction,Quantiles,KernelX,KernelY,AverageZLBShocks,NumZLBTimes] = ZLBDSGEPredictionDistribution(DSGEModel.n,DSGEModel.OutputDirectory,'DSGE','Cond',ShockMethod,DSGEModel.NameOfModel,DSGEModel.ZLBSampleT,DSGEModel.RtildePosition,DSGEModel.REqPosition,DSGEModel.ZLBRuleType,MaxPH,NumPostDrawsUsage,NumPredPaths,AnnualizeData,TransData,CurrChain,1,IsCentered,YDataStr,Weights,DSGEModel.Percentiles,DSGEModel.VariableNames,DSGEModel.ConfidenceRegionMethod,CurrINI);
                           %
                           % compute the forecast errors based on the mean predictions
                           %
                           PredictionErrors = CalculatePredictionErrors(MeanPrediction,AnnualizeData,TransData,'DSGE',DSGEModel);
                           %
                           % check if the user wants to save the data
                           %
                           txt = ['Would you like to save the conditional predictive distributions for the ' AnnualStr ' data to file?'];
                           answer = Query(txt,'question',140,'Save - Conditional Predictive Distributions',500,'no',CurrINI);
                           if strcmp(lower(answer),'yes')==1;
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod,PredDateVector] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,size(MeanPrediction,2));
                              file = FixFilePath([DSGEModel.OutputDirectory '\ZLBCondPredictiveDist-' int2str(IsCentered) YDataStr '-' ShockMethod AnnualStr '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.mat']);
                              YNames = DSGEModel.VariableNames;
                              DateVector = MakeDateVector(DSGEModel.SubBeginYear,DSGEModel.SubBeginPeriod,PredEndYear,PredEndPeriod,DSGEModel.DataFrequency);
                              Y = DSGEModel.Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                              if (length(DateVector)>size(Y,2))&(TransData==0);
                                 DateVector = DateVector(1,1:size(Y,2));
                              end;
                              EstimationStartPeriod = DSGEModel.KalmanFirstObservation;
                              EstimationStartPeriodOrig = EstimationStartPeriod;
                              YOrig = Y;
                              DateVectorOrig = DateVector;
                              if AnnualizeData==1;
                                 Y = Annualize(Y,DSGEModel.annual,DSGEModel.annualscale);
                                 EstimationStartPeriod = max(1,EstimationStartPeriod-(length(DateVector)-size(Y,2)));
                                 DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                              end;
                              if TransData==1;
                                 [YTStatus,YData,ErrorStr,HeaderStr] = CalcTransformData(DSGEModel.VariableNames,DSGEModel.Y',DSGEModel.YTransformation,DSGEModel.YTransMatrix);
                                 Y = YData';
                                 if size(Y,2)==DSGEModel.T;
                                    Y = Y(:,FirstPeriod:min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                 else;
                                    T2 = size(Y,2);
                                    if (FirstPeriod-(DSGEModel.T-T2))>=1;
                                       Y = Y(:,FirstPeriod-(DSGEModel.T-T2):min((LastPeriod+MaxPH),size(DSGEModel.Y,2)));
                                    else;
                                       Y = Y(:,1:min((LastPeriod+MaxPH),size(DSGEModel.Y,2))-(DSGEModel.T-T2));
                                       DateVector = DateVector(length(DateVector)-size(Y,2)+1:length(DateVector));
                                    end;
                                 end;
                              end;
                              if isempty(PredEventData)==1;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig');
                              else;
                                 save(file,'MeanPrediction','CovPrediction','PredictionErrors','Quantiles','KernelX','KernelY','PredDateVector','Y','DateVector','YNames','EstimationStartPeriod','EstimationStartPeriodOrig','YOrig','DateVectorOrig','PredictionEvent','PredEventData');
                              end;
                              if isempty(AverageZLBShocks)==0;
                                 save(file,'AverageZLBShocks','NumZLBTimes','-APPEND');
                              end;
                              %
                              % create a matlab m-file to read the output.
                              %
                              mfile = FixFilePath([DSGEModel.OutputDirectory '\ZLBCondPD' int2str(IsCentered) strrep(YDataStr,'-','') strrep(ShockMethod,'-','') AnnualStr int2str(CurrChain) int2str(ZLBSampleT) int2str(RtildePosition) int2str(REqPosition) int2str(ZLBRuleType) int2str(MaxPH) int2str(NumPostDrawsUsage) int2str(NumPredPaths) DSGEModel.SubEndYear DSGEModel.SubEndPeriod '.m']);
                              fid = fopen(mfile,'wt');
                              fprintf(fid,['%%\n%% load the data in ' GetFilename(file) '\n%%\n']);
                              fprintf(fid,'DataStructure = load(''%s'');\n',GetFilename(file));
                              fprintf(fid,'%%\n%% string matrix with the names of the observed variables\n%%\n');
                              fprintf(fid,'YNames = DataStructure.YNames;\n');
                              fprintf(fid,'%%\n%% matrix with the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'MeanPrediction = DataStructure.MeanPrediction;\n');
                              fprintf(fid,'%%\n%% 3D matrix (nxnxh) with the sample covariance of the predictions of the observed variables\n%%\n');
                              fprintf(fid,'CovPrediction = DataStructure.CovPrediction;\n');
                              fprintf(fid,'%%\n%% matrix with the errors from the mean predictions of the observed variables\n%%\n');
                              fprintf(fid,'PredictionErrors = DataStructure.PredictionErrors;\n');
                              fprintf(fid,'%%\n%% vector structure with quantiles from the predictive distribution\n%% the structure has %0.0f entries with fields "Quantile" and "percent"\n%% the first field is a matrix and the second a positive number between 0 and 100\n%%\n',length(Quantiles));
                              fprintf(fid,'Quantiles = DataStructure.Quantiles;\n');
                              fprintf(fid,'%%\n%% matrix with x-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelX = DataStructure.KernelX;\n');
                              fprintf(fid,'%%\n%% matrix with y-axis values for kernel density estimates of marginal predictive distribution\n%%\n');
                              fprintf(fid,'KernelY = DataStructure.KernelY;\n');
                              fprintf(fid,'%%\n%% vector with dates for the predictions\n%%\n');
                              fprintf(fid,'PredDateVector = DataStructure.PredDateVector;\n');
                              fprintf(fid,'%%\n%% observed variables\n%%\n');
                              fprintf(fid,'Y = DataStructure.Y;\n');
                              fprintf(fid,'%%\n%% vector with dates for the observed variables\n%%\n');
                              fprintf(fid,'DateVector = DataStructure.DateVector;\n');
                              CovDecompStr = '';
                              if isempty(PredEventData)==0;
                                 fprintf(fid,'%%\n%% matrix with the prediction event definitions. The first column\n%% holds the upper bound, the second column\n%% the lower bound, and third the length of the event.\n%%\n');
                                 fprintf(fid,'PredictionEvent = DataStructure.PredictionEvent;\n');
                                 fprintf(fid,'%%\n%% matrix with prediction event data. First column holds the number of true\%% events, the second the number of times below the lower bound of the event,\n%% the third the number of times above the upper bound of the event, the\n%% fourth the number of events, the fifth and sixth the sum and sum of\n%% squared deviations from the lower bound when below, and the seventh and\n%% eighth the sum and sum of squared deviations from the upper bound when above\n%%\n');
                                 fprintf(fid,'PredEventData = DataStructure.PredEventData;\n');
                                 NumEntries = '12';
                                 ExtraEntry = 'PredictionEvent (matrix with the upper and lower bound of the event and the length of the event), PredEventData (matrix with the number of true events, the number of events below the lower bound for the event, the number of events above the upper bound for the event, the number of events tested in the columns, the sum and sum of squared deviations from lower bound when below lower bound, the sum and sum of squared deviations from upper bound when above upper bound), ';
                              else;
                                 NumEntries = '10';
                                 ExtraEntry = '';
                              end;
                              if isempty(AverageZLBShocks)==0;
                                 fprintf(fid,'%%\n%% the names of the structural shocks\n%%\n');
                                 fprintf(fid,'ShockNames = DataStructure.ShockNames;\n');
                                 fprintf(fid,'%%\n%% the average shock values when the ZLB is binding\n%%\n');
                                 fprintf(fid,'AverageZLBShocks = DataStructure.AverageZLBShocks;\n');
                                 fprintf(fid,'%%\n%% the number of times the ZLB is binding\n%%\n');
                                 fprintf(fid,'NumZLBTimes = DataStructure.NumZLBTimes;\n');
                                 NumEntries = int2str(str2num(NumEntries)+3);
                                 ExtraEntry = [ExtraEntry 'AverageZLBShocks (matrix with the average shock values when the ZLB is binding), NumZLBTimes (number of times the ZLB is binding), '];
                              end;
                              fprintf(fid,'%%\n%% Add your own commands below\n%%\n\n\n');
                              %
                              % end the script file
                              %
                              fprintf(fid,'%%\n%% Created by YADA on %s\n%%\n',datestr(now,0));
                              fclose(fid);
                              %
                              txt = ['The conditional predictive distributions subject to the zero lower bound have been saved to the file "' GetFilename(file) '" in the directory "' GetPath(file) '". The file contains ' NumEntries ' entries: YNames (string matrix with the names of the observed variables), MeanPrediction (matrix with the mean predictions), CovPrediction (3D matrix with the sample covariances of the predictions), PredictionErrors (matrix with errors for the mean prediction), Quantiles (vector structure with 2 fields: Quantile and percent), KernelX (matrix with x-axis for kernel density estimates of marginal predictive distribution), KernelY (matrix with y-axis values for kernel density estimates of marginal predictive distribution), PredDateVector (vector with dates for the predictions), Y (matrix with the observed variables), ' CovDecompStr ExtraEntry 'and DateVector (vector with dates for the observed variables). To access these data you may run the matlab script file "' GetFilename(mfile) '".'];
                              About(txt,'information','ZLB Cond. Predictive Distributions',500,600,CurrINI);
                           end;
                           if isempty(PredEventData)==0;
                              %
                              % write prediction event results to file
                              %
                              file = FixFilePath([DSGEModel.OutputDirectory '\ZLBCondPredictionEvent-' int2str(IsCentered) '-' AnnualStr '-' int2str(CurrChain) '-' int2str(ZLBSampleT) '.' int2str(RtildePosition) '.' int2str(REqPosition) '.' int2str(ZLBRuleType) '-' int2str(MaxPH) '-' int2str(NumPostDrawsUsage) '.' int2str(NumPredPaths) '-' DSGEModel.SubEndYear '.' DSGEModel.SubEndPeriod '.txt']);
                              [PredStartYear,PredStartPeriod,PredEndYear,PredEndPeriod] = CreatePredictionSample(DSGEModel.SubEndYear,DSGEModel.SubEndPeriod,DSGEModel.DataFrequency,MaxPH);
                              PrintPredictionEvent(file,DSGEModel.VariableNames,PredEventData,[],PredictionEvent,[PredStartYear ':' PredStartPeriod '-' PredEndYear ':' PredEndPeriod],MaxPH,NumPredPaths,NumPostDrawsUsage,CurrINI);
                              %
                              % displays the file if it exists
                              %
                              if FileExist(file)==1;
                                 TextGUI(file,['Prediction Events - ' DSGEModel.NameOfModel],[(CurrINI.scrsz(3)-1000)/2 50 1000 CurrINI.scrsz(4)-100],'Posterior Distribution - Conditional Predictions',50,CurrINI,1,0);
                              end;
                           end;
                           %
                           % summarize the information in the uncertainty variables more conveniently
                           %
                           if (AnnualizeData==0)&(TransData==0);
                              %
                              % need to compute a nxh matrix with predictive variances from CovPrediction
                              %
                              [StateVar,ShockVar,MeasureVar,ParameterVar,TotalVar] = SummarizeDSGEPredictions(zeros(DSGEModel.n,DSGEModel.n,MaxPH),CovPrediction,zeros(DSGEModel.n,MaxPH),zeros(DSGEModel.n,DSGEModel.n,MaxPH));
                              [UData.KernelX,UData.KernelY] = NormalDensityValues(MeanPrediction,TotalVar,2^8);
                           else;
                              UData = [];
                           end;
                           %
                           % display the predictive distributions
                           %
                           ZLBPredictiveDistributionDLG('init',DSGEModel,CurrINI,MeanPrediction,ObsVarMean,Quantiles,KernelX,KernelY,[],[],[],'Posterior Distribution',AnnualStr,1,1,IsCentered,UData,SelectedYData);
                        end;
                     end;
                  else;
                     clear('thetaPostSample');
                  end;
               else;
                  txt = ['Some of the files for loading the posterior data for chain number ' num2str(CurrChain,'%0.0f') ' are not available.'];
                  About(txt,'error','Missing Posterior Draws',150,500,CurrINI);
               end;
            end;
         else;
            About(ErrorStr,'information',HeaderStr,500,600,CurrINI);
         end;
         %
         set(controls.about,'UserData',0);
      end;
end;
set(controls.open,'UserData',DSGEModel);
set(controls.filemenu,'UserData',CurrINI);
%
% make sure that controls are enabled and disabled as they should
%
YADAEnableControls(DSGEModel,controls);

%
% end of ZLBForecastToolsMenuFunctions.m
%
